/*  Finds the "rings" in the mk4 images.

    This code was written by Alice Lecinski, September 2001.

    It is modeled after David Elmore's mk4 calibraton
    routines written as idl procedures.  David developed these
    procedures in fall of 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>

#include "mk4v2.h"

/* The "rings" are not centered about sun center or the center of
   the mk4 fov.  
   So, to take them out, one must determine the ring's center,
   or the ring center's offset from mk4 center.  
   One does this by:
       1) using heights 340 to 370, for each azimuth, find out
	  which spots are the pixels where the maximum occurs,
       2) then, of these mx spot pixels, get the median pixel.
       3) from that median pixel height, check the nearest
	  +-3 pixel heights for where the maximum occurs.
	  Make the range go 0 to 6.
       4) Then get the median of these heights, subtract it
	  from the range (computed in step 3) and the
	  results are the offsets from mk4-fov-center
	  where the center of the ring occurs.

    Once the offset is known, one creates a vector with
    the data from that offset ring.  One can fix a specific height
    (above 300) by averaging and smoothing of the offset ring
    appropriate for that height.
*/

int FindRing( Img, FixedImg , xdim, ydim , StartP, EndP, FixP)
    float *Img;
    float *FixedImg;
    int    xdim,ydim;
    int    StartP, EndP, FixP;
{
    int   ii,jj;
    int   StartPix, EndPix, TotPix;
    float MyMax, AvgFixM1, derf;
    float *pYvec, *pWarp, *pY, *pW;
    int   MySpot;
    float *pR, *pRing;
    float Cen;
    int   CenRange, CenMin, CenMax, CenHalf;
    int   FixPM1;
    int   Yspot;
    float *pImg;
    float ovDistance;

    FixPM1   = FixP -1;
    CenRange = 6;
    CenHalf  = CenRange >> 1 ;

    /* Copy into FixedImg the original Img */
    VVcopy(Img,FixedImg,xdim*ydim);

    pRing    = FixedImg;         /* Temporarily use FixedImg as free memory */
    pYvec    = FixedImg+xdim;    /* It will get fixed below. */
    pWarp    = FixedImg+xdim;

    /* ; Look for a maximum in the 340 to 370 range
       ; At what height does it most often occur?
    */
    pR       = pRing;
    StartPix = StartP;
    EndPix   = EndP;
    TotPix   = 1+EndPix-StartPix;
    for(ii=0;ii<xdim;ii++){
	pY   = pYvec;
	pImg = Img + StartPix*xdim + ii;
	for(jj=StartPix;jj<=EndPix;jj++){
	    *pY = *pImg;
	    pY++;
	    pImg += xdim;
	}
	get_max(pYvec, TotPix , &MyMax, &MySpot);
	*pR = (float)(MySpot + StartPix);

	pR++;
    }

    /* ; Trim up the position of this ring
    */
    fmedian(pRing,xdim,&Cen,&MySpot,0);

    pR       = pRing;
    StartPix = (int)Cen-CenHalf;
    EndPix   = (int)Cen+CenHalf;
    TotPix   = 1+EndPix-StartPix;
    for(ii=0;ii<xdim;ii++){
	pY    = pYvec;
	pImg  = Img + StartPix*xdim + ii;
	for(jj=StartPix;jj<=EndPix;jj++){
	    *pY = *pImg;
	    pY++;
	    pImg += xdim;
	}
	get_max(pYvec, TotPix , &MyMax, &MySpot);
	*pR = (float)MySpot;               /* Don't add the StartPix offset. */
	pR++;
    }
    /* rng = rng - median(rng) */
    fmedian(pRing,xdim,&Cen,&MySpot,0);


    Vminus (pRing,Cen,xdim);

    /* tot299 = total(calq(*,299))/numaz */
    get_mean_fast( (Img+FixPM1*xdim), xdim, &AvgFixM1 ) ;
    /*
    fprintf(stderr,"AvgFixM1 is %f\n",AvgFixM1);
    */

    EndPix     = ydim - CenRange;
    ovDistance = 1./((float)(ydim-FixP));

    for(ii=FixP;ii<=EndPix;ii++) {

	pW = pWarp;
	pR = pRing;
	for(jj=0;jj<xdim;jj++) {
	    Yspot = xdim * (ii + (int)(*pR));
	    *pW   = *(Img + Yspot + jj);
	    pW++;
	    pR++;
	}
	get_mean_fast( pWarp, xdim, &derf);

	derf -= AvgFixM1 * (float)(ydim-ii) * ovDistance;

	/*
	fprintf(stderr,"derf %d %g  ",ii,derf);
	if((ii+1)%1 == 0 ) fprintf(stderr,"\n");
	*/

	Vminus       ( pWarp, derf,  xdim);

	pW = pWarp;
	pR = pRing;
	for(jj=0;jj<xdim;jj++) {
	    Yspot = xdim * (ii + (int)(*pR));
	    *(FixedImg + Yspot +jj) = *pW;
	    pW++;
	    pR++;
	}
    }

    /* Since the lower part of FixedImg was
       used as free memory, fill in the lower portion of
       the FixedImg with the original Img.
    */
    VVcopy(Img,FixedImg,xdim*3);


}
