;+
; NAME
;	dpmldsqr
;
; PURPOSE
;	Remove limb darkening from PICS disk images.
;	Convert ACOS/DPM images to 800x800 pixels.
;	Rotate image according to P-angle.
;
; SYNTAX
;	dpmldsqr, 'dpmimages.ls'
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   16 Dec 1998
;	Add limb darkening correction.
;-

PRO dpmldsqr, filelist

   nx = 800				; New X dimension.
   ny = 800				; New Y dimension.

   file = ''

   GET_LUN, lun
   OPENR, lun, filelist

   WHILE (NOT EOF (lun)) DO			$
   BEGIN
      READF, lun, file, FORMAT = '(A)'
      PRINT, 'Read file: ', file
      img = readfits (file, hdu)
      img [*,0] = 0				; Set first row to zero.
      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      xcen     = fxpar (hdu, 'CRPIX1')
      ycen     = fxpar (hdu, 'CRPIX2')
      cdelt1   = fxpar (hdu, 'CDELT1')
      cdelt2   = fxpar (hdu, 'CDELT2')
      crradius = fxpar (hdu, 'CRRADIUS')
      pang     = fxpar (hdu, 'SOLAR_P0')
      datamin  = fxpar (hdu, 'DATAMIN')
      datamax  = fxpar (hdu, 'DATAMAX')
      dispmin  = fxpar (hdu, 'DISPMIN')
      dispmax  = fxpar (hdu, 'DISPMAX')
      wavelen  = fxpar (hdu, 'WAVELEN')
      crradius = fxpar (hdu, 'CRRADIUS')

;      fxaddpar, hdu, 'DISPMIN', 0.0
;      fxaddpar, hdu, 'DISPMAX', 0.0

;      fxaddpar, hdu, 'DISPMIN', datamin
;      fxaddpar, hdu, 'DISPMAX', datamax

      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax

      ;--- Remove limb darkening.

      darklimb_correct, img, imgld, lambda=wavelen*10.0,	$
			limbxyr=[xcen,ycen,crradius]
      imgld = FIX (imgld)		; Convert image back to 16-bit integers.

      ;--- Find min/max pixel intensities.

      minld = min (imgld)
      maxld = max (imgld)
      PRINT, 'minld/maxld: ', minld, maxld

      ;--- Extract [nx,ny] pixel sub-image from [1534,1030] image array.

      xbeg = (xcen - nx / 2)
      ybeg = (ycen - ny / 2)
      xcen = (nx / 2.0) - 0.5
      ycen = (ny / 2.0) - 0.5
      imgsq = intarr (nx, ny)
      imgsq = imgld (xbeg : xbeg + nx - 1, ybeg : ybeg + ny - 1)

      ;--- Find min/max pixel intensities.

      minsq = min (imgsq)
      maxsq = max (imgsq)
      PRINT, 'minsq/maxsq: ', minsq, maxsq

      ;--- Rotate image to correct for P-angle (geocentric -> heliocentric).

      nxcen = FIX (xcen + 0.5)
      nycen = FIX (ycen + 0.5)
      mag = 1.0
      roll = pang
      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, INTERP=Cubic)

      ;--- Find min/max pixel intensities.

      minrot = min (imgrot)
      maxrot = max (imgrot)

      ;--- Update FITS header.

      fxaddpar, hdu, 'CRPIX1', xcen
      fxaddpar, hdu, 'CRPIX2', ycen
      fxaddpar, hdu, 'CDELT1', cdelt1 * 2.0
      fxaddpar, hdu, 'CDELT2', cdelt2 * 2.0
      fxaddpar, hdu, 'CRRADIUS', crradius / 2.0
      fxaddpar, hdu, 'DATAMIN', minsq * 1.0
      fxaddpar, hdu, 'DATAMAX', maxsq * 1.0
;      fxaddpar, hdu, 'DISPMIN', 0.0
;      fxaddpar, hdu, 'DISPMAX', 0.0
      fxaddpar, hdu, 'DISPMIN', minsq * 1.0
      fxaddpar, hdu, 'DISPMAX', maxsq * 1.0
      fxaddpar, hdu, 'COMMENT', $
		     ' LIMB DARKENING REMOVED BY DARKLIMB_CORRECT.PRO'

      ;--- Write image to disk file.

      extpos = strpos (file, '.fts')
      outfile = strmid (file, 0, extpos) + '.ldsqr.fts'
      PRINT, 'outfile: ', outfile
      writefits, outfile, imgrot, hdu
   END
   CLOSE, LUN
   
END
