;+
; NAME	fts2jpg512.pro
;
; PURPOSE	Convert FITS images to JPEG format.
;		Output resolution is 512x512 pixels.
;
; SYNTAX	fts2jpg, fits_list
;		fts2jpg, fits_list, color_lut, wmin=0, wmax=100, /label
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   30 January 2001
;		28 Feb 2002: add wmin & wmax.
;		30 Apr 2003: add label keyword.
;		11 Feb 2004: Use congrid to resize input image to 512x512.
;		10 Jan 2005: Default color table changed from 'quallab.lut'
;		             to 'bwcp.lut'.
;-

pro fts2jpg512, fits_list, color_lut, wmin=wmin, wmax=wmax, label=label

month_name = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', $
	      'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
CLOSE, 1
fits_name = ''
xdim_prev = 0
ydim_prev = 0
oxdim = 512			; Output x dimension
oydim = 512			; Output y dimension

red = bindgen (256)
green = red
blue  = red

SET_PLOT, 'Z'
DEVICE, set_resolution = [oxdim, oydim], set_colors = 256, z_buffering = 0

if (N_ELEMENTS (color_lut) GT 0) THEN	$
    lct, color_lut			$
ELSE					$
    lct,'/home/cordyn/color/bwcp.lut'	; color table.

tvlct, red, green, blue, /get

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
print, 'FITS image : ', fits_name

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

;   dispmin = MIN (fits_img, max=dispmax)

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   origin   = fxpar (hdu, 'ORIGIN')
   telescop = fxpar (hdu, 'TELESCOP')
   date     = fxpar (hdu, 'DATE-OBS')
   time     = fxpar (hdu, 'TIME-OBS')
   object   = fxpar (hdu, 'OBJECT')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   coordnam = fxpar (hdu, 'COORDNAM')
   bscale   = fxpar (hdu, 'BSCALE')
   bzero    = fxpar (hdu, 'BZERO')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')
   dispmin = fxpar (hdu, 'DISPMIN')
   dispmax = fxpar (hdu, 'DISPMAX')

   year  = STRMID (date, 0, 4)
   month = STRMID (date, 5, 2)
   day   = STRMID (date, 8, 2)
   iyear = FIX (year)
   syear = STRTRIM (STRING (iyear), 2)
   iday  = FIX (day)
   sday  = STRTRIM (STRING (iday), 2)
   IF (iday LT 10)  THEN sday = ' ' + sday
   imonth = FIX (month)
   smonth = month_name [imonth - 1]
   smonth = STRUPCASE (smonth)
   smonth = STRTRIM (smonth, 2)
   sdate  = sday + ' ' +  smonth + ' ' + syear
;   sdate  = STRTRIM (sdate, 2)

   stime = time + ' UT'

   doy = day_of_year (iyear, imonth, iday)

   IF (N_ELEMENTS (wmin) GT 0) THEN	$
       imin = wmin			$
   ELSE					$
       imin = dispmin

   IF (N_ELEMENTS (wmax) GT 0) THEN	$
       imax = wmax			$
   ELSE					$
       imax = dispmax

;   dispmin = -1000
;   dispmax =  5000
;   print, FORMAT = '("dispmin/max: ", E10.3, E10.3)', dispmin, dispmax

;   print, 'datamin/datamax: ', datamin, datamax
;   print, 'dispmin/dispmax: ', dispmin, dispmax
;   print, 'imin/imax: ',imin, imax

   ;--- Convert to byte pixels.
   ;--- Display image.

;   IF (xdim NE xdim_prev OR ydim NE ydim_prev) THEN	$
;      window, xsize=xdim, ysize=ydim

   byt_img = BYTSCL  (fits_img, min=imin, max=imax, top=249)
   tv_img  = CONGRID (byt_img, oxdim, oydim)		; Resize image.
   TV, tv_img						; Display image.

;   print, 'xdim/ydim: ', xdim, ydim

   ;--- Label image.

   IF (KEYWORD_SET (label)) THEN	$
   BEGIN
      cs = 1.0
;      IF (!version.os EQ 'IRIX')  THEN cs = 0.7
;      IF (!version.os EQ 'sunos') THEN cs = 0.7
;      IF (!version.os EQ 'linux') THEN cs = 0.7

      sorigin   = STRTRIM (origin, 2)
      stelescop = STRTRIM (telescop, 2)
      stype_obs = STRTRIM (type_obs, 2)
      sbunit    = STRTRIM (bunit, 2)
      sobject   = STRTRIM (object, 2)
      sobject   = STRUPCASE (sobject)
      scoordnam = STRTRIM (coordnam, 2)
      clen      = STRLEN  (scoordnam)

      stitle = sorigin + ' ' + stelescop + ' ' + stype_obs 
      stitle = 'MLSO ' + stelescop + ' ' + sbunit
      stitle = 'MLSO ' + stelescop + ' ' + stype_obs
      stitle = 'MLSO ' + stelescop + ' ' + sobject

      sdoy  = STRTRIM (STRING (doy), 2)
      IF (doy LT 100) THEN sdoy = ' ' + sdoy
      IF (doy LT 10)  THEN sdoy = ' ' + sdoy
      labeldoy = 'DOY: ' + sdoy

      swmin = STRTRIM (STRING (imin), 2)
      swmax = STRTRIM (STRING (imax), 2)
      sscale = 'Scaling: ' + swmin + ' to ' + swmax
      dmin = imin * bscale + bzero
      dmax = imax * bscale + bzero
;      sdmin = STRTRIM (STRING (dmin, FORMAT='(G9.2)'), 2)
;      sdmax = STRTRIM (STRING (dmax, FORMAT='(G9.2)'), 2)
      sdmin = STRING (dmin, FORMAT='(F9.2)')
      sdmax = STRING (dmax, FORMAT='(F9.2)')
;      sdmin = STRTRIM (STRING (dmin, FORMAT='(F9.2)'), 2)
;      sdmax = STRTRIM (STRING (dmax, FORMAT='(F9.2)'), 2)
      sdmin = 'MIN: ' + sdmin
      sdmax = 'MAX: ' + sdmax

      XYOUTS, 3,           oydim - 16, stitle, 	 /device, charsize=cs, color=255
      XYOUTS, 3,           oydim - 32, sdate,	 /device, charsize=cs, color=255
      XYOUTS, 3,           oydim - 48, stime,	 /device, charsize=cs, color=255
      XYOUTS, 3,           oydim - 64, labeldoy, /device, charsize=cs, color=255

;      XYOUTS, 3,           5,         sscale,	 /device, charsize=cs, color=255
      XYOUTS, 3,           36,         stype_obs, /device, charsize=cs, color=255
      XYOUTS, 3,           20,         sdmin,    /device, charsize=cs, color=255
      XYOUTS, 3,            4,         sdmax,    /device, charsize=cs, color=255

;     XYOUTS, oxdim/2 - 7, oydim   -20, 'N',     /device, charsize=cs, color=255
;     XYOUTS,  7,          oydim/2 - 7, 'E',     /device, charsize=cs, color=255
;     XYOUTS, oxdim/2 - 7, 10,          'S',     /device, charsize=cs, color=255
;     XYOUTS, oxdim   -20, oydim/2 - 7, 'W',     /device, charsize=cs, color=255
;     XYOUTS, oxdim   -19, 24,          '^',   /device, charsize=cs*2, color=255
;     XYOUTS, oxdim   -18, 18,          '|',     /device, charsize=cs, color=255
;     XYOUTS, oxdim   -20,  4,          'N',     /device, charsize=cs, color=255
      XYOUTS, oxdim - clen*8, 4, scoordnam,     /device, charsize=cs, color=255
   END
   byt_img = TVRD ()

   ;--- Create JPEG image [24-bits/pixel].

;   jpg_img = BYTARR (3, xdim, ydim)
   jpg_img = BYTARR (3, oxdim, oydim)
   jpg_img [0, *, *] = red   [byt_img [*, *]]
   jpg_img [1, *, *] = green [byt_img [*, *]]
   jpg_img [2, *, *] = blue  [byt_img [*, *]]

   ;--- Write jpeg image to disk.

   sloc     = RSTRPOS (fits_name, '.fts')
   jpg_name = STRMID  (fits_name, 0, sloc) + '.jpg'
   print, 'JPEG image : ', jpg_name
   WRITE_JPEG, jpg_name, jpg_img, true=1, quality=75	; Write new image.
   xdim_prev = xdim
   ydim_prev = ydim
END	;}

SET_PLOT, 'x'
CLOSE, 1
END
