;+
; rotdpmsq512.pro   IDL procedure
;
; PURPOSE
;	Read a MLSO/PICS FITS image and rotate it,
;       using CROTA1 and CRPIX1, CRPIX2.
;	Also, crop image to a square.
;
; SYNTAX
;	rotdpmsq3, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   9 October 1997
;	26 May 1999: Update FITS header: COORDNAM = 'HELIOCENTRIC'
;	27 May 1999: Crop image to 1000x1000 pixels (instead of 1030x1030).
;	12 Jan 2000: Output file name uses year-month-day format.
;	23 Jun 2000: Accept DATE-OBS as: DD/MM/YY or CCYY-MM-DD.
;	 6 Sep 2000: Output name includes "CODE-OBS" field from FITS header.
;		     output size is 768x768.
;	15 Nov 2002: Output size is 512x512.
;-

pro rotdpmsq512, fits_list

fits_name = ''				; Define FITS file name variable.
xdim = 0
ydim = 0
xcen = 0
ycen = 0
roll = 0.0
mag  = 1.0

CLOSE, 1
CLOSE, 2

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

year = ''
month = ''
day = ''
hour = ''
minute = ''
second = ''
datefts = ''
timefts = ''

i = 0
WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      dateobs = fxpar (hdu, 'DATE-OBS')

      slash = strpos (dateobs, '/')
      print, 'slash: ', slash
      IF (slash NE -1) THEN			$
         BEGIN
         day   = strmid (dateobs, 0, 2)
         month = strmid (dateobs, 3, 2)
         year  = strmid (dateobs, 6, 2)
         if (year > 70) then year  = '19' + year
         if (year < 50) then year  = '20' + year
         END

      dash = strpos (dateobs, '-')
      print, 'dash: ', dash
      IF (dash NE -1) THEN			$
         BEGIN
         year  = strmid (dateobs, 0, 4)
         month = strmid (dateobs, 5, 2)
         day   = strmid (dateobs, 8, 2)
         END

      datefts = year + month + day
      monthday = month + day
      print, 'dateobs: ', dateobs
      print, 'year, month, day: ', year, month, day
      print, 'datefts: ', datefts

      timeobs = fxpar (hdu, 'TIME-OBS')
      hour    = strmid (timeobs, 0, 2)
      minute  = strmid (timeobs, 3, 2)
      second  = strmid (timeobs, 6, 2)
      timefts = hour + minute + second
      print, 'timeobs: ', timeobs
      print, 'hour, minute, second: ', hour, minute, second
      print, 'timefts: ', timefts

      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')
      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')
      roll = fxpar (hdu, 'SOLAR_P0')
      telescop = STRTRIM (STRLOWCASE (fxpar (hdu, 'TELESCOP')), 2)
      object   = STRTRIM (STRLOWCASE (fxpar (hdu, 'OBJECT')), 2)
      code_obs = STRTRIM (STRLOWCASE (fxpar (hdu, 'CODE-OBS')), 2)
      datamin = fxpar (hdu, 'DATAMIN')
      datamax = fxpar (hdu, 'DATAMAX')

      IF (xcen LT 0 OR xcen GE xdim)  THEN	$
	 BEGIN
	 xcen = (xdim / 2) - 0.5
	 ycen = (ydim / 2) - 0.5
;	    PRINT, 'Invalid center . . . xcen: ', xcen
;	    GOTO, next
         END

      print, 'xdim, ydim: ', xdim, ydim
      print, 'xcen, ycen: ', xcen, ycen
      print, 'roll: ', roll

      ;--- Overwrite binary header line with 1st actual line of data.

      img [*,0] = img [*,1]

      ; -- Copy image into a square array in which sun is centered.

      xl = xcen + 1
      xr = xdim - (xcen + 1)
      yb = ycen + 1
      yt = ydim - (ycen + 1)
      print, 'xl, xr: ', xl, xr
      print, 'yb, yt: ', yb, yt
      xmax = MAX ([xl, xr])
      ymax = MAX ([yb, yt])
      print, 'xmax, ymax: ', xmax, ymax

      nxdim = MAX ([xmax, ymax]) * 2
      nydim = nxdim
      nxcen = nxdim / 2
      nycen = nydim / 2
      print, 'nxdim, nydim: ', nxdim, nydim
      imgsq = INTARR (nxdim, nydim)

      xbeg = nxcen - xl
      xend = xbeg + xdim - 1
      ybeg = nycen - yb
      yend = ybeg + ydim - 1
      print, 'xbeg/xend: ', xbeg, xend
      print, 'ybeg/yend: ', ybeg, yend
      imgsq (xbeg:xend, ybeg:yend) = img (0:xdim-1, 0:ydim-1)

      ;--- Rotate image.

;      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, INTERP=Cubic)
;      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, /CUBIC)
      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, /INTERP)

      sdim = MIN ([xdim, ydim])
      sdim = 1024
      print, 'sdim: ', sdim
      xcen = (sdim / 2) - 1
      ycen = (sdim / 2) - 1
      roll = 0.0
      print, 'new xcen, ycen: ', xcen, ycen
;      print, 'sizeof (imgrot): ', sizeof (imgrot)

      ;--- Crop image.

      imgout = intarr (sdim, sdim)
      xbeg = nxcen - xcen
      xend = xbeg + sdim - 1
      ybeg = nycen - ycen
      yend = ybeg + sdim - 1
      print, 'xbeg/xend: ', xbeg, xend
      print, 'ybeg/yend: ', ybeg, yend
      imgout (0:sdim-1, 0:sdim-1) = imgrot (xbeg:xend, ybeg:yend)

      imgout = rebin (imgout, sdim/2, sdim/2)
      xcen = xcen / 2
      ycen = ycen / 2

      ; -- Modify FITS header.

      xcen = xcen + 0.5
      ycen = ycen + 0.5
      fxaddpar, hdu, 'NAXIS1',   sdim
      fxaddpar, hdu, 'NAXIS2',   sdim
      fxaddpar, hdu, 'CRPIX1',   xcen,       ' SUN CENTER: X AXIS [PIXELS]'
      fxaddpar, hdu, 'CRPIX2',   ycen,       ' SUN CENTER: Y AXIS [PIXELS]'
      fxaddpar, hdu, 'CROTA1',   roll,       ' SOLAR NORTH ANGLE W.R.T. +Y AXIS'
      fxaddpar, hdu, 'CROTA2',   roll,       ' SOLAR NORTH ANGLE W.R.T. +Y AXIS'
      fxaddpar, hdu, 'DISPMIN', datamin, ' MINIMUM DISPLAY VALUE.'
      fxaddpar, hdu, 'DISPMAX', datamax, ' MAXIMUM DISPLAY VALUE.'
      fxaddpar, hdu, 'COORDNAM', 'HELIOCENTRIC', ' SOLAR NORTH ALIGNED WITH +Y AXIS'
;      fxaddpar, hdu, 'HISTORY',  ' IMAGE ROTATED TO ALIGN SOLAR NORTH WITH +Y AXIS.'

      ; -- Write FITS image to disk.
      ; -- Create output file name.

      sloc = RSTRPOS (fits_name, '.fts')
;
rfits_name = strmid (fits_name, 0, sloc) + '.sq.fts'
;      rfits_name = 'mlso_dpmn_asr_' + datefts + '_' + timefts + '.sq.fts'
      rfits_name = year + '.' + monthday + '.' + timefts + '.' +	$
		   telescop + '.' + code_obs + '.sq.fts'
      print, 'rfits_name: ', rfits_name
      writefits, rfits_name, imgout, hdu
next:
   END
CLOSE, 1

END
