;+
; IDL PROCEDURE
;
; NAME:
;	xinsrf
;
; PURPOSE:
;	Animate a sequence of SUN rasterfile images.
;
; CALLING SEQUENCE:
;	xinsrf, 'srf.ls'
;	xinsrf, 'srf.ls', rate=25
;	xinsrf, 'srf.ls', rate=8, /label
;	xinsrf, 'srf.ls', rate=2, /label, cm='~/color/sunset.lut'
;
; INPUTS:
;	listfile : character string containing the names of the SUN rasterfile
;		   images to be animated.
;	rate : animation rate (range: 0-100, default=50).
;	label: if present, label each image with frame #, date, time.
;	cm: name of color map file (ASCII, each line: index, red, green, blue).
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	READ_SRF
;	XINTERANIMATE
;	LCT
;
; MODIFICATION HISTORY:
;	created 7 August 1995, Andrew L. Stanger, HAO/NCAR
;	25 Sep 95 [ALS]: changed rate default from 4 to 100.
;-

pro xinsrf, listfile, rate=rate, label=label, cm=cm

if (not keyword_set (rate)) then rate = 100 

filename = ' '
idate    = ' '
itime    = ' '
ii = 0
winid = 20
;red   = bytarr (256)
;green = bytarr (256)
;blue  = bytarr (256)

; --- Open list file, read filenames, count the number of files in list.

close, 10
openr, 10, listfile

while (not eof (10))  do begin	;{
   readf, 10, filename
   ii = ii + 1
endwhile			;}
numimg = ii

close, 10

;--- Get size of last SUN rasterfile image in list.

read_srf, filename, imgbuf, red, green, blue	; Read last image in listfile.

sizimg = size (imgbuf)


if (sizimg (0) ne 2) then	$
begin
   print, 'size (imgbuf): ', sizimg
   print, '# dimensions: ', sizimg (0)
   print, 'xinsrf only works currently with 8-bit/pixel images.'
   return
end

nxpix  = sizimg (1)
nypix  = sizimg (2)
nbypix = sizimg (3)

;--- If label keyword set, create IDL window in which to display each image.

if (keyword_set (label))  then	$
begin
   window, winid, xsize = nxpix, ysize = nypix, color=256, /pixmap
   wset, winid
end

;--- Load color table.

;print, 'size (red): ', size (red)
sizred = size (red)
if (keyword_set (cm))  then lct, cm	$	; Load specified colormap.
else if (sizred (0) ne 0) then tvlct, red, green, blue	$
else			    loadct, 0		; Load greyscale colormap.
 
;--- Initialize 'xinteranimate'.

init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 

if ( init eq 'y') then begin ;{ 
   XINTERANIMATE, SET = [nxpix+4, nypix+4, numimg], /SHOWLOAD
 
   close, 10 & openr, 10, listfile	; Open file containing list of images.
 
   ii = 0 
   while ( not eof (10) ) do begin ;{
      readf, 10, filename

      read_srf, filename, imgbuf, red, green, blue	; Read image from list.

      imgbuf = reverse (imgbuf, 2)		; Flip image top-to-bottom.
      sizimg = size (imgbuf)
      nxpix  = sizimg (1)
      nypix  = sizimg (2)
      nbypix = sizimg (3)

      if (nbypix eq  1)  then imgbuf = byte ( imgbuf) else $
                              imgbuf = bytscl (imgbuf)		; Scale data.

      if (keyword_set (label))  then $
      begin
         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgbuf
         imglabel = sframe + '     ' + filename
	 print, 'imglabel: ', imglabel
         xyouts, xpos, ypos, imglabel, /device
         XINTERANIMATE, FRAME = ii, WINDOW = !d.window		; Load image
      end				$
      else				$
	 XINTERANIMATE, IMAGE = imgbuf, FRAME = ii		; Load image

      ii = ii+1
   endwhile ;} 
 
   close, 10

endif ;}

wdelete, winid					; Delete temp window.

;--- Activate animation.

XINTERANIMATE, rate 

; --- When you want to stop animating,  hit ^C. 
; --- You might then have to do a "retall" in idl.
 
end 
