/* sephem.c - solar ephemeris calculations.
 *------------------------------------------------------------------------------
 * Performs sun's ra, dec, az, el, p, b0, sradius calculations
 *------------------------------------------------------------------------------
 * SEE ALSO: astrotime.c for time calculations, jd, and last for unix commands
 * and astro.h for miscellaneous definitions.
 *------------------------------------------------------------------------------
 * Author: David Elmore      HAO/NCAR    3 Aug 1998   [solar.c/solar_filter.c]
 * Updates: Andrew Stanger   HAO/NCAR
 * 16 Feb 2000: Convert solar_filter.c [main program] to a subroutine [spehem.c]
 *  3 Dec 2002: Add solar_ephem: combination of azel & solarEphm.
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include "astro.h"

int
sephem (jd, hh, mm, ss,
	az, el, sradius, pang, b0, l0, lst, ra, dec, ha, airmass)

int jd ;		/* Julian date. */
int hh ;		/* Hours.       */
int mm ;		/* Minutes.     */
int ss ;		/* Seconds.     */

double *az ;		/* Azimuth. */
double *el ;		/* Elevation. */
double *sradius ;	/* Solar Radius. */
double *pang ;		/* P-angle. */
double *b0 ;		/* B-angle. */
double *l0 ;		/* Carrington Longitude. */
double *lst ;		/* Siderial Time. */
double *ra ;		/* Right Ascension. */
double *dec ;		/* Declination. */
double *ha ;		/* */
double *airmass ;	/* Airmass. */

{
    long gmt ;
    double Az, El, Sradius, P, B0, L0, Last, Ra, Dec, Ha, Airmass ;

    gmt = hmstosec ((long) hh, (long) mm, (long) ss) ;

/*
 *    fprintf (stderr, "jd: %d\n", jd) ;
 *    fprintf (stderr, "hh mm ss: %2d %2d %2d\n", hh, mm, ss) ;
 */

    /*** Compute siderial time. ***/

    Last = last (jd, gmt) ;				/* date -> radians */
    Last = fr2d (Last) * 24.0 / 360.0 ;			/* radians -> hours */
    Last = fmod (fmod (Last, 24.0) + 24.0, 24.0) ;	/* positive angle */

    /* Compute Azimuth, Elevation, Solar Radiuis, P-angle, B-angle, & Long. */

/*
 *  azel ((long) jd, (long) gmt, &Az, &El, &Sradius, &P, &B0, &L0) ;
 */

    /* Compute Right Ascension, Solar Radius, P-angle, B-angle, Longitude. */

/*
 *  solarEphm ((long) jd, (long) gmt, &Ra, &Dec, &Sradius, &P, &B0, &L0) ;
 */

    /* Compute Right Ascension, Declination, Azimuth, Elevation.
     *         Solar Radius, P-angle, B-angle, Longitude.
     */

    solar_ephem ((long) jd, (long) gmt, &Ra, &Dec, &Az, &El,
					  &Sradius, &P, &B0, &L0) ;

    /*** Convert to appropriate units. ***/

    Az = fr2d (Az) ;				/* Degrees. */
    if ( Az < 0.0 )  Az += 360.0 ;		/* Positive degrees. */

/*
 *    fprintf (stdout, "gmt %lg hrs, last %lg hrs, Az %lg deg., El %lg deg.\n", 
 *	    (double) gmt / 3600.0, Last, Az, fr2d (El)) ;
 *    fprintf (stdout, "sradius %lg, P %lg, B0 %lg, L0 %lg\n", 
 *	    fr2d (Sradius) * 3600.0, fr2d (P), fr2d (B0), fr2d (L0)) ;
 */

    Ra = fr2d (Ra) * 24.0 / 360.0 ;		/* Degrees -> Hours */
    Ra = fmod (fmod (Ra, 24.0) + 24.0, 24.0) ;	/* Positive angle */
    Dec = fr2d (Dec) ;
    Dec = fmod (Dec, 360.0) ;			/* Positive angle */
    Ha = (Last - Ra) * 360.0 / 24.0 ;
    Airmass = 1.0 / cos (1.5708-El) ;

/*
 *    fprintf (stdout, "Ra %lg hours., Dec %lg deg. H.A. %lg  Airmass %lg\n", 
 *	    Ra, Dec, Ha, Airmass ) ;
 */

/*
 *    fprintf ( stdout, "Bench %lg deg.\n", Ha+fr2d(P) ) ;
 */

    /*** Copy to output variables. ***/

    *az = Az ;
    *el = fr2d (El) ;				/* Degrees. */
    *sradius = fr2d (Sradius) * 3600.0 ;	/* Arcseconds. */
    *pang = fr2d (P) ;				/* Degrees. */
    *b0 = fr2d (B0) ;				/* Degrees. */
    *l0 = fr2d (L0) ;				/* Degrees. */
    if (*l0 < 0.0) *l0 += 360.0 ;		/* Positive degrees. */
    *lst = Last ;
    *ra = Ra ;
    *dec = Dec ;
    *ha = Ha ;
    *airmass = Airmass ;
    
    return (0) ;
 }
