#include <stdio.h>
#ifdef	VxWorks
#include "vxWorks.h"
#include	<math.h>
#include	"time.h"
#include	"astro.h"
#else
#include	"astro.h"
#endif
/*
 * solarEphm.c - ephemeris routines
 *
 * 	azel(day, gmt, Az, El, Sdiam, P, B0, L0)
 * 		long day = julian day
 * 		long gmt = UT time in seconds
 *	 	double *Az, *El = solar az, el in radians.
 *	 	double *Sdiam = solar diameter in radians.
 * 		*P, *B0 = P angle, B angle in radians.
 *	 	double *L0 = carrington longitude in radians.
 *
 *	solarEphm(day, gmt, Ra, Dec, Sdiam, P, B0, L0)
 *		Same arguments as azel, except Ra, Dec in radians
 *
 * These routines return the
 * current position in the sky, solar diameter,
 * P, B0, and L0.  These routines are built on
 * the sun_eph.c, abcd_lat.c, and astrotime.c
 * libraries and astro.h.
 *
 * This package calculates the ephemerides for 12h ut on the
 * current day, and day + 1, and interpolates between the points.
 * The LOCAL notion of day is kept, and if it changes, the points
 * are recalculated.  The time resolution is set by 
 * TICKS_PER_SEC in astro.h, and it is currently 1 second.  
 */

/*
 * declare locals:
 */
#define	slope(x, y)	(frem(((x)-(y)),2.*PI))

/*
 * reference values set for interpolation in a 24 hour period
 */
LOCAL long     Day = 0;
LOCAL long     ut0 = ONE_DAY / 2;	/* reference time = 12h UT on date Day */
LOCAL ANGLE    ra, dra, dec, ddec, sradius, dsradius, p, dp, b0, db0, l0, dl0;

/*
 * for this day amd gmt, return Az, El, ... for sun
 */
azel(day, gmt, Az, El, Sdiam, P, B0, L0)
long day, gmt;
ANGLE *Az, *El, *Sdiam, *P, *B0, *L0;
{				/* astro entry point */
	ANGLE Ra, Dec;
	if(day != Day)
		initSolar((long)day);	/* init solarEphm coordinate computations */
	do_coord((long)gmt,&Ra,&Dec, Sdiam, P, B0, L0);/* interp. values */
	rd2ae(gmt,Ra,Dec,Az,El);
}				/* end: astro main */

/*
 * for this day amd gmt, return Ra, Dec, ... for sun
 */
solarEphm(day, gmt, Ra, Dec, Sdiam, P, B0, L0)	/* ref. coordinates */
long day, gmt;
ANGLE *Ra, *Dec, *Sdiam, *P, *B0, *L0;
{				/* astro entry point */
	if(day != Day)
		initSolar(day);	/* initialize solarEphm coordinate computations */
	do_coord(gmt,Ra, Dec, Sdiam, P, B0, L0);/* interp. values */
}				/* end: astro main */

/*
 * rd2ae() - ra, dec to az el conversion
 */
LOCAL rd2ae(gmt,Ra,Dec,Az,El)
long gmt;
ANGLE Ra, Dec;
ANGLE *Az, *El;
{
	ANGLE ha, beta;

	ha = last(Day,gmt) - Ra;
	abcd_lat(ha,Dec,Az,El,&beta);	/* general spherical trig rotation */
 }

/*
 * initSolar() 
 *
 * initialize coordinate calculations This should be done more often to minimize
 * errors.  arg = FALSE, init for solarEphm, else use ra/dec from input. 
 */
LOCAL initSolar(day)
int day;
{
	FAST ANGLE      Tu;
	ut0 = ONE_DAY / 2;	/* reference time is 12h UT on date Day */
	/* call this anyway, for p, b0 values */
	Day = abs(day) > 10000 ? day - JD_EPOCH : day;
	sun_eph(Day, ut0, &ra, &dec, &sradius, &p, &b0, &l0);	/* reference coordinates */
	sun_eph(Day + 1, ut0, &dra, &ddec, &dsradius, &dp, &db0, &dl0);	/* next day's coords. */

		dra = slope(dra, ra);	/* prepare for interpolation */
		ddec = slope(ddec, dec);
		dsradius = slope(dsradius, sradius);
		dp = slope(dp, p);
		db0 = slope(db0, b0);
		dl0 = slope(dl0, l0);
}				/* end: intcoord */

/*
 * do_coord() 
 */
LOCAL int
do_coord(ut,Ra,Dec,Sdiam,P,B0,L0)	/* do the coordinate calculations
				   for given time: TICKS since ref. Day */
FAST long ut;
ANGLE *Ra, *Dec, *Sdiam, *P, *B0, *L0;
{
	FAST ANGLE      t;
	t = (ut - ut0) /	/* time in TICKS since reference date, time */
		((ANGLE) ONE_DAY);	/* fraction of day since reference
					 * date/time */
	*Ra = dra * t + ra;	/* do each coordinate axis interpolation */
	*Dec = ddec * t + dec;
	*Sdiam = dsradius * t + sradius;
	*P = dp * t + p;
	*B0 = db0 * t + b0;
	*L0 = fmod((dl0 * t + l0), 2 * PI);	/* L0 is a very large
						 * negative number */
}				/* end: do_coord */
