/* solar_eph.c - ephemeris routines
 *------------------------------------------------------------------------------
 * azel (day, gmt, Az, El, Sdiam, P, B0, L0)
 * 	long day = julian day
 * 	long gmt = UT time in seconds
 * 	double *Az, *El = solar az, el in radians.
 * 	double *Sdiam = solar diameter in radians.
 *	*P, *B0 = P angle, B angle in radians.
 * 	double *L0 = carrington longitude in radians.
 *
 * solarEphm (day, gmt, Ra, Dec, Sdiam, P, B0, L0)
 *	Same arguments as azel, except Ra, Dec in radians
 *
 * These routines return the current position in the sky, solar diameter,
 * P, B0, and L0.  These routines are built on the sun_eph.c, abcd_lat.c,
 * and astrotime.c libraries and astro.h.
 *
 * This package calculates the ephemerides for 12h ut on the current day,
 * and day + 1, and interpolates between the points.
 * The static notion of day is kept, and if it changes, the points
 * are recalculated.  The time resolution is set by TICKS_PER_SEC
 * in astro.h, and it is currently 1 second.  
 *------------------------------------------------------------------------------
 * Author:  David Elmore     HAO/NCAR    3 Aug 1998
 * Updates: Andrew Stanger   HAO/NCAR
 * 26 Nov 2002: Cosmetic changes.  Remove VxWorks directives.
 *  3 Dec 2002: Add "#include <stdio.h>".
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include "astro.h"

/*** Declare locals: ***/

#define	slope(x, y)	(frem(((x)-(y)),2.0*PI))

/*** Reference values set for interpolation in a 24 hour period. ***/

long   Day = 0;
long   ut0 = ONE_DAY / 2;   /* Reference time = 12h UT on date Day */
double ra, dra, dec, ddec, sradius, dsradius, p, dp, b0, db0, l0, dl0;

/* -------------------------------------------------------------------------- */
/* Given day & gmt, return Azimuth, Elevation,
 *                         Solar radius, P-angle, B-angle, and Longitude.
 */

azel (day, gmt, Az, El, Sdiam, P, B0, L0)

long day, gmt ;
double *Az, *El, *Sdiam, *P, *B0, *L0 ;
{
    double Ra, Dec ;
    if (day != Day)
	initSolar (day) ;   /* Initialize solarEphm coordinate computations. */

    do_coord (gmt, &Ra, &Dec, Sdiam, P, B0, L0) ;	/* Interp. values. */

    rd2ae (gmt, Ra, Dec, Az, El) ;

    return (0) ;
}

/* -------------------------------------------------------------------------- */
/* Given day & gmt, return Right Ascension, Declination,
 *                         Solar radius, P-angle, B-angle, and Longitude.
 */

solarEphm ( day, gmt, Ra, Dec, Sdiam, P, B0, L0)	/* Reference coords. */

long day, gmt ;
double *Ra, *Dec, *Sdiam, *P, *B0, *L0 ;
{
    if (day != Day)
	initSolar (day) ;   /* Initialize solarEphm coordinate computations. */

    do_coord (gmt, Ra, Dec, Sdiam, P, B0, L0) ;		/* Interp. values. */

    return (0) ;
}

/* -------------------------------------------------------------------------- */

/* For this day and gmt, return Ra, Dec, Az, El, Sdiam, P, B0, L0 for sun. */

solar_ephem (day, gmt, Ra, Dec, Az, El, Sdiam, P, B0, L0)

long day, gmt;
double *Ra, *Dec, *Az, *El, *Sdiam, *P, *B0, *L0;
{

    if (day != Day)
	initSolar (day);   /* Initialize solar_ephem coordinate computations. */

    do_coord (gmt, Ra, Dec, Sdiam, P, B0, L0);	/* interp. values */

/*
 *    printf ("solar_ephem> Ra: %9.2e  Dec: %9.2e\n", *Ra, *Dec) ;
 */

    rd2ae (gmt, *Ra, *Dec, Az, El);

    return (0) ;
}

/* -------------------------------------------------------------------------- */

/* rd2ae () : [ra, dec] to [az, el] conversion.  */

rd2ae (gmt, Ra, Dec, Az, El)

long gmt;
double Ra, Dec;
double *Az, *El;
{
    double ha, beta;

    ha = last (Day, gmt) - Ra;

    abcd_lat (ha, Dec, Az, El, &beta);	/* General spherical trig rotation. */

    return (0) ;
 }

/* -------------------------------------------------------------------------- */

/* initSolar ()
 *
 * Initialize coordinate calculations.
 * This should be done more often to minimize errors.
 * arg = FALSE, init for solarEphm, else use ra/dec from input. 
 */

initSolar (day)

int day;
{
    extern double dra, ddec, dsradius, dp, db0, dl0 ;
    FAST double      Tu;
    ut0 = ONE_DAY / 2;		/* Reference time is 12h UT on date Day */

    /*** Call this anyway, for p, b0 values. ***/

    Day = abs (day) > 10000 ? day - JD_EPOCH : day;

    /*** Reference coordinates ***/

    sun_eph (Day, ut0, &ra, &dec, &sradius, &p, &b0, &l0);

    /*** Next day's coordinates. ***/

    sun_eph (Day + 1, ut0, &dra, &ddec, &dsradius, &dp, &db0, &dl0);

    /*** Compute slopes for interpolation. ***/

    dra      = slope (dra,      ra);
    ddec     = slope (ddec,     dec);
    dsradius = slope (dsradius, sradius);
    dp       = slope (dp,       p);
    db0      = slope (db0,      b0);
    dl0      = slope (dl0,      l0);

/*
    printf ("initSolar> Day:      %d\n",    Day) ;
    printf ("initSolar> day:      %d\n",    day) ;
    printf ("initSolar> ra:       %9.2e\n", ra) ;
    printf ("initSolar> dec:      %9.2e\n", dec) ;
    printf ("initSolar> sradius:  %9.2e\n", sradius) ;
    printf ("initSolar> p:        %9.2e\n", p) ;
    printf ("initSolar> b0:       %9.2e\n", b0) ;
    printf ("initSolar> l0:       %9.2e\n", l0) ;
    printf ("initSolar> dra:      %9.2e\n", dra) ;
    printf ("initSolar> ddec:     %9.2e\n", ddec) ;
    printf ("initSolar> dsradius: %9.2e\n", dsradius) ;
    printf ("initSolar> dp:       %9.2e\n", dp) ;
    printf ("initSolar> db0:      %9.2e\n", db0) ;
    printf ("initSolar> dl0:      %9.2e\n", dl0) ;
 */

    return (0) ;
}

/* -------------------------------------------------------------------------- */

/* do_coord () */
/* Do the coordinate calculations for given time: TICKS since ref. Day */

int
do_coord (ut, Ra, Dec, Sdiam, P, B0, L0)

FAST long ut;
double *Ra, *Dec, *Sdiam, *P, *B0, *L0;
{
    FAST double      t;

    t = (ut - ut0) /		/* Time in TICKS since reference date, time   */
	((double) ONE_DAY);	/* fraction of day since reference date/time. */

/*
    printf ("do_coord> dra: %9.2e  t: %9.2e  ra: %9.2e\n", dra, t, ra) ;
    printf ("do_coord> ddec: %9.2e  dec: %9.2e\n", ddec, dec) ;
    printf ("do_coord> dra * t + ra: %7.2f  ddec * t + dec: %7.2f\n",
	     dra * t + ra, ddec * t + dec) ;
    printf ("do_coord> Ra: %9.2e  Dec: %9.2e\n", *Ra, *Dec) ;
 */

    /*** Do linear interpolation for each parameter. ***/

    *Ra    = dra      * t + ra;
    *Dec   = ddec     * t + dec;
    *Sdiam = dsradius * t + sradius;
    *P     = dp       * t + p;
    *B0    = db0      * t + b0;
    *L0  = fmod ((dl0 * t + l0), 2 * PI);	/* L0 is a very large
						 * negative number */
    return (0) ;
}

/* -------------------------------------------------------------------------- */
