/* sun_eph.c	sun's coordinates
 * -----------------------------------------------------------------------------
 * 13 FEB 85: phw
 * 21 FEB 86: phw	 move def of fd2r to math.h
 * -----------------------------------------------------------------------------
 * Author:  David Elmore    HAO/NCAR   3 Aug 1998
 * Updates: Andrew Stanger  HAO/NCAR
 *  3 Dec 2002: Remove VxWorks directive; add "#include <stdio.h>".
 * -----------------------------------------------------------------------------
 */
#include <stdio.h>
#include "astro.h"

sun_eph (nday, ut, ra,dec, diam, p, b0,l0)

long	nday;				/* days since EPOCH */
long	ut;				/* and TICKS into day 'n' */
ANGLE	*ra, *dec;			/* sun's R.A. and Dec */ 
ANGLE	*diam;				/* solar semi-diameter, as angle */
ANGLE	*p;				/* position angle of solar pole */
ANGLE	*b0, *l0;			/* center disk heliographic coords. */
{
#define	sin_I	((AMP) (0.126198969))	/* inclination of solar equator */
#define cos_I	((AMP) (0.992004950))	/* is 7.25 degrees */
#define	J1854	2398220			/* julian date for 1854 Jan 1.5 */

  ANGLE	L, g, lambda, xi, two_g;
  AMP	sin_l, cos_l, sin_xi, cos_xi;
  ANGLE	Omega, LmO;
  AMP	sin_LmO, cos_LmO, r, temp;

  /*** Low precision method from "The Astronomical Almanac", 1984, p C24. ***/

  /* aberration corrected mean long*/

  L = fd2r (280.460) + fd2r (0.9856474) * nday;

  /* mean anomaly */

  g = fd2r (357.528) + fd2r (0.9856003) * nday;
  two_g = 2 * g;

  lambda = L + fd2r (1.915) * SIN (g)
	     + fd2r (0.02) * SIN (two_g);	/* Ecliptic longitude */
  sin_l = SIN (lambda);
  cos_l = COS (lambda);

  xi = fd2r (23.439) - fd2r (0.0000004) * nday;	/* Obliquity of ecliptic */
  sin_xi = SIN (xi);
  cos_xi = COS (xi);

  /*** Right Ascension & Declination. ***/

  *ra  = ATAN2 (cos_xi * sin_l, cos_l);		/* Right ascension */
  temp = sin_xi * sin_l;
  *dec = ASIN (temp);				/* Declination */

  /*** Solar radius. ***/

  r = 0.00014 - 0.01671 * COS (g)
    - 0.00014 * COS (two_g);		/* R = earth/sun distance = 1 + r */
					/* 1/(1+r) = 1 - r + r**2 - r**3 ... */
  *diam = fd2r (0.2666) * (1.0 / (1.0 + r)); /* Solar semi-diameter, as ANGLE */

  /* Method from "Explanatory Supplement the Astronomical Ephemeris",
   * 1961, pp 307-310.
   * Omega is longitude of solar equator's ascending node.
   */

  Omega = fd2r (73.666667)				/* epoch of 1850 */
	+ fd2r ((EPOCH - 1850.0) * 50.25 / 3600.)	/* current EPOCH */
	+ fd2r (50.0 * 50.25 / 3600) * (AMP) (nday * 2) / 36525.0;	

  /* Change   in 1/2 century. */
  /* Fraction of 1/2 century. */

  LmO     = lambda - Omega;
  sin_LmO = SIN (LmO);
  cos_LmO = COS (LmO);

  *p = ATAN2 (-cos_l   * sin_xi, cos_xi)
     + ATAN2 (-cos_LmO * sin_I,  cos_I);

  /*** Latitude center of sun ***/

  temp = sin_LmO * sin_I;
  *b0  = ASIN (temp);

  /*** Carrington longitude ***/

  *l0 = ATAN2 (sin_LmO * cos_I, cos_LmO)
      + fd2r ((J1854 - JD_EPOCH) * 360.0 / 25.38 + 180)
      - fd2r (360.0 / 25.38) * nday;

/*#ifdef	DEBUG0 */
#define	DEGREES(x)	(((double) x)*180/PI)
  printf ("\n") ;
  printf ("L       =%12.4f\ng       =%12.4f\nlambda  =%12.4f\nxi      =%12.4f\n",
	 fr2d (L), fr2d (g), fr2d (lambda), fr2d (xi));
  printf ("Omega   =%12.4f\nLmO     =%12.4f\nsin_LmO =%f\n",
	 fr2d (Omega), fr2d (LmO), ((double) sin_LmO));
  printf ("\n") ;
/*#endif */

  return (0) ;
  }
