/* atar.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Special version of `tar' to handle multiple tar files on a storage medium.
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Option to write a volume (medium) name into the user file header record.
 * Option to write a `tar file name' (dataset) into the user file header record.
 * Option to produce only a `tar file' listing (tar file names).
 * Option to add new tar file at EOT.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Compile with -DSGI on Silicon Graphics machines
 *
 *
 * Modified by Andrew L. Stanger
 * High Altitude Observatory / National Center for Atmospheric Research
 * 15 June 1988: add `e' option: write volume name into header record.
 *               add `s' option: write dataset name into header record.
 *               add `d' option: do directory of only volume & dataset names.
 * 20 June 1988: add `g' option: go to specified dataset before extracting data.
 * 5 October 1988: allow g option to be used for directory listings.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Modifed by Ray Bovet
 * High Altitude Observatory / National Center for Atmospheric Research
 * 27 February 1989: add `D' option: write out crc32 checksum of data.
 * 28 February 1989; replace references to umt library with direct usage of
 *      mt ioctl calls.  Make sure we are dealing with a true tape
 *      before attempting these.
 * 10 March 1989: added 'T' option to request list info for all tarfiles
 *      on an entire tape!
 * 23 May 1989: fixed tarlist and dotable to avoid unnecessary gethdr and
 *      bsr combination which slowed things down.  Also made major changes
 *      to handling of blocksize so that we start over with the default
 *      or requested size at the beginning of each tarfile and so that the
 *      default blocksize on reads is 126 rather than 20.
 * 28 July 1989: added new Sun MTBSFM operations and try to do them first
 *      instead of the normal MTBSF since Sun has redefined what that does.
 *  6 June 1990: Sun reversed the meaning of BSF back to the way it was
 *	before 4.0.3 (BSF == Back Space Filemark).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*
 * Copyright (c) 1980 Regents of the University of California.
 */

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1980 Regents of the University of California.\n\
 All rights reserved.\n";
#endif not lint

#ifndef lint
static char sccsid[] = "@(#)atar.c	5.7 (Berkeley) 4/26/86";
#endif not lint

#ifndef lint
static char rcsid[] = "$Header: /hao/src/atar/RCS/atar.c,v 1.80 1993/04/06 20:15:21 bovet Exp $";
#endif not lint

/*
 ******* Tape Archival Program *******
 */
#include <stdio.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/dir.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>
#include <sys/time.h>
#include <sys/file.h>       /* makes the VAX happy! */
#include <signal.h>
#include <errno.h>
#include <fcntl.h>
#include "rtape_lib.h"
#include "os_type.h"

/*
 * Now that we can do tape operations remotely, we may have to force a local
 * definition for MTNBSF when the host doesn't know about it but the remote
 * does.
 */
#define TEMPORARY_MTNBSF  11
#ifdef MTNBSF
#if MTNBSF - TEMPORARY_MTNBSF
/* this is just to make sure our definition of MTNBSF is correct!! */
hey boss, the definition of MTNBSF is wrong - better fix it
/* Note that the above line is *not* a comment but rather a line that is
 * carefully designed to provoke a compiler error!
 */
#endif MTNBSF - TEMPORARY_MTNBSF
#else MTNBSF
#define MTNBSF TEMPORARY_MTNBSF
#endif MTNBSF

#define TBLOCK	512
#define WNBLOCK 20
#define RNBLOCK 126
#define NAMSIZ	100
#define SGI_BLKSIZE 4096

#define DEBUGTAPE 1
#define DEBUGFILENAMES 2
#define DEBUGREMOTE 4

#define	writetape(b)	writetbuf(b, 1)
#define	min(a,b)  ((a) < (b) ? (a) : (b))
#define	max(a,b)  ((a) > (b) ? (a) : (b))

union hblock
   {
   char dummy[TBLOCK];
      struct header
	 {
         char name[NAMSIZ];
         char mode[8];
         char uid[8];
         char gid[8];
         char size[12];
         char mtime[12];       /* also used to hold crc32 in blank header */
         char chksum[8];
         char linkflag;
         char linkname[NAMSIZ];
         char save1[182];
	 char crcflg;
         char sname[32];
         char ename[32];
         char save2[8];
         } dbuf;
   };

struct linkbuf
   {
   ino_t	inum;
   dev_t	devnum;
   int		count;
   char		pathname[NAMSIZ];
   struct	linkbuf *nextp;
   };

union	hblock dblock;
union	hblock *tbuf;
struct	linkbuf *ihead = NULL;
struct	stat stbuf;

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * NOTE: flags do not necessarily correspond exactly to option letters
 *       specified on the `atar' command argument line.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * In particular:
 *    r option => append specified user files to the last dataset file
 *                prior to EOT.
 *    c option => write user files to a newly created dataset file.
 *    a option => write user files to a newly created dataset file at EOT.
 *
 *    r flag => write user files to tape.
 *    c flag => create a new dataset file.
 *    a flag => write user files beginning at EOT.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Certain options are invoked only if associated options are selected.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * For instance:
 *    e option (specify volume name) is relevant only when used with the
 *        c option.
 *    s option (specify dataset name) is relevant only when used with the
 *        c or a option.
 *    g option (go to dataset specified after`-g' before extracting data)
 *        is relevant only when used with the x or t options.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int	rflag;	/* Write new user files to tape. */
int	xflag;	/* Extract user files from tape and copy to disk. */
int	vflag;	/* Verbose mode. */
int	tflag;	/* List all names of user files contained in atar file(s). */
int	cflag;	/* Create new atar dataset file. */
int	mflag;	/* Do NOT restore modification times of extracted files. */
int	fflag;	/* Specify tape device name in next command line argument. */
int	iflag;	/* Ignore directory checksum errors. */
int	oflag;	/* Suppress information specifying owner & directory modes. */
int	pflag;	/* Restore named files to their original modes. */
int	wflag;	/* Wait for user confirmation before taking specified action. */
int	hflag;	/* Follow symbolic links as if they were normal files. */
int	Bflag;	/* Perform multiple reads if needed to fill a block. */
int	Fflag;	/* Exclude all directories named SCCS from "tarfile". */
int	dflag;	/* Suppress user file listing; report only volume & dataset.*/
int	eflag;	/* Write volume name into header record. */
int	sflag;	/* Write dataset group name into header record. */
int	aflag;	/* Add new user files at EOT. */
int	gflag;	/* Go to specified dataset before extracting user files. */
int     Dflag;  /* Write and process crc32 data */
int     Tflag;  /* Do list of entire tape, not just this tarfile. */
int     debugmask;  /* Specify debug options, 1 <=> tape motion, 2 <=> filenames, 4 <=> remotetape & hosttype */

int taperrok = 0;
int	mt;
int	term;
int	chksum;
int	recno = 0;
int	first = 0;
int     bufset = 0;
int	prtlinkerr;
int	freemem = 1;
int	nblock = 0;
int     reqnblock = 0;
int     defnblock = 0;
int	onintr ();
int	onquit ();
int	onhup ();
#ifdef notdef
int	onterm ();
#endif
unsigned long int     crcsum, crcver, crc32buf();
int     istape, rfsdsize;                  /* will be true for real (reel) tape   */
int     isremote;                          /* true if tape drive is remote */
char    remote_system[64];
char    remote_login[64];
char    remote_device[64];
int     os_type;
daddr_t	low;
daddr_t	high;
daddr_t	bsrch ();

FILE	*vfile = stdout;
FILE	*tfile;
char	tname[] = "/tmp/atarXXXXXX";
char	*usefile;
char	magtape[] = "/dev/rmt8";
char	ename[32] = "volume";
char	sname[32] = "datset"; 
char	*malloc ();
char    *realloc ();
long	time ();
off_t	lseek ();
char	*mktemp ();
char	*strcat ();
char	*strcpy ();
char	*rindex ();
char	*getcwd ();
char	*getwd ();
char	*getmem ();

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
main (argc, argv)

int	argc;
char	*argv[];
{
   char *cp;			/* Argument pointer. */
   int ierr;			/* Error value. */

   if (argc < 2)
      usage ();


   tfile = NULL;
   usefile =  magtape;
   argv[argc] = 0;
   argv++;
   for (cp = *argv++; *cp; cp++) 
      switch (*cp)
      {

      case 'f':
         if (*argv == 0)
            {
            fprintf (stderr,
            "atar: tapefile must be specified with 'f' option\n");
            usage ();
            }
         usefile = *argv++;
         fflag++;
         break;

      case 'c':
         cflag++;
         rflag++;
         break;

      case 'o':
         oflag++;
         break;

      case 'p':
         pflag++;
         break;
		
      case 'u':
         mktemp (tname);
         if ((tfile = fopen (tname, "w")) == NULL)
            {
            fprintf (stderr,"atar: cannot create temporary file (%s)\n", tname);
            done (1);
            }
         fprintf (tfile, "!!!!!/!/!/!/!/!/!/! 000\n");
         /*FALL THRU*/

      case 'r':
	 fprintf (stderr,"atar warning: this option not feasible on Exabyte drive or 1/4 inch cartridge tape!\n");
         rflag++;
         aflag++;
         break;

      case 'v':
         vflag++;
         break;

      case 'w':
         wflag++;
         break;

      case 'x':
         xflag++;
         break;

      case 't':
         tflag++;
         break;

      case 'm':
         mflag++;
         break;

      case '-':
         break;

      case '0':
      case '1':
      case '4':
      case '5':
      case '7':
      case '8':
         magtape[8] = *cp;
         usefile = magtape;
         break;

      case 'b':
         if (*argv == 0)
            {
            fprintf (stderr,
	       "atar: blocksize must be specified with 'b' option\n");
            usage ();
            }
         reqnblock = atoi (*argv);
         if (reqnblock <= 0)
            {
            fprintf (stderr, "atar: invalid blocksize \"%s\"\n", *argv);
            done (1);
            }
	 if (reqnblock > 126)
	    {
	    fprintf (stderr, "atar: blocksize > 126 not allowed : %s\n", *argv);
	    done (1);
	    }
         argv++;
         break;

      case 'l':
         prtlinkerr++;
         break;

      case 'h':
         hflag++;
         break;

      case 'i':
         iflag++;
         break;

      case 'B':
         Bflag++;
         break;

      case 'F':
         Fflag++;
         break;

      case 'd':		/* Suppress user file listing. */
         dflag++;
         tflag++;
	 Tflag++;       /* Implies entire tape */
         break;

     case 'e':		/* Set flag to write medium name. */
         if (*argv == 0)
            {
            fprintf (stderr,"atar: volume name must follow 'e' option flag.\n");
            usage ();
            done (1);
            }
         eflag++;
         strcpy (ename, *argv);
         argv++;
         break;
			
      case 's':		/* Set flag to write dataset name. */
         if (*argv == 0)
            {
            fprintf (stderr,
	       "atar: dataset name must follow 's' option flag.\n");
            usage ();
            done (1);
            }
         sflag++;
         strcpy (sname, *argv);
         argv++;
         break;

      case 'a':		/* Add new dataset group file at EOT. */
         aflag++;
         cflag++;
         rflag++;
         break;

      case 'g':		/* Go to specified dataset name. */
         if (*argv == 0)
            {
            fprintf (stderr,
               "atar: dataset name must follow 'g' option flag.\n");
            usage ();
            done (1);
            }
         gflag++;
         strcpy (sname, *argv);
         argv++;
         break;

      case 'D':        /* Write crc32 info to tape and process it on read*/
	 Dflag++;
	 break;

      case 'T':       /* Do list of entire tape, not just this tarfile */
	 Tflag++;
	 tflag++;
	 break;

      case 'G':       /* Specify debug options */
         if (*argv == 0)
            {
            fprintf (stderr,
	       "atar: debug value must be specified with 'G' option\n");
            usage ();
            }
         debugmask = atoi (*argv);
	 argv++;
         break;

      default:
         fprintf (stderr, "atar: %c: unknown option\n", *cp);
         usage ();
      }   /* End of switch statement. */

   eflag++;	/* Store either a blank or supplied volume name. */
   sflag++;	/* Store either a blank or supplied dataset name. */

   if (!rflag && !xflag && !tflag)
     {
       usage ();
     }

/* It's too hard to do the D option when appending to an existing tarfile! */
   if (rflag && aflag && !cflag && Dflag)
     {
       fprintf (stderr, "D option incompatible with r or u option, ignored.\n");
       Dflag = 0;
     }

   if (Tflag && gflag)
     {
       fprintf (stderr, "g option incompatible with T option, ignored.\n");
       gflag = 0;
     }

/* Time to do a little housekeeping, like figuring out whether they are local
 * or remote, and which OS we have to deal with in the tape commands.
 */

   isremote = (index (usefile, ':') != NULL);
   if (debugmask & DEBUGREMOTE) fprintf (stderr, "Isremote : %d\n", isremote);
   if (isremote) {
     __rmt_parse (usefile, remote_system, remote_login, remote_device);
     if (debugmask & DEBUGREMOTE) {
       fprintf (stderr, "Remote_system : %s\n", remote_system);
       fprintf (stderr, "Remote_login : %s\n", remote_login);
       fprintf (stderr, "Remote_device : %s\n", remote_device);
     }
   }
   os_type = get_os_type (isremote, remote_system, remote_login);
   if (debugmask & DEBUGREMOTE) fprintf (stderr, "os_type : %d\n", os_type);
   if (os_type == 0) {
     fprintf (stderr, "Unable to determine OS type.\n");
     done(1);
   }

   if (rflag)
     {
       if (cflag && tfile != NULL)
	 usage ();
       if (signal (SIGINT, SIG_IGN) != SIG_IGN)
         (void) signal (SIGINT, onintr);
       if (signal (SIGHUP, SIG_IGN) != SIG_IGN)
         (void) signal (SIGHUP, onhup);
       if (signal (SIGQUIT, SIG_IGN) != SIG_IGN)
         (void) signal (SIGQUIT, onquit);
#ifdef notdef
       if (signal (SIGTERM, SIG_IGN) != SIG_IGN)
	 (void) signal (SIGTERM, onterm);
#endif
       if ((mt = openmt (usefile, 1)) < 0)
         {
/*
 *       fprintf (stderr, "openmt> file descriptor: %d\n", mt);
 */
	   fflush (stdout);
	   perror ("atar: openmt error: ");
	   exit (-1);
         }
       {
	 int i;
	 istape = (( i = mtnop (mt)) >= 0);
	 if (debugmask & DEBUGTAPE) {
	   fprintf (stderr, "istape is %d\n", istape);
	   if (istape == 0) {
	     fprintf (stderr, "mtnop : %d ", i);
	     perror ("mtnop said ");
	   }
	 }
       }
      defnblock = RNBLOCK;
      nblock = 0;
/*    It is an error to attempt an a operation on something other than
 *    a tape drive!
 */
      if (aflag) {
	if (istape) {
	  locend ();   /* Position tape. */
	} else {
	  fflush (stdout);
	  fprintf (stderr, "Can't position to end of tape on non-tape device.\n");
	  done(1);
	}
      }
      fflush (stdout);
      fprintf (stderr, "volume: %s   dataset: %s\n\n", ename, sname);

      crcsum = 0l;   /* initialize crc value */
      defnblock = WNBLOCK;
      nblock = reqnblock;
      dorep (argv);
      done (0);
      }

/* Open magtape for read only access. */
   if ((mt = openmt (usefile, 0)) < 0)
      {
/*
 *    fprintf (stderr, "openmt> file descriptor: %d\n", mt);
 */
	fflush (stdout);
      perror ("atar: openmt error: ");
      exit (-1);
      }
   defnblock = RNBLOCK;
   {
     int i;
     istape = ((i = mtnop (mt)) >= 0);
     if (debugmask & DEBUGTAPE) {
       fprintf (stderr, "istape is %d\n", istape);
       if (istape == 0) {
	 fprintf (stderr, "mtnop : %d ", i);
	 perror ("mtnop said ");
       }
     }
   }


   if (xflag)
      {
      if (gflag) {
	if (istape) {
	  locdataset (mt);	/* Position tape. */
	} else {
	  fprintf (stderr, "Can't search for saveset on non tape device.\n");
	  done (1);
	}
      }

      crcsum = 0l;   /* initialize crc value */

      nblock = reqnblock;
      doxtract (argv);			/* Copy images from tape to disk. */
      if (istape) mtfsf(1);             /* and position to next file. */
      }

   else
      {
          if (gflag) {
	    if (istape) {
	      locdataset (mt);	/* Position tape. */
	    } else {
	      fprintf (stderr, "Can't search for saveset on non tape device.\n");
	      done(1);
	    }
	  }
          tarlist (mt, argv);		/* Multiple-file directory listing. */
   }


   done (0);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * U S A G E :   Report command format & options.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

usage ()
{
   fprintf (stderr,
"atar: usage: atar -{cdtxru}[vfblmhopwBiesgD] [tapefile] [blocksize] [file1 file2...]\n");
   done (1);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * O P E N M T :   Open magnetic tape device.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int
openmt (tape, writing)
   char *tape;
   int writing;
{

   if (strcmp(tape, "-") == 0)
      {
      /*
       * Read from standard input or write to standard output.
       */
      if (writing)
	 {
         if (cflag == 0)
	    {
            fprintf(stderr,
               "atar: can only create standard output archives\n");
            done (1);
            }
         vfile = stderr;
         setlinebuf (vfile);
         mt = dup (1);
	 }
      else
         {
         mt = dup (0);
         Bflag++;
         }
      }
   else
      {
      /*
       * Use file or tape on local machine.
       */
      if (writing)
         {
         if (cflag)
	   {
	     if (debugmask & DEBUGTAPE) {
	       fprintf (stderr, "About to open %s with O_RDWR|O_CREAT|O_TRUNC\n", tape);
	     }
	     if (isremote)
	       mt = __rmt_open (tape, O_RDWR|O_CREAT|O_TRUNC, 0666, 0);
	     else
	       mt = open (tape, O_RDWR|O_CREAT|O_TRUNC, 0666);
	   }
         else
	   {
	     if (debugmask & DEBUGTAPE) {
	       fprintf (stderr, "About to open %s with O_RDWR\n", tape);
	     }
	     if (isremote)
	       mt = __rmt_open (tape, O_RDWR, 0666, 0);
	     else
	       mt = open (tape, O_RDWR);
	   }
         }
      else
	{
	  if (debugmask & DEBUGTAPE) {
	    fprintf (stderr, "About to open %s with O_RDONLY\n", tape);
	  }
	  if (isremote)
	    mt = __rmt_open (tape, O_RDONLY, 0666, 0);
	  else
	    mt = open (tape, O_RDONLY);
	}
    }
   if (mt < 0)
     {
       fflush (stdout);
       fprintf (stderr, "atar: ");
       perror (tape);
       done (1);
     }
   return (mt);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * D O R E P :   Write user disk files into one large atar tape file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dorep (argv)

char *argv[];
{
   register char *cp, *cp2;
   char wdir[MAXPATHLEN], tempdir[MAXPATHLEN], *parent;

/*
 * fprintf (stdout, "dorep> ename: %s   sname: %s\n", ename, sname);
 */

/* Make sure they gave us (at least) one file to archive! */
   if (*argv == 0) {
     fprintf (stderr,"You must include a list of files to archive!\n");
     usage ();
     done (1);
   }

   /*** Skip to EOF before writing new user files into atar file.***/

   if (!cflag)
      {
      getdir ();
      do {
         passtape ();
         if (term)
         done (0);
         getdir ();
         } while (!endtape ());

      backtape ();
      if (tfile != NULL)
         {
         char buf[200];

         sprintf (buf,
"sort +0 -1 +1nr %s -o %s; awk '$1 != prev {print; prev=$1}' %s >%sX; mv %sX %s",
            tname, tname, tname, tname, tname, tname);
         fflush (tfile);
         system (buf);
         freopen (tname, "r", tfile);
         fstat (fileno(tfile), &stbuf);
         high = stbuf.st_size;
         }
      }

   /*** Write user files onto tape. ***/

   (void) getcwd (wdir);
   while (*argv && ! term)
      {
	if (debugmask & DEBUGFILENAMES) {
	  fprintf (stderr, "dorep looping with argv = <%s>\n", *argv);
	}
      cp2 = *argv;
      if (!strcmp (cp2, "-C") && argv[1])
         {
         argv++;
         if (chdir (*argv) < 0)
            {
	      fflush (stdout);
            fprintf(stderr, "atar: can't change directories to ");
            perror (*argv);
            }
         else
            (void) getcwd (wdir);
         argv++;
         continue;
         }
      parent = wdir;
/*    Find the final "/" in argv.     */
      for (cp = *argv; *cp; cp++)
         if (*cp == '/')
            cp2 = cp;
/*    If argv includes a "/" change  to that subdirectory    */
      if (cp2 != *argv)
         {
         *cp2 = '\0';
         if (chdir(*argv) < 0)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: can't change directories to ");
            perror (*argv);
            continue;
            }
         parent = getcwd (tempdir);
         *cp2 = '/';
         cp2++;
         }
      else
	{
	if (*cp2 == '/')
	  {
	  if (chdir("/") < 0)
	    {
	      fflush (stdout);
            fprintf (stderr, "atar: can't change directories to ");
            perror ("/");
            continue;
          }
          parent = getcwd (tempdir);
          cp2++;
        }
      }
/*    Call putfile to do the real work.  cp2 is the tail of the path,
      parent is the complete path of its directory
 */
      putfile (*argv++, cp2, parent); /* Write user file to `tbuf'. */

      if (chdir (wdir) < 0)
         {
	   fflush (stdout);
	 fprintf (stderr, "atar: cannot change back?: ");
         perror (wdir);
         }
      }

/*
 *  Right about here we should write out the crc32 info.  What we will
 *  do is to write out a header record with the first character of the
 *  name nulled since this is what tar uses to detect the end of a
 *  tarfile.  We will gen up a dummy file name and force a 0 length
 *  just in case some other version of tar uses a different test for end
 *  of the tarfile.
 */
   if (Dflag)
      {
/*
 *    fprintf (stderr, "atar crc32 checksum %11lo written\n", crcsum);
 */
      bzero ((char *)&dblock, TBLOCK);
      strcpy (&dblock.dbuf.name[1], "atar.crc32");  /* just in case */
      sprintf (dblock.dbuf.size, "%11lo", 0);
      sprintf (dblock.dbuf.mtime, "%11lo", crcsum);
      dblock.dbuf.crcflg = 1;
      (void) writetape ((char *)&dblock);
   } else putempty ();	/* Write empty logical record to `tbuf'. */
   putempty ();	/* Write empty logical record to `tbuf'. */
   flushtape ();	/* Write current `tbuf' buffer to tape. */

   if (prtlinkerr == 0)
      return;
   for (; ihead != NULL; ihead = ihead->nextp)
      {
      if (ihead->count == 0)
         continue;
      fflush (stdout);
      fprintf (stderr, "atar: missing links to %s\n", ihead->pathname);
      }
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * E N D T A P E :   Return `true' if user file name is NULL (=> end of file).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

endtape ()
{
   return (dblock.dbuf.name[0] == '\0');
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * G E T D I R :   Read tape block & copy header into dblock structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

getdir ()
{
   register struct stat *sp;
   int i;

top:
   readtape ((char *) &dblock);		/* Read tape block. */
   if (dblock.dbuf.name[0] == '\0')
      return;
   sp = &stbuf;
   sscanf (dblock.dbuf.mode, "%o", &i);
   sp->st_mode = i;
   sscanf (dblock.dbuf.uid, "%o", &i);
   sp->st_uid = i;
   sscanf (dblock.dbuf.gid, "%o", &i);
   sp->st_gid = i;
   sscanf (dblock.dbuf.size, "%lo", &sp->st_size);
   sscanf (dblock.dbuf.mtime, "%lo", &sp->st_mtime);
   sscanf (dblock.dbuf.chksum, "%o", &chksum);
   if (chksum != (i = checksum()))
      {
	fflush (stdout);
      fprintf (stderr, "atar: directory checksum error (%d != %d)\n",
          chksum, i);
      if (iflag)
         goto top;
      done (2);
      }
   if (tfile != NULL)
      fprintf (tfile, "%s %s\n", dblock.dbuf.name, dblock.dbuf.mtime);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * P A S S T A P E :   Skip over remainder of user `file' on tape.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

passtape ()
{
   long blocks;
   char *bufp;

   if (dblock.dbuf.linkflag == '1')
      return;
   blocks = stbuf.st_size;
   blocks += TBLOCK-1;    /* round the size up to the next TBLOCK boundary */
   blocks /= TBLOCK;      /* and express as a multiple of TBLOCK bytes     */

   while (blocks-- > 0)
      (void) readtbuf (&bufp, TBLOCK);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * P U T F I L E :   Write user file to tape buffer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

putfile (longname, shortname, parent)

char *longname;
char *shortname;
char *parent;
{
   int infile = 0;
   long blocks;
   char buf[TBLOCK];
   char *bigbuf;
   register char *cp;
   struct direct *dp;
   DIR *dirp;
   register int i;
   long l;
   char newparent[NAMSIZ+64];
   extern int errno;
   int	maxread;
   int	hint;		/* amount to write to get "in sync" */

   if (!hflag)
      i = lstat (shortname, &stbuf);
   else
      i = stat (shortname, &stbuf);
   if (i < 0)
      {
	fflush (stdout);
      fprintf (stderr, "atar: ");
      perror (longname);
      return;
      }
   if (tfile != NULL && checkupdate(longname) == 0)
      return;
   if (checkw ('r', longname) == 0)
      return;
   if (Fflag && checkf (shortname, stbuf.st_mode, Fflag) == 0)
      return;

   /*** Write medium and/or dataset name(s) if options are selected. ***/

   bzero ((char *)&dblock, TBLOCK);

   if (eflag)
      {
      strcpy (dblock.dbuf.ename, ename);
/*
 *   printf ("putfile> ename: %s   dblock.dbuf.ename: %s\n",
 *	    ename, dblock.dbuf.ename);
 */
      }
   if (sflag)
      {
      strcpy (dblock.dbuf.sname, sname);
/*
 *   printf ("putfile> sname: %s   dblock.dbuf.sname: %s\n",
 *	    sname, dblock.dbuf.sname);
 *
 */
      }
   if (Dflag)
      dblock.dbuf.crcflg = 1;   /* we are currently on first version of crc  */

   /*** Cases for different types of files. ***/

   switch (stbuf.st_mode & S_IFMT)
      {
      case S_IFDIR:
         for (i = 0, cp = buf; *cp++ = longname[i++];)
		;
         *--cp = '/';
         *++cp = 0  ;
         if (!oflag)
            {
            if ((cp - buf) >= NAMSIZ)
               {
		 fflush (stdout);
               fprintf (stderr,"atar: %s: file name too long\n", longname);
               return;
               }

         stbuf.st_size = 0;
         tomodes (&stbuf);

         strcpy (dblock.dbuf.name,buf);
#ifdef   REDUNDANT
         if (eflag)
            {
            strcpy (dblock.dbuf.ename, ename);
/*
 *          printf ("putfile> ename: %s   dblock.dbuf.ename: %s\n",
 *             ename, dblock.dbuf.ename);
 */
            }
         if (sflag)
            {
            strcpy (dblock.dbuf.sname, sname);
/*
 *          printf ("putfile> sname: %s   dblock.dbuf.sname: %s\n",
 *             sname, dblock.dbuf.sname);
 */
            }
#endif   REDUNDANT
         sprintf (dblock.dbuf.chksum, "%6o", checksum ());
         (void) writetape ((char *)&dblock);
         }

      sprintf (newparent, "%s/%s", parent, shortname);
      if (chdir (shortname) < 0)
         {
	   fflush (stdout);
         perror (shortname);
         return;
         }

      if ((dirp = opendir (".")) == NULL)
         {
	   fflush (stdout);
         fprintf (stderr, "atar: %s: directory read error\n", longname);
         if (chdir (parent) < 0)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: cannot change back?: ");
            perror (parent);
            }
         return;
         }

      while ((dp = readdir (dirp)) != NULL && !term)
         {
         if (dp->d_ino == 0)
            continue;
         if (!strcmp (".", dp->d_name) || !strcmp ("..", dp->d_name))
            continue;
	 if (debugmask & DEBUGFILENAMES) {
	   fprintf (stderr, "putfiles read next directory entry as <%s>\n", dp->d_name);
	 }
         strcpy (cp, dp->d_name);
         l = telldir (dirp);
         closedir (dirp);
         putfile (buf, cp, newparent);
         dirp = opendir (".");
         seekdir (dirp, l);
         }
      closedir (dirp);

      if (chdir (parent) < 0)
         {
	   fflush (stdout);
         fprintf (stderr, "atar: cannot change back?: ");
         perror (parent);
         }
      break;

      case S_IFLNK:
         tomodes (&stbuf);
         if (strlen (longname) >= NAMSIZ)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: %s: file name too long\n", longname);
            return;
            }
         strcpy (dblock.dbuf.name, longname);
         if (stbuf.st_size + 1 >= NAMSIZ)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: %s: symbolic link too long\n", longname);
            return;
            }
         i = readlink (shortname, dblock.dbuf.linkname, NAMSIZ - 1);
         if (i < 0)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: can't read symbolic link ");
            perror (longname);
            return;
            }
         dblock.dbuf.linkname[i] = '\0';
         dblock.dbuf.linkflag = '2';
         if (vflag)
            fprintf (vfile, "a %s symbolic link to %s\n",
               longname, dblock.dbuf.linkname);
         sprintf (dblock.dbuf.size, "%11lo", 0);
#ifdef   REDUNDANT
         if (eflag)
            {
            strcpy (dblock.dbuf.ename, ename);
/*
 *  printf ("putfile> ename: %s   dblock.dbuf.ename: %s\n",
 *     ename, dblock.dbuf.ename);
 */
            }
         if (sflag)
            {
            strcpy (dblock.dbuf.sname, sname);
/*
 *   printf ("putfile> sname: %s   dblock.dbuf.sname: %s\n",
 *      sname, dblock.dbuf.sname);
 */
            }
#endif   REDUNDANT
         sprintf (dblock.dbuf.chksum, "%6o", checksum());
         (void) writetape ((char *)&dblock);
         break;

      case S_IFREG:
         if ((infile = open (shortname, 0)) < 0)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: ");
            perror (longname);
            return;
            }
         tomodes (&stbuf);
         if (strlen (longname) >= NAMSIZ)
            {
	      fflush (stdout);
            fprintf (stderr, "atar: %s: file name too long\n", longname);
            close (infile);
            return;
            }
         strcpy (dblock.dbuf.name, longname);
         if (stbuf.st_nlink > 1)
            {
            struct linkbuf *lp;
            int found = 0;

            for (lp = ihead; lp != NULL; lp = lp->nextp)
               if (lp->inum == stbuf.st_ino && lp->devnum == stbuf.st_dev)
                  {
                  found++;
                  break;
                  }
            if (found)
               {
               strcpy (dblock.dbuf.linkname, lp->pathname);
               dblock.dbuf.linkflag = '1';
#ifdef   REDUNDANT
               if (eflag)
                  {
                  strcpy (dblock.dbuf.ename, ename);
/*
 *   printf ("putfile> ename: %s   dblock.dbuf.ename: %s\n",
 *      ename, dblock.dbuf.ename);
 */
                  }
               if (sflag)
                  {
                  strcpy (dblock.dbuf.sname, sname);
/*
 *   printf ("putfile> sname: %s   dblock.dbuf.sname: %s\n",
 *      sname, dblock.dbuf.sname);
 */
                  }
#endif   REDUNDANT
               sprintf (dblock.dbuf.chksum, "%6o", checksum());
               (void) writetape ( (char *) &dblock);
               if (vflag)
                  fprintf (vfile, "a %s link to %s\n", longname, lp->pathname);
               lp->count--;
               close (infile);
               return;
               }
            lp = (struct linkbuf *) getmem (sizeof(*lp));
            if (lp != NULL)
               {
               lp->nextp = ihead;
               ihead = lp;
               lp->inum = stbuf.st_ino;
               lp->devnum = stbuf.st_dev;
               lp->count = stbuf.st_nlink - 1;
               strcpy (lp->pathname, longname);
               }
            }
         blocks = (stbuf.st_size + (TBLOCK-1)) / TBLOCK;
#ifdef   REDUNDANT
         if (eflag)
            {
            strcpy (dblock.dbuf.ename, ename);
/*
 *   printf ("putfile> ename: %s   dblock.dbuf.ename: %s\n",
 *   ename, dblock.dbuf.ename);
 */
            }
         if (sflag)
            {
            strcpy (dblock.dbuf.sname, sname);
/*
 *   printf ("putfile> sname: %s   dblock.dbuf.sname: %s\n",
 *      sname, dblock.dbuf.sname);
 */
            }
#endif   REDUNDANT
         if (vflag)
            fprintf (vfile, "a %s %ld blocks\n", longname, blocks);
            sprintf (dblock.dbuf.chksum, "%6o", checksum());
            hint = writetape ((char *)&dblock);
#ifdef   SGI
            maxread = max (SGI_BLKSIZE, (nblock * TBLOCK));
#else    SGI
            maxread = max (stbuf.st_blksize, (nblock * TBLOCK));
#endif    SGI
            if ((bigbuf = malloc ((unsigned)maxread)) == 0)
               {
               maxread = TBLOCK;
               bigbuf = buf;
               }

            while ((i = read (infile, bigbuf, min((hint*TBLOCK), maxread))) > 0
               && blocks > 0)
	       {
               register int nblks;

               nblks = ((i-1)/TBLOCK)+1;
               if (nblks > blocks)
               nblks = blocks;
               hint = writetbuf (bigbuf, nblks);
               blocks -= nblks;
               }
            close (infile);
            if (bigbuf != buf)
            free (bigbuf);
            if (i < 0)
               {
		 fflush (stdout);
               fprintf (stderr, "atar: Read error on ");
               perror (longname);
               }
            else if (blocks != 0 || i != 0) {
	      fflush (stdout);
               fprintf (stderr, "atar: %s: file changed size\n", longname);
	     }
            while (--blocks >=  0)
               putempty ();
            break;

      default:
	 fflush (stdout);
         fprintf (stderr, "atar: %s is not a file. Not dumped\n", longname);
         break;
      }
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * D O X T R A C T :   Copy user `files' from tape to disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

doxtract (argv)
	char *argv[];
{
	long blocks, bytes;
	int ofile, i;

	for (;;)
		{
		if ((i = wantit (argv)) == 0)
			continue;
		if (i == -1)
			break;		/* end of tape */
		if (checkw ('x', dblock.dbuf.name) == 0)
			{
			passtape ();
			continue;
			}
		if (Fflag)
			{
			char *s;

			if ((s = rindex (dblock.dbuf.name, '/')) == 0)
				s = dblock.dbuf.name;
			else
				s++;
			if (checkf (s, stbuf.st_mode, Fflag) == 0)
				{
				passtape ();
				continue;
				}
			}
		if (checkdir (dblock.dbuf.name))
			{	/* have a directory */
			if (mflag == 0)
				dodirtimes (&dblock);
			continue;
			}
		if (dblock.dbuf.linkflag == '2')
			{	/* symlink */
			/*
			 * only unlink non directories or empty
			 * directories
			 */
			if (rmdir (dblock.dbuf.name) < 0)
				{
				if (errno == ENOTDIR)
					unlink (dblock.dbuf.name);
				}
			if (symlink (dblock.dbuf.linkname, dblock.dbuf.name)<0)
				{
				  fflush (stdout);
				fprintf(stderr, "atar: %s: symbolic link failed: ",
				    dblock.dbuf.name);
				perror ("");
				continue;
				}
			if (vflag)
				fprintf (vfile, "x %s symbolic link to %s\n",
				    dblock.dbuf.name, dblock.dbuf.linkname);
#ifdef notdef
			/* ignore alien orders */
			chown (dblock.dbuf.name, stbuf.st_uid, stbuf.st_gid);
			if (mflag == 0)
				setimes (dblock.dbuf.name, stbuf.st_mtime);
			if (pflag)
				chmod (dblock.dbuf.name, stbuf.st_mode & 07777);
#endif
			continue;
			}
		if (dblock.dbuf.linkflag == '1')
			{	/* regular link */
			/*
			 * only unlink non directories or empty
			 * directories
			 */
			if (rmdir (dblock.dbuf.name) < 0)
				{
				if (errno == ENOTDIR)
					unlink (dblock.dbuf.name);
				}
			if (link (dblock.dbuf.linkname, dblock.dbuf.name) < 0)
				{
				  fflush (stdout);
				fprintf (stderr, "atar: can't link %s to %s: ",
				    dblock.dbuf.name, dblock.dbuf.linkname);
				perror ("");
				continue;
				}
			if (vflag)
				fprintf (vfile, "%s linked to %s\n",
				    dblock.dbuf.name, dblock.dbuf.linkname);
			continue;
			}
		if ((ofile = creat (dblock.dbuf.name,stbuf.st_mode&0xfff)) < 0)
			{
			  fflush (stdout);
			fprintf (stderr, "atar: can't create %s: ",
			    dblock.dbuf.name);
			perror ("");
			passtape ();
			continue;
			}
		chown (dblock.dbuf.name, stbuf.st_uid, stbuf.st_gid);
		blocks = ((bytes = stbuf.st_size) + TBLOCK-1)/TBLOCK;
		if (vflag)
			fprintf (vfile, "x %s, %ld bytes, %ld tape blocks\n",
			    dblock.dbuf.name, bytes, blocks);
		for (; blocks > 0;)
			{
			register int nread;
			char	*bufp;
			register int nwant;
			
			nwant = defnblock*TBLOCK;
			if (nwant > (blocks*TBLOCK))
				nwant = (blocks*TBLOCK);
			nread = readtbuf (&bufp, nwant);
			if (write (ofile, bufp, (int) min (nread, bytes)) < 0)
				{
				  fflush (stdout);
				fprintf (stderr,
				    "atar: %s: HELP - extract write error",
				    dblock.dbuf.name);
				perror ("");
				done (2);
				}
			bytes -= nread;
			blocks -= (((nread-1)/TBLOCK)+1);
			}
		close (ofile);
		if (mflag == 0)
			setimes (dblock.dbuf.name, stbuf.st_mtime);
		if (pflag)
			chmod (dblock.dbuf.name, stbuf.st_mode & 07777);
		}
	if (mflag == 0)
		{
		dblock.dbuf.name[0] = '\0';	/* process the whole stack */
		dodirtimes (&dblock);
	}
/*	Here, we need to verify the crc info.   */
	if ((dblock.dbuf.crcflg != 0) && Dflag)
	   {
/*
 *     fprintf (stderr, "atar crc32 computation is %11lo\n", crcsum);
 */
	      sscanf (dblock.dbuf.mtime, "%lo", &crcver);
	      if (crcver != crcsum)
		 {
		   fflush (stdout);
		    if (nblock) fprintf (stderr, "atar crc32 error, computed %11lo, read %11lo\n", crcsum, crcver);
		    done (-1);
              } else {
		fflush (stdout);
		fprintf (stderr, "atar crc32 correct.\n");
	      }
	}
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * D O T A B L E :   Produce directory listing of tape contents (one atar file).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dotable (argv, ifileno)
	char *argv[];
        int ifileno;
{
	register int i;
	int lsthdr = 1;

	for (;;)
		{
		  i = wantit (argv);
		  taperrok = 0;
		  if (i == 0)
			continue;
		if (i == -1)
			break;		/* end of tape */
		if (lsthdr)
		  {
		    if (ifileno) fprintf (stdout, "atar file #%3d   ",ifileno);
		    fprintf (stdout, "volume: %s   ", dblock.dbuf.ename);
		    fprintf (stdout, "dataset: %s\n\n", dblock.dbuf.sname);
		    lsthdr = 0;
		  }
		if (vflag)
			longt (&stbuf);		/* Verbose listing. */

		printf ("%s", dblock.dbuf.name);	/* file name. */

		if (dblock.dbuf.linkflag == '1')
			printf (" linked to %s", dblock.dbuf.linkname);
		if (dblock.dbuf.linkflag == '2')
			printf (" symbolic link to %s", dblock.dbuf.linkname);

		printf ("\n");
		passtape ();
		}
/*	Here, we need to verify the crc info.   */
	if ((dblock.dbuf.crcflg != 0) && Dflag)	   {
/*
 *     fprintf (stderr, "atar crc32 computation is %11lo\n", crcsum);
 */
	      sscanf (dblock.dbuf.mtime, "%lo", &crcver);
	      if (crcver != crcsum)
		 {
		   fflush (stdout);
		    fprintf (stderr, "atar crc32 error, computed %11lo, read %11lo\n", crcsum, crcver);
		    done (-1);
              } else {
		fflush (stdout);
		fprintf (stderr, "atar crc32 correct.\n");
	      }
	}
	return (lsthdr);   /* lsthdr is true => EOT */
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * P U T E M P T Y :   
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

putempty ()
{
	char buf[TBLOCK];

	bzero (buf, sizeof (buf));
	(void) writetape (buf);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * L O N G T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

longt (st)
	register struct stat *st;
{
	register char *cp;
	char *ctime ();

	pmode (st);
	printf ("%3d/%1d", st->st_uid, st->st_gid);
	printf ("%7ld", st->st_size);
	cp = ctime (&st->st_mtime);
	printf (" %-12.12s %-4.4s ", cp+4, cp+20);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * P M O D E
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define	SUID	04000
#define	SGID	02000
#define	ROWN	0400
#define	WOWN	0200
#define	XOWN	0100
#define	RGRP	040
#define	WGRP	020
#define	XGRP	010
#define	ROTH	04
#define	WOTH	02
#define	XOTH	01
#define	STXT	01000
int	m1[] = { 1, ROWN, 'r', '-' };
int	m2[] = { 1, WOWN, 'w', '-' };
int	m3[] = { 2, SUID, 's', XOWN, 'x', '-' };
int	m4[] = { 1, RGRP, 'r', '-' };
int	m5[] = { 1, WGRP, 'w', '-' };
int	m6[] = { 2, SGID, 's', XGRP, 'x', '-' };
int	m7[] = { 1, ROTH, 'r', '-' };
int	m8[] = { 1, WOTH, 'w', '-' };
int	m9[] = { 2, STXT, 't', XOTH, 'x', '-' };

int	*m[] = { m1, m2, m3, m4, m5, m6, m7, m8, m9};

pmode (st)
	register struct stat *st;
{
	register int **mp;

	for (mp = &m[0]; mp < &m[9];)
		selectbits (*mp++, st);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * S E L E C T B I T S
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

selectbits (pairp, st)
	int *pairp;
	struct stat *st;
{
	register int n, *ap;

	ap = pairp;
	n = *ap++;
	while (--n >= 0 && (st->st_mode & *ap++)==0)
		ap++;
	putchar (*ap);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C H E C K D I R
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*
 * Make all directories needed by `name'.  If `name' is itself
 * a directory on the atar tape (indicated by a trailing '/'),
 * return 1; else 0.
 */
checkdir (name)
	register char *name;
{
	register char *cp;

	/*
	 * Quick check for existence of directory.
	 */
	if ((cp = rindex(name, '/')) == 0)
		return (0);
	*cp = '\0';
	if (access (name, 0) == 0)
		{	/* already exists */
		*cp = '/';
		return (cp[1] == '\0');	/* return (lastchar == '/') */
		}
	*cp = '/';

	/*
	 * No luck, try to make all directories in path.
	 */
	for (cp = name; *cp; cp++)
		{
		if (*cp != '/')
			continue;
		*cp = '\0';
		if (access (name, 0) < 0)
			{
			if (mkdir (name, 0777) < 0)
				{
				  fflush (stdout);
				perror (name);
				*cp = '/';
				return (0);
				}
			chown (name, stbuf.st_uid, stbuf.st_gid);
			if (pflag && cp[1] == '\0')	/* dir on the tape */
				chmod (name, stbuf.st_mode & 07777);
			}
		*cp = '/';
		}
	return (cp[-1]=='/');
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * O N I N T R
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

onintr ()
{
	(void) signal (SIGINT, SIG_IGN);
	term++;
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * O N Q U I T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

onquit ()
{
	(void) signal (SIGQUIT, SIG_IGN);
	term++;
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * O N H U P
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

onhup ()
{
	(void) signal (SIGHUP, SIG_IGN);
	term++;
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * O N T E R M
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#ifdef notdef
onterm ()
{
	(void) signal (SIGTERM, SIG_IGN);
	term++;
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * T O M O D E S
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#endif

tomodes (sp)
register struct stat *sp;
{
	register char *cp;

	sprintf (dblock.dbuf.mode, "%6o ", sp->st_mode & 07777);
	sprintf (dblock.dbuf.uid, "%6o ", sp->st_uid);
	sprintf (dblock.dbuf.gid, "%6o ", sp->st_gid);
	sprintf (dblock.dbuf.size, "%11lo ", sp->st_size);
	sprintf (dblock.dbuf.mtime, "%11lo ", sp->st_mtime);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 C H E C K S U M
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

checksum ()
{
	register i;
	register char *cp;

	for (cp = dblock.dbuf.chksum;
	     cp < &dblock.dbuf.chksum[sizeof(dblock.dbuf.chksum)]; cp++)
		*cp = ' ';
	i = 0;
	for (cp = dblock.dummy; cp < &dblock.dummy[TBLOCK]; cp++)
		i += *cp;
	return (i);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C H E C K W
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

checkw (c, name)
	char *name;
{
	if (!wflag)
		return (1);
	printf ("%c ", c);
	if (vflag)
		longt (&stbuf);
	printf ("%s: ", name);
	return (response() == 'y');
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * R E S P O N S E
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

response ()
{
	char c;

	c = getchar ();
	if (c != '\n')
		while (getchar () != '\n')
			;
	else
		c = 'n';
	return (c);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C H E C K F
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

checkf (name, mode, howmuch)
	char *name;
	int mode, howmuch;
{
	int l;

	if ((mode & S_IFMT) == S_IFDIR)
		{
		if ((strcmp (name, "SCCS")==0) || (strcmp (name, "RCS")==0)) 
			return (0); 
		return(1);
		}
	if ((l = strlen (name)) < 3)
		return (1);
	if (howmuch > 1 && name[l-2] == '.' && name[l-1] == 'o')
		return (0);
	if (strcmp (name, "core") == 0 ||
	    strcmp (name, "errs") == 0 ||
	    (howmuch > 1 && strcmp (name, "a.out") == 0))
		return (0);
	/* SHOULD CHECK IF IT IS EXECUTABLE */
	return (1);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C H E C K U P D A T E
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Is the current file a new file, or the newest one of the same name? */
checkupdate (arg)
	char *arg;
{
	char name[100];
	long mtime;
	daddr_t seekp;
	daddr_t	lookup ();

	rewind (tfile);
	for (;;)
		{
		if ((seekp = lookup (arg)) < 0)
			return (1);
		fseek (tfile, seekp, 0);
		fscanf (tfile, "%s %lo", name, &mtime);
		return (stbuf.st_mtime > mtime);
		}
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * D O N E
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

done (n)
{
	unlink (tname);
	exit (n);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * W A N T I T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* 
 * Do we want the next entry on the tape, i.e. is it selected?  If
 * not, skip over the entire entry.  Return -1 if reached end of tape.
 */
wantit (argv)
	char *argv[];
{
	register char **cp;
	unsigned long int tempcrc = crcsum;

	getdir ();
	if (endtape ())
	  {
/* Nasty kludge to make crc come out right   */
	    crcsum = tempcrc;

		return (-1);
	  }
	if (*argv == 0)
		return (1);
	for (cp = argv; *cp; cp++)
		if (prefix (*cp, dblock.dbuf.name))
			return (1);
	passtape ();
	return (0);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * P R E F I X
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*
 * Does s2 begin with the string s1, on a directory boundary?
 */
prefix (s1, s2)
	register char *s1, *s2;
{
	while (*s1)
		if (*s1++ != *s2++)
			return (0);
	if (*s2)
		return (*s2 == '/');
	return (1);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * L O O K U P
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define	N	200
int	njab;

daddr_t
lookup (s)
	char *s;
{
	register i;
	daddr_t a;

	for (i=0; s[i]; i++)
		if (s[i] == ' ')
			break;
	a = bsrch (s, i, low, high);
	return (a);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * B S R C H
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

daddr_t
bsrch (s, n, l, h)
	daddr_t l, h;
	char *s;
{
	register i, j;
	char b[N];
	daddr_t m, m1;

	njab = 0;

loop:
	if (l >= h)
		return ((daddr_t) -1);
	m = l + (h-l)/2 - N/2;
	if (m < l)
		m = l;
	fseek (tfile, m, 0);
	fread (b, 1, N, tfile);
	njab++;
	for (i=0; i<N; i++)
		{
		if (b[i] == '\n')
			break;
		m++;
		}
	if (m >= h)
		return ((daddr_t) -1);
	m1 = m;
	j = i;
	for (i++; i<N; i++)
		{
		m1++;
		if (b[i] == '\n')
			break;
		}
	i = cmp (b+j, s, n);
	if (i < 0)
		{
		h = m;
		goto loop;
		}
	if (i > 0)
		{
		l = m1;
		goto loop;
		}
	return (m);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C M P
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

cmp (b, s, n)
	char *b, *s;
{
	register i;

	if (b[0] != '\n')
		exit (2);
	for (i=0; i<n; i++)
		{
		if (b[i+1] > s[i])
			return (-1);
		if (b[i+1] < s[i])
			return (1);
		}
	return (b[i+1] == ' '? 0 : -1);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * R E A D T A P E
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

readtape (buffer)
	char *buffer;
{
	char *bufp;

	if (first == 0)
		getbuf();
	(void) readtbuf (&bufp, TBLOCK);
	bcopy (bufp, buffer, TBLOCK);
	return (TBLOCK);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * R E A D T B U F
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

readtbuf (bufpp, size)
	char **bufpp;
	int size;
{
	register int i;

	if (recno >= nblock || first == 0)
		{
		if ((i = bread (mt, (char *)tbuf, TBLOCK*nblock)) < 0)
			mterr ("read", i, 3);
		if (first == 0)
			{
			if ((i % TBLOCK) != 0)
				{
				  fflush (stdout);
				fprintf (stderr,"atar: tape blocksize error\n");
				done (3);
				}
			i /= TBLOCK;
			if (i != nblock)
				{
				  fflush (stdout);
				fprintf (stderr, "atar: blocksize = %d\n", i);
				if (i == 0) {
					fprintf (stderr, ".  Hit EOT\n");
					done(0);
					}
				nblock = i;
				}
			first = 1;
			}
		recno = 0;
		}
	if (size > ((nblock-recno)*TBLOCK))
		size = (nblock-recno)*TBLOCK;
	*bufpp = (char *) &tbuf[recno];
/*      We _always_ compute CRC on read since we don't know till
	we've read the first header whether to do it or not         */
/*      Slight change to the above - only compute crc *if* Dflag is set */
	if (Dflag) {
	  crcsum = crc32buf (crcsum, (unsigned char *) &tbuf[recno], size);
	}
	recno += (size/TBLOCK);
	return (size);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * W R I T E T B U F :   Write `buffer' to `tbuf' array.  When full, dump to mt.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

writetbuf (buffer, n)
	register char *buffer;
	register int n;
{
	int i;

        if (Dflag)
	  {
	    crcsum = crc32buf (crcsum, buffer, n * TBLOCK);
	  }

	if (first == 0)
		{
		getbuf ();
		first = 1;
		}
	if (recno >= nblock)
		{
		  if (debugmask & DEBUGTAPE) {
		    fprintf (stderr,"about to write %d bytes\n", TBLOCK*nblock);
		  }
		i = mtwrite (mt, (char *)tbuf, TBLOCK*nblock);
		if (i != TBLOCK*nblock)
			mterr ("write", i, 2);
		recno = 0;
		}

	/*
	 *  Special case:  We have an empty tape buffer, and the
	 *  users data size is >= the tape block size:  Avoid
	 *  the bcopy and dma direct to tape.  BIG WIN.  Add the
	 *  residual to the tape buffer.
	 */
	while (recno == 0 && n >= nblock) {
	  if (debugmask & DEBUGTAPE) {
	    fprintf (stderr,"About to write %d bytes\n", TBLOCK*nblock);
	  }
		i = mtwrite (mt, buffer, TBLOCK*nblock);
		if (i != TBLOCK*nblock)
			mterr ("write", i, 2);
		n -= nblock;
		buffer += (nblock * TBLOCK);
	}
		
	while (n-- > 0) {
		bcopy (buffer, (char *) &tbuf[recno++], TBLOCK);
		buffer += TBLOCK;
		if (recno >= nblock)
			{
			  if (debugmask & DEBUGTAPE) {
			    fprintf (stderr, "3 about to write %d bytes\n", TBLOCK*nblock);
			  }
			i = mtwrite (mt, (char *)tbuf, TBLOCK*nblock);
			if (i != TBLOCK*nblock)
				mterr ("write", i, 2);
			recno = 0;
			}
	}

	/* Tell the user how much to write to get in sync */
	return (nblock - recno);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * B A C K T A P E :   Backspace one record.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

backtape ()
{
	
       	/*** Execute magnetic tape operation (backspace one record). ***/

	if (istape)
		{
		if (mtbsr(1) < 0)
			{
			  fflush (stdout);
			fprintf (stderr, "atar: tape backspace error: ");
			perror ("");
			done (4);
			}
		}
	else

	/*** Position tape device pointer back one record. ***/

		mtlseek (mt, (daddr_t) -TBLOCK*nblock, 1);
	recno--;
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * F L U S H T A P E :   Write out current `tbuf' array to magnetic tape.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

flushtape ()
{
	int i;

	if (debugmask & DEBUGTAPE) {
	  fprintf (stderr,"about to flush %d bytes to tape\n", TBLOCK*nblock);
	}
	i = mtwrite (mt, (char *) tbuf, TBLOCK*nblock);
	if (i != TBLOCK*nblock)
		mterr ("write", i, 2);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T E R R :   Report magnetic tape error.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

mterr (operation, i, exitcode)
	char *operation;
	int i;
{
  fflush (stdout);
	fprintf (stderr, "atar: tape %s error: ", operation);
	if (i < 0)
		perror ("");
	else
		fprintf (stderr, "unexpected EOF\n");
	if (taperrok) fprintf (stderr, "Don't worry - probably caused by driver suppressing double EOF.\n");
	done (exitcode);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * B R E A D :   Read magnetic tape block(s) into array `buf'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

bread (fd, buf, size)
	int fd;
	char *buf;
	int size;
{
	int count;
	static int lastread = 0;

	if (!Bflag)
	  {
	    if (debugmask & DEBUGTAPE) {
	      fprintf (stderr, "About to read %d bytes from tape\n", size);
	    }
		return (mtread (fd, buf, size)); 
	  }

	for (count = 0; count < size; count += lastread)
		{
		  if (debugmask & DEBUGTAPE) {
		    fprintf (stderr, "About to read %d bytes from tape\n", size - count);
		  }
		lastread = mtread (fd, buf, size - count);
		if (lastread <= 0)
			{
			if (count > 0)
				return (count);
			return (lastread);
			}
		buf += lastread;
		}
	return (count);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * G E T C W D :   Return current working directory on disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

char *
getcwd (buf)
	char *buf;
{
	if (getwd(buf) == NULL)
		{
		  fflush (stdout);
		fprintf (stderr, "atar: %s\n", buf);
		exit (1);
		}
	return (buf);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * G E T B U F :   Determine tape block size and allocate memory for `tbuf'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

getbuf ()
{
  
  /*** Determine tape block size. ***/
  
  if (nblock == 0)
    {
 
      if (istape)
	nblock = defnblock;
      else
	{
#ifdef  SGI
	  nblock = SGI_BLKSIZE / TBLOCK;
#else   SGI
	  fstat (mt, &stbuf);
	  nblock = stbuf.st_blksize / TBLOCK;
#endif   SGI
	  if (nblock == 0)
	    nblock = defnblock;
	}
    }
  
  /*** Allocate storage for `tbuf' array. ***/
  /*** but make sure to release the old storage if any ***/
  if (bufset)
    tbuf = (union hblock *) realloc ( (char *)tbuf,
				     (unsigned) nblock*TBLOCK);
  else
    tbuf = (union hblock *) malloc ((unsigned) nblock*TBLOCK);
  bufset = 1;
  if (tbuf == NULL)
    {
      fflush (stdout);
      fprintf (stderr, "atar: blocksize %d too big, can't get memory\n",
	       nblock);
      done (1);
    }
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * D O D I R T I M E S
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*
 * Save this directory and its mtime on the stack, popping and setting
 * the mtimes of any stacked dirs which aren't parents of this one.
 * A null directory causes the entire stack to be unwound and set.
 *
 * Since all the elements of the directory "stack" share a common
 * prefix, we can make do with one string.  We keep only the current
 * directory path, with an associated array of mtime's, one for each
 * '/' in the path.  A negative mtime means no mtime.  The mtime's are
 * offset by one (first index 1, not 0) because calling this with a null
 * directory causes mtime[0] to be set.
 * 
 * This stack algorithm is not guaranteed to work for tapes created
 * with the 'r' option, but the vast majority of tapes with
 * directories are not.  This avoids saving every directory record on
 * the tape and setting all the times at the end.
 */
char dirstack[NAMSIZ];
#define NTIM (NAMSIZ/2+1)		/* a/b/c/d/... */
time_t mtime[NTIM];

dodirtimes (hp)
	union hblock *hp;
{
	register char *p = dirstack;
	register char *q = hp->dbuf.name;
	register int ndir = 0;
	char *savp;
	int savndir;

	/* Find common prefix */
	while (*p == *q) {
		if (*p++ == '/')
			++ndir;
		q++;
	}

	savp = p;
	savndir = ndir;
	while (*p) {
		/*
		 * Not a child: unwind the stack, setting the times.
		 * The order we do this doesn't matter, so we go "forward."
		 */
		if (*p++ == '/')
			if (mtime[++ndir] >= 0)
				{
				*--p = '\0';	/* zap the slash */
				setimes (dirstack, mtime[ndir]);
				*p++ = '/';
				}
	}
	p = savp;
	ndir = savndir;

	/* Push this one on the "stack" */
	while (*p = *q++)	/* append the rest of the new dir */
		if (*p++ == '/')
			mtime[++ndir] = -1;
	mtime[ndir] = stbuf.st_mtime;	/* overwrite the last one */
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * S E T I M E S
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

setimes (path, mt)
	char *path;
	time_t mt;
{
	struct timeval tv[2];

	tv[0].tv_sec = time ((time_t *) 0);
	tv[1].tv_sec = mt;
	tv[0].tv_usec = tv[1].tv_usec = 0;
	if (utimes(path, tv) < 0)
		{
		  fflush (stdout);
		fprintf(stderr, "atar: can't set time on %s: ", path);
		perror ("");
		}
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * G E T M E M :   Allocate memory of length `size' bytes; return char pointer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

char *
getmem (size)
{
	char *p = malloc ((unsigned) size);

	if (p == NULL && freemem)
		{
		  fflush (stdout);
		fprintf (stderr,
		 "atar: out of memory, link and directory modtime info lost\n");
		freemem = 0;
		}
	return (p);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * T A R L I S T :   High level directory of multiple-atar-file tape.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * 14 June 1988
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

tarlist (mt, argv)

int mt;		/* Magnetic tape file descriptor. */
char *argv[];	/* Pointer to command line arguments. */

{
extern int first;
extern int recno;
extern int dflag;
extern int errno;
int ierr;
int ifile = 0;
int nskip, nback;

   if (istape && !gflag && Tflag)
/*** Read only first record of each file; retrieve volume & dataset names. ***/
     {
     if ((ierr = mtrew()) < 0)
        {
	  fflush (stdout);
	perror ("tarlist rew(1) error.\n");
	exit (ierr);
     }
     for (;;)
        {
        ifile++;
	nblock = reqnblock;
	if (dflag)
	  {
	    taperrok = 1;
	    ierr = gethdr (argv);	/* Read header, copy to dblock. */
	    taperrok = 0;
	    if (ierr == -1)		/* EOT encountered. */
	      {
		return (0);
	      }

	    fprintf (stdout, "atar file #%3d   ", ifile);
	    fprintf (stdout, "volume: %-20s   ", dblock.dbuf.ename);
	    fprintf (stdout, "dataset: %s\n\n", dblock.dbuf.sname);

	  } else	/* Normal file by file `atar' directory. */
	    {
	      crcsum = 0l;                   /* initialize crc value */
	      taperrok = 1;
	      if (dotable (argv, ifile)) {
		fprintf (stdout, "atar file #%3d has null header - ignored", ifile);
	      }
	    }
        fprintf (stdout, "\n");

	first = 0;		/* Reset `first' variable for `atar'. */
	recno = 0;		/* Reset `recno' variable for `atar'. */
/*      Do an lseek to 0 in order to reset offset variable in SunOS */
	if ((nskip = (int) mtlseek (mt, (off_t) 0, 0)) < 0) {
	  fflush (stdout);
	  perror ("lseek error in tarlist : ");
	  exit (-1);
	}
        if ((nskip = mtfsf(1)) < 0)	/* Skip to next file. */
	   {
	     fflush (stdout);
	   fprintf (stderr, "mtfsf ... status: %d\n", nskip);
	   return (nskip);
	   }
        }
   } else  /* this is not a multi-tarfile request! */
      {
	if (dflag)
	  {           /* all we need is the header stuff */
          ierr = gethdr (argv);		/* Read header, copy to dblock. */

          if (ierr == -1)			/* EOT encountered. */
             {
/*
 *           fprintf (stdout, "EOT: null filename\n");
 */
             return (0);
          }

          fprintf (stdout, "atar file #%3d   ", ifile);
	  fprintf (stdout, "volume: %-20s   ", dblock.dbuf.ename);
	  fprintf (stdout, "dataset: %s\n\n", dblock.dbuf.sname);
	  first = 0;
	  recno = 0;
	  mtlseek (mt, (off_t)0, 0);    /* go back to beginning  */
        } else
          {
	  crcsum = 0l;                   /* initialize crc value */
          dotable (argv, 0);		/* Normal `atar' directory. */
        }
	if (istape) if ((nskip = mtfsf(1)) < 0)
          {
	    fflush (stdout);
	  perror ("tarlist fsf(1) error");
	  return (nskip);
	}
     }
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * G E T H D R :   Read tape block & copy header into dblock structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

gethdr (argv)

char *argv[];

{
   recno = 0;
   first = 0;

   getdir ();

   if (endtape ())
      return (-1);
   if (*argv == 0)
      return (1);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * L O C E N D :   
 * Position magnetic tape just before end of tape or before end of last file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 14 June 1988
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */


locend ()

{
  int nbytes;			/* Size of buf array. */
  int nread;			/* # bytes read. */
  int ierr;			/* Error value. */
  int ifile;			/* File count. */
  int nfskip;			/* # files to skip. */
  int listfile = 0;
  extern int errno;

   /*** First, rewind tape. ***/
   /*** Unfortunately, we need to optimize tape motion a bit.
        The strategy is ... first do a bsf 3, then an fsf 1.
        If you are at the very end of the tape (past a double EOF)
        this is guaranteed to place you just before the last saveset
        that is on the tape.  You get the volume name by reading this
        saveset and go on from there.  If you are not at the end of the
        tape you may have an extra saveset to read through but never
        more than two.  If we get any errors in the bsf or fsf operations
        we revert to a rewind for safety.
    ***/

  ierr = mtbsf (3);
  if (debugmask & DEBUGTAPE) fprintf (stderr, "Result of bsf 3 was %d\n",
				      ierr);
  if (ierr >= 0) {
    ierr = mtfsf (1);
    if (debugmask & DEBUGTAPE) fprintf (stderr, "Result of fsf 1 was %d\n",
					ierr);
    if (ierr >= 0) goto skip_rewind;
  }

  fflush (stdout);
  fprintf (stderr, "We'll use a REWIND instead of file positioning.\n");
  listfile = 1;
  ierr = mtrew();
  if (ierr < 0) {
    fflush (stdout);
    perror ("Rewind failed, exiting");
    exit (ierr);
  }

skip_rewind:

   nblock = reqnblock;
   if (first == 0)   /* which it always will be   */
      {
      getbuf();
      }
   nbytes = nblock * TBLOCK;
   ifile = 0;
   /*
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    * Advance through tape, reading first record of each file,
    *  then skipping over the EOF.
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    */

  if (debugmask & DEBUGTAPE) fprintf (stderr, "locend> seek EOT.\n");
  if (debugmask & DEBUGTAPE) fprintf (stderr, "locend about to read %d bytes.\n", nbytes);
  nread = mtread (mt, tbuf, nbytes);
/* SCSI driver on sun4 can do MTEOM on 8mm under SunOS 4.1 */
/* but SGI requires that we close and reopen the device between the read
 * above and the writes to follow.  Do it before the mteom in case the
 * device is specified to be rewind on close! */
  if (os_type == SGI_OS) {
    ierr = mtclose (mt);
    if (ierr < 0) {
      fflush (stdout);
      perror ("locend> close error");
      exit (ierr);
    }
    if ((mt = openmt (usefile, 1)) < 0)
      {
/*
 *       fprintf (stderr, "openmt> file descriptor: %d\n", mt);
 */
	fflush (stdout);
	perror ("atar: openmt error: ");
	exit (-1);
      }
  }
  if (nread < 0) {
    perror ("Got an error on read in locend - fatal");
    exit (-1);
  } else if (nread == 0) {
    fprintf (stderr, "Tape is empty!");
    return 0;
  }
  bcopy (tbuf, &dblock, TBLOCK);
  strcpy (ename, dblock.dbuf.ename);  /* Store volume name. */
  if (listfile) {
    fprintf (stdout, "tarfile:%3d   volume: %-20s   dataset: %s\n",
	     ifile, dblock.dbuf.ename, dblock.dbuf.sname);
  }
  ierr = mteom();
  if (ierr >= 0) {
    fprintf (stdout, "further information not available - used MTEOM\n");
  } else {
/* Some machines don't support mteom for EXB-8200 drives */
    for (;;) {
/*     Skip forward one file, then read another record   */
      if (debugmask & DEBUGTAPE) {
	fprintf (stderr,"locend about to fsf 1\n");
      }
      if ((ierr = mtfsf(1)) < 0) {	/* Skip over next EOF. */
	fflush (stdout);
	perror ("locend> fsf(1) error");
	exit (ierr);
      }
      ifile++;
      if (debugmask & DEBUGTAPE) {
	fprintf (stderr,"locend about to read %d bytes from tape\n", nbytes);
      }
      nread = mtread (mt, tbuf, nbytes);
      if (nread > 0) {
	bcopy (tbuf, &dblock, TBLOCK);
	if (listfile) {
	  fprintf (stdout, "tarfile:%3d   volume: %-20s   dataset: %s\n",
		   ifile, dblock.dbuf.ename, dblock.dbuf.sname);
	} else {
	  fprintf (stdout, "tarfile:      volume: %-20s   dataset: %s\n",
		   dblock.dbuf.ename, dblock.dbuf.sname);
	}
	strcpy (ename, dblock.dbuf.ename);  /* Store volume name. */
	if (!cflag)  strcpy (sname, dblock.dbuf.sname);/* Store dataset name.*/
/*
 *	printf ("locend> file # %d   ename: %s   sname: %s\n",
 *	        ifile, ename, sname);
 */
      } else if (nread == 0) {
	fprintf (stdout, "locend> Found EOT after file # %d.\n", ifile);
	if (cflag) {	/*** Create new dataset file at EOT. ***/
	  if ((ierr = mtbsf(1)) < 0) {
	    fflush (stdout);
	    perror ("locend> bsf(1) error: ");
	  }
	} else {	/*** Add new user files to last dataset file. ***/
	  if (ifile < 2) {	/* Rewind tape if only one dataset exists. */
	    if ((ierr = mtrew()) < 0) {
	      fflush (stdout);
	      perror ("locend> rew(1) error: ");
	    }
	  } else {   /* Back 3 EOF's, Forward 1 EOF. */
	    if ((ierr = mtbsf(3)) < 0) {
	      fflush (stdout);
	      perror ("locend> bsf(1) error: ");
	    }
	    if ((ierr = mtfsf(1)) < 0) {
	      fflush (stdout);
	      perror ("locend> fsf(1) error: ");
	    }
	  }
	}
	return (ierr);
       } else {
/* nread is < 0 */
	 if (errno == EIO) {
	   fflush (stdout);
	   fprintf (stderr, "Got an IO error looking for EOT\n ");
	   fprintf (stderr, "Don't worry - probably caused by driver suppressing double EOF.\n");
	   return (nread);
	 } else {
	   fflush (stdout);
	   perror ("locend> read error");
	   exit (nread);
	 }
       }
    }
  }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * L O C D A T A S E T ( )
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
locdataset (fd)

int fd;			/* File descriptor for magnetic tape device file. */

{
  int nbytes;			/* Size of buf array. */
  int nread;			/* # bytes read. */
  int ierr;			/* Error value. */
  int ifile;			/* File count. */
  int nfskip;			/* # files to skip. */

   /*** First, rewind tape. ***/

   ierr = mtrew();
   if (ierr < 0)
     {
       fflush (stdout);
     perror ("locdataset rew error ");
     exit (ierr);
     }
   nblock = reqnblock;
   if (first == 0)    /* which it always will be   */
      {
      getbuf();
      }
   nbytes = nblock * TBLOCK;
   ifile = 0;
   /*
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    * Advance through tape, reading first record of each file,
    *  then skipping over the EOF until the requested dataset is found.
    * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
    */

   fprintf (stdout, "Seeking dataset %s.\n", sname);
   for (;;)
      {
      /*** Read first record of file, then skip forward over one EOF. ***/

	if (debugmask & DEBUGTAPE) {
	  fprintf (stderr,"locdataset about to read %d bytes from tape\n", nbytes);
	}
      if ((nread = read (fd, tbuf, nbytes)) > 0) /* First record is not EOF. */
         {
	 ifile++;
	 bcopy (tbuf, &dblock, TBLOCK);
	 fprintf (stdout, "file:%3d   volume: %-20s   dataset: %s\n",
		  ifile, dblock.dbuf.ename, dblock.dbuf.sname);
         strcpy (ename, dblock.dbuf.ename);  /* Store volume name. */
/*
 *	 fprintf (stdout, "locdataset> file # %d   ename: %s   sname: %s\n",
 *		  ifile, ename, sname);
 */

         if (debugmask & DEBUGTAPE) {
           fprintf (stderr, "About to get tape status \n");
	   mtiocget();
         }
         if (strcmp (dblock.dbuf.sname, sname) == 0)
	    {
            if (ifile < 2)	/* Rewind tape if only one dataset exists. */
               {
/*
 *      fprintf (stdout, "locdataset> rewind tape.\n");
 */
	       if ((ierr = mtrew()) < 0) {
		 fflush (stdout);
		  perror ("locdataset> rew(1) error: ");
		}
               }
            else	/* Back 1 EOF, Forward 1 EOF. */
	       {
/*
 *      fprintf (stdout, "r: Backspace 1 EOF, Forward 1 EOF.\n");
 */
		 if (os_type == SUN_OS) {
/*
 *    On Suns attempt an NBSF since on some Suns with the Sun driver a
 *    standard bsf results in a silent rewind.  Note that this is not
 *    the case with the APUNIX driver we use here at HAO!
 */
		   if ((ierr = mtnbsf(1)) < 0) {
		     fprintf (stderr, "locdataset> nbsf(1) error: %d\n", ierr);
		     fflush (stdout);
		     perror ("locdataset nbsf(1) error ");
		   }
		 } else {
		   if ((ierr = mtbsf(1)) < 0) {
		     fprintf (stderr, "locdataset> bsf(1) error: %d\n", ierr);
		     fflush (stdout);
		     perror ("locdataset bsf(1) error ");
		   }
		 }
               if (debugmask & DEBUGTAPE) {
                 fprintf (stderr, "About to get tape status \n");
	         mtiocget();
               }
	       if ((ierr = mtfsf(1)) < 0) {
	          fprintf (stderr, "locdataset> fsf(1) error: %d\n", ierr);
		 fflush (stdout);
		  perror ("locdataset> fsf(1) error ");
		}
               if (debugmask & DEBUGTAPE) {
                 fprintf (stderr, "About to get tape status \n");
	         mtiocget();
               }
               }
            return (0);
	    }

	 if ((ierr = mtfsf(1)) < 0)	/* Skip over next EOF. */
	    {
/*
 *	    fprintf (stderr, "locdataset> fsf(1) error: %d\n", ierr);
 */
	      fflush (stdout);
	    perror ("locdataset> fsf(1) error ");
	    return (ierr);
	    }
	 }

      /*** Found EOT if file is empty. ***/

      if (nread == 0)		/*** EOF detected. ***/
         {
	 if (ifile == 0)	/*** Empty tape.  Rewind to BOT. ***/
	    {
/*
 *          fprintf (stdout, "locdataset> Empty tape.  Rewind to BOT.\n");
 */
	    if ((ierr = mtrew()) < 0)
	       {
/*
 *	       fprintf (stderr, "locdataset> rewind(1) error: %d\n", ierr);
 */
		 fflush (stdout);
	       perror ("locdataset> rewind(1) error ");
	       exit (ierr);
	       }
            exit (0);
            }
         else			/*** EOT found.  Rewind tape. ***/
	    {
	    fprintf (stdout, "Found EOT after file # %d.\n", ifile);
	    fprintf (stdout, "dataset %s not on tape.\n", sname);
	    fprintf (stdout, "Rewinding tape.\n");
	    if ((ierr = mtrew()) < 0)
               {
/*
 *	       fprintf (stderr, "locdataset> rew(1) error: %d\n", ierr);
 */
		 fflush (stdout);
	       perror ("locdataset> rew(1) error ");
	       exit (ierr);
               }
	    exit (0);
	    }
         }

      /*** Encountered read error if nread < 0. ***/

      if (nread < 0)
         {
/*
 *         fprintf (stderr, "locdataset> read error...%d\n", nread);
 */
	   fflush (stdout);
	   perror ("locdataset> read error ");
         return (nread);
         }

      }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T I O C G E T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mtiocget()
{
  static struct mtget mt_status;
  int rval;

  if (isremote)
    rval = __rmt_ioctl (mt, MTIOCGET, (char *)&mt_status);
  else
    rval = ioctl (mt, MTIOCGET, (char *)&mt_status);
  fprintf (stderr, "tape status: \n");
  fprintf (stderr, "   file no= %d   block no= %d\n",
	mt_status.mt_fileno,mt_status.mt_blkno);
  return (rval);
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T R E W
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
}
mtrew()
{
  static struct mtop rew = { MTREW, 1};

  if (debugmask & DEBUGTAPE) {
    fprintf (stderr, "About to issue tape rewind\n");
  }
  if (isremote)
    return (__rmt_ioctl (mt, MTIOCTOP, (char *)&rew));
  else
    return (ioctl (mt, MTIOCTOP, (char *)&rew));
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T E O M
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mteom()
{
  static struct mtop eom = { MTEOM, 1};

  if (debugmask & DEBUGTAPE) {
    fprintf (stderr, "About to issue tape eom\n");
  }
  if (isremote)
    return (__rmt_ioctl (mt, MTIOCTOP, (char *)&eom));
  else
    return (ioctl (mt, MTIOCTOP, (char *)&eom));
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T F S F
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mtfsf(n)
int n;
{
  int ans;
  static struct mtop fsf = { MTFSF, 1};

  fsf.mt_count = (daddr_t) n;
  if (debugmask & DEBUGTAPE) {
    fprintf (stderr,"About to fsf %d on tape\n", n);
  }
  if (isremote)
    ans = __rmt_ioctl (mt, MTIOCTOP, (char *)&fsf);
  else
    ans = ioctl (mt, MTIOCTOP, (char *)&fsf);
  return (ans);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T B S F - BackSpace File MARK
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mtbsf(n)
int n;
{
  extern int errno;
  int ans;
  static struct mtop bsf = { MTBSF, 1};
  static struct mtget mt_status;
  int rval;
/* old code to deal with Sun's aberrant BSF and BSFM commands which only
 * related to SunOS 4.0.3 removed since the underlying conditions can't
 * be readily detected at runtime.
 */
  bsf.mt_count = (daddr_t) n;
  if (os_type == SUN_OS) {
/* APUNIX Exabyte driver doesn't return an error on BSF when it should.
 * We can catch this by checking file position before we start.      */
    if (debugmask & DEBUGTAPE) {
      fprintf (stderr,"About to get status on tape\n");
    }
    if (isremote)
      rval = __rmt_ioctl (mt, MTIOCGET, (char *)&mt_status);
    else
      rval = ioctl (mt, MTIOCGET, (char *)&mt_status);
    if (rval < 0) {
      fflush (stdout);
      perror ("Error getting tape status before BSF ");
      return (rval);
    }
    if (debugmask & DEBUGTAPE) {
      fprintf (stderr, "Called BSF on fileno %d\n", (int)mt_status.mt_fileno);
    }
    if (n > mt_status.mt_fileno) {
      errno = EIO;
      if (debugmask & DEBUGTAPE) {
	fprintf (stderr, "Forcing BSF error\n");
      }
      return (-1);
    }
  }
  if (debugmask & DEBUGTAPE) {
    fprintf (stderr,"About to bsf %d on tape\n", n);
  }
  if (isremote)
    ans = __rmt_ioctl (mt, MTIOCTOP, (char *)&bsf);
  else
    ans = ioctl (mt, MTIOCTOP, (char *)&bsf);
  return (ans);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T N B S F - BackSpace File 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mtnbsf(n)
int n;
{
  int ans;
  static struct mtop nbsf = { MTNBSF, 1};
  nbsf.mt_count = (daddr_t) n;
  if (debugmask & DEBUGTAPE) {
    fprintf (stderr,"About to nbsf %d on tape\n", n);
  }
  if (isremote)
    ans = __rmt_ioctl (mt, MTIOCTOP, (char *)&nbsf);
  else
    ans = ioctl (mt, MTIOCTOP, (char *)&nbsf);
/* APUNIX driver doesn't handle NBSF */
  if ((ans < 0) && (errno == ENXIO)) {
    if (debugmask & DEBUGTAPE) {
      fprintf (stderr,"NBSF not supported - using plain BSF\n");
    }
    ans = mtbsf (n);
  }
  return (ans);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T B S R
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mtbsr(n)
int n;
{
  static struct mtop bsr = { MTBSR, 1};

  bsr.mt_count = (daddr_t) n;
  if (debugmask & DEBUGTAPE) {
    fprintf (stderr,"About to bsr %d on tape\n", n);
  }
  if (isremote)
    return (__rmt_ioctl (mt, MTIOCTOP, (char *)&bsr));
  else
    return (ioctl (mt, MTIOCTOP, (char *)&bsr));
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M T N O P
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
mtnop()
{
  static struct mtop nop = { MTNOP, 1};

  if (debugmask & DEBUGTAPE) {
    fprintf (stderr, "About to issue tape no op\n");
  }
  if (isremote)
    return (__rmt_ioctl (mt, MTIOCTOP, (char *)&nop));
  else
    return (ioctl (mt, MTIOCTOP, (char *)&nop));
}

int mtread(fd,buf,nbytes)
int fd;
char *buf;
int nbytes;
{
  if (isremote)
    return (__rmt_read (fd, buf, nbytes));
  else
    return (read (fd, buf, nbytes));
}

int mtwrite(fd, buf, nbytes)
int fd;
char *buf;
int nbytes;
{
  if (isremote)
    return (__rmt_write(fd, buf, nbytes));
  else
    return (write(fd, buf, nbytes));
}

int mtclose (fd)
int fd;
{
  if (isremote)
    return (__rmt_close (fd));
  else
    return (close(fd));
}

off_t mtlseek (fd, offset, whence)
int fd;
off_t offset;
int whence;
{
  if (isremote)
    return ((off_t) __rmt_lseek (fd, (long) offset, whence));
  else
    return (lseek (fd, offset, whence));
}
