      program GETPB
c
c-----------------------------------------------------------------------
c
c ****** Calculate the polarization brightness from the
c ****** mass density (from the MAS code).
c
c-----------------------------------------------------------------------
c
c ****** Updates and bug fixes:
c
c        05/19/94, ZM, Version 1.00:
c
c         - Original version of program.
c
c        01/24/95, ZM, Version 1.01:
c
c         - Generalized to handle 3D fields.
c         - Added the capability to enhance the density perturbation.
c         - Added the capability to apply a "vignetting function"
c           to pB.
c
c        02/06/95, ZM, Version 1.02:
c
c         - Added the ability to rotate the output pB image
c           in the theta direction.
c
c        04/28/95, ZM, Version 1.03:
c
c         - Added the ability to multiply pB by a radial vignetting
c           function.
c
c        03/03/97, ZM, Version 1.04:
c
c         - Added the ability to specify the solar B0 and P angles.
c
c        12/03/2003, ZM, Version 1.05:
c
c         - Updated the program to be able to deal with
c           nonuniform meshes in phi.
c
c-----------------------------------------------------------------------
c
      character cname*(*),cvers*(*),cdate*(*)
c
      parameter (cname='GETPB')
      parameter (cvers='1.05')
      parameter (cdate='12/03/2003')
c
c-----------------------------------------------------------------------
c
      cliche syntax
c
c-----------------------------------------------------------------------
c
c ****** Group definitions.
c
c        GROUP 1: <kw>
c        GROUP 2: <arg>
c        GROUP 3: <kw> <arg>
c        GROUP 4: <kw> <arg> <arg>
c
      parameter (ngroup=4)
c
      parameter (IGRPK  =1)
      parameter (IGRPA  =2)
      parameter (IGRPKA =3)
      parameter (IGRPKAA=4)
c
      parameter (mxarg=100)
c
      parameter (IXGRP=1)
      parameter (IXSET=2)
      parameter (IXREQ=3)
      parameter (IXKEY=4)
      parameter (IXARG=5)
      parameter (IXNAM=6)
c
      integer narg,info(6,mxarg)
      common /sntx1/ narg,info
c
c ****** Buffer for the usage line.
c
      character*512 ulbuf
      common /sntx2/ ulbuf
c
c ****** Buffer for the error message.
c
      character*72 errmsg
      common /sntx3/ errmsg
c
c-----------------------------------------------------------------------
c
      endcliche
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      character*72 infile,outfile,vffile, brtfile
      common /parcom1/ infile,outfile,vffile, brtfile
c
      character*16 fmt
      common /parcom2/ fmt
c
      common /parcom3/ tmax,ntau,crlong,rocc,fac,power,
     &                 pangle,b0angle
c
c-----------------------------------------------------------------------
c
c ****** Pointers to the density array and its scales.
c
      pointer (iprh,rh),(ipth,th),(ipph,ph)
      pointer (iprho,rho)
c
c ****** Pointers to the pB array and its scales.
c
      pointer (ipr,r),(ipt,t)
      pointer (ippb,pb)
      pointer (ipbright, bright)
c
      common /fldcom/ nr,nt,np,iprh,ipth,ipph,iprho,
     &                nrm1,ntm1,ipr,ipt,ippb, ipbright
c
      common /phi2d/ phi0
c
      integer idim(3),ips(3)
c
c ****** Pointers to the vignetting function.
c
      pointer (iprvf,rvf),(ipvf,vf)
c
c-----------------------------------------------------------------------
c
c ****** Sin and cos of the solar P and B0 angles.
c
      common /pb0com/ sp,cp,sb,cb
c
c-----------------------------------------------------------------------
c
c ****** Central meridian longitude.
c
      common /cmdcom/ emu
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Set the parameters.
c
      call setpar (cname,cvers,cdate)
c
c ****** Read the density.
c
      lf=lenstr(infile)
c
      call rdsds (fmt,infile,3,ndim,idim,iprho,ifscale,ips,ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not read the data set.'
        write (*,*) 'IERR (from RDSDS) = ',ierr
        write (*,*) 'File name: ',infile(1:lf)
        call exit (1)
      end if
c
      if (ndim.eq.2) then
        nr=idim(1)
        nt=idim(2)
        np=1
        iprh=ips(1)
        ipth=ips(2)
        phi0=0.
        ipph=loc(phi0)
      else if (ndim.eq.3) then
        nr=idim(1)
        nt=idim(2)
        np=idim(3)
        iprh=ips(1)
        ipth=ips(2)
        ipph=ips(3)
c        write (*,*) 'rh (1): ', rh
c        write (*,*) 'rh (nr): ', rh (nr)
c        write (*,*) 'th range: ', th (1), th (nt)
c        write (*,*) 'ph range: ', ph (1), ph (np)
      else
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### The file does not have a 2D or 3D field.'
        write (*,*) 'Number of dimensions = ',ndim
        write (*,*) 'File name: ',infile(1:lf)
        call exit (1)
      end if
c
      if (ifscale.eq.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### The field does not have scales.'
        write (*,*) 'File name: ',infile(1:lf)
        call exit (1)
      end if
c
      nrm1=nr-1
      ntm1=nt-1
c
c ****** Set the dimensions for the pB image.
c ****** For a 2D case, the image includes only one limb
c ****** (theta between 0 and pi).
c ****** For a 3D case, the image includes both limbs
c ****** (theta between 0 and 2*pi).
c
      nrpb=nrm1
      if (ndim.eq.2) then
        ntpb=ntm1
      else
        ntpb=2*ntm1-1
      end if
c
c ****** Assign memory for PB and its scales.
c
      nwd=nrpb
      ipr=igetmem(nwd)
      if (ipr.eq.0) go to 900
c
      nwd=ntpb
      ipt=igetmem(nwd)
      if (ipt.eq.0) go to 900
c
      nwd = nrpb * ntpb
      ipbright = igetmem (nwd)
      if (ipbright .eq. 0)  go to 900
c
      nwd=nrpb*ntpb
      ippb=igetmem(nwd)
      if (ippb.eq.0) go to 900
c
c ****** Get the r and t arrays.
c
      call rmesh (nrm1,rh,r)
c
      if (ndim.eq.2) then
        call tmesh2d (ntm1,th,t)
      else
        call tmesh3d (ntm1,th,t)
      end if
c
c ****** Apply the enhancement factor to rho.
c
      call magrho (nr,nt,np,th,ph,rho,fac)
c
c ****** Convert the Carrington longitude to radians.
c
      emu=crlong*pi/180.
c
c ****** Convert the solar P and B0 angles to radians.
c
      pangle=pangle*pi/180.
      b0angle=b0angle*pi/180.
c
      sp=sin(pangle)
      cp=cos(pangle)
      sb=sin(b0angle)
      cb=cos(b0angle)
c
c ****** Make sure that the maximum tau does not exceed pi/2.
c
      tmax=amin1(abs(tmax),.5*pi)
      tmn=-tmax
      tmx=tmax
c
c ****** Caculate the integrated line-of-sight brightness.
c
      call findb  (nrpb, ntpb, r, t, tmn, tmx, ntau, bright)
c
c ****** Calculate the polarization brightness.
c
      call findpb (nrpb,ntpb,r,t,tmn,tmx,ntau,pb)
c
c ****** Multiply by the requested radial power.
c
      call pmult (nrpb,ntpb,r,rocc,power,bright)
      call pmult (nrpb,ntpb,r,rocc,power,pb)
c
c ****** Apply the vignetting function (if it was specified).
c
      lf=lenstr(vffile)
c
      if (vffile.ne.' ') then
c
        call rdsds (fmt,vffile,1,ndimvf,idimvf,
     &              ipvf,ifscale,iprvf,ierr)
c
        if (ierr.ne.0) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### Could not read the vignetting function.'
          write (*,*) 'IERR (from RDSDS) = ',ierr
          write (*,*) 'File name: ',vffile(1:lf)
          call exit (1)
        end if
c
        if (ndimvf.ne.1) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### The vignetting function file does'//
     &                ' not contain a 1D field.'
          write (*,*) 'Number of dimensions = ',ndimvf
          write (*,*) 'File name: ',vffile(1:lf)
          call exit (1)
        end if
c
        if (ifscale.ne.1) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### The vignetting function file does'//
     &                ' not contain a scale.'
          write (*,*) 'File name: ',vffile(1:lf)
          call exit (1)
        end if
c
        nrvf=idimvf
c
        if (nrvf.lt.2) then
          write (*,*)
          write (*,*) '### ERROR in ',cname,':'
          write (*,*) '### The vignetting function file has'//
     &                ' less than two points.'
          write (*,*) 'Number of points = ',nrvf
          write (*,*) 'File name: ',vffile(1:lf)
          call exit (1)
        end if
c
c ****** Apply the vignetting function.
c
        call vignette (nrpb,ntpb,r,nrvf,rvf,vf,bright)
        call vignette (nrpb,ntpb,r,nrvf,rvf,vf,pb)
c
      end if
c
c ****** Write PB.
c
      idim(1)=nrpb
      idim(2)=ntpb
      ips(1)=ipr
      ips(2)=ipt
c
      lf=lenstr(outfile)
c
      call wrsds (fmt, brtfile, 2, idim, bright, 1, ips, ierr)
      call wrsds (fmt, outfile, 2, idim, pb,     1, ips, ierr)
c
      if (ierr.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in ',cname,':'
        write (*,*) '### Could not write PB.'
        write (*,*) 'IERR (from WRSDS) = ',ierr
        write (*,*) 'File name: ',outfile(1:lf)
        call exit (1)
      end if
c
      call exit (0)
c
c ****** Error exit.
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in ',cname,':'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      call exit (1)
c
      end
c#######################################################################
      subroutine rmesh (nrm1,rh,r)
c
c-----------------------------------------------------------------------
c
      real rh(nrm1+1),r(nrm1)
c
c-----------------------------------------------------------------------
c
      write (*,*) 'rmesh> rh range: ', rh (1), rh (nrm1+1)

      do i=1,nrm1
        r(i)=.5*(rh(i)+rh(i+1))
      enddo
c
      return
      end
c#######################################################################
      subroutine tmesh2d (ntm1,th,t)
c
c-----------------------------------------------------------------------
c
      real th(ntm1+1),t(ntm1)
c
c-----------------------------------------------------------------------
c
      do j=1,ntm1
        t(j)=.5*(th(j)+th(j+1))
      enddo
c
      return
      end
c#######################################################################
      subroutine tmesh3d (ntm1,th,t)
c
c-----------------------------------------------------------------------
c
      real th(ntm1+1),t(2*ntm1-1)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      write (*,*) 'tmesh3d> th range: ', th (1), th (nrm1+1)

      do j=1,ntm1
        t(j)=.5*(th(j)+th(j+1))
      enddo
c
      do j=2,ntm1-1
        t(2*ntm1-j)=2.*pi-t(j)
      enddo
      t(2*ntm1-1)=2.*pi-t(1)
c
      return
      end
c#######################################################################
      subroutine magrho (nr,nt,np,th,ph,rho,fac)
c
c-----------------------------------------------------------------------
c
c ****** Enhance the density perturbations from a spherically
c ****** symmetric profile by the factor FAC.
c
c-----------------------------------------------------------------------
c
      real th(nt),ph(np),rho(nr,nt,np)
c
c-----------------------------------------------------------------------
c
      if (fac.eq.1.) return
c
      do i=1,nr
        avg=0.
        area=0.
        do j=2,nt-1
          st=sin(th(j))
          do k=2,np-1
            pm=(ph(k  )-ph(k-1))
            pp=(ph(k+1)-ph(k  ))
            da=(pp-pm)*st
            avg=avg+rho(i,j,k)*da
            area=area+da
          enddo
        enddo
        avg=avg/area
        do j=1,nt
          do k=1,np
            rho(i,j,k)=avg+fac*(rho(i,j,k)-avg)
          enddo
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine findb (nr, nt, r, t, tmin, tmax, ntau, bright)
c
c-----------------------------------------------------------------------
c
c ****** Find the integrated line-of-sight brightness.
c
c-----------------------------------------------------------------------
c
      real r (nr), t (nt), bright (nr, nt)
c
c-----------------------------------------------------------------------
c
      do i = 1, nr
        rv = r (i)
        do j = 1, nt
          tv = t (j)
          bright (i, j) = sbright (rv, tv, tmin, tmax, ntau)
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine findpb (nr,nt,r,t,tmin,tmax,ntau,pb)
c
c-----------------------------------------------------------------------
c
c ****** Find the polarization brightness.
c
c-----------------------------------------------------------------------
c
      real r(nr),t(nt),pb(nr,nt)
c
c-----------------------------------------------------------------------
c
      do i=1,nr
        rv=r(i)
        do j=1,nt
          tv=t(j)
          pb(i,j)=spb(rv,tv,tmin,tmax,ntau)
        enddo
      enddo
c
      return
      end
c#######################################################################
      subroutine pmult (nr,nt,r,rocc,power,pb)
c
c-----------------------------------------------------------------------
c
c ****** Multiply by the specified radial power.
c
c-----------------------------------------------------------------------
c
      real r(nr),pb(nr,nt)
c
c-----------------------------------------------------------------------
c
      do i=1,nr
c
        rv=r(i)
c
        if (rocc.ge.1.) then
          if (rv.ge.rocc) then
            occ=1.
          else
            occ=0.
          end if
        else
          occ=1.
        end if
c
        do j=1,nt
          pb(i,j)=occ*pb(i,j)*rv**power
        enddo
c
      enddo
c
      return
      end
c#######################################################################
      subroutine vignette (nrpb,ntpb,r,nrvf,rvf,vf,pb)
c
c-----------------------------------------------------------------------
c
c ****** Apply the vignetting function.
c
c-----------------------------------------------------------------------
c
      real r(nrpb),pb(nrpb,ntpb)
      real rvf(nrvf),vf(nrvf)
c
c-----------------------------------------------------------------------
c
c ****** Check that the vignetting function scale is monotonically
c ****** increasing.
c
      do i=1,nrvf-1
        if (rvf(i+1).le.rvf(i)) then
          write (*,*)
          write (*,*) '### ERROR in VIGNETTE:'
          write (*,*) '### The vignetting function scale is not'//
     &                ' monotonically increasing.'
          call exit (1)
        end if
      enddo
c
      do i=1,nrpb
        rv=r(i)
        if (rv.le.rvf(1)) then
          vfv=vf(1)
        else if (rv.ge.rvf(nrvf)) then
          vfv=vf(nrvf)
        else
          call interp (rvf,nrvf,rv,ivf,avf)
          vfv=(1.-avf)*vf(ivf)+avf*vf(ivf+1)
        end if
        do j=1,ntpb
          pb(i,j)=vfv*pb(i,j)
        enddo
      enddo
c
      return
      end
c#######################################################################
      function sbright (rho, eta, tmin, tmax, nt)
c
c-----------------------------------------------------------------------
c
c ****** Calculate brightness [adapted from F. Bagenal].
c
c-----------------------------------------------------------------------
c
c ****** RHO is the plane-of-sky radial distance
c ****** in units of R_sun.
c
c ****** ETA is the plane-of-sky polar angle in radians
c ****** (solar North pole ==> ETA=0 and ETA=2*pi).
c ****** Angles with ETA between 0 and pi specify the
c ****** right limb, and angles with ETA between pi and 2*pi
c ****** specify the left limb.
c
c ****** EMU is the central meridian longitude (Carrington
c ****** rotation longitude) in radians.
c
c ****** TMIN and TMAX are the limits in the angle tau for which
c ****** to integrate along the line of sight.
c ****** They should not specify an interval outside the
c ****** interval (-pi/2 to pi/2).
c
c ****** NT is the number of points to use in this
c ****** integration (with uniform tau interval).
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dtau = (tmax - tmin) / nt
c
      tau = tmin - 0.5 * dtau
      brt = 0.0
      do i = 1, nt
        tau = tau + dtau
        ct = cos (tau)
        r = rho / ct
        theta = acos (cos (eta) * ct)
        se = sin (eta)
        if (se .eq. 0.0) then
          if (tau .ge. 0.0) then
            zeta = 0.5 * pi
          else
            zeta = -0.5 * pi
          end if
        else
          tt = tan (tau)
          zeta = atan2 (tt, se)
        end if
        phi = 0.5 * pi + zeta
        phi = amod (phi, 2.0 * pi)
        if (phi .lt. 0.0) phi = phi + 2.0 * pi
        rhov = rhoval (r, theta, phi)
        brt = brt + rho * rhov * bfun (r) * dtau
      enddo
c
      sbright = brt
c
      return
      end
c#######################################################################
      function spb (rho,eta,tmin,tmax,nt)
c
c-----------------------------------------------------------------------
c
c ****** Calculate polarization brightness [adapted from F. Bagenal].
c
c-----------------------------------------------------------------------
c
c ****** RHO is the plane-of-sky radial distance
c ****** in units of R_sun.
c
c ****** ETA is the plane-of-sky polar angle in radians
c ****** (solar North pole ==> ETA=0 and ETA=2*pi).
c ****** Angles with ETA between 0 and pi specify the
c ****** right limb, and angles with ETA between pi and 2*pi
c ****** specify the left limb.
c
c ****** EMU is the central meridian longitude (Carrington
c ****** rotation longitude) in radians.
c
c ****** TMIN and TMAX are the limits in the angle tau for which
c ****** to integrate along the line of sight.
c ****** They should not specify an interval outside the
c ****** interval (-pi/2 to pi/2).
c
c ****** NT is the number of points to use in this
c ****** integration (with uniform tau interval).
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dtau=(tmax-tmin)/nt
c
      tau=tmin-.5*dtau
      pb=0.
      do i=1,nt
        tau=tau+dtau
        ct=cos(tau)
        r=rho/ct
        theta=acos(cos(eta)*ct)
        se=sin(eta)
        if (se.eq.0.) then
          if (tau.ge.0.) then
            zeta=.5*pi
          else
            zeta=-.5*pi
          end if
        else
          tt=tan(tau)
          zeta=atan2(tt,se)
        end if
        phi=.5*pi+zeta
        phi=amod(phi,2.*pi)
        if (phi.lt.0.) phi=phi+2.*pi
        rhov=rhoval(r,theta,phi)
        pb=pb+rho*rhov*cfun(r)*dtau
      enddo
c
      spb=pb
c
      return
      end
c#######################################################################
      function bfun (rr)
c
c-----------------------------------------------------------------------
c
c ****** Coronal light scattering function [Billings 1966].
c
c ****** R is the radius in units of R-sun.
c
c-----------------------------------------------------------------------
c
      parameter (a1=8.69e-7)
      parameter (a2=0.37)
      parameter (a3=0.63)
c
c-----------------------------------------------------------------------
c
      r = amax1 (1.0001, rr)
c
      cfunc = 4.0 / 3.0 - sqrt (1.0 - 1.0 / r / r)
     &      - ((1.0 - 1.0 / r / r) ** (1.5)) / 3.0

      df    = (r - 1.0 / r) * (5.0 - 1.0 / r / r)
     &      * log ((1.0 + 1.0 / r) / sqrt (1.0 - 1.0 / r / r))
      dfunc = 1.0 / 8.0 * (5.0 + 1.0 / r / r - df)

      bfun  = a1 * (a2 * cfunc + a3 * dfunc)
c
      return
      end
c#######################################################################
      function cfun (rr)
c
c-----------------------------------------------------------------------
c
c ****** Coronal light scattering function [Billings 1966].
c
c ****** R is the radius in units of R-sun.
c
c-----------------------------------------------------------------------
c
      parameter (a1=8.69e-7)
      parameter (a2=0.37)
      parameter (a3=0.63)
c
c-----------------------------------------------------------------------
c
      r=amax1(1.0001,rr)
c
      a=1./r/r
      ra=1./r
      r1a=sqrt(1.-a)
      afunc=r1a*a
      b1=(1.-a)/ra
      b2=1.+3.*a
      b3=alog((1.+ra)/r1a)
      bf=b1*b2*b3
      b4=1.-3.*a
      bfunc=-1./8.*(b4-bf)
      cfun=a1*(a2*afunc+a3*bfunc)
c
      return
      end
c#######################################################################
      function rhoval (rv,tv,pv)
c
c-----------------------------------------------------------------------
c
      common /fldcom/ nr,nt,np,iprh,ipth,ipph,iprho,
     &                nrm1,ntm1,ipr,ipt,ippb, ipbright
c
c-----------------------------------------------------------------------
c
      pointer (iprh,rh),(ipth,th),(ipph,ph)
      pointer (iprho,rho)
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
c ****** Sin and cos of the solar P and B0 angles.
c
      common /pb0com/ sp,cp,sb,cb
c
c-----------------------------------------------------------------------
c
c ****** Central meridian longitude.
c
      common /cmdcom/ emu
c
c-----------------------------------------------------------------------
c
c ****** Transform to Cartesian coordinates.
c
      call s2c (rv,tv,pv,x0,y0,z0)
c
c ****** Apply the solar P angle transformation.
c
      x1=x0
      y1= cp*y0+sp*z0
      z1=-sp*y0+cp*z0
c
c ****** Apply the solar B0 angle transformation.
c
      x2= cb*x1-sb*z1
      y2=y1
      z2= sb*x1+cb*z1
c
c ****** Transform back to spherical coordinates.
c
      call c2s (x2,y2,z2,rrv,ttv,ppv)
c
c ****** Transform to the appropriate central meridian longitude.
c
      ppv=ppv+emu
      ppv=amod(ppv,2.*pi)
      if (ppv.lt.0.) ppv=ppv+2.*pi
c
      if (np.eq.1) then
        rhoval=f2d(nr,nt,rh,th,rho,rrv,ttv)
      else
        rhoval=f3d(nr,nt,np,rh,th,ph,rho,rrv,ttv,ppv)
      end if
c
      return
      end
c#######################################################################
      subroutine c2s (x,y,z,r,t,p)
c
c-----------------------------------------------------------------------
c
c ****** Convert from Cartesian coordinates (X,Y,Z)
c ****** to spherical coordinates (R,T,P).
c
c ****** This routine returns T and P in radians, in the
c ****** following range:
c
c          0. .le. t .le. pi
c          0. .le. p .lt. 2.*pi
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
      parameter (pid2=.5*pi)
      parameter (pi2=2.*pi)
c
c-----------------------------------------------------------------------
c
      r=sqrt(x**2+y**2+z**2)
c
      if (r.eq.0.) then
        t=0.
      else
        t=acos(z/r)
      end if
c
      if (x.eq.0.) then
        if (y.ge.0.) then
          p=pid2
        else
          p=-pid2
        end if
      else
        p=atan2(y,x)
      end if
      if (p.lt.0.) p=p+pi2
c
      return
      end
c#######################################################################
      subroutine s2c (r,t,p,x,y,z)
c
c-----------------------------------------------------------------------
c
c ****** Convert from spherical coordinates (R,T,P) 
c ****** to Cartesian coordinates (X,Y,Z).
c
c ****** This routine assumes that T and P are in radians.
c
c-----------------------------------------------------------------------
c
      st=sin(t)
      x=r*st*cos(p)
      y=r*st*sin(p)
      z=r*cos(t)
c
      return
      end
c#######################################################################
      function f2d (nx,ny,x,y,f,xv,yv)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate the data in array F(NX,NY), with scales
c ****** in X(NX) and Y(NY), to the position (XV,YV).
c
c ****** Return the interpolated value as the function value.
c
c-----------------------------------------------------------------------
c
      real x(nx),y(ny),f(nx,ny)
c
c-----------------------------------------------------------------------
c
      xxv=amax1(xv,x(1))
      xxv=amin1(xxv,x(nx))
      yyv=amax1(yv,y(1))
      yyv=amin1(yyv,y(ny))
c
      call interp (x,nx,xxv,i,ax)
      call interp (y,ny,yyv,j,ay)
c
      f2d= (1.-ax)*((1.-ay)*f(i  ,j  )+ay*f(i  ,j+1))
     &    +    ax *((1.-ay)*f(i+1,j  )+ay*f(i+1,j+1))
c
      return
      end
c#######################################################################
      function f3d (nx,ny,nz,x,y,z,f,xv,yv,zv)
c
c-----------------------------------------------------------------------
c
c ****** Interpolate the data in array F(NX,NY,NZ), with scales
c ****** in X(NX), Y(NY), and Z(NZ), to the position (XV,YV,ZV).
c
c ****** The Z coordinate is assumed to be periodic, with a
c ****** repetition length of 2*pi.
c
c ****** Return the interpolated value as the function value.
c
c-----------------------------------------------------------------------
c
      real x(nx),y(ny),z(nz),f(nx,ny,nz)
c
c-----------------------------------------------------------------------
c
      xxv=amax1(xv,x(1))
      xxv=amin1(xxv,x(nx))
      yyv=amax1(yv,y(1))
      yyv=amin1(yyv,y(ny))
      zv = amax1 (zv, z (1))
      zv = amin1 (zv, z (nz))
c
      call interp (x,nx,xxv,i,ax)
      call interp (y,ny,yyv,j,ay)
      call interp (z,nz,zv,k,az)
c
      f3d= (1.-az)*((1.-ax)*((1.-ay)*f(i  ,j  ,k  )+ay*f(i  ,j+1,k  ))
     &    +             ax *((1.-ay)*f(i+1,j  ,k  )+ay*f(i+1,j+1,k  )))
     &    +    az *((1.-ax)*((1.-ay)*f(i  ,j  ,k+1)+ay*f(i  ,j+1,k+1))
     &    +             ax *((1.-ay)*f(i+1,j  ,k+1)+ay*f(i+1,j+1,k+1)))
c
      return
      end
c#######################################################################
      subroutine interp (x,n,xv,i,alpha)
c
c-----------------------------------------------------------------------
c
c ****** Get interpolation factor ALPHA and table index I.
c
c ****** This routine does not do the actual interpolation.  Use the
c ****** returned values of I and ALPHA to get the interpolated value.
c
c-----------------------------------------------------------------------
c
      real x(n)
c
c-----------------------------------------------------------------------
c
      do 100 i=1,n-1
        if (xv.ge.x(i).and.xv.le.x(i+1)) then
          alpha=(xv-x(i))/(x(i+1)-x(i))
          go to 200
        end if
  100 continue
c
c ****** Value not found --- signal error and stop.
c
      write (*,*)
      write (*,*) '### ERROR in INTERP:'
      write (*,*) 'Abscissa not found in table.'
      write (*,*) 'Abscissa requested = ',xv
      write (*,*)
      call exit (1)
c
  200 continue
c
      return
      end
c#######################################################################
      subroutine rdsds (fmt,fname,mxdim,ndim,idim,
     &                  ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set.
c
c ****** Use routine RDTXT or RDHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in RDSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine rdtxt (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from a text file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    Text data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'r')
c
c ****** Get the number of dimensions.
c
      call rdint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 910
c
      if (ndim.lt.1.or.ndim.gt.mxdim) then
        write (*,*)
        write (*,*) '### ERROR in RDTXT:'
        write (*,*) '### Invalid number of dimensions in file.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'NDIM = ',ndim
        write (*,*) 'MXDIM = ',mxdim
        ierr=1
        return
      end if
c
c ****** Read the dimensions.
c
      do 100 i=1,ndim
        call rdint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 910
        if (idim(i).le.0) go to 920
  100 continue
c
c ****** Check if the scales are present.
c
      call rdint (1,1,ifscale,ierr)
      if (ierr.ne.0) go to 910
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 300 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          call rdfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 910
  300   continue
      end if
c
c ****** Read the array.
c
      call rdfp (1,nwdf,f,ierr)
      if (ierr.ne.0) go to 910
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Error while reading text data.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=3
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDTXT:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine rdint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of INTEGER data into array I from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) i
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read N words of REAL data into array F from unit IUN
c ****** using a free format text read.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      read (iun,*,err=100,end=100) f
c
      return
c
  100 continue
c
c ****** Error in reading the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine rdhdf (fname,mxdim,ndim,idim,ipf,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Read a scientific data set from an HDF file.
c
c-----------------------------------------------------------------------
c
c ****** This routine allocates the required memory and returns
c ****** pointers to the data and scales arrays.
c
c-----------------------------------------------------------------------
c
c ****** Input arguments:
c
c          FNAME   : [character*(*)]
c                    HDF data file name.
c
c          MXDIM   : [integer]
c                    Maximum number of dimensions in the data set
c                    that are allowed.
c
c ****** Output arguments:
c
c          NDIM    : [integer]
c                    Number of dimensions found in the data set.
c
c          IDIM    : [integer array, dimension MXDIM]
c                    NDIM dimensions of data set.
c
c          IPF     : [integer]
c                    Pointer to the data array.
c
c          IFSCALE : [integer]
c                    Flag to indicate the presence of scales (axes)
c                    in the data set.  IFSCALE=0 means that scales
c                    were not found.  IFSCALE=1 means that scales
c                    were found.
c
c          IPS     : [integer array, dimension MXDIM]
c                    When IFSCALE=1, this array contains NDIM
c                    pointers to the scale arrays.
c
c          IERR    : [integer]
c                    IERR=0 is returned if the data set was read
c                    successfully.  Otherwise, IERR is set to a
c                    nonzero value.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      pointer (ipf,f)
      integer mxdim,ndim,idim(mxdim),ifscale,ips(mxdim),ierr
c
c-----------------------------------------------------------------------
c
c ****** Roving pointer.
c
      pointer (ipx,x)
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dsgdims,dsgdisc,dsgdata
      external dsgdims,dsgdisc,dsgdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
c ****** Get the dimensions of the SDS.
c
      lf=lenstr(fname)
      iret=dsgdims(fname(1:lf),ndim,idim,mxdim)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while retrieving HDF SDS dimensions.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDIMS) = ',iret,']'
        ierr=1
        return
      end if
c
c ****** Check the dimensions.
c
      do 100 i=1,ndim
        if (idim(i).le.0) go to 910
  100 continue
c
c ****** Calculate the number of words required for the data array.
c
      nwd=idim(1)
      do 200 i=2,ndim
        nwd=nwd*idim(i)
  200 continue
      nwdf=nwd
c
c ****** Allocate the memory for the array.
c
      nwd=nwdf
      ipf=igetmem(nwd)
      if (ipf.eq.0) go to 900
c
c ****** Check if all scales are present.
c ****** (Use array F for temporary storage.)
c
      nscal=0
      do 300 i=1,ndim
        iret=dsgdisc(i,idim(i),f)
        if (iret.eq.0) nscal=nscal+1
  300 continue
c
      if (nscal.eq.ndim) then
        ifscale=1
      else
        ifscale=0
      end if
c
c ****** Allocate memory and get the scales.
c
      if (ifscale.ne.0) then
        do 400 i=1,ndim
          nwd=idim(i)
          ips(i)=igetmem(nwd)
          if (ips(i).eq.0) go to 900
          ipx=ips(i)
          iret=dsgdisc(i,idim(i),x)
  400   continue
      end if
c
c ****** Read the data array.
c
      iret=dsgdata(fname(1:lf),ndim,idim,f)
c
      if (iret.ne.0) then
        write (*,*)
        write (*,*) '### ERROR in RDHDF:'
        write (*,*) '### Error while reading the SDS.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) '[Error return (from DSGDATA) = ',iret,']'
        ierr=3
        return
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Could not allocate the required memory.'
      write (*,*) 'Number of words required = ',nwd
      ierr=2
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in RDHDF:'
      write (*,*) '### Invalid value for dimension.'
      write (*,*) 'Dimension number = ',i
      write (*,*) 'Dimension value = ',idim(i)
      ierr=4
c
      return
      end
c#######################################################################
      subroutine wrsds (fmt,fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set.
c
c ****** Use routine WRTXT or WRHDF, depending on FMT.
c
c-----------------------------------------------------------------------
c
      character*(*) fmt,fname
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (fmt.eq.'text') then
        call wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
      else if (fmt.eq.'hdf') then
        call wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
      else
        write (*,*)
        write (*,*) '### ERROR in WRSDS:'
        write (*,*) '### Invalid file format specified.'
        write (*,*) 'File name: ',fname(1:lf)
        write (*,*) 'Format = ',fmt
        ierr=5
        return
      end if
c
      return
      end
c#######################################################################
      subroutine wrtxt (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to text file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      call ffopen (1,fname,'rw')
c
c ****** Write the number of dimensions.
c
      call wrint (1,1,ndim,ierr)
      if (ierr.ne.0) go to 900
c
c ****** Write the dimensions.
c
      nwd=1
      do 100 i=1,ndim
        call wrint (1,1,idim(i),ierr)
        if (ierr.ne.0) go to 900
        nwd=nwd*idim(i)
  100 continue
c
c ****** Write the scales.
c
      if (ifscale.ne.0) then
        call wrint (1,1,1,ierr)
        if (ierr.ne.0) go to 900
        do 200 i=1,ndim
          ipx=ips(i)
          call wrfp (1,idim(i),x,ierr)
          if (ierr.ne.0) go to 900
  200   continue
      else
        call wrint (1,1,0,ierr)
        if (ierr.ne.0) go to 900
      end if
c
c ****** Write the array.
c
      call wrfp (1,nwd,f,ierr)
      if (ierr.ne.0) go to 900
c
      close (1)
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRTXT:'
      write (*,*) '### Error in writing data to text file.'
      write (*,*) 'File name: ',fname(1:lf)
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrint (iun,n,i,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of INTEGER data into array I from unit IUN
c ****** using a free format text write.
c
c-----------------------------------------------------------------------
c
      integer i(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,*,err=100) i
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrfp (iun,n,f,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write N words of REAL data into array F from unit IUN.
c
c-----------------------------------------------------------------------
c
      real f(n)
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      write (iun,900,err=100) f
c
  900 format (5(1x,1pe13.6))
c
      return
c
  100 continue
c
c ****** Error in writing the data.
c
      ierr=1
c
      return
      end
c#######################################################################
      subroutine wrhdf (fname,ndim,idim,f,ifscale,ips,ierr)
c
c-----------------------------------------------------------------------
c
c ****** Write a scientific data set to HDF file FNAME.
c
c-----------------------------------------------------------------------
c
      character*(*) fname
      real f(*)
      integer ndim,idim(*),ifscale,ips(*)
c
c-----------------------------------------------------------------------
c
c ****** Pointer for scales.
c
      pointer (ipx,x(*))
c
c-----------------------------------------------------------------------
c
c ****** HDF routines.
c
      integer dssdims,dssdisc,dspdata
      external dssdims,dssdisc,dspdata
c
c-----------------------------------------------------------------------
c
      ierr=0
c
      lf=lenstr(fname)
c
      iret=dssdims(ndim,idim)
      if (iret.ne.0) go to 900
c
      if (ifscale.ne.0) then
        do 100 i=1,ndim
          ipx=ips(i)
          iret=dssdisc(i,idim(i),x)
          if (iret.ne.0) go to 910
  100   continue
      end if
c
      iret=dspdata(fname,ndim,idim,f)
      if (iret.ne.0) go to 920
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the dimensions.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDIMS) = ',iret,']'
      ierr=1
      return
c
  910 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not set the scales.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSSDISC) = ',iret,']'
      write (*,*) '[Dimension number = ',i,']'
      ierr=2
      return
c
  920 continue
c
      write (*,*)
      write (*,*) '### ERROR in WRHDF:'
      write (*,*) '### Could not write the data array.'
      write (*,*) 'File name: ',fname(1:lf)
      write (*,*) '[Error return (from DSPDATA) = ',iret,']'
      ierr=3
c
      return
      end
c#######################################################################
      function lenstr (str)
c
c-----------------------------------------------------------------------
c
c ****** Get the length of string STR, ignoring right-filling blanks.
c
c-----------------------------------------------------------------------
c
      character*(*) str
c
c-----------------------------------------------------------------------
c
      do i=len(str),1,-1
        if (str(i:i).ne.' ') go to 100
      enddo
      i=1
  100 continue
c
      lenstr=i
c
      return
      end
c#######################################################################
      integer function igetmem (nwd)
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for an array of length NWD 4-byte words.
c ****** The function returns the pointer to the array.
c
c-----------------------------------------------------------------------
c
c ****** Allocate memory for NWD words.
c
      igetmem=malloc(4*nwd)
c
      return
      end
c#######################################################################
      subroutine ffopen (iun,fname,mode)
c
c-----------------------------------------------------------------------
c
c ****** Open file FNAME and link it to unit IUN.
c
c-----------------------------------------------------------------------
c
c ****** When MODE='r', the file must exist.
c ****** When MODE='w', the file is created.
c ****** When MODE='rw', the file must exist, but can be overwritten.
c
c-----------------------------------------------------------------------
c
      character*(*) fname,mode
c
c-----------------------------------------------------------------------
c
      lf=lenstr(fname)
c
      if (mode.eq.'r') then
        open (iun,file=fname(1:lf),status='old',err=900)
      else if (mode.eq.'rw') then
        open (iun,file=fname(1:lf),err=900)
      else if (mode.eq.'w') then
        open (iun,file=fname(1:lf),status='new',err=900)
      else
        write (*,*)
        write (*,*) '### ERROR in FFOPEN:'
        write (*,*) '### Invalid MODE requested.'
        write (*,*) 'MODE = ',mode
        write (*,*) 'File name = ',fname(1:lf)
        call exit (1)
      end if
c
      return
c
  900 continue
c
      write (*,*)
      write (*,*) '### ERROR in FFOPEN:'
      write (*,*) '### Error while opening file.'
      write (*,*) 'File name = ',fname(1:lf)
      write (*,*) 'MODE =  ',mode
      call exit (1)
      end
c#######################################################################
      subroutine setpar (cname,cvers,cdate)
c
c-----------------------------------------------------------------------
c
c ****** Set parameters from the command line arguments.
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
      character*(*) cname,cvers,cdate
c
c-----------------------------------------------------------------------
c
c ****** Parameters.
c
      character*72 infile,outfile,vffile, brtfile
      common /parcom1/ infile,outfile,vffile, brtfile
c
      character*16 fmt
      common /parcom2/ fmt
c
      common /parcom3/ tmax,ntau,crlong,rocc,fac,power,
     &                 pangle,b0angle
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*72 arg
c
c-----------------------------------------------------------------------
c
      logical set
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax.
c
      call defarg (IGRPK ,'-v',' ',' ')
      call defarg (IGRPKA,'-fmt','hdf','text|hdf')
      call defarg (IGRPKA,'-long','0.','<angle>')
      call defarg (IGRPKA,'-tmax','1.5','<theta_max>')
      call defarg (IGRPKA,'-ntau','100','<#pts>')
      call defarg (IGRPKA,'-rocc','0.','<radius>')
      call defarg (IGRPKA,'-fac','1.','<factor>')
      call defarg (IGRPKA,'-power','3.','<r_power>')
      call defarg (IGRPKA,'-vf','<none>','<file>')
      call defarg (IGRPKA,'-p','0.','<angle>')
      call defarg (IGRPKA,'-b0','0.','<angle>')
      call defarg (IGRPA ,'infile',' ',' ')
      call defarg (IGRPA ,'outfile',' ',' ')
      call defarg (IGRPA ,'brtfile',' ',' ')
c
c ****** Parse the command line.
c
      call parse (ierr)
c
      if (ierr.ne.0) then
c
        write (*,*)
        write (*,*) '### ',cname,' Version ',cvers,' of ',cdate,'.'
        write (*,*) '### Calculate the polarization brightness.'
c
        if (ierr.gt.1) then
          write (*,*)
          write (*,*) errmsg
        end if
c
c ****** Print the usage.
c
        write (*,*)
        write (*,*) 'Usage:'
        write (*,*)
        call wrulin (ulbuf)
c
        write (*,*)
        write (*,*) 'The file format is specified by -fmt text|hdf.'
        write (*,*) 'The default file format is hdf.'
        write (*,*)
        write (*,*) '-long sets the Carrington rotation longitude'//
     &              ' of the central meridian'
        write (*,*) 'in degrees [default=0.].'
        write (*,*)
        write (*,*) '-tmax sets the maximum angle for the'//
     &              ' LOS integration.'
        write (*,*) 'It should not exceed pi/2 [default=1.5].'
        write (*,*) '-ntau sets the number of points to use in the'//
     &              ' LOS integration [default=100].'
        write (*,*)
        write (*,*) '-rocc sets the occulting disk radius'//
     &              ' [default=0.].'
        write (*,*) '-fac sets the enhancement factor for pB'//
     &              ' [default=1.].'
        write (*,*) '-power sets the radial power by which to'//
     &              ' multiply pB [default=3.].'
        write (*,*) '-vf specifies a file which defines a'//
     &              ' radial "vignetting function" by'
        write (*,*) 'which to multiply pB [default=none].'
        write (*,*)
        write (*,*) '-p is the solar P angle (in degrees)'//
     &              ' [default=0.].'
        write (*,*) '-b0 is the solar B0 angle (in degrees)'//
     &              ' [default=0.].'
c
        call exit (1)
c
      end if
c
c ****** Set the parameters.
c
c ****** File format.
c
      call fetcharg ('-fmt',set,arg)
      ls=lenstr(arg)
      fmt=arg(1:ls)
c
c ****** Carrington rotation longitude.
c
      call fetcharg ('-long',set,arg)
      crlong=fpval(arg,'-long')
c
c ****** Maximum tau angle, TMAX.
c
      call fetcharg ('-tmax',set,arg)
      tmax=fpval(arg,'-tmax')
c
c ****** Number of steps, NTAU.
c
      call fetcharg ('-ntau',set,arg)
      ntau=intval(arg,'-ntau')
c
c ****** Occulting disk radius.
c
      call fetcharg ('-rocc',set,arg)
      rocc=fpval(arg,'-rocc')
c
c ****** Enhancememnt factor.
c
      call fetcharg ('-fac',set,arg)
      fac=fpval(arg,'-fac')
c
c ****** Radial power exponent.
c
      call fetcharg ('-power',set,arg)
      power=fpval(arg,'-power')
c
c ****** Vignetting function file name.
c
      call fetcharg ('-vf',set,arg)
      ls=lenstr(arg)
      vffile=arg(1:ls)
      if (vffile.eq.'<none>') then
        vffile=' '
      end if
c
c ****** Solar P angle.
c
      call fetcharg ('-p',set,arg)
      pangle=fpval(arg,'-p')
c
c ****** Solar B0 angle.
c
      call fetcharg ('-b0',set,arg)
      b0angle=fpval(arg,'-b0')
c
c ****** Input file name.
c
      call fetcharg ('infile',set,arg)
      ls=lenstr(arg)
      infile=arg(1:ls)
c
c ****** Brightness file name.
c
      call fetcharg ('brtfile',set,arg)
      ls=lenstr(arg)
      brtfile=arg(1:ls)
c
c
c ****** Output file name.
c
      call fetcharg ('outfile',set,arg)
      ls=lenstr(arg)
      outfile=arg(1:ls)
c
      return
      end
c#######################################################################
      block data bdsntx
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data narg/0/
c
      end
c#######################################################################
      function fpval (avalue,name)
c
c-----------------------------------------------------------------------
c
c ****** Get the value of the floating point number in character
c ****** string AVALUE.
c
c-----------------------------------------------------------------------
c
      character*(*) avalue,name
c
c-----------------------------------------------------------------------
c
      logical iffp
      external iffp
c
c-----------------------------------------------------------------------
c
      ls=lenstr(avalue)
c
      if (.not.iffp(avalue(1:ls),value)) then
        write (*,*)
        write (*,*) '### ERROR in FPVAL:'
        write (*,*) '### Could not interpret a floating point '//
     &              'number while setting: ',name
        write (*,*) 'Invalid format: ',avalue(1:ls)
        call exit (1)
      end if
c
      fpval=value
c
      return
      end
c#######################################################################
      function intval (avalue,name)
c
c-----------------------------------------------------------------------
c
c ****** Get the value of the integer in character string AVALUE.
c
c-----------------------------------------------------------------------
c
      character*(*) avalue,name
c
c-----------------------------------------------------------------------
c
      logical ifint
      external ifint
c
c-----------------------------------------------------------------------
c
      ls=lenstr(avalue)
c
      if (.not.ifint(avalue(1:ls),ivalue)) then
        write (*,*)
        write (*,*) '### ERROR in INTVAL:'
        write (*,*) '### Could not interpret an integer '//
     &              'while setting: ',name
        write (*,*) 'Invalid format: ',avalue(1:ls)
        call exit (1)
      end if
c
      intval=ivalue
c
      return
      end
c#######################################################################
      subroutine wrulin (line)
c
c-----------------------------------------------------------------------
c
c ****** Write the string LINE to the TTY, taking into account
c ****** any carriage returns in the line.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
      ls=lenstr(line)
c
      i0=1
  100 continue
c
c ****** Find the next carriage return.
c
      icr=index(line(i0:ls),char(13))
c
      if (icr.gt.0) then
c
c ****** A CR was found.
c
        if (icr.eq.1) then
          write (*,*)
          i0=i0+1
        else
          write (*,*) line(i0:i0+icr-2)
          i0=i0+icr
        end if
        if (i0.le.ls) go to 100
c
      else
c
c ****** A CR was not found.
c
        write (*,*) line(i0:ls)
c
      end if
c
      return
      end
c#######################################################################
      subroutine parse (ierr)
c
c-----------------------------------------------------------------------
c
c ****** Parse the command line.
c
c-----------------------------------------------------------------------
c
c ****** The syntax for the keyword/argument items can be defined
c ****** by using routine DEFARG.
c
c ****** The usage line is constructed in character string ULBUF.
c ****** Use routine WRULIN to write the usage line, since it may
c ****** contain embedded carriage returns.
c
c ****** On return, IERR=0 indicates that the command line was
c ****** parsed successfully.
c
c ****** IERR=1 indicates that no arguments were present.  This
c ****** is usually used to print the usage line.
c
c ****** IERR=2 indicates that a syntax error occured.
c
c ****** IERR=3 indicates that one or more required arguments
c ****** was not supplied.
c
c ****** When IERR=2 or IERR=3, and error message is put into
c ****** character string ERRMSG.
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
c ****** Command line arguments.
c
      integer iargc
      external iargc,getarg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for an argument.
c
      character*72 arg
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      ierr=0
      errmsg=' '
c
c ****** Construct the usage line in ULBUF.
c
      ix=1
      ulbuf=' '
c
      call getarg (0,arg)
      ls=lenstr(arg)
      call addtobuf (arg(1:ls),ulbuf,ix)
      nc=ix-1
      ncc=min0(nc,6)
c
      i=1
      itried=0
  100 continue
      if (i.le.narg) then
        ix0=ix
        call addtobuf (' ',ulbuf,ix)
        if (info(IXREQ,i).eq.0) then
          call addtobuf ('[',ulbuf,ix)
        end if
        call getstr (info(IXKEY,i),arg)
        ls=lenstr(arg)
        call addtobuf (arg(1:ls),ulbuf,ix)
        if (info(IXNAM,i).gt.0) then
          call addtobuf (' ',ulbuf,ix)
          call getstr (info(IXNAM,i),arg)
          ls=lenstr(arg)
          call addtobuf (arg(1:ls),ulbuf,ix)
        end if
        if (info(IXREQ,i).eq.0) then
          call addtobuf (']',ulbuf,ix)
        end if
        nc=nc+ix-ix0
        if (nc.gt.78.and.itried.eq.0) then
          itried=1
          ix=ix0
          call addtobuf (char(13),ulbuf,ix)
          ulbuf(ix:ix+ncc-1)=' '
          ix=ix+ncc
          nc=ncc
          go to 100
        end if
        itried=0
        i=i+1
        go to 100
      end if
c
c ****** Get the number of command line arguments.
c
      na=iargc()
      if (na.eq.0) then
        ierr=1
        go to 900
      end if
c
      ia=1
  200 continue
c
      ia0=ia
c
c ****** Process arguments with syntax: <kw> <arg>
c
      if (na-ia+1.ge.2) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw> <arg> <arg>
c
      if (na-ia+1.ge.3) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPKAA,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            ia=ia+1
            call getarg (ia,arg)
            ls=lenstr(arg)
            ls=ls+1
            arg(ls:ls)=' '
            ia=ia+1
            call getarg (ia,arg(ls+1:))
            ls=lenstr(arg)
            info(IXARG,iarg)=iputstr(arg(1:ls))
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <kw>
c
      if (na-ia+1.ge.1) then
        call getarg (ia,arg)
        iarg=matchkw(IGRPK,arg)
        if (iarg.gt.0) then
          if (info(IXSET,iarg).eq.0) then
            info(IXARG,iarg)=0
            info(IXSET,iarg)=1
            ia=ia+1
            go to 300
          end if
        end if
      end if
c
c ****** Process arguments with syntax: <arg>
c
      if (na-ia+1.ge.1) then
        iarg=nxtarg(IGRPA)
        if (iarg.gt.0) then
          call getarg (ia,arg)
          ls=lenstr(arg)
          info(IXARG,iarg)=iputstr(arg(1:ls))
          info(IXSET,iarg)=1
          ia=ia+1
          go to 300
        end if
      end if
c
  300 continue
c
c ****** Check that an argument was found.
c
      if (ia.eq.ia0) then
        ierr=2
        go to 900
      end if
c
c ****** Keep processing arguments until done.
c
      if (na-ia+1.gt.0) go to 200
c
c ****** Check that the required arguments were supplied.
c
      do 400 i=1,narg
        if (info(IXREQ,i).ne.0.and.info(IXSET,i).eq.0) then
          ierr=3
          go to 900
        end if
  400 continue
c
      return
c
c ****** Error exit.
c
  900 continue
c
      if (ierr.eq.2) then
        errmsg='### Syntax error.'
      else if (ierr.eq.3) then
        errmsg='### A required argument was not supplied.'
      end if
c
      return
      end
c#######################################################################
      subroutine defarg (igroup,kw,default,argname)
c
c-----------------------------------------------------------------------
c
c ****** Define the syntax for a command line argument item.
c
c-----------------------------------------------------------------------
c
c ****** IGROUP is the syntax group index.
c ****** KW is the keyword.
c ****** DEFAULT is the default value of the argument.
c ****** ARGNAME is the name of the argument (for use in
c ****** constructing the usage line).
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw,default,argname
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
c ****** Check the group index.
c
      if (igroup.lt.0.or.igroup.gt.ngroup) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Invalid group index specified.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Check for a null keyword.
c
      if (kw.eq.' ') then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### The keyword is null.'
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
c ****** Increment the argument counter.
c
      if (narg.ge.mxarg) then
        write (*,*)
        write (*,*) '### ERROR in DEFARG:'
        write (*,*) '### Exceeded number of allowed arguments.'
        write (*,*) 'Maximum number of arguments = ',mxarg
        write (*,*) 'Group index = ',igroup
        call exit (1)
      end if
c
      narg=narg+1
c
c ****** Store the group index and keyword.
c
c ****** For group IGRPA (single arguments), the name of the
c ****** argument is stored as the "keyword".
c
      info(IXGRP,narg)=igroup
      ls=lenstr(kw)
      info(IXKEY,narg)=iputstr(kw(1:ls))
c
c ****** Initialize the flag which indicates whether an argument
c ****** has been set.
c
      info(IXSET,narg)=0
c
c ****** If a default argument was supplied, the argument
c ****** does not have to be set.  Use DEFAULT=' ' to
c ****** indicate that an argument is required.
c
c ****** If a default argument has been supplied, store it and
c ****** put its pointer in info(IXARG,narg).  If there is no
c ****** default, set info(IXARG,narg)=0.
c
c ****** Since group IGRPK doesn't have an argument,
c ****** DEFAULT is ignored for this group.
c
      if (igroup.eq.IGRPK) then
        info(IXREQ,narg)=0
        info(IXARG,narg)=0
      else
        if (default.eq.' ') then
          info(IXREQ,narg)=1
          info(IXARG,narg)=0
        else
          info(IXREQ,narg)=0
          ls=lenstr(default)
          info(IXARG,narg)=iputstr(default(1:ls))
        end if
      end if
c
c ****** Store the argument name.  For groups IGRPK (keywords) 
c ****** and IGRPA (single arguments), there is no argument name,
c ****** so ARGNAME is ignored.
c
      if (igroup.eq.IGRPK.or.igroup.eq.IGRPA) then
        info(IXNAM,narg)=0
      else
        ls=lenstr(argname)
        info(IXNAM,narg)=iputstr(argname(1:ls))
      end if
c
      return
      end
c#######################################################################
      subroutine fetcharg (kw,set,arg)
c
c-----------------------------------------------------------------------
c
c ****** Fetch the argument corresponding to keyword KW.
c
c-----------------------------------------------------------------------
c
c ****** If the corresponding argument was set, return SET=.true.,
c ****** and copy the argument to ARG.
c
c ****** If the argument was not set, return SET=.false..
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
      logical set
      character*(*) kw,arg
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
c ****** Find the index of the argument corresponding to keyword KW.
c
      do 100 i=narg,1,-1
        call getstr (info(IXKEY,i),tkw)
        if (tkw.eq.kw) then
          iarg=i
          go to 200
        end if
  100 continue
c
      ls=lenstr(kw)
      write (*,*)
      write (*,*) '### ERROR in FETCHARG:'
      write (*,*) '### Requested keyword could not be matched.'
      write (*,*) 'Keyword = ',kw(1:ls)
      call exit (1)
c
  200 continue
c
      set=info(IXSET,iarg).ne.0
      if (info(IXARG,iarg).gt.0) then
        call getstr (info(IXARG,iarg),arg)
      else
        arg=' '
      end if
c
      return
      end
c#######################################################################
      function matchkw (igroup,kw)
c
c-----------------------------------------------------------------------
c
c ****** Match keyword KW against the list of keywords in
c ****** group IGROUP.
c
c ****** If found, set the function value to the corresponding
c ****** argument number.  Otherwise, set MATCHKW=0.
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
      integer igroup
      character*(*) kw
c
c-----------------------------------------------------------------------
c
c ****** Temporary storage for a keyword.
c
      character*72 tkw
c
c-----------------------------------------------------------------------
c
      do 100 i=narg,1,-1
        if (info(IXGRP,i).eq.igroup) then
          call getstr (info(IXKEY,i),tkw)
          if (tkw.eq.kw) then
            matchkw=i
            return
          end if
        end if
  100 continue
c
      matchkw=0
c
      return
      end
c#######################################################################
      function nxtarg (igroup)
c
c-----------------------------------------------------------------------
c
c ****** Find the position of the next argument in group IGROUP
c ****** which has not been set.
c
c-----------------------------------------------------------------------
c
c ****** If an empty slot is found, set the function value
c ****** to the corresponding argument number.
c
c ****** Otherwise, set NXTARG=0.
c
c-----------------------------------------------------------------------
c
      include 'syntax.inc'
c
c-----------------------------------------------------------------------
c
      do 100 i=1,narg
        if (info(IXGRP,i).eq.igroup) then
          if (info(IXSET,i).eq.0) then
            nxtarg=i
            return
          end if
        end if
  100 continue
c
      nxtarg=0
c
      return
      end
c#######################################################################
      function iputstr (line)
c
c-----------------------------------------------------------------------
c
c ****** Put the string in LINE into internal storage, and return
c ****** a pointer to it as the function value.
c
c ****** The string can be retrieved by calling routine GETSTR
c ****** with the returned pointer.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
      ls=len(line)
c
c ****** Check that the string is not empty.
c
      if (ls.le.0) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Trying to store a zero-length string.'
        call exit (1)
      end if
c
c ****** Check that a free pointer exists.
c
      if (nstr.ge.mxstr) then
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for pointers.'
        write (*,*) 'Maximum number of pointers = ',mxstr
        call exit (1)
      end if
c
      nstr=nstr+1
c
c ****** Check that there is enough room to store the string.
c
      ix1=ix+ls-1
      if (ix1.gt.LENBUF) then
        nb=ix1-LENBUF
        write (*,*)
        write (*,*) '### ERROR in IPUTSTR:'
        write (*,*) '### Exceeded internal storage for strings.'
        write (*,*) 'Number of additional bytes needed = ',nb
        call exit (1)
      end if
c
      strbuf(ix:ix1)=line
c
c ****** Set IX to point to the next available character.
c
      ix=ix1+1
      indx(nstr)=ix
c
c ****** Return the pointer to the string as the function value.
c
      iputstr=nstr
c
      return
      end
c#######################################################################
      block data bdstr
c
c-----------------------------------------------------------------------
c
      parameter (mxstr=100)
      common /pstr1/ nstr,ix,indx(mxstr)
c
c-----------------------------------------------------------------------
c
c ****** Initialization.
c
      data nstr/0/
      data ix/1/
c
      end
c#######################################################################
      subroutine getstr (ip,line)
c
c-----------------------------------------------------------------------
c
c ****** Get the string associated with pointer IP and put it
c ****** into LINE.
c
c ****** This routine retrieves strings stored with routine IPUTSTR.
c
c-----------------------------------------------------------------------
c
      character*(*) line
c
c-----------------------------------------------------------------------
c
c ****** Storage for pointers and strings.
c
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr,ix,indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
c
c-----------------------------------------------------------------------
c
c ****** Check that the pointer is valid.
c
      if (ip.lt.0.or.ip.gt.nstr) then
        write (*,*)
        write (*,*) '### ERROR in GETSTR:'
        write (*,*) '### Invalid string pointer.'
        write (*,*) 'Pointer = ',ip
        call exit (1)
      end if
c
c ****** Retrieve the string.
c
      if (ip.eq.1) then
        i0=1
      else
        i0=indx(ip-1)
      end if
      i1=indx(ip)-1
c
      line=strbuf(i0:i1)
c
      return
      end
c#######################################################################
      subroutine addtobuf (line,buf,ix)
c
c-----------------------------------------------------------------------
c
c ****** Add the string in LINE to the buffer BUF, starting
c ****** at character position IX in BUF.
c
c-----------------------------------------------------------------------
c
c ****** When COMPRESS=.true., multiple spaces in LINE are
c ****** converted into a single space.
c
c-----------------------------------------------------------------------
c
      logical COMPRESS
      parameter (COMPRESS=.false.)
c
c-----------------------------------------------------------------------
c
      character*(*) line,buf
c
c-----------------------------------------------------------------------
c
      logical psp
c
c-----------------------------------------------------------------------
c
      lbuf=len(buf)
      ll=len(line)
c
      psp=.false.
      do 100 i=1,ll
c
        if (compress) then
c
c ****** Compress multiple spaces into a single space.
c
          if (line(i:i).eq.' ') then
            if (psp) go to 100
            psp=.true.
          else
            psp=.false.
          end if
        end if
c
c ****** Check for a buffer overflow.
c
        if (ix.gt.lbuf) then
          write (*,*)
          write (*,*) '### ERROR in ADDTOBUF:'
          write (*,*) '### Internal buffer overflow.'
          write (*,*) '[This is an internal error.]'
          call exit (1)
        end if
c
c ****** Write the character to the buffer.
c
        buf(ix:ix)=line(i:i)
        ix=ix+1
c
  100 continue
c
      return
      end
c#######################################################################
      logical function iffp (alpha,value)
c
c-----------------------------------------------------------------------
c
c ****** Determine if ALPHA represents a floating point number;
c ****** if so, return its value in VALUE.
c
c-----------------------------------------------------------------------
c
c ****** Set IFFP=.TRUE. if ALPHA contains an alphanumeric
c ****** string with the following format:
c
c       ALPHA = '[A][B...B][.][B...B][e[A]B[B...B]]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c ****** The exponent may be denoted by a lower or upper case e.
c ****** The mantissa must have at least one digit, and the
c ****** the exponent, if present, must have between 1 and 3 digits.
c ****** Otherwise, set IFFP=.FALSE.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      logical ifpoint,ifexp
c
c-----------------------------------------------------------------------
c
      character*7 fmt
c
c-----------------------------------------------------------------------
c
      iffp=.false.
      ifpoint=.false.
      ifexp=.false.
      nmant=0
      nexp=0
c
      do 100 k=1,len(alpha)
      i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
      if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
      if (i.ge.48.and.i.le.57) then
c
c ****** Count digits in mantissa and exponent.
c
        if (ifexp) then
          nexp=nexp+1
        else
          nmant=nmant+1
        end if
        go to 100
c
      end if
c
c ****** Check for a decimal point.
c
      if (.not.ifpoint.and.i.eq.46) then
c
c ****** Check that we are in the mantissa.
c
        if (.not.ifexp) then
          ifpoint=.true.
          go to 100
        end if
c
      end if
c
c ****** Check for an exponent.
c
      if (.not.ifexp.and.(i.eq.101.or.i.eq.69)) then
        ifexp=.true.
        ke=k
        go to 100
      end if
c
c ****** Check for an exponent sign.
c
      if (ifexp.and.k.eq.ke+1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Failed check: fall through here.
c
      iffp=.false.
c
      return
c
  100 continue
c
c ****** Final check of validity: check number of digits in
c ****** mantissa and exponent.
c
      if (nmant.ge.1) iffp=.true.
      if (ifexp.and.(nexp.lt.1.or.nexp.gt.3)) iffp=.false.
c
c ****** Obtain its numeric value.
c
      fmt='(f  .0)'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (iffp) read (alpha,fmt) value
c
      return
      end
c#######################################################################
      logical function ifint (alpha,ivalue)
c
c-----------------------------------------------------------------------
c
c ****** If ALPHA represents an integer, return IFINT=.true., and
c ****** put its value into IVALUE.
c
c ****** Otherwise, return IFINT=.false..
c
c-----------------------------------------------------------------------
c
c ****** A valid integer has the format:
c
c          ALPHA = '[A]B[B...B]',
c
c ****** where A represents a + or - sign, and B represents a digit
c ****** between 0 and 9, inclusive.
c
c-----------------------------------------------------------------------
c
      character*(*) alpha
      integer ivalue
c
c-----------------------------------------------------------------------
c
      character*5 fmt
c
c-----------------------------------------------------------------------
c
      ifint=.false.
c
      do 100 k=1,len(alpha)
c
        i=ichar(alpha(k:k))
c
c ****** Check for a sign in the first position.
c
        if (k.eq.1.and.(i.eq.43.or.i.eq.45)) go to 100
c
c ****** Check for a digit.
c
        if (i.ge.48.and.i.le.57) then
          ifint=.true.
          go to 100
        end if
c
c ****** Failed check: fall through here.
c
        ifint=.false.
c
        return
c
  100 continue
c
c ****** Obtain its numeric value.
c
      fmt='(i  )'
      write (fmt(3:4),'(i2.2)') len(alpha)
c
      if (ifint) read (alpha,fmt) ivalue
c
      return
      end
