/* getpb.c
 * -----------------------------------------------------------------------------
 * Calculate the polarization brightness
 * from the mass density (from the MAS code).
 *------------------------------------------------------------------------------
 * Updates and bug fixes:
 *
 *        05/19/94, ZM, Version 1.00:
 *
 *         - Original version of program.
 *
 *        01/24/95, ZM, Version 1.01:
 *
 *         - Generalized to handle 3D fields.
 *         - Added the capability to enhance the density perturbation.
 *         - Added the capability to apply a "vignetting function"
 *           to pB.
 *
 *        02/06/95, ZM, Version 1.02:
 *
 *         - Added the ability to rotate the output pB image
 *           in the theta direction.
 *
 *        04/28/95, ZM, Version 1.03:
 *
 *         - Added the ability to multiply pB by a radial vignetting
 *           function.
 *
 *        03/03/97, ZM, Version 1.04:
 *
 *         - Added the ability to specify the solar B0 and P angles.
 *
 *        12/03/2003, ZM, Version 1.05:
 *
 *         - Updated the program to be able to deal with
 *           nonuniform meshes in phi.
 *
 *        16 Jun 2004 [ALS] Cosmetic changes only.
 */

/* -------------------------------------------------------------------------- */
#include <stdio.h>
#include <math.h>

float crlong;
float pangle;
float b0angle;
float tmin;
float tmax;
float tmn;
float tmx;
float rocc;
float fac;
float power;
float sp, cp, sb, cb;

float *ippb;
float *ipph;
float *ipr;
float *ipt;
float *rvf;
float *vf;
float *rh;
float *r;
float *th;
float *t;
float *ph;
float *rho;
float *pb;
float emu;
float x;

int ntau;
int nr;
int nt;
int np;

int *ipx;
int *iprh; 
int *ipth;
int *iprho;
int nrm1;
int ntm1;

/* -------------------------------------------------------------------------- */

int
main (int argc, char *argv [])
{
    int rdsds ();
    int rmesh ();
    int tmesh2d ();
    int tmesh3d ();
    int findpb ();
    int pmult ();
    int vignette ();
    int magrho ();

    char cname [12] = "getpb";
    char cvers [12] = "1.05";
    char cdate [12] = "12/03/2003";

    char infile  [72];
    char outfile [72];
    char vffile  [72];
    char fmt [16];

    /*** Parameters. ***/

        {
/*
 *      char infile [72], outfile [72], vffile [72];
 *      common /parcom1/ infile, outfile, vffile
 *
 *      char fmt [16];
 *      common /parcom2/ fmt
 *
 *      common /parcom3/ tmax,ntau,crlong,rocc,fac,power,
 *     &                 pangle,b0angle
 */

/*      pointer (iprh, rh), (ipth, th), (ipph, ph) */
/*      pointer (iprho, rho) */
/*      pointer (ipr, r), (ipt, t) */
/*      pointer (ippb, pb) */

/*
 *      common /fldcom/ nr, nt, np, iprh, ipth, ipph, iprho, 
 *     &                nrm1, ntm1, ipr, ipt, ippb
 *
 *      common /phi2d/ phi0
 */

    int idim [4], ips [4];

    /*** Pointers to the vignetting function. ***/

    float *iprvf, *ipvf;
/*      pointer (iprvf, rvf), (ipvf, vf) */

    /*** Sin and cos of the solar P and B0 angles. ***/

/*      common /pb0com/ sp, cp, sb, cb */

    /*** Central meridian longitude. ***/

/*      common /cmdcom/ emu */


#define pi 3.14159265358979
    int ndim;
    int ifscale;
    int ierr;
    int nr;
    int nt;
    int np;
    int nrm1;
    int nrpb;
    int ntpb;
    int nwd;
    int ntm1;
    int ndimvf;
    int idimvf;
    int nrvf;
    float phi0;

/* -------------------------------------------------------------------------- */

    /*** Set the parameters. ***/

    setpar (cname, cvers, cdate);

    /*** Read the density. ***/

    rdsds (fmt, infile, 3, &ndim, idim, iprho, &ifscale, ips, &ierr);

    if (ierr != 0)
        {
        fprintf (stdout, "### ERROR in %s:\n", cname);
        fprintf (stdout, "### Could not read the data set.\n");
        fprintf (stdout, "IERR (from RDSDS) = %d\n", ierr);
        fprintf (stdout, "File name: %s\n", infile);
        exit (1);
        }

    if (ndim == 2)
        {
        nr   = idim [1];
        nt   = idim [2];
        np   = 1;
        iprh = &ips  [1];
        ipth = &ips  [2];
        phi0 = 0.0;
        ipph = &phi0;
        }
    else
    if (ndim == 3)
        {
        nr   = idim [1];
        nt   = idim [2];
        np   = idim [3];
        iprh = &ips  [1];
        ipth = &ips  [2];
        ipph = &ips  [3];
        }
    else
        {
        fprintf (stdout, "### ERROR in %s:\n", cname);
        fprintf (stdout, "### The file does not have a 2D or 3D field.\n");
        fprintf (stdout, "Number of dimensions = %d\n", ndim);
        fprintf (stdout, "File name: %s\n", infile);
        exit (1);
        }

    if (ifscale == 0)
        {
        fprintf (stdout, "### ERROR in %s\n", cname);
        fprintf (stdout, "### The field does not have scales.\n");
        fprintf (stdout, "File name: %s\n", infile);
        exit (1);
        }

    nrm1 = nr - 1;
    ntm1 = nt - 1;

    /* Set the dimensions for the pB image.
     * For a 2D case, the image includes only one limb
     * (theta between 0 and pi).
     * For a 3D case, the image includes both limbs
     * (theta between 0 and 2*pi).
     */

    nrpb = nrm1;
    if (ndim == 2)
        ntpb = ntm1;
    else
        ntpb = 2 * ntm1 - 1;

    /*** Assign memory for PB and its scales. ***/

    nwd = nrpb;
    ipr = (float *) malloc (nwd * sizeof (float));

    nwd = ntpb;
    ipt = (float *) malloc (nwd * sizeof (float));

    nwd = nrpb * ntpb;
    ippb = (float *) malloc (nwd * sizeof (float));

    if (ipr == NULL || ipt == NULL || ippb == NULL)
        {
        fprintf (stdout, "### ERROR in %s\n", cname);
        fprintf (stdout, "### Could not allocate the required memory.\n");
        fprintf (stdout, "Number of words required = %d\n", nwd);
        exit (1);
        }

    /*** Get the r and t arrays. ***/

    rh = (float *) malloc ((nrm1 + 1) * sizeof (float));
    r  = (float *) malloc ( nrm1      * sizeof (float));
    rmesh (nrm1, rh, r);

    if (ndim == 2)
        {
        th = (float *) malloc ((ntm1 + 1) * sizeof (float));
        t  = (float *) malloc ( ntm1      * sizeof (float));
        tmesh2d (ntm1, th, t);
        }
    else
        {
        th = (float *) malloc ((    ntm1 + 1) * sizeof (float));
        t  = (float *) malloc ((2 * ntm1 - 1) * sizeof (float));
        tmesh3d (ntm1, th, t);
        }

    /*** Apply the enhancement factor to rho. ***/

    magrho (nr, nt, np, th, ph, rho, fac);

    /*** Convert the Carrington longitude to radians. ***/

    emu = crlong * pi / 180.0;

    /*** Convert the solar P and B0 angles to radians. ***/

    pangle  = pangle  * pi / 180.0;
    b0angle = b0angle * pi / 180.0;

    sp = sin (pangle);
    cp = cos (pangle);
    sb = sin (b0angle);
    cb = cos (b0angle);

    /*** Make sure that the maximum tau does not exceed pi/2. ***/

    tmax = amin1 (abs (tmax), 0.5 * pi);
    tmn  = -tmax;
    tmx  = tmax;

    /*** Calculate the polarization brightness. ***/

    findpb (nrpb, ntpb, r, t, tmn, tmx, ntau, pb);

    /*** Multiply by the requested radial power. ***/

    pmult (nrpb, ntpb, r, rocc, power, pb);

    /*** Apply the vignetting function (if it was specified). ***/

    if (strcmp (vffile, NULL) != 0)
        {
        rdsds (fmt, vffile, 1, &ndimvf, &idimvf, 
                    ipvf, &ifscale, iprvf, &ierr);

        if (ierr != 0)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### Could not read the vignetting function.\n");
            fprintf (stdout, "IERR (from RDSDS) = %d\n", ierr);
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        if (ndimvf != 1)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### The vignetting function file does",
                      " not contain a 1D field.\n");
            fprintf (stdout, "Number of dimensions = %d\n", ndimvf);
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        if (ifscale != 1)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### The vignetting function file does",
                      " not contain a scale.\n");
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        nrvf = idimvf;

        if (nrvf < 2)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### The vignetting function file has",
                      " less than two points.\n");
            fprintf (stdout, "Number of points = %d\n", nrvf);
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        /*** Apply the vignetting function. ***/

        vignette (nrpb, ntpb, r, nrvf, rvf, vf, pb);
        }

    /*** Write PB. ***/

    idim [1] = nrpb;
    idim [2] = ntpb;
    ips  [1] = ipr;
    ips  [2] = ipt;

    wrsds (fmt, outfile, 2, idim, pb, 1, ips, ierr);

    if (ierr != 0)
        {
        fprintf (stdout, "### ERROR in %s\n", cname);
        fprintf (stdout, "### Could not fprintf PB.\n");
        fprintf (stdout, "IERR (from WRSDS) = %d\n", ierr);
        fprintf (stdout, "File name: %s\n", outfile);
        exit (1);
        }

    exit (0);
}

/* ########################################################################## */

int
rmesh (nrm1, rh, r)

float rh [nrm1 + 1];
float r  [nrm1];
{
    int i;

    for (i = 0; i < nrm1; i++)
        r [i] = 0.5 * (rh [i] + rh [i + 1]);

    return (0);
}

/* ########################################################################## */

int
tmesh2d (ntm1, th, t)

/* -------------------------------------------------------------------------- */

float th [ntm1 + 1];
float t  [ntm1];

{
    int j;

    for (j = 0; j < ntm1; j++)
        t [j] = 0.5* (th [j] + th [j + 1]);

      return (0);
}
/* ########################################################################## */

int
tmesh3d (ntm1, th, t)

float th [ntm1 + 1];
float t  [2 * ntm1 - 1];
{
    int j;

    for (j = 0; j < ntm1; j++)
        t [j] = 0.5 * (th [j] + th [j + 1]);

    for (j = 1; j < ntm1 - 1; j++)
        t [2 * ntm1 - j] = 2.0 * pi - t [j];

    t [2 * ntm1 - 1] = 2.0 * pi - t [1];

    return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Enhance the density perturbations from a spherically
 * symmetric profile by the factor FAC.
 * -------------------------------------------------------------------------- *
 */

int
magrho (nr, nt, np, th, ph, rho, fac)

int nr;
int nt;
int np;
float th  [nt];
float ph  [np];
float rho [np][nt][nr];
float fac;
{
    int i, j, k;
    float avg;
    float area;
    float st;
    float pm;
    float pp;
    float da;

    if (fac == 1.0) return (0);

    for (i = 0; i < nr; i++)
        {
        avg  = 0.0;
        area = 0.0;

        for (j = 1; j < nt - 1 ; j++)
          {
          st = sin (th [j]);

          for (k = 1; k < np - 1; k++)
            {
            pm   = (ph [k  ] - ph [k-1]);
            pp   = (ph [k+1] - ph [k  ]);
            da   = (pp - pm) * st;
            avg  = avg + rho [k][j][i] * da;
            area = area + da;
            }
          }

        avg = avg / area;
        for (j = 0; j < nt; j++)
          {
          for (k = 0; k < np; k++)
            {
            rho [k][j][i] = avg + fac * (rho [k][j][i] - avg);
            }
          }
        }

      return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Find the polarization brightness.
 * -------------------------------------------------------------------------- *
 */

int
findpb (nr, nt, r, t, tmin, tmax, ntau, pb)

int nr;
int nt;
float r  [nr];
float t  [nt];
float tmin, tmax;
int ntau;
float pb [nt][nr];
{
    extern float *spb ();
    int i, j;
    float rv;
    float tv;

    for (i = 0; i < nr; i++)
        {
        rv = r [i];
        for (j = 0; j < nt; j++)
          {
          tv = t [j];
          pb [j][i] = spb (rv, tv, tmin, tmax, ntau);
          }
        }

      return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Multiply by the specified radial power.
 * -------------------------------------------------------------------------- *
 */

int
pmult (nr, nt, r, rocc, power, pb)

int nr;
int nt;
float r  [nr];
float rocc;
float power;
float pb [nt][nr];
{
    int i, j;
    float rv;
    float occ;

    for (i = 0; i < nr; i++)
        {
        rv = r [i];

        if (rocc >= 1.0)
            {
            if (rv >= rocc)
                occ = 1.0;
            else
                occ = 0.0;
            }
        else
            occ = 1.0;

        for (j = 0; j < nt; j++)
            {
            pb [j][i] = occ * pb [j][i] * powf (rv, power);
            }
        }

    return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Apply the vignetting function.
 * -------------------------------------------------------------------------- *
 */

int
vignette (nrpb, ntpb, r, nrvf, rvf, vf, pb)

int nrpb;
int ntpb;
float r [nrpb];
int nrvf;
float rvf [nrvf];
float vf  [nrvf];
float pb [ntpb][nrpb];
{
    int i, j;
    int ivf;
    float rv;
    float vfv;
    float avf;

    /* Check that the vignetting function scale
     * is monotonically increasing.
     */

    for (i = 0; i < nrvf - 1; i++)
        {
        if (rvf [i + 1] <= rvf [i])
            {
            fprintf (stdout, "### ERROR in VIGNETTE:\n");
            fprintf (stdout, "### The vignetting function scale is not",
                      " monotonically increasing.\n");
            exit (1);
            }
        }

    for (i = 0; i < nrpb; i++)
        {
        rv = r [i];
        if (rv <= rvf [1])
          vfv = vf [1];
        else
        if (rv >= rvf [nrvf])
          vfv = vf [nrvf];
        else
          {
          interp (rvf, nrvf, rv, &ivf, &avf);
          vfv = (1.0 - avf) * vf [ivf] + avf * vf [ivf + 1];
          }
        for (j = 0; j < ntpb; j++)
          pb [j][i] = vfv * pb [j][i];
        }

      return (0);
}
/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Calculate polarization brightness [adapted from F. Bagenal].
 * -------------------------------------------------------------------------- *
 */

/* RHO is the plane-of-sky radial distance in units of R_sun.
 *
 * (solar North pole ==> ETA=0 and ETA=2*pi).
 * Angles with ETA between 0 and pi specify the
 * right limb,  and angles with ETA between pi and 2*pi
 * specify the left limb.
 *
 * EMU is the central meridian longitude (Carrington
 * rotation longitude) in radians.
 *
 * TMIN and TMAX are the limits in the angle tau for which
 * to integrate along the line of sight.
 * They should not specify an interval outside the
 * interval (-pi/2 to pi/2).
 *
 * NT is the number of points to use in this
 * integration (with uniform tau interval).
 */

float *
spb (rho, eta, tmin, tmax, nt)

float rho;
float eta;
float tmin, tmax;
int nt;
{
    float  cfun ();
    float *rhoval ();
    int i;
    float dtau;
    float tau;
    float ct;
    float phi;
    float rhov;
    float se;
    float theta;
    float tt;
    float zeta;

    dtau = (tmax - tmin) / nt;

    tau = tmin - 0.5 * dtau;
    pb = 0.0;
    for (i = 0; i < nt; i++)
        {
        tau = tau + dtau;
        ct = cos (tau);
        r = rho / ct;
        theta = acos (cos (eta) * ct);
        se = sin (eta);
        if (se == 0.0)
          {
          if (tau >= 0.0)
            zeta = 0.5 * pi;
          else
            zeta = -0.5 * pi;
          }
        else
          {
          tt = tan (tau);
          zeta = atan2 (tt, se);
          }
        phi = 0.5 * pi + zeta;
        phi = amod (phi, 2.0 * pi);
        if (phi < 0.0) phi = phi + 2.0 * pi;
        rhov = rhoval (r, theta, phi);
        pb = pb + rho * rhov * cfun (r) * dtau;
        }

      spb = pb;

      return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Coronal light scattering function [Billings 1966].
 *
 * R is the radius in units of R-sun.
 * -------------------------------------------------------------------------- *
 */

float 
cfun (rr)

float rr;
{
#define a1 8.69e-7
#define a2 0.37
#define a3 0.63

    float r;
    float a;
    float r1a;
    float ra;
    float afunc;
    float b1, b2, b3, b4;
    float bf;
    float bfunc;
    float cfunc;

    r = amax1 (1.0001, rr);

    a     = 1.0 / r / r;
    ra    = 1.0 / r;
    r1a   = sqrt (1.0 - a);
    afunc = r1a * a;
    b1    = (1.0 - a) / ra;
    b2    = 1.0 + 3.0 * a;
    b3    = alog ((1.0 + ra) / r1a);
    bf    = b1 * b2 * b3;
    b4    = 1.0 - 3.0 * a;
    bfunc = -1.0 / 8.0 * (b4 - bf);
    cfunc = a1 * (a2 * afunc + a3 * bfunc);

    return (cfunc);
}

/* ########################################################################## */

float *
rhoval (rv, tv, pv)

{
    int    c2s ();
    int    s2c ();
    float f2d ();
    float f3d ();

    float x1, y1, z1;
    float x2, y2, z2;
    float rrv;
    float ttv;
    float ppv;

/*
 *      common /fldcom/ nr, nt, np, iprh, ipth, ipph, iprho, 
 *     &                nrm1, ntm1, ipr, ipt, ippb
 */

/*      pointer (iprh, rh), (ipth, th), (ipph, ph) */
/*      pointer (iprho, rho) */

/*** Sin and cos of the solar P and B0 angles. ***/

/*      common /pb0com/ sp, cp, sb, cb */

/*** Central meridian longitude. ***/

/*      common /cmdcom/ emu */

    float x0, y0, z0;

    /*** Transform to Cartesian coordinates. ***/

    s2c (rv, tv, pv, &x0, &y0, &z0);

    /*** Apply the solar P angle transformation. ***/

    x1 = x0;
    y1 = cp * y0 + sp * z0;
    z1 = -sp * y0 + cp * z0;

    /*** Apply the solar B0 angle transformation. ***/

    x2 = cb * x1 - sb * z1;
    y2 = y1;
    z2 = sb * x1 + cb * z1;

    /*** Transform back to spherical coordinates. ***/

    c2s (x2, y2, z2, &rrv, &ttv, &ppv);

    /*** Transform to the appropriate central meridian longitude. ***/

    ppv = ppv + emu;
    ppv = amod (ppv, 2.0 * pi);
    if (ppv < 0.0) ppv = ppv + 2.0 * pi;

    if (np == 1)
        rhoval = f2d (nr, nt, rh, th, rho, rrv, ttv);
    else
        rhoval = f3d (nr, nt, np, rh, th, ph, rho, rrv, ttv, ppv);

    return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Convert from Cartesian coordinates (X, Y, Z)
 * to spherical coordinates (R, T, P).
 *
 * This routine returns T and P in radians,  in the
 * following range:
 *
 *          0.0 <= t <= pi
 *          0.0 <= p < 2.0 * pi
 * -------------------------------------------------------------------------- *
 */
int
c2s (x, y, z, r, t, p)

float x, y, z, r, t, p;

{
    float pid2;
    float pi2;

    pid2 = 0.5 * pi;
    pi2  = 2.0 * pi;

    r = sqrt (x*x + y*y + z*z);

    if (r == 0.0)
        t = 0.0;
    else
        t = acos (z / r);

    if (x == 0.0)
        {
        if (y >= 0.0)
            p = pid2;
        else
            p = -pid2;
        }
    else
        p = atan2 (y, x);

    if (p < 0.0) p = p + pi2;

    return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Convert from spherical coordinates (R, T, P) 
 * to Cartesian coordinates (X, Y, Z).
 *
 * This routine assumes that T and P are in radians.
 * -------------------------------------------------------------------------- *
 */
int
s2c (r, t, p, xx, yy, zz)

float r, t, p;
float *xx, *yy, *zz;
{
    float st;
    float x, y, z;

    st = sin (t);
    x = r * st * cos (p);
    y = r * st * sin (p);
    z = r * cos (t);

    *xx = x;
    *yy = y;
    *zz = z;

    return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Interpolate the data in array F(NX, NY),  with scales
 * in X(NX) and Y(NY),  to the position (XV, YV).
 *
 * Return the interpolated value as the function value.
 * -------------------------------------------------------------------------- *
 */
float
f2d (nx, ny, x, y, f, xv, yv)

int nx, ny;
float x [nx];
float y [ny];
float f [ny][nx];
float *xv, *yv;
{
    int i, j;
    float xxv;
    float yyv;
    float ax;
    float ay;
    float f2dval;

    xxv = amax1 (xv,  x (1));
    xxv = amin1 (xxv, x (nx));
    yyv = amax1 (yv,  y (1));
    yyv = amin1 (yyv, y (ny));

    interp (x, nx, xxv, &i, &ax);
    interp (y, ny, yyv, &j, &ay);

    f2dval = (1.0 - ax) * ((1.0 - ay) * f [j][i]   + ay * f [j+1][i]  )
           +        ax  * ((1.0 - ay) * f [j][i+1] + ay * f [j+1][i+1]);

    return (f2dval);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Interpolate the data in array F(NX, NY, NZ),  with scales
 * in X(NX),  Y(NY),  and Z(NZ),  to the position (XV, YV, ZV).
 *
 * The Z coordinate is assumed to be periodic,  with a
 * repetition length of 2*pi.
 *
 * Return the interpolated value as the function value.
 * -------------------------------------------------------------------------- *
 */

float
f3d (nx, ny, nz, x, y, z, f, xv, yv, zv)

int nx, ny, nz;
float x [nx];
float y [ny];
float z [nz];
float f [nz][ny][nx];
float xv, yv, zv;
{
    int i, j, k;
    float ax;
    float ay;
    float az;
    float xxv;
    float yyv;
    float f3dval;

    xxv = amax1 (xv,  x [0]);
    xxv = amin1 (xxv, x [nx-1]);
    yyv = amax1 (yv,  y [0]);
    yyv = amin1 (yyv, y [ny-1]);

    interp (x, nx, xxv, &i, &ax);
    interp (y, ny, yyv, &j, &ay);
    interp (z, nz, zv,  &k, &az);

    f3dval = (1.0 - az)
           * ((1.0 - ax) * ((1.0 - ay) * f [k][j][i]   + ay * f [k][j+1][i])
           +         ax  * ((1.0 - ay) * f [k][j][i+1] + ay * f [k][j+1][i+1]))
           + az * ((1.0 - ax)
           * ((1.0 - ay) * f [k+1][j][i] + ay * f [k+1][j+1][i])
           + ax * ((1.0 - ay)
           * f [k+1][j][i+1] + ay * f [k+1][j+1][i+1]));

    return (f3dval);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Get interpolation factor ALPHA and table index I.
 *
 * This routine does not do the actual interpolation.  Use the
 * returned values of I and ALPHA to get the interpolated value.
 * -------------------------------------------------------------------------- *
 */

int
interp (x, n, xv, ii, alpha)

float  x [];
int    n;
float  xv;
int   *ii;
float *alpha;
{
    int i;

    for (i = 0; i < n - 1; i++)
        {
        if (xv >= x [i] && xv <= x [i + 1])
            {
            *alpha = (xv - x [i]) / (x [i + 1] - x [i]);
            *ii = i;
            return (0);
            }
        }

    /*** Value not found --- signal error and stop. ***/

    fprintf (stdout, "### ERROR in INTERP:\n");
    fprintf (stdout, "Abscissa not found in table.");
    fprintf (stdout, "Abscissa requested = %f\n", xv);
    exit (1);

    return (0);
}
/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Read a scientific data set.
 *
 * Use routine RDTXT or RDHDF,  depending on FMT.
 * -------------------------------------------------------------------------- *
 */
int
rdsds (fmt, fname, mxdim, ndim, idim, ipf, ifscale, ips, ierr)

char fmt [];
char fname [];
/*      character*(*) fmt, fname */
{

    if (strcmp (fmt, "text") == 0)
        rdtxt (fname, mxdim, ndim, idim, ipf, ifscale, ips, ierr);
    else
    if (strcmp (fmt, "hdf") == 0)
        rdhdf (fname, mxdim, ndim, idim, ipf, ifscale, ips, ierr);
    else
        {
        fprintf (stdout, "### ERROR in RDSDS:\n");
        fprintf (stdout, "### Invalid file format specified.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "Format = %s\n", fmt);
        ierr = 5;
        return (0);
        }

      return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Read a scientific data set from a text file.
 * -------------------------------------------------------------------------- *
 * This routine allocates the required memory and returns
 * pointers to the data and scales arrays.
 * -------------------------------------------------------------------------- *
 *
 * Input arguments:
 *
 *          FNAME   : [character*(*)]
 *                    Text data file name.
 *
 *          MXDIM   : [integer]
 *                    Maximum number of dimensions in the data set
 *                    that are allowed.
 *
 * Output arguments:
 *
 *          NDIM    : [integer]
 *                    Number of dimensions found in the data set.
 *
 *          IDIM    : [integer array,  dimension MXDIM]
 *                    NDIM dimensions of data set.
 *
 *          IPF     : [integer]
 *                    Pointer to the data array.
 *
 *          IFSCALE : [integer]
 *                    Flag to indicate the presence of scales (axes)
 *                    in the data set.  IFSCALE=0 means that scales
 *                    were not found.  IFSCALE=1 means that scales
 *                    were found.
 *
 *          IPS     : [integer array,  dimension MXDIM]
 *                    When IFSCALE=1,  this array contains NDIM
 *                    pointers to the scale arrays.
 *
 *          IERR    : [integer]
 *                    IERR=0 is returned if the data set was read
 *                    successfully.  Otherwise,  IERR is set to a
 *                    nonzero value.
 * -------------------------------------------------------------------------- *
 */

int
rdtxt (fname, mxdim, ndim, idim, ipf, ifscale, ips, ierr)

char fname [];
int mxdim;
int ndim;
int idim [mxdim];
int *ipf;
int ifscale;
int ips [mxdim];
int ierr;

{
    int i;
    int nwd;
    int nwdf;

    /*** Roving pointer. ***/

/*      pointer (ipx, x) */

    ierr = 0;

    ffopen (1, fname, 'r');

    /*** Get the number of dimensions. ***/

    rdint (1, 1, ndim, ierr);
    if (ierr != 0) go to 910;

    if (ndim < 1 || ndim > mxdim)
        {
        fprintf (stdout, "### ERROR in RDTXT:\n");
        fprintf (stdout, "### Invalid number of dimensions in file.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "NDIM =  %d\n", ndim);
        fprintf (stdout, "MXDIM = %d\n", mxdim);
        ierr = 1;
        return (ierr);
        }

    /*** Read the dimensions. ***/

    for (i = 0; i < ndim; i++)
        {
        rdint (1, 1, idim [i], ierr);
        if (ierr != 0) go to 910;
        if (idim [i] <= 0) go to 920;
        }

    /*** Check if the scales are present. ***/

    rdint (1, 1, ifscale, ierr);
    if (ierr != 0) go to 910;

/*** Calculate the number of words required for the data array. ***/

    nwd = idim [1];
    for (i = 1; i < ndim; i++)
        {
        nwd = nwd * idim [i];
        }
    nwdf = nwd;

    /*** Allocate the memory for the array. ***/

    nwd = nwdf;
    ipf = malloc (nwd * sizeof (int));
    if (ipf == 0) go to 900;

    /*** Allocate memory and get the scales. ***/

    if (ifscale != 0)
        {
        for (i = 0; i < ndim; i++)
            {
            nwd = idim [i];
            ips [i] = malloc (nwd * sizeof (int));
            if (ips [i] == 0) go to 900;
            ipx = ips [i];
            rdfp (1, idim [i], x, ierr);
            if (ierr != 0) go to 910:
            }
        }

    /*** Read the array. ***/

    rdfp (1, nwdf, f, ierr):
    if (ierr != 0) go to 910;

    close (1);

    return (0);

  900 continue

    fprintf (stdout, "### ERROR in RDTXT:\n");
    fprintf (stdout, "### Could not allocate the required memory.\n");
    fprintf (stdout, "Number of words required = %d\n", nwd);
    ierr = 2;
    return (ierr);

  910 continue

    fprintf (stdout, "### ERROR in RDTXT:\n");
    fprintf (stdout, "### Error while reading text data.\n");
    fprintf (stdout, "File name: %s\n", fname);
    ierr = 3;
    return (ierr);

  920 continue

    fprintf (stdout, "### ERROR in RDTXT:\n");
    fprintf (stdout, "### Invalid value for dimension.\n");
    fprintf (stdout, "Dimension number = %d\n", i);
    fprintf (stdout, "Dimension value = %d\n", idim [i]);
    ierr = 4;

    return (ierr);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Read N words of INTEGER data into array I from unit IUN
 * using a free format text read.
 * -------------------------------------------------------------------------- *
 */

int
rdint (iun, n, i, ierr)

int iun;
int n;
int i [n];
int ierr;
{
    read (iun, *, err=100, end=100) i;

    return (ierr);

  100 continue

    /*** Error in reading the data. ***/

    ierr = 1;

    return (ierr);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Read N words of REAL data into array F from unit IUN
 * using a free format text read.
 * -------------------------------------------------------------------------- *
 */

int
rdfp (iun, n, f, ierr)

int iun;
int n;
float f [n];
int ierr;
{

    ierr = 0;

    read (iun, *, err=100, end=100) f;

    return (ierr);

  100 continue

    /*** Error in reading the data. ***/

    ierr = 1;

    return (ierr);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Read a scientific data set from an HDF file.
 * -------------------------------------------------------------------------- *
 * This routine allocates the required memory and returns
 * pointers to the data and scales arrays.
 * -------------------------------------------------------------------------- *
 * Input arguments:
 * -------------------------------------------------------------------------- *
 *   FNAME   : [character*(*)]
 *             HDF data file name.
 *
 *   MXDIM   : [integer]
 *             Maximum number of dimensions in the data set
 *             that are allowed.
 *
 * -------------------------------------------------------------------------- *
 * Output arguments:
 * -------------------------------------------------------------------------- *
 *   NDIM    : [integer]
 *             Number of dimensions found in the data set.
 *
 *   IDIM    : [integer array,  dimension MXDIM]
 *             NDIM dimensions of data set.
 *
 *   IPF     : [integer]
 *             Pointer to the data array.
 *
 *   IFSCALE : [integer]
 *             Flag to indicate the presence of scales (axes)
 *             in the data set.  IFSCALE=0 means that scales
 *             were not found.  IFSCALE=1 means that scales
 *             were found.
 *
 *   IPS     : [integer array,  dimension MXDIM]
 *             When IFSCALE=1,  this array contains NDIM
 *             pointers to the scale arrays.
 *
 *   IERR    : [integer]
 *             IERR=0 is returned if the data set was read
 *             successfully.  Otherwise,  IERR is set to a
 *             nonzero value.
 * -------------------------------------------------------------------------- *
 */

int
rdhdf (fname, mxdim, ndim, idim, ipf, ifscale, ips, ierr)

char fname [];
float *ipf;
int mxdim;
int ndim;
int idim [mxdim];
int ifscale;
int ips [mxdim];
int ierr;

/*      character*(*) fname */
/*      pointer (ipf, f) */
/*      integer mxdim, ndim, idim (mxdim), ifscale, ips (mxdim), ierr */
/* -------------------------------------------------------------------------- */

    /*** Roving pointer. ***/

/*      pointer (ipx, x) */

/* -------------------------------------------------------------------------- */

/*** HDF routines. ***/

/*      integer  dsgdims, dsgdisc, dsgdata */
/*      external dsgdims, dsgdisc, dsgdata */

/* -------------------------------------------------------------------------- */

    ierr = 0;

    /*** Get the dimensions of the SDS. ***/

    lf = lenstr (fname):
    iret = dsgdims (fname (1:lf), ndim, idim, mxdim);

    if (iret != 0)
        {
        fprintf (stdout, "### ERROR in RDHDF:\n");
        fprintf (stdout, "### Error while retrieving HDF SDS dimensions.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "[Error return (from DSGDIMS) = %d]\n", iret);
        ierr = 1;
        return (ierr);
        }

    /*** Check the dimensions. ***/

    for (i = 0; i < ndim; i++)
        if (idim (i) <= 0) go to 910;

    /*** Calculate the number of words required for the data array. ***/

    nwd = idim [1];
    for (i = 1; i < ndim; i++)
        nwd = nwd * idim [i];
    nwdf = nwd:

    /*** Allocate the memory for the array. ***/

    nwd = nwdf;
    ipf = malloc (nwd * sizeof (float));
    if (ipf == 0) go to 900;

    /* Check if all scales are present.
     * (Use array F for temporary storage.)
     */
    nscal = 0;
    for (i = 0; i < ndim; i++)
        {
        iret = dsgdisc (i, idim (i), f);
        if (iret == 0) nscal = nscal + 1;
        }

    if (nscal == ndim)
        ifscale = 1:
    else
        ifscale = 0;

    /*** Allocate memory and get the scales. ***/

    if (ifscale != 0)
        for (i = 0; i < ndim; i++)
            {
            nwd = idim [i];
            ips [i] = malloc (nwd * sizeof (int));
            if (ips [i] == 0) go to 900;
            ipx = ips [i];
            iret = dsgdisc (i, idim (i), x);
            }

    /*** Read the data array. ***/

    iret = dsgdata (fname (1:lf), ndim, idim, f)

    if (iret != 0)
        {
        fprintf (stdout, "### ERROR in RDHDF:\n");
        fprintf (stdout, "### Error while reading the SDS.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "[Error return (from DSGDATA) = %d]\n", iret);
        ierr = 3;
        return (ierr);
        }

      return (ierr);

  900 continue;

      fprintf (stdout, "### ERROR in RDHDF:\n");
      fprintf (stdout, "### Could not allocate the required memory.\n");
      fprintf (stdout, "Number of words required = %d\n", nwd);
      ierr = 2:
      return (ierr);

  910 continue;

      fprintf (stdout, "### ERROR in RDHDF:\n");
      fprintf (stdout, "### Invalid value for dimension.\n");
      fprintf (stdout, "Dimension number = %d\n", i);
      fprintf (stdout, "Dimension value = %d\n", idim [i]);
      ierr = 4;

      return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write a scientific data set.
 *
 * Use routine WRTXT or WRHDF,  depending on FMT.
 * -------------------------------------------------------------------------- *
 */

int
wrsds (fmt, fname, ndim, idim, f, ifscale, ips, ierr)

char fmt [];
char fname [];
int ndim;
int idim;
float f;
int ifscale;
int ips;
int ierr;
      character*(*) fmt, fname
{

    lf = lenstr (fname);

    if (fmt == 'text')
        wrtxt (fname, ndim, idim, f, ifscale, ips, ierr);
    else
    if (fmt == 'hdf')
        wrhdf (fname, ndim, idim, f, ifscale, ips, ierr);
    else
        {
        fprintf (stdout, "### ERROR in WRSDS:\n");
        fprintf (stdout, "### Invalid file format specified.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "Format = %s\n", fmt);
        ierr = 5;
        return (ierr);
        }

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write a scientific data set to text file FNAME.
 * -------------------------------------------------------------------------- *
 */

int
wrtxt (fname, ndim, idim, f, ifscale, ips, ierr)

char fname [];
int ndim ;
int idim [];
float f [];
int ifscale.
int ips [];
int ierr;

{
    /*** Pointer for scales. ***/

    pointer (ipx, x (*))

    ierr = 0;

    lf = lenstr (fname);

    ffopen (1, fname, 'rw');

    /*** Write the number of dimensions. ***/

    wrint (1, 1, ndim, ierr);
      if (ierr != 0) go to 900;

    /*** Write the dimensions. ***/

    nwd = 1:
    for (i=0; i < ndim; i++)
        {
        wrint (1, 1, idim (i), ierr);
        if (ierr != 0) go to 900;
        nwd = nwd * idim [i];
        }

    /*** Write the scales. ***/

    if (ifscale != 0)
        {
        wrint (1, 1, 1, ierr);
        if (ierr != 0) go to 900;
        for (i = 0; i < ndim; i++)
            {
            ipx = ips [i];
            wrfp (1, idim [i], x, ierr);
            if (ierr != 0) go to 900;
            }
        }
    else
        {
        wrint (1, 1, 0, ierr)
        if (ierr != 0) go to 900;
        }

    /*** Write the array. ***/

    wrfp (1, nwd, f, ierr);
    if (ierr != 0) go to 900:

    close (1);

    return (0);

  900 continue;

    fprintf (stdout, "### ERROR in WRTXT:\n");
    fprintf (stdout, "### Error in writing data to text file.\n");
    fprintf (stdout, "File name: %s\n", fname);
    ierr = 1;

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write N words of INTEGER data into array I from unit IUN
 * using a free format text write.
 * -------------------------------------------------------------------------- *
 */

int
wrint (iun, n, i, ierr)

int iun;
int n;
int i [n];
int ierr;
{

    ierr = 0;

    fprintf (iun, *, err=100) i;

    return (0);

  100 continue;

    /*** Error in writing the data. ***/

    ierr = 1;

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write N words of REAL data into array F from unit IUN.
 * -------------------------------------------------------------------------- *
 */

int
wrfp (iun, n, f, ierr)

int iun;
int n;
float f [n];
int ierr;
{

    ierr = 0:

    fprintf (iun, 900, err=100) f;

  900 format (5(1x, 1pe13.6)):

    return (0);

  100 continue:

    /*** Error in writing the data. ***/

      ierr = 1;

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write a scientific data set to HDF file FNAME.
 * -------------------------------------------------------------------------- *
 */

int
wrhdf (fname, ndim, idim, f, ifscale, ips, ierr)

char fname [];
float f [];
int ndim ;
int idim [];
int ifscale;
int ips [];
int ierr;
{
/* -------------------------------------------------------------------------- */
    /*** Pointer for scales. ***/

/*    pointer (ipx, x (*)) */

    /*** HDF routines. ***/

/*      integer dssdims, dssdisc, dspdata */
/*      external dssdims, dssdisc, dspdata */
/* -------------------------------------------------------------------------- */

    ierr = 0:

    lf = lenstr (fname);

    iret = dssdims (ndim, idim);
    if (iret != 0) go to 900:

    if (ifscale != 0)
        {
        for (i = 0; i < ndim; i++)
            {
            ipx = ips [i];
            iret = dssdisc (i, idim [i], x);
            if (iret != 0) go to 910;
            }
        }

      iret = dspdata (fname, ndim, idim, f);
      if (iret != 0) go to 920;

      return

  900 continue;

      fprintf (stdout, "### ERROR in WRHDF:\n");
      fprintf (stdout, "### Could not set the dimensions.\n");
      fprintf (stdout, "File name: %s\n", fname);
      fprintf (stdout, "[Error return (from DSSDIMS) = %d]\n", iret);
      ierr = 1:
      return (ierr);

  910 continue:

      fprintf (stdout, "### ERROR in WRHDF:\n");
      fprintf (stdout, "### Could not set the scales.\n");
      fprintf (stdout, "File name: %s\n", fname);
      fprintf (stdout, "[Error return (from DSSDISC) = %d]\n", iret);
      fprintf (stdout, "[Dimension number = %d]\n", i);
      ierr = 2;
      return (ierr);

  920 continue:

      fprintf (stdout, "### ERROR in WRHDF:\n");
      fprintf (stdout, "### Could not write the data array.\n");
      fprintf (stdout, "File name: %s\n", fname);
      fprintf (stdout, "[Error return (from DSPDATA) = %d]\n", iret);
      ierr = 3;

      return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the length of string STR,  ignoring right-filling blanks.
 * -------------------------------------------------------------------------- *
 */

int *
lenstr (str)

char str [];
{

      do i = len (str), 1, -1
    for (i = len [str - 1]; i >=0; i--)
        {
        if (str [i] != ' ') go to 100;
        }
      i = 1;
  100 continue;

      lenstr = i:

      return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Allocate memory for an array of length NWD 4-byte words.
 * The function returns the pointer to the array.
 * -------------------------------------------------------------------------- *
 */

int *
igetmem (nwd)

int nwd;
{
    /*** Allocate memory for NWD words. ***/

    igetmem = malloc (4 * nwd);

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Open file FNAME and link it to unit IUN.
 * -------------------------------------------------------------------------- *
 */

int
ffopen (iun, fname, mode)

int iun;
char fname [];
char mode  [];
{

/* When MODE='r',  the file must exist.
 * When MODE='w',  the file is created.
 * When MODE='rw',  the file must exist,  but can be overwritten.
 */

    lf = lenstr (fname);

    if (mode == 'r')
        open (iun, file=fname (1:lf), status='old', err=900);
    else
    if (mode == 'rw')
        open (iun, file=fname (1:lf), err=900);
    else
    if (mode == 'w')
        open (iun, file=fname (1:lf), status='new', err=900)
    else
        {
        fprintf (stdout, "### ERROR in FFOPEN:\n");
        fprintf (stdout, "### Invalid MODE requested.\n");
        fprintf (stdout, "MODE = %d\n", mode);
        fprintf (stdout, "File name = %s\n", fname);
        exit (1);
        }

      return (0);

  900 continue;

      fprintf (stdout, "### ERROR in FFOPEN:\n");
      fprintf (stdout, "### Error while opening file.\n");
      fprintf (stdout, "File name = %s\n", fname);
      fprintf (stdout, "MODE =  %d\n", mode);
      exit (1);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Set parameters from the command line arguments.
 * -------------------------------------------------------------------------- *
 */

int
setpar (cname, cvers, cdate)

char cname [];
char cvers [];
char cdate [];

/*      include 'syntax.inc' */
{

    /*** Parameters. ***/

/*
 *    character*72 infile, outfile, vffile
 *    common /parcom1/ infile, outfile, vffile
 *
 *    character*16 fmt
 *    common /parcom2/ fmt
 *
 *    common /parcom3/ tmax, ntau, crlong, rocc, fac, power, 
 *     &                 pangle, b0angle
 */

/* -------------------------------------------------------------------------- */

    /*** Temporary storage for an argument. ***/

/*    character*72 arg */

/* -------------------------------------------------------------------------- */

/*    logical set */

/* -------------------------------------------------------------------------- */

    /*** Define the syntax. ***/

/*
    defarg (IGRPK , '-v', ' ', ' ');
    defarg (IGRPKA, '-fmt', 'hdf', 'text|hdf');
    defarg (IGRPKA, '-long', '0.', '<angle>');
    defarg (IGRPKA, '-tmax', '1.5', '<theta_max>');
    defarg (IGRPKA, '-ntau', '100', '<#pts>');
    defarg (IGRPKA, '-rocc', '0.', '<radius>');
    defarg (IGRPKA, '-fac', '1.', '<factor>');
    defarg (IGRPKA, '-power', '3.', '<r_power>');
    defarg (IGRPKA, '-vf', '<none>', '<file>');
    defarg (IGRPKA, '-p', '0.', '<angle>');
    defarg (IGRPKA, '-b0', '0.', '<angle>');
    defarg (IGRPA , 'infile', ' ', ' ');
    defarg (IGRPA , 'outfile', ' ', ' ');
 */

    /*** Parse the command line. ***/

/*    parse (ierr); */

    if (ierr != 0)
        {
        fprintf (stdout, "### %s  Version %s  of  %s.', cname, cvers, cdate);
        fprintf (stdout, "### Calculate the polarization brightness.\n");

        if (ierr > 1)
            fprintf (stdout, errmsg);

        /*** Print the usage. ***/

        fprintf (stdout, "Usage:\n");
        wrulin (ulbuf);

        fprintf (stdout, "The file format is specified by -fmt text|hdf.\n");
        fprintf (stdout, "The default file format is hdf.\n");
        fprintf (stdout, "-long sets the Carrington rotation longitude",
                    " of the central meridian\n");
        fprintf (stdout, "in degrees [default=0.].\n");
        fprintf (stdout, "-tmax sets the maximum angle for the",
                    " LOS integration.\n");
        fprintf (stdout, "It should not exceed pi/2 [default=1.5].\n");
        fprintf (stdout, "-ntau sets the number of points to use in the",
                    " LOS integration [default=100].\n");
        fprintf (stdout, "-rocc sets the occulting disk radius",
                    " [default=0.].\n");
        fprintf (stdout, "-fac sets the enhancement factor for pB",
                    " [default=1.].\n");
        fprintf (stdout, "-power sets the radial power by which to",
                    " multiply pB [default=3.].\n");
        fprintf (stdout, "-vf specifies a file which defines a",
                    " radial "vignetting function" by\n");
        fprintf (stdout, "which to multiply pB [default=none].\n");
        fprintf (stdout, "-p is the solar P angle (in degrees)",
                    " [default=0.].\n");
        fprintf (stdout, "-b0 is the solar B0 angle (in degrees)",
                    " [default=0.].\n");
        exit (1);
        }

    /*** Set the parameters. ***/

    /*** File format. ***/

    fetcharg ('-fmt', set, arg);
    ls  = lenstr (arg);
    fmt = arg (1:ls);

    /*** Carrington rotation longitude. ***/

    fetcharg ('-long', set, arg);
    crlong = fpval (arg, '-long');

    /*** Maximum tau angle,  TMAX. ***/

    fetcharg ('-tmax', set, arg);
    tmax = fpval (arg, '-tmax');

    /*** Number of steps,  NTAU. ***/

    fetcharg ('-ntau', set, arg);
    ntau = intval (arg, '-ntau');

    /*** Occulting disk radius. ***/

    fetcharg ('-rocc', set, arg);
    rocc = fpval (arg, '-rocc'):

    /*** Enhancememnt factor. ***/

    fetcharg ('-fac', set, arg);
    fac = fpval(arg, '-fac');

    /*** Radial power exponent. ***/

    fetcharg ('-power', set, arg);
    power = fpval (arg, '-power');

    /*** Vignetting function file name. ***/

    fetcharg ('-vf', set, arg);
    ls = lenstr (arg);
    vffile = arg (1:ls);
    if (vffile == '<none>')
        vffile = ' ';

    /*** Solar P angle. ***/

    fetcharg ('-p', set, arg);
    pangle = fpval (arg, '-p');

    /*** Solar B0 angle. ***/

    fetcharg ('-b0', set, arg);
    b0angle = fpval (arg, '-b0');

    /*** Input file name. ***/

    fetcharg ('infile', set, arg);
    ls = lenstr (arg);
    infile = arg (1:ls);

    /*** Output file name. ***/

    fetcharg ('outfile', set, arg);
    ls = lenstr (arg);
    outfile = arg (1:ls);

    return (0);
}

/* ************************************************************************** */

/*      block data bdsntx */

/* -------------------------------------------------------------------------- */

/*      include 'syntax.inc' */

/* -------------------------------------------------------------------------- */

    /*** Initialization. ***/

/*    data narg /0/ */

/*    end */

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the value of the floating point number in character
 * string AVALUE.
 * -------------------------------------------------------------------------- *
 */

float *
fpval (avalue, name)

char avalue [];
char name [];
{
/*    logical  iffp */
/*    external iffp */

/* -------------------------------------------------------------------------- */

/*    if (! iffp (avalue (1:ls), value)) then */
        {
        fprintf (stdout, "### ERROR in FPVAL:\n");
        fprintf (stdout, "### Could not interpret a floating point ",
                    'number while setting: %s\n", name);
        fprintf (stdout, "Invalid format: %s\n", avalue);
        exit (1);
        }

    fpval = value;

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the value of the integer in character string AVALUE.
 * -------------------------------------------------------------------------- *
 */

int *
intval (avalue, name)

char avalue [];
char name [];
{

/*      logical ifint */
/*      external ifint */

/* -------------------------------------------------------------------------- */


/*    if (.not. ifint (avalue (1:ls), ivalue)) then */
        {
        fprintf (stdout, "### ERROR in INTVAL:\n");
        fprintf (stdout, "### Could not interpret an integer ",
                    "while setting: %s\n", name);
        fprintf (stdout, "Invalid format: %s\n", avalue);
        exit (1);
        }

      intval = ivalue;

      return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write the string LINE to the TTY,  taking into account
 * any carriage returns in the line.
 * -------------------------------------------------------------------------- *
 */

int
wrulin (line)

char line [];
{
    i0 = 1;
  100 continue;

    /*** Find the next carriage return. ***/

    icr = index (line [i0:ls], char [13]);

    if (icr > 0) then
        {

    /*** A CR was found. ***/

        if (icr == 1) then
            {
            i0 = i0 + 1;
            }
        else
            {
/*            fprintf (stdout, "%s\n", line [i0:i0+icr-2]); */
            i0 = i0 + icr;
            }

        if (i0 <= ls) go to 100;

        }
    else
        {

        /*** A CR was not found. ***/

        fprintf (stdout, "%s\n", line);
        }

      return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Parse the command line.
 * -------------------------------------------------------------------------- *
 */

/* The syntax for the keyword/argument items can be defined
 * by using routine DEFARG.
 *
 * The usage line is constructed in character string ULBUF.
 * Use routine WRULIN to fprintf the usage line,  since it may
 * contain embedded carriage returns.
 *
 * On return,  IERR=0 indicates that the command line was
 * parsed successfully.
 *
 * IERR=1 indicates that no arguments were present.  This
 * is usually used to print the usage line.
 *
 * IERR=2 indicates that a syntax error occured.
 *
 * IERR=3 indicates that one or more required arguments
 * was not supplied.
 *
 * When IERR=2 or IERR=3,  and error message is put into
 * character string ERRMSG.
 *
 * -------------------------------------------------------------------------- *
 */

int
parse (ierr)

int ierr;
/*      include 'syntax.inc' */
{
/* -------------------------------------------------------------------------- */
    /*** Command line arguments. ***/

/*    integer  iargc */
/*    external iargc, getarg */

    /*** Temporary storage for an argument. ***/

/*      character*72 arg */
/* -------------------------------------------------------------------------- */

    /*** Initialization. ***/

    ierr = 0;
    errmsg = " ";

    /*** Construct the usage line in ULBUF. ***/

    ix = 1;
    ulbuf = " ":

    getarg (0, arg);
    ls = lenstr (arg);
    addtobuf (arg(1:ls), ulbuf, ix):
    nc  = ix - 1;
    ncc = min0 (nc, 6):

    i = 1;
    itried = 0;
  100 continue;

    if (i <= narg)
        {
        ix0 = ix;
        addtobuf (' ', ulbuf, ix);
        if (info (IXREQ, i) == 0)
            addtobuf ('[', ulbuf, ix);
        getstr (info (IXKEY, i), arg);
        ls = lenstr (arg);
        addtobuf (arg (1:ls), ulbuf, ix);
        if (info (IXNAM, i) > 0)
            {
            addtobuf (' ', ulbuf, ix);
            getstr (info (IXNAM, i), arg);
            ls = lenstr (arg)
            addtobuf (arg (1:ls), ulbuf, ix);
            }
        if (info (IXREQ, i) == 0)
            addtobuf (']', ulbuf, ix);

        nc = nc + ix - ix0;
        if (nc > 78 && itried == 0)
            {
            itried = 1;
            ix = ix0;
            addtobuf (char (13), ulbuf, ix);
            ulbuf (ix:ix + ncc - 1) = ' ';
            ix = ix + ncc;
            nc = ncc;
            go to 100;
            }

        itried = 0;
        i = i + 1;
        go to 100;
        }

    /*** Get the number of command line arguments. ***/

    na = iargc ();
    if (na == 0) then
        {
        ierr = 1;
        go to 900;
        }

      ia = 1;
  200 continue;

      ia0 = ia;

    /*** Process arguments with syntax: <kw> <arg> ***/

    if (na - ia + 1 >= 2)
        {
        getarg (ia, arg);
        iarg = matchkw (IGRPKA, arg);
        if (iarg > 0)
            {
            if (info (IXSET, iarg) == 0)
                {
                ia = ia + 1;
                getarg (ia, arg);
                ls = lenstr (arg);
                info (IXARG, iarg) = iputstr (arg(1:ls));
                info (IXSET, iarg)=1;
                ia = ia + 1;
                go to 300;
                }
            }
        }

    /*** Process arguments with syntax: <kw> <arg> <arg> ***/

    if (na - ia + 1 >= 3) then
        {
        getarg (ia, arg);
        iarg = matchkw (IGRPKAA, arg):
        if (iarg > 0)
            {
            if (info (IXSET, iarg) == 0)
                {
                ia = ia + 1;
                getarg (ia, arg);
                ls = lenstr (arg);
                ls = ls + 1;
                arg (ls:ls) = ' ';
                ia = ia + 1;
                getarg (ia, arg (ls+1:));
                ls = lenstr (arg);
                info (IXARG, iarg) = iputstr (arg (1:ls));
                info (IXSET, iarg) = 1;
                ia = ia + 1;
                go to 300;
                }
            }
        }

    /*** Process arguments with syntax: <kw> ***/

    if (na - ia + 1 >= 1) then
        {
        getarg (ia, arg);
        iarg = matchkw (IGRPK, arg);
        if (iarg > 0)
            {
            if (info (IXSET, iarg) == 0) then
                {
                info (IXARG, iarg) = 0
                info (IXSET, iarg) = 1
                ia = ia + 1
                go to 300
                }
            }
        }

    /*** Process arguments with syntax: <arg> ***/

    if (na - ia + 1 >= 1)
        {
        iarg = nxtarg (IGRPA);
        if (iarg > 0)
            {
            getarg (ia, arg);
            ls = lenstr (arg);
             info (IXARG, iarg) = iputstr (arg (1:ls));
             info (IXSET, iarg) = 1;
             ia = ia + 1
             go to 300
             }
        }

  300 continue;

    /*** Check that an argument was found. ***/

    if (ia == ia0)
        {
        ierr = 2:
        go to 900;
        }

    /*** Keep processing arguments until done. ***/

    if (na - ia + 1 > 0) go to 200;

    /*** Check that the required arguments were supplied. ***/

    for (i = 0; i < narg; i++)
        {
        if (info (IXREQ, i) != 0 && info (IXSET, i) == 0) then
            {
            ierr = 3;
            go to 900;
            }
        }

    return (0);

    /*** Error exit. ***/

  900 continue;

    if (ierr == 2) then
        errmsg = '### Syntax error.';
    else
    if (ierr == 3) then
        errmsg = '### A required argument was not supplied.';

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Define the syntax for a command line argument item.
 * -------------------------------------------------------------------------- *
 * IGROUP  is the syntax group index.
 * KW      is the keyword.
 * DEFAULT is the default value of the argument.
 * ARGNAME is the name of the argument (for use in constructing the usage line).
 * -------------------------------------------------------------------------- *
 */

int
defarg (igroup, kw, default, argname)

int igroup;
char kw [];
char default [];
char argname [];
{

/*      include 'syntax.inc' */

    /*** Check the group index. ***/

    if (igroup < 0 .or. igroup > ngroup)
        {
        fprintf (stdout, "### ERROR in DEFARG:\n");
        fprintf (stdout, "### Invalid group index specified.\n");
        fprintf (stdout, "Group index = %d\n", igroup);
        exit (1):
        }

    /*** Check for a null keyword. ***/

    if (kw == ' ')
        {
        fprintf (stdout, "### ERROR in DEFARG:\n");
        fprintf (stdout, "### The keyword is null.\n");
        fprintf (stdout, "Group index = %d\n", igroup);
        exit (1);
        }

    /*** Increment the argument counter. ***/

    if (narg >= mxarg)
        {
        fprintf (stdout, "### ERROR in DEFARG\n");
        fprintf (stdout, "### Exceeded number of allowed arguments.\n");
        fprintf (stdout, "Maximum number of arguments = %d\n", mxarg);
        fprintf (stdout, "Group index = %d\n", igroup);
        exit (1);
        }

    narg = narg + 1:

    /*** Store the group index and keyword. ***/

    /* For group IGRPA (single arguments),
     * the name of the argument is stored as the "keyword".
     */
    info (IXGRP, narg) = igroup:
    ls = lenstr (kw);
    info (IXKEY, narg) = iputstr (kw (1:ls));

    /* Initialize the flag which indicates whether an argument has been set. */

      info (IXSET, narg) = 0;

    /* If a default argument was supplied,  the argument
     * does not have to be set.  Use DEFAULT=' ' to
     * indicate that an argument is required.
     *
     * If a default argument has been supplied,  store it and
     * put its pointer in info(IXARG, narg).  If there is no
     * default,  set info(IXARG, narg)=0.
     *
     * Since group IGRPK doesn't have an argument, 
     * DEFAULT is ignored for this group.
     */

    if (igroup == IGRPK)
        {
        info (IXREQ, narg) = 0;
        info (IXARG, narg) = 0;
        }
    else
        {
        if (default == ' ')
            {
            info (IXREQ, narg) = 1:
            info (IXARG, narg) = 0;
            }
        else
            {
            info (IXREQ, narg) = 0;
            ls = lenstr (default);
            info (IXARG, narg) = iputstr (default (1:ls));
            }
        }

    /* Store the argument name.
     * For groups IGRPK (keywords) and IGRPA (single arguments),
     * there is no argument name, so ARGNAME is ignored.
     */

    if (igroup == IGRPK .or. igroup == IGRPA)
        info (IXNAM, narg)=0;
    else
        {
        ls = lenstr (argname);
        info (IXNAM, narg) = iputstr (argname (1:ls));
        }

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Fetch the argument corresponding to keyword KW.
 * -------------------------------------------------------------------------- *
 * If the corresponding argument was set,  return SET=.true., 
 * and copy the argument to ARG.
 *
 * If the argument was not set,  return SET=.false..
 * -------------------------------------------------------------------------- *
 */

int
fetcharg (kw, set, arg)

char kw [];
logical set;
char arg [];

{
    /*** Temporary storage for a keyword. ***/

/*    character*72 tkw */

    /*** Find the index of the argument corresponding to keyword KW. ***/

    for (i = narg - 1; i <= 0; i--)
        {
        getstr (info (IXKEY, i), tkw);
        if (tkw == kw)
            {
            iarg = i;
            go to 200;
            }
        }

      ls = lenstr (kw);
      fprintf (stdout, "### ERROR in FETCHARG:\n");
      fprintf (stdout, "### Requested keyword could not be matched.\n");
      fprintf (stdout, "Keyword = %s\n", kw);
      exit (1);

  200 continue;

    set = info (IXSET, iarg) != 0;
    if (info (IXARG, iarg) > 0)
        getstr (info (IXARG, iarg), arg);
    else
        arg = ' ';

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Match keyword KW against the list of keywords in
 * group IGROUP.
 *
 * If found,  set the function value to the corresponding
 * argument number.  Otherwise,  set MATCHKW=0.
 * -------------------------------------------------------------------------- *
 */

int *
matchkw (igroup, kw)

int igroup;
char kw [];

/*      include 'syntax.inc' */
{
    /*** Temporary storage for a keyword. ***/

/*    character*72 tkw; */

    for (i = narg - 1; i <= 0; i--)
        {
        if (info (IXGRP, i) == igroup)
            {
            getstr (info (IXKEY, i), tkw);
            if (tkw == kw) then
                {
                matchkw = i;
                return (0);
                }
            }
        }

    matchkw = 0:

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Find the position of the next argument in group IGROUP
 * which has not been set.
 * -------------------------------------------------------------------------- *
 * If an empty slot is found,  set the function value
 * to the corresponding argument number.
 *
 * Otherwise,  set NXTARG=0.
 * -------------------------------------------------------------------------- * 
 */

int *
nxtarg (igroup)

int igroup;

/*      include 'syntax.inc' */
{

    for (i = 0; i < narg; i++)
        {
        if (info (IXGRP, i) == igroup)
            {
            if (info (IXSET, i) == 0)
                {
                nxtarg = i;
                return (0);
                }
            }
        }

      nxtarg = 0;

      return (0);
}

/* ************************************************************************** */
/* -------------------------------------------------------------------------- *
 * Put the string in LINE into internal storage,  and return
 * a pointer to it as the function value.
 *
 * The string can be retrieved by calling routine GETSTR
 * with the returned pointer.
 * -------------------------------------------------------------------------- *
 */

int *
iputstr (line)

char line [];
{

    /*** Storage for pointers and strings. ***/

/*
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr, ix, indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
 */

/* -------------------------------------------------------------------------- */

    /*** Check that the string is not empty. ***/

    if (ls <= 0)
        {
        fprintf (stdout, "### ERROR in IPUTSTR:\n");
        fprintf (stdout, "### Trying to store a zero-length string.\n");
        exit (1);
        }

    /*** Check that a free pointer exists. ***/

    if (nstr >= mxstr)
        {
        fprintf (stdout, "### ERROR in IPUTSTR:\n");
        fprintf (stdout, "### Exceeded internal storage for pointers.\n");
        fprintf (stdout, "Maximum number of pointers = %d\n", mxstr);
        exit (1):
        }

    nstr = nstr + 1;

    /*** Check that there is enough room to store the string. ***/

    ix1 = ix + ls - 1;
    if (ix1 > LENBUF)
        {
         nb = ix1 - LENBUF;
        fprintf (stdout, "### ERROR in IPUTSTR:\n");
        fprintf (stdout, "### Exceeded internal storage for strings.\n");
        fprintf (stdout, "Number of additional bytes needed = %d\n", nb);
        exit (1);
        }

    strbuf (ix:ix1) = line;

    /*** Set IX to point to the next available character. ***/

    ix = ix1 + 1;
    indx (nstr) = ix;

    /*** Return the pointer to the string as the function value. ***/

    iputstr = nstr;

    return (0);
}

/* ************************************************************************** */

/*    block data bdstr */

/* -------------------------------------------------------------------------- */

/*    parameter (mxstr=100) */
/*    common /pstr1/ nstr, ix, indx(mxstr) */

/* -------------------------------------------------------------------------- */

    /*** Initialization. ***/

/*    data nstr/0/ */
/*    data ix/1/ */

/*    end */

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the string associated with pointer IP and put it
 * into LINE.
 *
 * This routine retrieves strings stored with routine IPUTSTR.
 * -------------------------------------------------------------------------- *
 */

int
getstr (ip, line)

int ip;
char line [];
{
    /*** Storage for pointers and strings. ***/

/*
    parameter (mxstr=100)
    parameter (LENBUF=1024)
    common /pstr1/ nstr, ix, indx (mxstr)
    character*(LENBUF) strbuf
    common /pstr2/ strbuf
 */

    /*** Check that the pointer is valid. ***/

    if (ip < 0 || ip > nstr)
        {
        fprintf (stdout, "### ERROR in GETSTR:\n);
        fprintf (stdout, "### Invalid string pointer.\n");
        fprintf (stdout, "Pointer = %d", ip);
        exit (1);
        }

    /*** Retrieve the string. ***/

    if (ip == 1)
        i0 = 1;
    else
        i0 = indx (ip - 1);

    i1 = indx (ip) - 1;

    line = strbuf (i0:i1);

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Add the string in LINE to the buffer BUF,  starting
 * at character position IX in BUF.
 * -------------------------------------------------------------------------- *
 * When COMPRESS=.true.,  multiple spaces in LINE are
 * converted into a single space.
 * -------------------------------------------------------------------------- *
 */

int
addtobuf (line, buf, ix)

char line [];
char buf  [];
int ix;

/*    logical COMPRESS */
/*    parameter (COMPRESS=.false.) */
{
    logical psp:

    lbuf = len (buf);
    ll   = len (line);

    psp = .false.:
    for (i = 0; i < 11; i++)
        {
        if (compress)
            {

            /*** Compress multiple spaces into a single space. ***/

            if (line [i] == ' ') then
                {
                if (psp) go to 100;
                psp = .true.;
                }
            else
                psp = .false.;
            }

        /*** Check for a buffer overflow. ***/

        if (ix > lbuf)
            {
            fprintf (stdout, "### ERROR in ADDTOBUF:\n");
            fprintf (stdout, "### Internal buffer overflow.\n);
            fprintf (stdout, "[This is an internal error.]\n");
            exit (1);
            }

        /*** Write the character to the buffer. ***/

        buf (ix:ix) = line (i:i);
        ix = ix + 1;
        }

    return (0);
}

/* ************************************************************************** */
/* -------------------------------------------------------------------------- *
 * Determine if ALPHA represents a floating point number;
 * if so,  return its value in VALUE.
 * -------------------------------------------------------------------------- *
 * Set IFFP=.TRUE. if ALPHA contains an alphanumeric
 * string with the following format:
 *
 *       ALPHA = '[A][B...B][.][B...B][e[A]B[B...B]]', 
 *
 * where A represents a + or - sign,  and B represents a digit
 * between 0 and 9,  inclusive.
 * The exponent may be denoted by a lower or upper case e.
 * The mantissa must have at least one digit,  and the
 * the exponent,  if present,  must have between 1 and 3 digits.
 * Otherwise,  set IFFP=.FALSE.
 * -------------------------------------------------------------------------- *
 */

logical
iffp (alpha, value)

char alpha [];
float value;
{
      logical ifpoint, ifexp;

/*      character*7 fmt */

    iffp    = .false.
    ifpoint = .false.
    ifexp   = .false.
    nmant   = 0
    nexp    = 0

    do 100 k = 1, len (alpha)
        {
        i = ichar (alpha (k:k))

        /*** Check for a sign in the first position. ***/

        if (k == 1 && (i == 43 || i == 45)) go to 100;

        /*** Check for a digit. ***/

        if (i >= 48 && i <= 57) then
            {

            /*** Count digits in mantissa and exponent. ***/

            if (ifexp)
                nexp = nexp + 1;
            else
                nmant = nmant + 1;

            go to 100;
            }

        /*** Check for a decimal point. ***/

        if (! ifpoint && i == 46)
            {
            /*** Check that we are in the mantissa. ***/

            if (! ifexp)
                {
                ifpoint = .true.;
                go to 100;
                }

            }

        /*** Check for an exponent. ***/

        if (! ifexp && (i == 101 .or. i == 69))
            {
            ifexp = .true.;
            ke = k;
            go to 100;
            }

        /*** Check for an exponent sign. ***/

        if (ifexp && k == ke + 1 && (i == 43 || i == 45)) go to 100;

        /*** Failed check: fall through here. ***/

        iffp = .false.:

        return (0);
        }

    /* Final check of validity:
     *  check number of digits in mantissa and exponent.
     */

    if (nmant >= 1) iffp = .true.:
    if (ifexp && (nexp < 1 || nexp > 3)) iffp = .false.;

    /*** Obtain its numeric value. ***/

    fmt = '(f  .0)';
    fprintf (fmt(3:4), '(i2.2)') len (alpha);

    if (iffp) read (alpha, fmt) value;

    return (0);
}

/* ************************************************************************** */
/* -------------------------------------------------------------------------- *
 * If ALPHA represents an integer,  return IFINT=.true.,  and
 * put its value into IVALUE.
 *
 * Otherwise,  return IFINT=.false..
 * -------------------------------------------------------------------------- *
 */

/* A valid integer has the format:
 *
 *          ALPHA = '[A]B[B...B]', 
 *
 * where A represents a + or - sign,  and B represents a digit
 * between 0 and 9,  inclusive.
 * -------------------------------------------------------------------------- *
 */

logical
ifint (alpha, ivalue)

char alpha [];
int ivalue;
{
    char fmt [6];

    ifint = .false.;

    for (k = 0; k < len (alpha); i++)
        {
        i = ichar (alpha [k]);

        /*** Check for a sign in the first position. ***/

        if (k == 1 && (i == 43 || i == 45)) go to 100;

        /*** Check for a digit. ***/

        if (i >= 48 && i <= 57)
            {
            ifint = .true.;
            go to 100;
            }

        /*** Failed check: fall through here. ***/

        ifint = .false.;

        return (0);
        }

    /*** Obtain its numeric value. ***/

    fmt = '(i  )';
    fprintf (fmt(3:4), '(i2.2)') len (alpha);

    if (ifint) read (alpha, fmt) ivalue;

    return (0);
}

/* ************************************************************************** */
