/* getpb.v3.c
 * -----------------------------------------------------------------------------
 * Calculate the polarization brightness
 * from the mass density (from the MAS code).
 *------------------------------------------------------------------------------
 * Updates and bug fixes:
 *
 *        05/19/94, ZM, Version 1.00:
 *
 *         - Original version of program.
 *
 *        01/24/95, ZM, Version 1.01:
 *
 *         - Generalized to handle 3D fields.
 *         - Added the capability to enhance the density perturbation.
 *         - Added the capability to apply a "vignetting function"
 *           to pB.
 *
 *        02/06/95, ZM, Version 1.02:
 *
 *         - Added the ability to rotate the output pB image
 *           in the theta direction.
 *
 *        04/28/95, ZM, Version 1.03:
 *
 *         - Added the ability to multiply pB by a radial vignetting
 *           function.
 *
 *        03/03/97, ZM, Version 1.04:
 *
 *         - Added the ability to specify the solar B0 and P angles.
 *
 *        12/03/2003, ZM, Version 1.05:
 *
 *         - Updated the program to be able to deal with
 *           nonuniform meshes in phi.
 *
 *        16 Jun 2004 [ALS] Cosmetic changes only.
 */

/* -------------------------------------------------------------------------- */
#include <stdio.h>
#include <math.h>

#define pi 3.14159265358979

int ntau = 100;			/* # steps along line-of-sight integration. */
int nr;
int nt;
int np;

int nrm1;
int ntm1;

float b0angle = 0.0;		/* Solar Bzero angle. */
float crlong  = 0.0;		/* Carrington Longitude. */
float emu;			/* Central meridian longitude. */
float fac     = 1.0;		/* Enhancement factor. */
float pangle  = 0.0;		/* P angle. */
float phi0;
float power   = 3.0;		/* Radial power exponent. */
float rocc    = 0.0;		/* Occulting disk radius. */
float tmin;
float tmax    = 1.5;
float tmn;
float tmx;
float x;

float sp, cp;			/* sin (P),  cos (P) */
float sb, cb;			/* sin (B0), cos (B0) */

static char infile  [72] = "";			/* Input  file name. */
static char outfile [72] = "";			/* Output file name. */
static char vffile  [72] = "";			/* Vignetting function file. */

float *iprh; 
float *ipth;
float *ipph;

float *iprho;

float *rh;
float *th;
float *ph;
float *rho;

float *ipr;
float *ipt;
float *ippb;

float *r;
float *t;
float *pb;

/* -------------------------------------------------------------------------- */

int
main (int argc, char *argv [])
{
    /*** Function Prototypes. ***/

    int   rdhdf    (char *fname, int maxdim, int *ndim, int *idim,
                    float **dbuf, int *ifscale, float ***sbuf);
    int   rmesh    (int nrm1, float rh [], float r []);
    int   tmesh2d  (int ntm1, float th [], float t []);
    int   tmeshed  (int ntm1, float th [], float t []);
    int   findpb   (int nr, int nt, float r [], float t [],
                    float tmin, float tmax, int ntau, float *pb);
    int   pmult    (int nr, int nt, float r [], float rocc, float power,
                    float *pb);
    int   vignette (int nrpb, int ntbp, float r [], int nrvf,
                    float rvf [], float vf [], float *pb);
    int   magrho   (int nr, int nt, int np, float th [], float ph [],
                    float *rho, float fac);
    float fmin     (float v1, float v2);
    int   wrsds    (char *fmt, char *fname, int ndim, int *idim, float *f,
                    int ifscale, float *sbuf);


    /*** Local Variables. ***/

    static char cname [12] = "getpb";
    static char cvers [12] = "1.05";
    static char cdate [12] = "12/03/2003";
    static char fmt   [16] = "hdf";		/* File format. */

    static int idimrho [3] = {0, 0, 0};
    static int idimvf  [3] = {0, 0, 0};
    static int idimpb  [3] = {0, 0, 0};
/*    static int sbuf     [3] = {0, 0, 0}; */
    float **sbuf;

    /*** Pointers to the vignetting function. ***/

    float **iprvf;
    float *ipvf;

    float *rvf;
    float *vf;

    int ierr    = 0;
    int ifscale = 0;
    int maxdim  = 3;
    int maxdimv = 1;
    int ndim    = 0;
    int np      = 0;
    int nr      = 0;
    int nrm1    = 0;
    int nrpb    = 0;
    int nrvf    = 0;
    int nt      = 0;
    int ntpb    = 0;
    int ntm1    = 0;
    int nwd     = 0;
    int ndimvf  = 0;

/* -------------------------------------------------------------------------- */

    /*** Set the parameters. ***/

/*
 *    setpar (cname, cvers, cdate);
 */
    sprintf (infile,  "/telesto/d/stanger/spdx/spdx-dist/mas-data/bp002.hdf");
    sprintf (outfile, "/telesto/d/stanger/spdx/spdx-dist/mas-data/bp002.hdf");

    /*** Read the density. ***/

    ierr = rdhdf (infile, maxdim, &ndim, idimrho, &iprho, &ifscale, &sbuf);

    if (ierr != 0)
        {
        fprintf (stdout, "### ERROR in %s:\n", cname);
        fprintf (stdout, "### Could not read the data set.\n");
        fprintf (stdout, "IERR (from rdhdf) = %d\n", ierr);
        fprintf (stdout, "File name: %s\n", infile);
        exit (1);
        }

    if (ndim == 2)
        {
        nr    = *(idimrho + 0);
        nt    = *(idimrho + 1);
        np    = 1;
        *iprh = **(sbuf + 0);
        *ipth = **(sbuf + 1);
        phi0  = 0.0;
        ipph  = &phi0;
        }
    else
    if (ndim == 3)
        {
        nr    = *(idimrho + 0);
        nt    = *(idimrho + 1);
        np    = *(idimrho + 2);
        *iprh = **(sbuf + 0);
        *ipth = **(sbuf + 1);
        *ipph = **(sbuf + 2);
        }
    else
        {
        fprintf (stdout, "### ERROR in %s:\n", cname);
        fprintf (stdout, "### The file does not have a 2D or 3D field.\n");
        fprintf (stdout, "Number of dimensions = %d\n", ndim);
        fprintf (stdout, "File name: %s\n", infile);
        exit (1);
        }

    if (ifscale == 0)
        {
        fprintf (stdout, "### ERROR in %s\n", cname);
        fprintf (stdout, "### The field does not have scales.\n");
        fprintf (stdout, "File name: %s\n", infile);
        exit (1);
        }

    nrm1 = nr - 1;
    ntm1 = nt - 1;

    /* Set the dimensions for the pB image.
     * For a 2D case, the image includes only one limb
     * (theta between 0 and pi).
     * For a 3D case, the image includes both limbs
     * (theta between 0 and 2*pi).
     */

    nrpb = nrm1;
    if (ndim == 2)
        ntpb = ntm1;
    else
        ntpb = 2 * ntm1 - 1;

    /*** Assign memory for PB and its scales. ***/

    nwd = nrpb;
    ipr = (float *) malloc (nwd * sizeof (float));

    nwd = ntpb;
    ipt = (float *) malloc (nwd * sizeof (float));

    nwd = nrpb * ntpb;
    ippb = (float *) malloc (nwd * sizeof (float));

    if (ipr == NULL || ipt == NULL || ippb == NULL)
        {
        fprintf (stdout, "### ERROR in %s\n", cname);
        fprintf (stdout, "### Could not allocate the required memory.\n");
        fprintf (stdout, "Number of words required = %d\n", nwd);
        exit (1);
        }

    /*** Get the r and t arrays. ***/

    rh = (float *) malloc ((nrm1 + 1) * sizeof (float));
    r  = (float *) malloc ( nrm1      * sizeof (float));
    rmesh (nrm1, rh, r);

    if (ndim == 2)
        {
        th = (float *) malloc ((ntm1 + 1) * sizeof (float));
        t  = (float *) malloc ( ntm1      * sizeof (float));
        tmesh2d (ntm1, th, t);
        }
    else
        {
        th = (float *) malloc ((    ntm1 + 1) * sizeof (float));
        t  = (float *) malloc ((2 * ntm1 - 1) * sizeof (float));
        tmesh3d (ntm1, th, t);
        }

    /*** Apply the enhancement factor to rho. ***/

    ph = (float *) (np * sizeof (float));
    magrho (nr, nt, np, th, ph, rho, fac);

    /*** Convert the Carrington longitude to radians. ***/

    emu = crlong * pi / 180.0;

    /*** Convert the solar P and B0 angles to radians. ***/

    pangle  = pangle  * pi / 180.0;
    b0angle = b0angle * pi / 180.0;

    sp = sin (pangle);
    cp = cos (pangle);
    sb = sin (b0angle);
    cb = cos (b0angle);

    /*** Make sure that the maximum tau does not exceed pi/2. ***/

    tmax = fmin (abs (tmax), 0.5 * pi);
    tmn  = -tmax;
    tmx  = tmax;

    /*** Calculate the polarization brightness. ***/

    findpb (nrpb, ntpb, r, t, tmn, tmx, ntau, pb);

    /*** Multiply by the requested radial power. ***/

    pmult (nrpb, ntpb, r, rocc, power, pb);

    /*** Apply the vignetting function (if it was specified). ***/

    if (strcmp (vffile, NULL) != 0)
        {
        ierr = rdhdf (vffile, maxdimv, &ndimvf, idimvf, &ipvf,
                      &ifscale, &iprvf);

        if (ierr != 0)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### Could not read the vignetting function.\n");
            fprintf (stdout, "IERR (from rdhdf) = %d\n", ierr);
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        if (ndimvf != 1)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### The vignetting function file does not contain a 1D field.\n");
            fprintf (stdout, "Number of dimensions = %d\n", ndimvf);
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        if (ifscale != 1)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### The vignetting function file does not contain a scale.\n");
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        nrvf = *idimvf;

        if (nrvf < 2)
            {
            fprintf (stdout, "### ERROR in %s\n", cname);
            fprintf (stdout, "### The vignetting function file has less than two points.\n");
            fprintf (stdout, "Number of points = %d\n", nrvf);
            fprintf (stdout, "File name: %s\n", vffile);
            exit (1);
            }

        /*** Apply the vignetting function. ***/

        rvf = (float *) (nrvf * sizeof (float));
	vf  = (float *) (nrvf * sizeof (float));
        vignette (nrpb, ntpb, r, nrvf, rvf, vf, pb);
        }

    /*** Write PB. ***/

    idimpb [0] = nrpb;
    idimpb [1] = ntpb;
    **(sbuf + 0) = *(ipr + 0);
    **(sbuf + 1) = *(ipt + 0);

    ierr = wrsds (fmt, outfile, 2, idimpb, pb, 1, &**sbuf);

    if (ierr != 0)
        {
        fprintf (stdout, "### ERROR in %s\n", cname);
        fprintf (stdout, "### Could not fprintf PB.\n");
        fprintf (stdout, "IERR (from wrsds) = %d\n", ierr);
        fprintf (stdout, "File name: %s\n", outfile);
        exit (1);
        }

    exit (0);
}

/* ########################################################################## */

int
rmesh (int nrm1, float *rh, float *r)

/*
 * float rh [nrm1 + 1];
 * float r  [nrm1];
 */
{
    int i;

    for (i = 0; i < nrm1; i++)
        r [i] = 0.5 * (rh [i] + rh [i + 1]);

    return (0);
}

/* ########################################################################## */

int
tmesh2d (int ntm1, float *th, float *t)

/*
 * float th [ntm1 + 1];
 * float t  [ntm1];
 */

{
    int j;

    for (j = 0; j < ntm1; j++)
        t [j] = 0.5* (th [j] + th [j + 1]);

      return (0);
}
/* ########################################################################## */

int
tmesh3d (int ntm1, float *th, float *t)

/*
 * float th [ntm1 + 1];
 * float t  [2 * ntm1 - 1];
 */
{
    int j;

    for (j = 0; j < ntm1; j++)
        t [j] = 0.5 * (th [j] + th [j + 1]);

    for (j = 1; j < ntm1 - 1; j++)
        t [2 * ntm1 - j] = 2.0 * pi - t [j];

    t [2 * ntm1 - 1] = 2.0 * pi - t [1];

    return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Enhance the density perturbations from a spherically
 * symmetric profile by the factor FAC.
 * -------------------------------------------------------------------------- *
 */

int
magrho (int nr, int nt, int np, float th [], float ph [],
        float *rho, float fac)

/*
 * float th  [nt];
 * float ph  [np];
 * float rho [np][nt][nr];
 */
{
    int i, j, k;
    float avg;
    float area;
    float st;
    float pm;
    float pp;
    float da;

    if (fac == 1.0) return (0);

    for (i = 0; i < nr; i++)
        {
        avg  = 0.0;
        area = 0.0;

        for (j = 1; j < nt - 1 ; j++)
          {
          st = sin (th [j]);

          for (k = 1; k < np - 1; k++)
            {
            pm   = (ph [k  ] - ph [k-1]);
            pp   = (ph [k+1] - ph [k  ]);
            da   = (pp - pm) * st;
/*            avg  = avg + rho [k][j][i] * da; */
            avg  = avg + *(rho + i + j*nr + k*nr*nt) * da;
            area = area + da;
            }
          }

        avg = avg / area;
        for (j = 0; j < nt; j++)
          {
          for (k = 0; k < np; k++)
            {
/*            rho [k][j][i] = avg + fac * (rho [k][j][i] - avg); */
            *(rho + i + j*nr + k*nr*nt) =
                avg + fac * (*(rho + i + j*nr + k*nr*nt) - avg);
            }
          }
        }

      return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Find the polarization brightness.
 * -------------------------------------------------------------------------- *
 */

int findpb (int nr, int nt, float r [], float t [],
	    float tmin, float tmax, int ntau, float *pb)

/* float pb [nt][nr]; */
{
    extern float spb ();
    int i, j;
    float rv;
    float tv;

    for (i = 0; i < nr; i++)
        {
        rv = r [i];
        for (j = 0; j < nt; j++)
          {
          tv = t [j];
/*          pb [j][i] = spb (rv, tv, tmin, tmax, ntau); */
          *(pb + i + j*nr) = spb (rv, tv, tmin, tmax, ntau);
          }
        }

      return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Multiply by the specified radial power.
 * -------------------------------------------------------------------------- *
 */

int
pmult (int nr, int nt, float *r, float rocc, float power, float *pb)

/*
 * float r [nr];
 * float pb [nt][nr];
 */
{
    int i, j;
    float rv;
    float occ;

    for (i = 0; i < nr; i++)
        {
        rv = r [i];

        if (rocc >= 1.0)
            {
            if (rv >= rocc)
                occ = 1.0;
            else
                occ = 0.0;
            }
        else
            occ = 1.0;

        for (j = 0; j < nt; j++)
            {
/*            pb [j][i] = occ * pb [j][i] * powf (rv, power); */
            *(pb + i + j*nr) = occ * *(pb + i + j*nr) * powf (rv, power);
            }
        }

    return (0);
}

/* ########################################################################## */
/* -------------------------------------------------------------------------- *
 * Apply the vignetting function.
 * -------------------------------------------------------------------------- *
 */

int
vignette (int nrpb, int ntpb, float *r,
          int nrvf, float *rvf, float *vf, float *pb)

/*
 * float r [nrpb];
 * float rvf [nrvf];
 * float vf  [nrvf];
 * float pb [ntpb][nrpb];
 */
{
    int interp (float x [], int n, float xv, int *i, float *alpha);
    int i, j;
    int ivf;
    float rv;
    float vfv;
    float avf;

    /* Check that the vignetting function scale
     * is monotonically increasing.
     */

    for (i = 0; i < nrvf - 1; i++)
        {
        if (rvf [i + 1] <= rvf [i])
            {
            fprintf (stdout, "### ERROR in VIGNETTE:\n");
            fprintf (stdout, "### The vignetting function scale is not monotonically increasing.\n");
            exit (1);
            }
        }

    for (i = 0; i < nrpb; i++)
        {
        rv = r [i];
        if (rv <= rvf [1])
          vfv = vf [1];
        else
        if (rv >= rvf [nrvf])
          vfv = vf [nrvf];
        else
          {
          interp (rvf, nrvf, rv, &ivf, &avf);
          vfv = (1.0 - avf) * vf [ivf] + avf * vf [ivf + 1];
          }
        for (j = 0; j < ntpb; j++)
/*          pb [j][i] = vfv * pb [j][i]; */
          *(pb + i + j*nrpb) = vfv * *(pb + i + j*nrpb);
        }

      return (0);
}
/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Calculate polarization brightness [adapted from F. Bagenal].
 * -------------------------------------------------------------------------- *
 */

/* RHO is the plane-of-sky radial distance in units of R_sun.
 *
 * (solar North pole ==> ETA=0 and ETA=2*pi).
 * Angles with ETA between 0 and pi specify the
 * right limb,  and angles with ETA between pi and 2*pi
 * specify the left limb.
 *
 * EMU is the central meridian longitude (Carrington
 * rotation longitude) in radians.
 *
 * TMIN and TMAX are the limits in the angle tau for which
 * to integrate along the line of sight.
 * They should not specify an interval outside the
 * interval (-pi/2 to pi/2).
 *
 * NT is the number of points to use in this
 * integration (with uniform tau interval).
 */

float
spb (float rho, float eta, float tmin, float tmax, int ntau)

{
    float cfun ();
    float rhoval ();

    float ct;
    float dtau;
    float phi;
    float pbval;
    float rhov;
    float rval;
    float se;
    float tau;
    float theta;
    float tt;
    float zeta;

    int i;

    dtau = (tmax - tmin) / ntau;

    tau = tmin - 0.5 * dtau;
    pbval = 0.0;
    for (i = 0; i < ntau; i++)
        {
        tau = tau + dtau;
        ct = cos (tau);
        rval = rho / ct;
        theta = acos (cos (eta) * ct);
        se = sin (eta);
        if (se == 0.0)
          {
          if (tau >= 0.0)
            zeta = 0.5 * pi;
          else
            zeta = -0.5 * pi;
          }
        else
          {
          tt = tan (tau);
          zeta = atan2 (tt, se);
          }
        phi = 0.5 * pi + zeta;
/*        phi = phi % (2.0 * pi); */
        while (phi > 2.0 * pi) phi -= 2.0 * pi;
        if (phi < 0.0) phi = phi + 2.0 * pi;
        rhov = rhoval (rval, theta, phi);
        pbval = pbval + rho * rhov * cfun (rval) * dtau;
        }

      return (pbval);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Coronal light scattering function [Billings 1966].
 *
 * R is the radius in units of R-sun.
 * -------------------------------------------------------------------------- *
 */

float 
cfun (float rr)

{
#define a1 8.69e-7
#define a2 0.37
#define a3 0.63

    float fmax (float v1, float v2);
    float r;
    float a;
    float r1a;
    float ra;
    float afunc;
    float b1, b2, b3, b4;
    float bf;
    float bfunc;
    float cfunc;

    r = fmax (1.0001, rr);

    a     = 1.0 / r / r;
    ra    = 1.0 / r;
    r1a   = sqrt (1.0 - a);
    afunc = r1a * a;
    b1    = (1.0 - a) / ra;
    b2    = 1.0 + 3.0 * a;
    b3    = (float) log ((double)((1.0 + ra) / r1a));
    bf    = b1 * b2 * b3;
    b4    = 1.0 - 3.0 * a;
    bfunc = -1.0 / 8.0 * (b4 - bf);
    cfunc = a1 * (a2 * afunc + a3 * bfunc);

    return (cfunc);
}

/* ########################################################################## */

float
rhoval (float rv, float tv, float pv)

{
    int   c2s ();
    int   s2c ();
    float f2d ();
    float f3d ();
    float fval;

    float x1, y1, z1;
    float x2, y2, z2;
    float rrv;
    float ttv;
    float ppv;

    float x0, y0, z0;

    /*** Transform to Cartesian coordinates. ***/

    s2c (rv, tv, pv, &x0, &y0, &z0);

    /*** Apply the solar P angle transformation. ***/

    x1 = x0;
    y1 = cp * y0 + sp * z0;
    z1 = -sp * y0 + cp * z0;

    /*** Apply the solar B0 angle transformation. ***/

    x2 = cb * x1 - sb * z1;
    y2 = y1;
    z2 = sb * x1 + cb * z1;

    /*** Transform back to spherical coordinates. ***/

    c2s (x2, y2, z2, &rrv, &ttv, &ppv);

    /*** Transform to the appropriate central meridian longitude. ***/

    ppv = ppv + emu;
/*    ppv = ppv % (2.0 * pi); */
    while (ppv > 2.0 * pi)  ppv -= 2.0 * pi;
    if (ppv < 0.0) ppv = ppv + 2.0 * pi;

    if (np == 1)
        fval = f2d (nr, nt, rh, th, rho, rrv, ttv);
    else
        fval = f3d (nr, nt, np, rh, th, ph, rho, rrv, ttv, ppv);

    return (fval);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Convert from Cartesian coordinates (X, Y, Z)
 * to spherical coordinates (R, T, P).
 *
 * This routine returns T and P in radians,  in the
 * following range:
 *
 *          0.0 <= t <= pi
 *          0.0 <= p < 2.0 * pi
 * -------------------------------------------------------------------------- *
 */
int
c2s (float x, float y, float z, float r, float t, float p)

{
    float pid2;
    float pi2;

    pid2 = 0.5 * pi;
    pi2  = 2.0 * pi;

    r = sqrt (x*x + y*y + z*z);

    if (r == 0.0)
        t = 0.0;
    else
        t = acos (z / r);

    if (x == 0.0)
        {
        if (y >= 0.0)
            p = pid2;
        else
            p = -pid2;
        }
    else
        p = atan2 (y, x);

    if (p < 0.0) p = p + pi2;

    return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Convert from spherical coordinates (R, T, P) 
 * to Cartesian coordinates (X, Y, Z).
 *
 * This routine assumes that T and P are in radians.
 * -------------------------------------------------------------------------- *
 */
int
s2c (float r, float t, float p, float *xx, float *yy, float *zz)

{
    float st;
    float x, y, z;

    st = sin (t);
    x = r * st * cos (p);
    y = r * st * sin (p);
    z = r * cos (t);

    *xx = x;
    *yy = y;
    *zz = z;

    return (0);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Interpolate the data in array F(NX, NY),  with scales
 * in X(NX) and Y(NY),  to the position (XV, YV).
 *
 * Return the interpolated value as the function value.
 * -------------------------------------------------------------------------- *
 */
float
f2d (int nx, int ny, float x [nx], float y [ny], float f [ny][nx],
     float xv, float yv)

/*
 * float x [nx];
 * float y [ny];
 * float f [ny][nx];
 */
{
    int interp (float *x, int n, float xv, int *i, float *alpha);
    float fmax (float v1, float v2);
    float fmin (float v1, float v2);

    int i, j;
    float xxv;
    float yyv;
    float ax;
    float ay;
    float f2dval;

    xxv = fmax (xv,  x [0]);
    xxv = fmin (xxv, x [nx-1]);
    yyv = fmax (yv,  y [0]);
    yyv = fmin (yyv, y [ny-1]);

    interp (x, nx, xxv, &i, &ax);
    interp (y, ny, yyv, &j, &ay);

    f2dval = (1.0 - ax) * ((1.0 - ay) * f [j][i]   + ay * f [j+1][i]  )
           +        ax  * ((1.0 - ay) * f [j][i+1] + ay * f [j+1][i+1]);

/*
 *    f2dval = (1.0 - ax) * ((1.0 - ay) *
 *             *(f + i   + j*nx) + ay * *(f + i   + (j+1)*nx)
 *           + ax  * ((1.0 - ay) *
 *             *(f + i+1 + j*nx) + ay * *(f + i+1 + (j+1)*nx));
 */

    return (f2dval);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Interpolate the data in array F [nz][ny][nx], with scales
 * in x [nx], y [ny], and z [nz], to the position (xv, yv, zv).
 *
 * The z coordinate is assumed to be periodic,
 * with a repetition length of 2*pi.
 *
 * Return the interpolated value as the function value.
 * -------------------------------------------------------------------------- *
 */

float
f3d (int nx, int ny, int nz, float x [nx], float y [ny], float z [nz],
     float f [nz][ny][nx], float xv, float yv, float zv)

/*
 * float x [nx];
 * float y [ny];
 * float z [nz];
 * float f [nz][ny][nx];
 */
{
    int interp (float *x, int n, float xv, int *i, float *alpha);
    float fmax (float v1, float v2);
    float fmin (float v1, float v2);

    int i, j, k;
    float ax;
    float ay;
    float az;
    float xxv;
    float yyv;
    float f3dval;

    xxv = fmax (xv,  x [0]);
    xxv = fmin (xxv, x [nx-1]);
    yyv = fmax (yv,  y [0]);
    yyv = fmin (yyv, y [ny-1]);

    interp (x, nx, xxv, &i, &ax);
    interp (y, ny, yyv, &j, &ay);
    interp (z, nz, zv,  &k, &az);

    f3dval = (1.0 - az)
           * ((1.0 - ax) * ((1.0 - ay) * f [k][j][i]   + ay * f [k][j+1][i])
           +         ax  * ((1.0 - ay) * f [k][j][i+1] + ay * f [k][j+1][i+1]))
           + az * ((1.0 - ax)
           * ((1.0 - ay) * f [k+1][j][i] + ay * f [k+1][j+1][i])
           + ax * ((1.0 - ay)
           * f [k+1][j][i+1] + ay * f [k+1][j+1][i+1]));

    return (f3dval);
}

/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Get interpolation factor ALPHA and table index I.
 *
 * This routine does not do the actual interpolation.  Use the
 * returned values of I and ALPHA to get the interpolated value.
 * -------------------------------------------------------------------------- *
 */

int
interp (float *x, int n, float xv, int *ii, float *alpha)

{
    int i;

    for (i = 0; i < n - 1; i++)
        {
        if (xv >= x [i] && xv <= x [i + 1])
            {
            *alpha = (xv - x [i]) / (x [i + 1] - x [i]);
            *ii = i;
            return (0);
            }
        }

    /*** Value not found --- signal error and stop. ***/

    fprintf (stdout, "### ERROR in INTERP:\n");
    fprintf (stdout, "Abscissa not found in table.");
    fprintf (stdout, "Abscissa requested = %f\n", xv);
    exit (1);

    return (0);
}
/* ########################################################################## */

/* -------------------------------------------------------------------------- *
 * Read a scientific data set from an HDF file.
 * -------------------------------------------------------------------------- *
 * Allocates the required memory. Returns pointers to the data & scales arrays.
 * -------------------------------------------------------------------------- *
 * Input arguments:
 * -------------------------------------------------------------------------- *
 *   fname   : [character array]
 *             HDF data file name.
 *
 *   maxdim  : [integer]
 *             Maximum number of dimensions in the data set that are allowed.
 *
 * -------------------------------------------------------------------------- *
 * Output arguments:
 * -------------------------------------------------------------------------- *
 *   ndim    : [integer pointer]
 *             Number of dimensions found in the data set.
 *
 *   idim    : [integer array,  dimension maxdim]
 *             NDIM dimensions of data set.
 *
 *   dbuf    : [floating point array]
 *             Pointer to the data array.
 *
 *   ifscale : [integer pointer]
 *             Flag to indicate the presence of scales (axes) in the data set.
 *             ifscale=0 means that scales were not found.
 *             ifscale=1 means that scales were found.
 *
 *   sbuf    : [floating point array,  dimension maxdim]
 *             When ifscale=1,  this array contains ndim  pointers
 *             to the scale arrays.
 *
 * -------------------------------------------------------------------------- *
 * return value
 * -------------------------------------------------------------------------- *
 *   ierr    : [integer]
 *             ierr=0 is returned if the data set was read successfully.
 *             Otherwise,  ierr is set to a nonzero value.
 * -------------------------------------------------------------------------- *
 */

int
rdhdf (char *fname, int maxdim, int *ndim, int *idim,
       float **dbuf, int *ifscale, float ***sbuf)

{
    int i;
    int ierr;
    int iret;
    int npix;
    int npixs;
    int nscale;

    float *fbuf;

    ierr = 0;

    fprintf (stdout, "rdhdf> fname: %s\n", fname);
    fprintf (stdout, "rdhdf> ndim: %d  idim [0,1,2]: %d %d %d maxdim: %d\n",
             *ndim, idim [0], idim [1], idim [2], maxdim);
    fflush (stdout);

    /*** Get the dimensions of the HDF. ***/

    iret = DFSDgetdims (fname, ndim, idim, maxdim);

    if (iret != 0)
        {
        fprintf (stdout, "### ERROR in rdhdf:\n");
        fprintf (stdout, "### Error while retrieving HDF dimensions.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "[Error return (from DFSDdims) = %d]\n", iret);
        ierr = 1;
        return (ierr);
        }

    fprintf (stdout, "ndim: %d\n", *ndim);
    for (i = 0; i < *ndim; i++)
        fprintf (stdout, "dim: %d  size: %d\n", i, idim [i]);

    /*** Check the dimensions. ***/

    for (i = 0; i < *ndim; i++)
        if (idim [i] <= 0)
          {
          fprintf (stdout, "### ERROR in rdhdf:\n");
          fprintf (stdout, "### Invalid value for dimension.\n");
          fprintf (stdout, "Dimension number = %d\n", i);
          fprintf (stdout, "Dimension value = %d\n", idim [i]);
          ierr = 2;
          return (ierr);
          }

    /*** Compute the data array size [# pixels]. ***/

    npix = idim [0];
    for (i = 1; i < *ndim; i++)  npix *= idim [i];

    fprintf (stdout, "npix: %d\n", npix);

    /*** Allocate the memory for the array. ***/

    *dbuf = (float *) malloc (npix * sizeof (float));

    if (*dbuf == 0)
      {
      fprintf (stdout, "### ERROR in rdhdf:\n");
      fprintf (stdout, "### Could not allocate the required memory.\n");
      fprintf (stdout, "Number of words required = %d\n", npix);
      ierr = 3;
      return (ierr);
      }

    /*** Check if scales are present. ***/

    nscale = 0;
    for (i = 0; i < *ndim; i++)
        {
        iret = DFSDgetdimscale (i + 1, idim [i], *dbuf);
        fprintf (stdout, "rdhdf> iret: %2d  scale idim [%d]: %d\n",
                 iret, i, idim [i]);
        if (iret == 0) nscale++;

/*
        else
            {
            fprintf (stdout, "rdhdf> DFSDgetdimscale idim [%d] failed.\n", i);
            ierr = 4;
            return (ierr);
            }
 */
        }

    fprintf (stdout, "rdhdf> nscale: %d  ndim: %d\n", nscale, *ndim);

    if (nscale == *ndim)
        *ifscale = 1;
    else
        *ifscale = 0;

    /*** Allocate memory and get the scales. ***/

    *sbuf = (float **) malloc ((*ndim) * sizeof (float *));
/*    *sbuf = malloc ((*ndim) * sizeof (float *)); */

    if (*ifscale != 0)
        {
        for (i = 0; i < *ndim; i++)
            {
            fbuf = (float *) malloc (idim [i] * sizeof (float));
            fprintf (stdout, "rdhdf> i: %d  fbuf: %d  idim: %d  ndim: %d\n",
                     i, fbuf, idim [i], *ndim);
            fflush (stdout);

            (*sbuf) [i] = fbuf;

/*            (*sbuf) [i]= (float *) malloc (idim [i] * sizeof (float)); */
/*            (*sbuf) [i]= malloc (idim [i] * sizeof (float)); */

/*            if (*sbuf [i] == 0) */
            if (fbuf == 0)
              {
              fprintf (stdout, "### ERROR in rdhdf:\n");
              fprintf (stdout, "### Could not allocate the required memory.\n");
              fprintf (stdout, "# words required = %d [dim: %d] ndim: %d\n",
                       idim [i], i, *ndim);
              ierr = 5;
              return (ierr);
              }

            iret = DFSDgetdimscale (i + 1, idim [i], (*sbuf) [i]);

            if (iret != 0)
              {
              fprintf (stdout, "### ERROR in rdhdf:\n");
              fprintf (stdout, "### Could not read scale [dim: %d].\n", i);
              ierr = 6;
              return (ierr);
              }
            }
        }

    /*** Read the data array. ***/

    iret = DFSDgetdata (fname, *ndim, idim, *dbuf);

    if (iret != 0)
        {
        fprintf (stdout, "### ERROR in rdhdf:\n");
        fprintf (stdout, "### Error while reading the HDF.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "[Error return (from DFSDgetdata) = %d]\n", iret);
        ierr = 7;
        return (ierr);
        }

      return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write a scientific data set.
 *
 * Use routine WRTXT or WRHDF,  depending on FMT.
 * -------------------------------------------------------------------------- *
 */

int
wrsds (char *fmt, char *fname, int ndim, int *idim, float f,
       int ifscale, float *sbuf)

{
    int ierr;

    ierr = 0;
    if (strncmp (fmt, "text", 4) == 0)
        ierr = wrtxt (fname, ndim, idim, f, ifscale, sbuf);
    else
    if (strncmp (fmt, "hdf", 3) == 0)
        ierr = wrhdf (fname, ndim, idim, f, ifscale, sbuf);
    else
        {
        fprintf (stdout, "### ERROR in wrsds:\n");
        fprintf (stdout, "### Invalid file format specified.\n");
        fprintf (stdout, "File name: %s\n", fname);
        fprintf (stdout, "Format = %s\n", fmt);
        ierr = 5;
        return (ierr);
        }

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write a scientific data set to text file FNAME.
 * -------------------------------------------------------------------------- *
 */

int
wrtxt (char *fname, int ndim, int *idim, float *f, int ifscale, float *sbuf)

{
    int ierr;
    int i;
    int nwd;
    float *ipx;			/*** Pointer for scales. ***/
    FILE *fid;

    ierr = 0;

    fid = fopen (fname, "rw");

    /*** Write the number of dimensions. ***/

    ierr = wrint (fid, 1, ndim);
      if (ierr != 0) goto s900;

    /*** Write the dimensions. ***/

    nwd = 1;
    for (i = 0; i < ndim; i++)
        {
        ierr = wrint (fid, 1, idim [i]);
        if (ierr != 0) goto s900;
        nwd = nwd * idim [i];
        }

    /*** Write the scales. ***/

    if (ifscale != 0)
        {
        ierr = wrint (fid, 1, 1);
        if (ierr != 0) goto s900;
        for (i = 0; i < ndim; i++)
            {
            ipx = (sbuf + i);
            ierr = wrfp (1, idim [i], ipx);
            if (ierr != 0) goto s900;
            }
        }
    else
        {
        ierr = wrint (fid, 1, 0);
        if (ierr != 0) goto s900;
        }

    /*** Write the array. ***/

    ierr = wrfp (1, nwd, f);
    if (ierr != 0) goto s900;

    close (1);

    return (0);

  s900:

    fprintf (stdout, "### ERROR in WRTXT:\n");
    fprintf (stdout, "### Error in writing data to text file.\n");
    fprintf (stdout, "File name: %s\n", fname);
    ierr = 1;

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write N words of INTEGER data into array I from unit IUN
 * using a free format text write.
 * -------------------------------------------------------------------------- *
 */

int
wrint (FILE *fid, int n, int *i)

{
    int ierr;

    ierr = 0;
    fprintf (fid, "%d\n", i);
    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write N words of REAL data into array F from unit IUN.
 * -------------------------------------------------------------------------- *
 */

int
wrfp (FILE *fid, int n, float *f)

{
    int i;
    int ierr;

    ierr = 0;

    for (i = 0; i < n; i++)
        {
        ierr = fprintf (fid, " %13.6e", f [i]);
        if (ierr != 0)  return (ierr);
        }
    ierr = fprintf (fid, "\n");

    return (ierr);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write a scientific data set to HDF file FNAME.
 * -------------------------------------------------------------------------- *
 */

int
wrhdf (char *fname, int ndim, int *idim, float *f, int ifscale, float *sbuf)

{
    float *ipx;			/*** Pointer for scales. ***/
    int i;
    int ierr;
    int iret;

    ierr = 0;

    iret = DFSDsetdims (ndim, idim);
    if (iret != 0) goto s900;

    if (ifscale != 0)
        {
        for (i = 0; i < ndim; i++)
            {
            ipx = (sbuf + i);
            iret = DFSDsetdimscale  (i, idim [i], ipx);
            if (iret != 0) goto s910;
            }
        }

      iret = DFSDputdata (fname, ndim, idim, f);
      if (iret != 0) goto s920;

      return (ierr);

  s900:

      fprintf (stdout, "### ERROR in WRHDF:\n");
      fprintf (stdout, "### Could not set the dimensions.\n");
      fprintf (stdout, "File name: %s\n", fname);
      fprintf (stdout, "[Error return (from DSSDIMS) = %d]\n", iret);
      ierr = 1;
      return (ierr);

  s910:

      fprintf (stdout, "### ERROR in WRHDF:\n");
      fprintf (stdout, "### Could not set the scales.\n");
      fprintf (stdout, "File name: %s\n", fname);
      fprintf (stdout, "[Error return (from DSSDISC) = %d]\n", iret);
      fprintf (stdout, "[Dimension number = %d]\n", i);
      ierr = 2;
      return (ierr);

  s920:

      fprintf (stdout, "### ERROR in WRHDF:\n");
      fprintf (stdout, "### Could not write the data array.\n");
      fprintf (stdout, "File name: %s\n", fname);
      fprintf (stdout, "[Error return (from DSPDATA) = %d]\n", iret);
      ierr = 3;

      return (ierr);
}

/* ************************************************************************** */

/* fmax.c
 * -----------------------------------------------------------------------------
 * Compute the maximum of two floating point variables.
 * -----------------------------------------------------------------------------
 */

float
fmax (float v1, float v2)

{
    if (v1 >= v2)  return (v1);
    else           return (v2);
}

/* ************************************************************************** */

/* fmin.c
 * -----------------------------------------------------------------------------
 * Compute the minimum of two floating point variables.
 * -----------------------------------------------------------------------------
 */

float
fmin (float v1, float v2)

{
    if (v1 <= v2)  return (v1);
    else           return (v2);
}

/* ************************************************************************** */
