/* polcarfpb3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MAS Polar coordinate image to Cartesian coordinates.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Original Author: Alice Lecinski
 * Modifications by Andrew L. Stanger [ALS]
 * 15 Mar 2000: [ALS] Use BIGSCALE to output a larger cartesian coorinate image.
 *  1 Apr 2000: [ALS] Use simple coordinate conversion computation.
 * 23 May 2000: [ALS] Eliminate SQ & TRY variables.  Add -d flag to specify
 *                    output dimensions.
 * 29 Nov 2000: Replace "dofnm" with "dofname4" (replace "slit" with "vig").
 * 13 Jun 2000: Crop inner FOV edge (-e option).
 * 26 Sep 2001: Adjust r0 & dr (based on 960 arcsec/Rsun) using mk4h->Radius.
 *  6 Nov 2001: Use FITS format (instead of native Mk4 file format).
 *  9 Nov 2001: Output is the same pixel type as the input file.
 *		Previously, in 6 Nov 2001 version, the output was floating pt.
 *  9 Jan 2002: Adjust r0 & dr if image calibration is version 1.
 * 17 Sep 2004: Adapt for MAS non-uniform array.
 *		Input files: image, radius, azimuth.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define START

#include "fitsio.h"
#include "longnam.h"

char warnbyte [] = "\
ALL of the images will be scaled from 0-255\n\n" ;

char warndim [] = "\
Dimensions of output image:" ;

double *prr, r0, dr;
double xcen, ycen ;			/* Solar disk center. */
float r2d ;				/* radians -> degrees. */

fitsfile *fpol, *frec ;
fitsfile *fradpos, *fazipos;

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{
    extern void printerror () ;
    extern float blintra () ;
    extern char *ctime () ;

    register char *w ;

    static char fts_pb      [80];
    static char fts_radius  [80];
    static char fts_azimuth [80];

    static char card      [FLEN_CARD] ;
    static char rec_file  [80] ;
    static char comment   [48] ;
    static char cbitpix   [48] ;
    static char cnaxis1   [48] ;
    static char cnaxis1a  [48] ;
    static char cnaxis1r  [48] ;
    static char cnaxis2   [48] ;
    static char ccoordnam [48] ;
    static char cdataform [48] ;
    static char cobject   [48] = "SOLAR CORONA" ;
    static char corigin   [48] = "High Altitude Observatory [HAO]" ;
    static char cdate_obs [48] = "DATE OF OBSERVATION: CCYY/MM/DD [UT]" ;
    static char cinstrume [48] = "rho002.hdf -> IDL -> FITS -> polcarfpb.c" ;
    static char ctelescop [48] = "pB DERIVED FROM MAS DENSITY MODEL" ;
    static char ctime_obs [48] = "TIME OF OBSERVATION: HH:MM:SS" ;
    static char cctype1   [48] ;
    static char cctype2   [48] ;
    static char ccrpix1   [48] ;
    static char ccrpix2   [48] ;
    static char ccrval1   [48] ;
    static char ccrval2   [48] ;
    static char ccdelt1   [48] ;
    static char ccdelt2   [48] ;
    static char ccrota1   [48] ;
    static char ccrota2   [48] ;
    static char ccrradius [48] = "PIXELS PER SOLAR RADIUS";
    static char ctype_obs [48] = "POLARIZATION BRIGHTNESS" ;

    static char coordnam  [24] = "HELIO-CENTRIC" ;
    static char ctype1    [24] ;
    static char ctype2    [24] ;
    static char dataform  [24] = "CARTESIAN" ;
    static char date_obs  [12] = "2004/09/22" ;
    static char extname   [12] ;
    static char instrume  [24] = "polcarfpb.c" ;
    static char telescop  [24] = "MAS MODEL" ;
    static char object    [24] = "SOLAR CORONA" ;
    static char origin    [24] = "HAO" ;
    static char time_obs  [12] = "00:00:00" ;
    static char type_obs  [24] = "pB";
    static char xtension  [12] ;

    float azilo  = 0.0 ;
    float azihi  = 0.0 ;
    float bkgval = 0.0 ;
    float bscale ;
    float bzero ;
    float crradius ;
    float crpix1, crpix2 ;
    float crval1, crval2 ;
    float cdelt1, cdelt2 ;
    float crota1, crota2 ;
    float datamin ;
    float datamax ;
    float dispmin ;
    float dispmax ;
    float diskval = 0.0 ;
    float fmin, fmax ;
    float fov = 6.0 ;
    float fval ;
    float radius_range ;
    float rdp, adp ;
    float rsun = 960.0 ;		/* arcsec / Rsun */
    float scale ;
    float zero ;

    double angdeg ;			/* Azimuth angle [degrees]. */
    double radmax = 3.00 ;
    double rad1  = 1.0  ;
    double radlo = 1.07 ;
    double radhi = 2.91 ;
    double rd ;
    double rs ;
    double rspix ;
    double pixrs ;
    double xd, yd ;

    int anynull ;
    int bitpix ;
    int bitpixe ;
    int hdunum ;
    int hdutype ;
    int howfar ;
    int ia, ir ;
    int i ;
    int ii, jj ;
    int ix, iy ;
    int ihour, iminute, isecond ;
    int iyear, imonth, iday ;
    int morekeys = 0 ;
    int nkeys ;
    int keypos ;
    int nbypix ;
    int nbypixe ;
    int npxext ;
    int npximg ;
    int npximgc ;
    int nullval = 0 ;
    int pixlo, pixhi ;
    int pmode = 0 ;			/* =0 silent, =1 verbose print mode. */
    int proc_code ;
    int px, py ;
    int status = 0 ;
    int xdim ;
    int ydim ;
    int xdima ;
    int xdimr ;
    int xdimc = 960 ;
    int ydimc = 960 ;
    int who ;

    static int naxis [2] ;
    static long fpiximg [2] ;
    long fpixext = 1 ;
    static long naxes [2] ;

    long begtime, endtime ;
    long btime, etime ;

    unsigned char *xbuf ;
    unsigned char *cimg ;
    short	  *simg ;
    int		  *iimg ;
    float	  *fimg ;
    double	  *dimg ;
    float         *frad ;
    float         *fazi ;

    float *pimg ;
    float *rimg ;

    r2d = 180.0 / acos (-1.0) ;

    (void) time (&begtime) ;

    if ( argc < 4 ) { prsum () ; exit (3) ; }

    sscanf (argv [1], "%s", fts_pb);
    sscanf (argv [2], "%s", fts_radius);
    sscanf (argv [3], "%s", fts_azimuth);

    printf ("fts_pb:      %s\n", fts_pb);
    printf ("fts_radius:  %s\n", fts_radius);
    printf ("fts_azimuth: %s\n", fts_azimuth);
    fflush (stdout) ;

    npximgc = xdimc * ydimc ;
	
    xcen = (xdimc - 1.0) * 0.5 ;
    ycen = (ydimc - 1.0) * 0.5 ;

    /* ---------------------------------------------------------------------- */

    /*** Open polar coordinate FITS file. ***/

    if ( fits_open_file (&fpol, fts_pb, READONLY, &status) )
         printerror (status) ;
    if ( fits_open_file (&fradpos, fts_radius, READONLY, &status) )
         printerror (status) ;
    if ( fits_open_file (&fazipos, fts_azimuth, READONLY, &status) )
         printerror (status) ;

    if (fits_get_num_hdus (fpol, &hdunum, &status))  printerror (status) ;
    if (fits_get_hdu_num  (fpol, &hdunum))           printerror (status) ;
    if (fits_get_hdu_type (fpol, &hdutype, &status)) printerror (status) ;

    if (pmode)
        {
        fprintf (stdout, "# hdus: %d\n", hdunum) ;
        fprintf (stdout, "hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;	
        fflush  (stdout) ;
	}

    /*** Get selected keyword parameters from the primary FITS header. ***/

    if (fits_read_key (fpol, TINT, "BITPIX", &bitpix,
			     comment,   &status))  printerror (status) ;
    if (fits_read_key (fpol, TINT, "NAXIS1", &xdim,
			     cnaxis1,   &status))  printerror (status) ;
    if (fits_read_key (fpol, TINT, "NAXIS2", &ydim,
			     cnaxis2,   &status))  printerror (status) ;
    nbypix = abs (bitpix) / 8 ;

    if (fits_read_key (fradpos, TINT, "NAXIS1", &xdimr,
			        cnaxis1r,   &status))  printerror (status) ;
    if (fits_read_key (fazipos, TINT, "NAXIS1", &xdima,
			        cnaxis1a,   &status))  printerror (status) ;

    /* ------------------------------------------------------------------ */

    /*** Read FITS image data [polar coordinates]. ***/

    npximg  = xdim * ydim ;
    nullval = 0 ;          /* Don't check for null values in the image. */

    /*** Allocate storage for image arrays. ***/

    pimg = (float *) malloc (xdim  * ydim  * sizeof (float)) ;
    rimg = (float *) malloc (xdimc * ydimc * sizeof (float)) ;

    xbuf = (unsigned char *) malloc (xdim * ydim * nbypix) ;

    /*** Override automatic intensity scaling. ***/
    /*** i.e., read unscaled pixel data. ***/

    scale = 1.0 ;
    zero  = 0.0 ;
    if (fits_set_bscale (fpol, scale, zero, &status)) printerror (status) ;

    fpiximg [0] = 1 ;
    fpiximg [1] = 1 ;

    /*** Read primary image array. ***/

    switch (bitpix)
        {
	case   8:
		    printf ("case c\n") ;  fflush (stdout) ;
		    cimg = (unsigned char *) xbuf ;
		    if (fits_read_img (fpol, TBYTE,   fpiximg [0], npximg,
			&nullval, cimg, &anynull, &status) )
			printerror ( status ) ;
		    for (i = 0 ; i < npximg; i++)
			*(pimg + i) = (float) *(cimg + i) ;
		    break ;
	case  16:
		    printf ("case s\n") ;  fflush (stdout);
		    simg = (short  *) xbuf ;
		    if (fits_read_img (fpol, TSHORT,  fpiximg [0], npximg,
			&nullval, simg, &anynull, &status) )
			printerror ( status ) ;
		    for (i = 0 ; i < npximg; i++)
			*(pimg + i) = (float) *(simg + i) ;
		    break ;
	case  32:
		    printf ("case i\n") ;  fflush (stdout) ;
		    iimg = (int    *) xbuf ;
		    if (fits_read_img (fpol, TINT,    fpiximg [0], npximg,
			&nullval, iimg, &anynull, &status) )
			printerror ( status ) ;
		    for (i = 0 ; i < npximg; i++)
			*(pimg + i) = (float) *(iimg + i) ;
		    break ;
	case -32:
		    printf ("case f\n") ;  fflush (stdout) ;
		    fimg = (float  *) xbuf ;
		    if (fits_read_img (fpol, TFLOAT,  fpiximg [0], npximg,
			&nullval, fimg, &anynull, &status) )
			printerror ( status ) ;
		    for (i = 0 ; i < npximg; i++)
			*(pimg + i) = (float) *(fimg + i) ;
		    break ;
	case -64:
		    printf ("case d\n") ;  fflush (stdout) ;
		    dimg = (double *) xbuf ;
		    if (fits_read_img (fpol, TDOUBLE, fpiximg [0], npximg,
		        &nullval, dimg, &anynull, &status) )
			printerror ( status ) ;
		    for (i = 0 ; i < npximg; i++)
			*(pimg + i) = (float) *(dimg + i) ;
		    break ;

	default:
		    printf ("case default\n") ;  fflush (stdout) ;
		    fprintf (stderr, "Invalid bitpix: %d\n", bitpix) ;
		    goto next ;
	}

    free (xbuf) ;

    if (pmode)
	{
	ix = 0 ;
	for (iy = 0; iy < ydim; iy += 10)
	    printf ("ix/iy: %3d %3d  pimg: %8.2e\n",
		    ix, iy, *(pimg + ix + iy*xdim));
            fflush (stdout) ;
	}

    frad = (float *) malloc (xdimr * sizeof (float));
    fazi = (float *) malloc (xdima * sizeof (float));

    if (fits_read_img (fradpos, TFLOAT,  fpiximg [0], xdimr,
		                &nullval, frad, &anynull, &status) )
			        printerror ( status ) ;
    printf ("fits_read_img (frad)\n");  fflush (stdout);
    if (pmode)
    for (ix = 0; ix < xdimr; ix++)
	{
	printf ("frad [%4d]: %7.2f\n", ix, *(frad + ix));
	fflush (stdout) ;
	}

    if (fits_read_img (fazipos, TFLOAT,  fpiximg [0], xdima,
		                &nullval, fazi, &anynull, &status) )
			        printerror ( status ) ;
    printf ("fits_read_img (fazi)\n");  fflush (stdout);
    if (pmode)
    for (iy = 0; iy < xdima; iy++)
	{
	printf ("fazi [%4d]: %7.2f\n", iy, *(fazi + iy) * r2d);
	fflush (stdout) ;
	}

    radius_range = *(frad + xdimr - 1) * 2.0 ;
    radlo        = *(frad) ;
    radhi        = *(frad + xdimr - 1) ;
    azilo        = *(fazi) * r2d ;
    azihi        = *(fazi + xdima - 1) * r2d ;

    r0    = 0.0;
    dr    = fov / xdimc ;
    rspix = fov / xdimc ;
    pixrs = 1.0 / rspix ;

    printf ("azilo :       %f\n", azilo);
    printf ("azihi :       %f\n", azihi);
    printf ("radlo :       %f\n", radlo);
    printf ("radhi :       %f\n", radhi);
    printf ("radius_range: %f\n", radius_range);
    printf ("rad1 :        %f\n", rad1);
    printf ("rspix:        %f\n", rspix);
    printf ("pixrs:        %f\n", pixrs);
    printf ("xdimc         %d\n", xdimc       );
    fflush (stdout) ;

    /* ---------------------------------------------------------------------- */

    /*** Convert from Polar to Cartesian coordinates. ***/

    for (iy = 0 ; iy < ydimc ; iy++)
        {
	yd = iy - ycen ;		/* y distance from center. */
	for (ix = 0 ; ix < xdimc ; ix++)
	    {
	    xd = ix - xcen ;		  ;	/* x distance from center */
	    rd = sqrt (xd * xd + yd * yd) ;	/* radial distance: pixels */
	    rs = rd * rspix ;			/* radial distance: Rsun */

	    angdeg = atan2 ((double) yd, (double) xd) * r2d - 90.0 ;
	    while (angdeg < 0.0) angdeg += 360.0 ;
	    while (angdeg > 360.0)  angdeg -= 360.0 ;

	    if (rs < rad1)
	        fval = diskval ;
	    else
	    if (rs <= radhi)
		fval = blintra (pimg, frad, fazi, 4, xdim, ydim, rs, angdeg);
	    else
		fval = bkgval ;

            if (pmode)
	        {
	        if (iy == ydimc/2)
		printf ("ix/iy:%3d %3d  rd: %7.2f  rs: %7.2f  angdeg: %7.2f  fval: %8.2e\n",
			 ix, iy, rd, rs, angdeg, fval);
                fflush (stdout) ;
	        }

            *(rimg + ix + iy * xdimc) = fval ;
	    }
        }

    free (pimg) ;

    /*** Find image min/max values. ***/

    fmin =  1.0e+10;
    fmin = -1.0e+10;
    for (i = 0 ; i < npximgc; i++)
	{
	if (*(rimg + i) < fmin)  fmin = *(rimg + i);
	if (*(rimg + i) > fmax)  fmax = *(rimg + i);
	}
    printf ("fmin/fmax: %f   %f\n", fmin, fmax);  fflush (stdout) ;

    /*** Draw a polar grid on top of occulting disk region. ***/

/*
 *    suncir4 (cbuf, xdim, ydim, xcen, ycen, 1.0/rspix, 0.0) ;
 */


    /*** Create output file name. ***/

    sprintf (rec_file, "pbxy.fts");

    fprintf (stdout, "rec_file: %s\n", rec_file) ;
    fflush (stdout) ;

    remove (rec_file) ;		/* Delete pre-existing file. */

    /*** Create output file. ***/

    if ( fits_create_file (&frec, rec_file, &status) )
	 printerror ( status ) ;
    printf ("fits_create_file (%s)\n", rec_file);  fflush (stdout);

    /* ---------------------------------------------------------------------- */

    /*** Move to FITS primary HDU [input file]. ***/

    hdunum = 1 ;
    if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	printerror (status) ;
    printf ("fits_movabs_hdu (fpol:1)\n");  fflush (stdout);

    /*** Copy Primary FITS header to output file. ***/

    if (fits_get_hdrpos (fpol, &nkeys, &keypos, &status))
	printerror (status) ;
    printf ("fits_get_hdrpos (fpol: %d)\n", keypos);  fflush (stdout);

    for (ii = 1; ii <= nkeys; ii++)
        {
	if (fits_read_record (fpol, ii, card, &status))
	    printerror (status) ;
        printf ("fits_read_record (fpol)\n");
        printf ("%s\n", card);
        fflush (stdout);

	if (fits_write_record (frec, card, &status))
	    printerror (status) ;
        printf ("fits_write_record (frec)\n"); fflush (stdout);
	}

    /* ---------------------------------------------------------------------- */

    /*** Update keywords in primary HDU header. ***/

    if (fits_update_key (frec, TINT, "BITPIX", &bitpix, 0, &status))
	printerror ( status );
    printf ("fits_update_key (frec: bitpix)\n"); fflush (stdout);

    if (fits_update_key (frec, TINT, "NAXIS1", &xdimc, 0, &status))
	printerror ( status );
    printf ("fits_update_key (frec: naxis1)\n"); fflush (stdout);

    if (fits_update_key (frec, TINT, "NAXIS2", &ydimc, 0, &status))
        printerror ( status );
    printf ("fits_update_key (frec: naxis2)\n"); fflush (stdout);

    if (fits_update_key (frec, TSTRING, "DATE-OBS", date_obs, cdate_obs,
	&status))  printerror ( status );

    if (fits_update_key (frec, TSTRING, "TIME-OBS", time_obs, ctime_obs,
	&status))  printerror ( status );

    if (fits_update_key (frec, TSTRING, "TELESCOP", telescop, ctelescop,
	&status))  printerror ( status );

    if (fits_update_key (frec, TSTRING, "INSTRUME", instrume, cinstrume,
	&status))  printerror ( status );

    if (fits_update_key (frec, TSTRING, "TYPE-OBS", type_obs, ctype_obs,
	&status))  printerror ( status );

    if (fits_update_key (frec, TSTRING, "ORIGIN",   origin,   corigin,
	&status))  printerror ( status );

    if (fits_update_key (frec, TSTRING, "OBJECT",   object,   cobject,
	&status))  printerror ( status );

    sprintf (dataform, "CARTESIAN") ;
    if (fits_update_key (frec, TSTRING, "DATAFORM", dataform, cdataform,
	&status))  printerror ( status );
    printf ("fits_update_key (frec: dataform\n"); fflush (stdout);

    sprintf (coordnam, "HELIO-CENTRIC") ;
    if (fits_update_key (frec, TSTRING, "COORDNAM", coordnam, ccoordnam,
	&status))  printerror ( status );
    printf ("fits_update_key (frec: coordnam\n"); fflush (stdout);

    sprintf (ctype1, "X") ;
    sprintf (ctype2, "Y") ;
    crpix1 = xdimc/2 - 0.5 ;
    crpix2 = ydimc/2 - 0.5 ;
    crval1 = 0.0 ;
    crval2 = 0.0 ;
    crota1 = 0.0 ;
    crota2 = 0.0 ;
    cdelt1 = rsun * rspix ;
    cdelt2 = rsun * rspix ;
    crradius = 1.0 / rspix ;		/* pixels/Rsun */
    printf ("crradius: %f\n", crradius);  fflush (stdout);

    sprintf (cctype1, "AXIS 1 TYPE: X") ;
    sprintf (cctype2, "AXIS 2 TYPE: Y") ;
    sprintf (ccrpix1, "X SUN CENTER [PIXEL]") ;
    sprintf (ccrpix2, "Y SUN CENTER [PIXEL]") ;
    sprintf (ccrval1, "X SUN CENTER [ARCSEC]") ;
    sprintf (ccrval2, "Y SUN CENTER [ARCSEC]") ;
    sprintf (ccdelt1, "X COORD INCREMENT [ARCSEC/PIXEL]") ;
    sprintf (ccdelt2, "Y COORD INCREMENT [ARCSEC/PIXEL]") ;
    sprintf (ccrota1, "X ROTATION [DEG.] FROM REFERENCE") ;
    sprintf (ccrota2, "Y ROTATION [DEG.] FROM REFERENCE") ;

    if (fits_update_key (frec, TSTRING, "CTYPE1", ctype1, cctype1, &status))
	printerror ( status );
    printf ("fits_update_key (frec: ctype1)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CRPIX1", &crpix1, ccrpix1, &status))
	printerror ( status );
    printf ("fits_update_key (frec: crpix1)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CRVAL1", &crval1, ccrval1, &status))
	printerror ( status );
    printf ("fits_update_key (frec: crval1)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CDELT1", &cdelt1, ccdelt1, &status))
	printerror ( status );
    printf ("fits_update_key (frec: cdelt1)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CROTA1", &crota1, ccrota1, &status))
	printerror ( status );
    printf ("fits_update_key (frec: crota1)\n"); fflush (stdout);

    if (fits_update_key (frec, TSTRING, "CTYPE2", ctype2, cctype2, &status))
	printerror ( status );
    printf ("fits_update_key (frec: ctype2)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CRPIX2", &crpix2, ccrpix2, &status))
	printerror ( status );
    printf ("fits_update_key (frec: crpix2)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CRVAL2", &crval2, ccrval2, &status))
	printerror ( status );
    printf ("fits_update_key (frec: crval2)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CDELT2", &cdelt2, ccdelt2, &status))
	printerror ( status );
    printf ("fits_update_key (frec: cdelt2)\n"); fflush (stdout);
    if (fits_update_key (frec, TFLOAT, "CROTA2", &crota2, ccrota2, &status))
	printerror ( status );
    printf ("fits_update_key (frec: crota2)\n"); fflush (stdout);

    if (fits_update_key (frec, TFLOAT, "CRRADIUS", &crradius, ccrradius,
	&status))
        printerror ( status );
    printf ("fits_update_key (frec: crradius\n"); fflush (stdout);

    bscale = 1.0 ;
    bzero  = 0.0 ;
    if (fits_update_key (frec, TFLOAT, "BSCALE", &bscale, 0, &status))
	printerror ( status );

    if (fits_update_key (frec, TFLOAT, "BZERO", &bzero, 0, &status))
	printerror ( status );

    datamin = fmin ;
    if (fits_update_key (frec, TFLOAT, "DATAMIN", &datamin, 0, &status))
	 printerror( status );

    datamax = fmax ;
    if (fits_update_key (frec, TFLOAT, "DATAMAX", &datamax, 0, &status))
	printerror( status );

    dispmin = fmin ;
    dispmin = -1.e-8 ;
    if (fits_update_key (frec, TFLOAT, "DISPMIN", &dispmin, 0, &status))
	printerror( status );

    dispmax = fmax ;
    dispmax = 5.0e-7 ;
    if (fits_update_key (frec, TFLOAT, "DISPMAX", &dispmax, 0, &status))
	printerror( status );

    if (ffgidt (frec, &bitpix, &status))  printerror (status) ; ;
    if (ffgidm (frec, naxis,   &status))  printerror (status) ; ;

    if (pmode)
        {
	printf ("frec: bitpix = %d\n", bitpix) ;
	printf ("frec: naxis = %d %d\n", naxis [0], naxis [1]) ;
	fflush (stdout) ;
	}

    /*** Write cartesian image array into the FITS data array. ***/

    fpiximg [0] = 1 ;
    fpiximg [1] = 1 ;

/*
    if (pmode)
        {
	ix = 0 ;
	for (iy = 0; iy < ydimc; iy += 10)
	    printf ("ix/iy: %3d %3d  cbuf: %8.2e\n",
		     ix, iy, *(cbuf + ix + iy*xdimc));
            fflush (stdout) ;
	}
 */

    scale = 1.0 ;
    zero  = 0.0 ;
    if (fits_set_bscale (frec, scale, zero, &status)) printerror (status) ;


    /* ---------------------------------------------------------------------- */

    switch (bitpix)
	{
	case   8:
		printf ("case c\n") ;  fflush (stdout) ;
		strcpy (cbitpix, "1-BYTE UNSIGNED CHARACTERS") ;
		cimg = (unsigned char *) malloc (npximgc * sizeof (unsigned char)) ;
		for (i = 0 ; i < npximgc; i++)
		    *(cimg + i) = (unsigned char) *(rimg + i) ;
		if (fits_write_img (frec, TBYTE, fpiximg [0], npximgc,
		    cimg, &status) ) printerror ( status ) ;
		free (cimg) ;
		break ;
	case  16:
		printf ("case s  ") ;  fflush (stdout) ;
		fflush (stdout) ;
		strcpy (cbitpix, "2-BYTE SIGNED INTEGERS") ;
		simg = (short *) malloc (npximgc * sizeof (short)) ;
		for (i = 0 ; i < npximgc; i++)
		    *(simg + i) = (short) *(rimg + i) ;
		if (fits_write_img (frec, TSHORT, fpiximg [0], npximgc,
		    simg, &status) ) printerror ( status ) ;
		free (simg) ;
		break ;
	case  32:
		printf ("case i\n") ;  fflush (stdout) ;
		strcpy (cbitpix, "4-BYTE SIGNED INTEGERS") ;
		iimg = (int *) malloc (npximgc * sizeof (int)) ;
		for (i = 0 ; i < npximgc; i++)
		    *(iimg + i) = (int) *(rimg + i) ;
		if (fits_write_img (frec, TINT, fpiximg [0], npximgc,
		    iimg, &status) ) printerror ( status ) ;
		free (iimg) ;
		break ;
	case -32:
		printf ("case f\n") ;  fflush (stdout) ;
		fmin =  1.0e+10;
		fmin = -1.0e+10;
		strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
		fimg = (float *) malloc (npximgc * sizeof (float)) ;
		for (i = 0 ; i < npximgc; i++)
		    {
		    *(fimg + i) = (float) *(rimg + i) ;
		    if (*(fimg + i) < fmin)  fmin = *(fimg + i);
		    if (*(fimg + i) > fmax)  fmax = *(fimg + i);
		    }
                printf ("fmin/fmax: %f   %f\n", fmin, fmax);  fflush (stdout) ;
		if (fits_write_img (frec, TFLOAT, fpiximg [0], npximgc,
		    fimg, &status) ) printerror ( status ) ;
		free (fimg) ;
		break ;
	case -64:
		printf ("case d\n") ;  fflush (stdout) ;
		strcpy (cbitpix, "8-BYTE DOUBLE PRECISION") ;
		dimg = (double *) malloc (npximgc * sizeof (double)) ;
		for (i = 0 ; i < npximgc; i++)
		    *(dimg + i) = (double) *(rimg + i) ;
		if (fits_write_img (frec, TDOUBLE, fpiximg [0], npximgc,
		    dimg, &status) ) printerror ( status ) ;
		free (dimg) ;
		break ;
        default:
		printf ("case default\n") ;  fflush (stdout) ;
		bitpix = -32 ;
		strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
		fimg = (float *) malloc (npximgc * sizeof (float)) ;
		for (i = 0 ; i < npximgc; i++)
		    *(fimg + i) = (unsigned char) *(rimg + i) ;
		if (fits_write_img (frec, TFLOAT, fpiximg [0], npximgc,
		    fimg, &status) ) printerror ( status ) ;
		free (fimg) ;
	}

    free (rimg) ;

    /* ---------------------------------------------------------------------- */

    /*** Close both input & output FITS files. ***/

    if (fits_close_file (fpol, &status) ||
        fits_close_file (frec, &status))		/* close files */
	printerror ( status );

    fits_close_file (fradpos, &status);
    fits_close_file (fazipos, &status);

next:
    free (xbuf) ;
    free (pimg) ;
    free (rimg) ;

    (void) time (&etime) ;
    fprintf (stderr,
    "time : %ld sec\n", etime - btime) ;
    fflush (stderr) ;

    (void) time (&endtime) ;
    fprintf (stderr,"Total time :    %ld seconds\n", endtime - begtime) ;
    fflush  (stderr) ;

    exit (0) ;
}

/* -------------------------------------------------------------------------- */

prsum ()
{
    static char help [] = "\
polcarfpb [-b] [-c] [-d dim] [polar coord file1] [polar coord file2] ...\n\
\n\
Convert a MAS FITS polar coordinate (r,theta) image to a cartesian image.\n\
\n\
Example:  polcarfpb  pb.fts  radius.fts  azimuth.fts  \n\
\n\
Option Flags: \n\
\n\
-c		Solar disk pixels will be set to -32000.0 \n\
-d xdim		Specify output image size (default is 960x960). \n\
\n\
" ;

       fputs (help, stderr) ;

       return (0) ;
}

/* -------------------------------------------------------------------------- */

    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/

void
printerror ( int status)
{

    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit( status );    /* terminate the program, returning error status */
        }
    return;
}

/* -------------------------------------------------------------------------- */

/* blintra.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * rval, aval:    floating point values of radius & azimuth position in image.
 * blintra returns a floating point value representing the intensity
 *    value at position (rval, aval).
 * Remember to include the statement
 *    float blintra ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, or floating-point images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 29 March 2000: Add nbypix parameter (instead of include statement).
 *                Add xdim & ydim parameters.
 *                Replace "npixl" with "xdim".
 * 18 Sep 2004: Modify for non-linear grid spacing, polar coordinates.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .  */

float
blintra (img, rpos, apos, nbypix, rdim, adim, rval, aval)

char	*img;		/* Pointer to image array. */
float    rpos [] ;	/* X-axis position values. */
float    apos [];	/* Y-axis position values. */
int	 nbypix ;	/* # bytes per pixel. */
int	 rdim ;		/* radius  dimension. */
int	 adim ;		/* azimuth dimension. */
float	 rval, aval;	/* Coordinates of scan point in image. */

{
register char	*im;
register short	*ims;
register float	*imf;
int     i;
int	ir, ia;
float	fr, fa;
float	f00, f10, f01, f11;
float	fr0, fr1, fra;

   if (rval < rpos [0] || rval > rpos [rdim-1])
      {

      fprintf (stderr, "Invalid X-axis coordinate: %f\n", rval) ;
      fflush  (stderr);

      return  (-1.0) ;
      }

   if (aval < apos [0] * r2d || aval > apos [adim-1] * r2d)
      {

      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", aval) ;
      fflush  (stderr);

      return  (-2.0) ;
      }

   im  = (char *)  img;
   ims = (short *) img;
   imf = (float *) img;

/*
 *   ir = (int) rval;
 *   ia = (int) aval;
 */

   /*** Find the nearest neighbor index for x & y. ***/

   ir = 0;
   for (i = 0; i < rdim-1; i++)
      if (rval > rpos [i] && rval < rpos [i+1]) ir = i;

   ia = 0;
   for (i = 0; i < adim-1; i++)
      if (aval > apos [i] * r2d && aval < apos [i+1] * r2d) ia = i;

   /*** Fraction of coordinate beyond the lower-value neighbor. ***/

   fr = (rval - rpos [ir]) / (rpos [ir+1] - rpos [ir]) ;
   fa = (aval - apos [ia] * r2d) / ((apos [ia+1] - apos [ia]) * r2d) ;

   /*** Select 4 nearest neighbor elements in 'img' array. ***/

   /*** 1 byte per pirel. ***/

   if (nbypix == 1)
      {
      f00 = (float) (*(im +  ir    +  ia   *rdim) & 0377);
      f10 = (float) (*(im + (ir+1) +  ia   *rdim) & 0377);
      f01 = (float) (*(im +  ir    + (ia+1)*rdim) & 0377);
      f11 = (float) (*(im + (ir+1) + (ia+1)*rdim) & 0377);
      }
   else

   /*** 2 bytes per pirel (short integer). ***/

   if (nbypix == 2)
      {
      f00 = (float) *(ims +  ir    +  ia   *rdim);
      f10 = (float) *(ims + (ir+1) +  ia   *rdim);
      f01 = (float) *(ims +  ir    + (ia+1)*rdim);
      f11 = (float) *(ims + (ir+1) + (ia+1)*rdim);
      }
   else

   /*** 4 bytes per pirel (floating point). ***/

   if (nbypix == 4)
      {
      f00 = (float) *(imf +  ir    +  ia   *rdim);
      f10 = (float) *(imf + (ir+1) +  ia   *rdim);
      f01 = (float) *(imf +  ir    + (ia+1)*rdim);
      f11 = (float) *(imf + (ir+1) + (ia+1)*rdim);
      }

   /*** Bilinear interpolation. ***/

   fr0 = f00 + fr * (f10 - f00);
   fr1 = f01 + fr * (f11 - f01);
   fra = fr0 + fa * (fr1 - fr0);

/*
 *   printf ("blintra> ir/ia: %4d  %4d   rval/aval: %7.2f  %7.2f  fra: %8.2e\n",
 *	    ir, ia, rval, aval, fra);
 *   fflush  (stdout);
 */

   return (fra);
}

/* -------------------------------------------------------------------------- */

