/* rden.c */
/* -----------------------------------------------------------------------------
 * rden
 * -----------------------------------------------------------------------------
 * This OpenDX module reads the density data file
 * created by the MAS Corona model software.
 *
 * INPUTS
 *   directory name: Name of directory containing HDF data files.
 *   timestep :      Time step to read in data.
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 * -----------------------------------------------------------------------------
 * Michael Wiltberger   HAO/NCAR   20 Feb 2004   [readmas.c]
 * Andrew L. Stanger    HAO/NCAR   13 May 2004   [rden.c]
 * 14 May 2004: Adapted to read density data only.
 * -----------------------------------------------------------------------------
 */

#include "hdf.h"
/* #include <dx/dx.h> */
#include <stdio.h>
#include <math.h>

/*** Define some useful constants and indexing functions. ***/

#define SIZE 256 	/* Max Size of filenames.    */
#define MAXDIMS 3	/* Max number of dimensions. */

/*** cijk  : indexing macro for DX arrays. ***/

#define cijk(c, i, j, k)  ((c)+iNumVars*((i)+iNx*((j)+iNy*(k))))

/*** fijk  : indexing macro for arrays of size Nx, Ny, Nz. ***/

#define fijk(i, j, k)  ((k)+iNz*((j)+iNy*(i)))

/*** fsijk : indexing macro for arrays of size NxS, NyS, NzS. ***/

#define fsijk(i, j, k)  ((k)+iNzS*((j)+iNyS*(i)))

/*** Function Prototypes. ***/

void Copy2DX   (int, float*, float*, int, int, int, int);
int  average   (float*, int, int, int, float*, int, int, int);
void CreateXYZ (float*, float*, float*, float*, int, int, int, int, int, int);

/* -------------------------------------------------------------------------- */

main (int argc, char *argv []) 

{
  /* --- Function Prototypes. --- */

  void Copy2DX   (int,  float*,  float*,  int,  int,  int,  int);
  int  average   (float*,  int,  int,  int,  float*,  int,  int,  int);
  void CreateXYZ (float*, float*, float*, float*, int, int, int, int, int, int);
  void rdhdf     (char *cpFname,    int     *ipMaxDim,    int *ipNDim,
                  int  *ipDim,      float  **fppData,
                  int  *ipIfScale,  float ***fpppScales,  int *ipErr);

  /* --- Variable Declarations --- */

  /*** Inputs ***/

  char *cpDirName = NULL; 	/* Pointer to MHD data file name. */
  int   iTimeStep;		/* Time step. */
  
  /*** Outputs ***/

  float *fpGrid;		/* Pointer to complete Grid location data. */
  float *fpMHDData;		/* Pointer to MHD data. */

  int   *ipMHDConnect;		/* Pointer to MHD Connection data. */

/*  Field fMHD = NULL; */	/* MHD data Field. */

  /*** Other variables ***/

  char  caFileName [SIZE];	/* Array for file name. */

  int   i, j, k;		/* Loop indices. */
  int   iDimSizes [MAXDIMS];	/* Number of dimensions. */
  int   iErr;			/* Error flag. */
  int   iJoff, iJoff2;
  int   iKoff, iKoff2;
  int   iMaxDims = MAXDIMS;	/* Max Number of Dimensions. */
  int   iNumDims;		/* Dimension sizes. */
  int   iNumCells;
  int   iNumPts;		/* Number of points in output field. */
  int   iNumVars;		/* Number of variables in output field. */
  int   iNx,  iNy,  iNz;	/* Array sizes.  */
  int   iNxS, iNyS, iNzS;	/* Array sizes for staggered quanities. */
  int   ip1;
  int   iIfScale;		/* Flag if scales are present. */

  int   *ipRConnect;
  int   *ipThetaConnect;
  int   *ipPhiConnect;

  float *fpData;		/* Pointer for data.   */
  float **fppScales;		/* Pointer for scales. */
  float *fpIData;		/* Pointer for interpolated data. */
  float *fpR, *fpTh, *fpPhi;	/* Pointer to  arrays for R, Th, and Phi. */

/*
 * Array aRConnect     = NULL;
 * Array aThetaConnect = NULL;
 * Array aPhiConnect   = NULL;
 */

  cpDirName = (char *) malloc (80);
  sprintf (cpDirName, "/telesto/d/stanger/spdx/spdx-dist/mas-data");

  iTimeStep = 2;

  /*** Check for the existence of the data directory name. ***/

/*
 * if (!in [0])
 *   { DXSetError (ERROR_BAD_PARAMETER, "Missing Directory Name");
 *   goto error;
 *   }
 * else if (!DXExtractString (in [0], &cpDirName))
 *   {
 *   DXSetError (ERROR_BAD_PARAMETER, "Directory Name must be string");
 *   goto error;
 *   }
 */

  /*** Check for the existence of the Time Step parameter. ***/

/*
 * if (!in [1])
 *   {
 *   DXSetError (ERROR_BAD_PARAMETER, "Missing Time Step");
 *   goto error;
 *   }
 * else if (!DXExtractInteger(in [1], &iTimeStep))
 *   {
 *   DXSetError (ERROR_BAD_PARAMETER, "Time Step must be Integer");
 *   goto error;
 *   }
 */

  /* ===========================================================================
   * The core most common grid is for Bphi,
   * so we use that first to establish the sizes of the data space
   * we're going to need for the DX field.
   * ===========================================================================
   */

  sprintf (caFileName, "%s/bp%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
  fprintf (stdout, "Opening file %s\n", caFileName);
#endif

  /*** Read the Bphi data [HDF format]. ***/

  rdhdf ( caFileName,  &iMaxDims,   &iNumDims,
          iDimSizes,   &fpData,
         &iIfScale,    &fppScales,  &iErr);

  /*** Save the dimensions. ***/

  iNx = iDimSizes [0];
  iNy = iDimSizes [1];
  iNz = iDimSizes [2];

#ifdef DEBUG
  fprintf (stdout, "Num Dims %d Nx %d Ny %d Nz %d\n", iNumDims, iNx, iNy, iNz);
#endif

  /* Make the positions array for the DXField,
   * given the c-language ordering: k ==> r,  j ==> theta,  i ==> phi.
   */

  if (iNumDims == 3)
    {
    iNumVars = 3;
    iNumPts = iNx * iNy * iNz;
/*
 *   aGrid = DXNewArray (TYPE_FLOAT, CATEGORY_REAL, 1, iNumVars);
 *   DXAddArrayData (aGrid, 0, iNumPts, NULL);
 *   fpGrid = (float*) DXGetArrayData (aGrid);
 */
    fpGrid = (float *) malloc (iNumVars * iNumPts * sizeof (float));

    for (k = 0; k < iNz; k++)
      {
      for (j = 0; j < iNy; j++)
	{
        for (i = 0; i < iNx; i++)
	  {
	  /*** X-axis ***/
          fpGrid [cijk(0, i, j, k)] = 
            fppScales [2][k] * sin (fppScales [1][j]) * cos (fppScales [0][i]);

          /*** Y-axis ***/
          fpGrid [cijk(1, i, j, k)] = 
            fppScales [2][k] * sin (fppScales [1][j]) * sin (fppScales [0][i]);

          /*** Z axis ***/
          fpGrid [cijk(2, i, j, k)] = 
            fppScales [2][k] * cos (fppScales [1][j]);

          /* Just a quick ordering test:
	   *
           * fpGrid [cijk (0, i, j, k)] = fppScales [0][i];
           * fpGrid [cijk (1, i, j, k)] = fppScales [1][j];
           * fpGrid [cijk (2, i, j, k)] = fppScales [2][k];
	   */
          }
        }
      }

    /* Make copies of R, Th, and Phi
     * for the creation of X, Y, Z arrays later.
     */

    fpR   = malloc (iNz * sizeof (float));
    fpTh  = malloc (iNy * sizeof (float));
    fpPhi = malloc (iNx * sizeof (float));

    for (k = 0; k < iNz; k++) fpR   [k] = fppScales [2][k]; 
    for (j = 0; j < iNy; j++) fpTh  [j] = fppScales [1][j]; 
    for (i = 0; i < iNx; i++) fpPhi [i] = fppScales [0][i]; 
    }

  /*** Create the data array for the DX field. ***/

  /* For MHDData, we have the following assignment of variables :
   *
   * 0 - Density
   * 1 - Pressure
   * 2 - Br
   * 3 - Bth
   * 4 - Bphi
   * 5 - Vr
   * 6 - Vth
   * 7 - Vphi
   * 8 - Jr
   * 9 - Jth
   * 10 - Jphi
   * 11 - Bx
   * 12 - By
   * 13 - Bz
   * 14 - Vx
   * 15 - Vy
   * 16 - Vz
   * 17 - Jx
   * 18 - Jy
   * 19 - Jz
   * 20 - R
   */

  iNumVars = 21;

/*
 * aMHDData = DXNewArray (TYPE_FLOAT, CATEGORY_REAL, 1, iNumVars);
 * DXAddArrayData (aMHDData, 0, iNumPts, NULL);
 * fpMHDData = (float*) DXGetArrayData (aMHDData);
 */

  fpMHDData = (float *) malloc (iNumVars * iNumPts * sizeof (float));

  /*** Copy Bphi data into the DX array. ***/

  if (3 == iNumDims)
    {
    Copy2DX (4, fpMHDData, fpData, iNumVars, iNx, iNy, iNz);

    /* Free the memory in fpData and fppScales
     * before the next call to rdhdf.
     */

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);
         
    /* =========================================================================
     * density
     * =========================================================================
     */

    /*** Read the density data [HDF format]. ***/

    sprintf (caFileName, "%s/rho%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    fprintf (stdout, "Opening file %s\n", caFileName);
#endif

    rdhdf ( caFileName,  &iMaxDims,   &iNumDims,
            iDimSizes,   &fpData,
           &iIfScale,    &fppScales,  &iErr);

/*
 *   if (1 == iErr)
 *     {
 *     DXMessage ("Failure reading %s", caFileName);
 *     DXSetError (ERROR_INTERNAL, "HDF read failed");
 *     goto error;
 *     }
 */

    /*** Save array dimensions. ***/

    iNxS = iDimSizes [0];
    iNyS = iDimSizes [1];
    iNzS = iDimSizes [2];

#ifdef DEBUG
    fprintf (stdout, "Num Dims %d NxS %d NyS %d NzS %d\n",
             iNumDims, iNxS, iNyS, iNzS);
#endif
    
    /*** Allocate storage for the interpolated version of the data. ***/

    fpIData = malloc (iNumPts * sizeof (float));

    /*** Compute the interpolated density data. ***/

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);

    /*** Copy interpolated density data to the DX array. ***/

    Copy2DX (0, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    /*** Release storage for data & scale arrays. ***/

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /*** Create the XYZ vectors from the R, Th, Phi for mulation. ***/

    CreateXYZ (fpMHDData, fpR, fpTh, fpPhi, 2, 11, iNumVars, iNx, iNy, iNz);
    CreateXYZ (fpMHDData, fpR, fpTh, fpPhi, 5, 14, iNumVars, iNx, iNy, iNz);
    CreateXYZ (fpMHDData, fpR, fpTh, fpPhi, 8, 17, iNumVars, iNx, iNy, iNz);

    /*** Copy R into the DX array. ***/

    for (k = 0; k < iNz; k++)
      {
      for (j = 0; j < iNy; j++)
	{
        for (i = 0; i < iNx; i++)
	  {
          fpMHDData [cijk (20, i, j, k)] = fpR [k];
          }
        }
      }
    }		/* Done with 3D reads */

  /*** Do the final steps for the DX field. ***/

/*
 * fMHD = DXNewField ();
 * if (!fMHD)
 *   {
 *   DXSetError (ERROR_NO_MEMORY, "Creation of MHD field failed");
 *   goto error;
 *   }
 */

  /*** Set the dependence of the Data Array to be on positions. ***/

/*
 * if (!DXSetStringAttribute ((Object) aMHDData, "dep", "positions"))
 *   {
 *   DXSetError (ERROR_INTERNAL, "Couldn't set Data Array Dep");
 *   goto error;
 *   }
 */

  /*** Set the MHD Data array as the data component of the MHD field. ***/

/*
 * if (!DXSetComponentValue (fMHD, "data", (Object) aMHDData))
 *   {
 *   DXSetError (ERROR_INTERNAL, "Couldn't set Data Array of MHD Field");
 *   goto error;
 *   }
 */

  /*** Set the MHD Pts array as the position component of the MHD field. ***/

/*
 * if (!DXSetComponentValue (fMHD, "positions", (Object) aGrid))
 *   {
 *   DXSetError (ERROR_INTERNAL, "Couldn't set Position Array of MHD Field");
 *   goto error;
 *   }
 */
  
  /*** This code makes regular connections. ***/

/*
 *  aMHDConnect = DXMakeGridConnectionsV (3, iDimSizes);
 *  aMHDConnect = DXMakeGridConnections  (3, iNz, iNy, iNx);
 *
 *  if (!aMHDConnect)
 *    {
 *    DXSetError (ERROR_INTERNAL, "Could Make MHD Grid Connections");
 *    goto error;
 *    }
 *
 *  if (!DXSetComponentValue (fMHD, "connections", (Object) aMHDConnect))
 *    {
 *    DXSetError (ERROR_INTERNAL, "Could set MHD Grid Connections");
 *    goto error;
 *    }
 */
  
  /* This code makes phi (i) direction connections periodic.
   * Regular connections yields (iNx-1) * (iNy-1) * (iNz-1) cells.
   * Periodic in Phi     yields (iNx-1) * (iNy-1) * (iNz-1) cells.
   */

  iNumVars  = 8;
  iNumCells = (iNx) * (iNy - 1) * (iNz - 1);
  iNumPts   = (iNx) * (iNy    ) * (iNz    );

#ifdef DEBUG
  fprintf (stdout, "NumVars %d, NumPts %d\n", iNumVars, iNumCells);
#endif

/*
 * aMHDConnect = DXNewArray (TYPE_INT, CATEGORY_REAL, 1, iNumVars);
 */

/*
 * if (!aMHDConnect)
 *   {
 *   DXSetError (ERROR_INTERNAL, "Could Make MHD Grid Connections");
 *   goto error;
 *   }
 */

/*
 * DXAddArrayData (aMHDConnect, 0, iNumCells, NULL);
 * ipMHDConnect = (int*) DXGetArrayData (aMHDConnect);
 */
  ipMHDConnect = (int *) malloc (iNumCells * sizeof (int));

  for (k = 0; k < iNz-1; k++)
    {
    iKoff  = iNx * iNy * k;	/* Offset for back lower left  corner */
    iKoff2 = iKoff + iNx * iNy;	/* Offset for back lower right corner */

    for (j = 0; j < iNy-1; j++)
      {
      iJoff  = iNx * j;		/* Offset for front lower left  corner */
      iJoff2 = iJoff + iNx;	/* Offset for front lower right corner */

/*
 *     for (i = 0; i < iNx; i++)
 *       {
 *       ip1 = i + 1;
 *       if (iNx-1 == i) ip1 = 0;
 *       *ipMHDConnect       =  i   + iJoff  + iKoff;
 *       *(ipMHDConnect + 1) =  ip1 + iJoff  + iKoff;
 *       *(ipMHDConnect + 2) =  i   + iJoff2 + iKoff;
 *       *(ipMHDConnect + 3) =  ip1 + iJoff2 + iKoff;
 *       *(ipMHDConnect + 4) = (i   + iJoff  + iKoff2) % iNumPts;
 *       *(ipMHDConnect + 5) = (ip1 + iJoff  + iKoff2) % iNumPts;
 *       *(ipMHDConnect + 6) = (i   + iJoff2 + iKoff2) % iNumPts;
 *       *(ipMHDConnect + 7) = (ip1 + iJoff2 + iKoff2) % iNumPts;
 *
 * #ifdef DEBUG
 *      if (k < 1)
 *        fprintf (stdout, "%d %d %d %d %d %d %d %d\n", 
 *        *(ipMHDConnect + 0), *(ipMHDConnect + 1), *(ipMHDConnect + 2), 
 *        *(ipMHDConnect + 3), *(ipMHDConnect + 4), *(ipMHDConnect + 5), 
 *        *(ipMHDConnect + 6), *(ipMHDConnect + 7));
 * #endif
 *       ipMHDConnect += 8;
 *       }
 */
      }
    }		/* Close loop over cell corners. */

/*
 * if (!DXSetConnections (fMHD, "cubes", (Object) aMHDConnect))
 *   {
 *   DXSetError (ERROR_INTERNAL, "Couldn't set connections");
 *   goto error;
 *   }
 */

/*
 * if (!DXEndField (fMHD))
 *   {
 *   DXSetError (ERROR_INTERNAL, "Couldn't complete field");
 *   goto error;
 *   }
 */

/*
 * out [0] = (Object) fMHD;
 */

  /*** Cleanup temporary space. ***/

  free (fpR);
  free (fpTh);
  free (fpPhi);
  free (fpIData);

  exit (0);

  error:
/*
 *   DXDelete ((Object) fMHD);
 *   DXDelete ((Object) aMHDData);
 *   DXDelete ((Object) aGrid);
 *   DXDelete ((Object) aMHDConnect);
 */

    /*** Cleanup temporary space. ***/

    free (fpR);
    free (fpTh);
    free (fpPhi);
    free (fpIData);

  exit (0);
}

/* -------------------------------------------------------------------------- */

/*** Copy data to the DX array. ***/

void Copy2DX (int iLoc, float *fpMHDData, float *fpData, 
              int iNumVars, int iNx, int iNy, int iNz)
{
  int i, j, k;

  for (k = 0; k < iNz; k++)
    {
    for (j = 0; j < iNy; j++)
      {
      for (i = 0; i < iNx; i++)
	{
        fpMHDData [cijk (iLoc, i, j, k)] = fpData [fijk (i, j, k)];
        }
      }
    }

  return;
}

/* -------------------------------------------------------------------------- */

/*** Compute the interpolated data. ***/

int average (float *fpData,  int iNxS, int iNyS, int iNzS, 
             float *fpIData, int iNx,  int iNy,  int iNz) 
{

   /* There are three possiblities :
    *
    *  vars like p   which have NzS = Nz + 1 and NyS = Ny + 1
    *  vars like Vth which have NzS = Nz + 1
    *  vars like Vr  which have NyS = Ny + 1
    */

   int i, j, k, iErr;

#ifdef DEBUG
   fprintf (stdout, "Inside average \n");
#endif

   iErr = 1;	/* Set Default to be an error unless we do an interpolation. */
   if ((iNzS == iNz+1) && (iNyS == iNy+1))
     {
     for (k=0; k<iNz; k++)
       {
       for (j=0; j<iNy; j++)
	 {
         for (i=0; i<iNx; i++)
	   {
           fpIData [fijk (i, j, k)] = 0.25 * (fpData [fsijk (i, j,   k  )]   +
                                              fpData [fsijk (i, j+1, k  )]   +
                                              fpData [fsijk (i, j,   k  )]   +
                                              fpData [fsijk (i, j,   k+1)]);
           }
         }
       }
     iErr = 0;
     }

   if ((iNzS == iNz + 1) )
     {
     for (k = 0; k < iNz; k++)
       {
       for (j = 0; j < iNy; j++)
	 {
         for (i = 0; i < iNx; i++)
	   {
           fpIData [fijk (i, j, k)] = 0.5 * (fpData [fsijk (i, j, k  )]   +
                                             fpData [fsijk (i, j, k+1)]);
           }
         }
       }
     iErr = 0;
     }

   if ((iNyS == iNy+1) )
     {
     for (k = 0; k < iNz; k++)
       {
       for (j = 0; j < iNy; j++)
	 {
         for (i = 0; i < iNx; i++)
	   {
           fpIData [fijk (i, j, k)] = 0.5 * (fpData [fsijk (i, j,   k)]     +
                                             fpData [fsijk (i, j+1, k)]);
           }
         }
       }
     iErr = 0;
     }

   if (1 == iErr)
     {
     fprintf  (stdout, "Nx  %d Ny  %d Nz  %d", iNx,  iNy,  iNz);
     fprintf  (stdout, "NxS %d NyS %d NzS %d", iNxS, iNyS, iNzS);
/*
 *    DXSetError (ERROR_INTERNAL, "Parameter mis-match");
 */
     }

   return iErr;
}

/* -------------------------------------------------------------------------- */

/*** Create the XYZ vectors from the R, Th, Phi for mulation. ***/

void CreateXYZ (float *fpVar,     float *fpR,  float *fpTh,  float *fpPhi, 
                int    iFrom,     int    iTo, 
                int    iNumVars,  int    iNx,  int    iNy,   int    iNz)
{
   int i, j, k;

   for (k = 0; k < iNz; k++)
     {
     for (j = 0; j < iNy; j++)
       {
       for (i = 0; i < iNx; i++)
	 {
         /* ----------------------------------------------------------------- */
	 /* Br: use r = i sin(Th)cos(Phi) + j sin(Th)sin(Phi) + k cos(Th)     */
         /* ----------------------------------------------------------------- */

         fpVar [cijk (iTo+0, i, j, k)] = 
            fpVar [cijk (iFrom+0, i, j, k)] * sin (fpTh [j]) * cos (fpPhi [i]);

         fpVar [cijk (iTo+1, i, j, k)] =
            fpVar [cijk (iFrom+0, i, j, k)] * sin (fpTh [j]) * sin (fpPhi [i]);

         fpVar [cijk (iTo+2, i, j, k)] = 
            fpVar [cijk (iFrom+0, i, j, k)] * cos (fpTh [j]);

         /* ----------------------------------------------------------------- */
         /* Bth: use r = i cos(Th)cos(Phi) + j cos(Th)sin(Phi) - k sin(Th)    */
         /* ----------------------------------------------------------------- */

         fpVar [cijk (iTo+0, i, j, k)] += 
            fpVar [cijk (iFrom+1, i, j, k)] * cos (fpTh [j]) * cos (fpPhi [i]);

         fpVar [cijk (iTo+1, i, j, k)] += 
            fpVar [cijk (iFrom+1, i, j, k)] * cos (fpTh [j]) * sin (fpPhi [i]);

         fpVar [cijk (iTo+2, i, j, k)] +=
            -1.0 * fpVar [cijk (iFrom+1, i, j, k)] * sin (fpTh [j]);

         /* ----------------------------------------------------------------- */
         /* Bphi: use r = -i sin(Phi) + j cos(Phi)                            */
         /* ----------------------------------------------------------------- */

         fpVar [cijk (iTo+0, i, j, k)] +=
            -1.0 * fpVar [cijk (iFrom+2, i, j, k)] * sin (fpPhi [i]);

         fpVar [cijk (iTo+1, i, j, k)] += 
            fpVar [cijk (iFrom+2, i, j, k)] * cos (fpPhi [i]);
         }
       }
     }

   return;
}
 
/* -------------------------------------------------------------------------- */

/*** Read a dataset in HDF format. ***/

void
rdhdf (char  *cpFname,    int      *ipMaxDim,    int  *ipNDim,
       int   *ipDim,      float   **fppData,
       int   *ipIfScale,  float  ***fpppScales,  int  *ipErr)

{

    int i, j, iRet;
    int iSize,  iNumScales;

    /* printf ("Filename %s\n", cpFname); */

    iRet = DFSDgetdims (cpFname, ipNDim, ipDim, *ipMaxDim);
    if (0 != iRet)
        {
        printf ("DFSDgetdims failed\n");
        *ipErr = 1;
        return;			/* Abort futher read on error */
        }

    /*** Get size needed for read and allocate space for it. ***/

    iSize = ipDim [0];
    for (i = 1; i < *ipNDim; i++) iSize *= ipDim [i];
    *fppData = malloc (iSize * sizeof (float));

    printf ("rdhdf> ipDim [0]: %d\n", ipDim [0]);
    printf ("rdhdf> ipDim [1]: %d\n", ipDim [1]);
    printf ("rdhdf> ipDim [2]: %d\n", ipDim [2]);
    printf ("rdhdf> iSize: %d\n", iSize);

    /*** Check if scales are present. ***/

    iNumScales = 0;
    for (i = 0; i < (*ipNDim); i++)
        {
        iRet = DFSDgetdimscale (i + 1, ipDim [i], *fppData);
        if (0 == iRet) iNumScales++;
            if (0 != iRet)
            {
            printf ("DFSDgetdimscale failed\n");
            *ipErr = 1;
            return;			/* Abort futher read on error */
            }
        }
    if (iNumScales == *ipNDim)
        {
        *ipIfScale = 1;
        }
    else
        {
        *ipIfScale = 0;
        }

    /*** Allocate memory and get scales. ***/

    *fpppScales = malloc ((*ipNDim) * sizeof (float*));

    if (1 == *ipIfScale)
        {
        for (i = 0; i < (*ipNDim); i++)
            {
            (*fpppScales)[i] = malloc (ipDim [i] * sizeof (float));
            iRet = DFSDgetdimscale (i + 1, ipDim [i], (*fpppScales) [i]);
            if (0 != iRet)
                {
                printf ("DFSDgetdimscale failed\n");
                *ipErr = 1;
                return;			/* Abort futher read on error */
                }
            }

      /*** Output scales check. ***/
    
      printf ("*fpppScales [i][j] :\n");

      for (i = 0; i < (*ipNDim); i++)
          {
          for (j = 0; j < ipDim [i]; j++)
              {
              printf ("[i,j]: [%3d, %3d] %10.4f\n", i, j, (*fpppScales) [i][j]);
              }
          printf ("=============\n");
          }
     
        }

    /*** Read the data array. ***/

    iRet = DFSDgetdata (cpFname, *ipNDim, ipDim, *fppData);
    if (0 != iRet)
        {
        printf ("DFSDgetdata failed\n");
        *ipErr = 1;
        return;				/* Abort futher read on error */
        }
        
    if (*ipErr != 1) *ipErr = 0;
    return;
}

/* -------------------------------------------------------------------------- */
