/* readmas.c */
/* -----------------------------------------------------------------------------
 * readmas
 * -----------------------------------------------------------------------------
 * This module reads the data files created by the MAS Corona code.
 *
 * INPUTS
 *   directory name: Name of directory with HDF data files
 *   timestep : time step to read in data for
 *
 * OUTPUTS
 *   mhd: field containing mhd data
 * -----------------------------------------------------------------------------
 * 20 Feb 2004
 * 29 Jun 2004 [ALS]: cosmetic changes only.
 * -----------------------------------------------------------------------------
 */

#include "hdf.h"
#include <dx/dx.h>
#include <stdio.h>
#include <math.h>

/*** Define some useful constants and indexing functions ***/

#define SIZE 256 	/* Max Size of filenames */
#define MAXDIMS 3	/* Max number of dimensions */

/*** cijk : indexing macro for DX Arrays ***/

#define cijk(c, i, j, k)  ((c)+iNumVars*((i)+iNx*((j)+iNy*(k))))

/*** fijk : indexing macro for Fortran Arrays of Size Nx, Ny, Nz   ***/

#define fijk(i, j, k)  ((k)+iNz*((j)+iNy*(i)))

/*** fsijk : indexing macro for Fortran Arrays of Size NxS, NyS, NzS ***/

#define fsijk(i, j, k)  ((k)+iNzS*((j)+iNyS*(i)))

/*** Function Prototypes ***/

void Copy2DX   (int, float*, float*, int, int, int, int);
int average    (float*, int, int, int, float*, int, int, int);
void CreateXYZ (float*, float*, float*, float*, int, int, int, int, int, int);

/* -------------------------------------------------------------------------- */

Error
m_ReadMAS (Object *in, Object *out) 

{
  /*--- Variable Declarations ---*/

  /*** Inputs ***/

  char *cpDirName = NULL; 	/* Pointer to MHD data file name. */
  int iTimeStep;		/* Time step. */
  
  /*** Outputs ***/

  Array   aGrid       = NULL; 	/* Array for Grid locations */
  float *fpGrid;		/* Pointer to complete Grid  location data */

  Array   aMHDData    = NULL;	/* Array for MHD Field data */
  float *fpMHDData;		/* Pointer to MHD data */ 

  Array   aMHDConnect = NULL;	/* Array containing conncetions for MHD field */
  int   *ipMHDConnect;		/* Pointer to MHD Connection data */

  Field fMHD          = NULL;	/* MHD data Field */

  /*** Other variables ***/

  char  caFileName [SIZE];	/* Array for file name */

  int   i, j, k;		/* loop indices */
  int   iaDimSizes [MAXDIMS];	/* Number of dimensions. */
  int   iNumDims;		/* Dimension sizes */
  int   iErr;			/* Error flag */
  int   iMaxDims;		/* Max Number of Dims */
  int   iNumCells;
  int   iJoff, iJoff2;
  int   iKoff, iKoff2;
  int   iIfScale;		/* Flag if scales are present */
  int   iNumVars;		/* Number of Varibles in output field */
  int   iNx, iNy, iNz;		/* Array Sizes  */
  int   iNxS, iNyS, iNzS;	/* Array Sizes for staggered quanities. */
  int   iNumPts;		/* Num Pts in output field */
  int   ip1;

  int   *ipRConnect;
  int   *ipThetaConnect;
  int   *ipPhiConnect;

  float *fpData;		/* Pointer for data. */
  float **fppScales;		/* Pointer for scales. */
  float *fpIData;		/* Pointer for interpolated data. */
  float *fpR, *fpTh, *fpPhi;	/* Pointer to arrays for R, Th, and Phi. */

  Array aRConnect     = NULL;
  Array aThetaConnect = NULL;
  Array aPhiConnect   = NULL;

  if (!in [0])
    {
    DXSetError (ERROR_BAD_PARAMETER, "Missing Directory Name");
    goto error;
    }
  else if (!DXExtractString (in [0], &cpDirName))
    {
    DXSetError (ERROR_BAD_PARAMETER, "Directory Name must be string");
    goto error;
    }

  if (!in [1])
    {
    DXSetError (ERROR_BAD_PARAMETER, "Missing Time Step");
    goto error;
    }
  else if (!DXExtractInteger(in [1], &iTimeStep))
    {
    DXSetError (ERROR_BAD_PARAMETER, "Time Step must be Integer");
    goto error;
    }

  /* ---------------------------------------------------------------------------
   * The core most common grid is for Bphi,
   * so we use that first to establish the sizes of the data space
   * we're going to need for the DX field.
   * ---------------------------------------------------------------------------
   */

  sprintf (caFileName, "%s/bp%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
  DXMessage ("Opening file %s", caFileName);
#endif

  iMaxDims = MAXDIMS;

  /*** Read in the HDF data. ***/

  rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
         &fppScales, &iErr);

  if (1 ==  iErr)
    {
    DXMessage ("Failure reading %s", caFileName);
    DXSetError (ERROR_INTERNAL, "HDF read failed");
    goto error;
    }

  iNx = iaDimSizes [0];
  iNy = iaDimSizes [1];
  iNz = iaDimSizes [2];

#ifdef DEBUG
  DXMessage ("Num Dims %d Nx %d Ny %d Nz %d", iNumDims, iNx, iNy, iNz);
#endif

  /* Make the positions array for the DXField,
   * given the c-language ordering: k == > r, j == > theta, i == > phi.
   */

  if (3 ==  iNumDims)
    {
    iNumPts = iNx * iNy * iNz;

    aGrid = DXNewArray (TYPE_FLOAT, CATEGORY_REAL, 1, 3);
    DXAddArrayData (aGrid, 0, iNumPts, NULL);
    fpGrid = (float*) DXGetArrayData (aGrid);

    iNumVars = 3;

    for (k = 0; k < iNz; k++)
      {
      for (j = 0; j < iNy; j++)
	{
        for (i = 0; i < iNx; i++)
	  {
          fpGrid [cijk (0, i, j, k)] = 
            fppScales [2][k] * sin (fppScales [1][j]) * cos (fppScales [0][i]);
          fpGrid [cijk(1, i, j, k)] = 
            fppScales [2][k] * sin (fppScales [1][j]) * sin (fppScales [0][i]);
          fpGrid [cijk(2, i, j, k)] = 
            fppScales [2][k] * cos (fppScales [1][j]);

          /*  Just a quick ordering test
           * fpGrid [cijk (0, i, j, k)] = fppScales [0][i];
           * fpGrid [cijk (1, i, j, k)] = fppScales [1][j];
           * fpGrid [cijk (2, i, j, k)] = fppScales [2][k];
	   */
          }
        }
      }

    /* Make copies of R, Th, and Phi
     * for the creation of X, Y, Z arrays later.
     */

    fpR   = malloc (sizeof (float) * iNz);
    fpTh  = malloc (sizeof (float) * iNy);
    fpPhi = malloc (sizeof (float) * iNx);

    for (k = 0; k < iNz; k++) fpR   [k] = fppScales [2][k]; 
    for (j = 0; j < iNy; j++) fpTh  [j] = fppScales [1][j]; 
    for (i = 0; i < iNx; i++) fpPhi [i] = fppScales [0][i]; 
    }

  /*** Create the data array for the DX field. ***/

  /* For MHDData, we have the following assignment of variables :
   *
   * 0 - Density
   * 1 - Pressure
   * 2 - Br
   * 3 - Bth
   * 4 - Bphi
   * 5 - Vr
   * 6 - Vth
   * 7 - Vphi
   * 8 - Jr
   * 9 - Jth
   * 10 - Jphi
   * 11 - Bx
   * 12 - By
   * 13 - Bz
   * 14 - Vx
   * 15 - Vy
   * 16 - Vz
   * 17 - Jx
   * 18 - Jy
   * 19 - Jz
   * 20 - R
   */

  iNumVars = 21;
  aMHDData = DXNewArray (TYPE_FLOAT, CATEGORY_REAL, 1, iNumVars);
  DXAddArrayData (aMHDData, 0, iNumPts, NULL);
  fpMHDData = (float*) DXGetArrayData (aMHDData);

  if (3 ==  iNumDims)
    {
    Copy2DX (4, fpMHDData, fpData, iNumVars, iNx, iNy, iNz);

    /* Now that we've copied that data into the DXArray,
     * we can free the memory in fpData and fppScales
     * before the next call to rdhdf.
     */

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);
         
    /* -------------------------------------------------------------------------
     * density
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/rho%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif
    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);
    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    /*** Save dimension sizes. ***/

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif
    
    /*** Allocate storage for the interpolated version of the data. ***/

    fpIData = malloc (sizeof (float) * iNumPts);

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (0, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * pressure
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/p%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (1, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Br
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/br%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (2, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Bth
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/bt%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (3, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Vr
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/vr%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (5, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Vth
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/vt%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (6, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Vphi
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/vp%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (7, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Jr
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/jr%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (8, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Jth
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/jt%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (9, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /* -------------------------------------------------------------------------
     * Jphi
     * -------------------------------------------------------------------------
     */

    sprintf (caFileName, "%s/jp%3.3d.hdf", cpDirName, iTimeStep);
#ifdef DEBUG
    DXMessage ("Opening file %s", caFileName);
#endif

    rdhdf (caFileName, &iMaxDims, &iNumDims, iaDimSizes, &fpData, &iIfScale, 
           &fppScales, &iErr);

    if (1 ==  iErr)
      {
      DXMessage ("Failure reading %s", caFileName);
      DXSetError (ERROR_INTERNAL, "HDF read failed");
      goto error;
      }

    iNxS = iaDimSizes [0];
    iNyS = iaDimSizes [1];
    iNzS = iaDimSizes [2];

#ifdef DEBUG
    DXMessage ("Num Dims %d NxS %d NyS %d NzS %d", iNumDims, iNxS, iNyS, iNzS);
#endif

    average (fpData, iNxS, iNyS, iNzS, fpIData, iNx, iNy, iNz);
    Copy2DX (10, fpMHDData, fpIData, iNumVars, iNx, iNy, iNz);

    free (fpData);
    for (i = 0; i < iNumDims; i++) free (fppScales [i]);
    free (fppScales);

    /*** Create the XYZ vectors from the R, Th, Phi for mulation ***/

    CreateXYZ (fpMHDData, fpR, fpTh, fpPhi, 2, 11, iNumVars, iNx, iNy, iNz);
    CreateXYZ (fpMHDData, fpR, fpTh, fpPhi, 5, 14, iNumVars, iNx, iNy, iNz);
    CreateXYZ (fpMHDData, fpR, fpTh, fpPhi, 8, 17, iNumVars, iNx, iNy, iNz);

    /*** Copy R into variable ***/

    for (k = 0; k < iNz; k++)
      {
      for (j = 0; j < iNy; j++)
	{
        for (i = 0; i < iNx; i++)
	  {
          fpMHDData [cijk (20, i, j, k)] = fpR [k];
          }
        }
      }
    }		/* Done with 3D reads */

  /*** Do the final steps for the DXField. ***/

  fMHD = DXNewField ();
  if (!fMHD)
    {
    DXSetError (ERROR_NO_MEMORY, "Creation of MHD field failed");
    goto error;
    }

  /*** Set the dependence of the Data Array to be on positons. ***/

  if (!DXSetStringAttribute ((Object) aMHDData, "dep", "positions"))
    {
    DXSetError (ERROR_INTERNAL, "Couldn't set Data Array Dep");
    goto error;
    }

  /*** Set the MHD Data array as the data component of the MHD Field. ***/

  if (!DXSetComponentValue (fMHD, "data", (Object) aMHDData))
    {
    DXSetError (ERROR_INTERNAL, "Couldn't set Data Array of MHD Field");
    goto error;
    }

  /*** Set the MHD Pts array as the position component of the MHD Field. ***/

  if (!DXSetComponentValue (fMHD, "positions", (Object) aGrid))
    {
    DXSetError (ERROR_INTERNAL, "Couldn't set Position Array of MHD Field");
    goto error;
    }
  
  /*** This code makes regular connections. ***/

/*
 *  aMHDConnect = DXMakeGridConnectionsV (3, iaDimSizes);
 *  aMHDConnect = DXMakeGridConnections  (3, iNz, iNy, iNx);
 *
 *  if (!aMHDConnect)
 *    {
 *    DXSetError (ERROR_INTERNAL, "Could Make MHD Grid Connections");
 *    goto error;
 *    }
 *
 *  if (!DXSetComponentValue (fMHD, "connections", (Object) aMHDConnect))
 *    {
 *    DXSetError (ERROR_INTERNAL, "Could set MHD Grid Connections");
 *    goto error;
 *    }
 */
  
  /* This code makes phi (i) direction connections periodic.
   * Regular connections yields (iNx-1) * (iNy-1) * (iNz-1) cells.
   * Periodic in Phi     yields (iNx-1) * (iNy-1) * (iNz-1) cells.
   */

  iNumVars  = 8;
  iNumCells = (iNx) * (iNy - 1) * (iNz - 1);
  iNumPts   = (iNx) * (iNy) * iNz;

#ifdef DEBUG
  DXMessage ("NumVars %d, NumPts %d", iNumVars, iNumCells);
#endif

  aMHDConnect = DXNewArray (TYPE_INT, CATEGORY_REAL, 1, iNumVars);

  if (!aMHDConnect)
    {
    DXSetError (ERROR_INTERNAL, "Could Make MHD Grid Connections");
    goto error;
    }

  DXAddArrayData (aMHDConnect, 0, iNumCells, NULL);
  ipMHDConnect = (int*) DXGetArrayData (aMHDConnect);

  for (k = 0; k < iNz-1; k++)
    {
    iKoff  = iNx   * iNy * k;	/* Offset for back lower left  corner */
    iKoff2 = iKoff + iNx * iNy;	/* Offset for back lower right corner */

    for (j = 0; j < iNy-1; j++)
      {
      iJoff  = iNx * j;		/* Offset for front lower left  corner */
      iJoff2 = iJoff + iNx;	/* Offset for front lower right corner */

      for (i = 0; i < iNx; i++)
	{
        ip1 = i + 1;
        if (iNx-1 ==  i) ip1 = 0;  /* Use start as right for last set of cubes */
        *ipMHDConnect       =  i   + iJoff  + iKoff;
        *(ipMHDConnect + 1) =  ip1 + iJoff  + iKoff;
        *(ipMHDConnect + 2) =  i   + iJoff2 + iKoff;
        *(ipMHDConnect + 3) =  ip1 + iJoff2 + iKoff;
        *(ipMHDConnect + 4) = (i   + iJoff  + iKoff2) % iNumPts;
        *(ipMHDConnect + 5) = (ip1 + iJoff  + iKoff2) % iNumPts;
        *(ipMHDConnect + 6) = (i   + iJoff2 + iKoff2) % iNumPts;
        *(ipMHDConnect + 7) = (ip1 + iJoff2 + iKoff2) % iNumPts;

#ifdef DEBUG
       if(k < 1)
         DXMessage ("%d %d %d %d %d %d %d %d", 
         *(ipMHDConnect + 0), *(ipMHDConnect + 1), *(ipMHDConnect + 2), 
         *(ipMHDConnect + 3), *(ipMHDConnect + 4), *(ipMHDConnect + 5), 
         *(ipMHDConnect + 6), *(ipMHDConnect + 7));
#endif

        ipMHDConnect += 8;
        }
      }
    }		/* Close loop over cell corners. */

  if (!DXSetConnections (fMHD, "cubes", (Object) aMHDConnect))
    {
    DXSetError (ERROR_INTERNAL, "Couldn't set connections");
    goto error;
    }

  if (!DXEndField (fMHD))
    {
    DXSetError (ERROR_INTERNAL, "Couldn't complete field");
    goto error;
    }

  out [0] = (Object) fMHD;

  /*** Cleanup temporary space. ***/

  free (fpR);
  free (fpTh);
  free (fpPhi);
  free (fpIData);

  return OK;

  error:
    DXDelete ((Object) fMHD);
    DXDelete ((Object) aMHDData);
    DXDelete ((Object) aGrid);
    DXDelete ((Object) aMHDConnect);

    /*** Cleanup temporary space. ***/

    free (fpR);
    free (fpTh);
    free (fpPhi);
    free (fpIData);

  return ERROR;
}

/* -------------------------------------------------------------------------- */

void Copy2DX (int iLoc, float *fpMHDData, float *fpData, 
              int iNumVars, int iNx, int iNy, int iNz)
{
  int i, j, k;
  for (k = 0; k < iNz; k++)
    {
    for (j = 0; j < iNy; j++)
      {
      for (i = 0; i < iNx; i++)
	{
        fpMHDData [cijk (iLoc, i, j, k)] = fpData [fijk (i, j, k)];
        }
      }
    }

  return;
}

/* -------------------------------------------------------------------------- */

int average (float *fpData,  int iNxS, int iNyS, int iNzS, 
             float *fpIData, int iNx,  int iNy,  int iNz) 
{

   /* There are three possiblities :
    *
    *  vars like p   which have NzS = Nz + 1 and NyS = Ny + 1
    *  vars like Vth which have NzS = Nz + 1
    *  vars like Vr  which have NyS = Ny + 1
    */

   int i, j, k, iErr;

#ifdef DEBUG
   DXMessage ("Inside average ");
#endif

   iErr = 1;	/* Set Default to be an error unless we do an interpolation. */

   if ((iNzS ==  iNz+1) && (iNyS ==  iNy+1))
     {
     for (k = 0; k < iNz; k++)
       {
       for (j = 0; j < iNy; j++)
	 {
         for (i = 0; i < iNx; i++)
	   {
           fpIData [fijk (i, j, k)] = 0.25 * (fpData [fsijk (i, j,   k  )]   +
                                              fpData [fsijk (i, j+1, k  )]   +
                                              fpData [fsijk (i, j,   k  )]   +
                                              fpData [fsijk (i, j,   k+1)]);
           }
         }
       }
     iErr = 0;
     }

   if ((iNzS ==  iNz + 1) )
     {
     for (k = 0; k < iNz; k++)
       {
       for (j = 0; j < iNy; j++)
	 {
         for (i = 0; i < iNx; i++)
	   {
           fpIData [fijk (i, j, k)] = 0.5 * (fpData [fsijk (i, j, k  )]   +
                                             fpData [fsijk (i, j, k+1)]);
           }
         }
       }
     iErr = 0;
     }

   if ((iNyS ==  iNy+1) )
     {
     for (k = 0; k < iNz; k++)
       {
       for (j = 0; j < iNy; j++)
	 {
         for (i = 0; i < iNx; i++)
	   {
           fpIData [fijk (i, j, k)] = 0.5 * (fpData [fsijk (i, j,   k)]     +
                                             fpData [fsijk (i, j+1, k)]);
           }
         }
       }
     iErr = 0;
     }

   if (1 ==  iErr)
     {
     DXMessage ("Nx %d Ny %d Nz %d", iNx, iNy, iNz);
     DXMessage ("NxS %d NyS %d NzS %d", iNxS, iNyS, iNzS);
     DXSetError (ERROR_INTERNAL, "Parameter miss match");
     }

   return iErr;
}

/* -------------------------------------------------------------------------- */

void CreateXYZ (float *fpVar, float *fpR, float *fpTh, float *fpPhi, 
               int iFrom, int iTo, 
               int iNumVars, int iNx, int iNy, int iNz)
{
   int i, j, k;

   for (k = 0; k < iNz;k++)
     {
     for (j = 0; j < iNy;j++)
       {
       for (i = 0; i < iNx;i++)
	 {
	 
	 /* For Br, use r = i sin(Th)cos(Phi)+j sin(Th)sin(Phi)+k cos(Th). */

         fpVar [cijk (iTo+0, i, j, k)] = 
            fpVar [cijk (iFrom+0, i, j, k)] * sin (fpTh [j]) * cos (fpPhi [i]);
         fpVar [cijk (iTo+1, i, j, k)]  = 
            fpVar [cijk (iFrom+0, i, j, k)] * sin (fpTh [j]) * sin (fpPhi [i]);
         fpVar [cijk (iTo+2, i, j, k)] = 
            fpVar [cijk (iFrom+0, i, j, k)] * cos (fpTh [j]);

         /* For Bth, use r = i cos(Th)cos(Phi)+j cos(Th)sin(Phi)-k sin(Th). */

         fpVar [cijk (iTo+0, i, j, k)] + =  
            fpVar [cijk (iFrom+1, i, j, k)] * cos (fpTh [j]) * cos (fpPhi [i]);
         fpVar [cijk (iTo+1, i, j, k)] + =  
            fpVar [cijk (iFrom+1, i, j, k)] * cos (fpTh [j]) * sin (fpPhi [i]);
         fpVar [cijk (iTo+2, i, j, k)] + = 
            -1.0 * fpVar [cijk (iFrom+1, i, j, k)] * sin (fpTh [j]);

         /* For Bphi, use r = -i sin(Phi)+j cos(Phi). */

         fpVar [cijk (iTo+0, i, j, k)] + = 
            -1.0 * fpVar [cijk (iFrom+2, i, j, k)] * sin (fpPhi [i]);
         fpVar [cijk (iTo+1, i, j, k)] + =  
            fpVar [cijk (iFrom+2, i, j, k)] * cos (fpPhi [i]);
         }
       }
     }

   return;
}
 
/* -------------------------------------------------------------------------- */
