;+ --------------------------------------------------------------------------- +
; NAME:  
;	CPCOLOR
;
; PURPOSE:   
;	Creates arrays for CP black and white and CP differcence color tables
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
;	CPCOLOR,  DIFF_R=DIFF_R, DIFF_G=DIFF_G, DIFF_B=DIFF_B, $
;		  NORM_R=NORM_R, NORM_G=NORM_G, NORM_B=NORM_B, MAXCOLOR=MAXCOLOR
;
; CALLED BY:  
;	CP.PRO
;
; CALLS TO:  
;	none
;
; INPUTS: 
;	none
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS: 
;	DIFF_R = red color array for difference images
;	DIFF_G = green array for difference images
;	DIFF_B = blue  array for difference images
;
;	NORM_R = red color array for normal (not difference) images
;	NORM_G = green color array for normal (not difference) images
;	NORM_B = blue color array for normal (not difference) images
;
;	MAXCOLOR = maximum color used for image data.
;
; OPTIONAL OUTPUTS: none
;
; COMMON BLOCKS: 
;	none, but once the outputs are returned, they are in common CPCOLORS.
;
; SIDE EFFECTS: 
;	Temporarily creates a small IDL window. Users will see it "blip"
;	on the screen.  Necessary to obtain a corrrect value in !D.N_COLORS.
;
; RESTRICTIONS:
;	Must have X window system graphic device.
;
; PROCEDURE:
;	Create color array of size !D.N_COLORS. For normal (not difference)
;	images, the color table is linear black to white, with white at the
;	top index and red one index below that. For difference images,
;       the color table goes from blue to black (mid point index) to red
;       to yellow, with black at lowest index and white at maximun index.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	26 March 1993 - Andrew L. Stanger (HAO/NCAR)
;	Use top 5 levels of colormap for text annotation & graphics "overlays".
;	!p.color = grey, !p.color-1 = blue, !pcolor-2 = green,
;	!p.color-3 = yellow, !p.color-4 = red, !p.color-5 = white.
;	Also modified difference color table (cyan->blue->black->red->yellow).
;- --------------------------------------------------------------------------- -

pro cpcolor, diff_r=diff_r, diff_g=diff_g, diff_b=diff_b, $
	     norm_r=norm_r, norm_g=norm_g, norm_b=norm_b, $
	     maxcolor=maxcolor, colorbar=colorbar

   ; Get # colors not used by the system

   window, /free, xsize=10, ysize=10 & wdelete, !d.window

   maxcolor = !p.color - 5		; Reserve 5 colors for text & graphics.
   print, 'd.n_colors: ', !d.n_colors, '   p.color: ', !p.color, $
	'   maxcolor:', maxcolor

   ; Create colorbar.

   colorbar = bytarr (256, 10)
   for iy = 0, 9 do $
      for ix = 0, 255 do  colorbar (ix, iy) = ix

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

 ; Normal color table: black and white, with white line at max data color index.

   norm_r = indgen (!d.n_colors)		; colors to max available
   norm_g = norm_r
   norm_b = norm_r

   if (!version.os eq "IRIX") then $	; SGI has H/W gamma correction.

   ; SGI/IRIS colormap:
   ; 0-10% = black.
   ; 10-85% = black -> white (linear progression).
   ; 85-100% = red -> yellow.

   begin
      norm_r (0:23) = 0						; Black.

      minlev = fix (0.10 * maxcolor)
      maxlev = fix (0.85 * maxcolor)
      nlev = maxlev - minlev + 1
      norm_r (minlev:maxlev) = indgen (nlev) * (255.0 / (nlev-1)); Black->White.

      norm_g (0:maxlev) = norm_r (0:maxlev)
      norm_b (0:maxlev) = norm_r (0:maxlev)

      ;norm_r (maxlev:maxcolor) = 255				; Red->Yellow.
      ;nlev = maxcolor - maxlev + 1
      ;norm_g (maxlev:maxcolor) = indgen (nlev) * (255.0 / (nlev-1))
      ;norm_b (maxlev:maxcolor) = 0

      norm_r (maxlev:maxcolor) = 255
      midlev = (maxcolor - maxlev) / 2 + maxlev
      cpinterp, norm_g, maxlev,  midlev,  255,   0
      cpinterp, norm_b, maxlev,  midlev,  255,   0

      cpinterp, norm_g, midlev, maxcolor,   0, 255 
      norm_b (midlev:maxcolor) = 0
   end	$

   else $				; workstations w/o gamma control

   begin
      ; ------- colormap for text & graphics "overlays" -------

      ; Create B/W colormap which has gamma > 1 for lower pixel values,
      ; and a gamma < 1 for the upper intensity values.

      ; Interpolate between values at 0, 25, 50, 75 & 100% of colormap range.

      n0   = 0
      n25  = fix (maxcolor * 0.25)
      n50  = fix (maxcolor * 0.50)
      n75  = fix (maxcolor * 0.75)
      n100 = maxcolor

      v0   = 0
      v25  = fix (0.25 * 255.0 * 1.6)
      v50  = fix (0.50 * 255.0 * 1.4)
      v75  = fix (0.75 * 255.0 * 1.25)
      v100 = 255

      cpinterp, norm_r, n0,  n25,  v0,  v25
      cpinterp, norm_r, n25, n50,  v25, v50
      cpinterp, norm_r, n50, n75,  v50, v75
      cpinterp, norm_r, n75, n100, v75, v100

      ; For B/W colormap, green & blue tables are the same as red.

      norm_g = norm_r
      norm_b = norm_r
   endelse


   ; ------- colormap for text & graphics "overlays" -------

   ; Grey at max value.  Used for plotting labels.

   norm_r (!p.color) = 128
   norm_g (!p.color) = 128
   norm_b (!p.color) = 128

   ; Blue :

   norm_r (!p.color-1) =   0
   norm_g (!p.color-1) =   0
   norm_b (!p.color-1) = 255

   ; Green :

   norm_r (!p.color-2) =   0
   norm_g (!p.color-2) = 255
   norm_b (!p.color-2) =   0

   ; Yellow :

   norm_r (!p.color-3) = 255
   norm_g (!p.color-3) = 255
   norm_b (!p.color-3) =   0

   ; Red :

   norm_r (!p.color-4) = 255
   norm_g (!p.color-4) =   0
   norm_b (!p.color-4) =   0

   ; white at max image data value

   norm_r (!p.color-5) = 255
   norm_g (!p.color-5) = 255
   norm_b (!p.color-5) = 255

; savbwr=bwr  &  savbwg=bwg  &  savbwb=bwb           ; save b/w colors

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   ; Difference color table: cyan -> blue -> black -> red -> yellow.
   ; Black at min data index,  white at max data index.

   diff_r = indgen (!d.n_colors)		; colors to max available
   diff_g = diff_r
   diff_b = diff_r
   
   ; ------- colormap for image data -------


   ; If not integer, make two black levels at middle to center arrays.

   mid = (maxcolor) / 2.0			; middle value of blue/red scale
   if ((mid mod 1) ne 0) then $
     begin lomid = fix (mid)  &  himid = lomid+1  & endif $
     else $
     begin lomid = mid        &  himid = mid	  & endelse

   ; cyan -> blue.

   mid    = lomid / 2
   numlev = mid
   for i = 1, mid do $
      begin
      diff_r (i) = 0
      diff_g (i) = (mid - i) * (255.0 / numlev)
      diff_b (i) = 255
      end

   ; blue -> black.

   mid = lomid / 2 + 1
   numlev = lomid - mid + 1
   for i = mid, lomid do $
      begin
      diff_r (i) = 0
      diff_g (i) = 0
      diff_b (i) = (lomid - i) * (255.0 / numlev)
      end

   ; black -> red.

   numlev = (maxcolor - himid) / 2
   mid = himid + numlev - 1
   for i = himid, mid do $
      begin
      diff_r (i) = (i - himid) * (255.0 / numlev)
      diff_g (i) = 0
      diff_b (i) = 0
      end

   ; red -> yellow.

   numlev = maxcolor - mid + 1
   for i = mid, maxcolor do $
      begin
      diff_r (i) = 255
      diff_g (i) = (i - mid) * (255.0 / numlev)
      diff_b  (i) = 0
      end

   ; white at max data value

   diff_r (maxcolor) = 255   
   diff_g (maxcolor) = 255  
   diff_b (maxcolor) = 255 
 
   ; black line at min data value

   diff_r (0) = 0
   diff_g (0) = 0
   diff_b (0) = 0 
 
   ; ------- colormap for text & graphics "overlays" -------

   ; Grey at max value.  Used for plotting text labels.

   diff_r (!p.color) = 128
   diff_g (!p.color) = 128
   diff_b (!p.color) = 128

   ; Blue :

   diff_r (!p.color-1) =   0
   diff_g (!p.color-1) =   0
   diff_b (!p.color-1) = 255

   ; Green :

   diff_r (!p.color-2) =   0
   diff_g (!p.color-2) = 255
   diff_b (!p.color-2) =   0

   ; Yellow :

   diff_r (!p.color-3) = 255
   diff_g (!p.color-3) = 255
   diff_b (!p.color-3) =   0

   ; Red :

   diff_r (!p.color-4) = 255
   diff_g (!p.color-4) =   0
   diff_b (!p.color-4) =   0

; savbrr = brr  &  savbrg = brg  &  savbrb = brb	; save difference colors
end

;+ --------------------------------------------------------------------------- +
