;+ =========================================================================== +
;
; file CPSUNCEN.PRO - includes CPTVCOPY, CPCURSOR, CPGET_R_T,
;			       CPSUNCEN_EV, CPSUNCEN
;+ =========================================================================== +
;
; NAME: 
;	CPTVCOPY
;
; PURPOSE:
;	Output current window to selected printer
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;	CPTVCOPY, COLO=COLO, NORM_DIFF=NORM_DIFF
;
; CALLED BY:
;	CPONE
;
; CALLS TO:
;	PSPLOT
;
; INPUTS:
;	COLO : -1 means no hard copy allowed
;		1 means hard copy device is color
;		0 means hard copy device is color
;	NORM_DIFF :  0 means do a normal image
;                    1 means do a difference image
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	CPWIDGBASES for the widget IDs of the informational windows.
;
; SIDE EFFECTS: 
;	Prints a file.
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Read (TVRD) a window into an array and print it to the selected device.
;
; MODIFICATION HISTORY:
; 	1991 - Elaine Einfalt (HSTX)
;
;- =========================================================================== -


pro cptvcopy, colo=colo, norm_diff=norm_diff, filename=filename

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cpdirnames.common
; common cpdirnames, dirname, dirname1, dirname2

@cp_date.common
;COMMON cp_date, iyear, idoy, imonth, iday, ihour, iminute, isecond

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;               xcen, ycen, pixrs

  ; Copy the current window, then output it to specified device.
  ; colo=0 is black/white, colo=1 is color.

  ; Read IDL image window.

  widget_control, comment, set_value='Copy IDL window', /append
  snap = byte (tvrd (0, 0, !d.x_size, !d.y_size)) 

  ; 8.5x11" paper is 21.6x28 cm.  A 1" border leaves an area ~16.5x23 cm.
  ; Postscript units below are (by default) in cm units.  2.54 cm = 1 inch.
  ; xoffset & yoffset are always relative to lower left corner of page
  ; when viewed in portrait mode, which is the default orientation.

  if (norm_diff eq 0) then oxsiz = 18.7 $
		      else oxsiz = 22.0			; x-dim output size.
  oysiz = oxsiz * (!d.y_size / float (!d.x_size))	; compute y-dim size.
  xborder = (28.0 - oxsiz) / 2.0			; center image on paper.
  yoff = oxsiz + xborder		; yoff => x axis in landscape mode.
  xoff = 2.54				; xoff => y axis in landscape mode.

  ; Create postscript file name.

  case !version.os of
    'vms' : begin
	      if (colo eq 0) then  pfile = 'cpsbw.ps'	$
			     else  pfile = 'cpscolor.ps'
	    end

    else  : begin	; UNIX filesystems: ultrix, sunos, IRIX.
			; Create postscript filename, based upon image filename.
			; First, eliminate directory pathname.

	      if (norm_diff eq 0)  then $
		begin
		  imgindex = strlen (dirname)
		  imglength = strlen (filename) - imgindex
		  namimg = strmid (filename, imgindex, imglength)
		end	$
		else	$
		  begin
		    csm = ['sw', 'ss', 'se', 'ee', 'ne', 'nn', 'nw', 'ww', '4q']

		    cyear   = string (iyear (1),   format='(i2.2)')
		    cdoy    = string (idoy  (1),   format='(i3.3)')
		    chour1  = string (ihour (1),   format='(i2.2)')
		    cmin1   = string (iminute (1), format='(i2.2)')
		    chour0  = string (ihour (0),   format='(i2.2)')
		    cmin0   = string (iminute (0), format='(i2.2)')

		    cyear  = strtrim (cyear,  1)
		    cdoy   = strtrim (cdoy,   1)
		    chour1 = strtrim (chour1, 1)
		    cmin1  = strtrim (cmin1,  1)
		    chour0 = strtrim (chour0, 1)
		    cmin0  = strtrim (cmin0,  1)

		    namimg = cyear + cdoy + '.' + $
			     chour1 + cmin1 + '-' + chour0 + cmin0 + '.'

		    if (nbypx (1) eq 1)  then namimg = namimg + 'v'	$
		    else	$
		    if (nbypx (1) eq 2)  then namimg = namimg + 'c'	$
		    else	$
					      namimg = namimg + 'x'
		    namimg = namimg + csm (ism (1))
		  end

	      if (colo eq 0) then pfile = namimg + '.sbw.ps'	$
			     else pfile = namimg + '.scolor.ps'
	    end
  endcase

  ; Set postscript device & parameters.

  set_plot, 'ps', /copy   
  device, /landscape, bits_per_pixel=8, xsize=oxsiz, ysize=oysiz, $
          xoffset=xoff, yoffset=yoff, color=colo, filename=pfile

  ; Write image copied from IDL window to postscript file.

  tv, snap

  ; Close PS file & reset device to X-window server.
  ; Print message in text widget.

  device, /close  &  set_plot, 'x' 
  widget_control, comment, set_value = 'Wrote window to PS file: ' + pfile, $
		  /append
 
   if (colo eq 0) then $
      print, 'Wrote window to B/W postscript file: ' + pfile $
   else if (colo eq 1) then $
      print, 'Wrote window to color postscript file: ' + pfile

   ; Print PS image file.

;  if (colo eq 0) then spawn, 'print/setup=psi/delete/notify idl.ps' $
;	          else spawn, 'print/que=color$script/notify/delete idl.ps'

;25 Feb 93  psplot, filename = 'cp.ps', color = colo, /delete

  widget_control, onebase, sensitive=1

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	CPCURSOR
;
; PURPOSE:
; 	Draw user's requested R and theta.
;	
; CATEGORY:
;	CPDISPLAY
;
; CALLING SEQUENCE:
;	CPCURSOR, XCUR=XCUR, YCUR=YCUR, SC_SIZ=SC_SIZ, SIN_MOV=SIN_MOV, $
;			  RULECONDITION=RULECONDITION, RULETIMES=RULETIMES, $
;			  RADIUS=RADIUS, THETA=THETA, HGHLIT=HGHLIT, RD=R, $
;			  RADIALMAX=RADIALMAX, COLO=COLO, ROLL=ROLL
;
; CALLED BY:
;	CPSUNCEN_EV 
;
; CALLS TO:
;	none
;
; INPUTS:
;	XCUR :  Radius value unit of solar radii
;	YCUR :  Position angle value unit of degrees
;	SC_SIZ : radius of area to be marked as suncenter
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;			  
;	RADIUS :  radius array for solar disk
;	THETA  :  array of all degree in a circle (360) convert to radians
;	HGHLIT :  the color with which the lines are drawn
;		  white for (top index) for difference images and B/W printer
;		  red (top index-1) for normal images, except to B/W printer
;	R : the sun's radius in pixel, (and a terrible variable name)
;	RADIALMAX : the largest radial extent for this image
;	COLO  : -1 means no hard copy allowed
;		 1 means hard copy device is color
;		 0 means hard copy device is color
;	ROLL : the angle of solar north in radians, from up
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	CPNEW_WINDOW  for value of CURRENT_WINDOW
;	CPWIDGBASES   for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Gets user input (cursor click) and turns it into R and theta.
;	Draw over top of the already displayed image R/Theta using option
;	requested by the user.  
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;- =========================================================================== -

pro cpcursor, xcur=xcur, ycur=ycur, sc_siz=sc_siz, sin_mov=sin_mov, $
			  rulecondition=rulecondition, ruletimes=ruletimes, $
			  radius=radius, theta=theta, hghlit=hghlit, rd=r, $
			  radialmax=radialmax, colo=colo, roll=roll

@cpnew_window.common

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

   repeat begin	; keep doing R and theta until user clicks near sun center,
		; for multi-ruler case.

   if (ruletimes ne 3 and colo eq -1) then begin	; user clicked on screen
      rcur = xcur^2 + ycur^2  &  rcur = sqrt (rcur)	; radius of cursor click
      explain = 'Radius (solar radii)          : ' + strtrim(rcur/r)
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain,/append $
      else widget_control, news, set_value=explain, /append 
   
      Tclick = atan (ycur, xcur)		; theta (radians) of click
      pos = Tclick / !dtor			; theta in degrees
      if (pos lt 0) then pos = 360 + pos	; adjust to subscript values

;2/2/93      realpos = pos - 90 - 43		; correct for polar and CP

      realpos = pos - roll			; roll corrects for polar 
						; and  CP

      if (realpos lt 0) then realpos = realpos + 360	; if neg. position angle
							; c.c.w. from "+X axis"

      explain = 'Position angle (degrees) : ' + strtrim (realpos)
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain,/append $ 
      else widget_control, news, set_value=explain, /append
 
   endif else begin			; user input R and theta
      rcur = xcur * r			; radii times pixels per radii

      pos = (roll + ycur) mod 360	; add to polar adjusted roll
      tclick = pos * !dtor

   endelse

   if (rulecondition eq 1) then $			; mark cursor click
      oplot, [rcur], [Tclick], psym=4, color=hghlit, /polar	

   if (rulecondition eq 0) then begin			; make a ruler

      ; Line from sun center outward thru click.

      oplot, [0, radialmax], [Tclick, Tclick], /polar, color=hghlit     

      ; Each of the ruler tick marks.

      pos = fix (pos + 0.5)		; round pos to nearest integer

      thru_R  = 5.0			; ticks out to thru_R + 1 radii
      tick = 5				; degree length of long tick
      by = 1.0				; ticks every "by" of a solar radius

      for long_short=0,1 do begin	; long ticks then short ticks

        if ((pos ge tick) and (pos le 360-tick)) then $   	

         ; plot long ruler ticks at 1 thru 5 solar radii

         for i = 0.0, thru_R, by  do $
	    oplot, radius (pos - tick:pos+tick) + (r*i),  $
			   theta (pos-tick:pos+tick), /polar, color=hghlit $

         else begin		; just in case the ticks cross the "+X axis"

         ; How much above "+X" axis ?

         if (pos ge 0 and pos lt tick*2) then above = tick + pos $
					 else above = tick - (360-pos)
         below = 360 - ((tick*2)-above)		; how much below "+X" axis ?

         for i = 0.0, thru_R, by do begin	

	   ; From +X axis upward.

	   oplot, radius (0:above) + (r*i), theta (0:above), /polar, $
		  color=hghlit

	   ; From +X axis down.

	   oplot, radius (below:360) + (r*i), theta (below:360), /polar, $
		  color=hghlit
         endfor
      endelse 

        tick = 2			; degree length of short tick
        by = 0.1			; ticks every "by" of a solar radius
     endfor

   endif

   if (ruletimes eq 2 and colo eq -1) then begin  ; multiple R & theta to screen
      explain = 'Click cursor at desired position | click sun center to stop'
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain,/append $
      else widget_control, news, set_value=explain, /append

      wshow, current_window
      cursor, xcur,  ycur, /down
   endif
   
   endrep until  $
      ((abs (xcur) le sc_siz) and (abs (ycur) le sc_siz)) or     $
       (ruletimes ne 2)  or                                    $
       (colo ne -1) 
                                                             
   explain = 'R and theta done, continue processing.'
   if (sin_mov eq 0) then $
	 widget_control, comment, set_value=explain, /append $
    else widget_control, news, set_value=explain, /append 

end
;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	CPGET_R_T
;
; PURPOSE:
; 	Make widget that will allow use input of R and Theta.
;
; CATEGORY:
;	CP display	
;
; CALLING SEQUENCE:
;	CPGET_R_T
;
; CALLED BY:
;	CPSUNCEN
;
; CALLS TO:
;	CPSUNCEN_EV (via XMANAGER)
;
; INPUTS:
;	none
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	CPGET_R_T   widget IDs for R and Theta manual input and input status
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Create the widget which will prompt user to input R and Theta values.
;
; MODIFICATION HISTORY :
;	1991 - Elaine Einfalt (HSTX)
;
;- =========================================================================== -

pro cpget_r_t

@cpget_r_t.common
;common get_r_t, doingit, r_ok, t_ok

  r_ok=0  &  t_ok=0		; initialize

  inpget = widget_base (title='Enter Radius and Theta', /column, $
			space=20, xpad=20, ypad=20)
   doingit = widget_button (inpget, value='Ready', uvalue='READY')

   get_r_t = widget_base (inpget, /column, space=20)
   get_r   = widget_base (get_r_t, /column, space=10)
   junkr = widget_label (get_r, $
			 value='Enter Radius (solar radii) and hit RETURN')
   r_it = widget_text  (get_r, /editable, uvalue='RADII')
   get_t = widget_base (get_r_t, /column, space=10)   
   junkt= widget_label (get_t, $
			value='Enter position angle (degrees) and hit RETURN')
   t_it = widget_text (get_t, /editable, uvalue='THETA')

   canit = widget_button (inpget, value='Cancel', uvalue='CAN_R_T')

   widget_control, inpget, /realize

   widget_control, doingit, sensitive=0 

   xmanager, 'cpsuncen', inpget, event_handler='cpsuncen_ev', /modal

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	CPSUNCEN_EV 
;
; PURPOSE :
;	The event handler for manual R and Theta input widget
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;	CPSUNCEN_EV, EV
;
; CALLED BY:
;	CPGET_R_T (via XMANAGER)
;
; CALLS TO:
;	CPCURSOR
;
; INPUTS:
;	EV: the event structure, variable depending on type of item selected
;	also, values in commons
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	CPUSERINP	for passing to CPCURSOR
;	CPDISPFILES 	for passing COLO to CPCURSOR
;	CPGET_R_T   	widget IDs for R/Theta manual input and input status
;	CPRULE		for passing to CPCURSOR
;	CPWIDGBASES 	for the widget IDs of informational widgets
;	
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Process events representing user input of R and Theta until the 
;	buttons "Ready" or "Cancel" are clicked. At that point control is
;	passed to CPCURSOR or back to calling routine.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;- =========================================================================== -

pro cpsuncen_ev, ev

; handle the widget that is asking for the user to input R and theta

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cpuserinp.common
;common userinp, xcur, ycur, radialmax, sc_siz, radius, theta, hghlit, r, roll

@cprule.common
;common rule, times, condition, state

@cpdispfiles.common
;common dispfiles, filename, diffname1, diffname2, onewin, colo

@cpget_r_t.common
;common get_r_t, doingit, r_ok, t_ok

  widget_control, ev.id, get_uvalue=input	; which widget was clicked ?
  if (n_elements (input) eq 0) then input = ''
  type = strmid (tag_names (ev, /structure_name), 7, 1000)   ; get widget type

  case type of 

    'BUTTON' : case input of

      'READY': begin
		 widget_control, ev.top, /destroy 	; kill top widgets

		 if (colo ne -1) then set_plot,'ps'	; back to print device

		 cpcursor, xcur=xcur, ycur=ycur, sc_siz=sc_siz, roll=roll, $
			   rulecondition=condition, ruletimes=times, $
			   radius=radius, theta=theta, hghlit=hghlit, rd=r, $
			   radialmax=radialmax, sin_mov=state, colo=colo 
	       end 
                                         
      'CAN_R_T': begin
		   widget_control, ev.top, /destroy   ; kill top widgets
	 	   if (colo ne -1) then set_plot,'ps' ; back to print device
                 end

      else : begin
		explain ='Invalid BUTTON value in CPSUNCEN: ' + input
		if (state eq 0) then $
		     widget_control, comment, set_value=explain, /append $
		else widget_control, news, set_value=explain, /append 
	      end
    endcase			; of input

    'TEXT' : $
      begin
        widget_control, ev.id, get_value=value, set_value=''

        case input of 

          'RADII' : $
	    begin       ; get and test input radius
              on_ioerror, error_jump              
	      explain = 'Entered radius of '+ value(0) +' solar radii'
	      if (state eq 0) then $
                   widget_control, comment, set_value=explain, /append $
              else widget_control, news, set_value=explain, /append 

              xcur = float (value (0))
              on_ioerror, null

              xcur = xcur < 10.0 	; keep it reasonable, but
					; still may not be visible
              r_ok = 1
              if (r_ok and t_ok) then widget_control, doingit, sensitive=1
            end

          'THETA' : $
	    begin       ; get and test input theta
	      on_ioerror, error_jump
	      explain ='Entered postion angle of ' + value(0) + ' degrees'
	      if (state eq 0) then $
		   widget_control,comment,set_value=explain, /append $
              else widget_control, news, set_value=explain, /append 

              ycur = float (value (0)) mod 360
              if (ycur lt 0) then ycur = ycur + 360
              on_ioerror, null

              t_ok=1
	      if (r_ok and t_ok) then widget_control, doingit, sensitive=1
	    end

	  else : begin
		   explain = 'Invalid TEXT value in CPSUNCEN: ' + input
	           if (state eq 0) then $
			widget_control,comment, set_value=explain, /append $
          	   else widget_control, news, set_value=explain, /append 
                 end

        endcase 		; of input
      end			; of text    

    else : begin
	     explain = 'Invalid widget type in CPSUNCEN: '  + type
	     if (state eq 0) then $
		  widget_control,comment, set_value=explain, /append $
             else widget_control, news, set_value=explain,/append 
           end

  endcase			; of type

  error_jump:

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	CPSUNCEN
;
; PURPOSE:
;	Draw solar disk, solar north, and radial grid, as requested by user.
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;	CPSUNCEN, IMSIZ=IMSIZ, HK=HK,  DONORTH=DONORTH, DODISK=DODISK, $
;		DORADII=DORADII, DORADIAL=DORADIAL, COLO=COLO, $
;	  	RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		SIN_MOV=SIN_MOV, NORM_DIFF=NORM_DIFF, SAVPCOLOR=SAVPCOLOR, $
;		XPOS=XPOS, YPOS=YPOS, X_SIZ=X_SIZ, Y_SIZ=Y_SIZ,HOUSEKP=HOUSEKP
;
; CALLED BY:
; 	CPDISPLAY, CPMOVIE 
;
; CALLS TO:
;	CPCURSOR, CPGET_R_T
;
; INPUTS:
;	IMSIZ : the size of the CP image in pixels
;	HK    : Array of size (224 x N) where N is 1 for single images
;               and 2 for difference images. Contains the housekeeping
;               information for the current image(s)
;	DONORTH : 0 means don't draw solar north
;		  1 means do draw solar north on image
;	DODISK  : 0 means don't draw solar disk
;		  1 means do draw solar disk
;	DORADII : 0 means don't draw radius lines
;		  1 means do draw radius lines
;	DORADIAL : 0 means don't draw radial lines
;		   1 means do draw radial lines
;	COLO  : -1 means no hard copy allowed
;		 1 means hard copy device is color
;		 0 means hard copy device is color
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	NORM_DIFF : 0 means do a normal image
;                   1 means do a difference image
;	SAVPCOLOR : the value in !P.COLOR, which is the max color index
;	XPOS  : X direction screen coordinate (device pixels), to place window
;	YPOS  : Y direction screen coordinate (device pixels), to place window
;	X_SIZ : Image X size
;	Y_SIZ : Image Y size
;	HOUSEKP : X direction size reserved for house keeping information
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	CPNEW_WINDOW  for value of CURRENT_WINDOW
;	CPRULE	    for passing through the event handler destined for CPCURSOR
;	CPUSERINP   for passing through the event handler destined for CPCURSOR
;	CPWIDGBASES for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Convert solar and CP parameters for display over an already existing
;	CP image.  Add solar disk, solar north, and radial grid, as 
;	requested by user.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	24 March 1993 - Andrew L. Stanger (HAO/NCAR)
;	Use additonal colors specified in "cpcolor.pro".
;- =========================================================================== -

pro cpsuncen, imsiz=imsiz, hk=hk,  donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, colo=colo, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, savpcolor=savpcolor, $
		housekp=housekp, ylabel=ylabel, $
		xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz

@cpnew_window.common

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cpuserinp.common
;common userinp, xcur, ycur, radialmax, sc_siz, radius, theta, hghlit, r, roll

@cprule.common
;common rule, times, condition, state

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;                xcen, ycen, pixrs

   if (norm_diff eq 0)  then  ii = 0  else  ii = 1

   ; Ignore calibration path (CM = in) image.

   if (icm (ii) eq 2) then return

   r = pixrs (ii)				; sun's radius in pixels
   radius = fltarr (361) + r		; radius array
   theta = findgen (361) * !dtor	; all theta converted to radians
   range = findgen (imsiz)		; the X and Y scale in pixels (square)
   radialmax = sqrt (imsiz^2.0 + imsiz^2.0)	; max possible radial length

   ; Assign color levels.

   if (colo eq 0) then	$			; B/W printer.
      begin
	 hghlit = savpcolor			; white
	 suncol = savpcolor			; white
	 lincol = savpcolor			; white
	 norcol = savpcolor			; white
      end		$
   else			$
   if (norm_diff eq 0) then $			; Normal (direct) images.
      begin
         hghlit = !p.color - 4			; red
	 suncol = !p.color - 3			; yellow 
	 lincol = !p.color - 2			; green
	 norcol = !p.color - 1			; blue
      end		$
   else			$
   if (norm_diff eq 1) then $			; Difference images.
      begin
         hghlit = !p.color - 4			; red
	 suncol = !p.color - 3			; yellow 
	 lincol = savpcolor			; white
	 norcol = !p.color - 1			; blue
      end

   ; Set up an invisible coordinate system with (0,0) at sun center.

   fov_x = range - xcen (ii)
   fov_y = range - ycen (ii)			; coordinate system

   fov_pos_x0 = housekp
   fov_pos_y0 = ylabel				; plotting start pos.

   fov_pos_x1 = housekp + x_siz
   fov_pos_y1 = ylabel  + y_siz			; plotting end pos

   ; print, 'cpsuncen> fov_x:     ', fov_x,      '   fov_y:     ', fov_y
   ; print, 'cpsuncen> fov_pos_x0:', fov_pos_x0, '   fov_pos_y0:', fov_pos_y0
   ; print, 'cpsuncen> fov_pos_x1:', fov_pos_x1, '   fov_pos_y1:', fov_pos_y1

   plot, fov_x, fov_y, /nodata, xstyle=5, ystyle=5, /noerase, $ 	
	 position=[fov_pos_x0, fov_pos_y0, fov_pos_x1, fov_pos_y1] 

;   axis, xax=0, 0, 0, xstyle=1  &  axis,yax=0,0,0 ,ystyle=1    ; for debugging

   if (dodisk) then $				; draw solar disk
      oplot, radius, theta, /polar, linestyle=1, color=suncol

   if (doradii) then begin			; draw solar radius lines
      oplot, radius, theta, /polar, linestyle=2, color=suncol
      for i = 2.0,  6.0 do $
	 oplot, radius*i, theta, /polar, linesty=4, color=lincol   ; 1.0 Rsun
      for i = 1.5,  6.5 do $
         oplot, radius*i, theta, /polar, linesty=1, color=lincol   ; 0.5 Rsun
   endif

   percent = 0.10		; % of solar radius to mark with S/C circle
   sc_siz = r * percent 	; radius of area marked as suncenter (float)

   ; Get value of north for this image.

   roll = abs (iroll (ii) - 360)		; adjust to positive angles CCW
   roll = (roll + 90 + 43) mod 360	; adjust for polar and cp weirdness

   north = roll * !dtor

   if (donorth) then begin		; draw line indicating north

      ; Draw circle about sun center.

      center = radius * percent    ; radius of area marked as suncenter (fltarr)
      oplot, center, theta, /polar, color=lincol 	; small circle about s/c

      ; Draw line pointing north.

      oplot, /polar, [sc_siz, r], [north, north], thick=3,   color=norcol

      if (not (doradial)) then begin
         east  = (roll +  90) * !dtor
         south = (roll + 180) * !dtor
         west  = (roll + 270) * !dtor
         oplot, /polar, [sc_siz, r], [north, north], linesty=4, color=lincol
         oplot, /polar, [sc_siz, r], [east,  east ], linesty=4, color=lincol
         oplot, /polar, [sc_siz, r], [south, south], linesty=4, color=lincol
         oplot, /polar, [sc_siz, r], [west,  west ], linesty=4, color=lincol
      endif
   endif

   ; Cross hairs @ sun center are always drawn (a must if using multiple rulers)

   oplot, [-(sc_siz), sc_siz], [0, 0], color=lincol
   oplot, [0, 0], [-(sc_siz), sc_siz], color=lincol

   if (doradial) then begin

      ; Draw radial grid on image every 30 degrees.

      deg_sep = 30.0			   ; every 30 degrees
      three60 = 360 * !dtor  &  space = deg_sep * !dtor
      degctr = - fix (deg_sep)
      for i = north, north + three60, space do $		; radial line 
	 begin
	 degctr = degctr + fix (deg_sep)
	 if ((degctr mod 90) ne 0) then $
            oplot, /polar, [percent, radialmax], [i, i], linesty=1, color=lincol
	 end

      deg_sep = 90.0			   ; every 90 degrees
      three60 = 360 * !dtor  &  space = deg_sep * !dtor
      for i = north, north + three60, space do $		; radial line 
         oplot, /polar, [percent, radialmax], [i, i], linesty=4, color=lincol
   endif

   if (ruletimes ne 0) then begin	; some sort of R and theta is wanted

      if (ruletimes lt 3) then begin	; user wants to click in a R and theta
                                             
         if (colo eq -1) then begin	; output to screen
            explain = 'Click cursor on desired position'
            if (sin_mov eq 0) then $
	         widget_control, comment, set_value=explain, /append $
	    else widget_control, news, set_value=explain, /append 
            wshow, current_window
            cursor, xcur, ycur, /down
            cpcursor, xcur=xcur, ycur=ycur, sc_siz=sc_siz, sin_mov=sin_mov, $
			  rulecondition=rulecondition, ruletimes=ruletimes, $
			  radius=radius, theta=theta, hghlit=hghlit, rd=r, $
			  radialmax=radialmax, colo=colo, roll=roll

         endif else begin 		; can't cursor to device printer
            explain = 'Cursor invalid with printer, enter R and theta'
            if (sin_mov eq 0) then $
	         widget_control, comment, set_value=explain,/append $
	    else widget_control, news, set_value=explain, /append
            set_plot, 'X'		; so widget can ask for R and theta
            times = ruletimes  &  condition = rulecondition  &  state = sin_mov
            cpget_r_t
         endelse

      endif else begin                       ; user wants to input R and theta
         times = ruletimes  &  condition = rulecondition  &  state = sin_mov
         set_plot, 'X'		; so widget can ask for R and theta
         cpget_r_t 				; make widget to ask for input
      endelse

   endif 

end

;+ =========================================================================== +
