/* exden.c
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Convert image intensity (absolute or excess) to columnar density .
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Solar Maximum Mission   Coronagraph/Polarimeter
* Use ATM/WLC F-corona function.
* Compute electrons/cm^2 (instead of elec/pixel);
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
* Andrew L. Stanger   HAO/NCAR   SMM C/P
* 5 December 1986
* 8 June 1990: convert from F77 to C.
* 19 June 1990: reverse bytes if images were calibrated on HV (Vax-11/750).
* 12 July 1990: re-insert vignetting for output image.
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*/
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
#include <stdio.h>
#include "/u/smm/lib/src/cp/cplib.h"
#define npmax 448
#define cmsun 6.96e+10		/* solar radius in centimeters. */
extern int lelec;

float *
exden (namref, namcmp, nx, ny, xmin, xmax)

char namref [40], namcmp [40];	/* Calibrated SMM C/P image file names. */
int *nx, *ny;			/* Output image size. */
float *xmin;			/* Minimum density. */
float *xmax;			/* Maximum density. */

{
float *fimg;			/* Output density image array. */
short *imgref;			/* Reference image. */
short *imgcmp;			/* Comparison image. */

extern float calcor();		/* Computes Munro calibration correction. */
extern double nedsun ();	/* # electrons/cm2 for 1.0 Bsun brightness. */
extern float stray();		/* Computes C/P stray light. */
extern float vignet();		/* Computes C/P vignetting factor. */
extern float fcorona();		/* Computes F-corona. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
int min, max;
float r = 0.0, th = 0.0;
float x, y;
float cc;
int ix, iy;
int ierr;
int irad;

float ferr;
float str, vig, fc;	/*** Stray light, vignetting & F-corona values. ***/
float cm2pix;		/*** area per pixel. ***/
double eb;		/*** # electrons / cm2 / B/Bsun. ***/
float brite;		/*** coronal brightness. ***/
float emass = 2.0e-24;	/*** Mass per light-scattering electron. ***/

int lunref, luncmp;	/*** Logical unit numbers for images. ***/
int nbyimg;
int nbylin;
int nread, nclose;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/

   /*** Initialize stray light & vignetting variables. ***/

   *xmin =  1.e+20;		/* Minimum density. */
   *xmax = -1.e+20;		/* Maximum density. */
   brite = 0.0;
   vig   = 1.0;
   str   = 0.0;
   fc    = 0.0;
   cc    = pscale;		/* Calibration correction. */
   eb    = 0.0;

   /*** Compute the cross-sectional area (square cm) for one pixel. ***/

   cm2pix = (cmsun*rspix) * (cmsun*rspix);

   /*** Decommutate Housekeeping. ***/

   cphskp (namcmp);
   nbylin =        npix * nbypix;
   nbyimg = npix * npix * nbypix;
   fimg = (float *) malloc (npix * npix * sizeof (float));
   fprintf (stdout, "Housekeeping . . . npix: %d   nbypix: %d   scroll:%d\n",
	    npix, nbypix, scroll);

   /*** Open reference and comparison files. ***/

   if (ldif == 1)
      if (lunref <= 0)
	 {
         fprintf (stderr, "Invalid image file name: %s\n", namref);
         exit (-2);
          }

   luncmp = open (namcmp, 0);
   if (luncmp <= 0)
      {
      fprintf (stderr, "Invalid image file name: %s\n", namcmp);
      exit (-3);
      }

   /*** Skip over housekeeping and bitmap records in input image. ***/

   if (ldif == 1)
      {
      lunref = open (namref, 0);
      if (lunref <= 0)
         {
         fprintf (stderr, "Invalid image file name: %s\n", namref);
         exit (-3);
         }
      imgref = (short *) malloc (nbyimg);
      nread = read (lunref, imgref, nbylin);
      nread = read (lunref, imgref, nbylin);
      nread = read (lunref, imgref, nbyimg);
      if (bytord == 1)  srevbyt (imgref, 2, npix*npix);
      }

   imgcmp = (short *) malloc (nbyimg);
   nread = read (luncmp, imgcmp, nbylin);
   nread = read (luncmp, imgcmp, nbylin);
   nread = read (luncmp, imgcmp, nbyimg);
   if (bytord == 1)  srevbyt (imgcmp, 2, npix*npix);

   /*** ....... Image computation loop ....... ***/

   for (iy=0; iy<npix; iy++)
      {
      for (ix=0; ix<npix; ix++)
         {
 
         /*** Coordinate transformation :  (pixel,line) --> (theta,angle). ***/

         x = (float) ix;
	 y = (float) iy;
         ierr = rcoord (&r, &th, &x, &y, -1, scroll);
/*
 * 	 if (ix == iy)
 * 	    fprintf (stderr, "r: %f   th: %f   x: %f   y: %f\n", r, th, x, y);
 */

         if (ierr != 0)
	    {
            fprintf (stderr, "exden> coord error: %d\n", ierr);
            fprintf (stdout, "exden> coord error: %d\n", ierr);
	    ferr = (float) ierr;
            return (&ferr);
	    }

         /*** Use external stray/vignet/F-corona functions. ***/

         if (H.idtype == 3)
            {
            if (lcc)  cc = calcor (r) * pscale;
            lstr = 1;
            lvig = 1;
	    lfc  = 1; 
            if (!H.vig)  vig = vignet (r);
            if (!H.str)  str = stray (r);
	    if (!H.fc )  fc  = fcorona (r, th);
	    }

         if (lelec == 1)
	    eb = nedsun (r);	/* # electrons/cm2 for 1.0 Bsun. */

         /*** Compute electron density. ***/

         if (ldif == 0)
	    brite =
            (float)(eb * (((*(imgcmp  + iy*npix + ix) * cc - str) * vig) - fc));
         else
	    brite =
            (float) (eb * ( ( ( *(imgcmp + iy*npix + ix)
	                      - *(imgref + iy*npix + ix) ) * cc * vig) - fc));

         if (brite < 0.0)  brite = 0.0;
         else
         if (vig != 1.0)   brite /= vig;
         *(fimg + iy*npix + ix) = brite;

         if (*xmin > *(fimg + iy*npix + ix))  *xmin = *(fimg + iy*npix + ix);
         if (*xmax < *(fimg + iy*npix + ix))  *xmax = *(fimg + iy*npix + ix);
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
         if (ix == npix - iy - 1)
            {
/*          fprintf (stdout, "%3d cc:%9.2e s:%9.2e v:%9.2e fc:%9.2e eb:%9.2e fimg:%9.2e\n", iy, cc, str, vig, fc, eb, *(fimg+iy*npix+ix)); */
            fprintf (stdout, "%3d   r:%4.2f   eb:%9.2e   imgcmp:%4d   brite:%9.2e   fimg:%9.2e\n",
            iy, r, eb, *(imgcmp + iy*npix + ix), brite, *(fimg + iy*npix + ix));
/*
 *            fprintf (stderr, "imgcmp[%d][%d]: %6d   fimg: %g\n",
 *                     iy, ix, *(imgcmp+iy*npix+ix), *(fimg + iy*npix + ix));
 *            fprintf (stderr, "r: %5.2f  str: %g   vig: %g\n", r, str, vig);
 *            fprintf (stderr, "eb: %g   brite: %g   cc: %g\n",
 *                     eb, brite, cc);
 */
            }
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
         }
      }
   *nx = ix;
   *ny = iy;

   if (ldif == 1)  nclose = close (lunref);
   nclose = close (luncmp);
   if (lvig == 1)  lvig = 0;
   return (fimg);
}
