c * xden3.f
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c * Convert image intensity (absolute or excess) to columnar density .
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c * Solar Maximum Mission   Coronagraph/Polarimeter
c * Use ATM/WLC F-corona function.
c * Compute electrons/cm^2 (instead of elec/pixel);
c * 5 December 1986
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      subroutine xden (namref, namcmp, img, nx, ny, dmin, dmax)
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      implicit integer (u)

c--- Solar radius (cm).
      parameter (cmsun = 6.96e+10)
      parameter (npmax = 448)

      common /center/ ixcen, iycen
      common /const/  pi, raddeg, degrad, pixr, rpix, rsun
      common /hskdat/ ires, ip, ir, lp, lr, ifw, ism, ipw, iroll
      common /size/   npix, npix4, nbypix, idtype
      common /sun/    degnor, kcen, lcen
      common /opt/    lsv, lfc, lden, ldif

c--- Image file names.
      character*40 namref, namcmp

c--- Input image arrays.
      integer*2 linref(npmax)
      integer*2 lincmp(npmax)

c--- Output density array.
      real img(npmax,npmax)

c--- Function which applies calibration correction.
      external calcor

c--- Functions which remove stray light, vignetting & F-corona.
      external stray
      external vignet
      external fcorona
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      real dmin, dmax
      real r, th
      real cc
      integer ix, iy
      integer ierr
      integer irad
      integer nx, ny

c--- Stray light & vignetting values.
      real str, vig

c--- Area (cm^2) per pixel.
ccc      real cm2pix

c--- # pixels per cm^2.
ccc      real pixcm2

c--- # electrons / B/Bsun.
      real ebsun
      real elec(28)

c--- Coronal brightness.
      real cbrite

c--- Scaling factor for calibrated intensities.
      data Cexp /1.e-11/

c--- Mass per light-scattering electron.
      data emass /2.0e-24/

c--- #electrons per cm^2 per B/Bsun brightness at 0.25 Rsun intervals in corona.

      data elec
     .         /8.93e24,1.45e25,2.10e25,2.85e25,3.70e25,4.65e25,5.71e25,
     .          6.86e25,8.11e25,9.46e25,1.09e26,1.25e26,1.41e26,1.59e26,
     .          1.77e26,1.97e26,2.17e26,2.39e26,2.61e26,2.85e26,3.10e26,
     .          3.35e26,3.62e26,3.89e26,4.18e26,4.47e26,4.78e26,5.09e26/

c--- Logical unit numbers for images.

      data lunref/11/, luncmp/12/
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

c--- Initialize stray light & vignetting variables.

      rbrite = 0.
      cbrite = 0.
      cc = 1.
      str = 0.
      vig = 1.
      ebsun = 1.
      dmin = 1.e+30
      dmax = -1.e+30

c--- Compute the cross-sectional area (square cm) for one pixel.

ccc      cm2pix = (cmsun*rpix) * (cmsun*rpix)

c--- Decommutate Housekeeping.

      call cphskp (luncmp,namcmp)
      if (ires .ne. 2)  then
         write (*,*) 'Only low resolution images may be used.'
         stop
      endif
      nbytes = npix * nbypix
      kcen = ixcen / ires
      lcen = iycen / ires

c--- Open reference and comparison files.

      if (ldif .eq. 1)  then
         lunref = uopen (namref, 0)
         if (lunref .le. 0)  then
            write (*,6) namref
    6       format(1x,'Invalid image file name: ',a40)
            stop
         endif
      endif

      luncmp = uopen (namcmp, 0)
      if (luncmp .le. 0)  then
         write (*,*) 'luncmp: ', luncmp
         write (*,6) namcmp
         stop
      endif

c--- Skip over housekeeping and bitmap records in input image.

      if (ldif .eq. 1)  then
         nread = uread (lunref, linref, nbytes)
         nread = uread (lunref, linref, nbytes)
      endif
      nread = uread (luncmp, lincmp, nbytes)
      nread = uread (luncmp, lincmp, nbytes)

c ....... Image computation loop .......

      do 200 iy = 1, npix
      if (ldif .eq. 1)  nread = uread(lunref, linref, nbytes)
      nread = uread(luncmp, lincmp, nbytes)

      do 100 ix = 1, npix
 
c--- Coordinate transformation :  (pixel,line) --> (theta,angle).

      call rcoord(r,th,ix,iy,-1,iroll,ierr)

      if(ierr .ne. 0)  then
         write (*,*) 'xden3> coord error:', ierr
         return
      endif

c--- Apply Munro calibration adjustment, stray light & vignetting corrections.

      if(lsv .eq. 1)  then
         cc = calcor (r) * Cexp
         vig = vignet(r)
         str = stray(r)
      endif

      if (lden .eq. 1)  then
         irad = ((r - 1.0)/0.25)
         ebsun = ( elec(irad) + ( (r - irad*0.25 - 1.0) / 0.25)
     .          * (elec(irad+1) - elec(irad)) )
         if (r .lt. 1.5)  ebsun = 0.
      endif

c--- Reference image.

      if (ldif .eq. 1)  then
         rbrite = (linref(ix) * cc - str) * vig
         if (lfc .eq. 1)  rbrite = rbrite - fcorona(r,th)
         if (rbrite .lt. 0.)  rbrite = 0.
      endif

c--- Comparison image.

      cbrite = (lincmp(ix) * cc - str) * vig
      if (lfc .eq. 1)  cbrite = cbrite - fcorona(r,th)
      if (cbrite .lt. 0.)  cbrite = 0.
c
      if (ldif .eq. 0)  then
         img(ix,iy) = ebsun * cbrite
      else
         img(ix,iy) = ebsun * (cbrite - rbrite)
      endif
      if (dmin .gt. img(ix,iy))  dmin = img(ix,iy)
      if (dmax .lt. img(ix,iy))  dmax = img(ix,iy)
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c      if (ix .eq. 30)  then
c         write (*,*) 'line:',iy,'  image:',imgcmp(ix,iy),
c     .               '  img(30,iy):',img(30,iy)
c         write (*,*) 'r:',r,' str:',str,' vig:',vig
c         write (*,*) 'ebsun:',ebsun,' cbrite:',cbrite,' cc:',cc
c      endif
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
  100 continue
  200 continue
      nx = ix - 1
      ny = iy - 1

      if (ldif .eq. 1)  nclose = uclose (lunref)
      nclose = uclose (luncmp)
      return
      end
