/* calcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P image calibration (photometric & geometric).	               *
 * Output image (calibrated) is in Sun/SGI byte order.                 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	               *
 * High Altitude Observatory (HAO)			               *
 * National Center for Atmospheric Research (NCAR) 	               *
 * SGI IRIS   UNIX IRIX 3.3.1   C programming language                 *
 * author: Andrew L. Stanger   Associate Scientist                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Revision History:                                                   *
 *  9 May 1984: program created via translation from F77 to C.         *
 *  1 November 1984: accomodate all image sizes.                       *
 *  9 November 1984: close all files when no longer needed.            *
 * 12 November 1984: parameter files moved to ~/cal/tc.                *
 *  3 September 1985: use transfer curve created using                 *
 *                   cleaned-up cm images.                             *
 * 10 March 1988: replace cphk.c with cphskp.c                         *
 *                & hkshow.c with hkline.c                             *
 * 13 March 1988: add computer designation in hskp byte#70.            *
 * 21 March 1988: reverse bytes to conform with Vax order.             *
 *  8 Sept 1988: use 3rd order polynomial fit coefficients.            *
 * 28 February 1989: adapt for low resolution parameter files.         *
 *  1 March 1989: use 1980 transfer curve parameter files.             *
 * 17 March 1989: accomodates all years: 1980, 1984-89.                *
 * 19 October 1989: revised housekeeping format for AVIV/EXB archive.  *
 * 26 March 1991: adapt for use on SGI IRIS workstation.               *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*** Standard C system library and Math functions library. ***/

#include <stdio.h>
#include <math.h>

/*** Constant definitions. ***/

#define MAXREC 896		/* Maximum # records. */
#define MAXPIX 896		/* Maximum # pixels. */
#define LOWPIX 448		/* # pixels/line for low resolution images. */
#define MAXHSK 224		/* Number of bytes in housekeeping. */
#define TEOF   0		/* End-of-file. */
#define ERR    -1		/* Error indication. */
#define XOFF 70			/* Horizontal axis border zone size. */
#define YOFF 70			/* Vertical axis border zone size. */

/*** External variables defined. ***/

#include "/usr/people/smm/lib/src/cp/cplib.h"

/*** Function Declarations. ***/

int  tdvdsk ();			/* Read video data. */
int  eqs ();			/* Equivalent second calibration. */
int  geo ();			/* Geometric distortion correction. */
int  error ();			/* Error exit. */
char *outname ();		/* Convert video image name into cal. name. */

char hk [MAXPIX];		/* Housekeeping storage. */
char bitmap [MAXPIX];		/* Bit-map storage. */
char bytbuf [MAXPIX];		/* Vidicon raw data storage (one line). */

char pardir [] = "/picard/cp/cal/param/";	/* parameter directory. */
char fila0 [80];		/* a0 parameter file name. */
char fila1 [80];		/* a1  parameter file name. */
char fila2 [80];		/* a2 parameter file name. */
char fila3 [80];		/* a3 parameter file name. */
char filgeo [] = "/picard/cp/cal/tc/WW6COR.GEO"; /* Geometric distortion file.*/
char namea0 [80];		/* a0 parameter file name. */
char namea1 [80];		/* a1  parameter file name. */
char namea2 [80];		/* a2 parameter file name. */
char namea3 [80];		/* a3 parameter file name. */
char namgeo [] = "WW6COR";	/* Geometric distortion file name.*/
char namstr [] = "NONSTR";	/* Stray light file name. */
char namvig [] = "NONVIG";	/* Vignetting file name. */
char namlog [] = "CALCP.LOG";	/* Output summary file name. */
char namlst [80];		/* Filename containing list of images. */
char namvid [80];		/* Input image file name. */
char namcal [80];		/* Output image file name. */
char namtpv [7] = "SMMVID";	/* Raw image volume name. */
char namset [7] = "SMMCAL";	/* Calibrated image volume name. */
char namvol [7] = "YODA  ";	/* Storage medium name for calibrated image. */

int lunwr;			/* File descriptor for output summary. */
int luna0, luna1, luna2, luna3;	/* File descriptor for parameter files. */
int lungeo;			/* File descriptor for geometric correction. */
int lunvid, luncal;		/* File descriptor for input/output files. */

int icfpos = 1;			/* File position of image: calibrated volume. */

int lstr;			/* Stray light flag. */
int lvig;			/* Vignetting flag. */
int lreg;			/* Disk registration flag. */

long x;				/* Index for horizontal dimension. */
long y;				/* Index for vertical dimension. */
long dx = MAXPIX + 2*XOFF;	/* Size of horizontal dimension of cp[][]. */
long npix;			/* # pixels per vidicon line. */

int jres;			/* Resolution: 1=> Hi, 2=>Lo. */
int nres;			/* Resolution: 1=> Lo, 2=>Hi. */
int ip, lp;			/* Initial and last pixel indices. */
int ir, lr;			/* Initial and last record indices. */
int np, nr;			/* # pixels/line and # lines/image. */
int minx, maxx;			/* Minimum & maximum indices for X-axis. */
int miny, maxy;			/* Minimum & maximum indices for Y-axis. */

float fwmag;			/* Filter magnification. */
float pwcorx, pwcory;		/* Polaroid shift correction. */
float smcorx, smcory;		/* FW/SM shift correction. */
float cxcen, cycen;		/* Computed disk center (x,y). */
float dxcen, dycen;		/* Standard disk center (x,y). */
float sxcen, sycen;		/* Standard sun center (x,y). */
float escorr;			/* Equivalent Second filter correction. */
float radfac;			/* Radiance conversion factor. */

float a0 [MAXPIX];		/* Storage for a0 parameter array. */
float a1 [MAXPIX];		/* Storage for a1 parameter array. */
float a2 [MAXPIX];		/* Storage for a2 parameter array. */
float a3 [MAXPIX];		/* Storage for a3 parameter array. */
float cp [MAXREC+2*YOFF][MAXPIX+2*XOFF]; /* Storage for one entire image. */

FILE *LIST;		/* File descriptor for filename list. */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

main (argc, argv)

int argc;
char *argv [];
{
   int  i, lenvid, idot;

   /* Request input file name.
    * If it begins with '@', then it contains a list of image file names
    * to be calibrated.
    */

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [file_name | @file_name_list]\n", argv [0]);
      fprintf (stderr, "Enter C/P image file name [or @file_name_list]: ");
      fscanf  (stdin,  "%s", namvid);
      }
   else
   if (argc == 2)
      strcpy (namvid, argv [1]);

   if (namvid [0] == '@')
      {
      strcpy (namlst, &namvid [1]);

      /*** Read list of images.  Process one image at a time. ***/

      if ((LIST = fopen (namlst,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s\n", namlst);
         exit(-1);
         }

      /*** Fetch new image file name from list. ***/

      while (fscanf (LIST, "%s", namvid) != EOF)
         {

         /* Copy input image file name to output file name, then change
	  * character following the period (.) to a 'c'.
	  */

         strcpy (namcal, outname (namvid));
         fprintf (stdout, "Calibrated image file name : %s\n", namcal);

         /*** Read raw C/P image from disk to memory array. ***/

         rdvdsk ();

         /*** Photometric calibration : DN -> Equivalent Seconds.***/

         eqs ();

         /*** Geometric correction.  Write output file. ***/

         geo ();
         }
      fclose (LIST);
      }

   else

      /* Single image case.
       * Copy input file name to output file name, then change character
       * following the period (.) to a 'c'.
       */
      {
      strcpy (namcal, outname (namvid));
      fprintf (stdout, "Calibrated image file name : %s\n", namcal);

      /*** Read raw C/P image from disk to memory array. ***/

      rdvdsk ();

      /*** Photometric calibration : DN -> Equivalent Seconds. ***/

      eqs ();

      /*** Geometric distortion correction.  Write output file. ***/

      geo ();
      }

   exit (0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* rdvdsk: Read C/P raw (DN) video image from disk file.   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P image calibration.				   *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	   *
 * High Altitude Observatory (HAO)			   *
 * National Center for Atmospheric Research (NCAR) 	   *
 * SGI IRIS   UNIX IRIX 3.3.1   C programming language     *
 * author: Andrew L. Stanger   Associate Scientist                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * 21 March 1984				 	   *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

rdvdsk ()

{
   register float *cpp;
   int  nhk, nmap, ndata, ncvid;
   int  i, yy, ih, iv, lsw;

   cpp = &cp [0][0];

   /*** Open output image disk file. ***/

   if ((luncal = creat (namcal, 0644)) == -1)
      error ("Can't create %s", namcal);

   /*** Decommutate Housekeeping. ***/

   cphskp (namvid);
   hkline ();
   ih = H.ih;
   iv = H.iv;
   jres = H.ires + 1;
   nres = 1;
   if (H.ires == 0)  nres = 2;
   np = lp - ip + 1;
   nr = lr - ir + 1;
   npix = (long) np;

   /*** Open input image disk file and read housekeeping record. ***/

   if ((lunvid = open (namvid, 0)) == -1)
      error ("Can't open %s", namvid);
   if ((nhk = read (lunvid, hk, MAXHSK)) <= 0)
      error ("hk: bad read on file %s", namvid);

   /*** Set file pointer at beginning of bit-map data. ***/

   lseek (lunvid, npix, 0);

   /*** Read bit map record. ***/

   if ((nmap = read (lunvid, bitmap, np)) <= 0)
      error ("bitmap: bad read on file %s", namvid);

   /*** Read image data. ***/

   y = ir + YOFF -1;
   while ((ndata = read (lunvid, bytbuf, np)) != TEOF)
      {
      y++;
      yy = y * dx;
      minx = ip + XOFF;
      maxx = lp + XOFF;
      for (x = minx; x <= maxx; x++)  
         {
         i = x - minx;
         *(cpp + yy + x) = (float) ((int) (bytbuf [i]) & 0377);
         }
      }
   ncvid = close (lunvid);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * eqs :   Equivalent Second conversion. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Converts raw vidicon DN values to exposure units (green equivalent seconds)
 *   using a transfer function (tangent) with parameters a0, a1, a2, & a3.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P image calibration.				 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	 *
 * High Altitude Observatory (HAO)			 *
 * National Center for Atmospheric Research (NCAR) 	 *
 * SGI IRIS   UNIX IRIX 3.3.1   C programming language   *
 * author: Andrew L. Stanger   Associate Scientist                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * 21 March 1984					 *
 *  1 March 1989: use low resolution parameter files.	 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

eqs ()	/* Equivalent Second conversion. */
{
   register float *cpp;

   /*** ESCOLR: equivlent second adjustment for non-green filter exposures. ***/

   static float  escolr [8] = {1.266,15.798,0.200,1.025,0.444,1.0,94.16,94.16};

   float expdur;
   float eq;
   float z, z2, z3;
   long iskip;
   int  i, yy;
   int  nrd, nskp;

   cpp = &cp [0][0];

   /*** Fetch equivalent second correction; depends upon color filter used. ***/

   escorr = escolr [H.ifw];
   expdur = H.exp;

   /*** Open Alpha, Beta, Gamma, Geometric correction files. ***/

   sprintf (namea0, "a0.%dlo", H.year);
   sprintf (namea1, "a1.%dlo", H.year);
   sprintf (namea2, "a2.%dlo", H.year);
   sprintf (namea3, "a3.%dlo", H.year);
   sprintf (fila0, "%s%s", pardir, namea0);
   sprintf (fila1, "%s%s", pardir, namea1);
   sprintf (fila2, "%s%s", pardir, namea2);
   sprintf (fila3, "%s%s", pardir, namea3);
   fprintf (stdout, "parameter files: %s %s %s %s\n",
	    namea0, namea1, namea2, namea3);
	 
   if ((luna0 = open (fila0, 0)) == -1)
      error ("Can't open %s", namea0);
   if ((luna1 = open (fila1, 0)) == -1)
      error ("Can't open %s", namea1);
   if ((luna2 = open (fila2, 0)) == -1)
      error ("Can't open %s", namea2);
   if ((luna3 = open (fila3, 0)) == -1)
      error ("Can't open %s", namea3);
   if ((lungeo = open (filgeo, 0)) == -1)
      error ("Can't open %s", namgeo);

   /* Position pointer to start of image in parameter files.
    * (NOTE: a0, a1, a2, and a3 files are Low resolution format.)
    */

   iskip = ir/nres * LOWPIX * 4;
   nskp = lseek (luna0, iskip, 0);
   nskp = lseek (luna1, iskip, 0);
   nskp = lseek (luna2, iskip, 0);
   nskp = lseek (luna3, iskip, 0);

   /* ****** PHOTOMETRIC CALIBRATION ****** */

   /* Read one record each from a0, a1, a2, & a3 polynomial parameter files. */

   miny = ir + YOFF;
   maxy = lr + YOFF;

   for (y = miny; y <= maxy; y++)
      {
      /*
       * Since parameter files are low resolution,
       * use each line twice for Hi res images.
       */
      if ((H.ires == 0 && (y/2)*2 == y) || H.ires == 1)
         {
         nrd = read (luna0, a0, LOWPIX * 4);
         nrd = read (luna1, a1, LOWPIX * 4);
         nrd = read (luna2, a2, LOWPIX * 4);
         nrd = read (luna3, a3, LOWPIX * 4);

         if (H.ires == 0)	/* Expand parameter arrays for Hi resolution. */
            {
	    fexpand (a0, LOWPIX);
	    fexpand (a1, LOWPIX);
	    fexpand (a2, LOWPIX);
	    fexpand (a3, LOWPIX);
	    }
	 }

      /* Compute 'calflux' values from raw DN values.
       * Use 3rd order polynomial fit.
       * (calflux unit = equivalent second units / exposure duration.)
       */

      minx = ip + XOFF;
      maxx = lp + XOFF;
      yy = y * dx;

      for (x = minx; x <= maxx; x++)
         {
         i = (x - XOFF);
         z = *(cpp + yy + x);
         z2 = z * z;
         z3 = z2 * z;
         eq = (a0 [i] + a1 [i] * z + a2 [i] * z2 + a3 [i] * z3);
	 *(cpp + yy + x) = (eq * escorr) / expdur;
         }
      }
   close (luna0);
   close (luna1);
   close (luna2);
   close (luna3);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * geo : geometric distortion correction.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P image calibration.				 *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	 *
 * High Altitude Observatory (HAO)			 *
 * National Center for Atmospheric Research (NCAR) 	 *
 * SGI IRIS   UNIX IRIX 3.3.1   C programming language   *
 * author: Andrew L. Stanger   Associate Scientist                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * 21 March 1984				 	 *
 *  1 November 1984: ip0=XOFF; ir0=YOFF.		 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#define dr 59	/* # pixel coordinate pairs per line of geo distortion file. */

geo ()		/* Geometric Distortion Correction. */

{
   register float *cpp;
   static int ngeo2 = dr * dr * sizeof (float);
   static int ipw, ifw, ism;
   static int iglstw, iglpos;
   static int nrew, nrd, nwr, ncl;
   static int i, j, ii, jj, ip0, ir0;
   static int imin, imax, jmin, jmax;
   static int kr, kp, mp, mr, mrr, mrr1;

   static short outbuf[MAXPIX];	/* Storage for output array. */
   register short *op;		/* Pointer to array outbuf. */

   /* The geometric distortion file consists of
    * 59x59 pixel coordinates (floating point values).
    * Each coordinate pair represents an interpolated position
    * of the line intersections of an x/y grid test pattern
    * which was imaged onto the C/P vidicon during tests at Ball Aerospace.
    * Since the resolution of the geometric distortion grid is effectively
    * 1/16th that of a high resolution image (896/56 = 1/16), 
    */

   static float one16 = 0.0625;	/* 1/16. */
   static float pgeom [dr][dr];	/* Storage for X-axis distortion array. */
   static float rgeom [dr][dr];	/* Storage for Y-axis distortion array. */
   register float *pg;		/* Pointer to array pgeom. */
   register float *rg;		/* Pointer to array rgeom. */

   static float sp, sr, gp, gr, fx0, fx1, fy0, fy1, fp, fr, zp, zr;
   static float sxdif, sydif;
   static float shiftn, shiftw, angle = 5.493;

   float  cval, xmin = 0.0, xmax = 32767.0;

   /* Disk center (vs. sector mirror) in high resolution pixels.
    * (Derived from images on volume V34520, as computed by DC8CP program
    *  on CRAY-1A, 23 April 1982.)
    *
    *  sector     x        y
    *	sw	36.3	499.8
    *	s      213.1    193.6
    *	se     539.9      3.3
    *	e      820.0     87.0
    *	ne     892.4    361.5
    *	n      718.6    679.3
    *	nw     386.7    856.4
    *	w      104.3    792.1
    *
    * NOTE: disk center computation is performed on images which have NOT
    *       been corrected for geometric distortion.
    */

   static float cendsk [8][2] = { 36.3, 499.8,
                                 213.1, 193.6,
                                 539.9,   3.3,
                                 820.0,  87.0,
                                 892.4, 361.5,
                                 718.6, 679.3,
                                 386.7, 856.4,
                                 104.3, 792.1};

   /*** Radfil: coefficients of 10^-11 B/Bsun units.   W.Wagner 20 Sep 1982 ***/

   static float radfil [8] = {376.,463.,393.,334.,324.,324.,522.,522.};

   /* Filter magnification factors. FW order: r,ha,c,b,wb,g,gl,gc.
    * (Inverse of values from Stanger memo 11 Dec 1979.)
    */

   static float filmag [8] = {0.9889, 0.9781, 1.0069, 1.0022, 1.0022, 1.0000,
                              1.0035, 1.0035};

   /* Translation of polaroids (W.Wagner memo 7 December 1981).
    * Array storage: polcor[pixel,line][ipw], ipw=1,2,3,4.
    * NOTE: ipw = 1 for p0 (clear)
    *           = 2     p1 (-60 degrees)
    *           = 3     p3 (+60 degrees)
    *           = 4     p2 (  0 degrees)
    */

   static float polcor [2][4] = {0., -1.1, 3.0, -2.0,
                                 0., -2.9, 0.6,  2.4};

   /* Translation for sector mirror with filter position (W.Wagner 22 Jan 1982).
    * (sector mirror order: sw, s, se, e, ne, n, nw, w.)
    * (filter wheel order: r, ha, cl, b, wb, g.)
    * array storage: seccor [pixel,line][ifw][ism].
    */

   static float seccor [2][6][8] =
          {  2.02, 1.62,-1.09,-3.18,-3.64,-2.80,-1.14, 1.19,
             5.38, 4.29,-1.79,-6.62,-7.73,-5.81,-1.97, 2.83,
            -3.03,-2.35, .556, 3.26, 3.69, 2.48, .455,-2.15,
             0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
            -1.59,-1.59,-1.59,-1.59,-1.59,-1.59,-1.59,-1.59,
             0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
            -1.01, 1.01, 3.01, 2.83, .808,-1.54,-2.70,-2.40,
            -1.39, 3.18, 6.89, 6.01, 1.67,-3.49,-5.91,-5.05,
             .404,-1.64,-3.79,-3.71,-1.36, 1.57, 3.08, 2.40,
             0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00,
            -4.21,-4.21,-4.21,-4.21,-4.21,-4.21,-4.21,-4.21,
             0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00, 0.00 };

   cpp = &cp [0][0];
   op = &outbuf [0];
   pg = &pgeom [0][0];
   rg = &rgeom [0][0];

   /*** Disk center standard. ***/

   dxcen = cendsk [H.ism][0];
   dycen = cendsk [H.ism][1];

   /*** Sun center standard (assumed identical to disk center). ***/

   sxcen = cendsk [H.ism][0];
   sycen = cendsk [H.ism][1];

   cxcen = sxcen;
   cycen = sycen;

   /*** Difference in sun center: current image relative to reference. ***/

   sxdif = cxcen - sxcen;
   sydif = cycen - sycen;

   /* Select appropriate radiance conversion factor
    * based upon spectral filter.
    */

   ifw = H.ifw;
   radfac = radfil [ifw];

   /*** Polaroid, sector mirror, and magnification factors. ***/

   pwcorx = pwcory = 0.0;
   smcorx = smcory = 0.0;

   ipw = H.ipw;
   pwcorx = polcor [0][ipw];
   pwcory = polcor [1][ipw];

   ism = H.ism;
   smcorx = seccor [0][ifw][ism];
   smcory = seccor [1][ifw][ism];

   fwmag = filmag [H.ifw];

   /*** Calculate translation for green line (5303) filter. ***/

   iglstw = H.iglstw;
   iglpos = H.iglpos;
   if (iglstw == 1)
      { shiftn = ( iglpos*0.223 + 52.0) / 25.26;
        shiftw = (-iglpos*6.514 + 435.2) / 25.26;
        smcorx = shiftw * cos(angle) + shiftn * sin(angle);
        smcory = shiftw * sin(angle) + shiftn * cos(angle);
      }
   fprintf (stderr, "magnification: %7.4f  ", fwmag);
   fprintf (stderr, "fw/sm shift: %7.4f %7.4f  polaroid shift: %7.4f %7.4f\n",
           smcorx, smcory, pwcorx, pwcory);

   /*** Rewind calibration disk file. ***/

   nrew = lseek (luncal, 0L, 0);

   /*** Add information to housekeeping record & write to calibration file. ***/

   addhk (hk, 0);

   nwr = write (luncal, hk, np);
   for (i = 0; i < np; i++)  *(op + i) = 0;
   nwr = write (luncal, outbuf, np);

   /*** Write Bitmap record to calibration file. ***/

   nwr = write (luncal, bitmap, np);
   for (i = 0; i < np; i++) *(op + i) = 0;
   nwr = write (luncal, outbuf, np);

   /*** Read geometric correction data (records and pixels). ***/

   nrew = lseek (lungeo, 0L, 0); 
   if (nrd = read (lungeo, rgeom, ngeo2) <= 0)
      error ("Read error...nrd: %d file: %s", nrd, namgeo);
   if (nrd = read (lungeo, pgeom, ngeo2) <= 0)
      error ("Read error... nrd: %d file: %s", nrd, namgeo);
   close (lungeo);
   /*
    * printf ("pgeom: %f %f %f %f %f\n", pgeom [0][0], pgeom [0][1],
    *          pgeom [0][2], pgeom [0][3], pgeom [0][4]);
    * printf ("rgeom: %f %f %f %f %f\n", rgeom [0][0], rgeom [0][1],
    *          rgeom [0][2], rgeom [0][3], rgeom [0][4]);
    */

   /*** Compute offsets for horizontal and vertical axes. ***/

   ip0 = XOFF;
   ir0 = YOFF;

   /* Geometric distortion correction will be done in an array of dimensions
    * cp [nr+YOFF*2] [np+XOFF*2].
    */

   /* ******* Record Loop ******* */

   jj = -1;
   jmin = ir * jres;
   jmax = lr * jres;
   imin = ip * jres;
   imax = lp * jres;

   for (j = jmin; j <= jmax; j += jres)
   {
    jj++;

   /*** Magnification and translation for vertical axis. ***/

   sr = ((float) (j) - dycen) / fwmag + dycen
        + pwcory + smcory + sydif;

   /* kr: Y-axis index for upper left corner in interpolation.
    * fr: fractional displacement between records for interpolation.
    */

   kr = (int) (sr * one16);
   fr = sr * one16 - (float) kr;
   /* printf ("jj: %d   sr: %f   kr: %d   fr: %f\n", jj, sr, kr, fr); */

   /*** Magnification and translation in horizontal axis. ***/

   ii = -1;
   for (i = imin; i <= imax; i += jres)
      {
      ii++;
      sp = ((float) (i) - dxcen) / fwmag + dxcen
         + pwcorx + smcorx + sxdif;

      /* ******* Geometric Distortion: spatial function. *******/

      /*** Interpolate to create 896 points in x & y from 59x59. ***/

      /* kp: X-axis index for upper left corner in interpolation.
       * fp: Fractional displacement between pixels for interpolation.
       */

      kp = (int) (sp * one16);
      fp = sp * one16 - (float) kp;
      /* printf ("ii: %d   sp: %f   kp: %d   fp: %f\n", ii, sp, kp, fp); */

      /*** Bilinear interpolation for X-axis. ***/

      fx0 = *(pg + (kr+1)*dr + (kp+1))
            + fp * (*(pg + (kr+1)*dr + (kp+2)) - *(pg + (kr+1)*dr + (kp+1)) );

      fx1 = *(pg + (kr+2)*dr + (kp+1))
            + fp * (*(pg + (kr+2)*dr + (kp+2)) - *(pg + (kr+2)*dr + (kp+1)) );

      gp = ((fx0 + fr * (fx1 - fx0)) / (float) jres) + ip0;

      /*** Bilinear interpolation for Y-axis. ***/

      fy0 = *(rg + (kr+1)*dr + (kp+1))
            + fp * (*(rg + (kr+1)*dr + (kp+2)) - *(rg + (kr+1)*dr + (kp+1)) );

      fy1 = *(rg + (kr+2)*dr + (kp+1))
            + fp * (*(rg + (kr+2)*dr + (kp+2)) - *(rg + (kr+2)*dr + (kp+1)) );

      gr = ((fy0 + fr * (fy1 - fy0)) / (float) jres) + ir0;

      /*** Geometric distortion:  grey-scale bilinear interpolation. ***/

      /* mp: X-axis index for upper left corner in interpolation.
       * zp: fractional displacement between pixels for interpolation.
       * mr: Y-axis index for upper left corner in interpolation.
       * zr: fractional displacement between records for interpolation.
       */

      mp = (int) gp;
      zp = gp - (float) mp;
      mr = (int) gr;
      mrr = mr * dx;
      mrr1 = mrr + dx;
      zr = gr - (float) mr;

      fx0 = *(cpp + mrr + mp)
            + zp * (*(cpp + mrr + mp+1) - *(cpp + mrr + mp));
      fx1 = *(cpp + mrr1 + mp)
            + zp * (*(cpp + mrr1 + mp+1) - *(cpp + mrr1 + mp));
      cval = (fx0 + zr * (fx1 - fx0)) * radfac;
      if (cval < xmin)  cval = xmin;
      if (cval > xmax)  cval = xmax;
      *(op + ii) = cval;
     }

   nwr = write (luncal, outbuf, np*2);
   }
   ncl = close (luncal);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

error (str1, str2)   /* print error message and exit. */

char *str1, *str2;
{
   fprintf (stderr, str1, str2);
   fprintf (stderr, "\n");
   exit (1);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

char *
outname (namvid)	/* Create output file name (change v to c in suffix). */

char *namvid;

{
int lenvid;
int idot;
int i;
static char calnam [80];

   lenvid = strlen (namvid);
   idot = -1;

   strcpy (calnam, namvid);

   for (i=0; i<lenvid; i++) 
      if (namvid [i] == '.')  idot = i;

   calnam [idot+1] = 'c';
   calnam [lenvid] = '\0';

   return (calnam);
}


/* fexpand
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Expand floating point parameter line arrays for high resolution images.
 * Parameter files are in low resolution (448 numbers/line).
 * Double each point for high resolution.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * author: Andrew L. Stanger   Associate Scientist                     *
 * 28 February 1989
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

fexpand (fbuf, nelem)

float *fbuf;
int nelem;

{
register int i;
register float *fb;

   fb = fbuf;

   for (i=nelem-1; i>=0; i--)
      {
      *(fb + 2*i+1) = *(fb + i);
      *(fb + 2*i)   = *(fb + i);
      }
   return (0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
