/*  Compute the various ephemeris data.


    Some of this is slightly re-written ECHO code that David Elmore wrote.
    Some of this is slightly re-written ECHO code that Alice Lecinski wrote.
    Some of it is slightly re-written Jay Chalmers code.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "mk4v2.h"
#include "ephem.h"


long julianDay(year, month, day) int year, month, day; {
    
    long lDay;
    int  i;

    i = (month - 14)/12;

    if ( year<0) year += 1;

    year += i;

    lDay =   day - 1432 
           + ((1461L * (year + 4716)) >> 2)
           + 367 * (month -2 -i*12)/12;

    if (lDay >= 2299161L) 
        lDay -= ((3*((year-1500)/100)) >> 2) + 10;

    return lDay;
}


double last(jday, g_m_t) long jday; long g_m_t; {

    double tu, dt;

    if (abs(jday) > 10000)
        jday = jday - JD_EPOCH;

    tu = (double)((jday * 2.0) - 1.0) / (36525.0 * 2);
    
    tu = tu * ((tu * ((-tu * s2a*6.2E-06) + s2a*0.093104)) + s2a*184.812866)
         + (200.0 * pi * tu)
         + s2a*24110.54841 
         - siteLongitude ;    

    dt = (double)(g_m_t % ONE_DAY) / (double)(ONE_DAY);
    tu = tu +        
         (g_m_t / ONE_DAY)  * to_sidereal +    
         (dt * twopi) + (dt * to_sidereal);    

    return tu;
}


void abcd_latitude(ha, dec, Paz, Pel, PpAngle)
                   double ha, dec; 
                   double *Paz, *Pel, *PpAngle;
{
    double sin_ha,  cos_ha,  sin_dec, cos_dec;
    double tmp_pAngle, tmp_az, tmp_el;

    sin_ha  = sin(ha) ;
    cos_ha  = cos(ha) ;
    sin_dec = sin(dec);
    cos_dec = cos(dec);

    tmp_az     = (sin_dec * cos_Lat) - (cos_dec * cos_ha  * sin_Lat);
    tmp_pAngle = (sin_Lat * cos_dec) - (sin_dec * cos_Lat * cos_ha );
    tmp_el     = (sin_dec * sin_Lat) + (cos_dec * cos_ha  * cos_Lat) ;

    *Paz       = atan2( -cos_dec * sin_ha , tmp_az );
    *PpAngle   = atan2(  cos_Lat * sin_ha , tmp_pAngle);
    
    *Pel       = asin(tmp_el);
}



void sunEphemeris(myday, Pra,Pdec,Psunrad, PpAngle, PbAngle, PlAngle)
                  long myday;
                  double *Pra, *Pdec, *Psunrad, *PpAngle, *PbAngle, *PlAngle;
{
    double tmp_ra, tmp_dec, tmp_b , tmp_l , twog ;
    double mean_long, g, lambda, xi;                
    double sin_long, cos_long, sin_xi, cos_xi;        
    double omega, lmo;                                
    double sin_lmo, cos_lmo, r;                        

    /********************************************************************/
    /* Low precision method in "The Astronomical Almanac", 1984, p C24. */
    /********************************************************************/

    
    mean_long = d2r*280.460 + (d2r*0.9856474 * (double)myday);

    g         = d2r*357.528 + (d2r*0.9856003 * (double)myday);
    twog      = 2.*g ;
    lambda    = mean_long 
                + d2r*1.915 * sin(g) 
                + d2r*0.02  * sin(twog)  ;       
    xi        = d2r*23.439 
                - d2r*0.0000004 * (double)myday; 

    sin_long  = sin(lambda);
    cos_long  = cos(lambda);    
    sin_xi    = sin(xi); 
    cos_xi    = cos(xi);
    
    tmp_ra    = cos_xi * sin_long ;
    *Pra      = atan2(tmp_ra, cos_long);                
    *Pra      = fmod(fmod(*Pra, twopi) + twopi, twopi );
    
    tmp_dec   = sin_xi * sin_long ;
    *Pdec     = asin(tmp_dec);                            
    

    
    
    r         = 0.00014 - (0.01671 * cos(g)) - (0.00014 * cos(twog));

    *Psunrad  = d2r*(0.2666) * (1.0 / (1.0 + r));

    /*********************************************************************/
    /* Method in "Explanatory Supplement to the Astronomical Ephemeris", */
    /* 1961, p 307-310.                                                  */
    /* Omega is longitude of solar equator's ascending node              */
    /*********************************************************************/
    omega =   d2r*(73.666667)                              
           +  d2r*((EPOCH - 1850.0) * (50.25 / 3600.0))  
           + (d2r*(50.0 * (50.25 / 3600.0)) * ((double)myday * 2. / 36525.0));

    
    lmo      = lambda - omega;
    sin_lmo  = sin(lmo);
    cos_lmo  = cos(lmo);

    *PpAngle = atan2(-cos_long*sin_xi, cos_xi)  + atan2(-cos_lmo*sin_I,cos_I);

    tmp_b    = sin_lmo * sin_I ;
    *PbAngle = asin(tmp_b);                    

    tmp_l    =  sin_lmo * cos_I ;
    *PlAngle =  atan2(tmp_l, cos_lmo)  
              + d2r*((((J1854 - JD_EPOCH) * 360.0) / 25.38) + 180.) 
              - d2r*(360.0 / 25.38) * (double)myday; 

    *PlAngle = fmod(fmod(*PlAngle, twopi) + twopi, twopi);
}


void solarEphemeris(myday, g_m_t, 
                    Pra,   Pdec, Psunrad, PpAngle, PbAngle, PlAngle)
		    long   myday, g_m_t;
                    double *Pra,*Pdec, *Psunrad, *PpAngle,  *PbAngle, *PlAngle;
{
    long refTime ; 
    long lDay;
    double rra, rdec, rsunrad, rpAngle, rbAngle, rlAngle;
    double dra, ddec, dsunrad, dpAngle, dbAngle, dlAngle;
    double time;

    refTime = (long)(ONE_DAY)/(2L);    

    lDay = abs(myday) > 10000L ? myday - (long)JD_EPOCH : myday;
    sunEphemeris(lDay,      &rra,&rdec,&rsunrad,&rpAngle,&rbAngle,&rlAngle);
    sunEphemeris(lDay + 1L, &dra,&ddec,&dsunrad,&dpAngle,&dbAngle,&dlAngle);

    if      ((dra - rra) >       pi ) dra -= twopi ;
    else if ((dra - rra) < -1. * pi ) dra += twopi ;

    dra     = slope(    dra,rra    );
    ddec    = slope(   ddec,rdec   );
    dsunrad = slope(dsunrad,rsunrad);
    dpAngle = slope(dpAngle,rpAngle);
    dbAngle = slope(dbAngle,rbAngle);
    dlAngle = slope(dlAngle,rlAngle);
    
    time = (g_m_t - refTime) / (double)ONE_DAY;
    *Pra     = (    dra * time) + rra; 
    *Pdec    = (   ddec * time) + rdec;
    *Psunrad = (dsunrad * time) + rsunrad;
    *PpAngle = (dpAngle * time) + rpAngle;
    *PbAngle = (dbAngle * time) + rbAngle;
    *PlAngle = fmod(((dlAngle * time) + rlAngle), twopi);
    
}
int MRSUN_Isat() {
    
    long lJulianDay;
    long lGMT;
    double ra, declination, sunrad, pAngle, bAngle, lAngle, temp_pAngle;
    double sidereal_time, azimuth, elevation;    
    double ha ;
    int doprint=0;
    

    lJulianDay = julianDay(Eph_year, 
                           Eph_month,
                           Eph_day);

    lGMT = (long)(      Eph_second + 
                     60*Eph_minute + 
                  60*60*Eph_hour);

    if(doprint==1)
	fprintf(stderr,"lJulianDay %ld , lGMT %ld \n",lJulianDay,lGMT);

    solarEphemeris(lJulianDay, 
                   lGMT,&ra,&declination,&sunrad,&pAngle,&bAngle,&lAngle);

    
    ephRA      = r2d*(ra);
    ephDEC     = r2d*(declination);
    ephSRadius = 3600.0 * r2d*(sunrad);
    ephPangle  = r2d*(pAngle);
    ephBangle  = r2d*(bAngle);
    ephLangle  = r2d*(lAngle);

    sidereal_time = last(lJulianDay, lGMT);
    sidereal_time = fmod(fmod(sidereal_time,twopi)+twopi,twopi);

    ephST = r2d*(sidereal_time);
    ephHA = ephST - ephRA;

    if      (ephHA < -180.0) ephHA += 360.0;
    else if (ephHA >  180.0) ephHA -= 360.0;

    ha  = d2r*(ephHA);
    abcd_latitude(ha, declination, &azimuth, &elevation, &temp_pAngle);
    azimuth = fmod( fmod((azimuth),twopi) +twopi, twopi);

    ephAz = r2d*(azimuth);

    if (ephAz < 0.0) ephAz += 360.0;

    ephElev  = r2d*(elevation);
    ephAmass = 1./cos(piov2-elevation);

    return 0;
}
