/* mk4_namev2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create Mk4 FITS output file name.
 * filename format: YYYYMMDD_HHMM.mk4.CTYPE[.vig][.avg].fts
 * where
 *
 *	YYYY  = year  (4 digits)
 *	MM    = month (2 digits)
 *	DD    = day   (2 digits)
 *
 *	HH    = hour
 *	MM    = minute
 *
 *	C     = coordinate type: c=cylindrical/polar, r=rectangular/cartesian,
 *                               t=table.
 *	TYPE  = image type: i, q, u, I, k, Q, b, t, s, d.
 *	
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   16 June 2000 
 * 29 Nov 2000: add seconds to file name.
 * 14 Feb 2001: Put "avg" before "vig" in file name.
 * 15 Feb 2001: Put "mk4" in file name after the date.
 * 11 Jun 2001: Replace "pB" with "pb".
 *  8 Nov 2001: Add "bh" argument.
 *  2 Jul 2002: Revise extensions for ba, st, sk, tscp, drift files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "mk4v2.h"
#include "mk4headv2.h"
#include "mk4fitsv2.h"

char *
mk4_name (bh, doy_flag)

struct mk4_header *bh ;		/* Mk4 binary header structure. */
int doy_flag ;			/* =0 use month & day ;  =1 use day-of-year. */

{
    static char outnam [80]	= "YYYYMMDD.HHMM.mk4.CT.VIG.AVG.fts" ;
    static char telescope [8]	= "mk4" ;
    static char img_type [8]    = "" ;
    static char obs_date [12]	= "YYYYMMDD" ;
    static char obs_time [8]	= "HHMMSS" ;
    static char ext [24]	= "" ;

    int avg_flag ;
    int obs_year ;
    int obs_doy  ;

    /*** Ensure that year is a full 4-digit number. ***/

    obs_year = MK4F->year ;
    if (obs_year < 1900)  obs_year += 1900 ;
    obs_doy = day_of_year (obs_year, MK4F->month, MK4F->day) ;

    /*** Create date & time strings. ***/

/*
 *    printf ("mk4_name> doy_flag: %d\n", doy_flag) ;
 */
    if (doy_flag)
       sprintf (obs_date,"%02dd%03d", obs_year, obs_doy) ;
    else
        sprintf (obs_date, "%04d%02d%02d", obs_year, MK4F->month, MK4F->day) ;

/*
 *    sprintf (obs_time, "%02d%02d",     MK4F->hour, MK4F->minute) ;
 */
    sprintf (obs_time, "%02d%02d%02d", MK4F->hour, MK4F->minute, MK4F->second) ;

    /*** Set coordinate type: c[ylindrical], r[ectangular], t[able]. ***/

    strcpy (img_type, "") ;

    switch (bh->ProcSteps)
	{
	case IS_raw:
	case IS_raw_spln:
	case IS_cal:
	case IS_cal_slit:
	case IS_cal_avg:
	case IS_cal_avg_slit:
	case IS_raw_avg:
	case IS_raw_avg_spln:
					strcpy (img_type, "c") ; break ;

	case IS_raw_rect:
	case IS_cal_rect:
	case IS_cal_rect_slit:
	case IS_cal_rect_byte:
	case IS_cal_avg_rect:
	case IS_cal_avg_rect_slit:
	case IS_raw_avg_rect:
					strcpy (img_type, "r") ; break ;

	default:			strcpy (img_type, "x") ;
	}

    /*** Set data type: i, q, u, b, s, t, m, d. ***/

    switch (bh->ProcSteps)
	{
	case IS_raw:
	case IS_raw_spln:
	case IS_raw_rect:
	case IS_raw_avg:
	case IS_raw_avg_spln:
	case IS_raw_avg_rect:

		switch (bh->DataType)
		    {
		    case IS_i:		strcat (img_type, "i") ; break ;
		    case IS_q:		strcat (img_type, "q") ; break ;
		    case IS_u:		strcat (img_type, "u") ; break ;
		    case IS_ba:		strcpy (img_type, "ba") ; break ;
		    case IS_sk:		strcpy (img_type, "sk") ; break ;
		    case IS_st:		strcpy (img_type, "st") ; break ;
		    case IS_tscp:	strcpy (img_type, "ts") ; break ;
		    case IS_drift:	strcpy (img_type, "dr") ; break ;
		    default:		strcpy (img_type, "xx") ;
		    }
		break ;
      
/* old extensions:
		    case IS_ba:		strcpy (img_type, "tb") ; break ;
		    case IS_sk:		strcpy (img_type, "ts") ; break ;
		    case IS_st:		strcpy (img_type, "tt") ; break ;
		    case IS_tscp:	strcpy (img_type, "tm") ; break ;
		    case IS_drift:	strcpy (img_type, "td") ; break ;
 */
        case IS_cal:
	case IS_cal_slit:
        case IS_cal_rect:
        case IS_cal_rect_slit:
        case IS_cal_rect_byte:
        case IS_cal_avg:
        case IS_cal_avg_slit:
        case IS_cal_avg_rect:
        case IS_cal_avg_rect_slit:

		switch (bh->DataType)
		    {
		    case IS_i:		strcat (img_type, "I") ; break ;
/*		    case IS_q:		strcat (img_type, "k") ; break ; */
		    case IS_q:		strcat (img_type, "pb") ; break ;
		    case IS_u:		strcat (img_type, "U") ; break ;
		    case IS_ba:		strcpy (img_type, "BA") ; break ;
		    case IS_sk:		strcpy (img_type, "SK") ; break ;
		    case IS_st:		strcpy (img_type, "ST") ; break ;
		    case IS_tscp:	strcpy (img_type, "TS") ; break ;
		    case IS_drift:	strcpy (img_type, "DR") ; break ;
		    default:		strcpy (img_type, "xx") ;
		    }
	        break ;

/* old extensions:
		    case IS_ba:		strcpy (img_type, "tB") ; break ;
		    case IS_sk:		strcpy (img_type, "tS") ; break ;
		    case IS_st:		strcpy (img_type, "tT") ; break ;
		    case IS_tscp:	strcpy (img_type, "tm") ; break ;
		    case IS_drift:	strcpy (img_type, "tD") ; break ;
 */
	default: strcpy (img_type, "xx") ;
	}

    /*** Determine the extensions to append to the file name. ***/

    strcpy (ext, "") ;

    /*** For averaged images, append ".avg" to the extensions. ***/

    switch (bh->ProcSteps)
	{
	case IS_cal_avg:
	case IS_cal_avg_slit:
	case IS_cal_avg_rect:
	case IS_cal_avg_rect_slit:
	case IS_raw_avg:
	case IS_raw_avg_spln:
	case IS_raw_avg_rect:
					strcat (ext, ".avg") ; avg_flag = 1 ;
					break ;

	default:			avg_flag = 0 ;
	}

    /*** For "vignetted" images, append ".vig" to the extensions. */

    switch (bh->ProcSteps)
	{
	case IS_cal_slit:
	case IS_cal_rect_slit:
	case IS_cal_avg_slit:
	case IS_cal_avg_rect_slit:
					strcat (ext, ".vig") ; break ;

	default:			break ;
	}

    /*** Finally, append ".fts" to the extensions. ***/

    strcat (ext, ".fts") ;

    /*** Create output file name. ***/

    if (avg_flag)
        sprintf (outnam, "%s.mk4.%s%s",    obs_date,           img_type, ext) ;
    else
        sprintf (outnam, "%s.%s.mk4.%s%s", obs_date, obs_time, img_type, ext) ;

    return (outnam) ;
}

