/* mk3tofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Mark-3 image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 February 1994
 * 17 Feb 94: use for both primary and extension HDUs.
 * 18 Feb 94: adjust barrel angle for phase & pangle, then multiply by 60.
 * 11 May 94: dispmin/dispmax changed from int to float.
 * 13 May 94: add TYPE-OBS keyword.
 *  6 Jun 94: change keyword PANGLE to SOLAR_P0.  Add DIRAXIS1/2, DATE, TIME.
 * 18 Aug 94: change keywords SCNDUR & SCNDIR to SCAN-DUR and SCAN-DIR.
 * 24 Jun 96: Change time delimiter from '.' to ':'.
 * 10 Jul 96: Make VERSION a character string, not a floating point number.
 * 10 Jul 96: [Version 1.03]  Convert SLIT images to 8 bits per pixel.
 * 25 Oct 96: Inserted CRRADIUS parameter.
 * 24 Feb 97: Set soff = dsmin for slit image scaling.
 * 10 Mar 97: Add nhdu to make it easier to add new keyword/parameter lines.
 * 31 Mar 97: Clip rpB intensities to the ranger -3000 -> +15000.
 *  1 Apr 97: Change dispmin & dispmax from unscaled to scaled units.
 * 30 May 97: Accomodate optional photometric scaling parameters.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <string.h>
#include "/home/cordyn/mlsofits/lib/mk3/mk3lib.ext"
#include "/home/cordyn/mlsofits/img_scale.ext"
static char fitbuf [180][80] ;			/* FITS header buffer. */

mk3tofits (namimg, namext, hduflag, pmode)

   char namimg [] ;		/* Name of primary file. */

   char namext [] ;		/* Name of extension file. */

   int hduflag ;		/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */

   int pmode ;			/* Print mode. */

{
   extern char *malloc () ;		/* Function to allocate memory. */
   extern unsigned char *rdmk3 () ;	/* Function to read Mark-3 image. */

   static char str [80] ;
   char *sptr;
   char *padbuf ;			/* Pointer to padding buffer array. */
   unsigned char *mk3buf ;		/* MLSO Mark-3 file  array pointer. */
   unsigned char *imgbuf ;		/* MLSO Mark-3 image array pointer. */

   static char kangnorth [12] = "ANGNORTH" ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kchannel  [12] = "CHANNEL " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdataform [12] = "DATAFORM" ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_opc [12] = "DATE-OPC" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kdiraxis1 [12] = "DIRAXIS1" ;
   static char kdiraxis2 [12] = "DIRAXIS2" ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char kphase    [12] = "PHASE   " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char krmin     [12] = "RMIN    " ;
   static char krmax     [12] = "RMAX    " ;
   static char kscan_dur [12] = "SCAN-DUR" ;
   static char kscan_dir [12] = "SCAN-DIR" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktime_opc [12] = "TIME-OPC" ;
   static char ktmin     [12] = "TMIN    " ;
   static char ktmax     [12] = "TMAX    " ;
   static char kversion  [12] = "VERSION " ;

   static char atype1 [24] ;			/* Coord type axis 1. */
   static char atype2 [24] ;			/* Coord type axis 2. */
   static char bunit  [32] ;			/* Brightness unit. */
   static char ctype1 [24] ;			/* Coord type axis 1. */
   static char ctype2 [24] ;			/* Coord type axis 2. */
   static char cunit1 [24] ;			/* Coord units axis 1. */
   static char cunit2 [24] ;			/* Coord units axis 2. */
   static char outbuf [84] ;			/* output buffer. */
   static char datcom [24] ;			/* data scale comment. */
   static char cdate    [12] = "DD/MM/YY" ;	/* File creation date. */
   static char date_cal [12] = "DD/MM/YY" ;	/* Date image calibrated. */
   static char date_opc [12] = "DD/MM/YY" ;	/* Date of cal tilt plate img.*/
   static char date_obs [12] = "DD/MM/YY" ;	/* Observation date. */
   static char dform    [32] ;			/* Data form. */
   static char dformcom [32] = "";		/* Data form comment. */
   static char diraxis1 [12] = "RIGHT" ;	/* Direction of axis 1: right.*/
   static char diraxis2 [12] = "UP" ;		/* Direction of axis 2: up. */
   static char eq [4] = "= " ;			/* Equal symbol. */
   static char namfit [80] ;			/* FITS file name. */
   static char object [72] ;			/* Object. */
   static char pixcom [48] ;			/* data type comment. */
   static char refcom [48] ;			/* coord ref comment. */
   static char scan_dir [24] ;			/* Scan direction. */
   static char ctime    [12] = "HH:MM:SS" ;	/* File creation time. */
   static char time_obs [12] ;			/* Observation time. */
   static char time_opc [12] ;			/* Time of opal cal 0 img.*/
   static char type_obs [72] ;			/* Time of opal cal 0 img.*/
   static char version  [24] ;			/* MK3 version. */

   short *sbuf ;				/* short integer pointer. */
   short sval ;

   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */

   int bitpix ;				/* # bits per pixel. */
   int dsmax = 0;			/* Data scaling maximum. */
   int dsmin = 0;			/* Data scaling minimum. */
   int gcount = 1 ;			/* Extension gcount. */
   int iyear ;				/* year - 1900. */
   int i, ix, iy ;			/* index values. */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* integer value. */
   int iynum ;				/* # lines in FITS header. */
   int ndim = 0 ;			/* # dimensions for image (2=XY). */
   int nhdu ;				/* # HDU blocks to write. */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyfits = 2880 ;			/* # bytes per FITS logical record. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int nrec ;				/* # fits records for image. */
   int nelwr  = 0 ;			/* # elements written by fwrite. */
   int pcount = 0 ;			/* Extension pcount. */
   int smin = 0 ;			/* display min value. */
   int smax = 0 ;			/* display max value. */
   int swrite = 0 ;

   float angnorth =  0.0 ;		/* solar north w.r.t. +Y axis. */
   float bscale = 1.0e-10 ;		/* Brightness intensity scale factor. */
   float bzero  =     0.0 ;		/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float crpix1, crpix2 ;		/* Reference pixel location.*/
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin = 0.0 ;			/* Data minimum value. */
   float dmax = 0.0 ;			/* Data maximum value. */
   float fmin = 0.0 ;			/* Display min value. */
   float fmax = 0.0 ;			/* Display max value. */
   float fval   =     0.0 ;		/* floating point value. */
   float phase, pangle ;		/* MK3 phase angle & p-angle. */
   float ir, dr ;			/* diode 0 radius, r between diodes. */
   float dt ;				/* angle between rays. */
   float rad0, ang0 ;			/* radius/theta center coordinates. */
   float slope = 1.0 ;			/* Slope. */
   float soff = 0.0 ;			/* Offset. */
   FILE *FF ;				/* FITS file ID. */

   swrite = 0 ;
   sprintf (namfit, "%s.fts", namimg) ;

   if (pmode)
      {
      fprintf (stdout, "mk3tofits> namimg: %s  namext: %s   hduflag: %d\n",
	       namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- mk3tofits> %s -> %s -------\n\n",
	       namimg, namfit) ;
      }

   /*** Read MLSO Mark-3 image/data from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2)
      mk3buf = rdmk3 (namimg, pmode) ;	/* Read MK3 dataset into memory. */
   else
      mk3buf = rdmk3 (namext, pmode) ;	/* Read MK3 dataset into memory. */

   nx = MK3I->xdim ;			/* X dimension. */
   ny = MK3I->ydim ;			/* Y dimension. */
   npximg = nx * ny ;			/* # pixels in image. */
   nbyimg = npximg * MK3I->nbypx ;	/* # bytes in image. */
   bscale = MK3I->bscale ;
   bzero  = MK3I->bzero  ;
   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			nx, ny, MK3I->nbypx, npximg, nbyimg) ;

   imgbuf = mk3buf ;			/* Start of pixel/array data. */
   sbuf = (short *) imgbuf ;

   iyear = MK3I->year ;
   if (MK3I->year > 1900)  iyear = MK3I->year - 1900 ;

   sprintf (date_obs, "%02d/%02d/%02d", MK3I->day, MK3I->month, iyear) ;
   sprintf (time_obs, "%02d:%02d:%02d", MK3I->hour, MK3I->minute, MK3I->second) ;

   iyear = MK3I->cal_year ;
   if (MK3I->cal_year > 1900)  iyear = MK3I->cal_year - 1900 ;
   sprintf (date_cal, "%02d/%02d/%02d", MK3I->cal_day, MK3I->cal_month, iyear) ;

   iyear = MK3I->opc_year ;
   if (MK3I->opc_year > 1900)  iyear = MK3I->opc_year - 1900 ;
   sprintf (date_opc, "%02d/%02d/%02d", MK3I->opc_day, MK3I->opc_month, iyear) ;
   sprintf (time_opc, "%02d:%02d:%02d",
		       MK3I->opc_hour, MK3I->opc_minute, MK3I->opc_second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   sprintf (cdate, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (ctime, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   if (MK3I->scndir == -1)
      strcpy (scan_dir, "CW") ;
   else
   if (MK3I->scndir ==  1)
      strcpy (scan_dir, "CCW") ;
   else
      strcpy (scan_dir, "") ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (MK3I->xdim > 0)
      ndim++ ;
   if (MK3I->ydim > 1)
      ndim++ ;

   /* MK3 data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * q   --> cylindrical q data					 0
    * rq  --> rectangular q data, origin in SE				 1
    * srq --> scaled rectangular q data, origin in SE			 2
    * i   --> cylindrical i data					 3
    * ri  --> rectangular i data, origin in SE				 4
    * sri --> scaled rectangular i data, origin in SE			 5
    * u   --> cylindrical u data					 6
    * ru  --> rectangular u data, origin in SE				 7
    * sru --> scaled rectangular u data, origin in SE			 8
    * pB  --> cylindrical pB*1.e-10 data				 9
    * rpB  --> rectangular pB*1.e-10 data, origin in SE			10
    * srpB --> scaled rectangular pB data, origin in SE			11
    * Ne  --> cylindrical Ne data					12
    * rNe  --> rectangular Ne*1.e+33 data, origin in SE			13
    * srNe --> scaled rectangular Ne data, origin in SE			14
    * aba --> barrel angles as ascii in degrees *10			15
    * ba --> barrel angles (shorts) with 16384=360 degrees		16
    * ast --> scan times as ascii in seconds				17
    * st --> scan times (shorts) with 60=1 second (Hz)			18
    * a .calib file							19
    * a .tscp file							20
    * I  --> calibrated cylindrical I data				21
    * U  --> calibrated cylindrical U data				22
    * avg --> avgeraged rectangular pB*1.e-10 data, origin in SE	23
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;

   /*** Set default (Cartesian) coordinate parameters. ***/

   strcpy (object,   "SUN") ;   
   strcpy (type_obs, "W.LIGHT") ;   
   strcpy (datcom, "[BSUN]") ;		/* DATAMIN/DATAMAX UNITS. */
   crpix1 = (float) ((nx - 1) / 2) + 0.5 ;	/* X sun center [pixels]. */
   crpix2 = (float) ((ny - 1) / 2) + 0.5 ;	/* Y sun center [pixels]. */
   crval1 = 0.0 ;			/* X sun center [Rsun]. */
   crval2 = 0.0 ;			/* Y sun center [Rsun]. */
   cdelt1 = 1.0 / MK3I->pixrs ;		/* # Rsun per pixel in x-axis. */
   cdelt2 = 1.0 / MK3I->pixrs ;		/* # Rsun per pixel in y-axis. */
   crota1 = 0.0 ;			/* Rotation of x-axis w.r.t. ctype1. */
   crota2 = 0.0 ;			/* Rotation of y-axis w.r.t. ctype2. */

   /*** Set default data min/max values. ***/

   dmin = MK3I->dmin * MK3I->bscale + MK3I->bzero ; /* After scaling is done. */
   dmax = MK3I->dmax * MK3I->bscale + MK3I->bzero ; /* After scaling is done. */

   /*** Reference point:
    *** Cartesian coordinates: sun center.
    *** Polar coordinates: radius = 0, theta = 0. 
    ***/

   strcpy (refcom, "SUN CENTER") ;
   phase  = (float) MK3I->phase ;
   pangle = (float) MK3I->pangle ;
   ir = (float) MK3H->ir0 * 1.e-4 ;	/* Diode 0 radius (Rsun). */
   dr = (float) MK3H->idr * 1.e-7 ;	/* Diode increment (Rsun). */
   dt = 0.5 ;				/* angular increment (deg) for rays. */
   rad0 = - ir / dr ;			/* Sun center w.r.t. diode #0 [index].*/
   if (MK3I->tmin < 0)
      ang0 = (  0.0 - MK3I->tmin) / dt ;	/* Theta = 0 in index units. */
   else
      ang0 = (360.0 - MK3I->tmin) / dt ;	/* Theta = 0 in index units. */

   strcpy (cunit1, "RSUN") ;
   strcpy (cunit2, "RSUN") ;
   sprintf (version, "MK3 1.03") ;

   /*** Define various parameters, based upon MK3 data type:
   /*** object
    *** type_obs
    *** brightness units
    *** coordinate type, reference, increment
    *** bitpix
    *** display scaling
    ***/

   switch (MK3I->dtype)
      {
      case  0: /* q */
               strcpy (type_obs, "Q") ;
               strcpy (dform,    "CYLINDRICAL Q") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "RADIUS") ;
               strcpy (atype2,   "THETA") ;
	       crpix1 = rad0 ;   crpix2 = ang0 ; /* Sun center location. */
	       crval1 = 0.0 ;    crval2 = 0.0 ;	/* coord value at sun center. */
	       cdelt1 = dr ;     cdelt2 = dt ;	/* pixel increment. */
               strcpy (cunit1,   "RSUN") ;
               strcpy (cunit2,   "DEGREES") ;
	       crota1 = - phase - pangle ;   crota2 = - phase - pangle ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case  1: /* rq */	
               strcpy (type_obs, "Q") ;
               strcpy (dform,    "RECTANGULAR Q") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case  2: /* srq */
               strcpy (type_obs, "Q") ;
               strcpy (dform,    "SCALED RECT Q") ;
               strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =   8 ;			/* byte image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case  3: /* i */
               strcpy (type_obs, "I") ;
               strcpy (dform,    "CYLINDRICAL I") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "RADIUS") ;
               strcpy (atype2,   "THETA") ;
	       crpix1 = rad0 ;   crpix2 = ang0 ;/* Sun center location. */
	       crval1 = 0.0 ;    crval2 = 0.0 ; /* coord value at sun center. */
	       cdelt1 = dr ;     cdelt2 = dt ;	/* pixel increment. */
               strcpy (cunit1,   "RSUN") ;
               strcpy (cunit2,   "DEGREES") ;
	       crota1 = - phase - pangle ;   crota2 = - phase - pangle ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case  4: /* ri */
               strcpy (type_obs, "I") ;
               strcpy (dform,    "RECTANGULAR I") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case  5: /* sri */
               strcpy (type_obs, "I") ;
               strcpy (dform,    "SCALED RECT I") ;
               strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =   8 ;			/* byte image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case  6: /* u */
               strcpy (type_obs, "U") ;
               strcpy (dform,    "CYLINDRICAL U") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "RADIUS") ;
               strcpy (atype2,   "THETA") ;
	       crpix1 = rad0 ;   crpix2 = ang0 ;/* Sun center location. */
	       crval1 = 0.0 ;    crval2 = 0.0 ;	/* coord value at sun center. */
	       cdelt1 = dr ;     cdelt2 = dt ;	/* pixel increment. */
               strcpy (cunit1,   "RSUN") ;
               strcpy (cunit2,   "DEGREES") ;
	       crota1 = - phase - pangle ;   crota2 = - phase - pangle ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case  7: /* ru */
               strcpy (type_obs, "U") ;
               strcpy (dform,    "RECTANGULAR U") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case  8: /* sru */
               strcpy (type_obs, "U") ;
               strcpy (dform,    "SCALED RECT U") ;
               strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =   8 ;			/* byte image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case  9: /* pB */
               strcpy (type_obs, "PB") ;   
               strcpy (dform,    "CYLINDRICAL PB") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "RADIUS") ;
               strcpy (atype2,   "THETA") ;
	       crpix1 = rad0 ;   crpix2 = ang0 ;/* Sun center location. */
	       crval1 = 0.0 ;    crval2 = 0.0 ;	/* coord value at sun center. */
	       cdelt1 = dr ;     cdelt2 = dt ;	/* pixel increment. */
               strcpy (cunit1,   "RSUN") ;
               strcpy (cunit2,   "DEGREES") ;
	       crota1 = - phase - pangle ;   crota2 = - phase - pangle ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case 10: /* rpB */
               strcpy (type_obs, "PB") ;   
               strcpy (dform,    "RECTANGULAR PB") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
	       dsmin = -3000 ;   dsmax = 15000 ;
               for (i = 0; i < npximg; i++)
	          {
                  sval = (short) *(sbuf + i) ;
                  if (sval < dsmin)  sval = dsmin ;
                  if (sval > dsmax)  sval = dsmax ;
	          *(sbuf + i) = (short) sval ;
	          }
               break ;
      case 11: /* srpB */
               strcpy (type_obs, "PB") ;   
               strcpy (dform,    "SCALED RECT PB") ;
               strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =   8 ;			/* byte image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 12: /* Ne */
               strcpy (type_obs, "ELEC DEN") ;   
               strcpy (dform,    "CYLINDRICAL NE") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (datcom,   "[NELEC/CM^2]") ;
	       strcpy (bunit,    "NE/CM2") ;
               strcpy (atype1,   "RADIUS") ;
               strcpy (atype2,   "THETA") ;
	       crpix1 = rad0 ;   crpix2 = ang0 ;/* Sun center location. */
	       crval1 = 0.0 ;    crval2 = 0.0 ;	/* coord value at sun center. */
	       cdelt1 = dr ;     cdelt2 = dt ;	/* pixel increment. */
               strcpy (cunit1,   "RSUN") ;
               strcpy (cunit2,   "DEGREES") ;
	       crota1 = - phase - pangle ;   crota2 = - phase - pangle ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case 13: /* rNe */
               strcpy (type_obs, "ELEC DEN") ;   
               strcpy (dform,    "RECTANGULAR NE") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (datcom,   "[NELEC/CM^2]") ;
	       strcpy (bunit,    "NE/CM2") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case 14: /* srNe */
               strcpy (type_obs, "ELEC DEN") ;   
               strcpy (dform,    "SCALED RECT NE") ;
               strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
               strcpy (datcom,   "[NELEC/CM^2]") ;
	       strcpy (bunit,    "NE/CM2") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =   8 ;			/* byte image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 15: /* aba */
               strcpy (object,   "BARREL ANGLES") ;   
               strcpy (type_obs, "BA") ;   
               strcpy (dform,    "ASCII BARREL ANGLES") ;
               strcpy (pixcom,   "ASCII CHARACTERS") ;
               strcpy (datcom,   "[NELEC/CM^2]") ;
	       strcpy (bunit,    "DEGREES") ;
               strcpy (atype1,   "THETA") ;
               strcpy (atype2,   "NONE") ;
	       strcpy (refcom,   "SCAN START") ;
	       crpix1 = (float) (nx - 1) ;	/* Reference: last element. */
	       crval1 = atof (*(imgbuf+nx-1))
		      - phase - pangle ;		/* coord value at reference. */
	       cdelt1 = dt ;
	       crota1 = - phase - pangle ;	/* Coordinate rotation. */
               bitpix =  8 ;			/* ASCII characters. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 16: /* ba */
               strcpy (object,   "BARREL ANGLES") ;   
               strcpy (type_obs, "BA") ;   
               strcpy (dform,    "SHORT BARREL ANGLES") ;
               strcpy (pixcom,   "2-BYTE SIGNED DATA") ;
               strcpy (datcom,   "[DEGREES]") ;
	       strcpy (bunit,    "DEGREES") ;
               strcpy (atype1,   "THETA") ;
               strcpy (atype2,   "NONE") ;
	       strcpy (refcom,   "SCAN START") ;

               bscale = 1.0 / 60.0 ;
               bzero  = 0.0 ;
	       MK3I->bscale = bscale ;
	       MK3I->bzero  = bzero  ;
	       crpix1 = (float) (nx - 1) ;	/* Reference: last element. */
	       sbuf = (short *) imgbuf ;
	       crval1 = (float) (*(sbuf + nx - 1))
		      * bscale + bzero
		      - phase - pangle ;		/* coord value at reference. */
	       cdelt1 = dt ;
               strcpy (cunit1,   "DEGREES") ;	/* Coordinate units. */
	       crota1 = - phase - pangle ;	/* Coordinate rotation. */

               /*** Find data min/max. ***/

	       dmin =  32767 ;
	       dmax = -32767 ;
	       for (i = 0 ; i < MK3I->xdim ; i++)
		  {
		  fval = ((float) *(sbuf + i) * MK3I->bscale + MK3I->bzero
                                  - phase - pangle ) ;
                  if (fval < 0.0)  fval += 360.0 ;
		  fval *= 60.0 ;		/* Scaling for barrel angles. */
		  sval = (short) fval ;
		  if (sval < dmin)  dmin = sval ;
		  if (sval > dmax)  dmax = sval ;
		  *(sbuf + i) = sval ;
		  }
               MK3I->dmin = dmin ;
               MK3I->dmax = dmax ;
               dmin = dmin * bscale + bzero ;	/* after scaling is done. */
               dmax = dmax * bscale + bzero ;	/* after scaling is done. */

               bitpix =  16 ;			/* short integer image. */
               smin =     0 ;    smax = 21600 ;/* (0-360) * 60 */
               break ;
      case 17: /* ast */
               strcpy (object,   "SCAN TIMES") ;   
               strcpy (type_obs, "ST") ;   
               strcpy (dform,    "ASCII SCAN TIMES") ;
               strcpy (pixcom,   "ASCII CHARACTERS") ;
               strcpy (datcom,   "[HERTZ]") ;
	       strcpy (bunit,    "HERTZ") ;
               strcpy (atype1,   "TIME") ;
               strcpy (atype2,   "NONE") ;
	       strcpy (refcom,   "SCAN START") ;
	       crpix1 = (float) (nx - 1) ;	/* Reference: last element. */
	       crval1 = atof (*(imgbuf+nx-1)) ;	/* coord value at reference. */
	       cdelt1 = atof (*(imgbuf+nx-2))
		      - atof (*(imgbuf+nx-1)) ;	/* Approx time increment. */
	       crota1 = - phase - pangle ;	/* Coordinate rotation. */
               bitpix =  8 ;			/* ASCII characters. */
               smin =    0 ;    smax =  250 ;
               break ;
      case 18: /* st */
               strcpy (object,   "SCAN TIMES") ;   
               strcpy (type_obs, "ST") ;   
               strcpy (dform,    "SHORT SCAN TIMES") ;
               strcpy (pixcom,   "2-BYTE SIGNED DATA") ;
               strcpy (datcom,   "[SECONDS]") ;
	       strcpy (bunit,    "SECONDS") ;
               strcpy (atype1,   "TIME") ;
               strcpy (atype2,   "NONE") ;
	       strcpy (refcom,   "SCAN START") ;

               bscale = 1.0 / 60.0 ;
               bzero  = 0.0 ;
	       MK3I->bscale = bscale ;
	       MK3I->bzero  = bzero  ;
	       crpix1 = (float) (nx - 1) ;	/* Reference: last element. */
	       sbuf = (short *) imgbuf ;
	       crval1 = (float) (*(sbuf + nx - 1))
		      * bscale + bzero ;	/* coord value at reference. */
	       cdelt1 = (float) (*(sbuf+nx-2)
		               - *(sbuf+nx-1))
                      * bscale + bzero ;		/* scan time increment. */
               strcpy (cunit1, "SECONDS") ;	/* Coordinate units. */
	       crota1 = - phase - pangle ;	/* Coordinate rotation. */

               /*** Find data min/max. ***/

	       dmin =  32767 ;
	       dmax = -32767 ;
	       for (i = 0 ; i < MK3I->xdim ; i++)
		  {
		  sval = *(sbuf + i) ;
		  if (sval < dmin)  dmin = sval ;
		  if (sval > dmax)  dmax = sval ;
		  }
               MK3I->dmin = dmin ;
               MK3I->dmax = dmax ;
               dmin = dmin * bscale + bzero ;	/* after scaling is done. */
               dmax = dmax * bscale + bzero ;	/* after scaling is done. */

               bitpix =  16 ;			/* short integer image. */
               smin =     0 ;    smax = 10800 ;	/* Hertz = sec * 60 */
               break ;
      case 19: /* calibration information */
               strcpy (object,   "CALIBRATION INFO") ;   
               strcpy (type_obs, "CAL INFO") ;   
               strcpy (dform,    "CALIB PARAMETERS") ;
               strcpy (pixcom,   "2-BYTE SIGNED DATA") ;
               strcpy (datcom,   "[?]") ;
	       strcpy (bunit,    "NONE") ;
               strcpy (atype1,   "?") ;
               strcpy (atype2,   "?") ;
               bitpix =  16 ;			/* short integer image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 20: /* tscp */
               strcpy (object,   "TELESCOPE INFO") ;   
               strcpy (type_obs, "TSCP") ;   
               strcpy (dform,    "TSCP PARAMETERS") ;
               strcpy (pixcom,   "2-BYTE SIGNED DATA") ;
               strcpy (datcom,   "[?]") ;
	       strcpy (bunit,    "NONE") ;
               strcpy (atype1,   "?") ;
               strcpy (atype2,   "?") ;
               bitpix =  16 ;			 /* short integer image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 21: /* I calibrated */
               strcpy (type_obs, "I CAL") ;   
               strcpy (dform,    "I CAL") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 22: /* U calibrated */
               strcpy (type_obs, "U CAL") ;   
               strcpy (dform,    "U CAL") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin =     0 ;    smax =  250 ;
               break ;
      case 23: /* avgpB */
               strcpy (type_obs, "PB AVG") ;   
               strcpy (dform,    "CYLINDRICAL AVG PB") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "RADIUS") ;
               strcpy (atype2,   "THETA") ;
	       crpix1 = rad0 ;   crpix2 = ang0 ;/* Sun center location. */
	       crval1 = 0.0 ;    crval2 = 0.0 ;	/* coord value at sun center. */
	       cdelt1 = dr ;     cdelt2 = dt ;	/* pixel increment. */
               strcpy (cunit1,   "RSUN") ;
               strcpy (cunit2,   "DEGREES") ;
	       crota1 = - phase - pangle ;   crota2 = - phase - pangle ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
               break ;
      case 24: /* avgrpB */
               strcpy (type_obs, "PB AVG") ;   
               strcpy (dform,    "RECTANGULAR AVG PB") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
	       strcpy (bunit,    "BSUN") ;
               strcpy (atype1,   "X") ;
               strcpy (atype2,   "Y") ;
               bitpix =  16 ;			/* short integer image. */
               smin = -1000 ;    smax = 5000 ;
	       dsmin = -3000 ;   dsmax = 15000 ;
               for (i = 0; i < npximg; i++)
	          {
                  sval = (short) *(sbuf + i) ;
                  if (sval < dsmin)  sval = dsmin ;
                  if (sval > dsmax)  sval = dsmax ;
	          *(sbuf + i) = (short) sval ;
	          }
               break ;
      default:
               strcpy (object,   "UNKNOWN") ;   
               strcpy (type_obs, "UNKNOWN") ;   
               strcpy (dform,    "UNKNOWN") ;
               strcpy (bunit,    "UNKNOWN") ;
               strcpy (atype1,   "UNKNOWN") ;
               strcpy (atype2,   "UNKNOWN") ;
               bitpix =  16 ;			/* short integer image. */
               smin =     0 ;    smax =  250 ;
	       fprintf (stderr, "Invalid dtype: %d\n", MK3I->dtype) ;
	       exit (-1) ;
      }


   /*** Special case: convert to 8-bits/pixel. ***/

   sptr = str ;
   if ((sptr = strstr (namimg, "slit")) != NULL)
      {
      fprintf (stdout, "Image converted to 8-bits/pixel.\n") ;
      strcpy (type_obs, "PB AVG") ;
      strcpy (dform,    "RECT.SLIT AVG PB") ;
      strcpy (dformcom, ": SLIT VIGNETTED") ;
      strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
      strcpy (bunit,    "BSUN") ;
      strcpy (atype1,   "X") ;
      strcpy (atype2,   "Y") ;
      bitpix =  8 ;
      MK3I->nbypx = 1 ;
      nbyimg = npximg * MK3I->nbypx ;	/* # bytes in image. */

      /*** Convert from short to unsigned char. ***/

      bscale = 1.0 ;
      bzero  = 0.0 ;
      smin =   0 ;
      smax = 250 ;
      if (lscale == 1)
	 {
	 dsmin = foldmin ;
	 dsmax = foldmax ;
	 }
      else
	 {
         dsmin = -1000 ;
         dsmax =  7000 ;
	 }
      soff  = dsmin ;
      slope = (float) (smax - smin) / (float) (dsmax - dsmin) ;
      for (i = 0; i < npximg; i++)
	 {
         fval = ((float) *(sbuf + i) - soff) * slope ;
	 sval = (short) fval ;
         if (sval < smin)  sval = smin ;
         if (sval > smax)  sval = smax ;
	 *(imgbuf + i) = (unsigned char) sval ;
	 }
      dmin = dsmin * MK3I->bscale ;
      dmax = dsmax * MK3I->bscale ;
      }
   else
      strcpy (dformcom, "") ;

   sprintf (ctype1, "%s", atype1) ;
   sprintf (ctype2, "%s", atype2) ;

   MK3I->smin = smin ;
   MK3I->smax = smax ;
   fmin = (float) smin * bscale + bzero ;
   fmax = (float) smax * bscale + bzero ;

   /*** Create FITS file. ***/

   if (hduflag == 3)				/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- FIRST HDU ---\n") ;

   /*** Fill FITS header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   strcpy  (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: MLSO MARK-3 K-CORONAMETER") ;
   else
   sprintf (&fitbuf [iy++][0],  "XTENSION= 'IMAGE   '           / MLSO MARK-3 %s EXTENSION", dform) ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, ny, atype2) ;
   else
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;

   /*** Extension information. ***/

   if (hduflag == 1)
      {
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;
      }
   else
   if (hduflag == 2)
      {
      strcpy  (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
      }
   else
   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;
      }

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "OBJECT  = '%-8s'", object) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR CORONA") ;
   sprintf (&fitbuf [iy  ][0],  "TYPE-OBS= '%-8s'", type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ WHITE LIGHT [700 - 1080 NANOMETERS]") ;
   sprintf (&fitbuf [iy  ][0],  "ORIGIN  = 'HAO     '") ;
   sprintf (&fitbuf [iy++][31], "/ HIGH ALTITUDE OBSERVATORY (HAO)") ;
   strcpy  (&fitbuf [iy  ][0],  "TELESCOP= 'MK3     '") ;
   strcpy  (&fitbuf [iy++][31], "/ MARK-III K-CORONAMETER (MK3)") ;
   strcpy  (&fitbuf [iy  ][0],  "INSTRUME= 'RETICON '") ;
   strcpy  (&fitbuf [iy++][31], "/ 128-DIODE LINEAR RETICON") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [DOY %03d 19%02d]",
				MK3I->doy, MK3I->year) ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", ktime_obs, eq, time_obs) ;
   strcpy  (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS") ;

   /*** Coordinate system. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s", atype1) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / %s %s [PIXEL]",
			      kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s ROTATION [DEGREES] FROM REFERENCE", 
				kcrota1, eq, crota1, atype1) ;

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s", atype2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [PIXEL]",
			           kcrpix2, eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4e / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4e / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2, eq, cdelt2, atype2, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s ROTATION [DEGREES] FROM REFERENCE", 
				   kcrota2, eq, crota2, atype2) ;
      }

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / PIXELS PER SOLAR RADIUS", 
				 kcrradius, eq, MK3I->pixrs) ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'",     kbunit, eq, bunit) ;
   strcpy  (&fitbuf [iy++][31], "/ MAGNITUDE UNITS") ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / PHYSICAL = DATA * BSCALE + BZERO",
			       kbscale, eq, bscale) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e", kbzero , eq, bzero ) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / MINIMUM DATA VALUE %s",
			       kdatamin , eq, dmin, datcom) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / MAXIMUM DATA VALUE %s",
			       kdatamax , eq, dmax, datcom) ;

   strcpy  (&fitbuf [iy++][0],  "          **********************************************************************") ;

   /*** All keywords after this point are unique to the MLSO MK3. ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4e / MIN VALUE FOR DISPLAY %s",
			        kdispmin , eq, fmin, datcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4e / MAX VALUE FOR DISPLAY %s",
			        kdispmax , eq, fmax, datcom) ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdiraxis1, eq, diraxis1) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 1 [X].") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdiraxis2, eq, diraxis2) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 2 [Y].") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kversion,  eq, version) ;
   strcpy  (&fitbuf [iy++][31], "/ MK3 VERSION NUMBER") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdate,     eq, cdate) ;
   strcpy  (&fitbuf [iy++][31],"/ DATE OF FITS FILE CREATION : DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", ktime,     eq, ctime) ;
   strcpy  (&fitbuf [iy++][31],"/ TIME OF FITS FILE CREATION : HH:MM:SS") ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdataform, eq, dform) ;
   sprintf (&fitbuf [iy++][31], "/ MK3 DATA FORM %s", dformcom) ;

   switch (MK3I->dtype)
      {
      case  1:			/*  rq     */
      case  2:			/* srq     */
      case  4:			/*  ri     */
      case  5:			/* sri     */
      case  7:			/*  ru     */
      case  8:			/* sru     */
      case 10:			/*  rpB    */
      case 11:			/* srpB    */
      case 13:			/*  rNe    */
      case 14:			/* srNe    */
      case 21:			/*   I     */
      case 22:			/*   U     */
      case 24:			/* avg rpB */
/*
 *               sprintf (&fitbuf [iy++][0],
 *			"%8s%2s%20.2f / DIR OF SOLAR NORTH W.R.T. TO +Y AXIS [CCW DEG]",
 *                        kangnorth, eq, angnorth) ;
 */

               break ;

      case  0:			/*  q    */
      case  3:			/*  i    */
      case  6:			/*  u    */
      case 15:			/* aba   */
      case 16:			/*  ba   */
      case 17:			/* ast   */
      case 18:			/*  st   */
      case 19:			/* calib */
      case 20:			/* tscp  */
      case 23:			/* avg pB  */
               sprintf (&fitbuf [iy++][0],
			"%8s%2s%20.2f / PHASE ANGLE [DEGREES]",
		        kphase, eq, MK3I->phase) ;

               sprintf (&fitbuf [iy++][0],
			"%8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
			ksolar_p0, eq, MK3I->pangle) ;
               break ;

      default:
               break ;
      }

   sprintf (&fitbuf [iy++][0], "%8s%2s%20d / MK3 CHANNEL. 0: RMIN=1.12 RSUN, 1: RMIN=1.40",
			      kchannel , eq, MK3I->channel) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4f / INNER FIELD-OF-VIEW LIMIT [RSUN]",
			      krmin    , eq, MK3I->rmin) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4f / OUTER FIELD-OF-VIEW LIMIT [RSUN]",
			      krmax    , eq, MK3I->rmax) ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SCAN DURATION [SECONDS]",
			      kscan_dur, eq, MK3I->scndur) ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'",     kscan_dir,   eq, scan_dir) ;

   strcpy  (&fitbuf [iy++][31], "/ SCAN DIRECTION: CW=CLOCKWISE, CCW=COUNTER-CW.") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / START BARREL ANGLE [DEGREES]",
		               ktmin, eq, MK3I->tmin) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / STOP  BARREL ANGLE [DEGREES]",
		               ktmax, eq, MK3I->tmax) ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdate_cal, eq, date_cal) ;
   strcpy  (&fitbuf [iy++][31],"/ DATE MK3 IMAGE CALIBRATED  : DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdate_opc, eq, date_opc) ;
   strcpy  (&fitbuf [iy++][31],"/ DATE OF MK3 OPAL CAL 0 IMG : DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'",     ktime_opc, eq, time_opc) ;
   strcpy  (&fitbuf [iy++][31],"/ TIME OF MK3 OPAL CAL 0 IMG : HH:MM:SS") ;

   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;
   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   THE MARK-3 K-CORONAMETER (MK3) OBSERVES THE LOWER CORONA [700-1080 NM]") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FROM %3.1f to %3.1f SOLAR RADII [RSUN], USING A 128-ELEMENT LINEAR RETICON", MK3I->rmin, MK3I->rmax) ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   DIODE ARRAY ALIGNED ALONG A RADIAL DIRECTION OUTWARD FROM THE CENTER") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   OF THE SUN.  THIS DETECTOR IS ROTATED OVER A PERIOD OF ABOUT 3") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   MINUTES TO ACCUMULATE A SET OF RADIAL RAYS (OR SCANS) WHICH ARE LATER") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   CONVERTED FROM POLAR TO CARTESIAN (I.E., RECTANGULAR) COORDINATES.") ;
   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;

   switch (MK3I->dtype)
      {
      case  1:			/*  rq     */
      case  2:			/* srq     */
      case  4:			/*  ri     */
      case  5:			/* sri     */
      case  7:			/*  ru     */
      case  8:			/* sru     */
      case 10:			/*  rpB    */
      case 11:			/* srpB    */
      case 21:			/*   I     */
      case 22:			/*   U     */
      case 24:			/* avg rpB */
              strcpy  (&fitbuf [iy++][0],  "COMMENT   FOR CARTESIAN COORDINATE IMAGES, PIXELS ARE DISPLAYED WITH THE ORIGIN") ;
              strcpy  (&fitbuf [iy++][0],  "COMMENT   AT THE LOWER LEFT CORNER; I.E., HORIZONTAL SCANS (LEFT TO RIGHT), WITH") ;
              strcpy  (&fitbuf [iy++][0],  "COMMENT   THE FIRST SCAN AT THE BOTTOM OF THE IMAGE, & THE LAST SCAN AT THE TOP.") ;
               strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   BRIGHTNESS MEASUREMENTS IN THE MARK-3 DATA ARE SPECIFIED IN UNITS") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   OF BSUN, WHICH IS A RATIO OF BRIGHTNESS OF THE CORONA TO THE MEAN") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   BRIGHTNESS OF THE SOLAR DISK.  TO CONVERT INTEGER PIXEL BRIGHTNESS") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   VALUES TO BSUN UNITS, MULTIPLY EACH PIXEL VALUE BY BSCALE & ADD BZERO.") ;
               strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
	       break ;
         
      default:
               break ;
      }

   switch (MK3I->dtype)
      {
      case  0:			/*   q   */
      case  3:			/*   i   */
      case  6:			/*   u   */
      case 12:			/*   Ne  */
               strcpy  (&fitbuf [iy++][0],  "COMMENT   THE BARREL ANGLES INDICATE THE ANGULAR POSITION IN DEGREES") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   (CCW W.R.T. GEOCENTRIC NORTH) OF EACH RADIAL RAY (128 DIODE SAMPLES).") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   TO CONVERT FROM GEOCENTRIC TO HELIOCENTRIC COORDINATES,") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   SUBTRACT PHASE ANGLE AND SUBTRACT P-ANGLE.") ;
               strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
	       break ;
         
      default:
               break ;
      }
   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;
   swrite += nelwr ;
   if (pmode)
      fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (imgbuf, MK3I->nbypx, npximg, FF) ;
   swrite += nelwr ;
   if (pmode)
      fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;
   free (mk3buf) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      if (pmode)
         fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n",
	    nrec, npad, nbyimg) ;
      fflush (stdout) ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FF) ;
      swrite += nelwr ;
      if (pmode)
         fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;
      free (padbuf) ;
      }
   
   if (pmode)
      {
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg) ;
      printf ("End of mk3tofits\n") ;
      fflush (stdout) ;
      }
   close (FF) ;
   return (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
