/* sun_pos.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C-language version adapted from IDL code: $SSW/gen/idl/solar/sun_pos.pro
 * Andrew L. Stanger   HAO/NCAR   25 March 1999
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Project     : SOHO - CDS     
 *                   
 * Name        : SUN_POS
 *               
 * Purpose     : Calculate solar ephemeris parameters.
 *               
 * Explanation : Allows for planetary and lunar perturbations in the calculation
 *               of solar longitude at date and various other solar positional
 *               parameters.
 *               
 * Use         : sun_pos (date, longitude, ra, dec, app_long, obliq) ;
 *    
 * Inputs      : date - fractional number of days since JD 2415020.0 
 *               
 * Opt. Inputs : None
 *               
 * Outputs     : longitude  -  Longitude of sun for mean equinox of date (degs)
 *               ra         -  Apparent RA for true equinox of date (degs)
 *               dec        -  Apparent declination for true equinox of date (degs)
 *               app_long   -  Apparent longitude (degs)
 *               obliq      -  True obliquity (degs)
 *               
 * Opt. Outputs: All above
 *               
 * Keywords    : None
 *
 * Calls       : None
 *
 * Common      : None
 *               
 * Restrictions: None
 *               
 * Side effects: None
 *               
 * Category    : Util, coords
 *               
 * Prev. Hist. : From Fortran routine by B Emerson (RGO).
 *
 * Written     : CDS/IDL version by C D Pike, RAL, 17-May-94
 *               
 * Modified    : 
 *
 * Version     : Version 1, 17-May-94
 *		 C-language version: Andrew L. Stanger, HAO/NCAR, 26 March 1999.
 */

/* pro  sun_pos, dd, longmed, ra, dec, l, oblt */

#include <math.h>

int
sun_pos (jd, longmed, ra, dec, app_long, obliq)

double jd ;
double *longmed ;
double *ra ;
double *dec ;
double *app_long ;
double *obliq ;

{
   extern double moduloc () ;
   double degrad ;
   double raddeg ;
   static double pi = 3.14159265 ;

   double d ;
   double ellcor ;
   double jupcorr ;
   double l ;
   double longterm ;
   double me ;
   double mj ;
   double mm ;
   double mv ;
   double marscorr ;
   double mooncorr ;
   double omega ;
   double vencorr ;
   double t ;
   double p1, p2, p3, p4, p5, p6, p7, p8 ;

   degrad = 180.0 / pi ;
   raddeg = pi / 180.0 ;

   printf ("--> Start of sun_pos.\n") ;
   printf ("jd: %8.3f\n", jd) ;
   /*
    *  This routine is a truncated version of Newcomb's Sun and
    *  is designed to give apparent angular coordinates (T.E.D) to a
    *  precision of one second of time.
    */

   /* Form time in Julian centuries from 1900.0 */

   t = jd / 36525.0L ;
   printf ("t: %8.3f\n", t) ;

   /*  Form sun's mean longitude. */

   l = (279.696678L + moduloc ((36000.768925L * t), 360.0L)) * 3600.0L ;
   p1 = t ;
   p2 = 279.696678L ;
   p3 = 36000.768925L * t ;
   p4 = 360.0L ;
   p5 = moduloc (p3, p4) ;
   p6 = 3600.0L ;
   p7 = (p2 + p5) * p6 ;
   printf ("p1: %8.3f\n", p1) ;
   printf ("p2: %8.3f\n", p2) ;
   printf ("p3: %8.3f\n", p3) ;
   printf ("p4: %8.3f\n", p4) ;
   printf ("p5: %8.3f\n", p5) ;
   printf ("p6: %8.3f\n", p6) ;
   printf ("p7: %8.3f\n", p7) ;
   printf ("l: %8.3f\n", l) ;

   /*  Allow for ellipticity of the orbit (equation of centre)
    *  using the Earth's mean anomoly ME.
    */

   me = 358.475844L + moduloc ((35999.049750L * t), 360.0L) ;
   ellcor  = (6910.1L - 17.2L * t) * sin (me * raddeg) +
	      72.3L * sin (2.0L * me * raddeg) ;
   l = l + ellcor ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for the Venus perturbations using the mean anomaly of Venus MV. */

   mv = 212.603219L + moduloc ((58517.803875L * t), 360.0L) ;
   vencorr = 4.8L * cos ((299.1017L + mv - me) * raddeg) +
             5.5L * cos ((148.3133L +  2.0L * mv  -  2.0L * me ) * raddeg) +
             2.5L * cos ((315.9433L +  2.0L * mv  -  3.0L * me ) * raddeg) +
             1.6L * cos ((345.2533L +  3.0L * mv  -  4.0L * me ) * raddeg) +
             1.0L * cos ((318.15L   +  3.0L * mv  -  5.0L * me ) * raddeg) ;
   l = l + vencorr ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for the Mars perturbations using the mean anomaly of Mars MM. */

   mm = 319.529425L + moduloc (( 19139.858500L * t), 360.0L ) ;
   marscorr = 2.0L * cos ((343.8883L - 2.0L * mm + 2.0L * me) * raddeg ) +
              1.8L * cos ((200.4017L - 2.0L * mm  + me) * raddeg) ;
   l = l + marscorr ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for the Jupiter perturbations
    * using the mean anomaly of Jupiter mj.
    */

   mj = 225.328328L + moduloc (( 3034.6920239L * t),  360.0L ) ;
   jupcorr = 7.2L * cos (( 179.5317L - mj + me ) * raddeg) +
             2.6L * cos ((263.2167L  - mj ) * raddeg) +
             2.7L * cos (( 87.1450L  - 2.0L * mj  +  2.0L * me ) * raddeg) +
             1.6L * cos ((109.4933L  - 2.0L * mj  +  me ) * raddeg) ;
   l = l + jupcorr ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for the Moons perturbations using the mean elongation of
    * the Moon from the Sun D.
    */

   d = 350.7376814L + moduloc (( 445267.11422L * t), 360.0L ) ;
   mooncorr  = 6.5L * sin (d * raddeg) ;
   l = l + mooncorr ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for long period terms. */

   longterm  = + 6.4L * sin (( 231.19L + 20.20L * t ) * raddeg) ;
   l  =    l + longterm ;
   l  =  moduloc ((l + 2592000.0L), 1296000.0L) ;
   *longmed = l / 3600.0L ;
   printf ("longterm: %8.3e\n", longterm) ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for Aberration. */

   l  =  l - 20.5L ;
   printf ("l: %8.3f\n", l) ;


   /* Allow for Nutation using the longitude of the Moons mean node OMEGA. */

   omega = 259.183275L - moduloc (( 1934.142008L * t ), 360.0L ) ;
   l  =  l - 17.2L * sin (omega * raddeg) ;
   printf ("omega: %8.3f\n", omega) ;
   printf ("l: %8.3f\n", l) ;


   /* Form the True Obliquity. */

   *obliq  = 23.452294L - 0.0130125L * t +
	     (9.2L * cos (omega * raddeg)) / 3600.0L ;


   /* Form Right Ascension and Declination. */

   *app_long = l / 3600.0L ;
   l = l / 3600.0L ;
   *ra  = atan2 ( sin (l * raddeg) * cos (*obliq * raddeg),
		cos (l * raddeg) ) * degrad ;
   printf ("l: %8.3f\n", l) ;

   if (*ra < 0.0L)  *ra = *ra + 360.0L ;

   *dec = asin (sin (l * raddeg) * sin (*obliq * raddeg)) * degrad ;

   printf ("obliq    : %8.3f\n", *obliq) ;
   printf ("app_long : %8.3f\n", *app_long) ;
   printf ("ra       : %8.3f\n", *ra) ;
   printf ("dec      : %8.3f\n", *dec) ;
   printf ("<-- End of sun_pos.\n") ;

   return (0) ;
}
