;+ fts24.pro	IDL procedure
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; NAME:		fts24
; PURPOSE:	Combine 2 or 3 FITS images into a 24-bit composite image.
; SYNTAX:	fts24, rfile='file1.fts', gfile='file2.fts', bfile='file3.fts'
;       	fts24, rfile='file1.fts', gfile='file2.fts', bfile='file3.fts',$
;                      outfile='file123.ras'
; REQUIREMENTS:	Images must have sun center and solar radius size defined in
;               the FITS header.
; AUTHOR:	Andrew L. Stanger   HAO/NCAR   7 May 1996
; HISTORY:	
;	10 Jun 1996: [ALS] remove 'fts' extension from rasterfile output names.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-
PRO fts24, RFILE=rfile, GFILE=gfile, BFILE=bfile, OUTFILE=outfile

;--- Initialize variables.

rxdim = 0   &   gxdim = 0   &   bxdim = 0	; X-dimension.
rydim = 0   &   gydim = 0   &   bydim = 0	; Y-dimension.
rsrad = 0   &   gsrad = 0   &   bsrad = 0	; Solar radius (pixels).
rxcen = 0   &   gxcen = 0   &   bxcen = 0	; X Sun center.
rycen = 0   &   gycen = 0   &   bycen = 0	; Y Sun center.
disptop = 249					; Maximum display level.

DEVICE, GET_SCREEN_SIZE = scrsiz		; Get screen size.
sxdim = scrsiz (0)
sydim = scrsiz (1)

;--- Read FITS images.
;--- If a color channel file is not specified, create a black image.

IF (N_ELEMENTS (rfile) EQ 0) THEN	$
BEGIN
   rfile = 'r.fts'
   rf = 0
   rimg = bytarr (512, 512)
   rimg (*, *) = 0
   fxhmake, rhdu, rimg
   fxaddpar, rhdu, 'CRPIX1', 255.5
   fxaddpar, rhdu, 'CRPIX2', 255.5
   fxaddpar, rhdu, 'CRRADIUS', 256.0
;   fxaddpar, rhdu, 'BSCALE,', 1.0
;   fxaddpar, rhdu, 'BZERO', 0.0
END					$
ELSE					$
BEGIN
   rf = 1
   print, 'Reading FITS file: ', rfile
   rimg = readfits (rfile, rhdu)
END

;--- Get image dimensions.

rxdim   = fxpar (rhdu, 'NAXIS1')			; Image x dimension
rydim   = fxpar (rhdu, 'NAXIS2')			; Image y dimension
rbitpix = fxpar (rhdu, 'BITPIX')
window, xsize = rxdim, ysize = rydim
tvscl, rimg

IF (N_ELEMENTS (gfile) EQ 0) THEN	$
BEGIN
   gfile = 'g.fts'
   gf = 0
   gimg = bytarr (512, 512)
   gimg (*, *) = 0
   fxhmake, ghdu, gimg
   fxaddpar, ghdu, 'CRPIX1', 255.5
   fxaddpar, ghdu, 'CRPIX2', 255.5
   fxaddpar, ghdu, 'CRRADIUS', 256.0
;   fxaddpar, ghdu, 'BSCALE,', 1.0
;   fxaddpar, ghdu, 'BZERO', 0.0
END					$
ELSE					$
BEGIN
   gf = 1
   print, 'Reading FITS file: ', gfile
   gimg = readfits (gfile, ghdu)
END

;--- Get image dimensions.

gxdim   = fxpar (ghdu, 'NAXIS1')			; Image x dimension
gydim   = fxpar (ghdu, 'NAXIS2')			; Image y dimension
gbitpix = fxpar (ghdu, 'BITPIX')
window, xsize = gxdim, ysize = gydim
tvscl, gimg

IF (N_ELEMENTS (bfile) EQ 0) THEN	$
BEGIN
   bfile = 'b.fts'
   bf = 0
   bimg = bytarr (512, 512)
   bimg (*, *) = 0
   fxhmake, bhdu, bimg
   fxaddpar, bhdu, 'CRPIX1', 255.5
   fxaddpar, bhdu, 'CRPIX2', 255.5
   fxaddpar, bhdu, 'CRRADIUS', 256.0
;   fxaddpar, bhdu, 'BSCALE,', 1.0
;   fxaddpar, bhdu, 'BZERO', 0.0
END					$
ELSE					$
BEGIN
   bf = 1
   print, 'Reading FITS file: ', bfile
   bimg = readfits (bfile, bhdu)
END

;--- Get image dimensions.

bxdim   = fxpar (bhdu, 'NAXIS1')			; Image x dimension
bydim   = fxpar (bhdu, 'NAXIS2')			; Image y dimension
bbitpix = fxpar (bhdu, 'BITPIX')
window, xsize = bxdim, ysize = bydim
tvscl, bimg

;--- Get selected parameters from FITS headers.

rdate = fxpar (rhdu, 'DATE-OBS')
;IF (!err EQ -1) THEN rdate=''
gdate = fxpar (ghdu, 'DATE-OBS')
;IF (!err EQ -1) THEN gdate=''
bdate = fxpar (bhdu, 'DATE-OBS')
;IF (!err EQ -1) THEN bdate=''

rtime = fxpar (rhdu, 'TIME-OBS')
;IF (!err EQ -1) THEN rtime=''
gtime = fxpar (ghdu, 'TIME-OBS')
;IF (!err EQ -1) THEN gtime=''
btime = fxpar (bhdu, 'TIME-OBS')
;IF (!err EQ -1) THEN btime=''

rbscale = fxpar (rhdu, 'BSCALE')
IF (!err EQ -1) THEN rbscale = 1.0
gbscale = fxpar (ghdu, 'BSCALE')
IF (!err EQ -1) THEN gbscale = 1.0
bbscale = fxpar (bhdu, 'BSCALE')
IF (!err EQ -1) THEN bbscale = 1.0

rbzero  = fxpar (rhdu, 'BBZERO')
IF (!err EQ -1) THEN rbzero = 0.0
gbzero  = fxpar (ghdu, 'BBZERO')
IF (!err EQ -1) THEN gbzero = 0.0
bbzero  = fxpar (bhdu, 'BBZERO')
IF (!err EQ -1) THEN bbzero = 0.0

rdispmin  = fxpar (rhdu, 'BDISPMIN')
IF (!err EQ -1) THEN rdispmin = MIN (rimg)
gdispmin  = fxpar (ghdu, 'BDISPMIN')
IF (!err EQ -1) THEN gdispmin = MIN (gimg)
bdispmin  = fxpar (bhdu, 'BDISPMIN')
IF (!err EQ -1) THEN bdispmin = MIN (bimg)

rdispmax  = fxpar (rhdu, 'BDISPMAX')
IF (!err EQ -1) THEN rdispmax = MAX (rimg)
gdispmax  = fxpar (ghdu, 'BDISPMAX')
IF (!err EQ -1) THEN gdispmax = MAX (gimg)
bdispmax  = fxpar (bhdu, 'BDISPMAX')
IF (!err EQ -1) THEN bdispmax = MAX (bimg)

;--- Get solar radius.

rsrad = fxpar (rhdu, 'CRRADIUS')		; Image solar radius (pixels).
rsrad_err = !err
IF (rsrad_err EQ -1) THEN		$
BEGIN
   rsrad = fxpar (rhdu, 'RADIUS')		; Image solar radius (pixels).
   rsrad_err = !err
END

gsrad = fxpar (ghdu, 'CRRADIUS')		; Image solar radius (pixels).
gsrad_err = !err
IF (gsrad_err EQ -1) THEN		$
BEGIN
   gsrad = fxpar (ghdu, 'RADIUS')		; Image solar radius (pixels).
   gsrad_err = !err
END

bsrad = fxpar (bhdu, 'CRRADIUS')		; Image solar radius (pixels).
bsrad_err = !err
IF (bsrad_err EQ -1) THEN		$
BEGIN
   bsrad = fxpar (bhdu, 'RADIUS')		; Image solar radius (pixels).
   bsrad_err = !err
END

;--- Get sun center.

rxcen = fxpar (rhdu, 'CRPIX1')			; Image x axis sun center.
rxcen_err = !err
gxcen = fxpar (ghdu, 'CRPIX1')
gxcen_err = !err
bxcen = fxpar (bhdu, 'CRPIX1')
bxcen_err = !err

rycen = fxpar (rhdu, 'CRPIX2')			; Image y axis sun center.
rycen_err = !err
gycen = fxpar (ghdu, 'CRPIX2')
gycen_err = !err
bycen = fxpar (bhdu, 'CRPIX2')
bycen_err = !err

print, 'rimg: xdim,ydim,srad,xcen,ycen: ', rxdim, rydim, rsrad, rxcen, rycen
print, 'gimg: xdim,ydim,srad,xcen,ycen: ', gxdim, gydim, gsrad, gxcen, gycen
print, 'bimg: xdim,ydim,srad,xcen,ycen: ', bxdim, bydim, bsrad, bxcen, bycen

rmin = rdispmin * rbscale + rbzero
gmin = gdispmin * gbscale + gbzero
bmin = bdispmin * bbscale + bbzero

rmax = rdispmax * rbscale + rbzero
gmax = gdispmax * gbscale + gbzero
bmax = bdispmax * bbscale + bbzero

print, 'rbscale,rbzero: ', rbscale, rbzero
print, 'gbscale,gbzero: ', gbscale, gbzero
print, 'bbscale,bbzero: ', bbscale, bbzero

print, 'rdispmin,rdispmax: ', rdispmin, rdispmax
print, 'gdispmin,gdispmax: ', gdispmin, gdispmax
print, 'bdispmin,bdispmax: ', bdispmin, bdispmax

print, 'rmin,rmax: ', rmin, rmax
print, 'gmin,gmax: ', gmin, gmax
print, 'bmin,bmax: ', bmin, bmax

;--- Scale images to byte arrays.

IF (rbitpix NE 8) THEN rimg = bytscl (rimg, MIN=rmin, MAX=rmax, TOP=disptop)
IF (gbitpix NE 8) THEN gimg = bytscl (gimg, MIN=gmin, MAX=gmax, TOP=disptop)
IF (bbitpix NE 8) THEN bimg = bytscl (bimg, MIN=bmin, MAX=bmax, TOP=disptop)

;--- If sun center & solar radius is not defined, locate sun center & limb.

IF (rsrad_err EQ -1 OR rxcen_err EQ -1 OR rycen_err EQ -1) THEN	$
BEGIN
   print, '*** Calling gen_loc : ', rfile
   window, xsize = rxdim, ysize = rydim
   gen_loc, rimg, CENTROID=rcen, HORZRAD=rdiam, /CIR
   rxcen = rcen (0)
   rycen = rcen (1)
   rsrad = rdiam / 2.0
END

IF (gsrad_err EQ -1 OR gxcen_err EQ -1 OR gycen_err EQ -1) THEN	$
BEGIN
   print, '*** Calling gen_loc : ', gfile
   window, xsize = gxdim, ysize = gydim
   gen_loc, gimg, CENTROID=gcen, HORZRAD=gdiam, /CIR
   gxcen = gcen (0)
   gycen = gcen (1)
   gsrad = gdiam / 2.0
END

IF (bsrad_err EQ -1 OR bxcen_err EQ -1 OR bycen_err EQ -1) THEN	$
BEGIN
   print, '*** Calling gen_loc : ', bfile
   window, xsize = bxdim, ysize = bydim
   gen_loc, bimg, CENTROID=bcen, HORZRAD=bdiam, /CIR
   bxcen = bcen (0)
   bycen = bcen (1)
   bsrad = bdiam / 2.0
END

;--- Determine maximum solar radius.

radmax = 0.0
IF (rsrad GT radmax) THEN radmax = rsrad
IF (gsrad GT radmax) THEN radmax = gsrad
IF (bsrad GT radmax) THEN radmax = bsrad

;--- Determine maximum Field-Of-View (FOV).

xfovmax = 0.0
yfovmax = 0.0

;--- Number of pixels left, right, top, bottom from sun center.

rxlcen = rxcen
rxrcen = rxdim - rxcen - 1
gxlcen = gxcen
gxrcen = gxdim - gxcen - 1
bxlcen = bxcen
bxrcen = bxdim - bxcen - 1

rylcen = rycen
ryrcen = rydim - rycen - 1
gylcen = gycen
gyrcen = gydim - gycen - 1
bylcen = bycen
byrcen = bydim - bycen - 1

;--- Compute left, right, top, bottom fields of view.

rxlfov = rxlcen / rsrad
rxrfov = rxrcen / rsrad
gxlfov = gxlcen / gsrad
gxrfov = gxrcen / gsrad
bxlfov = bxlcen / bsrad
bxrfov = bxrcen / bsrad

rylfov = rylcen / rsrad
ryrfov = ryrcen / rsrad
gylfov = gylcen / gsrad
gyrfov = gyrcen / gsrad
bylfov = bylcen / bsrad
byrfov = byrcen / bsrad

IF (rxlfov GT xfovmax) THEN	$
BEGIN
   xfovmax = rxlfov
   radmax  = rsrad
END

IF (gxlfov GT xfovmax) THEN	$
BEGIN
   xfovmax = gxlfov
   radmax  = gsrad
END

IF (bxlfov GT xfovmax) THEN	$
BEGIN
   xfovmax = bxlfov
   radmax  = gsrad
END

IF (rxrfov GT xfovmax) THEN	$
BEGIN
   xfovmax = rxrfov
   radmax  = rsrad
END

IF (gxrfov GT xfovmax) THEN	$
BEGIN
   xfovmax = gxrfov
   radmax  = gsrad
END

IF (bxrfov GT xfovmax) THEN	$
BEGIN
   xfovmax = bxrfov
   radmax  = gsrad
END

IF (rylfov GT yfovmax) THEN	$
BEGIN
   yfovmax = rylfov
   radmax  = rsrad
END

IF (gylfov GT yfovmax) THEN	$
BEGIN
   yfovmax = gylfov
   radmax  = gsrad
END

IF (bylfov GT yfovmax) THEN	$
BEGIN
   yfovmax = bylfov
   radmax  = gsrad
END

IF (ryrfov GT yfovmax) THEN	$
BEGIN
   yfovmax = ryrfov
   radmax  = rsrad
END

IF (gyrfov GT yfovmax) THEN	$
BEGIN
   yfovmax = gyrfov
   radmax  = gsrad
END

IF (byrfov GT yfovmax) THEN	$
BEGIN
   yfovmax = byrfov
   radmax  = gsrad
END

IF (xfovmax GT yfovmax) THEN fovmax = xfovmax ELSE fovmax = yfovmax

print, 'rxdim,gxdim,bxdim :     ', rxdim, gxdim, bxdim
print, 'rydim,gydim,bydim :     ', rydim, gydim, bydim
print, 'rsrad,gsrad,bsrad :     ', rsrad, gsrad, bsrad
print, 'rxlfov,gxlfov,bxlfov :  ', rxlfov, gxlfov, bxlfov
print, 'rxrfov,gxrfov,bxrfov :  ', rxrfov, gxrfov, bxrfov
print, 'rylfov,gylfov,bylfov :  ', rylfov, gylfov, bylfov
print, 'ryrfov,gyrfov,byrfov :  ', ryrfov, gyrfov, byrfov
print, 'radmax:                 ', radmax
print, 'xfovmax,yfovmax,fovmax: ', xfovmax, yfovmax, fovmax

;--- Set output dimensions and solar radius size.

oxdim  = fix (fovmax * radmax) * 2
oydim  = fix (fovmax * radmax) * 2
osrad  = radmax
oxcen  = fix (oxdim / 2.0 - 0.5)
oycen  = fix (oydim / 2.0 - 0.5)

print, 'oxdim,oydim: ', oxdim, oydim
print, 'oxcen,oycen,osrad: ', oxcen, oycen, osrad

IF (oxdim LE sxdim) THEN wxdim = oxdim ELSE wxdim = sxdim - 20
IF (oydim LE sydim) THEN wydim = oydim ELSE wydim = sydim - 20

;--- Resize images to be sun-centered & registered.

orimg = imgfmt (rimg,	$
		ICENTER=[rxcen,rycen], IRADIUS=rsrad, ISIZE=[rxdim,rydim], $
		OCENTER=[oxcen,oycen], ORADIUS=osrad, OSIZE=[oxdim,oydim])

print, 'size (orimg) : ', size (orimg)
window, xsize = wxdim, ysize = wydim
IF (rf) THEN	$
   tv, orimg

ogimg = imgfmt (gimg,	$
		ICENTER=[gxcen,gycen], IRADIUS=gsrad, ISIZE=[gxdim,gydim], $
		OCENTER=[oxcen,oycen], ORADIUS=osrad, OSIZE=[oxdim,oydim])

print, 'size (ogimg) : ', size (ogimg)
IF (gf) THEN	$
   tv, ogimg

obimg = imgfmt (bimg,	$
		ICENTER=[bxcen,bycen], IRADIUS=bsrad, ISIZE=[bxdim,bydim], $
		OCENTER=[oxcen,oycen], ORADIUS=osrad, OSIZE=[oxdim,oydim])

print, 'size (obimg) : ', size (obimg)
IF (bf) THEN	$
   tv, obimg

; --- Create 24-bit color image: 8 bit image for each channel: red, green, blue.
;
; --- Note: For standard 24-bit Sun Rasterfile,
;     color channel order is blue, green, red.

bgr_img = bytarr (3, oxdim, oydim)

bgr_img (0, *, *) = reverse (obimg, 2)              ; blue channel.

bgr_img (1, *, *) = reverse (ogimg, 2)              ; green channel.

bgr_img (2, *, *) = reverse (orimg, 2)              ; red channel.

; --- Create RGB colormap (equal brightness for all 3 colors).

map_red   = bindgen (256)

map_green = bindgen (256)

map_blue  = bindgen (256)

map_black = bindgen (256)
;map_black = bytarr (256)
;map_black (*,*) = 0

;--- Write each color channel image to disk as a Sun rasterfile.

;routfile = rfile + '.r.ras'
;goutfile = gfile + '.g.ras'
;boutfile = bfile + '.b.ras'

extloc = strpos (rfile, '.fts')
IF (extloc EQ -1) THEN extloc = strpos (rfile, '.fits')
IF (extloc NE -1) THEN				$
   out_file = strmid (rfile, 0, extloc)		$
ELSE						$
   out_file = rfile
routfile = out_file + '.r.ras'

extloc = strpos (gfile, '.fts')
IF (extloc EQ -1) THEN extloc = strpos (gfile, '.fits')
IF (extloc NE -1) THEN				$
   out_file = strmid (gfile, 0, extloc)		$
ELSE						$
   out_file = gfile
goutfile = out_file + '.g.ras'

extloc = strpos (bfile, '.fts')
IF (extloc EQ -1) THEN extloc = strpos (bfile, '.fits')
IF (extloc NE -1) THEN				$
   out_file = strmid (bfile, 0, extloc)		$
ELSE						$
   out_file = bfile
boutfile = out_file + '.b.ras'

IF (rf) THEN	$
BEGIN
   print, 'Writing Sun Rasterfile : ', routfile
   write_ras, routfile, reverse (orimg, 2), map_red,   map_black, map_black
END

IF (gf) THEN	$
BEGIN
   print, 'Writing Sun Rasterfile : ', goutfile
   write_ras, goutfile, reverse (ogimg, 2), map_black, map_green, map_black
END

IF (bf) THEN	$
BEGIN
   print, 'Writing Sun Rasterfile : ', boutfile
   write_ras, boutfile, reverse (obimg, 2), map_black, map_black, map_blue
END

;--- Output file name.

;outfile = 'fts24.sun'

IF (N_ELEMENTS (outfile) EQ 0) THEN	$
BEGIN

   print, 'n_elements (rdate): ', n_elements (rdate)
   print, 'size (rdate) : ', size (rdate)
   print, 'n_elements (gdate): ', n_elements (gdate)
   print, 'size (gdate) : ', size (gdate)
   print, 'n_elements (bdate): ', n_elements (bdate)
   print, 'size (bdate) : ', size (bdate)

   obsdate = ''
   obstime = ''
   obsdate = '00-00-00'
   IF (rdate NE 0) THEN obsdate = rdate
   IF (gdate NE 0) THEN obsdate = gdate
   IF (bdate NE 0) THEN obsdate = bdate

   iday   = fix (strmid (obsdate, 0, 2))
   imonth = fix (strmid (obsdate, 3, 2)) - 1
   iyear  = fix (strmid (obsdate, 6, 2))
   smonth = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 	$
             'jul', 'aug', 'sep', 'oct', 'nov', 'dec']
   outdate = string (iday, FORMAT = '(I2.2)') + '-' +	$
             smonth (imonth) + '-' + string (iyear, FORMAT = '(i2.2)')

   IF (rtime NE 0) THEN routtime = strmid (rtime, 0, 5)	$
   ELSE							$
   routtime = 'r'

   IF (gtime NE 0) THEN gouttime = strmid (gtime, 0, 5)	$
   ELSE							$
   gouttime = 'g'

   IF (btime NE 0) THEN bouttime = strmid (btime, 0, 5)	$
   ELSE							$
   bouttime = 'b'

   print, 'rdate: ', rdate
   print, 'gdate: ', gdate
   print, 'bdate: ', bdate
   print, 'outdate: ', outdate

   print, 'rtime: ', rtime
   print, 'gtime: ', gtime
   print, 'btime: ', btime
   print, 'routtime: ', routtime
   print, 'gouttime: ', gouttime
   print, 'bouttime: ', bouttime

   outfile = 'rgb' + '_' + outdate + '_' +	$
	     routtime + '_' + gouttime + '_' + bouttime + '.ras'

END
 
;   outfile = 'fts24.ras'

print, 'Writing RGB Sun Rasterfile : ', outfile

; --- Write image to disk as a 24-bit (BGR) Sun rasterfile.

write_ras, outfile, bgr_img, map_red, map_green, map_blue

END
