/* hs.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Histogram Specification Scaling.
 * From Winter 1997 CCD Astronomy article,
 * "An Optimized Stretch for Deep-Sky Images", * by Blaise Canzain.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 April 1997
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

unsigned char *
hs (imgbuf, xdim, ydim, nbypx, pmode)

unsigned char *imgbuf ;			/* Image array. */
int xdim, ydim ;			/* Image dimensions. */
int nbypx ;				/* # bytes per pixel. */
int pmode ;				/* Print mode. */
{
   unsigned char *ibuf ;
   unsigned char *cbuf ;
   unsigned char *gbuf ;
   short *sbuf ;
   float *fbuf ;

   unsigned char cmin, cmax ;
   short smin, smax ;
   int imin, imax ;

   static float pixel [256] ;
   float slope, offset ;
   float fmin, fmax ;
   float fmincum, fmaxcum ;
   float fmininv, fmaxinv ;
   float num ;

   static int   igauss [256] ;		/* Gaussian array (int). */
   static int   igausscuminv [256] ;	/* Gaussian cum. inverse (int). */

   static float gauss [256] ;		/* Gaussian array (float). */
   static float gausscum [256];		/* Gaussian cumulative integral. */
   static float gausscuminv [256] ;	/* Gaussian cum. inverse (float). */
   static float gausscumref [256] ;	/* Gaussian cum. reflect (float). */
   static float opix [256] ;		/* Output Pixel Values (float). */

   float gbeg, gend ;
   float gdiag ;
   float gdif ;
   float gsum ;				/* Gaussian  sum. */
   int hist [256] ;			/* Histogram array. */
   int histcum [256];			/* Histogram cumulative integral. */
   int hsum ;				/* Histogram sum. */
   int i ;
   int j ;
   int ibeg, iend ;
   int nz ;
   int ival ;
   int nelem ;				/* # pixels in image. */
   int nlevels = 256;			/* # Histogram Levels. */

   float ndif ;
   float nzero = 127.5 ;
   float s     = 256.0 / 6.0 ;

   nelem = xdim * ydim ;

/*   if (pmode) */
      fprintf (stdout, "hs> xdim/ydim/nbypx/nelem: %3d %3d %3d %6d\n",
	       xdim, ydim, nbypx, nelem) ;
   fprintf (stdout, "hs>s: %f\n", s) ;
   fflush (stdout) ;

   ibuf = (unsigned char *) malloc (nelem) ;
   gbuf = (unsigned char *) malloc (nelem) ;

   /*** Convert input array into a byte array. ***/

   switch (nbypx)
      {

      /*** Byte Array. ***/

      case 1: cbuf = (unsigned char *) imgbuf ;
	 for (i = 0; i < nelem; i++)
	    *(ibuf + i) = *(cbuf + i) ;
	 break ;

      /*** Short Int Array. ***/

      case 2: sbuf = (short *) imgbuf ;
	 for (i = 0; i < nelem; i++)
	    {
            smin =  32767 ;
            smax = -32767 ;
	    for (i = 0; i < nelem; i++)
               {
               if (*(sbuf + i) < smin)  smin = *(sbuf + i) ;
               if (*(sbuf + i) > smax)  smax = *(sbuf + i) ;
               }
	    }

         fprintf (stdout, "hs>sbuf>smin/smax: %d %d\n", smin, smax) ;

	 if (smax > smin)
            slope = 256.0 / (smax - smin) ;
         else
	    slope = 1.0 ;

         cmin = 255 ;
	 cmax =   0 ;
	 for (i = 0; i < nelem; i++)
	    {
	    *(ibuf + i) = (unsigned char) ( (*(sbuf + i) - smin) * slope ) ;
	    if (*(ibuf + i) < cmin)  cmin = *(ibuf + i) ;
	    if (*(ibuf + i) > cmax)  cmax = *(ibuf + i) ;
	    }
         fprintf (stdout, "hs>ibuf>cmin/cmax: %3d %3d\n", cmin, cmax) ;
         break ;

      /*** Floating Point Array. ***/

      case 4: fbuf = (float *) imgbuf ;
	 fmin =  100000.0 ;
	 fmax = -100000.0 ;
	 for (i = 0; i < nelem; i++)
	    {
	    if (*(fbuf + i) < fmin)  fmin = *(fbuf + i) ;
	    if (*(fbuf + i) > fmax)  fmax = *(fbuf + i) ;
	    }
         slope = 256.0 / (fmax - fmin) ;
	 for (i = 0; i < nelem; i++)
	    {
	    *(ibuf + i) = (unsigned char) ( (*(fbuf + i) - fmin) * slope ) ;
	    }
      }
   
   /*** Clear Arrays to Zero. ***/

   for (i = 0; i < nlevels; i++)
      {
      pixel [i] = (float) i ;
      hist [i]  = 0 ;
      histcum [i]  = 0 ;

      igauss [i] = 0 ;
      gauss [i] = 0.0 ;

      gausscum [i] = 0.0 ;

      gausscuminv [i] = 0.0 ;
      gausscumref [i] = 0.0 ;
      igausscuminv [i] = 0 ;
      }

   /*** Compute Historgram. ***/

   for (i = 0; i < nelem; i++)
      {
      hist [(int) *(ibuf + i)] += 1 ;
      }

   imin =  nelem ;
   imax = -nelem ;
   for (i = 0; i < nlevels; i++)
      {
      if (hist [i] < imin)  imin = hist [i] ;
      if (hist [i] > imax)  imax = hist [i] ;
/*      fprintf (stdout, "i: %3d  hist: %6d\n", i, hist [i]) ; */
      }

   fprintf (stdout, "hs>hist>imin/imax: %3d %3d\n", imin, imax) ;

   /*** Compute Cumulative Histogram Function. ***/

   hsum = 0 ;
   for (i = 0 ; i < nlevels; i++)
      {
      hsum += hist [i] ;		/* Histogram Integral. */
      histcum [i] = hsum ;		/* Cumulative Histogram Integral. */
      }

   imin =  nelem ;
   imax = -nelem ;
   for (i = 0; i < nlevels; i++)
      {
      if (histcum [i] < imin)  imin = histcum [i] ;
      if (histcum [i] > imax)  imax = histcum [i] ;
      }

   fprintf (stdout, "hs>histcum>imin/imax: %3d %3d\n", imin, imax) ;

   /*** Normalize Cumulative Histogram. ***/

   for (i = 0; i < nlevels; i++)
      {
      histcum [i] /= (nelem / 255) ;
      }

   imin =  nelem ;
   imax = -nelem ;
   for (i = 0; i < nlevels; i++)
      {
      if (histcum [i] < imin)  imin = histcum [i] ;
      if (histcum [i] > imax)  imax = histcum [i] ;
      }

   fprintf (stdout, "hs>histcum.norm>imin/imax: %3d %3d\n", imin, imax) ;

   /*** Compute Gaussian Function. ***/

   for (i = 0 ; i < nlevels; i++)
      {
      ival = i ;
      ndif = (float) ival - nzero ;
      num =  -(ndif * ndif) / (2 * s * s) ;
      gauss [ival] = (float) exp ((double) num) ;
/*
      fprintf (stdout, "i: %6d  ndif: %10.4f  num: %10.4f  gauss: %10.4f\n",
	       i, ndif, num, gauss [ival]) ;
 */
      }

   fmin =  100000.0 ;
   fmax = -100000.0 ;
   for (i = 0; i < nlevels; i++)
      {
      if (gauss [i] < fmin)  fmin = gauss [i] ;
      if (gauss [i] > fmax)  fmax = gauss [i] ;
      }

   fprintf (stdout, "hs>gauss>fmin/fmax: %10.4f %10.4f\n", fmin, fmax) ;

   xyplot (pixel, gauss, nlevels, "fitw3.cgm", "Gaussian Function",
	   "Pixel Value", "Gaussian Function g", 0, 0, 0) ;

   /*** Compute Cumulative Gaussian Integral. ***/

   gsum = 0.0 ;
   for (i = 0 ; i < nlevels; i++)
      {
      gsum += gauss [i] ;		/* Gaussian Integral. */
      gausscum [i] = gsum ;		/* Cumulative Gaussian Integral. */
      }

   xyplot (pixel, gausscum, nlevels, "fitw3.cgm", "Gaussian Cumulative",
	   "Pixel Value", "Sum of Gaussian Function G", 0, 0, 0) ;

   xyplot (gausscum, pixel, nlevels, "fitw3.cgm",
	   "Reflection of Gaussian Cumulative",
	   "Sum of Gaussian Function G", "Pixel Value", 0, 0, 0) ;

   fmincum =  100000.0 ;
   fmaxcum = -100000.0 ;
   for (i = 0; i < nlevels; i++)
      {
      if (gausscum [i] < fmincum)  fmincum = gausscum [i] ;
      if (gausscum [i] > fmaxcum)  fmaxcum = gausscum [i] ;
      }

   fprintf (stdout, "hs>gausscum>fmin/fmax: %10.4f %10.4f\n",
	    fmincum, fmaxcum) ;

   /*** Normalize Cumulative Gaussian Integral. ***/

   for (i = 0; i < nlevels; i++)
      {
      gausscum [i] /= (fmaxcum / 255) ;
      }

   xyplot (pixel, gausscum, nlevels, "fitw3.cgm",
	   "Normalized Gaussian Cumulative",
	   "Pixel Value", "Cumulative Gaussian Function G", 0, 0, 0) ;

   fmin =  nelem ;
   fmax = -nelem ;
   for (i = 0; i < nlevels; i++)
      {
      if (gausscum [i] < fmin)  fmin = gausscum [i] ;
      if (gausscum [i] > fmax)  fmax = gausscum [i] ;
      }

   fprintf (stdout, "hs>gausscum.norm>fmin/fmax: %10.4f %10.4f\n", fmin, fmax) ;

   /*** Compute Inverse Cumulative Gaussian Integral. ***/

   for (i = 0 ; i < nlevels; i++)
      {
      gausscuminv [(int) (gausscum [i] + 0.5)] = (float) i ;
      }

   fmin =  100000.0 ;
   fmax = -100000.0 ;
   for (i = 0; i < nlevels; i++)
      {
      if (gausscuminv [i] < fmin)  fmin = gausscuminv [i] ;
      if (gausscuminv [i] > fmax)  fmax = gausscuminv [i] ;
      }

   fprintf (stdout, "hs>gausscuminv>fmin/fmax: %10.4f %10.4f\n", fmin, fmax) ;

   /*** Interpolate over gaps (zeros) in array. ***/

   for (i = 1 ; i < nlevels; i++)
      {
      if (gausscuminv [i] == 0.0)
	 {
	 if (i != 1)
	    for (j = i-1; j > 0; j--)
	       {
               ibeg = j ;
               if (gausscuminv [j] != 0.0)  break ;
	       }
         else
	    ibeg = 0 ;

	 for (j = i+1; j < nlevels; j++)
	    {
            iend = j ;
            if (gausscuminv [j] != 0.0)  break ;
	    }
         nz = iend - ibeg - 1 ;
	 gbeg = gausscuminv [ibeg] ;
	 gend = gausscuminv [iend] ;
	 gdif = gend - gbeg ;
         for (j = 0; j <= nz; j++)
	    {
	    gausscuminv [i] = ((nz - j + 1) / nz) * gdif + gbeg ;
	    }
	 }
      }

/*
 *   for (i = 0; i < nlevels; i++)
 *      printf ("gausscuminv [%3d]: %10.4f\n", i, gausscuminv [i]) ;
 */

   xyplot (pixel, gausscuminv, nlevels, "fitw3.cgm",
	   "Inverse Gaussian Cumulative",
	   "Pixel Value", "Inverse Cumulative Gaussian Function G'", 0, 0, 0) ;

   fmin =  100000.0 ;
   fmax = -100000.0 ;
   for (i = 0; i < nlevels; i++)
      {
      if (gausscuminv [i] < fmin)  fmin = gausscuminv [i] ;
      if (gausscuminv [i] > fmax)  fmax = gausscuminv [i] ;
      }

   fprintf (stdout, "hs>gausscuminv>fmin/fmax: %10.4f %10.4f\n", fmin, fmax) ;

   /*** Normalize Inverse Cumulative Gaussian Integral. ***/

/*
   for (i = 0; i < nlevels; i++)
      {
      gausscuminv [i] /= (fmaxinv / 255) ;
      }

   xyplot (pixel, gausscuminv, nlevels, "fitw3.cgm",
	   "Normalized Inverse Gaussian Cumulative",
	   "Pixel Value", "Inverse Cumulative Gaussian Function G'", 0, 0, 0) ;

   fmin =  nelem ;
   fmax = -nelem ;
   for (i = 0; i < nlevels; i++)
      {
      if (gausscuminv [i] < fmin)  fmin = gausscuminv [i] ;
      if (gausscuminv [i] > fmax)  fmax = gausscuminv [i] ;
      }

   fprintf (stdout, "hs>gausscuminv.norm>fmin/fmax: %10.4f %10.4f\n", fmin, fmax) ;
 */

   for (i = 0; i < nlevels; i++)
      {
      opix [i] = gausscuminv [histcum [i]] ;
      }

   xyplot (pixel, opix, nlevels, "fitw3.cgm",
	   "Transfer Function",
	   "Input Pixel Value", "Output Pixel Value", 0, 0, 0) ;

   /*** Transfer Function. ***/

   for (i = 0; i < nelem; i++)
      {
      ival = *(ibuf + i) ;
      *(gbuf + i) = (unsigned char) gausscuminv [ histcum [ival] ] ;
      }

   cmin = 255 ;
   cmax =   0 ;
   for (i = 0; i < nelem; i++)
      {
      if (*(gbuf + i) < cmin)  cmin = *(gbuf + i) ;
      if (*(gbuf + i) > cmax)  cmax = *(gbuf + i) ;
      }

   fprintf (stdout, "hs>gbuf>cmin/cmax: %3d %3d\n", cmin, cmax) ;

   /*** Expand intensity range to 0 -> 255. ***/

   if (cmax > cmin)
      slope = 256.0 / (cmax - cmin) ;
   else
      slope = 1.0 ;
   for (i = 0; i < nelem; i++)
      {
      *(gbuf + i) = (unsigned char) ( (*(gbuf + i) - cmin) * slope ) ;
      }

   free (ibuf) ;
   return (gbuf) ;
}
