;+
; NAME:
;	align_cube
;
; PURPOSE:
;	 To align a data cube of full disk images to a consistant sun center.
;	 These can be either SXT or GBO or both, and the SXT images may be of 
;	 mixed resolutions. 
;
; METHOD: 
;	 Find the centroid and radii of the target and then feed each image
;        into IMGSCL. A "bit-map" of 1's and 0's indicates which images are
;	 SXT and which GBO. There must be an index entry for each SXT image.
;	 There is no restriction on the ordering of the images, except that
;	 the indices must appear in the same order as the corresponding SXT
;	 images (interviening GBO images are factored in). The target image
;	 can be any where in the cube.
;        When using the target keyword to pass it in from outside, you MUST
;	 add put an extra `1' on the end of sxtmsk and set IT to index that
;	 last value.
;
; CALLING SEQUENCE:
;
;	     cube=align_cube(cube,sxtmsk,i,index[,target=trgt][,/rough][,/cir]
;				[,/inter][,/quiet])
;
; 
; PARAMETERS:
;		cube      the datacube to be aligned
;		sxtmsk    a msk of 1s and 0s indicating SXT/GBO data
;		index	  corresponding index elements
;		it	  the index of the image to align the others to
;
; KEYWORDS: 	rough	  tells it to use the /ROUGH option for the SXT
;			  limbfinder. This makes it MUCH faster, with a
;			  slight reduction in accuracy (less than 1 FR pixel).
;		target    a 6 component vector which contains: the CENTROID (0,1),
;			  the DIAMETERS(horizontal,vertical;2,3), and the pixel
;			  size (x,y:4,5) of the target image.
;			  ***See notes above****
;	   	quiet	  pass through to DSK_LOCG, no display.
;	   	inter	  pass through to DSK_LOCG, interactive point edit.
;
; WARNING: This uses the same cube for out put as input, in order to
;	   allow larger data cubes. If you want to keep your un-aligned cube,
;	   you must save it first.
;          
; RESTRICTIONS:The target can generally be anything, but if it is a GBO image  
;              the SXT sizes must be the same to within about 5%.              
;
; RETURN TYPE: Same as input.
;
; CALLS: GEN_LOC, IMGSCL2
;
; HISTORY:
;	  drafted by A. McAllister. 8-May-92.
;	  changed to avoid centering traget each time. By A.McA. 6-Oct-92.
;	  added target keyword, AMcA, 3-june-93.
;	  Added HIS_INDEX calls, AMcA, 4-april-94.
;	  Added pass-through keywords QUIET and INTER, AMcA, 24-aug-94.
;	  Added pass-through keyword CIR, AMcA, 2-sep-94.
;
;-
FUNCTION align_cube,incube,sxtmsk,it,index,rough=rough,target=trgdt, $
			inter=inter,quiet=quiet,cir=cir

his_index,/enable

if n_params() gt 3 then his_index,index			;add .HIS tag if its missing

cubesz=size(incube)

;a few input traps

if cubesz(0) ne 3 then begin 
		  print,'ALIGN_CUBE: Input not properly dimensioned' & return,-1
endif

if keyword_set(trgdt) then begin
  if n_elements(sxtmsk) ne cubesz(3)+1 then begin
		  print,'ALIGN_CUBE: Mask has wrong number of elements' & return,-1
  endif
endif else begin
  if n_elements(sxtmsk) ne cubesz(3) then begin
		  print,'ALIGN_CUBE: Mask has wrong number of elements' & return,-1
  endif
endelse

if n_elements(index) gt n_elements(sxtmsk) then begin
		  print,'ALIGN_CUBE: More indices than data' &  return,-1
endif

if n_elements(sxtmsk) gt (n_elements(index)+keyword_set(trgdt)) then begin

  print,'ALIGN_CUBE: This is a mixed data set'

endif else begin 

  print,'ALIGN_CUBE: All SXT images'

endelse

ind_ss=where(sxtmsk eq 1)
itt=where(ind_ss eq it)

if not keyword_set(trgdt) then begin		;target not provided, create one

  if sxtmsk(it) eq 1 then begin

    print, 'ALIGN_CUBE: Target SXT image'

    gen_loc,incube(*,*,it),index(itt),CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt, $
				     /SXT,rough=rough

  endif else begin

    print, 'ALIGN_CUBE: Target not SXT image'
	
    gen_loc,incube(*,*,it),CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt, $
			   inter=inter,quiet=quiet,cir=cir

  endelse

    trgdt=[cent,rdhz,rdvt]    
    trgsz=size(incube(*,*,it))			
    trgdt=[trgdt,trgsz(1),trgsz(2)]    

endif 

;replace with some sort of case statement? 

for i=0,cubesz(3)-1 do begin

  if i ne it then begin                ;skip the target index

    if sxtmsk(it) eq 1 then begin 			;target is SXT

      if sxtmsk(i) eq 1 then begin 

        sxtpr=[1,1]
        print,'ALIGN_CUBE: Image # ',i,' is an SXT image.'
	ii=where(ind_ss eq i)
        out=imgscl2(trgdt,incube(*,*,i),sxtpr,index(itt),index(ii),rough=rough)

      endif else begin

        sxtpr=[1,0]
        print,'ALIGN_CUBE: Image # ',i,' is not an SXT image.'
        out=imgscl2(trgdt,incube(*,*,i),sxtpr,index(itt),inter=inter,quiet=quiet,cir=cir)

      endelse

    endif else begin					;target is not SXT

      if sxtmsk(i) eq 1 then begin 

        sxtpr=[0,1]
        print,'ALIGN_CUBE: Image # ',i,' is an SXT image.'
        out=imgscl2(trgdt,incube(*,*,i),sxtpr,index(ii),rough=rough)

      endif else begin

        sxtpr=[0,0]
        print,'ALIGN_CUBE: Image # ',i,' is not an SXT image.'
        out=imgscl2(trgdt,incube(*,*,i),sxtpr,inter=inter,quiet=quiet,cir=cir)

      endelse

    endelse

  endif else begin

    print,'ALIGN_CUBE: Skipped target frame'

    out=incube(*,*,i)

  endelse

  incube(0,0,i)=out

endfor

;  else begin	  ;whole cube is SXT case, this only works if they are all the same size

;  print,'All SXT images'

;  sunc=hxa_suncenter(index=index)

;  diffx=sunc(0,it)-sunc(0,*)
;  diffy=sunc(1,it)-sunc(1,*)

;  out=bytarr(cubesz(1),cubesz(2),cubesz(3))

;  for i=0,cubesz(3)-1 do begin

;    if i ne it then begin                ;skip the target index

;      out(*,*,i)=poly_2d(incube(*,*,i),[-diffx(i),0,1,0],[-diffy(i),1,0,0],1,missing=incube(0,0,i))

;    endif else begin

;      out(*,*,it)=incube(*,*,it)

;    endelse

;  endfor

;endelse   

return,incube

end
