;+
; NAME: background
;
; PURPOSE:  Calculate the featureless background of an optical image. 
;           Also referred to as the limbdarkening profile.
;
; METHOD:   Find the center and do toroidal fourier transform. Keep ndnm modes
;           and transform back. Nearest neighbor averaging is used to fill in
; 	    pixels missed by the ring generation algorythm.
;
;
; CALLING SEQUENCE: 
;		      profile=background(image,target=trgt,modes=mdnm)
;
; PARAMETERS:
;
;           image     image from which to create the profile.
;	    profile   the calculated profile.
;
; KEYWORDS:
;
;   	target     contains the centroid and radius of the input image, 
;		   in pixels. If this is missing they will be calculated.
;    	modes      the number of modes to keep. 
;		   1, gives basic limb-darkening, average value at each r. 
;		   2, takes out components with a wavelength equal to 2 pi. 
;		  >2, removes increasingly high frequency components. 
;		   Note that there is no inherent correlation from one ring
;	           to the next. 
;	           Modes (starting at 0) less than MDNM are kept, so setting
;                  MDNM = 1 keeps one mode (i.e. the 0 mode).
;	           A default value of 2 is used if the keyword is omitted.
;
;
; CAUTIONS: The raw background still shows the effects of intense features  
;  	    in the original data, due in part to limited accuracy in the FT. 
;           Smoothing 2 or 3 times with a box of 5 helps alot, but affects 
;	    the limb.
;
;	    The input is assumed to have been corrected for any artificial 
;	    oblateness.
;
; RETURN TYPE: FLOAT
;
; CALLS:  CRCLE, TARGET                                               
;
; HISTORY: drafted apr. 1992, A. McAllister
;          added target and modes keywords, nov. 1992 AHMcA. 
;	   added nearest neighbor averaging for missing pixels, jan 1993. AHMcA.
;	   changed to only calculate to the limb, mar. 1993, AHMcA.
;
;-
FUNCTION background,img,target=trgt,modes=mdnm

 if keyword_set(mdnm) eq 0 then mdnm = 2

 if keyword_set(trgt) eq 0 then trgt=target(img)

 imgsz=size(img)

 backgrd=img                                     ;initialize result array
 backgrd(*,*,*)=0				 ;maintains variable type

 for ir=1,fix(trgt(2)) do begin

   dum=crcle(trgt(0),trgt(1),ir,imgsz(1),VCOORDS=ss)    ;throw a ring and get pix coords
   ringsz=size(ss)                                	;how many points?
   ringdat=intarr(ringsz(1))                      	;initialize data vector
   backring=ringdat

   ringdat=img(ss)                              ;store data
   datfft=fft(ringdat,-1)                       ;transform to fourier space
   datfft(mdnm:*)=0                             ;keep first mdmn modes, counting from 1
   backring=fft(datfft,1)                       ;tranform back
  
   backgrd(ss)=backring                         ;store background

 endfor

 lind=lindgen(imgsz(1),imgsz(2))
 rcds=r_coord(lind,imgsz(2),trgt(0:1))

 ssz=where(backgrd eq 0 and rcds le trgt(2))   ;find missed pixels, inside the limb
						 
 bck1=shift(backgrd,0,1)
 bck2=shift(backgrd,0,-1)
 bck3=shift(backgrd,1,0)
 bck4=shift(backgrd,-1,0)

 backgrd(ssz)=(float(bck1(ssz))+float(bck2(ssz))+float(bck3(ssz))+float(bck4(ssz)))/4

 return,backgrd

end