;+
;       NAME:  box_lc_plot
;
;
;       PURPOSE:   To format output of BOX_LC for printing figures
;
;
;       METHOD:  Takes the output of BOX_LC and creates finished figures.
;		 The TYPE keyword allows various types of data presentation.
;		 Default is to plot the data as is, i.e. TYPE=1.
;		 
;
;       CALLING SEQUENCE:  box_lc_plot,index,values,boxes,backi[,title][,/bw]
;					[,type=#][,colors=colors][,/port][,/sym]
;
;
;       PARAMETERS:	index	index for data (as in call to BOX_LC)
;			values  output of BOX_LCs TIMESERIES variable
;			boxes   output of BOX_LCs BOXOUT variable
;			backi	image to display boxes on
;			title   string, to display as title
;
;       KEYWORDS: 	port	portrait mode, the image at top, the plot underneath
;			bw 	don't use color for the plots
;			symbol  when set display symbol index. Should be used with BW.
;			reset   when set call XLOADCT to reset base color table.
;			colors  colors to use for plots, use DOC_LIBRARY on LINECOLORS
;			type    keyword for selecting the type of plot
;				1: straight forward plot.
;				2: each curve normalized to 1 at t=0.
;				3: log plot.
;				4: curves each normalized by peak.
;				5: first series used as refference for others.
;
;	NOTES:	You can plot a selection of the boxes by appropriate indexing of the boxes and 
;		values inputs.
;
;	SIDE-EFFECTS:	Resets the color table. Old table saved in common block RGB2. Because
;			the colors are fixed in the color plot mode we can not guarentee good
;			contrast
;
;	COMMONS: RGB2
;
;       HISTORY: Drafted, A.McAllister 7-May-93.
;		 Various improvements and options added, AMcA 11-May-93.
;		 Fixed to handle negative numbers, AMcA 14-May-93.
;		 Reworked to allow color plotting, AMcA 16-Jun-93.
;		 Ensure box outline contrast, AMcA 29-Jun-93.
;		 Added NOSQUEEZE to LINECOLORS call, AMcA 10-Jul-93.
;		 Increased the number of plots to 10, AMcA 14-dec-94.
;		 Fixed a bug in option 5, AMcA 15-dec-94.
;
;-
PRO box_lc_plot,index,values,boxes,backi,title,portrait=port,type=plot_type,bw=bw, $
				colors=colors,symbol=sym,reset=reset

COMMON rgb2,rr,gg,bb,lcrr,lcgg,lcbb,lcset

if n_params() lt 5 then title=' '
if not keyword_set(plot_type) then plot_type=1

if n_elements(lcset) eq 0 then begin
  tvlct,rr,gg,bb,/get						;save table first time only
  linecolors,line_r=lcrr,line_g=lcgg,line_b=lcbb,/noload,/notop,/nosqueeze
  lcset=0
endif

if keyword_set(reset) then begin				;redo base color table
  xloadct
  tvlct,rr,gg,bb,/get
  linecolors,line_r=lcrr,line_g=lcgg,line_b=lcbb,/noload,/notop,/nosqueeze ;redo special table
  lcset=0
endif

if not keyword_set(colors) then if not keyword_set(bw) then $
	colors=[2,4,5,7,9,12,3,1,10,6] else colors=replicate(255,n_elements(boxes(0,0,*)))

if not keyword_set(bw) then begin				;set color table

  if lcset eq 0 then begin 
    tvlct,lcrr,lcgg,lcbb
    lcset=1							
  endif
  back=bytscl(backi,top=238)+17					;scale background image

endif else begin

  if lcset eq 1 then begin
    tvlct,rr,gg,bb						;reload to go back to BW
    lcset=0
  endif
  back=bytscl(backi)

endelse

if not keyword_set(port) then begin				;****set basic display layout****

  num_main=new_win(1024,546) 
  tv,congrid(back,512,512,/interp),0,0

  hblank=replicate(20,1024,3)
  vblank=replicate(20,3,546)

  tv,hblank,0,543
  tv,hblank,0,511
  tv,hblank,0,0
  tv,vblank,0,0
  tv,vblank,510,0
  tv,vblank,1021,0

  blank=replicate(240,1018,30)
  tv,blank,3,514
  xyouts,100,518,'!17'+title+'!3',/device,size=3.0,color=0

endif else begin

  num_main=new_win(513,1024)
  tv,congrid(back,512,512,/interp),0,476

  hblank=replicate(20,512,3)
  vblank=replicate(20,3,1024)

  tv,hblank,0,1021
  tv,hblank,0,988
  tv,hblank,0,475
  tv,hblank,0,0
  tv,vblank,0,0
  tv,vblank,510,0

  blank=replicate(200,506,30)
  tv,blank,3,991
  xyouts,10,995,'!17'+title+'!3',/device,size=3.0,color=0

endelse

sz=size(boxes)
sz2=size(values)
szb=size(back)
scale=8.0*(64.0/szb(1))
pboxes=boxes*scale		
frames=sz(3)
points=sz2(1)

for i=0,frames-1 do begin					;*****draw boxes and number******

  if not keyword_set(port) then begin

    ntest=fix(total(tvrd(pboxes(0,0,i),pboxes(1,0,i),15,15))/225)
    x0=pboxes(0,0,i)
    x1=pboxes(0,1,i)+scale
    y0=pboxes(1,0,i)
    y1=pboxes(1,1,i)+scale
    bdt1=reform(tvrd(x0,y0,1,y1-y0+1))
    bdt2=tvrd(x0,y0,x1-x0+1,1)
    bdt3=reform(tvrd(x1,y0,1,y1-y0+1))
    bdt4=tvrd(x0,y1,x1-x0+1,1)
    btest=fix(total([bdt1,bdt2,bdt3,bdt4])/n_elements([bdt1,bdt2,bdt3,bdt4]))

    if btest gt 120. then clr=0 else clr=250
      
    draw_boxcorn,pboxes(0,0,i),pboxes(1,0,i),pboxes(0,1,i)+scale,pboxes(1,1,i)+scale,color=clr

    if keyword_set(bw) then begin					 	;ensure number contrast
      if ntest gt 120. then clr=10 else clr=250
    endif else begin

      tvlct,tr,tg,tb,/get
      dr=sqrt(tr(i)*tr(i)-tr(ntest)*tr(ntest))
      dg=sqrt(tg(i)*tg(i)-tg(ntest)*tg(ntest))
      db=sqrt(tb(i)*tb(i)-tb(ntest)*tb(ntest))
      if dr+dg+db lt 5 then begin					;match color to plot 
        if i ge median(indgen(frames)) then begin
          dum=colors(i) & colors(i)=colors(0) & colors(0)=dum 
        endif else dum=colors(i) & colors(i)=colors(frames-1) & colors(frames-1)=dum
      endif
      clr=colors(i)
    endelse

    xyouts,pboxes(0,0,i)-45,pboxes(1,0,i)+5,'!5'+string(i+1)+'!3',size=1.5,/device,color=clr

  endif else begin

    ntest=fix(total(tvrd(pboxes(0,0,i),pboxes(1,0,i)+476,15,15))/225)
;    bdt=tvrd(pboxes(0,0,i),pboxes(1,0,i)+476,pboxes(0,1,i),pboxes(1,1,i))+476
;    btest=fix(total(bdt)/n_elements(bdt))

    draw_boxcorn,pboxes(0,0,i),pboxes(1,0,i)+476,pboxes(0,1,i),pboxes(1,1,i)+476

    if keyword_set(bw) then begin						;ensure number contrast
      if ntest gt 120. then clr=15 else clr=250
    endif else begin
      tvlct,tr,tg,tb,/get						;rearrange color order to make #s readable
      dr=sqrt(tr(i)*tr(i)-tr(ntest)*tr(ntest))
      dg=sqrt(tg(i)*tg(i)-tg(ntest)*tg(ntest))
      db=sqrt(tb(i)*tb(i)-tb(ntest)*tb(ntest))
      if dr+dg+db lt 5 then begin					;match color to plot 
        if i ge median(indgen(frames)) then begin
	  dum=colors(i) & colors(i)=colors(0) & colors(0)=dum 
        endif else dum=colors(i) & colors(i)=colors(frames-1) & colors(frames-1)=dum
      endif
      clr=colors(i)
    endelse

    xyouts,pboxes(0,0,i)-45,pboxes(1,0,i)+481,'!5'+string(i+1)+'!3',size=1.5,/device,color=clr

  endelse

endfor

;do plots

pnum=new_win(512,471)
psyms=[-1,-2,-4,-5,-6,-7,-1,-2,-4,-5,-6,-7]			;repeat for large #s

if plot_type ge 5 then ii=0 else ii=1
maxv=-10000000000.						;get max 
minv=100000000000.						;get min
for i=0,frames-1 do begin

    case plot_type of

	1: begin
                  maxval=max(values(*,i))
                  minval=min(values(*,i))
	   end

	2: begin
                  maxval=max(values(*,i)/values(0,i))
                  minval=min(values(*,i)/values(0,i))
           end

	3: begin
                  maxval=max(values(*,i))
                  minval=min(values(*,i))
		  if minval le 0 then begin				;trap negative #s
		    print,'There are negative numbers in this data set.'
		    print,'Please select another plotting option.'
		    wdelete,num_main
		    wdelete,pnum
		    return
		  endif
           end

	4: begin
                  maxval=max(values(*,i)/max(values(*,i)))
                  minval=min(values(*,i)/max(values(*,i)))
           end

	5: begin
		  if i eq 0 then i=1				;skip i=0
                  maxval=max(values(*,i)/values(*,0))
                  minval=min(values(*,i)/values(*,0))
           end

	6:

    endcase

   if maxval gt maxv then begin
     maxv=maxval
     ii=i
   endif
   if minval lt minv then begin
     minv=minval
   endif

endfor

case plot_type of					;*****first set the frame, then do plots*****

	1: begin 						;raw data

		utplot,index,values(*,ii),psym=psyms(ii),yrange=[minv,maxv]
		for i=0,frames-1 do outplot,index,values(*,i),psym=psyms(i),color=colors(i)

	   end

	2: begin						;normalize to first in each series

		utplot,index,values(*,ii)/values(0,ii),psym=psyms(ii),yrange=[minv,maxv]
		for i=0,frames-1 do outplot,index,values(*,i)/values(0,i),psym=psyms(i),color=colors(i)

	   end

	3: begin 						;log plots

		maxt=max(alog10(values))
		mint=min(alog10(values))
		utplot,index,alog10(values(*,ii)),psym=psyms(ii),yrange=[mint,maxt]
		for i=0,frames-1 do outplot,index,alog10(values(*,i)),psym=psyms(i),color=colors(i)

	   end

	4: begin							;normalized by each peak

		
		utplot,index,values(*,ii)/max(values(*,ii)),psym=psyms(ii),yrange=[minv,maxv]
		for i=0,frames-1 do outplot,index,values(*,i)/max(values(*,i)),psym=psyms(i),color=colors(i)

	   end

	5: begin							;first sequence used as reference.

		utplot,index,values(*,ii)/values(*,0),psym=psyms(ii),yrange=[minv,maxv]
		for i=1,frames-1 do outplot,index,values(*,i)/values(*,0),psym=psyms(i),color=colors(i)

	   end 

endcase

if keyword_set(sym) then begin 						;****symbol index*****

  xend=int2secarr(index(points-1),index(0))
  xrange=!x.crange
  yrange=!y.crange
  xdelt=xrange(1)/20.0
  ydelt=(yrange(1)-yrange(0))/20.0

  if plot_type ne 5 then srt=0 else srt=1
  for i=srt,frames-1 do begin

    xx=[xrange(1)-1.0*xdelt,xrange(1)-1.0*xdelt]
    yy=[yrange(1)-(2*(i+1.1)*ydelt),yrange(1)-(2*(i+1.1)*ydelt)]

    if keyword_set(bw) then begin

      xyouts,xrange(1)-5*xdelt,yrange(1)-(2*(i+1.2)*ydelt),'!5'+string(i+1)+'!3',size=1.5,/data
      outplot,xx,yy,psym=psyms(i)

    endif else begin
    
      clr=colors(i)
      xyouts,xrange(1)-5*xdelt,yrange(1)-(2*(i+1.2)*ydelt),'!5'+string(i+1)+'!3',size=1.5,color=clr,/data
      outplot,xx,yy,psym=psyms(i),color=clr

    endelse
    
 endfor

endif

temp=tvrd(0,0,505,471)
wdelete,pnum

wset,num_main
if not keyword_set(port) then tv,temp,512,3 $
		     else tv,temp,3,3

end




