;+
;       NAME:  ellipse_fit
;
;       PURPOSE:  To fit an ellipse to a set of data points.
;		 
;       METHOD: Estimate parameters from data points and feed to a
;		chi squared search routine. Itterate until covergence.
;
;       CALLING SEQUENCE:	trgt=ellipse_fit(limb,fit,mode,/major)
;
;       PARAMETERS:	limb	points to fit, in [x,y] form
;			fit	fitted points, return	
;			mode	mode [-1,0,+1] switch for chi square
;			trgt	return fit parameters
;					centroid: [x,y]
;					minor axis
;					eccentricity
;					major axis angle: x-axis is 0.
;
;       KEYWORDS:	major	when major axis rather than minor axis
;				is available
;
;	CALLS:	RECT2POL,ELLIPSE_FN,GRADLS
;
;       HISTORY: Drafted AMcA, August 1994.
;		 Added QUIET keyword, Sept. 1994
;
;-
FUNCTION ellipse_fit,limb,fit,mode,major=major,quiet=quiet

x = limb(0,*)
y = limb(1,*)

mode = 0.0

npnts = n_elements(x)

;find the initial parameters (for circle)

xcen = total(x+.5)/npnts				;estimate centroid
ycen = total(y+.5)/npnts

polar = rect2pol(x-xcen,y-ycen)
radius = total(polar(0,*))/npnts			;average radius

if not keyword_set(major) then begin

  minax = min(polar(0,*))				;estimate minor axis
; rot_ang = polar(1,where(polar(0,*) eq minax)) + 90/!radeg	
  rot_ang = 90/!radeg					;fix ellipse as upright
  majax = minax + 2.5*(radius - minax)			;estimate major axis
  
endif else begin

  majax = max(polar(0,*)) 				;estimate major axis
  rot_ang = polar(1,where(polar(0,*) eq majax)) 
  minax = majax + 2.5*(radius - majax)			;estimate minor axis
  
endelse

eccent = sqrt(1. - (minax*minax)/(majax*majax)) 	;estimate eccentricity

a = [xcen,ycen,minax,eccent,rot_ang]			;parameters for fitting

if not keyword_set(quiet) then begin

  curve=ellipse_fn(a,x,y)
  plots,curve(0,*),curve(1,*),psym=0,/device,color=255	;check first quess

endif

delta=[2.0,2.0,1.0,0.01,0.1]				;set "reasonable" search increments

fit = gradls(x,y,a,delta,chisqr,mode,sigmay,funct="ellipse")		;do the fit

if (chisqr gt 1.0) then begin

  repeat begin					;repeat until convergence

    chisq0=chisqr
    fit = gradls(x,y,a,delta,chisqr,mode,sigmay,funct="ellipse")

  endrep until chisq0-chisqr lt 0.001
  
  chisqr=chisq0
  
endif

return,a					;return fit parameters

end
