;+
; NAME:
;       enhancer
;
; PURPOSE:
;	A function to enhance the sharpness of features, of single frames.
;       The data is not decompressed or otherwise preprocessed. Any such
;       processing should be done prior to calling enhancer.
;       A difference (delta) field is created by subtracting a smoothed
;       image from the input image. This delta field is then cropped (if
;       so desired), multiplied by a constant and added back to the 
;       original field. Lower and upper cutoffs can also be set to limit
;       the enhancement to a specified range of initial field intensity.
;
; CALLING SEQUENCE:
;
;	imageout=enhancer(imagein,index,keyword=##,keyword=##,..)
;
;       where imagein & imageout are 2d arrays, and index is the
;       index entry that corresponds to imagein.
;
; Optional Keyword Parameters:
;
;	smooth    Size of smoothing box, integer.
;	lowdelt   Lower delta cutoff, a percentage of minimum, decimal 0.xx.
;	updelt    Upper delta cutoff, a percentage of maximum, decimal 0.xx.
;	lowint    Saturation cutoff, don't add back when signal is BELOW a
;                 given percentage of the saturation value, decimal 0.xx.
;       upint     Saturation cutoff, don't add back when signal is ABOVE a  
;                 given percentage of the saturation value, decimal 0.xx.
;	deltcoef  Multiplier for Delta field,  float x.x.
;
;       If a keyword is not specified then a default value is used, based on
;       the resolution of the input data array.
;
; HISTORY:
;       drafted by A. McAllister and K. Strong, Apr. 1992
;       
;-

FUNCTION enhancer,data,indx,smooth=smth,lowdelt=lwct,updelt=upct,lowint=stctl,upint=stctu,deltcoef=dltmlt

;check the keywords, if not set use defaults

swtch=gt_res(indx)                             ;check resolution

case 1 of

  (swtch eq 2):  begin                                  ;quarter resolution

	if keyword_set(smth) eq 0 then smth = 3
	if keyword_set(lwct) eq 0 then lwct = 0.0
	if keyword_set(upct) eq 0 then upct = 0.5
	if keyword_set(stctl) eq 0 then stctl = 0.0
	if keyword_set(stctu) eq 0 then stctu = 0.7
	if keyword_set(dltmlt) eq 0 then dltmlt = 5.0
        end

  (swtch eq 1):  begin                                  ;half resolution

	if keyword_set(smth) eq 0 then smth = 5
	if keyword_set(lwct) eq 0 then lwct = 0.0
	if keyword_set(upct) eq 0 then upct = 0.5
	if keyword_set(stctl) eq 0 then stctl = 0.0
	if keyword_set(stctu) eq 0 then stctu = 0.7
	if keyword_set(dltmlt) eq 0 then dltmlt = 3.0
        end

  (swtch eq 0):  begin                                  ;full resolution

	if keyword_set(smth) eq 0 then smth = 9
	if keyword_set(lwct) eq 0 then lwct = 0.0
	if keyword_set(upct) eq 0 then upct = 0.5
	if keyword_set(stctl) eq 0 then stctl = 0.0
	if keyword_set(stctu) eq 0 then stctu = 0.7
	if keyword_set(dltmlt) eq 0 then dltmlt = 1.0
	end

endcase

print,smth,lwct,upct,stctl,stctu,dltmlt

;make averaged image, and subtract from the original data

dum=data-smooth(data,smth) 
 
lwlmt=(lwct*min(dum))                    ;set lower delta cutoff
uplmt=(upct*max(dum))                    ;set upper delta cutoff

dum(where(dum le lwlmt))=lwlmt           ;clip lower end
dum(where(dum ge uplmt))=uplmt           ;clip upper end

tst=size(where(data le stctl*max(data)))

if (tst(0) gt 0) then begin

   dum(where(data le stctl*max(data)))=0   ;only add when dum > stctl*max(dum)

endif

tst=size(where(data ge stctu*max(data)))

if (tst(0) gt 0) then begin

   dum(where(data ge stctu*max(data)))=0   ;only add when dum < stctu*max(dum)

endif

dum=bytscl(long(data)+long(dltmlt)*long(dum))     ;add the delta field back to base

return,dum

end                      

