pro ffi2pfi, index_out, data_out, $
	index_in=index_in, data_in=data_in, $
	infile=infil, dset_arr=dset_arr, $
	helio=helio, date_helio=date_helio, $
	suncoord=suncoord, $
	ccdcoord=ccdcoord, $
	outres=outres, outsize=outsize, $
	nocorrections=nocorrections, $
	qdebug=qdebug, qstop=qstop
;+
;NAME:
;	ffi2pfi
;PURPOSE:
;	To extract and align a portion of an FFI image by specifying either
;		(1) heliocentric coordinates for a given time/date
;		    (corrects for differential rotation and S/C jitter)
;		(2) the fixed coordinates relative to sun center in arcseconds
;		    (corrects for S/C jitter)
;		(3) the CCD fixed coordinates in FR IDL coordinates
;		    (would not take out S/C drift)
;SAMPLE CALLING SEQUENCE:
;	ffi2pfi, index_out, data_out, infil=infil, dset_arr=dset_arr, $
;	ffi2pfi, index_out, data_out, infil=infil, dset_arr=-1, $
;	ffi2pfi, index_out, data_out, index_in=index_in, data_in=data_in, $
;
;		<< one of the two above, plus one of the below >>
;
;	   helio=[-9.8,-20.3], date_helio='14-JUN-92  02:37:41'
;	   suncoord = [-300,300]
;
;	ffi2pfi,index,data,helio=[-9.8,-20.3],date_helio=ii,infil=ff(0),dset_arr=-1
;METHOD:
;	Optionally decompress, dark sub, and leak sub
;
;     Relies on HXA/IRU data for alignment, the accuracy of the alignment
;     will be greatly reduced if there is an HXA problem (warning is printed).
;
;INPUTS:
;OUTPUTS:
;	index_out- The output index structure
;	data_out - The output data array
;OPTIONAL KEYWORD INPUTS:
;	index_in - The input index structure (if not using INFIL/DSET_ARR
;		   option)
;	data_in	- The input data array (if not using INFIL/DSET_ARR option)
;
;	infil	- The list of input files
;	dset_arr- The list of dataset numbers of the images within the
;		file "infil" to be processed.
;
;	outres- The resolution of the output image (0=FR, 1=HR, 2=QR).
;		If not set, then convert all to full res
;		** NOT AVAILABLE RIGHT NOW - ALWAY FULL RESOLUTION **
;	outsize- The dimention of the ouptut image [#col, #lin]
;		If it is a scalar value, it will make the #lin = #col
;		If not set, it will make it 128x128

;	helio - the reference latitude and longitude for the alignment.
;		This should be the position of the active region at some 
;		time, preferably near the time it crossed closest to the 
;		center of the disk.  
;			helio(0) = longitude E/W (E is negative)
;			helio(1) = latitude N/S (S is negative)
;	date_helio - the reference date/time for the heliocentric input.
;
;	suncoord - The coordinates of the center of the portion to extract
;		   in angular arcseconds relative to sun center.
;			suncoord(0) = E-W coordinate (E is negative)
;			suncoord(1) = N-S coordinate (S is negative)
;
;	ccdcoord - The FRE (full resolution CCD pixel equivalent) of the 
;		   center of the portion to extract
;
;	nocorrections - If set, turns off ALL corrections to the data (does not
;		  call sxt_decomp,dark_sub, or leak_sub).  This is 
;                 useful if you have already done these corrections
;                 and are passing data to be dejittered.
;		  Do corrections only if data comes in as BYTE type
;OPTIONAL KEYWORD OUTPUTS:
;
;RESTRICTIONS:
;     When aligning data which has already been read in with yodat, it is
;     very important that no other alignment routines have been run on the 
;     data.  The data should be "raw", straight out of yodat.  FFI images
;     must be full size.
;
;QUESTIONS:
;	* how to specify the output size -arcsec or FR pixels?
;	* resolution of the output image
;	* what image to register to - first or middle?
;HISTORY:
;	Written 1-Jun-93 by M.Morrison using Metcalf RD_AR as starting point
;	 8-Jul-93 (MDM) - Do corrections only if data comes in as BYTE type
;			- Set the SAVE option on LEAK_SUB when it is not the
;			  last image being processed
;	 4-Aug-93 (MDM) - Added /CONV2FR switch to call to EXT_SUBIMG2
;			  because of changes to EXT_SUBIMG2
;	 28-Feb-94 (AHM) - Replaced GET_SXTCENTER with SXT_CEN
;-
;
progverno = 1.0*1000


;;pro rd_ar, infil, ss, index, data, roadmap, ref_image=refimage, $
;;             korrel=korrel, quiet=quiet, latitude=lat, longitude=lon, $
;;             date=date, size_ffi=size_ffi,ar_size=ar_size,missing=missing, $
;;             compress=compress,nocorrections=nocorrections

;To carry over from RD_AR (?)
;     missing = the data value which is used in the portions of the data array
;               which are left undefined (PFI's only).
;     /compress = Re-compress the data after alignment.


;     A combination of FFI and PFI data does not align very well over the 
;     limb.  This is because the PFI FOV center is off the limb and it is 
;     impossible to compute the differential rotation.  To correct this, use
;     the ref_image keyword to select a reference PFI whose center is on
;     the disk.  Or use the latitude/longitude keywords to specify the 
;     position of the co-alignment.
;PROCEDURE:
;     The PFI's are aligned with the align_ar function.  A piece of the
;     FFI of the same size as the PFI's and at the same heliographic
;     position is extracted.  The output is thus a data cube which has
;     a mixture of PFI and FFI data all aligned to the accuracy of the
;     HXA/IRU data (sub-arcsecond??).  The heliographic position is derived 
;     from an expression for the differential rotation.
;
code = -1
if (keyword_set(helio)) then code = 1		;using 
if (keyword_set(suncoord)) then code = 2		;using coordinates relative to sun center
if (keyword_set(ccdcoord)) then code = 3		;using fixed IDL coordinates
;
if ((n_params() LT 2) or (code eq -1)) then begin
    print, 'FFI2PFI Sample Calling Sequences:
    print, '   ffi2pfi, index_out, data_out, infil=infil, dset_arr=dset_arr
    print, '   ffi2pfi, index_out, data_out, index_in=index_in, data_in=data_in
    print, ' '
    print, 'Need to specify one of the following:
    print, '       HELIO and HELIO_DATE		- to track by heliocentric coord
    print, '       SUNCOORD			- to track on fixed location on sun
    print, '       CCDCOORD			- to track on fixed CCD location
    print, 'Returning...'
    return
end
;
ARCSEC_PER_PIX = gt_pix_size()
;
;
;---------------------------------------- Figure out how the input images are specified
;
qread = 0
if ((n_elements(index_in) eq 0) or keyword_set(infil) ) then begin
    if ((n_elements(infil) eq 0) or (n_elements(dset_arr) eq 0)) then begin
	message, /info, 'If INDEX and DATA are not passed in, then INFIL and DSET_ARR must be used'
	return
    end
    if (keyword_set(qdebug)) then print, 'Reading from the data file'
    qread = 1
    dset = mk_dset_str(infil, dset_arr)
    rd_xda, infil, dset, index_in, /nodata
end
nout = n_elements(index_in)
;
index_out = index_in
his_index, index_out	;append on the .HIS structure
;
;---------------------------------------- Figure out the offsets
;
case code of
    1: begin
		sc = sxt_cen(index_out) 				;need the sun center locations (in FR pixels)
		h_ang = track_h2a(index_out, helio, date_helio)		;need heliocentric motion (in arcsec)
		h_ang = h_ang / gt_pix_size()				;converted to pixels from sun center

		xx = sc(0,*) + h_ang(0,*)				;center of the FOV in FR IDL coordinates
		yy = sc(1,*) + h_ang(1,*)
	end
    2: begin
		sc = sxt_cen(index_out) 				;need the sun center locations (in FR pixels)
		xy = suncoord/ARCSEC_PER_PIX				;FR pixels relative to sun center

		xx = xy(0) + sc(0,*)					;center of the FOV in FR IDL coordinates
		yy = xy(1) + sc(1,*)
	end
    3: begin
		xx = ccdcoord(0)
		yy = ccdcoord(1)
	end
endcase
;
if (keyword_set(qstop)) then stop
;
;---------------------------------------- Figure out the output image size
;
case n_elements(outsize) of
    0: begin
	    nx = 128
	    ny = 128
	end
    1: begin
	    nx = outsize
	    ny = outsize
	end
    else: begin
	    nx = outsize(0)
	    ny = outsize(1)
	end
endcase
;
data_out = make_array(nx, ny, nout, type=2)	;output is always integer*2
;
for iimg=0,nout-1 do begin
    if (qread) then begin
	if (keyword_set(qdebug)) then print, 'Reading dataset # ', iimg
	rd_xda, infil, dset(iimg), index0, data0, roadmap
    end else begin
	index0 = index_in(iimg)
	data0 = data_in(*,*,iimg)	;TODO - what if QR is imbedded inside a 512x512xN array?
    end
    if (keyword_set(qdebug)) then print, 'Now processing ', fmt_tim(index0)

    siz = size(data0)
    typ = siz( siz(0)+1 )
    ;
    if ((not keyword_set(nocorrections)) and (typ eq 1)) then begin
	data0 = leak_sub(index0, temporary(data0), save=(iimg ne nout-1) )	;leak_sub does DARK_SUB and SXT_DECOMP work also
    end

    nx_in = gt_shape_cmd(index0, /x)
    ny_in = gt_shape_cmd(index0, /y)

    ;---------------------------------------- Extract the image

    x = xx(iimg)
    y = yy(iimg)

    x0 = (fix(x)-nx/2) >0 < 1023
    x1 = (x0 + (nx-1)) >0 < 1023
    y0 = (fix(y)-ny/2) >0 < 1023
    y1 = (y0 + (ny-1)) >0 < 1023
    xoffset = -(x-(x0+x1)/2.0)		;fractional pixel offsets (??) - in FR pixels
    yoffset = -(y-(y0+y1)/2.0)

    ext_subimg2, index0, data0, index1, data1, fri=[x0,y0], nfr_pix=[nx, ny], /conv2fr

    data2 = poly_2d(temporary(data1), [-xoffset,0,1,0], [-yoffset,1,0,0], 1)

    str_insert, index_out, index1, iimg
    his_index, index_out, iimg, 'q_sun_rot', progverno
    his_index, index_out, iimg, 'q_registered', progverno
    his_index, index_out, iimg, 'arcsec_shift', [-xoffset, -yoffset]*gt_pix_size()
    corner_sav = gt_corner(index1)
    corner_sav(0) = corner_sav(0) - xoffset
    corner_sav(1) = corner_sav(1) - yoffset
    his_index, index_out, iimg, 'corner_sav', corner_sav
    his_index, index_out, iimg, 'sun_center', sc(*,iimg)
    
    data_out(0, 0, iimg) = data2
end
;
if (keyword_set(qstop)) then stop
end
