;+
; NAME: ffi_prep
;
;
; PURPOSE: general data preparation for sections of FFI's, and/or FFI's
;
;
; METHOD: This procedure is designed to prep large FFI data sets all at once,
;	  by reading the data in in manageble sets. You can input a long list
;	  of SFR files (using wild cards), select specific images using
;	  SSWHERE, and then process that desired images only. The processing
;  	  is controled by keywords, and is essentially the same as for SXT_PREP.
;	  With the SAVE keyword the results of this section are saved to disk.
;	  The results of SATPIX and UNCERT keywords (controlling the FFI prep
;	  only) are always saved if preformed.
;
;	  There is a second section that allows you to extract a data cube 
;	  covering a smaller field of view.  This data set is aligned with 
;	  FFI2PFI in one of two ways: 1) all frames taken with the same angular
;	  location and 2) tracking a region to remove rotation.  The results of 
;	  this alignment along with saturated pixel maps are always saved
;	  to disk. 
;
;	  These two operations can be performed in seguence or one at a time
;	  using keywords to shut off either the preparation or the alignment.
;    
; 	  Before using you should set the environmental variable DIR_MYPREP
;	  to indicate the directory to which the output files are to be
;	  written, otherwise the current directory is the defualt. 
;
;
; CALLING SEQUENCE:  ffi_prep,filestr[,/uncert][,/satpix][,align='algn'][,/all]
;				    [,/noprep][,/norm]....
;
;	  For PFI's: ffi_prep,filestr,align='region',/sparse,/upper_only
;
;
;       PARAMETERS:	filestr	 	path/filename, wildcards O.K. STRING.
;
;
;       KEYWORDS:   all		skip the call to SSWHERE, take everything.
;		    noprep 	when set the data in FILESTR is read
;				and durectly passed to the alignment section.
;		    align	For limb alignment 'limb',
;				for region alignment, 'region'
;			  	for no alignment, use 'no'.
;		    uncert	to return decomp uncertainty
;		    satpix	when set calculates the saturation map for 
;				the FFIs as well.
;		    upper_only	pass-through for SXT_SATPIX
;		    sparse	return satpix as 'sparse' structure
;		    mult_str	when set do not try to create a single
;				sparse structure for the PFIs.
;		    nodark	to suppress DARK_SUB call
;		    leak	to correct for light leak
;		    roll_do	do roll sorrection
;		    norm  	to call EXP_NORM, should be single filter
;		    save	save the processed FFI data to disk
;		    inc		size of batches to process, default is 10
;
;	CAUTIONS:     * This program expects the DIR_MYPREP environmental
;			variable to be defined to indicate where to put the
;			output files. If it is not the default is to use the
;			'current directory'.
;		      * If the NOPREP keyword is set then the ALIGN keyword must
;			also be set.
;		      * The NORM,LEAK,NODARK keywords have no effect
;			if NOPREP is set.
;		      * For FFIs, with the SPARSE keyword SATPIX is stored
;			in pieces, as it is probably too large to make into
;			a single structure.
;			That for the PFI regions is made into a single
;			structure. Be careful as there may be limits to this
;			as well! In that case use MULT_STR keyword.
;
;	****** For now best to use MULT_STR with the SPARSE option ********
;
;
;  CALLS: RT_SXTDT,SXT_SATPIX,SXT_DECOMP,DARK_SUB,LEAK_SUB,EXP_NORM,GET_RECT,
;	  AR_PREP,FFI2PFI,RD_XDA,FMT_TIM,HIS_INDEX,EXP_FILE,WMENU_SEL,BREAKFILE,
;	  GT_FILTB,SAV_SDA,SPARSE
;
;
;       HISTORY: Drafted by A. McAllister, mar-93.
;		 Drafted prep sections AMcA, apr-93.
;		 Moved the region selection up front, and replaced RD_AR
;		 with FFI2PFI. AMcA, jun-93.
;		 Changed UNCERT and SATPIX to keywords & save results, 
;		 allow selection of reference image, AMcA, 1-jul-93.
;		 Calculate SATPIX for the region seperately, AMcA, 2-jul-93.
;		 Added test for near night images, AMcA, 3-jul-93.
;		 Limit ref images to large dpes, AMcA, 8-jul-93.
;		 Fixed calls to FFI2PFI for no-correction, AMcA, 8-jul-93.
;		 Replaced calls to FFI2PFI with calls to SXT_PREP, AMcA,
;		 29-apr-94.
;		 Corrected the some of the preparation calls and
;		 removed the /NOCORRECTIONS keyword from the SXT_PREP calls,
;		 AMcA, 14-dec-94.
;		 enabled reselection of region, AMcA, 4-mar-95.
;
;-
PRO ffi_prep, filestr, uncert=uncrt, satpix=satpx, all=all, noprep=noprep, $
	      align=algn, upper_only=upper_only, sparse=sparse, nodark=nodark, $
	      roll_do=roll_do, leak=leak, norm=norm, save=save, inc=inc, $
	      mult_str=mult_str

  his_index, /enable			;enable rewriting the index

  if not keyword_set(inc) then inc=10	;number of images per batch
  sat_limit = [235,  255,  255]		;full,  half,  quarter resolution

  prep_dir = getenv ('DIR_MYPREP')
  if prep_dir eq '' then spawn, 'pwd', prep_dir
 
  if not keyword_set(algn) then begin 

    algn = 'no'

    if keyword_set (noprep) then return		;no align,  no prep,  return

  endif

  if not keyword_set (all) then begin		;*** select the data ****

    rt_sxtdt, filestr, index, ss=ss, infil=infil, /nodata  	

  endif else begin

    infil = exp_file (filestr)
    rd_xda, infil, -1, index, /nodata
    ss=indgen (n_elements (index))

  endelse

  tim2orbit, index, tim2night=nght			;screen out 'night' images
  ssnt = where (nght gt 1.0)
  if ssnt(0) ne -1 then begin
    nghtimg = n_elements (where (nght le 1.0))
    print, 'FFI_PREP: Eliminated ' + strcompress (string (nghtimg)) + $
	   ' near night images.'
    index = index (ssnt)
    ss = ss (ssnt)
  end
  
  dpes = gt_dpe (index)				;eliminate short dpe images from list
  sdpes = dpes (sort (dpes))
  udpes = sdpes (uniq (sdpes))
  tstdpe = udpes ((n_elements (udpes) - 1)) < 16
  sst = where (gt_dpe (index) ge tstdpe)

  times = fmt_tim (index (sst))			;times for reference images

  imgnum = n_elements (ss)					
  count = 0

  while (imgnum-count gt 0) do begin

    last = (count+inc-1) < (imgnum-1)
    ssn = indgen (last+1-count)		                ;indexing for prep file
    rd_xda, infil, ss (count:last), index, cube		;get set of images  
    
    his_index, index

    if count eq 0 then begin					

      if algn ne 'no' then  begin		;**** select region *****

        print, 'FFI_PREP: Please choose a reference image for region selection.'
        print, 'A null choice results in use of the first image.'

	done = 0
	while not done do begin

          reff = sst (wmenu_sel (times, /one))
          if reff eq -1 then reff = 0

          if reff gt last then begin
	    rd_xda, infil, ss (reff), ref_ind, ref_img 
          endif else begin 
	    ref_img = cube (*, *, reff) 
 	    ref_ind = index (reff)
	  endelse

          case algn of

            'limb':   if count eq 0 then ar_prep, ref_img, ref_ind, lonlat, dum, arsize, /table

            'region': if count eq 0 then ar_prep, ref_img, ref_ind, lonlat, dat0, arsize, /table

            else:		;do nothing

          endcase

	  print, 'Your reference image was at ', gt_time (ref_ind, /string), $
		 ' on ', gt_day(ref_ind, /string), ' .'
	  print, 'You selected a ', strtrim (string (arsize (0))), ' by ', $
		 strtrim (string (arsize(1))),  $
		' FR pixel region centered at ',  $
		'LON: ', string (lonlat (0), format='$(f8.3)'),  $
		'LAT: ', string (lonlat (1), format='$(f7.3)'), '.'

	  yesnox, 'Use these values? (y/n)', ans, 'y'	;change color table
   	  if ans then done = 1 

	endwhile

      endif

      break_file, infil (0), dum, dum1, stem		;set stem for savefiles

    endif

    if not keyword_set (noprep) then begin		;***** start prep ****

      filts = gt_filtb (index (ssn))			;what filters do we have?
      if n_elements (filts) gt 1 then $
          unq_filts = uniq (filts (sort (filts))) $
      else unq_filts = filts	

      if n_elements (unq_filts) gt 1 then begin

        print, 'FFI_PREP: There is more than one filter in this data set.'

      endif 

; Do the prep work,  mostly taken from SXT_PREP,  if calling sequence is changed
; replace with a direct call

      if keyword_set (satpx) then $
        satpix = sxt_satpix (index, cube, sat_limit=sat_limit, $
			     upper_only=upper_only)

      if keyword_set (uncrt) then uncert = sxt_decomp (cube, /uncert)    ;this must go first

      if not keyword_set (nodark) then $
	 cube = dark_sub (index, cube, /interpolate)   $  ;calls SXT_DECOMP
       else  cube = sxt_decomp (cube)    

      if keyword_set (leak) then $
	  cube = leak_sub (index, cube)	;for data after 13-nov-92,  
					;currently dosn't work for pfi's

      cube = cube > 0			;set negative values to zero

      ;add call to SXT_INTERP

      ;add call to SXT_SUMXY

      if keyword_set (norm) then cube = exp_norm (cube, index)

      if keyword_set (save) then begin				;store the data

        if count eq 0 then begin 

	  prep_file   = prep_dir + '/' + stem + '_prep.sda'
          uncert_file = prep_dir + '/' + stem + '_unc.sda'

          sav_sda, prep_file, index, cube		
          if keyword_set (uncrt)  then sav_sda, uncert_file, index, uncert

	  if keyword_set (satpx) then begin	;store satpix as sparce struct?
    	    if keyword_set (sparse) then begin
	      satpix_str_lg = sparse (satpix, low=1)
	    endif else begin
	      sat_file = prep_dir + '/' + stem + '_sat.sda'
	      sav_sda, sat_file, index, satpix		
	    endelse
	  endif

        endif else begin

	  sav_sda, prep_file, index, cube, /append		
          if keyword_set (uncrt)  then $
	      sav_sda, uncert_file, index, uncert, /append

	  if keyword_set (satpx) then begin
	    if keyword_set (sparse) then begin
              new_str = sparse (satpix, low=1)
	      satpix_str_lg = str_concat (satpix_str_lg, new_str)		;something flakey here!
	    endif else sav_sda, sat_file, index, satpix, /append	
	  endif
	endelse
      endif
    endif  						;***** end prep*****

    case algn of

      'limb':  	begin

		  if data_type (cube) ne 1 then begin
		  
		    sxt_prep, index, cube, smcube_i, smcube, $
			      suncoord=conv_h2a(lonlat),  $
		    	      outsize=arsize, roll_do=roll_do

;		    ffi2pfi, smcube_i, smcube, index_in=index, data_in=cube,  $
;			     suncoord=conv_h2a(lonlat), outsize=arsize, $
;			     /nocorrections

		  endif else begin	;will run dark_sub and leak_sub on PFIs

		    sxt_prep, index, cube, smcube_i, smcube, $
                              suncoord=conv_h2a(lonlat),  $
		              outsize=arsize, roll_do=roll_do

;		    ffi2pfi, smcube_i, smcube, index_in=index, data_in=cube,  $
;			    suncoord=conv_h2a(lonlat), outsize=arsize

		  endelse
		  
                  satpix = sxt_satpix (smcube_i, sxt_comp (smcube),  $
				       sat_limit=sat_limit, $
				       upper_only=upper_only)
	      	  if keyword_set (sparse) then satpix = sparse (satpix, low=1)	  

	          if count eq 0 then begin 

		    algn_file = prep_dir + '/' + stem + '_algn.sda'
		    sav_sda, algn_file, smcube_i, smcube

	      	    if keyword_set (sparse) then begin
		      satpix_str_sm = sparse (satpix, low=1)
		    endif else begin
		      asat_file = prep_dir + '/' + stem + '_algn_sat.sda'
		      sav_sda, asat_file, smcube_i, satpix
		    endelse

    		  endif else begin

		    sav_sda, algn_file, smcube_i, smcube, /append

	      	    if keyword_set (sparse) then begin
		      new_str = sparse (satpix, low=1)
		      satpix_str_sm = str_concat (satpix_str_sm, new_str)
		    endif else sav_sda, asat_file, smcube_i, satpix, /append

		  endelse
	        end

      'region': begin

print, 'FFI_PREP: data_type is ', data_type (cube)

		  if data_type (cube) ne 1 then begin

		    sxt_prep, index, cube, smcube_i, smcube, helio=lonlat, $
			      date_helio=dat0,  outsize=arsize, $
			      roll_do=roll_do
		
;		    ffi2pfi, smcube_i, smcube, index_in=index, data_in=cube,  $
;                            helio=lonlat, date_helio=dat0, outsize=arsize, $
;                            /nocorrections

		  endif else begin	;will run dark_sub and leak_sub on PFIs

		    sxt_prep, index, cube, smcube_i, smcube, helio=lonlat, $
			      date_helio=dat0,  $
		    	     outsize=arsize, roll_do=roll_do

;		    ffi2pfi, smcube_i, smcube, index_in=index, data_in=cube,  $
;			  helio=lonlat, date_helio=dat0, outsize=arsize

		  endelse
		 
                  satpix = sxt_satpix (smcube_i, sxt_comp(smcube),  $
                                       sat_limit=sat_limit, $
				       upper_only=upper_only)

	          if count eq 0 then begin 

		    algn_file=prep_dir + '/' + stem + '_algn.sda'
		    sav_sda, algn_file, smcube_i, smcube		

	      	    if keyword_set (sparse) then begin
		      satpix_str_sm = sparse (satpix, low=1)
		    endif else begin
		      asat_file=prep_dir + '/' + stem + '_algn_sat.sda'
		      sav_sda, asat_file, smcube_i, satpix		
		    endelse

    		  endif else begin

		    sav_sda, algn_file, smcube_i, smcube, /append	

	      	    if keyword_set (sparse) then begin
		      new_str = sparse (satpix, low=1)
		      satpix_str_sm = str_concat (satpix_str_sm, new_str)
		    endif else sav_sda, asat_file, smcube_i, satpix, /append	

		  endelse
	        end

      else: 	print, 'No alignments done.'

    endcase

    count = count + inc

  endwhile

  cube=0 & smcube=0					;clean up  

  if keyword_set (sparse) then begin			;save sparse structures

    if not keyword (mult_str) then begin
      array = sparse (satpix_str_sm)			;create single structure
      satpix = array (0)
      for i = 1, n_elements (array) - 1 do satpix = [[[satpix]], [[array (i)]]]
    end

    asat_file = prep_dir + '/' + stem + '_algn_sat.sprs'
    save, filename = asat_file, satpix_str_sm

    if keyword_set (satpx) and keyword_set  (save) then begin

      asat_file = prep_dir + '/' + stem + '_sat.sprs'
      save, filename=asat_file, satpix_str_lg

    endif
  endif

  his_index, /disable
  clearplot			;clean up after SXT_GRID
  clear_utplot 

end
