;+
; NAME: gbo_pfi
;
;
; PURPOSE: get a matching GBO patch for PFI data
;
;
; METHOD:  Figure PFI corners from INDEX, rotate these to the
;	   time of the GBO image, get section of GBO and rotate
;	   back, finally take exact matching section. The GBO
;	   image is first matched to the location and size of
;	   the full disk coresponding to the PFI.
;	   You can either pass in the GBO image itself, in which 
;	   case you MUST also supply the time for it, or a string
;	   with the path/filename of a YS style GBO image file.
;	   The key here is that the extension be .HHMM for the image.
;	   The greater the time difference the less appropriate the
;	   RIGID keyword becomes.
;
;
; CALLING SEQUENCE:  gbo_part=gbo_pfi(gboin,indx[,gbotim][,/rigid)
;
;
; PARAMETERS: 	gboin		the gbo image data, or a string with 
;				path/filename of gbo image
;		indx		PFI index
;		gbotim		if gboin is data, then this must have 
;				the image's time. Any format acceptable
;				to ANYTIM2EX is OK.
;
; KEYWORDS:	rigid		when set does rigid rotation
;
; CALLS: SXT_CEN,GT_PIX_SIZE,GET_RB0P,RFITS,IMGSCL2,SOL_ROT,GET_RECT,
;	 RECT_ROT,ANYTIM2EX
;
;       HISTORY: Drafted by A. McAllister, early Mar. 1993
;		 Fixed and released, AMcA  29-Apr-93.
;		 Adjusted the corner, AMcA 12-may-93.
;		 Fixed definition of RES, AMcA 19-may-93.
;		 Replaced direct references to index tags, AMcA 1-jun-93.
;		 Adjusted for new index routines, AMcA 6-jul-93.
;		 Added code to adjust for corners over the limb, AMcA 1-dec-93. 
;-
FUNCTION gbo_pfi,gboin,indx,gbotim,rigid=rigid

his_index,/enable

roll= 1.0				;SXT roll angle

res=2-gt_res(indx)			;get PFI resolution,FR=2,HR=1,QR=0
sxt_pix = gt_pix_size(indx)
sunc=sxt_cen(indx)			;gives, x,y,time,day
dat_trg = anytim2ex(indx)		;target time
ans=get_rb0p(indx)
sunr=ans(0)/sxt_pix			;target radius in FR pixels

if n_params() eq 2 then  begin

  mag_img=rfits(gboin,/scale) 				;get magnetogram 
  dat_mag=fid2ex(strmid(gboin,strlen(gboin)-11,11))

endif else begin

  mag_img=gboin
  dat_mag=anytim2ex(gbotim)

endelse

imgsz=256*2^res
fac=4/2^res

trgdt=[sunc(0:1)/fac,2*sunr/fac,2*sunr/fac,imgsz,imgsz] ;make target, sunr -> diameter
match_img=imgscl2(trgdt,mag_img,[0,0]) 			;match to target, at PFI res

;add roll correction here or in IMGSCL2.;
;match_img=rott(match_img,roll,sunc(0)/fac,sunc(1)/fac)

;can we parse the date from the fits header?

dd0=int2secarr(anytim2ints(dat_trg),anytim2ints(dat_mag))/86400. ;decimal days
ndd0=-dd0

;get coords of PFI

corn=gt_corner(indx)
shape_cmd=gt_shape(indx)

lft= corn(0)+1				   ;left edge
low= corn(1)+1				   ;lower edge
rgt= corn(0)+shape_cmd(0)         	   ;right edge
upp= corn(1)+shape_cmd(1)         	   ;upper edge

corns=fltarr(2,4)
corns(*,0)=[lft,low]
corns(*,1)=[rgt,low]
corns(*,2)=[lft,upp]
corns(*,3)=[rgt,upp]

trgt=[trgdt(0:1),trgdt(2)/2]
bck_crn=sol_rot(corns(0,*),corns(1,*),trgt,dat_trg,ndd0)

;trap for corners off the limb

ssoff=where(bck_crn(0,*) eq -1.0)

case n_elements(ssoff) of

	1: begin
	
	   if ssoff ne -1 then begin		;catch null where return
	
	     print,'One corner is behind the limb.'
	   
	     ssoff=ssoff(0)
	     case fix(ssoff) of
	   
	         ;pad these a bit? Only need to pad in x really.
	   
	   	 0: bck_crn(*,ssoff)=[bck_crn(0,ssoff+2),bck_crn(1,ssoff+1)]
	   	
	   	 1: bck_crn(*,ssoff)=[bck_crn(0,ssoff+2),bck_crn(1,ssoff-1)]
	   	
	   	 2: bck_crn(*,ssoff)=[bck_crn(0,ssoff-2),bck_crn(1,ssoff+1)]
	   	
	   	 3: bck_crn(*,ssoff)=[bck_crn(0,ssoff-2),bck_crn(1,ssoff-1)]
	
	     endcase
	   endif
	   end
	
	2: begin
	
	   print,'Two corners are behind the limb.'
	   ;assume for now its both east, or both west corners.
	   
	   case fix(total(ssoff)) of
	   
	        1: print,'Both bottom corners are over the limb.'
	   
	      	2: begin
	      	
	      	   side=bck_crn(1,3)-bck_crn(1,1)
	      	   
	      	   bck_crn(*,0)=[bck_crn(0,1)-side,bck_crn(1,1)]
	      	   bck_crn(*,2)=[bck_crn(0,3)-side,bck_crn(1,3)]
	      	
	      	   end
	      	
	      	4: begin
	      	
	      	   side=bck_crn(1,2)-bck_crn(1,0)
	      	
	      	   bck_crn(*,1)=[bck_crn(0,0)+side,bck_crn(1,0)]
	      	   bck_crn(*,3)=[bck_crn(0,2)+side,bck_crn(1,2)]
	      	   	      	
	      	   end
	      	
	      	5: print,'Both upper corners are over the limb.' 
	      	
	   endcase
	   end
	
	3: begin
	
	   print,'Three corners are behind the limb.'
	
	   ;if needed fix using imgsz to estimate the other three
	   ;corners.
	
	   end

	4: begin
	
	   print,'The image is on the back of the Sun.'
	   return,-1
	     
	   end
	   
     else:  ;do nothing
	
endcase

rgtt= max(bck_crn(0,*))	;edges of box inclosing inverse rotated 
loww= min(bck_crn(1,*))	;region
lftt= min(bck_crn(0,*))
uppp= max(bck_crn(1,*))

if keyword_set(rigid) then begin
  rot_mag=rect_rot(match_img,dat_mag,dd0,lftt,rgtt,loww,uppp,target=trgt,/rigid) 
endif else begin
;what happens if one corner (or more are off the limb).
  rot_mag=rect_rot(match_img,dat_mag,dd0,lftt,rgtt,loww,uppp,target=trgt)
endelse

return,get_rect(rot_mag,lft,rgt,low,upp)

end
