;+
; NAME: gbo_rot_t
;
;
; PURPOSE: To take a GBO image and rotate it to extrapolate
;	   backward or forward in time.
;
; METHOD: Take the image, map it to heliocentric coordinates, add the
;	  appropriate differential rotation, and map it back. This 
;	  version is ment for optical full disk images, and selects the
;	  on disk pixels before doing the rotation. The input images 
;	  must be tractable to the limbfinder, or else you may run out
;	  of memory. The pixels are mapped individually and then mapped 
;	  back to the disk, so there wil be over writing in the hemisphere
;	  being rotated away from disk center, and gaps on the hemisphere
;	  being rotated towards disk center. Blank pixels are filled in
;	  by simple east-west averaging of the nearest neighbors, but near
;	  the limb everything is blank and no replacement is done.
;
;
; CALLING SEQUENCE:
;			out=gbo_rot_t(image,date,delt[,roll=roll])
;
;
; PARAMETERS: 	image	the image data.
;		date 	the time of the image, in standard 7
;			part form (use fmt_tim(index), etc.).
;		delt	the rotation time, in decimal days.
;
; KEYWORDS:	roll 	optional roll correction. For SXT 
;			images you can use 1.0.
;
; RESTRICTIONS:	There is a limit (somewhere between 116,000 and 
;		262,000) to the number of pixels this will handle.
;		When the number is too high it runs out of memory
;		in VEC_H2P.                                       
;
; CALLS: SOL_ROT, SGN, TARGET, COORD_L2V
;
;
; HISTORY: Drafted by A.McAllister, Feb. 1993.                    
;	   Made correction to add ROLL in call to SOL_ROT, jun 15, 1993.     
;
;-
FUNCTION gbo_rot_t,image,date,delt,roll=roll

 trgt=target(image)

 if keyword_set(roll) eq 0 then roll=0

 imgsz=size(image)

 linpix=lindgen(long(imgsz(1))*long(imgsz(2)))
 xy=coord_l2v(linpix,imgsz) + .5 		;add .5 to get pixel center
 
;reduce to only on disk pixels

 trgt=float(trgt)				;sqrt bug?
 x0=xy(0,*)-trgt(0)				;offset by suncenter
 y0=xy(1,*)-trgt(1)		
 dsk_pix_ss=where(sqrt(x0*x0 + y0*y0) le trgt(2))      ;on disk pixels
 pixx=xy(0,dsk_pix_ss)
 pixy=xy(1,dsk_pix_ss)

 newpix=sol_rot(pixx,pixy,trgt,date,delt,stretch=stretch,roll=roll) ;do rotation

 od_ss=where(newpix(0,*) ne -1)         ;on disk pixels

;for now assuming square pixels

; stretch=(0.5>stretch(od_ss)<2.0)	;limit the transform and eliminate otl pix

;start with lower left corner, figure other three relative to this.

; pix1_ss=newpix(*,od_ss)		 	;start with pixel of center
; pix1_ss(0,*)=pix1_ss(0,*)-.5*stretch  	;move to left border
; pix1_ss(1,*)=pix1_ss(1,*)-.5 	   		;move to lower left corner
						
; pix2_ss=pix1_ss
; pix2_ss(0,*)=pix2_ss(0,*)+stretch		;lower right corner

; pix3_ss=pix1_ss
; pix3_ss(1,*)=pix3_ss(1,*)+1			;upper left corner

; pix4_ss=pix2_ss
; pix4_ss(1,*)=pix3_ss(1,*)			;upper right corner

; pix1_ss=fix(pix1_ss)				;get pixel coordinates by rounding down
; pix2_ss=fix(pix2_ss)
; pix3_ss=fix(pix3_ss)
; pix4_ss=fix(pix4_ss)

;now the areas to add to each pixel

; pix1_ar=abs(pix2_ss(0,*)-(newpix(0,od_ss)-.5*stretch)) * $
;          abs(pix3_ss(1,*)-(newpix(1,od_ss)-.5))
; pix2_ar=abs(pix2_ss(0,*)-(newpix(0,od_ss)+.5*stretch)) * $
;         abs(pix3_ss(1,*)-(newpix(1,od_ss)-.5))
; pix3_ar=abs(pix2_ss(0,*)-(newpix(0,od_ss)-.5*stretch)) * $
;         abs(pix3_ss(1,*)-(newpix(1,od_ss)+.5))
; pix4_ar=abs(pix2_ss(0,*)-(newpix(0,od_ss)+.5*stretch)) * $
;         abs(pix3_ss(1,*)-(newpix(1,od_ss)+.5))

;to normalize

;now assign the pixels

 out=fltarr(imgsz(1),imgsz(2))

; out(pix1_ss(0,*),pix1_ss(1,*))=pix1_ar*image(pixx(od_ss),pixy(od_ss))/stretch
; out(pix2_ss(0,*),pix2_ss(1,*))=pix2_ar*image(pixx(od_ss),pixy(od_ss))/stretch
; out(pix3_ss(0,*),pix3_ss(1,*))=pix3_ar*image(pixx(od_ss),pixy(od_ss))/stretch
; out(pix4_ss(0,*),pix4_ss(1,*))=pix4_ar*image(pixx(od_ss),pixy(od_ss))/stretch

 out(newpix(0,od_ss),newpix(1,od_ss))=image(pixx(od_ss),pixy(od_ss))

 zeros=where( out eq 0 )

 out(zeros) = (out(zeros+1)+out(zeros-1))/2.0 	;use simple averaging to fill blanks

return,out

end