;+
;       NAME:
;             gbo_scale2
;
;       PURPOSE:
;               To take two images of the sun and resize, and repostion
;               the second one so that it lies on top of the first one.
;      
;       METHOD: Both images are passed through gen_loc to find the centroid
;               and the axial diameters, the second is then resized and 
;               reposisitioned to match the first (TARGET).
;               When the target, or both images, are SXT images set the SXT
;               keyword to 1 or 2, respectively.
;
;	CALLING SEQUENCE:
;
;               outimg=gbo_scale2(trgimg,gboimg[,indx1,indx2][,sxt=sxt]
;                                                            [,mktv=mktv])
;
;               indx1 and indx2 are index entries for trgimg and gboimg
;               respectively. Only needed when these are SXT images. Then
;               you must also set the SXT keyword (see below).
;       NOTES:
;             When working with fits images the top 5 lines or so 
;             should be removed before input. These contain the 
;             header and gum things up.
;
;	PARAMETERS:
;	   gboimg      the input GBO image.
;          trgimg      the target image to which it is to be matched.
;	   outimg      the rescaled output image.
;
;	KEYWORDS:
;                SXT    This is the flag indicating wiether the images
;                       are sxt images of not. If the target is an SXT
;                       image set this to 1, if both are SXT images,
;                       set it to two.
; 
;	         MKTV   if you want to put outimg on a window set /mktv.
;
;       OUTPUT TYPE:
; 
;       CALLS: GEN_LOC 
;
;	HISTORY:
;	   Procedure developed to shrink BBSO Halpha image and 
;	   place it correctly in a 512x512 SXT image 
;	   Written by L. Acton  7-nov-91
;	   Modified for generality  10-apr-92
;          Added automatic sun finder, A. McA. 16-apr-92
;          Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;          Version 2. Rewrote for general sized matrices, using dsk_loc.
;          changed calling sequence to include target image, A.McA 24-apr-92
;          Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;          Added fractional pixel repositioning, A.McA. 7-may-92
;          Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;          Output variable type now same as input type, A.McA. 2-dec-92
;
;-
FUNCTION gbo_scale2,trgimg,gboimg,indx1,indx2,sxt=sxt,mktv=mktv

;find location and size of the target image

if keyword_set(sxt) eq 0 then sxt=0

if sxt gt 0 then begin        ;find target diameters & centriod

  print, 'target sxt image'

  gen_loc,trgimg,indx1,CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt,/SXT

endif else begin

  print, 'target not sxt image'

  gen_loc,trgimg,CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt 

endelse

x0=cent(0)
y0=cent(1)

gbosz=size(gboimg)                            ;size of input image

if (sxt eq 2) then begin                   ;find input diameters

  print, 'input sxt image'

  gen_loc,gboimg,indx2,HORZRAD=hzrd,VERTRAD=vtrd,/SXT

endif else begin

  print, 'input not sxt image'

  gen_loc,gboimg,HORZRAD=hzrd,VERTRAD=vtrd   

endelse

print,hzrd,vtrd

ha=congrid(gboimg,gbosz(1)*rdhz/hzrd,gbosz(2)*rdvt/vtrd,/interp)   ;resize the input

if (sxt eq 2) then begin                       ;find center of resized image

  indx2.sxt.imgparam=indx1.sxt.imgparam        ;swap in the target resolution

  gen_loc,ha,indx2,CENTROID=cent,/SXT          ;for now assume its basically
                                               ;unchanged
                                              
endif else begin

  gen_loc,ha,CENTROID=cent

endelse

dltx=fix(x0-cent(0))                      ;difference to one pixel
dlty=fix(y0-cent(1))

hasz = size(ha)                           ;size of resized image
trgsz=size(trgimg)                        ;size of target image

imgout=replicate(ha(0,0),trgsz(1),trgsz(2))  ;output image same as input        

;copy resized input to output, while centering to nearest pixel

imgout((0 > dltx):((trgsz(1)-1) < (dltx+hasz(1)-1)),         $
       (0 > dlty):((trgsz(2)-1) < (dlty+hasz(2)-1))) =       $
     ha((0 > (-1*dltx)):((hasz(1)-1) < (trgsz(1)-1-dltx)),   $
        (0 > (-1*dlty)):((hasz(2)-1) < (trgsz(2)-1-dlty)))

;shift the last fraction, uses interpolation which may change intensity range
;use min value for 'uncovered' pixels to avoid shifting intensity range.

fdltx=(x0-cent(0))-fix(x0-cent(0))        ;fractional pixel difference
fdlty=(y0-cent(1))-fix(y0-cent(1))
 
imgout=poly_2d(imgout,[-fdltx,0,1,0],[-fdlty,1,0,0],1,missing=ha(0,0))

if keyword_set(mktv) ne 0 then tv,imgout             ;plot on screen

return,imgout

end

