;+
;  NAME: GEN_LOC
;
; PURPOSE: 
;          a general disk locator for all types of image.
;
; METHOD:
;          for SXT images this calls FIT_LIMB.
;          for other images it uses DSK_LOCG.
;	   Availablity of ATT files is checked for and when absent 
;	   FIT_LIMB is called interactively.
;
; CALLING SEQUENCE:
;                    gen_loc,image,[indx],[KEYWORD=variable,..]
;
; KEYWORDS:
;
;          SXT          when set indicates SXT data, requires indx.
;          ROUGH        a fast rough fit, may not work well with sxt.
;          BFRAD        RADIUS of best fit circle, use w/ sxt option.
;	   CIR		Use to invoke a circle fit to a GBO image.
;          CENTROID     horizontal, vertical centers, floating point pixels.
;          HORZRAD      horizontal DIAMETER, floating point pixels.
;          VERTRAD      vertical DIAMETER, floating point pixels.
;          EASTLIMB     east limb, in pixels.
;          WESTLIMB     west limb, in pixels.
;          NPOLE        north pole, in pixels.
;          SPOLE        south pole, in pixels.
;	   QUIET	pass through to DSK_LOCG, no display.
;	   INTER	pass through to DSK_LOCG, interactive point edit.
;
; NOTES: When run with /ROUGH, it is much faster, but the results may be 
;	 off on the order of a FR pixel (due to round off). There may also 
;        be some error message generation.
;
;	 It is assumed that elliptical GBO images have the major axis along
;	 the vertical. This may be grneralized in the future.
;
; RETURN TYPE: FLOAT
;
; CALLS:   SXT_CENTER, DSK_LOCG, GET_RB0P, PNT_EXST, ATT_EXST
;
; HISTORY:
;           drafted by A. McAllister, May 5, 1992.
;	    added evironmental switch for PNT data, A.McA. Dec. 1992.
;	    corrected BFRAD calc for GBO case, AHMcA, mar. 1993.
;	    replaced PNT data check, june. 22, 1993.
;	    added ATT data check, AMcA, jan. 12, 1994.
;	    replaced SXT_CENTER with FIT_LIMB and set kill=5, AMcA, feb. 11, 1994.
;	    added the SUMMED_PIXELS keyword to the FIT_LIMB calls
;	    and removed the resolution corrections, AMcA, feb. 15, 1994.
;	    replaced DSK_LOCB with DSK_LOCG and added QUIET and INTER
;	    keywords, AMcA, aug. 24, 1994.
;	    added CIR keyword, as the test on BFRAD didn't seem to work, nov 2, 1994.
;
;-
PRO gen_loc,imgin,indx,CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt, $
             EASTLIMB=mnhz,WESTLIMB=mxhz,NPOLE=mxvt,SPOLE=mnvt, $
             SXT=sxt,ROUGH=rough,BFRAD=rad,inter=inter,quiet=quiet,cir=cir

if (keyword_set(sxt) eq 0) then sxt=0               ;test for SXT flag

if (sxt ne 0 ) then goto,sxt_dat                    ;go to SXT case

;if its zero

if keyword_set(cir) then begin

	trgt=dsk_locg(imgin,100,inter=inter,quiet=quiet,/cir)	;returns centroid,radius
	
	cent = [trgt(0),trgt(1)]
	rdhz = 2.0*trgt(2)
	rdvt = 2.0*trgt(2)
	rad  = trgt(2)

endif else begin

	trgt=dsk_locg(imgin,100,inter=inter,quiet=quiet)	;returns centroid, minor axis,
						   		;eccentricity, angle
	cent = [trgt(0),trgt(1)]
	
;factor in the angle first? Currently assumes major axis is vertical.

	rdhz = 2.0*trgt(2)	
	rdvt = 2.0*sqrt((trgt(2)*trgt(2))/(1 - trgt(3)*trgt(3)))
	rad  = (rdhz + rdvt)/4.0
	  
endelse                  

mnhz = cent(0) - rdhz/2.0
mxhz = cent(0) + rdhz/2.0

mnvt = cent(1) - rdvt/2.0
mxvt = cent(1) + rdvt/2.0

print,'GEL_LOC: centroid is: ',cent,', the diameters are: ',rdhz,rdvt,', and the radius is: ',rad

return
stop

;SXT call

sxt_dat:

resol=2^gt_res(indx)

if NOT(keyword_set(rough)) then begin

  if not (pnt_exst(indx) or att_exst(indx)) then begin 

    print,'GEN_LOC: No pointing data, calling SXT_CENTER interactively.'

    fit_limb,imgin,cnthz,cntvt,rad,index=indx,oblateness=obl,ob_angle=oba,kill=5 $
           ,/interactive,/summed_pixels

;the nordpnt option should be obsolete now 2/11/94.

  endif else  begin
  
    fit_limb,imgin,cnthz,cntvt,rad,index=indx,oblateness=obl,ob_angle=oba,kill=5, $
       /quiet,/summed_pixels

  endelse

endif else begin

;for a pretty rough result use /fast, but it is MUCH faster.

  if not (pnt_exst(indx) or att_exst(indx)) then begin 
  
      fit_limb,imgin,cnthz,cntvt,rad,index=indx,oblateness=obl,ob_angle=oba,kill=5 $
          ,/interactive,/fast,/summed_pixels 
           
  endif else  begin
  
     fit_limb,imgin,cnthz,cntvt,rad,index=indx,oblateness=obl,ob_angle=oba,kill=5 $
          ,/quiet,/fast,/summed_pixels  
          
  endelse

;slightly faster, radius taken from computed value, not from image

;  cent=sxt_cen(indx)
;  cnthz=cent(*,0)
;  cntvt=cent(*,1)
;  ans=get_rb0p(indx)
;  rad=ans(0)/gt_pix_size()

endelse

mnhz=(cnthz-rad)
mxhz=(cnthz+rad)

mnvt=(cntvt-rad)
mxvt=(cntvt+rad)

rdhz=mxhz-mnhz   	  		;get diameter
rdvt=mxvt-mnvt

cnthz=cnthz
cntvt=cntvt

mnhz=fix(mnhz)
mxhz=fix(mxhz)

mnvt=fix(mnvt)
mxvt=fix(mxvt)

rad=rad           		;best fit circle radius
cent=[cnthz,cntvt]

return

end
