;+
;       NAME: gradls
;
;       PURPOSE: To do a least squares fit to a non-linear function
;		 with a parabolic expansion of chi square.		 
;
;       METHOD: Uses the gradient in chi-squared space to search for
;		minimum chi-squared and optimize the parameters for
;		a curve to get the best fit to a set of data points. 
;
;       CALLING SEQUENCE: fit=gradls(x,y,a,delta,chisqr,mode,sigmay,funct="circle")
;
;       PARAMETERS:	x	data points for independent variable
;			y	data points for dependent variable
;			mode	determines weighting method
;				+1 (instrumental) weight(i) = 1./sigmay(i)**2
;				 0 (no wieghting) weight(i) = 1.
;				-1 (statistical)  weight(i) = 1./y(i)
;			a       fit parameters
;                       sigmay  standard deviations of the y data points
;			delta	parameter increments
;			chisqr	reduced chi square for fit
;			fit	the point on the fit curve: [xfit,yfit] 
;
;	KEYWORDS:	funct	Set to indicate what function to fit to.
;				Current options are:
;				circle (default) and ellipse.
;
;	CALLS: 	CIRCLE_FN,ELLIPSE_FN,FCHISQ
;
;       HISTORY: Converted from Bornmann C procedure, AMcA, August 1994.
;		 added trap for convergent chi squares, AMcA, Aug. 24 1994
;-
FUNCTION gradls,x,y,a,delta,chisqr,mode,sigmay,funct=funct

if not keyword_set(funct) then funct = "circle"		;default

npts = n_elements(x)
nfree = npts - n_elements(a)

;original fit

 case funct of

	"circle":	nfits = circle_fn(a,x,y)
	"ellipse":	nfits = ellipse_fn(a,x,y)
	else:		begin 
				print,"GRADLS: function not properly defined"
				return,-1
			end	
 endcase

 chisq1 = fchisq(x,y,nfits(0,*),nfits(1,*),mode,sigmay,nfree)
 
 if (chisq1 eq 1) then print, "exact initial quess"

;get chisqr gradient for each parameter

 del = 0.001 * delta		;local gradient, very small steps
 sum = 0
 grad = replicate(0.0,n_elements(a))
 
 for i = 0, n_elements(a)-1 do begin
 
   a(i) = a(i) + del(i)
   case funct of

	  "circle":	nfits = circle_fn(a,x,y)
	  "ellipse":	nfits = ellipse_fn(a,x,y)

   endcase

   a(i) = a(i) - del(i)
   chinew = fchisq(x,y,nfits(0,*),nfits(1,*),mode,sigmay,nfree)
   grad(i) = (chisq1 - chinew)/del(i)	;negative if chinew gt chi1
   sum = sum + (grad(i)*grad(i))
   
 endfor

 if abs(sum gt 1e-7) then grad = delta * grad / sqrt(sum)
 
; chi square at new point
; make sure chi squared decreases

 repeat begin
 
 	a = a + grad
	case funct of

	  "circle":	nfits = circle_fn(a,x,y)
	  "ellipse":	nfits = ellipse_fn(a,x,y)

 	endcase
 	
 	chisq2 = fchisq(x,y,nfits(0,*),nfits(1,*),mode,sigmay,nfree)
 	
 	if chisq1 le chisq2 then begin
 	
 		a = a - grad		;make sure point 2 is between
 		grad = grad/2.0		;point 1 and the minimum
 		
 	if total(abs(grad)) lt 1e-5 then goto, exit1   ;empirical fix
 		
 	endif
; print,chisq1,chisq2
 endrep until chisq1 gt chisq2

 exit1:
;increment parameters until chi squared starts to increase

repeat begin

	a = a + grad
	case funct of

	  "circle":	nfits = circle_fn(a,x,y)
	  "ellipse":	nfits = ellipse_fn(a,x,y)

 	endcase
 	
 	chisq3 = fchisq(x,y,nfits(0,*),nfits(1,*),mode,sigmay,nfree)
 
	if chisq2 gt chisq3 then begin
 	
 		chisq1 = chisq2
 		chisq2 = chisq3
 		
 	endif
;print,chisq1,chisq2,chisq3	
endrep until chisq3 ge chisq2			;change from gt to ge

 denom = ( chisq3 - 2*chisq2 + chisq1 )		;check for small values?
 delt = ((chisq3 - chisq2) / denom + 0.5)
 a = a - grad * delt				;find parabola minimum

 case funct of

	  "circle":	nfits = circle_fn(a,x,y)
	  "ellipse":	nfits = ellipse_fn(a,x,y)

 endcase
 		
 chisqr = fchisq(x,y,nfits(0,*),nfits(1,*),mode,sigmay,nfree)
 
 if chisq2 lt chisqr then begin			;what's this for?
 
 	a = a + (delt -1.0) * grad
		
 	case funct of

	  "circle":	nfits = circle_fn(a,x,y)
	  "ellipse":	nfits = ellipse_fn(a,x,y)

 	endcase
 		
 	chisqr = chisq2

 endif
 	
return,nfits

end
