;+
; NAME:
;        IMG_MATCH
;
; PURPOSE:
;         To take two images of the sun and resize, and repostion
;         the second one so that it lies on top of the first one.
;      
; METHOD: Both images are passed through GEN_LOC to find the centroid
;         and the axial diameters, the second is then resized and 
;         reposisitioned to match the first (the TARGET).
;         The SXT variable indicates which images, if any, are SXT images. 
;
; CALLING SEQUENCE:
;
;          outimg=img_match(trgimg,gboimg,sxt[,indx1][,indx2][,mktv=mktv])
;                                                            
; PARAMETERS:
;
;	    outimg      the rescaled output image.
;	    gboimg      the input image.
;           trgimg      the target image to which it is to be matched.
; 	    sxt         a binary two-vector mask: 1=sxt, 0=gbo.
;
;           indx1       a single index entry for either trgimg or gboimg. 
;           indx2       the second index when both are SXT images.       
;
;
; KEYWORDS:
;	   MKTV   	if you want to put outimg on a window set /mktv.
;	   QUIET	pass through to DSK_LOCG, no display.
;	   INTER	pass through to DSK_LOCG, interactive point edit.

;
; RESTRICTIONS:
;
;             When working with fits images the top few lines or so 
;             should be removed before input. These contain the 
;             header and gum things up.
;
;	      When matching an SXT to a GBO image the sizes must be the
;	      same to within about 5%. In all other cases there is no
;	      restriction on relative size.
;
; RETURN TYPE: Same as input.
; 
; CALLS:     GEN_LOC                                                               
;
; HISTORY:
;	   Procedure developed to shrink BBSO Halpha image and place it
;	   correctly in a 512x512 SXT image. Written by L. Acton  7-nov-91
;	   Modified for generality  A. McAllister 10-apr-92
;          Added automatic sun finder, A. McA. 16-apr-92
;          Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;
;          Version 2. Rewrote for general sized matrices, using DSK_LOC.
;          changed calling sequence to include target image, A.McA 24-apr-92
;          Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;          Added fractional pixel repositioning, A.McA. 7-may-92
;          Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;          Output variable type now same as input type, A.McA. 2-dec-92
;
;	   Version 3. Reworked for flexiable image type. Renamed from
;	   GBO_SCALE2, A.McA. 14-dec-92.
;	   Minor change to match DSK_LOCG use, added QUIET and INTER 
;	   keywords, A.McA. 24-aug-94.
;          Changed congrid call to use cubic convolution interpolation, A.McA. 24-aug-94
;
;-
FUNCTION img_match,trgimg,gboimg,sxt,indx1,indx2,mktv=mktv, $
		 inter=inter,quiet=quiet

;print,n_params(0)

;add test for n_params(0) eq 3    total(sxt) should be 0

if n_params(0) eq 4 then begin		   ;only one sxt image

  if total(sxt) ne 1 then begin		   ;trap sxt/index mismatch

    print,"The number of indices dosn't match SXT vector"
    return,-1
    stop

  endif

  if sxt(0) eq 1 then indxt=indx1
  if sxt(1) eq 1 then indxi=indx1

endif else begin

  if n_params(0) eq 5 then begin		   ;two sxt images

    if total(sxt) ne 2 then begin		   ;trap sxt/index mismatch

      print,"The number of indices dosn't match SXT vector"
      return,-1
      stop

    endif

    indxt=indx1
    indxi=indx2

  endif

endelse

;find location and size of the target image

if sxt(0) eq 0 then begin                  ;find target diameters & centriod

  print, 'Target not SXT image'

  gen_loc,trgimg,CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt,inter=inter,quiet=quiet ;ellipse fit

endif else begin

  print, 'Target SXT image'

  gen_loc,trgimg,indxt,CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt,/SXT

endelse

x0=cent(0)
y0=cent(1)

gbosz=size(gboimg)                            ;size of input image

if sxt(1) eq 0 then begin                     ;find input diameters & centroid

  print, 'Input not SXT image'

  gen_loc,gboimg,HORZRAD=hzrd,VERTRAD=vtrd,inter=inter,quiet=quiet 	;ellipse fit

endif else begin

  print, 'Input SXT image'

  gen_loc,gboimg,indxi,HORZRAD=hzrd,VERTRAD=vtrd,/SXT

endelse

print,rdhz,rdvt,hzrd,vtrd

;resize the input

ha=congrid(gboimg,gbosz(1)*rdhz/hzrd,gbosz(2)*rdvt/vtrd,/cubic)   

if sxt(1) eq 1 then begin                      ;find center of resized image

  if sxt(0) eq 1 then begin

    indxi.sxt.imgparam=indxt.sxt.imgparam        ;swap in the target resolution

  endif else begin

    indxi.sxt.imgparam=indxi.sxt.imgparam       ;for now do nothing, assume same scale

  endelse

  gen_loc,ha,indxi,CENTROID=cent,/SXT          	;for now assume its basically
                                               	;unchanged
                                              
endif else begin

  gen_loc,ha,CENTROID=cent,BFRAD=rad,/quiet	;circle fit

endelse

dltx=fix(x0-cent(0))                      ;difference to one pixel
dlty=fix(y0-cent(1))

hasz = size(ha)                           ;size of resized image
trgsz=size(trgimg)                        ;size of target image

imgout=replicate(ha(0,0),trgsz(1),trgsz(2))  ;output image same as input        

;copy resized input to output, while centering to nearest pixel

imgout((0 > dltx):((trgsz(1)-1) < (dltx+hasz(1)-1)),         $
       (0 > dlty):((trgsz(2)-1) < (dlty+hasz(2)-1))) =       $
     ha((0 > (-1*dltx)):((hasz(1)-1) < (trgsz(1)-1-dltx)),   $
        (0 > (-1*dlty)):((hasz(2)-1) < (trgsz(2)-1-dlty)))

;shift the last fraction, uses interpolation which may change intensity range
;use min value for 'uncovered' pixels to avoid shifting intensity range.

fdltx=(x0-cent(0))-fix(x0-cent(0))        ;fractional pixel difference
fdlty=(y0-cent(1))-fix(y0-cent(1))
 
imgout=poly_2d(imgout,[-fdltx,0,1,0],[-fdlty,1,0,0],1,missing=ha(0,0))

if keyword_set(mktv) ne 0 then tv,imgout             ;plot on screen

return,imgout

end

