;+
;       NAME:
;             imgscl
;
;       PURPOSE:
;               To take two images of the sun and resize, and repostion
;               the second one so that it lies on top of the first one.
;		Like GBO_SCALE2, except it takes traget data as input,
;		rather than the target image.
;      
;       METHOD: The image is passed through gen_loc to find the centroid
;               and the axial diameters, it is then resized and reposisitioned 
;               to match the data in TRGDT.
;               When the target, or both images, are SXT images set the SXT
;               keyword to 1 or 2, respectively.
;
;	CALLING SEQUENCE:
;
;               outimg=imgscl(trgdt,gboimg[,indx1,indx2][,sxt=sxt][,mktv=mktv])
;
;		trgdt is a 6 component vector which contains: the centroid (0,1), 
;               the radii(horizontal,vertical;2,3), and the pixel size (x,y:4,5) 
;		of the target image.
;
;	        gboimg is the input GBO image.
;
;               indx1 is the index entry for the target image and indx2 is 
;               the index entry for gboimg. These are necessary for
;               SXT images. Then you must also set the SXT keyword (see below).
;
;       NOTES:
;               When working with fits images the top 5 lines or so 
;               should be removed before input. These contain the 
;               header and gum things up.
;
;
;	KEYWORDS:
;                SXT    This is the flag indicating weither the images
;                       are sxt images or not. If only the target is an
;                       sxt image the keyword should be set to 1, if
;                       they both are it should be set to 2.
; 
;	         MKTV   if you want to put outimg on a window set /mktv.
;
;       OUTPUT TYPE:
;
;       CALLS: GEN_LOC
;
;	HISTORY:
;	   Procedure developed to shrink BBSO Halpha image and 
;	   place it correctly in a 512x512 SXT image 
;	   Written by L. Acton  7-nov-91
;	   Modified for generality  10-apr-92
;          Added automatic sun finder, A. McA. 16-apr-92
;          Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;          Version 2. Rewrote for general sized matrices, using dsk_loc.
;          changed calling sequence to include target image, A.McA 24-apr-92
;          Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;          Added fractional pixel repositioning, A.McA. 7-may-92
;          Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;	   Rewritten from GBO_SCALE2, takes target info as input, A.McA. 6-oct-92
;          Output variable type now same as input type, A.McA. 2-dec-92
;
;-
FUNCTION imgscl,trgdt,gboimg,indx1,indx2,sxt=sxt,mktv=mktv

x0=trgdt(0)
y0=trgdt(1)

gbosz=size(gboimg)                            ;size of input image

if (sxt eq 2)  then begin                          ;find input diameters

  print, 'input sxt image'

  gen_loc,gboimg,indx1,HORZRAD=hzrd,VERTRAD=vtrd,/SXT

endif else begin

  print, 'input not sxt image'

  gen_loc,gboimg,HORZRAD=hzrd,VERTRAD=vtrd   

endelse

ha=congrid(gboimg,gbosz(1)*trgdt(2)/hzrd,gbosz(2)*trgdt(3)/vtrd,/interp)   ;resize the input

if (sxt eq 2) then begin                       ;find center of resized image

  indx2.sxt.imgparam=indx1.sxt.imgparam        ;swap in the target resolution

  gen_loc,ha,indx2,CENTROID=cent,/SXT          ;for now assume its basically
                                               ;unchanged
                                              
endif else begin

  gen_loc,ha,CENTROID=cent

endelse

dltx=fix(x0-cent(0))                      ;difference to one pixel
dlty=fix(y0-cent(1))

hasz = size(ha)                           ;size of resized image

imgout=replicate(ha(0,0),trgdt(4),trgdt(5))  ;initialize output image

;copy resized input to output, while centering to nearest pixel

imgout((0 > dltx):((trgdt(4)-1) < (dltx+hasz(1)-1)),         $
       (0 > dlty):((trgdt(5)-1) < (dlty+hasz(2)-1))) =       $
     ha((0 > (-1*dltx)):((hasz(1)-1) < (trgdt(4)-1-dltx)),   $
        (0 > (-1*dlty)):((hasz(2)-1) < (trgdt(5)-1-dlty)))

;shift the last fraction, uses interpolation which may change intensity range
;use min value for 'uncovered' pixels to avoid shifting intensity range.

fdltx=(x0-cent(0))-fix(x0-cent(0))        ;fractional pixel difference
fdlty=(y0-cent(1))-fix(y0-cent(1))
 
imgout=poly_2d(imgout,[-fdltx,0,1,0],[-fdlty,1,0,0],1,missing=ha(0,0))

if keyword_set(mktv) ne 0 then tv,imgout             ;plot on screen

return,imgout

end

