;+
; NAME:
;         IMGSCL2
;
; PURPOSE:
;         To an image of the sun and resize, and repostion it to match
;         an input centroid and radius.
;	  Like IMG_MATCH, except it takes traget data as input, rather
;	  than the target image.
;      
; METHOD: The image is passed through GEN_LOC to find the centroid
;         and the axial diameters, it is then resized and reposisitioned 
;         to match the data in TRGDT.
;         The SXT variable indicates which images, if any, are SXT images.
;
; CALLING SEQUENCE:
;
;         outimg=imgscl2(trgdt,gboimg,sxt[,indx1][,indx2][,mktv=mktv])
;
; PARAMETERS:
;
;	    outimg      the rescaled output image.
;	    trgdt       a 6 component vector which contains: the CENTROID (0,1), 
;                       the DIAMETERS(horizontal,vertical;2,3), and the pixel 
;	                size (x,y:4,5) of the target image. 
;	    gboimg      the input image.
; 	    sxt         a binary two-vector mask: 1=sxt, 0=gbo.
;           indx1       a single index entry for either trgimg or gboimg.
;           indx2       the second index when both are SXT images.
;
; KEYWORDS:
; 
;	   MKTV   	if you want to put outimg on a window set /mktv.
;	   QUIET	pass through to DSK_LOCG, no display.
;	   INTER	pass through to DSK_LOCG, interactive point edit.
;       
; RESTRICTIONS:
;             When working with fits images the top few lines or so 
;             should be removed before input. These contain the 
;             header and gum things up.
;
;	      When matching an SXT to a GBO image the sizes must be the 
;	      same to within about 5%. In all other cases there is no
;	      restriction on relative size.
;
; RETURN TYPE: Same as input.
;
; CALLS: GEN_LOC  
;
; NOTE:      Expects the indeices to have .HIS tags and that "his_index,/enable"
;	     has been invoked.                                                    
;
; HISTORY:
;	   Procedure developed to shrink BBSO Halpha image and 
;	   place it correctly in a 512x512 SXT image 
;	   Written by L. Acton  7-nov-91
;	   Modified for generality  10-apr-92
;          Added automatic sun finder, A. McA. 16-apr-92
;          Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;
;          Version 2. Rewrote for general sized matrices, using dsk_loc.
;          changed calling sequence to include target image, A.McA 24-apr-92
;          Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;          Added fractional pixel repositioning, A.McA. 7-may-92
;          Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;
;	   Rewritten from GBO_SCALE2, takes target info as input, A.McA. 6-oct-92
;          Output variable type now same as input type, A.McA. 2-dec-92
;
;	   Version 2. Reworked for flexiable image type. A.McA. 14-dec-92 
;	   Return radius rather than diameters for first GEN_LOC/SXT 
;	   call,A.McA 14-feb-94 
;	   Reworked to index update to function with FIT_LIMB. A.McA. 4-apr-94.
;	   Minor change to match DSK_LOCG use, added QUIET and INTER 
;	   keywords, A.McA. 24-aug-94.
;          Changed congrid call to use cubic convolution interpolation, A.McA. 24-aug-94
;	   Added CIR keyword, for use with corrected images, A.McA. 2-sep-94.
;
;-
FUNCTION imgscl2,trgdt,gboimg,sxt,indx1,indx2,rough=rough,mktv=mktv, $
		 inter=inter,quiet=quiet,cir=cir

if n_params(0) eq 4 then begin		   ;only one sxt image

  if total(sxt) ne 1 then begin		   ;trap sxt/index mismatch

    print,"IMGSCL2: The number of indices dosn't match SXT vector"
    return,-1
    stop

  endif

  if sxt(0) eq 1 then indxt=indx1
  if sxt(1) eq 1 then indxi=indx1

endif else begin

  if n_params(0) eq 5 then begin		   ;two sxt images

    if total(sxt) ne 2 then begin		   ;trap sxt/index mismatch

      print,"IMGSCL2: The number of indices dosn't match SXT vector"
      return,-1
      stop

    endif

    indxt=indx1
    indxi=indx2

  endif

endelse

if sxt(1) eq 0  then begin                          ;find input diameters

  print, 'IMGSCL2: Input not SXT image'
  
  gen_loc,gboimg,HORZRAD=hzrd,VERTRAD=vtrd,inter=inter,quiet=quiet,cir=cir	    ;invoking the ellipse fit
 
endif else begin

  print, 'IMGSCL2: Input SXT image'

  gen_loc,gboimg,indxi,BFRAD=rad,inter=inter,quiet=quiet,/SXT,rough=rough  

  hzrd=2.0*rad &  vtrd=2.0*rad 

endelse

x0=trgdt(0)				   ;target centroid
y0=trgdt(1)

gbosz=size(gboimg)                         ;size of input image

;Note that if conditions are right (large gradients in data with absolute
;values near 0) then the congrid may introduce negative numbers into a
;previously positive definite field. This may have later consequences.

ha=congrid(gboimg,gbosz(1)*trgdt(2)/hzrd,gbosz(2)*trgdt(3)/vtrd,/cubic)

;resize the input

if sxt(1) eq 1 then begin                       ;image is sxt image

  if sxt(0) eq 1 then begin			;target also sxt image
  
    tres=gt_res(indxt)				
    tshape=indxt.sxt.shape_sav
       
  endif else begin				;target not sxt image

    tres=gt_res(indxi)				;assume its the same resolution
    tshape=indxi.sxt.shape_sav
    
  endelse
  
  his_index,indxi , 0, 'pixel_size', tres		;update index
  indxi.sxt.shape_sav = tshape
  
  gen_loc,ha,indxi,CENTROID=cent,/SXT,rough=rough       ;find center of resized image
                                               		                                            		
endif else begin					;everything non-sxt				
  
  gen_loc,ha,CENTROID=cent,BFRAD=rad,/quiet,/cir	;invoke the circle fit

endelse

dltx=fix(x0-cent(0))                      	;difference to one pixel
dlty=fix(y0-cent(1))

hasz = size(ha)                          	;size of resized image

imgout=replicate(ha(0,0),trgdt(4),trgdt(5))  	;initialize output image

;copy resized input to output, while centering to nearest pixel

imgout((0 > dltx):((trgdt(4)-1) < (dltx+hasz(1)-1)),         $
       (0 > dlty):((trgdt(5)-1) < (dlty+hasz(2)-1))) =       $
     ha((0 > (-1*dltx)):((hasz(1)-1) < (trgdt(4)-1-dltx)),   $
        (0 > (-1*dlty)):((hasz(2)-1) < (trgdt(5)-1-dlty)))

;shift the last fraction, uses interpolation which may change intensity range
;use min value for 'uncovered' pixels to avoid shifting intensity range.

fdltx=(x0-cent(0))-fix(x0-cent(0))        ;fractional pixel difference
fdlty=(y0-cent(1))-fix(y0-cent(1))
 
imgout=poly_2d(imgout,[-fdltx,0,1,0],[-fdlty,1,0,0],1,missing=ha(0,0))

if keyword_set(mktv) ne 0 then tv,imgout             ;plot on screen

return,imgout

end

