;+
; NAME: mode_fit
;
; PURPOSE: For fitting a unique value to a set of data using histograms.
;           
;
; METHOD: A histogram is performed on the data, and the results are 
;	  fitted with a guassian, the center-value of which is taken		
;	  as the best fit to the data. When there are less than four
;         values the cases are handled specially, as the guassian fit
;	  won't converge. For a unigue value, that is simply past back,
;	  for 2 or 3 values a weighted average is taken.
;
;	  In order to use with data where you want a binsize less than
;	  1, you must first scale up the data.
;
;	  Note: the sigma value is that defined by IDL convention.
;
;
; CALLING SEQUENCE: mean=mode_fit(data[,binsize=2][,/second][,fit=gauss]
;                   			          [,/display])
;
; PARAMETERS:	data		input vector 
;          
;
; KEYWORDS: 	binsize		sets the size of the histogram bin
;				default is 1.0, this is also the minimum.
;
;		second		when set does a second pass using data
;				within 3 sigma of the peak value
;
;		display  	when set display the histogram and fit.
;
;		fit		a three vector with peak value, peak 
;				location and sigma value.
;
; CAUTIONS: Dosn't work well on histograms with many discrete spikes.
;
; CALLS: HISTOGRAM, GAUSSFIT2
;
; HISTORY: cut out as seperate routine, June 1, 1993, A. McAllister
;	   adjusted bin definitions to integers, Feb. 1, 1994, AMcA.
;
;-
FUNCTION mode_fit,data,binsize=delv,second=sec,display=disp,fit=fit
 
 if keyword_set(delv) eq 0 then delv = 1.0	;set intensity sensitivity

 histmin=fix(min(data)-3.0*delv+0.5)		;use min/max to avoid srink bin count
 histmax=fix(max(data)+3.0*delv+0.5)
 hist=histogram(data,min=histmin,max=histmax,binsize=delv)
 
;  zero bins with less than 10% peak count, not currently needed
;  hist(where( hist lt max(hist)/10))=0

 szh=n_elements(hist)
 bincount=szh-6

 case 1 of

   (bincount le 1): value = data(0)		;unique value

   (bincount eq 2): value = (min(data)*hist(3)+max(data)*hist(4)) $
			    /(total(hist(3:4)))

   (bincount eq 3): value = (min(data)*hist(3)+(min(data)+1)*hist(4)+  $
                             max(data)*hist(5))/(total(hist(3:5)))

   (bincount gt 3): begin			;need 4 points for curvefit
   
     xx=indgen(szh)*delv+histmin+0.5		;set bin values, offset from min
     
     if keyword_set(disp) then begin
;       num=new_win(512,512)
       plot,xx,hist,yrange=[0,max(hist)]
     endif
     
     a=fltarr(3)				;zero background fit
;stop
     yfit=gaussfit2(xx,hist,a)

     value = a(1)
     fit = a

     if keyword_set(disp) then oplot,xx,yfit
;stop
;a second pass focusing on the peak value

     if keyword_set(sec) then begin

       dmax=a(1)+3*a(2)				;peak + 1/e value
       dmin=a(1)-3*a(2)				;peak - 1/e value
       pkdt=data(where((dmin > data) < dmax))
       histmin=dmin-3.0*delv              
       histmax=dmax+3.0*delv
       hist2=histogram(pkdt,min=histmin,max=histmax,binsize=delv)	
       a2=fltarr(3)
       xx2=indgen(n_elements(hist2))*delv+histmin 		
       yfit=gaussfit2(xx2,hist2,a2)
       value = a2(1)
       fit = a2

     endif

     if value lt min(data) then goto,error
     if value gt max(data) then goto,error
  
    end

  endcase

return,value

error: return,value=-1

end
