;+
; NAME: rad_bck
;
; PURPOSE:  Calculate the featureless background of an Halpha or white light 
;           image. It is mainly ment for use with SXT aspect sensor images
;	    but with the /GBO keyword it should work on other spectral data.
;
; METHOD:   Find the center, calculate radial values, use a gaussian fit to 
;	    histograms of pixels in radial bins to select `background' value 
;           for those pixels.
;	    This has been fixed to handle signals at the center.
;	    
;
; CALLING SEQUENCE: 
;                   back=rad_bck(image,profile[,target=trgt][,step=step]
;				      [,res=res][,binsize=binsize][,/gbo])
;
; PARAMETERS:
;
;          image   	is assumed to have be circular. (how much tolorance?)
;
;	   profile	returns the limbdarkening profile, vector. The first
;			entry gives the step size.
;
; KEYWORDS:
;
;	res		for QR or FR SXT images this must be set as 4.0 and 1.0
;			respectively.
;
;   	target  	contains the centroid and radius of the input image.
;               	if this is missing they will be calculated.
;
;   	step    	the step size for incrementing the radius. 
;			Note reducing this increases the running time.
;
;   	binsize 	sets the binsize for the histogram routine. 
;			Default value is 1.0, this is also the minimum.
;
;   	gbo	    	when set expects a GBO image and calls GEN_LOC to 
;			make the target.
;
; CALLS:  FINDLIMB, GEN_LOC, R_COORD, MODE_FIT	  
;
; HISTORY: Rewrote from BACKGROUND using histograms, jan. 1993 A. McA.
;	   Cut out MODE_FIT as a separate routine, 
;	   and added GBO keyword, june 1993 AMcA.
;	   added special central treatment, june 8, 1993, AMcA.
;	   added profile variable, june 15, 1993, AMcA.
;	   added trap for infinities, june 16, 1993, AMcA.
;	   reworked bad case handling, june 18, 1993, AMcA.
;	   set up graded stepsize, june 24, 1993, AMcA.
;	   trap for residual noise in creating REFF, july 2, 1993, AMcA.
;	   fix bug in recording last profile entry, jan. 31, 1994, AMcA.
;	   added case for average values less than szero, feb. 2, 1994, AMcA.
;	   replaced MODE_FIT call with straight average, feb. 7, 1994, AMcA.
;	   added RES keyword for non-GBO resolution, feb. 10, 1994, AMcA. 
;-
FUNCTION rad_bck,img,prof,res=res,target=trgt,step=step,binsize=delv0,gbo=gbo

print,'begin: ',!stime

 if keyword_set(res)  eq 0 then res= 2.0	;default for Half Resolution
 if keyword_set(step) eq 0 then step = 1.0	;set radial step
 if keyword_set(delv) eq 0 then delv0 = 1.0	;set intensity sensitivity

 if keyword_set(trgt) eq 0 then begin

   if not keyword_set(gbo) then begin 

     find_limb,img,x,y,r,err 			;be careful about resolution
     trgt=[x,y,r,r]/res				;changes ?? this returns FR

   endif else begin

     gen_loc,img,CENTROID=cent,HORZRAD=rdhz,VERTRAD=rdvt	;make this /SXT ?		
     trgt=[cent,rdhz/2.0,rdvt/2.0]				;centroid & radius.

   endelse

 endif

 imgsz=size(img)
 backgrd=fltarr(imgsz(1),imgsz(2))               ;initialize result array
 pixind=fltarr(imgsz(1),imgsz(2))                ;initialize array for index

 indx=lindgen(imgsz(1),imgsz(2))		 ;create matrix with linear coords
 rads=r_coord(indx,imgsz(1),trgt(0:1))		 ;get radii 

 deltr= step > 1.0				 ;force step to be >= 1
 
 tmp=img
 ref=-1
 while ref(0) eq -1 do begin				;take central pix, minus sunspots
   maxval=max(tmp)
   ref=where(tmp gt .9*maxval and rads lt trgt(2)/15.)  ;changed to '/15.' 2-jul-93.
   if ref(0) eq -1 then tmp(where(tmp eq maxval)) = 0	;trap for remaining noise
 endwhile
 tmp=0
 reff=total(img(ref))/n_elements(img(ref))	 	;rough central average

 jj=0

 for ir= 0.0 , float(fix(trgt(2))+3.0), step do begin

   if jj lt 10 then begin

     ssp=where(rads ge ir and rads lt (ir+3.0*deltr))   ;get index to points

   endif else begin

     ssp=where(rads ge ir and rads lt (ir+1.0*deltr))   ;get index to points

   endelse

   if ssp(0) ne -1 then begin

     fstdt=img(ssp)

;TODO recheck this value (50).
     if n_elements(fstdt) lt 50 then begin	;special treatment at center

       fstdt=median(fstdt,3)  

       case 1 of				;check for central sunspots or plage

          median(fstdt) lt .8*reff:  	fstdt(*)=reff   

          median(fstdt) gt 1.1*reff: 	fstdt(*)=reff   

	  else: 		;dummy pass through

       endcase

     endif

     medn=median(fstdt)	
     if medn gt 0.0 then begin 				;screen out wings beyond +/- 10%
       tst=where(fstdt gt .9*medn)
       if tst(0) ne -1 then fstdt=fstdt(tst)
       tst=where(fstdt lt 1.1*medn)
       if tst(0) ne -1 then fstdt=fstdt(tst)
     endif else begin					;in case medn is lt 0
       tst=where(fstdt gt 1.1*medn)
       if tst(0) ne -1 then fstdt=fstdt(tst)
       tst=where(fstdt lt .9*medn)
       if tst(0) ne -1 then fstdt=fstdt(tst)
     endelse

     case 1 of						;set histogram binsize 

        ir le 15: delv=3.0*delv0

        ir gt 15 and ir le 29: delv=2.0*delv0

        ir gt 29: delv=1.0*delv0

     endcase
     
;print,"ir = ",ir
;if ir gt float(fix(trgt(2)))*.9 then stop
;if ir gt float(fix(trgt(2))) then fstdt=median(fstdt,5)
 
 szf=n_elements(fstdt)					;straight average
 value=total(fstdt)/szf

;  value=mode_fit(fstdt,binsize=delv,/disp) 
;  value=mode_fit(fstdt,binsize=delv)

     if finite(value) ne 1 then value=-1		;trap overflows

     if value lt 0 then  $				;out of bounds answer

	print,'RAD_BCK: out of bounds answer. ir ='+string(ir) 	

     if ir eq 0.0 then begin				;store profile

       profile=fltarr(4)
       profile(0)=deltr					;first entry is step size.
       profile(1:3)=value
       jj=jj+3
       ir=ir+2.0*deltr

     endif else profile=[profile,value]

     backgrd(ssp)=value					;store background
     pixind(ssp)=jj				  	;index pixels to prof values
     jj=jj+1

   endif
 endfor

 for kk = 3 , n_elements(profile)-1 do begin		;take care of problems

   if profile(kk) lt 0 then begin

     case 1 of

        (ir le .05*trgt(2)):	profile(kk)=reff		;near suncenter

	(ir gt .05*trgt(2)) and (ir le .9*trgt(2)):  profile(kk)=(profile(kk-1)+profile(kk+1))/2.0

	(ir gt .9*trgt(2)):  begin $					;near limb
	
				if kk eq n_elements(profile)-1 then begin
				  profile(kk)=0.0			;overlimb
				endif else begin
 				  profile(kk)=(.75*profile(kk-1)+1.25*profile(kk+1))/2.0
 				endelse
 				
 			     end	

        else:	print,'no problem'		;pass through

     endcase

     backgrd(where(pixind eq kk)) = profile(kk)

   endif

 endfor

 if n_params() eq 2 then prof=profile

print,'end: ',!stime

 endhere: return,backgrd

end


