;+
; NAME: rect_rot
;
;
; PURPOSE: To rotate a part of the solar disk forward or backward in time. 
;
;
; METHOD: You can enter the edges of the rectangle, or leave them
;	  out and choose them interactively. The selected rectangle 
;	  is rotated using SOL_ROT. The /RIGID keyword invokes rigid
;	  rotation, meaning the whole region is rotated at the rate
;	  calculated for its center point (using the differential 
;	  formula). You can use HMS2DD to generate DELT.             
;
; CALLING SEQUENCE: trans=rect_rot(img,date,delt[,x0,x1,y0,y1][,target=trgt]
;					         [,rigid=rdgd][,edges=edges])
;
; PARAMETERS:	img	the input image, you wish to rotate a part of
;		date    the time of the image, in standard 7 part form, 
;			[DD-MMM-YY HH:MM:SS].
;		delt	the rotation time, in decimal days.
;		trans 	the output is a matrix the same size as the input
;			with the rotated pixels in their new locations. The
;			rest is zero.
;
; OPTIONAL INPUT: x0,x1,y0,y1  	when you already know the edges you want
;				then you can pass them in
;
; KEYWORDS:	trgt	a three vector with the centroid and radius of 
;			the input image, in pixel units matching the image
;		rigid  When set rotate the whole region at the rate for
;			the center.
;		edges 	returns the four selected edge values [x0,x1,y0,y1]
;
; RETURN TYPE:
;
; CALLS: COORD_L2V, GET_BOXCORN, DRAW_BOXCORN, SOL_ROT, TARGET                       
;
; HISTORY: Drafted by A. McAllister Feb. 1993.
;	   [Interactive code borrowed from PLOT_FOV by MDM]
;	   Changed target to keyword, added rigid keyword, AMcA, Mar. 1993.
;	   Added fill for blanks, AMcA, Jul. 1993.
;	   Fixed a bug in the blank filling line, AMcA, Dec. 1993.
;
;-
FUNCTION rect_rot,img,date,delt,x0,x1,y0,y1,target=trgt,rigid=rgd,edges=edgs

 if keyword_set(trgt) eq 0 then trgt=target(img)

 imgsz=size(img)
 linind=lindgen(imgsz(1),imgsz(2))                      ;generate linear indices
 xy=coord_l2v(linind,imgsz)                             ;convert to xy indices

 if n_params() lt 7 then begin				;select rectangle

    get_boxcorn, x0, y0, x1, y1, /device
    draw_boxcorn, x0, y0, x1, y1, /device

 endif

 edgs=[x0,x1,y0,y1]

;get indices of selected rectangle and convert to xy

 rec_ss=where(xy(0,*) ge x0 and xy(0,*) le x1 and xy(1,*) ge y0 and xy(1,*) le y1)
 sm_xy=coord_l2v(rec_ss,imgsz)

 if keyword_set(rgd) then begin

   print,'rotating rigidly'

   xc=(x1+x0+1)/2                             	;find region center
   yc=(y1+y0+1)/2

   newcnt=sol_rot(xc,yc,trgt,date,delt)	      	;rotate center

   trans=sm_xy 
   trans(0,*)=trans(0,*)+(newcnt(0)-xc)		;add rigid rotation
   trans(1,*)=trans(1,*)+(newcnt(1)-yc)

 endif else begin           

   print,'rotating differentially'      		

   trans=sol_rot(sm_xy(0,*),sm_xy(1,*),trgt,date,delt)  ;rotate differentially

 endelse

;take only on the disk points

 od_ss=where(trans(0,*) ne -1)				
 
;create ouput array and assign rotated pixels
 rimg=img & rimg(*,*)=min(img)
 rimg(trans(0,od_ss),trans(1,od_ss)) = img(sm_xy(0,od_ss),sm_xy(1,od_ss)) 

;fill in the blanks

 zeros=where( rimg eq min(img) )
 rimg(zeros) =  (rimg(zeros+1)+float(rimg(zeros-1)))/2.0 
 
 rimg=match_type(rimg,img)
                 	
 return,rimg

end
