;+
; NAME:	rm_darklimb
;
; PURPOSE: Script to remove the limb darkening from optical images.
;
; METHOD: Starting from the center, takes concentric circles and performs
;	  azimuthal fourier transforms, retains the first n modes (default
;	  2) and inverse transforms them to construct a background. This is
;	  then smoothed and subtracted from the original image. The SETBACK
;	  keyword causes the surrounding background to be set to the minimum 
;	  on disk value to leave a sharp limb for the limb-finding routines.
;	  To retain an image of prominances etc. you can use the PROM keyword
;	  Use of the MARGIN keyword can handle these cases, will exclude 
;	  bright "features" above the limb such as labels.
; 
; 
; CALLING SEQUENCE:
;
;         outimg=rm_darklimb(data,[target=trgt],[modes=mdnm],[/setback]
;                                    [,/prom][,margin=margin])
;
; PARAMETERS:
;
;	inimage   image/s from which the limbdarkening is to be removed.
;	outimg    the processed result.
;
; KEYWORDS:
;
;    	target 	  contains the centroid and radius of the input image.
;	    	  if this is missing they will be calculated.
;    	modes     sets the number of fourier modes used to construct the 
;            	  background profile. Default is 2.
;   	setback   to reset the background to the minimum on disk value.
;	prom	  to retain the shape of prominances.
;	margin    when set everything at a radius greater than limb + margin
;		  is reset. In pixels.
;
; RESTRICTIONS: It is expected that the image is already corrected for 
;   	        oblateness due to the image format.
;		The margin keyword cannot be set exactly to zero. If 
;		so the keyword is ignored.
;
; RETURN TYPE:  Same as input data type.                                
;
; CALLS:  TARGET, BACKGROUND, SET_BACK, CUT_LEV, MATCH_TYPE                    
;
; HISTORY: 
;           drafted by A. McAllister, 17-apr-92.
;           added optional mode number and target keywords, A.McA., 29-nov-92. 
;	    added SET_BACK call, A.McA, 14-Jan-93.
;	    added margin keyword for SET_BACK, A. McA. 4-Feb-93.
;	    freed return data type, A.McA. 9-mar-93.
;	    added prom keyword, A.McA. 10-mar-93.
;	    reworked to take data cubes, A.McA. 5-dec-94.
;-
FUNCTION rm_darklimb,data,target=targt,modes=mdnm,setback=sb,margin=mrg,prom=pr

  if keyword_set(modes) eq 0 then modes = 2

  if keyword_set(targt) then trgt=targt

  szi=size(data)

  case szi(0) of

	1: begin

		print,'RM_DARKLIMB: This is not image data.'
		return,-1

	   end

	2: begin

		count=1					;single image
		flat=fltarr(szi(1),szi(2))

	   end

	3: begin

		count=szi(3)				;data cube
		flat=fltarr(szi(1),szi(2),szi(3))

	   end

  endcase

for ii=0,count-1 do begin

  img=data(*,*,ii)

  if keyword_set(targt) eq 0 then trgt=target(img)

  if n_elements(trgt) eq 3 then begin		;add cut if necessary

     cut=cut_lev(img,size(img))
     trgt=[trgt,cut]

  endif

  if keyword_set(sb) ne 0 then begin

     out=set_back(img,trgt=trgt,bakmsk=msk,prom=pr,margin=mrg)

   endif else  out=img                                         

  back=background(out,target=trgt,modes=mdnm)

;limit smoothing at the edge? this will change radius?

; out=float(img)-float(smooth(smooth(smooth(back,5),5),5))

  out=float(img)-float(back)		;leave out smoothing
  out=match_type(out,img)

  if keyword_set(sb) ne 0 then out(where(msk eq 1)) = min(out)	;adjust background down
  if keyword_set(pr) ne 0 then out(where(pr eq 1)) = img(where(pr eq 1)) ;set prominences

  flat(*,*,ii)=out

endfor

return,flat

end
