;+
; NAME: 
;        rt_fits
;
; PURPOSE: A script to read GBO fits files. It also allows you to match it 
;          to another image (GBO or SXT), remove limbdarkening, and convert to 
;          byte type. 
;
; METHOD:  Reads the file with RFITS, and calls a series of routines to do the
;	   processing. The exact function can be controlled with keywords.
;	   When limbdarkening removal is on, the default is to use the setback
;	   keyword, which sets the background pixels to the minimum on disk
;	   value. In this case using the /prom keyword preserves prominences.
;
; CALLING SEQUENCE:
;
;       outimg=rt_fits(filename[,targt_img][,indx][,header=header][,modes=modes]
;			       [,delang=da][,/nomatch][,/limbdark][,/nosetback]
;                              [,/invert][,/prom][,/byte][,/sel])
;
;	raw=rt_fits('path/filename',header=head,/nomatch,/limbdark,/byte)
;       matched=rt_fits('path/filename',image,index,header=head,/limbdark,/byte)
;       ld_rmvd=rt_fits('path/filename',image,index,header=head,/byte)
;	calcium=rt_fits('path/filename',image,index,header=head,/invert,/byte)  
;
; PARAMETERS: 
;            filename     GBO filename (with path).
;            target       target image matched for size and location. Can
;			  also be a 6 component vector,the CENTROID (0,1), 
;                         the DIAMETERS(horizontal,vertical;2,3), and the pixel 
;	                  size (x,y:4,5) of the target image.  
;            indx         if the target is an sxt image you need to pass
;                         in the associated index.
;            outimg       output image scaled and centered to match the
;                         target image.
;
; KEYWORDS:
;    	     header       receives the header info for the Halpha image.
;	     modes        number of fourier modes to use for LD profile.
;			  The default is 2.
;	     delang	  optional roll angle correction.
;	     nomatch      omits matching. 
;	     limbdark     if set leave limbdarkening.
;	     nosetback    suppresses the setback function.
;	     invert	  when set invert the image before matching.
;	     prom	  when set leaves prominence outlines, when 
;			  setback is called.
;	     byte         if set return byte scaled output.
;	     bbso	  removes the labels on BBSO processed GBO data.
;	     sel	  removes the labels on SEL processed GBO data.             
;
; RETURN TYPE:  BYTE      if the byte keyword is set
;               FLOAT     otherwise
;
; CALLS: RFITS, IMG_MATCH, RM_DARKLIMB, TARGET, ROTT                                      
;
; HISTORY: drafted by A. McAllister, Apr. 1992. 
;          slightly reworked to use gen_loc,  A.McA, May 1992.
;          added keywords modes,limbdark,byte,nomatch, A.McA, Dec. 1992.
;          added setback option, A. McA, Jan. 1993.
;	   added sel keyword, A. McA, Feb. 1993.  
;	   added delang keyword, made setback default, added invert 
;	         keyword, added prom keyword, A. McA, Mar. 1993.
;	   fixed but in nomatch section, AMcA, Dec. 1, 1993. 
;	   Allow a vector target for matching, AMcA, Feb. 21, 1994.
;	   Added BBSO keyword, AMcA, Feb. 3, 1995.   
;
;-
FUNCTION rt_fits,infl,target,indx,header=hdr,modes=mdnm,limbdark=ld, $
                 invert=ivrt,delang=da,nomatch=nomatch,nosetback=nsb,sel=sel, $
		 prom=prom,byte=bt,bbso=bbso

 if keyword_set(mdnm) eq 0 then mdnm = 2

 img=rfits(infl,header=hdr,/scale)                ;retrieve a fits image as float

 if keyword_set(sel) ne 0 then begin		  ;suppress labels on SEL images

   img(246:265,496:511)=0			  ;'N'
   img(0:150,475:511)=0			  	  ;data type / site [upper-left]
   img(400:511,475:511)=0			  ;time / date [upper-right]
   img(370:511,0:20)=0			  	  ;'Preprocesed' [lower-right]
 
 endif

if keyword_set(bbso) ne 0 then begin		  ;suppress labels on BBSOimages

   imgsz=size(img)				  ;remove fits header
   img=img(0:imgsz(1)-1,2:imgsz(2)-2)		  ;images (sometimes at bottom) 
   img(where(img lt 10))=0.0			  ;gludge to get rid of garbage
   img(*,0:20)=0.0				  ;BBSO cludge redo this later
   img(590:*,420:*)=0.0				  ;BBSO logo

endif

dum=img

 if keyword_set(ivrt) ne 0 then dum=rotate(dum,7) ;invert if necessary

 if keyword_set(nomatch) ne 0 then begin 

   if keyword_set(ld) eq 0 then begin

     print,'warning: for non-circular images limbdarkening'
     print,'         removal will give strange results'

     if keyword_set(nsb) ne 0 then begin

       dum=rm_darklimb(dum,modes=mdnm)

     endif else begin

       dum=rm_darklimb(dum,modes=mdnm,/setback,prom=prom)

     endelse

   endif

   if keyword_set(bt) then dum=bytscl(dum)

   return,dum     				  ;for nomatch return here
   stop

 endif

 if n_elements(target) gt 6 then begin		    ;test target as image or vector

   if n_elements(indx) gt 0 then begin

     out = img_match( target,dum,[1,0],indx )       ;reformat to match sxt image

   endif else begin

     out = img_match( target,dum,[0,0] )      	    ;reformat to match non sxt image

   endelse
   
 endif else begin
 
   out = imgscl2( target,dum,[0,0],/rough ) 	    ;match to a vector target

 endelse

 if keyword_set(ld) eq 0 then begin

   if keyword_set(nsb) ne 0 then begin

     out=rm_darklimb(out,modes=mdnm)

   endif else begin

     out=rm_darklimb(out,modes=mdnm,/setback)

   endelse

 endif

; if n_params() eq 3 then da=1.0		   ;future SXT default, & test (da ne 0)

 if keyword_set(da) ne 0 then begin		   ;do rotation

   trgt=target(out)

   out=rott(out,da,trgt(0),trgt(1),missing=out(0,0),/interp)

 endif 

if keyword_set(bt) then out=bytscl(out)

return,out

end
