;+
; NAME: 	rt_hdha
;
; PURPOSE: 	To read and optionally process a Hida H alpha image for use 
;               with Yohkoh SXT or other GBO data.
;
; METHOD: 	This reads an Hida H alpha image, inverts it, rotates it (if
;		necessary), and repositions it to match a target (either SXT or 
;		GBO). The result is then fed to RM_DARKLIMB to remove the 
;		limb-darkening. To get the rotation angle it calls GET_RB0P. 
;		To add roll correction use the DELANG keyword. As the resultant 
;    		background is a mess, it is reset, unless you suppress it with 
;		the NOSETBACK keyword. 
;
; CALLING SEQUENCE: 
;
;       output=rt_hdha(filename,[target],[target_i],[delang=delang],[header=head],
;                               [modes=mdnm],[/limbdark],[/nosetback],[/nomatch])
;
;               filename  a string with the path/filename of a Hida Halpha image
;               target    the SXT target image
;               target_i  the index entry for the target image
;
;
; KEYWORDS:   
;		delang    corrections to the solar axis tilt angle (clockwise degrees)
;               header    if you wish, this returns the header.
;		limbdark  if set leave limbdarkening
;               modes	  number of fourier modes to use for LD profile
;			  The default is 2.
;		nosetback suppresses resetting the background.
;		nomatch   omits rotation and matching
;  			  
;
; RESTRICTIONS: The angle DELANG can be used to correct for the small (~1 deg) 
;		SXT pointing error. There may also be small errors in GBO pointing. 
;		When using a non-SXT image as target delang must be the whole
;		rotation angle. Running GET_RB0P with the desired date and
;		time will return it in the third component.
;
;     		Omiting the NOSETBACK keyword will obscure the details of 
;		limb prominances.
;
;
;	CALLS:  NKRGET, GEN_LOC, GET_RB0P, ROTT, IMGSCL2, SET_BACK, RM_DARKLIMB, 
;		TARGET                                                    
;
;       HISTORY: Drafted by A. McAllister feb-93.
;		 Changed default to call SET_BACK, replaced, A.McA mar-93. 
;-
FUNCTION rt_hdha,filename,trgt,trgt_i,delang=delang,header=head,modes=mdnm,$
                 limbdark=ld,nosetback=nsb,nomatch=nomatch

  if keyword_set(mdnm) eq 0 then mdnm = 2
  if keyword_set(delang) eq 0 then delang = 0

  nkrget,filename,head,data

  if keyword_set(nomatch) ne 0 then begin

    if keyword_set(ld) eq 0 then begin 

      if keyword_set(nsb) eq 0 then begin

        data=rm_darklimb(data,modes=mdnm,/setback,margin=0.1)

      endif else begin

        data=rm_darklimb(data,modes=mdnm)

      endelse

    endif

    return,data           

  stop						

  endif

; matching case: put together all needed target info

  if n_elements(trgt_i) gt 0 then begin 

    targt=target(trgt,trgt_i)
    sxt=[1,0]

  endif else begin

    targt=target(trgt) 
    sxt=[0,0] 

  endelse

  trgsz=size(trgt)
  trgdt=[targt,targt(2),trgsz(1),trgsz(2)]    

; When the SXT roll is calculated then use this for angle and redo this section.

;  vect=[0,0,0]
;  if n_elements(trgt_i) gt 0 then  vect=get_rb0p(fmt_tim(trgt_i))
   angle = 0
   if keyword_set(delang) ne 0 then angle=delang

; flip first, then scale, then rotate.

  temp=rotate(data,7)

  if sxt(0) eq 1 then begin

    temp=imgscl2(trgdt,temp,sxt,trgt_i)
 
  endif else begin

    temp=imgscl2(trgdt,temp,sxt)

  endelse

  if angle ne 0 then temp=rott(temp,angle,cent(0),cent(1),missing=data(0,0),/interp)

  cut=cut_lev(temp,size(temp))
  trgtt=[trgdt(0:3),cut]

  if keyword_set(nsb) eq 0 then set_back,temp,trgt=trgtt,bakmsk=msk,margin=0.1    ;set background

  if keyword_set(ld) ne 0 then return, temp     ;return with limbdarkening

; remove the limb darkening, pass sun center in

  out=rm_darklimb(temp,target=trgdt(0:3),modes=mdnm)
  
  if keyword_set(nsb) eq 0 then out(where(msk eq 1)) = min(out)     ;adjust background down

return,out

end


