;+
; NAME: 
;        rt_mag
;
; PURPOSE: A script to retrieve a fits format magnetogram and resize and
;          position to match an SXT or GBO image. Keywords allow suppression 
;	   of the matching function, or invocation of a croping function that
;	   truncates all insensities greater than +/- 127 Guass. This is to
;	   allow conversion to byte type without scaling. When the CROP
;	   keyword is called with a value (crop=value) this replaces the 
;	   default. You may also rotate about the centroid to align solar
;	   north properly.
;
; CALLING SEQUENCES:
;
;         outimg=rt_mag(filename,[targt_img],[indx][,header=header][,/crop]
;                                [,zero=cent][,/nomatch][,delang=delang])
;
;         outimg=rt_mag(filename,header=header,/nomatch)
;         outimg=rt_mag(filename,header=header,/crop,zero=cent,/nomatch)
;         outimg=rt_mag(filename,targt_img,[indx],header=header)
;         outimg=rt_mag(filename,targt_img,[indx],header=header,/crop,zero=cent)
;
; PARAMETERS:
;
;         filename     is the Magnetogram filename (with path).
;         targt_img    is the target image matched for size and location.
;         indx         if the target is an sxt image you need to pass
;                      in the associated index.
;         outimg       the output image scaled and centered to match the
;                      target image.
;
; KEYWORDS:
;
;         header       receives the header info for the magnetogram.
;	  crop         if set then the image is truncated to +/- 127.
;		       a value from 2-127 sets the truncation value.
;	  zero 	       when cropping, output the new zero.
;	  nomatch      don't match to another image.
;	  delang       to rotate about the centroid (clockwise degrees).
;
; NOTES:	This works OK when /crop is NOT set, but the new limbfinder
;		has troubled with the cropped data. Will try to adjust. AHM 3/8/95
;
; RETURN TYPE: FLOAT  when not cropped.
;              BYTE   when cropped. 
;
; CALLS: RFITS, BYTCON, IMG_MATCH, SET_BACK, ROTT
;
; HISTORY: drafted by A. McAllister, Dec. 2, 1992. 
;          implimented user defined cropping, A.McA. Dec. 23, 1992.
;	   added delang keyword, A.McA. Mar. 10, 1993.
;
;-
FUNCTION rt_mag,infl,trg_img,indx,header=hdr,crop=crop,zero=cent, $
                     nomatch=nomatch,delang=da

img=rfits(infl,header=hdr,/scale)        ;retrieve a fits image

imgsz=size(img)
dum=img(0:imgsz(1)-1,0:imgsz(2)-2)  ;remove fits header (check this)

if keyword_set(crop) ne 0 then begin		;if /crop set, crop it

  if (crop gt 1 and crop le 127) then begin

    img=bytcon(img,min=(-1*crop),max=crop,cent=cent) ;crop to specified value

  endif else begin

    img=bytcon(img,cent=cent) ;crop to default

  endelse

endif

if keyword_set(nomatch) ne 0 then return,img  

;For matching case.

if keyword_set(crop) eq 0 then img=set_back(img)  		

if n_elements(indx) gt 0 then begin

  out=img_match(trg_img,img,[1,0],indx)  ;reformat to match sxt target

endif else begin

  out=img_match(trg_img,img,[0,0])       ;reformat to match gbo target

endelse 

if keyword_set(da) ne 0 then begin                ;do rotation

   trgt=target(out)

   out=rott(out,da,trgt(0),trgt(1),missing=out(0,0),/interp)

 endif 

 return,out

end
