;+
; NAME: 	rt_nkr
;
;
; PURPOSE: 	To read and process a Mitaka H alpha image for use with SXT
;               or other GBO data. 
;
; METHOD: 	This reads an NKR format image, such as Mitaka or Hida H alpha, 
;		inverts and rotates it, and repositions it to match a target 
;		(either SXT or GBO). The result is then fed to RM_DARKLIMB to 
;		remove the limb-darkening. To get the rotation angle it calls 
;               GET_RB0P (the norbp keyword suppresses this). To add further correction 
;		for SXT roll error (or additional GBO uncertainty) use the DELANG 
;		keyword. As the resultant background is a mess, it is reset, unless 
;		you suppress it with the NOSETBACK keyword. To keep prominences when
;		setback is called use /PROM.
;
; CALLING SEQUENCE: 
;
;       output=rt_mtha(filename,[target],[target_i][,delang=delang][,header=head]
;                               [,modes=mdnm][,/limbdark][,/nosetback][,/nomatch]
;				[,/prom][,/norbp])
;
;               filename  a string with the path/filename of an NKR image
;               target    the SXT target image
;               target_i  the index entry for the target image
;
;
; KEYWORDS:   
;		delang    corrections to the solar axis tilt angle 
;			  (clockwise degrees).
;               header    if you wish, this returns the header.
;		limbdark  if set leave limbdarkening.
;               modes	  number of fourier modes to use for LD profile
;			  The default is 2.
;		nosetback suppresses resetting the background.
;		nomatch   omits rotation and matching.
;		prom	  when set leaves prominance outlines, when
;			  setback is called.
;		norbp	  uses only the delang value for rotation. 		  
;
; RESTRICTIONS: The angle DELANG can be used to correct for the small (~1 deg) 
;		SXT pointing error. There may also be small errors in GBO 
;               pointing. When using a non-SXT image as target DELANG must be 
;               the whole rotation angle. Running GET_RB0P with the desired 
;		date and time will return it in the P angle third component.
;
;     		Omiting the NOSETBACK keyword will remove limb prominances.
;		This can be somewhat aliviated by using the /prom keyword. 
;
; RETURN TYPE:  BYTE
;
;
; CALLS:  NKRGET, GET_RB0P, ROTT, IMGSCL2, SET_BACK, RM_DARKLIMB, TARGET 
;		
;
; HISTORY: Drafted by A. McAllister dec-92.
;	   Changed default to call SET_BACK, added norbp keyword, 
;	   added the prom keyword, etc. A.McA mar-93.
;		 
;-
FUNCTION rt_nkr,filename,trgt,trgt_i,delang=delang,header=head,modes=mdnm,$
                 limbdark=ld,nosetback=nsb,nomatch=nomatch,norbp=nrbp,prom=prom

  if keyword_set(mdnm) eq 0 then mdnm = 2
  if keyword_set(delang) eq 0 then delang = 0

  nkrget,filename,head,data

  if keyword_set(nomatch) ne 0 then begin

    if keyword_set(ld) eq 0 then begin 

      if keyword_set(nsb) eq 0 then begin

        data=rm_darklimb(data,modes=mdnm,/setback)

      endif else begin

        data=rm_darklimb(data,modes=mdnm)

      endelse

    endif

    return,data           

  stop						

  endif

; matching case: put together all needed target info

  if n_elements(trgt_i) gt 0 then begin 

    targt=target(trgt,trgt_i)
    sxt=[1,0]

  endif else begin

    targt=target(trgt)
    sxt=[0,0] 

  endelse

  trgsz=size(trgt)
  trgdt=[targt(0:1),targt(2)*2.0,targt(2)*2.0,trgsz(1),trgsz(2)]  ;imgscl wants diameter 

; get the rotation angle with get_rb0p. If not matching SXT image then angle=delang. 

  if keyword_set(nrbp) eq 0 then begin

    vect=[0,0,0]
    if n_elements(trgt_i) gt 0 then  vect=get_rb0p(fmt_tim(trgt_i))
    angle=vect(2)*180/(4.0*atan(1.0))

  endif else begin

    angle=0

  endelse

  angle=angle+delang

 print, angle

; flip first, then scale, then rotate about centroid.

  if sxt(0) eq 1 then begin

    temp=rott(imgscl2(trgdt,rotate(data,7),sxt,trgt_i),angle,targt(0),targt(1),missing=data(0,0),/interp)

  endif else begin

    temp=rott(imgscl2(trgdt,rotate(data,7),sxt),angle,targt(0),targt(1),missing=data(0,0),/interp)

  endelse

  if keyword_set(ld) ne 0 then return, temp     ;return with limbdarkening

; remove the limb darkening, pass sun center in

  if keyword_set(nsb) eq 0 then  begin

    out=rm_darklimb(temp,target=targt(0:2),modes=mdnm,/setback,prom=prom)

  endif else begin

    out=rm_darklimb(temp,target=targt(0:2),modes=mdnm)

  endelse
  
return,out

end


