;+
; NAME: set_back
;
; PURPOSE: To reset the background in optical images, or get a background 
;          bitmask. Also return a mask of prominance features.
;
; METHOD: Uses DSK_LOCB to find the radii, tests the corner for the 
;         current background value (returned from DSK_LOCB) and adjusts
;	  all pixels that above the limb. Keywords allow you to return a 
;	  mask for above limb features, and exclude "features" well over 
;         the limb. 
;
;
; CALLING SEQUENCE:  
;
;	out=set_back(image[,trgt=target][,back=background][,bakmsk=mask]
;			                [,prom=prom][,margin=margin])
;
; PARAMETERS:	image   the input GBO image, also the output.
;		out	the same image with a reset backgorund.
;
; KEYWORDS:	back    option to pass in the background value
;			the default is the minimum value in image.
;		trgt    option to pass in the centroid & radius if
;			it is already known, in pixels.
;		bakmsk  option to return a mask of (0 & 1) indicating 
;			the background pixels.
;		prom    bits for all bright pixels over the limb.
;		margin  option to zero prom for more than a set number 
;			of pixels above the limb. 
;
; RETURN TYPE: Same as the input.
;
; CALLS: TARGET, R_COORD, CUT_LEV                                             
;
; HISTORY: Drafted dec-92 by A. McAllister.
; 	   Added margin keyword, feb-93 AMcA.
;	   Added prom keyword, redefined bakmsk keyword, mar-93 AMcA.
;	   Trapped all the WHERE index statments, jan-94 AMcA.
;
;-
FUNCTION set_back,indata,trgt=trgt,back=backgrnd,bakmsk=mmsk,prom=prom,$
                         margin=mrg

  if keyword_set(backgrnd) eq 0 then backgrnd=min(indata)

  if keyword_set(trgt) eq 0 then trgt=target(indata)

  if n_elements(trgt) eq 3 then begin		;add cut if necessary

     cut=cut_lev(indata,size(indata))
     trgt=[trgt,cut]

  endif

  imgsz=size(indata)
  inds=lindgen(imgsz(1),imgsz(2))
  radi = r_coord(inds,imgsz(1),trgt(0:1))
  
  mmsk=replicate(0,imgsz(1),imgsz(2))
  
  otl=where(radi gt trgt(2))
  if otl(0) ne -1 then begin
    mmsk(otl) = 1	   			;set everything over the limb
  endif else return,-1
 
  if keyword_set(prom) ne 0 then begin

    prom=replicate(0,imgsz(1),imgsz(2))
    ptst=where((radi gt trgt(2)) and indata gt .8*trgt(3))
    if ptst(0) ne -1 then prom(ptst)=1

    if keyword_set(mrg) ne 0 then begin
      junk=where(radi ge trgt(2)+mrg)
      prom(junk) = 0
    endif

  endif

  out=indata
  bck=where( mmsk eq 1 )
  if bck(0) ne -1 then out(bck) = backgrnd

  return,out

end
