;+
;       NAME: spikes
;
;       PURPOSE: To locate single pixel spikes, like Cosmic Rays,
;		 in 2-D images.
;
;       METHOD: Use a three point median as a reference, take the
;		difference and compair with a cutoff.
;		The returned variable is a vector of pointers to 
;		spikes in the input variable.
;
;
;       CALLING SEQUENCE:  spikes=(img,cutoff,type='per',/remove)
;
;
;       PARAMETERS: 	img		input image 
;			cutoff		comparison cutoff
;
;	KEYWORDS:	remove		when set return corrected 
;					image
;			type		cutoff type, set to
;					"mmp", for percentage of total range,
;					"per", for percentage of local median,
;					"abs", for absolute value,
;					       this is default.
;
;	CALLS: 	 COORD_V2L
;
;	NOTES:   When the image data is an index into a cube, e.g. data(*,*,4),
;		 the REMOVE keyword is not effective. 
;		 Needs to be modified for SXT images, to account for ARs.
;
;
;       HISTORY: Drafted by A.McAllister, following a B.Labonte
;		 algorythm, 10-jun-93.
;		 Added code to handle borders, and keyword to
;		 return corrected image, 2-jul-93.
;		 trap for no spikes, 9-aug-93.
;		 added TYPE keyword, 21-jan-94.
;-
FUNCTION spikes,img,cutoff,remove=remove,type=type

if not keyword_set(type) then type="abs"

sz=size(img)
xmax=sz(1)-1
ymax=sz(2)-1

medn=median(img,3)

med0=where(medn eq 0.0)
if med0(0) ne -1 then medn(med0)=0.0000001			;trap for zeros

;add case for percentage of max(img) ??

case type of

	"abs": spkes=where((img - medn) gt cutoff)
	"mmp": spkes=where((img - medn) gt cutoff*(max(img)-min(img)))
	"per": spkes=where((img - medn)/abs(medn) gt cutoff)
endcase	

if keyword_set(remove) and spkes(0) ne -1 then begin

 help,spkes
 img(spkes)=medn(spkes)

endif

for i=0,ymax do begin			;vertical boundries

   ym=(i-1)>0
   yp=(i+1)<ymax
   medy0=median(img(0:1,ym:yp))
   medym=median(img((xmax-1):xmax,ym:yp))
   x0ss=where((img(0,i)-medy0) gt cutoff)
   xmss=where((img(xmax,i)-medym) gt cutoff)
   if x0ss(0) ne -1 then begin
     ll=coord_v2l([0,i],sz)
     spkes=[spkes,ll]
     if keyword_set(remove) then img(ll)=medy0
   end
   if xmss(0) ne -1 then begin
     ll=coord_v2l([xmax,i],sz)
     spkes=[spkes,ll]
     if keyword_set(remove) then img(ll)=medym
   end

endfor

for j=0,xmax do	begin		;horizontal boundries

   xm=(j-1)>0
   xp=(j+1)<xmax
   medx0=median(img(xm:xp,0:1))
   medxm=median(img(xm:xp,(ymax-1):ymax))
   y0ss=where((img(j,0)-medx0) gt cutoff)
   ymss=where((img(j,ymax)-medxm) gt cutoff)
   if y0ss(0) ne -1 then begin
     ll=coord_v2l([j,0],sz)
     spkes=[spkes,ll]
     if keyword_set(remove) then img(ll)=medx0
   endif
   if ymss(0) ne -1 then begin
     ll=coord_v2l([j,ymax],sz)
     spkes=[spkes,ll]
     if keyword_set(remove) then img(ll)=medxm
   endif

endfor

return,spkes

end
