;+
;       NAME: wl_process
;
;
;       PURPOSE: To process the SXT aspect sensor images and generate limbdarkening
;		 profiles, flat field images, white light images minus the dark
;		 current, flat field, and limbdarkening signals.
;
;
;       METHOD: First you select the images to process. Darkframe removal is 
;		performed. Single image limbdarkening profiles are created and
;		subtracted, and major sunspots are removed, before batch averaging
;		is done to derive the flat fields. These are removed from the 
;		dark frame corrected data, which are then aligned. Limbdarkening
;		profiles are made again, and averaged over the batches. Finally
;		fully processed images are created and stored to disk. The batch
;		averaged darkening profiles and flatfields are also stored.
;		The processing is done in batches of 10 unless the INC keyword 
;		is set. This should be set depending on your machines memory
;		capacity, and such as to break the dataset evenly, if there is
;		a small left over batch at the end the results may be poor.
;		In averaging for the flatfields 4-6 images are taken at
;		intervals.
;
;
;       CALLING SEQUENCE:	wl_process,infil[,inc=inc][,sscf=sscf][plcf=plcf]
;						[,/saa][,/night][,/all]
;
;
;       PARAMETERS:	infil	 	path/filename of SXT white light file 
;
;
;       KEYWORDS:	inc		size of batches to process. This will vary
;					depending on your machine's memory and the
;					number of selected images.
;			sscf		set cutoff for "sunspots", > flatfield 
;			plcf		factor to set plage cutoff, in flatfield
;			all		take all image in the file
;			saa		when set omit images taken in saa
;			night		when set omit images within 1 min of S/C night
;
;	CALLS: RT_SXTDT,DARKSUB,FINDLIMB,RAD_BCK,COORD_L2V,AREA_MEAN,TARGET,ALIGN_CUBE
;
;
;       HISTORY: Drafted by A. McAllister jun-93
;		 First release version, 16-jun-93, AMcA.
;		 Fixed data useage and cleanup, 1-jul-93, AMcA.
;		 Corrected application of SAA keyword, 25-jan-94, AMcA.
;		 Enhanced cosmic ray removal, 26-jan-94, AMcA.
;		 Added code to enlarge QR's in mixed sets, 27-jan-94, AMcA.
;		 Added call to SET_BACK before second LD pass, 2-feb-94, AMcA.
;		 Version 1.1  feb-2-94, AMcA.
;		 Use target in second call to RAD_BCK, feb-8-94, AMcA.
;		 Fixed the background at -100., feb-9-94, AMcA.
;		 Fixed indexing bug in resolution code, feb-10-94, AMcA.
;		 Replaced call to SXT_CENTER with call to GEN_LOC, feb-11-94, AMcA.
;     		 Removed new target formation after alignment, feb-14-94, AMcA.
;		 Version 1.2 feb-15-94, AMcA.
;
;-
PRO wl_process,infil,inc=inc,sscf=sscf,plcf=plcf,all=all,saa=saa,night=night

  version='-1.2'
  his_index,/enable					;turn on access to the index history tag
  crctof = 0.08						;percentage of image data range to use for SPIKES

  if not keyword_set(sscf) then  sscf = -600.		;get rid of deep spots (uncompressed)
  if not keyword_set(plcf) then  plcf =  50.		;get rid bright plage (uncompressed)

  prep_dir= getenv('DIR_WL_PREP')
  if prep_dir eq '' then spawn,'pwd',prep_dir

  if not keyword_set(all) then begin

    rt_sxtdt,infil,index,ss=ss,/nodata    		;select data from sda file

    tim2orbit,index,saa=sa,tim2night=nght		;optinally remove saa and sunset images
    if keyword_set(saa) then begin
      sssa=where(sa eq 0)
      if sssa(0) ne -1 then begin
        index=index(sssa)
        print, "Removed ",n_elements(where(sa eq 1))," saa frames."
        ss=ss(sssa)
      endif
    endif
    if keyword_set(night) then begin 
      ssdy=where(nght gt 1.0)
      if ssdy(0) ne -1 and n_elements(nght)-n_elements(ssdy) ne 0 then begin
        index=index(ssdy)
        print, "Removed ",n_elements(where(nght le 1.))," sunset frames."
        ss=ss(ssdy)		;what if we have both night and saa?
      endif
    endif

  endif else begin

    rd_xda,infil,-1,index,/nodata
    ss=indgen(n_elements(index))

  endelse
  
;  his_index,index						;add history tag to index

  if not keyword_set(inc) then begin

    input=' '
    read,'Please set the batch size: ',input			;add to this some helpful hints
    inc=fix(input)

  endif

  imgnum=n_elements(ss)
  count=0
  is=0
 
  rsz=0								;set resize flag
  res=gt_res(index)
								;for mixed resolution sets blow up
  qrt=where(res eq 2) & hlf=where(res eq 1)			;the quarter res to half res size
  if hlf(0) ne -1 and qrt(0) ne -1 then begin
     print,"WL_PROCESS: Mixed resolution data set, will expand QR frames."
     rsz=1
  endif 
   								;********start processing*******    
  while (imgnum-count gt 0) do begin			
    
    last=(count+inc-1)<(imgnum-1)
    ssn=indgen(last+1-count)                           	 	;indexing for prep file
    rd_xda,infil,ss(count:last),index,data_in                   ;get set of images
    his_index,index						;add history tag to index
    
    data_ds = float(dark_sub(index, data_in))			;feed to DARK_SUB
  
    if rsz eq 1 then begin					;resize QRs
    
      dsz=size(data_ds)
      qrt=where(res(count:last) eq 2)				;are there QRs in this batch?
      if qrt(0) ne -1 then begin
      
        case dsz(1) of 
        
          512:	data_ds(*,*,qrt)=mod_res(index(qrt),data_ds(*,*,qrt),nindex,outres=1)
      
          256:	data_ds=mod_res(index(qrt),data_ds,nindex,outres=1)  ;all QRs       	        
        	
        endcase
        
        index(qrt)=nindex & nindex=0
      endif
      
    endif 
 
    data_in = 0							;clean up
       
    if count eq 0 then begin					;create over all target
    
       mtch=0												
       hlf=where(res(count:last) eq 1)
       if rsz eq 1 and hlf(0) ne -1 then begin 
         mtch=hlf(0)						;use first HR image
       endif else mtch=0
       
       trgt=target(data_ds(*,*,mtch),index(mtch))
       trgsz=size(data_ds(*,*,mtch))
       trgta=[trgt(0:1),2*trgt(2),2*trgt(2),trgsz(1),trgsz(2)]	;set alignment target
       trgti=index(mtch)					;save alignment index
       
    endif

    for jj = 0, n_elements(ssn)-1 do begin
       data_ds(*,*,jj) = data_ds(*,*,jj)/gt_expdur(index(jj))*100. 	;gives DN/.1sec
    endfor

    data_sf=data_ds & data_sf(*,*,*)=0.0 			;initialize data_sf

print,'starting flat fielding',check_math(1,0)

    tbck = data_ds(*,*,0) & tbck(*,*)=0.0
    for jj = 0, n_elements(ssn)-1 do begin			;******** first LD pass and flat fielding

      tmp=data_ds(*,*,jj)					; when passed as a pointer, no update
      repeat tss=spikes(tmp,crctof,type="mmp",/remove) $
      					 until tss(0) eq -1	;screen for CRs
      data_ds(*,*,jj)=tmp 

      bck = rad_bck(data_ds(*,*,jj),step=1.0,res=2^gt_res(index(jj)))	;create background

      data_sf(*,*,jj) = data_ds(*,*,jj)-bck			;ss+ff

      tbck=tbck+bck						;add to composit background

      tmp2=data_sf(*,*,jj)
      sunspots=where(tmp2 lt sscf)	  			;find major spots
      if sunspots(0) ne -1 then tmp2(sunspots)=0.0
      plage=where(tmp2 gt plcf)		 			;find major plage
      if plage(0) ne -1 then tmp2(plage)=0.0
      data_sf(*,*,jj)=temporary(tmp2)			 	;changed to rough ff

print,'finished pass #',jj,check_math(1,0)

    endfor

    disk_ss = where(tbck gt 0.0)				;what points have some signal

    tmp=0							;clean up 
    med=0
    tmp2=0
    bck=0
    tbck=0

    spc=fix(n_elements(ssn)/7)+1				;every nth image
    dum=fix(indgen(n_elements(ssn))/spc)

    ffss=where(dum ne shift(dum,1))
    flatf = area_mean(data_sf(*,*,[ffss]),disk_ss)		;create average Flat Field

print,'created batch flatfield',check_math(1,0)

    data_sf=0							;clean up
    data_sd=data_ds & data_sd(*,*,*)=0.0 			;create some cubes

    for jj = 0, n_elements(ssn)-1 do  $
		data_sd(*,*,jj)=data_ds(*,*,jj)-flatf  		;ss+ld ..for now subtract

    data_ds=0							;clean up

print,'subtracted flatfield',check_math(1,0)

    len=n_elements(data_sd(0,0,*))+1
    sxtmsk=replicate(1,len)
    data_sd = align_cube(data_sd,sxtmsk,len-1,index,target=trgta,/rough) 	;feed to ALIGN_CUBE

print,'aligned cube',check_math(1,0)

    back_ar=data_sd & back_ar(*,*,*)=0.0 			;create back_ar
    profs=fltarr(n_elements(ssn),500)				;array for DL profiles

    for jj = 0, n_elements(ssn)-1 do begin		

      trget=[trgta(0:1),trgta(2)/2.,50.]
      data_sd(*,*,jj)=set_back(data_sd(*,*,jj),trgt=trget,back=-100.0)	;get rid of any crud beyond the limb
      
      back_ar(*,*,jj) = rad_bck(data_sd(*,*,jj),profile,target=trget,step=1.0)	
      proln=n_elements(profile)-1
      profs(jj,0:proln)=profile*10				;bring it back to DN/sec

    endfor

print,'created second pass backgrounds',check_math(1,0)

    disk_ss = where(back_ar(*,*,0) gt 0)
    back = area_mean(temporary(back_ar),disk_ss)		;average background
    back_ar=0							;clean up

print,'created average background',check_math(1,0)

    data_pro=data_sd & data_pro(*,*,*)=0.0			
								;remove back, DN/sec	
    for jj = 0,n_elements(ssn)-1 do $
				    data_pro(*,*,jj) = (data_sd(*,*,jj)-back)*10.

    data_sd=0							;clean up

print,'finished processing',check_math(1,0)

    if count eq 0 then begin					;save the results

      break_file,infil,dum,dum1,stem
      prep_file=prep_dir(0)+'/'+stem+'_proc.sda'+version
      sav_sda,prep_file,index,data_pro

    endif else begin

      sav_sda,prep_file,index,data_pro,/append

    endelse

    data_pro=0							;clean up
 
    iss=strtrim(string(is),1)
    dl_file=prep_dir(0)+'/'+stem+'_'+iss+'.dl'+version
    ff_file=prep_dir(0)+'/'+stem+'_'+iss+'.ff'+version

    save,filename=dl_file,index,back,profs
    save,filename=ff_file,index,flatf

    back=0
    profs=0
    flatf=0

    is=is+1
    count=count+inc

  endwhile

end






