;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: cntr_rnd
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, May 14, 1998
;
; Last Modified: Mon Jul 27 10:37:45 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO cntr_rnd, file, im, head=head, minint=minint, trace=trace, help=help, $
	      pause1=pause1, pause2=pause2
;+
;
; PROJECT:
;       SunRISE/PSPT
;
; NAME:
;       cntr_rnd     (procedure)
;
; PURPOSE:
;       To make a PSPT solar disk image round and to center it in the field of
;       view.
;
; CALLING SEQUENCE:
;
;       cntr_rnd, file [,im [,head=head, minint=minint, trace=trace,
;                             help=help]]
;
; INPUTS:
;       file - file name of the image to process.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       Writes centered-rounded image to disk in FITS format with a file name
;       file.geom.
;
; OPTIONAL OUTPUTS:
;       im - a variable to return the centered-rounded image to IDL session.
;     head - contains the updated header of the processed image.
;   minint - limb intensity threshold to use for approximating image geometry
;            (default 400.0).
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;      /trace.  Will display progress messages.
;
; CALLS:
;       readfits.pro, sxpar.pro, solargeom.pro, linchk.pro, sxaddpar.pro,
;       sxaddhist.pro, delvarx.pro, writefits.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written May 14, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;       Sept. 14, 1998 by Randy Meisner - added code to fill "dead pixels"
;            with average space value (fill_val).
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, May 14, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'cntr_rnd'
  RETURN
ENDIF

IF(N_PARAMS() NE 2) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  cntr_rnd, file, im [,head=head, minint=minint, ' + $
     'trace=trace, help=help]'
  PRINT, ''
  RETURN
ENDIF

; Housekeeping.

; Read in image and header.

IF(keyword_set(trace)) THEN $
   print, '-> Reading image and header...'

im = readfits(file, head, /silent)


IF(n_elements(minint) GT 0) THEN BEGIN 
  th = minint
ENDIF ELSE BEGIN
; th = 400.0
  th = find_minint(im)
  if(th le 0) then th=10.0
ENDELSE

IF(keyword_set(trace)) THEN $
  print, '-> cntr_rnd:  minint = ', th

;tvscl, rebin(im, 512, 512)
;xyouts, 10, 10, 'Original Image', charsize = 2, color = 255, /dev


; Get image dimensions.

sz = size(im)

nx = sz(1) & ny = sz(2)

ddx = fix(0.01*nx) & ddy = fix(0.01*ny)


; Get solar P angle from header for rotation of image.

p0 = sxpar(head, 'SOLAR_P0')


; Calculate solar disk center using least-squares fit to an ellipse.

IF(keyword_set(trace)) THEN $
   print, '-> Determining initial center coordinates...'


solargeom, im, th, 20, x0, y0, erx, ery

if(keyword_set(pause1)) then stop

; Rotate image by p0 and center in FOV.

IF(keyword_set(trace)) THEN $
   print, '-> Rotating and centering image...'

;sub_black = im(30:39, 5:14)
sub_black = im(100:109, 2000:2009)

fill_val = float(total(sub_black))/float(n_elements(sub_black))

IF(keyword_set(trace)) THEN $
   print, '-> Fill_Val = ', fill_val

im= rot(im, p0, 1.0, x0, y0, /interp, missing = fill_val)

;tvscl, rebin(im, 512, 512)
;xyouts, 10, 10, 'Rotated Image', charsize = 2, color = 255, /dev

; Set "dead pixel" columns and rows to value of fill_val

idx = where(im LT fill_val)

im(idx) = fill_val


; Sent new image center coordinate to center of FOV.

x0 = nx/2.0
y0 = ny/2.0


; Determine Rx and Ry.

IF(keyword_set(trace)) THEN $
   print, '-> Determining Rx and Ry...'

bim = im GT th                  ; generate bi-level image based on int. thresh

; determine pixel coord. proximity of east limb point and south limb point
; using derivatives of bi-level image.

bx = deriv(bim(0:x0, y0)) & by = deriv(bim(x0, 0:y0))

sx = where(bx EQ max(bx)) & sy = where(by EQ max(by))


; calculate 2nd derivative of x-y center profiles and use zero-crossings to
; determine the limb radius in each direction.

xx = deriv(deriv(im(0:x0, y0))) & yy = deriv(deriv(im(x0, 0:y0)))

xx(0:ddx) = 0.0 & yy(0:ddy) = 0.0

xx = xx(0:sx(0)+50) & yy = yy(0:sy(0)+50)


linchk, smooth(xx/max(xx), 5), y0, x0, y0, abs(x0-sx(0)), 0.25, xz, /row

linchk, smooth(yy/max(yy), 5), x0, x0, y0, abs(y0-sy(0)), 0.25, yz, /col

if(keyword_set(pause2)) then stop

rx = x0-xz(0) & ry = y0-yz(0)

print,'Rx: ',rx,'     Ry: ',ry

if(xz(0) eq -1 or yz(0) eq -1) then begin

  print,'xz, yz, rx, ry: ',xz(0), yz(0), rx, ry
  exit
  ;return

endif


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Randy Meisner modification July 7, 1998 BEGIN
; remove code to make image round
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


; Make solar disk round, using poly_2d to shrink/expand image in
; y-direction.

;IF(keyword_set(trace)) THEN $
;   print, '-> Rounding solar disk...'

;dr = ry/rx                      ; expansion factor for y dimension.

;print,'Rx=',rx,'  Ry=',ry,'  Dr=',dr

;p = [0, 0, 1, 0]
;q = [0, dr, 0, 0]

;im = poly_2d(im, p, q, 1.0, missing = 0.0)

;tvscl, rebin(im, 512, 512)
;xyouts, 10, 10, 'Rounded Image', charsize = 2, color = 255, /dev

; Determine new center of corrected image, using solargeom.pro

;IF(keyword_set(trace)) THEN $
;   print, '-> Determining final center coordinates...'

;xc = x0 & yc = y0
;solargeom, im, th, 25, xc, yc, erx, ery

; Center solar disk in field of view, using poly_2d.pro.

;IF(keyword_set(trace)) THEN $
;   print, '-> Re-centering rounded image...'

;p = [xc-x0, 0, 1, 0]
;q = [yc-y0, 1, 0, 0]
;im = poly_2d(im, p, q, 1.0, missing = 0.0)

;tvscl, rebin(im, 512, 512)
;xyouts, 10, 10, 'Centered Image', charsize = 2, color = 255, /dev

; Determine Rx and Ry of corrected image, using same method as above.

;IF(keyword_set(trace)) THEN $
;   print, '-> Determining final Rx and Ry...'
;bim = im GT th
;bx = deriv(bim(0:x0, y0)) & by = deriv(bim(x0, 0:y0))
;sx = where(bx EQ max(bx)) & sy = where(by EQ max(by))
;xx = deriv(deriv(im(0:x0, y0))) & yy = deriv(deriv(im(x0, 0:y0)))
;linchk, xx/max(xx), y0, x0, y0, abs(x0-sx(0)), 0.5, xz, /row
;linchk, yy/max(yy), x0, x0, y0, abs(y0-sy(0)), 0.5, yz, /col
;rx = x0-xz(0) & ry = y0-yz(0)
;print, xz
;print, yz

;print,'Rx =', rx, '   Ry =', ry

delvarx, bim

; Update header information.

IF(keyword_set(trace)) THEN $
   print, '-> Updating header...'

sxaddpar, head, 'CENTER_X', x0
sxaddpar, head, 'CENTER_Y', y0

sxaddpar, head, 'XRADIUS', rx
sxaddpar, head, 'YRADIUS', ry

today = systime()

sxaddhist, 'P angle removed ' + today, head

;sxaddhist, 'Solar disk rounded ' + today, head

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Randy Meisner modification July 7, 1998 END
; remove code to make image round
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

sxaddhist, 'Solar disk centered in FOV ' + today, head


; Save geometry corrected image to file.

IF(keyword_set(trace)) THEN $
   print, '-> Saving image to file ' + file + '.geom...'

writefits, file+'.geom', im, head

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'cntr_rnd.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
