;+
; NAME		flmb6
;
; PURPOSE	Call Peter Bandurian's flmbiter.f routine
;		to remove limb darkening.
;
; SUBROUTINES	flmbiter.f (Fortran, part of flmb library)
;		/riker/d/acos/band/sun/flmb.so
;		/riker/d/acos/band/sgi/flmb_sgi.so
;
; AUTHOR	Randy Meisner   HAO/NCAR
;
; HISTORY	Updates by Andrew L. Stanger   HAO/NCAR
;		22 Feb 2001: Add 1 to xc & yc for use in "flmbiter.f"
;                            since xc & yc are determined via C-language,
;			     and flmbiter.f uses Fortran indexing.
;		28 Feb 2001: Add rms_l & rms_h to flmbext argument list.
;		27 Jul 2001: reduce image size for limb darkening fit.
;-

PRO flmb6, file

   PRINT, 'flmb6> file: ', file
   niter = 20
   img = readfits (file, hdu)

   ;--- Extract infomation from FITS header.

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   xcen     = fxpar (hdu, 'CRPIX1')
   ycen     = fxpar (hdu, 'CRPIX2')
   rx       = fxpar (hdu, 'CRRADIUS')
   ry       = rx
   obs_date = fxpar (hdu, "DATE-OBS")
   obs_time = fxpar (hdu, "TIME-OBS")
   code_obs = fxpar (hdu, "CODE-OBS")
   telescop = fxpar (hdu, "TELESCOP")

   code_obs = STRLOWCASE (STRTRIM (code_obs, 2))
   telescop = STRLOWCASE (STRTRIM (telescop, 2))

;   PRINT, 'obs_date:', obs_date
;   PRINT, 'obs_time:', obs_time
;   PRINT, 'telescop: ', telescop
;   PRINT, 'code_obs: ', code_obs

   ;--- OBS-DATE: YYYY-MM-DD

   year  = strmid (obs_date, 0, 4)
   month = strmid (obs_date, 5, 2)
   day   = strmid (obs_date, 8, 2)
;   PRINT, 'year/month/day: ', year, month, day

   ;--- OBS-TIME: HH:MM:SS

   hour   = strmid (obs_time, 0, 2)
   minute = strmid (obs_time, 3, 2)
   second = strmid (obs_time, 6, 2)
;   PRINT, 'hour/minute/second: ', hour, minute, second

   if (strpos (telescop, "chip") GE 0) THEN tscp_code = "chp"
   if (strpos (telescop, "pics") GE 0) THEN tscp_code = "dpm"
   if (strpos (telescop, "mk4")  GE 0) THEN tscp_code = "mk4"

   niter = LONG (niter)

   ;--- Image size (pixels).

   xdim2 = xdim / 2
   ydim2 = ydim / 2

   PRINT, 'xdim2/ydim2: ', xdim2, ydim2

   ;--- Reduce image size.

   img  = FLOAT (img)
   img2 = rebin (img, xdim2, ydim2)
   img2 = FLOAT (img2)

   ;--- Sun center & radius.
   ;--- Add 1 to center coordinates, since Fortran indices start at 1, not 0.

   xcen  = (xdim - 1) * 0.5
   ycen  = (ydim - 1) * 0.5
   xcen2 = xcen / 2.0
   ycen2 = ycen / 2.0
   xc2 = FLOAT (xcen2) + 1.0
   yc2 = FLOAT (ycen2) + 1.0

   PRINT, 'xc2/yc2: ', xc2, yc2

   ;--- Solar radius (pixels).

   rx2 = rx / 2.0
   ry2 = ry / 2.0
   rad_t   = MAX ([rx2, ry2])
   rad_lim = rad_t - 4.0

   PRINT, 'rad_t: ', rad_t

;   rad_lim = rad_t

   ;--- Storage for Weight array.

   weight = FLTARR (xdim2, ydim2)

   ;--- m1: degree of polynomial in axis 1.
   ;--- m2: degree of polynomial in axis 2.

   m1 = 2L & m2 = 2L

   ;--- ex: array of powers in R (mu) to be fitted.

   ex = [1L, 3] & nex = N_ELEMENTS (ex)

   ;--- Dimension of fitting coefficients array.

   dimcoef = 8L

   ;--- coef: fitting coefficients to be returned.

   coef = FLTARR (dimcoef, niter)

   ;--- Fit array.

   fit2 = FLTARR (xdim2, ydim2, 2)

   ;--- Root Mean Square array.

   rms = FLTARR (niter)

   ;--- Set RMS limits.

   CASE code_obs OF
   'dsr':		$
       BEGIN
       rms_l = -1.0
       rms_h =  2.0
       END
   'csr':		$
       BEGIN
       rms_l = -1.0
       rms_h =  2.0
       END
   'bsr':		$
       BEGIN
       rms_l = -0.25
       rms_h =  3.0
       END
   'hsr':		$
       BEGIN
       rms_l = -0.25
       rms_h =  3.0
       END
   'rsr':		$
       BEGIN
       rms_l = -0.25
       rms_h =  3.0
       END
   'ssr':		$
       BEGIN
       rms_l = -1.0
       rms_h =  2.0
       END
   'tsr':		$
       BEGIN
       rms_l = -1.0
       rms_h =  2.0
       END
   ELSE:		$
       BEGIN
       rms_l = -1.0
       rms_h =  2.0
       END
   ENDCASE

   PRINT, 'rms_l/rms_h: ', rms_l, rms_h

   npts = LONARR (3, niter)

   envchk = strpos (strupcase (!version.os), 'IRIX')

   IF (envchk EQ 0) THEN BEGIN

      PRINT, 'SGI version of flmb.so'
;     dummy = call_external ('/riker/d/acos/band/sgi/flmb_sgi.so', 'flmbext_', $
      dummy = call_external ('/riker/d/acos/band/sgi/band.so', 'flmbext_', $
			     xc2, yc2, rad_t, rad_lim,			$
			     img2, xdim2, ydim2,			$
			     weight, m1, m2, ex, nex, fit2, niter,	$
			     rms, rms_l, rms_h,				$
			     coef, dimcoef, npts)

   ENDIF ELSE BEGIN

      PRINT, 'Solaris version of flmb.so'

;     dummy = call_external ('/riker/d/acos/band/sun/flmb_pll.so', 'flmbext_', $
      dummy = call_external ('/riker/d/acos/band/sun/flmb.so', 'flmbext_', $
			     xc2, yc2, rad_t, rad_lim,			$
			     img2, xdim2, ydim2,			$
			     weight, m1, m2, ex, nex, fit2, niter,	$
			     rms, rms_l, rms_h,				$
			     coef, dimcoef, npts)
   ENDELSE

   indx_it = WHERE (rms EQ 0, n_it)

   n_iter = niter - n_it

   PRINT, '# flmb iterations = ', n_iter

   ;--- Rebin fit image back to full size.

   fit = rebin (fit2 [*, *, 0], xdim, ydim)
   resid = img - fit

   indx_fit = WHERE (fit NE 0, n_fit)

   irr_2d = TOTAL (fit)

   irr_2d = irr_2d / 1.0e6

   resid = temporary (resid) / irr_2d

   PRINT, 'Integrated image intensity =', TOTAL (img (indx_fit)) / 1.0e6
   PRINT, '  Normalization value used = ', irr_2d

   min_res = MIN (resid)

   max_res = MAX (abs (resid))

   PRINT, 'min_res/max_res: ', min_res, max_res

   indx_zero = WHERE (fit eq 0)

   resid (indx_zero) = min_res - 0.1 * min_res

   BSCALE = max_res / 30000.0

   resid = fix ( temporary (resid) / BSCALE )

   PRINT, 'BSCALE = ', bscale

   mkhdr, hdr, resid
   hdu_resid = hdu

   sxaddpar, hdu_resid, 'CRRADIUS', (rx + ry) / 2.0
   sxaddpar, hdu_resid, 'CRPIX1', xcen
   sxaddpar, hdu_resid, 'CRPIX2', ycen
   sxaddpar, hdu_resid, 'TYPE-DP', 'LD REMOVED'
   sxaddpar, hdu_resid, 'DATAMIN', min_res
   sxaddpar, hdu_resid, 'DATAMAX', max_res
   sxaddpar, hdu_resid, 'DISPMIN', min_res
   sxaddpar, hdu_resid, 'DISPMAX', max_res

   sxaddpar, hdu_resid, 'BSCALE', bscale, 'DATA = BSCALE * ARRAY + BZERO', 'SUMX'
   sxaddpar, hdu_resid, 'BZERO', 0.0, 'DATA = BSCALE * ARRAY + BZERO', 'SUMX'
   sxaddpar, hdu_resid, 'IRR_2D', irr_2d, '2-D QUIET SUN IRRADIENCE ESTIMATE / 1.0e6'
   sxaddpar, hdu_resid, 'RAD_LIM', rad_lim, 'RESIDUAL RADIUS, IN PIXELS.'
   sxaddpar, hdu_resid, 'C0', coef (0,n_iter-1), 'FIT COEFF CONSTANT'
   sxaddpar, hdu_resid, 'C1', coef (1,n_iter-1), 'FIT COEFF X'
   sxaddpar, hdu_resid, 'C2', coef (2,n_iter-1), 'FIT COEFF Y'
   sxaddpar, hdu_resid, 'C3', coef (3,n_iter-1), 'FIT COEFF X^2'
   sxaddpar, hdu_resid, 'C4', coef (4,n_iter-1), 'FIT COEFF X*Y'
   sxaddpar, hdu_resid, 'C5', coef (5,n_iter-1), 'FIT COEFF Y^2'
   sxaddpar, hdu_resid, 'C6', coef (6,n_iter-1), 'FIT COEFF MU'
   sxaddpar, hdu_resid, 'C7', coef (7,n_iter-1), 'FIT COEFF MU^3'
   sxaddhist, 'RESIDUAL IMAGE IS SCALED BY IRR_2D.', hdu_resid
   sxaddhist, 'BACKGROUND = RESIDUAL MINIMUM MINUS 0.1 * MINIMUM.', hdu_resid

   ;--- Write residual image to disk.

   resfile = year + month + day + '.' + hour + minute + second + '.' + $
	     tscp_code + '.' + code_obs + '.resid.fts'

;   extpos = strpos (file, '.fts')
;   PRINT, 'extpos: ', extpos
;   resfile = strmid (file, 0, extpos) + '.resid.fts'

   WRITEFITS, resfile, resid, hdu_resid

   hdufit = hdu
   min_fit = MIN (fit (*, *, 0))
   max_fit = MAX (fit (*, *, 0))
   PRINT, 'min_fit/max_fit: ', min_fit, max_fit

   sxaddpar, hdufit, 'OBJECT', 'LIMB DARK FIT'
   sxaddpar, hdufit, 'DATAMIN', min_fit
   sxaddpar, hdufit, 'DATAMAX', max_fit
   sxaddpar, hdufit, 'DISPMIN', min_fit
   sxaddpar, hdufit, 'DISPMAX', max_fit

   ;--- Write limb-darkening-fitted image to disk.

   ldfile = year + month + day + '.' + hour + minute + second + '.' + $
	     tscp_code + '.' + code_obs + '.ldf.fts'

;   extpos = strpos (file, '.fts')
;   PRINT, 'extpos: ', extpos
;   ldfile = strmid (file, 0, extpos) + '.ldf.fts'

   WRITEFITS, ldfile, fit (*, *, 0), hdufit

END
