;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: flmb_chip.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, May 22, 1998
;
; Last Modified: Fri Jun 26 12:01:49 1998 by meisner (Randy Meisner) on hoth
; 9 Oct 2000: Adapted for use with ACOS/CHIP images [ALS].
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO flmb_chip, file, itr=itr, trace=trace, jrnl=jrnl, minint=minint, help=help
;+
;
; PROJECT:
;       ACOS/CHIP
;
; NAME:
;       flmb_chip     (procedure)
;
; PURPOSE:
;       To center and round a CHIP full disk image, remove limb darkening,
;       remove any gradients in the image, and create a residual image.
;
; CALLING SEQUENCE:
;
;       flmb_chip, file [,itr=itr, trace=trace, minint=minint, help=help]
;
; INPUTS:
;       file - file name of the CHIP image to process.
;
; OPTIONAL INPUTS: 
;       itr - number of iterations to perform in the Bandurian fit
;             (default is 20).
;
; OUTPUTS:
;       Writes residual image to disk in FITS format, file named file.resid.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;      /trace.  Will display progress messages.
;       /jrnl.  Will generate an IDL journal file for the run of this
;               procedure.
;
; CALLS:
;       headfits.pro, fxpar.jpro, cntr_rnd.pro, flmb_pll.so (external),
;       fxaddpar.pro, sxaddhist.pro, writefits.pro, hprint.pro, w512.pro,
;       find_minint.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;	MLSO/CHIP software.
;
; PREVIOUS HISTORY:
;       Written May 22, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;	Modifications by Andrew L. Stanger, HAO/NCAR.
;
; VERSION:
;       Version 1, May 22, 1998
;	Version 2: 9 Oct 2000 [ALS] CHIP images.
;-
;
ON_ERROR, 2

IF (N_ELEMENTS (help) GT 0) THEN BEGIN
  doc_library, 'flmb_chip'
  RETURN
ENDIF

IF (N_PARAMS () NE 1) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  flmb_chip, file [,itr=itr, trace=trace, minint=minint, help=help]'
  PRINT, ''
  RETURN
ENDIF

;--- Begin journal file, if requested.

IF (keyword_set (jrnl)) THEN journal, file + '.jrnl'

print, systime ()

;--- Set number of iterations.

IF (n_elements (itr) GT 0) THEN niter = long (itr) ELSE niter = 20L


;--- Open 512x512 graphic window, if one is not open

;IF (!d.window EQ -1) THEN w512, 8
;IF (!d.window NE -1 AND !d.x_size NE 512 AND !d.y_size NE 512) THEN w512, 8


;--- Center and round image.

;IF (keyword_set (trace)) THEN  $
;   print, '%FLMB_CHIP:  Centering and rounding image'


;--- Read in original header.

;h = headfits (file)

img = readfits (file, hdr)

;--- Determine limb thresholding.

;IF (n_elements (minint) GT 0) THEN mnt = minint $
;   ELSE mnt = find_minint (readfits (file))


;IF (keyword_set (trace)) THEN $
;   print, '%FLMB_CHIP:  Limb threshold is ' + strn (mnt) + '.'

;IF (keyword_set (trace)) THEN $
;   cntr_rnd, file, img, head = hdr, minint = minint, /trace  $
;ELSE $
;   cntr_rnd, file, img, head = hdr, minint = mnt


;--- Get new image geometry variables.

IF (keyword_set (trace)) THEN  $
   print, '%FLMB_CHIP:  Setting up flmb variables.'

;rx = fxpar (hdr, 'XRADIUS')
;ry = fxpar (hdr, 'YRADIUS')

;xc = fxpar (hdr, 'CENTER_X')
;yc = fxpar (hdr, 'CENTER_Y')

nx = fxpar (hdr, 'NAXIS1')
ny = fxpar (hdr, 'NAXIS2')
rx = fxpar (hdr, 'CRRADIUS')
ry = fxpar (hdr, 'CRRADIUS')
xc = fxpar (hdr, 'CRPIX1')
yc = fxpar (hdr, 'CRPIX2')

;--- Reduce image size.

;img = rebin (img, nx/2, ny/2)
;nx = nx / 2
;ny = ny / 2
;rx = rx / 2.0
;ry = ry / 2.0
;xc = xc / 2.0
;yc = yc / 2.0

;sz = size (img) & nx = sz (1) & ny = sz (2)

IF (xc LT 0.0) THEN xc = nx / 2 - 0.5
IF (yc LT 0.0) THEN yc = ny / 2 - 0.5

;window, xsize=nx, ysize=ny
;tvscl, img
;xyouts, 5, 5, file, /dev
;wait, 3

;--- Setup flmb.so input variables.

rad_t = max ([rx, ry]) & rad_lim = rad_t - 4.0

weight = fltarr (nx, ny)

m1 = 2L & m2 = 2L

ex = [1L, 3] & nex = n_elements (ex)

dimcoef = 8L

fit = fltarr (nx, ny, 2)

rms = fltarr (niter)

coef = fltarr (dimcoef, niter)

npts = lonarr (3, niter)

PRINT, 'xc,yc: ', xc, yc
PRINT, 'rad_t, rad_lim: ', rad_t, rad_lim
PRINT, 'size (img): ', size (img)
PRINT, 'nx, ny: ', nx, ny
PRINT, 'size (weight): ', size (weight)
PRINT, 'm1, m2: ', m1, m2
PRINT, 'size (ex): ', size (ex)
PRINT, 'nex: ', nex
PRINT, 'size (fit): ', size (fit)
PRINT, 'niter: ', niter
PRINT, 'size (rms): ', size (rms)
PRINT, 'size (coef): ', size (coef)
PRINT, 'dimcoef: ', dimcoef
PRINT, 'size (npts): ', size (npts)

;--- Perform the flmb fit.

IF (keyword_set (trace)) THEN  $
   print, '%FLMB_CHIP:  Running flmb fit.'

t0 = systime(1)

envchk = strpos (strupcase (!version.os), 'IRIX')

IF (envchk EQ 0) THEN BEGIN

    dummy = call_external ('/riker/d/acos/chip/flmb.so', 'flmbext_', $
		xc, yc, rad_t, rad_lim, img, nx, ny, weight, m1, m2, $
		ex, nex, fit, niter, rms, coef, dimcoef, npts)

ENDIF ELSE BEGIN

    dummy = call_external ('/riker/d/acos/chip/sun/flmb.so', 'flmbext_', $
                      xc, yc, rad_t, rad_lim, img, nx, ny, weight, m1, m2, $
                      ex, nex, fit, niter, rms, coef, dimcoef, npts)
ENDELSE

t1 = systime (1)

print, 'Total flmb time: ', (t1-t0)/60.0, 'min.', $
	format = '(/, a, 2x, f7.3, 2x, a, /)'


;--- Find number of iterations taken.

indx_it = where (rms EQ 0, n_it)
n_iter = niter - n_it

print, '# flmb iterations = ', n_iter

print, 'writefits: ', file + '.fit'
fxaddpar, hdr, 'DISPMIN', min (fit)
fxaddpar, hdr, 'DISPMAX', max (fit)
writefits, file + '.fit', fit, hdr

;tvscl, fit
;xyouts, 5, 5, 'FIT', /dev
;wait, 3

;--- Compute residual image.

IF (keyword_set (trace)) THEN  $
   print, '%FLMB_CHIP:  Creating residual image.'

resid = img - fit (*, *, 0)


;--- Compute 2D fit normalization.

indx_fit = where (fit (*, *, 0) NE 0, n_fit)

;irr_2d = total (fit (*, *, 0))


;--- Rationalize irradiances.

;irr_2d = irr_2d / 1.0e6


;--- Normalize residual image.

;resid = temporary (resid) / irr_2d

;print, 'Integrated image intensity =', total (img (indx_fit)) / 1.0e6

;print, '  Normalization value used = ', irr_2d


;writefits, file + '.resid', resid, hdr

;tvscl, rebin (resid, nx/2, ny/2)
;tvscl, resid
;xyouts, 5, 5, 'RESID', /dev
;wait, 3


;--- "Zero" residual image outside fitting radius.

min_res = min (resid)
max_res = max (abs (resid))

indx_zero = where (fit eq 0)

resid (indx_zero) = min_res - 0.1 * min_res

BSCALE = max_res / 30000.0

resid = fix ( temporary (resid) / BSCALE )

print, 'BSCALE = ', bscale

;tvscl, rebin (resid, nx/2, ny/2)
;tvscl, resid
;xyouts, 5, 5, 'RESIDUAL', /dev
;wait, 3


;--- Update header information.

IF (keyword_set (trace)) THEN  $
   print, '%FLMB_CHIP:  Updating residual header.'

fxaddpar, hdr, 'PROCSTAG', 5
;fxaddpar, hdr, 'BSCALE', bscale, 'DATA = BSCALE*ARRAY+BZERO', 'SUMX'
fxaddpar, hdr, 'BSCALE', bscale, 'DATA = BSCALE*ARRAY+BZERO'
;fxaddpar, hdr, 'BZERO', 0.0, 'DATA = BSCALE*ARRAY+BZERO', 'SUMX'
fxaddpar, hdr, 'BZERO', 0.0, 'DATA = BSCALE*ARRAY+BZERO'

;fxaddpar, hdr, 'IRR_2D', irr_2d, '2-D QUIET SUN IRRADIENCE ESTIMATE / 1.0e6.'

fxaddpar, hdr, 'RAD_LIM', rad_lim, 'RESIDUAL RADIUS, IN PIXELS.'

fxaddpar, hdr, 'C0', coef (0, n_iter-1), 'FIT COEFF CONSTANT'
fxaddpar, hdr, 'C1', coef (1, n_iter-1), 'FIT COEFF X'
fxaddpar, hdr, 'C2', coef (2, n_iter-1), 'FIT COEFF Y'
fxaddpar, hdr, 'C3', coef (3, n_iter-1), 'FIT COEFF X^2'
fxaddpar, hdr, 'C4', coef (4, n_iter-1), 'FIT COEFF X*Y'
fxaddpar, hdr, 'C5', coef (5, n_iter-1), 'FIT COEFF Y^2'
fxaddpar, hdr, 'C6', coef (6, n_iter-1), 'FIT COEFF MU'
fxaddpar, hdr, 'C7', coef (7, n_iter-1), 'FIT COEFF MU^3'

;fxaddpar, hdr, 'HISTORY', 'RESIDUAL IMAGE IS SCALED BY IRR_2D.', hdr

;fxaddpar, hdr, 'HISTORY', 'BACKGROUND = RESIDUAL MINIMUM MINUS .1*MINIMUM.', hdr


;--- Write residual image and header to disk.

IF (keyword_set (trace)) THEN  $
   print, '%FLMB_CHIP:  Saving residual image in ' + file + '.resid'

print, 'writefits: ', file + '.resid'
fxaddpar, hdr, 'DISPMIN', min (resid)
fxaddpar, hdr, 'DISPMAX', max (resid)
writefits, file + '.resid', resid, hdr


;--- Add new header info to database

;resid_db, hdr


;--- Display new header if requested.

;IF (keyword_set (trace)) THEN hprint, hdr

IF (keyword_set (trace)) THEN BEGIN

   FOR i = 0, n_elements (hdr)-1 DO print, strtrim (hdr [i], 2)

ENDIF

print, systime ()


;--- Close journal file.

IF (keyword_set (jrnl)) THEN journal

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'flmb_chip.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
