;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: flmb_rise.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, May 22, 1998
;
; Last Modified: Fri Jun 26 12:01:49 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO flmb_rise, file, itr=itr, trace=trace, jrnl=jrnl, minint=minint, help=help
;+
;
; PROJECT:
;       SunRISE/PSPT
;
; NAME:
;       flmb_rise     (procedure)
;
; PURPOSE:
;       To center and round a PSPT full disk image, remove limb darkening,
;       remove any gradients in the image, and create a residual image.
;
; CALLING SEQUENCE:
;
;       flmb_rise, file [,itr=itr, trace=trace, minint=minint, help=help]
;
; INPUTS:
;       file - file name of the PSPT image to process.
;
; OPTIONAL INPUTS: 
;       itr - number of iterations to perform in the Bandurian fit
;             (default is 20).
;
; OUTPUTS:
;       Writes residual image to disk in FITS format, file named file.resid.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;      /trace.  Will display progress messages.
;       /jrnl.  Will generate an IDL journal file for the run of this
;               procedure.
;
; CALLS:
;       headfits.pro, sxpar.jpro, cntr_rnd.pro, flmb_pll.so (external),
;       sxaddpar.pro, sxaddhist.pro, writefits.pro, hprint.pro, w512.pro,
;       find_minint.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written May 22, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, May 22, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'flmb_rise'
  RETURN
ENDIF

IF(N_PARAMS() NE 1) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  flmb_rise, file [,itr=itr, trace=trace, minint=minint, help=help]'
  PRINT, ''
  RETURN
ENDIF

; Begin journal file, if requested.

IF(keyword_set(jrnl)) THEN journal, file+'.jrnl'

print, systime()

; Set number of iterations.

IF(n_elements(itr) GT 0) THEN niter = long(itr) ELSE niter = 20L


; Open 512x512 graphic window, if one is not open

;IF(!d.window EQ -1) THEN w512, 8
;IF(!d.window NE -1 AND !d.x_size NE 512 AND !d.y_size NE 512) THEN w512, 8


; Center and round image.


IF(keyword_set(trace)) THEN  $
   print, '%FLMB_RISE:  Centering and rounding image'


; read in original header.

h = headfits(file)


; determine limb thresholding.

IF(n_elements(minint) GT 0) THEN mnt = minint $
   ELSE mnt = find_minint(readfits(file))


IF(keyword_set(trace)) THEN $
   print, '%FLMB_RISE:  Limb threshold is ' + strn(mnt) + '.'

IF(keyword_set(trace)) THEN $
   cntr_rnd, file, im, head = hdr, minint = minint, /trace  $
ELSE $
   cntr_rnd, file, im, head = hdr, minint = mnt


; Get new image geometry variables.

IF(keyword_set(trace)) THEN  $
   print, '%FLMB_RISE:  Setting up flmb variables.'

rx = sxpar(hdr, 'XRADIUS')
ry = sxpar(hdr, 'YRADIUS')

xc = sxpar(hdr, 'CENTER_X')
yc = sxpar(hdr, 'CENTER_Y')


sz = size(im) & nx = sz(1) & ny = sz(2)

; Setup flmb.so input variables.

rad_t = max([rx, ry]) & rad_lim = rad_t-4.0

weight = fltarr(nx, ny)

m1 = 2L & m2 = 2L

ex = [1L, 3] & nex = n_elements(ex)

dimcoef = 8L

fit = fltarr(nx, ny, 2)

rms = fltarr(niter)

coef = fltarr(dimcoef, niter)

npts = lonarr(3, niter)



; Perform the flmb fit.

IF(keyword_set(trace)) THEN  $
   print, '%FLMB_RISE:  Running flmb fit.'

t0 = systime(1)

envchk = strpos(strupcase(!version.os), 'IRIX')

IF(envchk EQ 0) THEN BEGIN

    dummy = call_external('/pspt/d/pspt/IDL/SO/IRIX/flmb_sgi.so', 'flmbext_', $
		xc, yc, rad_t, rad_lim, im, nx, ny, weight, m1, m2, $
		ex, nex, fit, niter, rms, coef, dimcoef, npts)

ENDIF ELSE BEGIN

    dummy = call_external('/pspt/d/pspt/IDL/SO/flmb_pll.so', 'flmbext_', $
                      xc, yc, rad_t, rad_lim, im, nx, ny, weight, m1, m2, $
                      ex, nex, fit, niter, rms, coef, dimcoef, npts)
ENDELSE

t1 = systime(1)

print, 'Total flmb time: ', (t1-t0)/60., 'min.', $
	format = '(/, a, 2x, f7.3, 2x, a, /)'


; Find number of iterations taken.

indx_it = where(rms EQ 0, n_it)
n_iter = niter-n_it

print, '# flmb iterations = ', n_iter


; Compute residual image.

IF(keyword_set(trace)) THEN  $
   print, '%FLMB_RISE:  Creating residual image.'

resid = im - fit(*, *, 0)


; Compute 2D fit normalization.

indx_fit = where(fit(*, *, 0) NE 0, n_fit)

irr_2d = total(fit(*, *, 0))


; Rationalize irradiances.

irr_2d = irr_2d/1.0e6


; Normalize residual image.

resid = temporary(resid)/irr_2d

print, 'Integrated image intensity =', total(im(indx_fit))/1.0e6

print, '  Normalization value used = ', irr_2d


;tvscl, rebin(resid, 512, 512)


; "Zero" residual image outside fitting radius.

min_res = min(resid)
max_res = max(abs(resid))

indx_zero = where(fit eq 0)

resid(indx_zero) = min_res - .1*min_res

BSCALE = max_res/30000.0

resid = fix( temporary(resid)/BSCALE )

print, 'BSCALE = ', bscale

;tvscl, rebin(resid, 512, 512)


; Update header information.

IF(keyword_set(trace)) THEN  $
   print, '%FLMB_RISE:  Updating residual header.'

sxaddpar, hdr, 'PROCSTAG', 5
sxaddpar, hdr, 'BSCALE', bscale, 'DATA = BSCALE*ARRAY+BZERO', 'SUMX'
sxaddpar, hdr, 'BZERO', 0.0, 'DATA = BSCALE*ARRAY+BZERO', 'SUMX'

sxaddpar, hdr, 'IRR_2D', irr_2d, '2-D QUIET SUN IRRADIENCE ESTIMATE / 1.0e6.'

sxaddpar, hdr, 'RAD_LIM', rad_lim, 'RESIDUAL RADIUS, IN PIXELS.'

sxaddpar, hdr, 'C0', coef(0,n_iter-1), 'FIT COEFF CONSTANT'
sxaddpar, hdr, 'C1', coef(1,n_iter-1), 'FIT COEFF X'
sxaddpar, hdr, 'C2', coef(2,n_iter-1), 'FIT COEFF Y'
sxaddpar, hdr, 'C3', coef(3,n_iter-1), 'FIT COEFF X^2'
sxaddpar, hdr, 'C4', coef(4,n_iter-1), 'FIT COEFF X*Y'
sxaddpar, hdr, 'C5', coef(5,n_iter-1), 'FIT COEFF Y^2'
sxaddpar, hdr, 'C6', coef(6,n_iter-1), 'FIT COEFF MU'
sxaddpar, hdr, 'C7', coef(7,n_iter-1), 'FIT COEFF MU^3'

sxaddhist, 'RESIDUAL IMAGE IS SCALED BY IRR_2D.', hdr

sxaddhist, 'BACKGROUND = RESIDUAL MINIMUM MINUS .1*MINIMUM.', hdr


; Write residual image and header to disk.

IF(keyword_set(trace)) THEN  $
   print, '%FLMB_RISE:  Saving residual image in ' + file+'.resid'

writefits, file+'.resid', resid, hdr


; Add new header info to database

resid_db, hdr


; Display new header if requested.

;IF(keyword_set(trace)) THEN hprint, hdr

IF(keyword_set(trace)) THEN BEGIN

   FOR i = 0, n_elements(hdr)-1 DO print, strtrim(hdr(i), 2)

ENDIF

print, systime()


; Close journal file.

IF(keyword_set(jrnl)) THEN journal

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'flmb_rise.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
