;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: linchk.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 20, 1998
;
; Last Modified: Mon Feb 23 16:07:25 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO linchk, x, idx, xc, yc, r, thresh, zcross, row=row, col=col, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       linchk     (procedure)
;
; PURPOSE:
;       To locate zero crossing points in a 1D vector from a Laplacian image.
;
; CALLING SEQUENCE:
;
;       linchk, x, idx, xc, yc, r, thresh, zcross, row=row, col=col
;               [,help=help]
;
; INPUTS:
;       x - the 1D vector (row or column) from a Laplacian image.
;     idx - the image index of the row or column being passed.
;      xc - the estimated x coordinate of the center of the solar disk.
;      yc - the estimated y coordinate of the center of the solar disk.
;  thresh - the minimum amplitude threshold to consider in the Laplacian image
;           to denote the location of an edge.
;
;     row=row, col=col - one of these keywords must be set to denote if a row
;     or a column is being passed from the Laplacian image.
;   
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       zcross - a 1D array containing the interpolated indicies of the input
;                vector of where the qualifying zero crossings occur.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;
; CALLS:
;       lrminmax.pro, rtspline.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written February 20, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 20, 1998
;-
;
;ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'SunRISE'
  RETURN
ENDIF

IF(N_PARAMS() NE 7) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  linchk, x, idx, xc, yc, r, thresh, zcross, row=row, ' + $
     'col=col [,help=help]'
  PRINT, ''
  RETURN
ENDIF

; Initialize zero crossing index array.

zcross = [-1.0]

; Get number of elements in input vector.

nx = n_elements(x)


; Loop over all elements of vector and check for zero crossings.

FOR i = 0.0, nx-2 DO BEGIN
   
                                ; Get incremental pair of data points from
                                ; vector.
   a = x(i) & b = x(i+1)
   

   IF(a*b LT 0) THEN BEGIN      ; check for zero crossing.
      
                                ; determine location of point relative to
                                ; estimated solar disk center/limb.
      
      IF(keyword_set(row)) THEN rc = sqrt( (i-xc)^2 + (idx-yc)^2 )
      IF(keyword_set(col)) THEN rc = sqrt( (idx-xc)^2 + (i-yc)^2 )
      
                                ; if point within +/- 7 pixels of estimated
                                ; limb, then process/record it.
      
      IF(abs(r-rc) LT 10) THEN BEGIN
         
                                ; find surrounding local min and max.
         
         IF(a GT 0) THEN BEGIN
            
            ll = lrminmax(x, i, dir = -1, mm = 1)
            
            rr = lrminmax(x, i, dir = 1, mm = -1)
            
         ENDIF ELSE BEGIN
            
            ll = lrminmax(x, i, dir = -1, mm = -1)
            
            rr = lrminmax(x, i, dir = 1, mm = 1)
            
         ENDELSE
         
                                ; check amplitude of local min/max against
                                ; threshold.
         
         IF(abs(x(rr)-x(ll)) GE thresh AND abs(x(rr)) GE thresh/3.0 $
            AND abs(x(ll)) GE thresh/3.0) THEN BEGIN
            
            xx = findgen(rr-ll+1)+ll
            
            yy = x(ll:rr)
            
                                ; check for more than 2 interpolating points.
            
            IF(n_elements(xx) GT 2) THEN BEGIN
               
                                ; interpolate zero crossing.
               ;zc = rtspline(xx, yy, float(ll), float(rr))
               zc = rtsafe(xx, yy, float(ll), float(rr), acc = 1.e-3)
               
                                ; record zero crossing.
               zcross = [zcross, zc]
               
            ENDIF
            
         ENDIF
         
      ENDIF
      
   ENDIF
   
   
ENDFOR

; If zero crossings found, then remove default -1 value from output.

IF(n_elements(zcross) GE 2) THEN zcross = zcross(where(zcross NE -1))

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'linchk.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
