;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: find_minint.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, May 26, 1998
;
; Last Modified: Thu May 28 16:10:12 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION find_minint, im, help=help
;+
;
; PROJECT:
;       SunRISE/PSPT
;
; NAME:
;       find_minint     (function)
;
; PURPOSE:
;       To find an approximate minimum value of the solar disk intensity at
;       the limb for defining an intensity threshold.
;
; CALLING SEQUENCE:
;
;       result = find_minint(im [,help=help])
;
; INPUTS:
;       im - a full disk solar image.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       Returns the intensity value of the solar disk limb.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;
; CALLS:
;       None.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written May 26, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, May 26, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'find_minint'
  RETURN, -999
ENDIF

IF(N_PARAMS() NE 1) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  result = find_minint(im [,help=help])'
  PRINT, ''
  RETURN, -999
ENDIF

; Get the dimensions of the image.

sz = size(im) & nx2 = sz(1)/2 & ny2 = sz(2)/2

; Extract the first half of the center row of pixels.

step = long(0.01*sz(1))

row = im(step:nx2, ny2)

; Take the derivative of the row of pixels.

drow = deriv(row)

; Normalize the absolute value of the derivative of the row of pixels.

adrow = abs(drow)/max(abs(drow))

; Find the one or two pixels near the limb which have the steepest gradient
; from the row of pixels.

lmbpt = where(adrow GT 0.75) + step

; Return the limb intensity of the pixel value at/near the limb.

minint0 = im(lmbpt(0), ny2)


; Perform the above steps 2 more time and take min of 3 samples.

dy = long(0.25*sz(2))

row = im(step:nx2, ny2+dy)
drow = deriv(row)
adrow = abs(drow)/max(abs(drow))
lmbpt = where(adrow GT 0.75) + step
minint1 = im(lmbpt(0), ny2+dy)


row = im(step:nx2, ny2-dy)
drow = deriv(row)
adrow = abs(drow)/max(abs(drow))
lmbpt = where(adrow GT 0.75) + step
minint2 = im(lmbpt(0), ny2-dy)

mmtt = [minint0, minint1, minint2]

mmtt = mmtt(where(mmtt ne -1))

minint = min(mmtt)

RETURN, minint

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'find_minint.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
