;+
; NAME		calmk4a.pro
;
; PURPOSE
;		Routine to calibrate MkIV images
;
; INPUT:	The root name for a coronal image file
;		contained in the file "callist".
;		This file has +I, _Q, and _U appended
;		to read the three coronal stokes files
;		Also read are
;		bias, gain, X21, X31, X22, X23, X32, X33
;		and Pangle
;
;		All files must have been run through a spline fit
;		to have NUMPIX x NUMAZ elements and to be
;		in the instrument coordinate frame.
;
;		Calibrations from xmk4 need to have been either
;		moved, smoothed, or fit with movex4a, smoothx4a or fitx4a.
;		from the Xnn to Xnnfit files.
;
; OUTPUT:
;		Calibrated corona files in heliocentric coordinates
;		called cali, calq, and calu
;
; AUTHOR	 David Elmore	11/5/98   5/11/01
;
; HISTORY	16 May 2001 [ALS]: Add PRO statement.
;-

PRO calmk4a, listfile

dodepol = 'y'
dostop  = 'n'

;--- As of 5/11/01, the cameara moves so little, the stretch correction
;--- is not needed.

dostretch = 'n'

print, "dodepol=", dodepol, "  dostretch=", dostretch

listfile = 'callist'
filename = 'mk4'
close, 2 & openr, 2, listfile 
readf, 2, filename & close, 2

head   = 0 & head   = bytarr (1024) ;;
headib = 0 & headib = lonarr (16)  
headf  = 0 & headf  = fltarr (24)
headie = 0 & headie = lonarr (16)  

top =long (0)

close, 1 & openr, 1, filename + '.Ci' & point_lun, -1, top & readu, 1, headib
                                                             readu, 1, headf
                                                             readu, 1, headie
                                        point_lun,  1, top & readu, 1, head
close, 1

;;;;;;;;;;;;;;;;;;;;;;;;;

numaz = headib  [9]
numpix = headib [10]
pangle = headf [0]
print, numaz, numpix, pangle

;;;;;;;;;;;;;;;;;;;;;;;;;

loadct, 16
window, 1, xsize=numaz, ysize=numpix-53, title='U'
window, 2, xsize=numaz, ysize=numpix-53, title='I'
window, 0, xsize=numaz, ysize=numpix-53, title='Q'
window, 3, xsize=numaz, ysize=numpix-53, xpos = 1280-numaz, ypos=numpix+50

;--- Should be set to total number of files.

meanq = fltarr (numpix, 200)
meanu = fltarr (numpix, 200)
meani = fltarr (numpix, 200)
imglinelow  = fltarr (200)
imglinehigh = fltarr (200)

corona_I = fltarr (numpix, numaz)
corona_Q = fltarr (numpix, numaz)
corona_U = fltarr (numpix, numaz)

gain = fltarr (numpix, numaz)
bias = fltarr (numpix, numaz)
X21 =  fltarr (numpix, numaz)
X31 =  fltarr (numpix, numaz)
X22 =  fltarr (numpix, numaz)
X23 =  fltarr (numpix, numaz)
X32 =  fltarr (numpix, numaz)
X33 =  fltarr (numpix, numaz)
Xinv11 =  fltarr (numpix, numaz)
Xinv12 =  fltarr (numpix, numaz)
Xinv13 =  fltarr (numpix, numaz)
Xinv21 =  fltarr (numpix, numaz)
Xinv22 =  fltarr (numpix, numaz)
Xinv23 =  fltarr (numpix, numaz)
Xinv31 =  fltarr (numpix, numaz)
Xinv32 =  fltarr (numpix, numaz)
Xinv33 =  fltarr (numpix, numaz)

pheight = findgen (numpix)
lines = [[90, 260], [120, 290]]

;--- All use of drift function has been removed 5/01/2001.

str = filename
tmpstr = 'fred'

;--- Read in calibration matrix elements

close, 1 & openr, 1, 'gain.fit' & readu, 1, gain
close, 1 & openr, 1, 'bias.fit' & readu, 1, bias
close, 1 & openr, 1, 'x21.fit'  & readu, 1, X21
close, 1 & openr, 1, 'x31.fit'  & readu, 1, X31
close, 1 & openr, 1, 'x22.fit'  & readu, 1, X22
close, 1 & openr, 1, 'x23.fit'  & readu, 1, X23
close, 1 & openr, 1, 'x32.fit'  & readu, 1, X32
close, 1 & openr, 1, 'x33.fit'  & readu, 1, X33

;--- What are the heights of the hairlines in the gain image?

if ( dostretch eq 'y' ) then begin
	meangain = total (gain, 2)
	fit = gaussfit (pheight (lines [0, 0]:lines[0, 1]),  $
		        meangain [lines [0, 0]:lines [0, 1]], r, nterms=6)
	linelow = r [1]
	fit = gaussfit (pheight (lines [1, 0]:lines [1, 1]),  $
                        meangain [lines [1, 0]:lines [1, 1]], r, nterms=6)
	linehigh = r [1]

	print, "hairlines", linelow, linehigh
endif

;--- Create inverse polarimeter response matrix.

det = 1.0 / (x22 * x33 - x23 * x32)
xinv11 [*, *] = 1.0
xinv12 (*, *) = 0.0
xinv13 (*, *) = 0.0
xinv21 = det * (x23 * x31 - x21 * x33)
xinv22 = det * (x33)
xinv23 = det * (-x23)
xinv31 = det * (x21 * x32 - x31 * x22)
xinv32 = det * (-x32)
xinv33 = det * (x22)

;--- Loop through coronal image files applying
;--- calibration and writing out result.
;--- Depol is applied to calibrated pB.

close, 2 & openr, 2, 'callist' 

first = 1
!y.range = [-1.e-8, 1.e-7]
!y.style = 1
!x.style = 1
!x.ticklen = 1
!y.ticklen = 1

ifile = 0
while ( not eof (2) ) do begin
	readf, 2, filename

	str = string (filename + '.Ci')
	close, 1 & openr, 1, str & readu, 1, head & readu, 1, corona_I

	str = string (filename + '.Cq')
	close,  1 & openr, 1, str &  readu, 1, head & readu, 1, corona_Q

	str = string (filename + '.Cu')
	close,  1 & openr, 1, str &  readu, 1, head & readu, 1, corona_U

	cali = fltarr (numpix, numaz)
	calq = fltarr (numpix, numaz)
	calu = fltarr (numpix, numaz)

	corona_i = corona_i - float (bias)

if ( dostretch eq 'y' ) then begin
	print, "doing stretch"
	avgi = total (corona_I, 2)
	fit = gaussfit (pheight (lines [0, 0]:lines [0, 1]),  $
		        avgi (lines [0, 0]:lines [0, 1]), rlow, nterms=6)
	fit = gaussfit (pheight (lines [1, 0]:lines [1, 1]),  $
		        avgi (lines [1, 0]:lines [1, 1]), rhigh, nterms=6)
	print, 'Image harilines', rlow [1], rhigh [1]
	newpix = numpix * (rhigh [1] - rlow [1]) / (linehigh - linelow)
	if ( newpix ge numpix ) then begin
		fred = congrid (corona_I, newpix, numaz)
		corona_I = fred (0:numpix-1, *)
		fred = congrid (corona_Q, newpix, numaz)
		corona_Q = fred (0:numpix-1, *)
		fred = congrid (corona_U, newpix, numaz)
		corona_U = fred (0:numpix-1, *)
	endif else begin
		fred = congrid (corona_I, newpix, numaz)
		corona_I [0:newpix-1, *] = fred
		fred = congrid (corona_Q, newpix, numaz)
		corona_Q [0:newpix-1, *] = fred
		fred = congrid (corona_U, newpix, numaz)
		corona_U [0:newpix-1, *] = fred
	endelse

	;--- Correct the offset.

	avgi = total (corona_I, 2)
	fit = gaussfit (pheight (lines [0, 0]:lines [0, 1]),  $
		        avgi (lines [0, 0]:lines [0, 1]), r, nterms=6)
	imglinelow [ifile] = r [1]
	fit = gaussfit (pheight (lines [1, 0]:lines [1, 1]),  $
		        avgi (lines [1, 0]:lines [1, 1]), r, nterms=6)
	imglinehigh [ifile] = r [1]
	imgshift = linelow - imglinelow [ifile]
	ishift = fix (imgshift)
	frac = imgshift - ishift
	if ( imgshift ge 0 ) then begin
		corona_I = shift (corona_I, ishift, 0) * (1.0 - frac) + $
			          shift (corona_I, ishift + 1, 0) * frac
		corona_Q = shift (corona_Q, ishift, 0) * (1.0 - frac) + $
			          shift (corona_Q, ishift + 1, 0) * frac
		corona_U = shift (corona_U, ishift, 0) * (1.0 - frac) + $
			          shift (corona_U, ishift + 1, 0) * frac
	endif else begin
		corona_I = shift (corona_I, ishift, 0) * (1 + frac) + $
			          shift (corona_I, ishift - 1, 0) * (-frac)
		corona_Q = shift (corona_Q, ishift, 0) * (1 + frac) + $
			          shift (corona_Q, ishift - 1, 0) * (-frac)
		corona_U = shift (corona_U, ishift, 0) * (1 + frac) + $
			          shift (corona_U, ishift - 1, 0) * (-frac)
	endelse
		
endif

;--- Apply inverse X matrix to measured data.

cali = xinv11 * corona_i + xinv12 * corona_q + xinv13 * corona_u
calq = xinv21 * corona_i + xinv22 * corona_q + xinv23 * corona_u
calu = xinv31 * corona_i + xinv32 * corona_q + xinv33 * corona_u

calq = calq / gain
calu = calu / gain
cali = cali / gain

if ( dodepol eq 'y' ) then begin

	;--- Remove two per scan component.

	depol, numpix, numaz, cali, calq, calu, skyq, skyu

	calq = calq - skyq
	calu = calu - skyu

	;--- Remove one per scan component.

	depolscan, numpix, numaz, cali, calq, calu, skyq, skyu

	calq = calq - skyq
	calu = calu - skyu

	;--- Attempt to extract corona from intensity data.

	sky = cali
	meansky = total (sky, 2) / numaz
	for ii=0, numaz-1 do cali [*, ii] = cali [*, ii] - meansky
	sintrend , numpix, numaz, cali, trendi
	cali = cali - trendi
	
endif

;--- Shift for pangle.

pixshift = -pangle * NUMAZ / 360.0
	cali = shift (cali, 0, pixshift)
	calq = shift (calq, 0, pixshift)
	calu = shift (calu, 0, pixshift)

	str = string (filename + '.CalI')
	close, 1 & openw, 1, str & writeu, 1, head & writeu, 1, (cali * 1.e9)

	str = string (filename + '.CalpB')
	close, 1 & openw, 1, str & writeu, 1, head & writeu, 1, (calq * 1.e10)

	str = string (filename + '.CalU')
	close, 1 & openw, 1, str & writeu, 1, head & writeu, 1, (calu * 1.e10)

	print, filename
	wset, 0
	tv, bytscl (transpose (alog10 (calq [30:370, *] + 5.e-9)), -9, -5.7)
	wset, 1
	tv, bytscl (transpose (calu [30:370, *]), -1.e-8, 1.e-8)
	wset, 2
	tv, bytscl (transpose (cali [30:370, *]), -1.e-5, 2.e-5)
	wset, 3

	meanq (*, ifile) = total (calq, 2) / float (numaz)
	meanu (*, ifile) = total (calu, 2) / float (numaz)
	meani (*, ifile) = total (cali, 2) / float (numaz)

	if ( first eq 1 ) then begin
		!p.title = "pB vs Height"
		plot,  meanq [*, ifile]
		oplot, meanu [*, ifile], color=128
		first = 0
	endif else begin
		oplot, meanq [*, ifile]
		oplot, meanu [*, ifile], color=128
	endelse

	ifile = ifile + 1

if ( dostop eq 'y' ) then stop

endwhile
close, 1
print, 'done'
stop
!y.range = [1.e-9, 1.e-6]
!p.title = 'Average pB vs. Scan # for Various Coronal Heights'
!y.title = "Average Calibrated pB"
!x.title = "Scan Number"
plot_io, meanq [275, 0:ifile-1], psym=1
for ii=50, 250, 25 do oplot, meanq [ii, 0:ifile-1], psym=1

openw, 1, 'meanq'
writeu, 1, meanq
close, 1

;stop


end

