;+
; NAME		xmk4a.pro
;		Array polarimeter response matrices
;
; PURPOSE	Find the X matrix for MkIV ;
;
; INPUT 	File names are in xmk4list.
;		Already spline-fit cal0, cal15, dark, and corona images.
;		File names must be in order for cal0, cal15, dark, and corona.
;
; OUTPUT 	2D arrays 
;		bias, X21, x22, x23, x31, x32, x33, and gain
;
;		It is assumed bias for Q and U is zero
;		It is assumed X12 and X13 are zero
;
; METHOD	Routine follows procedure in MkIII Calibration Technote.
;		Matrix convention is element ( row, column ).
;
; AUTHOR	David Elmore   HAO/NCAR   9 May 2001
;-
PRO xmk4a, listfile

numpix = 384
numaz  = 720
scalepix = 150
radfac = !pi / 180.0

!x.ticklen = 1
!y.ticklen = 1
!x.style = 1
!y.style = 1
!y.range = 0
!x.range = 0

;
;--- This is the difference between reported barrel angle
;--- and that needed to minimize calibrated coronal U in degrees.

phaseoffset = 2.35

;--- Values from technote.

;--- Transmission of opal.

Bopal = 10.11e-6

;--- Fractional polarization of tilt plates at 15 degrees.

fpt = 0.01543

gain = fltarr (numpix, numaz)
bias = fltarr (numpix, numaz)
x21  = fltarr (numpix, numaz)
x31  = fltarr (numpix, numaz)
x22  = fltarr (numpix, numaz)
x23  = fltarr (numpix, numaz)
x33  = fltarr (numpix, numaz)
x32  = fltarr (numpix, numaz)

;listfile = 'xmk4list'
filename = 'mk4'
cal0_I   = fltarr (numpix, numaz)
cal0_Q   = fltarr (numpix, numaz)
cal0_U   = fltarr (numpix, numaz)
cal15_I  = fltarr (numpix, numaz)
cal15_Q  = fltarr (numpix, numaz)
cal15_U  = fltarr (numpix, numaz)
dark_I   = fltarr (numpix, numaz)
dark_Q   = fltarr (numpix, numaz)
dark_U   = fltarr (numpix, numaz)
corona_I = fltarr (numpix, numaz)
corona_Q = fltarr (numpix, numaz)
corona_U = fltarr (numpix, numaz)
cx21     = fltarr (numpix, numaz)
cx31     = fltarr (numpix, numaz)
headl = lonarr (256)

close, 1
close, 2 & openr, 2, listfile

;--- Cal 0 [opal]:

readf, 2, filename
print, filename

openr, 1, filename + '.Ci' & readu, 1, headl & readu, 1, cal0_I & close, 1
openr, 1, filename + '.Cq' & readu, 1, headl & readu, 1, cal0_Q & close, 1
openr, 1, filename + '.Cu' & readu, 1, headl & readu, 1, cal0_U & close, 1

;--- Cal 15 [tilt plate]:

readf, 2, filename
print, filename

openr, 1, filename + '.Ci' & readu, 1, headl & readu, 1, cal15_I & close, 1
openr, 1, filename + '.Cq' & readu, 1, headl & readu, 1, cal15_Q & close, 1
openr, 1, filename + '.Cu' & readu, 1, headl & readu, 1, cal15_U & close, 1

;--- Dark:

readf, 2, filename
print, filename

openr, 1, filename + '.Ci' & readu, 1, headl & readu, 1, dark_I & close, 1
openr, 1, filename + '.Cq' & readu, 1, headl & readu, 1, dark_Q & close, 1
openr, 1, filename + '.Cu' & readu, 1, headl & readu, 1, dark_U & close, 1

;--- Corona:

readf, 2, filename
print, filename

openr, 1, filename + '.Ci' & readu, 1, headl & readu, 1, corona_I & close, 1
openr, 1, filename + '.Cq' & readu, 1, headl & readu, 1, corona_Q & close, 1
openr, 1, filename + '.Cu' & readu, 1, headl & readu, 1, corona_U & close, 1

;--- Perform bias correction.

bias = dark_I

cal0_I   = cal0_I   - bias
cal15_I  = cal15_I  - bias
corona_I = corona_I - bias

;--- In this version, all cal arrays are 2D.

;--- Gain:

gain = cal0_I / Bopal

;--- Intensity to polarization.

;--- Use coronal Intensity to polarization if it shows
;--- more crosstalk than that from the opal.
;--- Smooth corona.  Smooth opal.  Take the difference
;--- between smoothed corona and corona and add to opal

X21 = cal0_Q / cal0_I
X31 = cal0_U / cal0_I

OX21 = leefilt (X21)

cx21 = corona_q / corona_i - leefilt (corona_q / corona_i)
cx31 = corona_u / corona_i

; a = where ( cx21 lt x21 )
; x21(a) = cx21(a)
; x21(300:numpix-1, *) = cx21(300:numpix-1, *)
; a = where ( cx31 lt x31 )
; x31(a) = cx31(a)
; x31(300:numpix-1, *) = cx31(300:numpix-1, *)

X21 = OX21 + CX21

;--- Polarized diffraction around occulter shows up only in corona data.

for ii = 29, 37 do begin
	x21 [ii, *] = x21 [ii, *] + cx21 [ii, *] - cx21 [38, *]
	x31 [ii, *] = x31 [ii, *] + cx31 [ii, *] - cx31 [38, *]
end

tv, bytscl (transpose (x21), -0.2, 0.1)

;--- Angles will run from -180 to 180.

ang = (findgen (numaz) - numaz / 2.0) * 2.0 * !pi / float (numaz)
sang = sin (2.0 * ang)
cang = cos (2.0 * ang)

qphi  = fltarr (numpix)
uphi  = fltarr (numpix)
phi   = fltarr (numpix)
sprod = fltarr (numpix)
cprod = fltarr (numpix)
qamp  = fltarr (numpix)
uamp  = fltarr (numpix)
amp   = fltarr (numpix)
rat   = fltarr (numpix)
qmean = fltarr (numpix)
umean = fltarr (numpix)

;--- Find amplitude and phase of cal 15
;--- at position angle of channel 4.

fitq = fltarr (numpix, numaz)
fitu = fltarr (numpix, numaz)

;--- Compute X22 and X33.
;--- Use fractional polarization normalizaton.
;--- Remove fractional polarization due to opal alone
;--- from fractional polarization of cal 15.

fred = cal15_q / cal15_i - cal0_q / cal0_i
mary = cal15_u / cal15_i - cal0_u / cal0_i

for ii = 0, NUMPIX-1 do begin
	sprod [ii] = total (sang * fred [ii, *]) / numaz
	cprod [ii] = total (cang * fred [ii, *]) / numaz
endfor

qphi = atan (sprod, cprod)
qamp = 2.0 * sqrt (sprod * sprod + cprod * cprod)
qmean = total (fred, 2) / numaz

for ii = 0, NUMPIX-1 do begin
	sprod [ii] = total (sang * mary [ii, *]) / numaz
	cprod [ii] = total (cang * mary [ii, *]) / numaz
endfor

uphi = atan (sprod, cprod)
uamp = 2.0 * sqrt (sprod * sprod + cprod * cprod)
umean = total (mary, 2) / numaz

fitu = uamp [200] * cos (2.0 * ang - uphi [200])
fitq = qamp [200] * cos (2.0 * ang - qphi [200])

amp = (qamp + uamp) / 2.0
phi = (qphi + (uphi + !pi / 2.0)) / 2.0

;--- This is where correction to coronal coordinate frame is done.
;--- Two cycles of tilt plate modulation per scan.

phi = phi - 2.0 * phaseoffset * !pi / 180.0

plot,  fred [200, *], psym=3
oplot, fitq
oplot, mary [200, *], psym=3
oplot, fitu
;stop

;--- Signs here do not match the paper.
;--- This is because when the barrel rotates, the orientation
;--- of the calibration polarizer rotates backwards.

!p.title = "X21"
!y.range = [-.3, .3]
plot,  ox21 [*, numaz / 2], color=200
oplot, cx21 [*, numaz / 2], color=128
oplot, x21  [*, numaz / 2]
;stop

!p.title = "X31"
plot,  x31  [*, numaz / 2]
oplot, cx31 [*, numaz / 2], color=0
;stop

sx22 =  amp * cos (2.0 * phi) / fpt
!p.title = "X22"
!y.range = [-40, 40]
plot, sx22
;stop

sX23 = -amp * sin (2.0 * phi) / fpt
!p.title = "X23"
plot, sx23
;stop

sx32 = amp * sin (2.0 * phi) / fpt
!p.title = "X32"
plot, sx32
;stop

sx33 = amp * cos (2.0 * phi) / fpt
!p.title = "X33"
plot, sx33
;stop

for ii=0, NUMAZ-1 do begin
	x33 [*, ii] = sx33
	x22 [*, ii] = sx22
	x32 [*, ii] = sx32
	x23 [*, ii] = sx23
endfor

;--- Write bias to disc.

close, 1 & openw, 1, 'bias' & writeu, 1, bias
!p.title='Bias'
!y.range = [-200, 200]
plot, bias [*, numaz / 2]
;stop

;--- Write gain to disc.

close, 1 & openw, 1, 'gain' & writeu, 1, gain
!y.range = [0, 1.7e9]
!p.title = 'Gain'
plot, gain [*, numaz / 2]
;stop

;--- Write X-matrix values to disc.

close, 1 & openw, 1, 'x21' & writeu, 1, x21
close, 1 & openw, 1, 'x31' & writeu, 1, x31
close, 1 & openw, 1, 'x22' & writeu, 1, x22
close, 1 & openw, 1, 'x23' & writeu, 1, x23
close, 1 & openw, 1, 'x32' & writeu, 1, x32
close, 1 & openw, 1, 'x33' & writeu, 1, x33

print, 'done'
;stop

end
