function dpm2fits, dpmfile

;+
; IDL function
;
; NAME:
;	DPM2FITS
; PURPOSE:
;	Reads a binary MLSO/DPM image file from disk and writes a FITS
;	file contaning the image.
;	A detailed FITS header appears at the beginning of the FITS output file.
; CATEGORY:
;	Input/Output
; CALLING SEQUENCE:
;	dpm2fits, dpmfile
; INPUTS:
;	dpmfile
; OUTPUTS:
;	FITS file containing MLSO/DPM image.
; COMMON BLOCKS:
;	none.
; SIDE EFFECTS:
;	A DPM FITS file is created on disk.
; RESTRICTIONS:
;	none.
; MODIFICATION HISTORY:
;	Andrew L. Stanger   HAO/NCAR   22 August 1994
;-

   dpmh = {dpmhdr, version : 0L, $
	   year     :  0L, month    :  0L, day      : 0L, $
	   hour     :  0L, minute   :  0L, sec      : 0L, $
	   xdim     :  0L, ydim     :  0L, nsum     : 0L, $
	   exposure :  0L, gain     :  0L, $
	   odisk    :  0L, type     :  0L, $
	   optic1   :  0L, optic2   :  0L, optic3   :  0L, optic4   :  0L, $
	   optic5   :  0L, optic6   :  0L, optic7   :  0L, optic8   :  0L, $
	   pangle   : 0.0, bangle   : 0.0, langle   : 0.0, hangle   : 0.0, $
	  elevation : 0.0, azimuth  : 0.0, amass    : 0.0, radius   : 0.0, $
	  empty     : bytarr (2946)}


   ; Open DPM file.  Read DPM header and image pixels.

   get_lun, unit
   openr, unit, dpmfile
   readu, unit, dpmh

   xdim = 1534
   ydim = 1029

   dpmimg = intarr (xdim, ydim)			; image array (16-bit int)

   readu, unit, dpmimg
   close, unit

   ; Define arguments for write_fits.

   date_obs = string (dpmh.day,    format='(i2)') + "/" $
	    + string (dpmh.month,  format='(i2)') + "/" $
	    + string (dpmh.year,   format='(i2)')

   time_obs = string (dpmh.hour,   format='(i2)') + "." $
	    + string (dpmh.minute, format='(i2)') + "." $
	    + string (dpmh.sec,    format='(i2)')

   origin   = "HAO"
   telescop = "DPM"
   instrume = "CCD"

   comment = "   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF"
   comment = "   HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),"
   comment = "   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),"
   comment = "   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF)."
   comment = "   THE DIGITAL PROMINENCE MONITOR (DPM) OBSERVES THE SOLAR DISK & LIMB"
   comment = "   WITH AN APPROXIMATE 1.6 x 2.2 SOLAR RADIUS FIELD OF VIEW, USING A"
   comment = "   KODAK CCD DETECTOR (1534x1032 PIXELS, OF WHICH 1534x1029 ARE USED)."

   comment = "   IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP."

   ; Create FITS file name.

   fitsfile = dpmfile + ".fit"

   ; Write FITS file to disk.

   write_fits, dpmimg, fitsfile, $
	       date_obs=date_obs, time_obs=time_obs, $
	       origin=origin,     telescop=telescop, instrume=instrume, $
	       comment=comment

   return, fitsfile
end

