function dpm3fits, dpmfile

;+
; IDL function
;
; NAME:
;	DPM3FITS
; PURPOSE:
;	Reads a binary MLSO/DPM image file from disk and writes a FITS
;	file contaning the image.
;	A detailed FITS header appears at the beginning of the FITS output file.
; CATEGORY:
;	Input/Output
; CALLING SEQUENCE:
;	dpm3fits, dpmfile
; INPUTS:
;	dpmfile
; OUTPUTS:
;	FITS file containing MLSO/DPM image.
; COMMON BLOCKS:
;	none.
; SIDE EFFECTS:
;	A DPM FITS file is created on disk.
; RESTRICTIONS:
;	none.
; MODIFICATION HISTORY:
;	Andrew L. Stanger   HAO/NCAR   22 August 1994
;-

@dpm.struct

   print, 'size (dpmh): ', size (dpmh)

   ; Open DPM file.  Read DPM header and image pixels.

   get_lun, unit
   openr, unit, dpmfile
   readu, unit, dpmh

;   print, 'dpmh: ', dpmh

   point_lun, -unit, dpmpos

   print, 'dpmpos: ', dpmpos

   dpmimg = intarr (dpmh.xdim, dpmh.ydim)	; image array (16-bit int)

   print, 'xdim: ', dpmh.xdim, '  ydim: ', dpmh.ydim

   readu, unit, dpmimg
   close, unit
   free_lun, unit

   ; Locate disk center. 

   simg = dpmimg (256:1279,0:1023)		; central subset of DPM image.
   if (dpmh.type EQ 1) then $
      find_limb, simg, xcen, ycen, disc_rad, rad_err, oblateness 

   print, 'xcen: ', xcen, '  ycen: ', ycen, '  disc_rad: ', disc_rad
   xcen = xcen + 256.0

   ; Define parameters for wdpm_fits.

   dpmsiz = size (dpmimg)				; array size info
   ndim   = dpmsiz (0)					; # dimensions

   print, 'size (dpmimg): ', size (dpmimg)

   fkey.naxis    = dpmsiz (0)				; # dimensions
   fkey.naxis1   = dpmsiz (1)				; dimension 1 size
   if (ndim GT 1) then fkey.naxis2   = dpmsiz (2)	; dimension 2 size
   type_code = dpmsiz (ndim + 1)			; type code
   nelem     = dpmsiz (ndim + 2)			; # pixels in dpmimg
   nbypix = 2						; bytes/pixel
   fkey.bitpix = nbypix * 8				; bits/pixel

   print, 'naxis1: ', fkey.naxis1, '  naxis2: ', fkey.naxis2

   fkey.object   = 'SUN     '
   fkey.origin   = 'HAO     '
   fkey.telescop = 'DPM     '
   fkey.instrume = 'CDD     '

   fkey.date_obs = string (dpmh.day,    format='(i2)') + "/" $
	         + string (dpmh.month,  format='(i2)') + "/" $
	         + string (dpmh.year,   format='(i2)')

   fkey.time_obs = string (dpmh.hour,   format='(i2)') + "." $
	         + string (dpmh.minute, format='(i2)') + "." $
	         + string (dpmh.sec,    format='(i2)')

   fkey.ctype1   = 'X       '		; axis 1 type: cartesian horizontal
   fkey.crpix1   = xcen			; axis 1 reference sun center [pixels]
   fkey.crval1   = 0.0			; axis 1 reference sun center [arcsec]
   fkey.cdelt1   = 3.0060		; axis 1 increment [arcsec/pixel]

   fkey.ctype2   = 'Y       '		; axis 2 type: cartesian vertical  
   fkey.crpix2   = ycen			; axis 2 reference sun center [pixels]
   fkey.crval2   = 0.0			; axis 2 reference sun center [arcsec]
   fkey.cdelt2   = 3.0060		; axis 2 increment [arcsec/pixel]

   fkey.disp_dir = 'RU      '		; display direction

   fkey.crradius = dpmh.radius/3.0060	; solar radius [pixels]
   fkey.rsun     = dpmh.radius		; solar radius [arcsec]
   fkey.occ_disc = float (dpmh.odisk)	; occulting disc diameter [arcsec]

   fkey.solar_p0 = dpmh.pangle		; P0 angle [degrees]
   fkey.solar_b0 = dpmh.bangle		; B0 angle [degrees]
   fkey.solar_l0 = dpmh.langle		; L0 angle [degrees] = Carr. Longitude
   fkey.hour_ang = dpmh.hangle		; solar hour angle

   fkey.elev     = dpmh.elevation	; elevation [degrees]
   fkey.azimuth  = dpmh.azimuth		; azimuth [degrees]

   fkey.airmass  = dpmh.amass		; airmass [atmospheres]

   obstype = ['OTHER   ', $
              'DISC    ', 'LIMB    ', 'REF     ', 'DIFFUSER', $
	      'DISCDARK', 'LIMBDARK', 'DIFUDARK', 'REF_DARK']

   fkey.type_obs = obstype (dpmh.type)	; observation type
   fkey.code_obs = dpmh.type		; observation code

   fkey.nimavg   = dpmh.nsum		; # images averaged
   fkey.expdur   = dpmh.exposure / 100.0	; exposure duration [seconds]

   fkey.bunit    = 'CCD     '		; brightness units
   fkey.bscale   = 1.0			; physical = data * bscale + bzero
   fkey.bzero    = 0.0			; offset

   dpmmax = max (dpmimg, min=dpmmin)
   print, 'max (dpmimg): ', dpmmax, '  !C: ', !C, '   dpmmin: ', dpmmin

   dpmmax = 0
;   for iy = 0, fkey.naxis2 - 1 do $
;      for ix = 0, fkey.naxis1 - 1 do $
;         if (dpmimg (ix,iy) > dpmmax)  then  dpmmax = dpmimg (ix, iy)
;   print, 'dpmmax: ', dpmmax

   fkey.datamin  = float (min (dpmimg))	; data minimum value in image array
   fkey.datamax  = float (max (dpmimg < 4000))	; data maximum value in image array
   fkey.dispmin  = 0.0			; data minimum value for display
   fkey.dispmax  = 0.0			; data maximum value for display

   fkey.optdev1  = 'BANDPASS'		; optical device 1
   fkey.optdev2  = 'APERTURE'		; optical device 2
   fkey.optdev3  = 'DAYSTAR '		; optical device 3
   fkey.optdev4  = 'FILTER  '		; optical device 4
   fkey.optdev5  = 'OCCULT  '		; optical device 5
   fkey.optdev6  = 'PICTURE '		; optical device 6
   fkey.optdev7  = 'STEPS   '		; optical device 7
   fkey.optdev8  = 'FLIP    '		; optical device 8

   dev1pos = strarr (11)
   dev1pos = ['UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'TIMEOUT ']

   dev2pos = strarr (11)
   dev2pos = ['CLEAR   ', 'DIFFUSER', 'OPAL    ', 'POLARIZER', 'UNKNOWN ', $
	      'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ',  'UNKNOWN ', $
	      'TIMEOUT ']

   dev3pos = strarr (11)
   dev3pos = ['IN      ', 'OUT     ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'TIMEOUT ']

   dev4pos = strarr (11)
   dev4pos = ['658.1   ', '654.0   ', 'BLOCKED ', 'BLOCKED ', 'BLOCKED ', $
	      '655.5   ', '656.3   ', '657.1   ', 'UNKNOWN ', 'UNKNOWN ', $
	      'TIMEOUT ']

   dev5pos = strarr (11)
   dev5pos = ['WEDGE   ', '2.0R0_OCCULT', 'CLEAR   ', '1.25R0_OCCULT', $
	      'LIMB    ', 'RETICLE ',     'UNKNOWN ', 'UNKNOWN ',      $
	      'UNKNOWN ', 'UNKNOWN ',     'TIMEOUT ']

   dev6pos = strarr (11)
   dev6pos = ['UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'TIMEOUT ']

   dev7pos = strarr (11)
   dev7pos = ['UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'TIMEOUT ']

   dev8pos = strarr (11)
   dev8pos = ['UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', 'UNKNOWN ', $
	      'TIMEOUT ']

   fkey.optpos1  = dev1pos (dpmh.optic1)	; optical position 1
   fkey.optpos2  = dev2pos (dpmh.optic2)	; optical position 2
   fkey.optpos3  = dev3pos (dpmh.optic3)	; optical position 3
   fkey.optpos4  = dev4pos (dpmh.optic4)	; optical position 4
   fkey.optpos5  = dev5pos (dpmh.optic5)	; optical position 5
   fkey.optpos6  = dev6pos (dpmh.optic6)	; optical position 6
   fkey.optpos7  = dev7pos (dpmh.optic7)	; optical position 7
   fkey.optpos8  = dev8pos (dpmh.optic8)	; optical position 8

   fkey.version  = 'DPM 1.01'		; version designation

   cdate = systime (0)			; current date: DOW MON DD HH:MM:SS YEAR
   cmon  = strmid (systime (0),  4, 3) 	; current month
   cday  = strmid (systime (0),  8, 2)	; current day of month
   chour = strmid (systime (0), 11, 2)	; current hour
   cmin  = strmid (systime (0), 14, 2)	; current minute
   csec  = strmid (systime (0), 17, 2)	; current second
   cyear = strmid (systime (0), 20, 4)	; current year

   print, 'size (cdate): ', size (cdate)
   print, 'cdate: ', cdate
   print, 'cmon: ', cmon, '  cday: ', cday, '  cyear: ', cyear
   print, 'chour: ', chour, '  cmin: ', cmin, '  csec: ', csec

   case cmon of
      'Jan' : smon = 1
      'Feb' : smon = 2
      'Mar' : smon = 3
      'Apr' : smon = 4
      'May' : smon = 5
      'Jun' : smon = 6
      'Jul' : smon = 7
      'Aug' : smon = 8
      'Sep' : smon = 9
      'Oct' : smon = 10
      'Nov' : smon = 11
      'Dec' : smon = 12
      else: smon = 0
   endcase

   cmon = string (smon)

   sday  = fix (cday )
   shour = fix (chour)
   smin  = fix (cmin )
   ssec  = fix (csec )
   syear = fix (cyear)

   print, 'sday: ', sday, '  smon: ', smon, '  syear: ', syear

   syear = syear - 1900
   cyear = string (syear)

   print, 'cday: ', cday, '  cmon: ', cmon, '  cyear: ', cyear

   fkey.date     = string (cday, format='(i2)') + "/" $
	         + string (cmon, format='(i2)') + "/" $
	         + string (cyear,format='(i2)')

   print, 'fkey.date: ', fkey.date
;   fkey.date     = '00/00/00'		; date when FITS file is created
   fkey.date_cal = '00/00/00'		; date when FITS file was calibrated

   ; Create FITS file name.

   fitsfile = dpmfile + ".fst"

   ; Write FITS file to disk.

   wdpm_fits, dpmimg, fitsfile, fkey

   return, fitsfile
end
