pro wdpm_fits, img, filnam, fkey, index = fnum

;+
; NAME:
;        WDPM_FITS
; PURPOSE:
;        Writes a MLSO/DPM array into a standard FITS file on the disk.
;        A complete FITS header is created automatically.
; CATEGORY:
;        Input/Output
; CALLING SEQUENCE:
;        wfits, array, filename, fkey
; INPUTS:
;        img    = image array.
;        filnam = string containing the file name.
;        fkey   = fits keyword structure
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;        index = nonnegative integer. If this parameter is present, a period
;              and the index number are appended to the filename (e.g., '.34').
;              This option makes handling of data in the MCCD file naming
;              convention easier.
; OUTPUTS:
;        No explicit outputs.
; COMMON BLOCKS:
;        None.
; SIDE EFFECTS:
;        A file is created, or overwritten.
; RESTRICTIONS:
;        If array is not of type byte, integer, or long, then a conversion
;        to type integer is done. No scaling is made beforehand, so significant
;        loss of information is possible.
; MODIFICATION HISTORY:
;        JPW, Nov, 1989.
;        Andrew Stanger (HAO/NCAR), 6 May 1994:
;           Added keywords origin, telescop, instrume, object, type_obs.
;        Andrew Stanger (HAO/NCAR), 10 May 1994:
;           Added keywords bscale, bzero.
;	 Andrew Stanger (HAO/NCAR), 23 August 1994:
;	    replace fits keyword arguments with fkey structure
;-

; open FITS file

if (n_elements (fnum) ne 0) $
   then file = filnam + '.' + string (format = '(i0)', fnum) $
   else file = filnam
get_lun, unit
openw, unit, file

; Check data type, and make conversion if necessary.

dpmsiz = size (img)
case dpmsiz (dpmsiz (0) + 1) of
   1 : begin
          bitpix = 8
       ;    print, 'Note : FITS image is  8 bits deep (BITPIX = 8) '
       end
   2 : begin
	  bitpix = 16
          print, 'Note : FITS image is 16 bits deep (BITPIX = 8) '
       end
   3 : begin
          bitpix = 32
          print, 'Note : FITS image is 32 bits deep (BITPIX = 32) '
       end
   else : begin
          bitpix = 16
          imgb = fix (img)
          print, 'Note : img converted to 16 bit integer,  no scaling made '
          endelse
endcase

; Create header.

space = 32b				; ASCII blank value
blank = bytarr (80) + space

; SIMPLE

skey = string (format = '("SIMPLE  = ", 19x, a1)', 'T')
scom = '/ FITS FILE: MLSO DIGITAL PROMINENCE MONITOR (DPM)'
bkey = bytarr (80) + space
bkey ( 0) = byte (skey)
;bkey (31) = byte (scom)
head = bkey 

; BITPIX

skey = string (format = '("BITPIX  = ", i20)', bitpix)
bkey = bytarr (80) + space
bkey (0) = byte  (skey)
head = [[head], [bkey]]

; NAXIS

skey = string (format = '("NAXIS   = ", i20)', dpmsiz (0))
bkey = bytarr (80) + space
bkey (0) = byte  (skey)
head = [[head], [bkey]]

; NAXISi

for i = 1, dpmsiz (0) do begin
   skey = string (format = '("NAXIS", i1, "  = ", i20)', i, dpmsiz (i))
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
endfor

; BLANK

   head = [[head], [blank]]
   head = [[head], [blank]]

; OBJECT

;if (n_elements (object) eq 1) then begin
   skey = "OBJECT  = '" + strmid (fkey.object, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; ORIGIN

;if (n_elements (origin) eq 1) then begin
   skey = "ORIGIN  = '" + strmid (fkey.origin, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; TELESCOP

;if (n_elements (telescop) eq 1) then begin
   skey = "TELESCOP= '" + strmid (fkey.telescop, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; INSTRUME

;if (n_elements (instrume) eq 1) then begin
   skey = "INSTRUME= '" + strmid (fkey.instrume, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;;endif

; DATE-OBS

;if (n_elements (date_obs) eq 1) then begin
   skey = "DATE-OBS= '" + strmid (fkey.date_obs, 0, 18) + "'"
   bkey = bytarr (80) + space
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
;endif

; TIME-OBS

;if (n_elements (time_obs) eq 1) then begin
   skey = "TIME-OBS= '" + strmid (fkey.time_obs, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CTYPE1

;if (n_elements (ctype1  ) eq 1) then begin
   skey = "CTYPE1  = '" + strmid (fkey.ctype1  , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CRPIX1

;if (n_elements (crpix1) eq 1) then begin
   skey = string (format = '("CRPIX1  ", "= ", f20.2)', fkey.crpix1)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CRVAL1

;if (n_elements (crval1) eq 1) then begin
   skey = string (format = '("CRVAL1  ", "= ", f20.2)', fkey.crval1)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CDELT1

;if (n_elements (cdelt1) eq 1) then begin
   skey = string (format = '("CDELT1  ", "= ", f20.4)', fkey.cdelt1)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CTYPE2

;if (n_elements (ctype2  ) eq 1) then begin
   skey = "CTYPE2  = '" + strmid (fkey.ctype2  , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CRPIX2

;if (n_elements (crpix2) eq 1) then begin
   skey = string (format = '("CRPIX2  ", "= ", f20.2)', fkey.crpix2)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CRVAL2

;if (n_elements (crval2) eq 1) then begin
   skey = string (format = '("CRVAL2  ", "= ", f20.2)', fkey.crval2)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CDELT2

;if (n_elements (cdelt2) eq 1) then begin
   skey = string (format = '("CDELT2  ", "= ", f20.4)', fkey.cdelt2)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DISP-DIR

;if (n_elements (disp_dir) eq 1) then begin
   skey = "DISP-DIR= '" + strmid (fkey.disp_dir, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CRRADIUS

;if (n_elements (crradius) eq 1) then begin
   skey = string (format = '("CRRADIUS", "= ", f20.2)', fkey.crradius)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; RSUN

;if (n_elements (rsun    ) eq 1) then begin
   skey = string (format = '("RSUN    ", "= ", f20.2)', fkey.rsun    )
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OCC-DISC

;if (n_elements (occ_disc) eq 1) then begin
   skey = string (format = '("OCC-DISC", "= ", f20.2)', fkey.occ_disc)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; SOLAR_P0

;if (n_elements (solar_p0) eq 1) then begin
   skey = string (format = '("SOLAR_P0", "= ", f20.2)', fkey.solar_p0)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; SOLAR_B0

;if (n_elements (solar_b0) eq 1) then begin
   skey = string (format = '("SOLAR_B0", "= ", f20.2)', fkey.solar_b0)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; SOLAR_L0

;if (n_elements (solar_l0) eq 1) then begin
   skey = string (format = '("SOLAR_L0", "= ", f20.2)', fkey.solar_l0)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; HOUR-ANG

;if (n_elements (hour_ang) eq 1) then begin
   skey = string (format = '("HOUR-ANG", "= ", f20.2)', fkey.hour_ang)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; ELEV

;if (n_elements (elev    ) eq 1) then begin
   skey = string (format = '("ELEV    ", "= ", f20.2)', fkey.elev    )
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; AZIMUTH

;if (n_elements (azimuth ) eq 1) then begin
   skey = string (format = '("AZIMUTH ", "= ", f20.2)', fkey.azimuth )
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; AIRMASS

;if (n_elements (airmass ) eq 1) then begin
   skey = string (format = '("AIRMASS ", "= ", f20.2)', fkey.airmass )
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; TYPE-OBS

;if (n_elements (type_obs) eq 1) then begin
   skey = "TYPE-OBS= '" + strmid (fkey.type_obs, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; CODE-OBS

;if (n_elements (code_obs) eq 1) then begin
   skey = string (format = '("CODE-OBS", "= ", i20)', fkey.code_obs)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; NIMAVG

;if (n_elements (nimavg  ) eq 1) then begin
   skey = string (format = '("NIMAVG  ", "= ", i20)', fkey.nimavg  )
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; EXPDUR

;if (n_elements (expdur) eq 1) then begin
   skey = string (format = '("EXPDUR  ", "= ", f20.2)', fkey.expdur)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; BUNIT

;if (n_elements (bunit   ) eq 1) then begin
   skey = "BUNIT   = '" + strmid (fkey.bunit   , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; BSCALE

;if (n_elements (bscale) eq 1) then begin
   skey = string (format = '("BSCALE  ", "= ", e20.4)', fkey.bscale)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; BZERO

;if (n_elements (bzero) eq 1) then begin
   skey = string (format = '("BZERO   ", "= ", e20.4)', fkey.bzero)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DATAMIN

;if (n_elements (datamin) eq 1) then begin
   skey = string (format = '("DATAMIN ", "= ", f20.2)', fkey.datamin)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DATAMAX

;if (n_elements (datamax) eq 1) then begin
   skey = string (format = '("DATAMAX ", "= ", f20.2)', fkey.datamax)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DISPMIN

;if (n_elements (dispmin) eq 1) then begin
   skey = string (format = '("DISPMIN ", "= ", f20.2)', fkey.dispmin)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DISPMAX

;if (n_elements (dispmax) eq 1) then begin
   skey = string (format = '("DISPMAX ", "= ", f20.2)', fkey.dispmax)
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV1

;if (n_elements (optdev1) eq 1) then begin
   skey = "OPTDEV1 = '" + strmid (fkey.optdev1 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS1

;if (n_elements (optpos1) eq 1) then begin
   skey = "OPTPOS1 = '" + strmid (fkey.optpos1 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV2

;if (n_elements (optdev2) eq 1) then begin
   skey = "OPTDEV2 = '" + strmid (fkey.optdev2 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS2

;if (n_elements (optpos2) eq 1) then begin
   skey = "OPTPOS2 = '" + strmid (fkey.optpos2 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV3

;if (n_elements (optdev3) eq 1) then begin
   skey = "OPTDEV3 = '" + strmid (fkey.optdev3 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS3

;if (n_elements (optpos3) eq 1) then begin
   skey = "OPTPOS3 = '" + strmid (fkey.optpos3 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV4

;if (n_elements (optdev4) eq 1) then begin
   skey = "OPTDEV4 = '" + strmid (fkey.optdev4 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS4

;if (n_elements (optpos4) eq 1) then begin
   skey = "OPTPOS4 = '" + strmid (fkey.optpos4 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV5

;if (n_elements (optdev5) eq 1) then begin
   skey = "OPTDEV5 = '" + strmid (fkey.optdev5 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS5

;if (n_elements (optpos5) eq 1) then begin
   skey = "OPTPOS5 = '" + strmid (fkey.optpos5 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV6

;if (n_elements (optdev6) eq 1) then begin
   skey = "OPTDEV6 = '" + strmid (fkey.optdev6 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS6

;if (n_elements (optpos6) eq 1) then begin
   skey = "OPTPOS6 = '" + strmid (fkey.optpos6 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV7

;if (n_elements (optdev7) eq 1) then begin
   skey = "OPTDEV7 = '" + strmid (fkey.optdev7 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS7

;if (n_elements (optpos7) eq 1) then begin
   skey = "OPTPOS7 = '" + strmid (fkey.optpos7 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTDEV8

;if (n_elements (optdev8) eq 1) then begin
   skey = "OPTDEV8 = '" + strmid (fkey.optdev8 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; OPTPOS8

;if (n_elements (optpos8) eq 1) then begin
   skey = "OPTPOS8 = '" + strmid (fkey.optpos8 , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; VERSION

;if (n_elements (version) eq 1) then begin
   skey = "VERSION = '" + strmid (fkey.version , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DATE

;if (n_elements (date   ) eq 1) then begin
   skey = "DATE    = '" + strmid (fkey.date    , 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; DATE-CAL

;if (n_elements (date_cal eq 1) then begin
   skey = "DATE_CAL= '" + strmid (fkey.date_cal, 0, 18)+"'"
   bkey = bytarr (80) + space
   bkey (0) = byte  (skey)
   head = [[head], [bkey]]
;endif

; HISTORY

;if (n_elements (his) eq 1) then begin
;   skey = "HISTORY " + strmid (his, 0, 72)
;   bkey = bytarr(80) + space
;   bkey (0) = byte  (skey)
;   head = [[head], [bkey]]
;endif

; BLANK

   head = [[head], [blank]]

; COMMENT

com = strarr (8)
com(0)= 'THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF'
com(1)= 'HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),'
com(2)= 'A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),'
com(3)= 'UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).'
com(4)= 'THE DIGITAL PROMINENCE MONITOR (DPM) OBSERVES THE SOLAR DISC & LIMB'
com(5)= 'WITH AN APPROXIMATE 1.6X2.2 SOLAR RADIUS FIELD OF VIEW, USING A'
com(6)= 'KODAK CCD DETECTOR (1534X1032 PIXELS, OF WHICH 1534X1029 ARE USED).'
com(7)= 'IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.'

for i = 0, 7  do begin
   skey = "COMMENT   " + strmid (com (i), 0, 70)
   bkey = bytarr (80) + space
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endfor

; END

skey = 'END'
bkey = bytarr (80) + space
bkey (0) = byte (skey)
head = [[head], [bkey]]

; Add blank lines to fill last FITS header record.

print, 'n_elements (head): ', n_elements (head)

i = (3600 - n_elements (head) / 80) mod 36
if (i ne 0) then begin
   bkey = bytarr (80, i) + space
   head = [[head], [bkey]]
endif

; Write the FITS header & image data to disk file.

writeu, unit, head
if (dpmsiz (dpmsiz (0) + 1) gt 3) $
   then writeu, unit, imgb $
   else writeu, unit, img

; Add null data to fill last FITS data record.

i = (n_elements (img) * (bitpix/8)) mod 2880
i = (2880-i) mod 2880
if (i gt 0) then begin
   imgb = bytarr (i)
   writeu, unit, imgb
endif

free_lun, unit
return
end

