pro write_fits, data, filnam, index = fnum, $
		date_obs = date_obs, time_obs = time_obs,  $
		bscale = bscale, bzero = bzero, $
                origin = origin, telescop = telescop, instrume = instrume, $
		object = object, type_obs = type_obs, $
		history = his, comment = com
;+
; NAME:
;        WRITE_FITS
; PURPOSE:
;        Writes an array into a standard FITS file on the disk.
;        A simple FITS header is created automatically.
; CATEGORY:
;        Input/Output
; CALLING SEQUENCE:
;        wfits, array, filename
; INPUTS:
;        data   = data array.
;        filnam = string containing the file name.
; OPTIONAL (KEYWORD) INPUT PARAMETERS:
;        index = nonnegative integer. If this parameter is present, a period
;              and the index number are appended to the filename (e.g., '.34').
;              This option makes handling of data in the MCCD file naming
;              convention easier.
;        date_obs = string. A FITS keyword parameter "DATE_OBS= 'string'" is
;                   added to the header.
;        time_obs = string. A FITS keyword parameter "TIME_OBS= 'string'" is
;                   added to the header.
;        bscale   = float.  multiplicative scaling factor.
;        bzero    = float.  additive offset.
;	 origin   = string. Institution name.
;	 telescop = string. Telescope name.
;	 instrume = string. Instrument name.
;	 object   = string. Name of object observed. 
;	 type_obs = string. Observation type.
;        history = string. A FITS keyword parameter "HISTORY string" is added
;                   to the header.
;        comment = string. A FITS keyword parameter "COMMENT string" is added
;                   to the header.
; OUTPUTS:
;        No explicit outputs.
; COMMON BLOCKS:
;        None.
; SIDE EFFECTS:
;        A file is created, or overwritten.
; RESTRICTIONS:
;        If array is not of type byte, integer, or long, then a conversion
;        to type integer is done. No scaling is made beforehand, so significant
;        loss of information is possible.
;        Only a very simple FITS header can be created.
; MODIFICATION HISTORY:
;        JPW, Nov, 1989.
;        Andrew Stanger (HAO/NCAR), 6 May 1994:
;           Added keywords origin, telescop, instrume, object, type_obs.
;        Andrew Stanger (NAO/NCAR), 10 May 1994:
;           Added keywords bscale, bzero.
;-

; open FITS file

if (n_elements (fnum) ne 0) $
   then file = filnam + '.' + string (format = '(i0)', fnum) $
   else file = filnam
get_lun, unit
openw, unit, file

; Check data type, and make conversion if necessary.

type = size (data)
case type (type (0) + 1) of
   1 : begin
          bitpix = 8
       ;    print, 'Note : FITS image is  8 bits deep (BITPIX = 8) '
       end
   2 : begin
	  bitpix = 16
          print, 'Note : FITS image is 16 bits deep (BITPIX = 8) '
       end
   3 : begin
          bitpix = 32
          print, 'Note : FITS image is 32 bits deep (BITPIX = 32) '
       end
   else : begin
          bitpix = 16
          datb = fix (data)
          print, 'Note : data converted to 16 bit integer,  no scaling made '
          endelse
endcase

; Create header.

; SIMPLE

skey = string (format = '("SIMPLE  = ", 19x, a1)', 'T')
bkey = bytarr (80) + 32b                            ; 32b = blank
bkey(0) = byte (skey)
head = bkey

; BITPIX

skey = string (format = '("BITPIX  = ", i20)', bitpix)
bkey = bytarr (80) + 32b
bkey(0) = byte (skey)
head = [[head], [bkey]]

; NAXIS

skey = string (format = '("NAXIS   = ", i20)', type(0))
bkey = bytarr (80) + 32b
bkey(0) = byte (skey)
head = [[head], [bkey]]

; NAXISi

for i = 1, type (0) do begin
   skey = string (format = '("NAXIS", i1, "  = ", i20)', i, type(i))
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endfor

; DATE-OBS

if (n_elements (date_obs) eq 1) then begin
   skey = "DATE-OBS= '" + strmid (date_obs, 0, 18) + "'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte(skey)
   head = [[head], [bkey]]
endif

; TIME-OBS

if (n_elements (time_obs) eq 1) then begin
   skey = "TIME-OBS= '" + strmid (time_obs, 0, 18)+"'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; BSCALE

if (n_elements (bscale) eq 1) then begin
   skey = string (format = '("BSCALE  ", "= ", e20.4)', bscale)
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; BZERO

if (n_elements (bzero) eq 1) then begin
   skey = string (format = '("BZERO   ", "= ", e20.4)', bzero)
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; ORIGIN

if (n_elements (origin) eq 1) then begin
   skey = "ORIGIN  = '" + strmid (origin, 0, 18)+"'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; TELESCOP

if (n_elements (telescop) eq 1) then begin
   skey = "TELESCOP= '" + strmid (telescop, 0, 18)+"'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; INSTRUME

if (n_elements (instrume) eq 1) then begin
   skey = "INSTRUME= '" + strmid (instrume, 0, 18)+"'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; OBJECT

if (n_elements (object) eq 1) then begin
   skey = "OBJECT  = '" + strmid (object, 0, 18)+"'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; TYPE-OBS

if (n_elements (type_obs) eq 1) then begin
   skey = "TYPE-OBS= '" + strmid (type_obs, 0, 18)+"'"
   bkey = bytarr (80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; HISTORY

if (n_elements (his) eq 1) then begin
   skey = "HISTORY " + strmid (his, 0, 72)
   bkey = bytarr(80) + 32b
   bkey (0) = byte (skey)
   head = [[head], [bkey]]
endif

; COMMENT

if (n_elements (com) eq 1) then begin
   skey = "COMMENT " + strmid (com, 0, 72)
   bkey = bytarr (80) + 32b
   bkey(0) = byte(skey)
   head = [[head], [bkey]]
endif

; END

skey = 'END'
bkey = bytarr (80) + 32b
bkey(0) = byte(skey)
head = [[head], [bkey]]

; Add blank lines to fill last FITS header record.

i = (3600-n_elements(head)/80) mod 36
if (i ne 0) then begin
   bkey = bytarr (80, i) + 32b
   head = [[head], [bkey]]
endif

; Write the FITS header & data to disk file.

writeu, unit, head
if (type (type (0) + 1) gt 3) $
   then writeu, unit, datb $
   else writeu, unit, data

; Add null data to fill last FITS data record.

i = (n_elements (data) * (bitpix/8)) mod 2880
i = (2880-i) mod 2880
if (i gt 0) then begin
   datb = bytarr (i)
   writeu, unit, datb
endif

free_lun, unit
return
end

