;+
; NAME	fts2gif_mk4.pro
;
; PURPOSE	Convert FITS images to GIF format.
;		Default output resolution is 960x960 pixels.
;		Default color map is "quallab.lut".
;
; SYNTAX	fts2jpg_mk4, fits_list
;		fts2jpg_mk4, fits_list, color_lut, wmin=0, wmax=100, /label
;		fts2jpg_mk4, fits_list, /grid
;		fts2jpg_mk4, fits_list, size=512
;
;		fits_list: file containing a list of FITS images.
;		color_lut: file containing an ASCII color map [index, r, g, b]
;		wmin/wmax: min/max scaling range (maps to [0 --> 249])
;		label:     annotate image
;		grid:      draw a polar grid inside the occulting dis.
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   30 January 2001
;		28 Feb 2002: add wmin & wmax.
;		30 Apr 2003: add label keyword.
;		11 Feb 2004: Use congrid to resize input image to 512x512.
;		23 Nov 2004: add grid keyword; add size keyword.
;-

pro fts2gif_mk4, fits_list, color_lut, wmin=wmin, wmax=wmax,		$
				       label=label, grid=grid, size=size

month_name = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', $
	      'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
CLOSE, 1
fits_name = ''
xdim_prev = 0
ydim_prev = 0
oxdim = 960			; Output x dimension
oydim = 960			; Output y dimension
mag   = 1.0

; white  = 255
; red    = 254
; green  = 253
; blue   = 252
; grey   = 251
; yellow = 250
; black  =   0

red   = bindgen (256)
green = bindgen (256)
blue  = bindgen (256)

IF (N_ELEMENTS (size) GT 0)  THEN	$
BEGIN ;{
   mag = float (size) / float (oxdim)
   print, 'oxdim/oydim: ', oxdim, oydim
   print, 'mag: ', mag
   oxdim = size
   oydim = size
END   ;}

SET_PLOT, 'Z'
DEVICE, set_resolution = [oxdim, oydim], set_colors = 256, z_buffering = 0

if (N_ELEMENTS (color_lut) GT 0) THEN	$
    lct, color_lut			$
ELSE					$
    lct,'/home/cordyn/color/quallab.lut'	; color table.

tvlct, red, green, blue, /get

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   print, 'FITS image: ', fits_name

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

;   dispmin = MIN (fits_img, max=dispmax)
   roll = 0.0

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   crpix1   = fxpar (hdu, 'CRPIX1')
   crpix2   = fxpar (hdu, 'CRPIX2')
   roll     = fxpar (hdu, 'CROTA1')
   origin   = fxpar (hdu, 'ORIGIN')
   telescop = fxpar (hdu, 'TELESCOP')
   date     = fxpar (hdu, 'DATE-OBS')
   time     = fxpar (hdu, 'TIME-OBS')
   object   = fxpar (hdu, 'OBJECT')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   coordnam = fxpar (hdu, 'COORDNAM')
   crradius = fxpar (hdu, 'CRRADIUS')
   bscale   = fxpar (hdu, 'BSCALE')
   bzero    = fxpar (hdu, 'BZERO')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')
   dispmin  = fxpar (hdu, 'DISPMIN')
   dispmax  = fxpar (hdu, 'DISPMAX')

   telescop = STRTRIM (telescop, 2)
   pixrs = crradius
   xcen  = crpix1
   ycen  = crpix2

   year  = STRMID (date, 0, 4)
   month = STRMID (date, 5, 2)
   day   = STRMID (date, 8, 2)
   iyear = FIX (year)
   syear = STRTRIM (STRING (iyear), 2)
   iday  = FIX (day)
   sday  = STRTRIM (STRING (iday), 2)
   IF (iday LT 10)  THEN sday = ' ' + sday
   imonth = FIX (month)
   smonth = month_name [imonth - 1]
   smonth = STRUPCASE (smonth)
   smonth = STRTRIM (smonth, 2)
   sdate  = sday + ' ' +  smonth + ' ' + syear
;   sdate  = STRTRIM (sdate, 2)

   stime = time + ' UT'

   doy = day_of_year (iyear, imonth, iday)

   ;--- Set display minimum intensity.

   IF (N_ELEMENTS (wmin) GT 0) THEN	$
       imin = wmin			$
   ELSE					$
       imin = dispmin

   ;--- Set display maximum intensity.

   IF (N_ELEMENTS (wmax) GT 0) THEN	$
       imax = wmax			$
   ELSE					$
       imax = dispmax

;   dispmin = -1000
;   dispmax =  5000
;   print, FORMAT = '("dispmin/max: ", E10.3, E10.3)', dispmin, dispmax

;   print, 'datamin/datamax: ', datamin, datamax
;   print, 'dispmin/dispmax: ', dispmin, dispmax
;   print, 'imin/imax: ',imin, imax

   ;--- Convert to byte pixels.

;   IF (xdim NE xdim_prev OR ydim NE ydim_prev) THEN	$
;      WINDOW, xsize=xdim, ysize=ydim

   byt_img = BYTSCL  (fits_img, min=imin, max=imax, top=249)

   ;--- Resize image (if needed). 

   IF (xdim NE oxdim) THEN 				$
   BEGIN						$
      tv_img  = CONGRID (byt_img, oxdim, oydim)		
      pixrs = crradius * mag
      xcen  = crpix1   * mag
      ycen  = crpix2   * mag
   END							$
   ELSE							$
      tv_img = byt_img

   ;--- Display image.

   TV, tv_img

;   print, 'xdim/ydim: ', xdim, ydim

   ;--- Label image.

   IF (KEYWORD_SET (label)) THEN	$
   BEGIN ;{
      IF (mag LT 0.9) THEN		$
         cs = 1.0			$
      ELSE				$
         cs = 1.2

;      IF (!version.os EQ 'IRIX')  THEN cs = 0.7
;      IF (!version.os EQ 'sunos') THEN cs = 0.7
;      IF (!version.os EQ 'linux') THEN cs = 0.7

      sorigin   = STRTRIM (origin, 2)
      stelescop = STRTRIM (telescop, 2)
      stype_obs = STRTRIM (type_obs, 2)
      sbunit    = STRTRIM (bunit, 2)
      sobject   = STRTRIM (object, 2)
      scoordnam = STRTRIM (coordnam, 2)
      clen      = STRLEN  (scoordnam)

      stelescop = STRUPCASE (stelescop)
      sobject   = STRUPCASE (sobject)

      stitle = sorigin + ' ' + stelescop + ' ' + stype_obs 
      stitle = 'MLSO ' + stelescop + ' ' + sbunit
      stitle = 'MLSO ' + stelescop + ' ' + stype_obs
      stitle = 'MLSO ' + stelescop + ' ' + sobject

      sdoy  = STRTRIM (STRING (doy), 2)
      IF (doy LT 100) THEN sdoy = ' ' + sdoy
      IF (doy LT 10)  THEN sdoy = ' ' + sdoy
      labeldoy = 'DOY: ' + sdoy

      swmin = STRTRIM (STRING (imin), 2)
      swmax = STRTRIM (STRING (imax), 2)
      sscale = 'Scaling: ' + swmin + ' to ' + swmax
      dmin = imin * bscale + bzero
      dmax = imax * bscale + bzero
;      sdmin = STRTRIM (STRING (dmin, FORMAT='(G9.2)'), 2)
;      sdmax = STRTRIM (STRING (dmax, FORMAT='(G9.2)'), 2)
      sdmin = STRING (dmin, FORMAT='(F9.2)')
      sdmax = STRING (dmax, FORMAT='(F9.2)')
;      sdmin = STRTRIM (STRING (dmin, FORMAT='(F9.2)'), 2)
;      sdmax = STRTRIM (STRING (dmax, FORMAT='(F9.2)'), 2)
      sdmin = 'MIN: ' + sdmin
      sdmax = 'MAX: ' + sdmax

      XYOUTS, 3,           oydim - 16, stitle, 	 /device, charsize=cs, color=255
      XYOUTS, 3,           oydim - 32, sdate,	 /device, charsize=cs, color=255
      XYOUTS, 3,           oydim - 48, stime,	 /device, charsize=cs, color=255
      XYOUTS, 3,           oydim - 64, labeldoy, /device, charsize=cs, color=255

;      XYOUTS, 3,           5,         sscale,	 /device, charsize=cs, color=255
      XYOUTS, 3,           36,        stype_obs, /device, charsize=cs, color=255
      XYOUTS, 3,           20,         sdmin,    /device, charsize=cs, color=255
      XYOUTS, 3,            4,         sdmax,    /device, charsize=cs, color=255

;     XYOUTS, oxdim/2 - 7, oydim   -20, 'N',     /device, charsize=cs, color=255
;     XYOUTS,  7,          oydim/2 - 7, 'E',     /device, charsize=cs, color=255
;     XYOUTS, oxdim/2 - 7, 10,          'S',     /device, charsize=cs, color=255
;     XYOUTS, oxdim   -20, oydim/2 - 7, 'W',     /device, charsize=cs, color=255
;     XYOUTS, oxdim   -19, 24,          '^',   /device, charsize=cs*2, color=255
;     XYOUTS, oxdim   -18, 18,          '|',     /device, charsize=cs, color=255
;     XYOUTS, oxdim   -20,  4,          'N',     /device, charsize=cs, color=255
      lm = FIX (14 * mag)
      XYOUTS, oxdim - clen*lm, 4, scoordnam,     /device, charsize=cs, color=255
   END  ;}

   telescop = STRLOWCASE (telescop)
   IF (KEYWORD_SET (grid) AND telescop EQ 'mk4') THEN	$
   BEGIN ;{
      print, 'oxdim/oydim: ', oxdim, oydim
      print, 'xcen/ycen:   ', xcen, ycen 
      print, 'pixrs:       ', pixrs
      xb = 0
      yb = 0
      suncir_mk4, oxdim, oydim, xcen, ycen, xb, yb, pixrs, roll
   END   ;}

   gif_img = TVRD ()

   ;--- Write jpeg image to disk.

   sloc     = RSTRPOS (fits_name, '.fts')
   gif_name = STRMID  (fits_name, 0, sloc) + '.gif'
   print, 'GIF image : ', gif_name
   WRITE_GIF, gif_name, gif_img, red, green, blue 	; Write new image.
   xdim_prev = xdim
   ydim_prev = ydim
END	;}

SET_PLOT, 'x'
CLOSE, 1
END
