;+
; NAME
;	CHIP_MATCH
;
; PURPOSE
;	Resize and reposition a FITS image to match the geometric
;	format of a MLSO CHIP (He-1083nm) image.
;
; CALLING SEQUENCE
;	chip_match, fits_filename
;
; INPUT:
;	fits_filename:	Name of FITS input file (string enclosed by quotes).
;
; OPTIONAL INPUT PARAMETERS:
;	chipcen:	CHIP sun center vector; default is [255.5, 255.5].
;	chiprad:	CHIP solar radius size (pixels); default is 201.
;	chipsiz:	CHIP image array size; default is [512, 512].
;	gamma:		gamma correction value (do intensity transformation).
; OUTPUT
;	FITS file:	Same base as input name, with '.chip.fts' suffix.
;
; MODIFICATION HISTORY
;	17 Aug 1995: Created by Andrew L. Stanger, HAO/NCAR
;	18 Aug 1995: Add optional parameters: chipcen, chiprad, chipsiz, gamma.
;-

pro chip_match, fits_inp, chipcen=chipcen, chiprad=chiprad, chipsiz=chipsiz, $
                          gamma=gamma

;--- Define MLSO CHIP geometry for output.

chipxdim = 512
chipydim = 512
chipbitpix = 8
chipradius = 201				; CHIP solar radius (pixels).
chipxcen = ((chipxdim - 1) / 2) + 0.5
chipycen = ((chipydim - 1) / 2) + 0.5

if (keyword_set (centroid)) then begin
   chipxcen = chipcen (0)
   chipycen = chipcen (1)
endif

if (keyword_set (chiprad)) then chipradius = chiprad

if (keyword_set (chipsiz)) then begin
   chipxdim = chipsiz (0)
   chipydim = chipsiz (1)
endif

;--- Create name for output FITS file.
;
;    Search for extension 'fts' or 'fits' in input name.
;    Extract base name prior to extenstion,
;    then add '.chip.fts' to form output name.

fits_out = ''
extpos = strpos (fits_inp, '.fts')
if (extpos ge 0) then begin	$
   fits_out = strmid (fits_inp, 0, extpos)
end else begin
   extpos = strpos (fits_inp, '.fits')
   if (extpos ge 0) then	$
      fits_out = strmid (fits_inp, 0, extpos)
endelse

fits_out = fits_out + '.chip.fts'

;--- Read FITS image from disk.

ftsbuf = readfits (fits_inp, hdu)

;--- Get parameters from FITS header.

xdim   = fxpar (hdu, 'NAXIS1')
ydim   = fxpar (hdu, 'NAXIS2')
bitpix = fxpar (hdu, 'BITPIX')
crpix1 = fxpar (hdu, 'CRPIX1')			; sun center E/W axis (pixels).
crpix2 = fxpar (hdu, 'CRPIX2')			; sun center N/W axis (pixels).
crota  = fxpar (hdu, 'CROTA' )			; degrees CCW.
radius = fxpar (hdu, 'RADIUS')			; radius of sun (pixels).

center = [crpix1, crpix2]
diam   = radius * 2.0
outsiz = [xdim, ydim]

refvec = [chipxcen, chipycen, chipradius*2.0, chipradius*2.0,	$
	  chipxdim, chipydim]
ftsvec = [crpix1, crpix2, radius*2.0, radius*2.0, xdim, ydim]

;--- Scale FITS image to match geometrical format of MLSO CHIP image.
;--- image centered in 512x512 pixel array.
;--- solar disk radius is 201 pixels.

imgbuf = img_scale (refvec, ftsbuf, [0,0], gbovec=ftsvec)

imgbuf = bytscl (imgbuf)
bitpix = 8

;--- If gamma parameter is supplied, do gamma correction of image.

if (keyword_set (gamma)) then imgbuf = gamtran (imgbuf, gamma)

;--- Modify and/or add parameters in output FITS header.

fxaddpar, hdu, 'NAXIS1', xdim
fxaddpar, hdu, 'NAXIS2', ydim
fxaddpar, hdu, 'BITPIX', bitpix
fxaddpar, hdu, 'CRPIX1', 255.5,  'SUN CENTER E/W AXIS (PIXELS).'
fxaddpar, hdu, 'CRPIX2', 255.5,  'SUN CENTER N/S AXIS (PIXELS).'
fxaddpar, hdu, 'RADIUS', radius, 'RADIUS OF SUN (PIXELS).'
fxaddpar, hdu, 'CROTA',  crota,  'CCW ROTATION OF SOLAR NORTH.'

if (keyword_set (gamma)) then begin
   gammacom = 'GAMMA FUNCTION WITH VALUE ' $
	    + string (gamma, FORMAT = "(F7.3)") + ' APPLIED.'
   fxaddpar, hdu, 'COMMENT', gammacom
endif

;--- Write output image to disk in FITS format.

writefits, fits_out, imgbuf, hdu

end
