pro chipsxt, chip_file, sxt_file, $
;           chip_out=chip_out, sxt_out=sxt_out, $
	   chip_brt=chip_brt, sxt_brt=sxt_brt, $
	   nolabel=nolabel, ld=ld, outres=outres

;+
; NAME
;	chipsxt
;
; PURPOSE
;	Creates a true-color Sun rasterfile by combining
;	a CHIP disk image and a Yohkoh SXT (soft X-ray) image.
;
; METHOD
;	Calls rd_sda to input SXT image from disk file (SXT format).
;	Calls rt_fits to register, magnify/contract, and remove limb
;	darkening from CHIP image stored in a FITS file.
;	Calls write_srf to write 24-bit color Sun rasterfile to disk.
;
; CALLING SEQUENCE
;	chipsxt, chip_file, sxt_file
;	chipsxt, chip_file, sxt_file, /nolabel
;	chipsxt, chip_file, sxt_file, /nolabel, /ld
;	chipsxt, chip_file, sxt_file, /nolabel, /ld, outres=0
;
; PARAMETERS
;	chip_file	CHIP filename (FITS format)
;	sxt_file	Yohkoh SXT image (SXT format)
;	chip_out	If set, write registered CHIP image to disk (FITS)
;	sxt_out		If set, write registered SXT    image to disk (FITS)
;	chip_brt	If present, use as Ha  brightness scaling factor (1.00).
;	sxt_brt		If present, use as SXT brightness scaling factor (1.33).
;	nolabel		Do NOT write image ID into each image array
;	ld		remove limbdarkening
;	outres		Output resolution:
;			0=full[1024x1024], 1=half[512x512], 2=quarter[256x256]
;			default is 1=half [512x512]
;
; KEYWORDS
;	none
;
; OUTPUT
;	Sun rasterfile (RGB): composite image CHIP + Yohkoh SXT.
;	FITS file: CHIP image (registered).
;	FITS file: Yohkoh SXT image (registered).
;
; CALLS
;	rd_sda, rt_fits, write_ras, writefits, sxt2fits, imgscl3, rm_darklimb
;
; HISTORY
;	written 14 Februay 1995 by Andrew L. Stanger   HAO/NCAR
;       15 February 1995: ALS, Add bbso check for CHIP FITS image.
;			       Add label option
;	21 Mar 1995 ALS: Added resout keyword and mod_res function to
;                        allow user to specify output resolution.
;	12 Apr 1995 ALS: Use sxt2fts instead of sxt2fits (string parameters
;			 weren't enclosed by quotes, unreadable by 'xv').
;                        Also, use write_ras instead of write_srf (ras_length
;                        incorrectly written in header, unreadable by 'bit').
;	13 Apr 1995 ALS: Make label the default; replace keyword with 'nolabel'.
;                        Make chip_out and sxt_out options the default.
;			 Use imgscl3 instead of rt_fits; use reference image
;			 target: radius=360 pixels, centered in 512x480 region
;			 (offset 32 pixels from bottom for labels).
;	10 Nov 1995 ALS: Use sxt2fits.pro.  New version is now acceptable.
;-

!P.font = 0					; Use X-window hardware font
device, font='-sgi-fixed-bold-r-normal--15-140-75-75-c-90-iso8859-9'

; device, font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'

; window, xsize=512, ysize=512, colors=256		; Define display window.

window, xsize=512, ysize=512, colors=256, /pixmap	; Define display window.

; loadct, 0					; Load greyscale colormap.

defres = 1					; Default output res = half.

; --- Verify that both CHIP and SXT filenames are non-zero.

if (n_elements (chip_file) eq 0 or n_elements (sxt_file) eq 0) then $
   begin
      print, 'usage: chipsxt, chip_FITS_file, sxt_file'
      return
   end

print, 'chip_file: ', chip_file, '   sxt_file: ', sxt_file

chip_out = 1
sxt_out = 1

; --- Test for origin of CHIP file.

   fits_hdr = headfits (chip_file)		; Read FITS header from file.
   chip_origin = fxpar (fits_hdr, 'ORIGIN')	; Get origin parameter.
   chip_origin = strtrim (chip_origin)

   bbpos = strpos (chip_origin, 'BBSO')
   if (bbpos ge 0) then bbso = 1  else  bbso = 0

   kboupos = strpos (chip_origin, 'KBOU')
   khmnpos = strpos (chip_origin, 'KHMN')
   learpos = strpos (chip_origin, 'LEAR')
   if (kboupos eq 0 or khmnpos eq 0 or learpos eq 0) then $
      sel = 1 else sel = 0

   print, 'origin: ', chip_origin, '   bbso: ', bbso, '   sel: ', sel

; --- Read SXT image from disk file.

rd_sda, sxt_file, -1, sxt_index, sxt_img

; --- Modify SXT resolution (& update index) if image does not match
;     the desired output resolution..

if (keyword_set (outres)) then res_out = outres else res_out = defres

res_in = gt_res (sxt_index)			; Resolution of input SXT image.
print, 'res_in/res_out: ', res_in, res_out
if (res_in ne res_out) then begin		; Modify SXT resolution.
   sxt_img = mod_res (sxt_index, sxt_img, index_out, /update_index, $
                      outres=res_out, /override, /norm_fact)
endif

; --- Define reference target (6-element vector: x/y centroid, diam, pixel size)

ref_xdim = 512
ref_ydim = 512
ref_xcen = ref_xdim / 2.0 - 0.5
ref_ycen = ref_ydim / 2.0 - 0.5 + 32
ref_xrad = 360.0
ref_yrad = 360.0
ref_target = [ref_xcen, ref_ycen, ref_xrad, ref_yrad, ref_xdim, ref_ydim]

; --- Rescale image size [1 Rsun = 200 pixels]
; --- and reposition to center of image array.

sxt_img = imgscl3 (ref_target, sxt_img, [0,1], sxt_index)

; --- Read CHIP FITS file from disk.
; --- Match it (same size and location) to SXT image.
; --- Remove limb darkening (if requested).

if (keyword_set (ld)) then ldrm = 0  else ldrm = 1	; ldrm: 0=remove, 1=keep

;if (keyword_set (ld)) then $
;   chip_img = rt_fits (chip_file, sxt_img, sxt_index (0), $	; remove ld
;                     header=chip_hdr, /byte, bbso=bbso, sel=sel) $
;else	$
;   chip_img = rt_fits (chip_file, sxt_img, sxt_index (0), $	; no ld removal
;                     header=chip_hdr, /byte, limbdark=ldrm, bbso=bbso, sel=sel)

chip_img = readfits (chip_file, chip_hdr)		; Read CHIP image.

if (bbso EQ 1) then $
begin
   chip_size = size (chip_img) 
   chip_img (*, chip_size (2) - 1) = 0			; Blank out top line.
end

if (sel EQ 1) then chip_img = reverse (chip_img, 2)	; Invert top to bottom

chip_img = imgscl3 (ref_target, chip_img, [0,0], /cir)	; Register CHIP image.

;if (keyword_set (ld)) then $				; Remove limb-darkening.
;   chip_img = rm_darklimb (chip_img, target=ref_target)

; --- Expand intensity scale to fill brightness range (0-249).

brt_max = 249					; Maximum brightness
chip_fac  = 1.00				; Default CHIP brightness factor
sxt_fac = 0.75					; Default SXT brightness factor
if (keyword_set (chip_brt)) then chip_fac = 1.0 / chip_brt; CHIP brightness factor
if (keyword_set (sxt_brt)) then sxt_fac = 1.0 / sxt_brt	; SXT brightness factor

chip_min = MIN (chip_img, max=chip_max)		; Get intensity min/max values.
print, 'chip_min/max: ', chip_min, ' ', chip_max
print, 'chip_fac: ', chip_fac
chip_maxbrt = fix (chip_max * chip_fac)
;if (chip_maxbrt > brt_max)  then chip_maxbrt = brt_max
print, 'chip_maxbrt: ', chip_maxbrt

   chip_img_expand = bytscl (chip_img, min=chip_min,	$
			max=chip_maxbrt, top=brt_max) 	   ; Rescale brightness.

;if (chip_fac NE 1.00)  then				$
;   chip_img_expand = bytscl (chip_img, min=chip_min,	$
;			max=chip_maxbrt, top=brt_max) 	$  ; Rescale brightness.
;else							$
;begin							$
;   chip_img_expand = chip_img
;   print, '*** No H-a brightness rescaling. ***'
;end

chip_nx   = fxpar (fits_hdr, 'NAXIS1')			; CHIP X dimension.
chip_ny   = fxpar (fits_hdr, 'NAXIS2')			; CHIP Y dimension.
print, 'chip_nx/chip_ny: ', chip_nx, chip_ny

sxt_min = MIN (sxt_img, max=sxt_max)		; Get intensity min/max values.
print, 'sxt_min/max: ', sxt_min, ' ', sxt_max
print, 'sxt_fac: ', sxt_fac
sxt_maxbrt = fix (sxt_max * sxt_fac)
;if (sxt_maxbrt > brt_max)  then sxt_maxbrt = brt_max
print, 'sxt_maxbrt: ', sxt_maxbrt

   sxt_img_expand = bytscl (sxt_img, min=sxt_min,	$
			    max=sxt_maxbrt, top=brt_max)   ; Rescale brightness.

;if (sxt_fac NE 1.00)  then				$
;   sxt_img_expand = bytscl (sxt_img, min=sxt_min,	$
;			    max=sxt_maxbrt, top=brt_max)$  ; Rescale brightness.
;else							$
;begin							$
;   sxt_img_expand = sxt_img
;   print, '*** No SXT brightness rescaling. ***'
;end

sxt_size = size (sxt_img)				; Get SXT dimensions.
sxt_nx = sxt_size (1)					; SXT X dimension.
sxt_ny = sxt_size (2)					; SXT Y dimension.
print, 'sxt_nx/sxt_ny: ', sxt_nx, sxt_ny

; --- Image labels requested ?

if (keyword_set (nolabel)) then $
   print, 'No label written into output images.' $
   else $
   begin
      chip_date = fxpar (fits_hdr, 'DATE-OBS')
      chip_time = fxpar (fits_hdr, 'TIME-OBS')
      htime = fmt_tim (anytim2ex (chip_date + ' ' + chip_time))	; SXT format
      chip_nx = sxt_nx 
      chip_ny = sxt_ny
      print, 'label...chip_nx/chip_ny: ', chip_nx, chip_ny
      chip_name = chip_origin + ' CHIP'
      nchip_name = strlen (chip_name)

      sxt_time = string (gt_time (sxt_index, /str))
      sxt_date = string (gt_day  (sxt_index, /str))
      sxt_name = 'Yohkoh SXT'
      nsxt_name = strlen (sxt_name)

      ; --- Pad ID field to "lenmax" characters.

      lenmax = 24

      if ( nchip_name lt lenmax) then $
	 for i = 0, lenmax -  nchip_name - 1 do  chip_name =  chip_name + ' '

      if (nsxt_name lt lenmax) then $
	 for i = 0, lenmax - nsxt_name - 1 do sxt_name = sxt_name + ' '

      chip_label  =  chip_name + '          ' + htime
      sxt_label = sxt_name + '          ' + sxt_date + '  ' + sxt_time

      ; --- Display CHIP image, write annotation, then read back to array.

      chip_img_expand (*, 0:31) = 0		; Set bottom 32 rows to black.
      tv, chip_img_expand
      xyouts, 10, 18, chip_label, /device
      chip_img_expand = tvrd (0, 0, chip_nx, chip_ny)

      ; --- Display SXT image, write annotation, then read back to array.

      sxt_img_expand (*, 0:31) = 0		; Set bottom 32 rows to black.
      tv, sxt_img_expand
      xyouts, 10, 2, sxt_label, /device
      sxt_img_expand = tvrd (0, 0, sxt_nx, sxt_ny)
   end

; --- Write CHIP image to disk file (if requested).

; if (keyword_set (chip_out)) then $
if (chip_out EQ 1) then $
   begin
      chip_out_file = chip_file
      outlen = strlen (chip_out_file)
      begpos = strpos (chip_out_file, '.fts')
      if (keyword_set (ld)) then ldext = '.ldr'  else ldext = '.ldp'

      if (begpos eq outlen - 4) then $
	 begin
	    strput, chip_out_file, ldext, begpos
	    chip_out_file = chip_out_file + '.fts'
	 end	$
      else	$
	 chip_out_file = chip_out_file + ldext + '.fts'

      print, 'CHIP output file: ', chip_out_file

      if (sel eq 1) then begin
	 fxaddpar, chip_hdr, 'ORIGIN', 'SEL: ' + chip_origin, 'NOAA/SEL'
	 fxaddpar, chip_hdr, 'COMMENT', ' IMAGE REVERSED TOP-TO-BOTTOM SO THAT ORIGIN IS LOWER LEFT CORNER.'
      endif
      fxaddpar, chip_hdr, 'CRPIX1',   ref_xcen, 'X SUN CENTER (PIXELS)'
      fxaddpar, chip_hdr, 'CRPIX2',   ref_ycen, 'Y SUN CENTER (PIXELS)'
      fxaddpar, chip_hdr, 'CRRADIUS', ref_xrad, 'PIXELS PER SOLAR RADIUS'
      writefits, chip_out_file, chip_img_expand, chip_hdr
   end

; --- Write SXT image to disk file (if requested).

; if (keyword_set (sxt_out)) then $
if (sxt_out EQ 1) then $
   begin
      stime   = anytim2ints (sxt_index)			; date/time conversion
      sdaytim = ex2fid ( anytim2ex (stime))		; make 'YYMMDD.HHMM'
      sxt_out_file = 'sfi.' + sdaytim + '.fts'
      print, 'SXT output file: ', sxt_out_file
      rsun = ref_xrad
      suncen = [ref_xcen, ref_ycen]
      comment = 'IMAGE HAS BEEN RESIZED AND POSITIONED (SEE CRRADIUS AND CRPIX1/2).'
;      sxt2fts, sxt_index, sxt_img_expand, outfil=sxt_out_file, $
;	       rsun=rsun, suncen=suncen, comment=comment
      sxt2fits, sxt_index, sxt_img_expand, outfil=sxt_out_file
   end

; --- Flip images top-to-bottom since Sun Rasterfile's origin is upper left,
;     whereas SXT and IDL use lower left corner as the origin.

chip_img_exprev  = reverse ( chip_img_expand, 2)

sxt_img_exprev = reverse (sxt_img_expand, 2)

; --- Create 24-bit color image.
;     CHIP   --> red channel.
;     SXT image --> green & blue channels.
;
; --- Note: For standard 24-bit Sun Rasterfile,
;     color channel order is blue, green, red.

bgr_img = bytarr (3, sxt_nx, sxt_ny)

bgr_img (0, *, *) = sxt_img_exprev		; blue channel.

bgr_img (1, *, *) = sxt_img_exprev		; green channel.

bgr_img (2, *, *) = chip_img_exprev		; red channel.

; --- Create RGB colormap (equal brightness for all 3 colors).

map_red   = bindgen (256)

map_green = bindgen (256)

map_blue  = bindgen (256)

; --- Write image to disk as a 24-bit (BGR) Sun rasterfile.
 
srf_file = 'hs.' + sdaytim + '.ras'		; Sun rasterfile name.

; write_srf, srf_file, bgr_img, map_red, map_green, map_blue

write_ras, srf_file, bgr_img

!P.font = -1

end
