;+
; IDL PROCEDURE
;
; NAME:
;	ftsgamld
;
; PURPOSE:
;	Apply a gamma function to a FITS image.
;	Remove limb darkening.
;
; CALLING SEQUENCE:
;	ftsgamld, 'image.fts', gammaval
;	ftsgamld, 'image.fts', gammaval, /order
;	ftsgamld, 'image.fts', gammaval, /order, /label
;
; INPUTS:
;	image.fts:	Input FITS image filename
;	gammaval:	gamma function value
;
; OUTPUTS:
;	image.fts.gam:	Output FITS image filename
;
; PROCEDURES USED:
;	READFITS
;	WRITEFITS
;	GAMTRAN
;	RM_LIMBDARK
;
; MODIFICATION HISTORY:
;	17 Aug 1995: procedure created by Andrew L. Stanger, HAO/NCAR
;	13 Mar 1996: add limb darkening removal, ALS.
;-

pro ftsgamld, inp_file, gammaval, label=label, order=order

out_file = 'img.gam.fts'

      out_file = inp_file + '.gam'

      ;--- Read image into array & scale intensity levels to range: 0-255.

      imgbuf = readfits (inp_file, hdu)	; Read first FITS image in listfile.

      nbitpx = fxpar (hdu, 'BITPIX')
      naxis  = fxpar (hdu, 'NAXIS')
      nxpix  = fxpar (hdu, 'NAXIS1')
      nypix  = fxpar (hdu, 'NAXIS2')

      ;--- Add comment to output FITS header.

      hdu_out = hdu
      comment = 'GAMMA FUNCTION APPLIED WITH VALUE: ' $
	      + string (gammaval, FORMAT = "(F7.3)")
      fxaddpar, hdu_out, 'COMMENT', comment
      if (nbitpx eq   8)  then imgbuf = byte   (imgbuf) else $
                               imgbuf = bytscl (imgbuf)

      ;--- If order keyword is set, reverse image top to bottom.

      if (keyword_set (order))  then imgbuf = reverse (imgbuf, 2)

      loadct, 0
      window, 0, xsize = nxpix, ysize = nypix
      wset, 0
      tv, imgbuf

      imgbuf = rm_darklimb (imgbuf)			; Remove limb darkening.
      imgbuf = bytscl (imgbuf)
      tv, imgbuf

      imgout = gamtran (imgbuf, gammaval)		; Transform image.
      tv, imgbuf

      ;--- Label image if label keyword is set.

      if (keyword_set (label))  then $
      begin
         window, 20, xsize = nxpix, ysize = nypix, color=256, /pixmap
         wset, 20
         idate  = fxpar (hdu, 'DATE-OBS', count=count)
         sizdate = size (idate)
         if (count eq 0 or sizdate (1) lt 8)  then idate = inp_file

         itime  = fxpar (hdu, 'TIME-OBS', count=count)
         siztime = size (itime)
         if (count eq 0 or siztime (1) lt 8)  then itime = ''

         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgout
         imglabel = sframe + '     ' + idate + '   ' + itime
	 gamlabel = 'gamma: ' + string (gammaval, FORMAT = "(F7.3)")
         xyouts, xpos, ypos, imglabel, /device
	 xyouts, xpos, ypos - 20, gamlabel, /device
         imgout = tvrd ()
	 wdelete, 20
      end

      imgout = bytscl (imgout)
      fxaddpar, hdu, 'BITPIX', 8

      writefits, out_file, imgout, hdu_out

end 
