pro ha_match, ha_file, $
;          ha_out=ha_out,$
	   ha_brt=ha_brt, $
	   nolabel=nolabel, ld=ld

;+
; NAME
;	ha_match
;
; PURPOSE
;	Finds sun center of an H-alpha disk image.
;
; METHOD
;	Calls rt_fits to register, magnify/contract, and remove limb
;	darkening from H-alpha image stored in a FITS file.
;
; CALLING SEQUENCE
;	ha_match, ha_file
;	ha_match, ha_file, /nolabel
;	ha_match, ha_file, /nolabel, /ld
;
; PARAMETERS
;	ha_file		H-alpha filename (FITS format)
;	ha_out		If set, write registered Halpha image to disk (FITS)
;	ha_brt		If present, use as Ha  brightness scaling factor (1.00).
;	nolabel		Do NOT write image ID into each image array
;	ld		remove limbdarkening
;
; KEYWORDS
;	none
;
; OUTPUT
;	FITS file: H-alpha image (registered).
;
; CALLS
;	rt_fits, write_ras, writefits, imgscl3, rm_darklimb
;
; HISTORY
;	written 14 Februay 1995 by Andrew L. Stanger   HAO/NCAR
;       15 February 1995: ALS, Add bbso check for H-alpha FITS image.
;			       Add label option
;	21 Mar 1995 ALS: Added resout keyword and mod_res function to
;                        allow user to specify output resolution.
;	12 Apr 1995 ALS: Use sxt2fts instead of sxt2fits (string parameters
;			 weren't enclosed by quotes, unreadable by 'xv').
;                        Also, use write_ras instead of write_srf (ras_length
;                        incorrectly written in header, unreadable by 'bit').
;	13 Apr 1995 ALS: Make label the default; replace keyword with 'nolabel'.
;                        Make ha_out and sxt_out options the default.
;			 Use imgscl3 instead of rt_fits; use reference image
;			 target: radius=360 pixels, centered in 512x480 region
;			 (offset 32 pixels from bottom for labels).
;	17 Jun 1997 ALS: Modified from hasxt3.pro.
;-

!P.font = 0					; Use X-window hardware font
device, $
   font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'

; window, xsize=512, ysize=512, /pixmap			; Define pixmap window.
; window, xsize=512, ysize=512, colors=256		; Define display window.

window, xsize=512, ysize=512, colors=256, /pixmap	; Define pixmap window.

nx = 512
ny = 512

; loadct, 0					; Load greyscale colormap.

defres = 1					; Default output res = half.

; --- Verify that both H-alpha and SXT filenames are non-zero.

if (n_elements (ha_file) eq 0) then $
   begin
      print, 'usage: ha_match, ha_FITS_file'
      return
   end

print, 'ha_file: ', ha_file

ha_out = 1

; --- Test for origin of H-alpha file.

   fits_hdr = headfits (ha_file)		; Read FITS header from file.
   ha_origin = fxpar (fits_hdr, 'ORIGIN')	; Get origin parameter.
   ha_origin = strtrim (ha_origin)

   bbpos = strpos (ha_origin, 'BBSO')
   if (bbpos ge 0) then bbso = 1  else  bbso = 0

   kboupos = strpos (ha_origin, 'KBOU')
   khmnpos = strpos (ha_origin, 'KHMN')
   learpos = strpos (ha_origin, 'LEAR')
   if (kboupos eq 0 or khmnpos eq 0 or learpos eq 0) then $
      sel = 1 else sel = 0

   print, 'origin: ', ha_origin, '   bbso: ', bbso, '   sel: ', sel


; --- Define reference target (6-element vector: x/y centroid, diam, pixel size)

ref_xdim = 512
ref_ydim = 512
ref_xcen = ref_xdim / 2.0 - 0.5
ref_ycen = ref_ydim / 2.0 - 0.5 + 32
ref_xrad = 360.0
ref_yrad = 360.0
ref_target = [ref_xcen, ref_ycen, ref_xrad*2, ref_yrad*2, ref_xdim, ref_ydim]

; --- Read H-alpha FITS file from disk.
; --- Match it (same size and location) to SXT image.
; --- Remove limb darkening (if requested).

if (keyword_set (ld)) then ldrm = 0  else ldrm = 1	; ldrm: 0=remove, 1=keep

ha_img = readfits (ha_file, ha_hdr)			; Read H-alpha image.

if (bbso EQ 1) then $
begin
   ha_size = size (ha_img) 
   ha_img (*, ha_size (2) - 1) = 0			; Blank out top line.
end

if (sel EQ 1) then ha_img = reverse (ha_img, 2)		; Invert top to bottom

ha_img = imgscl3 (ref_target, ha_img, [0,0], /cir)	; Register Halpha image.
if (keyword_set (ld)) then $				; Remove limb-darkening.
   ha_img = rm_darklimb (ha_img, target=ref_target)

; --- Expand intensity scale to fill brightness range (0-249).

brt_max = 249					; Maximum brightness
ha_fac  = 1.00					; Default Ha  brightness factor
if (keyword_set (ha_brt))  then ha_fac  = 1.0 / ha_brt	; Ha  brightness factor

ha_min = MIN (ha_img, max=ha_max)		; Get intensity min/max values.
print, 'ha_min/max: ', ha_min, ' ', ha_max
print, 'ha_fac: ', ha_fac
ha_maxbrt = fix (ha_max * ha_fac)
;if (ha_maxbrt > brt_max)  then ha_maxbrt = brt_max
print, 'ha_maxbrt: ', ha_maxbrt

if (keyword_set (ha_brt))  then				$
begin							$
   ha_img_expand = ha_img
   print, 'No H-a brightness rescaling.'
end							$
else							$
   ha_img_expand = bytscl (ha_img, min=ha_min,		$
			   max=ha_maxbrt, top=brt_max) 	; Rescale brightness.

ha_nx   = fxpar (fits_hdr, 'NAXIS1')			; H-alpha X dimension.
ha_ny   = fxpar (fits_hdr, 'NAXIS2')			; H-alpha Y dimension.
print, 'ha_nx/ha_ny: ', ha_nx, ha_ny

; --- Image labels requested ?

if (keyword_set (nolabel)) then $
   print, 'No label written into output images.' $
   else $
   begin
      ha_date = fxpar (fits_hdr, 'DATE-OBS')
      ha_time = fxpar (fits_hdr, 'TIME-OBS')
      htime = fmt_tim (anytim2ex (ha_date + ' ' + ha_time))	; SXT format
      sdaytim = htime
      ha_nx = nx 
      ha_ny = ny
      print, 'label...ha_nx/ha_ny: ', ha_nx, ha_ny
      ha_name = ha_origin + ' Halpha'
      nha_name = strlen (ha_name)

      ; --- Pad ID field to "lenmax" characters.

      lenmax = 24

      if ( nha_name lt lenmax) then $
	 for i = 0, lenmax -  nha_name - 1 do  ha_name =  ha_name + ' '

      ha_label  =  ha_name + '     ' + htime

      ; --- Display H-alpha image, write annotation, then read back to array.

      tv, ha_img_expand
      xyouts, 10, 20, ha_label, /device
      ha_img_expand = tvrd (0, 0, ha_nx, ha_ny)

   end

; --- Write H-alpha image to disk file (if requested).

; if (keyword_set (ha_out)) then $
if (ha_out EQ 1) then $
   begin
      ha_out_file = ha_file
      outlen = strlen (ha_out_file)
      begpos = strpos (ha_out_file, '.fts')
      if (keyword_set (ld)) then ldext = '.ldr'  else ldext = '.ldp'

      if (begpos eq outlen - 4) then $
	 begin
	    strput, ha_out_file, ldext, begpos
	    ha_out_file = ha_out_file + '.fts'
	 end	$
      else	$
	 ha_out_file = ha_out_file + ldext + '.fts'

      print, 'H-alpha output file: ', ha_out_file

      if (sel eq 1) then begin
	 fxaddpar, ha_hdr, 'ORIGIN', 'SEL: ' + ha_origin, 'NOAA/SEL'
	 fxaddpar, ha_hdr, 'COMMENT', ' IMAGE REVERSED TOP-TO-BOTTOM SO THAT ORIGIN IS LOWER LEFT CORNER.'
      endif
      fxaddpar, ha_hdr, 'CRPIX1',   ref_xcen, 'X SUN CENTER (PIXELS)'
      fxaddpar, ha_hdr, 'CRPIX2',   ref_ycen, 'Y SUN CENTER (PIXELS)'
      fxaddpar, ha_hdr, 'CRRADIUS', ref_xrad, 'PIXELS PER SOLAR RADIUS'
      writefits, ha_out_file, ha_img_expand, ha_hdr
   end

; --- Flip images top-to-bottom since Sun Rasterfile's origin is upper left,
;     whereas SXT and IDL use lower left corner as the origin.

ha_img_exprev  = reverse ( ha_img_expand, 2)

; --- Create 24-bit color image.
;     H-alpha   --> red channel.
;
; --- Note: For standard 24-bit Sun Rasterfile,
;     color channel order is blue, green, red.

bgr_img = bytarr (3, nx, ny)

bgr_img (2, *, *) = ha_img_exprev		; red channel.

; --- Create RGB colormap (equal brightness for all 3 colors).

map_red   = bindgen (256)

map_green = bindgen (256)

map_blue  = bindgen (256)

; --- Write image to disk as a 24-bit (BGR) Sun rasterfile.
 
srf_file = 'hs.' + sdaytim + '.ras'		; Sun rasterfile name.

; write_srf, srf_file, bgr_img, map_red, map_green, map_blue

write_ras, srf_file, bgr_img

!P.font = -1

end
