;+
; NAME:
;        IMGFMT2
;
; PURPOSE:
;         Resize and repostion a solar disk image so that it is registered
;         with a specified image center & diameter.
;      
; METHOD: The image is resized and repositioned to match the output specs.
;
; CALLING SEQUENCE:
;
; obuf = imgfmt2 (ibuf, MKTV=mktv					  $
;           ISIZE=[ixdim,iydim], ICENTER=[ixcen, iycen], IRADIUS=iradius, $
;           OSIZE=[oxdim,oydim], OCENTER=[oxcen, oycen], ORADIUS=oradius)
;                                                            
; PARAMETERS:
;
;	obuf		The rescaled output image.
;
;	ibuf		The input image.
;
;	ISIZE		input image size [xdim, ydim]	default: [512,512]
;	ICENTER		input [x,y] center		default: [255.5,255.5]
;	IRADIUS		input solar radius		default: 224
;
;	OSIZE		output image size [xdim, ydim]	default: [512,512]
;	OCENTER		output [x,y] center		default: [255.5,255.5]
;	ORADIUS		output solar radius		default: 224 
;
; KEYWORDS:
;
;	MKTV   		if you want to put obuf on a window set /mktv.
;
; CAUTION:
;
; RETURN TYPE: Same as input.
; 
; HISTORY:
;	Andrew L. Stanger (ALS), HAO/NCAR, 6 May 1996.
;        9 May 1996: Add gen_loc call.
;	15 Jun 1998: Clip cixbeg/end & ciybeg/end at array boundaries.
;-

FUNCTION imgfmt2, ibuf, MKTV=mktv,					$
		  ISIZE=isize, ICENTER=icenter, IRADIUS=iradius,	$
		  OSIZE=osize, OCENTER=ocenter, ORADIUS=oradius

; --- Specify location and size of the target image (video frame).

IF (KEYWORD_SET (isize)) THEN BEGIN
   ixdim = isize (0)
   iydim = isize (1)
ENDIF ELSE BEGIN			; Default output size.
   ixdim = 512
   iydim = 512
ENDELSE

IF (KEYWORD_SET (icenter)) THEN BEGIN
   ixcen = icenter (0)
   iycen = icenter (1)
ENDIF ELSE BEGIN			; Center of default image [512,512].
   ixcen = 255.5
   iycen = 255.5
ENDELSE

IF (KEYWORD_SET (iradius)) THEN BEGIN
   ixrad = iradius
   iyrad = iradius
ENDIF ELSE BEGIN			; Default radius.
   ixrad = 224.0
   iyrad = 224.0
ENDELSE

print, 'imgfmt> ixdim,iydim: ', ixdim, iydim
print, 'imgfmt> ixcen,iycen: ', ixcen, iycen
print, 'imgfmt> ixrad,iyrad: ', iyrad, iyrad

IF (KEYWORD_SET (osize)) THEN BEGIN
   oxdim = osize (0)
   oydim = osize (1)
ENDIF ELSE BEGIN			; Default output size.
   oxdim = 512
   oydim = 512
ENDELSE

IF (KEYWORD_SET (ocenter)) THEN BEGIN
   oxcen = ocenter (0)
   oycen = ocenter (1)
ENDIF ELSE BEGIN			; Center of default image [512,512].
   oxcen = 255.5
   oycen = 255.5
ENDELSE

IF (KEYWORD_SET (oradius)) THEN BEGIN
   oxrad = oradius
   oyrad = oradius
ENDIF ELSE BEGIN			; Default radius.
   oxrad = 224.0
   oyrad = 224.0
ENDELSE

print, 'imgfmt> oxdim,oydim: ', oxdim, oydim
print, 'imgfmt> oxcen,oycen: ', oxcen, oycen
print, 'imgfmt> oxrad,oyrad: ', oyrad, oyrad

;--- Create new array in which solar disk is centered and has the
;    same FOV ratio as the specified output image.
;
;--- Make new dimension based upon Field-Of-View ratio (solar radii / dimension)

xfovin  = ixdim / ixrad			; Input  x-dimension FOV in Rsun units
yfovin  = iydim / iyrad			; Input  y-dimension FOV in Rsun units
xfovout = oxdim / oxrad			; Output x-dimension FOV in Rsun units
yfovout = oydim / oyrad			; Output y-dimension FOV in Rsun units
iratio = xfovin  / yfovin
oratio = xfovout / yfovout

;cixdim = fix (xfovout * ixrad + 0.5)	; x-dimension output size (pixels)
;ciydim = fix (yfovout * iyrad + 0.5)	; y-dimension output size (pixels)
;cixdim = (cixdim/2) * 2			; Round UP.
;ciydim = (ciydim/2) * 2			; Round UP.

xmag = oxrad / ixrad			; x-axis magnification factor.
ymag = oyrad / iyrad			; y-axis magnification factor.

cixdim = xmag * ixdim + 0.5
ciydim = ymag * iydim + 0.5
cixdim = (cixdim/2) * 2			; Round UP.
ciydim = (ciydim/2) * 2			; Round UP.

print, 'imgfmt> xfovin, yfovin, ratio: ', xfovin, yfovin, xfovin/yfovin
print, 'imgfmt> xfovout,yfovout,ratio: ', xfovout, yfovout, xfovout/yfovout

;cibuf = bytarr (cixdim, ciydim)

;cixcen = cixdim / 2 - 1
;ciycen = ciydim / 2 - 1

;cixbeg = cixcen - ixcen
;cixend = cixbeg + ixdim - 1
;ciybeg = ciycen - iycen
;ciyend = ciybeg + iydim - 1

;IF (cixbeg < 0)         THEN cixbeg = 0
;IF (cixend > cixdim-1)  THEN cixend = cixdim - 1
;IF (ciybeg < 0)         THEN ciybeg = 0
;IF (ciyend > ciydim-1)  THEN ciyend = ciydim - 1

;print, 'imgfmt> size ( ibuf): ', size ( ibuf)
;print, 'imgfmt> size (cibuf): ', size (cibuf)
;print, 'imgfmt> cixdim,ciydim: ', cixdim, ciydim
;print, 'imgfmt> cixcen,ciycen: ', cixcen, ciycen
;print, 'imgfmt> cixbeg,cixend,ciybeg,ciyend: ', cixbeg, cixend, ciybeg, ciyend

;cibuf (cixbeg : cixend, ciybeg : ciyend) =  ibuf (*, *)

; --- Resize the input image.

;obuf = congrid (cibuf, oxdim, oydim, 0, /cubic)

;obuf = congrid (cibuf, oxdim, oydim, 0)

cibuf = bytarr (oxdim, oydim)

cibuf = congrid (ibuf, cixdim, ciydim, 0)

print, 'size (cibuf): ', size (cibuf)
cixsize = size (cibuf)
cixdim = cixsize [1]
ciydim = cixsize [2]
cixcen = cixdim / 2
ciycen = ciydim / 2

obuf = bytarr (oxdim, oydim)

oxcen  = fix (oxcen)
oycen  = fix (oycen)
;cixcen = fix (ixcen * xmag)
;ciycen = fix (iycen * ymag)
cixbeg = cixcen - oxcen
cixend = cixcen + oxcen
ciybeg = ciycen - oycen
ciyend = ciycen + oycen
if (cixbeg < 0)  then cixbeg = 0
if (cixend > cixdim - 1)  then cixend = cixdim - 1
if (ciybeg < 0)  then ciybeg = 0
if (ciyend > ciydim - 1)  then ciyend = ciydim - 1

print, 'imgfmt> cixdim,ciydim: ', cixdim, ciydim
print, 'imgfmt> cixcen,ciycen: ', cixcen, ciycen
print, 'imgfmt> cixbeg,cixend,ciybeg,ciyend: ', cixbeg, cixend, ciybeg, ciyend

oxbeg = cixbeg
oxend = cixend
oybeg = ciybeg
oyend = ciyend
print, 'oxdim, oydim: ', oxdim, oydim
print, 'imgfmt> oxbeg,oxend,oybeg,oyend: ', oxbeg, oxend, oybeg, oyend

obuf [oxbeg:oxend, oybeg:oyend]= cibuf [cixbeg:cixend, ciybeg:ciyend]

IF (KEYWORD_SET (mktv) NE 0) THEN TVSCL, obuf		; Plot on screen.

RETURN, obuf

END
