;+
; NAME:
;        IMGFMT_MATCH
;
; PURPOSE:
;         Resize and repostion a solar disk image so that it is registered
;         with a specified image center & diameter (Default is Mark-III).
;      
; METHOD: The image is passed through GEN_LOC to find the centroid
;         and the axial diameters, then resized and reposisitioned
;         to match the first (the TARGET).
;         The SXT variable indicates which images, if any, are SXT images. 
;
; CALLING SEQUENCE:
;
;          img_out = imgfmt_match (img_inp[, index][, mktv=mktv])
;                                                            
; PARAMETERS:
;
;	img_out		The rescaled output image.
;
;	img_inp		The input image.
;	index		If img_inp is SXT, a single index entry is required. 
;
; OPTIONAL KEYWORDS:
;
;	CENTER		[x,y] center			default: [319.5,239.5]
;	DIAM		solar diameter			default: 360
;	OUTSIZ		output image size [xdim, ydim]	default: [640,480]
;
; KEYWORDS:
;
;	MKTV   		if you want to put outimg on a window set /mktv.
;	QUIET		pass through to DSK_LOCG, no display.
;	INTER		pass through to DSK_LOCG, interactive point edit.
;
; RESTRICTIONS:
;
;	When working with fits images, the top few lines or so 
;	should be removed before input. These contain the 
;	header and gum things up.
;
;	When matching an SXT to a GBO image the sizes must be the
;	same to within about 5%. In all other cases there is no
;	restriction on relative size.
;
; RETURN TYPE: Same as input.
; 
; CALLS:     GEN_LOC
;
; HISTORY:
;
;	Procedure developed to shrink BBSO Halpha image and place it
;	correctly in a 512x512 SXT image. Written by L. Acton  7-nov-91
;	Modified for generality  A. McAllister 10-apr-92
;	Added automatic sun finder, A. McA. 16-apr-92
;	Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;
;	Version 2. Rewrote for general sized matrices, using DSK_LOC.
;	changed calling sequence to include target image, A.McA 24-apr-92
;	Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;	Added fractional pixel repositioning, A.McA. 7-may-92
;	Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;	Output variable type now same as input type, A.McA. 2-dec-92
;
;	Version 3. Reworked for flexible image type. Renamed from
;	GBO_SCALE2, A.McA. 14-dec-92.
;	Minor change to match DSK_LOCG use, added QUIET and INTER 
;	keywords, A.McA. 24-aug-94.
;	Changed congrid call to use cubic convolution interpolation,
;	A.McA. 24-aug-94
;
;	Adapted from IMG_MATCH, written by Alan McAllister, for MK3 target.
;	Andrew L. Stanger (ALS), HAO/NCAR, 22 March 1995.
;-

FUNCTION imgfmt_match, img_inp, index, mktv=mktv, $
		       center=center, diam=diam, outsiz=outsiz

; --- sxt	image type: 1=sxt, 0=gbo.

IF (N_PARAMS (0) EQ 2) THEN BEGIN		   ; One SXT image.

  sxt = 1

ENDIF ELSE BEGIN 

   sxt = 0

ENDELSE

; --- Specify location and size of the target image (video frame).

if (keyword_set (center)) then begin
   oxc = center (0)
   oyc = center (1)
endif else begin			; Center of video frame [640,480].
   oxc = 319.5
   oyc = 239.5
endelse

if (keyword_set (diam)) then begin
   oxd = diam
   oyd = diam
endif else begin			; Default radius.
   oxd = 360.0
   oyd = 360.0
endelse

if (keyword_set (outsiz)) then begin
   oxdim = outsiz (0)
   oydim = outsiz (1)
endif else begin			; Default output size.
   oxdim = 640
   oydim = 480
endelse

target = [oxc, oyc, oxd, oxd, oxdim, oydim]	; output target.

PRINT, 'oxc,oyc,oxd,oyd,oxdim,oydim: ', oxc, oyc, oxd, oyd, oxdim, oydim

; --- Resize the input image.

if (sxt EQ 0)  then					$
img_out = imgscl4 (target, img_inp, [0,0])		$
else							$
img_out = imgscl4 (target, img_inp, [0,1], index)

IF (KEYWORD_SET (mktv) NE 0) THEN TV, img_out		; Plot on screen.

RETURN, img_out

END
