;+
; NAME:
;	IMGSCL3
;
; PURPOSE:
;	Take an image of the sun and resize, and reposition it to match
;	an input centroid and radius.
;	Like IMG_MATCH, except it takes target data (refvec) as input,
;	rather than the target image.
;      
; METHOD:
;	The image is passed through GEN_LOC to find the centroid
;	and the axial diameters, it is then resized and reposisitioned 
;	to match the data in TRGDT.
;	The SXT variable indicates which images, if any, are SXT images.
;
; CALLING SEQUENCE:
;
;	outimg = imgscl3 (refvec, gboimg, sxt[, indx1][, indx2][, mktv=mktv])
;
; PARAMETERS:
;
;	outimg	The rescaled output image.
;	refvec	A 6 component vector which contains: the CENTROID (0,1),
;		the DIAMETERS(horizontal,vertical;2,3), and the pixel 
;		size (x,y:4,5) of the target image. 
;	gboimg	The input image.
;	sxt	A binary two-vector mask: 1=sxt, 0=gbo.
;	indx1	A single index entry for either trgimg or gboimg.
;	indx2	The second index when both are SXT images.
;
; KEYWORDS:
; 
;	MKTV   	If you want to put outimg on a window set /mktv.
;	QUIET	Pass through to DSK_LOCG, no display.
;	INTER	Pass through to DSK_LOCG, interactive point edit.
;       
; RESTRICTIONS:
;	When working with fits images the top few lines or so 
;	should be removed before input. These contain the 
;	header and gum things up.
;
;	When matching an SXT to a GBO image the sizes must be the 
;	same to within about 5%. In all other cases there is no
;	restriction on relative size.
;
; RETURN TYPE: Same as input.
;
; CALLS: GEN_LOC  
;
; NOTE:
;	Expects the indices to have .HIS tags and that "his_index,/enable"
;	has been invoked.
;
; HISTORY:
;	Procedure developed to shrink BBSO Halpha image and 
;	place it correctly in a 512x512 SXT image 
;	Written by L. Acton  7-nov-91
;	Modified for generality  10-apr-92
;	Added automatic sun finder, A. McA. 16-apr-92
;	Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;
;	Version 2. Rewrote for general sized matrices, using dsk_loc.
;	changed calling sequence to include target image, A.McA 24-apr-92
;	Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;	Added fractional pixel repositioning, A.McA. 7-may-92
;	Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;
;	Rewritten from GBO_SCALE2, takes target info as input,
;	A.McA. 6-oct-92
;
;	Output variable type now same as input type, A.McA. 2-dec-92
;
;	Version 2. Reworked for flexible image type. A.McA. 14-dec-92 
;	Return radius rather than diameters for first GEN_LOC/SXT call,
;       A.McA 14-feb-94 
;	Reworked to index update to function with FIT_LIMB. A.McA. 4-apr-94.
;	Minor change to match DSK_LOCG use, added QUIET and INTER keywords,
;	A.McA. 24-aug-94.
;	Changed congrid call to use cubic convolution interpolation,
;	A.McA. 24-aug-94
;	Added CIR keyword, for use with corrected images, A.McA. 2-sep-94.
;-
FUNCTION imgscl3, refvec, gboimg, sxt, indx1, indx2, rough=rough, mktv=mktv, $
		  inter=inter, quiet=quiet, cir=cir

cir = 1						; Circle fit.

if n_params (0) eq 4 then begin			; Only one sxt image.

  if total (sxt) ne 1 then begin		; Trap sxt/index mismatch.

    print, "IMGSCL3: The number of indices doesn't match SXT vector"
    return, -1
    stop

  endif

  if sxt (0) eq 1 then indxt=indx1
  if sxt (1) eq 1 then indxi=indx1

endif else begin

  if n_params (0) eq 5 then begin		; Two sxt images.

    if total (sxt) ne 2 then begin		; Trap sxt/index mismatch.

      print, "IMGSCL3: The number of indices dosn't match SXT vector"
      return, -1
      stop

    endif

    indxt = indx1
    indxi = indx2

  endif

endelse

; --- Find input x/y diameters.

if sxt (1) eq 0  then begin

  print, 'IMGSCL3: Input not SXT image'
  
  ; --- Invoking the ellipse fit.

  gen_loc, gboimg, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
           inter=inter, quiet=quiet, cir=cir
 
endif else begin

  print, 'IMGSCL3: Input SXT image'

  gen_loc, gboimg, indxi, BFRAD=rad, CENTROID=centroid, $
	   inter=inter, quiet=quiet, /SXT, rough=rough, cir=cir
  hzrd = 2.0 * rad   &   vtrd = 2.0 * rad 	; Horiz/Vert. diameters.

endelse

xc = centroid (0)
yc = centroid (1)
print, 'centroid: ', centroid
print, 'refvec: ', refvec
print, 'hzrd: ', hzrd, '   vtrd: ', vtrd

x0 = refvec (0)					; Target centroid.
y0 = refvec (1)

gbosz = size (gboimg)				; Size of input image.

; --- Resize image based upon ratio of target and input image diameters.
;
; Note that if conditions are right (large gradients in data with absolute
; values near 0) then congrid may introduce negative numbers into a
; previously positive definite field. This may have later consequences.

ha = congrid (gboimg, gbosz (1) * refvec (2) / hzrd, $
	      gbosz (2) * refvec (3) / vtrd, /cubic)

; --- Find sun center of resized image.

if sxt (1) eq 1 then begin			; Image is sxt image.

  if sxt (0) eq 1 then begin			; Target also sxt image.
  
    tres = gt_res (indxt) * vtrd / refvec (3)
    tshape = indxt.sxt.shape_sav
       
  endif else begin				; Target not sxt image.

    tres = gt_res (indxi) * vtrd / refvec (3)	; Adjust resolution.
    tshape = indxi.sxt.shape_sav
    
  endelse
  
  his_index, indxi , 0, 'pixel_size', tres		; Update index.
  indxi.sxt.shape_sav = tshape
  
  gen_loc, ha, indxi, CENTROID=cent, /SXT, $
	   rough=rough, cir=cir				; Find center
							; of resized image.
 
endif else begin					; Everything non-sxt.
  
  gen_loc, ha, CENTROID=cent, BFRAD=rad, /quiet, /cir	; Invoke the circle fit

endelse

dltx = fix (x0 - cent (0))                     	; Difference to one pixel.
dlty = fix (y0 - cent (1))

magfac = refvec (3) / vtrd
xc = xc * magfac
yc = yc * magfac
print, 'xc/yc: ', xc, yc

dltx = fix (x0 - xc)
dlty = fix (y0 - yc)

print, 'cent: ', cent
print, 'dltx: ', dltx, '   dlty: ', dlty

hasz = size (ha)                          	; Size of resized image.
print, 'hasz: ', hasz

imgout = replicate (0, refvec (4), refvec (5))   ; Initialize output image

; --- Copy resized input to output, while centering to nearest pixel.

imgout ((0 > dltx):((refvec (4) - 1) < (dltx + hasz (1) - 1)),		$
        (0 > dlty):((refvec (5) - 1) < (dlty + hasz (2) - 1))) =		$
    ha ((0 > (-1*dltx)):((hasz (1) - 1) < (refvec (4) - 1 - dltx)),	$
        (0 > (-1*dlty)):((hasz (2) - 1) < (refvec (5) - 1 - dlty)))

; Shift the last fraction, uses interpolation which may change intensity range.
; Use min value for 'uncovered' pixels to avoid shifting intensity range.

fdltx = (x0 - cent (0)) - fix (x0 - cent (0))	; Fractional pixel difference.
fdlty = (y0 - cent (1)) - fix (y0 - cent (1))
 
print, 'fdltx: ', fdltx, '   fdlty: ', fdlty

; --- Linear transformation.

imgout = poly_2d (imgout, [-fdltx, 0, 1, 0], [-fdlty, 1, 0, 0], $
		  1, missing=0)

if keyword_set (mktv) ne 0 then tv, imgout	; Plot on screen.

return, imgout

end

