;+
; NAME:
;        MK3_MATCH
;
; PURPOSE:
;         Resize and repostion a solar disk image so that it is registered
;         with an MLSO Mark-III image.
;      
; METHOD: The image is passed through GEN_LOC to find the centroid
;         and the axial diameters, then resized and repositioned
;         to match the first (the TARGET).
;         The SXT variable indicates which images, if any, are SXT images. 
;
; CALLING SEQUENCE:
;
;          img_out = mk3_match (img_inp[, index][, mktv=mktv])
;                                                            
; PARAMETERS:
;
;	    img_out	the rescaled output image.
;
;	    img_inp	the input image.
;           index	if img_inp is SXT, a single index entry is required. 
;
; KEYWORDS:
;	   MKTV   	if you want to put outimg on a window set /mktv.
;	   QUIET	pass through to DSK_LOCG, no display.
;	   INTER	pass through to DSK_LOCG, interactive point edit.
;
; RESTRICTIONS:
;
;             When working with fits images, the top few lines or so 
;             should be removed before input. These contain the 
;             header and gum things up.
;
;	      When matching an SXT to a GBO image the sizes must be the
;	      same to within about 5%. In all other cases there is no
;	      restriction on relative size.
;
; RETURN TYPE: Same as input.
; 
; CALLS:     GEN_LOC
;
; HISTORY:
;	   Procedure developed to shrink BBSO Halpha image and place it
;	   correctly in a 512x512 SXT image. Written by L. Acton  7-nov-91
;	   Modified for generality  A. McAllister 10-apr-92
;          Added automatic sun finder, A. McA. 16-apr-92
;          Corrected for different vertical/horizontal scaling A.McA 18-apr-92
;
;          Version 2. Rewrote for general sized matrices, using DSK_LOC.
;          changed calling sequence to include target image, A.McA 24-apr-92
;          Replaced location routine. Can now handle SXT images, A.McA. 6-may-92
;          Added fractional pixel repositioning, A.McA. 7-may-92
;          Changed congrid call to use bilinear extrapolation, A.McA. 26-may-92
;          Output variable type now same as input type, A.McA. 2-dec-92
;
;	   Version 3. Reworked for flexiable image type. Renamed from
;	   GBO_SCALE2, A.McA. 14-dec-92.
;	   Minor change to match DSK_LOCG use, added QUIET and INTER 
;	   keywords, A.McA. 24-aug-94.
;          Changed congrid call to use cubic convolution interpolation,
;          A.McA. 24-aug-94
;
;          Adapted from IMG_MATCH, written by Alan McAllister, for MK3 target.
;          Andrew L. Stanger (ALS), HAO/NCAR, 22 March 1995.
;
;-

FUNCTION mk3_match, img_inp, index, mktv=mktv, inter=inter, quiet=quiet

; --- sxt	image type: 1=sxt, 0=gbo.

IF (N_PARAMS (0) EQ 2) THEN BEGIN		   ; One SXT image.

  sxt = 1

ENDIF ELSE BEGIN 

   sxt = 0

ENDELSE

; --- Specify location and size of the target image (MLSO Mark-III).

x0 = 255.5
y0 = 255.5
dht = 104.4 * 2
dvt = 104.4 * 2

gbosz = SIZE (img_inp)			; Size of input image.

IF (sxt eq 0) THEN BEGIN		; Find input diameters & centroid.

  PRINT, 'Input not SXT image'

  gen_loc, img_inp, HORZRAD=dhi, VERTRAD=dvi, $
	   CENTROID=cent, inter=inter, quiet=quiet		; Ellipse fit.

ENDIF ELSE BEGIN

  PRINT, 'Input SXT image'

  gen_loc, img_inp, index, HORZRAD=dhi, VERTRAD=dvi, $		; SXT fit.
	   CENTROID=cent, /SXT

ENDELSE

PRINT, 'dht,dvt,dhi,dvi: ', dht, dvt, dhi, dvi

; --- Resize the input image.

img_new = CONGRID (img_inp, gbosz (1) * dht/dhi, gbosz (2) * dvt/dvi, /cubic)   

cent (0) = cent (0) * dht / dhi
cent (1) = cent (1) * dvt / dvi

;if (sxt eq 1) then begin			; Find center of resized image
;
;  index.sxt.imgparam=index.sxt.imgparam		; For now do nothing;
;						; assume same scale.
;
;
;  gen_loc, img_new, index, CENTROID=cent, /SXT	; For now assume its basically
;						; unchanged.
;                                              
;endif else begin
;
;  gen_loc, img_new, CENTROID=cent, BFRAD=rad, /quiet		; Circle fit.
;
;endelse

dltx = fix (x0 - cent (0))			; Difference to one pixel.
dlty = fix (y0 - cent (1))

imgsz  = SIZE (img_new)				; Size of resized image.
mk3xdim = 512					; Size of target image.
mk3ydim = 512

img_out = REPLICATE (0, mk3xdim, mk3ydim)   ; Output image same as input.

; --- Copy resized input to output, while centering to nearest pixel.

img_out ((0 > dltx):((mk3xdim-1) < (dltx+imgsz (1)-1)),         $
         (0 > dlty):((mk3ydim-1) < (dlty+imgsz (2)-1))) =       $
     img_new ((0 > (-1*dltx)):((imgsz (1)-1) < (mk3xdim-1-dltx)),   $
         (0 > (-1*dlty)):((imgsz (2)-1) < (mk3ydim-1-dlty)))

; --- Shift the last fraction, uses interpolation which may change
; --- the intensity range use min value for 'uncovered' pixels to avoid
; --- shifting the intensity range.

fdltx = (x0 - cent (0)) - fix (x0 - cent (0))	; Fractional pixel difference
fdlty = (y0 - cent (1)) - fix (y0 - cent (1))
 
img_out = POLY_2D (img_out, [-fdltx, 0, 1, 0], [-fdlty, 1, 0, 0], 1, $
		  MISSING=0, /CUBIC)

if (KEYWORD_SET (mktv) NE 0) THEN TV, img_out		; Plot on screen.

RETURN, img_out

END
