;+
; NAME
;	rotimg
;
; PURPOSE
;	Rotates an image about a specified center.
;
; METHOD
;	Puts images into a new array in which the specified center (cx, cy)
;	is at the center of the arry.  Then rot is called to perform
;	the rotation.
;
; CALLING SEQUENCE
;	img2 = rotimg (img1, angle, cx, cy)
;
; PARAMETERS
;	img1		Input Image Array.
;	angle		Rotation angle (Clockwise)
;	cx		x-axis center coordinate
;	cy		y-axis center coordinate
;
; KEYWORDS
;	none
;
; OUTPUT
;	img2		Image array containing the rotated image.
;
; CALLS
;	rot
;
; HISTORY
;	written 6 March 1997 by Andrew L. Stanger   HAO/NCAR
;-

FUNCTION rotimg, img1, angle, cx, cy

   imgsiz1 = size (img1)
   ndim = imgsiz1 (0)
   IF (ndim NE 2) THEN $
   BEGIN
      PRINT, 'Input variable must be a 2-D array.'
      RETURN, img1
   END

   ; Get X & Y dimensions of input array.

   dx1  = imgsiz1 (1)
   dy1  = imgsiz1 (2)
   print, 'dx1, dy1: ', dx1, dy1

   ; Get the data type of the input array.
   ; 0=undefined
   ; 1=byte
   ; 2=integer
   ; 3=longword integer
   ; 4=floating point
   ; 5=double precision floating point
   ; 6=complex floating point
   ; 7=string
   ; 8=structure
   ; 9=double precision floating point
   
   dtype = imgsiz1 (ndim + 1)

   ; Determine size of new array.

   bl = cx			; border left 
   br = dx1 - cx - 1		; border right
   bb = cy			; border bottom
   bt = dy1 - cy - 1		; border top

   xll = long (cx - 0)
   yll = long (cy - 0)

   xul = long (cx - 0)
   yul = long (dy1 - 1 - cy)

   xlr = long (dx1 - 1 - cx)
   ylr = long (cy - 0)

   xur = dx1 - 1 - cx
   yur = dy1 - 1 - cy

   ll = fix (sqrt (xll * xll + yll * yll) + 0.5)
   ul = fix (sqrt (xul * xul + yul * yul) + 0.5)
   lr = fix (sqrt (xlr * xlr + ylr * ylr) + 0.5)
   ur = fix (sqrt (xur * xur + yur * yur) + 0.5)

   print, 'xll, yll: ', xll, yll
   print, 'xul, yul: ', xul, yul
   print, 'xlr, ylr: ', xlr, ylr
   print, 'xur, yur: ', xur, yur

   print, 'xll * xll + yll * yll : ', xll * xll + yll * yll
   print, 'xul * xul + yul * yul : ', xul * xul + yul * yul
   print, 'xlr * xlr + ylr * ylr : ', xlr * xlr + ylr * ylr
   print, 'xur * xur + yur * yur : ', xur * xur + yur * yur

   print, 'bl, br, bb, bt: ', bl, br, bb, bt
   print, 'll, ul, lr, ur: ', ll, ul, lr, ur

   maxdim = MAX ([bl, br, bb, bt, ll, ul, lr, ur])
   dx2 = maxdim * 2
   if ( (dx2 / 2) * 2 NE dx2 ) THEN dx2 = dx2 + 1
   dy2 = dx2

   ; Create new array.

   cx2 = dx2 / 2
   cy2 = dy2 / 2

   CASE dtype OF
   1: img2 = bytarr (dx2, dy2)
   2: img2 = intarr (dx2, dy2)
   3: img2 = lonarr (dx2, dy2)
   4: img2 = fltarr (dx2, dy2)
   ELSE: BEGIN
            PRINT, 'Only byte, int & float arrays are supported.'
            RETURN, img1
	 END
   ENDCASE
   
   ; Copy input array into new array.

   xmin = cx2 - cx
   xmax = xmin + dx1 - 1
   ymin = cy2 - cy
   ymax = ymin + dy1 - 1

   print, 'dx2, dy2: ', dx2, dy2
   print, 'xmin, xmax, ymin, ymax: ', xmin, xmax, ymin, ymax

   img2 (xmin:xmax, ymin:ymax) = img1 (0:dx1-1, 0:dy1-1)

   mag = 1.0						; Magnification factor.
   img2 = rot (img2, angle, mag, cx2, cy2)

   RETURN, img2
END
