pro svid, index, data, outfil=outfil, infil=infil, ss=ss, $
            outdir=outdir, qstop=qstop
;
;+
;NAME:
;	svid
;PURPOSE:
;	Convert SXT image(s) to video format (640x480).
;METHOD:
;	Read a SXT index and data.
;	Resize/position to desired video frame location.
;       Write output image(s) to disk as Sun Rasterfile(s).
;	The output filename syntax is "sfvYYMMDD.HHMM"
;CALLING SEQUENCE:
;	svid, index, data
;	svid, index, data, outfil='sxt_img.sfi'
;	svid, infil=infil
;	svid, infil=infil, ss=ss
;	svid, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd0/stanger/sfi'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	outfil	- The output file name(s).  There should be one filename
;		  per image saved.  If "outfil" is not specified, then
;		  the output file name is:
;			SFVyymmdd.hhmmss	(for FFI)
;			SPVyymmdd.hhmmss	(for PFI)
;		  If "outfil" is passed as a scalar and there are
;		  multiple images passed in, then a .0001, .0002, .0003
;		  and so forth is appended to the end.
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the SRF file created
;	outres  - Output resolution; 0=full, 1=half, 2=quarter
;		  Default is to leave resolution unchanged.
;OUTPUT:
;	Creates SFV files
;HISTORY:
;	Written  8-May-95 by Andrew L. Stanger (ALS), HAO/NCAR
;       Adapted from SXT2MK3
;	outres keyword removed; Output resolution is fixed at 1 (512x512).
;-
 
!P.font = 0					; Use X-window hardware font
device, $
   font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'

window, colors = 256, xsize = 640, ysize = 480	; Force private colormap

;loadct, 3					; Load red temperature colormap.

lct, '/usr/people/stanger/color/sunsetvid.lut'	; load sunsetvid color table.

if (n_elements (outdir) eq 0) then outdir = ''

if (keyword_set (infil)) then begin

    qinfil = 1
    rd_roadmap, infil, roadmap
    print, 'n_elements (roadmap): ', n_elements (roadmap)
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)

end else begin

    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'svid: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)

end

noutfil = n_elements (ss)

if (keyword_set (outfil)) then begin

    outfil0 = outfil
    if (n_elements (outfil) ne noutfil) then $
       outfil0 = outfil (0) + '.' + string (indgen (noutfil), format='(i4.4)')

end else begin						; Create output filename

    prefix_arr = ['spv', 'sfv']				; partial or full frame
    ii = gt_pfi_ffi (input1) mod 2			; b0 = 0 is PFI
    prefix = prefix_arr (ii)				; sp_fits or sf_fits
    tims = anytim2ints (input1)				; date/time conversion
    out_fids = ex2fid ( anytim2ex (tims))		; make 'YYMMDD.HHMM'
    outfil0 = prefix + out_fids + '.ras'

end

if (keyword_set (qstop)) then stop

; --- Color assignments.

c0 = 250
c1 = 251
c2 = 252
c3 = 253
c4 = 254
c5 = 255
;c0 = 249
;c1 = c0
;c2 = c0
;c3 = c0
;c4 = c0
;c5 = c0

; --- Storage for color LUT.

r = intarr (256)
g = intarr (256)
b = intarr (256)

; --- File Loop ---

for ifil = 0, noutfil - 1 do begin

    if (qinfil) then begin
	rd_xda, infil, ss (ifil), index0, data0, roadmap	; Read image
    end else begin
	index0 = index (ss (ifil))
	data0 = data (*, *, ss (ifil))	; TODO - what about images that
					; have been padded during the read?
    end

    outres = 1
   if (keyword_set (outres)) then begin
      res_in = gt_res (index0)		; image resolution (0=full,1=half,2=qtr)
      res_out = outres			; 0=full, 1=half, 2=quarter resolution

       if (res_in ne res_out) then begin
          data0  = mod_res (index0, data0, index_out, outres=res_out, $
			    /update_index, /override, /norm_fact)
       endif

  endif

;    sav_sda, concat_dir (outdir, outfil0 (ifil)), index0, data0

   ; --- Resize & position SXT image to match output image resolution.

;   data0 (*,0:4) = 0				; Set bottom 5 rows to black.
;   data0 (0:1,*) = 0				; Set left two columns to black.

   xoff = 640 - 512				; xoff = annotation offset.
   xc = 239.5 + xoff
   yc = 239.5
   diam = 360
   outsiz = [640,480]

;   img = imgfmt_match (data0, index0,			$
;	 center=[xc,yc], diam=diam, outsiz=outsiz)

   ; --- Copy SXT image into video output buffer.

   smin = min (data0, max=smax)
   data0 = bytscl (data0, min=smin, max=smax, top=249)
   img = bytarr (640, 480)
   img (128:639,0:479) = data0 (0:511, 16:495)

   ; --- Display SXT image.

   imgsiz = size (img)
   sxt_nx = imgsiz (1)
   sxt_ny = imgsiz (2)

   tv, img

   ; --- Draw box around border.

   plots,   0,   0,            /device, color=c1
   plots, 639,   0, /continue, /device, color=c1
   plots, 639, 479, /continue, /device, color=c1
   plots,   0, 479, /continue, /device, color=c1
   plots,   0,   0, /continue, /device, color=c1
   plots, 128,   0,            /device, color=c1
   plots, 128, 479, /continue, /device, color=c1

   ; --- Write image annotation.

   sxt_name  = 'Yohkoh SXT'
   sxt_time  = gt_time (index0, /str)
   sxt_date  = gt_day  (index0, /str)
   sxt_filta = gt_filta  (index0, /string)
   sxt_filtb = gt_filtb  (index0, /string)
   sxt_exp   = gt_expdur (index0)
   sxt_exp   = string (sxt_exp, FORMAT="(F7.2)")
;   sxt_exp   = strcompress (sxt_exp, /remove_all)
   roll  = gt_sxt_roll (index0)
   case res_in of
   0: resval = 1024
   1: resval =  512
   2: resval =  256
   else: resval = 0
   endcase
   sxt_res   = string (resval, FORMAT="(I4)")
;   print, 'roll: ', roll
   sxt_roll = string (roll, FORMAT="(F7.2)")
;   sxt_cen   = gt_sxt_cen (index0)
;   print, 'sxt_cen: ', sxt_cen

   xloc =  12
   yinc =  64
   yloc = 462 - 42

   device, $
      font='-misc-fixed-medium-r-normal--14-110-100-100-c-70-iso8859-1'
   xyouts, xloc, yloc,            'DATE',     /device, color=c1
   xyouts, xloc, yloc - 1 * yinc, 'TIME',     /device, color=c1
   xyouts, xloc, yloc - 2 * yinc, 'FILTER-A', /device, color=c1
   xyouts, xloc, yloc - 3 * yinc, 'FILTER-B', /device, color=c1
   xyouts, xloc, yloc - 4 * yinc, 'EXP.DUR.', /device, color=c1
   xyouts, xloc, yloc - 5 * yinc, 'ROLL',     /device, color=c1
   xyouts, xloc, yloc - 6 * yinc, 'RES.IN',   /device, color=c1

   xloc =  12
   yinc =  64
   yloc = 462

   device, $
      font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'
   xyouts, xloc, yloc,          sxt_name,  /device, color=c3
   xyouts, xloc, yloc - 1*yinc, sxt_date,  /device, color=c4
   xyouts, xloc, yloc - 2*yinc, sxt_time,  /device, color=c4
   xyouts, xloc, yloc - 3*yinc, sxt_filta, /device, color=c4
   xyouts, xloc, yloc - 4*yinc, sxt_filtb, /device, color=c4
   xyouts, xloc, yloc - 5*yinc, sxt_exp,   /device, color=c4
   xyouts, xloc, yloc - 6*yinc, sxt_roll,  /device, color=c4
   xyouts, xloc, yloc - 7*yinc, sxt_res,   /device, color=c4
;   xyouts, xloc, yloc - 7*yinc, sxt_cen,   /device, color=c4

   ; --- Read annotated image back into IDL array.

   img = tvrd (0, 0, sxt_nx, sxt_ny)
   tvlct, r, g, b, /get				; Read color LUT.

   ; --- Write to disk as a Sun rasterfile.

   imgrev = reverse (img, 2)				; Flip top to bottom.
   write_srf, outfil0 (ifil), imgrev, r, g, b		; Write Sun rasterfile.
   print, 'svid: output file ', outfil0 (ifil)

endfor

!P.font = 0

end
