pro svid2, index, data, outfil=outfil, infil=infil, ss=ss, $
            outdir=outdir, qstop=qstop
;
;+
;NAME:
;	svid2
;PURPOSE:
;	Convert SXT image(s) to video format (640x480).
;METHOD:
;	Read a SXT index and data.
;	Resize/position to desired video frame location.
;       Write output image(s) to disk as Sun Rasterfile(s).
;	The output filename syntax is "sfvYYMMDD.HHMM"
;CALLING SEQUENCE:
;	svid2, index, data
;	svid2, index, data, outfil='sxt_img.sfi'
;	svid2, infil=infil
;	svid2, infil=infil, ss=ss
;	svid2, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd0/stanger/sfi'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	outfil	- The output file name(s).  There should be one filename
;		  per image saved.  If "outfil" is not specified, then
;		  the output file name is:
;			SFVyymmdd.hhmmss	(for FFI)
;			SPVyymmdd.hhmmss	(for PFI)
;		  If "outfil" is passed as a scalar and there are
;		  multiple images passed in, then a .0001, .0002, .0003
;		  and so forth is appended to the end.
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the SRF file created
;	outres  - Output resolution; 0=full, 1=half, 2=quarter
;		  Default is to leave resolution unchanged.
;OUTPUT:
;	Creates SFV files
;HISTORY:
;	Written  8-May-95 by Andrew L. Stanger (ALS), HAO/NCAR
;       Adapted from SXT2MK3
;	outres keyword removed; Output resolution is fixed at 1 (512x512).
;	7 June 1995: Reduced size of image from 432x432 to 400x400.
;                    Changed color table from sunsetvid.lut to blue_white.2.lut.
;
;-
 
xframe = 640				; x dimension of video frame.
yframe = 480				; y dimension of video frame.

!P.font = 0					; Use X-window hardware font
device, $
   font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'

window, colors = 256,	$
	xsize = xframe, ysize = yframe,	/pixmap		; Force private colormap

lct, '/usr/people/stanger/color/blue_white.2.lut'	; load color table.

if (n_elements (outdir) eq 0) then outdir = ''

if (keyword_set (infil)) then begin

    qinfil = 1
    rd_roadmap, infil, roadmap
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)

end else begin

    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'svid: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)

end

noutfil = n_elements (ss)

if (keyword_set (outfil)) then begin

    outfil0 = outfil
    if (n_elements (outfil) ne noutfil) then $
       outfil0 = outfil (0) + '.' + string (indgen (noutfil), format='(i4.4)')

end else begin						; Create output filename

    prefix_arr = ['spv', 'sfv']				; partial or full frame
    ii = gt_pfi_ffi (input1) mod 2			; b0 = 0 is PFI
    prefix = prefix_arr (ii)				; sp_fits or sf_fits
    tims = anytim2ints (input1)				; date/time conversion
    out_fids = ex2fid ( anytim2ex (tims))		; make 'YYMMDD.HHMM'
    outfil0 = prefix + out_fids + '.sun'

end

if (keyword_set (qstop)) then stop

; --- Color assignments.

c0 = 250
c1 = 251
c2 = 252
c3 = 253
c4 = 254
c5 = 255

; --- Storage for color LUT.

r = intarr (256)
g = intarr (256)
b = intarr (256)

; --- File Loop ---

for ifil = 0, noutfil - 1 do begin

    if (qinfil) then begin
	rd_xda, infil, ss (ifil), index0, data0, roadmap	; Read image
    end else begin
	index0 = index (ss (ifil))
	data0 = data (*, *, ss (ifil))	; TODO - what about images that
					; have been padded during the read?
    end

    outres = 1
   if (keyword_set (outres)) then begin
      res_in = gt_res (index0)		; image resolution (0=full,1=half,2=qtr)
      res_out = outres			; 0=full, 1=half, 2=quarter resolution

       if (res_in ne res_out) then begin
          data0  = mod_res (index0, data0, index_out, outres=res_out, $
			    /update_index, /override, /norm_fact)
       endif

  endif

;    sav_sda, concat_dir (outdir, outfil0 (ifil)), index0, data0

   ; --- Rescale pixel intensities.

   smin = min (data0, max=smax)
   data0 = bytscl (data0, min=smin, max=smax, top=249)

   ; --- Resize & position SXT image to match output image resolution.

;   data0 (*,0:4) = 0				; Set bottom 5 rows to black.
;   data0 (0:1,*) = 0				; Set left two columns to black.

;   xoff = 640 - 512				; xoff = annotation offset.
;   xc = 239.5 + xoff
;   yc = 239.5
;   diam = 360
;   outsiz = [640,480]

;   img = imgfmt_match (data0, index0,			$
;	 center=[xc,yc], diam=diam, outsiz=outsiz)

   ; --- Resize image for video frame (83% of 480x480 = 400x400).

   ximg   = 400				; x dimension of SXT image: video frame.
   yimg   = 400				; y dimension of SXT image: video frame.

   xmin =   0				; x begin.
   ymin =   0				; y begin.
   xmax = 639				; x end.
   ymax = 479				; y end.

   sxtimg = congrid (data0, ximg, yimg)	; resize image to fit video frame.

   ; --- Copy SXT image into digital video frame buffer.

   ybor = (yframe - yimg) / 2		; y border size.
   xbor = ybor				; x border size.
   xoff = xframe - ximg - xbor		; x lower left corner of image in frame.
   yoff = ybor				; y lower left corner of image in frame.
   fb = bytarr (xframe, yframe)
   fb (xoff:xoff+ximg-1, yoff:yoff+yimg-1) = sxtimg (0:ximg-1, 0:yimg-1)

   ; --- Display SXT image.

   fbsiz = size (fb)
   fb_nx = fbsiz (1)
   fb_ny = fbsiz (2)

   tv, fb

   ; --- Video frame perimeter.

   plots, xmin,   ymin,              /device, color=c1
   plots, xmax,   ymin,   /continue, /device, color=c1
   plots, xmax,   ymax,   /continue, /device, color=c1
   plots, xmin,   ymax,   /continue, /device, color=c1
   plots, xmin,   ymin,   /continue, /device, color=c1

   plots, xmin+1, ymin+1,            /device, color=c1
   plots, xmax-1, ymin+1, /continue, /device, color=c1
   plots, xmax-1, ymax-1, /continue, /device, color=c1
   plots, xmin+1, ymax-1, /continue, /device, color=c1
   plots, xmin+1, ymin+1, /continue, /device, color=c1

   yborder = (yframe - yimg) / 2		;  40 = (480 - 400) / 2
   xborder = yborder				;  40
   xannot  = xframe - ximg - xborder * 2	; 160 = 640 - 400 - 80
   yannot  = yframe        - yborder * 2	; 400 = 480       - 80

   xmin1 = xborder - 1
   xmin2 = xborder - 2
   xmax1 = xborder + xannot + ximg
   xmax2 = xborder + xannot + ximg + 1
   ymin1 = yborder - 1
   ymin2 = yborder - 2
   ymax1 = yborder + yannot
   ymax2 = yborder + yannot + 1

   ; --- Inner box.

   plots, xmin1, ymin1,            /device, color=c1
   plots, xmax1, ymin1, /continue, /device, color=c1
   plots, xmax1, ymax1, /continue, /device, color=c1
   plots, xmin1, ymax1, /continue, /device, color=c1
   plots, xmin1, ymin1, /continue, /device, color=c1

   ; --- Inner box.

   plots, xmin2, ymin2,            /device, color=c1
   plots, xmax2, ymin2, /continue, /device, color=c1
   plots, xmax2, ymax2, /continue, /device, color=c1
   plots, xmin2, ymax2, /continue, /device, color=c1
   plots, xmin2, ymin2, /continue, /device, color=c1

   ; --- Vertical line separating annotation and image.

   plots, xborder + xannot - 1, yborder,          /device, color=c1
   plots, xborder + xannot - 1, yborder + yannot, /device, color=c1, /continue
   plots, xborder + xannot - 2, yborder,          /device, color=c1
   plots, xborder + xannot - 2, yborder + yannot, /device, color=c1, /continue

   ; --- Color bar size & corner coordinates.

   xtick1 = 4					; small tick length.
   xtick2 = 8					; large tick length.
   xbar  = 12					; color bar x dimension.
   ybar = yimg					; color bar y dimension.

   xbmin = xborder + xtick2 + 1			; color bar x begin.
   ybmin = yborder				; color bar y begin.
   xbmax = xbmin   + xbar - 1			; color bar x end.
   ybmax = ybmin   + ybar - 1			; color bar y end.

   ; --- Create color bar and write to window.

   cbar = bytarr (xbar, ybar)			; color bar array.
   for i = 0, ybar -1 do	$
      cbar (*, i) = byte ((float (i)/ float (ybar)) * 255.0)
   tv, cbar, xbmin, ybmin			; Write colorbar to window.

   ; --- Tick marks at x11, 95, 159, 223.

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Tick marks at 63, 127, 191.

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Vertical line at right edge of color bar.

   plots, xbmax + x2 + 1, yborder,          /device, color=c1
   plots, xbmax + x2 + 1, yborder + yannot, /device, color=c1, /continue
   plots, xbmax + x2 + 2, yborder,          /device, color=c1
   plots, xbmax + x2 + 2, yborder + yannot, /device, color=c1, /continue

   ; --- Write image annotation.

   sxt_name  = 'Yohkoh SXT'
   sxt_time  = gt_time (index0, /str)
   sxt_date  = gt_day  (index0, /str)
   sxt_filta = gt_filta  (index0, /string)
   sxt_filtb = gt_filtb  (index0, /string)
   sxt_exp   = gt_expdur (index0)
   sxt_exp   = string (sxt_exp, FORMAT="(F7.2)")
   roll      = gt_sxt_roll (index0)
   case res_in of
   0: resval = 1024
   1: resval =  512
   2: resval =  256
   else: resval = 0
   endcase
   sxt_res  = string (resval, FORMAT="(I4)")
   sxt_roll = string (roll, FORMAT="(F7.2)")

;   sxt_cen   = gt_sxt_cen (index0)

;   print, 'roll: ', roll
;   print, 'sxt_cen: ', sxt_cen

   xloc = xborder + xbar + 32 
   yinc = 52
   yloc = yborder + yimg - 56

   device, $
      font='-*-times-bold-r-normal--14-*-*-*-*-*-*-*'
   xyouts, xloc, yloc,            'DATE',     /device, color=c1
   xyouts, xloc, yloc - 1 * yinc, 'TIME',     /device, color=c1
   xyouts, xloc, yloc - 2 * yinc, 'FILTER-A', /device, color=c1
   xyouts, xloc, yloc - 3 * yinc, 'FILTER-B', /device, color=c1
   xyouts, xloc, yloc - 4 * yinc, 'EXPOSURE', /device, color=c1
   xyouts, xloc, yloc - 5 * yinc, 'ROLL',     /device, color=c1
   xyouts, xloc, yloc - 6 * yinc, 'RESOLUTION',   /device, color=c1

   xloc = xborder + xbar + 32 
   yinc = 52
   yloc = yborder + yimg - 24 

   device, $
      font='-*-times-bold-r-normal--20-*-*-*-*-*-*-*'
   xyouts, xloc, yloc,          sxt_name,  /device, color=c4

   device, $
      font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'
   xyouts, xloc, yloc - 1*yinc, sxt_date,  /device, color=c0
   xyouts, xloc, yloc - 2*yinc, sxt_time,  /device, color=c0
   xyouts, xloc, yloc - 3*yinc, sxt_filta, /device, color=c3
   xyouts, xloc, yloc - 4*yinc, sxt_filtb, /device, color=c3
   xyouts, xloc, yloc - 5*yinc, sxt_exp,   /device, color=c3
   xyouts, xloc, yloc - 6*yinc, sxt_roll,  /device, color=c3
   xyouts, xloc, yloc - 7*yinc, sxt_res,   /device, color=c3
;   xyouts, xloc, yloc - 7*yinc, sxt_cen,   /device, color=c3

   ; --- Read annotated image back into IDL array.

   fb = tvrd (0, 0, fb_nx, fb_ny)
   tvlct, r, g, b, /get					; Read color LUT.

   ; --- Write to disk as a Sun rasterfile.

   fbrev = reverse (fb, 2)				; Flip top to bottom.
   write_srf, outfil0 (ifil), fbrev, r, g, b		; Write Sun rasterfile.
   print, 'svid: output file ', outfil0 (ifil)

endfor

!P.font = 0

end
