pro svid7, index, data, infil=infil, ss=ss, $
            outdir=outdir, qstop=qstop
;
;+
;NAME:
;	svid7
;PURPOSE:
;	Convert SXT image(s) to SFV video format (640x480).
;METHOD:
;	Read a SXT index and data.
;	Resize/position to desired video frame location.
;       Write output image(s) to disk as Sun Rasterfile(s).
;	The output filename syntax is "sfvYYMMDD.HHMM.sun"
;CALLING SEQUENCE:
;	svid7, index, data
;	svid7, infil=infil
;	svid7, infil=infil, ss=ss
;	svid7, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd0/stanger/sfi'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the SRF file created
;	outres  - Output resolution; 0=full, 1=half, 2=quarter
;		  Default is to leave resolution unchanged.
;OUTPUT:
;	Creates SFV files (sun rasterfile format).
;HISTORY:
;	Written  8-May-95 by Andrew L. Stanger [ALS], HAO/NCAR
;       Adapted from SXT2MK3
;	outres keyword removed.  Output resolution is fixed at 1 (512x512).
;
;	 7 Jun 1995: [ALS] Reduced size of image from 432x432 to 400x400.
;                    Changed color table from sunsetvid.lut to blue_white.2.lut.
;       27 Jun 1995: [ALS] Use imgscl3 to register images.
;                    Changed color table to 'blue_white.3.lut'.
;	31 Aug 1995: [ALS] Do bytscl AFTER call to imgscl3.
;	 1 Sep 1995: [ALS] Output file name now includes filter-b info.
;	 1 Sep 1995: [ALS] Delete arrays at end of each for loop cycle.
;	 1 Sep 1995: [ALS] Removed as much code as possible from for loop.
;	11 Sep 1995: [ALS] Add seconds to output filename.
;-
 
; --- Define frame buffer size.

xframe = 640				; x dimension of video frame.
yframe = 480				; y dimension of video frame.

;  --- Set up font, window, and colormap.

!P.font = 0				; Use X-window hardware font
device, $
   font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'

window, colors = 256,	$
	xsize = xframe, ysize = yframe,	/pixmap		; Force private colormap

lct, '/riker/usr/people/stanger/color/blue_white.3.lut'	; load color table.

; --- Color assignments.

c0 = 250						; yellow
c1 = 251						; grey
c2 = 252						; blue
c3 = 253						; green
c4 = 254						; red
c5 = 255						; white

; --- Storage for color LUT.

red   = intarr (256)
green = intarr (256)
blue  = intarr (256)

; --- Frame buffer initialization.

fb = bytarr (xframe, yframe)
fbsiz = size (fb)
fb_nx = fbsiz (1)			; = xframe
fb_ny = fbsiz (2)			; = yframe

; --- Define video frame buffer dimensions & borders.

ximg   = 400				; x dimension of SXT image: video frame.
yimg   = 400				; y dimension of SXT image: video frame.

xmin =   0				; x begin.
ymin =   0				; y begin.
xmax = 639				; x end.
ymax = 479				; y end.


yborder = (yframe - yimg) / 2		;  40 = (480 - 400) / 2
xborder = yborder			;  40
xoff = xframe - ximg - xborder		; x lower left corner of image in frame.
yoff = yborder				; y lower left corner of image in frame.
xannot  = xframe - ximg - xborder * 2	; 160 = 640 - 400 - 80
yannot  = yframe        - yborder * 2	; 400 = 480       - 80

xmin1 = xborder - 1
xmin2 = xborder - 2
xmax1 = xborder + xannot + ximg
xmax2 = xborder + xannot + ximg + 1
ymin1 = yborder - 1
ymin2 = yborder - 2
ymax1 = yborder + yannot
ymax2 = yborder + yannot + 1

; --- Color bar size & corner coordinates.

xtick1 = 4					; small tick length.
xtick2 = 8					; large tick length.
xbar  = 12					; color bar x dimension.
ybar = yimg					; color bar y dimension.

xbmin = xborder + xtick2 + 1			; color bar x begin.
ybmin = yborder				; color bar y begin.
xbmax = xbmin   + xbar - 1			; color bar x end.
ybmax = ybmin   + ybar - 1			; color bar y end.

cbar = bytarr (xbar, ybar)			; color bar array.

   ; --- Draw video frame perimeter (2 pixels thick).

   plots, xmin,   ymin,              /device, color=c1
   plots, xmax,   ymin,   /continue, /device, color=c1
   plots, xmax,   ymax,   /continue, /device, color=c1
   plots, xmin,   ymax,   /continue, /device, color=c1
   plots, xmin,   ymin,   /continue, /device, color=c1

   plots, xmin+1, ymin+1,            /device, color=c1
   plots, xmax-1, ymin+1, /continue, /device, color=c1
   plots, xmax-1, ymax-1, /continue, /device, color=c1
   plots, xmin+1, ymax-1, /continue, /device, color=c1
   plots, xmin+1, ymin+1, /continue, /device, color=c1

   ; --- Inner box (2 pixels thick).

   plots, xmin1, ymin1,            /device, color=c1
   plots, xmax1, ymin1, /continue, /device, color=c1
   plots, xmax1, ymax1, /continue, /device, color=c1
   plots, xmin1, ymax1, /continue, /device, color=c1
   plots, xmin1, ymin1, /continue, /device, color=c1

   plots, xmin2, ymin2,            /device, color=c1
   plots, xmax2, ymin2, /continue, /device, color=c1
   plots, xmax2, ymax2, /continue, /device, color=c1
   plots, xmin2, ymax2, /continue, /device, color=c1
   plots, xmin2, ymin2, /continue, /device, color=c1

   ; --- Vertical line separating annotation and image.

   plots, xborder + xannot - 1, yborder,          /device, color=c1
   plots, xborder + xannot - 1, yborder + yannot, /device, color=c1, /continue
   plots, xborder + xannot - 2, yborder,          /device, color=c1
   plots, xborder + xannot - 2, yborder + yannot, /device, color=c1, /continue

   ; --- Draw color bar in window.

   for i = 0, ybar -1 do	$
      cbar (*, i) = byte ((float (i)/ float (ybar)) * 255.0)
   tv, cbar, xbmin, ybmin			; Write colorbar to window.

   ; --- Tick marks at x11, 95, 159, 223.

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Tick marks at 63, 127, 191.

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * yimg) + ybmin
   plots, xbmax + x1, yloc,   /device, color=c1
   plots, xbmax + x2, yloc,   /device, color=c1, /continue
   plots, xbmax + x1, yloc+1, /device, color=c1
   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * yimg) + ybmin
   plots, xbmin - x1, yloc,   /device, color=c1
   plots, xbmin - x2, yloc,   /device, color=c1, /continue
   plots, xbmin - x1, yloc+1, /device, color=c1
   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Vertical line at right edge of color bar.

   plots, xbmax + x2 + 1, yborder,          /device, color=c1
   plots, xbmax + x2 + 1, yborder + yannot, /device, color=c1, /continue
   plots, xbmax + x2 + 2, yborder,          /device, color=c1
   plots, xbmax + x2 + 2, yborder + yannot, /device, color=c1, /continue

   ; --- Write labels to window.

   xloc = xborder + xbar + 32 
   yinc = 52
   yloc = yborder + yimg - 56

   device, $
      font='-*-times-bold-r-normal--14-*-*-*-*-*-*-*'
   xyouts, xloc, yloc,            'DATE',     /device, color=c1
   xyouts, xloc, yloc - 1 * yinc, 'TIME',     /device, color=c1
   xyouts, xloc, yloc - 2 * yinc, 'FILTER-A', /device, color=c1
   xyouts, xloc, yloc - 3 * yinc, 'FILTER-B', /device, color=c1
   xyouts, xloc, yloc - 4 * yinc, 'EXPOSURE', /device, color=c1
   xyouts, xloc, yloc - 5 * yinc, 'ROLL',     /device, color=c1
   xyouts, xloc, yloc - 6 * yinc, 'RESOLUTION',   /device, color=c1

   xloc = xborder + xbar + 32 
   yinc = 52
   yloc = yborder + yimg - 24 

   sxt_name  = 'Yohkoh SXT'
   device, $
      font='-*-times-bold-r-normal--20-*-*-*-*-*-*-*'
   xyouts, xloc, yloc,          sxt_name,  /device, color=c4

   fb = tvrd (0, 0, fb_nx, fb_ny)

; --- Define reference target
;     (6-element vector: x/y centroid, diam, pixel size)

ref_xdim = ximg 
ref_ydim = yimg
ref_xcen = ref_xdim / 2.0 - 0.5
ref_ycen = ref_ydim / 2.0 - 0.5
ref_xdia = 312.0				; 312 = 400 * (ref_xdim / 512)
ref_ydia = 312.0
ref_target = [ref_xcen, ref_ycen, ref_xdia, ref_ydia, ref_xdim, ref_ydim]

if (n_elements (outdir) eq 0) then outdir = ''

if (keyword_set (infil)) then begin	; Input from an SXT disk file.

    qinfil = 1
    rd_roadmap, infil, roadmap
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)

end else begin				; Input from IDL array.

    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'svid: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)

end

noutfil = n_elements (ss)

prefix_arr = ['spv', 'sfv']			; partial or full frame
ii = gt_pfi_ffi (input1) mod 2			; b0 = 0 is PFI
prefix = prefix_arr (ii)			; spv or sfv
tims = anytim2ints (input1)			; date/time conversion
out_fids = ex2fid (anytim2ex (tims), /sec)	; make 'YYMMDD.HHMMSS'

if (keyword_set (qstop)) then stop

; --- File Loop ---

for ifil = 0, noutfil - 1 do begin

   ; --- Get input SXT image.

   if (qinfil) then begin
      rd_xda, infil, ss (ifil), indexi, sxtimg, roadmap		; Read image
   end else begin
      indexi = index (ss (ifil))
      sxtimg = data (*, *, ss (ifil))	; TODO - what about images that
					; have been padded during the read?
   end

   ; --- Create output file name.

   sxt_filtb = gt_filtb  (indexi, /string)
   sxt_filtb = strcompress (sxt_filtb, /remove_all)
   outname = outdir + prefix (ifil) + out_fids (ifil)		$
	   + '.' + sxt_filtb + '.sun'			; Output filename.

   ; --- Adjust image to normalize resolution as specified (512x512 default).

   outres = 1
   if (keyword_set (outres)) then begin
      res_in = gt_res (indexi)		; image resolution (0=full,1=half,2=qtr)
      res_out = outres			; 0=full, 1=half, 2=quarter resolution

       if (res_in ne res_out) then begin
          sxtimg = mod_res (indexi, sxtimg, index_out, outres=res_out, $
			    /update_index, /override, /norm_fact)
       endif

  endif

   ; --- Rescale image size and reposition to center of image array.

   sxtimg = imgscl3 (ref_target, sxtimg, [0,1], indexi)

   ; --- Rescale pixel intensities to the range: 0 -> 249.

   sxtimg = bytscl (sxtimg, top=249)

   ; --- Display SXT image.

   tv, fb
   tv, sxtimg, xoff, yoff

   ; --- Write image annotation.

   sxt_time  = gt_time (indexi, /str)
   sxt_date  = gt_day  (indexi, /str)
   sxt_filta = gt_filta  (indexi, /string)
   sxt_filtb = gt_filtb  (indexi, /string)
   sxt_exp   = gt_expdur (indexi)
   sxt_exp   = string (sxt_exp, FORMAT="(F7.2)")
   roll      = gt_sxt_roll (indexi)

   case res_in of
         0: resval = 1024
         1: resval =  512
         2: resval =  256
      else: resval =    0
   endcase

   sxt_res  = string (resval, FORMAT="(I4)")
   sxt_roll = string (roll, FORMAT="(F7.2)")

;   sxt_cen   = gt_sxt_cen (indexi)

;   print, 'roll: ', roll
;   print, 'sxt_cen: ', sxt_cen

   ; --- Write parameters to window.

   device, $
      font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'
   xyouts, xloc, yloc - 1*yinc, sxt_date,  /device, color=c0
   xyouts, xloc, yloc - 2*yinc, sxt_time,  /device, color=c0
   xyouts, xloc, yloc - 3*yinc, sxt_filta, /device, color=c3
   xyouts, xloc, yloc - 4*yinc, sxt_filtb, /device, color=c3
   xyouts, xloc, yloc - 5*yinc, sxt_exp,   /device, color=c3
   xyouts, xloc, yloc - 6*yinc, sxt_roll,  /device, color=c3
   xyouts, xloc, yloc - 7*yinc, sxt_res,   /device, color=c3

;   xyouts, xloc, yloc - 8*yinc, sxt_cen,   /device, color=c3

   ; --- Read annotated image back into IDL array.

   vf = tvrd (0, 0, fb_nx, fb_ny)
   tvlct, red, green, blue, /get			; Read color LUT.

   ; --- Write to disk as a Sun rasterfile.

   vf = reverse (vf, 2)					; Flip top to bottom.
   write_srf, outname, vf, red, green, blue		; Write Sun rasterfile.
   print, '---> output file ', outname

endfor

!P.font = 0

end
