pro svidahm2a, index, data, infil=infil, ss=ss, $
            outdir=outdir, qstop=qstop
;
;+
;NAME:
;	svidahm2
;PURPOSE:
;	Convert SXT image(s) to SFV video format (640x480).
;METHOD:
;	Read a SXT index and data.
;	Resize/position to desired video frame location.
;       Write output image(s) to disk as Sun Rasterfile(s).
;	The output filename syntax is "sfvYYMMDD.HHMM.sun"
;CALLING SEQUENCE:
;	svidahm2a, index, data
;	svidahm2a, infil=infil
;	svidahm2a, infil=infil, ss=ss
;	svidahm2a, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd0/stanger/sfi'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the SRF file created
;	outres  - Output resolution; 0=full, 1=half, 2=quarter
;		  Default is to leave resolution unchanged.
;OUTPUT:
;	Creates SFV files (sun rasterfile format).
;HISTORY:
;	Written  8-May-95 by Andrew L. Stanger [ALS], HAO/NCAR
;       Adapted from SXT2MK3
;	outres keyword removed.  Output resolution is fixed at 1 (512x512).
;
;	 7 Jun 1995: [ALS] Reduced size of image from 432x432 to 400x400.
;                    Changed color table from sunsetvid.lut to blue_white.2.lut.
;       27 Jun 1995: [ALS] Use imgscl3 to register images.
;                    Changed color table to 'blue_white.3.lut'.
;	31 Aug 1995: [ALS] Do bytscl AFTER call to imgscl3.
;	 1 Sep 1995: [ALS] Output file name now includes filter-b info.
;	 1 Sep 1995: [ALS] Delete arrays at end of each for loop cycle.
;	 1 Sep 1995: [ALS] Removed as much code as possible from for loop.
;	11 Sep 1995: [ALS] Add seconds to output filename.
;	12 Sep 1995: [ALS] Changed fonts to itc bookman.
;	13 Sep 1995: [ALS] Add min/max parameters to 'bytscl (sxtimg)'.
;	14 Sep 1995: [ALS] For AlMg images, adjust intensity levels to
;			   approximate Al.1 images:
;			   gamtab = make_gamma (1.1) and
;                          sxtimg = gamtab (sxtimg) + 10
;-
 
; --- Define frame buffer size.

dxframe = 640				; x dimension of video frame.
dyframe = 480				; y dimension of video frame.

;  --- Set up font, window, and colormap.

!P.font = 0				; Use X-window hardware font
device, $
   font='-adobe-itc bookman-demi-r-normal--19-140-100-100-p-113-iso8859-1'

window, colors = 256,	$
	xsize = dxframe, ysize = dyframe, /pixmap	; Force private colormap

;lct, '/riker/usr/people/stanger/color/blue_white.3.lut'	; load color table.
;lct, '/riker/usr/people/stanger/color/greycp.lut'	; load color table.
lct, '/riker/usr/people/stanger/color/ahm.lut'	; load color table.

; --- Color assignments.

c0 = 250						; yellow
c1 = 251						; grey
c2 = 252						; blue
c3 = 253						; green
c4 = 254						; red
c5 = 255						; white

; --- Storage for color LUT.

red   = intarr (256)
green = intarr (256)
blue  = intarr (256)

; --- Frame buffer initialization.

fb = bytarr (dxframe, dyframe)
fbsiz = size (fb)
fb_nx = fbsiz (1)			; = dxframe
fb_ny = fbsiz (2)			; = dyframe

; --- Define video frame buffer dimensions & borders.

dximg   = 500				; x dimension of SXT image: video frame.
dyimg   = 350				; y dimension of SXT image: video frame.

xmin =   0				; x begin.
ymin =   0				; y begin.
xmax = dxframe - 1			; x end [639].
ymax = dyframe - 1			; y end [479].

dybor1 = (dyframe - dyimg) / 3		; 45 = (480 - 300) / 4
dybor2 =  dyframe - dyimg - dybor1	;135 =  480 - 300 - 45
dybor2 = dybor1
dxbor = (dxframe - dximg) / 2		; 70 = (640 - 500) / 2
xoff = dxframe - dximg - dxbor		; x lower left corner of image in frame.
yoff = dybor1				; y lower left corner of image in frame.
dxannot  = dxframe - dximg - dxbor * 2	;  70 = 640 - 500 - 70
dyannot  = dybor1 * 1			; 90
;dyannot = 0

xmin1 = dxbor - 1
xmin2 = dxbor - 2
xmax1 = dxbor + dxannot + dximg
xmax2 = dxbor + dxannot + dximg + 1
ymin1 = dybor1 - 1
ymin2 = dybor1 - 2
ymax1 = dybor1 + dyimg + dyannot
ymax2 = dybor1 + dyimg + dyannot + 1

; --- Color bar size & corner coordinates.

xtick1 = 0
xtick2 = 0
;xtick1 = 4					; small tick length.
;xtick2 = 8					; large tick length.
xbar = 0
ybar = 0
;xbar  = 12					; color bar x dimension.
;ybar = dyimg					; color bar y dimension.

xbmin = dxbor + xtick2 + 1			; color bar x begin.
ybmin = dybor1					; color bar y begin.
xbmax = xbmin   + xbar - 1			; color bar x end.
ybmax = ybmin   + ybar - 1			; color bar y end.

;cbar = bytarr (xbar, ybar)			; color bar array.

   ; --- Draw video frame perimeter (2 pixels thick).

   plots, xmin,   ymin,              /device, color=c1
   plots, xmax,   ymin,   /continue, /device, color=c1
   plots, xmax,   ymax,   /continue, /device, color=c1
   plots, xmin,   ymax,   /continue, /device, color=c1
   plots, xmin,   ymin,   /continue, /device, color=c1

   plots, xmin+1, ymin+1,            /device, color=c1
   plots, xmax-1, ymin+1, /continue, /device, color=c1
   plots, xmax-1, ymax-1, /continue, /device, color=c1
   plots, xmin+1, ymax-1, /continue, /device, color=c1
   plots, xmin+1, ymin+1, /continue, /device, color=c1

   ; --- Inner box (2 pixels thick).

   plots, xmin1, ymin1,            /device, color=c1
   plots, xmax1, ymin1, /continue, /device, color=c1
   plots, xmax1, ymax1, /continue, /device, color=c1
   plots, xmin1, ymax1, /continue, /device, color=c1
   plots, xmin1, ymin1, /continue, /device, color=c1

   plots, xmin2, ymin2,            /device, color=c1
   plots, xmax2, ymin2, /continue, /device, color=c1
   plots, xmax2, ymax2, /continue, /device, color=c1
   plots, xmin2, ymax2, /continue, /device, color=c1
   plots, xmin2, ymin2, /continue, /device, color=c1

   ; --- Horizontal line separating annotation and image.

   plots, dxbor,               dybor1 + dyimg + 2, /device, color=c1
   plots, dxbor + dximg - 1, dybor1 + dyimg + 2, /device, color=c1, /continue
   plots, dxbor,               dybor1 + dyimg + 3, /device, color=c1
   plots, dxbor + dximg - 1, dybor1 + dyimg + 3, /device, color=c1, /continue

   ; --- Draw color bar in window.

;   for i = 0, ybar -1 do	$
;      cbar (*, i) = byte ((float (i)/ float (ybar)) * 255.0)
;   tv, cbar, xbmin, ybmin			; Write colorbar to window.

   ; --- Tick marks at x11, 95, 159, 223.

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Tick marks at 63, 127, 191.

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * dyimg) + ybmin
;   plots, xbmax + x1, yloc,   /device, color=c1
;   plots, xbmax + x2, yloc,   /device, color=c1, /continue
;   plots, xbmax + x1, yloc+1, /device, color=c1
;   plots, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * dyimg) + ybmin
;   plots, xbmin - x1, yloc,   /device, color=c1
;   plots, xbmin - x2, yloc,   /device, color=c1, /continue
;   plots, xbmin - x1, yloc+1, /device, color=c1
;   plots, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Vertical line at right edge of color bar.

   plots, xbmax + x2 + 1, dybor1,          /device, color=c1
   plots, xbmax + x2 + 1, dybor1 + dyannot, /device, color=c1, /continue
   plots, xbmax + x2 + 2, dybor1,          /device, color=c1
   plots, xbmax + x2 + 2, dybor1 + dyannot, /device, color=c1, /continue

   ; --- Write labels to window.

   xannot = xbmax + x2 + 3		; X start location for annotation.
;   xloc = dxbor + xbar + 32 
   xloc = xannot + 4
   yinc = 20
   xloc = dxframe / 2
   yloc = dybor1*2 + dyimg - 10

   device, $
      font='-adobe-itc bookman-demi-r-normal--14-100-100-100-p-88-iso8859-1'
;   xyouts, xloc, yloc - 1 * yinc, 'DATE',     /device, color=c1
;   xyouts, xloc, yloc - 2 * yinc, 'TIME',     /device, color=c1
;   xyouts, xloc, yloc - 3 * yinc, 'FILTER-A', /device, color=c1

;   xyouts, xloc, yloc - 3 * yinc, 'FILTER-B', /device, color=c1
;   xyouts, xloc, yloc - 2 * yinc, 'EXPOSURE', /device, color=c1
;   xyouts, xloc, yloc - 3 * yinc, 'ROLL',     /device, color=c1
;   xyouts, xloc, yloc - 6 * yinc, 'RESOLUTION',   /device, color=c1

;   xloc = dxbor + xbar + 32 
   xloc = xannot + 4
   xinc = 100
   yinc = 20
   yloc = dybor1*2 + dyimg - 10

   sxt_name  = 'Yohkoh SXT'
   device, $
      font='-adobe-itc bookman-demi-r-normal--19-140-100-100-p-113-iso8859-1'
   xyouts, xloc, yloc - 1 * yinc,  sxt_name,  /device, color=c4

   fb = tvrd (0, 0, fb_nx, fb_ny)

; --- Define reference target
;     (6-element vector: x/y centroid, diam, pixel size)

ref_xdim = dximg 
ref_ydim = dyimg
ref_xcen = ref_xdim / 2.0 - 0.5
ref_ycen = ref_ydim / 2.0 - 0.5
ref_xdia = 312.0				; 312 = 400 * (ref_xdim / 512)
ref_ydia = 312.0
ref_target = [ref_xcen, ref_ycen, ref_xdia, ref_ydia, ref_xdim, ref_ydim]

if (n_elements (outdir) eq 0) then outdir = ''

if (keyword_set (infil)) then begin	; Input from an SXT disk file.

    qinfil = 1
    rd_roadmap, infil, roadmap
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)

end else begin				; Input from IDL array.

    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'svid: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)

end

noutfil = n_elements (ss)

prefix_arr = ['spv', 'sfv']			; partial or full frame
ii = gt_pfi_ffi (input1) mod 2			; b0 = 0 is PFI
prefix = prefix_arr (ii)			; spv or sfv
tims = anytim2ints (input1)			; date/time conversion
out_fids = ex2fid (anytim2ex (tims), /sec)	; make 'YYMMDD.HHMMSS'

if (keyword_set (qstop)) then stop

; --- Make gamma table.

gamval = 2.0
offval = 0
gamtab = make_gamma (gamval)

; --- File Loop ---

for ifil = 0, noutfil - 1 do begin

   ; --- Get input SXT image.

   if (qinfil) then begin
      rd_xda, infil, ss (ifil), indexi, imgbuf, roadmap		; Read image
   end else begin
      indexi = index (ss (ifil))
      imgbuf = data (*, *, ss (ifil))	; TODO - what about images that
					; have been padded during the read?
   end
;   sxtimg = imgbuf (*, 25:374)
   sxtimg = sfd_comp (imgbuf (*, 25:374))

;   sizsxt = size (sxtimg)
;   print, 'sizsxt: ', sizsxt
;   nbypix = sizsxt (3)

   ; --- Create output file name.

   ifiltb    = gt_filtb (indexi)	; =2 for Al.1, =3 for AlMg.
   sxt_filtb = gt_filtb (indexi, /string)
   sxt_filtb = strcompress (sxt_filtb, /remove_all)
   outname = outdir + prefix (ifil) + out_fids (ifil)		$
	   + '.' + sxt_filtb + '.sun'			; Output filename.
;   print, 'sxt_filtb, ifiltb: ', sxt_filtb, ifiltb

   ; --- Adjust image to normalize resolution as specified (512x512 default).

;   outres = 1
;   if (keyword_set (outres)) then begin
;      res_in = gt_res (indexi)		; image resolution (0=full,1=half,2=qtr)
;      res_out = outres			; 0=full, 1=half, 2=quarter resolution

;       if (res_in ne res_out) then begin
;          sxtimg = mod_res (indexi, sxtimg, index_out, outres=res_out, $
;			    /update_index, /override, /norm_fact)
;       endif

;  endif

   ; --- Rescale image size and reposition to center of image array.

;   sxtimg = imgscl3 (ref_target, sxtimg, [0,1], indexi)

   ; --- For AlMg images, adjust pixel intensities to simulate Al.1 images.

;   if (ifiltb eq 3) then	$
;   begin
;      sxtimg = (gamtab (sxtimg) + offval) < 255
;      print, 'Applying gamma (', string (gamval),			$
;	     ') & offset (', string (offval), ') to AlMg image.'
;   end

   ; --- Rescale pixel intensities to the range: 0 -> 249.

   smin =    0
   smax = 1000 
;   sxtimg = bytscl (sxtimg, min=smin, max=smax, top=249)
   sxtimg = bytscl (sxtimg, top=249)

      sxtimg = (gamtab (sxtimg) + offval) < 249

   ; --- Display SXT image.

   tv, fb
   tv, sxtimg, xoff, yoff

   ; --- Write image annotation.

   sxt_time  = gt_time (indexi, /str)
   sxt_date  = gt_day  (indexi, /str)
   sxt_filta = gt_filta  (indexi, /string)
   sxt_filtb = gt_filtb  (indexi, /string)
   sxt_exp   = gt_expdur (indexi)
   sxt_exp   = string (sxt_exp, FORMAT="(F7.2)")
   roll      = gt_sxt_roll (indexi)

;   case res_in of
;         0: resval = 1024
;         1: resval =  512
;         2: resval =  256
;      else: resval =    0
;   endcase

;   sxt_res  = string (resval, FORMAT="(I4)")
   sxt_roll = string (roll, FORMAT="(F9.2)")

;   sxt_cen   = gt_sxt_cen (indexi)

;   print, 'roll: ', roll
;   print, 'sxt_cen: ', sxt_cen

   ; --- Write parameters to window.

   xloc = (dxframe / 2) + 90
   yloc = dybor1*2 + dyimg - 10
   device, $
      font='-adobe-itc bookman-demi-r-normal--19-140-100-100-p-113-iso8859-1'

   xloc = xoff + 150

   xyouts, xloc, yloc - 1*yinc, sxt_date,  /device, color=c0
   xloc = xloc + 150
   xyouts, xloc, yloc - 1*yinc, sxt_time,  /device, color=c0
;   xyouts, xloc, yloc - 3*yinc, sxt_filta, /device, color=c3

   xloc = xloc + 140
   xyouts, xloc, yloc - 1*yinc, sxt_filtb, /device, color=c3
;   xyouts, xloc, yloc - 2*yinc, sxt_exp,   /device, color=c3
;   xyouts, xloc, yloc - 6*yinc, sxt_roll,  /device, color=c3
;   xyouts, xloc, yloc - 7*yinc, sxt_res,   /device, color=c3

   ; --- Read annotated image back into IDL array.

   vf = tvrd (0, 0, fb_nx, fb_ny)
   tvlct, red, green, blue, /get			; Read color LUT.

   ; --- Write to disk as a Sun rasterfile.

   vf = reverse (vf, 2)					; Flip top to bottom.
   write_srf, outname, vf, red, green, blue		; Write Sun rasterfile.
   print, '---> output file ', outname

endfor

!P.font = 0

end
