;+
;NAME:
;	svidfts2
;PURPOSE:
;	Convert FITS image(s) to video format (640x480).
;METHOD:
;	Read a FITS index and data.
;	Resize/position to desired video frame location.
;       Write output image(s) to disk as Sun Rasterfile(s).
;	The output filename syntax is "*.sun"
;CALLING SEQUENCE:
;	svid9, 'fits_list'
;	svid9, '/riker/d/img/smm/fits_list', outdir='/riker/d/img/vid'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	outdir	- The output directory where to put the SRF file created
;OUTPUT:
;	Creates video files (sun rasterfile format).
;HISTORY:
;	Written  8-May-95 by Andrew L. Stanger [ALS], HAO/NCAR
;	Adapted from SXT2MK3
;
;	 7 Jun 1995: [ALS] Reduced size of image from 432x432 to 400x400.
;                    Changed color table from sunsetvid.lut to blue_white.2.lut.
;       27 Jun 1995: [ALS] Use imgscl3 to register images.
;                    Changed color table to 'blue_white.3.lut'.
;	31 Aug 1995: [ALS] Do BYTSCL AFTER call to imgscl3.
;	 1 Sep 1995: [ALS] Output file name now includes filter-b info.
;	 1 Sep 1995: [ALS] Delete arrays at end of each for loop cycle.
;	 1 Sep 1995: [ALS] Removed as much code as possible from for loop.
;	11 Sep 1995: [ALS] Add seconds to output filename.
;	12 Sep 1995: [ALS] Changed fonts to itc bookman.
;	12 Sep 1995: [ALS] Add de_spiker routine.
;	24 Feb 1999: [ALS] Modify to accept FITS images (instead of SXT).
;-
 
PRO svidfts2, file_list, outdir=outdir, qstop=qstop

; --- Define frame buffer size.

dxframe = 640				; x dimension of video frame.
dyframe = 480				; y dimension of video frame.

;  --- Set up font, window, and colormap.

!P.font = 0				; Use X-window hardware font
device, $
   font='-adobe-itc bookman-demi-r-normal--19-140-100-100-p-113-iso8859-1'

window, colors = 256,	$
	xsize = dxframe, ysize = dyframe, /pixmap	; Force private colormap

lct, '/home/stanger/color/blue_white.3.lut'	; load color table.

; --- Color assignments.

c0 = 250						; yellow
c1 = 251						; grey
c2 = 252						; blue
c3 = 253						; green
c4 = 254						; red
c5 = 255						; white

; --- Storage for color LUT.

red   = intarr (256)
green = intarr (256)
blue  = intarr (256)

; --- Frame buffer initialization.

fb = bytarr (dxframe, dyframe)
fbsiz = size (fb)
fb_nx = fbsiz (1)			; = dxframe
fb_ny = fbsiz (2)			; = dyframe

; --- Define video frame buffer dimensions & borders.

dximg   = 400			; x dimension of FITS image: video frame.
dyimg   = 400			; y dimension of FITS image: video frame.

xmin =   0				; x BEGIN.
ymin =   0				; y BEGIN.
xmax = dxframe - 1			; x END [639].
ymax = dyframe - 1			; y END [479].

dybor = (dyframe - dyimg) / 2		; 40 = (480 - 400) / 2
dxbor = dybor				; 40
xoff = dxframe - dximg - dxbor		; x lower left corner of image in frame.
yoff = dybor				; y lower left corner of image in frame.
dxannot  = dxframe - dximg - dxbor * 2	; 160 = 640 - 400 - 80
dyannot  = dyframe         - dybor * 2	; 400 = 480       - 80

xmin1 = dxbor - 1
xmin2 = dxbor - 2
xmax1 = dxbor + dxannot + dximg
xmax2 = dxbor + dxannot + dximg + 1
ymin1 = dybor - 1
ymin2 = dybor - 2
ymax1 = dybor + dyannot
ymax2 = dybor + dyannot + 1

; --- Color bar size & corner coordinates.

xtick1 = 4					; small tick length.
xtick2 = 8					; large tick length.
xbar  = 12					; color bar x dimension.
ybar = dyimg					; color bar y dimension.

xbmin = dxbor + xtick2 + 1			; color bar x BEGIN.
ybmin = dybor					; color bar y BEGIN.
xbmax = xbmin   + xbar - 1			; color bar x END.
ybmax = ybmin   + ybar - 1			; color bar y END.

cbar = BYTARR (xbar, ybar)			; color bar array.

   ; --- Draw video frame perimeter (2 pixels thick).

   PLOTS, xmin,   ymin,              /device, color=c1
   PLOTS, xmax,   ymin,   /continue, /device, color=c1
   PLOTS, xmax,   ymax,   /continue, /device, color=c1
   PLOTS, xmin,   ymax,   /continue, /device, color=c1
   PLOTS, xmin,   ymin,   /continue, /device, color=c1

   PLOTS, xmin+1, ymin+1,            /device, color=c1
   PLOTS, xmax-1, ymin+1, /continue, /device, color=c1
   PLOTS, xmax-1, ymax-1, /continue, /device, color=c1
   PLOTS, xmin+1, ymax-1, /continue, /device, color=c1
   PLOTS, xmin+1, ymin+1, /continue, /device, color=c1

   ; --- Inner box (2 pixels thick).

   PLOTS, xmin1, ymin1,            /device, color=c1
   PLOTS, xmax1, ymin1, /continue, /device, color=c1
   PLOTS, xmax1, ymax1, /continue, /device, color=c1
   PLOTS, xmin1, ymax1, /continue, /device, color=c1
   PLOTS, xmin1, ymin1, /continue, /device, color=c1

   PLOTS, xmin2, ymin2,            /device, color=c1
   PLOTS, xmax2, ymin2, /continue, /device, color=c1
   PLOTS, xmax2, ymax2, /continue, /device, color=c1
   PLOTS, xmin2, ymax2, /continue, /device, color=c1
   PLOTS, xmin2, ymin2, /continue, /device, color=c1

   ; --- Vertical line separating annotation and image.

   PLOTS, dxbor + dxannot - 1, dybor,           /device, color=c1
   PLOTS, dxbor + dxannot - 1, dybor + dyannot, /device, color=c1, /continue
   PLOTS, dxbor + dxannot - 2, dybor,           /device, color=c1
   PLOTS, dxbor + dxannot - 2, dybor + dyannot, /device, color=c1, /continue

   ; --- Draw color bar in window.

   FOR i = 0, ybar -1 DO	$
      cbar (*, i) = BYTE ((float (i)/ float (ybar)) * 255.0)
   TV, cbar, xbmin, ybmin			; Write colorbar to window.

   ; --- Tick marks at x11, 95, 159, 223.

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1 + xtick1
   yloc = fix (0.125 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.375 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.625 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.875 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Tick marks at 63, 127, 191.

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * dyimg) + ybmin
   PLOTS, xbmax + x1, yloc,   /device, color=c1
   PLOTS, xbmax + x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmax + x1, yloc+1, /device, color=c1
   PLOTS, xbmax + x2, yloc+1, /device, color=c1, /continue

   x1 = 2
   x2 = x1  + xtick2 
   yloc = fix (0.25 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.50 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   yloc = fix (0.75 * dyimg) + ybmin
   PLOTS, xbmin - x1, yloc,   /device, color=c1
   PLOTS, xbmin - x2, yloc,   /device, color=c1, /continue
   PLOTS, xbmin - x1, yloc+1, /device, color=c1
   PLOTS, xbmin - x2, yloc+1, /device, color=c1, /continue

   ; --- Vertical line at right edge of color bar.

   PLOTS, xbmax + x2 + 1, dybor,          /device, color=c1
   PLOTS, xbmax + x2 + 1, dybor + dyannot, /device, color=c1, /continue
   PLOTS, xbmax + x2 + 2, dybor,          /device, color=c1
   PLOTS, xbmax + x2 + 2, dybor + dyannot, /device, color=c1, /continue

   ; --- Write labels to window.

   xannot = xbmax + x2 + 3		; X start location for annotation.
;   xloc = dxbor + xbar + 32 
   xloc = xannot + 4
   yinc = 52
   yloc = dybor + dyimg - 56

   device, $
      font='-adobe-itc bookman-demi-r-normal--14-100-100-100-p-88-iso8859-1'
   XYOUTS, xloc, yloc,            'DATE',     /device, color=c1
   XYOUTS, xloc, yloc - 1 * yinc, 'TIME',     /device, color=c1
   XYOUTS, xloc, yloc - 2 * yinc, 'ORIGIN',   /device, color=c1
   XYOUTS, xloc, yloc - 3 * yinc, 'TELESCOP', /device, color=c1
   XYOUTS, xloc, yloc - 4 * yinc, 'TYPE-OBS', /device, color=c1
   XYOUTS, xloc, yloc - 5 * yinc, 'EXPOSURE', /device, color=c1
   XYOUTS, xloc, yloc - 6 * yinc, 'ROLL',     /device, color=c1

;   xloc = dxbor + xbar + 32 
   xloc = xannot + 4
   yinc = 52
   yloc = dybor + dyimg - 24 

   fts_name  = 'FITS'
   DEVICE, $
      font='-adobe-itc bookman-demi-r-normal--19-140-100-100-p-113-iso8859-1'
   XYOUTS, xloc, yloc,          fts_name,  /device, color=c4

   fb = TVRD (0, 0, fb_nx, fb_ny)

IF (N_ELEMENTS (outdir) EQ 0) THEN outdir = ''

IF (KEYWORD_SET (qstop)) THEN stop

; --- File Loop ---

OPENR, lunlist, file_list, /GET_LUN
file_name = ''

WHILE (NOT EOF (lunlist)) DO			$

   BEGIN
   READF, lunlist, file_name

   ftsimg = readfits (file_name, hdu)

   fts_xdim   = fxpar (hdu, 'NAXIS1')
   fts_ydim   = fxpar (hdu, 'NAXIS2')
   fts_xcen   = fxpar (hdu, 'CRPIX1')
   fts_ycen   = fxpar (hdu, 'CRPIX2')
   fts_rsun   = fxpar (hdu, 'CRRADIUS')
   fts_time   = fxpar (hdu, 'TIME-OBS')
   fts_date   = fxpar (hdu, 'DATE-OBS')
   fts_origin = fxpar (hdu, 'ORIGIN')
   fts_tscope = fxpar (hdu, 'TELESCOP')
   fts_type   = fxpar (hdu, 'TYPE-OBS')
   fts_exp    = fxpar (hdu, 'EXPDUR')
   fts_exp    = STRING (fts_exp, FORMAT="(F7.2)")
   roll       = fxpar (hdu, 'CROTA1')
   fts_roll   = STRING (roll, FORMAT="(F10.2)")
   fts_min    = fxpar (hdu, 'DISPMIN')
   fts_max    = fxpar (hdu, 'DISPMAX')
   print, 'xcen,ycen: ', fts_xcen, fts_ycen

;   fts_cen   = gt_fts_cen (indexi)

;   print, 'roll: ', roll
;   print, 'fts_cen: ', fts_cen

   ; --- Define reference target
   ;     (6-element vector: x/y centroid, diam, pixel size)

   vid_xdim = dxframe 
   vid_ydim = dyframe
   vid_xcen = dxframe - 1 - (dximg / 2.0)
   vid_ycen = dyframe / 2.0
   vid_rsun = dximg / 2.5

   ; --- Create output file name.

   outname = file_name  + ".vid.sun"

   ; --- Remove spikes from image.

   threshold = 21
;   ftsimg = despike (ftsimg, threshold)

   ; --- Rescale image size and reposition to center of image array.

   vidimg = imgfmt2 (ftsimg,				$
		    ISIZE=[fts_xdim, fts_ydim],		$
		    ICENTER=[fts_xcen, fts_ycen],	$
		    IRADIUS=fts_rsun,			$
;		    OSIZE=[vid_xdim, vid_ydim],		$
		    OSIZE=[dximg, dyimg],		$
;		    OCENTER=[vid_xcen, vid_ycen],	$
		    OCENTER=[dximg/2.0, dyimg/2.0],	$
		    ORADIUS=vid_rsun)


   ; --- Rescale pixel intensities to the range: 0 -> 249.

;   vidimg = BYTSCL (vidimg, top=249)
   vidimg = BYTSCL (vidimg, min=fts_min, max=fts_max, top=249)

   ; --- Display FITS image.

   TV, fb
   TV, vidimg, xoff, yoff

   ; --- Write parameters to window.

   DEVICE, $
      font='-adobe-itc bookman-demi-r-normal--19-140-100-100-p-113-iso8859-1'
   XYOUTS, xloc, yloc - 1*yinc, fts_date,   /device, color=c0
   XYOUTS, xloc, yloc - 2*yinc, fts_time,   /device, color=c0
   XYOUTS, xloc, yloc - 3*yinc, fts_origin, /device, color=c3
   XYOUTS, xloc, yloc - 4*yinc, fts_tscope, /device, color=c3
   XYOUTS, xloc, yloc - 5*yinc, fts_type,   /device, color=c3
   XYOUTS, xloc, yloc - 6*yinc, fts_exp,    /device, color=c3
   XYOUTS, xloc, yloc - 7*yinc, fts_roll,   /device, color=c3

   ; --- Read annotated image back into IDL array.

   vf = TVRD (0, 0, fb_nx, fb_ny)
   TVLCT, red, green, blue, /get			; Read color LUT.

   ; --- Write to disk as a Sun rasterfile.

   vf = REVERSE (vf, 2)					; Flip top to bottom.
   WRITE_SRF, outname, vf, red, green, blue		; Write Sun rasterfile.
   print, '---> output file ', outname

ENDWHILE

!P.font = 0

END
