pro sxt2fts, index, data, outfil=outfil, infil=infil, ss=ss, $
	     outdir=outdir, qstop=qstop, $
	     rsun=rsun, suncen=suncen, comment=comment
;
;+
;NAME:
;	sxt2fts
;PURPOSE:
;	Take a SXT index and data and write a FITS file
;CALLING SEQUENCE:
;	sxt2fts, index, data
;	sxt2fts, index, data, outfil='sda_sample.fts'
;	sxt2fts, infil=infil
;	sxt2fts, infil=infil, ss=ss
;	sxt2fts, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd3/morrison/sgd'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	outfil	- The output file name(s).  There should be one filename
;		  per image saved.  If "outfil" is not specified, then
;		  the output file name is:
;			SF_FITSyymmdd.hhmmss	(for FFI)
;			SP_FITSyymmdd.hhmmss	(for FFI)
;		  If "outfil" is passed as a scalar and there are
;		  multiple images passed in, then a .0001, .0002, .0003
;		  and so forth is appended to the end.
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the FITS file created
;	rsun    - The number of pixels per solar radius.
;	suncen  - The sun center location (2-element vector in pixel units)
;	comment - Comment to include in FITS header.
;OUTPUT:
;	Creates FITS files
;HISTORY:
;	Written 19-Oct-92 by M.Morrison
;	 5-May-93 (MDM) - Modification to the header
;
;	12-Apr-1995 Andrew L. Stanger (ALS), HAO/NCAR
;	Header modifications: string parameters must be enclosed
;	by single quote marks.
;-
;
if (n_elements (outdir) eq 0) then outdir = ''

if (keyword_set (infil)) then begin
    qinfil = 1
    rd_roadmap, infil, roadmap
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)
end else begin
    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'SXT2FITS: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)
end
noutfil = n_elements (ss)
 
if (keyword_set (outfil)) then begin
    outfil0 = outfil
    if (n_elements (outfil) ne noutfil) then $
       outfil0 = outfil (0) + '.' + string (indgen (noutfil), format='(i4.4)')
end else begin
    prefix_arr = ['sp_fts', 'sf_fts']
    ii = gt_pfi_ffi (input1) mod 2	;b0 = 0 is PFI
    prefix   = prefix_arr (ii)
    tims     = anytim2ints (input1)
    out_fids = ex2fid ( anytim2ex (tims), /sec )
    outfil0  = prefix + out_fids
end
 
if (keyword_set (qstop)) then stop
 
for ifil = 0, noutfil - 1 do begin
    if (qinfil) then begin
	rd_xda, infil, ss (ifil), index0, data0, roadmap
    end else begin
	index0 = index (ss (ifil))
	data0  = data (*, *, ss (ifil))	; TODO - what about images
					; that have been padded during the read?
    end

    nbits  = get_nbytes (data0 (0,0))*8

    ; --- Get information from index.

    time   = gt_time (index0, /str)
    date   = gt_day (index0, /leadzero, /str)

    filta   = gt_filta (index0, /str)
    filtb   = gt_filtb (index0, /str)
    expmode = gt_expmode (index0, /str)
    res     = gt_res (index0, /str)
    dp_mode = gt_dp_mode (index0, /str)
    dp_rate = gt_dp_rate (index0, /str)
    print, '-',date,'-'
    print, '-',time,'-'
    print, '-',expmode,'-'
    print, '-',res,'-'
    print, '-',dp_mode,'-'
    print, '-',dp_rate,'-'

    ; --- Add single quote mark to begin/end of parameter strings.

    qtime     = "'        '"
    qdate     = "'        '"
    qfilta    = "'        '"
    qfiltb    = "'        '"
    qexpmode  = "'        '"
    qres      = "'        '"
    qdp_mode  = "'        '"
    qdp_rate  = "'        '"

    strput, qtime    , time   , 1
    strput, qdate    , date   , 1
    strput, qfilta   , filta  , 1
    strput, qfiltb   , filtb  , 1
    strput, qexpmode , expmode, 1
    strput, qres     , res    , 1
    strput, qdp_mode , dp_mode, 1
    strput, qdp_rate , dp_rate, 1

    siz    = size (data0)
    naxis1 = siz (1)
    naxis2 = siz (2)
    fits_head = strarr (35)

    fits_head (0)  = 'SIMPLE  = ' + string ('T', format="(a20, ' /')")
    fits_head (1)  = 'BITPIX  = ' + string (nbits, format="(i20, ' /')")
    fits_head (2)  = 'NAXIS   = ' + string (2, format="(i20, ' /')")
    fits_head (3)  = 'NAXIS1  = ' + string (naxis1, format="(i20, ' /')")
    fits_head (4)  = 'NAXIS2  = ' + string (naxis2, format="(i20, ' /')")
    fits_head (5)  = '  '
    fits_head (6)  = 'TIME-OBS= ' + string (qtime, format="(a10, 10x, ' /')")
    fits_head (7)  = 'DATE-OBS= ' + string (qdate, format="(a10, 10x, ' /')")
    fits_head (8)  = '  '
    fits_head (9)  = 'FILTER-A= ' + string (qfilta, format="(a10, 10x, ' / 1=Open , 2=NaBan, 3=Quart')")
    fits_head (10) = '          ' + string (' ', format="(a20, ' / 4=Diffu, 5=WdBan, 6=NuDen')")
    fits_head (11) = 'FILTER-B= ' + string (qfiltb, format="(a10, 10x, ' / 1=Open , 2=Al.1 , 3=AlMg ')")
    fits_head (12) = '          ' + string (' ', format="(a20, ' / 4=Be119, 5=Al12 , 6=Mg3  ')")
    fits_head (13) = 'RESOLUT = ' + string (qres, format="(a10, 10x, ' / Full=2.46 arcsec pixel, half=4.92, qrtr=9.84')")
    fits_head (14) = 'EXP-TYPE= ' + string (qexpmode, format="(a10, 10x, ' / Normal, dark, calibration')")
    fits_head (15) = 'EXP-DUR = ' + string (gt_expdur(index0), format="(f20.3, ' / Exposure duration in milliseconds')")
    fits_head (16) = 'EXP-LEV = ' + string (gt_dpe(index0), format="(i20, ' / DPE (commanded exposure level 0 to 32)')")
    fits_head (17) = '
    fits_head (18) = 'DP-MODE = ' + string (qdp_mode, format="(a10, 10x, ' / ')")
    fits_head (19) = 'DP-RATE = ' + string (qdp_rate, format="(a10, 10x, ' / ')")
    fits_head (20) = '
    i = 20
    if (keyword_set (suncen)) then begin
    i = i + 1
    fits_head (i) = 'CRPIX1  = ' + string (suncen (0), format="(f20.3, ' / X SUN CENTER (PIXELS).')")
    i = i + 1
    fits_head (i) = 'CRPIX2  = ' + string (suncen (1), format="(f20.3, ' / Y SUN CENTER (PIXELS).')")
    if (keyword_set (rsun)) then begin
    i = i + 1
    fits_head (i) = 'CRRADIUS= ' + string (rsun, format="(f20.3, ' / PIXELS PER SOLAR RADIUS')")
    end
    end
    ;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
    i = i + 1
    fits_head (i) = ' '
    i = i + 1
    if (keyword_set (comment)) then begin
    i  = i + 1
    fits_head (i) = 'COMMENT   ' + comment
    end

    print, 'SDA2FITS: Now saving ', outfil0 (ifil)
    wrt_fits, concat_dir (outdir, outfil0 (ifil)), fits_head , data0
end
;
end
