pro sxt2mk3, index, data, outfil=outfil, infil=infil, ss=ss, $
             outdir=outdir, qstop=qstop, outres=outres
;
;+
;NAME:
;	sxt2mk3
;PURPOSE:
;	Read a SXT index and data.
;	Resize/position to match MLSO Mark-III image format.
;       Write output image(s) to disk as Sun Rasterfile(s).
;	The output filename syntax is "sfimYYMMDD.HHMM.sun"
;CALLING SEQUENCE:
;	sxt2mk3, index, data
;	sxt2mk3, index, data, outfil='sxt_img.sfi'
;	sxt2mk3, infil=infil
;	sxt2mk3, infil=infil, ss=ss
;	sxt2mk3, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd0/stanger/sfi'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	outfil	- The output file name(s).  There should be one filename
;		  per image saved.  If "outfil" is not specified, then
;		  the output file name is:
;			SFIyymmdd.hhmmss	(for FFI)
;			SPIyymmdd.hhmmss	(for PFI)
;		  If "outfil" is passed as a scalar and there are
;		  multiple images passed in, then a .0001, .0002, .0003
;		  and so forth is appended to the end.
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the FITS file created
;	outres  - Output resolution; 0=full, 1=half, 2=quarter
;		  Default is to leave resolution unchanged.
;OUTPUT:
;	Creates SFI files
;
;PROCEDURES CALLED:
; 	rd_xda - Read SXT image.
;	anytim2ints - date/time conversion
;	ex2fid - create date/time string: YYMMDD.HHMM
;	index - get SXT index
;	gt_res - get SXT resolution
;	mod_res - modify resolution
;	mk3_match - resize image to match MLSO Mark-III geometry.
;	write_srf - write sun rasterfile to disk.
;
HISTORY:
;	Written  23-Mar-95 by Andrew L. Stanger (ALS), HAO/NCAR
;       Adapted from SXT2SFI
;-
 
!P.font = 0					; Use X-window hardware font
device, $
   font='-misc-fixed-medium-r-normal--20-200-75-75-c-100-iso8859-1'

window, colors = 256, xsize = 512, ysize = 512

loadct, 0					; Load greyscale colormap.

if (n_elements (outdir) eq 0) then outdir = ''

if (keyword_set (infil)) then begin

    qinfil = 1
    rd_roadmap, infil, roadmap
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)

end else begin

    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'SXT2MK3: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)

end

noutfil = n_elements (ss)

if (keyword_set (outfil)) then begin

    outfil0 = outfil
    if (n_elements (outfil) ne noutfil) then $
       outfil0 = outfil (0) + '.' + string (indgen (noutfil), format='(i4.4)')

end else begin						; Create output filename

    prefix_arr = ['spim', 'sfim']			; partial or full frame
    ii = gt_pfi_ffi (input1) mod 2			; b0 = 0 is PFI
    prefix = prefix_arr (ii)				; sp_fits or sf_fits
    tims = anytim2ints (input1)				; date/time conversion
    out_fids = ex2fid ( anytim2ex (tims))		; make 'YYMMDD.HHMM'
    outfil0 = prefix + out_fids + '.sun'

end

if (keyword_set (qstop)) then stop

; --- File Loop ---

for ifil = 0, noutfil - 1 do begin

    if (qinfil) then begin
	rd_xda, infil, ss (ifil), index0, data0, roadmap	; Read image
    end else begin
	index0 = index (ss (ifil))
	data0 = data (*, *, ss (ifil))	; TODO - what about images that
					; have been padded during the read?
    end

   if (keyword_set (outres)) then begin
      res_in = gt_res (index0)		; image resolution (0=full,1=half,2=qtr)
      res_out = outres			; 0=full, 1=half, 2=quarter resolution

       if (res_in ne res_out) then begin
          data0  = mod_res (index0, data0, index_out, outres=res_out, $
			    /update_index, /override, /norm_fact)
       endif

   endif

   ; --- Resize & position SXT image to match MLSO MK3 image resolution.

   data0 (*,0:4) = 0				; Set bottom 5 rows to black.
   data0 (0:1,*) = 0				; Set left two columns to black.
   img = mk3_match (data0, index0)

   print, 'SXT2MK3: Writing file ', outfil0 (ifil)

   ; --- Image annotation.

   sxt_time = string (gt_time (index0, /str))
   sxt_date = string (gt_day  (index0, /str))
   sxt_name = 'Yohkoh SXT'
   nsxt_name = strlen (sxt_name)

   ; --- Display SXT image, write annotation, then read back to array.

   imgsiz = size (img)
   sxt_nx = imgsiz (1)
   sxt_ny = imgsiz (2)
   tv, img
   xyouts, 400, 490, sxt_name, /device
   xyouts, 400, 470, sxt_date, /device
   xyouts, 400, 450, sxt_time, /device
   img = tvrd (0, 0, sxt_nx, sxt_ny)
;   img (*,0:4) = 0				; Set bottom 5 rows to black.
;   img (0:1,*) = 0				; Set left two columns to black.

   ; --- Write to disk as a Sun rasterfile.

   imgrev = reverse (img, 2)				; Flip top to bottom.
   write_srf, outfil0 (ifil), imgrev

endfor

end
