pro sxt2srf, index, data, outfil=outfil, infil=infil, ss=ss, $
	     outdir=outdir, outres=outres, qstop=qstop, cmname=cmname
;
;+
;NAME:
;	sxt2srf
;PURPOSE:
;	Take a SXT index and data and write a Sun Rasterfile(s).
;CALLING SEQUENCE:
;	sxt2srf, index, data
;	sxt2srf, index, data, outfil='sda_sample.srf'
;	sxt2srf, infil=infil
;	sxt2srf, infil=infil, ss=ss
;	sxt2srf, infil='$DIR_SXT_SFM/sfm930300.0000', outdir='/yd3/morrison/sgd'
;INPUT:
;	index	- The index structure to be saved
;	data	- The data array to be saved
;OPTIONAL KEYWORD INPUT:
;	outfil	- The output file name(s).  There should be one filename
;		  per image saved.  If "outfil" is not specified, then
;		  the output file name is:
;			SFRyymmdd.hhmmss.sun	(for FFI)
;			SPRyymmdd.hhmmss.sun	(for FFI)
;		  If "outfil" is passed as a scalar and there are
;		  multiple images passed in, then a .0001, .0002, .0003
;		  and so forth is appended to the end.
;	infil	- Instead of passing in the "index" and "data", it
;		  is possible to pass in the SDA file name.  If 
;		  "ss" is not passed, then all data sets are saved
;	ss	- A list of the indicies to be saved (either within
;		  the "index" variable, or datasets within the
;		  "infil"
;	outdir	- The output directory where to put the SRF file is created
;	outres  - Output resolution; 0=full, 1=half, 2=quarter
;		  Default is to leave resolution unchanged.
;	cmname  - Name of ASCII file containing colormap [index red green blue].
;OUTPUT:
;	Creates one Sun rasterfile for each SXT image.
;HISTORY:
;	15-Sep-1995 Andrew L. Stanger (ALS), HAO/NCAR
;	            Adapted from 'sxt2fts.pro'.
;-

if (not keyword_set (cmname)) then	$
   cmname = '/riker/usr/people/stanger/color/blue_white.3.lut'

lct, cmname					; load color table.

; --- Storage for color LUT.

red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)

tvlct, red, green, blue, /get

if (n_elements (outdir) eq 0) then outdir = ''

; --- Input: file or array ?

if (keyword_set (infil)) then begin
    qinfil = 1
    rd_roadmap, infil, roadmap
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (roadmap))
    input1 = roadmap (ss)
end else begin
    qinfil = 0
    if (n_elements (index) eq 0) then begin
	print, 'SXT2SRF: No input file specified and no index passed
	print, 'Returning...
	return
    end
    if (n_elements (ss) eq 0) then ss = indgen (n_elements (index))
    input1 = index (ss)
end

noutfil = n_elements (ss)
 
; --- Create output file name(s).

suffix = '.sun'

if (keyword_set (outfil)) then begin
    outfil0 = outfil
    if (n_elements (outfil) ne noutfil) then $
       outfil0 = outfil (0) + '.' + string (indgen (noutfil), format='(i4.4)') $
	       + suffix
end else begin
    prefix_arr = ['spr', 'sfr']
    ii = gt_pfi_ffi (input1) mod 2	;b0 = 0 is PFI
    prefix   = prefix_arr (ii)
    tims     = anytim2ints (input1)
    out_fids = ex2fid ( anytim2ex (tims), /sec )
    outfil0  = prefix + out_fids + suffix
end
 
if (keyword_set (qstop)) then stop
 
; --- Image loop.

for ifil = 0, noutfil - 1 do begin
   if (qinfil) then begin
      rd_xda, infil, ss (ifil), index0, data0, roadmap	; Read image
   end else begin
      index0 = index (ss (ifil))
      data0  = data (*, *, ss (ifil))	; TODO - what about images
					; that have been padded during the read?
   end

   ; --- Adjust image to normalize resolution as specified (512x512 default).

   if (not keyword_set (outres)) then outres = 1
   res_in = gt_res (index0)		; image resolution (0=full,1=half,2=qtr)
   res_out = outres			; 0=full, 1=half, 2=quarter resolution

   if (res_in ne res_out) then begin
      data0 = mod_res (index0, data0, index_out, outres=res_out, $
                        /update_index, /override, /norm_fact)
   endif

   nbits  = get_nbytes (data0 (0,0))*8

   ; --- Get information from index.

   time   = gt_time (index0, /str)
;    date   = gt_date (index0, /leadzero, /str)
    date = gt_day (index0, /str, /fits)

    filta   = gt_filta (index0, /str)
    filtb   = gt_filtb (index0, /str)
    expmode = gt_expmode (index0, /str)
    res     = gt_res (index0, /str)
    dp_mode = gt_dp_mode (index0, /str)
    dp_rate = gt_dp_rate (index0, /str)
;    print, '-',date,'-'
;    print, '-',time,'-'
;    print, '-',expmode,'-'
;    print, '-',res,'-'
;    print, '-',dp_mode,'-'
;    print, '-',dp_rate,'-'

    ; --- Add single quote mark to begin/end of parameter strings.

    qtime     = "'        '"
    qdate     = "'        '"
    qfilta    = "'        '"
    qfiltb    = "'        '"
    qexpmode  = "'        '"
    qres      = "'        '"
    qdp_mode  = "'        '"
    qdp_rate  = "'        '"

    strput, qtime    , time   , 1
    strput, qdate    , date   , 1
    strput, qfilta   , filta  , 1
    strput, qfiltb   , filtb  , 1
    strput, qexpmode , expmode, 1
    strput, qres     , res    , 1
    strput, qdp_mode , dp_mode, 1
    strput, qdp_rate , dp_rate, 1

    siz    = size (data0)
    naxis1 = siz (1)
    naxis2 = siz (2)

    print, 'sxt2srf: Now saving ', outfil0 (ifil)
    write_srf, concat_dir (outdir, outfil0 (ifil)), data0, red, green, blue
    print, 'writing file: ', outfil0 (ifil), ' to directory: ', outdir
endfor

end
