pro tif2fts, tif_file, date=date, time=time, $
	     origin=origin, object=object, type_obs=type_obs, $
	     telescop=telescop, instrume=instrume, comment=comment

;+
; NAME
;	tif2fts
;
; PURPOSE
;	Convert a TIFF file to a FITS file
;
; CALLING SEQUENCE
;	tif2fts, 'spo.15dec93.1641.tiff', date='15/12/93', time='16:41:00', $
;                origin='SPO', object='SUN', type_obs='H-ALPHA'
;
; PARAMETERS
;	date		string format: 'DD/MM/YY'
;	time		string format: 'HH:MM:SS'
;	origin		Institute or Observatory name
;	object		Name of object observed
;	type_obs	Type of observation, i.e., H-ALPHA
;	telescop	Telescope name
;	instrume	Instrument name
;	comment		Remark relating to image
;
; HISTORY
;	written 16 February 1995, ALS (Andrew L. Stanger) HAO/NCAR
;-

if (n_params () eq 0) then $
   begin
   print, 'tif2fts: convert TIFF file to FITS file.'
   print, 'usage: tif2fts, tif_file, date=date, time=time, origin=origin,'
   print, '                object=object, type_obs=type_obs, telescop=telescop,'
   print, '                instrume=instrume, comment=comment'
   print, ' '
   print, 'tif_file        TIFF image file'
   print, "date            'dd/mm/yy'"
   print, "time            'hh:mm:ss'"
   print, 'origin          name of institute or observatory'
   print, 'object          name of object observed'
   print, 'type_obs        type of observation, i.e., H-alpha'
   print, 'telescop        name of telescope'
   print, 'instrume        name of instrument'
   print, 'comment         remark relating to image'
   return
   end

; --- Read TIFF image into IDL array.

img = tiff_read (tif_file, red, green, blue)
img = reverse (img, 2)			; flip image top to bottom for FITS.

; --- Make standard FITS header.

fxhmake, fits_hdr, img

; --- Add additional keywords & parameters.

if (n_elements (date) ne 0) then $
   begin
      fxaddpar, fits_hdr, 'DATE-OBS', date
   end

if (n_elements (time) ne 0) then $
   begin
      fxaddpar, fits_hdr, 'TIME-OBS', time
   end

if (n_elements (origin) ne 0) then $
   begin
      origin = strupcase (origin)
      fxaddpar, fits_hdr, 'ORIGIN', origin
   end

if (n_elements (object) ne 0) then $
   begin
      object = strupcase (object)
      fxaddpar, fits_hdr, 'OBJECT', object
   end

if (n_elements (type_obs) ne 0) then $
   begin
      type_obs = strupcase (type_obs)
      fxaddpar, fits_hdr, 'TYPE-OBS', type_obs
   end

if (n_elements (telescop) ne 0) then $
   begin
      telescop = strupcase (telescop)
      fxaddpar, fits_hdr, 'TELESCOP', telescop
   end

if (n_elements (instrume) ne 0) then $
   begin
      instrume = strupcase (instrume)
      fxaddpar, fits_hdr, 'INSTRUME', instrume
   end

if (n_elements (comment) ne 0) then $
   begin
      comment = strupcase (comment)
      comment = ' ' + comment
      fxaddpar, fits_hdr, 'COMMENT', comment
   end

print, 'FITS HEADER: '
print, fits_hdr

; --- Create FITS file name.

fts_file = tif_file

extpos = strpos (tif_file, 'tif')
print, 'extpos: ', extpos

if (extpos ge 0) then $
   begin
      strput, fts_file, 'fts ', extpos
      fts_file = strtrim (fts_file)
   end	$
else	$
   begin
      fts_file = tif_file + '.fts'
   end

; --- Write FITS file to disk.

print, 'fts_file: ', fts_file

writefits, fts_file, img, fits_hdr

end
