;+
; IDL PROCEDURE
;
; NAME:
;	xanfts
;
; PURPOSE:
;	Animate a sequence of FITS images.
;
; CALLING SEQUENCE:
;	xanfts, 'fits.ls'
;	xanfts, 'fits.ls', rate=1
;	xanfts, 'fits.ls', rate=1, /label
;	xanfts, 'fits.ls', rate=1, /label, cm='~/color/sunset.lut'
;
; INPUTS:
;	listfile : character string containing the names of the FITS
;		   files to be animated.
;	rate : approximate animation rate (frames/second).
;	label: if present, label each image with frame #, date, time.
;	cm: name of color map file (ASCII, each line: index, red, green, blue).
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	READFITS
;	XANIMATE
;	LCT
;
; MODIFICATION HISTORY:
;	created 3 August 1995, Andrew L. Stanger, HAO/NCAR
;-

pro xanfts, listfile, rate=rate, label=label, cm=cm

if (not keyword_set (rate)) then rate = 2.0 

filename = ' '
idate    = ' '
itime    = ' '
ii = 0

; --- Open list file, read filenames, count the number of files in list.

close, 10
openr, 10, listfile

while (not eof (10))  do begin	;{
   readf, 10, filename
   ii = ii + 1
endwhile			;}
numimg = ii

close, 10

;--- Get size of last FITS image in list.

imgbuf = readfits (filename, hdu)	; Read first FITS image in listfile.

nbitpx = fxpar (hdu, 'BITPIX',   count=count)
naxis  = fxpar (hdu, 'NAXIS',    count=count)
nxpix  = fxpar (hdu, 'NAXIS1',   count=count)
nypix  = fxpar (hdu, 'NAXIS2',   count=count)
idate  = fxpar (hdu, 'DATE-OBS', count=count)
if (count eq 0)  then idate = ' '
itime  = fxpar (hdu, 'TIME-OBS', count=count)
if (count eq 0)  then itime = ' '

;if (nbitpx eq 8)       then imgbuf = bytarr (nxpix, nypix) $
;else if (nbitpx eq 16) then imgbuf = intarr (nxpix, nypix) $
;else if (nbitpx eq 32) then imgbuf = fltarr (nxpix, nypix) $
;else exit

;--- If label keyword set, create IDL window in which to display each image.

if (keyword_set (label))  then	$
begin
   window, 20, xsize = nxpix, ysize = nypix, color=256, /pixmap
   wset, 20
end

;--- Load color table.

if (keyword_set (cm))  then lct, cm	$	; Load specified colormap.
else			    loadct, 0		; Load greyscale colormap.
 
;--- Initialize 'xanimate'.

init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 

if ( init eq 'y') then begin ;{ 
   XANIMATE, SET = [nxpix, nypix, numimg, 0] 
 
   close, 10 & openr, 10, listfile	; Open file containing list of images.
 
   ii = 0 
   while ( not eof (10) ) do begin ;{
      readf, 10, filename

      imgbuf = readfits (filename, hdu)		; Read FITS image in listfile.
      if (nbitpx eq   8)  then imgbuf = byte ( imgbuf) else $
                               imgbuf = bytscl (imgbuf)		; Scale data.

      nbitpx = fxpar (hdu, 'BITPIX')
      naxis  = fxpar (hdu, 'NAXIS')
      nxpix  = fxpar (hdu, 'NAXIS1')
      nypix  = fxpar (hdu, 'NAXIS2')

      if (keyword_set (label))  then $
      begin
         idate  = fxpar (hdu, 'DATE-OBS', count=count)
         sizdate = size (idate)
         if (count eq 0 or sizdate (1) lt 8)  then idate = filename

         itime  = fxpar (hdu, 'TIME-OBS', count=count)
         siztime = size (itime)
         if (count eq 0 or siztime (1) lt 8)  then itime = ''

         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgbuf
         imglabel = sframe + '     ' + idate + '   ' + itime
         xyouts, xpos, ypos, imglabel, /device
         imgfrm = tvrd ()
      end				$
      else				$
	 imgfrm = imgbuf

       XANIMATE, IMAGE = imgfrm, FRAME = ii

      ii = ii+1
   endwhile ;} 
 
   close, 10

endif ;}

wdelete, 20					; Delete temp window.

;--- Activate animation.

XANIMATE, rate 

; --- When you want to stop animating,  hit ^C. 
; --- You might then have to do a "retall" in idl.
 
end 
