;+
; IDL PROCEDURE
;
; NAME:
;	xansrf
;
; PURPOSE:
;	Animate a sequence of SUN rasterfile images.
;
; CALLING SEQUENCE:
;	xansrf, 'fits.ls'
;	xansrf, 'fits.ls', rate=1
;	xansrf, 'fits.ls', rate=1, /label
;	xansrf, 'fits.ls', rate=1, /label, cm='~/color/sunset.lut'
;
; INPUTS:
;	listfile : character string containing the names of the SUN rasterfile
;		   images to be animated.
;	rate : approximate animation rate (frames/second).
;	label: if present, label each image with frame #, date, time.
;	cm: name of color map file (ASCII, each line: index, red, green, blue).
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	READ_SRF
;	XANIMATE
;	LCT
;
; MODIFICATION HISTORY:
;	created 4 August 1995, Andrew L. Stanger, HAO/NCAR
;-

pro xansrf, listfile, rate=rate, label=label, cm=cm

if (not keyword_set (rate)) then rate = 2.0 

filename = ' '
idate    = ' '
itime    = ' '
ii = 0
;red   = bytarr (256)
;green = bytarr (256)
;blue  = bytarr (256)

; --- Open list file, read filenames, count the number of files in list.

close, 10
openr, 10, listfile

while (not eof (10))  do begin	;{
   readf, 10, filename
   ii = ii + 1
endwhile			;}
numimg = ii

close, 10

;--- Get size of last SUN rasterfile image in list.

read_srf, filename, imgbuf, red, green, blue	; Read last image in listfile.

sizred = size (red)
; print, 'size (red): ', size (red)

sizimg = size (imgbuf)

print, 'size (imgbuf): ', sizimg

if (sizimg (0) ne 2) then	$
begin
   print, '# dimensions: ', sizimg (0)
   return
end

nxpix  = sizimg (1)
nypix  = sizimg (2)
nbypix = sizimg (3)

;--- If label keyword set, create IDL window in which to display each image.

if (keyword_set (label))  then	$
begin
   window, 20, xsize = nxpix, ysize = nypix, color=256, /pixmap
   wset, 20
end

;--- Load color table.

if (keyword_set (cm))  then lct, cm	$	; Load specified colormap.
else if (sizred (0) ne 0) then tvlct, red, green, blue	$
else			    loadct, 0		; Load greyscale colormap.
 
;--- Initialize 'xanimate'.

init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 

if ( init eq 'y') then begin ;{ 
   XANIMATE, SET = [nxpix, nypix, numimg, 0] 
 
   close, 10 & openr, 10, listfile	; Open file containing list of images.
 
   ii = 0 
   while ( not eof (10) ) do begin ;{
      readf, 10, filename

      read_srf, filename, imgbuf, red, green, blue	; Read image from list.

      imgbuf = reverse (imgbuf, 2)		; Flip image top-to-bottom.
      sizimg = size (imgbuf)
      nxpix  = sizimg (1)
      nypix  = sizimg (2)
      nbypix = sizimg (3)

      if (nbypix eq  1)  then imgbuf = byte ( imgbuf) else $
                              imgbuf = bytscl (imgbuf)		; Scale data.

      if (keyword_set (label))  then $
      begin
         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgbuf
         imglabel = sframe + '     ' + filename
	 print, 'imglabel: ', imglabel
         xyouts, xpos, ypos, imglabel, /device
         imgfrm = tvrd ()
      end				$
      else				$
	 imgfrm = imgbuf

       XANIMATE, IMAGE = imgfrm, FRAME = ii		; Load image

      ii = ii+1
   endwhile ;} 
 
   close, 10

endif ;}

wdelete, 20

;--- Activate animation.

XANIMATE, rate 

; --- When you want to stop animating,  hit ^C. 
; --- You might then have to do a "retall" in idl.
 
end 
