;+
; IDL PROCEDURE
;
; NAME:
;	xinchip
;
; PURPOSE:
;	Animate a sequence of MLSO/CHIP binary images.
;
; CALLING SEQUENCE:
;	xinchip, 'chip.ls'
;	xinchip, 'chip.ls', pixtype=pixtype
;	xinchip, 'chip.ls', rate=25
;	xinchip, 'chip.ls', rate=8, /label
;	xinchip, 'chip.ls', rate=2, /label, cm='~/color/sunset.lut'
;
; INPUTS:
;	listfile : character string containing the names of the SUN rasterfile
;		   images to be animated.
;	rate : animation rate (range: 0-100).
;	label: if present, label each image with frame #, date, time.
;	cm: name of color map file (ASCII, each line: index, red, green, blue).
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	XINTERANIMATE
;	LCT
;
; MODIFICATION HISTORY:
;	created 9 August 1995, Andrew L. Stanger, HAO/NCAR
;-

pro xinchip, listfile, pixtype=pixtype, rate=rate, label=label, cm=cm

if (not keyword_set (rate)) then rate = 4

filename = ' '
ii = 0
winid = 20

; --- Open list file, read filenames, count the number of files in list.

get_lun, lun
openr, lun, listfile

while (not eof (lun))  do begin	;{
   readf, lun, filename
   ii = ii + 1
endwhile			;}
numimg = ii

close, lun
free_lun, lun

;--- Set size of CHIP images.

nxpix = 767
nypix = 516
nbypix = 2				; default: 16-bit integer.
if (keyword_set (pixtype))  then	$
begin
   if (pixtype eq 'byt')  then nbypix = 1
   if (pixtype eq 'int')  then nbypix = 2
   if (pixtype eq 'flt')  then nbypix = 4
end

if (nbypix eq 1)  then	$
   begin
   chipbuf = bytarr (nxpix, nypix)
   chipmin =   0
   chipmax = 255
   end
if (nbypix eq 2)  then	$
   begin
   chipbuf = intarr (nxpix, nypix)
   chipmin =  9000
   chipmax = 10500
   end
if (nbypix eq 4)  then	$
   begin
   chipbuf = fltarr (nxpix, nypix)
   chipmin = -0.19				; 9000
   chipmax = -0.05				; 10500
   end

;--- If label keyword set, create IDL window in which to display each image.

if (keyword_set (label))  then	$
begin
   window, winid, xsize = nxpix, ysize = nypix, color=256, /pixmap
   wset, winid
end

;--- Load color table.

;print, 'size (red): ', size (red)
sizred = size (red)
if (keyword_set (cm))  then lct, cm	$	; Load specified colormap.
else if (sizred (0) ne 0) then tvlct, red, green, blue	$
else			    loadct, 1		; Load blue-white colormap.
 
;--- Initialize 'xinteranimate'.

init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 

if ( init eq 'y') then begin ;{ 
   XINTERANIMATE, SET = [nxpix+4, nypix+4, numimg], /SHOWLOAD
 
    
   get_lun, lun
   openr, lun, listfile			; Open file containing list of images.
 
   ii = 0 
   while ( not eof (lun) ) do begin ;{
      readf, lun, filename

      print, 'Read file: ', filename

      chipbuf = read_chip (filename, nxpix, nypix, nbypix)	; Read image.
      cmin = min (chipbuf, max=cmax)
      print, filename, '   min/max: ', cmin, cmax

      imgbuf = bytscl (chipbuf, min=chipmin, max=chipmax)	; Scale image.

      if (keyword_set (label))  then $
      begin
         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgbuf
         imglabel = sframe + '     ' + filename
	 print, 'imglabel: ', imglabel
         xyouts, xpos, ypos, imglabel, /device
         XINTERANIMATE, FRAME = ii, WINDOW = !d.window		; Load image
      end				$
      else				$
	 XINTERANIMATE, IMAGE = imgbuf, FRAME = ii		; Load image

      ii = ii+1
   endwhile ;} 
 
   close, lun
   free_lun, lun

endif ;}

wdelete, winid					; Delete temp window.

;--- Activate animation.

XINTERANIMATE, rate 

end 
