;+
; NAME	brtpix
;
; PURPOSE	Find the brightest pixels (top brtpct%) in an image.
;
; SYNTAX	pixloc = brtpix (img, pixrs)
;
; INPUTS	imgbuf	image array
;		pixrs	# pixels / Rsun
;		brtpct	% brightness level
;
; OUTPUTS	ixbeg, ixend	start/stop x-coordinates of equatorial band.
;		iybeg, iyend	start/stop y-coordinates of equatorial band.
;		pixloc		coordinate locations of brtpct brightest pixels.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   6 June 2000
;-

FUNCTION brtpix, imgbuf, pixrs, brtpct, ixbeg, ixend, iybeg, iyend

sizeimg = size (imgbuf)

;PRINT, 'brtpix> size (imgbuf): ', size (imgbuf)

xdim = sizeimg [1]
ydim = sizeimg [2]
npximg = xdim * ydim

xcen = (npximg / 2 - 0.5)
ycen = (npximg / 2 - 0.5)

ixcen = xdim / 2
iycen = ydim / 2

;--- Extract a "band" (very wide box) across the equator of the disk.

ixbeg = ixcen - fix (pixrs * 0.9)
ixend = ixcen + fix (pixrs * 0.9)
iybeg = iycen - fix (pixrs * 0.1)
iyend = iycen + fix (pixrs * 0.1)
ixdif = ixend - ixbeg + 1
iydif = iyend - iybeg + 1

xdimband = long (ixend - ixbeg + 1)
ydimband = long (iyend - iybeg + 1)
npxband  = xdimband * ydimband

;PRINT, 'ixbeg/ixend/ixdif: ', ixbeg, ixend, ixdif
;PRINT, 'iybeg/iyend/iydif: ', iybeg, iyend, iydif
;PRINT, 'xdimband/ydimband: ', xdimband, ydimband
;PRINT, 'npxband: ', npxband

band = FLTARR (xdimband, ydimband)

band = imgbuf [ixbeg:ixend, iybeg:iyend]

bandsort = band (SORT (band))        ; order w.r.t. increasing brightness.

;--- Compute the "brtpct % median":
;    the level at which 20% of the pixels lie above the computed brightness.

IF (KEYWORD_SET (brtpct)) THEN		$
    frac = 1.0 - (brtpct / 100.0)	$
ELSE					$
    frac = 0.8

sfrac = strtrim (string (fix ((1.0 - frac) * 100.00001), FORMAT='(I4)'), 2)

nbrtval = long (frac * npxband)	; Fraction of brightest pixels to sample
ibrtval = npxband - nbrtval
brtval  = bandsort (ibrtval)		; Brightness level threshold.
brtloc  = WHERE (band GE brtval)	; Indices corresponding to bright pixels
brtlocsize = SIZE (brtloc)
IF (brtlocsize [0] GT 0) THEN		$
    nloc = long (brtlocsize [1])	$
ELSE					$
    nloc = 0L

;PRINT, 'nbrtval: ', nbrtval
;PRINT, 'ibrtval: ', ibrtval
;PRINT, 'brtval:  ', brtval

RETURN, brtloc
END
