;+
; NAME		chip_doppler
;
; PURPOSE	Determine the doppler shift for chip filter positions.
;
; SYNTAX	chip_doppler, fits_list
;
;		fits_list	File containing a list of 7 images, spanning
;				the wavelength range 1082.745 - 1083.315 nm.
;				Images are required to be in FITS format.
;
; METHOD	Read FITS images.
;		Remove limb darkening.
;		Rotate image (using p-angle) to convert to heliocentric coords.
;		Find the brightest 20% of the pixels in an equatorial band.
;		Use these brightest pixels and average them in each subset
;		box within the band.  Assign the average value as the
;		representative value for each box, then fit a curve of
;		apparent "longitude" versus intensity.  This profile may
;		then be used to remove the doppler variation across the
;		solar disk.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   22 May 2000
;-

PRO chip_doppler, fits_list

file1 = ''
file2 = ''
file3 = ''
file4 = ''
file5 = ''
file6 = ''
file7 = ''

print, 'fits_list: ', fits_list

;--- Read the FITS file names from the input list file.

CLOSE,  10

imgld = fltarr (936, 936)
OPENR, 10, 'chip.ldcor.img'		; limb darkening image file.
READU, 10, imgld
CLOSE, 10

OPENR,  10, fits_list
READF,  10, file1
READF,  10, file2
READF,  10, file3
READF,  10, file4
READF,  10, file5
READF,  10, file6
READF,  10, file7
CLOSE, 10

print, 'file1: ', file1
print, 'file2: ', file2
print, 'file3: ', file3
print, 'file4: ', file4
print, 'file5: ', file5
print, 'file6: ', file6
print, 'file7: ', file7

;--- Read each of the images into memory.

img1 = readfits (file1, hdu1, /noscale)
img2 = readfits (file2, hdu2, /noscale)
img3 = readfits (file3, hdu3, /noscale)
img4 = readfits (file4, hdu4, /noscale)
img5 = readfits (file5, hdu5, /noscale)
img6 = readfits (file6, hdu6, /noscale)
img7 = readfits (file7, hdu7, /noscale)

;--- Overwrite the first line (bottom), which contains the binary header.

img1 [*, 0] = img1 [*, 1]
img2 [*, 0] = img2 [*, 1]
img3 [*, 0] = img3 [*, 1]
img4 [*, 0] = img4 [*, 1]
img5 [*, 0] = img5 [*, 1]
img6 [*, 0] = img6 [*, 1]
img7 [*, 0] = img7 [*, 1]

;--- Get information from FITS header.

pixrs  = fxpar (hdu1, 'CRRADIUS')	; # Pixels / solar radius 
rsun   = fxpar (hdu1, 'RSUN')		; Solar Radius (arcsec)
xdim   = fxpar (hdu1, 'NAXIS1')
ydim   = fxpar (hdu1, 'NAXIS2')
bitpix = fxpar (hdu1, 'BITPIX')
nbypix = bitpix / 8

ipixrs = fix (pixrs + 0.5)
border = 50
xdimc = (ipixrs + border) * 2
ydimc = xdimc

print, 'xdim/ydim:', xdim, ydim
print, 'nbypix: ', nbypix
print, 'rsun: ',  rsun
print, 'pixrs: ', pixrs
print, 'xdimc/ydimc: ', xdimc, ydimc

;--- Create storage for cropped images.

img   = fltarr (xdimc, ydimc)
img1c = fltarr (xdimc, ydimc)
img2c = fltarr (xdimc, ydimc)
img3c = fltarr (xdimc, ydimc)
img4c = fltarr (xdimc, ydimc)
img5c = fltarr (xdimc, ydimc)
img6c = fltarr (xdimc, ydimc)
img7c = fltarr (xdimc, ydimc)

;--- Get the disk center locations.

xc1 = fxpar (hdu1, 'CRPIX1')
yc1 = fxpar (hdu1, 'CRPIX2')
xc2 = fxpar (hdu2, 'CRPIX1')
yc2 = fxpar (hdu2, 'CRPIX2')
xc3 = fxpar (hdu3, 'CRPIX1')
yc3 = fxpar (hdu3, 'CRPIX2')
xc4 = fxpar (hdu4, 'CRPIX1')
yc4 = fxpar (hdu4, 'CRPIX2')
xc5 = fxpar (hdu5, 'CRPIX1')
yc5 = fxpar (hdu5, 'CRPIX2')
xc6 = fxpar (hdu6, 'CRPIX1')
yc6 = fxpar (hdu6, 'CRPIX2')
xc7 = fxpar (hdu7, 'CRPIX1')
yc7 = fxpar (hdu7, 'CRPIX2')

xc1 = xdim / 2.0 - 0.5
yc1 = ydim / 2.0 - 0.5
xc2 = xdim / 2.0 - 0.5
yc2 = ydim / 2.0 - 0.5
xc3 = xdim / 2.0 - 0.5
yc3 = ydim / 2.0 - 0.5
xc4 = xdim / 2.0 - 0.5
yc4 = ydim / 2.0 - 0.5
xc5 = xdim / 2.0 - 0.5
yc5 = ydim / 2.0 - 0.5
xc6 = xdim / 2.0 - 0.5
yc6 = ydim / 2.0 - 0.5
xc7 = xdim / 2.0 - 0.5
yc7 = ydim / 2.0 - 0.5

;--- Get date & time.
;--- Get WSR (wavelength/Source/Reduction) code. 

date_obs1 = fxpar (hdu1, 'DATE-OBS')
date_obs2 = fxpar (hdu2, 'DATE-OBS')
date_obs3 = fxpar (hdu3, 'DATE-OBS')
date_obs4 = fxpar (hdu4, 'DATE-OBS')
date_obs5 = fxpar (hdu5, 'DATE-OBS')
date_obs6 = fxpar (hdu6, 'DATE-OBS')
date_obs7 = fxpar (hdu7, 'DATE-OBS')

time_obs1 = fxpar (hdu1, 'TIME-OBS')
time_obs2 = fxpar (hdu2, 'TIME-OBS')
time_obs3 = fxpar (hdu3, 'TIME-OBS')
time_obs4 = fxpar (hdu4, 'TIME-OBS')
time_obs5 = fxpar (hdu5, 'TIME-OBS')
time_obs6 = fxpar (hdu6, 'TIME-OBS')
time_obs7 = fxpar (hdu7, 'TIME-OBS')

code_obs1 = strtrim (strlowcase (fxpar (hdu1, 'CODE-OBS')))
code_obs2 = strtrim (strlowcase (fxpar (hdu2, 'CODE-OBS')))
code_obs3 = strtrim (strlowcase (fxpar (hdu3, 'CODE-OBS')))
code_obs4 = strtrim (strlowcase (fxpar (hdu4, 'CODE-OBS')))
code_obs5 = strtrim (strlowcase (fxpar (hdu5, 'CODE-OBS')))
code_obs6 = strtrim (strlowcase (fxpar (hdu6, 'CODE-OBS')))
code_obs7 = strtrim (strlowcase (fxpar (hdu7, 'CODE-OBS')))

year1 = strmid (date_obs1, 0, 4)
year2 = strmid (date_obs2, 0, 4)
year3 = strmid (date_obs3, 0, 4)
year4 = strmid (date_obs4, 0, 4)
year5 = strmid (date_obs5, 0, 4)
year6 = strmid (date_obs6, 0, 4)
year7 = strmid (date_obs7, 0, 4)

month1 = strmid (date_obs1, 5, 2)
month2 = strmid (date_obs2, 5, 2)
month3 = strmid (date_obs3, 5, 2)
month4 = strmid (date_obs4, 5, 2)
month5 = strmid (date_obs5, 5, 2)
month6 = strmid (date_obs6, 5, 2)
month7 = strmid (date_obs7, 5, 2)

day1 = strmid (date_obs1, 8, 2)
day2 = strmid (date_obs2, 8, 2)
day3 = strmid (date_obs3, 8, 2)
day4 = strmid (date_obs4, 8, 2)
day5 = strmid (date_obs5, 8, 2)
day6 = strmid (date_obs6, 8, 2)
day7 = strmid (date_obs7, 8, 2)

hour1 = strmid (time_obs1, 0, 2)
hour2 = strmid (time_obs2, 0, 2)
hour3 = strmid (time_obs3, 0, 2)
hour4 = strmid (time_obs4, 0, 2)
hour5 = strmid (time_obs5, 0, 2)
hour6 = strmid (time_obs6, 0, 2)
hour7 = strmid (time_obs7, 0, 2)

minute1 = strmid (time_obs1, 3, 2)
minute2 = strmid (time_obs2, 3, 2)
minute3 = strmid (time_obs3, 3, 2)
minute4 = strmid (time_obs4, 3, 2)
minute5 = strmid (time_obs5, 3, 2)
minute6 = strmid (time_obs6, 3, 2)
minute7 = strmid (time_obs7, 3, 2)

second1 = strmid (time_obs1, 6, 2)
second2 = strmid (time_obs2, 6, 2)
second3 = strmid (time_obs3, 6, 2)
second4 = strmid (time_obs4, 6, 2)
second5 = strmid (time_obs5, 6, 2)
second6 = strmid (time_obs6, 6, 2)
second7 = strmid (time_obs7, 6, 2)

;--- Create filenames for output cropped images.

file1c = year1 + month1  + day1    + '_' +	$
	 hour1 + minute1 + second1 + '_' +	$
	 'chip_' + code_obs1 + '.sq.fts'

file2c = year2 + month2  + day2    + '_' +	$
	 hour2 + minute2 + second2 + '_' +	$
	 'chip_' + code_obs2 + '.sq.fts'

file3c = year3 + month3  + day3    + '_' +	$
	 hour3 + minute3 + second3 + '_' +	$
	 'chip_' + code_obs3 + '.sq.fts'

file4c = year4 + month4  + day4    + '_' +	$
	 hour4 + minute4 + second4 + '_' +	$
	 'chip_' + code_obs4 + '.sq.fts'

file5c = year5 + month5  + day5    + '_' +	$
	 hour5 + minute5 + second5 + '_' +	$
	 'chip_' + code_obs5 + '.sq.fts'

file6c = year6 + month6  + day6    + '_' +	$
	 hour6 + minute6 + second6 + '_' +	$
	 'chip_' + code_obs6 + '.sq.fts'

file7c = year7 + month7  + day7    + '_' +	$
	 hour7 + minute7 + second7 + '_' +	$
	 'chip_' + code_obs7 + '.sq.fts'

IF (xc1 LT 0.0 OR yc1 LT 0.0) THEN	$
   BEGIN
   gen_loc, img1, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file1: ', file1
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc1 = centroid [0]
   yc1 = centroid [1]
   END

;*******************************************************************************

x1beg = fix (xc1 + 0.5) - ipixrs - border
x1end = x1beg + (ipixrs + border) * 2 - 1
y1beg = yc1 - ipixrs - border
y1end = y1beg + (ipixrs + border) * 2 - 1

x1dif = x1end - x1beg + 1
print, 'x1beg/x1end: ', x1beg, x1end, ' x1dif: ', x1dif
;*******************************************************************************

IF (xc2 LT 0.0 OR yc2 LT 0.0) THEN	$
   BEGIN
   gen_loc, img2, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file2: ', file2
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc2 = centroid [0]
   yc2 = centroid [1]
   END

IF (xc3 LT 0.0 OR yc3 LT 0.0) THEN	$
   BEGIN
   gen_loc, img3, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file3: ', file3
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc3 = centroid [0]
   yc3 = centroid [1]
   END

IF (xc4 LT 0.0 OR yc4 LT 0.0) THEN	$
   BEGIN
   gen_loc, img4, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file4: ', file4
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc4 = centroid [0]
   yc4 = centroid [1]
   END

IF (xc5 LT 0.0 OR yc5 LT 0.0) THEN	$
   BEGIN
   gen_loc, img5, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file5: ', file5
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc5 = centroid [0]
   yc5 = centroid [1]
   END

IF (xc6 LT 0.0 OR yc6 LT 0.0) THEN	$
   BEGIN
   gen_loc, img6, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file6: ', file6
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc6 = centroid [0]
   yc6 = centroid [1]
   END

IF (xc7 LT 0.0 OR yc7 LT 0.0) THEN	$
   BEGIN
   gen_loc, img7, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   print, 'file7: ', file7
   print, 'hzrd/vtrd: ', hzrd, vtrd
   print, 'centroid: ', centroid
   xc7 = centroid [0]
   yc7 = centroid [1]
   END

;--- Crop each image to a square, centered at solar disk center.
;--- Also, convert to floating point.

x1beg = fix (xc1 + 0.5) - ipixrs - border
x1end = x1beg + (ipixrs + border) * 2 - 1
y1beg = fix (yc1 + 0.5) - ipixrs - border
y1end = y1beg + (ipixrs + border) * 2 - 1

x2beg = fix (xc2 + 0.5) - ipixrs - border
x2end = x2beg + (ipixrs + border) * 2 - 1
y2beg = fix (yc2 + 0.5) - ipixrs - border
y2end = y2beg + (ipixrs + border) * 2 - 1

x3beg = fix (xc1 + 0.5) - ipixrs - border
x3end = x3beg + (ipixrs + border) * 2 - 1
y3beg = fix (yc1 + 0.5) - ipixrs - border
y3end = y3beg + (ipixrs + border) * 2 - 1

x4beg = fix (xc4 + 0.5) - ipixrs - border
x4end = x4beg + (ipixrs + border) * 2 - 1
y4beg = fix (yc4 + 0.5) - ipixrs - border
y4end = y4beg + (ipixrs + border) * 2 - 1

x5beg = fix (xc5 + 0.5) - ipixrs - border
x5end = x5beg + (ipixrs + border) * 2 - 1
y5beg = fix (yc5 + 0.5) - ipixrs - border
y5end = y5beg + (ipixrs + border) * 2 - 1

x6beg = fix (xc6 + 0.5) - ipixrs - border
x6end = x6beg + (ipixrs + border) * 2 - 1
y6beg = fix (yc6 + 0.5) - ipixrs - border
y6end = y6beg + (ipixrs + border) * 2 - 1

x7beg = fix (xc7 + 0.5) - ipixrs - border
x7end = x7beg + (ipixrs + border) * 2 - 1
y7beg = fix (yc7 + 0.5) - ipixrs - border
y7end = y7beg + (ipixrs + border) * 2 - 1

img1c [*, *] = img1 [x1beg:x1end, y1beg:y1end]
img2c [*, *] = img2 [x2beg:x2end, y2beg:y2end]
img3c [*, *] = img3 [x3beg:x3end, y3beg:y3end]
img4c [*, *] = img4 [x4beg:x4end, y4beg:y4end]
img5c [*, *] = img5 [x5beg:x5end, y5beg:y5end]
img6c [*, *] = img6 [x6beg:x6end, y6beg:y6end]
img7c [*, *] = img7 [x7beg:x7end, y7beg:y7end]

tvscl, img1c
wait, 1
tvscl, img2c
wait, 1
tvscl, img3c
wait, 1
tvscl, img4c
wait, 1
tvscl, img5c
wait, 1
tvscl, img6c
wait, 1
tvscl, img7c

;--- For cropped, square images, set sun center equal to array center.

xc1 = xdimc / 2.0 - 0.5
yc1 = ydimc / 2.0 - 0.5

xc2 = xdimc / 2.0 - 0.5
yc2 = ydimc / 2.0 - 0.5

xc3 = xdimc / 2.0 - 0.5
yc3 = ydimc / 2.0 - 0.5

xc4 = xdimc / 2.0 - 0.5
yc4 = ydimc / 2.0 - 0.5

xc5 = xdimc / 2.0 - 0.5
yc5 = ydimc / 2.0 - 0.5

xc6 = xdimc / 2.0 - 0.5
yc6 = ydimc / 2.0 - 0.5

xc7 = xdimc / 2.0 - 0.5
yc7 = ydimc / 2.0 - 0.5

;--- Now, find sun center again in the square images.

;window, xsize=xdimc, ysize=ydimc

;gen_loc, img1c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc1 = centroid [0]
;yc1 = centroid [1]

;gen_loc, img2c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc2 = centroid [0]
;yc2 = centroid [1]

;gen_loc, img3c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc3 = centroid [0]
;yc3 = centroid [1]

;gen_loc, img4c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc4 = centroid [0]
;yc4 = centroid [1]

;gen_loc, img5c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc5 = centroid [0]
;yc5 = centroid [1]

;gen_loc, img6c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc6 = centroid [0]
;yc6 = centroid [1]

;gen_loc, img7c, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc7 = centroid [0]
;yc7 = centroid [1]

;--- Modify FITS header to update the sun center information.

fxaddpar, hdu1, 'CRPIX1', xc1, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu1, 'CRPIX2', yc1, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu2, 'CRPIX1', xc2, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu2, 'CRPIX2', yc2, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu3, 'CRPIX1', xc3, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu3, 'CRPIX2', yc3, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu4, 'CRPIX1', xc4, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu4, 'CRPIX2', yc4, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu5, 'CRPIX1', xc5, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu5, 'CRPIX2', yc5, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu6, 'CRPIX1', xc6, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu6, 'CRPIX2', yc6, ' Y-AXIS SUN CENTER [PIXELS]' 

fxaddpar, hdu7, 'CRPIX1', xc7, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu7, 'CRPIX2', yc7, ' Y-AXIS SUN CENTER [PIXELS]' 

;--- Write square images to disk if FITS format.

writefits, file1c, fix (img1c), hdu1
writefits, file2c, fix (img2c), hdu2
writefits, file3c, fix (img3c), hdu3
writefits, file4c, fix (img4c), hdu4
writefits, file5c, fix (img5c), hdu5
writefits, file6c, fix (img6c), hdu6
writefits, file7c, fix (img7c), hdu7

;--- Find the middle red filter image (CODE-OBS = "ssr").

imgnum = 0

FOR i = 1, 7 DO			$
   BEGIN
   IF (strpos (code_obs1, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 1
      imgr2 = img1c
      hdur2 = hdu1
      xcen = xc1
      ycen = yc1
      END
   IF (strpos (code_obs2, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 2
      imgr2 = img2c
      hdur2 = hdu2
      xcen = xc2
      ycen = yc2
      END
   IF (strpos (code_obs3, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 3
      imgr2 = img3c
      hdur2 = hdu3
      xcen = xc3
      ycen = yc3
      END
   IF (strpos (code_obs4, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 4
      imgr2 = img4c
      hdur2 = hdu4
      xcen = xc4
      ycen = yc4
      END
   IF (strpos (code_obs5, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 5
      imgr2 = img5c
      hdur2 = hdu5
      xcen = xc5
      ycen = yc5
      END
   IF (strpos (code_obs6, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 6
      imgr2 = img6c
      hdur2 = hdu6
      xcen = xc6
      ycen = yc6
      END
   IF (strpos (code_obs7, 'ssr') GE 0) THEN		$
      BEGIN
      imgnum = 7
      imgr2 = img7c
      hdur2 = hdu7
      xcen = xc7
      ycen = yc7
      END
   END

   
   numpixrs = fix (pixrs)
   rinc = 1.0 / pixrs
   numang = 360

   scanval = fltarr (numpixrs, numang)
   scanrad = fltarr (numpixrs, numang)
   scanang = fltarr (numpixrs, numang)
   scanavg = fltarr (numpixrs + 1)
   scandst = fltarr (numpixrs + 1)

   scanavg [*] = 0.0
   scandst [*] = 0.0

   FOR it = 0, 359   DO		$
      BEGIN	;{		$
      angle = float (it)
;      print, 'it: ', it
      radius = 0.0
      FOR ir = 0, numpixrs - 1   DO			$
	 BEGIN	;{
	 ierr = rcoord (radius, angle, x, y, 1, 0.0, xcen, ycen, pixrs)
	 ix = fix (x + 0.5)
	 iy = fix (y + 0.5)
	 scanval [ir, it] = imgr2 [ix, iy]
	 scanrad [ir, it] = radius
	 scanang [ir, it] = angle
	 scanavg [ir] = scanavg [ir] + imgr2 [ix, iy]	; Add radial scans.
	 scandst [ir] = radius
	 radius = radius + rinc
	 END	;}
      END	;}

scanavg = scanavg / 360.0		; Average radial scan.
scanavg [numpixrs] =  0.0
scandst [numpixrs] =  1.0
print, 'scandst [numpixrs - 1]: ', scandst [numpixrs - 1]

set_plot, 'ps'

PLOT, scandst, scanavg,			$
      title = '13 Apr 2000 Pixel Intensity Vs. Radial Distance',	$
      xtitle = 'Radial Distance [Rsun]',		$
      ytitle = 'Pixel Intensity'

set_plot, 'x'

;--- Compute average of all 7 images.

img = (img1c + img2c + img3c + img4c + img5c + img6c + img7c) / 7.0

;--- Remove limb darkening.

img = img * imgld 

;FOR iy = 0, ydimc - 1 DO		$
;   BEGIN
;   print, 'iy: ', iy
;   FOR ix = 0, xdimc - 1 DO	$
;      BEGIN
;      ierr = rcoord (radius, angle, ix, iy, -1, 0.0, xcen, ycen, pixrs)
;      IF (radius LT 1.0) THEN	$
;	 BEGIN
;	 limbdark = lint (radius, scandst, scanavg, numpixrs + 1) ; Interpolate.
;	 ldcor = scanavg [0] / limbdark				  ; Normalize.
;	 img [ix, iy] = img [ix, iy] * ldcor			  ; Adjust.
;	 END			$
;      ELSE			$
;	 img [ix, iy] = 0.0			; Set to zero beyond limb.
;      END
;   END

print, 'size (img): ', size (img)

imin = MIN (img, max=imax)

print, 'average image min/max: ', imin, imax

disk = img
xcen = xc1
ycen = yc1

print, 'size (disk): ', size (disk)
print, 'xcen/ycen: ', xcen, ycen

offdiskval = -1.0E+30

;--- Set all pixel magnitudes outside the solar limb  to a unique number.
;--- Then the "on-disk" pixels may be identified.

FOR iy = 0, ydimc - 1 DO	$
   BEGIN
   IF (iy LT 0 OR iy GE ydimc) THEN print, 'iy: ', iy
   ydist = float (iy) - ycen
   ydist2 = ydist * ydist
   FOR ix = 0, xdimc -1 DO	$
      BEGIN
      IF (ix LT 0 OR ix GE xdimc) THEN print, 'ix: ', ix
      xdist = float (ix) - xcen
      xdist2 = xdist * xdist
      dist = sqrt (xdist2 + ydist2)	; distance from sun center.
      IF (dist GE pixrs) THEN	$	; set pixels outside disk to zero.
         disk [ix, iy] = offdiskval
      END
   END

diskb = bytscl (disk, min=imin, max=imax, top=249)
;tv, diskb
;wait, 1

diskpix = WHERE (disk NE offdiskval)	; Coordinates of "on-disk" pixels.
diskmed = disk (diskpix)		; Array of pixels inside solar limb.

dmin = MIN (diskmed, max=dmax)
print, 'dmin, dmax: ', dmin, dmax
print, 'size (diskmed): ', size (diskmed)

;--- Compute the median intensity level for the solar disk pixels.

medianval = median (diskmed)

print, 'median: ', medianval

;--- Create an image of pixels whose intensities are greater than the median.

diskbrt = disk > medianval

brtmin = MIN (diskbrt, max=brtmax)
print, 'brtmin/brtmax: ', brtmin, brtmax
write_gif, 'chip.00d104.gtmedian.gif', bytscl (diskbrt, min=brtmin, max=brtmax)
;tvscl, diskbrt
;wait, 3

diskdim = disk < medianval
dimmin = MIN (diskdim, max=dimmax)
print, 'dimmin/dimmax: ', dimmin, dimmax
;tvscl, diskdim

END
