;+
; NAME
;	chipbg1.pro
;
; PURPOSE
;	Remove bias & CCD gain from an ACOS/CHIP image.
;	Process one wavelength at a time.
;
; SYNTAX
;	chipbg1, solar_image, bias_image, gain_image
;
;	list	contains a list of raw CHIP images (FITS format):
;		?sr.fts		solar image
;		?br.fts		bias image
;		?dd.fts		diffuser correction image
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR    4 April 2001
;	14 June 2001 : adapted from "chipbgc4.pro".
;-

PRO chipbg1, file_solar, file_bias, file_gain

    file = ''
;   print, 'filelist: ', filelist

    ;--- Open list file.

;    GET_LUN, lun
;    OPENR, lun, filelist

    ;--- Read CHIP solar image.

;    READF, lun, file, FORMAT = '(A)'

    imgs = readfits (file_solar, hdus, /noscale)
    hsks = readfits (file_solar, exts, exten_no=1, /noscale)

    ;--- Read CHIP bias  image.

;    READF, lun, file, FORMAT = '(A)'

    imgb = readfits (file_bias, hdub, /noscale)
    hskb = readfits (file_bias, extb, exten_no=1, /noscale)

    ;--- Read CHIP gain  image.

;    READF, lun, file, FORMAT = '(A)'

    imgd = readfits (file_gain, hdud, /noscale)
    hskd = readfits (file_gain, extd, exten_no=1, /noscale)

;    CLOSE, lun

    xdim     = fxpar (hdus, 'NAXIS1')
    ydim     = fxpar (hdus, 'NAXIS2')
    code_obs = fxpar (hdus, 'CODE-OBS') 
    date     = fxpar (hdus, 'DATE-OBS')
    time     = fxpar (hdus, 'TIME-OBS')
    dispmin  = fxpar (hdus, 'DISPMIN')
    dispmax  = fxpar (hdus, 'DISPMAX')

    ;--- Decipher date & time.
    
    syear    = STRMID (date, 0, 4)
    smonth   = STRMID (date, 5, 2)
    sday     = STRMID (date, 8, 2)
    shour    = STRMID (time, 0, 2)
    sminute  = STRMID (time, 3, 2)
    ssecond  = STRMID (time, 6, 2)
    
    print, 'syear: ', syear, ' smonth: ', smonth, ' sday: ', sday
    print, 'shour: ', shour, ' sminute: ', sminute, ' ssecond: ', ssecond
    
    ;--- Subtract bias & multiply by gain.

    imgc = (imgs - imgb) * imgd		; remove bias & "gain".
    imgc = imgc > 0			; Clip to positive pixel values.
    imgc = FIX (imgc)			; Convert to short integer.

    smin = MIN (imgs, max=smax)
    print, 'smin,smax: ', smin, smax

    bmin = MIN (imgb, max=bmax)
    print, 'bmin,bmax: ', bmin, bmax

    cmin = MIN (imgc, max=cmax)
    print, 'cmin,cmax: ', cmin, cmax

    ;--- Display original image.

    WINDOW, xsize=xdim/2, ysize=ydim/2
    lct, '/home/stanger/color/chip1.lut'	; Load color table.

    imgs2 = REBIN (imgs, xdim/2, ydim/2)
    TV, BYTSCL (imgs2, min=dispmin, max=dispmax)
    XYOUTS, 10, 30, 'ORIGINAL IMG', /device
    WAIT, 3

    ;--- Display corrected image.

    imgc2 = REBIN (imgc, xdim/2, ydim/2)

    dmin = 0
    dmax = cmax * 1.05
    dmax = 1024

    TV, BYTSCL (imgc2, min=dmin, max=dmax)
    XYOUTS, 10, 30, '(IMG - BIAS) * GAIN', /device
;    XYOUTS, 10, 10, 'DISPMIN=-100   DISPMAX=+100', /device
    
    type_dpc  = 'B&G ADJ'	; bias/gain correction applied.

;    extloc = STRPOS (file, code_obs)
;    filec  = STRMID (file, 0, extloc) + 'dsg.'
;    extloc = STRPOS (file, 'r')
;    fext   = STRMID (file, extloc+1, strlen (filec)-extloc+1)
;    filec  = filec + fext
;    print, 'filec: ', filec

    hduc = hdus

    code_obs2 = STRMID (code_obs, 0, 2)		; Get first two characters.
    code_obs4 = STRMID (code_obs, 3, 1)
    code_obsc = code_obs2 + 'G' + code_obs4	; replace 3rd char with "G".
    code_obsc = STRTRIM (code_obsc, 2)
    fxaddpar, hduc, 'CODE-OBS', code_obsc
    fxaddpar, hduc, 'TYPE-DP',  type_dpc

    filec = syear + smonth + sday + '.' + shour + sminute + ssecond + $
               '.chp.' + STRLOWCASE (code_obsc) + '.fts'
    PRINT, 'filec: ', filec
    
    writefits, filec, imgc, hduc
    writefits, filec, hsks, exts, /append

END
