;+
; NAME
;	chipbg4.pro
;
; PURPOSE
;	Remove bias & CCD gain from ACOS/CHIP images.
;
; SYNTAX
;	chipbg4, 'list'
;
;	list	contains a list of raw CHIP images (FITS format):
;
;		hbr		bias
;
;		ddd		B3 gain correction (from mklsdgains.pro)
;		cdd		B2 gain correction
;		bdd		B1 gain correction
;		hdd		He gain correction
;		rdd		R1 gain correction
;		sdd		R2 gain correction
;		tdd		R3 gain correction
;
;		dsr		B3 solar
;		csr		B2 solar
;		bsr		B1 solar
;		hsr		He solar
;		rsr		R1 solar
;		ssr		R2 solar
;		tsr		R3 solar
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   21 March 2001
;-

PRO chipbg4, filelist

   file = ''
;   print, 'filelist: ', filelist

   GET_LUN, lun
;   print, 'lun: ', lun
   OPENR, lun, filelist

   WHILE (NOT EOF (lun)) DO			$
   BEGIN ;{

      ;--- Read file name from list.

      READF, lun, file, FORMAT = '(A)'
      PRINT, 'Read file: ', file

      ;--- Read image from FITS file.
      ;--- Get information from FITS header.

      img = readfits (file, hdu, /noscale)
      ext = readfits (file, hduext, exten_no=1, /noscale)

      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      xcen     = fxpar (hdu, 'CRPIX1')
      ycen     = fxpar (hdu, 'CRPIX2')
      cdelt1   = fxpar (hdu, 'CDELT1')
      cdelt2   = fxpar (hdu, 'CDELT2')
      code_obs = STRTRIM (STRLOWCASE (fxpar (hdu, 'CODE-OBS')))
      nsum     = fxpar (hdu, 'NIMSUM')
      crradius = fxpar (hdu, 'CRRADIUS')
      datamin  = fxpar (hdu, 'DATAMIN')
      datamax  = fxpar (hdu, 'DATAMAX')
      dispmin  = fxpar (hdu, 'DISPMIN')
      dispmax  = fxpar (hdu, 'DISPMAX')
      wavelen  = fxpar (hdu, 'WAVELEN')

      wsr = STRMID (code_obs, 0, 3)
      IF (STRLEN (code_obs) EQ 4) THEN		$
	  expnum = STRMID (code_obs, 3, 1)	$
      ELSE					$
	  expnum = ''

      ;--- Set sun center equal to the array center.

      fxaddpar, hdu, 'CRPIX1', (xdim - 1.0) / 2.0
      fxaddpar, hdu, 'CRPIX2', (ydim - 1.0) / 2.0

      PRINT, 'code_obs: ', code_obs
      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax

      ;--- Assign image arrays according to the following labels:
      ;
      ;    img4b : hbr   Helium-I    bias
      ;
      ;    img1d : ddr   far  blue   diffuser
      ;    img2d : cdr   mid  blue   diffuser
      ;    img3d : bdr   near blue   diffuser
      ;    img4d : hdr   Helium-I    diffuser
      ;    img5d : rdr   near red    diffuser
      ;    img6d : sdr   mid  red    diffuser
      ;    img7d : tdr   far  red    diffuser

      ;    img1  : dsr   far  blue   solar
      ;    img2  : csr   mid  blue   solar
      ;    img3  : bsr   near blue   solar
      ;    img4  : hsr   Helium-I    solar
      ;    img5  : rsr   near red    solar
      ;    img6  : ssr   mid  red    solar
      ;    img7  : tsr   far  red    solar

      CASE wsr OF	;{
      'hbr': BEGIN
	     hdu4b = hdu
             img4b = img
	     ext4b = hduext
	     hsk4b = ext
	     code_obs4b = code_obs
	     file4b = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'ddd': BEGIN
	     hdu1d = hdu
             img1d = img
	     ext1d = hduext
	     hsk1d = ext
	     code_obs1d = code_obs
	     file1d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'cdd': BEGIN
	     hdu2d = hdu
	     img2d = img
	     ext2d = hduext
	     hsk2d = ext
	     code_obs2d = code_obs
	     file2d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'bdd': BEGIN
	     hdu3d = hdu
	     img3d = img
	     ext3d = hduext
	     hsk3d = ext
	     code_obs3d = code_obs
	     file3d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'hdd': BEGIN
	     hdu4d = hdu
	     img4d = img
	     ext4d = hduext
	     hsk4d = ext
	     code_obs4d = code_obs
	     file4d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'rdd': BEGIN
	     hdu5d = hdu
	     img5d = img
	     ext5d = hduext
	     hsk5d = ext
	     code_obs5d = code_obs
	     file5d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'sdd': BEGIN
	     hdu6d = hdu
	     img6d = img
	     ext6d = hduext
	     hsk6d = ext
	     code_obs6d = code_obs
	     file6d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'tdd': BEGIN
	     hdu7d = hdu
	     img7d = img
	     ext7d = hduext
	     hsk7d = ext
	     code_obs7d = code_obs
	     file7d = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'dsr': BEGIN
	     hdu1 = hdu
             img1 = img
	     ext1 = hduext
	     hsk1 = ext
	     code_obs1 = code_obs
	     file1 = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'csr': BEGIN
	     hdu2 = hdu
	     img2 = img
	     ext2 = hduext
	     hsk2 = ext
	     code_obs2 = code_obs
	     file2 = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'bsr': BEGIN
	     hdu3 = hdu
	     img3 = img
	     ext3 = hduext
	     hsk3 = ext
	     file3 = file
	     code_obs3 = code_obs
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'hsr': BEGIN
	     hdu4 = hdu
	     img4 = img
	     ext4 = hduext
	     hsk4 = ext
	     code_obs4 = code_obs
	     file4 = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'rsr': BEGIN
	     hdu5 = hdu
	     img5 = img
	     ext5 = hduext
	     hsk5 = ext
	     code_obs5 = code_obs
	     file5 = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'ssr': BEGIN
	     hdu6 = hdu
	     img6 = img
	     ext6 = hduext
	     hsk6 = ext
	     code_obs6 = code_obs
	     file6 = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'tsr': BEGIN
	     hdu7 = hdu
	     img7 = img
	     ext7 = hduext
	     hsk7 = ext
	     code_obs7 = code_obs
	     file7 = file
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
	     END
      ELSE:  BEGIN
	     PRINT, 'Invalid code-obs: ', code_obs
	     END
      ENDCASE ;}

   END ;}
   CLOSE, LUN


   ;--- Subtract bias, multiply by gain.

   img1c = (img1 - img4b) * img1d
   img1c = FIX (img1c)

   img2c = (img2 - img4b) * img2d
   img2c = FIX (img2c)

   img3c = (img3 - img4b) * img3d
   img3c = FIX (img3c)

   img4c = (img4 - img4b) * img4d
   img4c = FIX (img4c)

   img5c = (img5 - img4b) * img5d
   img5c = FIX (img5c)

   img6c = (img6 - img4b) * img6d
   img6c = FIX (img6c)

   img7c = (img7 - img4b) * img7d
   img7c = FIX (img7c)

    ;--- Subtract corrected image from original.

    border = 6
    img1dif = img1 - img1c

    img1difc = INTARR (xdim-border*2, ydim-border*2)
    img1difc = img1dif [border:xdim-1-border, border:ydim-1-border]
    img1dif2 = REBIN (img1dif, xdim/2, ydim/2)
    imin = MIN (img1difc, max=imax)
    dmin = -100
    dmax =  100
    print, 'imin,imax: ', imin, imax
    lct, '/home/stanger/color/dif.lut'		; Load difference color table.
    WINDOW, xsize=xdim/2, ysize=ydim/2
    TV, BYTSCL (img1dif2, min=dmin, max=dmax)
    XYOUTS, 10, 30, 'IMG1 - IMG1C', /device
    XYOUTS, 10, 10, 'DISPMIN=-100   DISPMAX=+100', /device
    
    type_dpc  = 'B&G COR'	; bias/gain correction applied.

    extloc = STRPOS (file1, code_obs1)
    file1c = STRMID (file1, 0, extloc) + 'dsg'
    extloc = STRPOS (file1, 'r')
    f1ext  = STRMID (file1, extloc+1, strlen (file1c)-extloc+1)
;    file1c = file1c + expnum + '.' + f1ext
    file1c = file1c + expnum + '.fts'
    print, 'file1c: ', file1c
    hdu1c = hdu1
    code_obs1c = 'DSG' + expnum
    fxaddpar, hdu1c, 'CODE-OBS', code_obs1c
    fxaddpar, hdu1c, 'TYPE-DP',  type_dpc
    writefits, file1c, img1c, hdu1c
    writefits, file1c, hsk1, ext1, /append

    extloc = STRPOS (file2, code_obs2)
    file2c = STRMID (file2, 0, extloc) + 'csg'
    extloc = STRPOS (file2, 'r')
    f2ext  = STRMID (file2, extloc+1, strlen (file2c)-extloc+1)
;    file2c = file2c + expnum + '.' + f2ext
    file2c = file2c + expnum + '.fts'
    hdu2c = hdu2
    code_obs2c = 'CSG' + expnum
    fxaddpar, hdu2c, 'CODE-OBS', code_obs2c
    fxaddpar, hdu2c, 'TYPE-DP',  type_dpc
    writefits, file2c, img2c, hdu2c
    writefits, file2c, hsk2, ext2, /append

    extloc = STRPOS (file3, code_obs3)
    file3c = STRMID (file3, 0, extloc) + 'bsg'
    f3ext  = STRMID (file3, extloc+1, strlen (file3c)-extloc+1)
;    file3c = file3c + expnum + '.' + f3ext
    file3c = file3c + expnum + '.fts'
    print, 'file3c: ', file3c
    hdu3c = hdu3
    code_obs3c = 'BSG' + expnum
    fxaddpar, hdu3c, 'CODE-OBS', code_obs3c
    fxaddpar, hdu3c, 'TYPE-DP',  type_dpc
    writefits, file3c, img3c, hdu3c
    writefits, file3c, hsk3, ext3, /append

    extloc = STRPOS (file4, code_obs4)
    file4c = STRMID (file4, 0, extloc) + 'hsg'
    f4ext  = STRMID (file4, extloc+1, strlen (file4c)-extloc+1)
;    file4c = file4c + expnum + '.' + f4ext
    file4c = file4c + expnum + '.fts'
    print, 'file4c: ', file4c
    hdu4c = hdu4
    code_obs4c = 'HSG' + expnum
    fxaddpar, hdu4c, 'CODE-OBS', code_obs4c
    fxaddpar, hdu4c, 'TYPE-DP',  type_dpc
    writefits, file4c, img4c, hdu4c
    writefits, file4c, hsk4, ext4, /append

    extloc = STRPOS (file5, code_obs5)
    file5c = STRMID (file5, 0, extloc) + 'rsg'
    f5ext  = STRMID (file5, extloc+1, strlen (file5c)-extloc+1)
;    file5c = file5c + expnum + '.' + f5ext
    file5c = file5c + expnum + '.fts'
    print, 'file5c: ', file5c
    hdu5c = hdu5
    code_obs5c = 'RSG' + expnum
    fxaddpar, hdu5c, 'CODE-OBS', code_obs5c
    fxaddpar, hdu5c, 'TYPE-DP',  type_dpc
    writefits, file5c, img5c, hdu5c
    writefits, file5c, hsk5, ext5, /append

    extloc = STRPOS (file6, code_obs6)
    file6c = STRMID (file6, 0, extloc) + 'ssg'
    f6ext  = STRMID (file6, extloc+1, strlen (file6c)-extloc+1)
;    file6c = file6c + expnum + '.' + f6ext
    file6c = file6c + expnum + '.fts'
    print, 'file6c: ', file6c
    hdu6c = hdu6
    code_obs6c = 'SSG' + expnum
    fxaddpar, hdu6c, 'CODE-OBS', code_obs6c
    fxaddpar, hdu6c, 'TYPE-DP',  type_dpc
    writefits, file6c, img6c, hdu6c
    writefits, file6c, hsk6, ext6, /append

    extloc = STRPOS (file7, code_obs7)
    file7c = STRMID (file7, 0, extloc) + 'tsg'
    f7ext  = STRMID (file7, extloc+1, strlen (file7c)-extloc+1)
;    file7c = file7c + expnum + '.' + f7ext
    file7c = file7c + expnum + '.fts'
    print, 'file7c: ', file7c
    hdu7c = hdu7
    code_obs7c = 'TSG' + expnum
    fxaddpar, hdu7c, 'CODE-OBS', code_obs7c
    fxaddpar, hdu7c, 'TYPE-DP',  type_dpc
    writefits, file7c, img7c, hdu7c
    writefits, file7c, hsk7, ext7, /append

END
