;+
; NAME
;	chipbg7.pro
;
; PURPOSE
;	Remove bias & CCD gain from ACOS/CHIP images.
;
; SYNTAX
;	chipbg, 'list'
;
;	list	contains a list of raw CHIP images (FITS format):
;
;		hbr		bias
;
;		ddr.lsc		B3 gain correction (from mklsdgains.pro)
;		cdr.lsc		B2 gain correction
;		bdr.lsc		B1 gain correction
;		hdr.lsc		He gain correction
;		rdr.lsc		R1 gain correction
;		sdr.lsc		R2 gain correction
;		tdr.lsc		R3 gain correction
;
;		dsr		B3 solar
;		csr		B2 solar
;		bsr		B1 solar
;		hsr		He solar
;		rsr		R1 solar
;		ssr		R2 solar
;		tsr		R3 solar
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   21 March 2001
;-

PRO chipbg7, filelist

   file = ''
;   print, 'filelist: ', filelist

   GET_LUN, lun
;   print, 'lun: ', lun
   OPENR, lun, filelist

   WHILE (NOT EOF (lun)) DO			$
   BEGIN ;{

      ;--- Read file name from list.

      READF, lun, file, FORMAT = '(A)'
      PRINT, 'Read file: ', file

      ;--- Read image from FITS file.
      ;--- Get information from FITS header.

      img = readfits (file, hdu, /noscale)
      ext = readfits (file, hduext, exten_no=1, /noscale)

      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      xcen     = fxpar (hdu, 'CRPIX1')
      ycen     = fxpar (hdu, 'CRPIX2')
      cdelt1   = fxpar (hdu, 'CDELT1')
      cdelt2   = fxpar (hdu, 'CDELT2')
      code_obs = STRTRIM (STRLOWCASE (fxpar (hdu, 'CODE-OBS')))
      nsum     = fxpar (hdu, 'NIMSUM')
      crradius = fxpar (hdu, 'CRRADIUS')
      datamin  = fxpar (hdu, 'DATAMIN')
      datamax  = fxpar (hdu, 'DATAMAX')
      dispmin  = fxpar (hdu, 'DISPMIN')
      dispmax  = fxpar (hdu, 'DISPMAX')
      wavelen  = fxpar (hdu, 'WAVELEN')

      ;--- Set sun center equal to the array center.

      fxaddpar, hdu, 'CRPIX1', (xdim - 1.0) / 2.0
      fxaddpar, hdu, 'CRPIX2', (ydim - 1.0) / 2.0

      PRINT, 'code_obs: ', code_obs
      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax

      ;--- Compute the average x/y shift values.

      xsh = FLTARR (nsum)
      ysh = FLTARR (nsum)
      xshsum = 0
      yshsum = 0

      FOR i = 0, nsum - 1 DO	$
      BEGIN ;{
         snum = STRTRIM (STRING (i), 2)
         IF (nsum LT 10) THEN snum = '0' + snum
         xshkey = 'XSHIFT' + snum
         print, 'xshkey: ', xshkey
         xsh [i] = fxpar (hduext, 'XSHIFT' + snum)
         print, 'xsh: ', xsh [i]
         xshsum  = xshsum + xsh [i]
         ysh [i] = fxpar (hduext, 'YSHIFT' + snum)
         yshsum  = yshsum + ysh [i]
         print, 'ysh: ', ysh [i]
      END ;}

      ;--- Average X & Y shifts:

      xshave = xshsum / nsum
      yshave = yshsum / nsum
      PRINT, 'xshave/yshave: ', xshave, yshave

      ;--- Temporarily set values for testing only:

;      xshave =  60.0
;      yshave = -20.0
;      PRINT, 'Test Values . . . xshave/yshave: ', xshave, yshave

      ;--- Assign image arrays according to the following labels:
      ;
      ;    img4b : hbr   Helium-I    bias
      ;
      ;    img1d : ddr   far  blue   diffuser
      ;    img2d : cdr   mid  blue   diffuser
      ;    img3d : bdr   near blue   diffuser
      ;    img4d : hdr   Helium-I    diffuser
      ;    img5d : rdr   near red    diffuser
      ;    img6d : sdr   mid  red    diffuser
      ;    img7d : tdr   far  red    diffuser

      ;    img1  : dsr   far  blue   solar
      ;    img2  : csr   mid  blue   solar
      ;    img3  : bsr   near blue   solar
      ;    img4  : hsr   Helium-I    solar
      ;    img5  : rsr   near red    solar
      ;    img6  : ssr   mid  red    solar
      ;    img7  : tsr   far  red    solar

      CASE code_obs OF	;{
      'hbr': BEGIN
	     hdu4b = hdu
             img4b = img
	     ext4b = hduext
	     hsk4b = ext
	     code_obs4b = code_obs
	     file4b = file
	     xsh4b = xshave
	     ysh4b = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'ddr': BEGIN
	     hdu1d = hdu
             img1d = img
	     ext1d = hduext
	     hsk1d = ext
	     code_obs1d = code_obs
	     file1d = file
	     xsh1d = xshave
	     ysh1d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'cdr': BEGIN
	     hdu2d = hdu
	     img2d = img
	     ext2d = hduext
	     hsk2d = ext
	     code_obs2d = code_obs
	     file2d = file
	     xsh2d = xshave
	     ysh2d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'bdr': BEGIN
	     hdu3d = hdu
	     img3d = img
	     ext3d = hduext
	     hsk3d = ext
	     code_obs3d = code_obs
	     file3d = file
	     xsh3d = xshave
	     ysh3d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'hdr': BEGIN
	     hdu4d = hdu
	     img4d = img
	     ext4d = hduext
	     hsk4d = ext
	     code_obs4d = code_obs
	     file4d = file
	     xsh4d = xshave
	     ysh4d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'rdr': BEGIN
	     hdu5d = hdu
	     img5d = img
	     ext5d = hduext
	     hsk5d = ext
	     code_obs5d = code_obs
	     file5d = file
	     xsh5d = xshave
	     ysh5d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'sdr': BEGIN
	     hdu6d = hdu
	     img6d = img
	     ext6d = hduext
	     hsk6d = ext
	     code_obs6d = code_obs
	     file6d = file
	     xsh6d = xshave
	     ysh6d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'tdr': BEGIN
	     hdu7d = hdu
	     img7d = img
	     ext7d = hduext
	     hsk7d = ext
	     code_obs7d = code_obs
	     file7d = file
	     xsh7d = xshave
	     ysh7d = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'dsr': BEGIN
	     hdu1 = hdu
             img1 = img
	     ext1 = hduext
	     hsk1 = ext
	     code_obs1 = code_obs
	     file1 = file
	     xsh1 = xshave
	     ysh1 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'csr': BEGIN
	     hdu2 = hdu
	     img2 = img
	     ext2 = hduext
	     hsk2 = ext
	     code_obs2 = code_obs
	     file2 = file
	     xsh2 = xshave
	     ysh2 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'bsr': BEGIN
	     hdu3 = hdu
	     img3 = img
	     ext3 = hduext
	     hsk3 = ext
	     file3 = file
	     code_obs3 = code_obs
	     xsh3 = xshave
	     ysh3 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'hsr': BEGIN
	     hdu4 = hdu
	     img4 = img
	     ext4 = hduext
	     hsk4 = ext
	     code_obs4 = code_obs
	     file4 = file
	     xsh4 = xshave
	     ysh4 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'rsr': BEGIN
	     hdu5 = hdu
	     img5 = img
	     ext5 = hduext
	     hsk5 = ext
	     code_obs5 = code_obs
	     file5 = file
	     xsh5 = xshave
	     ysh5 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'ssr': BEGIN
	     hdu6 = hdu
	     img6 = img
	     ext6 = hduext
	     hsk6 = ext
	     code_obs6 = code_obs
	     file6 = file
	     xsh6 = xshave
	     ysh6 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
             END
      'tsr': BEGIN
	     hdu7 = hdu
	     img7 = img
	     ext7 = hduext
	     hsk7 = ext
	     code_obs7 = code_obs
	     file7 = file
	     xsh7 = xshave
	     ysh7 = yshave
	     print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
	     END
      ELSE:  BEGIN
	     PRINT, 'Invalid code-obs: ', code_obs
	     END
      ENDCASE ;}

   END ;}
   CLOSE, LUN

   ;--- Shift diffuser & bias images.

   PRINT, 'Shifting images . . . [dsr]'

   imgbs = imgshift (img4b, xdim, ydim, xsh1, ysh1, 0.0)
   imgds = imgshift (img1d, xdim, ydim, xsh1, ysh1, 1.0)
   img1c = (img1 - imgbs) * imgds
   img1c = FIX (img1c)

   PRINT, 'Shifting images . . . [csr]'
   imgbs = imgshift (img4b, xdim, ydim, xsh2, ysh2, 0.0)
   imgds = imgshift (img2d, xdim, ydim, xsh2, ysh2, 1.0)
   img2c = (img2 - imgbs) * imgds
   img2c = FIX (img2c)

   PRINT, 'Shifting images . . . [bsr]'
   imgbs = imgshift (img4b, xdim, ydim, xsh3, ysh3, 0.0)
   imgds = imgshift (img3d, xdim, ydim, xsh3, ysh3, 1.0)
   img3c = (img3 - imgbs) * imgds
   img3c = FIX (img3c)

   PRINT, 'Shifting images . . . [hsr]'
   imgbs = imgshift (img4b, xdim, ydim, xsh4, ysh4, 0.0)
   imgds = imgshift (img4d, xdim, ydim, xsh4, ysh4, 1.0)
   img4c = (img4 - imgbs) * imgds
   img4c = FIX (img4c)

   PRINT, 'Shifting images . . . [rsr]'
   imgbs = imgshift (img4b, xdim, ydim, xsh5, ysh5, 0.0)
   imgds = imgshift (img5d, xdim, ydim, xsh5, ysh5, 1.0)
   img5c = (img5 - imgbs) * imgds
   img5c = FIX (img5c)

   PRINT, 'Shifting images . . . [ssr]'
   imgbs = imgshift (img4b, xdim, ydim, xsh6, ysh6, 0.0)
   imgds = imgshift (img6d, xdim, ydim, xsh6, ysh6, 1.0)
   img6c = (img6 - imgbs) * imgds
   img6c = FIX (img6c)

   PRINT, 'Shifting images . . . [tsr]'
   imgbs = imgshift (img4b, xdim, ydim, xsh7, ysh7, 0.0)
   imgds = imgshift (img7d, xdim, ydim, xsh7, ysh7, 1.0)
   img7c = (img7 - imgbs) * imgds
   img7c = FIX (img7c)

    ;--- Subtract corrected image from original.

    border = 6
    img1dif = img1 - img1c

    img1difc = INTARR (xdim-border*2, ydim-border*2)
    img1difc = img1dif [border:xdim-1-border, border:ydim-1-border]
    img1dif2 = REBIN (img1dif, xdim/2, ydim/2)
    imin = MIN (img1difc, max=imax)
    dmin = -100
    dmax =  100
    print, 'imin,imax: ', imin, imax
    lct, '/home/stanger/color/dif.lut'		; Load difference color table.
    WINDOW, xsize=xdim/2, ysize=ydim/2
    TV, BYTSCL (img1dif2, min=dmin, max=dmax)
    XYOUTS, 10, 30, 'IMG1 - IMG1C', /device
    XYOUTS, 10, 10, 'DISPMIN=-100   DISPMAX=+100', /device
    
    type_dpc  = 'B&G COR'	; bias/gain correction applied.

    extloc = STRPOS (file1, code_obs1)
    file1c = STRMID (file1, 0, extloc) + 'dsg.fts'
    print, 'file1c: ', file1c
    hdu1c = hdu1
    code_obs1c = 'DSG'
    fxaddpar, hdu1c, 'CODE-OBS', code_obs1c
    fxaddpar, hdu1c, 'TYPE-DP',  type_dpc
    writefits, file1c, img1c, hdu1c
    writefits, file1c, hsk1, ext1, /append

    extloc = STRPOS (file2, code_obs2)
    file2c = STRMID (file2, 0, extloc) + 'csg.fts'
    hdu2c = hdu2
    code_obs2c = 'CSG'
    fxaddpar, hdu2c, 'CODE-OBS', code_obs2c
    fxaddpar, hdu2c, 'TYPE-DP',  type_dpc
    writefits, file2c, img2c, hdu2c
    writefits, file2c, hsk2, ext2, /append

    extloc = STRPOS (file3, code_obs3)
    file3c = STRMID (file3, 0, extloc) + 'bsg.fts'
    hdu3c = hdu3
    code_obs3c = 'BSG'
    fxaddpar, hdu3c, 'CODE-OBS', code_obs3c
    fxaddpar, hdu3c, 'TYPE-DP',  type_dpc
    writefits, file3c, img3c, hdu3c
    writefits, file3c, hsk3, ext3, /append

    extloc = STRPOS (file4, code_obs4)
    file4c = STRMID (file4, 0, extloc) + 'hsg.fts'
    hdu4c = hdu4
    code_obs4c = 'HSG'
    fxaddpar, hdu4c, 'CODE-OBS', code_obs4c
    fxaddpar, hdu4c, 'TYPE-DP',  type_dpc
    writefits, file4c, img4c, hdu4c
    writefits, file4c, hsk4, ext4, /append

    extloc = STRPOS (file5, code_obs5)
    file5c = STRMID (file5, 0, extloc) + 'rsg.fts'
    hdu5c = hdu5
    code_obs5c = 'RSG'
    fxaddpar, hdu5c, 'CODE-OBS', code_obs5c
    fxaddpar, hdu5c, 'TYPE-DP',  type_dpc
    writefits, file5c, img5c, hdu5c
    writefits, file5c, hsk5, ext5, /append

    extloc = STRPOS (file6, code_obs6)
    file6c = STRMID (file6, 0, extloc) + 'ssg.fts'
    hdu6c = hdu6
    code_obs6c = 'SSG'
    fxaddpar, hdu6c, 'CODE-OBS', code_obs6c
    fxaddpar, hdu6c, 'TYPE-DP',  type_dpc
    writefits, file6c, img6c, hdu6c
    writefits, file6c, hsk6, ext6, /append

    extloc = STRPOS (file7, code_obs7)
    file7c = STRMID (file7, 0, extloc) + 'tsg.fts'
    hdu7c = hdu7
    code_obs7c = 'TSG'
    fxaddpar, hdu7c, 'CODE-OBS', code_obs7c
    fxaddpar, hdu7c, 'TYPE-DP',  type_dpc
    writefits, file7c, img7c, hdu7c
    writefits, file7c, hsk7, ext7, /append

END
