;+
; NAME		chipnorm14
;
; PURPOSE	Normalize filter responses.
;
; SYNTAX	chipnorm14, fits_list
;
;		fits_list	File containing a list of 7 images, spanning
;				the wavelength range 1082.745 - 1083.315 nm.
;				Images are required to be in FITS format.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR    2 Jun 2000
;		19 Sep 2000: Use ssr limb darkening for all except dsr.
;		             Normalization: all agree at sun center.
;		 3 Oct 2000: Normalize all filters in same fashion,
;			     using brightest 5% pixels over the disk
;			     to create an average maximum brightness.
;		12 Oct 2000: Use Peter Bandurian's Fortran routine (flmbiter.f)
;			     to compute limb darkening.
;			     Use initgeom to determine disc center.
;		 4 Jan 2001: Use Randy Meisner's initgeom to find disc center.	
;		17 Jan 2001: Use disc center info while cropping images.
;		21 Feb 2001: Use FITS file for radial distance mask.
;		22 Mar 2001: Add FITS extension to output files.
;		18 Apr 2001: Compute normalized average brightness to
;			     verify normalization procedure.
;			     Fix xbeg/ybeg: use xcsq/ycsq for sq img center.
;		19 Apr 2001: shift image by 1 pixel:
;			     x1beg = x1beg + 1, y1beg = y1beg + 1.
;-

PRO chipnorm14, fits_list

red    = BYTARR (256)
green  = BYTARR (256)
blue   = BYTARR (256)
cindex = BINDGEN (256)
cbar   = BYTARR (256, 20)
FOR i = 0, 19 DO cbar [*, i] = cindex

file  = ''
file1 = ''
file2 = ''
file3 = ''
file4 = ''
file5 = ''
file6 = ''
file7 = ''

daytab = INTARR (13,2)
daytab = [[0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ],  $
          [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]]

;--- Cropped square image size.

xdimsq = 936
ydimsq = 936

;--- For cropped, square images, set sun center equal to array center.

xcsq = (xdimsq - 1) / 2.0
ycsq = (ydimsq - 1) / 2.0
print, 'xcsq/ycsq: ', xcsq, ycsq

window, xsize=xdimsq, ysize=ydimsq
lct, '/home/stanger/color/chip1.lut'

;--- Read the FITS file names from the input list file.
;
;    Note: Prior to using this IDL program, use the  "acosfits" c-language
;    program to convert native format CHIP images into FITS format.

CLOSE,  10
OPENR,  10, fits_list

;--- Read each of the images into memory.

FOR i = 0, 6 DO		$
    BEGIN
    READF,  10, file
    PRINT, 'file: ', file
    img = readfits (file, hdu, /noscale)
    hsk = readfits (file, hduext, exten_no=1, /noscale)
    code_obs = strtrim (strlowcase (fxpar (hdu, 'CODE-OBS')))
    wsr = STRMID (code_obs, 0, 3)
    wavelen = fxpar (hdu, 'WAVELEN')
    pangle = fxpar (hdu, 'SOLAR_P0')

;--- Assign image arrays according to the following labels:
;
;    img1: dsr   far  blue
;    img2: csr   mid  blue
;    img3: bsr   near blue
;    img4: hsr   on-line helium
;    img5: rsr   near red
;    img6: ssr   mid  red
;    img7: tsr   far  red

    CASE wsr OF
        'dsr': BEGIN
	       img1 = img
	       hdu1 = hdu
	       ext1 = hduext
	       hsk1 = hsk
	       code_obs1 = code_obs
	       file1 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'csr': BEGIN
	       img2 = img
	       hdu2 = hdu
	       ext2 = hduext
	       hsk2 = hsk
	       code_obs2 = code_obs
	       file2 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'bsr': BEGIN
	       img3 = img
	       hdu3 = hdu
	       ext3 = hduext
	       hsk3 = hsk
	       file3 = file
	       code_obs3 = code_obs
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'hsr': BEGIN
	       img4 = img
	       hdu4 = hdu
	       ext4 = hduext
	       hsk4 = hsk
	       code_obs4 = code_obs
	       file4 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'rsr': BEGIN
	       img5 = img
	       hdu5 = hdu
	       ext5 = hduext
	       hsk5 = hsk
	       code_obs5 = code_obs
	       file5 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'ssr': BEGIN
	       img6 = img
	       hdu6 = hdu
	       ext6 = hduext
	       hsk6 = hsk
	       code_obs6 = code_obs
	       file6 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'tsr': BEGIN
	       img7 = img
	       hdu7 = hdu
	       ext7 = hduext
	       hsk7 = hsk
	       code_obs7 = code_obs
	       file7 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'dsg': BEGIN
	       img1 = img
	       hdu1 = hdu
	       ext1 = hduext
	       hsk1 = hsk
	       code_obs1 = code_obs
	       file1 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'csg': BEGIN
	       img2 = img
	       hdu2 = hdu
	       ext2 = hduext
	       hsk2 = hsk
	       code_obs2 = code_obs
	       file2 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'bsg': BEGIN
	       img3 = img
	       hdu3 = hdu
	       ext3 = hduext
	       hsk3 = hsk
	       file3 = file
	       code_obs3 = code_obs
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'hsg': BEGIN
	       img4 = img
	       hdu4 = hdu
	       ext4 = hduext
	       hsk4 = hsk
	       code_obs4 = code_obs
	       file4 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'rsg': BEGIN
	       img5 = img
	       hdu5 = hdu
	       ext5 = hduext
	       hsk5 = hsk
	       code_obs5 = code_obs
	       file5 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'ssg': BEGIN
	       img6 = img
	       hdu6 = hdu
	       ext6 = hduext
	       hsk6 = hsk
	       code_obs6 = code_obs
	       file6 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'tsg': BEGIN
	       img7 = img
	       hdu7 = hdu
	       ext7 = hduext
	       hsk7 = hsk
	       code_obs7 = code_obs
	       file7 = file
	       PRINT, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        ELSE:  BEGIN
	       PRINT, 'Invalid code-obs: ', code_obs
	       END
        ENDCASE
    END
CLOSE, 10

img = 0
hdu = 0

;--- Get information from FITS header.

pixrs  = fxpar (hdu4, 'CRRADIUS')	; # Pixels / solar radius 
rsun   = fxpar (hdu4, 'RSUN')		; Solar Radius (arcsec)
xdim   = fxpar (hdu4, 'NAXIS1')		; X-axis dimension.
ydim   = fxpar (hdu4, 'NAXIS2')		; Y-axis dimension.
bitpix = fxpar (hdu4, 'BITPIX')		; Bits per pixel.
nbypix = bitpix / 8			; Bytes per pixel.

ipixrs = FIX (pixrs + 0.5)

;border = 50				; "Border" beyond the limb for cropping.
;xdimc = (ipixrs + border) * 2
;ydimc = xdimc
;xdimc = 936				; X-axis dimension for cropped image.
;ydimc = 936				; Y-axis dimension for cropped image.

PRINT, ''
PRINT, 'xdim/ydim :   ', xdim, ydim, '   nbypix: ', nbypix
PRINT, 'rsun  : ', rsun
PRINT, 'pixrs : ', pixrs
PRINT, ''

;--- Overwrite the first line (bottom), which contains the binary header.
;--- Replace it with the adjacent horizontal line of the image.

img1 [*, 0] = img1 [*, 1]
img2 [*, 0] = img2 [*, 1]
img3 [*, 0] = img3 [*, 1]
img4 [*, 0] = img4 [*, 1]
img5 [*, 0] = img5 [*, 1]
img6 [*, 0] = img6 [*, 1]
img7 [*, 0] = img7 [*, 1]

;--- Create storage for cropped images (square shape).

img1f = FLTARR (xdimsq, ydimsq)
img2f = FLTARR (xdimsq, ydimsq)
img3f = FLTARR (xdimsq, ydimsq)
img4f = FLTARR (xdimsq, ydimsq)
img5f = FLTARR (xdimsq, ydimsq)
img6f = FLTARR (xdimsq, ydimsq)
img7f = FLTARR (xdimsq, ydimsq)

;--- Get the disk center locations from the FITS header.

xc1 = fxpar (hdu1, 'CRPIX1')
yc1 = fxpar (hdu1, 'CRPIX2')
xc2 = fxpar (hdu2, 'CRPIX1')
yc2 = fxpar (hdu2, 'CRPIX2')
xc3 = fxpar (hdu3, 'CRPIX1')
yc3 = fxpar (hdu3, 'CRPIX2')
xc4 = fxpar (hdu4, 'CRPIX1')
yc4 = fxpar (hdu4, 'CRPIX2')
xc5 = fxpar (hdu5, 'CRPIX1')
yc5 = fxpar (hdu5, 'CRPIX2')
xc6 = fxpar (hdu6, 'CRPIX1')
yc6 = fxpar (hdu6, 'CRPIX2')
xc7 = fxpar (hdu7, 'CRPIX1')
yc7 = fxpar (hdu7, 'CRPIX2')

;--- Find disc center (using Randy's Meisner's software).

;if (xc1 LT 0.0)  THEN print, 'initgeom: finding disc centers.'
;edgeval = 1000.0
;if (xc1 LT 0.0) THEN initgeom, img1, edgeval, xc1, yc1, r1
;print, 'xc1/yc1: ', xc1, yc1
;if (xc2 LT 0.0) THEN initgeom, img2, edgeval, xc2, yc2, r2
;print, 'xc2/yc2: ', xc2, yc2
;if (xc3 LT 0.0) THEN initgeom, img3, edgeval, xc3, yc3, r3
;print, 'xc3/yc3: ', xc3, yc3
;if (xc4 LT 0.0) THEN initgeom, img4, edgeval, xc4, yc4, r4
;print, 'xc4/yc4: ', xc4, yc4
;if (xc5 LT 0.0) THEN initgeom, img5, edgeval, xc5, yc5, r5
;print, 'xc5/yc5: ', xc5, yc5
;if (xc6 LT 0.0) THEN initgeom, img6, edgeval, xc6, yc6, r6
;print, 'xc6/yc6: ', xc6, yc6
;if (xc7 LT 0.0) THEN initgeom, img7, edgeval, xc7, yc7, r7
;print, 'xc7/yc7: ', xc7, yc7

;--- If center information is invalid, assume disk center is array center.

IF (xc1 LT 0.0)  THEN xc1 = xdim / 2.0 - 0.5
IF (yc1 LT 0.0)  THEN yc1 = ydim / 2.0 - 0.5
IF (xc2 LT 0.0)  THEN xc2 = xdim / 2.0 - 0.5
IF (yc2 LT 0.0)  THEN yc2 = ydim / 2.0 - 0.5
IF (xc3 LT 0.0)  THEN xc3 = xdim / 2.0 - 0.5
IF (yc3 LT 0.0)  THEN yc3 = ydim / 2.0 - 0.5
IF (xc4 LT 0.0)  THEN xc4 = xdim / 2.0 - 0.5
IF (yc4 LT 0.0)  THEN yc4 = ydim / 2.0 - 0.5
IF (xc5 LT 0.0)  THEN xc5 = xdim / 2.0 - 0.5
IF (yc5 LT 0.0)  THEN yc5 = ydim / 2.0 - 0.5
IF (xc6 LT 0.0)  THEN xc6 = xdim / 2.0 - 0.5
IF (yc6 LT 0.0)  THEN yc6 = ydim / 2.0 - 0.5
IF (xc7 LT 0.0)  THEN xc7 = xdim / 2.0 - 0.5
IF (yc7 LT 0.0)  THEN yc7 = ydim / 2.0 - 0.5

PRINT, 'xc1/yc1: ', xc1, yc1

;--- Get date & time.
;--- Get WSR (wavelength/Source/Reduction) code. 

date_obs1 = fxpar (hdu1, 'DATE-OBS')
date_obs2 = fxpar (hdu2, 'DATE-OBS')
date_obs3 = fxpar (hdu3, 'DATE-OBS')
date_obs4 = fxpar (hdu4, 'DATE-OBS')
date_obs5 = fxpar (hdu5, 'DATE-OBS')
date_obs6 = fxpar (hdu6, 'DATE-OBS')
date_obs7 = fxpar (hdu7, 'DATE-OBS')

time_obs1 = fxpar (hdu1, 'TIME-OBS')
time_obs2 = fxpar (hdu2, 'TIME-OBS')
time_obs3 = fxpar (hdu3, 'TIME-OBS')
time_obs4 = fxpar (hdu4, 'TIME-OBS')
time_obs5 = fxpar (hdu5, 'TIME-OBS')
time_obs6 = fxpar (hdu6, 'TIME-OBS')
time_obs7 = fxpar (hdu7, 'TIME-OBS')

year1 = strmid (date_obs1, 0, 4)
year2 = strmid (date_obs2, 0, 4)
year3 = strmid (date_obs3, 0, 4)
year4 = strmid (date_obs4, 0, 4)
year5 = strmid (date_obs5, 0, 4)
year6 = strmid (date_obs6, 0, 4)
year7 = strmid (date_obs7, 0, 4)

month1 = strmid (date_obs1, 5, 2)
month2 = strmid (date_obs2, 5, 2)
month3 = strmid (date_obs3, 5, 2)
month4 = strmid (date_obs4, 5, 2)
month5 = strmid (date_obs5, 5, 2)
month6 = strmid (date_obs6, 5, 2)
month7 = strmid (date_obs7, 5, 2)

day1 = strmid (date_obs1, 8, 2)
day2 = strmid (date_obs2, 8, 2)
day3 = strmid (date_obs3, 8, 2)
day4 = strmid (date_obs4, 8, 2)
day5 = strmid (date_obs5, 8, 2)
day6 = strmid (date_obs6, 8, 2)
day7 = strmid (date_obs7, 8, 2)

hour1 = strmid (time_obs1, 0, 2)
hour2 = strmid (time_obs2, 0, 2)
hour3 = strmid (time_obs3, 0, 2)
hour4 = strmid (time_obs4, 0, 2)
hour5 = strmid (time_obs5, 0, 2)
hour6 = strmid (time_obs6, 0, 2)
hour7 = strmid (time_obs7, 0, 2)

minute1 = strmid (time_obs1, 3, 2)
minute2 = strmid (time_obs2, 3, 2)
minute3 = strmid (time_obs3, 3, 2)
minute4 = strmid (time_obs4, 3, 2)
minute5 = strmid (time_obs5, 3, 2)
minute6 = strmid (time_obs6, 3, 2)
minute7 = strmid (time_obs7, 3, 2)

second1 = strmid (time_obs1, 6, 2)
second2 = strmid (time_obs2, 6, 2)
second3 = strmid (time_obs3, 6, 2)
second4 = strmid (time_obs4, 6, 2)
second5 = strmid (time_obs5, 6, 2)
second6 = strmid (time_obs6, 6, 2)
second7 = strmid (time_obs7, 6, 2)

;--- Create filenames for output cropped images.

file1c = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.sq.fts'

file2c = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.sq.fts'

file3c = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.sq.fts'

file4c = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.sq.fts'

file5c = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.sq.fts'

file6c = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.sq.fts'

file7c = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.sq.fts'

;--- Find disc center (using Alan McAllister's software).

IF (xc1 LT 0.0 OR yc1 LT 0.0) THEN	$
   BEGIN
   gen_loc, img1, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file1: ', file1
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc1 = centroid [0]
   yc1 = centroid [1]
   END

IF (xc2 LT 0.0 OR yc2 LT 0.0) THEN	$
   BEGIN
   gen_loc, img2, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file2: ', file2
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc2 = centroid [0]
   yc2 = centroid [1]
   END

IF (xc3 LT 0.0 OR yc3 LT 0.0) THEN	$
   BEGIN
   gen_loc, img3, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file3: ', file3
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc3 = centroid [0]
   yc3 = centroid [1]
   END

IF (xc4 LT 0.0 OR yc4 LT 0.0) THEN	$
   BEGIN
   gen_loc, img4, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file4: ', file4
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc4 = centroid [0]
   yc4 = centroid [1]
   END

IF (xc5 LT 0.0 OR yc5 LT 0.0) THEN	$
   BEGIN
   gen_loc, img5, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file5: ', file5
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc5 = centroid [0]
   yc5 = centroid [1]
   END

IF (xc6 LT 0.0 OR yc6 LT 0.0) THEN	$
   BEGIN
   gen_loc, img6, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file6: ', file6
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc6 = centroid [0]
   yc6 = centroid [1]
   END

IF (xc7 LT 0.0 OR yc7 LT 0.0) THEN	$
   BEGIN
   gen_loc, img7, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file7: ', file7
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc7 = centroid [0]
   yc7 = centroid [1]
   END

;--- Determine beginning & ending pixels to crop images to a square size.

x1beg = FIX ((xc1 - xcsq) + 0.5)
x2beg = FIX ((xc2 - xcsq) + 0.5)
x3beg = FIX ((xc3 - xcsq) + 0.5)
x4beg = FIX ((xc4 - xcsq) + 0.5)
x5beg = FIX ((xc5 - xcsq) + 0.5)
x6beg = FIX ((xc6 - xcsq) + 0.5)
x7beg = FIX ((xc7 - xcsq) + 0.5)

;--- Disc center adjustment.

x1beg = x1beg + 1
x2beg = x2beg + 1
x3beg = x3beg + 1
x4beg = x4beg + 1
x5beg = x5beg + 1
x6beg = x6beg + 1
x7beg = x7beg + 1

x1end = x1beg + xdimsq - 1
x2end = x2beg + xdimsq - 1
x3end = x3beg + xdimsq - 1
x4end = x4beg + xdimsq - 1
x5end = x5beg + xdimsq - 1
x6end = x6beg + xdimsq - 1
x7end = x7beg + xdimsq - 1

y1beg = FIX ((yc1 - ycsq) + 0.5)
y2beg = FIX ((yc2 - ycsq) + 0.5)
y3beg = FIX ((yc3 - ycsq) + 0.5)
y4beg = FIX ((yc4 - ycsq) + 0.5)
y5beg = FIX ((yc5 - ycsq) + 0.5)
y6beg = FIX ((yc6 - ycsq) + 0.5)
y7beg = FIX ((yc7 - ycsq) + 0.5)

;--- Disc center adjustment.

y1beg = y1beg + 1
y2beg = y2beg + 1
y3beg = y3beg + 1
y4beg = y4beg + 1
y5beg = y5beg + 1
y6beg = y6beg + 1
y7beg = y7beg + 1

y1end = y1beg + ydimsq - 1
y2end = y2beg + ydimsq - 1
y3end = y3beg + ydimsq - 1
y4end = y4beg + ydimsq - 1
y5end = y5beg + ydimsq - 1
y6end = y6beg + ydimsq - 1
y7end = y7beg + ydimsq - 1

;x1beg = FIX (xc1 + 0.5) - ipixrs - border
;x1end = x1beg + (ipixrs + border) * 2 - 1
;y1beg = yc1 - ipixrs - border
;y1end = y1beg + (ipixrs + border) * 2 - 1

x1dif = x1end - x1beg + 1
x2dif = x2end - x2beg + 1
x3dif = x3end - x3beg + 1
x4dif = x4end - x4beg + 1
x5dif = x5end - x5beg + 1
x6dif = x6end - x6beg + 1
x7dif = x7end - x7beg + 1

PRINT, 'x1beg/x1end: ', x1beg, x1end, ' x1dif: ', x1dif
PRINT, 'x2beg/x2end: ', x2beg, x2end, ' x2dif: ', x2dif
PRINT, 'x3beg/x3end: ', x3beg, x3end, ' x3dif: ', x3dif
PRINT, 'x4beg/x4end: ', x4beg, x4end, ' x4dif: ', x4dif
PRINT, 'x5beg/x5end: ', x5beg, x5end, ' x5dif: ', x5dif
PRINT, 'x6beg/x6end: ', x6beg, x6end, ' x6dif: ', x6dif
PRINT, 'x7beg/x7end: ', x7beg, x7end, ' x7dif: ', x7dif

;--- Crop each image to a square, centered at solar disk center.
;--- Also, convert to floating point.

img1f [*, *] = float (img1 [x1beg:x1end, y1beg:y1end])
img2f [*, *] = float (img2 [x2beg:x2end, y2beg:y2end])
img3f [*, *] = float (img3 [x3beg:x3end, y3beg:y3end])
img4f [*, *] = float (img4 [x4beg:x4end, y4beg:y4end])
img5f [*, *] = float (img5 [x5beg:x5end, y5beg:y5end])
img6f [*, *] = float (img6 [x6beg:x6end, y6beg:y6end])
img7f [*, *] = float (img7 [x7beg:x7end, y7beg:y7end])

;--- Display cropped images & label with filter designation.

TVSCL, img1f
XYOUTS,  5, 30, 'B3 Geocentric', /device, color=251
WAIT, 1
TVSCL, img2f
XYOUTS,  5, 30, 'B2 Geocentric', /device, color=251
WAIT, 1
TVSCL, img3f
XYOUTS,  5, 30, 'B1 Geocentric', /device, color=251
WAIT, 1
TVSCL, img4f
XYOUTS,  5, 30, 'He Geocentric', /device, color=251
WAIT, 1
TVSCL, img5f
XYOUTS,  5, 30, 'R1 Geocentric', /device, color=251
WAIT, 1
TVSCL, img6f
XYOUTS,  5, 30, 'R2 Geocentric', /device, color=251
WAIT, 1
TVSCL, img7f
XYOUTS,  5, 30, 'R3 Geocentric', /device, color=251
WAIT, 1

;--- Now, find sun center again in the square images.

;window, xsize=xdimsq, ysize=ydimsq

;gen_loc, img1f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc1 = centroid [0]
;yc1 = centroid [1]

;gen_loc, img2f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc2 = centroid [0]
;yc2 = centroid [1]

;gen_loc, img3f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc3 = centroid [0]
;yc3 = centroid [1]

;gen_loc, img4f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc4 = centroid [0]
;yc4 = centroid [1]

;gen_loc, img5f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc5 = centroid [0]
;yc5 = centroid [1]

;gen_loc, img6f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc6 = centroid [0]
;yc6 = centroid [1]

;gen_loc, img7f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc7 = centroid [0]
;yc7 = centroid [1]

;--- Modify FITS header to update the sun center information.

fxaddpar, hdu1, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu1, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu2, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu2, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu3, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu3, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu4, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu4, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu5, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu5, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu6, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu6, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]' 

fxaddpar, hdu7, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu7, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]' 

;--- Rotate image to heliocentric coordinates.

mag = 1.0
img1f = rot (img1f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)
img2f = rot (img2f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)
img3f = rot (img3f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)
img4f = rot (img4f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)
img5f = rot (img5f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)
img6f = rot (img6f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)
img7f = rot (img7f, pangle, mag, xcsq, ycsq, missing=0.0, /pivot)

;--- Display the square, rotated images.

TVSCL, img1f
XYOUTS,  5, 30, 'B3 Heliocentric', /device, color=251
WAIT, 1
TVSCL, img2f
XYOUTS,  5, 30, 'B2 Heliocentric', /device, color=251
WAIT, 1
TVSCL, img3f
XYOUTS,  5, 30, 'B1 Heliocentric', /device, color=251
WAIT, 1
TVSCL, img4f
XYOUTS,  5, 30, 'He Heliocentric', /device, color=251
WAIT, 1
TVSCL, img5f
XYOUTS,  5, 30, 'R1 Heliocentric', /device, color=251
WAIT, 1
TVSCL, img6f
XYOUTS,  5, 30, 'R2 Heliocentric', /device, color=251
WAIT, 1
TVSCL, img7f
XYOUTS,  5, 30, 'R3 Heliocentric', /device, color=251
WAIT, 1

;---  Update FITS header to indicate heliocentric coordinate orientation.

fxaddpar, hdu1, 'CROTA1', 0.0
fxaddpar, hdu2, 'CROTA1', 0.0
fxaddpar, hdu3, 'CROTA1', 0.0
fxaddpar, hdu4, 'CROTA1', 0.0
fxaddpar, hdu5, 'CROTA1', 0.0
fxaddpar, hdu6, 'CROTA1', 0.0
fxaddpar, hdu7, 'CROTA1', 0.0

fxaddpar, hdu1, 'CROTA2', 0.0
fxaddpar, hdu2, 'CROTA2', 0.0
fxaddpar, hdu3, 'CROTA2', 0.0
fxaddpar, hdu4, 'CROTA2', 0.0
fxaddpar, hdu5, 'CROTA2', 0.0
fxaddpar, hdu6, 'CROTA2', 0.0
fxaddpar, hdu7, 'CROTA2', 0.0

fxaddpar, hdu1, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu2, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu3, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu4, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu5, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu6, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu7, 'COORDNAM', 'HELIOCENTRIC'

;--- Write square, heliocentric images to disk.

writefits, file1c, FIX (img1f), hdu1
writefits, file1c, hsk1, ext1, /append
writefits, file2c, FIX (img2f), hdu2
writefits, file2c, hsk2, ext2, /append
writefits, file3c, FIX (img3f), hdu3
writefits, file3c, hsk3, ext3, /append
writefits, file4c, FIX (img4f), hdu4
writefits, file4c, hsk4, ext4, /append
writefits, file5c, FIX (img5f), hdu5
writefits, file5c, hsk5, ext5, /append
writefits, file6c, FIX (img6f), hdu6
writefits, file6c, hsk6, ext6, /append
writefits, file7c, FIX (img7f), hdu7
writefits, file7c, hsk7, ext7, /append

;--- Create the limb darkening correction file names.

syr1 = strmid (year1, 2, 2)
syr2 = strmid (year2, 2, 2)
syr3 = strmid (year3, 2, 2)
syr4 = strmid (year4, 2, 2)
syr5 = strmid (year5, 2, 2)
syr6 = strmid (year6, 2, 2)
syr7 = strmid (year7, 2, 2)

iyear1 = FIX (year1)
leap = LONG ((iyear1 MOD 4 EQ 0) AND (iyear1 MOD 100 NE 0) OR $
             (iyear1 MOD 400 EQ 0))
idoy1 = FIX (day1)
idoy2 = FIX (day2)
idoy3 = FIX (day3)
idoy4 = FIX (day4)
idoy5 = FIX (day5)
idoy6 = FIX (day6)
idoy7 = FIX (day7)

FOR i = 0, month1 - 1 DO idoy1 = idoy1 + daytab (i, leap)
FOR i = 0, month2 - 1 DO idoy2 = idoy2 + daytab (i, leap)
FOR i = 0, month3 - 1 DO idoy3 = idoy3 + daytab (i, leap)
FOR i = 0, month4 - 1 DO idoy4 = idoy4 + daytab (i, leap)
FOR i = 0, month5 - 1 DO idoy5 = idoy5 + daytab (i, leap)
FOR i = 0, month6 - 1 DO idoy6 = idoy6 + daytab (i, leap)
FOR i = 0, month7 - 1 DO idoy7 = idoy7 + daytab (i, leap)

sdoy1 = STRTRIM (STRING (idoy1), 2)
sdoy2 = STRTRIM (STRING (idoy2), 2)
sdoy3 = STRTRIM (STRING (idoy3), 2)
sdoy4 = STRTRIM (STRING (idoy4), 2)
sdoy5 = STRTRIM (STRING (idoy5), 2)
sdoy6 = STRTRIM (STRING (idoy6), 2)
sdoy7 = STRTRIM (STRING (idoy7), 2)

IF (idoy1 LT 10)  THEN sdoy1 = '00' + sdoy1
IF (idoy2 LT 10)  THEN sdoy2 = '00' + sdoy2
IF (idoy3 LT 10)  THEN sdoy3 = '00' + sdoy3
IF (idoy4 LT 10)  THEN sdoy4 = '00' + sdoy4
IF (idoy5 LT 10)  THEN sdoy5 = '00' + sdoy5
IF (idoy6 LT 10)  THEN sdoy6 = '00' + sdoy6
IF (idoy7 LT 10)  THEN sdoy7 = '00' + sdoy7

IF (idoy1 LT 100)  THEN sdoy1 = '0' + sdoy1
IF (idoy2 LT 100)  THEN sdoy2 = '0' + sdoy2
IF (idoy3 LT 100)  THEN sdoy3 = '0' + sdoy3
IF (idoy4 LT 100)  THEN sdoy4 = '0' + sdoy4
IF (idoy5 LT 100)  THEN sdoy5 = '0' + sdoy5
IF (idoy6 LT 100)  THEN sdoy6 = '0' + sdoy6
IF (idoy7 LT 100)  THEN sdoy7 = '0' + sdoy7

;--- Read image distance image.  Each pixel contains the radial distance
;    from sun center in Rsun units.

;syr4 = strmid (year4, 2, 2)
;sdoy4 = STRTRIM (STRING (idoy4), 2)

;filerad = syr4 + 'd' + sdoy4 + '.' + hour4 + minute4 + second4 + '.' + $
;	   code_obs4 + '.rad'
;imgrad = FLTARR (xdimsq, ydimsq)
;OPENR, 11, filerad
;READU, 11, imgrad			; File contains radial distances.
;CLOSE, 11

filerad = year4 + month4  + day4    + '.' +      $
	  hour4 + minute4 + second4 + '.' +      $
	  'chp.' + code_obs4 + '.rad' + '.fts'

PRINT, 'filerad: ', filerad

imgrad = readfits (filerad, hdurad)

PRINT, 'size (imgrad): ', SIZE (imgrad)
TVSCL, imgrad
XYOUTS,  5, 30, 'Radial Distance Image', /device, color=251
WAIT, 1

;--- Doppler correction image (B3 filter only).

;dc_name1 = syr1 + 'd' + sdoy1 + '.' + hour1 + minute1 + second1 + '.' + $
;	   code_obs1 + '.dc'

;filedc1 = year4 + month4  + day4    + '.' +      $
;	  hour4 + minute4 + second4 + '.' +      $
;	  'chp.' + code_obs4 + '.dc'

;imgdc1 = FLTARR (xdimsq, ydimsq)
;CLOSE, 11
;OPENR, 11, filedc1
;READU, 11, imgdc1                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;plot, imgdc1 [*, 468], title='B3 Equatorial doppler correction'
;WAIT, 3

;lct, '/home/stanger/color/dif.lut'
;TVSCL, imgdc1
;XYOUTS,  5, 30, 'Doppler Correction B3', /device, color=251
;WAIT, 1

;ld_name1 = syr1 + 'd' + sdoy1 + '.' + hour1 + minute1 + second1 + '.' + $
;	   code_obs1 + '.ldcor'
;ld_name2 = syr2 + 'd' + sdoy2 + '.' + hour2 + minute2 + second2 + '.' + $
;	   code_obs2 + '.ldcor'
;ld_name3 = syr3 + 'd' + sdoy3 + '.' + hour3 + minute3 + second3 + '.' + $
;	   code_obs3 + '.ldcor'
;ld_name4 = syr4 + 'd' + sdoy4 + '.' + hour4 + minute4 + second4 + '.' + $
;	   code_obs4 + '.ldcor'
;ld_name5 = syr5 + 'd' + sdoy5 + '.' + hour5 + minute5 + second5 + '.' + $
;	   code_obs5 + '.ldcor'
;ld_name6 = syr6 + 'd' + sdoy6 + '.' + hour6 + minute6 + second6 + '.' + $
;	   code_obs6 + '.ldcor'
;ld_name7 = syr7 + 'd' + sdoy7 + '.' + hour7 + minute7 + second7 + '.' + $
;	   code_obs7 + '.ldcor'

;PRINT, 'ld_name1: ', ld_name1
;PRINT, 'ld_name2: ', ld_name2
;PRINT, 'ld_name3: ', ld_name3
;PRINT, 'ld_name4: ', ld_name4
;PRINT, 'ld_name5: ', ld_name5
;PRINT, 'ld_name6: ', ld_name6
;PRINT, 'ld_name7: ', ld_name7

;--- Read limb darkening correction images.

;--- These images are created by the c-language program "chip_ldark2.c"
;    in the directory: /home/stanger/mlso/chip/als".

;CLOSE, 11
;OPENR, 11, ld_name1
;READU, 11, imgld1                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld1): ', SIZE (imgld1)

;CLOSE, 11
;OPENR, 11, ld_name2
;READU, 11, imgld2                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld2): ', SIZE (imgld2)

;CLOSE, 11
;OPENR, 11, ld_name3
;READU, 11, imgld3                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld3): ', SIZE (imgld3)

;CLOSE, 11
;OPENR, 11, ld_name4
;READU, 11, imgld4                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld4): ', SIZE (imgld4)

;CLOSE, 11
;OPENR, 11, ld_name5
;READU, 11, imgld5                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld5): ', SIZE (imgld5)

;CLOSE, 11
;OPENR, 11, ld_name6
;READU, 11, imgld6                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld6): ', SIZE (imgld6)

;CLOSE, 11
;OPENR, 11, ld_name7
;READU, 11, imgld7                      ; 936 x 936 floating point pixels.
;CLOSE, 11
;PRINT, 'size (imgld7): ', SIZE (imgld7)

;--- Limb darkening fit files.

;--- Create filenames for limb darkening fit images.

ldf_name1 = year1 + month1  + day1    + '.' +	$
	    hour1 + minute1 + second1 + '.' +	$
	    'chp.' + code_obs1 + '.ldf.fts'

ldf_name2 = year2 + month2  + day2    + '.' +	$
	    hour2 + minute2 + second2 + '.' +	$
	    'chp.' + code_obs2 + '.ldf.fts'

ldf_name3 = year3 + month3  + day3    + '.' +	$
	    hour3 + minute3 + second3 + '.' +	$
	    'chp.' + code_obs3 + '.ldf.fts'

ldf_name4 = year4 + month4  + day4    + '.' +	$
	    hour4 + minute4 + second4 + '.' +	$
	    'chp.' + code_obs4 + '.ldf.fts'

ldf_name5 = year5 + month5  + day5    + '.' +	$
	    hour5 + minute5 + second5 + '.' +	$
	    'chp.' + code_obs5 + '.ldf.fts'

ldf_name6 = year6 + month6  + day6    + '.' +	$
	    hour6 + minute6 + second6 + '.' +	$
	    'chp.' + code_obs6 + '.ldf.fts'

ldf_name7 = year7 + month7  + day7    + '.' +	$
	    hour7 + minute7 + second7 + '.' +	$
	    'chp.' + code_obs7 + '.ldf.fts'

;--- Read limb darkening fit images.

;--- These images are created by the IDL procedure "ld.pro"
;    in the directory: /home/stanger/idl/lib/band".

PRINT, ''
PRINT, 'ldf_name1: ', ldf_name1
imgldf1 = readfits (ldf_name1, hduldf1)
;CLOSE, 11
;OPENR, 11, ldf_name1
;READU, 11, imgldf1                      ; 936 x 936 floating point pixels.
;CLOSE, 11

PRINT, 'ldf_name2: ', ldf_name2
imgldf2 = readfits (ldf_name2, hduldf2)
;CLOSE, 11
;OPENR, 11, ldf_name2
;READU, 11, imgldf2                      ; 936 x 936 floating point pixels.
;CLOSE, 11

PRINT, 'ldf_name3: ', ldf_name3
imgldf3 = readfits (ldf_name3, hduldf3)

;CLOSE, 11
;OPENR, 11, ldf_name3
;READU, 11, imgldf3                      ; 936 x 936 floating point pixels.
;CLOSE, 11

PRINT, 'ldf_name4: ', ldf_name4
imgldf4 = readfits (ldf_name4, hduldf4)

;CLOSE, 11
;OPENR, 11, ldf_name4
;READU, 11, imgldf4                      ; 936 x 936 floating point pixels.
;CLOSE, 11

PRINT, 'ldf_name5: ', ldf_name5
imgldf5 = readfits (ldf_name5, hduldf5)

;CLOSE, 11
;OPENR, 11, ldf_name5
;READU, 11, imgldf5                      ; 936 x 936 floating point pixels.
;CLOSE, 11

PRINT, 'ldf_name6: ', ldf_name6
imgldf6 = readfits (ldf_name6, hduldf6)

;CLOSE, 11
;OPENR, 11, ldf_name6
;READU, 11, imgldf6                      ; 936 x 936 floating point pixels.
;CLOSE, 11

PRINT, 'ldf_name7: ', ldf_name7
imgldf7 = readfits (ldf_name7, hduldf7)

;CLOSE, 11
;OPENR, 11, ldf_name7
;READU, 11, imgldf7                      ; 936 x 936 floating point pixels.
;CLOSE, 11

;--- Invoke 'flmbiter.f' to compute limb darkening.

;rx = pixrs
;ry = pixrs
;niter = 20

;PRINT, 'Computing limb darkening for ', code_obs1, ' image.'
;imgld1 = ldf (img1f, xcsq, ycsq, rx, ry, niter)
;PRINT, 'Computing limb darkening for ', code_obs2, ' image.'
;imgld2 = ldf (img2f, xcsq, ycsq, rx, ry, niter)
;PRINT, 'Computing limb darkening for ', code_obs3, ' image.'
;imgld3 = ldf (img3f, xcsq, ycsq, rx, ry, niter)
;PRINT, 'Computing limb darkening for ', code_obs4, ' image.'
;imgld4 = ldf (img4f, xcsq, ycsq, rx, ry, niter)
;PRINT, 'Computing limb darkening for ', code_obs5, ' image.'
;imgld5 = ldf (img5f, xcsq, ycsq, rx, ry, niter)
;PRINT, 'Computing limb darkening for ', code_obs6, ' image.'
;imgld6 = ldf (img6f, xcsq, ycsq, rx, ry, niter)
;PRINT, 'Computing limb darkening for ', code_obs7, ' image.'
;imgld7 = ldf (img7f, xcsq, ycsq, rx, ry, niter)

;ldmin1 = MIN (imgld1, max=ldmax1)
;ldmin2 = MIN (imgld2, max=ldmax2)
;ldmin3 = MIN (imgld3, max=ldmax3)
;ldmin4 = MIN (imgld4, max=ldmax4)
;ldmin5 = MIN (imgld5, max=ldmax5)
;ldmin6 = MIN (imgld6, max=ldmax6)
;ldmin7 = MIN (imgld7, max=ldmax7)

;PRINT, 'ldmin1/ldmax1: ', ldmin1, ldmax1
;PRINT, 'ldmin2/ldmax2: ', ldmin2, ldmax2
;PRINT, 'ldmin3/ldmax3: ', ldmin3, ldmax3
;PRINT, 'ldmin4/ldmax4: ', ldmin4, ldmax4
;PRINT, 'ldmin5/ldmax5: ', ldmin5, ldmax5
;PRINT, 'ldmin6/ldmax6: ', ldmin6, ldmax6
;PRINT, 'ldmin7/ldmax7: ', ldmin7, ldmax7

;--- Display the limb darkening correction images.

lct, '/home/stanger/color/chip1.lut'

TVSCL, imgldf1
XYOUTS,  5, 30, 'Limb Darkening Fit B3', /device, color=251
WAIT, 1

TVSCL, imgldf2
XYOUTS,  5, 30, 'Limb Darkening Fit B2', /device, color=251
WAIT, 1

TVSCL, imgldf3
XYOUTS,  5, 30, 'Limb Darkening Fit B1', /device, color=251
WAIT, 1

TVSCL, imgldf4
XYOUTS,  5, 30, 'Limb Darkening Fit He', /device, color=251
WAIT, 1

TVSCL, imgldf5
XYOUTS,  5, 30, 'Limb Darkening Fit R1', /device, color=251
WAIT, 1

TVSCL, imgldf6
XYOUTS,  5, 30, 'Limb Darkening Fit R2', /device, color=251
WAIT, 1

TVSCL, imgldf7
XYOUTS,  5, 30, 'Limb Darkening Fit R3', /device
WAIT, 1

ldfmin1 = MIN (imgldf1, max=ldfmax1)
ldfmin2 = MIN (imgldf2, max=ldfmax2)
ldfmin3 = MIN (imgldf3, max=ldfmax3)
ldfmin4 = MIN (imgldf4, max=ldfmax4)
ldfmin5 = MIN (imgldf5, max=ldfmax5)
ldfmin6 = MIN (imgldf6, max=ldfmax6)
ldfmin7 = MIN (imgldf7, max=ldfmax7)

PRINT, 'min/max (imgldf1): ', ldfmin1, ldfmax1
PRINT, 'min/max (imgldf2): ', ldfmin2, ldfmax2
PRINT, 'min/max (imgldf3): ', ldfmin3, ldfmax3
PRINT, 'min/max (imgldf4): ', ldfmin4, ldfmax4
PRINT, 'min/max (imgldf5): ', ldfmin5, ldfmax5
PRINT, 'min/max (imgldf6): ', ldfmin6, ldfmax6
PRINT, 'min/max (imgldf7): ', ldfmin7, ldfmax7

;ldfmax1 = maxbrt (imgldf1, imgrad)
;ldfmax2 = maxbrt (imgldf2, imgrad)
;ldfmax3 = maxbrt (imgldf3, imgrad)
;ldfmax4 = maxbrt (imgldf4, imgrad)
;ldfmax5 = maxbrt (imgldf5, imgrad)
;ldfmax6 = maxbrt (imgldf6, imgrad)
;ldfmax7 = maxbrt (imgldf7, imgrad)

;--- Compute limb darkening correction.

discpix = WHERE (imgrad LT 1.0)
sizedisc = SIZE (discpix)
ndisc = sizedisc [1]
PRINT, 'size (discpix): ', SIZE (discpix)

imgldc1 = FLTARR (xdimsq, ydimsq)
imgldc2 = FLTARR (xdimsq, ydimsq)
imgldc3 = FLTARR (xdimsq, ydimsq)
imgldc4 = FLTARR (xdimsq, ydimsq)
imgldc5 = FLTARR (xdimsq, ydimsq)
imgldc6 = FLTARR (xdimsq, ydimsq)
imgldc7 = FLTARR (xdimsq, ydimsq)

;--- Default value for limb darkening correction is 1.0

imgldc1 [*, *] = 1.0
imgldc2 [*, *] = 1.0
imgldc3 [*, *] = 1.0
imgldc4 [*, *] = 1.0
imgldc5 [*, *] = 1.0
imgldc6 [*, *] = 1.0
imgldc7 [*, *] = 1.0

;--- For pixels ON the solar disc, compute limb darkening correction.
;--- For pixels with a limb darkening value GT 1.0,
;--- compute limb darkening CORRECTION.
;
;--- LDC [x,y] = LDF (maximum) / LDF [x,y]

nonneg = WHERE (imgldf1 GT 1.0)
PRINT, 'size (nonneg): ', SIZE (nonneg)
imgldc1 [nonneg] = ldfmax1 / imgldf1 [nonneg]
nonneg = WHERE (imgldf2 GT 1.0)
imgldc2 [nonneg] = ldfmax2 / imgldf2 [nonneg]
nonneg = WHERE (imgldf3 GT 1.0)
imgldc3 [nonneg] = ldfmax3 / imgldf3 [nonneg]
nonneg = WHERE (imgldf4 GT 1.0)
imgldc4 [nonneg] = ldfmax4 / imgldf4 [nonneg]
nonneg = WHERE (imgldf5 GT 1.0)
imgldc5 [nonneg] = ldfmax5 / imgldf5 [nonneg]
nonneg = WHERE (imgldf6 GT 1.0)
imgldc6 [nonneg] = ldfmax6 / imgldf6 [nonneg]
nonneg = WHERE (imgldf7 GT 1.0)
imgldc7 [nonneg] = ldfmax7 / imgldf7 [nonneg]

PRINT, 'min/max (imgldc1): ', min (imgldc1), max (imgldc1)
PRINT, 'min/max (imgldc2): ', min (imgldc2), max (imgldc2)
PRINT, 'min/max (imgldc3): ', min (imgldc3), max (imgldc3)
PRINT, 'min/max (imgldc4): ', min (imgldc4), max (imgldc4)
PRINT, 'min/max (imgldc5): ', min (imgldc5), max (imgldc5)
PRINT, 'min/max (imgldc6): ', min (imgldc6), max (imgldc6)
PRINT, 'min/max (imgldc7): ', min (imgldc7), max (imgldc7)
PRINT, ''

;--- Clip limb darkening correction to 2.0

bigloc = WHERE (imgldc1 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc1 [bigloc] = 2.0

bigloc = WHERE (imgldc2 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc2 [bigloc] = 2.0

bigloc = WHERE (imgldc3 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc3 [bigloc] = 2.0

bigloc = WHERE (imgldc4 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc4 [bigloc] = 2.0

bigloc = WHERE (imgldc5 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc5 [bigloc] = 2.0

bigloc = WHERE (imgldc6 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc6 [bigloc] = 2.0

bigloc = WHERE (imgldc7 GT 2.0)
;IF (N_ELEMENTS (bigloc) GT 0) THEN imgldc7 [bigloc] = 2.0

ldcmin1 = MIN (imgldc1, max=ldcmax1)
ldcmin2 = MIN (imgldc2, max=ldcmax2)
ldcmin3 = MIN (imgldc3, max=ldcmax3)
ldcmin4 = MIN (imgldc4, max=ldcmax4)
ldcmin5 = MIN (imgldc5, max=ldcmax5)
ldcmin6 = MIN (imgldc6, max=ldcmax6)
ldcmin7 = MIN (imgldc7, max=ldcmax7)

PRINT, 'min/max (imgldc1): ', ldcmin1, ldcmax1
PRINT, 'min/max (imgldc2): ', ldcmin2, ldcmax2
PRINT, 'min/max (imgldc3): ', ldcmin3, ldcmax3
PRINT, 'min/max (imgldc4): ', ldcmin4, ldcmax4
PRINT, 'min/max (imgldc5): ', ldcmin5, ldcmax5
PRINT, 'min/max (imgldc6): ', ldcmin6, ldcmax6
PRINT, 'min/max (imgldc7): ', ldcmin7, ldcmax7
PRINT, ''

;--- Display the limb darkening correction images.

TVSCL, imgldc1
XYOUTS,  5, 30, 'Limb Darkening Correction B3', /device, color=251
WAIT, 1

TVSCL, imgldc2
XYOUTS,  5, 30, 'Limb Darkening Correction B2', /device, color=251
WAIT, 1

TVSCL, imgldc3
XYOUTS,  5, 30, 'Limb Darkening Correction B1', /device, color=251
WAIT, 1

TVSCL, imgldc4
XYOUTS,  5, 30, 'Limb Darkening Correction He', /device, color=251
WAIT, 1

TVSCL, imgldc5
XYOUTS,  5, 30, 'Limb Darkening Correction R1', /device, color=251
WAIT, 1

TVSCL, imgldc6
XYOUTS,  5, 30, 'Limb Darkening Correction R2', /device, color=251
WAIT, 1

TVSCL, imgldc7
XYOUTS,  5, 30, 'Limb Darkening Correction R3', /device, color=251
WAIT, 1

;--- Create filenames for limb darkening correction images.

file1c = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.ldc.fts'

file2c = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.ldc.fts'

file3c = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.ldc.fts'

file4c = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.ldc.fts'

file5c = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.ldc.fts'

file6c = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.ldc.fts'

file7c = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.ldc.fts'

;--- Write limb darkening correction images to disc.

hduldc1 = hdu1
hduldc2 = hdu2
hduldc3 = hdu3
hduldc4 = hdu4
hduldc5 = hdu5
hduldc6 = hdu6
hduldc7 = hdu7

fxaddpar, hduldc1, 'TYPE-OBS', 'LD COR'
fxaddpar, hduldc2, 'TYPE-OBS', 'LD COR'
fxaddpar, hduldc3, 'TYPE-OBS', 'LD COR'
fxaddpar, hduldc4, 'TYPE-OBS', 'LD COR'
fxaddpar, hduldc5, 'TYPE-OBS', 'LD COR'
fxaddpar, hduldc6, 'TYPE-OBS', 'LD COR'
fxaddpar, hduldc7, 'TYPE-OBS', 'LD COR'

fxaddpar, hduldc1, 'TYPE-DP', 'LD COR'
fxaddpar, hduldc2, 'TYPE-DP', 'LD COR'
fxaddpar, hduldc3, 'TYPE-DP', 'LD COR'
fxaddpar, hduldc4, 'TYPE-DP', 'LD COR'
fxaddpar, hduldc5, 'TYPE-DP', 'LD COR'
fxaddpar, hduldc6, 'TYPE-DP', 'LD COR'
fxaddpar, hduldc7, 'TYPE-DP', 'LD COR'

fxaddpar, hduldc1, 'DATAMIN', ldcmin1
fxaddpar, hduldc1, 'DATAMAX', ldcmax1
fxaddpar, hduldc1, 'DISPMIN', ldcmin1
fxaddpar, hduldc1, 'DISPMAX', ldcmax1
fxaddpar, hduldc2, 'DATAMIN', ldcmin2
fxaddpar, hduldc2, 'DATAMAX', ldcmax2
fxaddpar, hduldc2, 'DISPMIN', ldcmin2
fxaddpar, hduldc2, 'DISPMAX', ldcmax2
fxaddpar, hduldc3, 'DATAMIN', ldcmin3
fxaddpar, hduldc3, 'DATAMAX', ldcmax3
fxaddpar, hduldc3, 'DISPMIN', ldcmin3
fxaddpar, hduldc3, 'DISPMAX', ldcmax3
fxaddpar, hduldc4, 'DATAMIN', ldcmin4
fxaddpar, hduldc4, 'DATAMAX', ldcmax4
fxaddpar, hduldc4, 'DISPMIN', ldcmin4
fxaddpar, hduldc4, 'DISPMAX', ldcmax4
fxaddpar, hduldc5, 'DATAMIN', ldcmin5
fxaddpar, hduldc5, 'DATAMAX', ldcmax5
fxaddpar, hduldc5, 'DISPMIN', ldcmin5
fxaddpar, hduldc5, 'DISPMAX', ldcmax5
fxaddpar, hduldc6, 'DATAMIN', ldcmin6
fxaddpar, hduldc6, 'DATAMAX', ldcmax6
fxaddpar, hduldc6, 'DISPMIN', ldcmin6
fxaddpar, hduldc6, 'DISPMAX', ldcmax6
fxaddpar, hduldc7, 'DATAMIN', ldcmin7
fxaddpar, hduldc7, 'DATAMAX', ldcmax7
fxaddpar, hduldc7, 'DISPMIN', ldcmin7
fxaddpar, hduldc7, 'DISPMAX', ldcmax7

fxaddpar, hduldc1, 'HISTORY', 'Limb darkening correction.'
fxaddpar, hduldc2, 'HISTORY', 'Limb darkening correction.'
fxaddpar, hduldc3, 'HISTORY', 'Limb darkening correction.'
fxaddpar, hduldc4, 'HISTORY', 'Limb darkening correction.'
fxaddpar, hduldc5, 'HISTORY', 'Limb darkening correction.'
fxaddpar, hduldc6, 'HISTORY', 'Limb darkening correction.'
fxaddpar, hduldc7, 'HISTORY', 'Limb darkening correction.'

writefits, file1c, imgldc1, hduldc1
writefits, file1c, hsk1, ext1, /append
writefits, file2c, imgldc2, hduldc2
writefits, file2c, hsk2, ext2, /append
writefits, file3c, imgldc3, hduldc3
writefits, file3c, hsk3, ext3, /append
writefits, file4c, imgldc4, hduldc4
writefits, file4c, hsk4, ext4, /append
writefits, file5c, imgldc5, hduldc5
writefits, file5c, hsk5, ext5, /append
writefits, file6c, imgldc6, hduldc6
writefits, file6c, hsk6, ext6, /append
writefits, file7c, imgldc7, hduldc7
writefits, file7c, hsk7, ext7, /append

;--- Remove limb darkening.

;img1f = img1f * imgldc1
;img2f = img2f * imgldc2
;img3f = img3f * imgldc3
;img4f = img4f * imgldc4
;img5f = img5f * imgldc5
;img6f = img6f * imgldc6
;img7f = img7f * imgldc7

;--- Use dsr limb darkening for dsr image.
;--- Use ssr limb darkening for all other images.
;
;--- Use computed limb darkening determined for each image (12 Oct 2000).

;img1f = img1f * imgldc1 + imgdc1

img1f = img1f * imgldc1
img2f = img2f * imgldc2
img3f = img3f * imgldc3
img4f = img4f * imgldc4
img5f = img5f * imgldc5
img6f = img6f * imgldc6
img7f = img7f * imgldc7

;--- Display the limb-darkening corrected images.

lct, '/home/stanger/color/chip1.lut'
TVSCL, img1f
WAIT, 1
XYOUTS,  5, 30, 'B3 Heliocentric LD removed', /device, color=251
WAIT, 1
TVSCL, img2f
XYOUTS,  5, 30, 'B2 Heliocentric LD removed', /device, color=251
WAIT, 1
TVSCL, img3f
XYOUTS,  5, 30, 'B1 Heliocentric LD removed', /device, color=251
WAIT, 1
TVSCL, img4f
XYOUTS,  5, 30, 'He Heliocentric LD removed', /device, color=251
WAIT, 1
TVSCL, img5f
XYOUTS,  5, 30, 'R1 Heliocentric LD removed', /device, color=251
WAIT, 1
TVSCL, img6f
XYOUTS,  5, 30, 'R2 Heliocentric LD removed', /device, color=251
WAIT, 1
TVSCL, img7f
XYOUTS,  5, 30, 'R3 Heliocentric LD removed', /device, color=251
WAIT, 1

;--- Create filenames for limb darkening corrected images.

file1c = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.ld.fts'

file2c = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.ld.fts'

file3c = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.ld.fts'

file4c = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.ld.fts'

file5c = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.ld.fts'

file6c = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.ld.fts'

file7c = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.ld.fts'

;---  Update FITS header to indicate limb darkening correction.

hduld1 = hdu1
hduld2 = hdu2
hduld3 = hdu3
hduld4 = hdu4
hduld5 = hdu5
hduld6 = hdu6
hduld7 = hdu7

fxaddpar, hduld1, 'TYPE-DP', 'LD REM'
fxaddpar, hduld2, 'TYPE-DP', 'LD REM'
fxaddpar, hduld3, 'TYPE-DP', 'LD REM'
fxaddpar, hduld4, 'TYPE-DP', 'LD REM'
fxaddpar, hduld5, 'TYPE-DP', 'LD REM'
fxaddpar, hduld6, 'TYPE-DP', 'LD REM'
fxaddpar, hduld7, 'TYPE-DP', 'LD REM'

fxaddpar, hduld1, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hduld2, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hduld3, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hduld4, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hduld5, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hduld6, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hduld7, 'HISTORY', 'Limb darkening has been removed.'

;--- Write square, limb darkening-corrected images to disk in FITS format.

writefits, file1c, FIX (img1f), hduld1
writefits, file1c, hsk1, ext1, /append
writefits, file2c, FIX (img2f), hduld2
writefits, file2c, hsk2, ext2, /append
writefits, file3c, FIX (img3f), hduld3
writefits, file3c, hsk3, ext3, /append
writefits, file4c, FIX (img4f), hduld4
writefits, file4c, hsk4, ext4, /append
writefits, file5c, FIX (img5f), hduld5
writefits, file5c, hsk5, ext5, /append
writefits, file6c, FIX (img6f), hduld6
writefits, file6c, hsk6, ext6, /append
writefits, file7c, FIX (img7f), hduld7
writefits, file7c, hsk7, ext7, /append

;--- Find average of brightest 5% pixels on the solar disc.

;bmax1 = maxbrt (img1f, imgrad)
bmax2 = maxbrt (img2f, imgrad)
;bmax3 = maxbrt (img3f, imgrad)
;bmax4 = maxbrt (img4f, imgrad)
;bmax5 = maxbrt (img5f, imgrad)
;bmax6 = maxbrt (img6f, imgrad)
;bmax7 = maxbrt (img7f, imgrad)

;PRINT, 'bmax1: ', bmax1
PRINT, 'bmax2: ', bmax2
;PRINT, 'bmax3: ', bmax3
;PRINT, 'bmax4: ', bmax4
;PRINT, 'bmax5: ', bmax5
;PRINT, 'bmax6: ', bmax6
;PRINT, 'bmax7: ', bmax7
;PRINT, ''

;--- Determine continuum brightness.

bcont = bmax2 / 0.97

PRINT, 'bcont: ', bcont

;--- Find brightest pixels (top 20%) within an equatorial band
;    of a Helium-1 image.

brtloc = brtpix (img4f, pixrs, 20.0, ixbeg, ixend, iybeg, iyend)

sizeloc = SIZE (brtloc)				; Vector array.
PRINT, 'size (brtloc): ', sizeloc
nloc = LONG (sizeloc [3])

;--- Extract a horizontal "band" across the equator of each image (except He-I).

xdimband = long (ixend - ixbeg + 1)
ydimband = long (iyend - iybeg + 1)
npxband  = xdimband * ydimband

band1 = FLTARR (xdimband, ydimband)
band2 = FLTARR (xdimband, ydimband)
band3 = FLTARR (xdimband, ydimband)
band4 = FLTARR (xdimband, ydimband)
band5 = FLTARR (xdimband, ydimband)
band6 = FLTARR (xdimband, ydimband)
band7 = FLTARR (xdimband, ydimband)

band1 = img1f [ixbeg:ixend, iybeg:iyend]
band2 = img2f [ixbeg:ixend, iybeg:iyend]
band3 = img3f [ixbeg:ixend, iybeg:iyend]
band4 = img4f [ixbeg:ixend, iybeg:iyend]
band5 = img5f [ixbeg:ixend, iybeg:iyend]
band6 = img6f [ixbeg:ixend, iybeg:iyend]
band7 = img7f [ixbeg:ixend, iybeg:iyend]

;--- Use brightest He-I pixel locations to compute average brightness
;--- for each filter (other than He-I).

brtavg1 = TOTAL (band1 (brtloc)) / nloc
brtavg2 = TOTAL (band2 (brtloc)) / nloc
brtavg3 = TOTAL (band3 (brtloc)) / nloc

brtavg4 = TOTAL (band4 (brtloc)) / nloc

brtavg5 = TOTAL (band5 (brtloc)) / nloc
brtavg6 = TOTAL (band6 (brtloc)) / nloc
brtavg7 = TOTAL (band7 (brtloc)) / nloc

PRINT, ''
PRINT, 'Average limb-darkening-corrected brightness: (for brightest 20% pixels)'
PRINT, 'brtavg1: ', brtavg1
PRINT, 'brtavg2: ', brtavg2
PRINT, 'brtavg3: ', brtavg3
PRINT, 'brtavg4: ', brtavg4
PRINT, 'brtavg5: ', brtavg5
PRINT, 'brtavg6: ', brtavg6
PRINT, 'brtavg7: ', brtavg7
PRINT, ''

;--- Compute normalization correction terms.

nc1 = bcont - brtavg1
nc2 = bcont - brtavg2
nc6 = bcont - brtavg6
nc7 = bcont - brtavg7

;nc3 = bcont - brtavg3
;nc4 = bcont - brtavg4
;nc5 = bcont - brtavg5

;nc3 = 0.8 * nc2			; Theoretical normalization.
;nc4 = 0.4 * nc2 + 0.04 * nc6		; Theoretical normalization.
;nc5 = 0.2 * nc2 + 0.6  * nc6		; Theoretical normalization.

;nc3 = 0.745 * nc2			; Experimental value.
;nc4 = 0.74  * nc2 + 0.075 * nc6		; Experimental value.
;nc5 = 0.07  * nc2 + 0.270 * nc6		; Experimental value.

;--- Normalize inner 3 filters the same way as the outer 4 filters.

nc3 = bcont - brtavg3			
nc4 = bcont - brtavg4
nc5 = bcont - brtavg5

;nc1 = bcont - bmax1
;nc2 = bcont - bmax2
;nc3 = bcont - bmax3
;nc4 = bcont - bmax4
;nc5 = bcont - bmax5
;nc6 = bcont - bmax6
;nc7 = bcont - bmax7

;--- Normalize filters by computing an additive correction value
;    based upon the difference of the average maximum brightness
;    from the continuum brightness.

;normavg1 = brtavg1 + nc1
;normavg2 = brtavg2 + nc2
;normavg3 = brtavg3 + nc3
;normavg4 = brtavg4 + nc4
;normavg5 = brtavg5 + nc5
;normavg6 = brtavg6 + nc6
;normavg7 = brtavg7 + nc7

;PRINT, 'Average normalized brightness: (for brightest 20% pixels)'
;PRINT, 'normavg1: ', normavg1
;PRINT, 'normavg2: ', normavg2
;PRINT, 'normavg3: ', normavg3
;PRINT, 'normavg4: ', normavg4
;PRINT, 'normavg5: ', normavg5
;PRINT, 'normavg6: ', normavg6
;PRINT, 'normavg7: ', normavg7
;PRINT, ''

PRINT, 'Normalization corrections (additive): (for brightest 20% pixels)'
PRINT, 'nc1: ', nc1
PRINT, 'nc2: ', nc2
PRINT, 'nc3: ', nc3
PRINT, 'nc4: ', nc4
PRINT, 'nc5: ', nc5
PRINT, 'nc6: ', nc6
PRINT, 'nc7: ', nc7
PRINT, ''

img1n = img1f + nc1
img2n = img2f + nc2
img3n = img3f + nc3
img4n = img4f + nc4
img5n = img5f + nc5
img6n = img6f + nc6
img7n = img7f + nc7

;--- Recompute brightest pixel average, using normalized images.

band1 = img1n [ixbeg:ixend, iybeg:iyend]
band2 = img2n [ixbeg:ixend, iybeg:iyend]
band3 = img3n [ixbeg:ixend, iybeg:iyend]
band4 = img4n [ixbeg:ixend, iybeg:iyend]
band5 = img5n [ixbeg:ixend, iybeg:iyend]
band6 = img6n [ixbeg:ixend, iybeg:iyend]
band7 = img7n [ixbeg:ixend, iybeg:iyend]

;--- Use brightest He-I pixel locations to compute average brightness
;--- for each filter (other than He-I).

brtavg1 = TOTAL (band1 (brtloc)) / nloc
brtavg2 = TOTAL (band2 (brtloc)) / nloc
brtavg3 = TOTAL (band3 (brtloc)) / nloc

brtavg4 = TOTAL (band4 (brtloc)) / nloc

brtavg5 = TOTAL (band5 (brtloc)) / nloc
brtavg6 = TOTAL (band6 (brtloc)) / nloc
brtavg7 = TOTAL (band7 (brtloc)) / nloc

PRINT, ''
PRINT, 'Average normalized brightness: (for brightest 20% pixels)'
PRINT, 'brtavg1: ', brtavg1
PRINT, 'brtavg2: ', brtavg2
PRINT, 'brtavg3: ', brtavg3
PRINT, 'brtavg4: ', brtavg4
PRINT, 'brtavg5: ', brtavg5
PRINT, 'brtavg6: ', brtavg6
PRINT, 'brtavg7: ', brtavg7
PRINT, ''

;nmax1 = maxbrt (img1n, imgrad)
;nmax2 = maxbrt (img2n, imgrad)
;nmax3 = maxbrt (img3n, imgrad)
;nmax4 = maxbrt (img4n, imgrad)
;nmax5 = maxbrt (img5n, imgrad)
;nmax6 = maxbrt (img6n, imgrad)
;nmax7 = maxbrt (img7n, imgrad)

;PRINT, 'nmax1: ', nmax1
;PRINT, 'nmax2: ', nmax2
;PRINT, 'nmax3: ', nmax3
;PRINT, 'nmax4: ', nmax4
;PRINT, 'nmax5: ', nmax5
;PRINT, 'nmax6: ', nmax6
;PRINT, 'nmax7: ', nmax7
;PRINT, ''

;--- Display normalized images & label accordingly.

TVSCL, img1n
XYOUTS,  5, 30, 'B3 Norm', /device, color=251
WAIT, 1
TVSCL, img2n
XYOUTS,  5, 30, 'B2 Norm', /device, color=251
WAIT, 1
TVSCL, img3n
XYOUTS,  5, 30, 'B1 Norm', /device, color=251
WAIT, 1
TVSCL, img4n
XYOUTS,  5, 30, 'He Norm', /device, color=251
WAIT, 1
TVSCL, img5n
XYOUTS,  5, 30, 'R1 Norm', /device, color=251
WAIT, 1
TVSCL, img6n
XYOUTS,  5, 30, 'R2 Norm', /device, color=251
WAIT, 1
TVSCL, img7n
XYOUTS,  5, 30, 'R3 Norm', /device, color=251
WAIT, 1

;--- Create filenames for output normalized images.

file1n = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.norm.fts'

file2n = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.norm.fts'

file3n = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.norm.fts'

file4n = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.norm.fts'

file5n = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.norm.fts'

file6n = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.norm.fts'

file7n = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.norm.fts'

;--- Write normalized images to disk.

imin1  = MIN (img1n, max = imax1)
imin2  = MIN (img2n, max = imax2)
imin3  = MIN (img3n, max = imax3)
imin4  = MIN (img4n, max = imax4)
imin5  = MIN (img5n, max = imax5)
imin6  = MIN (img6n, max = imax6)
imin7  = MIN (img7n, max = imax7)

fxaddpar, hdu1, 'TYPE-DP', 'NORM'
fxaddpar, hdu2, 'TYPE-DP', 'NORM'
fxaddpar, hdu3, 'TYPE-DP', 'NORM'
fxaddpar, hdu4, 'TYPE-DP', 'NORM'
fxaddpar, hdu5, 'TYPE-DP', 'NORM'
fxaddpar, hdu6, 'TYPE-DP', 'NORM'
fxaddpar, hdu7, 'TYPE-DP', 'NORM'

fxaddpar, hdu1, 'DATAMIN', imin1
fxaddpar, hdu1, 'DATAMAX', imax1
fxaddpar, hdu1, 'DISPMIN', imin1
fxaddpar, hdu1, 'DISPMAX', imax1
fxaddpar, hdu2, 'DATAMIN', imin2
fxaddpar, hdu2, 'DATAMAX', imax2
fxaddpar, hdu2, 'DISPMIN', imin2
fxaddpar, hdu2, 'DISPMAX', imax2
fxaddpar, hdu3, 'DATAMIN', imin3
fxaddpar, hdu3, 'DATAMAX', imax3
fxaddpar, hdu3, 'DISPMIN', imin3
fxaddpar, hdu3, 'DISPMAX', imax3
fxaddpar, hdu4, 'DATAMIN', imin4
fxaddpar, hdu4, 'DATAMAX', imax4
fxaddpar, hdu4, 'DISPMIN', imin4
fxaddpar, hdu4, 'DISPMAX', imax4
fxaddpar, hdu5, 'DATAMIN', imin5
fxaddpar, hdu5, 'DATAMAX', imax5
fxaddpar, hdu5, 'DISPMIN', imin5
fxaddpar, hdu5, 'DISPMAX', imax5
fxaddpar, hdu6, 'DATAMIN', imin6
fxaddpar, hdu6, 'DATAMAX', imax6
fxaddpar, hdu6, 'DISPMIN', imin6
fxaddpar, hdu6, 'DISPMAX', imax6
fxaddpar, hdu7, 'DATAMIN', imin7
fxaddpar, hdu7, 'DATAMAX', imax7
fxaddpar, hdu7, 'DISPMIN', imin7
fxaddpar, hdu7, 'DISPMAX', imax7

fxaddpar, hdu1, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu2, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu3, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu4, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu5, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu6, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu7, 'HISTORY', 'NORMALIZED BRIGHTNESS'

writefits, file1n, FIX (img1n), hdu1
writefits, file1n, hsk1, ext1, /append
writefits, file2n, FIX (img2n), hdu2
writefits, file2n, hsk2, ext2, /append
writefits, file3n, FIX (img3n), hdu3
writefits, file3n, hsk3, ext3, /append
writefits, file4n, FIX (img4n), hdu4
writefits, file4n, hsk4, ext4, /append
writefits, file5n, FIX (img5n), hdu5
writefits, file5n, hsk5, ext5, /append
writefits, file6n, FIX (img6n), hdu6
writefits, file6n, hsk6, ext6, /append
writefits, file7n, FIX (img7n), hdu7
writefits, file7n, hsk7, ext7, /append

;--- Compute He-I image by subtracting continuum from on-band image.

imghe1 = img4n - bcont
he1min = MIN (imghe1, max=he1max)
hduhe1 = hdu4
fxaddpar, hduhe1, 'TYPE-DP', 'CAL'
fxaddpar, hduhe1, 'DATAMIN', he1min
fxaddpar, hduhe1, 'DATAMAX', he1max
fxaddpar, hduhe1, 'DISPMIN', he1min
fxaddpar, hduhe1, 'DISPMAX', he1max
writefits, 'he1.fts', FIX (imghe1), hduhe1
writefits, 'he1.fts', hsk4, ext4, /append

TVSCL, imghe1
XYOUTS,  5, 30, 'He-I', /device, color=251
WAIT, 1

;--- Compute average of all 7 normalized images.

imgavg = (img1n + img2n + img3n + img4n + img5n + img6n + img7n) / 7.0
TVSCL, imgavg
XYOUTS,  5, 30, 'Average Normalized', /device, color=251
WAIT, 1

;PRINT, 'size (img): ', SIZE (imgavg)

imin = MIN (imgavg, max=imax)

PRINT, 'average image min/max: ', imin, imax

;--- Compute velocity image.

imgdif0 = 1.0 - ( imgavg / bcont)
dif0min = MIN (imgdif0, max=dif0max)
PRINT, 'dif0min/dif0max: ', dif0min, dif0max

imgdif1 = (img3n - img5n) / bcont
imgdif2 = (img2n - img6n) / bcont
imgdif3 = (img1n - img7n) / bcont

d1min = MIN (imgdif1, max=d1max)
d2min = MIN (imgdif2, max=d2max)
d3min = MIN (imgdif3, max=d3max)

PRINT, ''
PRINT, 'd1min/d1max: ', d1min, d1max
PRINT, 'd2min/d2max: ', d2min, d2max
PRINT, 'd3min/d3max: ', d3min, d3max
PRINT, ''

;--- Display "component images" of the velocity image equation.

lct, '/home/stanger/color/dif.lut'
TVLCT, red, green, blue, /get

IF (abs (dif0min) LT abs (dif0max)) THEN		$
   BEGIN
   d0min =   dif0min
   d0max = - dif0min
   END							$
else							$
   BEGIN
   d0min = - dif0max
   d0max =   dif0max
   END
imgb = bytscl (imgdif0, min=d0min, max=d0max, top=249, /nan)

;TVSCL, imgdif0
TV, imgb
XYOUTS,   5,  70, '1.0 - Iave/Icont', /device, color=251
XYOUTS,   5,  50, 'MIN: ' + STRING (d0min), /device, color=251
XYOUTS,   5,  30, 'MAX: ' + STRING (d0max), /device, color=251
tv, cbar
WRITE_GIF, 'dif0.gif', tvrd (), red, green, blue
WAIT, 1

hdudif0 = hdu4
fxaddpar, hdudif0, 'TYPE-OBS', '1.0-IAVE/ICONT'
fxaddpar, hdudif0, 'TYPE-DP',  'NORM'
fxaddpar, hdudif0, 'DATAMIN', dif0min
fxaddpar, hdudif0, 'DATAMAX', dif0max
fxaddpar, hdudif0, 'DISPMIN', dif0min
fxaddpar, hdudif0, 'DISPMAX', dif0max
filedif0 = year4 + month4  + day4    + '.' +	$
	   hour4 + minute4 + second4 + '.' +	$
	   'chp' + '.dif0.fts'
writefits, filedif0, imgdif0, hdudif0
writefits, 'dif0.fts', imgdif0, hdudif0

dif1min = MIN (imgdif1, max=dif1max)
IF (abs (dif1min) LT abs (dif1max)) THEN		$
   BEGIN
   d1min =   dif1min
   d1max = - dif1min
   END							$
else							$
   BEGIN
   d1min = - dif1max
   d1max =   dif1max
   END
imgb = bytscl (imgdif1, min=d1min, max=d1max, top=249, /nan)

;TVSCL, imgdif1
TV, imgb
XYOUTS,   5,  70, 'B1 - R1', /device, color=251
XYOUTS,   5,  50, 'MIN: ' + STRING (d1min), /device, color=251
XYOUTS,   5,  30, 'MAX: ' + STRING (d1max), /device, color=251
tv, cbar
WRITE_GIF, 'dif1.gif', tvrd (), red, green, blue
WAIT, 1

hdudif1 = hdu3
fxaddpar, hdudif1, 'TYPE-OBS', '(B1-R1)/ICONT'
fxaddpar, hdudif1, 'TYPE-DP',  'NORM'
fxaddpar, hdudif1, 'DATAMIN', dif1min
fxaddpar, hdudif1, 'DATAMAX', dif1max
fxaddpar, hdudif1, 'DISPMIN', d1min
fxaddpar, hdudif1, 'DISPMAX', d1max
filedif1 = year3 + month3  + day3    + '.' +	$
	   hour3 + minute3 + second3 + '.' +	$
	   'chp' + '.dif1.fts'
writefits, filedif1, imgdif1, hdudif1
writefits, 'dif1.fts', imgdif1, hdudif1

dif2min = MIN (imgdif2, max=dif2max)
IF (abs (dif2min) LT abs (dif2max)) THEN		$
   BEGIN
   d2min =   dif2min
   d2max = - dif2min
   END							$
else							$
   BEGIN
   d2min = - dif2max
   d2max =   dif2max
   END
imgb = bytscl (imgdif2, min=d2min, max=d2max, top=249, /nan)

;TVSCL, imgdif2
TV, imgb
XYOUTS,   5,  70, 'B2 - R2', /device, color=251
XYOUTS,   5,  50, 'MIN: ' + STRING (d2min), /device, color=251
XYOUTS,   5,  30, 'MAX: ' + STRING (d2max), /device, color=251
tv, cbar
WRITE_GIF, 'dif2.gif', tvrd (), red, green, blue
WAIT, 1

hdudif2 = hdu2
fxaddpar, hdudif2, 'TYPE-OBS', '(B2-R2)/ICONT'
fxaddpar, hdudif2, 'TYPE-DP',  'NORM'
fxaddpar, hdudif2, 'DATAMIN', dif2min
fxaddpar, hdudif2, 'DATAMAX', dif2max
fxaddpar, hdudif2, 'DISPMIN', d2min
fxaddpar, hdudif2, 'DISPMAX', d2max
filedif2 = year2 + month2  + day2    + '.' +	$
	   hour2 + minute2 + second2 + '.' +	$
	   'chp' + '.dif2.fts'
writefits, filedif2, imgdif2, hdudif2
writefits, 'dif2.fts', imgdif2, hdudif2

dif3min = MIN (imgdif3, max=dif3max)
IF (abs (dif3min) LT abs (dif3max)) THEN		$
   BEGIN
   d3min =   dif3min
   d3max = - dif3min
   END							$
else							$
   BEGIN
   d3min = - dif3max
   d3max =   dif3max
   END
imgb = bytscl (imgdif3, min=d3min, max=d3max, top=249, /nan)

;TVSCL, imgdif3
TV, imgb
XYOUTS,   5,  70, 'B3 - R3', /device, color=251
XYOUTS,   5,  50, 'MIN: ' + STRING (d3min), /device, color=251
XYOUTS,   5,  30, 'MAX: ' + STRING (d3max), /device, color=251
tv, cbar
WRITE_GIF, 'dif3.gif', tvrd (), red, green, blue
WAIT, 1

hdudif3 = hdu1
fxaddpar, hdudif3, 'TYPE-OBS', '(B3-R3)/ICONT'
fxaddpar, hdudif3, 'TYPE-DP',  'NORM'
fxaddpar, hdudif3, 'DATAMIN', dif3min
fxaddpar, hdudif3, 'DATAMAX', dif3max
fxaddpar, hdudif3, 'DISPMIN', d3min
fxaddpar, hdudif3, 'DISPMAX', d3max
filedif3 = year1 + month1  + day1    + '.' +	$
	   hour1 + minute1 + second1 + '.' +	$
	   'chp' + '.dif3.fts'
writefits, filedif3, imgdif3, hdudif3
writefits, 'dif3.fts', imgdif3, hdudif3

;--- Now, put the pieces together to get the velocity image.

dif0loc = WHERE (abs (imgdif0) LT 0.01, ndif0)
sizdif0 = SIZE (imgdif0)

PRINT, 'size (imgdif0): ', SIZE (imgdif0)
PRINT, 'ndif0: ', ndif0
PRINT, 'sizdif0 [4]: ', sizdif0 [4]
PRINT, 'fraction of imgdif0 < 0.01 : ', float (ndif0) / float (sizdif0 [4]) 
PRINT, ''

imgvel =   ( 4.0 /  imgdif0 )			$
	 * (        imgdif1 			$
	 +   2.0 *  imgdif2 			$
	 +   3.5 *  imgdif3 			$
	 +   6.0 * (imgdif0 ^ 1.4) )

imgvel [dif0loc] = 0.0		;--- Set velocity to zero where imgdif0 < 0.01

;--- Convert to short integer.

;imgvel = FIX (imgvel)

;--- Display velocity image.

velmin = MIN (imgvel, max=velmax)
PRINT, 'velmin/velmax: ', velmin, velmax

IF (abs (velmin) LT abs (velmax)) THEN			$
   BEGIN
   vmin =   velmin
   vmax = - velmin
   END							$
else							$
   BEGIN
   vmin = - velmax
   vmax =   velmax
   END
imgvelb = bytscl (imgvel, min=vmin, max=vmax, top=249, /nan)

;TVSCL, imgvel
TV, imgvelb
XYOUTS,   5,  70, 'Velocity', /device, color=251
XYOUTS,   5,  50, 'MIN: ' + STRING (vmin), /device, color=251
XYOUTS,   5,  30, 'MAX: ' + STRING (vmax), /device, color=251
tv, cbar
WRITE_GIF, 'vel.gif', tvrd (), red, green, blue
WAIT, 1

;--- Write velocity image to disk as a FITS file.

hduvel = hdu4
fxaddpar, hduvel, 'TYPE-OBS', 'VELOCITY'
fxaddpar, hduvel, 'TYPE-DP',  'CAL'
fxaddpar, hduvel, 'BUNIT',    'KM/SEC'
fxaddpar, hduvel, 'DATAMIN', velmin
fxaddpar, hduvel, 'DATAMAX', velmax
fxaddpar, hduvel, 'DISPMIN', vmin
fxaddpar, hduvel, 'DISPMAX', vmax
fxaddpar, hduvel, 'TYPE-OBS', 'HEI-VEL'

PRINT, 'vmin/vmax: ', vmin, vmax

writefits, 'vel.fts', imgvel, hduvel
writefits, 'vel.fts', hsk4, ext4, /append

fxaddpar, hduvel, 'DATAMIN',   0.0
fxaddpar, hduvel, 'DATAMAX', 249.0
fxaddpar, hduvel, 'DISPMIN',   0.0 
fxaddpar, hduvel, 'DISPMAX', 249.0 
writefits, 'vel.byte.fts', imgvelb, hduvel

END
