;+
; NAME		chipsq.pro
;
; PURPOSE	Get sun center from FITS header.
;		Register image by co-aligning sun center with array center.
;		Crop image to square size [default: 936x936].
;
; SYNTAX	chipsq, 'fits_list'
;       	chipsq, 'fits_list', dim=1000
;
; EXTERNAL ROUTINES	readfits	astronomy library or SOHO SSW library
;			writefits	astronomy library or SOHO SSW library
;			drawcir
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   18 April 2001
;-

PRO chipsq, fits_list, dim=dim, disp=disp

file_name = 'img.fts'
naxis1com = ''
naxis2com = ''
crpix1com = ''
crpix2com = ''
crradiuscom = ''

;--- Get optional output dimension.

IF (KEYWORD_SET (dim)) THEN	$
    BEGIN
    xdimsq = dim
    ydimsq = dim
    END 			$
ELSE				$
    BEGIN
    xdimsq = 936
    ydimsq = 936
;    PRINT, 'Must specify keyword dim=num, where num=x/y dimension [output].'
;    RETURN
    END

PRINT, 'xdimsq/ydimsq: ', xdimsq, ydimsq

;--- Open input list file.

CLOSE, 11
OPENR, 11, fits_list
i = -1

;--- File loop:

WHILE (NOT EOF (11)) DO		$
    BEGIN ;{
    i = i + 1
    READF, 11, file_name, FORMAT = '(A)'	; Read filename from list.
    PRINT, 'input file: ', file_name

    ;--- Read image & extension.
    
    img = readfits (file_name, hdu, /noscale)
    hsk = readfits (file_name, ext, exten_no=1, /noscale)
    
    imin = MIN (img, max=imax)			; Find min/max intensities.
    print, 'imin/imax: ', imin, imax
    
    ;--- Get information from FITS header.

    xdim     = fxpar (hdu, 'NAXIS1', comment=naxis1com)
    ydim     = fxpar (hdu, 'NAXIS2', comment=naxis2com)
    xcen     = fxpar (hdu, 'CRPIX1', comment=crpix1com)
    ycen     = fxpar (hdu, 'CRPIX2', comment=crpix2com)
    crradius = fxpar (hdu, 'CRRADIUS', comment=crradiuscom)
    radius = crradius
    dispmin  = fxpar (hdu, 'DISPMIN')
    dispmax  = fxpar (hdu, 'DISPMAX')

    ;--- If center < 0, set to array center.

;    IF (xcen LT 0.0) THEN xcen = (xdim - 1) * 0.5
;    IF (ycen LT 0.0) THEN ycen = (ydim - 1) * 0.5

    xcen = (xdim - 1) * 0.5
    ycen = (ydim - 1) * 0.5
    
    print, ' xcen/ycen: ', xcen, ycen, ' crradius: ', crradius

    ;--- Display image.
    
    xdim2 = xdim / 2
    ydim2 = ydim / 2
    
    ;--- Reduce image size for display.

    img2 = REBIN (img, xdim2, ydim2)
    
    IF (KEYWORD_SET (disp)) THEN	$
    BEGIN ;{
        IF (i EQ 0) THEN  WINDOW, 0, xsize = xdim2, ysize=ydim2
        WSET, 0
    
        TV, BYTSCL (img2, min=dispmin, max=dispmax)
    
        XYOUTS, 10, 10, 'ORIGINAL IMAGE', /device

        ;--- Draw circle at 1.0 Rsun.

        drawcir, xcen/2, ycen/2, crradius/2
    END ;}
    
    ;--- Compute array center of output image.
    
    imgsq = INTARR (xdimsq, ydimsq)
    xcensq = (xdimsq - 1) * 0.5
    ycensq = (ydimsq - 1) * 0.5
    
    ;--- Translate image using x/y shifts.
    
    print, 'Translate image.'

    xsh = xcensq - xcen - 1
    ysh = ycensq - ycen - 1
    print, 'xsh/ysh: ', xsh, ysh
    
    ;--- Generate output cropped image using interpolation.

    FOR iy = 0, ydimsq - 1 DO	$
    BEGIN ;{
       FOR ix = 0, xdimsq - 1 DO	$
       BEGIN ;{
       imgsq [ix, iy] = INTERPOLATE (img, ix - xsh, iy - ysh, MISSING=0)
       END ;}
    END ;}

    ;--- Display registered image.
    
    IF (KEYWORD_SET (disp)) THEN	$
    BEGIN ;{
        IF (xdimsq GT 1200 OR ydimsq GT 1000)  THEN	$
	    BEGIN ;{
            xdimsq2 = xdimsq / 2
            ydimsq2 = ydimsq / 2

            imgsq2 = REBIN (imgsq, xdimsq2, ydimsq2)
    
            IF (i EQ 0) THEN WINDOW, 1, xsize = xdimsq2, ysize=ydimsq2
	    WSET, 1
            TV, BYTSCL (imgsq2, min=dispmin, max=dispmax)
            XYOUTS, 10, 10, 'REGISTERED IMAGE', /device

            ;--- Draw a circle of radius 1.0 Rsun.

            drawcir, xcensq/2, ycensq/2, crradius/2

	    WAIT, 2
            END					$
        ELSE					$	;}
            BEGIN ;{

            IF (i EQ 0) THEN  WINDOW, 1, xsize = xdimsq, ysize=ydimsq
	    WSET, 1
            TV, BYTSCL (imgsq, min=dispmin, max=dispmax)
            XYOUTS, 10, 10, 'REGISTERED IMAGE', /device

            ;--- Draw a circle of radius 1.0 Rsun.

            drawcir, xcensq, ycensq, crradius

 	    WAIT, 2
	    END ;}
    END ;}
    
    ;--- Write square image to disk.
    
    fpos = STRPOS (file_name, '.fts')
    out_name = STRMID (file_name, 0, fpos) + '.sq.fts'
    PRINT, 'out_name: ', out_name
    
    ;--- Update FITS header for output image file.

    hdusq = hdu
    fxaddpar, hdusq, 'NAXIS1', xdimsq, naxis1com
    fxaddpar, hdusq, 'NAXIS2', ydimsq, naxis2com
    fxaddpar, hdusq, 'CRPIX1', xcensq, crpix1com
    fxaddpar, hdusq, 'CRPIX2', ycensq, crpix2com
    fxaddpar, hdusq, 'HISTORY', ' IMAGE REGISTERED WITH SUN CENTER EQUAL TO ARRAY CENTER.'
    
    writefits, out_name, imgsq, hdusq
    writefits, out_name, hsk, ext, /APPEND

    END ;}

CLOSE, 11

END
