;+
; NAME	chipsqrot.pro
;
; PURPOSE	Get sun center from FITS header [use array center if < 0].
;		Register image by co-aligning sun center with array center.
;		Crop image to square size [default: 936x936].
;		Rotate according to P-angle.
;
;		As a visual check, draw 1.0 Rsun circle with center
;		at array center.
;
; SYNTAX	chipsqrot, 'fits_list'
;		chipsqrot, 'fits_list', dim=500
;		chipsqrot, 'fits_list', dim=1000, /interp
;
; INPUT		fits_list	file containing a list of FITS images
;		dim		output dimension; default = 936
;		interp		if set, use interpolation; default: pixel crop
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   18 April 2001
;
; HISTORY	19 Apr 2001 : Add interpolation option.
;-

PRO chipsqrot, fits_list, dim=dim, interp=interp

file_name = ' '

IF (KEYWORD_SET (dim)) THEN	$
    BEGIN
    xdimsq = dim
    ydimsq = dim
    END 			$
ELSE				$
    BEGIN
    xdimsq = 936
    ydimsq = 936
;    PRINT, 'Must specify keyword dim=num, where num=x/y dimension [output].'
;    RETURN
    END

CLOSE, 11
OPENR, 11, fits_list
i = -1

WHILE (NOT EOF (11)) DO		$
    BEGIN 
    i = i + 1
    READF, 11, file_name, FORMAT = '(A)'
    PRINT, 'input file: ', file_name

    fpos = STRPOS (file_name, '.fts')
    out_name = STRMID (file_name, 0, fpos) + '.sqrot.fts'
    PRINT, 'out_name: ', out_name
    
    ;--- Read image & extension.
    
    img = readfits (file_name, hdu, /noscale)
    hsk = readfits (file_name, ext, exten_no=1, /noscale)
    
    xdim     = fxpar (hdu, 'NAXIS1', comment=naxis1com)
    ydim     = fxpar (hdu, 'NAXIS2', comment=naxis2com)
    xcen     = fxpar (hdu, 'CRPIX1', comment=crpix1com)
    ycen     = fxpar (hdu, 'CRPIX2', comment=crpix2com)
    crradius = fxpar (hdu, 'CRRADIUS', comment=crradiuscom)
    radius = crradius
    dispmin  = fxpar (hdu, 'DISPMIN')
    dispmax  = fxpar (hdu, 'DISPMAX')
    crota1   = fxpar (hdu, 'CROTA1', comment=crota1com)
    crota2   = fxpar (hdu, 'CROTA2', comment=crota2com)
    pangle   = fxpar (hdu, 'SOLAR_P0')

    ;--- If center < 0, set to array center.

    IF (xcen LT 0.0) THEN xcen = (xdim - 1) * 0.5
    IF (ycen LT 0.0) THEN ycen = (ydim - 1) * 0.5
    
    print, ' xcen/ycen: ', xcen, ycen, ' crradius: ', crradius

    ;--- Display image.
    
    xdim2 = xdim / 2
    ydim2 = ydim / 2
    
    img2 = REBIN (img, xdim2, ydim2)
    
    WSET, 0
    IF (i EQ 0) THEN  WINDOW, 0, xsize = xdim2, ysize=ydim2, retain=2
    
    TV, BYTSCL (img2, min=dispmin, max=dispmax)
    
    XYOUTS, 10, 10, 'ORIGINAL IMAGE: ' + file_name, /device

    drawcir, xcen/2, ycen/2, crradius/2

    WAIT, 2
    
    ;--- Compute array center of output image.
    
    imgsq = INTARR (xdimsq, ydimsq)
    xcensq = (xdimsq - 1) * 0.5
    ycensq = (ydimsq - 1) * 0.5
    
    ;--- Crop image to a square.

    IF (KEYWORD_SET (interp)) THEN		$
    BEGIN

        ;--- Translate image using x/y shifts.
    
        print, 'Translate image using interpolation.'

        xsh = xcensq - xcen - 1
        ysh = ycensq - ycen - 1
        print, 'xsh/ysh: ', xsh, ysh
    
        FOR iy = 0, ydimsq - 1 DO	$
        BEGIN
           FOR ix = 0, xdimsq - 1 DO	$
           BEGIN
           imgsq [ix, iy] = INTERPOLATE (img, ix - xsh, iy - ysh, MISSING=0)
           END
        END
    END						$
    ELSE					$
    BEGIN

        ;--- Determine beginning & ending pixels to crop images to a square size
    
        PRINT, 'Translate image using pixel cropping.'

        xbeg = FIX ((xcen - xcensq) + 0.5) + 1
        xend = xbeg + xdimsq - 1
        ybeg = FIX ((ycen - ycensq) + 0.5) + 1
        yend = ybeg + ydimsq - 1
        imgsq [*, *] = img [xbeg:xend, ybeg:yend]
        imgsq [*, *] = img [xbeg:xend, ybeg:yend]
    END

    mag = 1.0
    imgsqrot = rot (imgsq, pangle, mag, xcensq, ycensq, missing=0.0, /pivot)

    ;--- Display registered image.
    
    IF (xdimsq GT 1200 OR ydimsq GT 1000)  THEN	$
	BEGIN
        xdimsq2 = xdimsq / 2
        ydimsq2 = ydimsq / 2

        imgsq2 = REBIN (imgsqrot, xdimsq2, ydimsq2)
    
        IF (i EQ 0) THEN WINDOW, 1, xsize = xdimsq2, ysize=ydimsq2, retain=2
	WSET, 1
        TV, BYTSCL (imgsq2, min=dispmin, max=dispmax)
        XYOUTS, 10, 10, 'REGISTERED IMAGE: ' + out_name, /device

        ;--- Draw a circle of radius 1.0 Rsun.

        drawcir, xcensq/2, ycensq/2, crradius/2

	WAIT, 2
        END					$
    ELSE					$
        BEGIN

        IF (i EQ 0) THEN  WINDOW, 1, xsize = xdimsq, ysize=ydimsq, retain=2
	WSET, 1
        TV, BYTSCL (imgsqrot, min=dispmin, max=dispmax)
        XYOUTS, 10, 10, 'REGISTERED IMAGE: ' + out_name, /device

        ;--- Draw a circle of radius 1.0 Rsun.

        drawcir, xcensq, ycensq, crradius

	WAIT, 2
	END
    
    ;--- Write registered image to disk.
    
    hdusq = hdu
    fxaddpar, hdusq, 'NAXIS1', xdimsq, naxis1com
    fxaddpar, hdusq, 'NAXIS2', ydimsq, naxis2com
    fxaddpar, hdusq, 'CRPIX1', xcensq, crpix1com
    fxaddpar, hdusq, 'CRPIX2', ycensq, crpix2com
    fxaddpar, hdusq, 'CROTA1', 0.0, crota1com
    fxaddpar, hdusq, 'CROTA2', 0.0, crota2com
    fxaddpar, hdusq, 'COORDNAM', 'HELIOCENTRIC'
    fxaddpar, hdusq, 'HISTORY', ' IMAGE REGISTERED WITH SUN CENTER EQUAL TO ARRAY CENTER.'
    
    writefits, out_name, imgsqrot, hdusq
    writefits, out_name, hsk, ext, /APPEND

    END

CLOSE, 11

END
