;+
; fixchiphd.pro
;
; Change the code_obs parameter in the FITS header.
; Modify the Type code in the FITS extension.
; Also, rename the file.
;
; The individual exposures for CHIP sometimes have the WRONG binary header,
; resulting in incorrect information in the FITS headers as well.
;
; The file name is used to correct the CODE-OBS and OBJECT parameters.
;
; Andrew L. Stanger   HAO/NCAR   4 April 2001
; 10 May 2002: Fixed type_obs for b & d.
; 13 May 2002: Set sun center to array center if crpix1 or crpix2 are negative.
;-

PRO fixchiphd, fits_list

ccrpix1 = ''
ccrpix2 = ''

fits_file = ''
CLOSE, 11

OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO                  $
    BEGIN
    READF, 11, fits_file, FORMAT = '(A)'

    print, fits_file

    ;--- Read image, primary HDU, and extension HDU.

    img = readfits (fits_file, hdu, /noscale)
    hsk = readfits (fits_file, ext, /noscale, exten_no=1)

    ;--- Get information from FITS header.

    dateobs = fxpar (hdu, 'DATE-OBS')
    timeobs = fxpar (hdu, 'TIME-OBS')
    year   = STRMID (dateobs, 0, 4)
    month  = STRMID (dateobs, 5, 2)
    day    = STRMID (dateobs, 8, 2)
    hour   = STRMID (timeobs, 0, 2)
    minute = STRMID (timeobs, 3, 2)
    second = STRMID (timeobs, 6, 2)
    xdim   = fxpar (hdu, 'NAXIS1')
    ydim   = fxpar (hdu, 'NAXIS2')
    xcen   = (xdim - 1) * 0.5
    ycen   = (ydim - 1) * 0.5
    crpix1 = fxpar (hdu, 'CRPIX1', ccrpix1)
    crpix2 = fxpar (hdu, 'CRPIX2', ccrpix2)
    IF (crpix1 LT 0.0) THEN fxaddpar, hdu, 'CRPIX1', xcen, ccrpix1
    IF (crpix2 LT 0.0) THEN fxaddpar, hdu, 'CRPIX2', xcen, ccrpix2

    print, 'FITS date: ', dateobs
    print, 'FITS time: ', timeobs

    ;--- Extract information from file name.

    syear   = STRMID (fits_file,  0, 4)
    smonth  = STRMID (fits_file,  5, 2)
    sday    = STRMID (fits_file,  8, 2)
    shour   = STRMID (fits_file, 11, 2)
    sminute = STRMID (fits_file, 14, 2)
    ssecond = STRMID (fits_file, 17, 2)
    wsr     = STRMID (fits_file, 20, 3)
    wavecode      = STRMID (fits_file, 20, 1)
    sourcecode    = STRMID (fits_file, 21, 1)
    reductioncode = STRMID (fits_file, 22, 1)
    expnum  = STRMID (fits_file, 24, 1)

    date = syear + '-' + smonth + '-' + sday
    time = shour + ":" + sminute + ':' + ssecond
    print, 'filename date: ', date
    print, 'filename time: ', time

    print, 'filename wsr: ', wsr
    code_obs = wsr + expnum

    ;--- Set correct wavelength description.

    CASE wavecode OF  ;{
    'x':  BEGIN
	  type_obs = 'UNKNOWN'
	  wavenum = 0
	  wavelen = 0.0
	  wavewid = 0.0
	  END
    'h':  BEGIN
	  type_obs = 'HELIUM-I'
	  wavenum = 1
	  wavelen = 1083.03 
	  wavewid = 0.14
	  END
    'd':  BEGIN
	  type_obs = 'FAR BLUE'
	  wavenum = 2
	  wavelen = 1082.74 
	  wavewid = 0.14
	  END
    't':  BEGIN
	  type_obs = 'FAR RED'
	  wavenum = 3
	  wavelen = 1083.31 
	  wavewid = 0.14
	  END
    'c':  BEGIN
	  type_obs = 'MID BLUE'
	  wavenum = 4
	  wavelen = 1082.85 
	  wavewid = 0.14
	  END
    's':  BEGIN
	  type_obs = 'MID RED'
	  wavenum = 5
	  wavelen = 1083.21 
	  wavewid = 0.14
	  END
    'b':  BEGIN
	  type_obs = 'NEAR BLUE'
	  wavenum = 6
	  wavelen = 1082.96 
	  wavewid = 0.14
	  END
    'r':  BEGIN
	  type_obs = 'NEAR RED'
	  wavenum = 7
	  wavelen = 1083.10 
	  wavewid = 0.14
	  END
    'k':  BEGIN
	  type_obs = 'K-CORONA'
	  wavenum = 8
	  wavelen = 656.28 
	  wavewid = 0.80
	  END
    'o':  BEGIN
	  type_obs = 'OTHER'
	  wavenum = 9
	  wavelen = 0.0
	  wavewid = 0.0
          END
    ELSE: BEGIN
	  type_obs = 'UNKNOWN'
	  wavenum = 0
	  wavelen = 0.0
	  wavewid = 0.0
	  END
    ENDCASE ;}

    ;--- Set correct object name.

    CASE sourcecode OF  ;{
    'x':  BEGIN
	  object = 'UNDEF'
	  sourcenum = 0
	  END
    's':  BEGIN
	  object = 'SUN'
	  sourcenum = 1
	  END
    'l':  BEGIN
	  object = 'LIMB'
	  sourcenum = 2
	  END
    'o':  BEGIN
	  object = 'OPAL'
	  sourcenum = 3
	  END
    'd':  BEGIN
	  object = 'DIFFUSER'
	  sourcenum = 4
	  END
    'b':  BEGIN
	  object = 'BIAS'
	  sourcenum = 5
	  END
    'c':  BEGIN
	  object = 'CAL-OPT'
	  sourcenum = 6
	  END
    'f':  BEGIN
	  object = 'FLAT-FLD'
	  sourcenum = 7
	  END
    ELSE: BEGIN
	  object = 'UNKNOWN'
	  sourcenum = 0
	  END
    ENDCASE ;}

    CASE reductioncode OF ;{
    'x':  reducenum = 0
    'r':  reducenum = 1
    'c':  reducenum = 2
    'l':  reducenum = 3
    'd':  reducenum = 4
    'm':  reducenum = 5
    't':  reducenum = 6
    'h':  reducenum = 7
    ELSE: reducenum = 0
    ENDCASE ;}

    ;--- Update CODE-OBS parameter in FITS header.
    
;    object = object + expnum
;    print, 'object: ', object

    fxaddpar, hdu, 'DATE-OBS', date			; Update 'DATE-OBS'
    fxaddpar, hdu, 'TIME-OBS', time			; Update 'TIME-OBS'
    fxaddpar, hdu, 'CODE-OBS', STRUPCASE (code_obs)	; Update 'CODE-OBS'.
    fxaddpar, hdu, 'TYPE-OBS', STRUPCASE (type_obs)	; Update 'TYPE-OBS'.
    fxaddpar, hdu, 'OBJECT',   STRUPCASE (object  )	; Update 'OBJECT'.
    fxaddpar, hdu, 'WAVELEN', wavelen, ' CENTRAL WAVE LENGTH OF OBSERVATION [NM]
    fxaddpar, hdu, 'WAVEWID', wavewid, ' BANDPASS OF FILTER [NM]

    fxaddpar, ext, 'DATE-OBS', date			; Update 'DATE-OBS'
    fxaddpar, ext, 'TIME-OBS', time			; Update 'TIME-OBS'
    fxaddpar, ext, 'TYPE-OBS', STRUPCASE (type_obs)	; Update 'TYPE-OBS'.
    fxaddpar, ext, 'OBJECT',   STRUPCASE (object  )	; Update 'OBJECT'.
    
    wsrnum   = wavenum * 100 + sourcenum * 10 + reducenum	; Type.
    hsk [13] = wsrnum
    print, 'wsrnum: ', wsrnum

    ;--- Write FITS image to disk.
    
;    out_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
;               '.chp.' + STRLOWCASE (code_obs) + '.fts'

    out_name = year + month + day + '.' + hour + minute + second + $
               '.chp.' + STRLOWCASE (code_obs) + '.fts'

    PRINT, 'out_name: ', out_name
    PRINT, ' '
    
    writefits, out_name, img, hdu
    writefits, out_name, hsk, ext, /APPEND
    END
CLOSE, 11
    
END
    
