;+ mklsdgain00.pro
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MLSO/CHIP Helium-I instrument
;
; Use a Light Shaping Diffuser (LSD) image to create a CCD correction image.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; SYNTAX	mklsdgain0, 'lsd.fts'
;
; AUTHOR	David Elmore   HAO/NCAR   2 March 2001
; UPDATES	Andrew L. Stanger
;		 5 March 2001 : Input/Output FITS images.
;		 6 March 2001 : Compute a correction for each LSD filter.
;		21 March 2001 : Add FITS extension.
;		14 June 2001  : Only one diffuser file is used.
;		20 March 2002 : No bias subtraction.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-

PRO mklsdgain00, filename

border  = 6
refval  = 1
frefval = float (refval)

LOADCT, 3

;--- Read LSD diffuser image.

    lsd = readfits (filename, hdu, /noscale)
    hsk = readfits (filename, ext, /noscale, exten_no=1)
    code_obs = strlowcase ( strtrim (fxpar (hdu, 'CODE-OBS'), 2) )
    print, 'code_obs: ...', code_obs, '...'

    xdim  = fxpar (hdu, 'NAXIS1')
    ydim  = fxpar (hdu, 'NAXIS2')
    xdim2 = xdim / 2
    ydim2 = ydim / 2

    WINDOW, xsize = xdim2, ysize = ydim2, retain=2

    ;--- Reduce image size for 4th order surface fit.

    lsd2 = REBIN (lsd, xdim2, ydim2)
    TV, BYTSCL (lsd2, min=0, max=600)

    XYOUTS, 5, 5, 'LSD', /device
 
    ;--- Compute a 4th order polynomial fit.

    PRINT, 'Running SFIT'

    lsdf2 = SFIT (lsd2, 4, kx=kx)

    print, 'size (kx): ', kx
    print, 'kx: ', kx

    TVSCL, lsdf2
    XYOUTS, 5, 5, 'LSD FIT', /device
;    WAIT, 3
;    stop

    ;--- Expand surface fit image back to original size [cropped].

    lsdf = float (REBIN (lsdf2, xdim, ydim))

    ;--- Subtract fit from LSD image.

    lsds = lsd - lsdf

    lsds2 = REBIN (lsds, xdim2, ydim2)
    TV, BYTSCL (lsds2)
    XYOUTS, 5, 5, 'LSD - FIT', /device
;    WAIT, 3
;    stop

    ;--- Divide original LSD image by the fit.

    lsdr = float (lsdf) / float (lsd)

    ;--- Display image.

    lsdr2 = REBIN (lsdr, xdim2, ydim2)
    TV, BYTSCL (lsdr2, min=0.9, max=1.1)

    XYOUTS, 5, 5, 'LSD CORRECTION', /device

    rmin = MIN (lsdr, max=rmax)
    rmin_norm = rmin / frefval
    rmax_norm = rmax / frefval

    ;--- Update FITS header.

    hdulsdr = hdu
    fxaddpar, hdulsdr, 'HISTORY', 'DIFFUSER CORRECTION [MULTIPLICATIVE].'
    fxaddpar, hdulsdr, 'BSCALE',  frefval
    fxaddpar, hdulsdr, 'DATAMIN', rmin_norm
    fxaddpar, hdulsdr, 'DATAMAX', rmax_norm
    fxaddpar, hdulsdr, 'DISPMIN', 0.9
    fxaddpar, hdulsdr, 'DISPMAX', 1.1

    date     = fxpar (hdulsdr, 'DATE-OBS')
    time     = fxpar (hdulsdr, 'TIME-OBS')
    code_obs_com = ''
    code_obs = strtrim (fxpar (hdulsdr, 'CODE-OBS', code_obs_com), 2)
    code_obs = strlowcase (code_obs)
    code_obs2 = STRMID (code_obs, 0, 2)
    code_obs  = code_obs2 + 'd'			; 'd' => ratio image.
    fxaddpar, hdulsdr, 'CODE-OBS', STRUPCASE (code_obs), code_obs_com
    print, 'date: ', date
    print, 'time: ', time
    print, 'code_obs: ', code_obs
    syear    = STRMID (date, 0, 4)
    smonth   = STRMID (date, 5, 2)
    sday     = STRMID (date, 8, 2)
    shour    = STRMID (time, 0, 2)
    sminute  = STRMID (time, 3, 2)
    ssecond  = STRMID (time, 6, 2)
    print, 'syear/smonth/sday: ', syear, smonth, sday
    print, 'shour/sminute/ssecond: ', shour, sminute, ssecond

    ;--- Write ratio image to disk.

    lsc_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
	       '.chp.' + code_obs + '.lsc.fts'
    PRINT, 'lsc_name: ', lsc_name

    writefits, lsc_name, lsdr, hdulsdr
    writefits, lsc_name, hsk,  ext, /APPEND

    ;--- Write LSD fit image to disk.

    fmin = MIN (lsdf, max=fmax)
    print, 'fmin/fmax: ', fmin, fmax
    hdulsdf = hdu
    code_obs  = code_obs2 + 'x'			; 'x' => unknown (fit).
    fxaddpar, hdulsdf, 'CODE-OBS', STRUPCASE (code_obs), code_obs_com
    fxaddpar, hdulsdf, 'HISTORY', 'DIFFUSER FIT [4th ORDER].'
    fxaddpar, hdulsdf, 'OBJECT',  'DIFFUSER FIT'
    fxaddpar, hdulsdf, 'DATAMIN', fmin
    fxaddpar, hdulsdf, 'DATAMAX', fmax
    fxaddpar, hdulsdf, 'DISPMIN', fmin
    fxaddpar, hdulsdf, 'DISPMAX', fmax
    lsf_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
	       '.chp.' + code_obs + '.lsf.fts'
    PRINT, 'lsf_name: ', lsf_name
    writefits, lsf_name, lsdf, hdulsdf
    writefits, lsf_name, hsk,  ext, /APPEND

    ;--- Write LSD-fit image to disk.

    smin = MIN (lsds, max=smax)
    print, 'smin/smax: ', smin, smax
    hdulsds = hdu
    code_obs  = code_obs2 + 'm'			; 'm' => contrast (minus).
    fxaddpar, hdulsds, 'CODE-OBS', STRUPCASE (code_obs), code_obs_com
    fxaddpar, hdulsds, 'HISTORY', 'DIFFUSER - FIT [4th ORDER].'
    fxaddpar, hdulsds, 'OBJECT',  'DIFFUSER - FIT'
    fxaddpar, hdulsds, 'DATAMIN', smin
    fxaddpar, hdulsds, 'DATAMAX', smax
    fxaddpar, hdulsds, 'DISPMIN', -25.0
    fxaddpar, hdulsds, 'DISPMAX',  25.0
    lss_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
	       '.chp.' + code_obs + '.lss.fts'
    PRINT, 'lss_name: ', lss_name
    writefits, lss_name, lsds, hdulsds
    writefits, lsf_name, hsk,  ext, /APPEND

END
