; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MLSO/CHIP Helium-I instrument
;
; Use a bias image and a set of Light Shaping Diffuser (LSD) images
; to create a new gain to pass to MLSO.
;
; "list" contains the names of the files.
; The first file is a bias, the remaining ones are for the various passbands.
;
; The mean of the LSD gain files is turned into a multiplicative gain file
; for chip and written out as 'gainlsd'
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; SYNTAX	mklsdgain, 'list'
;		list: file containing a list of image names.
;
; AUTHOR	David Elmore   HAO/NCAR   2 March 2001
; UPDATES	Andrew L. Stanger
;		5 March 2001 : Input/Output FITS images.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-

PRO mklsdgainfts, list

border  = 6
numband = 7
hdulsd  = STRARR (numband)
refval  = 8192

filename = 'chip'

;--- Open list file & read LSD bias image.

CLOSE, 15
OPENR, 15, list
READF, 15, filename

bias  = readfits (filename, hdubias, /noscale)
xdim  = fxpar (hdubias, 'NAXIS1')
ydim  = fxpar (hdubias, 'NAXIS2')
xdimc = xdim - border * 2
ydimc = ydim - border * 2
xdimc2 = xdimc / 2
ydimc2 = ydimc / 2
fr    = INTARR (numband, xdimc, ydimc)

biasc = bias [border:xdim-1-border, border:ydim-1-border]

WINDOW, xsize = xdimc2, ysize = ydimc2
LOADCT, 3

;--- Remove header and edge effects from bias

bias [*, 0] = bias [*, 2]
bias [*, 1] = bias [*, 2]
bias [*, ydimc - 1] = bias [*, ydimc - 2]

;--- Crop image.
;--- Subtract bias

FOR ii = 0, numband - 1 DO BEGIN
    READF, 15, filename 
    lsd = readfits (filename, hdu, /noscale)
    code_obs = strlowcase ( strtrim (fxpar (hdu, 'CODE-OBS'), 2) )
    print, 'code_obs: ...', code_obs, '...'
    IF (code_obs EQ 'hdr') THEN		$
	BEGIN
	hdulsd = hdu
	ihdu = ii
	END

    lsdc = lsd [border:xdim-1-border, border:ydim-1-border]
 
    lsdc = lsdc - biasc
    fr [ii, *, *] = lsdc

    lsdc2 = REBIN (lsdc, xdimc2, ydimc2)
    TV, BYTSCL (lsdc2, 1200, 2500)
ENDFOR
CLOSE, 15
print, 'ihdu: ', ihdu

;stop

;--- Compute average bias-corrected LSD image.

meanfr = 1.0 * fr [0, *, *]
FOR ii= 1, numband - 1 DO BEGIN
    meanfr = meanfr + fr [ii, *, *]
ENDFOR
meanfr = meanfr / 7
meanfr = REFORM (meanfr)

;STOP

;--- Compute a 4th order polynomial fit to the average LSD image.

PRINT, "Doing polynomial fit"
frfit = SFIT (meanfr, 4)
gainc = frfit / meanfr * float (refval)
gainc = FIX (gainc)

gainc2 = REBIN (gainc, xdimc2, ydimc2)
TV, BYTSCL (gainc2, 1.1 * MAX (gainc2), 0.8 * MAX (gainc2))

;STOP

;--- Write gain image to disk.

gain  = INTARR (xdim, ydim)
gain [0:border-1,         *]                  = refval
gain [xdim-border:xdim-1, *]                  = refval
gain [*,                  0:border-1]         = refval
gain [*,                  ydim-border:ydim-1] = refval
gain [border:xdim-1-border, border:ydim-1-border] = gainc
gmin = MIN (gain, max=gmax)
gmin_norm = gmin / float (refval)
gmax_norm = gmax / float (refval)

hdugain = hdulsd
fxaddpar, hdugain, 'HISTORY', 'AVERAGE LIGHT SHAPING DIFFUSER CORRECTION.'
fxaddpar, hdugain, 'BSCALE',  1.0 / float (refval)
fxaddpar, hdugain, 'DATAMIN', gmin_norm
fxaddpar, hdugain, 'DATAMAX', gmax_norm
fxaddpar, hdugain, 'DISPMIN', 0.9
fxaddpar, hdugain, 'DISPMAX', 1.1

date     = fxpar (hdugain, 'DATE-OBS')
time     = fxpar (hdugain, 'TIME-OBS')
code_obs = strlowcase ( strtrim (fxpar (hdugain, 'CODE-OBS'), 2) )
print, 'date: ', date
print, 'time: ', time
print, 'code_obs: ', code_obs
syear    = STRMID (date, 0, 4)
smonth   = STRMID (date, 5, 2)
sday     = STRMID (date, 8, 2)
shour    = STRMID (time, 0, 2)
sminute  = STRMID (time, 3, 2)
ssecond  = STRMID (time, 6, 2)
print, 'syear/smonth/sday: ', syear, smonth, sday
print, 'shour/sminute/ssecond: ', shour, sminute, ssecond

gain_name = syear + smonth + sday + '.' + shour + sminute + ssecond +	 $
	    '.chp.' + code_obs + '.lsc.fts'
PRINT, 'gain_name: ', gain_name

writefits, gain_name, gain, hdugain
OPENW,  11, 'gainlsd'
WRITEU, 11, gain
CLOSE,  11

;--- Compare new vs old gain.

;OPENR, 11, 'chip_gain'
;READU, 11, lsd
;CLOSE, 11

;gaindif = gain - lsd
;gaindif = REBIN (gaindif, xdimc2, ydimc2)
;TV, BYTSCL (gaindif, -400, 400)

;STOP

END
