;+ mklsdgains0.pro
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MLSO/CHIP Helium-I instrument
;
; Use a bias image and a set of Light Shaping Diffuser (LSD) images
; to create a CCD correction image for each LSD image.
;
; "list" contains the names of the files.
; The first file is a bias, the remaining ones are for the various wavelengths.
;
; Each of the LSD files is turned into a multiplicative correction file.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; SYNTAX	mklsdgain, 'list'
;		list: file containing a list of image names:
;		      ddr
;		      cdr
;		      bdr
;		      hdr
;		      rdr
;		      sdr
;		      tdr
;
; AUTHOR	David Elmore   HAO/NCAR   2 March 2001
; UPDATES	Andrew L. Stanger
;		 5 March 2001 : Input/Output FITS images.
;		 6 March 2001 : Compute a correction for each LSD filter.
;		21 March 2001 : Add FITS extension.
;		20 March 2002 : Add retain=2 to WINDOW command.
;		22 March 2002 : Remove bias subtraction.
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-

PRO mklsdgains0, list

border  = 6
numband = 7
hdulsd  = STRARR (numband)
refval  = 1
frefval = float (refval)

filename = 'chip'

;--- Open list file & read LSD bias image.

CLOSE, 11
OPENR, 11, list
READF, 11, filename

CLOSE, 11
OPENR, 11, list

bias  = readfits (filename, hdubias, /noscale)
xdim  = fxpar (hdubias, 'NAXIS1')
ydim  = fxpar (hdubias, 'NAXIS2')
xdim2 = xdim / 2
ydim2 = ydim / 2

xdimc = xdim - border * 2
ydimc = ydim - border * 2
xdimc2 = xdimc / 2
ydimc2 = ydimc / 2

WINDOW, xsize = xdimc2, ysize = ydimc2, retain=2
LOADCT, 3

;--- Remove header and edge effects from bias

;bias [*, 0] = bias [*, 2]
;bias [*, 1] = bias [*, 2]
;bias [*, ydim - 1] = bias [*, ydim - 2]

;;;biasc = bias [border:xdim-1-border, border:ydim-1-border]

;bias2 = REBIN (bias, xdim2, ydim2)
;TVSCL, bias2

;;;biasc2 = REBIN (biasc, xdimc2, ydimc2)
;;;TVSCL, biasc2

;;;XYOUTS, 5, 5, 'BIAS', /device
;;;WAIT, 3

;--- Process light shaping diffuser (LSD) images.

FOR ii = 0, numband - 1 DO BEGIN
    READF, 11, filename 
    lsd = readfits (filename, hdu, /noscale)
    hsk = readfits (filename, ext, /noscale, exten_no=1)
    code_obs = strlowcase ( strtrim (fxpar (hdu, 'CODE-OBS'), 2) )
    print, 'code_obs: ...', code_obs, '...'
    hdulsd = hdu

;    lsd (*, 0) = lsd (*, 7)
;    lsd (*, 1) = lsd (*, 7)
;    lsd (*, 2) = lsd (*, 7)
;    lsd (*, 3) = lsd (*, 7)
;    lsd (*, 4) = lsd (*, 7)
;    lsd (*, 5) = lsd (*, 7)
;    lsd (*, 6) = lsd (*, 7)
;    lsd (*, ydim-1) = lsd (*, ydim-2)

    ;--- Crop image.

    lsdc = lsd [border:xdim-1-border, border:ydim-1-border]

;    lsd2 = REBIN (lsd, xdim2, ydim2)
;    TVSCL, lsd2
;    TV, BYTSCL (lsd2, 1200, 2500)

    lsdc2 = REBIN (lsdc, xdimc2, ydimc2)
    TVSCL, lsdc2

    XYOUTS, 5, 5, 'LSD', /device
 
    ;--- Subtract bias

;    lsdb = float (lsd2 - bias2)
;    lsdbc = float (lsdc2 - biasc2)

    lsdbc = float (lsdc - biasc)

    ;--- Reduce image size for 4th order surface fit.

    lsdbc2 = REBIN (lsdbc, xdimc2, ydimc2)

    ;--- Compute a 4th order polynomial fit.

    PRINT, 'Running SFIT'

;    lsdf = SFIT (lsdb, 4, kx=kx)

;    lsdfc = SFIT (lsdbc, 4, kx=kx)
    lsdfc2 = SFIT (lsdbc2, 4, kx=kx)

    print, 'size (kx): ', kx
    print, 'kx: ', kx

    ;--- Expand image back to original size.

    lsdfc = REBIN (lsdbc2, xdimc, ydimc)

    ;--- Divide original LSD image by the fit.

;    lsdr = lsdb / lsdf

    lsdrc = float (lsdfc) / float (lsdbc)

    ;--- Display image.

;    TV, BYTSCL (lsdr, min=0.9, max=1.1)
;    TV, BYTSCL (lsdrc, min=0.9, max=1.1)

    lsdrc2 = REBIN (lsdrc, xdimc2, ydimc2)
    TV, BYTSCL (lsdrc2, min=0.9, max=1.1)

    XYOUTS, 5, 5, 'LSD CORRECTION', /device

    ;--- Add borders to restore image back to original size.

    lsdr  = FLTARR (xdim, ydim)

;    lsdr  = FLTARR (xdim2, ydim2)

    lsdr [0:border-1,         *]                  = refval
    lsdr [xdim-border:xdim-1, *]                  = refval
    lsdr [*,                  0:border-1]         = refval
    lsdr [*,                  ydim-border:ydim-1] = refval
    lsdr [border:xdim-1-border, border:ydim-1-border] = lsdrc

    rmin = MIN (lsdr, max=rmax)
;    rmin = MIN (lsdrc, max=rmax)
    rmin_norm = rmin / frefval
    rmax_norm = rmax / frefval

    ;--- Update FITS header.

    hdulsdr = hdu
    fxaddpar, hdulsdr, 'HISTORY', 'DIFFUSER CORRECTION [MULTIPLICATIVE].'
    fxaddpar, hdulsdr, 'BSCALE',  frefval
    fxaddpar, hdulsdr, 'DATAMIN', rmin_norm
    fxaddpar, hdulsdr, 'DATAMAX', rmax_norm
    fxaddpar, hdulsdr, 'DISPMIN', 0.9
    fxaddpar, hdulsdr, 'DISPMAX', 1.1

    date     = fxpar (hdulsdr, 'DATE-OBS')
    time     = fxpar (hdulsdr, 'TIME-OBS')
    code_obs = strlowcase ( strtrim (fxpar (hdulsdr, 'CODE-OBS'), 2) )
    print, 'date: ', date
    print, 'time: ', time
    print, 'code_obs: ', code_obs
    syear    = STRMID (date, 0, 4)
    smonth   = STRMID (date, 5, 2)
    sday     = STRMID (date, 8, 2)
    shour    = STRMID (time, 0, 2)
    sminute  = STRMID (time, 3, 2)
    ssecond  = STRMID (time, 6, 2)
    print, 'syear/smonth/sday: ', syear, smonth, sday
    print, 'shour/sminute/ssecond: ', shour, sminute, ssecond

    ;--- Write ratio image to disk.

    lsc_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
	       '.chp.' + code_obs + '.lsc.fts'
    PRINT, 'lsc_name: ', lsc_name

    writefits, lsc_name, lsdr, hdulsdr
    writefits, lsc_name, hsk,  ext, /APPEND
;    writefits, lsc_name, lsdrc, hdulsdr

ENDFOR
CLOSE, 11

END
