; pro cme

;
;
;  This program calculates a magnetic field, pressure and density
;  distribution of a magnetic bubble at the surface of the sun,
;  which evolves self-similarly.  
;
;  The model is based on transforming a reasonably simple, symmetric,
;  time independent system to one that allows asymmetry, 3-d currents,
;  and self-similar evolution.
;
;  The density distribution attained from this model can be integrated
;  along the line of sight using existing programs to reproduce pB
;  that can be compared to observations.
;
;  The parameters that will be set in the datafile pramsset.d are (the
;  ones marked by asterskes are those that are likely to be changed):  
;
;	*1. apar- just as in the Bogdan and Low model,
;          this allows an expansion of the field and introduces currents.   	
;
;	*2. xo - this is the shift coordinates between
;	   the bubble system and the sun's coordinate system.
; 	   A choice of xo =1should put the bubble at
;	   the solar surface at the equator.
;
;	*2.5 thetao,phio - these are the spherical shifts. thetao=0
;	     and phio = 0. centers the bubble equator limb
;
;	*3. rbub - this is the radius of the bubble.
;	
;	*4. ao - this is the constant of proportionality between
;	   the stream function S and the pressure Pi
;
;	*5. Pio - this is the background pressure
;
;	*6.  alpha, eta - these are parameters of the self-similar
;    	     transform - eta normalizes the time to meaningful units,
;	     and alpha positive = accel, neg = decel, zero = const. V
;	     note that this program requires alpha = 0, although it 
;	     could be easily adapted
;
;
;    	7.  cpar,dpar - background density, pressure parameters, 
;	as in BandL - I usually set these to zero - but can be used
;	to set up a hydrostatic equilibrium background solution - note
;
;	8. alnot - the eigenvalue parameter of the twisted flux
;	    rope model
;		*note - read in as alnot*rbub, normalized in readprams*
;
; 	*9. sig1 the rotation about the CME neutral line
;	*10. out a parameter which scales the outer field solution
;
;**************************************************************************
	pro drive

	common sscoord,rsquig,phiss,time
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out
	common readphiss,phissuse,accel,etaprams


;
; set time step
; NOTE: times can also be put into an array in order to
; get time-dependent behavior of CME 
; or called as drive,t
;

	time = 60.*60.

;
; determine if accelerating case
; NOTE - for now I've only got one accelerating solution, but it
; is not difficult to make more, just go to FIT/veluse.pro and
; make a new profile there.  Otherwise, the parameter velmult
; will allow speeding up and slowing down, with the basic profile
; the same (just a different asymptotic speed)
;

	read,'constant velocity (0) or accelerating (1)?',accel

	if accel eq 1 then begin
	  alphaprime = alpha/eta
	  restore,'phiss_vs_t_'+strtrim(string(alphaprime),2)+'_'+strtrim(string(eta),2)+'.dat'
;
; but be careful - time is normalized to etaprams
;
          timearr=timearr*sqrt(etaprams)/sqrt(eta)
	  test = where(abs(timearr-time) eq min(abs(timearr-time)))
	  phissuse = phissarr(test)
	  phissuse = phissuse(0)
	endif

;
; read in the parameters of the model
;
	if accel eq 0 then openr,5,'pramsset.d'
	if accel eq 1 then openr,5,'pramsset_accel.d'

	readprams

	close,5

;
; read in the coordinates r,theta,phi 
; - if thetao and phio are nonzero in the parameters file, you may
; want to explicitly include these in your theta,phi definition in this
; drive program - this is useful if theta,phi are not inputted but are 
; defined in the program and you want to adjust them without adjusting
; the program.  I have not done so now, though.
; NOTE: r,theta,phi can be any dimension arrays resulting in outputted 
; physical parameters of the same dimensions which can then be plotted
; For example, a r,theta,phi could be 3-dimensional arrays and
; then so would be the outputted physical parameters - thus the output
; info could be viewed along any line-of-sight.  Or else they
; could be written in terms of plane-of-sky variables rho&th and a 
; line-of-sight coordinate along which density could be integrated to get pB.
; 

	read,'input r, theta, phi',r,theta,phi

;
; then call subroutine denscalc which will determine plasma parameters
; density, temperature, pressure, and magnetic field
;
	denscalc,r,theta,phi

;
;
; calculate temperature and plasma beta
;
	Tmp = 6.02e-9*Pres/Dens/1.6696e-24

	magpres=bstrengthphys*bstrengthphys/8./!pi

	beta=magpres*0.
	test = where(magpres ne 0.)
	if min(test) ne -1 then beta(test)= pres(test)/magpres(test)

;
; calculate velocity (in km/sec)
;

	dphidt = sqrt((eta*phiss - 2.d0*alpha)/phiss)
	v = (r*6.96d5*dphidt)/phiss
;
; plot these, or whatever
;
	stop
	end

;**************************************************************************
	pro readprams
	
	 
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out

	readf,5,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out

	alnot = alnot/rbub

	etaprams=eta
	eta = eta*(velmult^2)
;
;  but keep alphaprime = alpha/eta unchanged
;
	alpha = alpha*(velmult^2)

	close,5

	end

;**************************************************************************
	pro coords,rpB,thetapB,phipB

	common transcoord,rlam
	common shiftcoord,Rcap,Thcap,Phcap
	common sscoord,rsquig,phiss,time
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out
	common readphiss,phissuse,accel,etaprams

;
;  CALLED BY DENSCALC
;

;  
;  there are so many coordinate transforms, most (all but
;  the transform from plane-of-sky coords to the physical pB integration
;  coords, which is done in pBint *IN ORIGINAL PROGRAM*) will be done
;  in this procedure to keep them straight.
;
;  the self-similar transformation:
;  phiss will be set proportional to time, in other words,
;  alpha = 0

		if alpha eq 0. then phiss = sqrt(eta)*time + 1.

;
;  for non-zero alpha,
;  phiss will be determined from the diffeq as in Gibson and Low 1998
;

		if alpha ne 0. then phiss = phissuse
;
		rsquig = rpB/phiss
;
;  the radial coordinate is now transformed introducing expansion of
;  the field
;

		rlam = rsquig + apar

;  the angular coordintes are the same in the general coordinate
;  system.  HOWEVER, the force balance equations will actually
;  be solved in yet another system, this one with a shifted center,
;  This shifted symmetry introduces asymmetry.
;  We must first move to Cartesian coordinates.
;  For now we will just shift in x-  this will put the bubble at
;  the equator
;
;  xtr and ztr are swapped from standard because phi = 0
;  is the central meridian in our case
;
		xtr = rlam*sin(thetapB*!pi/180.)*sin(phipB*!pi/180.) - xo
		ytr = rlam*sin(thetapB*!pi/180.)*cos(phipB*!pi/180.)
		ztr = rlam*cos(thetapB*!pi/180.) 

; now transform to spherical coords

		Rcap = sqrt(xtr*xtr + ytr*ytr + ztr*ztr)
;
; we want to rotate the coordintes in the bubble 
;  about the x-axis by an angle sig1 (parameter)
;
		sigma = sig1
;
		ytilde = ytr*cos(sigma*!pi/180.) + ztr*sin(sigma*!pi/180.)
		ztilde = ztr*cos(sigma*!pi/180.) - ytr*sin(sigma*!pi/180.)
;
		Thcap = acos(ztilde/Rcap)*180./!pi
;
		Phcap = atan(xtr/ytilde)*180./!pi
;
; but be careful of the degeneracy in tangent!
;
		tandegen1 = where (ytilde lt 0. )
		if min(tandegen1) ne -1 then Phcap(tandegen1) = 180 + atan(xtr(tandegen1)/ytilde(tandegen1))*180/!pi
;
		end

;**************************************************************************
	pro getinside,rin


; 
; CALLED BY DENSCALC
;
	common sscoord,rsquig,phiss,time
	common transcoord,rlam
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out
	common inblock,Pbackin,Dbackin

;
        r1 = xo 
 
;   just so I dont have to type a lot I am calling the radius and theta
;   coors rout and thout
;
        rout = rin
;
; calculte theta along the boundary for each r, and rotate so that the
; bubble is at the equator
;
        thout = acos((r1*r1 + rout*rout - rbub*rbub)/(2*rout*r1))*180/!pi
;
;    break down bits of stream function
;
        mu = cos(thout*!pi/180.)
        al1 = r1*r1 - rbub*rbub
        al2 = 2*r1*r1 - rbub*rbub
;
        psi0 = mu
;
        f = r1*(rout*rout + al1) - rout*mu*al2
        g = al1*al1 + r1*r1*rout*rout - 2*rout*r1*al1*mu
        h = 1/(sqrt(g))
        psi1 = (-1/rbub)*f*h
;
        i = rout*rout + r1*r1 - 2*rout*r1*mu
        psi3 = (1/rbub)*sqrt(i)
;
        Streamout = psi0 + psi1 + psi3
;
;  now take derivatives for magnetic fields, etc.
;
        dpsi0dr = 0.
        dpsi0dth = - sin(thout*!pi/180.)
        dpsi0dr2 = 0.
        dpsi0drdth = 0.
        dpsi0dth2 = -1*cos(thout*!pi/180.)
;
        dfdr = 2*r1*rout - mu*al2
        dfdmu = -rout*al2
        dfdrdmu = -al2
        dfdr2 = 2*r1
        dfdmu2 = 0.
;
        dgdr = 2*rout*r1*r1 - 2*r1*al1*mu
        dgdmu = -2*rout*r1*al1
        dgdrdmu = -2*r1*al1
        dgdr2 = 2*r1*r1
        dgdmu2 = 0.
;
        dhdr = -dgdr/2/(g^1.5)
        dhdmu = -dgdmu/2/(g^1.5)
        dhdrdmu = -dgdrdmu/2/(g^1.5) + 3*dgdmu*dgdr/4/(g^2.5)
        dhdr2 = -dgdr2/2/(g^1.5) + 3*dgdr*dgdr/4/(g^2.5)
        dhdmu2 = 1.5*(dgdmu^2)/2/(g^2.5)
;
        dpsi1dr = (-1/rbub)*(dfdr*h + f*dhdr)
        dpsi1dr2 = (-1/rbub)*(2*dfdr*dhdr + dfdr2*h + f*dhdr2)
;
        dpsi1dmu = (-1/rbub)*(dfdmu*h + f*dhdmu)
        dpsi1drdmu = (-1/rbub)*(dfdmu*dhdr + dfdr*dhdmu + dfdrdmu*h + f*dhdrdmu)
        dpsi1dmu2 = (-1/rbub)*(dfdmu2*h + 2*dfdmu*dhdmu + f*dhdmu2)
;
        dpsi1dth = -sin(thout*!pi/180.)*dpsi1dmu
        dpsi1drdth = -sin(thout*!pi/180.)*dpsi1drdmu
        dpsi1dth2 = -cos(thout*!pi/180.)*dpsi1dmu +(sin(thout*!pi/180.))^2*dpsi1dmu2
;
        didr = 2*rout - 2*r1*mu
        didmu = -2*rout*r1
        didrdmu = -2*r1
        didr2 = 2.
        didmu2 = 0.
;
        dpsi3dr = (1/rbub)*didr/2/sqrt(i)
        dpsi3dmu = (1/rbub)*didmu/2/sqrt(i)
        dpsi3drdmu = (1/rbub)*(didrdmu/2/sqrt(i) - didmu*didr/4/(i^1.5))
        dpsi3dr2 = (1/rbub)*(didr2/2/sqrt(i) - (didr)^2/4/(i^1.5))
        dpsi3dmu2 = (1/rbub)*didmu2/2/sqrt(i) -(1/rbub)*(didmu^2)/4/(i^1.5)
;
        dpsi3dth = -sin(thout*!pi/180.)*dpsi3dmu
        dpsi3drdth = -sin(thout*!pi/180.)*dpsi3drdmu
        dpsi3dth2 = -cos(thout*!pi/180.)*dpsi3dmu + (sin(thout*!pi/180.))^2*dpsi3dmu2
;
        dAdr = dpsi0dr + dpsi1dr + dpsi3dr
        dAdth = dpsi0dth + dpsi1dth + dpsi3dth
        dAdrdth = dpsi0drdth + dpsi1drdth + dpsi3drdth
        dAdr2 = dpsi0dr2 + dpsi1dr2 + dpsi3dr2
        dAdth2 = dpsi0dth2 + dpsi1dth2 + dpsi3dth2
;
;  now calculate field
;
        brlambout =  dAdth/rout/rout/sin(thout*!pi/180.)
        bthlambout =  -dAdr/rout/sin(thout*!pi/180.)
        bphlambout = 0.
;
;  now calculate field derivatives
;
        dbrdr = (-2.*dAdth/rout+ dAdrdth)/rout/rout/sin(thout*!pi/180.)
        dbthdr = (dAdr/rout - dAdr2)/rout/sin(thout*!pi/180.)
;
        dbrdth = -cos(thout*!pi/180.)*dAdth/rout/rout/(sin(thout*!pi/180.))^2
        dbrdth = dbrdth + dAdth2/rout/rout/sin(thout*!pi/180.)
        dbthdth = cos(thout*!pi/180.)*dAdr/rout/(sin(thout*!pi/180.))^2
        dbthdth = dbthdth - dAdrdth/rout/sin(thout*!pi/180.)
;
        rnolam = rout - apar
        Presin= (rout^2/rnolam^2)*(1-(rout^2/rnolam^2))*(brlambout^2)/8/!pi
        Presin= Presin/(phiss^4)
;
;
;  here are the magnetic pressure in the rpB,thetapB
;  phipB coordinate systems !  we have to also get rid of selfsim stuff
;
;  dlamdr is one, because rlam = rsquig + apar.
;
        dlamdr = 1
;
        Bpresin = ((brlambout*(rout^2/rnolam^2)/(phiss^2))^2 + (bthlambout*(rout/rnolam)*dlamdr/(phiss^2))^2)/8./!pi
;
        Pbackin = Presin + Bpresin
        Pbackin = Pbackin*out*out
;
;  now calculate the background density
;
        dldr = -apar/rnolam/rnolam
        lr = (apar+rnolam)/rnolam
;
        ufunc = (r1*r1 + rout*rout - rbub*rbub)/(2*rout*r1)
        dufdr = (1/r1) - ufunc/rout
 
        dthdr = -(1/sqrt(1-ufunc*ufunc))*dufdr
 
        dstuffdr = -(lr*dldr*brlambout*brlambout + lr*lr*brlambout*dbrdr + .5*dldr*bthlambout*bthlambout + lr*bthlambout*dbthdr)/4/!pi
;
;
        dstuffdth= -(lr*lr*brlambout*dbrdth + lr*bthlambout*dbthdth)/4/!pi
;
        dpdr = dstuffdr + dstuffdth*dthdr
 
        Rsun = 6.96d10
        GMm = 221.3d0
	mprot = 1.674d-24
        F = GMm/((rout-apar)^2)/(Rsun^2) + alpha*(rout-apar)*Rsun*mprot
;
        Dbackin = dpdr/F/Rsun/phiss/phiss/phiss
        Dbackin = Dbackin*out*out
;
        end

;**************************************************************************
	pro getoutside,rtemp,thtemp,phtemp


; 
; CALLED BY FIELDCALC
;
	common sscoord,rsquig,phiss,time
	common transcoord,rlam
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out
	common outblock,Streamout,brlambout,bthlambout,bphlambout,tderivout,brderivout,bthphderivout
;
;  we are going to rotate the outside field so that its axis
;  of symmetry lies along the vector r1 defined by offset
;  xo , with angle sigma to the equator.
;  we do this by transforming through cartesian coords
;
;  x and y are defined so that phi = 0 is the central meridian
;
		xtr = rtemp*sin(thtemp*!pi/180.)*sin(phtemp*!pi/180.) 
		ytr = rtemp*sin(thtemp*!pi/180.)*cos(phtemp*!pi/180.)
		ztr = rtemp*cos(thtemp*!pi/180.) 

; now transform to spherical coords (radius is unchanged)

		rout = rtemp
;
		sigma = 0.
;
; but be careful of the degeneracy in tangent!
;
		tandegen1 = where (xo lt 0.)
		if min(tandegen1) ne -1 then sigma(tandegen1) = 180. 
;
; the tilde coords are the new x and z with up defined as above
; note, because the outside field is azimuthally symmetric, we dont
; have to worry about y- note, however, that sin and cos have
; been swapped compared to the inner solution rotation.  This
;  is because the inner solution has its axis of symmetry naturally
;  at the equator, and the outer at the pole
;
		xtilde = xtr*sin(sigma*!pi/180.) - ztr*cos(sigma*!pi/180.)
		ztilde = ztr*sin(sigma*!pi/180.) + xtr*cos(sigma*!pi/180.)
;
		thout = acos(ztilde/rout)*180./!pi
;
		phout =  atan(xtilde/ytr)*180./!pi
;
; but be careful of the degeneracy in tangent!
;
		tandegen1 = where (ytr lt 0.)
		if min(tandegen1) ne -1 then phout(tandegen1) = 180 + atan(xtilde(tandegen1)/ytr(tandegen1))*180/!pi
;
;
        r1 = sqrt(xo*xo )
;
;    break down bits of stream function
;
	mu = cos(thout*!pi/180.)
	al1 = r1*r1 - rbub*rbub 
	al2 = 2*r1*r1 - rbub*rbub
;
	psi0 = mu
;
	f = r1*(rout*rout + al1) - rout*mu*al2
	g = al1*al1 + r1*r1*rout*rout - 2*rout*r1*al1*mu
	h = 1/(sqrt(g))
	psi1 = (-1/rbub)*f*h 
;
	i = rout*rout + r1*r1 - 2*rout*r1*mu
	psi3 = (1/rbub)*sqrt(i)
;
	Streamout = psi0 + psi1 + psi3  
;
;  now take derivatives for magnetic fields, etc.
;
	dpsi0dr = 0.
	dpsi0dth = - sin(thout*!pi/180.)
	dpsi0dr2 = 0.
	dpsi0drdth = 0.
;
	dfdr = 2*r1*rout - mu*al2
	dfdmu = -rout*al2
	dfdrdmu = -al2
	dfdr2 = 2*r1
;
	dgdr = 2*rout*r1*r1 - 2*r1*al1*mu
	dgdmu = -2*rout*r1*al1
	dgdrdmu = -2*r1*al1
	dgdr2 = 2*r1*r1 
;
	dhdr = -dgdr/2/(g^1.5)
	dhdmu = -dgdmu/2/(g^1.5)
	dhdrdmu = -dgdrdmu/2/(g^1.5) + 3*dgdmu*dgdr/4/(g^2.5)
	dhdr2 = -dgdr2/2/(g^1.5) + 3*dgdr*dgdr/4/(g^2.5)
;
	dpsi1dr = (-1/rbub)*(dfdr*h + f*dhdr)
	dpsi1dr2 = (-1/rbub)*(2*dfdr*dhdr + dfdr2*h + f*dhdr2)
;
	dpsi1dmu = (-1/rbub)*(dfdmu*h + f*dhdmu)
	dpsi1drdmu = (-1/rbub)*(dfdmu*dhdr + dfdr*dhdmu + dfdrdmu*h + f*dhdrdmu)
;	
	dpsi1dth = -sin(thout*!pi/180.)*dpsi1dmu
	dpsi1drdth = -sin(thout*!pi/180.)*dpsi1drdmu
;
	didr = 2*rout - 2*r1*mu
	didmu = -2*rout*r1
	didrdmu = -2*r1
	didr2 = 2.
;	
	dpsi3dr = (1/rbub)*didr/2/sqrt(i)
	dpsi3dmu = (1/rbub)*didmu/2/sqrt(i)
	dpsi3drdmu = (1/rbub)*(didrdmu/2/sqrt(i) - didmu*didr/4/(i^1.5))
	dpsi3dr2 = (1/rbub)*(didr2/2/sqrt(i) - (didr)^2/4/(i^1.5))
;
	dpsi3dth = -sin(thout*!pi/180.)*dpsi3dmu
	dpsi3drdth = -sin(thout*!pi/180.)*dpsi3drdmu
;
	dAdr = dpsi0dr + dpsi1dr + dpsi3dr
	dAdth = dpsi0dth + dpsi1dth + dpsi3dth
	dAdrdth = dpsi0drdth + dpsi1drdth + dpsi3drdth
	dAdr2 = dpsi0dr2 + dpsi1dr2 + dpsi3dr2
;
;  now calculate field
;
	brlambout1 =  dAdth/rout/rout/sin(thout*!pi/180.)
	bthlambout1 =  -dAdr/rout/sin(thout*!pi/180.)
	bphlambout1 = 0.
;
; But now we need to get derivatives of blittle with respect to
; the physical coord r, which will give us pressure and density
;  We do so by going through the cartesian coordinates.
;
		st = sin(thout*!pi/180.)
		ct = cos(thout*!pi/180.)
		sp = sin(phout*!pi/180.)
		cp = cos(phout*!pi/180.)
;
		blittleX1 = brlambout1*st*sp + bthlambout1*ct*sp	
		blittleY = brlambout1*st*cp + bthlambout1*ct*cp	
		blittleZ1 = brlambout1*ct - bthlambout1*st	
;
; but now we have to translate to the rotated frame for x and z
;
		blittleX = sin(sigma*!pi/180.)*blittleX1 + cos(sigma*!pi/180.)*blittleZ1 
		blittleZ = sin(sigma*!pi/180.)*blittleZ1 - cos(sigma*!pi/180.)*blittleX1 
;
; and finally back to spherical coords
;
		streal = sin(thtemp*!pi/180.)
		ctreal = cos(thtemp*!pi/180.)
		spreal = sin(phtemp*!pi/180.)
		cpreal = cos(phtemp*!pi/180.)
		brlambout = blittleX*streal*spreal + blittleY*streal*cpreal+ blittleZ*ctreal
;	
		bthlambout =blittleX*ctreal*spreal+blittleY*ctreal*cpreal - blittleZ*streal
;
		bphlambout = -blittleY*spreal + blittleX*cpreal
;
;  now calculated tderivout
;
	dbr1dr = (-2.*dAdth/rout+ dAdrdth)/rout/rout/sin(thout*!pi/180.)
	dbth1dr = (dAdr/rout - dAdr2)/rout/sin(thout*!pi/180.) 
;
	tderivout = (brlambout1*dbr1dr + bthlambout1*dbth1dr)/4/!pi	
;
	ss = sin(sigma*!pi/180.)
	cs = cos(sigma*!pi/180.)
;
	dbrdr = (ss*(dbr1dr*st*sp+dbth1dr*ct*sp)+cs*(dbr1dr*ct-dbth1dr*st))*streal*spreal
	dbrdr = dbrdr + (dbr1dr*st*cp + dbth1dr*ct*cp)*streal*cpreal
	dbrdr = dbrdr + (-cs*(dbr1dr*st*sp+dbth1dr*ct*sp)+ss*(dbr1dr*ct-dbth1dr*st))*ctreal
;
	brderivout = brlambout*dbrdr/4./!pi
	bthphderivout = tderivout - brderivout
;
;
;
;   and also flip the sign about the equator
;
		south = where(phout gt 0. and phout lt 180.)
		if min(south) ne -1 then brlambout(south) = -1*brlambout(south)
		if min(south) ne -1 then bthlambout(south) = -1*bthlambout(south)
		if min(south) ne -1 then bphlambout(south) = -1*bphlambout(south)
	end

;**************************************************************************
	pro fieldcalc,rpB,thetapB,phipB

        common results,Dens,Pres,bstrengthphys,brphys,bthphys,bphphys
	common sscoord,rsquig,phiss,time
	common transcoord,rlam
	common shiftcoord,Rcap,Thcap,Phcap
	common fieldprams,Pi,blittlerlamb,blittlethlamb,blittlephlamb,Stream
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out
	common outblock,Streamout,brlambout,bthlambout,bphlambout,tderivout,brderivout,bthphderivout


;
;  CALLED BY DENSCALC
;


;  first calculate axisymm. stream function in shifted coordinates
;
		st= sin(Thcap*!pi/180.)
		ct= cos(Thcap*!pi/180.)
		sp= sin(Phcap*!pi/180.)
		cp= cos(Phcap*!pi/180.)
;
;  in the region Rcap lt rbub
		
		gfun = sin(alnot*Rcap)/alnot/Rcap - cos(alnot*Rcap)
		gfunot = sin(alnot*rbub)/alnot/rbub - cos(alnot*rbub)

		Stream = (4*ao*!pi/alnot/alnot)*(gfun*rbub^2/gfunot-Rcap^2)*st^2
;  
;  in the region Rcap ge rbub
;  call routine that calculates stream function, etc

 		outside = where(Rcap ge rbub)
		if min(outside) ge 0 then getoutside,rlam(outside),thetapB(outside),phipB(outside)
;
		if min(outside) ge 0 then Stream(outside) = out*Streamout(*)
;
;  define pressure
;
		Pi = ao*Stream + Pio	
		if min(outside) ge 0 then Pi(outside) = Pio
;
; now calculate little b (blittle), that goes with this Stream function
; and pressure in these coordinates
; To do this, we need to use the derivatives of Stream with respect
; to the cap coords.
;
		dSdT = Stream*2*ct/st
;
		dgfundr = cos(alnot*Rcap)/Rcap - sin(alnot*Rcap)/Rcap/Rcap/alnot + alnot*sin(alnot*Rcap)
;
		dSdR = (4*ao*!pi/alnot/alnot)*(dgfundr*rbub^2/gfunot-2*Rcap)*st^2
;
		blittleR = (1/Rcap/st)*((1/Rcap)*dSdT)
		blittleT = -(1/Rcap/st)*dSdR
;
;  ADD THE PHI FIELD!!!!!
;
		Q = alnot*Stream
;
		blittleP = -(1/Rcap/st)*Q
;
; But now we need to get derivatives of blittle with respect to
; the physical coord r, which will give us pressure and density
;  We do so by going through the cartesian coordinates.
;
		blittleX = blittleR*st*sp + blittleT*ct*sp + blittleP*cp
		blittleY1 = blittleR*st*cp + blittleT*ct*cp - blittleP*sp	
		blittleZ1 = blittleR*ct - blittleT*st	
;
; but now we have to translate to the rotated frame for y and z
;
		sigma = sig1
;
		blittleY = cos(sigma*!pi/180.)*blittleY1 - sin(sigma*!pi/180.)*blittleZ1 
		blittleZ = cos(sigma*!pi/180.)*blittleZ1 + sin(sigma*!pi/180.)*blittleY1 
;
		ctreal = cos(thetapB*!pi/180.)
		streal = sin(thetapB*!pi/180.)
		cpreal = cos(phipB*!pi/180.)
		spreal = sin(phipB*!pi/180.)
;
;  these are the "little b" magnetic field components- the little b r
;  is needed in calculating pressure and density in the pB coords
;
		blittlerlamb = blittleX*streal*spreal + blittleY*streal*cpreal+ blittleZ*ctreal
;	
		blittlethlamb =blittleX*ctreal*spreal+blittleY*ctreal*cpreal - blittleZ*streal
;
		blittlephlamb = -blittleY*spreal + blittleX*cpreal
;
;   we need to put in the proper values for the outside field
;
		if min(outside) ge 0 then blittlerlamb(outside)= out*brlambout(*)
		if min(outside) ge 0 then blittlethlamb(outside)= out*bthlambout(*)
		if min(outside) ge 0 then blittlephlamb(outside)= out*bphlambout(*)
;
;  and finally, here are the magnetic field coordinates in the rpB,thetapB
;  phipB coordinate systems !  we have to also get rid of selfsim stuff
;
;  dlamdr is one, because rlam = rsquig - apar.
;
		dlamdr = 1
;
		brphys = (blittlerlamb*(rlam/rsquig)^2)/(phiss^2)
		bthphys = blittlethlamb*(rlam/rsquig)*dlamdr/(phiss^2)
		bphphys = blittlephlamb*(rlam/rsquig)*dlamdr/(phiss^2)
;
; check to make sure the geometry is right by comparing field strengths
;
;		bstrength1 = sqrt(blittleR^2+blittleT^2)
;		bstrength2 = sqrt(blittleX^2+blittleY^2+blittleZ^2)
;		bstrength3 = sqrt(blittlerlamb^2 + blittlethlamb^2 + blittlephlamb^2)
;
;  but bstrengthphys is the actual physical field strength in the rpB, 
; thetapB, phipB coords
;
		bstrengthphys = sqrt(brphys^2 + bthphys^2 + bphphys^2)
		end
;**************************************************************************

	pro denscalc,rpB,thetapB,phipB

	common shiftcoord,Rcap,Thcap,Phcap
	common results,Dens,Pres,bstrengthphys,brphys,bthphys,bphphys
	common fieldprams,Pi,blittlerlamb,blittlethlamb,blittlephlamb,Stream
	common prams,apar,xo,thetao,phio,rbub,ao,Pio,alpha,eta,velmult,cpar,dpar,alnot,sig1,out
	common transcoord,rlam
	common sscoord,rsquig,phiss,time
	common outblock,Streamout,brlambout,bthlambout,bphlambout,tderivout,brderivout,bthphderivout
	common inblock,Pbackin,Dbackin

;
;  first shift coordinate systems
;
		coords,rpB,thetapB,phipB
;
;  now calculate the field
;
		fieldcalc,rpB,thetapB,phipB
;
;

		Pres= (rlam^2/rsquig^2)*(1-(rlam^2/rsquig^2))*(blittlerlamb^2)/8/!pi + (rlam^2/rsquig^2)*Pi
;
;  put in background pressure
;  but also match total pressure at boundary
;
		Presback = (cpar/(dpar+1))*rsquig^(-dpar-1)
;
		inside = where(Rcap lt rbub)
;
		if min(inside) ge 0 then begin
			getinside,rlam(inside)
			Presback(inside) = Pbackin*(phiss^4) + (cpar/(dpar+1))*rsquig(inside)^(-dpar-1)
		endif
;
		Pres = Pres + Presback
;
		Pres = Pres/(phiss^4)
;
;  now calculate density
; 
;  first need to calculate tderiv = dT/drlam, where T = Pi + Bstrength (squared)
;  in the Rcap,Thcap system
;
		st = sin(thetapB*!pi/180.)
		ct = cos(thetapB*!pi/180.)
		sp = sin(phipB*!pi/180.)
		cp = cos(phipB*!pi/180.)
;
		sTbig = sin(Thcap*!pi/180.)
		cTbig = cos(Thcap*!pi/180.)
		sPbig = sin(Phcap*!pi/180.)
		cPbig = cos(Phcap*!pi/180.)
;
		mu = Rcap*sTbig
;
		
		gfunot = sin(alnot*rbub)/alnot/rbub - cos(alnot*rbub)
;
		dSdT = Stream*2*cTbig/sTbig
;
		dgfundr = cos(alnot*Rcap)/Rcap - sin(alnot*Rcap)/Rcap/Rcap/alnot + alnot*sin(alnot*Rcap)
;
		dSdR = (4*ao*!pi/alnot/alnot)*(dgfundr*rbub^2/gfunot-2*Rcap)*sTbig^2
;

;
;
		d2SdTdR = dSdR*2.*cTbig/sTbig
;		
		d2gfundr2 = (-Rcap*Rcap*alnot*sin(alnot*Rcap) - Rcap*cos(alnot*Rcap) + alnot*Rcap*Rcap*Rcap*alnot*cos(alnot*Rcap) - cos(alnot*Rcap)*Rcap + 2*sin(alnot*Rcap)/alnot)/Rcap/Rcap/Rcap
;
		d2SdR2 = (4*ao*!pi/alnot/alnot)*(d2gfundr2*rbub^2/gfunot-2)*sTbig^2
;		
		d2SdT2 = Stream*2*(cTbig^2-sTbig^2)/(sTbig^2)
;
		dRdmu = sTbig
		dthetadmu = cTbig/Rcap
;
		dPidR = ao*dSdR
;
		dPidmu=ao*(dRdmu*dSdR + dthetadmu*dSdT)
;
		dTdR = - 4*(dSdT^2)/(Rcap^5)/(sTbig^2)
		dTdR = dTdR +2*dSdT*d2SdTdR/(Rcap^4)/(sTbig^2)
		dTdR = dTdR -2*(dSdR^2)/(Rcap^3)/(sTbig^2)
		dTdR = dTdR +2*dSdR*d2SdR2/(Rcap^2)/(sTbig^2)
;
		dTdR = dTdR -2*alnot*alnot*(Stream^2)/(Rcap^3)/(sTbig^2)
		dTdR = dTdR + 2*alnot*alnot*Stream*dSdR/(Rcap^2)/(sTbig^2)
;
		dTdR = dTdR/8/!pi
		dTdR = dTdR + dPidR
;
		dTdmu =  -2*((dSdT/Rcap)^2)/(mu^3)
		dTdmu = dTdmu +2*(dSdT/Rcap)*(dRdmu*(-dSdT/(Rcap^2)+d2SdTdR/Rcap)+dthetadmu*d2SdT2/Rcap)/(mu^2)
		dTdmu = dTdmu -2*(dSdR^2)/(mu^3)
		dTdmu = dTdmu +2*dSdR*(dRdmu*d2SdR2+dthetadmu*d2SdTdR)/(mu^2)
;
		dTdmu = dTdmu -2*alnot*alnot*(Stream^2)/(mu^3)
		dTdmu = dTdmu +2*alnot*alnot*Stream*(dRdmu*dSdR+dthetadmu*dSdT)/(mu^2)
;
		dTdmu = dTdmu/8/!pi
		dTdmu = dTdmu + dPidmu

		dRdlam = (2*rlam - 2*xo*st*sp)/2/Rcap
;
;  remember we are in a rotated mu frame
;
		
		sigma = sig1
;	
		cs = cos(sigma*!pi/180.)
		ss = sin(sigma*!pi/180.)
;	
		dmudlam = (2*rlam*(st^2*(sp^2+cp^2*cs^2) + ct^2*ss^2 + 2*st*ct*cp*ss*cs) - 2*xo*st*sp)/2/mu
;	
		tderivR = dTdR*dRdlam
		tderivmu = dTdmu*dmudlam
;
		tderiv = tderivR + tderivmu
;
		outside = where(Rcap ge rbub)
		if min(outside) ge 0 then tderiv(outside) = out*out*tderivout(*)
;
;  the force F could include the effects of the self-similar
;  acceleration (alpha ne 0) - otherwise its just gravity.
; Note that to properly include alpha ne 0, however, phiss needs
; to be defined as a differential equation
;
        	Rsun = 6.96d10
        	GMm = 221.3d0
		mprot = 1.674d-24
		F = GMm/(rsquig^2)/(Rsun^2) + alpha*rsquig*Rsun*mprot
;
		Dens = -((rlam/rsquig)^2)*(1-((rlam/rsquig)^2))*tderiv
		Dens = Dens + 2*(rlam/rsquig)*(apar/(rsquig^2))*Pi
		Dens = Dens + (rlam/rsquig)*(apar/(rsquig^2))*(1-2*((rlam/rsquig)^2))*blittlerlamb^2/4/!pi
		Dens = Dens + ((rlam/rsquig)^2)*((apar^2)/(rsquig^2)+2*(apar/rsquig))*(blittlethlamb^2+blittlephlamb^2)/4/!pi/rlam
		Dens = Dens/F/Rsun

;  put in background density
;
		Densback = cpar*rsquig^(-dpar)*Rsun/GMm
;
		if min(inside) ge 0 then Densback(inside) = cpar*rsquig(inside)^(-dpar)*Rsun/Gmm + Dbackin*(phiss^3)

;
		DensbackHEonly = cpar*rsquig^(-dpar)*Rsun/GMm
;
		Dens = Dens + Densback
;
;  put dens in ss coords
;
		Dens = Dens/(phiss^3)
		test = where(dens*0. ne 0.)
		if min(test) ne -1 then stop


;
; now calculate forces (radial for now)
;
; NOTE NONE OF THE REST OF THIS ROUTINE IS ESSENTIAL TO THE 
; THE CALCULATION - THEY ARE JUST FOR TESTING PURPOSES
; (e.g. THE FORCES SHOULD BALANCE)

		densforce = F*Rsun*Dens/phiss^2
;
		tensionforce  = ((rlam/rsquig)^4)*tderiv
		tensionforce = tensionforce - (rlam/rsquig)*(apar/(rsquig^2))*(2*((rlam/rsquig)^2))*blittlerlamb^2/4/!pi
		tensionforce = tensionforce + ((rlam/rsquig)^2)*(apar/rsquig + (apar^2)/(rsquig^2))*(blittlethlamb^2+blittlephlamb^2)/4/!pi/rlam
		tensionforce = tensionforce/phiss^5
;
		tpresforce = -((rlam/rsquig)^2)*tderiv
		tpresforce = tpresforce + 2*(rlam/rsquig)*(apar/(rsquig^2))*Pi
		tpresforce = tpresforce + (rlam/rsquig)*(apar/(rsquig)^2)*(blittlerlamb^2)/4/!pi
		tpresforce = tpresforce + ((rlam/rsquig)^2)*(apar/(rsquig))*(blittlethlamb^2 + blittlephlamb^2)/4/!pi/rlam
;	
; and background pressure
;
		tpresforce = tpresforce/phiss^5 + Densback*GMm/(rsquig^2)/Rsun/phiss^5
;
;  now separate the gas pressure and magnetic pressure forces
;
		dpidrlam = dPidR*dRdlam + dPidmu*dmudlam
;
		dbrdR = -2*dSdT/Rcap/Rcap/Rcap/sTbig +d2SdTdR/Rcap/Rcap/sTbig
		dbrdmu = -dSdT/Rcap/mu/mu + d2SdT2*dthetadmu/Rcap/mu - dSdT*dRdmu/Rcap/Rcap/mu
		dbrdrlam1 = dbrdR*dRdlam + dbrdmu*dmudlam
;
		dbthdR = (dSdR)/(Rcap^2)/(sTbig)
		dbthdR = dbthdR -d2SdR2/(Rcap)/(sTbig)
;
		dbthdmu = (dSdR)/(mu^2)
		DBTHdmu = dbthdmu -(dRdmu*d2SdR2+dthetadmu*d2SdTdR)/(mu)
;
		dbthdrlam1 = dbthdR*dRdlam + dbthdmu*dmudlam
;
		dbphdR =  alnot*(Stream)/(Rcap^2)/(sTbig)
		dbphdR = dbphdR - alnot*dSdR/(Rcap)/(sTbig)

;
		dbphdmu =  alnot*(Stream)/(mu^2)
		dbphdmu = dbphdmu -alnot*(dRdmu*dSdR+dthetadmu*dSdT)/(mu)
;
		dbphdrlam1 = dbphdR*dRdlam + dbphdmu*dmudlam
;
;  but transform to a derivative in the littlelamb coords
;
	dbrdrlam = (dbrdrlam1*sTbig*sPbig+dbthdrlam1*cTbig*sPbig + dbphdrlam1*cPbig)*st*sp
	dbrdrlam = dbrdrlam + (cs*(dbrdrlam1*sTbig*cPbig + dbthdrlam1*cTbig*cPbig - dbphdrlam1*sPbig) -ss*(dbrdrlam1*cTbig - dbthdrlam1*sTbig))*st*cp
	dbrdrlam = dbrdrlam + (ss*(dbrdrlam1*sTbig*cPbig+dbthdrlam1*cTbig*cPbig - dbphdrlam1*sPbig)+cs*(dbrdrlam1*cTbig-dbthdrlam1*sTbig))*ct
;
                dbr2drlam = (2*dbrdrlam*blittlerlamb)/8./!pi
;
                tderivnopi = tderiv - dpidrlam
;
                dbthph2drlam =  tderivnopi - dbr2drlam
;
                if min(outside) ge 0 then begin
                        dpidrlam(outside) = 0.
                        dbr2drlam(outside) = out*out*brderivout(*)
                        dbthph2drlam(outside) = out*out*bthphderivout(*)                       
		endif
;
;  gas pressure
;
                gaspresforce = (2.*rlam*apar/(rsquig^3) - 4*apar*(rlam^3)/(rsquig^5))*(blittlerlamb^2)/8./!pi  + 2.*rlam*apar*Pi/(rsquig^3)
                gaspresforce = gaspresforce - ((rlam/rsquig)^2)*(1 - (rlam/rsquig)^2)*(dbr2drlam)
                gaspresforce = gaspresforce - dpidrlam*(rlam/rsquig)^2
;
                gaspresforce = gaspresforce/phiss^5 + Densback*GMm/(rsquig^2)/Rsun/phiss^5
;
; magnetic pressure
;
                magpresforce = 4.*apar*(rlam^3)*(blittlerlamb^2)/8./!pi/(rsquig^5) +2.*rlam*apar*(blittlethlamb^2+blittlephlamb^2)/8./!pi/(rsquig^3)
;
                magpresforce = magpresforce - dbr2drlam*((rlam/rsquig)^4)
                magpresforce = magpresforce/phiss^5 - (dbthph2drlam)*((rlam/rsquig)^2)/phiss^5
;
;
                gaspresforce = -1*gaspresforce
                magpresforce = -1*magpresforce
                tensionforce = -1*tensionforce
;

                end
;***********************************************************************
