;+
; NAME	mk3_rscan.pro
;
; PURPOSE	Read Mk3 FITS images (polar coordinates)
;		& plot radial scan.
;
; SYNTAX	mk3_rscan, fits_list, angle, /text
;
;               fits_list       file containing a list of FITS images
;               angle           angle (degrees) for radial scan
;               text            option to output an ASCII data file.
;
; EXAMPLE	mk3_rscan, 'list',  0.0 
;		mk3_rscan, 'list', 90.0, /text
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    31 August 2001
;-

pro mk3_rscan, fits_list, angle, text=text

CLOSE, 1
fits_name = ''
basename  = ''
plotname  = ''
dataname  = ''

set_plot, 'PS'

posang = findgen (720) / 2.0		; Position Angle array.
sangle = STRTRIM (STRING (angle, FORMAT='(F7.2)'), 2)

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   plotname = basename + '.a' + sangle + '.ps'
   print, plotname
   device, filename = plotname

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   telescop = fxpar (hdu, 'TELESCOP')
   dataform = fxpar (hdu, 'DATAFORM')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')

   ;--- Verify that intstrument is MK3.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'MK3')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be MK3.'
      RETURN
   END

   ;--- Verify that dataform is CYLINDRICAL.

   dataform = STRTRIM (dataform, 2)
   polpos = -1
   cylpos = -1
   cylpos = STRPOS (dataform, 'CYL')
   polpos = STRPOS (dataform, 'POL')
;   PRINT, 'cylpos: ', cylpos
;   PRINT, 'polpos: ', polpos
   IF (cylpos EQ -1 AND polpos EQ -1)  THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.'
      PRINT, 'It should be CYLINDRICAL or POLAR.'
      PRINT, 'ONLY polar coordinate data is valid with this program.'
      RETURN
   END

   ;--- r0 = radius of first pixel height.
   ;--- dr = increment in radius (Rsun units) between adjacent pixels.
   ;
   ;--- CRPIX1 is the pixel location of sun center (r=0) [pixel units].
   ;--- CDELT1 is the radial increment [Rsun/pixel].
   ;--- Compute the r0 value from the FITS parameters crpix1 & cdelt1,
   ;--- since r0 does not appear explicitly in the FITS header.

   crpix1   = fxpar (hdu, 'CRPIX1')
   cdelt1   = fxpar (hdu, 'CDELT1')
   r0       = - (crpix1 * cdelt1)
   dr       = cdelt1

   ;--- Get r0 & dr from rmin & rmax parameters.

   rmin = fxpar (hdu, "RMIN")
   rmax = fxpar (hdu, "RMAX")

   r0 = rmin
   dr = (rmax - rmin) / 127.0

   radbuf = fltarr (xdim)
   FOR i = 0, xdim - 1 DO	$
   BEGIN
      radbuf [i] = r0 + i * dr
   END

   bunit    = fxpar (hdu, 'BUNIT')
   bunit    = STRTRIM (bunit, 2)

;   print, 'xdim: ', xdim, '   ydim: ', ydim
;   print, 'r0: ', r0, '   dr: ', dr

;   r0       = fxpar (hduext, 'R0')
;   dr       = fxpar (hduext, 'DR')

;   rpix = (radius - r0) / dr			; Radial index.

;   print, 'radius: ', radius, '   rpix: ', rpix

   ;--- Do linear interpolation to get the intensity at the exact angle.

   apix = angle * 2.0
   ia1 = FIX (angle * 2.0)
   ia2 = ia1 + 1
   if (ia2 EQ 720)  THEN ia2 = 0
   adelta = apix - ia1
   rscan = fits_img [*, ia1] + (fits_img [*, ia2] - fits_img [*, ia1]) * adelta

;   ir1 = fix (rpix)			; pixel below the desired radius.
;   ir2 = ir1 + 1			; pixel above the desired radius.
;   rdelta = rpix - ir1			; fractional pixel above ir1.
;   tscan = fits_img [ir1, *] + (fits_img [ir2, *] - fits_img [ir1, *]) * rdelta

;   print, 'ir1: ', ir1, '   ir2: ', ir2
;   print, 'rdelta: ', rdelta

   ;--- Plot the theta scan into a postscipt file.

   ititle = fits_name + ' Radial Scan @' + sangle + ' degrees'
   xlab = 'Radius [Rsun]'
   ylab = 'Intensity'
   ylab = bunit
   ylab = 'Bsun'

;   plot, posang, fits_img [ir1, *],	$
;	 title=ititle,		$
;	 xtitle = xlab,		$
;	 ytitle = ylab,		$
;	 xrange = [0.0, 360.0]
   
   plot, radbuf, rscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [1.0, 2.2]
;	 xrange = [0.0, 360.0]
   
   device, /close

   ;--- Print out intensity values into a text file.

   IF (KEYWORD_SET (text)) THEN	$
      BEGIN
      dataname = basename + '.a' + sangle + '.txt'
      print, dataname
      CLOSE, 2
      OPENW, 2, dataname
      PRINTF, 2, 'MLSO Mark-III K-coronameter'
      PRINTF, 2, fits_name
      PRINTF, 2, 'Radial Scan   ' + sangle + ' degrees'
      PRINTF, 2, 'Solar Radius  '    + srsun   + ' arcsec'
      PRINTF, 2, ''
      PRINTF, 2, 'Position   pB'
      PRINTF, 2, 'Radius     [B/Bsun]'
      PRINTF, 2, '--------   --------

      FOR i = 0, xdim - 1 DO	$
         BEGIN
         PRINTF, 2, radbuf [i], rscan [i], FORMAT='(F8.2, 3x, E9.2)'
         END
      CLOSE, 2
      END

END	;}

CLOSE, 1

set_plot,'x'

END
