;+
; NAME	mk3_tscangif.pro
;
; PURPOSE	Read Mk3 FITS images (polar coordinates)
;		& plot theta scan.
;
; SYNTAX	mk3_tscan, fits_list, radius, /text
;
;               fits_list       file containing a list of FITS images
;               radius          radius (Rsun units) for azimuthal scan
;               text            option to output an ASCII data file.
;		wmin		minimum value for y-axis plot scale.
;		wmax		maximum value for y-axis plot scale.
;
; EXAMPLE	mk3_tscan, 'list', 1.2
;		mk3_tscan, 'list', 1.2, /text
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    28 June 2001
;		 2 Jul 2001: Add text file output.
;		16 Jul 2001: use rmin & rmax to determine r0 & dr.
;		 5 Nov 2002: Add wmin & wmax keywords.
;		 5 Nov 2002: Write plot to a GIF file.
;-

PRO mk3_tscangif, fits_list, radius, text=text, wmin=wmin, wmax=wmax

CLOSE, 1
fits_name = ''
basename  = ''
plotname  = ''
dataname  = ''

set_plot, 'Z'
LOADCT, 0				; Load greyscale color map.
fcol =   0
bcol = 255
!P.BACKGROUND = bcol

posang = findgen (720) / 2.0		; Position Angle array.
sradius = STRING (radius, FORMAT='(F4.2)')

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   plotname = basename + '.r' + sradius + '.ps'
   gifname  = basename + '.r' + sradius + '.gif'
;   print, plotname
;   device, filename = plotname

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   telescop = fxpar (hdu, 'TELESCOP')
   dataform = fxpar (hdu, 'DATAFORM')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')

   ;--- Verify that intstrument is MK3.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'MK3')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be MK3.'
      RETURN
   END

   ;--- Verify that dataform is CYLINDRICAL.

   dataform = STRTRIM (dataform, 2)
   polpos = -1
   cylpos = -1
   cylpos = STRPOS (dataform, 'CYL')
   polpos = STRPOS (dataform, 'POL')
   PRINT, 'cylpos: ', cylpos
   PRINT, 'polpos: ', polpos
   IF (cylpos EQ -1 AND polpos EQ -1)  THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.'
      PRINT, 'It should be CYLINDRICAL or POLAR.'
      PRINT, 'ONLY polar coordinate data is valid with this program.'
      RETURN
   END

   ;--- r0 = radius of first pixel height.
   ;--- dr = increment in radius (Rsun units) between adjacent pixels.
   ;
   ;--- CRPIX1 is the pixel location of sun center (r=0) [pixel units].
   ;--- CDELT1 is the radial increment [Rsun/pixel].
   ;--- Compute the r0 value from the FITS parameters crpix1 & cdelt1,
   ;--- since r0 does not appear explicitly in the FITS header.

   crpix1   = fxpar (hdu, 'CRPIX1')
   cdelt1   = fxpar (hdu, 'CDELT1')
   r0       = - (crpix1 * cdelt1)
   dr       = cdelt1

   ;--- Get r0 & dr from rmin & rmax parameters.

   rmin = fxpar (hdu, "RMIN")
   rmax = fxpar (hdu, "RMAX")

   r0 = rmin
   dr = (rmax - rmin) / 127.0

   bunit    = fxpar (hdu, 'BUNIT')
   bunit    = STRTRIM (bunit, 2)

;   print, 'xdim: ', xdim, '   ydim: ', ydim
;   print, 'r0: ', r0, '   dr: ', dr

;   r0       = fxpar (hduext, 'R0')
;   dr       = fxpar (hduext, 'DR')

   rpix = (radius - r0) / dr			; Radial index.

   print, 'radius: ', radius, '   rpix: ', rpix

   ;--- Do linear interpolation to get the intensity at the exact height.

   ir1 = fix (rpix)			; pixel below the desired radius.
   ir2 = ir1 + 1			; pixel above the desired radius.
   rdelta = rpix - ir1			; fractional pixel above ir1.
   tscan = fits_img [ir1, *] + (fits_img [ir2, *] - fits_img [ir1, *]) * rdelta

;   print, 'ir1: ', ir1, '   ir2: ', ir2
;   print, 'rdelta: ', rdelta

   ;--- Plot the theta scan into a postscipt file.

   tymin = MIN (tscan)
   tymax = MAX (tscan)
   IF (KEYWORD_SET (wmin)) THEN ymin = wmin ELSE ymin = tymin
   IF (KEYWORD_SET (wmax)) THEN ymax = wmax ELSE ymax = tymax

   ititle = fits_name + ' Theta Scan @' + sradius + ' Rsun'
   xlab = 'Position Angle [deg]'
   ylab = 'Intensity'
   ylab = bunit
   ylab = 'Bsun'

;   plot, posang, fits_img [ir1, *],	$
;	 title=ititle,		$
;	 xtitle = xlab,		$
;	 ytitle = ylab,		$
;	 xrange = [0.0, 360.0]
   
   plot, posang, tscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0],	$
	 yrange = [ymin, ymax],	$
	 color  = fcol
   
;   device, /close

   ;--- Print out intensity values into a text file.

   IF (KEYWORD_SET (text)) THEN	$
      BEGIN
      dataname = basename + '.r' + sradius + '.txt'
      print, dataname
      CLOSE, 2
      OPENW, 2, dataname
      PRINTF, 2, 'MLSO Mark-III K-coronameter'
      PRINTF, 2, fits_name
      PRINTF, 2, 'Azimuthal Scan   ' + sradius + ' Rsun'
      PRINTF, 2, 'Solar Radius  '    + srsun   + ' arcsec'
      PRINTF, 2, ''
      PRINTF, 2, 'Position   pB'
      PRINTF, 2, 'Angle      [B/Bsun]'
      PRINTF, 2, '--------   --------

      FOR i = 0, ydim - 1 DO	$
         BEGIN
         PRINTF, 2, posang [i], tscan [i], FORMAT='(F8.2, 3x, E9.2)'
         END
      CLOSE, 2
      END

   plotimg = TVRD ()
   WRITE_GIF, gifname, plotimg

END	;}

CLOSE, 1

set_plot,'X'

END
