;+
; NAME		mk3skfit.pro
;
; PURPOSE	Read Mk3 sky transmission file (text).
;		Plot max skytx vs airmass.
;		Eliminate bad points (if any are present).
;		Fit a straight line to the skytx between 1.5 & 6.0 airmass.
;		Store plot in both a GIF file.
;
; SYNTAX	mk3skfit, sk_file
;		amass keyword causes airmass vs. time to be plotted in PS file.
;
; HISTORY	Andrew L. Stanger   HAO/NCAR     18 July 2002
;		23 Jul 2002: Add standard deviation to 'txt' output.
;		25 Jul 2002: Add skmin/skmax, airmin/airmax.
;-

PRO mk3skfit, sk_file, display=display, postscript=postscript, airmass=airmass

amin = 1.5
amax = 6.0
smin = 240.0
smax = 320.0

red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)

ttab = STRARR (200)
stab = FLTARR (200)
atab = FLTARR (200 )

stab [*] = ''
ttab [*] = 0
atab [*] = 0

answer = ''
efitlab = 'Elmore: skytx = -11.0 * airmass + 376.00'

a = FINDGEN (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), sin (a), /FILL
;USERSYM, [-0.5, -0.5], [ 0.5, -0.5], [0.5, 0.5], [-0.5, 0.5], [-0.5, -0.5]

CLOSE, 11
line    = ''

;--- File list loop.

i = 0
OPENR, 11, sk_file
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 11, line				; Get line.
   print, 'line: ', line

   ;--- Extract items from text line.

   dateobs = STRMID (line, 0, 10)
   timeobs = STRMID (line, 11, 8)
   skytx   = STRMID (line, 22, 6)
   amass   = STRMID (line, 31, 6)

   fsk = FLOAT (skytx)
   fam = FLOAT (amass)

   syear    = STRMID (dateobs, 0, 4)
   smonth   = STRMID (dateobs, 5, 2)
   sday     = STRMID (dateobs, 8, 2)

   year     = FIX (syear)
   month    = FIX (smonth)
   day      = FIX (sday)
   doy      = day_of_year (year, month, day)
   sdoy     = STRING (doy, FORMAT = '(i3)')

   shour    = STRMID (timeobs, 0, 2)
   sminute  = STRMID (timeobs, 3, 2)
   ssecond  = STRMID (timeobs, 6, 2)

   hour     = FIX (shour)
   minute   = FIX (sminute)
   second   = FIX (ssecond)

   IF (i EQ 0)  THEN	$
   BEGIN
      skfile = syear + smonth + sday + '.mk3.sk.ps'
      amfile = syear + smonth + sday + '.mk3.am.ps'
      begday = day
   END

   IF (day GT begday) THEN hour = hour + 24.0
   time = hour + minute / 60.0 + second / 3600.0

   ;--- Plot sky transmission values for each image.

;   PLOT, fsk,							$
;	 title='Mk3 Sky Transmission ' + fits_name,		$
;	 charsize=0.8,						$
;	 xtitle='Angle Index', ytitle='Sky Transmission',	$
;	 xrange=[0,1000],					$
;	 yrange=[0,400],					$
;	 psym=4
;   WAIT, 1

   ;--- Save sky transmission & airmass values.

   ttab [i] = timeobs
   stab [i] = fsk
   atab [i] = fam
   i = i + 1

END	;}

CLOSE, 11

;window, xsize = 640, ysize = 480, RETAIN=2

nimg = i
print, 'nimg: ', nimg

tt = ttab [0:nimg-1]
st = stab [0:nimg-1]
am = atab [0:nimg-1]

;--- Determine how many data points fulfill criteria:
;    amin < am < amax
;    smin < st < smax

ii = 0
FOR i = 0, nimg -1 DO	$
BEGIN
   if (st [i] GE smin  AND st [i] LE smax AND $
       am [i] GE amin  AND am [i] LE amax) THEN ii = ii + 1
END
nimgok = ii

IF (nimgok LE 0) THEN			$
    BEGIN
    print, 'sky transmission and airmass values do not meet criteria'
    print, 'No plot generated.'
    goto, finish
    END

;--- Eliminate entries where airmass is outside the range 1.5 to 6.0.

st_fix = FLTARR (nimgok)
am_fix = FLTARR (nimgok)
tt_fix = STRARR (nimgok)
ii = 0

FOR i = 0, nimg-1 DO	$
BEGIN
    if (st [i] GE smin AND st [i] LE smax AND $
	am [i] GE amin AND am [i] LE amax) THEN	$
    BEGIN
	st_fix [ii] = st [i]
	am_fix [ii] = am [i]
	tt_fix [ii] = tt [i]
	ii = ii + 1
    END
END

st = st_fix
am = am_fix
nimg = nimgok

ste = FLTARR (nimg)

;--- Compute Elmore's sky transmission based upon airmass.

FOR i = 0, nimg -1 DO	$
BEGIN
    ste [i] = -11.0 * am [i] + 376.0
END

stcoef = POLY_FIT (am, st, 1, st_fit, st_err, st_sigma)

stdifsum = 0.0
skmin  = 10000.0
skmax  =    -1.0
airmin = 10000.0
airmax =    -1.0
for i = 0, nimgok-1 DO	$
    BEGIN
    stdif = st_fit [i] - st [i]
    stdif2 = stdif * stdif
    stdifsum = stdifsum + stdif2
    IF (st [i] LT skmin)  THEN skmin  = st [i]
    IF (st [i] GT skmax)  THEN skmax  = st [i]
    IF (am [i] LT airmin) THEN airmin = am [i]
    IF (am [i] GT airmax) THEN airmax = am [i]
    END
var = stdifsum / nimgok
stdev = sqrt (var)
sstdev     = STRING (stdev, FORMAT='(F9.3)')
sdevlab = 'std.dev: ' + sstdev

sskmin  = STRING (skmin,  FORMAT='(F7.1)')
sskmax  = STRING (skmax,  FORMAT='(F7.1)')
sairmin = STRING (airmin, FORMAT='(F9.3)')
sairmax = STRING (airmax, FORMAT='(F9.3)')

intercept = stcoef [0]
slope     = stcoef [1]
sintercept = STRING (intercept, FORMAT='(F9.3)')
sslope     = STRING (slope,     FORMAT='(F9.3)')
sfitlab = 'fit : skytx = ' + sslope + ' * airmass + ' + sintercept
print, 'sfitlab: ', sfitlab

;--- Set postscript device.

IF (KEYWORD_SET (postscript)) THEN			$
BEGIN ;{
    SET_PLOT, 'PS'
    DEVICE, filename = skfile
    print, 'skfile: ', skfile

    ;--- Plot airmass versus sky transmission.

    ptitle = dateobs + ' Mk3 Airmass vs. Sky Transmission' 
    PLOT, am, st,					$
    ;      charsize = 0.8,				$
          xrange = [0.0, 10.0],				$
	  yrange = [smin, smax],			$
          psym = 1,					$
          title = ptitle,				$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'

    ;--- Overplot Elmore's sky transmission function (based on airmass).

;    OPLOT, am, ste
;    XYOUTS, 0.2, 0.15, efitlab, /normal

    device, /close

    IF (KEYWORD_SET (airmass)) THEN			$
    BEGIN ;{
        SET_PLOT, 'PS'
        DEVICE, filename = amfile
        print, 'amfile: ', amfile

        ;--- Plot airmass vs. time.

        ptitle = dateobs + ' Mk3 Airmass vs. Time' 
        PLOT, tt, am, 					$
        ;      charsize = 0.8,				$
              xrange = [16.0, 24.0],			$
        ;      yrange = [0.0, 8.0],			$
              psym = 1,					$
              title = ptitle,				$
              xtitle = 'Time (hours)',			$
              ytitle = 'Airmass'
        device, /close
    END ;}
END ;}

;--- Plot airmass versus sky transmission.

IF (KEYWORD_SET (display)) THEN			$
BEGIN ;{
    SET_PLOT, 'X'
    lct, '/home/cordyn/color/bwy.lut'
    ymax = MAX (st)
    ymin = MIN (st)
    ptitle = dateobs + ' Mk3 Airmass vs. Sky Transmission' 
    PLOT, am, st,						$
          charsize = 0.75,					$
          xrange = [1.0, 6.0],				$
          yrange = [smin, smax],				$
    ;      yrange = [ymin-10, ymax+10],			$
          psym = 1,						$
          title = ptitle,					$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'
    
    ;--- Overplot Elmore's sky transmission function (based on airmass).
    
;    OPLOT, am, ste,   color=252, linestyle=1
    OPLOT, am, st_fit, color=254, linestyle=0
    
;    XYOUTS, 0.2, 0.20, efitlab, /normal, charsize=0.75, color=252
    XYOUTS, 0.2, 0.15, sfitlab, /normal, charsize=0.75, color=254
    XYOUTS, 0.2, 0.10, sdevlab, /normal, charsize=0.75, color=254
END ;}


;--- Write plot to a GIF file.

SET_PLOT, 'Z'
lct, '/home/cordyn/color/bwy.lut'
ptitle = dateobs + ' Mk3 Airmass vs. Sky Transmission' 

PLOT, am, st,						$
      background = 255, color=0,			$
;      charsize = 0.75,					$
      xrange = [1.0, 6.0],				$
      yrange = [smin, smax],				$
;      yrange = [ymin-10, ymax+10],			$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Air Mass',				$
      ytitle = 'Sky Transmission'

;--- Overplot straight line fit.

;OPLOT, am, ste,   color=252, linestyle=1
OPLOT, am, st_fit, color=254, linestyle=0

;XYOUTS, 0.2, 0.25, efitlab, /normal, charsize=1.00, color=252
XYOUTS, 0.2, 0.20, sfitlab, /normal, charsize=1.00, color=254
XYOUTS, 0.2, 0.15, sdevlab, /normal, charsize=1.00, color=254

tvlct, red, green, blue, /get
gifplot = TVRD ()
gifname = syear + smonth + sday + '.mk3.skfit.gif'
write_gif, gifname, gifplot, red, green, blue

print, 'gifname: ', gifname


;--- Plot airmass vs. time.

IF (KEYWORD_SET (airmass)) THEN				$
BEGIN ;{
    SET_PLOT, 'X'
    lct, '/home/cordyn/color/bwy.lut'
    ptitle = dateobs + ' Mk3 Airmass vs. Time' 
    PLOT, tt, am, 	$
    ;      charsize = 0.8,				$
          xrange = [16.0, 24.0],			$
    ;      yrange = [0.0, 8.0],				$
          psym = 1,					$
          title = ptitle,				$
          xtitle = 'Time (hours)',			$
          ytitle = 'Airmass'
    gifplot = TVRD ()
    gifname = syear + smonth + sday + '.mk3.airmass.gif'
    write_gif, gifname, gifplot
    print, 'gifname: ', gifname
END ;}


FOR i = 0, nimg-1 DO	$
    print, 'st: ', st [i], '  am: ', am [i]

intercept = stcoef [0]
slope     = stcoef [1]
sintercept = STRING (intercept, FORMAT='(F9.3)')
sslope     = STRING (slope,     FORMAT='(F9.3)')
fitname = syear + smonth + sday + '.mk3.skfit.txt'

print, 'sslope/sintercept: ', sslope, ' ', sintercept

OPENW,  11, fitname
PRINTF, 11, sslope, sintercept, sstdev, sskmin, sskmax, sairmin, sairmax
PRINTF, 11, 'Mk3 Sky Transmission vs. Airmass Fit'
PRINTF, 11, '-- date --            slope  intercept   stdev  skmin  skmax   airmin   airmax'
PRINTF, 11, dateobs, '  DOY ',  $
	    sdoy, sslope, sintercept, sstdev, sskmin, sskmax, sairmin, sairmax
PRINTF, 11, '   index     time        sky tx       airmass'
FOR i = 0, nimg-1 DO $
    PRINTF, 11, i, '   ', tt [i], st [i], am [i]
CLOSE,  11

finish:		$
SET_PLOT, 'X'
lct, '/home/cordyn/color/bwy.lut'

END
