;+
; NAME	fixmk4_elmore.pro
;
; PURPOSE	Read Mk4 FITS images & fix FITS parameters.
;
; SYNTAX	fixmk4_elmore, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    21 September 2001
;-

pro fixmk4_elmore, fits_list

;--- Mk4 Geometric scale factors from height mask test 28 April 2001 :

;r0 = 0.00526475
;dr = 0.938592

comment_dr = ''
comment_r0 = ''
comment_rmin = ''
comment_rmax = ''
comment_rsun = ''
comment_cdelt1 = ''
comment_cdelt2 = ''
comment_crpix1 = ''
comment_crpix2 = ''
comment_crradius = ''
out_name = ''

CLOSE, 11
fits_name = ''

;--- File list loop.

OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 11, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')
   out_name = STRMID (fits_name, 0, ftspos)
   out_name = out_name + '.fix.fts' 
   print, 'out_name:  ', out_name

   img = readfits (fits_name, hdu, /noscale)		; Read image.
   ext = readfits (fits_name, hduext, exten_no=1)	; Read extension.

   ;--- Extract information from header.

   xdim     = fxpar (hdu,    'NAXIS1')
   ydim     = fxpar (hdu,    'NAXIS2')
   crpix1   = fxpar (hdu,    'CRPIX1',   comment_crpix1)
   crpix2   = fxpar (hdu,    'CRPIX2',   comment_crpix2)
   dateobs  = fxpar (hdu,    'DATE-OBS')
   timeobs  = fxpar (hdu,    'TIME-OBS')
   crradius = fxpar (hdu,    'CRRADIUS', comment_crradius)
   dataform = fxpar (hdu,    'DATAFORM')
   rmin     = fxpar (hdu,    'RMIN',     comment_rmin)		; Rsun units
   rmax     = fxpar (hdu,    'RMAX',     comment_rmax)		; Rsun units
   rsun     = fxpar (hdu,    'RSUN',     comment_rsun)		; arcsec/Rsun
   cdelt1   = fxpar (hdu,    'CDELT1',   comment_cdelt1)	; arcsec/pixel
   cdelt2   = fxpar (hdu,    'CDELT2',   comment_cdelt2)	; arcsec/pixel

   fr0      = fxpar (hduext, 'R0',       comment_r0)	; Height [Rsun] @pixel#0
   fdr      = fxpar (hduext, 'DR',       comment_dr)	; Scale: Rsun / pixel

   r0 = fr0
   dr = fdr
;   rdr = fdr * (384.0 * 2.0) / xdim

   print, 'rsun: ', rsun

   ;--- Adjust rmin & rmax for actual solar radius size.

   rmin = r0 * 960.0 / rsun
   rmax = (383.0 * dr + r0) * 960.0 / rsun

   print, 'rmin/rmax: ', rmin, rmax

   fxaddpar, hdu,    'RMIN', rmin, comment_rmin
   fxaddpar, hdu,    'RMAX', rmax, comment_rmax

   ;--- Assume polar->cartesian conversion takes a diameter (2*384 pixels)
   ;    and expands them to fill the width of the desired image dimension
   ;    (in this case, xdim=538).
   ;
   ;    This yields a pixel scale rdr = rmax * 2.0 / xdim   [Rsun/pixel units]

   rdr    = rmax * 2.0 / xdim			; Rsun/pixel
   cdelt1 = rdr * rsun				; arcsec/pixel

   print, 'rdr: ', rdr
   print, 'cdelt1: ', cdelt1

   ;--- Compute crpix1/crpix2:

   crpix1 = (xdim - 1) * 0.5
   crpix2 = (ydim - 1) * 0.5

   fxaddpar, hdu, 'CRPIX1', crpix1, comment_crpix1
   fxaddpar, hdu, 'CRPIX2', crpix2, comment_crpix2

   ;--- Re-compute pixel scale.

;   rspix = rdr * (960.0 / rsun)		; Corrected scale [Rsun / pixel]

   rspix = rdr				; Rsun / pixel
   pixrs = 1.0 / rspix			; Corrected scale [pixel / Rsun]

   ;--- Modify cdelt1 & cdelt2.

   dataform = STRTRIM (dataform, 2)

   IF (dataform EQ 'CARTESIAN')  THEN	$
   BEGIN
      cdelt1 = rsun / pixrs		; arcsec/pixel
      cdelt2 = rsun / pixrs		; arcsec/pixel
   ENDIF

   IF (dataform EQ 'POLAR')  THEN	$
   BEGIN
      cdelt1 = 0.5			; degrees/pixel
      cdelt2 = rsun / pixrs		; arcsec/pixel
   ENDIF

   fxaddpar, hdu,    'CDELT1', cdelt1, comment_cdelt1
   fxaddpar, hdu,    'CDELT2', cdelt2, comment_cdelt2

   ;--- Compute crradius.

   crradius = rsun / cdelt1
   print, 'crradius: ', crradius

   fxaddpar, hdu,    'CRRADIUS', crradius, comment_crradius

   ;--- Update DR & R0.

;   fxaddpar, hduext, 'DR', dr, comment_dr
;   fxaddpar, hduext, 'R0', r0, comment_r0

   ;--- Write modified FITS file to disk.

   writefits, out_name, img, hdu
   writefits, out_name, ext, hduext, /APPEND

END	;}

CLOSE, 11

END
