;+
; NAME		mk4_annuli.pro
;
; PURPOSE	Plot annuli averages versus time in native Mk4 raw q images.
;		Use cylindrical coordinate data.
;
; SYNTAX	mk4_annuli, 'mk4_list'
;
; HISTORY	Andrew L. Stanger
;		 4 Mar 2002: annuli averages.
;		 8 Mar 2002: apply sky transmission correction.
;-

PRO mk4_annuli11, mk4_file_list

;--- Initialize variables.

filename = 'mk4.q'
red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)

;--- Read a header to set some variables.

CLOSE, 2
OPENR, 2, mk4_file_list 
READF, 2, filename
CLOSE, 2
filename = STRTRIM (filename, 2)

headib  = 0 & headib  = LONARR (16)
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

CLOSE, 1
OPENR, 1, filename
READU, 1, headib
READU, 1, headf
READU, 1, headie
READU, 1, headpad
CLOSE, 1

numaz    = headib  [9]				; Xdim
numpix   = headib [10]				; Ydim
pixoff   = headf  [23]				; r0
pixscale = headf  [24]				; dr

year    = headib [0]			; Image.Year
month   = headib [1]			; Image.Month
day     = headib [2]			; Image.Day

IF (year LT 1900) THEN year = year + 1900

syear   = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
smonth  = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
sday    = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)

IF (month LT 10) THEN smonth = '0' + smonth
IF (day   LT 10) THEN sday   = '0' + sday

date = syear + '-' + smonth + '-' + sday

;PRINT, 'syear: ', syear, ' smonth: ', smonth, ' sday: ', sday

plotfile = syear + smonth + sday + '.mk4.cq.plot.ps'

;PRINT, 'numaz/numpix: ', numaz, numpix
;PRINT, 'pixscale/pixoff: ', pixscale, pixoff


;--- Specify low, middle & high radius sample heights.

r1min = 1.2							; Rsun units
r1max = 1.5
r2min = 1.5
r2max = 1.8
r3min = 1.8
r3max = 2.1

PRINT, 'MIN r1/r2/r3: ', r1min, r2min, r3min
PRINT, 'MAX r1/r2/r3: ', r1max, r2max, r3max

p1min  = FIX ((r1min  - pixoff) / pixscale + 0.5)		; Pixel units.
p1max  = FIX ((r1max  - pixoff) / pixscale + 0.5) - 1
p2min  = FIX ((r2min  - pixoff) / pixscale + 0.5)
p2max  = FIX ((r2max  - pixoff) / pixscale + 0.5) - 1
p3min  = FIX ((r3min  - pixoff) / pixscale + 0.5)
p3max  = FIX ((r3max  - pixoff) / pixscale + 0.5) - 1

PRINT, 'MIN p1/p2/p3: ', p1min, p2min, p3min
PRINT, 'MAX p1/p2/p3: ', p1max, p2max, p3max

num1 = p1max - p1min + 1					; # azimuths.
num2 = p2max - p2min + 1
num3 = p3max - p3min + 1

PRINT, 'num1/num2/num3: ', num1, num2, num3

sp1min  = STRTRIM (STRING (p1min), 2)				; String.
sp1max  = STRTRIM (STRING (p1max), 2)
sp2min  = STRTRIM (STRING (p2min), 2)
sp2max  = STRTRIM (STRING (p2max), 2)
sp3min  = STRTRIM (STRING (p3min), 2)
sp3max  = STRTRIM (STRING (p3max), 2)

sr1min  = STRTRIM (STRING (r1min,  FORMAT='(F3.1)'), 2)		; String.
sr1max  = STRTRIM (STRING (r1max,  FORMAT='(F3.1)'), 2)
sr2min  = STRTRIM (STRING (r2min,  FORMAT='(F3.1)'), 2)
sr2max  = STRTRIM (STRING (r2max,  FORMAT='(F3.1)'), 2)
sr3min  = STRTRIM (STRING (r3min,  FORMAT='(F3.1)'), 2)
sr3max  = STRTRIM (STRING (r3max,  FORMAT='(F3.1)'), 2)

;--- Heights from header of first file.

height = FINDGEN (numpix) * pixscale + pixoff

;--- Determine number of images to average.

CLOSE, 2
OPENR, 2, mk4_file_list
numimg = 0

WHILE ( NOT EOF (2) ) DO BEGIN
    numimg = numimg + 1
    READF, 2, filename
ENDWHILE

;--- Define arrays.

inimg  = INTARR (NUMAZ, NUMPIX)
avgimg = FLTARR (NUMAZ, NUMPIX)

b1ave  = FLTARR (numimg)
b2ave  = FLTARR (numimg)
b3ave  = FLTARR (numimg)
hours  = FLTARR (numimg)

;--- Open list file.

CLOSE, 2
OPENR, 2, mk4_file_list

imgnum = 0

;--- File loop.

WHILE ( NOT EOF (2) ) DO BEGIN	;{
    READF, 2, filename
    filename = STRTRIM (filename, 2)

    CLOSE, 1
    OPENR, 1, filename				; Open binary image file.

    READU, 1, headib				; 32-bit integers
    READU, 1, headf				; floats
    READU, 1, headie				; 32-bit integers
    READU, 1, headpad				; char

    ;--- Extract items from binary header.

    numaz   = headib  [9]				; Xdim
    numpix  = headib [10]				; Ydim
    scale   = headf  [13]			; data_scale
    offset  = headf  [14]			; data_offset
    hour    = headib  [3]			; image hour
    minute  = headib  [4]			; image minute

    shour   = STRTRIM (STRING (hour),   2)
    sminute = STRTRIM (STRING (minute), 2)
    if (hour LT 10)   then shour   = '0' + shour
    if (minute LT 10) then sminute = '0' + sminute
    imgtime = shour + sminute
    hours [imgnum] = FLOAT (hour) + FLOAT (minute) / 60.0

    obstype   = headie [0] 	; IS_cal, IS_cal15, IS_dark, IS_coronal
    datatype  = headie [1]	; i, q, u, ba, sk, st, tscp, drift, tscp_inv
    procsteps = headie [2]	; IS_raw ... IS_cal_avg_rect_slit

    CASE datatype OF
    0:    dtype = 'i'
    1:    dtype = 'q'
    2:    dtype = 'u'
    3:    dtype = 'ba'
    4:    dtype = 'sk'
    5:    dtype = 'st'
    6:    dtype = 'tscp'
    7:    dtype = 'drift'
    8:    dtype = 'tscp_inv'
    ELSE: dtype = '?'
    ENDCASE

    CASE procsteps OF
    10:   ptype = 'raw'
    11:   ptype = 'raw_spln'
    12:   ptype = 'raw_rect'
    13:   ptype = 'cal'
    14:   ptype = 'cal_slit'
    15:   ptype = 'cal_rect'
    16:   ptype = 'cal_rect_slit'
    17:   ptype = 'cal_rect_byte'
    18:   ptype = 'cal_avg'
    19:   ptype = 'cal_avg_slit'
    20:   ptype = 'cal_avg_rect'
    21:   ptype = 'cal_avg_rect_slit'
    24:   ptype = 'raw_avg'
    25:   ptype = 'raw_avg_spln'
    26:   ptype = 'raw_avg_rect'
    ELSE: ptype = '?'
    ENDCASE

    ilab = dtype + ' ' + ptype

    skytx    = headf  [10]				; Skytx
    zerosky  = 376.0
    IF (skytx NE 0.0) THEN		$
       skycor = zerosky / skytx		$	; Sky transmission correction.	
    ELSE				$
       skycor = 1.0

    ylab = ilab
    IF (skytx NE 1.0)  THEN ylab = ylab + ' [Skytx corrected]'

;    print, 'headf: ', headf
;    print, 'skytx: ', skytx, '   skycor: ', skycor

    ;--- Read FITS image.

    inimg = INTARR (numaz, numpix)
    READU, 1, inimg				; Read image array.
    CLOSE, 1

    ;--- Apply photometric scaling.
    ;--- Accumulate image sum.

    IF (procsteps GE 13 AND procsteps LE 21) THEN	$
        img = (FLOAT (inimg) - offset) / scale 		$
    ELSE						$
	img = inimg * skycor			; Apply skytx correction.

;    print, 'size (img): ', size (img)
;    print, img [*, p1min]
;    plot, img [*, p1min]


    ;--- Compute azimuthal averages at low, middle & high radius positions.

    b1ave [imgnum] = TOTAL (img [*, p1min:p1max]) / (numaz * num1)
    b2ave [imgnum] = TOTAL (img [*, p2min:p2max]) / (numaz * num2)
    b3ave [imgnum] = TOTAL (img [*, p3min:p3max]) / (numaz * num3)

    PRINT, filename, ' ave b1/b2/b3: ', $
	   b1ave [imgnum], b2ave [imgnum], b3ave [imgnum]

    imgnum = imgnum + 1

ENDWHILE	;}

;--- Write plots to GIF files.

SET_PLOT, 'Z'				; Use Z-buffer for plots.

;PRINT, !version.os
;IF (!version.os EQ 'IRIX') THEN cs = 0.8 ELSE cs = 1.6
cs = 1.0

LOADCT, 0				; Load greyscale color map
lct, '/home/stanger/color/bwcp.lut'
tvlct, red, green, blue, /get
fcol =   0
bcol = 255
!P.BACKGROUND=bcol

;PRINT, "Set plot device"
;SET_PLOT, 'PS'
;DEVICE, filename = plotfile

;WINDOW, xsize=640, ysize=480

;--- Plot annulus average vs. time at 3 different heights.

xmin = MIN (hours)
xmax = MAX (hours)
;PRINT, 'xmin/xmax: ', xmin, xmax

ymin = MIN (b1ave)
ymax = MAX (b1ave)

b1aves = SMOOTH (b1ave, 11)

ptitle = 'Average ' + ilab + ' @' + sr1min + '-' + sr1max + $
	 ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, b1ave,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
      charsize=cs, ticklen=1, color=fcol, 			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

OPLOT, hours, b1aves, color=254

XYOUTS, 0.3, 0.15, '11-point smoothing', color=254, /NORMAL
plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.a' + sr1min + '.s11.gif'
write_gif, gifname, plotimg, red, green, blue

;STOP

ymin = MIN (b2ave)
ymax = MAX (b2ave)
;PRINT, 'ymin/ymax: ', ymin, ymax

b2aves = SMOOTH (b2ave, 11)

ptitle = 'Average ' + ilab + ' @' + sr2min + '-' + sr2max + $
	 ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, b2ave,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
      charsize=cs, ticklen=1, color=fcol,			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

OPLOT, hours, b2aves, color=254

XYOUTS, 0.3, 0.15, '11-point smoothing', color=254, /NORMAL
plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.a' + sr2min + '.s11.gif'
write_gif, gifname, plotimg, red, green, blue

ymin = MIN (b3ave)
ymax = MAX (b3ave)
;PRINT, 'ymin/ymax: ', ymin, ymax

b3aves = SMOOTH (b3ave, 11)

ptitle = 'Average ' + ilab + ' @' + sr3min + '-' + sr3max + $
	 ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, b3ave,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
      charsize=cs, ticklen=1, color=fcol,			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

OPLOT, hours, b3aves, color=254

XYOUTS, 0.3, 0.15, '11-point smoothing', color=254, /NORMAL
plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.a' + sr3min + '.s11.gif'
write_gif, gifname, plotimg, red, green, blue

!P.BACKGROUND=0
;DEVICE, /close

SET_PLOT, 'X'

END

