;+
; NAME	mk4_cpbrot.pro
;
; PURPOSE	Read Mk4 FITS polar coordinate images & "rotate" image.
;
; SYNTAX	mk4_cpbrot, fits_list, rotdeg
;		rotdeg		rotation in degrees CCW.
;
;		fits_list	file containing a list of FITS images
;
; EXAMPLE	mk4_cpbrot, 'list'
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   26 Aug 2003
;-

PRO mk4_cpbrot, fits_list, rotdeg

CLOSE, 1
fits_name = ''
basename  = '' 
plotname  = ''
dataname  = ''
xdim = 720
ydim = 384
WINDOW, xsize=xdim, ysize=ydim*2, retain=2

posang = FINDGEN (720) / 2.0			; Position Angle array.

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN  ;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   PRINT, 'fits_name: ', fits_name

   img = readfits (fits_name, hdu, /noscale)
   ext = readfits (fits_name, hduext, /noscale, exten_no=1)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   telescop = fxpar (hdu, 'TELESCOP')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')

   ;--- Verify that intstrument is MK4.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'MK4')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be MK4.'
      RETURN
   END

   ;--- Verify that dataform is POLAR.

   dataform = STRTRIM (dataform, 2)
   IF (dataform NE 'POLAR') THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.  It should be POLAR.'
      PRINT, 'ONLY polar coordinate data is valid with this program.'
      RETURN
   END

   ;--- Rotate image CCW 9.5 degrees.

   nrot = FIX ((rotdeg / 0.5) + 0.5)	; Rotation CCW in x-axis pixels.
   imgrot = INTARR (xdim, ydim) 

   FOR i = 0, xdim - 1 DO	$
   BEGIN
      irot = i + nrot
      IF (irot GE xdim) THEN irot = irot - xdim
      IF (irot LT    0) THEN irot = irot + xdim
      imgrot [irot, *] = img [i, *]
   END

   ;--- Create file name for output file.

   ftspos   = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   rotname = basename + '.rot.fts'
   print, 'rotname: ', rotname

   ;--- Write rotated image to disk.

   hdurot = hdu
   protdeg = abs (rotdeg)
   srotdeg = STRTRIM (STRING (protdeg, FORMAT='(F5.1)'), 2)
   IF (rotdeg LT 0) THEN sdir = 'CW'  ELSE sdir = 'CCW'
   rolladjcom = ' IMAGE ROTATED ' + srotdeg + ' DEGREES ' + sdir
   PRINT, 'rolladjcom: ', rolladjcom
   fxaddpar, hdurot, 'ROLLADJ', rotdeg, rolladjcom 
   writefits, rotname, imgrot, hdurot
   writefits, rotname, ext, hduext, /APPEND

   ;--- Display original image & rotated image in a window.

   imin = MIN (img,    max=imax)
   rmin = MIN (imgrot, max=rmax)
   PRINT, 'imin/imax: ', imin, imax
   PRINT, 'rmin/rmax: ', rmin, rmax

   TV, BYTSCL (img,    min=500, max=6000, top=249), 0, 0
   TV, BYTSCL (imgrot, min=500, max=6000, top=249), 0, ydim 

   XYOUTS, xdim/2 - 120, ydim   - 20, fits_name, /device, charsize=2.0
   XYOUTS, xdim/2 - 120, ydim*2 - 20, rotname,   /device, charsize=2.0

END    ;}

CLOSE, 1

END
