;+
; NAME	mk4_rpbrot.pro
;
; PURPOSE	Read Mk4 FITS Cartesian coordinate images & "rotate" image.
;
; SYNTAX	mk4_rpbrot, fits_list, rotdeg
;		rotdeg		rotation in degrees CCW.
;
;		fits_list	file containing a list of FITS images
;
; EXAMPLE	mk4_rpbrot, 'list'
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    1 Dec 2004
;-

PRO mk4_rpbrot, fits_list, rotdeg

CLOSE, 1
fits_name = ''
basename  = '' 
plotname  = ''
dataname  = ''
xdim = 960
ydim = 960

cwidth  = 8
cheight = FIX ((cwidth * 1.5) + 0.5)

WINDOW, xsize=xdim, ysize=ydim, retain=2
DEVICE, SET_CHARACTER_SIZE = [cwidth, cheight]          ; Set character size.

;posang = FINDGEN (720) / 2.0			; Position Angle array.

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN  ;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   PRINT, 'fits_name: ', fits_name

   img = readfits (fits_name, hdu, /noscale)
   ext = readfits (fits_name, hduext, /noscale, exten_no=1)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   xcen     = fxpar (hdu, 'CRPIX1')
   ycen     = fxpar (hdu, 'CRPIX2')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   vignet   = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   telescop = fxpar (hdu, 'TELESCOP')
   rsun     = fxpar (hdu, 'RSUN')

   vignet   = STRTRIM (vignet, 2)
   vignet   = STRUPCASE (vignet)
   IF (vignet EQ 'N') THEN	$
   BEGIN
      dmin = -500
      dmax = 5000
   END				$
   ELSE				$
   BEGIN
      dmin = 0
      dmax = 600
   END

   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')

   ;--- Verify that intstrument is MK4.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'MK4')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be MK4.'
      RETURN
   END

   ;--- Verify that dataform is POLAR.

   dataform = STRTRIM (dataform, 2)
   IF (dataform NE 'CARTESIAN') THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.  It should be CARTESIAN.'
      PRINT, 'ONLY Cartesian coordinate data is valid with this program.'
      RETURN
   END

   ;--- Rotate image CW (for positive roll values).
   ;--- For Mk4, the corona needs 9.5 degrees CW rotation prior to 28 Apr 2001.

   imgrot = rot (img, rotdeg, 1.0, xcen, ycen)

   ;--- Create file name for output file.

   ftspos   = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   rotname = basename + '.rot.fts'
   print, 'rotname: ', rotname

   ;--- Write rotated image to disk.

   hdurot = hdu
   protdeg = abs (rotdeg)
   srotdeg = STRTRIM (STRING (protdeg, FORMAT='(F5.1)'), 2)
   IF (rotdeg GE 0) THEN sdir = 'CW'  ELSE sdir = 'CCW'
   rolladjcom = ' Image rotated ' + srotdeg + ' degrees ' + sdir
   PRINT, 'rolladjcom: ', rolladjcom
   fxaddpar, hdurot, 'ROLLADJ', rotdeg, rolladjcom 
   writefits, rotname, imgrot, hdurot
   writefits, rotname, ext, hduext, /APPEND

   ;--- Display original image & rotated image in a window.

   imin = MIN (img,    max=imax)
   rmin = MIN (imgrot, max=rmax)
   PRINT, 'imin/imax: ', imin, imax
   PRINT, 'rmin/rmax: ', rmin, rmax

   TV, BYTSCL (img,    min=dmin, max=dmax, top=249), 0, 0
   TV, BYTSCL (imgrot, min=dmin, max=dmax, top=249), 0, ydim 

   XYOUTS, xdim/2 - 120, ydim   - 20, fits_name, /device, charsize=1.2
   XYOUTS, xdim/2 - 120, ydim*2 - 20, rotname,   /device, charsize=1.2

END    ;}

CLOSE, 1

END
