;+
; NAME	mk4_tscan.pro
;
; PURPOSE	Read Mk4 FITS polar coordinate images & plot theta scan.
;
; SYNTAX	mk4_tscan, fits_list, radius, /text
;
;		fits_list	file containing a list of FITS images
;		radius		radius (Rsun units) for azimuthal scan
;		text		option to output an ASCII data file.
;		wmin		minimum value for y-axis lot scale.
;		wmax		maximum value for y-axis lot scale.
;
; EXAMPLE	mk4_tscan, 'list', 1.2
;		mk4_tscan, 'list', 1.2, /text
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    6 June 2001
;		27 Aug 2001: Add linear interpolation.
;		28 Aug 2001: Add text data file option.
;               31 Jan 2002: Text loop limit changed from ydim to xdim.
;		 7 Nov 2002: Add wmin & wmax keywords.
;-

PRO mk4_tscan, fits_list, radius, text=text, wmin=wmin, wmax=wmax

CLOSE, 1
fits_name = ''
basename  = '' 
plotname  = ''
dataname  = ''
SET_PLOT, 'PS'

posang = FINDGEN (720) / 2.0			; Position Angle array.
sradius = STRING (radius, FORMAT='(F4.2)')

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN  ;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   PRINT, 'fits_name: ', fits_name

   fits_img = readfits (fits_name, hdu)
   ext = readfits (fits_name, hduext, exten_no=1)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   telescop = fxpar (hdu, 'TELESCOP')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')

   ;--- Verify that intstrument is MK4.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'Mk4')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be Mk4.'
      RETURN
   END

   ;--- Verify that dataform is POLAR.

   dataform = STRTRIM (dataform, 2)
   IF (dataform NE 'Polar') THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.  It should be Polar.'
      PRINT, 'ONLY polar coordinate data is valid with this program.'
      RETURN
   END

   ;--- r0 = radius of first pixel height.
   ;--- dr = increment in radius (Rsun units) between adjacent pixels.

   r0   = fxpar (hduext, 'R0')
   dr   = fxpar (hduext, 'DR')
   rpix = (radius - r0) / dr			; Radial index.

   ;--- Do linear interpolation to get the intensity at the exact height.

   ir1 = fix (rpix)                     ; pixel below the desired radius.
   ir2 = ir1 + 1                        ; pixel above the desired radius.
   rdelta = rpix - ir1                  ; fractional pixel above ir1.
   tscan = fits_img [*, ir1] + (fits_img [*, ir2] - fits_img [*, ir1]) * rdelta

   PRINT, 'radius: ', radius, ' Rsun   rpix: ', rpix, ' pixels'

   ;--- Create file name for postscript plot.

   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   plotname = basename + '.r' + sradius + '.ps'
   print, plotname

   DEVICE, filename = plotname

   ;--- Plot the azimuthal scan and store it in a postscript file.

   tymin = MIN (tscan)
   tymax = MAX (tscan)
   IF (KEYWORD_SET (wmin))  THEN ymin = wmin  ELSE ymin = tymin
   IF (KEYWORD_SET (wmax))  THEN ymax = wmax  ELSE ymax = tymax

   ititle = fits_name + ' Theta Scan @' + sradius + ' Rsun'
   xlab = 'Position Angle [deg]'
   ylab = 'Intensity'
   ylab = type_obs + ' [' + bunit + ']'

   PLOT, posang, tscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0],	$
	 yrange = [ymin, ymax]
   
   DEVICE, /close

   ;--- Print out intensity values into a text file.

   IF (KEYWORD_SET (text)) THEN	$
   BEGIN  ;{
      dataname = basename + '.r' + sradius + '.txt'
      print, dataname
      CLOSE, 2
      OPENW, 2, dataname
      PRINTF, 2, 'MLSO Mark-IV K-coronameter'
      PRINTF, 2, fits_name
      PRINTF, 2, 'Azimuthal Scan   ' + sradius + ' Rsun'
      PRINTF, 2, 'Solar Radius  '    + srsun   + ' arcsec'
      PRINTF, 2, ''
      PRINTF, 2, 'Position   pB'
      PRINTF, 2, 'Angle      [B/Bsun]'
      PRINTF, 2, '--------   --------

      FOR i = 0, xdim - 1 DO	$
      BEGIN  ;{
         PRINTF, 2, posang [i], tscan [i], FORMAT='(F8.2, 3x, E9.2)'
      END    ;}

      CLOSE, 2
   END    ;}

END    ;}

CLOSE, 1

SET_PLOT,'x'

END
