;+
; NAME	mk4_tscanp.pro
;
; PURPOSE	Read Mk4 FITS polar coordinate images & plot theta scan.
;		Scan height is plotted in red.
;		Solar surface is plotted in green.
;
; SYNTAX	mk4_tscanp, fits_list, pixel_index, /text
;
;		fits_list	file containing a list of FITS images
;		pixel_index	radius (pixel index) for azimuthal scan
;				NOTE: pixel_index is an INTEGER !
;		text		option to output an ASCII data file.
;		ymin		Y-axis minimum value
;		ymax		Y-axis maximum value
;		gif		if set, a GIF image file is written.
;
; EXAMPLE	mk4_tscanp, 'list', 40
;		mk4_tscanp, 'list', 42, /text
;		mk4_tscanp, 'list', 44, /text, ymin=1500, ymax=3000
;		mk4_tscanp, 'list', 44, /text, ymin=1500, ymax=3000, /gif
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    6 June 2001
;		27 Aug 2001: Add linear interpolation.
;		28 Aug 2001: Add text data file option.
;		 4 Sep 2001: Radius is in pixel index units.
;		 5 Sep 2001: Display image & plot theta scan line.
;		31 Jan 2002: Text loop limit changed from ydim to xdim.
;-

PRO mk4_tscanp, fits_list, rpix, text=text, ymin=ymin, ymax=ymax, gif=gif

CLOSE, 11
ybor      = 32
bar       = BYTARR (720, ybor)
bar [*]   = 0
fits_name = ''
basename  = '' 
plotname  = ''
dataname  = ''
r = bindgen (256)
g = bindgen (256)
b = bindgen (256)

posang = FINDGEN (720) / 2.0			; Position Angle array.
srpix = STRTRIM (STRING (rpix, FORMAT='(I4)'), 2)

;--- File list loop.

ii = -1
OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN  ;{

   ii = ii + 1

   ;--- Read FITS image header & pixel data.

   READF, 11, fits_name				; Get file name.
   PRINT, 'fits_name: ', fits_name

   fits_img = readfits (fits_name, hdu)
   ext      = readfits (fits_name, hduext, exten_no=1)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   telescop = fxpar (hdu, 'TELESCOP')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')
   dispmin  = fxpar (hdu, 'DISPMIN')
   dispmax  = fxpar (hdu, 'DISPMAX')

   ;--- Verify that intstrument is MK4.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'MK4')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be MK4.'
      RETURN
   END

   ;--- Verify that dataform is POLAR.

   dataform = STRTRIM (dataform, 2)
   IF (dataform NE 'POLAR') THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.  It should be POLAR.'
      PRINT, 'ONLY polar coordinate data is valid with this program.'
      RETURN
   END

   ;--- r0 = radius of first pixel height.
   ;--- dr = increment in radius (Rsun units) between adjacent pixels.

   r0   = fxpar (hduext, 'R0')
   dr   = fxpar (hduext, 'DR')

;   rpix = (radius - r0) / dr		; Radial index.

   radius = (rpix * dr) + r0		; Solar radius (Rsun units).
   sradius = STRTRIM (STRING (radius, FORMAT='(F5.2)'), 2)

   ;--- Do linear interpolation to get the intensity at the exact height.

   ir0 = FIX (((1.0 - r0) / dr) + 0.5)	; pixel corresponding to 1.0 Rsun.
   ir1 = fix (rpix)                     ; pixel below the desired radius.

;   ir2 = ir1 + 1                        ; pixel above the desired radius.
;   rdelta = rpix - ir1                  ; fractional pixel above ir1.
;   tscan = fits_img [*, ir1] + (fits_img [*, ir2] - fits_img [*, ir1]) * rdelta

   tscan = fits_img [*, ir1]

   PRINT, 'radius: ', radius, ' Rsun   rpix: ', rpix, ' pixels'
   PRINT, 'rsun:   ', rsun, ' arcsec'

   ;--- Create file name for postscript plot.

   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   plotname = basename + '.rp' + srpix + '.ps'
   PRINT, 'plot name: ', plotname

   SET_PLOT, 'PS'
   DEVICE, filename = plotname

   ;--- Plot the azimuthal scan and store it in a postscript file.

   ititle = fits_name + ' Theta Scan' + ' pixel#' + srpix + $
	    '=' + sradius + 'Rsun'
   xlab = 'Position Angle [deg]'
   ylab = 'Intensity'
   ylab = type_obs + ' [' + bunit + ']'

   yminset = 0
   ymaxset = 0
   IF (KEYWORD_SET (ymin)) THEN yminset = 1  ELSE ymin = 0.0
   IF (KEYWORD_SET (ymax)) THEN ymaxset = 1  ELSE ymax = 0.0

   PRINT, 'yminset: ', yminset, '   ymin: ', ymin
   PRINT, 'ymaxset: ', ymaxset, '   ymax: ', ymax

   IF (yminset OR ymaxset) THEN                    $
   BEGIN
      PLOT, posang, tscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0],	$
	 yrange = [ymin, ymax]
   
      DEVICE, /close
   END				$
   ELSE				$
   BEGIN
      PLOT, posang, tscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0]
   
      DEVICE, /close
   END

   ;--- Print out intensity values into a text file.

   IF (KEYWORD_SET (text)) THEN	$
   BEGIN  ;{
      dataname = basename + '.rp' + srpix + '.txt'
      PRINT, dataname
      CLOSE, 2
      OPENW, 2, dataname
      PRINTF, 2, 'MLSO Mark-IV K-coronameter'
      PRINTF, 2, fits_name
      PRINTF, 2, 'Azimuthal Scan   ' + ' pixel #', + srpix + $
		 ' = ' + sradius + ' Rsun'
      PRINTF, 2, 'Solar Radius  '    + srsun   + ' arcsec'
      PRINTF, 2, ''
      PRINTF, 2, 'Position   ', type_obs 
      PRINTF, 2, 'Angle      ', bunit
      PRINTF, 2, '--------   --------

      FOR i = 0, xdim - 1 DO	$
      BEGIN  ;{
         PRINTF, 2, posang [i], tscan [i], FORMAT='(F8.2, 3x, E9.2)'
      END    ;}

      CLOSE, 2
   END    ;}

   ;--- Display image & draw line at specified pixel height.

   SET_PLOT, 'X'
   IF (ii EQ 0)  THEN WINDOW, xs=xdim, ys=ydim + ybor, retain=2

   lct, '/home/cordyn/color/bwy.lut'
   TVLCT, r, g, b, /get

   TV, bar, 0, ydim-1
   TV, BYTSCL (fits_img, min=dispmin, max=dispmax, top=249)

   PLOTS, [0, xdim-1], [ir1, ir1], /device, color=254
   PLOTS, [0, xdim-1], [ir0, ir0], /device, color=253
   XYOUTS, 2, ydim+8, fits_name + ' Theta Scan at pixel # ' + srpix + $
	   ' = ' + sradius + ' Rsun', /device, color=251

   IF (KEYWORD_SET (gif))  THEN		$
   BEGIN
      gifname = basename + '.gif'
      PRINT, 'gifname: ', gifname
      img = TVRD ()
      WRITE_GIF, gifname, img, r, g, b
   END

END    ;}

CLOSE, 11
SET_PLOT, 'X'

END
