;+
; NAME	mk4_tscanqs.pro
;
; PURPOSE	Read Mk4 FITS polar coordinate images & plot theta scan.
;
; SYNTAX	mk4_tscanps, fits_list, pixel_index, /text, ymin=ymin, ymax=ymax
;
;		fits_list	file containing a list of FITS images
;		pixel_index	radius (pixel index) for azimuthal scan
;				NOTE: pixel_index is an INTEGER !
;		text		option to output an ASCII data file.
;		ymin		Y-axis minimum value
;		ymax		Y-axis maximum value
;
; EXAMPLE	mk4_tscanps, 'list', 40
;		mk4_tscanps, 'list', 42, /text
;		mk4_tscanps, 'list', 44, /text, ymin=1500, ymax=3000
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    6 June 2001
;		27 Aug 2001: Add linear interpolation.
;		28 Aug 2001: Add text data file option.
;		 4 Sep 2001: Radius is in pixel index units.
;               31 Jan 2002: Text loop limit changed from ydim to xdim.
;-

PRO mk4_tscanqs, fits_list, rpix, text=text, ymin=ymin, ymax=ymax

CLOSE, 1
fits_name = ''
basename  = '' 
plotname  = ''
dataname  = ''
SET_PLOT, 'PS'

;posang = FINDGEN (720) / 2.0			; Position Angle array.
srpix = STRTRIM (STRING (rpix, FORMAT='(I4)'), 2)

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN  ;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   PRINT, 'fits_name: ', fits_name

   fits_img = readfits (fits_name, hdu)
   ext = readfits (fits_name, hduext, exten_no=1)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   telescop = fxpar (hdu, 'TELESCOP')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')

   ;--- Verify that intstrument is MK4.

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'MK4')  THEN		$
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' is incorrect.  It should be MK4.'
      RETURN
   END

   ;--- Verify that dataform is POLAR.

   dataform = STRTRIM (dataform, 2)
   IF (dataform NE 'POLAR') THEN	$
   BEGIN
      PRINT, 'DATAFORM: ', dataform, ' is incorrect.  It should be POLAR.'
      PRINT, 'ONLY polar coordinate data is valid with this program.'
      RETURN
   END

   IF (STRMID (type_obs, 0, 1) EQ 'p') THEN	$
   BEGIN
      ptype = 1
      posang = FINDGEN (xdim) / 2.0		; Position Angle array.
   END						$
   ELSE						$
   BEGIN
      ptype = 0
      posang = FINDGEN (xdim)			; azimuthal index.
   END

   ;--- r0 = radius of first pixel height.
   ;--- dr = increment in radius (Rsun units) between adjacent pixels.

   r0   = fxpar (hduext, 'R0')
   dr   = fxpar (hduext, 'DR')

;   rpix = (radius - r0) / dr			; Radial index.

   radius = (rpix * dr) + r0			; Solar radius (Rsun units).
   sradius = STRTRIM (STRING (radius, FORMAT='(F5.2)'), 2)

   ;--- Do linear interpolation to get the intensity at the exact height.

   ir1 = fix (rpix)                     ; pixel below the desired radius.

;   ir2 = ir1 + 1                        ; pixel above the desired radius.
;   rdelta = rpix - ir1                  ; fractional pixel above ir1.
;   tscan = fits_img [*, ir1] + (fits_img [*, ir2] - fits_img [*, ir1]) * rdelta

   tscan = fits_img [*, ir1]

   PRINT, 'radius: ', radius, ' Rsun   rpix: ', rpix, ' pixels'

   ;--- Create file name for postscript plot.

   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   plotname = basename + '.rp' + srpix + '.ps'
   print, plotname

   DEVICE, filename = plotname

   ;--- Plot the azimuthal scan and store it in a postscript file.

   ititle = fits_name + ' Theta Scan' + ' pixel #' + srpix
   IF (ptype EQ 1) THEN xlab = 'Position Angle [deg]'	$
   ELSE                 xlab = 'Position Angle [index]'
   ylab = 'Intensity'
   ylab = type_obs + ' [' + bunit + ']'

   yminset = 0
   ymaxset = 0
   IF (KEYWORD_SET (ymin)) THEN yminset = 1  ELSE ymin = 0.0
   IF (KEYWORD_SET (ymax)) THEN ymaxset = 1  ELSE ymax = 0.0

   print, 'ymin/ymax: ', ymin, ymax

   IF (yminset OR ymaxset) THEN                    $
   BEGIN
      PLOT, posang, tscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0],	$
	 yrange = [ymin, ymax]
   
      DEVICE, /close
   END				$
   ELSE				$
   BEGIN
      PLOT, posang, tscan,	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0]
   
      DEVICE, /close
   END

   ;--- Print out intensity values into a text file.

   IF (KEYWORD_SET (text)) THEN	$
   BEGIN  ;{
      dataname = basename + '.rp' + srpix + '.txt'
      print, dataname
      CLOSE, 2
      OPENW, 2, dataname
      PRINTF, 2, 'MLSO Mark-IV K-coronameter'
      PRINTF, 2, fits_name
      PRINTF, 2, 'Azimuthal Scan   ' + ' pixel #', + srpix + $
		 ' = ' + sradius + ' Rsun'
      PRINTF, 2, 'Solar Radius  '    + srsun   + ' arcsec'
      PRINTF, 2, ''
      PRINTF, 2, 'Position   ', type_obs 
      PRINTF, 2, 'Angle      ', bunit
      PRINTF, 2, '--------   --------

      FOR i = 0, xdim - 1 DO	$
      BEGIN  ;{
         PRINTF, 2, posang [i], tscan [i], FORMAT='(F8.2, 3x, E9.2)'
      END    ;}

      CLOSE, 2
   END    ;}

END    ;}

CLOSE, 1

SET_PLOT,'x'

END
