;+
; NAME		mk4airmass.pro
;
; PURPOSE	Read Mk4 FITS images & plot time vs. airmass.
;
; SYNTAX	mk4airmass, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR     1 July 2002
;-

PRO mk4airmass, fits_list

stab = FLTARR (120)
atab = FLTARR (120)
ttab = FLTARR (120)

CLOSE, 11
fits_name = ''

;--- File list loop.

i = 0
OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 11, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')

   img = readfits (fits_name, hdu, /noscale)		; Read image.
   ext = readfits (fits_name, hduext, exten_no=1)	; Read extension.

   ;--- Extract information from header.

   dateobs  = fxpar (hdu,    'DATE-OBS')
   timeobs  = fxpar (hdu,    'TIME-OBS')
   dataform = fxpar (hdu,    'DATAFORM')
   dataform = STRTRIM (dataform, 2)

   syear    = STRMID (dateobs, 0, 4)
   smonth   = STRMID (dateobs, 5, 2)
   sday     = STRMID (dateobs, 8, 2)

   year     = FIX (syear)
   month    = FIX (smonth)
   day      = FIX (sday)

   shour    = STRMID (timeobs, 0, 2)
   sminute  = STRMID (timeobs, 3, 2)
   ssecond  = STRMID (timeobs, 6, 2)

   hour     = FIX (shour)
   minute   = FIX (sminute)
   second   = FIX (ssecond)

   IF (i EQ 0)  THEN	$
   BEGIN
      plotfile = syear + smonth + sday + '.mk4.airmass.ps'
      begday = day
   END

   IF (day GT begday) THEN hour = hour + 24.0
   time = hour + minute / 60.0 + second / 3600.0

   skytx    = fxpar (hduext, 'SKYTX')
   amass    = fxpar (hduext, 'AMASS')

   print, 'skytx/amass: ', skytx, amass

   ;--- Save sky transmission & airmass values.

   stab [i] = skytx
   atab [i] = amass
   ttab [i] = time
   i = i + 1

END	;}

CLOSE, 11

;--- Set postscript device.

;window, xsize = 640, ysize = 480, RETAIN=2

set_plot, 'PS'
device, filename = plotfile
print, 'plotfile: ', plotfile

nimg = i
st = stab [0:nimg-1]
am = atab [0:nimg-1]
tt = ttab [0:nimg-1]

ste = FLTARR (nimg)

;--- Compute Elmore's sky transmission based upon airmass.

FOR i = 0, nimg -1 DO	$
BEGIN
    ste [i] = -11.0 * am [i] + 376.0
END

FOR i = 0, nimg - 1 DO	$
    print, 'time: ', tt [i], ' airmass: ', am [i]

;--- Plot airmass vs. time.

ptitle = dateobs + ' Mk4 Airmass vs. Time' 
PLOT, tt, am, 	$
      xrange = [16.0, 24.0],				$
;      yrange = [0.0, 8.0],				$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Time (hours)',					$
      ytitle = 'Airmass'

;--- Plot sky transmission vs. time.

ptitle = dateobs + ' Mk4 Sky Transmission vs. Time' 
PLOT, tt, st, 	$
      xrange = [16.0, 24.0],				$
      yrange = [300.0, 400.0],				$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Time (hours)',				$
      ytitle = 'Sky Transmission'

;--- Plot airmass versus sky transmission.

;ptitle = dateobs + ' Mk4 Airmass vs. Sky Transmission' 
;PLOT, am, st,	$
;      xrange = [1.0, 6.0],				$
;      yrange = [280.0, 380.0],				$
;      psym = 1,						$
;      title = ptitle,					$
;      xtitle = 'Air Mass',				$
;      ytitle = 'Sky Transmission'

;--- Overplot Sky transmission function (based on airmass).

;OPLOT, am, ste

;XYOUTS, 0.2, 0.15, 'line: y = -11 * airmass + 376', /normal

device, /close
set_plot, 'X'

END
