;+
; NAME		mk4qas.pro
;
; PURPOSE	Generate Mk4 azimuthal scan plots [raw q]
;		to monitor image quality.
;
; SYNTAX	mk4qas, file_list
;		file_list	file containing a list of native raw q files.
;		ymin		overrides default Y-axis minimum.
;		ymax		overrides default Y-axis maximum.
;
; AUTHOR 	Andrew L. Stanger
;		1 Aug 2002: Add ymin & ymax parameters.
;		30 Oct 2002: Modify plot title (remove "x.x Rsun").
;			     Renamed from "mk4plotq.pro" to mk4qas.pro".
;-

PRO mk4qas, file_list, ymin=ymin, ymax=ymax

;--- Write plots to GIF files.

SET_PLOT, 'Z'				; Use Z-buffer for plots.

;--- Initialize variables.

LOADCT, 0				; Load greyscale color map
lct, '/home/cordyn/color/bwy.lut'	; Load blue-white-yellow color map.
r = BYTARR (256)
g = BYTARR (256)
b = BYTARR (256)
TVLCT, r, g, b, /get
fcol   =   0
bcol   = 255
!P.BACKGROUND=bcol

white  = 255
red    = 254
green  = 253
blue   = 252
grey   = 251
yellow = 250
black  =   0

filename = 'mk4.img'
PRINT, !version.os
IF (!version.os EQ 'IRIX') THEN cs = 0.8 ELSE cs = 1.6
cs = 1.0

headib  = 0 & headib  = LONARR (16)
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

;--- Specify low, middle & high radius sample heights.

radlo  = 1.2			; Rsun units
radmid = 1.5
radhi  = 2.0

;--- Read first file header.

CLOSE, 10
OPENR, 10, file_list
READF, 10, filename
filename = STRTRIM (filename, 2)

CLOSE, 11
OPENR, 11, filename
READU, 11, headib
READU, 11, headf
READU, 11, headie
READU, 11, headpad
CLOSE, 11

CLOSE, 10

;PRINT, headib
;PRINT, headf
;PRINT, headie

numaz    = headib  [9]				; Xdim
numpix   = headib [10]				; Ydim

pixoff   = headf  [23]				; r0
pixscale = headf  [24]				; dr

;--- Convert radial heights from Rsun to pixel units.

pixlo  = FIX ((radlo  - pixoff) / pixscale + 0.5)
pixmid = FIX ((radmid - pixoff) / pixscale + 0.5)
pixhi  = FIX ((radhi  - pixoff) / pixscale + 0.5)

spixlo  = STRTRIM (STRING (pixlo), 2)
spixmid = STRTRIM (STRING (pixmid), 2)
spixhi  = STRTRIM (STRING (pixhi), 2)
sradlo  = STRTRIM (STRING (radlo,  FORMAT='(F3.1)'), 2)
sradmid = STRTRIM (STRING (radmid, FORMAT='(F3.1)'), 2)
sradhi  = STRTRIM (STRING (radhi,  FORMAT='(F3.1)'), 2)

;PRINT, 'pixlo/pixmid/pixhi: ', pixlo, pixmid, pixhi

;--- Heights from header of first file.

height = FINDGEN (numpix) * pixscale + pixoff

;--- Open list file.

CLOSE, 10 
OPENR, 10, file_list

;WINDOW, 0, xs=numaz, ys=numpix
;WINDOW, 1, xs=numaz, ys=numpix

;--- File loop.

WHILE ( NOT EOF (10) ) DO BEGIN	;{
    READF, 10, filename
    filename = STRTRIM (filename, 2)
;    PRINT, filename

    CLOSE, 11
    OPENR, 11, filename				; Open binary image file.

    ;--- Read binary header.

    READU, 11, headib				; 32-bit integers
    READU, 11, headf				; floats
    READU, 11, headie				; 32-bit integers
    READU, 11, headpad				; char

    ;--- Extract items from binary header.

    year    = headib  [0]			; TimeDate.Year
    month   = headib  [1]			; TimeDate.Month
    day     = headib  [2]			; TimeDate.Day
    hour    = headib  [3]			; TimeDate.Hour
    minute  = headib  [4]			; TimeDate.Minute
    second  = headib  [5] / 100			; TimeDate.CentiSec / 100
    doy     = headib  [7]			; TimeDate.Doy

    numaz   = headib  [9]			; Xdim
    numpix  = headib [10]			; Ydim

    skytx   = headf  [10]			; sky transmission
    scale   = headf  [13]			; data_scale
    offset  = headf  [14]			; data_offset

    obstype   = headie [0]	; IS_cal, IS_cal15, IS_dark, IS_coronal
    datatype  = headie [1]	; i, q, u, ba, sk, st, tscp, drift, tscp_inv
    procsteps = headie [2]	; IS_raw ... IS_cal_rect_slit

    IF (year LT 1900) THEN year = year + 1900

    syear   = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
    smonth  = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
    sday    = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)

    IF (month LT 10) THEN smonth = '0' + smonth
    IF (day   LT 10) THEN sday   = '0' + sday

    date = syear + '-' + smonth + '-' + sday

    shour   = STRTRIM (STRING (hour,   FORMAT='(I2)'), 2)
    sminute = STRTRIM (STRING (minute, FORMAT='(I2)'), 2)
    ssecond = STRTRIM (STRING (second, FORMAT='(I2)'), 2)

    IF (hour   LT 10) THEN shour   = '0' + shour
    IF (minute LT 10) THEN sminute = '0' + sminute
    IF (second LT 10) THEN ssecond = '0' + ssecond

    time = shour + ':' + sminute + ':' + ssecond

    datetime = syear + smonth + sday + '.' + shour + sminute + ssecond

    qlo  = FLTARR (numaz)
    qmid = FLTARR (numaz)
    qhi  = FLTARR (numaz)

    ;--- Get data type & processing level.

    CASE datatype OF
    0:    dtype = 'i'
    1:    dtype = 'q'
    2:    dtype = 'u'
    3:    dtype = 'ba'
    4:    dtype = 'sk'
    5:    dtype = 'st'
    6:    dtype = 'tscp'
    7:    dtype = 'drift'
    8:    dtype = 'tscp_inv'
    ELSE: dtype = '?'
    ENDCASE

    CASE procsteps OF
    10:   ptype = 'raw'
    11:   ptype = 'raw_spln'
    12:   ptype = 'raw_rect'
    13:   ptype = 'cal'
    14:   ptype = 'cal_slit'
    15:   ptype = 'cal_rect'
    16:   ptype = 'cal_rect_slit'
    17:   ptype = 'cal_rect_byte'
    18:   ptype = 'cal_avg'
    19:   ptype = 'cal_avg_slit'
    20:   ptype = 'cal_avg_rect'
    21:   ptype = 'cal_avg_rect_slit'
    22:   ptype = 'raw_avg'
    23:   ptype = 'raw_avg_spln'
    24:   ptype = 'raw_avg_rect'
    ELSE: ptype = '?'
    ENDCASE

    ylab = dtype + ' [' + ptype + ']'

    shour   = STRTRIM (STRING (hour),   2)
    sminute = STRTRIM (STRING (minute), 2)
    if (hour LT 10)   then shour   = '0' + shour
    if (minute LT 10) then sminute = '0' + sminute
    imgtime = shour + sminute

    PRINT, filename, '   skytx: ', skytx

    ;--- Read image.

    inimg  = INTARR (numaz, numpix)
    READU, 11, inimg				; Read image array.
    CLOSE, 11

    ;--- Apply photometric scaling.
    ;--- Accumulate image sum.

    IF (procsteps GE 13 AND procsteps LE 21) THEN	$
        img = (FLOAT (inimg) - offset) / scale 		$
    ELSE						$
	img = inimg

    ;--- Extract azimuthal scan data at low, middle & high radius positions.

    qlo   = img [*, pixlo]
    qmid  = img [*, pixmid]
    qhi   = img [*, pixhi]

    ;--- Display image & average image.

;    WSET, 0
;    TV, BYTSCL (ALOG10 (img + 2.e-9), -9, -5)

;--- Plot i vs. azimuthal index at 3 different heights.

xmin =    0
xmax = 1000
;PRINT, 'xmin/xmax: ', xmin, xmax

IF (NOT KEYWORD_SET (ymin)) THEN	$
    BEGIN
;    ymin = MIN (qlo)
    ymin = -200.0
    END

IF (NOT KEYWORD_SET (ymax)) THEN	$
    BEGIN
;    ymin = MAX (qlo)
    ymax = 5000.0
    END

;PRINT, 'ymin/ymax: ', ymin, ymax

;ptitle = ylab + ' @' + sradlo + 'Rsun   Mk4 ' + date + ' ' + time
ptitle = ylab + ' Azimuthal Scan  Mk4 ' + date + ' ' + time

PLOT, qlo,							$
      linestyle=0,						$
      color=fcol, 						$
      /nodata,							$
;      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)',	$
      charsize=cs, ticklen=1,					$
      xrange=[xmin, xmax],					$
      yrange=[ymin, ymax],					$
      title=ptitle,						$
      xtitle='Azimuthal Index',					$
      ytitle=ylab

lablo  = 'solid  line : ' + sradlo  + ' Rsun'
labmid = 'dotted line : ' + sradmid + ' Rsun'
labhi  = 'dashed line : ' + sradhi  + ' Rsun'

XYOUTS, 0.19, 0.90, labhi,  charsize=0.8, /normal, color=blue
XYOUTS, 0.19, 0.87, labmid, charsize=0.8, /normal, color=green
XYOUTS, 0.19, 0.84, lablo,  charsize=0.8, /normal, color=red

OPLOT, qlo, color=red, linestyle=0

;plotimg = TVRD ()
;gifname = datetime + '.mk4.q.r' + sradlo + '.gif'
;write_gif, gifname, plotimg

;IF (NOT KEYWORD_SET (ymin)) THEN	$
;    BEGIN
;    ymin = MIN (qmid)
;    ymin =    0.0
;    END

;IF (NOT KEYWORD_SET (ymax)) THEN	$
;    BEGIN
;    ymin = MAX (qmid)
;    ymax = 5000.0
;    END

;PRINT, 'ymin/ymax: ', ymin, ymax

;ptitle = ylab + ' @' + sradmid + 'Rsun   Mk4 ' + date + ' ' + time

;PLOT, qmid,							$
;      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
;      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
;      charsize=cs, ticklen=1, color=fcol,			$
;      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

OPLOT, qmid, color=green, linestyle=1

;plotimg = TVRD ()
;gifname = datetime + '.mk4.q.r' + sradmid + '.gif'
;write_gif, gifname, plotimg

;IF (NOT KEYWORD_SET (ymin)) THEN	$
;    BEGIN
;    ymin = MIN (qhi)
;    ymin =    0.0
;    END

;IF (NOT KEYWORD_SET (ymax)) THEN	$
;    BEGIN
;    ymin = MAX (qhi)
;    ymax = 5000.0
;    END

;PRINT, 'ymin/ymax: ', ymin, ymax

;ptitle = ylab + ' @' + sradhi + 'Rsun   Mk4 ' + date + ' ' + time

;PLOT, qhi,							$
;      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
;      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
;      charsize=cs, ticklen=1, color=fcol,			$
;      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

OPLOT, qhi, color=blue, linestyle=2

plotimg = TVRD ()

;gifname = datetime + '.mk4.q.r' + sradhi + '.gif'
;write_gif, gifname, plotimg

;--- Save plot as a GIF file.

gifname = datetime + '.mk4.q.ascan3' + '.gif'
write_gif, gifname, plotimg, r, g, b

ENDWHILE	;}
CLOSE, 10

!P.BACKGROUND=0
SET_PLOT, 'X'
END

