;+
; NAME		mk4qtxt.pro
;
; PURPOSE	Generate MK4 q vs time plots to monitor image quality.
;
; AUTHOR	Andrew L. Stanger  6 Feb 2002
;
; HISTORY	Andrew L. Stanger
;		16 May 2002: Correct for sky transmission.
;		17 May 2002: Set fixed y-axis range for plots.
;		21 May 2002: Set fixed x-axis range [16-24] for plots.
;		30 Oct 2002: Renamed "mk4plotqtx.pro" to "mk4qtxt.pro".
;-

PRO mk4qtxt, file_list

;--- Initialize variables.

filename = 'mk4.fts'

;--- Read a header to set some variables.

CLOSE, 2
OPENR, 2, file_list 
READF, 2, filename
CLOSE, 2
filename = STRTRIM (filename, 2)

headib  = 0 & headib  = LONARR (16)
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

CLOSE, 1
OPENR, 1, filename
READU, 1, headib
READU, 1, headf
READU, 1, headie
READU, 1, headpad
CLOSE, 1

numaz    = headib  [9]				; Xdim
numpix   = headib [10]				; Ydim
pixoff   = headf  [23]				; r0
pixscale = headf  [24]				; dr

;PRINT, numaz, numpix, pixscale, pixoff

year    = headib [0]			; Image.Year
month   = headib [1]			; Image.Month
day     = headib [2]			; Image.Day

IF (year LT 1900) THEN year = year + 1900

syear   = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
smonth  = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
sday    = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)

IF (month LT 10) THEN smonth = '0' + smonth
IF (day   LT 10) THEN sday   = '0' + sday

date = syear + '-' + smonth + '-' + sday

;PRINT, 'syear: ', syear, ' smonth: ', smonth, ' sday: ', sday

plotfile = syear + smonth + sday + '.mk4.cq.plot.ps'

;PRINT, 'numaz/numpix: ', numaz, numpix
;PRINT, 'pixscale/pixoff: ', pixscale, pixoff

;--- Specify low, middle & high radius sample heights.

radlo  = 1.2			; Rsun units
radmid = 1.5
radhi  = 1.8
pixlo  = FIX ((radlo  - pixoff) / pixscale + 0.5)
pixmid = FIX ((radmid - pixoff) / pixscale + 0.5)
pixhi  = FIX ((radhi  - pixoff) / pixscale + 0.5)

spixlo  = STRTRIM (STRING (pixlo), 2)
spixmid = STRTRIM (STRING (pixmid), 2)
spixhi  = STRTRIM (STRING (pixhi), 2)
sradlo  = STRTRIM (STRING (radlo,  FORMAT='(F3.1)'), 2)
sradmid = STRTRIM (STRING (radmid, FORMAT='(F3.1)'), 2)
sradhi  = STRTRIM (STRING (radhi,  FORMAT='(F3.1)'), 2)

;PRINT, 'pixlo/pixmid/pixhi: ', pixlo, pixmid, pixhi

;--- Heights from header of first file.

height = FINDGEN (numpix) * pixscale + pixoff

;--- Determine number of images to average.

CLOSE, 2
OPENR, 2, file_list
numimg = 0

WHILE ( NOT EOF (2) ) DO BEGIN
    numimg = numimg + 1
    READF, 2, filename
ENDWHILE

;--- Define arrays.

;PRINT, 'numimg: ', numimg

qlo    = FLTARR (numimg)
qmid   = FLTARR (numimg)
qhi    = FLTARR (numimg)
hours  = FLTARR (numimg)

;--- Open list file.

CLOSE, 2
OPENR, 2, file_list

imgnum = 0

;--- File loop.

WHILE ( NOT EOF (2) ) DO BEGIN	;{
    READF, 2, filename
    filename = STRTRIM (filename, 2)
;    PRINT, filename

    CLOSE, 1
    OPENR, 1, filename				; Open binary image file.

    READU, 1, headib				; 32-bit integers
    READU, 1, headf				; floats
    READU, 1, headie				; 32-bit integers
    READU, 1, headpad				; char

    ;--- Extract items from binary header.

    numaz   = headib  [9]				; Xdim
    numpix  = headib [10]				; Ydim
    scale   = headf  [13]			; data_scale
    offset  = headf  [14]			; data_offset
    hour    = headib  [3]			; image hour
    minute  = headib  [4]			; image minute

    obstype   = headie [0]	; IS_cal, IS_cal15, IS_dark, IS_coronal
    datatype  = headie [1]	; i, q, u, ba, sk, st, tscp, drift, tscp_inv
    procsteps = headie [2]	; IS_raw ... IS_cal_rect_slit

    skytx   = headf  [10]			; sky transmission

    CASE datatype OF
    0:    dtype = 'i'
    1:    dtype = 'q'
    2:    dtype = 'u'
    3:    dtype = 'ba'
    4:    dtype = 'sk'
    5:    dtype = 'st'
    6:    dtype = 'tscp'
    7:    dtype = 'drift'
    8:    dtype = 'tscp_inv'
    ELSE: dtype = '?'
    ENDCASE

    CASE procsteps OF
    10:   ptype = 'raw'
    11:   ptype = 'raw_spln'
    12:   ptype = 'raw_rect'
    13:   ptype = 'cal'
    14:   ptype = 'cal_slit'
    15:   ptype = 'cal_rect'
    16:   ptype = 'cal_rect_slit'
    17:   ptype = 'cal_rect_byte'
    18:   ptype = 'cal_avg'
    19:   ptype = 'cal_avg_slit'
    20:   ptype = 'cal_avg_rect'
    21:   ptype = 'cal_avg_rect_slit'
    22:   ptype = 'raw_avg'
    23:   ptype = 'raw_avg_spln'
    24:   ptype = 'raw_avg_rect'
    ELSE: ptype = '?'
    ENDCASE

    ilab = dtype + ' ' + ptype

    zerosky = 376.0
    IF (skytx NE 0.0) THEN		$
	skycor = zerosky / skytx	$	; sky transmission correction.
    ELSE				$
	skycor = 1.0

    ylab = ilab
    IF (skytx NE 1.0) THEN ylab = ylab + ' [Skytx corrected]'

    shour   = STRTRIM (STRING (hour),   2)
    sminute = STRTRIM (STRING (minute), 2)
    if (hour LT 10)   then shour   = '0' + shour
    if (minute LT 10) then sminute = '0' + sminute
    imgtime = shour + sminute
    hours [imgnum] = FLOAT (hour) + FLOAT (minute) / 60.0

    PRINT, filename, '   skytx: ', skytx, '   skycor: ', skycor, '   hours: ', hours [imgnum]

    ;--- Read image [Mk4 native format].

    inimg  = INTARR (numaz, numpix)
    READU, 1, inimg				; Read image array.
    CLOSE, 1

    ;--- Apply photometric scaling.
    ;--- Accumulate image sum.

    IF (procsteps GE 13 AND procsteps LE 21) THEN	$
        img = (FLOAT (inimg) - offset) / scale 		$
    ELSE						$
	img = inimg * skycor

    ;--- Compute azimuthal averages at low, middle & high radius positions.

    qlo  [imgnum] = TOTAL (img [*, pixlo])  / numaz
    qmid [imgnum] = TOTAL (img [*, pixmid]) / numaz
    qhi  [imgnum] = TOTAL (img [*, pixhi])  / numaz
    PRINT, 'qlo/mid/hi: ', qlo [imgnum], qmid [imgnum], qhi [imgnum]

    imgnum = imgnum + 1

    ;--- Display image.

;    WSET, 0
;    TV, BYTSCL (ALOG10 (img + 2.e-9), -9, -5)

    ;--- Concatenate image times to be stored in header.

    inum1 = (imgnum - 1) / 4
    inum2 = (imgnum - 1) - inum1 * 4
;    PRINT, 'imgnum/inum1/inum2: ', imgnum, inum1, inum2, ' imgtime: ', imgtime

ENDWHILE	;}

;--- Write plots to GIF files.

SET_PLOT, 'Z'				; Use Z-buffer for plots.

;PRINT, !version.os
;IF (!version.os EQ 'IRIX') THEN cs = 0.8 ELSE cs = 1.6
cs = 1.0

LOADCT, 0				; Load greyscale color map
fcol =   0
bcol = 255
!P.BACKGROUND=bcol

;PRINT, "Set plot device"
;SET_PLOT, 'PS'
;DEVICE, filename = plotfile

;--- Plot average q vs. time at 3 different heights.

xmin = MIN (hours)
xmax = MAX (hours)
xmin = 16.0
xmax = 24.0
;PRINT, 'xmin/xmax: ', xmin, xmax

ymin = MIN (qlo)
ymax = MAX (qlo)
ymin = 1000.0
ymax = 1600.0
ymin =    0.0			; 31 Jul 2002
ymax = 1600.0			; 31 Jul 2002
ymax = 3000.0			;  2 Aug 2002
;PRINT, 'ymin/ymax: ', ymin, ymax

ptitle = 'Average q @' + sradlo + ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, qlo,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
      charsize=cs, ticklen=1, color=fcol, 			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.qtxt.r' + sradlo + '.gif'
write_gif, gifname, plotimg

ymin = MIN (qmid)
ymax = MAX (qmid)
ymin = 200.0
;ymax =  500.0
ymax =  600.0
ymin =    0.0			; 31 Jul 2002
ymax =  600.0			; 31 Jul 2002
ymax = 1000.0			;  2 Aug 2002
;PRINT, 'ymin/ymax: ', ymin, ymax

ptitle = 'Average q @' + sradmid + ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, qmid,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
      charsize=cs, ticklen=1, color=fcol,			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.qtxt.r' + sradmid + '.gif'
write_gif, gifname, plotimg

ymin = MIN (qhi)
ymax = MAX (qhi)
ymin =    0.0
;ymax = 150.0
ymax = 200.0
ymax = 400.0			;  2 Aug 2002
;PRINT, 'ymin/ymax: ', ymin, ymax

ptitle = 'Average q @' + sradhi + ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, qhi,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle=ylab,		$
      charsize=cs, ticklen=1, color=fcol,			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.qtxt.r' + sradhi + '.gif'
write_gif, gifname, plotimg

!P.BACKGROUND=0
;DEVICE, /close
SET_PLOT, 'X'

END

