;+
; NAME		mk4sk.pro
;
; PURPOSE	Read Mk4 sky transmission files (FITS) & plot skytx vs time.
;
; SYNTAX	mk4sk, sk_fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR     2 July 2002
;-

PRO mk4sk, fits_list

stab = FLTARR (1100, 120)
atab = FLTARR (120 )
ttab = FLTARR (120)
xtab = FLTARR (120)

stab [*,*] = 0
a = FINDGEN (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), sin (a), /FILL
;USERSYM, [-0.5, -0.5], [ 0.5, -0.5], [0.5, 0.5], [-0.5, 0.5], [-0.5, -0.5]

CLOSE, 11
fits_name = ''

;--- File list loop.

i = 0
OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 11, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')

   sk  = readfits (fits_name, hdu, /noscale)		; Read image.
   ext = readfits (fits_name, hduext, exten_no=1)	; Read extension.

   ;--- Extract information from header.

   nsk      = fxpar (hdu,    'NAXIS1')
   dateobs  = fxpar (hdu,    'DATE-OBS')
   timeobs  = fxpar (hdu,    'TIME-OBS')
   dataform = fxpar (hdu,    'DATAFORM')
   dataform = STRTRIM (dataform, 2)

   syear    = STRMID (dateobs, 0, 4)
   smonth   = STRMID (dateobs, 5, 2)
   sday     = STRMID (dateobs, 8, 2)

   year     = FIX (syear)
   month    = FIX (smonth)
   day      = FIX (sday)

   shour    = STRMID (timeobs, 0, 2)
   sminute  = STRMID (timeobs, 3, 2)
   ssecond  = STRMID (timeobs, 6, 2)

   hour     = FIX (shour)
   minute   = FIX (sminute)
   second   = FIX (ssecond)

   IF (i EQ 0)  THEN	$
   BEGIN
      plotfile = syear + smonth + sday + '.mk4.sk.ps'
      begday = day
   END

   IF (day GT begday) THEN hour = hour + 24.0
   time = hour + minute / 60.0 + second / 3600.0

   skytx    = fxpar (hduext, 'SKYTX')
   amass    = fxpar (hduext, 'AMASS')

   print, 'skytx/amass: ', skytx, amass


   ;--- Plot sky transmission values for each image.

   PLOT, sk,							$
	 title='MK4 Sky Transmission ' + fits_name,		$
	 charsize=0.8,						$
	 xtitle='Angle Index', ytitle='Sky Transmission',	$
	 xrange=[0,1000],					$
	 yrange=[0,400],					$
	 psym=4
   WAIT, 1

   ;--- Save sky transmission & airmass values.

   stab [0:nsk-1, i] = sk
   atab [i] = amass
   ttab [i] = time
   xtab [i] = skytx
   i = i + 1

END	;}

CLOSE, 11

;--- Set postscript device.

;window, xsize = 640, ysize = 480, RETAIN=2

set_plot, 'PS'
device, filename = plotfile
print, 'plotfile: ', plotfile

nimg = i
print, 'nimg: ', nimg

;st = stab [0:nsk-1, nimg]
am = atab [0:nimg-1]
tt = ttab [0:nimg-1]
sx = xtab [0:nimg-1]

print, 'size (st): ', size (st)

stmin = FLTARR (nimg)
stmax = FLTARR (nimg)
ste   = FLTARR (nimg)

;--- Compute min & max sky transmission for each ray.

FOR i = 0, nimg -1 DO	$
BEGIN
   stmin [i] = MIN (stab [0:999, i])
   stmax [i] = MAX (stab [0:999, i])
   print, 'skytx/max/min: ', sx [i], stmax [i], stmin [i]
END

;--- Compute Elmore's sky transmission based upon airmass.

FOR i = 0, nimg -1 DO	$
BEGIN
    ste [i] = -11.0 * am [i] + 376.0
END

;FOR i = 0, nimg - 1 DO	$
;    print, 'time: ', tt [i], ' airmass: ', am [i]

;--- Plot min & max sky transmission vs. time.

ptitle = dateobs + ' Mk4 Min Sky Transmission vs. Time' 
PLOT, tt, stmin,				 	$
      xrange = [16.0, 24.0],				$
;      yrange = [0.0, 400.0],				$
      psym = -1,					$
      title = ptitle,					$
      xtitle = 'Time (hours)',				$
      ytitle = 'Sky Transmission'

ptitle = dateobs + ' Mk4 Max Sky Transmission vs. Time' 
PLOT, tt, stmax,				 	$
      xrange = [16.0, 24.0],				$
;      yrange = [0.0, 400.0],				$
      psym = -1,					$
      title = ptitle,					$
      xtitle = 'Time (hours)',				$
      ytitle = 'Sky Transmission'
OPLOT, sx, psym=4
XYOUTS, 0.2, 0.15, 'diamond: SkyTx', /normal

ptitle = dateobs + ' Mk4 Min/Max Sky Transmission vs. Time' 
PLOT, tt, stmax,				 	$
      xrange = [16.0, 24.0],				$
      yrange = [0.0, 400.0],				$
      psym = 8,						$
      title = ptitle,					$
      xtitle = 'Time (hours)',				$
      ytitle = 'Sky Transmission'
OPLOT, tt, stmin, psym=1
XYOUTS, 0.2, 0.15, 'plus: min  circle: max', /normal

;--- Plot airmass vs. time.

ptitle = dateobs + ' Mk4 Airmass vs. Time' 
PLOT, tt, am, 	$
      xrange = [16.0, 24.0],				$
;      yrange = [0.0, 8.0],				$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Time (hours)',					$
      ytitle = 'Airmass'

;--- Plot airmass versus sky transmission.

ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
PLOT, am, stmax,					$
      xrange = [1.0, 6.0],				$
      yrange = [280.0, 380.0],				$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Air Mass',				$
      ytitle = 'Sky Transmission'

;--- Overplot Sky transmission function (based on airmass).

OPLOT, am, ste

XYOUTS, 0.2, 0.15, 'line: y = -11 * airmass + 376', /normal

ptitle = dateobs + ' Mk4 Airmass vs. SkyTx' 
PLOT, am, sx,						$
      xrange = [1.0, 6.0],				$
      yrange = [280.0, 380.0],				$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Air Mass',				$
      ytitle = 'Sky Transmission'
OPLOT, am, ste
XYOUTS, 0.2, 0.15, 'line: y = -11 * airmass + 376', /normal

device, /close
set_plot, 'X'

END
