;+
; NAME		mk4skfitc.pro
;
; PURPOSE	Read Mk4 sky transmission files (FITS).
;		Plot max skytx vs airmass.
;		Eliminate bad points (if any are present).
;		Fit a straight line to the skytx between 1.5 & 6.0 airmass.
;		Store plot in both a GIF file.
;
; SYNTAX	mk4skfitc, sk_fits_list
;		amass keyword causes airmass vs. time to be plotted in PS file.
;
; HISTORY	Andrew L. Stanger   HAO/NCAR     8 July 2002
;		23 Jul 2002: Add standard deviation.
;-

;PRO mk4skfitc, fits_list, display=display, postscript=postscript

PRO mk4skfitc, fits_list

display = 1
amin = 1.5
amax = 6.0
smin = 280.0
smax = 380.0

red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)

stab = FLTARR (1100, 200)
atab = FLTARR (200 )
xtab = FLTARR (200)
ttab = STRARR (200)

answer = ''
efitlab = 'Elmore: skytx = -11.0 * airmass + 376.00'

stab [*,*] = 0
a = FINDGEN (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), sin (a), /FILL
;USERSYM, [-0.5, -0.5], [ 0.5, -0.5], [0.5, 0.5], [-0.5, 0.5], [-0.5, -0.5]

CLOSE, 11
fits_name = ''

;--- File list loop.

i = 0
OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Get file name from list.

   READF, 11, fits_name
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')

   ;--- Read FITS image header & pixel data.

   skdat = readfits (fits_name, hdu, /noscale)		; Read image.
   ext   = readfits (fits_name, hduext, exten_no=1)	; Read extension.

   ;--- Extract information from header.

   nsk      = fxpar (hdu,    'NAXIS1')
   dateobs  = fxpar (hdu,    'DATE-OBS')
   timeobs  = fxpar (hdu,    'TIME-OBS')
   dataform = fxpar (hdu,    'DATAFORM')
   dataform = STRTRIM (dataform, 2)

   syear    = STRMID (dateobs, 0, 4)
   smonth   = STRMID (dateobs, 5, 2)
   sday     = STRMID (dateobs, 8, 2)

   year     = FIX (syear)
   month    = FIX (smonth)
   day      = FIX (sday)
   doy      = day_of_year (year, month, day)
   sdoy     = STRING (doy, FORMAT = '(i3)')

   shour    = STRMID (timeobs, 0, 2)
   sminute  = STRMID (timeobs, 3, 2)
   ssecond  = STRMID (timeobs, 6, 2)

   hour     = FIX (shour)
   minute   = FIX (sminute)
   second   = FIX (ssecond)

   IF (i EQ 0)  THEN	$
   BEGIN
      skfile = syear + smonth + sday + '.mk4.sk.ps'
      amfile = syear + smonth + sday + '.mk4.am.ps'
      begday = day
   END

   IF (day GT begday) THEN hour = hour + 24.0
   time = hour + minute / 60.0 + second / 3600.0

   skytx    = fxpar (hduext, 'SKYTX')
   amass    = fxpar (hduext, 'AMASS')

   print, 'skytx/amass: ', skytx, amass

   ;--- Plot sky transmission values for each image.

;   PLOT, skdat,						$
;	 title='MK4 Sky Transmission ' + fits_name,		$
;	 charsize=0.8,						$
;	 xtitle='Angle Index', ytitle='Sky Transmission',	$
;	 xrange=[0,1000],					$
;	 yrange=[0,400],					$
;	 psym=4
;   WAIT, 1

   ;--- Save sky transmission & airmass values.

   xtab [i] = skytx
   stab [0:nsk-1, i] = skdat
   atab [i] = amass
   ttab [i] = timeobs
   i = i + 1

END	;}

CLOSE, 11

;window, xsize = 640, ysize = 480, RETAIN=2

;--- Resize arrays.

nimg = i
print, 'nimg: ', nimg

;st = stab [0:nsk-1, nimg]
am = atab [0:nimg-1]
tt = ttab [0:nimg-1]
sx = xtab [0:nimg-1]

print, 'size (st): ', size (st)

stmin = FLTARR (nimg)
stmax = FLTARR (nimg)
ste   = FLTARR (nimg)

;--- Compute min & max sky transmission for each ray.

ii = 0
FOR i = 0, nimg -1 DO	$
BEGIN
   stmin [i] = MIN (stab [0:999, i])
   stmax [i] = MAX (stab [0:999, i])
   print, 'skytx/max/min: ', sx [i], stmax [i], stmin [i]
   if (stmax [i] GE smin  AND stmax [i] LE smax AND $
       am [i]    GE amin  AND am [i]    LE amax) THEN ii = ii + 1
END
nimgok = ii

IF (nimgok LE 0) THEN			$
    BEGIN
    print, 'sky transmission and airmass values do not meet criteria'
    print, 'No plot generated.'
    goto, finish
    END


;--- Eliminate entries where sky transmission is less than 300.

st_fix = FLTARR (nimgok)
am_fix = FLTARR (nimgok)
tt_fix = STRARR (nimgok)
ii = 0

FOR i = 0, nimg-1 DO	$
BEGIN
    if (stmax [i] GE smin AND stmax [i] LE smax AND $
	am [i]    GE amin AND am [i]    LE amax) THEN	$
    BEGIN
	st_fix [ii] = stmax [i]
	am_fix [ii] = am [i]
	tt_fix [ii] = tt [i]
	ii = ii + 1
    END
END

st = st_fix
am = am_fix
nimg = nimgok

;--- Compute Elmore's sky transmission based upon airmass.

FOR i = 0, nimg -1 DO	$
BEGIN
    ste [i] = -11.0 * am [i] + 376.0
END

;--- Compute straight-line fit.

stcoef = POLY_FIT (am, st, 1, st_fit, st_err, st_sigma)

;--- Compute standard deviation.

stdifsum = 0.0
for i = 0, nimgok-1 DO	$
    BEGIN
    stdif = st_fit [i] - st [i]
    stdif2 = stdif * stdif
    stdifsum = stdifsum + stdif2
    END
var = stdifsum / nimgok
stdev = sqrt (var)
sstdev     = STRING (stdev, FORMAT='(F7.2)')
sdevlab = 'std.dev: ' + sstdev

;--- Get slope & intercept of straight-line fit.

intercept = stcoef [0]
slope     = stcoef [1]
sintercept = STRING (intercept, FORMAT='(F9.2)')
sslope     = STRING (slope,     FORMAT='(F9.2)')
sfitlab = 'fit : skytx = ' + sslope + ' * airmass + ' + sintercept
print, 'sfitlab: ', sfitlab

;--- Plot to a postscript file.

IF (KEYWORD_SET (postscript)) THEN			$
BEGIN ;{
    SET_PLOT, 'PS'
    DEVICE, filename = skfile
    print, 'skfile: ', skfile

    ;--- Plot airmass versus sky transmission.

    ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
    PLOT, am, st,					$
    ;      charsize = 0.8,				$
          xrange = [0.0, 10.0],				$
	  yrange = [smin, smax],			$
          psym = 1,					$
          title = ptitle,				$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'

    ;--- Overplot Elmore's sky transmission function (based on airmass).

    OPLOT, am, ste
    XYOUTS, 0.2, 0.15, efitlab, /normal

    device, /close

    IF (KEYWORD_SET (airmass)) THEN			$
    BEGIN ;{
        SET_PLOT, 'PS'
        DEVICE, filename = amfile
        print, 'amfile: ', amfile

        ;--- Plot airmass vs. time.

        ptitle = dateobs + ' Mk4 Airmass vs. Time' 
        PLOT, tt, am, 					$
        ;      charsize = 0.8,				$
              xrange = [16.0, 24.0],			$
        ;      yrange = [0.0, 8.0],			$
              psym = 1,					$
              title = ptitle,				$
              xtitle = 'Time (hours)',			$
              ytitle = 'Airmass'
        device, /close
    END ;}
END ;}

;--- Plot airmass versus sky transmission.

IF (KEYWORD_SET (display)) THEN			$
BEGIN
    SET_PLOT, 'X'
    lct, '/home/cordyn/color/bwy.lut'
    ymax = MAX (st)
    ymin = MIN (st)
    ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
    PLOT, am, st,						$
          charsize = 0.75,					$
          xrange = [1.0, 6.0],				$
          yrange = [smin, smax],				$
    ;      yrange = [ymin-10, ymax+10],			$
          psym = 1,						$
          title = ptitle,					$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'
    
    ;--- Overplot Elmore's sky transmission function (based on airmass).
    
    OPLOT, am, ste,    color=252, linestyle=1
    OPLOT, am, st_fit, color=254, linestyle=0
    
    XYOUTS, 0.2, 0.15, efitlab, /normal, charsize=0.75, color=252
    XYOUTS, 0.2, 0.10, sfitlab, /normal, charsize=0.75, color=254
END

loop: read, answer, $
	    prompt='Do you want to eliminate a point from the graph [y,n] ?'

ans = STRMID (answer, 0, 1)
if (ans EQ 'n') THEN goto, done
;goto, done

;--- Get the cursor position.

;--- Display a box cursor, which may be resized.

;x0 = am [0]
;y0 = st [0]
;nx =  0.2
;ny =  2.0

box_cursor, x0, y0, nx, ny, /init, /message
;box_cursor, x0, y0, nx, ny, /message
print, 'x0/y0: ', x0, y0, '  nx/ny: ', nx, ny

cursor, x0, y0, wait=4, /DATA

print, 'x0, y0, nx, ny: ', x0, y0, nx, ny

x1 = x0 - nx / 2
y1 = y0 - ny / 2
x2 = x1 + nx
y2 = y1 + ny

;--- Draw a box centered around the cursor position.

PLOTS, x1, y1,            color=254
PLOTS, x2, y1, /CONTINUE, color=254
PLOTS, x2, y2, /CONTINUE, color=254
PLOTS, x1, y2, /CONTINUE, color=254
PLOTS, x1, y1, /CONTINUE, color=254

j = 0
FOR i = 0, nimg-1 DO	$
BEGIN
    IF ((am [i] GE x1 AND am [i] LE x2) AND	$
        (st [i] GE y1 AND st [i] LE y2)) THEN	$
    BEGIN
	j = j + 1
    END
END

nimg_fix = nimg - j
print, 'nimg: ', nimg, '  j: ', j, '   nimg_fix: ', nimg_fix

j = 0
am_fix = FLTARR (nimg_fix)
st_fix = FLTARR (nimg_fix)

FOR i = 0, nimg-1 DO	$
BEGIN
    IF (NOT ((am [i] GE x1 OR  am [i] GE x2) AND	$
             (st [i] GE y1 AND st [i] LE y2))) THEN	$
    BEGIN
	print, 'j: ', j, '  i: ', i
        am_fix [j] = am [i]
	st_fix [j] = st [i]
	tt_fix [j] = tt [i]
	j = j + 1
    END
END

;--- Fit a straight line to the data.

stcoef = POLY_FIT (am_fix, st_fix, 1, st_fit, st_err, st_sigma)

print, 'stcoef: ', stcoef
intercept = stcoef [0]
slope     = stcoef [1]
sintercept = STRING (intercept, FORMAT='(F12.5)')
sslope     = STRING (slope,     FORMAT='(F9.3)')
sfitlab = 'fit : skytx = ' + sslope + ' * airmass + ' + sintercept
print, 'sfitlab: ', sfitlab

;--- Plot airmass versus sky transmission [edited version].

ymax = MAX (st_fix)
ymin = MIN (st_fix)
ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
PLOT, am_fix, st_fix,					$
      charsize = 0.75,					$
      xrange = [1.0, 6.0],				$
      yrange = [smin, smax],				$
;      yrange = [ymin-10, ymax+10],			$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Air Mass',				$
      ytitle = 'Sky Transmission'

;--- Overplot straight line fit.

OPLOT, am, ste,        color=252, linestyle=1
OPLOT, am_fix, st_fit, color=254, linestyle=0

XYOUTS, 0.2, 0.15, efitlab, /normal, charsize=0.75, color=252
XYOUTS, 0.2, 0.10, sfitlab, /normal, charsize=0.75, color=254

am = am_fix
st = st_fix
tt = tt_fix
nimg = nimg_fix
goto, loop

done: print, ' '

;--- Write plot to a GIF file.

SET_PLOT, 'Z'
lct, '/home/cordyn/color/bwy.lut'

ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
PLOT, am_fix, st_fix,					$
      background = 255, color=0,			$
;      charsize = 0.75,					$
      xrange = [1.0, 6.0],				$
      yrange = [smin, smax],				$
;      yrange = [ymin-10, ymax+10],			$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Air Mass',				$
      ytitle = 'Sky Transmission'

;--- Overplot straight line fit.

OPLOT, am, ste,        color=252, linestyle=1
OPLOT, am_fix, st_fit, color=254, linestyle=0

XYOUTS, 0.2, 0.25, efitlab, /normal, charsize=1.00, color=252
XYOUTS, 0.2, 0.20, sfitlab, /normal, charsize=1.00, color=254
XYOUTS, 0.2, 0.15, sdevlab, /normal, charsize=1.00, color=254

tvlct, red, green, blue, /get
gifplot = TVRD ()
gifname = syear + smonth + sday + '.mk4.skfit.gif'
write_gif, gifname, gifplot, red, green, blue

print, 'gifname: ', gifname


;--- Plot airmass vs. time.

IF (KEYWORD_SET (airmass)) THEN				$
BEGIN ;{
    SET_PLOT, 'X'
    lct, '/home/cordyn/color/bwy.lut'
    ptitle = dateobs + ' Mk4 Airmass vs. Time' 
    PLOT, tt, am, 	$
    ;      charsize = 0.8,				$
          xrange = [16.0, 24.0],			$
    ;      yrange = [0.0, 8.0],				$
          psym = 1,					$
          title = ptitle,				$
          xtitle = 'Time (hours)',			$
          ytitle = 'Airmass'
    gifplot = TVRD ()
    gifname = syear + smonth + sday + '.mk4.airmass.gif'
    write_gif, gifname, gifplot
    print, 'gifname: ', gifname
END ;}


FOR i = 0, nimg-1 DO	$
    print, 'st_fix: ', st_fix [i], '  am_fix: ', am_fix [i]

intercept = stcoef [0]
slope     = stcoef [1]
sintercept = STRING (intercept, FORMAT='(F9.2)')
sslope     = STRING (slope,     FORMAT='(F9.2)')
fitname = syear + smonth + sday + '.mk4.skfit.txt'

print, 'sslope/sintercept: ', sslope, ' ', sintercept

OPENW,  11, fitname
PRINTF, 11, sslope, sintercept
PRINTF, 11, 'Mk4 Sky Transmission vs. Airmass Fit'
PRINTF, 11, dateobs, '  DOY ', sdoy, $
	    '      slope/intercept: ', sslope, sintercept, ' std.dev: ', sstdev
PRINTF, 11, '   index     time        sky tx       airmass'
FOR i = 0, nimg-1 DO $
    PRINTF, 11, i, '   ', tt_fix [i], st_fix [i], am_fix [i]
CLOSE,  11

finish:		$
SET_PLOT, 'X'
lct, '/home/cordyn/color/bwy.lut'

END
