;+
; NAME		mk4skfitnl.pro
;
; PURPOSE	Read Mk4 sky transmission files (native).
;		Plot max skytx vs airmass.
;		Fit a straight line to the skytx between 1.5 & 6.0 airmass.
;		Optionally, select two points for straight-line fit.
;		Store plot in both a GIF file.
;
; SYNTAX	mk4skfitnl, file_list
;		amass keyword causes airmass vs. time to be plotted in PS file.
;
; HISTORY	Andrew L. Stanger   HAO/NCAR     8 July 2002
;		23 Jul 2002: Add standard deviation.
;		25 Jul 2002: Adapt for native file format.
;		25 Feb 2003: Use cursor to set two points for straight-line fit.
;-

;PRO mk4skfitnc, sk_list, display=display

PRO mk4skfitnl, sk_list

display = 1
x0 =   2
y0 = 300
x1 =   0
y1 =   0
x2 =   0
y2 =   0
nx =  20
ny =  20

amin = 1.5
amax = 6.0
smin = 280.0
smax = 380.0

red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)

stab = FLTARR (1100, 200)
atab = FLTARR (200 )
xtab = FLTARR (200)
ttab = STRARR (200)

answer = ''
efitlab = 'Elmore: skytx = -11.0 * airmass + 376.00'

stab [*,*] = 0
a = FINDGEN (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), sin (a), /FILL
;USERSYM, [-0.5, -0.5], [ 0.5, -0.5], [0.5, 0.5], [-0.5, 0.5], [-0.5, -0.5]

sk_name = ''
headib  = 0 & headib  = LONARR (16)
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

syear   = ''
smonth  = ''
sday    = ''
shour   = ''
sminute = ''
ssecond = ''

;--- File list loop.

i = 0
CLOSE, 11
OPENR, 11, sk_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

    ;--- Read FITS image header & pixel data.

    READF, 11, sk_name				; Get file name.
    print, 'sk_name: ', sk_name
    ftspos = STRPOS (sk_name, '.fts')

;   skdat = readfits (fits_name, hdu, /noscale)		; Read image.
;   ext   = readfits (fits_name, hduext, exten_no=1)	; Read extension.

   ;--- Extract information from header.

    CLOSE, 12
    OPENR, 12, sk_name
    READU, 12, headib		; Long integer header (begin)
    READU, 12, headf		; Floating point header
    READU, 12, headie		; Long integer header (end)
    READU, 12, headpad		; header padding.

    nsk    = headib  [9]	; Xdim
    year   = headib  [0]	; TimeDate.Year
    month  = headib  [1]	; TimeDate.Month
    day    = headib  [2]	; TimeDate.Day
    hour   = headib  [3]	; TimeDate.Hour
    minute = headib  [4]	; TimeDate.Minute
    second = headib  [5] / 100	; TimeDate.CentiSec
    doy    = headib  [7]	; TimeDate.Doy

    skdat = INTARR (nsk)
    READU, 12, skdat		; Read sky transmission vector.
    CLOSE, 12

    IF (year LT 1900) THEN year = year + 1900

    syear   = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
    smonth  = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
    sday    = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)
    sdoy    = STRTRIM (STRING (doy,   FORMAT='(I3)'), 2)

    IF (month LT 10) THEN smonth = '0' + smonth
    IF (day   LT 10) THEN sday   = '0' + sday
    IF (doy   LT 10) THEN sdoy   = '00' + sdoy
    IF (doy   GT  9 AND doy LT 100) THEN sdoy = '0' + sdoy
    dateobs = syear + '/' + smonth + '/' + sday

    shour   = STRTRIM (STRING (hour),   2)
    sminute = STRTRIM (STRING (minute), 2)
    ssecond = STRTRIM (STRING (second), 2)
    IF (hour   LT 10) THEN shour   = '0' + shour
    IF (minute LT 10) THEN sminute = '0' + sminute
    IF (second LT 10) THEN ssecond = '0' + ssecond
    timeobs = shour + ':' + sminute + ':' + ssecond

    IF (i EQ 0)  THEN	$
    BEGIN
      skfile = syear + smonth + sday + '.mk4.sk.ps'
      amfile = syear + smonth + sday + '.mk4.am.ps'
      begday = day
    END

    IF (day GT begday) THEN hour = hour + 24.0
    time = hour + minute / 60.0 + second / 3600.0

;    skytx    = fxpar (hduext, 'SKYTX')
;    amass    = fxpar (hduext, 'AMASS')

    amass = headf  [8]		; Amass
    skytx = headf [10]		; SkyTx

    print, 'amass/skytx: ', amass, skytx

    ;--- Plot sky transmission values for each image.

;    PLOT, skdat,						$
; 	   title='MK4 Sky Transmission ' + fits_name,		$
;	   charsize=0.8,					$
;	   xtitle='Angle Index', ytitle='Sky Transmission',	$
;	   xrange=[0,1000],					$
;	   yrange=[0,400],					$
;	   psym=4
;    WAIT, 1

    ;--- Save sky transmission & airmass values.

    xtab [i] = skytx
    stab [0:nsk-1, i] = skdat
    atab [i] = amass
    ttab [i] = timeobs
    i = i + 1

END	;}

CLOSE, 11

;window, xsize = 640, ysize = 480, RETAIN=2

nimg = i
print, 'nimg: ', nimg

;st = stab [0:nsk-1, nimg]
am = atab [0:nimg-1]
tt = ttab [0:nimg-1]
sx = xtab [0:nimg-1]

print, 'size (st): ', size (st)

stmin = FLTARR (nimg)
stmax = FLTARR (nimg)
ste   = FLTARR (nimg)

;--- Compute min & max sky transmission for each ray.

ii = 0
FOR i = 0, nimg -1 DO	$
BEGIN
   stmin [i] = MIN (stab [0:999, i])
   stmax [i] = MAX (stab [0:999, i])
   print, 'skytx/max/min: ', sx [i], stmax [i], stmin [i]
   if (stmax [i] GE smin  AND stmax [i] LE smax AND $
       am [i]    GE amin  AND am [i]    LE amax) THEN ii = ii + 1
END
nimgok = ii

IF (nimgok LE 0) THEN			$
    BEGIN
    print, 'sky transmission and airmass values do not meet criteria'
    print, 'No plot generated.'
    goto, finish
    END

;--- Eliminate entries where sky transmission is less than 300.

st_fix = FLTARR (nimgok)
am_fix = FLTARR (nimgok)
tt_fix = STRARR (nimgok)
ii = 0

FOR i = 0, nimg-1 DO	$
BEGIN
    if (stmax [i] GE smin AND stmax [i] LE smax AND $
	am [i]    GE amin AND am [i]    LE amax) THEN	$
    BEGIN
	st_fix [ii] = stmax [i]
	am_fix [ii] = am [i]
	tt_fix [ii] = tt [i]
	ii = ii + 1
    END
END

st = st_fix
am = am_fix
nimg = nimgok

;--- Compute Elmore's sky transmission based upon airmass.

FOR i = 0, nimg -1 DO	$
BEGIN
    ste [i] = -11.0 * am [i] + 376.0
END

stcoef = POLY_FIT (am, st, 1, st_fit, st_err, st_sigma)

stdifsum = 0.0
skmin  = 10000.0
skmax  =    -1.0
airmin = 10000.0
airmax =    -1.0
for i = 0, nimgok-1 DO	$
    BEGIN
    stdif = st_fit [i] - st [i]
    stdif2 = stdif * stdif
    stdifsum = stdifsum + stdif2
    IF (st [i] LT skmin)  THEN skmin  = st [i]
    IF (st [i] GT skmax)  THEN skmax  = st [i]
    IF (am [i] LT airmin) THEN airmin = am [i]
    IF (am [i] GT airmax) THEN airmax = am [i]
    END
var     = stdifsum / nimgok
stdev   = sqrt (var)
sstdev  = STRING (stdev, FORMAT='(F9.3)')
sdevlab = 'std.dev: ' + sstdev

sskmin  = STRING (skmin,  FORMAT='(F7.1)')
sskmax  = STRING (skmax,  FORMAT='(F7.1)')
sairmin = STRING (airmin, FORMAT='(F9.3)')
sairmax = STRING (airmax, FORMAT='(F9.3)')

intercept  = stcoef [0]
slope      = stcoef [1]
sintercept = STRING (intercept, FORMAT='(F9.3)')
sslope     = STRING (slope,     FORMAT='(F9.3)')
sfitlab    = 'fit : skytx = ' + sslope + ' * airmass + ' + sintercept

print, 'sfitlab: ', sfitlab
print, 'standard deviation: ', stdev

;--- Set postscript device.

IF (KEYWORD_SET (postscript)) THEN			$
BEGIN ;{
    SET_PLOT, 'PS'
    DEVICE, filename = skfile
    print, 'skfile: ', skfile

    ;--- Plot airmass versus sky transmission.

    ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
    PLOT, am, st,					$
    ;      charsize = 0.8,				$
          xrange = [0.0, 10.0],				$
	  yrange = [smin, smax],			$
          psym = 1,					$
          title = ptitle,				$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'

    ;--- Overplot Elmore's sky transmission function (based on airmass).

    OPLOT, am, ste
    XYOUTS, 0.2, 0.15, efitlab, /normal

    device, /close

    IF (KEYWORD_SET (airmass)) THEN			$
    BEGIN ;{
        SET_PLOT, 'PS'
        DEVICE, filename = amfile
        print, 'amfile: ', amfile

        ;--- Plot airmass vs. time.

        ptitle = dateobs + ' Mk4 Airmass vs. Time' 
        PLOT, tt, am, 					$
        ;      charsize = 0.8,				$
              xrange = [16.0, 24.0],			$
        ;      yrange = [0.0, 8.0],			$
              psym = 1,					$
              title = ptitle,				$
              xtitle = 'Time (hours)',			$
              ytitle = 'Airmass'
        device, /close
    END ;}
END ;}

;--- Plot airmass versus sky transmission.

IF (KEYWORD_SET (display)) THEN			$
BEGIN
    SET_PLOT, 'X'
    lct, '/home/cordyn/color/bwy.lut'
    ymax = MAX (st)
    ymin = MIN (st)
    ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
    PLOT, am, st,						$
          charsize = 0.75,					$
          xrange = [0.0, 6.0],				$
          yrange = [smin, smax],				$
    ;      yrange = [ymin-10, ymax+10],			$
          psym = 1,						$
          title = ptitle,					$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'
    
    ;--- Overplot Elmore's sky transmission function (based on airmass).
    
    OPLOT, am, ste,    color=252, linestyle=1
    OPLOT, am, st_fit, color=254, linestyle=0
    
    XYOUTS, 0.2, 0.15, efitlab, /normal, charsize=0.75, color=252
    XYOUTS, 0.2, 0.10, sfitlab, /normal, charsize=0.75, color=254
END

loop: read, answer, $
	    prompt='Set 2 points to determine a straight line fit [y,n] ?'

ans = STRMID (answer, 0, 1)
if (ans EQ 'y') THEN	$
BEGIN	;{
    
    redo: print, 'Optional Path.'

    ERASE
    PLOT, am, st,						$
          charsize = 0.75,					$
          xrange = [0.0, 6.0],				$
          yrange = [smin, smax],				$
    ;      yrange = [ymin-10, ymax+10],			$
          psym = 1,						$
          title = ptitle,					$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'
    
    ;--- Get the cursor position.
    
    print, 'Position cursor and click to set first point."
    
    CURSOR, x1, y1, /UP, /DATA

    print, 'x1/y1: ', x1, y1
    
    print, 'Position cursor and click to set second point."
    
    CURSOR, x2, y2, /UP, /DATA

    print, 'x2/y2: ', x2, y2
    
    IF (x1 GT x2) THEN $			; Interchange points 1 & 2.
    BEGIN
        xtmp = x1
        ytmp = y1
        x1 = x2
        y1 = y2
        x2 = xtmp
        y2 = ytmp
    END
    
    print, 'x1/y1: ', x1, y1, '  x2/y2: ', x2, y2

    slope = (y2 - y1) / (x2 - x1)
    intercept = - x1 * slope + y1

    print, 'slope: ', slope, '  intercept: ', intercept

    stdifsum = 0.0
    skmin  = 10000.0
    skmax  =    -1.0
    airmin = 10000.0
    airmax =    -1.0

    FOR i = 0, nimg - 1 DO	$
    BEGIN
	st_fit [i] = am [i] * slope + intercept
        stdif = st_fit [i] - st [i]
        stdif2 = stdif * stdif
        stdifsum = stdifsum + stdif2
        IF (st [i] LT skmin)  THEN skmin  = st [i]
        IF (st [i] GT skmax)  THEN skmax  = st [i]
        IF (am [i] LT airmin) THEN airmin = am [i]
        IF (am [i] GT airmax) THEN airmax = am [i]
    END
    var = stdifsum / nimgok
    stdev = sqrt (var)
    sstdev     = STRING (stdev, FORMAT='(F9.3)')
    sdevlab = 'std.dev: ' + sstdev

    sskmin  = STRING (skmin,  FORMAT='(F7.1)')
    sskmax  = STRING (skmax,  FORMAT='(F7.1)')
    sairmin = STRING (airmin, FORMAT='(F9.3)')
    sairmax = STRING (airmax, FORMAT='(F9.3)')

    ;--- Overplot Elmore's sky transmission function (based on airmass).
    
    OPLOT, am, ste,    color=252, linestyle=1

    ;--- Overplot Straight Line fit.

    xmin = 0
    ymin = intercept
    xmax = 6
    ymax = intercept + slope * 6
    PLOTS, [xmin, xmax], [ymin, ymax], /DATA, color=254

    sintercept = STRING (intercept, FORMAT='(F9.3)')
    sslope     = STRING (slope,     FORMAT='(F9.3)')
    sfitlab = 'fit : skytx = ' + sslope + ' * airmass + ' + sintercept

    print, 'sfitlab: ', sfitlab
    print, 'standard deviation: ', stdev

    XYOUTS, 0.2, 0.15, efitlab, /normal, charsize=0.75, color=252
    XYOUTS, 0.2, 0.10, sfitlab, /normal, charsize=0.75, color=254
    
    read, answer, $
	    prompt='straight line fit OK [y,n] ?'

    ans = STRMID (answer, 0, 1)
    if (ans EQ 'n') THEN GOTO, redo

END	$	;}
ELSE	$
BEGIN ;{
    
    print, 'Normal Path'

    j = 0
    FOR i = 0, nimg-1 DO	$
    BEGIN
        IF ((am [i] GE x1 AND am [i] LE x2) AND	$
            (st [i] GE y1 AND st [i] LE y2)) THEN	$
        BEGIN
    	j = j + 1
        END
    END
    
    nimg_fix = nimg - j
    print, 'nimg: ', nimg, '  j: ', j, '   nimg_fix: ', nimg_fix
    
    j = 0
    am_fix = FLTARR (nimg_fix)
    st_fix = FLTARR (nimg_fix)
    
    FOR i = 0, nimg-1 DO	$
    BEGIN
        IF (NOT ((am [i] GE x1 OR  am [i] GE x2) AND	$
                 (st [i] GE y1 AND st [i] LE y2))) THEN	$
        BEGIN
    	print, 'j: ', j, '  i: ', i
            am_fix [j] = am [i]
    	st_fix [j] = st [i]
    	tt_fix [j] = tt [i]
    	j = j + 1
        END
    END
    
    ;--- Fit a straight line to the data.
    
    stcoef = POLY_FIT (am_fix, st_fix, 1, st_fit, st_err, st_sigma)
    
    print, 'stcoef: ', stcoef [0], stcoef [1]

    intercept = stcoef [0]
    slope     = stcoef [1]

    sintercept = STRING (intercept, FORMAT='(F9.3)')
    sslope     = STRING (slope,     FORMAT='(F9.3)')
    sfitlab = 'fit : skytx = ' + sslope + ' * airmass + ' + sintercept

    print, 'sfitlab: ', sfitlab
    
    ;--- Plot airmass versus sky transmission [edited version].
    
    ymax = MAX (st_fix)
    ymin = MIN (st_fix)
    ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
    PLOT, am_fix, st_fix,					$
          charsize = 0.75,					$
          xrange = [0.0, 6.0],				$
          yrange = [smin, smax],				$
    ;      yrange = [ymin-10, ymax+10],			$
          psym = 1,						$
          title = ptitle,					$
          xtitle = 'Air Mass',				$
          ytitle = 'Sky Transmission'
    
    ;--- Overplot straight line fit.
    
    OPLOT, am, ste,        color=252, linestyle=1
    OPLOT, am_fix, st_fit, color=254, linestyle=0
    
    XYOUTS, 0.2, 0.15, efitlab, /normal, charsize=0.75, color=252
    XYOUTS, 0.2, 0.10, sfitlab, /normal, charsize=0.75, color=254
    
    am = am_fix
    st = st_fix
    tt = tt_fix
    nimg = nimg_fix
    
END ;}

done: print, ' '

;--- Write plot to a GIF file.

SET_PLOT, 'Z'
lct, '/home/cordyn/color/bwy.lut'

ptitle = dateobs + ' Mk4 Airmass vs. Max Sky Transmission' 
PLOT, am_fix, st_fix,					$
      background = 255, color=0,			$
;      charsize = 0.75,					$
      xrange = [1.0, 6.0],				$
      yrange = [smin, smax],				$
;      yrange = [ymin-10, ymax+10],			$
      psym = 1,						$
      title = ptitle,					$
      xtitle = 'Air Mass',				$
      ytitle = 'Sky Transmission'

;--- Overplot straight line fit.

OPLOT, am, ste,        color=252, linestyle=1
OPLOT, am_fix, st_fit, color=254, linestyle=0

XYOUTS, 0.2, 0.25, efitlab, /normal, charsize=1.00, color=252
XYOUTS, 0.2, 0.20, sfitlab, /normal, charsize=1.00, color=254
XYOUTS, 0.2, 0.15, sdevlab, /normal, charsize=1.00, color=254

tvlct, red, green, blue, /get
gifplot = TVRD ()
gifname = syear + smonth + sday + '.mk4.skfit.gif'
write_gif, gifname, gifplot, red, green, blue

print, 'gifname: ', gifname


;--- Plot airmass vs. time.

IF (KEYWORD_SET (airmass)) THEN				$
BEGIN ;{
    SET_PLOT, 'X'
    lct, '/home/cordyn/color/bwy.lut'
    ptitle = dateobs + ' Mk4 Airmass vs. Time' 
    PLOT, tt, am, 	$
    ;      charsize = 0.8,				$
          xrange = [16.0, 24.0],			$
    ;      yrange = [0.0, 8.0],				$
          psym = 1,					$
          title = ptitle,				$
          xtitle = 'Time (hours)',			$
          ytitle = 'Airmass'
    gifplot = TVRD ()
    gifname = syear + smonth + sday + '.mk4.airmass.gif'
    WRITE_GIF, gifname, gifplot
    print, 'gifname: ', gifname
END ;}


;FOR i = 0, nimg-1 DO	$
;    print, 'st_fix: ', st_fix [i], '  am_fix: ', am_fix [i]

fitname = syear + smonth + sday + '.mk4.skfit.txt'

OPENW,  11, fitname
PRINTF, 11, sslope, sintercept, sstdev, sskmin, sskmax, sairmin, sairmax
PRINTF, 11, 'Mk4 Sky Transmission vs. Airmass Fit'
PRINTF, 11, '-- date --            slope  intercept   stdev  skmin  skmax   airmin   airmax'
PRINTF, 11, dateobs, '  DOY ',	$
	    sdoy, sslope, sintercept, sstdev, sskmin, sskmax, sairmin, sairmax
PRINTF, 11, '   index     time        sky tx       airmass'
FOR i = 0, nimg-1 DO $
    PRINTF, 11, i, '   ', tt_fix [i], st_fix [i], am_fix [i]
CLOSE,  11

finish:		$
SET_PLOT, 'X'
lct, '/home/cordyn/color/bwy.lut'

END
