;+
; mk4tplot.pro   IDL procedure
;
; PURPOSE
;	Plot pixel magnitude versus time.
;
; SYNTAX
;	mk4tplot, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxpar
;
; HISTORY
;	Author: Andrew L. Stanger   HAO/NCAR    7 Feb 2002
;-

pro mk4tplot, fits_list

fits_name = ''				; Define FITS file name variable.
comment   = ''
shour     = ''
sminute   = ''
ssecond   = ''

r1 = 1.2
r2 = 1.5
r3 = 1.8

srad1 = STRTRIM (STRING (r1, FORMAT='(F3.1)'), 2)
srad2 = STRTRIM (STRING (r2, FORMAT='(F3.1)'), 2)
srad3 = STRTRIM (STRING (r3, FORMAT='(F3.1)'), 2)

CLOSE, 1

print, 'fits_list: ', fits_list

;--- Count the number of image files to process.

OPENR, 1, fits_list			; Open list file.
numimg = 0
WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      numimg = numimg + 1
   END
CLOSE, 1

print, 'numimg: ', numimg
hours = FLTARR (numimg)

pixval  = FLTARR (numimg)
avgval1 = FLTARR (numimg)
avgval2 = FLTARR (numimg)
avgval3 = FLTARR (numimg)

;--- File Loop.

inum = 0
OPENR, 1, fits_list			; Open list file.
WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.

      ;--- Read FITS file & extension.

      print, fits_name
      img   = readfits (fits_name, hdu)
      ext   = readfits (fits_name, hduext, exten_no=1)

      ;--- Get selected items from FITS headers.

      telescop = fxpar (hdu, 'TELESCOP')
      date     = fxpar (hdu, 'DATE-OBS')
      time     = fxpar (hdu, 'TIME-OBS')
      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      type_obs = fxpar (hdu, 'TYPE-OBS')

      telescop = STRLOWCASE (telescop)
      shour   = STRMID (time, 0, 2)
      sminute = STRMID (time, 3, 2)
      ssecond = STRMID (time, 6, 2)
      hour = FIX (shour)
      minute = FIX (sminute)
      second = FIX (ssecond)

      hours [inum] = hour + minute / 60.0 + second / 3600.0

      year     = fxpar (hduext, 'I_YEAR')
      month    = fxpar (hduext, 'I_MONTH')
      day      = fxpar (hduext, 'I_DAY')
      dr       = fxpar (hduext, 'DR')
      r0       = fxpar (hduext, 'R0')
      skytx    = fxpar (hduext, 'SKYTX')
      zerosky  = fxpar (hduext, 'ZEROSKY')
      datatype = fxpar (hduext, 'DATATYPE')
      procstep = fxpar (hduext, 'PROCSTEP')

      syear  = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
      smonth = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
      sday   = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)
      IF (month LT 10) THEN smonth = '0' + smonth
      IF (day   LT 10) THEN sday   = '0' + sday
      sdate = syear + smonth + sday
      sdate = STRTRIM (sdate, 2)

      ylab = type_obs

      ;--- Apply sky transmission correction for raw images.

      IF (STRPOS (type_obs, 'RAW') GE 0) THEN	$
      BEGIN
	  skycor = zerosky / skytx
	  img = img * skycor
	  print, 'Corrected for Sky Transmission ... skycor: ', skycor
	  ylab = ylab + ' [skytx corrected]'
      END

      ;--- Convert solar radius values to pixel indices.

      pix1 = FIX ((r1 - r0) / dr + 0.5)
      pix2 = FIX ((r2 - r0) / dr + 0.5)
      pix3 = FIX ((r3 - r0) / dr + 0.5)

      pixval [inum] = img [0, pix1]
      print, 'radius: ', r1, '  magnitude @azimuth 0: ', img [0, pix1]

      ;--- Compute average pixel value at the specified heights.

      avgval1 [inum] = TOTAL (img [*, pix1]) / FLOAT (xdim)
      avgval2 [inum] = TOTAL (img [*, pix2]) / FLOAT (xdim)
      avgval3 [inum] = TOTAL (img [*, pix3]) / FLOAT (xdim)
      inum = inum + 1
   END
CLOSE, 1

;--- Determine data type.

CASE datatype OF
    0:    dtype = 'i'
    1:    dtype = 'q'
    2:    dtype = 'u'
    3:    dtype = 'ba'
    4:    dtype = 'sk'
    5:    dtype = 'st'
    6:    dtype = 'tscp'
    7:    dtype = 'drift'
    8:    dtype = 'tscp-inv'
    ELSE: dtype = 'x'
ENDCASE

;--- Calibrated "q" is labeled "pb".

IF (dtype EQ 'q' AND procstep GE 13 and procstep LE 21) THEN	$
BEGIN
    dtype = 'pb'
END

plotname = sdate + '.' +  telescop + '.' + dtype + '.ps'
print, 'plotname: ', plotname

;--- Output plots as postscript files.

SET_PLOT, 'PS'
DEVICE, filename = plotname

;--- Generate plots.

;PLOT, title = 'Pixel Magnitude vs. Time',	$
;      xtitle = 'Azimuth Index',			$
;      ytitle = 'Magnitude',			$
;      pixval

;STOP

amin1 = MIN (avgval1)
amax1 = MAX (avgval1)

amin2 = MIN (avgval2)
amax2 = MAX (avgval2)

amin3 = MIN (avgval3)
amax3 = MAX (avgval3)

plab1 = ' Average ' + type_obs + ' ' + ' vs. Time @' + srad1 + ' Rsun  ' $
      + telescop + ' ' + date
plab2 = ' Average ' + type_obs + ' ' + ' vs. Time @' + srad2 + ' Rsun  ' $
      + telescop + ' ' + date
plab3 = ' Average ' + type_obs + ' ' + ' vs. Time @' + srad3 + ' Rsun  ' $
      + telescop + ' ' + date

PLOT, title  = plab1,						$
      xtitle = 'Time [hours]',					$
      ytitle = ylab,						$
      charsize=0.8,						$
      hours, avgval1, yrange=[amin1,amax1]

PLOT, title  = plab2, 						$
      xtitle = 'Time [hours]',					$
      ytitle = ylab,						$
      charsize=0.8,						$
      hours, avgval2, yrange=[amin2,amax2]

PLOT, title  = plab3,						$
      xtitle = 'Time [hours]',					$
      ytitle = ylab,						$
      charsize=0.8,						$
      hours, avgval3, yrange=[amin3,amax3]

DEVICE, /close
SET_PLOT, 'X'

END
