;
; plottscp.pro
;
; This IDL procedure will plot the new version of the tscp file with
; the 2d x21,x31 and Gain elements.
; 
; This procedure will read both the native binary format (checking for
; byte-order as it does so) and fits format.
;
; The output is a 1440x768 srf image with a png extension on output file.
; 
; usage:  plottscp,filename[,/FITS,/VIEW]
; 
; The FITS keyword must be used if you're reading in a fits image.
; The VIEW keyword will plot output to a window instead of a file.
;
; Tony Darnell
; August, 2004
;

PRO plottscp,filename,FITS=FITS,VIEW=VIEW

NAZ=720
NPX=384

if (!VERSION.release LT 5.6) THEN BEGIN
  print, 'This program needs IDL version 5.6 or later to run'
  RETURN
ENDIF

;
; Read in binary file
;
IF NOT KEYWORD_SET(FITS) THEN BEGIN
  
  ; Read in the various headers in their various data types

  byte_hdr = bytarr(1024)
  long1_hdr = lonarr(16)
  long2_hdr = lonarr(16)
  float_hdr = fltarr(32)


  buffer = intarr(2)
  TiltPhs =fltarr(2)

  Bopal=0.
  ang1=0.
  fp1=0.
  ang0=0.
  fp0=0.
  trt=0.
  fpt=0.

  x11=1.
  x12=0.
  x13=0.
  x21 = fltarr(NPX)
  x22 = fltarr(NPX)
  x23 = fltarr(NPX)
  x31 = fltarr(NPX)
  x32 = fltarr(NPX)
  x33 = fltarr(NPX)
  vgain = fltarr(NPX)

  ;
  ; These are the 2D elements
  ;
  Gain = fltarr(NAZ,NPX)
  x21a = fltarr(NAZ,NPX)
  x31a = fltarr(NAZ,NPX)

  top =long(0)
  ;
  ; Open the binary file and fill the headers and arrays
  ;
  close,1 & openr,1,filename & point_lun,-1,top & readu,1,long1_hdr
  readu,1,float_hdr
  readu,1,long2_hdr
  

  point_lun, 1,top & readu,1,byte_hdr
  point_lun, 1,top & readu,1,byte_hdr
  point_lun, 1,top & readu,1,byte_hdr
  
  point_lun, 1,top & readu,1,byte_hdr
  point_lun, 1,top & readu,1,byte_hdr
  point_lun, 1,top & readu,1,byte_hdr

  point_lun, 1,top & readu,1,byte_hdr
  point_lun, 1,top & readu,1,byte_hdr
  point_lun, 1,top & readu,1,byte_hdr

  readu,1,buffer
  readu,1,TiltPhs

  readu,1, Bopal
  readu,1, ang1
  readu,1, fp1
  readu,1, ang0
  readu,1, fp0
  readu,1, trt
  readu,1, fpt
  
  readu,1, x11
  readu,1, x12
  readu,1, x13
  readu,1, x21
  readu,1, x22
  readu,1, x23
  readu,1, x31
  readu,1, x32
  readu,1, x33
  
  readu,1, vgain
  readu,1, Gain
  readu,1, x21a
  readu,1, x31a
  close,1

  ;
  ;   Everything is read in.  Check for byteorder and swap if little endian
  ;
  ; tscp files are currently made on a big endian machine, so we need to 
  ; swap if we're runnng on anything else.
  ;
  
  long1_hdr = swap_endian(long1_hdr,/SWAP_IF_LITTLE_ENDIAN)
  long2_hdr = swap_endian(long2_hdr,/SWAP_IF_LITTLE_ENDIAN)
  float_hdr = swap_endian(float_hdr,/SWAP_IF_LITTLE_ENDIAN)
  byte_hdr  = swap_endian(byte_hdr,/SWAP_IF_LITTLE_ENDIAN)

  buffer  = swap_endian(buffer,/SWAP_IF_LITTLE_ENDIAN)
  TiltPhs = swap_endian(TiltPhs,/SWAP_IF_LITTLE_ENDIAN)
  Bopal   = swap_endian(Bopal,/SWAP_IF_LITTLE_ENDIAN)
  ang1    = swap_endian(ang1,/SWAP_IF_LITTLE_ENDIAN)
  fp1     = swap_endian(fp1,/SWAP_IF_LITTLE_ENDIAN)
  ang0    = swap_endian(ang0,/SWAP_IF_LITTLE_ENDIAN)
  fp0     = swap_endian(fp0,/SWAP_IF_LITTLE_ENDIAN)
  trt     = swap_endian(trt,/SWAP_IF_LITTLE_ENDIAN)
  fpt     = swap_endian(fpt,/SWAP_IF_LITTLE_ENDIAN)

  x11     = swap_endian(x11,/SWAP_IF_LITTLE_ENDIAN)
  x12     = swap_endian(x12,/SWAP_IF_LITTLE_ENDIAN)
  x13     = swap_endian(x13,/SWAP_IF_LITTLE_ENDIAN)
  x21     = swap_endian(x21,/SWAP_IF_LITTLE_ENDIAN)
  x22     = swap_endian(x22,/SWAP_IF_LITTLE_ENDIAN)
  x23     = swap_endian(x23,/SWAP_IF_LITTLE_ENDIAN)
  x31     = swap_endian(x31,/SWAP_IF_LITTLE_ENDIAN)
  x32     = swap_endian(x32,/SWAP_IF_LITTLE_ENDIAN)
  x33     = swap_endian(x33,/SWAP_IF_LITTLE_ENDIAN)
  vgain   = swap_endian(vgain,/SWAP_IF_LITTLE_ENDIAN)
  
  Gain   = swap_endian(Gain,/SWAP_IF_LITTLE_ENDIAN)
  x21a   = swap_endian(x21a,/SWAP_IF_LITTLE_ENDIAN)
  x31a   = swap_endian(x31a,/SWAP_IF_LITTLE_ENDIAN)

  year = strtrim(string(long1_hdr[0]),1)
  mon  = strtrim(string(long1_hdr[1],format='(I2.2)'),1)
  day  = strtrim(string(long1_hdr[2],format='(I2.2)'),1)

  date = year+'/'+mon+'/'+day
  outputname = filename+'.srf'

ENDIF ELSE BEGIN
  ;
  ; That's it for the binary stuff, here's the fits file stuff
  ;

  ; Get the 1D stuff from the binary table

  fits_read,filename,data,hdr,exten_no=1

  date = fxpar(hdr,'DATE-OBS');

  x11 = tbget(hdr,data,'X11')
  x12 = tbget(hdr,data,'X12')
  x13 = tbget(hdr,data,'X13')

  x21 = tbget(hdr,data,'X21')
  x22 = tbget(hdr,data,'X23')
  x23 = tbget(hdr,data,'X23')

  x31 = tbget(hdr,data,'X31')
  x32 = tbget(hdr,data,'X32')
  x33 = tbget(hdr,data,'X33')

  vgain = tbget(hdr,data,'VGAIN')

  ; Read in the 2D stuff as images
  fits_read,filename,x21a,x21a_hdr,exten_no=2
  fits_read,filename,x31a,x31a_hdr,exten_no=3
  fits_read,filename,gain,gain_hdr,exten_no=4

  outputname=filename
  strput,outputname,'srf',strpos(outputname,'fts')
  print,outputname

ENDELSE

;
; Set up plotting environment.  Output will be a gif image.
; Initially, I'll set up a plot of 1440x768 and divided the image
; up into 4 quadrants.
;

;
; Some scaling numbers, play with these till you find something you like
; I have no idea what these numbers should look like.  To me, all plots
; look bad.  These values were set up by Alice so I guess they're close
; to what they should be.
;
ms123 = 0.15
ms23  = 15.0
mx22_lo =  0.
mx22_hi = 40.
vg_lo   = 0.
vg_hi   = 40.0 
mysz=1.5

;
; Make the 1D plots in its own NAZ by NPX image for insertion into the
; main plot.
;
SET_PLOT,'Z'
DEVICE,SET_RESOLUTION=[NAZ,NPX]

!P.Multi = [0,3,3]
plot,[0,400],[-1.*ms123,ms123],title='x21',/ynoz,xstyle=1,ystyle=1,/nodata,$
	charsize=mysz
oplot,X21(*),thick=1

plot,[0,400],[mx22_lo,mx22_hi],title='x22',/ynoz,xstyle=1,ystyle=1,/nodata,$
   charsize=mysz
oplot,X22(*),thick=1

plot,[0,400],[-1.*ms23,ms23],title='x23',/ynoz,xstyle=1,ystyle=1,/nodata,$
   charsize=mysz
oplot,X23(*),thick=1

plot,[0,400],[-1.*ms123,ms123],title='x31',/ynoz,xstyle=1,ystyle=1,$
	/nodata, charsize=mysz
oplot,X31(*),thick=1

plot,[0,400],[-1.*ms23,ms23],title='x32',/ynoz,xstyle=1,ystyle=1,/nodata,$
   charsize=mysz
oplot,X32(*),thick=1

plot,[0,400],[mx22_lo,mx22_hi],title='x33',/ynoz,xstyle=1,ystyle=1,/nodata,$
   charsize=mysz
oplot,X33(*),thick=1

plot,[0,400],[vg_lo,vg_hi],title='Vgain',/ynoz,xstyle=1,ystyle=1,/nodata,$
   charsize=mysz
oplot,Vgain(*),thick=1

xyouts,300,60,filename,charsize=2.0,/device
xyouts,300,30,date,charsize=2.0,/device

plts = tvrd();

;
; Check for output format:  window or gif image
;
IF (KEYWORD_SET(VIEW)) THEN BEGIN

SET_PLOT,'X'
;Size of window
xsz = NAZ*2
ysz = NPX*2

WINDOW, xsize=xsz, ysize = ysz 
tv,plts,0
tv,bytscl(x21a,min=-0.1,max=0.),1
tv,bytscl(x31a,min=-0.05,max=0.05),2
tv,bytscl(gain,min=100,max=1000),3

ENDIF ELSE BEGIN

SET_PLOT,'Z'
DEVICE,SET_RESOLUTION=[NAZ*2,NPX*2]

tv,plts,0
tv,bytscl(x21a,min=-0.1,max=0.),1
tv,bytscl(x31a,min=-0.05,max=0.05),2
tv,bytscl(gain,min=100,max=1000),3

!order = 1
write_srf,outputname,tvrd()

ENDELSE

!order = 0
SET_PLOT,'X'

END
