;+
; IDL PROCEDURE
;
; NAME:
;	xiafts
;
; PURPOSE:
;	Animate a sequence of FITS images.
;
; CALLING SEQUENCE:
;	xiafts, 'fits.ls'
;	xiafts, 'fits.ls', rate=25
;	xiafts, 'fits.ls', rate=8, /label
;	xiafts, 'fits.ls', rate=2, /label, cm='~/color/sunset.lut'
;	xiafts, 'fits.ls', wmin=-1200, wmax=600
;	xiafts, 'fits.ls', cm='dif'
;	xiafts, 'fits.ls', cm='chip1'
;	xiafts, 'fits.ls', cm='bwy'
;
; INPUTS:
;	listfile : Character string containing the names of the FITS
;		   files to be animated.
;	rate : Animation rate (range: 0-100).
;	label: If present, label each image with frame #, date, time.
;	cm: Name of color map file (ASCII, each line: index, red, green, blue).
;	order: If set, image is displayed top to bottom;
;	       default is bottom to top.
;	wmin: window minimum intensity scaling value (overrides 'DISPMIN').
;	wmax: window maximum intensity scaling value (overrides 'DISPMAX').
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	READFITS
;	FXPAR
;	XIA
;	LCT
;
; MODIFICATION HISTORY:
;	 7 Aug 1995: procedure created by Andrew L. Stanger, HAO/NCAR
;	11 Aug 1995: added order keyword, ALS, HAO/NCAR
;	11 Mar 1996: reduce image size if image is larger than 1000x1000.
;	14 Apr 1998: replaced "readfits" with "fxread", since "readfits"
;		     did not always apply 'BSCALE/BZERO' to MK3 FITS images.
;	14 Oct 1999: Fix label position: ypos = nypix - 20 --> ypos = ydim - 20 
;		     Move label to bottom of window.
;		     Increase label size for SunOS.
;	 7 Apr 2000: Rotate image if CROTA1 is not zero.
;		     Also rotate PICS image if CROTA1 missing
;		     but SOLAR_P0 is present in FITS header.
;	20 Apr 2000: Do NOT rotate image IF 'DATAFORM'='POLAR'.
;	20 Apr 2000: Use either 'CRROTA1' and 'CROTA1' in PICS images.
;		     Note: CRROTA1 (incorrect spelling) was used for some
;		     cropped & rotated PICS images.
;	15 Nov 2000: Add min & max keywords so that user may set scaling.
;	25 Jan 2001: Rotate ONLY if crpix1 & crpix2 are non-negative.
;	 4 Sep 2001: Add /NAN keyword to BYTSCL call.
;	 5 Sep 2001: Add default directory for color table (/home/cordyn/color).
;-

PRO xiafts, listfile, rate=rate, label=label, cm=cm, order=order,	$
	    wmin=wmin, wmax=wmax

IF (NOT KEYWORD_SET (rate)) THEN rate = 100

labsiz = 1
IF (STRLOWCASE (!version.os) EQ 'sunos') THEN labsiz = 2

rotang = 0.0

filename = ' '
dataform = ''
ii = 0
CLOSE, 10
OPENR, 10, listfile

;--- Find number of images in file list.

WHILE (NOT EOF (10))  DO BEGIN	;{
   READF, 10, filename
   ii = ii + 1
ENDWHILE			;}
numimg = ii

CLOSE, 10

;--- Read first image to get size information.

fxread, filename, imgbuf, hdu		; Read first FITS image in listfile.

nbitpx = fxpar (hdu, 'BITPIX')
naxis  = fxpar (hdu, 'NAXIS')
nxpix  = fxpar (hdu, 'NAXIS1')
nypix  = fxpar (hdu, 'NAXIS2')
idate  = fxpar (hdu, 'DATE-OBS')
itime  = fxpar (hdu, 'TIME-OBS')

nxmax = 1000
nymax = 1000
xdim = nxpix
ydim = nypix

WHILE (xdim GT nxmax OR ydim GT nymax)  DO      $
   BEGIN
      xdim = xdim / 2
      ydim = ydim / 2
   ENDWHILE

IF      (nbitpx EQ   8) THEN imgbuf = BYTARR (xdim, nypix) $
ELSE IF (nbitpx EQ  16) THEN imgbuf = INTARR (xdim, nypix) $
ELSE IF (nbitpx EQ  32) THEN imgbuf = LONARR (xdim, nypix) $
ELSE IF (nbitpx EQ -32) THEN imgbuf = FLTARR (xdim, nypix) $
ELSE IF (nbitpx EQ -64) THEN imgbuf = DBLARR (xdim, nypix) $
ELSE exit

bkgbuf = INTARR (xdim, ydim) 
bkgbuf (*, *) = 0

IF (KEYWORD_SET (label))  THEN $
BEGIN
   WINDOW, 20, xsize = xdim, ysize = ydim, color=256, /pixmap
   WSET,   20
END

;--- Load color table.

IF (KEYWORD_SET (cm))  THEN 		$
BEGIN
   PRINT, 'cm: ', cm
   dirend = -1

   FOR i = 0, STRLEN (cm) - 1 DO		$
   BEGIN
      dirloc = STRPOS (cm, '/', i)
      IF (dirloc GE 0)  THEN dirend = dirloc
   END

   IF (dirend NE -1)  THEN		$
   BEGIN
      PRINT, 'dirend: ', dirend
      coldir = STRMID (cm, 0, dirend)
      PRINT, 'coldir: ', coldir
      ccm = STRMID (cm, dirend+1, strlen (cm) - dirend - 1)
      PRINT, 'ccm: ', ccm
   END

   IF (dirend EQ -1)  THEN		$
      lct, '/home/cordyn/color/' + cm + '.lut'	$
   ELSE					$
   lct, cm					; Load specified colormap.
END					$
ELSE   LOADCT, 0				; Load greyscale colormap.
 
init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 
 
IF ( init EQ 'y') THEN BEGIN ;{ 
   XIA, SET = [xdim+4, ydim+4, numimg], /SHOWLOAD
 
   CLOSE, 10 & OPENR, 10, listfile
 
   ;--- Image loop.

   ii = 0 
   WHILE ( NOT EOF (10) ) DO BEGIN ;{
      READF, 10, filename

      ;--- Read image into array & scale intensity levels to range: 0-255.

      fxread, filename, imgbuf, hdu	; Read FITS image in listfile.

      dispmin = 0.0
      dispmax = 0.0
      wintop  = 249

      nbitpx   = fxpar (hdu, 'BITPIX')
      naxis    = fxpar (hdu, 'NAXIS')
      nxpix    = fxpar (hdu, 'NAXIS1')
      nypix    = fxpar (hdu, 'NAXIS2')
      dispmin  = fxpar (hdu, 'DISPMIN')
      dispmax  = fxpar (hdu, 'DISPMAX')
      telescop = fxpar (hdu, 'TELESCOP')
      crpix1   = fxpar (hdu, 'CRPIX1',   COUNT=crpix1count)
      crpix2   = fxpar (hdu, 'CRPIX2',   COUNT=crpix2count)
      crota1   = fxpar (hdu, 'CROTA1',   COUNT=rotcount)
      crrota1  = fxpar (hdu, 'CRROTA1',  COUNT=rrotcount)
      pangle   = fxpar (hdu, 'SOLAR_P0', COUNT=pangcount)
      dataform = fxpar (hdu, 'DATAFORM', COUNT=dformcount)
      IF (dformcount EQ 1) THEN	$
	  dform = dataform		$
      ELSE				$
	 dform = 'NONE'

;      PRINT, 'crpix1count: ', crpix1count, '  crpix1 :, ', crpix1
;      PRINT, 'crpix2count: ', crpix2count, '  crpix2 :, ', crpix2
;      PRINT, 'rrotcount:     ', rrotcount, ' crrota1  : ', crrota1
;      PRINT, 'rotcount:      ', rotcount, ' crota1  : ', crota1
;      PRINT, 'pangcount:     ', pangcount, ' solar_p0: ', pangle
;      PRINT, 'dataformcount: ', dformcount, ' dataform: ', dataform


      IF (KEYWORD_SET (wmin)) THEN winmin = wmin ELSE winmin = dispmin
      IF (KEYWORD_SET (wmax)) THEN winmax = wmax ELSE winmax = dispmax

      PRINT, 'winmin/winmax: ', winmin, winmax
      dimfac = 1.0

      nxmax = 1000
      nymax = 1000
      xdim = nxpix
      ydim = nypix

      WHILE (xdim GT nxmax OR ydim GT nymax)	DO	$
	 BEGIN
	    dimfac = dimfac * 2.0
	    xdim = xdim / 2
	    ydim = ydim / 2
         ENDWHILE
;      PRINT, 'dimfac: ', dimfac, '  xdim/ydim: ', xdim, ydim

      ;--- Reduce image size if xdim or ydim > 1000 pixels.

      IF (nxpix NE xdim OR ydim NE nypix)  THEN	$
	    imgbuf = REBIN (imgbuf, xdim, ydim)

      ;--- Scale image to the range 0-249.

      IF (nbitpx EQ   8)  THEN $
            imgbuf = BYTSCL (imgbuf, top=wintop, /NAN)	$
      ELSE						$
	 IF (winmin LT winmax)  THEN	$
            imgbuf = BYTSCL (imgbuf, min=winmin, max=winmax, top=wintop, /NAN) $
         ELSE				$
            imgbuf = BYTSCL (imgbuf, top=wintop, /NAN)

      ;--- If order keyword is set, reverse image top to bottom.

      IF (KEYWORD_SET (order))  THEN imgbuf = REVERSE (imgbuf, 2)

      ;--- Rotate image (if needed).

      dpmpos = STRPOS (telescop, 'DPM')
      IF (dpmpos GE 0 AND rotcount EQ 0 AND rrotcount EQ 0) THEN	$
      BEGIN
	 IF (pangcount EQ 1) THEN	$
            rotang = pangle		$
         ELSE				$
	    BEGIN
	    IF (rrotcount EQ 1) THEN	$
	       rotang = -crrota1	$
            ELSE			$
	    IF (rotcount EQ 1) THEN	$
	       rotang = -crota1		$
            ELSE			$
	       rotang = 0.0
	    END
      END
;      PRINT, 'rotang: ', rotang

      IF (rotcount EQ 1) THEN				$
      BEGIN
	 IF (crota1 NE 0.0 AND STRPOS ('POLAR', dform) LT 0) THEN	$
	    rotang = -crota1		$
	 ELSE				$
	    rotang = 0.0
      END

      mag = 1.0
      IF (crpix1count EQ 1 AND crpix1 GE 0.0) THEN	$
	 xcen = crpix1 / dimfac - 0.5	$
      ELSE				$
         xcen = (xdim * 0.5) - 0.5

      IF (crpix2count EQ 1 AND crpix2 GE 0.0) THEN	$
	 ycen = crpix2 / dimfac - 0.5	$
      ELSE				$
         ycen = (ydim * 0.5) - 0.5

;      PRINT, 'xcen/ycen: ', xcen, ycen

      arotang = ABS (rotang)
      IF (rotang GT 0.0) THEN rotdir = 'CW'  ELSE rotdir = 'CCW'
      IF (rotang NE 0.0) THEN		$
	 BEGIN
	 PRINT, filename, '  rotated ', arotang, ' degrees ', rotdir
	 imgbuf = ROT (imgbuf, rotang, mag, xcen, ycen, MISSING=0)
	 END

      ;--- Label image if "label" keyword is set.

      IF (KEYWORD_SET (label))  THEN $
      BEGIN
         idate  = fxpar (hdu, 'DATE-OBS', count=count)
         sizdate = SIZE (idate)
         IF (count EQ 0 OR sizdate (1) LT 8)  THEN idate = filename

         itime  = fxpar (hdu, 'TIME-OBS', count=count)
         siztime = SIZE (itime)
         IF (count EQ 0 OR siztime (1) LT 8)  THEN itime = ''

         xpos = 10
         ypos = 10
         sframe = STRING (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         TV, imgbuf
         imglabel = sframe + '     ' + idate + '   ' + itime
         XYOUTS, xpos, ypos, imglabel, charsize=labsiz, /device
         XIA, FRAME = ii, WINDOW = !d.window	; Load image window.
      END				$
      ELSE				$
         XIA, FRAME = ii, IMAGE = imgbuf	; Load image array.

      ii = ii + 1
   ENDWHILE ;} 
 
   CLOSE, 10
 
ENDIF ;} 

;WDELETE, 20					; Delete temp window.

;--- Activate animation.

XIA, rate 

END 
