;+
; NAME:
;     XMOVIEDC
;
; PURPOSE:
;
;     This program is a simplified version of XINTERANIMATE. It is written
;     to illustrate the proper way to write an animation loop in a widget
;     program using the WIDGET_TIMER functionality and pixmaps.
;
; AUTHOR:
;
;       FANNING SOFTWARE CONSULTING
;       David Fanning, Ph.D.
;       1645 Sheely Drive
;       Fort Collins, CO 80526 USA
;       Phone: 970-221-0438
;       E-mail: davidf@dfanning.com
;       Coyote's Guide to IDL Programming: http://www.dfanning.com
;
; CATEGORY:
;
;     Widgets.
;
; CALLING SEQUENCE:
;
;      XMOVIEDC, image3d
;
; INPUTS:
;
;     image3d: A three-dimensional image array. The animation occurs over
;              the third dimension.
;
; KEYWORD PARAMETERS:
;
;     GROUP:   The group leader of the program. When the group leader dies,
;              this program dies as well.
;
;     TITLE:   The window title of the program. The default is "Animation
;              Example...".
;
; COMMON BLOCKS:
;
;     None.
;
; SIDE EFFECTS:
;
;     None.
;
; EXAMPLE:
;
;     To open the abnormal heart data and animate it, type:
;
;        filename = FILEPATH (SUBDIR=['examples', 'data'], 'abnorm.dat')
;        OPENR, lun, filename, /GET_LUN
;        data = BYTARR (64, 64, 15)
;        READU, lun, data
;        FREE_LUN, lun
;        data = REBIN(data, 256, 256, 15)
;
;        XMOVIEDC, data
;
; MODIFICATION HISTORY:
;
;      Written by: David W. Fanning, June 96.
;      Added slider for controlling animation speed. 30 June 99. DWF
;      Added pixmap operations. 15 May 2002. DWF
;      25 Jul 2003 : moviec.pro [ALS]  Added "colors" button
;                    & call to "xcolors.pro".
;-
;
;###########################################################################
;
; LICENSE
;
; This software is OSI Certified Open Source Software.
; OSI Certified is a certification mark of the Open Source Initiative.
;
; Copyright  1996-2002 Fanning Software Consulting.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
; For more information on Open Source Software, visit the Open Source
; web site: http://www.opensource.org.
;
;###########################################################################


PRO XMOVIEDC_CLEANUP, tlb

; --- Come here when the TLB is destroyed so you can clean up.

WIDGET_CONTROL, tlb, Get_UValue=info
IF N_Elements (info) NE 0 THEN WDelete, info.pixid

END


; ------------------------------------------------------------------------------


PRO XMOVIEDC_DELAY_EVENTS, event

; --- Get the INFO structure.

WIDGET_CONTROL, event.top, GET_UVALUE=info, /NO_COPY

; --- Set the delay.

info.delay = (100 - event.value) / 200.0	; Max delay 0.5 seconds.

; --- Put INFO structure back.

WIDGET_CONTROL, event.top, SET_UVALUE=info, /NO_COPY


END


; ------------------------------------------------------------------------------


PRO XMOVIEDC_EVENT, event

; --- Get the INFO structure.

WIDGET_CONTROL, event.top, GET_UVALUE=info, /NO_COPY

; --- What kind of event is this?

eventName = TAG_NAMES (event, /STRUCTURE_NAME)

; --- Code for BUTTON events.

IF eventName EQ 'WIDGET_BUTTON' THEN BEGIN

   CASE event.id OF

      info.start: BEGIN

         ; --- If stop flag is 0, then display next frame.

         IF info.stopflag EQ 0 THEN BEGIN
            WSET, info.wid
            DEVICE, Copy=[info.xsize*info.curr_img, 0, info.xsize, info.ysize,$
			  0, 0, info.pixid]
         ENDIF

         ; --- Update frame number.

         info.curr_img = info.curr_img + 1
         IF info.curr_img GT (info.nframes - 1) THEN info.curr_img = 0

         ; --- Set a timer event to get back into this event handler.

         WIDGET_CONTROL, event.ID, TIMER=0

         ; --- Update stop flag.

         info.stopflag = 0
      END ; of START button CASE.


      info.restart: BEGIN

         ; --- If stop flag is 0, then display next frame.

         IF info.stopflag EQ 0 THEN BEGIN
            WSET, info.wid
            DEVICE, Copy=[info.xsize*info.curr_img, 0, info.xsize, info.ysize,$
			  0, 0, info.pixid]
         ENDIF

         ; --- Update frame number.

         info.curr_img = info.curr_img + 1
         IF info.curr_img GT (info.nframes - 1) THEN info.curr_img = 0

         ; --- Set a timer event to get back into this event handler.

         WIDGET_CONTROL, event.ID, TIMER=0

         ; --- Update stop flag.

         info.stopflag = 0
      END ; of RESTART button CASE.


      info.stop: BEGIN
         info.stopflag = 1
         info.curr_img = info.curr_img - 1  ; Subtract a frame.
         END


      info.quit: WIDGET_CONTROL, event.top, /DESTROY
   ENDCASE

   ; --- Put INFO structure back.

   IF WIDGET_INFO(event.top, /VALID_ID) THEN $
      WIDGET_CONTROL, event.top, SET_UVALUE=info, /NO_COPY
   RETURN
ENDIF ; of BUTTON code

; --- Code for TIMER events.

IF (eventName EQ 'WIDGET_TIMER') THEN BEGIN

   IF (info.stopflag EQ 0) THEN BEGIN

      WSET, info.wid
      IF (info.true) THEN DEVICE, Decomposed=0

      ; --- Delay for a moment.

      Wait, info.delay

      ; --- Display the image by copying from the pixmap.

      DEVICE, Copy=[info.xsize*info.curr_img, 0, info.xsize, info.ysize, $
		    0, 0, info.pixid]

      ; --- Update frame number.

      info.curr_img = info.curr_img + 1
      IF info.curr_img GT (info.nframes - 1) THEN info.curr_img = 0

      ; --- Set a timer event to get back into this event handler.

      IF info.stopflag EQ 0 THEN WIDGET_CONTROL, event.ID, TIMER=0
   ENDIF

   ; --- Put INFO structure back.

   WIDGET_CONTROL, event.top, SET_UVALUE=info, /NO_COPY
ENDIF ; of TIMER code

END


; ------------------------------------------------------------------------------


PRO XMOVIEDC_DRAW_EVENTS, event

; --- This event handler is responsible for color processing
; --- and EXPOSE events.

Widget_Control, event.top, Get_UValue=info, /No_Copy

WSET,  info.wid
DEVICE, Decomposed=0

thisEvent = TAG_NAMES (event, /Structure)

; --- Reload colors for this widget. This is necessary
; --- if, for example, colors are changed from the command line.
; --- If this is a 24-bit display, the graphic must also be
; --- redisplayed by generating a button event.

IF (thisEvent EQ 'XCOLORS_LOAD') THEN BEGIN
   WSET, info.wid
   info.r = event.r [info.bottom:info.ncolors-1+info.bottom]
   info.g = event.g [info.bottom:info.ncolors-1+info.bottom]
   info.b = event.b [info.bottom:info.ncolors-1+info.bottom]

   IF (info.true) THEN BEGIN
      DEVICE, Decomposed=0
      pseudoEvent = {WIDGET_BUTTON, ID:info.lastid, TOP:event.top, $
                                    HANDLER:0L, SELECT:1}
      Widget_Control, info.lastid, Send_Event=pseudoEvent
   ENDIF
ENDIF

; --- Expose events processed here.

IF (thisEvent EQ 'WIDGET_DRAW') THEN BEGIN		; Button events

   TVLCT, info.r, info.g, info.b, info.bottom

   IF (event.type EQ 4) THEN BEGIN			; Expose events only

      IF (NOT info.true) THEN BEGIN			; 8-bit
         WSET, info.wid
;         DEVICE, COPY=[0, 0,info.xsize, info.ysize, 0, 0, info.pixid]
         DEVICE, Copy=[info.xsize*info.curr_img, 0, info.xsize, info.ysize,$
		       0, 0, info.pixid]

      ENDIF ELSE BEGIN					; 24-bit
         DEVICE, Decomposed=0
         pseudoEvent = {WIDGET_BUTTON, ID:info.lastid, TOP:event.top, $
                                       HANDLER:0L, SELECT:1}
         Widget_Control, info.lastid, Send_Event=pseudoEvent
      ENDELSE

   ENDIF
ENDIF

Widget_Control, event.top, Set_UValue=info, /No_Copy
END


; ------------------------------------------------------------------------------


PRO XMOVIEDC_LOAD_COLORS, event

; --- This event handler responds to the Data Colors button
; --- by calling XCOLORS. A "callback" ID is furnished with the
; --- NOTIFYID keyword.

Catch, error
IF (error NE 0) THEN BEGIN
   IF (error EQ -77) THEN BEGIN
      ok = Dialog_Message ('Program XCOLORS required.')
      WIDGET_CONTROL, event.top, Set_UValue=info, /No_Copy
      RETURN
   ENDIF

   ok = Dialog_Message (!Err_String)
   WIDGET_CONTROL, event.top, Set_UValue=info, /No_Copy
   RETURN
ENDIF

WIDGET_CONTROL, event.top, Get_UValue=info, /No_Copy

;--- Color Table tool.

XCOLORS, Group=event.top, $
	 Title='xmovie colors', $
         NotifyID=[event.id, event.top], $
;         NotifyID=[info.drawid, event.top], $
	 /drag

; --- Update the display for 24-bit displays.

;DEVICE, GET_VISUAL_DEPTH=this_depth
;IF (this_depth GT 8)  THEN	$
;BEGIN
;    WSET, info.wid
;    DEVICE, Copy=[info.xsize*info.curr_img, 0, info.xsize, info.ysize,$
;		  0, 0, info.pixid], decomposed=0
;;ok = Dialog_Message ('>>> Truecolor: redraw image.')
;ENDIF

WIDGET_CONTROL, event.top, Set_UValue=info, /No_Copy
END


; ------------------------------------------------------------------------------


PRO XMOVIEDC, image3d, GROUP=group, TITLE=title

ON_ERROR, 1

; --- Read some 3D example data for animation if none passed in.

IF N_PARAMS () EQ 0 THEN BEGIN
    PRINT, 'A 3D datacube must be provided to xmoviec.'
    RETURN
ENDIF

IF N_ELEMENTS (title) EQ 0 THEN title='xmoviedc'

; --- Collect information about the dimensions of the image cube.

imgsize = SIZE (image3d)
IF (imgsize [0] NE 3) THEN MESSAGE, 'Image parameter must be 3D.'
xdim   = imgsize [1]
ydim   = imgsize [2]
zdim   = imgsize [3]

; --- 8-bit color or something else ?

WINDOW, xsize=10, ysize=10, /pixmap, /free
WDELETE, !D.WINDOW
IF (!D.N_COLORS GE 256) THEN	$
BEGIN
    true = 1
    DEVICE, Decomposed=0
END				$
ELSE				$
    true = 0

bottom = 0
ncolors = (256 < !D.N_Colors)

; --- Create the widgets.

tlb     = WIDGET_BASE   (COLUMN=2, TITLE=title)

bbase   = WIDGET_BASE   (tlb, ROW=6)
ibase   = WIDGET_BASE   (tlb)

start   = WIDGET_BUTTON (bbase, Value='Start Animation')
restart = WIDGET_BUTTON (bbase, Value='Restart Animation')
stop    = WIDGET_BUTTON (bbase, Value='Stop Animation')
delayid = WIDGET_SLIDER (bbase, Max=100, Min=0, /Suppress_Value, $
                                Title='Frame Delay', $
			        Value=50, Event_Pro='XMOVIEDC_DELAY_EVENTS')
colors  = WIDGET_BUTTON (bbase, Value='Modify Color Table',	$
			        Event_Pro='XMOVIEDC_LOAD_COLORS')
quit    = WIDGET_BUTTON (bbase, Value='Exit Program')

;drawid  = WIDGET_DRAW   (ibase, XSIZE=xdim, YSIZE=ydim)

drawid  = WIDGET_DRAW   (ibase, XSIZE=xdim, YSIZE=ydim, retain=2,	$
				Event_Pro='XMOVIEDC_DRAW_EVENTS')

; --- Realize widgets.

WIDGET_CONTROL, tlb,    /REALIZE

; --- Set the draw widget to be the active graphics window.

WIDGET_CONTROL, drawid, Get_Value=wid, Draw_Expose_Events=1
WSET, wid
DEVICE, Decomposed=0

; --- Load color table.

;LOADCT, 3

; --- Get the color table.

TVLCT, r, g, b, /GET

; --- Load first image.

TVSCL, image3d [*, *, 0]
XYOUTS, 15, 15, /device, 'Frame 0'

; --- Create a pixmap and load the images onto it
;     for fast copy to the display.

WINDOW, /Free, /Pixmap, XSize=xdim * zdim, YSize=ydim
pixid = !D.Window

WIDGET_CONTROL, Hourglass=1

FOR j = 0, zdim - 1 DO BEGIN
   TVSCL, image3d [*, *, j], j * xdim, 0
   XYOUTS, 15 + (xdim * j), 15, /device, 'Frame ' + StrTrim (j, 2)
ENDFOR

WIDGET_CONTROL, Hourglass=0

; --- Create an "INFO" structure for program info. Store in TLB.

info = { start:start,         $	; ID of START button.
         restart:restart,     $ ; ID of RESTART button.
         stop:stop,           $	; ID of STOP button.
         quit:quit,           $	; ID of QUIT button.
         pixid:pixid,         $	; The window index number of the pixmap.
         wid:wid,             $	; Graphics window index number of draw widget.
	 drawid:drawid,       $	; Draw widget ID.
	 true:true,           $ ; Truecolor [0=no, 1=year].
	 r:r,                 $ ; Red   color table.
	 g:g,                 $ ; Green color table.
	 b:b,                 $ ; Blue  color table.
	 bottom:bottom,       $ ; Index for bottom of color table.
	 ncolors:ncolors,     $ ; Number of colors.
         delay:0.25,          $	; Amount of time (sec) to delay between frames.
         stopflag:0,          $	; A stop animation flag.
         xsize:xdim,          $	; The X size of the image.
         ysize:ydim,          $	; The Y size of the image.
         nframes:zdim,        $ ; The total number of animation frames.
	 lastid:start,        $	; ID of last command button.
         curr_img:0 }		; The current animation frame.

WIDGET_CONTROL, tlb, SET_UVALUE=info, /NO_COPY

; --- Run the program.

XMANAGER, 'XMOVIEDC', tlb, GROUP_LEADER=group, CLEANUP='XMOVIEDC_CLEANUP'
END

; ------------------------------------------------------------------------------

