;+ =========================================================================== +
;
; NAME: MK3HK_DECODE
;
; PURPOSE: 
;	Decode information in Mark-III binary "header" (actually a tailer).
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
; 	MK3HK_DECODE, MK3HK=MK3HK
;
; CALLED BY: 
;	MK3DISPLAY
;
; CALLS TO: 
;	none
;
; INPUTS:
;	MK3HK =  binary "header"; intarr (20)
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	
;		 
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	MK3_IMG
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Copy information in MK3 "header" into various variables in MK3_IMG
;	common block.
;
; MODIFICATION HISORY:
;	31 Mar 93 - Andrew L. Stanger (HAO/NCAR)
;- =========================================================================== -


pro mk3hk_decode, mk3hk=mk3hk

@mk3_img.common
; COMMON mk3_img, iyear, idoy, imonth, iday, ihour, iminute, isecond, $
;                 dtype, nbypx, npx, npy, channel, dmin, dmax, smin, smax, $
;                 scndir, scndur, pixrs, pangle

   ; Mark-III file "header" : 40 bytes.

;   mk3hdr = {mk3, $
;	flagch:		FIX  (0), $
;	juld:		FIX  (0), $
;	time:		LONG (0), $
;	phase:		FIX  (0), $
;	bright:		FIX  (0), $
;	pangle:		FIX  (0), $
;	nray:		FIX  (0), $
;	qmax:		FIX  (0), $
;	qmin:		FIX  (0), $
;	didxx:		FIX  (0), $
;	didxn:		FIX  (0), $
;	exdate:		FIX  (0), $
;	scndur:		FIX  (0), $
;	ipixscl:	LONG (0), $
;	ir0:		LONG (0), $
;	idr:		LONG (0)  $
;	}

   imask  = '177777'O
   lmask  = '37777777777'OL

   ;*** Decode flags in `filehd.flagch'. ***

   hkflags = FIX (ISHFT (FIX (mk3hk (0) AND imask), -3))
   mk3flags, hkflags, $
             quickflag, pBflag, qdflag, udflag, idflag, posflag, negflag

   ; MK3 "Julian" date.

   hkdate = fix (mk3hk (1)) AND imask

   mjdate, hkdate, imonth, iday, iyear

   ; Convert month & day to day-of-year.

   daymon  = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]  ; normal year.
   daymonl = [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]  ; leap year.

    if (((iyear mod 4) EQ 0)    AND $
	((iyear mod 100) NE  0) OR (iyear mod 400) EQ 0)  $
	then  leap = 1   else leap = 0

   idoy = iday
   for i = 0, imonth-1 do $
   begin
      if (leap) then $
         idoy = idoy + daymonl (i) $
      else $
         idoy = idoy + daymon (i)

   ; Compute time in hours, minutes & seconds.

   hktime = LONG (mk3hk (2) AND imask) * 65536 + LONG (mk3hk (3) AND imask)
   mk3time, hktime, ihour, iminute, isecond

   ; MK3 data types:
   ;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   ; q   --> cylindrical q data					/*  0 */
   ; rq  --> rectangular q data, origin in SE			/*  1 */
   ; srq --> scaled rectangular q data, origin in SE		/*  2 */

   ; i   --> cylindrical i data					/*  3 */
   ; ri  --> rectangular i data, origin in SE			/*  4 */
   ; sri --> scaled rectangular i data, origin in SE		/*  5 */

   ; u   --> cylindrical u data					/*  6 */
   ; ru  --> rectangular u data, origin in SE			/*  7 */
   ; sru --> scaled rectangular u data, origin in SE 		/*  8 */

   ; pB  --> cylindrical pB data				/*  9 */
   ; rpB  --> rectangular pB data, origin in SE			/* 10 */
   ; srpB --> scaled rectangular pB data, origin in SE		/* 11 */

   ; aba --> barrel angles as ascii in degrees *10		/* 12 */
   ; ba --> barrel angles (shorts) with 16384=360 degrees	/* 13 */
   ; ast --> scan times as ascii in seconds			/* 14 */
   ; st --> scan times (shorts) with 60=1 second (Hz)		/* 15 */
   ;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   if (qdflag) then dtype =  0
   if (idflag) then dtype =  3
   if (udflag) then dtype =  6
   if (pbflag) then dtype =  9

   if (nbypx eq 1)  then dtype = dtype + 2	$
		    else			$
   if (nbypx eq 2)  then dtype = dtype + 1

   ; Channel number.

   channel = FIX (mk3hk (0)) AND '000007'O	; least significant 3 bits.

   ; Pixel data min & max values.

   dmax = FIX (mk3hk (8))				; Data max value.
   dmin = FIX (mk3hk (9))				; Data min value.

   ; Scan direction.

   scndir = '*****'
   if (posflag) then scndir = 'CCW +' $
   else $
   if (negflag) then scndir = ' CW -'

   ; Scan duration (seconds).

   scndur = FIX ((mk3hk (13) AND imask) / 60)		; convert to seconds.

   ; Geometric scale size: pixels/Rsun.

   pixrs = (LONG (mk3hk (14) AND imask) * 65536 + $
	    LONG (mk3hk (15) AND imask)) / 10000.0		; # Pixels/Rsun.

   ; P-angle.

   pangle = FLOAT (mk3hk (6) / 60.0)	; Convert P-angle to degrees.

   end
   return
end
;+ =========================================================================== +
