;+
; NAME		sfv.pro
;
; PURPOSE	Display a Spartan sp201 image (FITS format).
;
; SYNTAX	svf, fits_image, /gif, cm='colormap.lut'
;
;		fits_image	filename of Spartan WLC FITS image.
;		gif		write displayed image as a GIF file.
;		cm		pathname of ASCII colormap file.
;				Each line has the syntax: index red green blue
;				where index = 0, 1, 2, ... 255,
;				and red/green/blue are in the range [0:255].
;
; EXAMPLES	svf, 'sp.19981101.1234.fts'
; 		svf, 'sp.19981101.1234.fts', cm='/home/stanger/color/bwy.lut'
; 		svf, 'sp.19981101.1234.fts', /gif
;
; EXTERNAL
; PROCEDURES	spdisp.pro
;		spartan_annotate.pro
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   29 August 2001
;-

PRO sfv, fits_name, gif=gif, cm=cm

;--- Load color map.

;loadct, 3				; standard RSI color table #3.
;lct, '/home/stanger/color/sunsetcp.lut'	; Load color table.

lct, '/home/stanger/color/bwy.lut'	; Load color table.

IF (KEYWORD_SET (cm))  THEN	$
   lct, cm

red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)
tvlct, red, green, blue, /GET		; Fetch RGB color look-up tables.

;CLOSE, 1
;fits_name = ''
xb = 160 
yb =  88
xdim_prev = 0
ydim_prev = 0

;OPENR, 1, fits_list
;WHILE (NOT EOF (1)) DO  $
;BEGIN  ;{

   ;--- Read FITS image header & pixel data.

;   READF, 1, fits_name                          ; Get file name.
;   PRINT, 'fits_name: ', fits_name

   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   print, 'basename: ', basename

   img = readfits (fits_name, hdu)

;   ext = readfits (fits_name, hduext, exten_no=1)

   ;--- Extract information from header.

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')

   xdim2 = xdim * 2
   ydim2 = ydim * 2

;   window, xsize=xdim2, ys=ydim2

   ;--- Resize window [if image size has changed].

   if (xdim NE xdim_prev OR ydim NE ydim_prev)  THEN	$
       window, xsize=xdim2+xb, ys=ydim2+yb

   print, 'xdim2 + xb: ', xdim2 + xb
   print, 'ydim2 + yb: ', ydim2 + yb

   xdim_prev = xdim
   ydim_prev = ydim

   ;--- Annotate image.

   spartan_annotate, hdu, xdim2, ydim2, xb, yb

   ;--- Get information from FITS header.

   orbit_id = fxpar (hdu, 'ORBIT-ID')
   image_id = fxpar (hdu, 'IMAGE-ID')
   telescop = fxpar (hdu, 'TELESCOP')
   instrume = fxpar (hdu, 'INSTRUME')
   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')

   telescop = STRTRIM (telescop, 2)
      IF (telescop NE 'SPARTAN 201' AND instrume NE 'WLC')  THEN         $
      BEGIN
       PRINT, 'TELESCOP: ', telescop, ' should be "SPARTAN 201".'
       PRINT, 'INSTRUME: ', instrume, ' should be "WLC".'
       RETURN
      END

   ;--- Expand image (double its size).

   img2 = rebin (img, xdim2, ydim2)

   ;--- Display image.

   tv, bytscl (img2, min=datamin, max=datamax, top=249), xb, yb

   ;--- Write displayed image as a GIF file (if "gif" keyword is set).

   IF (KEYWORD_SET (gif))  THEN		$
   BEGIN
      gif_file = basename + '.gif'
      img_gif = tvrd ()
      write_gif, gif_file, img_gif, red, green, blue
   END

;END    ;}
;CLOSE, 1

END

