;+
; NAME		mkfovmask.pro
;
; PURPOSE	Create FOV mask.
;
; SYNTAX	mkfovmask, fits_image
;
;		fits_image	filename of Spartan WLC FITS image.
;
; EXAMPLES	mkfovmask, '19981101.1234.mk3.cpb.fts'
;
; EXTERNAL
; PROCEDURES	fitsdisp.pro
;		fits_annotate.pro
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   14 September 2001
;-

PRO mkfovmask, fits_name

;--- Load color map.

;loadct, 3					; standard RSI color table #3.
;lct, '/home/stanger/color/sunsetcp.lut'	; Load color table.

lct, '/home/stanger/color/bwy.lut'	; Load color table.

IF (KEYWORD_SET (cm))  THEN lct, cm

red   = bytarr (256)
green = bytarr (256)
blue  = bytarr (256)
tvlct, red, green, blue, /GET		; Fetch RGB color look-up tables.

xb = 160 
yb =  88
xb =   0
yb =   0
xdim_prev = 0
ydim_prev = 0
dispmin   = 0.0
dispmax   = 0.0
rmin      = 1.15
rmax      = 6.0

   ;--- Read FITS image header & pixel data.

   ftspos   = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)
   print, 'basename: ', basename

   img = readfits (fits_name, hdu)

   ;--- Extract information from header.

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')

   ;--- Resize window [if image size has changed].

   if (xdim NE xdim_prev OR ydim NE ydim_prev)  THEN	$
       WINDOW, xsize=xdim+xb, ys=ydim+yb

   print, 'xdim + xb: ', xdim + xb
   print, 'ydim + yb: ', ydim + yb

   xdim_prev = xdim
   ydim_prev = ydim

   ;--- Annotate image.

;   fits_annotate, hdu, xdim, ydim, xb, yb

   ;--- Get information from FITS header.

   orbit_id = fxpar (hdu, 'ORBIT-ID')
   image_id = fxpar (hdu, 'IMAGE-ID')
   telescop = fxpar (hdu, 'TELESCOP')
   instrume = fxpar (hdu, 'INSTRUME')
   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   xcen     = fxpar (hdu, 'CRPIX1')
   ycen     = fxpar (hdu, 'CRPIX2')
   cdelt1   = fxpar (hdu, 'CDELT1')
   cdelt2   = fxpar (hdu, 'CDELT2')
   pixrs    = fxpar (hdu, 'CRRADIUS')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   solar_p0 = fxpar (hdu, 'SOLAR_P0')
   solar_b0 = fxpar (hdu, 'SOLAR_B0')
   bunit    = fxpar (hdu, 'BUNIT')
   bscale   = fxpar (hdu, 'BSCALE')
   bzero    = fxpar (hdu, 'BZERO')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')
   dispmin  = fxpar (hdu, 'DISPMIN', count=countmin)
   dispmax  = fxpar (hdu, 'DISPMAX', count=countmax)

   print, 'rsun:         ', rsun
   print, 'solar_p0:     ', solar_p0
   print, 'solar_b0:     ', solar_b0
   print, 'bscale/bzero: ', bscale, bzero
   print, 'datamin/datamax: ', datamin, datamax
   print, 'dispmin/dispmax: ', dispmin, dispmax

;   telescop = STRTRIM (telescop, 2)
;      IF (telescop NE 'SPARTAN 201' AND instrume NE 'WLC')  THEN         $
;      BEGIN
;       PRINT, 'TELESCOP: ', telescop, ' should be "SPARTAN 201".'
;       PRINT, 'INSTRUME: ', instrume, ' should be "WLC".'
;       RETURN
;      END

   imin = MIN (img, max=imax)
   print, 'imin/imax: ', imin, imax

   ;--- Display image.

   dmin = dispmin
   dmax = dispmax
;   IF (KEYWORD_SET (wmin) OR wmin EQ 0.0)  THEN dmin = wmin
   IF (KEYWORD_SET (wmin))  THEN dmin = wmin
   IF (KEYWORD_SET (wmax))  THEN dmax = wmax
   print, 'dmin/dmax: ', dmin, dmax

   TV, BYTSCL (img, min=dispmin, max=dispmax, top=249), xb, yb

;   fitsdisp, fits_name, xdim_prev, ydim_prev, gif=gif, wmin=dmin, wmax=dmax

   ;--- Write displayed image as a GIF file (if "gif" keyword is set).

   IF (KEYWORD_SET (gif))  THEN		$
   BEGIN
      gif_file = basename + '.gif'
      img_gif = TVRD ()
      WRITE_GIF, gif_file, img_gif, red, green, blue
   END

   imgb = BYTSCL (img, min=dispmin, max=dispmax, top=249)

   rdist = float (xdim-1) - xcen		; radial distance FOV (pixels).
   xmax  = rdist / pixrs			; radial distance FOV (Rsun).

   FOR iy = 0, ydim - 1 DO $
   BEGIN
      y = float (iy) - ycen
      FOR ix = 0, xdim - 1 DO $
      BEGIN
	 x = float (ix) - xcen
	 d = sqrt (x*X + y*y)				; pixel units
	 r = d / pixrs			 		; solar radius units.
	 IF (telescop EQ 'Mk4' OR telescop EQ '') THEN $
	 BEGIN
	    IF (r LT rmin) THEN imgb [ix, iy] = 254	; inner FOV limit.
	 END
	 IF (telescop EQ 'Mk4')  THEN rmax = 2.8 ELSE rmax = xmax
	 IF (r GT rmax) THEN imgb [ix, iy] = 253	; outer FOV limit.
      END
   END

   TV, imgb
END

