;+		IDL procedure
;
; NAME		hgw.pro
;
; PURPOSE	Convert Cartesian coordinate images in FITS format
;		to Heliographic coordinates [long, sin (lat)].
;
; SYNTAX	hgw, 'fits_list'
;
; OUTPUT	FITS images in heliographic coordinate form [*.hgw.fts].
;
; NOTICE      	Input images should already be in Heliocentric coordinates
;		[i.e., solar north should be aligned with +Y axis].
;
; EXTERNAL	carrlong.pro
;		Solar Software library : pb0r.pro, utc2tai,
;					 fxpar.pro, fxaddpar.pro, readfits.pro
;		C-language library "cint2d.so" : cint2d.c, wintdr.c
;
; AUTHOR	Guiliana DeToma   HAO/NCAR   14 Nov 2002
;
; HISTORY	13 Mar 2003 : [ALS] Replace interp2d.pro call with
;			      CALL_EXTERNAL to cint2d.c & wintdr.c routines.
;-

; ------------------------------------------------------------------------------

PRO  hgw,  listfile,  soho=soho

  ; PROGRAM ASSUMES THAT THE IMAGES ARE ALREADY ROTATED FOR P ANGLE
  ; PROGRAM ASSUMES THAT HEADER COORDINATES ARE CORRECT, DOES NOT CHECK THEM 

  ;--- Find if soho or not.

  IF (NOT KEYWORD_SET (soho)) THEN soho = 0

  ;--- Find number of images in file list.

  file    = ' '
  newfile = ' '
  ii      = 0

  OPENR, lun, listfile, /get_lun
    WHILE (NOT EOF (lun))  DO BEGIN 
      READF, lun, file
      ii = ii + 1
    ENDWHILE                       
    num = ii
  CLOSE, lun
  FREE_LUN, lun
    
  ;--- Define new arrays in 0.2 degree step = 1x181.

  newsize = 901.0
  new_image = DBLARR (901, 901)

  ;--- This is in equal steps of 0.2 degress in longitude.
  ;--- in 1D it would be FINDGEN (901) / 5.0 -90.0

  newlon = FINDGEN (901, 901) mod (901) / 5.0 - 90

  ;--- This is in equal steps of 0.2 degress in latitude.

  newlat = TRANSPOSE (FINDGEN (901, 901) mod (901) / 5.0 ) - 90

  ;--- This is in equal steps in sin (latitude).
  ;--- in 1D would be l = (FINDGEN (901) - 450.0) / 450.

  l = TRANSPOSE (FINDGEN (901, 901) mod (901) - 450) / 450.0
  newlat = ASIN (l)
  newlat = newlat * !radeg
   

  ;--- Reopen file list.

  OPENR, lun, listfile, /get_lun
  file= ' '

  ;--- Read images and convert to heliographic coordinates.

  FOR i = 0, num - 1 DO BEGIN 

      READF, lun, file
    
      image = readfits (file, hdu)
      IF (soho EQ 0) THEN temp = readfits (file, exten,/exten)

      xsize = (SIZE (image))[1]
      ysize = (SIZE (image))[2]

      xsize = LONG (xsize)
      ysize = LONG (ysize)

      IF (xsize NE ysize) THEN PRINT, '!!!!! WARNING: IMAGE NOT SQUARE !!!!!' 
 
      ;--- Read header information.

      IF (soho EQ 1) THEN date_str= fxpar (hdu, 'DATE') else $
      date_str = fxpar (hdu, 'DATE-OBS')
      time_str = fxpar (hdu, 'TIME-OBS')
      date     = date_str + ' ' + time_str 
      t_ave    = date_str + '_' +  time_str +  '_TAI'

      year   = STRMID (date_str, 0, 4)	   
      month  = STRMID (date_str, 5, 2)         
      day    = STRMID (date_str, 8, 2)         
    
      t_ave = year + '.' + month + '.' + day + '_' +  time_str + '_TAI'

      year  = FLOAT (year)
      month = FLOAT (month)
      day   = FLOAT (day)
    
      hour = STRMID (time_str, 0, 2)         & hour = FLOAT (hour)
      min  = STRMID (time_str, 3, 2)         & min  = FLOAT (min)
      sec  = STRMID (time_str, 6, 2)         & sec  = FLOAT (sec)
 
      time = hour + min / 60.0 + sec / 3600.0	; time in hours

      xcen   =  fxpar (hdu, 'CRPIX1')      ; x center in pixel 
      ycen   =  fxpar (hdu, 'CRPIX2')      ; y center in pixel 
      dx     =  fxpar (hdu, 'CDELT1')      ; arcsec/pixel
      dy     =  fxpar (hdu, 'CDELT2')      ; arcsec/pixel
  
      ;--- Find time of observations.

      obs_time = utc2tai (anytim2utc (date))
     
      ;--- Find solar p,b angles and radius at time of the image using pb0r.
      ;--- pb0r is more reliable than the values in the header for MLSO !! 

      angles = pb0r (obs_time, soho=soho)
     
      p_ang = angles [0]
      b_ang = angles [1]
      ; r0  = 1.0 / TAN (angles [2] / (60.0 * !radeg))
      radius_pix = angles [2] * 60.0 / dx
     
      ;--- Compute b angle in radians.

      b0 = angles [1] / !radeg
    
      PRINT, ' p, b, and radius :  ' , p_ang, b_ang, radius_pix

 
      IF (soho eq 0) THEN BEGIN 
        coords  = fxpar (hdu, 'COORDNAM')
        sol_p  =  fxpar (hdu, 'SOLAR_P0')
        sol_b  =  fxpar (hdu, 'SOLAR_B0')

        solar_l0 = fxpar (hdu, 'SOLAR_L0')
        IF (solar_l0 LT 0) THEN solar_l0 = solar_l0 + 360.0

        datamin = fxpar (hdu, 'DATAMIN')
        datamax = fxpar (hdu, 'DATAMAX')
 
        hour = STRMID (time_str, 0, 2)         & hour = FLOAT(hour)
        min  = STRMID (time_str, 3, 2)         & min  = FLOAT(min)
        sec  = STRMID (time_str, 6, 2)         & sec  = FLOAT(sec)
 
        time = hour + min / 60.0 + sec / 3600.0		; time in hours

        xcen   =  fxpar (hdu, 'CRPIX1')      ; x center in pixel 
        ycen   =  fxpar (hdu, 'CRPIX2')      ; y center in pixel 
        dx     =  fxpar (hdu, 'CDELT1')      ; arcsec/pixel
        dy     =  fxpar (hdu, 'CDELT2')      ; arcsec/pixel
      ENDIF

      PRINT, 'date: ', date
 
      ;--- Find Carrington number and longitude for central meridian.
    
      carrlong, date, cr, clong, cmp=1

 
      ;--- Define coordinates for interpolartion on the disk =
      ;--- regular array centered at image center in unit of radius.

      xx = FINDGEN (xsize, ysize) mod (xsize) - xcen   
      yy = TRANSPOSE (FINDGEN (ysize, xsize) mod (ysize) ) - ycen 

      xx = REFORM (temporary (xx), xsize * ysize)
      yy = REFORM (temporary (yy), xsize * ysize)
    
      xx = xx / radius_pix
      yy = yy / radius_pix
 

      ;--- Find pixels within and outside the solar disk.

      r_2D = SQRT ( xx ^ 2.0 + yy ^ 2.0 )
      on_disk = WHERE (r_2D LT 1.0)
      ; off_disk= WHERE (r_2D GT 1.0)

      ;--- There are bad values around edge of the Sun.

      off_disk= WHERE (r_2D GT 0.975)


      ;--- Transform sin(latitude0 and longitude = newlat and newlon in 
      ;--- new x and y coord on the disk.

      ;--- First find 3D cartesian coordinates.
     
      newlat = newlat / !radeg
      newlon = newlon / !radeg

      newzz  = SIN (newlat)
      newyy  = COS (newlat) * SIN (newlon)
      newxx  = COS (newlat) * COS (newlon)
     
      newxx = REFORM (temporary (newxx), newsize * newsize)
      newyy = REFORM (temporary (newyy), newsize * newsize)
      newzz = REFORM (temporary (newzz), newsize * newsize)
  
      xyz = TRANSPOSE ([ [newxx], [newyy], [newzz] ])

      ;--- Rotate for -b0 to simulate disk.
   
      rotmx = [[ COS (-b0), 0.0, SIN (-b0)], [0.0, 1.0, 0.0], $
	       [-SIN (-b0), 0.0, COS (-b0)]]
      xyz = rotmx # temporary (xyz)

      ;--- Find coordinates in 2D.

      newx = xyz [1, *]
      newy = xyz [2, *]

      ;--- Latitude in 2D is = arc_tangent of z/y.
      ;--- Latitude= ATAN (xyz [2, *], xyz [1, *])
      ;--- Radius in 2D is  
      ;--- rad_2d = SQRT (xyz [2, *] ^ 2 +  xyz [1, *] ^ 2)
      
      ;--- Interpolate original image on the new coordinate grid
      ;--- equally spaced in latitude and longitude in 0.2 degree increments.

      xx = REFORM (temporary (xx), xsize, ysize)
      yy = REFORM (temporary (yy), xsize, ysize)
   
      newx = REFORM (temporary (newx), newsize, newsize)
      newy = REFORM (temporary(newy), newsize, newsize)
     
      image (off_disk) = 0


;      new_image= interp2d (image, xx, yy, newx, newy)  

      ;--- Make integers LONG and FLOATing point arrays DOUBLE precision.

      image      = DOUBLE (image)
      xsize      = LONG   (xsize)
      ysize      = LONG   (ysize)
      xcen       = DOUBLE (xcen)
      ycen       = DOUBLE (ycen)
      radius_pix = DOUBLE (radius_pix)
      newx       = DOUBLE (newx)
      newy       = DOUBLE (newy)
      newxsize   = LONG   (newsize)
      newysize   = LONG   (newsize)

      ;--- Convert angle coordinates to pixel coordinates for interpolation.
    
      xslope = (MAX (xx) - MIN (xx)) / (xsize - 1)
      yslope = (MAX (yy) - MIN (yy)) / (ysize - 1)
      xint = (newx - MIN (xx)) / xslope
      yint = (newy - MIN (yy)) / yslope

      PRINT, 'xslope/yslope: ', xslope, yslope

      PRINT, ''
;      PRINT, 'xx [*, ysize/2]:'
;      PRINT, xx [*, ysize/2]
;      PRINT, ''
;      PRINT, 'newx [*, newysize/2]:'
;      PRINT, newx [*, newysize/2]
;      PRINT, ''
      PRINT, 'xint [*, newysize/2]:'
      PRINT, xint [*, newysize/2]

      PRINT, ''
      PRINT, 'size (xint):      ', size (xint)
      PRINT, 'size (yint):      ', size (yint)
      PRINT, ''
      PRINT, 'xsize/ysize:      ', xsize, ysize
      PRINT, 'size (xx):        ', size (xx)
      PRINT, 'size (yy):        ', size (yy)
      PRINT, 'newxsize/newysize:', newxsize, newysize
      PRINT, 'size (newx):      ', size (newx)
      PRINT, 'size (newy):      ', size (newy)
      PRINT, ''
      PRINT, 'size (image):     ', size (image)
      PRINT, 'size (xsize):     ', size (xsize)
      PRINT, 'size (ysize):     ', size (ysize)
      PRINT, 'size (xcen):      ', size (xcen)
      PRINT, 'size (ycen):      ', size (ycen)
      PRINT, 'size (radius_pix):', size (radius_pix)
      PRINT, 'size (new_image): ', size (new_image)
      PRINT, 'size (newx):      ', size (newx)
      PRINT, 'size (newy):      ', size (newy)
      PRINT, 'size (newxsize):  ', size (newxsize)
      PRINT, 'size (newysize):  ', size (newysize)

      ;--- Call C-language routine to do 2D weighted interpolation.

      iext = CALL_EXTERNAL ('/home/stanger/syn/cint2dlib.so', 'cint2d',$
			     image, xsize, ysize, xcen, ycen, radius_pix,$
			     new_image, xint, yint, newxsize, newxsize) 

;      PRINT, 'iext: ', iext

;      PRINT, 'size (new_image): ', size (new_image)

      imin = MIN (new_image, max=imax)
      PRINT, 'imin/imax: ', imin, imax

      new_image = FIX (new_image)

;      PRINT, 'hdu: '
;      PRINT, hdu

      ;--- Add keywords about carrington totations and time of observation.
      ;--- Find carrington number and longitude for central meridian.
    
      carrlong, date, cr, clong, cmp=1

      ;--- Test carr rot.

      IF (year EQ 2002 AND month eq 4 AND day GE 27) THEN cr = 1989

    
      fxaddpar, hdu, 'COORDNAM', 'HELIOCENTRIC', $
		                 ' SOLAR NORTH ALIGNED WITH +Y AXIS'
      fxaddpar, hdu, 'OBS_L0',  solar_l0, ' SOLAR LONGITUDE L0' 
      fxaddpar, hdu, 'OBS_CR',  cr, ' CARRINGTON ROTATION'
      fxaddpar, hdu, 'T_REC', t_ave, 'AVERAGE TIME OF IMAGE'
      fxaddpar, hdu, 'DATAMIN', imin, 'MINIMUM DATA VALUE'
      fxaddpar, hdu, 'DATAMAX', imax, 'MAXIMUM DATA VALUE'
      fxaddpar, hdu, 'DISPMIN', imin, 'MIN VALUE FOR DISPLAY [UNSCALED]'
      fxaddpar, hdu, 'DISPMAX', imax, 'MAX VALUE FOR DISPLAY [UNSCALED]'
      IF (soho EQ 0) THEN $
          fxaddpar, hdu, 'SOLAR_L0', solar_l0, $
			 ' SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE'

 
      ;--- Write new square images in files.

      flen    = STRLEN (file) - 3
      newfile = STRMID (file, 0,flen) + 'hgw.fts'

      writefits, newfile, new_image, hdu
      IF (soho EQ 0) THEN writefits, newfile, temp , exten, /append

  ENDFOR

  CLOSE, lun
  FREE_LUN, lun
  CLOSE, /all

END

; ------------------------------------------------------------------------------
