;+
; NAME		remap
;
; PURPOSE	Convert Cartesian coordinate images to Heliographic coordinates.
;
; SYNTAX	remap_v2, 'fits_list', soho=soho, stepsize=stepsize, lat=lat $
;		       rmax=rmax, minpix=minpix, bkg=bkg, /rot_pangle
;
;		'fits_list"	Name of a file containing a list of FITS images.
;		soho		Image from SOHO satellite
;				default: 0
;		stepsize	Pixel increment [deg] for output image.
;				default: 0.2
;		lat		If set, use equal steps of latitude.
;				default: 0
;		rmax		Maximum radius to use for interpolation.
;				default: 0.99
;		minpix		Minimum # neighboring pixels to use for interp.
;				default: 3
;		bkg		Background level to set for output image.
;				default: 0.0
;;		rot_pangle       Rotate for p angle, if set to 1
;                               assumes images have not been rotated for p angle
;                               default: 0
;
; OUTPUT	Floating point FITS image in Heliographic Coordinates.
;
; LIMITATIONS	Program assumes that the images are already rotated for P-angle.
;		Program assumes that header coordinates are correct, and
;		it does not check them.
;
; IDL ROUTINES
;		anytim2jd
;		get_date_time
;		jd_carr
;		nearest_gridcell
;		pb0r
;		fxaddpar
;		fxpar
;		readfits
;		writefits
;		utc2tai
;
; C-lang. ROUTINES
;		libfwint2d.so library: [Linux & Irix supported]
;		    fwint2d.c
;		    fwintrb.c
;
; AUTHORS	Giuliana DeToma + Leslie Mayer + Andy Stanger
;
; HISTORY
; March 27, 2003: [LM] adj_lon,  step default set to 1.0,  cmlong,
;                 adjustment, function:nearest_gridcell, 
;
; April 2,  2003  [GdT] set grid step default to 0.2
;                       added keyword rot_pangle and code to rotate for p angle
;                       changed Sun-Earth distance to dd=1.0/sin(sunr)
;
;
; April 4, 2003  [LM]  added if statement to find C library in case I'm runing
;                      at NOAA
;
; April 17, 2003 added FLOAT(rmax) b/f call to C program
;                added LONG(minpix)
;                added FLOAT(bkg)
;                (To make sure values get passed correctly to C program)
;
;-=============================================================================

pro remap_v2, listfile, soho=soho, stepsize=stepsize, lat=lat,	$
	   rmax=rmax, minpix=minpix, bkg=bkg, rot_pangle=rot_pangle

; IDL> remap, 'list.helium', step=1.0d, /rot


; new March 27, 2003:  adj_lon,  step default set to 1.0,  cmlong,
; adjustment, function:nearest_gridcell, 

T = SYSTIME(1)

;------------------------------------------------
;  Check keywords, if not set,Set default values
;------------------------------------------------
IF (NOT KEYWORD_SET (rmax))	THEN rmax   = 0.995
IF (NOT KEYWORD_SET (minpix))	THEN minpix = 3L
IF (NOT KEYWORD_SET (bkg))	THEN bkg    = -99999.


minpix = LONG (minpix)

;---------------------
; Find if soho or not
;---------------------
if not keyword_set(soho) then soho = 0

;---------------------
; 1st longitude value 
;---------------------
lonstart = -90.D

;---------------------------------
; Grid step size 0, 0.2, 0.4, ...
;---------------------------------
if not keyword_set(stepsize) then stepsize = 0.2d0

;------------------------------------
; Find number of images in file list
;------------------------------------
file = ' '
num = 0l

openr, lun, listfile, /get_lun
while (not eof (lun))  do begin 
       readf, lun, file
       num = num + 1
endwhile                       

	
;-----------------------------------
; Define new arrays in for stepsize 
;-----------------------------------

newsize = FIX (180.d0 / double (stepsize) + 0.5)  ; new april 4, 2003
new_image = fltarr(newsize,newsize) ; was double

;--------------------------------------------------------
; This is in equal steps of stepsize degrees in longitude
;--------------------------------------------------------
genarr = dindgen(newsize,newsize) mod (newsize)

;------------------------------------------------
; Use double(stepsize)*0.5d0 to get cell centers
; Use lonstart to shift it -90. degrees
;------------------------------------------------
newlon = genarr*double(stepsize) + lonStart + double(stepsize)*0.5d0


;---------------------------------------------------------------
; convert to radians - keep in degrees to add adjustment later!!
;---------------------------------------------------------------
;newlon = temporary(newlon)/!radeg

;---------------
; Set latitude
;---------------
if keyword_set(lat) then begin

    ;-----------------------------------------------------------------------
    ; Define latitude in equal steps in latitude if  if "lat" keyword is set.
    ; old equation : newlat= transpose(dindgen(901,901)mod(901)/5. ) -90
    ;-----------------------------------------------------------------------
    ;newlat = transpose(genarr*double(step)) - 90.d0 + double(step)*0.5d0
    ;newlat=  temporary(newlat)/!radeg

    ;-------------------------------------------------------------------
    ; this is in equal steps of stepsize degrees in latitude in radians
    ;-------------------------------------------------------------------
    newlat = transpose(newlon)
    newlat = temporary(newlat)/!radeg

endif else begin

    l = transpose(genarr - newsize*0.5d0 + 0.5d0)/(newsize*0.5d0) 
    newlat =  asin(l)

endelse ; lat 

   
;------------------
; Rewind file list 
;------------------
point_lun, lun, 0

file = ' '

;----------------------------------------------------
; Read images and convert to heliographic coordinates
;----------------------------------------------------

FOR i = 0, num-1 do begin 

     readf, lun, file
     print, 'file = ', file

     image = readfits (file, hdu)
     image = float(image)

     ;---------------
     ; Read extension
     ;---------------
     if (not soho)  then temp  = readfits (file, exten,/exten)

   
     xsize = (size(image))[1]
     ysize = (size(image))[2]

     if (xsize ne ysize) then print, '!!!!!!!! WARNING: IMAGE NOT SQUARE  !!!!!!!!!' 
 
     ;------------------------
     ;Read header information
     ;------------------------

     get_date_time, soho, hdu, $ ; inputs
                    date, t_ave  ; output
  
     ;---------------
     ;Find julian day 
     ;---------------
     jd = anytim2jd(date)

     ;--------------------------------------------------
     ; Get carrington rotation # and central meridian L0
     ;--------------------------------------------------
     jul_day = double(jd.int) + jd.frac
     jd_carr, jul_day, carr, L0

     ;-----------------------------------
     ; Find nearest center gridcell to L0
     ;-----------------------------------
     cmlong = nearest_gridcell(L0, stepsize)

     adjustment = cmlong - L0
     print,'L0, cmlong, adjustment = ',L0, cmlong, adjustment

     ;-----------------------------
     ; Adjust longitude grid center
     ; Convert to radians
     ;-----------------------------
     adj_lon = newlon + adjustment

     adj_lon = temporary(adj_lon)/!radeg

     
     ;--------------------------------
     ; Get center of map, arcsec/pixel
     ;--------------------------------
     xcen   =  fxpar (hdu, 'CRPIX1')      ; x center in pixel 
     ycen   =  fxpar (hdu, 'CRPIX2')      ; y center in pixel 
     dx     =  fxpar (hdu, 'CDELT1')      ; arcsec/pixel
     dy     =  fxpar (hdu, 'CDELT2')      ; arcsec/pixel

     ;-------------------------
     ;find time of observations
     ;-------------------------
     obs_time = utc2tai(anytim2utc(date))
     

     ;-------------------------------------------------------------------
     ; Find solar p,b angles and radius at time of the image using pb0r.
     ; pb0r has enough accuracy and it is is more reliable than the values 
     ; in the header for MLSO data 
     ;-------------------------------------------------------------------

     angles = pb0r(obs_time, soho=soho)
     
     ;p_ang = angles(0)
     ;b_ang = angles(1)

     ;-----------------------------------------------
     ; angles(2) = radius in arcmin
     ; convert to arcsec and divide by arcsec/pixel
     ; to get size of radius in terms of pixel size
     ;-----------------------------------------------
     sunr_min = angles(2)                   ; sun radius in arcmin
     radius_pix = sunr_min*60.d/float(dx)  ; about 420 pixels from center, step =.2

     ;---------------------------------
     ; Compute p and B angle in radians
     ;---------------------------------
     b0 = angles(1)/!radeg
     p_angle = angles(0)/!radeg

     ;--------------------------------
     ; Compute sun's radius in radians
     ;--------------------------------
     sunr = sunr_min / 60.d0 / !radeg
      
     ;---------------------------------------------------
     ; fix solar central meridian longitude for MLSO data
     ;---------------------------------------------------
     IF (not soho) then begin 

        solar_l0 = fxpar (hdu, 'SOLAR_L0')
        if (solar_l0 lt 0) then solar_l0 = solar_l0 + 360.
  
     ENDIF

     ;-----------------------------------------------
     ; find pixels within and outside the solar disk
     ;----------------------------------------------
     disk_edge = double (rmax)


     ;----------------------------------------------------------
     ; define coordinates for interpolation on the disk =
     ; regular array centered at image center in units of radius 
     ;----------------------------------------------------------
     genarr = dindgen(xsize,ysize)mod(xsize)
    
     xx = genarr - xcen 
     yy = transpose(dindgen(ysize,xsize)mod(ysize) ) - ycen 

     xx = xx / radius_pix
     yy = yy / radius_pix

     ;---------------------------------------
     ; set off-disk values on image to zero
     ;---------------------------------------
     zero_offdisk_v2, disk_edge, xx, yy, bkg, image


     ;---------------------------------------------------------------
     ; Transform sin(latitude) (newlat) and longitude (newlon) in 
     ; new x and y coordinates on the disk.
     ;---------------------------------------------------------------

     ;---------------------------------------------------------------
     ; First find 3D cartesian coordinates.
     ;--------------------------------------------------------------- 

     newzz  = sin(newlat)
     newyy  = cos(newlat) * sin(adj_lon)
     newxx  = cos(newlat) * cos(adj_lon)

     ;----------------------------------------------------------------
     ; Account for the B angle 
     ; Rotate around the y-axis by -b0 to introduce b angle
     ;----------------------------------------------------------------

     newbxx  =  COS(b0)*newxx + SIN(b0)*newzz
     newbyy  =  newyy
     newbzz  =  COS(b0)*newzz - SIN(b0)*newxx 


     ;----------------------------------------------------------------
     ; Account for the p angle 
     ; If the disk image has not been corrected for the p angle, 
     ; rotate by -p angle around x axis to introduce the p angle
     ;----------------------------------------------------------------

     IF keyword_set(rot_pangle) THEN BEGIN 

       newxx  =  newbxx
       newyy  =  COS(p_angle)*newbyy - SIN(p_angle)*newbzz
       newzz  =  COS(p_angle)*newbzz + SIN(p_angle)*newbyy 
   
     ENDIF ELSE BEGIN

       newxx  =  newbxx
       newyy  =  newbyy
       newzz  =  newbzz

     ENDELSE
     
    ;----------------------------------------------------------------
    ;  Find the disk coordinates 
    ;  Transform from the 3D coord. on the sphere to the 2D coord. on the disk
    ;  y-axis becomes x-axis (east-west)
    ;  z-axis becomes y-axis (north-south) 
    ;  x-axis in the Sun-Earth direction 
    ;----------------------------------------------------------------

     dd = 1.0 / sin (sunr)       ; Sun-Earth distance
     rr =  dd - newxx            ; distance from plane of the sky ?
     newx = atan ( newyy, rr)    ; x-coord = angle whose tangent is newyy/rr
     newy = atan ( newzz, rr)    ; y-coord = angle whose tangent is newzz/rr
 
     ;----------------------------------
     ; Divide by sun's radius in radians
     ; if B0 = 0,
     ; 0.,-90. maps to 0.,-1.
     ; 0.,0.  maps to 0.,0.
     ; 0.,90. maps to 0.,1.
     ;----------------------------------

     newx = temporary (newx) / sunr
     newy = temporary (newy) / sunr

     ;--------------------------------------------------------------------------
     ; Interpolate original image onto the new coordinate grid, newx, newy,
     ; in equally spaced in latitude and longitude in stepsize degree increments
     ;--------------------------------------------------------------------------

     ; Andy's routine =============================================================

      ;------------------------------------------------------------------
      ; Make integers LONG and FLOATing point arrays DOUBLE precision.
      ;------------------------------------------------------------------

      image      = FLOAT (image)  ; changed to float april 4, 2003
      xsize      = LONG  (xsize)
      ysize      = LONG  (ysize)
      xx         = FLOAT (xx)
      yy         = FLOAT (yy)
      xcen       = FLOAT (xcen)
      ycen       = FLOAT (ycen)
      radius_pix = FLOAT (radius_pix)
      rmax       = FLOAT (rmax)  ; added april 17, 2003
      newx       = FLOAT (newx)
      newy       = FLOAT (newy)
      newxsize   = LONG  (newsize)
      newysize   = LONG  (newsize)
      minpix     = LONG  (minpix)  ; added april 17, 2003
      bkg        = FLOAT (bkg)     ; added april 17, 2003

      ;------------------------------------------------------------------
      ; Convert angle coordinates to pixel coordinates for interpolation.
      ;------------------------------------------------------------------

      xslope = (MAX (xx) - MIN (xx)) / (xsize - 1)
      yslope = (MAX (yy) - MIN (yy)) / (ysize - 1)
      xint = (newx - MIN (xx)) / xslope
      yint = (newy - MIN (yy)) / yslope

      xslope = FLOAT (xslope)
      yslope = FLOAT (yslope)
      xint   = FLOAT (xint)
      yint   = FLOAT (yint)

      PRINT, 'newxsize/newysize:', newxsize, newysize
      PRINT, 'rmax:             ', rmax
      PRINT, 'minpix:           ', minpix
      PRINT, 'bkg:              ', bkg
      PRINT, 'xslope/yslope: ', xslope, yslope

      PRINT, ''
      PRINT, 'size (xint):      ', size (xint)
      PRINT, 'size (yint):      ', size (yint)
      PRINT, ''
      PRINT, 'xsize/ysize:      ', xsize, ysize
      PRINT, 'size (xx):        ', size (xx)
      PRINT, 'size (yy):        ', size (yy)
      PRINT, 'newxsize/newysize:', newxsize, newysize
      PRINT, 'size (newx):      ', size (newx)
      PRINT, 'size (newy):      ', size (newy)
      PRINT, ''
      PRINT, 'size (image):     ', size (image)
      PRINT, 'size (xsize):     ', size (xsize)
      PRINT, 'size (ysize):     ', size (ysize)
      PRINT, 'size (xcen):      ', size (xcen)
      PRINT, 'size (ycen):      ', size (ycen)
      PRINT, 'size (radius_pix):', size (radius_pix)
      PRINT, 'size (new_image): ', size (new_image)
      PRINT, 'size (newx):      ', size (newx)
      PRINT, 'size (newy):      ', size (newy)
      PRINT, 'size (newxsize):  ', size (newxsize)
      PRINT, 'size (newysize):  ', size (newysize)
      PRINT, 'size (minpix):    ', size (minpix)
      PRINT, 'size (bkg):       ', size (bkg)


      ;------------------------------------------------------------------
      ; Call C-language routine to do 2D weighted interpolation.
      ;------------------------------------------------------------------
      cobj = 'fwint2d2'

      ;--------------------------------------------------
      ; Check to see what computer we're on to set clib 
      ;--------------------------------------------------
      hostname = getenv('HOSTNAME'); get hostname environment variable

      If (hostname eq 'lmayerlx.sec.noaa.gov') then begin

          clib = '/home/lmayer/HAO/interp/C_interp/libfwint2d.so'

      endif ELSE begin  ; assuming we're on an HAO machine

        IF (!version.os EQ 'linux') THEN		        $
	   clib = 'home/stanger/lib/linux/libfwint2d2.so'	$
        ELSE IF (!version.os EQ 'IRIX') THEN			$
	   clib = '/home/stanger/lib/sgi/libfwint2d2.so'	$
        ELSE							$
	  BEGIN
	  PRINT, 'Only linux & IRIX are currently supported.'
	  RETURN
	  END

      ENDELSE


      window, xs=xsize, ys=ysize, retain=2
      tvscl, image
	
      iext = CALL_EXTERNAL (clib, cobj,					$
			    image, xsize, ysize, xcen, ycen,		$
			    radius_pix, Rmax,				$
			    new_image, xint, yint, newxsize, newxsize,	$
			    minpix, bkg)

;      PRINT, 'iext: ', iext
;      PRINT, 'size (new_image): ', size (new_image)

      imin = MIN (new_image, max=imax)
      PRINT, 'imin/imax: ', imin, imax

      window, xs=newxsize, ys=newysize, retain=2
      tvscl, new_image
   
     ;------------------------------------------------------------------------
     ; Andy this is where I call the bilinear interpolation 
     ; this is a solarsoft routine that calls the IDL routine interpolate.pro
     ; I suspect interpolate.pro calls or is a c routine
     ;------------------------------------------------------------------------   
     ;new_image = interp2d(image, xx, yy, newx, newy, missing=-9999.99)  


     ;------------------------------------------
     ; Trim data near the limb to avoid bad data.
     ; I do not think we need this anymore    
     ;------------------------------------------

     ;new_edge = double (rmax) ; new april 4, 2003
     ;zero_offdisk_v2, new_edge, newx, newy, new_image
 
    ;-----------------------------------------------------------------
    ; add keywords about carrington totations and time of observation
    ; find carrington number and longitude for central meridian 
    ;-----------------------------------------------------------------
  
      fxaddpar, hdu, 'COORDNAM', 'HELIOCENTRIC', ' SOLAR NORTH ALIGNED WITH +Y AXIS'
      fxaddpar, hdu, 'OBS_L0',  L0, ' SOLAR LONGITUDE L0' 
      fxaddpar, hdu, 'OBS_CR',  carr, ' CARRINGTON ROTATION'
      ;---------------------------------
      ; write out adjusted center of map
      ;---------------------------------
      fxaddpar, hdu, 'CMLONG',  cmlong, ' ADJUSTED CENTRAL MERIDIAN' 
      fxaddpar, hdu, 'T_REC', t_ave, 'AVERAGE TIME OF IMAGE'
      if (not soho) then $
         fxaddpar, hdu, 'SOLAR_L0',solar_l0, ' SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE'


   ;----------------------------------
   ; write new square images in files 
   ;----------------------------------

   if (not soho) then begin

       ;flen = strlen(file)-3
       newfile = strmid(file, 0, strlen(file)-3) +'remap2.fts'
       writefits, newfile, new_image, hdu
       writefits, newfile, temp , exten, /append

   endif else begin
   
       newfile = strmid(file, 0, 23) +'remap.fts' 
       writefits, newfile, new_image, hdu

   endelse

ENDFOR

close, lun
free_lun, lun
close, /all

PRINT, SYSTIME(1) - T, 'Seconds'


END

