;+
; lowkmat.pro
;
; Routine to produce the LowK X matrix
; from picscal data in FITS file format.
;
;	Data are created when the MLSO observers run a
;	Special observation with the Picscal file linked
;	to Special.  Data are stored to disk on nene.mtn.hawaii.gov.
;
;	Input are the picscal0/1 files with extensions of 1 ... 10
;	1 = opal only
;	2,3,...,9 = opal + polarizer at 315, 270, ... 45, 0  degrees.
;	10 is a corona image
;	picscal0 files are modulation states 1 and 3.
;	picscal1 files are modulation states 2 and 4.
;
;	Calibration is similar to that for MkIII described in
;	tech note, but needs its own technote to fully explain
;	the use of a liquid crystal variable retarder as a 
;	polarization modulator.
;
;	For each array position,
;
;	A bias is subtracted from each image.
;	A "dark" region of the frame is used to infer the bias.
;	If instrument alignment changes so too may the location
;	of this dark region.
;
;	Here is a brief description of the calibration technique.
;	t = Xs where t is the measured stokes vector, X is the
;	polarimeter response matrix, and s in the coronal input
;	stokes vector.  Only a 3 element Stokes vector is used
;	as circular polarization is thought to be insignificant.
;	s = X-1 t
;	The inverse polarimeter response matrix X-1 acting upon
;	the measured vector gives the input stokes vector.
;	The t vector is created from linear combinations of the
;	images recorded at the four modulation states.
;	I = ( m1 + m2 + m3 + m4)/4.
;	Q = ( m1 + m2 - m3 - m4 )/4.
;	U = ( m1 - m2 - m3 + m4 )/4.
;	Where m1, m4 are the images taken at modulation states
;	1 through 4.  This is done independently for the P(top) 
;	and S (bottom) beams.
;	Measurements of the opal alone are used to determine the
;	gain for the P and S beams.
;	The amount of Q and U seen for the opal only gives the
;	X21 and X31 elements.  X11 is normalized by the gain to 1.
;	Opal + linear polarizer at 0 + 180 - 90 - 270 are used
;	to compute the Q to I, Q, and U column, X12, X22, and X32.
;	Opal  + linear polarizer at 45 + 225 - 135 - 315 are used
;	to compute the U to I, Q, and U column, X13, X23, and X33
;
;	Once X is computed for P and for S, the inverse X matrix at 
;	each field position is computed and stored in a file
;	to be picked up be the calibration routine picscal.pro.
;
;	Output is:
;	pxinv	Inverse polarimeter response matrix for "p" beam
;	sxinv	Inverse polarimeter response matrix for "s" beam
;	gainp	Gain for "p" beam
;	gains	Gain for "s" beam
;
;	David Elmore
;	1/5/99
;-

PRO lowkmat

printem = 0

;--- Constant: transmission of the calibration opal

Bopal = 1.0e-5

window, 0, xsize=768, ysize=768

file0 = 'picscal0.x'
file1 = 'picscal1.x'
 
;--- Define the polarizer positions

opal   = 0
pol0   = 1
pol45  = 8
pol90  = 7
pol135 = 6
pol180 = 5
pol225 = 4
pol270 = 3
pol315 = 2

LOADCT, 0		/* Load B/W color table. */

xpix = 767
ypix = 2060
xsiz = xpix
ysiz = ypix / 4

;--- Where to find frame bias.
;--- This should be checked.  As of 7/1/99 this was OK. DFE

bxmin = xpix - 6
bxmax = xpix - 1
bymin = ypix / 4
bymax = ypix / 2 -1
btot = (bxmax-bxmin) * (bymax-bymin)

shortimg = INTARR (xpix, ypix)
hdr      = INTARR (xpix)
img      = FLTARR (xpix, ypix)

;--- Element 0 is opal only.
;--- Polarizers are 1 - 8.

s1 = FLTARR (9, xsiz, ysiz)
p1 = FLTARR (9, xsiz, ysiz)
s2 = FLTARR (9, xsiz, ysiz)
p2 = FLTARR (9, xsiz, ysiz)
s3 = FLTARR (9, xsiz, ysiz)
p3 = FLTARR (9, xsiz, ysiz)
s4 = FLTARR (9, xsiz, ysiz)
p4 = FLTARR (9, xsiz, ysiz)

x = INTARR (2)
y = INTARR (2)

xdim = xsiz
ydim = ysiz
x0 = 1.0 * xsiz / 2.0
y0 = 1.0 * ysiz / 2.0

;--- Offsets lower half of image vs upper half.
;--- The two beams are called s and p.

;--- The polarizers are      0,  45,  90,  ...
;--- corresponding to files  2,   3,   4, ...

for i = 1, 9 do begin ;{
	pol = i - 1

	file0 = STRING ( 'picscal0.', i, format='(a9, i1)' + '.fts' )
	PRINT, file0
;        CLOSE, 1 & OPENR, 1, file0 & READU, 1, shortimg
	shortimg = readfits (file0, hdu0)

;--- Remove header from processing.
;--- Remove bias.
;--- Normalize by exposure time.

;        hdr = shortimg (*, 0)
;        exposure = FLOAT (hdr(21))

	exposure = fxpar (hdu0, 'EXPDUR')
	img = FLOAT (shortimg)
	img (*, 0)        = img (*, 2)
	img (*, 1)        = img (*, 2)
	img (*, ypix/2)   = img (*, ypix/2+2)
	img (*, ypix/2+1) = img (*, ypix/2+2)
	bias = TOTAL (img (bxmin:bxmax, bymin:bymax)) / btot
	img = img - bias
	img = img / exposure

	p1 (pol, *, *) = img (0:766, ysiz:2*ysiz-1)
	s1 (pol, *, *) = img (0:766, 0:ysiz-1)
	p3 (pol, *, *) = img (0:766, 3*ysiz:4*ysiz-1)
	s3 (pol, *, *) = img (0:766, 2*ysiz:3*ysiz-1)

	file1 = STRING ( 'picscal1.', i, FORMAT='(a9, i1)' + '.fts' )
	PRINT, file1
;        CLOSE, 1 & OPENR, 1, file1 & READU, 1, shortimg
	shortimg = readfits (file1, hdu1)

;--- Remove header from processing.
;--- Remove bias.
;--- Normalize by exposure time.

;        hdr = shortimg (*, 0)
;        exposure = FLOAT (hdr(21))

	exposure = fxpar (hdu1, 'EXPDUR')
	img = FLOAT (shortimg)
	img (*, 0)        = img (*, 2)
	img (*, 1)        = img (*, 2)
	img (*, ypix/2)   = img (*, ypix/2+2)
	img (*, ypix/2+1) = img (*, ypix/2+2)
	bias = TOTAL (img (bxmin:bxmax, bymin:bymax)) / btot
PRINT, "Bias value = ", bias
	img = img - bias
	img = img / exposure

	p2 (pol, *, *) = img (0:766, ysiz:2*ysiz-1)
	s2 (pol, *, *) = img (0:766, 0:ysiz-1)
	p4 (pol, *, *) = img (0:766, 3*ysiz:4*ysiz-1)
	s4 (pol, *, *) = img (0:766, 2*ysiz:3*ysiz-1)

end ;}

;--- Create measured stokes parameters.

	pi = ( p1 + p2 + p3 + p4 ) / 4.0
	si = ( s1 + s2 + s3 + s4 ) / 4.0
	pq = ( p1 + p2 - p3 - p4 ) / 4.0
	sq = ( s1 + s2 - s3 - s4 ) / 4.0
	pu = ( p1 - p2 - p3 + p4 ) / 4.0
	su = ( s1 - s2 - s3 + s4 ) / 4.0

;--- The first column is just what is seen.

px11 = 1.0 * pi (opal, *, *)
px11 = REFORM (px11)
sx11 = 1.0 * si (opal, *, *)
sx11 = REFORM (sx11)

px21 = 1.0 * pq (opal, *, *)
px21 = REFORM (px21)
sx21 = 1.0 * sq (opal, *, *)
sx21 = REFORM (sx21)

px31 = 1.0 * pu (opal, *, *)
px31 = REFORM (px31)
sx31 = 1.0 * su (opal, *, *)
sx31 = REFORM (sx31)

;--- The transmission of the polarizer is needed;
;--- therefore, average of polarizer transmissions is used.

stp = 1.0 * si (pol0,   *, *) + 1.0 * si (pol45,  *, *) + $
      1.0 * si (pol90,  *, *) + 1.0 * si (pol135, *, *) + $
      1.0 * si (pol180, *, *) + 1.0 * si (pol225, *, *) + $
      1.0 * si (pol270, *, *) + 1.0 * si (pol315, *, *)
stp = stp / (8.0 * si (opal, *, *))
stp = REFORM (stp)

tv, bytscl (stp, 0, 2)

ptp = 1.0 * pi (pol0,   *, *) + 1.0 * pi (pol45,  *, *) + $
      1.0 * pi (pol90,  *, *) + 1.0 * pi (pol135, *, *) + $
      1.0 * pi (pol180, *, *) + 1.0 * pi (pol225, *, *) + $
      1.0 * pi (pol270, *, *) + 1.0 * pi (pol315, *, *)
ptp = ptp / (8.0 * pi (opal, *, *))
ptp = REFORM (ptp)

tv, bytscl (ptp, 0, 2)


;--- Second column Q > I,  Q,  and U.

sx12 = 1.0 * (si (pol0, *, *) - si (pol90, *, *) + $
	      si (pol180, *, *) - si (pol270, *, *)) / ( 4.0 * stp )
sx12 = REFORM (sx12)
px12 = 1.0 * (pi (pol0, *, *) - pi (pol90, *, *) + $
	      pi (pol180, *, *) - pi (pol270, *, *)) / ( 2.0 * ptp )
px12 = REFORM (px12)

sx22 = 1.0 * (sq (pol0, *, *) - sq (pol90, *, *) + $
	      sq (pol180, *, *) - sq (pol270, *, *)) / ( 4.0 * stp )
sx22 = REFORM (sx22)
px22 = 1.0 * (pq (pol0, *, *) - pq (pol90, *, *) + $
	      pq (pol180, *, *) - pq (pol270, *, *)) / ( 4.0 * ptp )
px22 = REFORM (px22)

sx32 = 1.0 * (su (pol0, *, *) - su (pol90, *, *) + $
	      su (pol180, *, *) - su (pol270, *, *)) / ( 4.0 * stp )
sx32 = REFORM (sx32)
px32 = 1.0 * (pu (pol0, *, *) - pu (pol90, *, *) + $
	      pu (pol180, *, *) - pu (pol270, *, *)) / ( 4.0 * ptp )
px32 = REFORM (px32)


;--- Third column U > I,  Q,  and U.

sx13 = 1.0 * (si (pol45, *, *) - si (pol135, *, *) + $
	      si (pol225, *, *) - si (pol315, *, *)) / ( 4.0 * stp )
sx13 = REFORM (sx13)
px13 = 1.0 * (pi (pol45, *, *) + pi (pol135, *, *) + $
	      pi (pol225, *, *) - pi (pol315, *, *)) / ( 4.0 * ptp )
px13 = REFORM (px13)

sx23 = 1.0 * (sq (pol45, *, *) - sq (pol135, *, *) + $
	      sq (pol225, *, *) - sq (pol315, *, *)) / ( 4.0 * stp )
sx23 = REFORM (sx23)
px23 = 1.0 * (pq (pol45, *, *) - pq (pol135, *, *) + $
	      pq (pol225, *, *) - pq (pol315, *, *)) / ( 4.0 * ptp )
px23 = REFORM (px23)

sx33 = 1.0 * (su (pol45, *, *) - su (pol135, *, *) + $
	      su (pol225, *, *) - su (pol315, *, *)) / ( 4.0 * stp )
sx33 = REFORM (sx33)
px33 = 1.0 * (pu (pol45, *, *) - pu (pol135, *, *) + $
	      pu (pol225, *, *) - pu (pol315, *, *)) / ( 4.0 * ptp )
px33 = REFORM (px33)


;--- Figure out where to force matrix elements to 0.

	pa = where ( pi (opal, *, *) lt 2 )
	sa = where ( si (opal, *, *) lt 2 )

;--- Now normalize...
;--- Keep x11 as the normalization factor,  it should be the unit matrix.
;--- Divide all other elements by x11.

gains = sx11
sx11 = sx11 / gains
sx21 = sx21 / gains
if ( sa (0) ne -1 ) then sx21 (sa) = 0
sx31 = sx31 / gains
if ( sa (0) ne -1 ) then sx31 (sa) = 0
sx12 = sx12 / gains
if ( sa (0) ne -1 ) then sx12 (sa) = 0
sx22 = sx22 / gains
if ( sa (0) ne -1 ) then sx22 (sa) = 0
sx32 = sx32 / gains
if ( sa (0) ne -1 ) then sx32 (sa) = 0
sx13 = sx13 / gains
if ( sa (0) ne -1 ) then sx13 (sa) = 0
sx23 = sx23 / gains
if ( sa (0) ne -1 ) then sx23 (sa) = 0
sx33 = sx33 / gains
if ( sa (0) ne -1 ) then sx33 (sa) = 0

gainp = px11 
px11 = px11 / gainp
px21 = px21 / gainp
if ( pa (0) ne -1 ) then px21 (pa) = 0
px31 = px31 / gainp
if ( pa (0) ne -1 ) then px31 (pa) = 0
px12 = px12 / gainp
if ( pa (0) ne -1 ) then px12 (pa) = 0
px22 = px22 / gainp
if ( pa (0) ne -1 ) then px22 (pa) = 0
px32 = px32 / gainp
if ( pa (0) ne -1 ) then px32 (pa) = 0
px13 = px13 / gainp
if ( pa (0) ne -1 ) then px13 (pa) = 0
px23 = px23 / gainp
if ( pa (0) ne -1 ) then px23 (pa) = 0
px33 = px33 / gainp
if ( pa (0) ne -1 ) then px33 (pa) = 0

gains = gains / Bopal
if ( sa (0) ne -1 ) then gains (sa) = max (gains (*, 258))
gainp = gainp / Bopal
if ( pa (0) ne -1 ) then gainp (pa) = max (gainp (*, 258))

p1 = 0
p2 = 0
p3 = 0
p4 = 0
s1 = 0
s2 = 0
s3 = 0
s4 = 0

px = FLTARR (3, 3, xsiz, ysiz)
sx = FLTARR (3, 3, xsiz, ysiz)

sx (0, 0, *, *) = sx11
sx (0, 1, *, *) = sx12
sx (0, 2, *, *) = sx13
sx (1, 0, *, *) = sx21
sx (1, 1, *, *) = sx22
sx (1, 2, *, *) = sx23
sx (2, 0, *, *) = sx31
sx (2, 1, *, *) = sx32
sx (2, 2, *, *) = sx33

px (0, 0, *, *) = px11
px (0, 1, *, *) = px12
px (0, 2, *, *) = px13
px (1, 0, *, *) = px21
px (1, 1, *, *) = px22
px (1, 2, *, *) = px23
px (2, 0, *, *) = px31
px (2, 1, *, *) = px32
px (2, 2, *, *) = px33

pltpicsmat, sx, px, gains, gainp
STOP

if ( printem ) then begin
	set_plot, 'ps'
	pltpicsmat, sx, px, gains, gainp
	DEVICE, /close
	SET_PLOT, 'x'
	SPAWN, 'lp -d haow idl.ps'
endif

pxinv = px
sxinv = sx

for i = 0, xsiz - 1 do begin
	for j = 0, ysiz - 1 do begin
		pxinv (*, *, i, j) = invert (px(*, *, i, j))
		sxinv (*, *, i, j) = invert (sx(*, *, i, j))
	end
	PRINT, i
end

pltpicsmat1, sxinv, pxinv, gains, gainp
if ( printem ) then begin
	set_plot, 'ps'
	pltpicsmat1, sxinv, pxinv, gains, gainp
	DEVICE, /close
	SET_PLOT, 'x'
	SPAWN, 'lp -d haow idl.ps'
endif


CLOSE, 1
OPENW, 1, 'pxinv' & WRITEU, 1, pxinv  & CLOSE, 1
OPENW, 1, 'sxinv' & WRITEU, 1, sxinv  & CLOSE, 1
OPENW, 1, 'gains' & WRITEU, 1, gains  & CLOSE, 1
OPENW, 1, 'gainp' & WRITEU, 1, gainp  & CLOSE, 1

PRINT, 'done'

STOP
END

