;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: edgefind.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 13, 1998
;
; Last Modified: Thu Feb 19 11:03:23 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO edgefind, im, minint, xc, yc, rx, ry, thresh=thresh, tv=tv, $
              err=err, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       edgefind     (procedure)
;
; PURPOSE:
;       To determine the center coordinates, the x-radius, and the
;       y-radius of a full disk solar image.
;
; CALLING SEQUENCE:
;
;       edgefind, im, minint, xc, yc, rx, ry [,thresh=thresh, tv=tv, $
;                                              err=err, help=help]
;
; INPUTS:
;       im - image of full solar disk.
;   minint - minimum solar disk intensity at the solar limb.  This
;            value is used to determine an initial guess of the image
;            geometry to set a limiting range of values/pixels to
;            check during the actual determination of the limb points.
;
; OPTIONAL INPUTS: 
;       thresh - minimum amplitude threshold to consider when
;                detecting zero-crossing points in the Laplacian image
;                which denote the solar limb edges.  (default=50.0)
;
; OUTPUTS:
;       xc - x pixel coordinate of the center of the solar disk.
;       yc - y pixel coordinate of the center of the solar disk.
;       rx - radius of the solar disk in the x dimension (pixels).
;       ry - radius of the solar disk in the y dimension (pixels).   
;
; OPTIONAL OUTPUTS:
;       err - ellipse fitting error values for each limb point fit.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;         /tv.  Will display the solar image with the limb points used
;               in the fit.
;
; CALLS:
;       xycenter.pro, linedge.pro, fitellipse.pro,
;       fitcircle.pro, delvarx.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       It is assumed that the input image is square.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written February 13, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 13, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'edgefind'
  RETURN
ENDIF

IF(N_PARAMS() NE 6) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  edgefind, im, minint, xc, yc, rx, ry'
  PRINT, '                  [,thresh=thresh, tv=tv, err=err, help=help]'
  PRINT, ''
  RETURN
ENDIF

; Set default amplitude threshold for Laplacian edge detection.

IF(n_elements(thresh) EQ 0) THEN thresh = 50.0


; Get image dimensions.

sz = size(im)

nx = sz(1) & ny = sz(2)


; Get approximate geometry using Haosheng's xycenter.pro code.

xycenter, im, minint, hxc, hyc, hr


; Create Laplacian image.

kern = [[-1., -1., -1.], [-1., 8., -1.], [-1., -1., -1.]]

lap = convol(im, kern)


; Set edges of Laplacian image to zero - remove edge effects.

dx = fix(nx/100.) & dy = fix(ny/100.)

lap(*, 0:dy) = 0.0           ; bottom

lap(0:dx, *) = 0.0           ; left

lap(nx-dx:nx-1, *) = 0.0     ; right

lap(*, ny-dy:ny-1) = 0.0     ; top


; Define arrays to store edge points found.

xpts = [-999.] & ypts = [-999.]


; Setup for progress reporting.

prog = ' '

print, 'Working:   ', format = '(a, $)'


; Loop over the middle half of the image in both the x and y
; dimensions and find the solar limb/edge.

FOR i = nx/4, 3*(nx/4) DO BEGIN
   
                                ; Process rows.
   
   line = lap(*, i)             ; Extract a row from the Laplacian image.
   
                                ; Determine the limb/edge positions.
                                ; Use three point smoothing to remove
                                ; "false" zero-crossings.
   
   linedge, smooth(line, 3), thresh, xa, xb
   
                                ; If left edge found, then check and record it.
   IF(xa NE -1) THEN BEGIN
                                ; Determine edge points approximate
                                ; radial position from disk center.
      
      rr = sqrt( (xa-hxc)^2 + (i-hyc)^2 )
      
                                ; If approx. radial position in within
                                ; +/- 10 pixels of the radius found by
                                ; Haosheng's code, then store the
                                ; position.
      
      IF(abs(rr-hr) LE 10) THEN BEGIN
         xpts = [xpts, xa]
         ypts = [ypts, i]
      ENDIF
   ENDIF
   
                                ; Check and record the other edge in
                                ; the extracted row.
   
   IF(xb NE -1) THEN BEGIN
      rr = sqrt( (xb-hxc)^2 + (i-hyc)^2 )
      IF(abs(rr-hr) LE 10) THEN BEGIN
         xpts = [xpts, xb]
         ypts = [ypts, i]
      ENDIF
   ENDIF
   
                                ; Process columns in the same fashion
                                ; as the rows.
   
   line = lap(i, *)
   linedge, smooth(line, 3), thresh, xa, xb
   IF(xa NE -1) THEN BEGIN
      rr = sqrt( (i-hxc)^2 + (xa-hyc)^2 )
      IF(abs(rr-hr) LE 10) THEN BEGIN
         xpts = [xpts, i]
         ypts = [ypts, xa]
      ENDIF
   ENDIF
   
   IF(xb NE -1) THEN BEGIN
      rr = sqrt( (i-hxc)^2 + (xb-hyc)^2 )
      IF(abs(rr-hr) LE 10) THEN BEGIN
         xpts = [xpts, i]
         ypts = [ypts, xb]
      ENDIF
   ENDIF
   
ENDFOR


; Disregard the array initialization points.

xpts = xpts(where(xpts NE -999))
ypts = ypts(where(ypts NE -999))


; Least squares fit these 8 points to an ellipse.

; Display the limb/edge points found, if requested.

IF(keyword_set(tv)) THEN BEGIN
   
   set_plot, 'Z'
   device, set_resolution = [nx, ny], set_colors = 256
   tv, bytscl(im)
   FOR i = 0, n_elements(xpts)-1 DO plots,[xpts(i),xpts(i)], $
      [ypts(i),ypts(i)], /dev, color = 255, psym = 1
   a = tvrd()
   device,/close
   set_plot, 'X'
   a = rebin(a, 512, 512)
   tvscl, a
   
ENDIF


; Transpose the limb/edge points to column vectors as required by the
; LSQ ellipse fitting procedure.

xpts = transpose(xpts) & ypts = transpose(ypts)


; Perform a least-squares fit of the limb points to an ellipse.

fitellipse, xpts, ypts, coeff, xc, yc, rx, ry, phi, err = err



; Free the Laplacian image from memory.

delvarx, lap, line


; For comparison, perform a least-squares fit of the limb points to a
; circle.

fitcircle, xpts, ypts, cx, cy, cr


; Print the results to the screen.

print, ''
print, 'XC', 'YC', 'RX', 'RY', 'PHI', 'Ellipse Fit:', xc, yc, rx, $
   ry, phi, format = '(14x, a, 6x, a, 7x, a, 6x, a, 6x, a, /, a, 1x,' + $
   'f7.2, 1x, f7.2, 1x, f7.2, 1x, f7.2, 1x, f7.2)'

print, ' Circle Fit:', cx, cy, cr, $
   format = '(/, a, 1x, f7.2, 1x, f7.2, 1x, f7.2)'

print, ''
print, n_elements(xpts), ' Points fit.'
print, ''

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'edgefind.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
