;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: flatten.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, October 9, 1998
;
; Last Modified: 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO flatten2, fname, rname, help=help
;+
;
; PROJECT:
;       SunRISE/PSPT
;
; NAME:
;       flatten     (procedure)
;
; PURPOSE:
;       To flatten raw PSPT images and remove the linear gradient introduced
;       by the flat field calculation.
;
; CALLING SEQUENCE:
;
;       flatten, fname, rname [,help=help]
;
; INPUTS:
;       fname - flat field file name
;       rname - raw image file name
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       Writes flattened image to disk in file named rname.rdc.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;
; CALLS:
;       delvarx.pro, find_minint.pro, xycenter.pro, lsqcenter.pro,
;       readfits.pro, writefits.pro, minmax.pro, gradient.pro, sxaddhist.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written October 9, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, October 9, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'flatten'
  RETURN
ENDIF

IF(N_PARAMS() NE 2) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  flatten, fname, rname [,help=help]'
  PRINT, ''
  RETURN
ENDIF

; Read in flat and raw images.

flat = readfits(fname)

im = readfits(rname, h)


; Divide raw image by flat.

mm = minmax(flat)

IF(mm(0) LE 0) THEN im = im/(flat > 0.01) ELSE im = im/flat

delvarx, flat

; Remove linear gradient.

limb_cut = find_minint(im)

xycenter, im, limb_cut, xc, yc
ss=lsqcenter(im, xc, yc, 0.2, 2048, imga, imgr)
im = im/gradient(ss)

ss=lsqcenter(im, xc, yc, 0.2, 2048, imga, imgr)
im = im/gradient(ss)

delvarx, imga, imgr

today = systime(0)

im = reverse(im, 2)

sxaddhist, 'Image flipped N-S ' + today, h

sxaddhist, 'Flat fielded ' + today + ' with ' + fname, h


; Save flattened image.

writefits, rname+'.rdc', im, h


RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'flatten.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
