;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: linedge.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 17, 1998
;
; Last Modified: Thu Feb 19 10:05:38 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO linedge, vec, thresh, e1, e2, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       linedge     (procedure)
;
; PURPOSE:
;       To find first and last zero crossings in a vector from a Laplacian
;       filtered image, denoting the position of the edges, based on an input
;       amplitude threshold limit.
;
; CALLING SEQUENCE:
;
;       linedge, vec, thresh, e1, e2 [,help=help]
;
; INPUTS:
;       vec - single vector (1D array) from a Laplacian image.
;    thresh - amplitude threshold to use to determine the location of the
;             edges.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       e1 - interpolated index of vec of the first edge/zero crossing of the
;            Laplacian vector.
;
;       e2 - interpolated index of vec of the last edge/zero crossing of the
;            Laplacian vector.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;
; CALLS:
;       lrminmax.pro, rtspline.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written February 17, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 17, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'linedge'
  RETURN
ENDIF

IF(N_PARAMS() NE 4) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  linedge, vec, thresh, e1, e2 [,help=help]'
  PRINT, ''
  RETURN
ENDIF

x = vec                         ; Copy input vector.

nx = n_elements(x)-1


; Set default values to return.

e1 = -1.0 & e2 = -1.0


; Search for the first edge in the input vector.

found = 0 & i = 0               ; Define edge finding flag and index.


WHILE NOT found DO BEGIN
   
   IF(i LE nx-1) THEN BEGIN     ; Check for edge.
      
      a = x(i) & b = x(i+1)     ; Check sequential indicies for 0-crossing.
      
      IF(a*b LT 0) THEN BEGIN   ; If 0-crossing, then interpolate.
         
         
                                ; Find local min and max around 0-crossing.
         
         IF(a GT 0) THEN BEGIN
            
            ll = lrminmax(x, i, dir = -1, mm = 1)
            
            rr = lrminmax(x, i, dir = 1, mm = -1)
            
         ENDIF ELSE BEGIN
            
            ll = lrminmax(x, i, dir = -1, mm = -1)
            
            rr = lrminmax(x, i, dir = 1, mm = 1)
            
         ENDELSE
         
                                ; If amplitude of local min/max around
                                ; 0-crossing greater than input threshold,
                                ; then interpolate zero-crossing index.
         
         IF(abs(x(rr)-x(ll)) GE thresh AND abs(x(rr)) GE thresh/4.0 $
            AND abs(x(ll)) GE thresh/4.0) THEN BEGIN
            
            found = 1
            
            xx = findgen(rr-ll+1)+ll
            
            yy = x(ll:rr)
            
                                ; Use simple search/spline method to determine
                                ; zero-crossing.
            
            e1 = rtspline(xx, yy, float(ll), float(rr))
            
         ENDIF
         
      ENDIF
      
   ENDIF
   
   i = i+1
   
ENDWHILE


; Now use the same method to find the edge/zero-crossing for the other side of
; the vector.  Reverse the input vector and use the same steps as above to
; locate the zero-crossing of the Laplacian, remembering to subtract indicies
; properly to obtain the correct index for the "right side" edge.

x = reverse(vec)

found = 0 & i = 0

WHILE NOT found DO BEGIN
   
   IF(i LE nx-1) THEN BEGIN
      
      a = x(i) & b = x(i+1)
      
      IF(a*b LT 0) THEN BEGIN
         
         IF(a GT 0) THEN BEGIN
            
            ll = lrminmax(x, i, dir = -1, mm = 1)
            
            rr = lrminmax(x, i, dir = 1, mm = -1)
            
         ENDIF ELSE BEGIN
            
            ll = lrminmax(x, i, dir = -1, mm = -1)
            
            rr = lrminmax(x, i, dir = 1, mm = 1)
            
         ENDELSE
         
         IF(abs(x(rr)-x(ll)) GE thresh AND abs(x(rr)) GE thresh/4.0 $
            AND abs(x(ll)) GE thresh/4.0) THEN BEGIN
            
            found = 1
            
            xx = findgen(rr-ll+1)+ll
            
            yy = x(ll:rr)
            
            e2 = float(nx)-rtspline(xx, yy, float(ll), float(rr))
            
         ENDIF
         
      ENDIF
      
   ENDIF
   
   i = i+1
   
ENDWHILE


RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'linedge.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
