;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: lrminmax.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 17, 1998
;
; Last Modified: Thu Feb 19 10:08:57 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION lrminmax, x, idx, direction = direction, mm = mm, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       lrminmax     (function)
;
; PURPOSE:
;       To find a local minimum or maximum in a vector, given a starting index.
;
; CALLING SEQUENCE:
;
;       result = lrminmax(x, idx, direction=direction, mm=mm [,help=help])
;
; INPUTS:
;       x - vector for which to find the local extrema.
;     idx - vector index of where to begin the search.
;
; direction - set to a value of 1 to search to the right of the starting
;             index, set to a value of -1 to search to the left of the
;             starting index.
;   
;      mm - set to a value of 1 to search for a local maximum, set to a value
;           of -1 to search for a local minimum.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       Returns the index of the input vector where the local extrema was
;       found.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;
; CALLS:
;       None.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       SunRISE/PSPT general use.
;
; PREVIOUS HISTORY:
;       Written February 17, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 17, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'lrminmax'
  RETURN, -999
ENDIF

IF(N_PARAMS() NE 2) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  result = lrminmax(x, idx, direction=direction, mm=mm' + $
     ' [,help=help])'
  PRINT, ''
  RETURN, -999
ENDIF

; Housekeeping of inputs.

; direction:  1=right, -1=left

IF(abs(direction) NE 1) THEN BEGIN
   
   print, '%LRMINMAX:  Direction keyword must be specified as either'
   print, '            right (1) or left (-1).
   return, -1
   
ENDIF

; mm:  1=max, -1=min

IF(abs(mm) NE 1) THEN BEGIN
   
   print, '%LRMINMAX:  MM keyword must be specified to fine either the'
   print, '            maximum (1) or minimum (-1).'
   return, -1
   
ENDIF


; Set default index value to return.

rtn = -1

nx = n_elements(x)-1


; Find maximum to the right.

IF(direction EQ 1 AND mm EQ 1) THEN BEGIN
   
   i = idx & found = 0
   
   WHILE NOT found DO BEGIN
      
      IF(i EQ nx) THEN BEGIN    ; Check if starting point is right edge.
         found = 1
         rtn = i
      ENDIF ELSE BEGIN
         
                                ; Check for right edge.
         
         IF(i+1 EQ nx AND x(i+1) GT x(i)) THEN BEGIN
            found = 1
            rtn = i+1
         ENDIF ELSE BEGIN
            
                                ; Check if local maximum has been found.
            
            IF(x(i+1) LT x(i)) THEN BEGIN
               found = 1
               rtn = i
            ENDIF ELSE BEGIN
               i = i+1
            ENDELSE
            
         ENDELSE
      ENDELSE
   ENDWHILE
   
   
ENDIF

; Find maximum to the left.

IF(direction EQ -1 AND mm EQ 1) THEN BEGIN
   
   i = idx & found = 0
   
   WHILE NOT found DO BEGIN
      
      IF(i EQ 0) THEN BEGIN     ; Check if starting point is left edge.
         found = 1
         rtn = i
      ENDIF ELSE BEGIN
         
                                ; Check for left edge.
         
         IF(i-1 EQ 0 AND x(i-1) GT x(i)) THEN BEGIN
            found = 1
            rtn = i-1
         ENDIF ELSE BEGIN
            
                                ; Check if local maximum has been found.
            
            IF(x(i-1) LT x(i)) THEN BEGIN
               found = 1
               rtn = i
            ENDIF ELSE BEGIN
               i = i-1
            ENDELSE
            
         ENDELSE
      ENDELSE
   ENDWHILE
   
ENDIF

; Find minimum to the right.

IF(direction EQ 1 AND mm EQ -1) THEN BEGIN
   
   i = idx & found = 0
   
   WHILE NOT found DO BEGIN
      
      IF(i EQ nx) THEN BEGIN    ; Check if starting point is right edge.
         found = 1
         rtn = i
      ENDIF ELSE BEGIN
         
                                ; Check for right edge.
         
         IF(i+1 EQ nx AND x(i+1) LT x(i)) THEN BEGIN
            found = 1
            rtn = i+1
         ENDIF ELSE BEGIN
            
                                ; Check if local minimum has been found.
            
            IF(x(i+1) GT x(i)) THEN BEGIN
               found = 1
               rtn = i
            ENDIF ELSE BEGIN
               i = i+1
            ENDELSE
            
         ENDELSE
      ENDELSE
   ENDWHILE
   
ENDIF

; Find minimum to the left.

IF(direction EQ -1 AND mm EQ -1) THEN BEGIN
   
   i = idx & found = 0
   
   WHILE NOT found DO BEGIN
      
      IF(i EQ 0) THEN BEGIN     ; Check if starting point is left edge.
         found = 1
         rtn = i
      ENDIF ELSE BEGIN
         
                                ; Check for left edge.
         
         IF(i-1 EQ 0 AND x(i-1) LT x(i)) THEN BEGIN
            found = 1
            rtn = i-1
         ENDIF ELSE BEGIN
            
                                ; Check if local minimum has been found.
            
            IF(x(i-1) GT x(i)) THEN BEGIN
               found = 1
               rtn = i
            ENDIF ELSE BEGIN
               i = i-1
            ENDELSE
            
         ENDELSE
      ENDELSE
   ENDWHILE
   
ENDIF


RETURN, rtn

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'lrminmax.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
