;**************************************************************************
;+
; NAME       : f1calib (procedure)
; PURPOSE    : convert pol.deg. to mag.strength (Gauss) for T1
; CALLING SEQUENCE :
;	f1calib, img, q, u, v, bx, by, bl, h=h, scatter=scatter, dwl=dwl, $
;		kdpfact=kdpfact, Bl_cal=Bl_cal, Bt_cal=Bt_cal
; INPUTS :
;     img	-- image
;     q,u,v     -- pol. degree *10000 (q,u,v already devided by img)
; KEYWORD PARAMETERS :
;     dwl       -- wave length position of Lyot filter from line center (mA)
;			if dwl is not specified, -80 mA is assumed
;     scatter   -- sacttered light intensity
;		   if not set, scatter=imin     (imin/imax < 0.17, limb image)
;				      =(1-0.7)*imin*imax/(imax-0.7*imin)
;						(	   > 0.17, disk image)
;     kdpfact	-- kdpfactor, default=1.05
;     h		-- header structure
;     		header modification
;		h.value   =>   I,  Bx,  By,  Bl
;		h.correct =>  101, 101, 101, 101
;		h.dx	  =>  pix1*bin
;		h.unit    =>  'G*10'
;		h.com     =>  'f1calib1, dwl=##   Mon dd hh:mm:ss'
;		h.d_parm(10)  =>  dwl
;		h.d_parm(11)  =>  scatter
;		h.d_parm(12)  =>  kdpfact
;		h.d_parm(15)  =>  blfact*1000
;     Bl_cal    -- calibration function of Bl,    p(*10000) => B(G*10)
;     Bt_cal    -- calibration function of Bt,    p(*10000) => B(G*10)
; OUTPUTS :
;     img	-- midified image for scatter
;     bx,by,bl  -- magnetic field (transversal & long.) in Gauss*10
; 
; MODIFICATION HISTORY :
;	K.Ichimoto, 1992/10
;	K.Ichimoto, 1992/10/25	rename to f1calib
;	K.Ichimoto, 1992/10/28	change order of Bl and Bx,By
;	K.Ichimoto, 1992/11/20	include img, h=h
;	K.Ichimoto, 1992/11/21	determine scatter
;	K.Ichimoto, 1993/05/18	quv2b included here, Bl modification
;-
;==========================================================================
pro f1calib,img,q,u,v,bx,by,bl,h=h,scatter=scatter,dwl=dwl,kdpfact=kdpfact, $
	Bl_cal=Bl_cal,Bt_cal=Bt_cal

blmodif=1	; if 1, make modification of Bl magnitude to barance with Bt
polfact=2	; pol.degree = q,u,v /10000./polfact

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;>> scattered light correction <<
;--------------------------------------------------------------------------
if n_elements(scatter) eq 0 then begin
	imax=max(img)
	imin=min(img>0)
	r=float(imin)/float(imax)
	sctfact=0.7
	if r lt 0.17 then begin
		print,'This is limb image...'
		scatter=imin
	endif else begin
		scatter=fix((1.-sctfact)*imin*imax/(imax-sctfact*imin))
	endelse
endif
if scatter ne 0 then scatter,scatter,img,q,u,v

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;>> KDP error correction <<
;--------------------------------------------------------------------------
if n_elements(kdpfact) eq 0 then kdpfact=1.05
if kdpfact ne 1 then begin
	print,'kdp error correction, fact=',kdpfact
	q=fix(q*kdpfact)
	u=fix(u*kdpfact)
	v=fix(v*kdpfact)
endif

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;>> convert pol. degree to B magnitude <<
;--------------------------------------------------------------------------
if not keyword_set(dwl) then dwl=-80
if not keyword_set(polfact) then polfact=2
if not keyword_set(Bl_cal) or not keyword_set(Bl_cal) then begin
	getcalib1,Bl_cal,Bt_cal,dwl=dwl,/extra
	print,'calibration table loaded,  dwl=',dwl,' mA'
endif
nn2=n_elements(Bl_cal)/2
print,'getting Bx, By & Bl ...'
bl=Bl_cal( (-v/polfact+nn2) >0<(n_elements(Bl_cal)-1) )
pl=float(q/polfact)^2+float(u/polfact)^2
pl=fix(sqrt(pl>0))
bt=Bt_cal( pl >0<(n_elements(Bt_cal)-1) )
ii=where( q eq 0 and u eq 0 , count )
if count ne 0 then q(ii)=1
th=atan(u,q)/2.
bx=fix(bt*cos(th))
by=-fix(bt*sin(th))

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;>> determine the Bt direction <<
;--------------------------------------------------------------------------
s=size(bl)	&	nx=s(1)	&	ny=s(2)
if nx eq 1 or ny eq 1 then return
print,'getting potential field ...'
pfact=nx/64
cff1n,rebin(bl,nx/pfact,ny/pfact),bxp,byp
bxp=rebin(bxp,nx,ny)
byp=rebin(byp,nx,ny)
print,'setting Bt direction...'
flip = where( bx*bxp + by*byp lt 0.0, count )
if count gt 0 then begin
	bx( flip ) = -bx( flip )
	by( flip ) = -by( flip )
endif

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;>> correct the Bl magnitude <<
;--------------------------------------------------------------------------
if blmodif eq 1 then begin
	btp=fix(sqrt(float(bxp)^2+float(byp)^2))
	costh=(float(bx)*bxp+float(by)*byp)/float(bt>1)/float(btp>1)
	btmax=max(bt)
	btpmax=max(btp)
	ii=where( (abs(costh) gt 0.9) and 	$
		  (bt gt btmax*0.5 )  and	$
		  (btp gt btpmax*0.5), count )
; check count gt 0 --- added by T.Sakurai, 93/10/25
	blfact=1.0
	if count gt 0 then begin
		blfact=total(float(bt(ii))/btp(ii))
		blfact=blfact/n_elements(ii)
		blfact=min([blfact,7.0])
		blfact=max([blfact,0.5])
		print,blfact,'   is multiplied on Bl'
		bl=fix(bl*blfact)
	endif
endif

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;>> set header information <<
;--------------------------------------------------------------------------
if keyword_set(h) then begin
	h.com='f1calib, dwl='+string(dwl,format='(i4)')+'  ' $
		+ strmid(systime(),4,strlen(systime())-4)
	h(1).value='Bx'		& h(1).unit='G*10'
	h(2).value='By'		& h(2).unit='G*10'
	h(3).value='Bl'		& h(3).unit='G*10'
	h.d_parm(10)=dwl
	h.d_parm(11)=fix(scatter)
	h.d_parm(12)=fix(kdpfact*100)
	h.correct=101
	if polfact eq 2 then h.correct=h.correct+1
	if blmodif eq 1 then begin
		h.correct=h.correct+100
		h(3).d_parm(15)=fix(blfact*1000)
	endif
endif

end

