;**************************************************************************
; NAME       : f4calib (procedure)
; PURPOSE    : convert pol.deg. to mag.strength (Gauss) 
;		       Dopp.signal to velocity for T4
; CALLING SEQUENCE :
;	f4calib, img, s, v, vel, bl, h=h,  Bl_cal=Bl_cal, Vel_cal=Vel_cal,  $
;		scatter=scatter,kdpfact=kdpfact,dwl=dwl,bmag=bmag
; INPUTS :
;     img	-- image
;     s,v       -- Dopp.signal & pol. degree *10000 (already devided by img)
; KEYWORD PARAMETERS :
;     dwl       -- wave length position of Lyot filter from line center (mA)
;			if dwl is not specified, dwl=0 mA is assumed
;     bmag      -- magnetic field strength in Gauss
;			if bmag is not specified, bmag=0 G is assumed
;     scatter   -- sacttered light intensity
;		   if not set, scatter=imin     (imin/imax < 0.17, limb image)
;				      =(1-0.7)*imin*imax/(imax-0.7*imin)
;						(	   > 0.17, disk image)
;     kdpfact	-- kdpfactor, default=1.05
;     Bl_cal    -- calibration function of Bl,    p(*10000) => B(G*10)
;     Vel_cal   -- calibration function of Vel,   s(*10000) => Vel(m/s)
; OUTPUTS :
;     img	-- modified image for scatter
;     vel,bl    -- magnetic field in Gauss*10 and velocity in m/s
; 
; MODIFICATION HISTORY :
;	K.Ichimoto, 1992/10/28	rename to f1calib
;	K.Ichimoto, 1992/11/21	use sv2vb, include img & h=h, sactter
;==========================================================================
;
pro f4calib,img,s,v,vel,bl,h=h,Bl_cal=Bl_cal,Vel_cal=Vel_cal, 	$
	scatter=scatter,kdpfact=kdpfact,dwl=dwl,bmag=bmag

if n_elements(scatter) eq 0 then begin
	imax=max(img)
	imin=min(img>0)
	r=float(imin)/float(imax)
	sctfact=0.7
	if r lt 0.17 then begin
		print,'This is limb image...'
		scatter=imin
	endif else begin
		scatter=fix((1.-sctfact)*imin*imax/(imax-sctfact*imin))
	endelse
endif
if scatter ne 0 then scatter,scatter,img,s,v
if not keyword_set(dwl) then dwl=0
if not keyword_set(bmag) then bmag=0
if n_elements(kdpfact) eq 0 then kdpfact=1.05
if kdpfact ne 1 then begin
	print,'kdp error correction, fact=',kdpfact
	s=fix(s*kdpfact)
	v=fix(v*kdpfact)
endif

sv2vb,s,v,vel,bl,Bl_cal=Bl_cal,Vel_cal=Vel_cal,dwl=dwl,bmag=bmag

if keyword_set(h) then begin
	h.com='f4calib, dwl='+string(dwl,format='(i4)')+'  ' $
		+'bmag='+string(bmag,format='(i4)')+'  ' $
		+ strmid(systime(),4,strlen(systime())-4)
	h(1).value='Vel'	& h(1).unit='m/s'
	h(2).value='Bl'		& h(2).unit='G*10'
	h(0).correct=101
	h(1).correct=101
	h(2).correct=101
	h.d_parm(10)=fix(dwl)
	h.d_parm(11)=fix(scatter)
	h.d_parm(12)=fix(kdpfact*100)
	h.d_parm(13)=fix(kdpfact*100)
	h.d_parm(14)=fix(bmag)
endif


end
