;=======================================================================
;+
; NAME :	magtv2 (procedure)
; PURPOSE :
;	display vector mag.field as contour on sunspot image
; CATEGORY :
;	idl/lib/flare
; CALLING SEQUENCE :
;	magtv2,img,q,u,v,h=h,/spline,/order,save=filename,/seq, $
;		magnify=magnify,bin=bin,/nocont,x0=x0,y0=y0,/notitle
;  or	magtv2,img,Bx,By,Bl,/cal,h=h,/spline,/order,save=filename,/seq, $
;		magnify=magnify,bin=bin,/nocont,x0=x0,y0=y0,/notitle
; INPUTS :
;	img     : image (*,*) or i,q,u,v as (*,*,4)
;       q,u,v   : polarization degree*10000
;       Bx,By,Bl: long. & trans. mag fields, gauss*10
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	h       : header array
;       spline  : if set, draw spline contour
; 	order   : if set, up-down direction is opposite
; 	save=filename  : save tvrd to file 
; 	/cal	: filenamefor calibrated data	Bl=>q,  Bx=>u, By=>v
; 	/seq	: display seq.no
; 	magnify : magnification factor
; 	bin	: binning factor
; 	nocont  : no contour of img
;       x0,y0   : lower-left position for drwaing
;       /notitle: omit title
; OUTPUTS :
;	none
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;	if h(1).value eq 'Bl', it displays as calibrated data (/cal)
; MODIFICATION HISTORY :
;	K.I. '92/02/08
;	K.I. '92/05/01
;	K.I. '92/10/20
;	K.I. '92/10/28	change order of Bl & bx,by
;	K.I. '92/11/19	change v sign
;	K.I. '92/11/29	h.dx --> bin
;	K.I. '92/12/06	x0,y0, /notitle
;	K.I. '93/03/08	nstep=max([nstep/magnify,2])
;	K.I. '93/12/21	rebin -> congrid
;-
;----------------------------------------------------------------------


pro colortbl,color	; define color table

color = { color,	$
	icmin:		0,	$
	icmax:		0,	$
	frame:		0,	$
	contour:	0,	$
	contour_p:	0,	$
	contour_n:	0,	$
	lpol:		0,	$
	sky:		0,	$
	bg:		0	$
	}
color.icmin=1
color.icmax=!d.table_size-6
color.frame=!d.table_size-1
color.contour=!d.table_size-2
color.contour_p=!d.table_size-2
color.contour_n=!d.table_size-3
color.lpol=!d.table_size-4
color.sky=!d.table_size-5
color.bg=0
	
end

;********************************************************************
;NAME       : magtv2c (procedure)
;FUNCTION   : display vector mag.field on spot image
;		for calibrated data
;						K.I. '92/10/20
;====================================================================
pro magtv2c,img,Bx,By,Bl,spline=spline,nocont=nocont,magnify=magnify,	$
	x0=x0,y0=y0,notitle=notitle

; img 		: image
; Bx,By		: tang. mag. field (G*10)
; Bl		: long. mag. field (G*10)
;-------------------   parameters   --------------------
Blmax=1500.	; max. of long. mag.	(Gauss)
Btmin=150.	; min. of tang. mag.	(Gauss)
nstep=10	; step of linear pol. bar plot
lfact=0.2	; factor for length of linear pol. bar
Bfact=10.	; ratio-fact  q,u,v -> q,u,v/Bfact (gauss)
thfact=1000.	; th-fact, angle=th/thfact (radian)
imgmin=0.2	; minimum level to display pol.
cmpfact=2	; compress factor for drawing contour
arrow=1		; display arrow for Bt

;------ initialise keywords ------
if keyword_set(bin) then pix1=pix1*bin
if keyword_set(magnify) then magnify=magnify else magnify=1
if keyword_set(spline) then spline=1 else spline=0
if keyword_set(nocont) then nocont=1 else nocont=0
if keyword_set(spline) then begin
	spline=1
	c_labels=[0]
endif else begin
	spline=0
	c_labels=0
endelse
s=size(img)
nx=s(1)	& ny=s(2)
colortbl,color
if n_elements(x0) eq 0 then x0=50
if n_elements(y0) eq 0 then y0=50
imax=max(img)
imin=min(img)
ifact=float(color.icmax-1)/float(imax-imin)
bimg=byte( (img-imin)*ifact+1)
;tv,rebin(bimg,nx*magnify,ny*magnify),x0,y0,/dev
tv,congrid(bimg,nx*magnify,ny*magnify,/interp),x0,y0,/dev

pos=[x0,y0,x0+nx*magnify,y0+ny*magnify]
if not keyword_set(nocont) then begin
    levp=[50,200,500,1000,1500]
    levn=levp
    nn=n_elements(levp)
    for i=0,nn-1 do levn(i)=-levp(nn-i-1)
    r=float(smooth(rebin(Bl,nx/cmpfact,ny/cmpfact),2))/Bfact
    ;img=rebin(img,nx/cmpfact,ny/cmpfact)
    img=congrid(img,nx/cmpfact,ny/cmpfact,/interp)
    if !order eq 1 then begin
	mirror,r
	mirror,img
    endif
    imgmax=max(img)
    sky=where(img lt imgmax*imgmin, nsky)
    if(nsky ne 0l) then r(sky) = 0.
    contour,r,	$		
	pos=pos,/dev,xstyle=1+4,ystyle=1+4,	$
	xrange=[0.,nx/cmpfact],yrange=[0.,ny/cmpfact],color=color.contour_p, $
	level=levp,/noerase,spline=spline,	$
	c_labels=c_labels	; use follow method and surpress labels
    contour,r,	$		
	pos=pos,/dev,xstyle=1+4,ystyle=1+4,	$
	xrange=[0.,nx/cmpfact],yrange=[0.,ny/cmpfact],color=color.contour_n, $
	level=levn,/noerase,spline=spline,	$
	c_labels=c_labels	; use follow method and surpress labels

    if !order eq 1 then mirror,img
endif else begin
    imgmax=max(img)
endelse

if n_elements(Bx) gt 1 and n_elements(By) gt 1 then begin
plot,[0,0],[0,0],/nodata,pos=pos,/dev,/noerase,	$
	xrange=[0.,nx*magnify],yrange=[0.,ny*magnify],xstyle=1+4,ystyle=1+4
nstep=max([nstep/magnify,2])
for j=nstep,ny-nstep,nstep do begin
for i=nstep,nx-nstep,nstep do begin
    if img(i/cmpfact,j/cmpfact) gt imgmax*imgmin then begin
	Bt1=float(Bx(i,j))^2+float(By(i,j))^2
	if Bt1 gt 0. then Bt1=sqrt(Bt1)/Bfact else Bt1=0.
	if Bt1 gt Btmin then begin
		th=atan(By(i,j),Bx(i,j))
		x=sqrt(Bt1)*cos(th)*lfact/magnify
		y=sqrt(Bt1)*sin(th)*lfact/magnify
		if !order eq 0 then begin
			x1=[i-x,i+x]
			y1=[j-y,j+y]
		endif else begin
			x1=[i-x,i+x]
			y1=[ny-j-1+y,ny-j-1-y]
		endelse
		oplot,x1*magnify,y1*magnify,color=color.lpol
		if arrow eq 1 then begin
			dth=25./180.*!pi
			len=sqrt((x1(1)-x1(0))^2+(y1(1)-y1(0))^2)
			larrow=min([len/4.,5.])
			ddx=larrow*(x1(0)-x1(1))/len
			ddy=larrow*(y1(0)-y1(1))/len
			xx1=x1(1)+ddx*cos(dth)+ddy*sin(dth)
			yy1=y1(1)-ddx*sin(dth)+ddy*cos(dth)
			xx2=x1(1)+ddx*cos(-dth)+ddy*sin(-dth)
			yy2=y1(1)-ddx*sin(-dth)+ddy*cos(-dth)
			oplot,  [xx1,x1(1),xx2,xx1]*magnify,	$
				[yy1,y1(1),yy2,yy1]*magnify,	$
				color=color.lpol
		endif
	endif
    endif
endfor
endfor
endif else begin
	print,'Bx & By not defined. Only Bl map is displayed!'
endelse
levstr='!3level=!9+!3'
if not keyword_set(notitle) then begin 
    for i=0,n_elements(levp)-2 do	$
	levstr=levstr+strcompress(string(levp(i),format='(i)'),/remove_all)+','
    levstr=levstr+string(levp(n_elements(levp)-1),format='(I4)')+'  Gauss'
	xyouts,x0,y0+ny*magnify+5,levstr,/dev,charsize=1.5
endif

end

;********************************************************************
;NAME       : magtv2r (procedure)
;FUNCTION   : display vector mag.field on spot image
;		for Q,U,V data
;						K.I. '92/10/20
;====================================================================
pro magtv2r,img,q,u,v,spline=spline,nocont=nocont,magnify=magnify,	$
	x0=x0,y0=y0,notitle=notitle

; img 		: image
; q,u,v		: Stokes Q,U,V * 10000

;-------------------   parameters   --------------------
pcmax=0.2	; max. of circular pol. for display
plmin=0.002	; min. of linear pol. for display
;pix1=0.6738     ; arc sec. for 1 pixel
pix1=pix_size('ft1')	; arc sec. for 1 pixel
nstep=10	; step of linear pol. bar plot
lfact=0.4	; factor for length of linear pol. bar
rfact=10000.	; ratio-fact  q,u,v -> q,u,v/rfact
imgmin=0.2	; minimum level to display pol.
cmpfact=2	; compress factor for drawing contour

;------ initialise keywords ------
if keyword_set(bin) then pix1=pix1*bin
if keyword_set(magnify) then magnify=magnify else magnify=1
if keyword_set(spline) then spline=1 else spline=0
if keyword_set(nocont) then nocont=1 else nocont=0
if keyword_set(spline) then begin
	spline=1
	c_labels=[0]
endif else begin
	spline=0
	c_labels=0
endelse
s=size(img)
nx=s(1)	& ny=s(2)
colortbl,color
if n_elements(x0) eq 0 then x0=50
if n_elements(y0) eq 0 then y0=50
;--------------------   draw   ------------------------------   
imax=max(img)
imin=min(img)
ifact=float(color.icmax-1)/float(imax-imin)
bimg=byte( (img-imin)*ifact+1)
;tv,rebin(bimg,nx*magnify,ny*magnify),x0,y0,/dev
tv,congrid(bimg,nx*magnify,ny*magnify,/interp),x0,y0,/dev

pos=[x0,y0,x0+nx*magnify,y0+ny*magnify]
if not keyword_set(nocont) then begin
    levp=[0.01,0.03,0.05,0.1,0.15]
    levn=levp
    nn=n_elements(levp)
    for i=0,nn-1 do levn(i)=-levp(nn-i-1)
    r=float(smooth(rebin(-v,nx/cmpfact,ny/cmpfact),2))/rfact
    ;img=rebin(img,nx/cmpfact,ny/cmpfact)
    img=congrid(img,nx/cmpfact,ny/cmpfact,/interp)
    if !order eq 1 then begin
	mirror,r
	mirror,img
    endif
    imgmax=max(img)
    sky=where(img lt imgmax*imgmin, nsky)
    if(nsky ne 0l) then r(sky) = 0.
    contour,r,	$		
	pos=pos,/dev,xstyle=1+4,ystyle=1+4,	$
	xrange=[0.,nx/cmpfact],yrange=[0.,ny/cmpfact],color=color.contour_p, $
	level=levp,/noerase,spline=spline,	$
	c_labels=c_labels	; use follow method and surpress labels
    contour,r,	$		
	pos=pos,/dev,xstyle=1+4,ystyle=1+4,	$
	xrange=[0.,nx/cmpfact],yrange=[0.,ny/cmpfact],color=color.contour_n, $
	level=levn,/noerase,spline=spline,	$
	c_labels=c_labels	; use follow method and surpress labels

    if !order eq 1 then mirror,img
endif

if n_elements(q) gt 1 and n_elements(u) gt 1 then begin
    plot,[0,0],[0,0],/nodata,pos=pos,/dev,/noerase,	$
	xrange=[0.,nx*magnify],yrange=[0.,ny*magnify],xstyle=1+4,ystyle=1+4

nstep=nstep/magnify
for j=nstep,ny-nstep,nstep do begin
for i=nstep,nx-nstep,nstep do begin
    if img(i/cmpfact,j/cmpfact) gt imgmax*imgmin then begin
	pl=fix(sqrt(float(q(i,j))^2+float(u(i,j))^2))
	if pl gt plmin*rfact   then begin
		th=atan(u(i,j),q(i,j))/2.
		x=cos(th)*sqrt(pl)*lfact/magnify
		y=-sin(th)*sqrt(pl)*lfact/magnify
		if !order eq 0 then begin
			x1=[i-x,i+x]
			y1=[j-y,j+y]
		endif else begin
			x1=[i-x,i+x]
			y1=[ny-j-1+y,ny-j-1-y]
		endelse
		oplot,x1*magnify,y1*magnify,color=color.lpol
	endif
    endif
endfor
endfor
endif else begin
	print,'Q & U not defined. Only V map is displayed!'
endelse
levstr='!3level = !9+!3'
for i=0,n_elements(levp)-2 do levstr=levstr+string(levp(i),format='(f4.2)')+', '
levstr=levstr+string(levp(n_elements(levp)-1),format='(f4.2)')+'  pol.degree'
if not keyword_set(notitle) then 	$
	xyouts,x0,y0+ny*magnify+5,levstr,/dev,charsize=1.5

end

;********************************************************************
;NAME       : magtv2 (procedure)
;FUNCTION   : display vector mag.field and draw intensity contour
;====================================================================
pro magtv2,img0,q,u,v,h=h,spline=spline,order=order,save=save,cal=cal,seq=seq, $
	magnify=magnify,bin=bin,nocont=nocont,x0=x0,y0=y0,notitle=notitle
; img0    : image (*,*) or i,q,u,v as (*,*,4)
;  h      : header array
; spline  : if set, draw spline contour
; order   : if set, up-down direction is opposite
; save=filename  : save tvrd to file
; /cal	  : for calibrated data	Bl=>q,  Bx=>u, By=>v
; /seq	  : display seq.no
; magnify : magnification factor
; bin	  : binning factor
; nocont  : no contour of img
; x0,y0	  : lower-left position for drwaing
; /notitle: omit title

pix1=pix_size('ft1')	; arc sec. for 1 pixel

;------ initialise keywords ------
if keyword_set(bin) then bin=bin else bin=1
if keyword_set(magnify) then begin
	magnify=magnify 
	print,'magnification factor : ',magnify
endif else begin
	magnify=1
endelse
if keyword_set(spline) then spline=1 else spline=0
if keyword_set(nocont) then nocont=1 else nocont=0
if keyword_set(notitle) then notitle=1 else notitle=0
if keyword_set(h) then begin
	if h(1).value eq 'Bx' then begin 
		cal=1
		print,'It is calibrated data...'
	endif
	if strmid(h(0).dx,0,1) eq ' ' then dx=0 $
	else dx=float(h(0).dx)
	if dx ne 0. then bin=dx/pix1
endif
if n_params() eq 0 then begin
	colortbl,color
	return
endif
;------ initialise array ------
s=size(img0)
nx=s(1)	& ny=s(2)
if s(0) eq 2 then img=img0 else img=img0(*,*,0)
if n_params() eq 2 then v=q
if n_params() eq 1 then begin	; case only img0 given
	if s(0) eq 2 then begin	; case img0 is 2-dimensional
		img=img0
	endif else begin	; case img0 is 3-dimensional
		img=img0(*,*,0)
		if s(3) eq 4 then begin
			q=img0(*,*,1)
			u=img0(*,*,2)
			v=img0(*,*,3)
		endif else begin
			v=img0(*,*,1)
		endelse
	endelse
endif

;------- initialize window & color table ----------
if n_elements(x0) eq 0 then begin
	x0=50
	ers=1
endif else ers=0
if n_elements(y0) eq 0 then y0=50
if keyword_set(notitle) then begin
	wwx=nx*magnify+x0+5	;expected window size
	wwy=ny*magnify+y0+5
endif else begin
	wwx=nx*magnify+x0+50	;expected window size
	wwy=ny*magnify+y0+55
endelse
if !d.window ne -1 then wno=!d.window else wno=0
if (!d.window eq -1) or (wwx gt !d.x_size+1) or (wwy gt !d.y_size+1) then $
	window,wno,xsize=wwx,ysize=wwy
if ers eq 1 then erase

colortbl,color
loadct,0
tvlct,0,0,0,color.frame		; color of frame in icontour
tvlct,255,0,0,color.contour_p	; color of v-positive contour
tvlct,0,0,255,color.contour_n	; color of v-negative contour
tvlct,0,255,0,color.lpol	; color of linear pol.
tvlct,0,0,0,color.sky		; color of sky
tvlct,255,255,255,color.bg		; back ground
!order=1
if keyword_set(order) then !order=0

;------- display map ----------
if keyword_set(cal) then begin 
	magtv2c,img,q,u,v,spline=spline,nocont=nocont,magnify=magnify,	$
		x0=x0,y0=y0,notitle=notitle
endif else begin
	magtv2r,img,q,u,v,spline=spline,nocont=nocont,magnify=magnify,	$
		x0=x0,y0=y0,notitle=notitle
endelse


;--------------------   frame display   ---------------------
if keyword_set(h) then begin
	if n_elements(h) gt 1 then begin
		timeint,h,time1,time2,date1,date2
		time_sys=h(0).time_sys
		g_x=h(0).g_x
		g_y=h(0).g_y
		seqno=h(0).seqno
		hql=h(0)
	endif else begin
		date1=h.date1
		time1=h.time1
		date2=h.date2
		time2=h.time2
		time_sys=h.time_sys
		g_x=h.g_x
		g_y=h.g_y
		seqno=h.seqno
		hql=h
	endelse
	if keyword_set(save) then begin
		hql.nbyt=1
		s=size(img)
		hql.nx=s(1)	& hql.ny=s(2)
		hql.date1=date1
		hql.date2=date2
		hql.time1=time1
		hql.time2=time2
		hql.nset=1
		hql.value='map'
		wshow
		rd=tvrd(x0,y0,wx,wy)
		nkrsave,save,hql,rd,/nocheck
		print,'display saved in '+save
	endif
	if time_sys eq 'JST' then begin
		jst2ut,date1,time1
		jst2ut,date2,time2
	endif
	title1='!N!3Solar Flare Telescope (MTK) : vector magnetic field'
	title2=date1+'  '+strmid(time1,0,8)+'-'+strmid(time2,0,8)+' UT' $
		+'    '+str_gpos(g_x,g_y)
endif else begin
	title1=''
	title2='!N!3Solar Flare Telescope (MTK) : vector magnetic field'
endelse

nx=nx*magnify
ny=ny*magnify
pix1=pix1*bin/magnify
xrange=[0.,pix1*(nx-1)]
yrange=[0.,pix1*(ny-1)]
tickv=[0,100,200,300,400,500,600]
pos=[x0,y0,x0+nx,y0+ny]
if not keyword_set(notitle) then begin
	xyouts,x0,y0+ny+35,title1,/dev,charsize=1.7
	xyouts,x0,y0+ny+20,title2,/dev,charsize=1.5
	if keyword_set(seq) then $
		xyouts,x0+nx-35,y0+ny+5,	$
		'('+string(seqno,format='(i3)')+')',/dev,charsize=1.5
	plot,[0,0],[0,0],/nodata,pos=pos,/dev,/noerase,color=color.frame, $
	  xrange=xrange,yrange=yrange,xstyle=1,ystyle=1,	$
          xticks=fix(nx*pix1/100.),xtickv=tickv,xminor=10,xticklen=0.03, $ 
          yticks=fix(ny*pix1/100.),ytickv=tickv,yminor=10,yticklen=0.03, $ 
	  xtitle='arc sec.',ytitle='arc sec.'
endif else begin
	plot,[0,0],[0,0],/nodata,pos=pos,/dev,/noerase,color=color.frame, $
	xrange=xrange,yrange=yrange,xstyle=1,ystyle=1,	$
        xticks=fix(nx*pix1/100.),xtickv=tickv,xminor=10,xticklen=0.03, $ 
        yticks=fix(ny*pix1/100.),ytickv=tickv,yminor=10,yticklen=0.03,	$
	xtickname=replicate(' ',7),ytickname=replicate(' ',7)
endelse

end
