;+
; IDL PROCEDURE
;
; NAME:
;	xinfts
;
; PURPOSE:
;	Animate a sequence of FITS images.
;
; CALLING SEQUENCE:
;	xinfts, 'fits.ls'
;	xinfts, 'fits.ls', rate=25
;	xinfts, 'fits.ls', rate=8, /label
;	xinfts, 'fits.ls', rate=2, /label, cm='~/color/sunset.lut'
;
; INPUTS:
;	listfile : Character string containing the names of the FITS
;		   files to be animated.
;	rate : Animation rate (range: 0-100).
;	label: If present, label each image with frame #, date, time.
;	cm: Name of color map file (ASCII, each line: index, red, green, blue).
;	order: If set, image is displayed top to bottom;
;	       default is bottom to top.
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	READFITS
;	FXPAR
;	XINTERANIMATE
;	LCT
;
; MODIFICATION HISTORY:
;	 7 Aug 1995: procedure created by Andrew L. Stanger, HAO/NCAR
;	11 Aug 1995: added order keyword, ALS, HAO/NCAR
;	11 Mar 1996: reduce image size if image is larger than 1000x1000.
;-

pro xinfts, listfile, rate=rate, label=label, cm=cm, order=order

if (not keyword_set (rate)) then rate = 100

filename = ' '
ii = 0
close, 10
openr, 10, listfile

;--- Find number of images in file list.

while (not eof (10))  do begin	;{
   readf, 10, filename
   ii = ii + 1
endwhile			;}
numimg = ii

close, 10

;--- Read first image to get size information.

imgbuf = readfits (filename, hdu)	; Read first FITS image in listfile.

nbitpx = fxpar (hdu, 'BITPIX')
naxis  = fxpar (hdu, 'NAXIS')
nxpix  = fxpar (hdu, 'NAXIS1')
nypix  = fxpar (hdu, 'NAXIS2')
idate  = fxpar (hdu, 'DATE-OBS')
itime  = fxpar (hdu, 'TIME-OBS')

nxmax = 1000
nymax = 1000
xdim = nxpix
ydim = nypix

while (xdim gt nxmax or ydim gt nymax)  do      $
   begin
      xdim = xdim / 2
      ydim = ydim / 2
   endwhile

if (nbitpx eq 8)       then imgbuf = bytarr (xdim, nypix) $
else if (nbitpx eq 16) then imgbuf = intarr (xdim, nypix) $
else if (nbitpx eq 32) then imgbuf = fltarr (xdim, nypix) $
else exit

bkgbuf = intarr (xdim, ydim) 
bkgbuf (*, *) = 0

if (keyword_set (label))  then $
begin
   window, 20, xsize = xdim, ysize = ydim, color=256, /pixmap
   wset, 20
end

;--- Load color table.

if (keyword_set (cm))  then lct, cm	$	; Load specified colormap.
else			    loadct, 0		; Load greyscale colormap.
 
init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 
 
if ( init eq 'y') then begin ;{ 
   XINTERANIMATE, SET = [xdim+4, ydim+4, numimg], /SHOWLOAD
 
   close, 10 & openr, 10, listfile
 
   ;--- Image loop.

   ii = 0 
   while ( not eof (10) ) do begin ;{
      readf, 10, filename

      ;--- Read image into array & scale intensity levels to range: 0-255.

      imgbuf = readfits (filename, hdu)	; Read FITS image in listfile.

      dispmin = 0.0
      dispmax = 0.0
      disptop = 249

      nbitpx  = fxpar (hdu, 'BITPIX')
      naxis   = fxpar (hdu, 'NAXIS')
      nxpix   = fxpar (hdu, 'NAXIS1')
      nypix   = fxpar (hdu, 'NAXIS2')
      dispmin = fxpar (hdu, 'DISPMIN')
      dispmax = fxpar (hdu, 'DISPMAX')

      nxmax = 1000
      nymax = 1000
      xdim = nxpix
      ydim = nypix

      while (xdim gt nxmax or ydim gt nymax)	do	$
	 begin
	    xdim = xdim / 2
	    ydim = ydim / 2
         endwhile

      ;--- Reduce image size if xdim or ydim > 1000 pixels.

      if (nxpix ne xdim or ydim ne nypix)  then	$
	    imgbuf = rebin (imgbuf, xdim, ydim)

      ;--- Scale image to the range 0-249.

      if (nbitpx eq   8)  then $
            imgbuf = bytscl (imgbuf, top=disptop)	$
      else						$
	 if (dispmin lt dispmax)  then	$
            imgbuf = bytscl (imgbuf, min=dispmin, max=dispmax, top=disptop) $
         else				$
            imgbuf = bytscl (imgbuf, top=disptop)

      ;--- If order keyword is set, reverse image top to bottom.

      if (keyword_set (order))  then imgbuf = reverse (imgbuf, 2)

      ;--- Label image if label keyword is set.

      if (keyword_set (label))  then $
      begin
         idate  = fxpar (hdu, 'DATE-OBS', count=count)
         sizdate = size (idate)
         if (count eq 0 or sizdate (1) lt 8)  then idate = filename

         itime  = fxpar (hdu, 'TIME-OBS', count=count)
         siztime = size (itime)
         if (count eq 0 or siztime (1) lt 8)  then itime = ''

         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgbuf
         imglabel = sframe + '     ' + idate + '   ' + itime
         xyouts, xpos, ypos, imglabel, /device
         XINTERANIMATE, FRAME = ii, WINDOW = !d.window	; Load image window.
      end				$
      else				$
         XINTERANIMATE, FRAME = ii, IMAGE = imgbuf	; Load image array.

      ii = ii+1
   endwhile ;} 
 
   close, 10
 
endif ;} 

wdelete, 20					; Delete temp window.

;--- Activate animation.

XINTERANIMATE, rate 

end 
