;+ =========================================================================== +
;
; NAME: 
;	MK3HOUSE
;
; PURPOSE:  
;	Output housekeeping information.
;
; CATEGORY: 
;	Mark-III diplay
;
; CALLING SEQUENCE: 
;	pro mk3house, hk=hk, ntime=ntime, colo=colo
;
; CALLED BY:  
;	MK3TV, MK3MOVIE
;
; CALLS TO:  
;	none
;
; INPUTS:
;	HK	: Array of size (40 x N) where N is 1 for single images
;		  and 2 for difference images. Contains the housekeeping
;		  information for the current image(s)
;	NTIME	:  0 means display the house keeping for the normal image 
;		     or base image of a difference display
;		   1 means use the subtraction image values in HK
;	COLO    : -1 means no hard copy allowed
;		   1 means hard copy device is color
;		   0 means hard copy device is color
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:  
;	none
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 
;	none
;
; SIDE EFFECTS: 
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Writes the house keeping information to the output device.
;	If a difference image, the base image house keeeping is on the
;	left and the subtraction image house keeping is written on the right.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;	1 March 1993 Andrew L. Stanger (HAO/NCAR)
;	   IRIX apparently requires font size 1/2 as large as for sunos.
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cphouse" for MLSO Mark-III images.
;
;	5 November 1993 - Andrew L. Stanger
;	Changed osfac from 0.5 to 0.4 for IRIX.
;	Other minor adjustments in annotation formats.
;	Also removed call to 'mk3hk_decode', since it is now done by 'mk3disk'.
;- =========================================================================== -

pro mk3house, hk=hk, ntime=ntime, colo=colo

@mk3_img.common
; COMMON mk3_img, iyear, idoy, imonth, iday, ihour, iminute, isecond, $
;                 dtype, nbypx, npx, npy, channel, dmin, dmax, smin, smax, $
;                 scndir, scndur, pixrs, pangle

   ;mk3hk_decode, mk3hk=hk

   ; x = 0.01 + (0.84 * ntime)  &  y = 0.96  &  down = 0.05  &  skip = 0.01

   x = 0.01 + (0.84 * ntime)
   y = 0.96
   textfac = 512 / 576.0		; Image is 512x512. Window height is 576
   down = 0.05 * textfac		; y increment for adjacent text.
   skip = 0.01 * textfac		; y increment for space between text.

   labcs = 1.2				; Set standard char size for labels.
   valcs = 2.0				; Set standard char size for values.

   case !version.os of
      'IRIX'	: osfac = 0.4		; Reduce size by 1/2 for SGI/IRIX.
      else	: osfac = 1.0		; Use standard char size for all others.
   endcase

   if (colo eq -1) then $
        colofac = 1.0   $		; Display size factor.
   else colofac = 1.5			; Print   size factor.

   labsiz = labcs * colofac * osfac	; Label font size for xyouts.
   valsiz = valcs * colofac * osfac	; Value font size for xyouts.

   c2 = !p.color			; color for labels.
   c1 = !p.color - 4			; color for data values.

   ; *** date: year & DOY ***

   xyouts, x, y, 'YEAR : DOY', /normal, size = labsiz, color = c2
   y = y - down

   xyouts, x, y, strtrim (iyear - 1900, 2) + ' : ' + $
		 string (idoy, format = '(i3.3)'), $
	   /normal, size=valsiz, color = c1
   y = y - down - skip

   ; *** GMT time ***

   xyouts, x, y, 'TIME  (UT)', /normal, size = labsiz, color = c2
   y = y - down

   time = string (fix (ihour), format='(i2.2)') + ' : ' + $
          string (fix (iminute), '(i2.2)') + ' : ' + $
	  string (fix (isecond), '(i2.2)')
   xyouts, x, y, time, /normal, size=valsiz, color = c1  &  y = y - down - skip

   ; *** day, month, and year ***

   month = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', $
            'Oct', 'Nov', 'Dec']
   xyouts, x, y, string (iday, format = '(i2)') + ' ' + month (imonth-1), $
	   /normal, size = valsiz, color = c1
   y = y - down

   xyouts, x, y, string (FIX (iyear)), /normal, size = valsiz, color = c1
   y = y - down - skip 

   ; *** Data type & channel ***

   xyouts, x, y, 'TYPE : CHANNEL', /normal, size=labsiz, color = c2
   y = y - down

   ; Mark-III data types:
   ;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   ; q   --> cylindrical q data					/*  0 */
   ; rq  --> rectangular q data, origin in SE			/*  1 */
   ; srq --> scaled rectangular q data, origin in SE		/*  2 */

   ; i   --> cylindrical i data					/*  3 */
   ; ri  --> rectangular i data, origin in SE			/*  4 */
   ; sri --> scaled rectangular i data, origin in SE		/*  5 */

   ; u   --> cylindrical u data					/*  6 */
   ; ru  --> rectangular u data, origin in SE			/*  7 */
   ; sru --> scaled rectangular u data, origin in SE 		/*  8 */

   ; pB  --> cylindrical pB data				/*  9 */
   ; rpB  --> rectangular pB data, origin in SE			/* 10 */
   ; srpB --> scaled rectangular pB data, origin in SE		/* 11 */

   ; aba --> barrel angles as ascii in degrees *10		/* 12 */
   ; ba --> barrel angles (shorts) with 16384=360 degrees	/* 13 */
   ; ast --> scan times as ascii in seconds			/* 14 */
   ; st --> scan times (shorts) with 60=1 second (Hz)		/* 15 */
   ;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

   itype = ['  q ', ' rq ', 'srq ', '  i ', ' ri ', 'sri ', $
	   '  u ', ' ru ', 'sru ', '  pB', ' rpB', 'srpB', $
	   'aba ', ' ba ', 'ast ', ' st ']
   type = itype (dtype)
   schan = type + STRING (channel, format='(i3)')
   xyouts, x, y, schan, /normal, size=valsiz, color=c1 & y = y - down - skip

   ;*** Data min/max. ***

   xyouts, x, y, 'MIN : MAX  INTENSITY', /normal, size=labsiz, color = c2
   y = y - down

   if (smin eq 0 and smax eq 0)  then				$
      sminmax = STRING (FIX (dmin), format='(i5)') + ':' +	$
		STRING (FIX (dmax), format='(i5)')		$
   else								$
      sminmax = STRING (FIX (smin), format='(i5)') + ':' +	$
		STRING (FIX (smax), format='(i5)')
   xyouts, x, y, sminmax, /normal, size=valsiz, color = c1
   y = y - down - skip

   ; *** scan direction ***

   xyouts, x, y, 'SCAN DIRECTION', /normal, size=labsiz, color = c2
   y = y - down

   xyouts, x, y, scndir, /normal, size=valsiz, color = c1
   y = y - down - skip

   ; *** scan duration ***

   xyouts, x, y, 'SCAN DURATION', /normal, size=labsiz, color = c2
   y = y - down
   sscndur = STRING (scndur, format='(i3)') + ' sec'	; Convert to string.

   xyouts, x, y, sscndur, /normal, size=valsiz, color = c1
   y = y - down - skip

   ; *** scale size ***

   xyouts, x, y, 'PIXELS/Rsun', /normal, size=labsiz, color = c2
   y = y - down

   spixrs = STRING (pixrs, format='(f6.1)')
   xyouts, x, y, spixrs, /normal, size=valsiz, color = c1
   y = y - down - skip

   ; *** P-angle ***

   xyouts, x, y, 'P-ANGLE', /normal, size=labsiz, color = c2  &  y = y - down
   spangle = STRING (pangle, format='(f6.1)') + ' deg'

   xyouts, x, y, spangle, /normal, size=valsiz, color = c1
   y = y - down - skip

end 

;+ =========================================================================== +
