;+
;===========================================================================
;
; file CPDISPLAY.PRO -- includes routines CPTV, CPDIFF, CPDISPLAY
;
;===========================================================================
;
;
; NAME:
;	CPTV
;
; PURPOSE:
; 	Send a single (not movie) image, housekeeping info and 
;	requested drawing option output to screen or printer.
;
; CATEGORY:  
;	CP display
;
; CALLING SEQUENCE:
;	CPTV, DATA=DATA, HK=HK, XPOS=XPOS, YPOS=YPOS, X_SIZ=X_SIZ, 
;		Y_SIZ=Y_SIZ, XWINSIZ=XWINSIZ, YWINSIZ=YWINSIZ, COLO=COLO, $
;		IMSIZ=IMSIZ, DONORTH=DONORTH, DODISK=DODISK, $
;		DORADII=DORADII, DORADIAL=DORADIAL, HOUSEKP=HOUSEKP, $
;		RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		SIN_MOV=SIN_MOV, NORM_DIFF=NORM_DIFF, PROCESSED=PROCESSED, $
;		FILENAME=FILENAME, DIFFNAME1=DIFFNAME1, DIFFNAME2=DIFFNAME2, $
;		SAVEME=SAVEME, WMOVIE=W, NFILES=NFILES
;
; CALLED BY:
;	CPDISPLAY
;
; CALLS TO:  
;	CPHOUSE, CPSUNCEN, PSPLOT, CPINTERANIMATE
;         
; INPUTS:
;	DATA  : Byte array of size equal to CP image.
;	HK	: Array of size (224 x N) where N is 1 for single images
;		  and 2 for difference images. Contains the housekeeping
;		  information for the current image(s)
;	XPOS  : X direction screen coordinate (device pixels), to place window
;	YPOS  : Y direction screen coordinate (device pixels), to place window
;	X_SIZ : Image X size
;	Y_SIZ : Image Y size
;	XWINSIZ :  X direction width of window w/ housekeeping
;	YWINSIZ :  Y direction width of window 
;	COLO  : -1 means no hard copy allowed
;		 1 means hard copy device is color
;		 0 means hard copy device is color
;	IMSIZ : Size of one dimension of CP image, CP images are square.  
;	DONORTH : 0 means don't draw solar north
;		  1 means do draw solar north on image
;	DODISK  : 0 means don't draw solar disk
;		  1 means do draw solar disk
;	DORADII : 0 means don't draw radius lines
;		  1 means do draw radius lines
;	DORADIAL : 0 means don't draw radial lines
;		   1 means do draw radial lines
;	HOUSEKP : X direction size reserved for house keeping information
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	NORM_DIFF : 0 means do a normal image
;                   1 means do a difference image
;	PROCESSED : for movies a number indicating the current movie frame
;	FILENAME  : string filename normal images	
;	DIFFNAME1 : string filename for base image 
;	DIFFNAME2 : string filename for subtraction image 
;	SAVEME    : for movies 0 means don't save movie to a file
;			       1 means do save movie to a file
;	W 	: associated record array for file output
;	NFILES  : for movies the number of frames in movie
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	PROCESSED : for movies a number indicating the next movie frame
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	CPWIDGBASES for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	If SAVEME is set then creates a file containing the images and
;	housekeeping information of a movie.
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:  
;	Output image to selected device and call CPSUNCEN to draw line options.
;	For movies to be saved write necessary arrays to an already opened file.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;-


pro cptv, data=data, hk=hk, xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz, $
		xwinsiz=xwinsiz, ywinsiz=ywinsiz, colo=colo, $
		imsiz=imsiz, donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, housekp=housekp, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, processed=processed, $
		filename=filename, diffname1=diffname1, diffname2=diffname2, $
		saveme=saveme, wmovie=w, nfiles=nfiles

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal


    if sin_mov eq 1 then colo = -1		; can't print movies

    savpcolor = !p.color	; windows color max
    if colo ge 0 then begin	; making a hardcopy
      set_plot,'ps',/copy   &   !p.font=0
      if norm_diff eq 0 then oxsiz=21. else oxsiz=23.  ; output size
      oysiz =  oxsiz *(ywinsiz/xwinsiz)   ; square up output
      xoff= 3.0 + norm_diff  &  yoff=25.5

      device, /landscape, bits_per_pixel=8, xsize=oxsiz, ysize=oysiz, $
	      xoffset=xoff, yoffset=yoff, color=colo, /helvetica, $
	      filename='CP.PS'
    endif

    xpos = xpos/xwinsiz  &  ypos = ypos/ywinsiz		; convert to normalized
    x_siz = x_siz/xwinsiz  &  y_siz = y_siz/ywinsiz 	; convert to normalized
    housekp=housekp/xwinsiz

    tv, data, xpos, ypos, /normal  

    ; do twice if a difference
    for ntime = 0,norm_diff do cphouse, hk=hk(*,ntime), ntime=ntime, colo=colo
 		
    cpsuncen, imsiz=imsiz, hk=hk(*,0), donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, colo=colo, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		housekp=housekp, $
		sin_mov=sin_mov, norm_diff=norm_diff, savpcolor=savpcolor, $
		xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz 

    if colo ge 0 then begin			; close and print image
	device,/close  &  set_plot,'x'  &  !p.font=-1 
        widget_control, comment, set_value='Printing output', /append

;	if colo eq 0 then spawn,'print/setup=psi/delete/notify idl.ps' $
;	   else spawn,'print/que=color$script/notify/delete idl.ps'

    psplot, filename='CP.PS', color=colo, /delete

    endif

    if sin_mov ne 0 then begin          ; load a movie frame to movie widget
      	   cpinteranimate, frame=processed, window=!d.window

	   if saveme then begin			
		;
		; Save a movie frame by writing it to a file
		;
		ndata = bytarr(imsiz,imsiz+2)	; 2 headers and image

		; write header(s) to file 
		for ntime = 0,norm_diff do $	; do twice if difference
			ndata(0,ntime) = [hk(*,ntime),nfiles-1b, $
					 xpos*100b, ypos*100b]	
		; write information needed by CPSUNCEN.PRO once the movie	
		;	is restored
                ndata(227) = donorth
		ndata(228) = dodisk
		ndata(229) = doradii
		ndata(230) = doradial
		ndata(231) = colo
		ndata(232) = ruletimes
		ndata(233) = rulecondition
		ndata(234) = norm_diff          ; normal / difference image
		ndata(235) = housekp*100	; percent space left for info
		ndata(236) = x_siz*100		; percent image X size
		ndata(237) = y_siz*100		; percent image Y size

		; add image of size IMSIZ x IMSIZ
		ndata(*,2:*) = data		
		w(processed)=ndata	; saving movie frame w/ headers to file
		ndata = 0b		; clear memory
	   endif

           processed = processed + 1


	   mes = 'Proccessed frame ' + strtrim(processed,2) + ': ' 
	   if norm_diff eq 0 then mes = mes + filename $
	    		     else mes = mes + diffname1 + ' sub ' + diffname2
           widget_control, news, set_value= mes, /append 

           erase
    endif

end

;===========================================================================

;+
;
; NAME: 
;	CPDIFF
;
; PURPOSE:
; 	Scale the difference images to align with the difference color table.
;
; CATEGORY:  
;	CP diplay
;
; CALLING SEQUENCE: 
;      	CPDIFF, DATA=DATA, POSFAC=POSFAC, NEGFAC=NEGFAC
;
; CALLED BY:  
;	CPDISPLAY
;
; CALLS TO: 
;	none
;
; INPUTS:
;	DATA  : Byte array of size equal to CP image.  Array values
;		range from -255 to +255.
;
; OPTIONAL INPUTS:  
;	none
;
; OUTPUTS:
;	DATA   : Byte array of size equal to CP image.  Array values
;		 range from 1 to the value of !D.N_COLORS-2
;	POSFAC : image red color scale factor
;	NEGFAC : image blue color scale factor
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 
;	none
;
; SIDE EFFECTS: 
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Align data array values to match the difference color table and
;	the users (or default) values for scaling the negative and positive
;	values resulting from the image subtraction.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;-

pro cpdiff, data=data, posfac=posfac, negfac=negfac

       blk=(!p.color)/2.                    ; middle value(s) of blue/red scale
       if (blk mod 1) ne 0 then begin loblk=fix(blk)  &  hiblk=loblk+1
     	             endif else begin loblk=blk       &  hiblk=blk    & endelse

       neg = where(data lt 0)  &  sizn = size(neg)   ; where are the negatives
       pos = where(data ge 0)  &  sizp = size(pos)   ; where are the positives

       ; scale the negative from 0 to the faintest blue color
       fntb = loblk-1.		; the faintest blue, 1 index from black
       if sizn(0) ne 0 then data(neg) = (data(neg) * (fntb/negfac)) + loblk

       ; scale the positives from faintest red color to !p.color
       fntr = hiblk+1.		; the faintest red, 1 index from black
       if sizp(0) ne 0 then data(pos) = (data(pos) * ((!p.color-fntr)/posfac))$
						+ hiblk
                                            
       ; truncate data values equal to zero to 1 (index 0 is a black line) and
       ;   data values equal to !p.color to !p.color-1 (index !p.color is 
       ;   a white line)
       data = 1 > byte(data) < !p.color-1 
     
end

;====================================================================

;+
;
; NAME: 
;	CPDISPLAY
;
; PURPOSE:
;	Setup image for display
;
; CATEGORY:  
;	CP display
;
; CALLING SEQUENCE:
; 	CPDISPLAY, NORM_DIFF=NORM_DIFF, DONORTH=DONORTH, $
;	         DODISK=DODISK, DORADII=DORADII, DORADIAL=DORADIAL,  $
;		 RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		 POSFAC=POSFAC, NEGFAC=NEGFAC, SIN_MOV=SIN_MOV, $
;		 PROCESSED=PROCESSED, SAVEME=SAVEME, WMOVIE=W, $
;		 NFILES=NFILES, OK=OK
;
; CALLED BY:
;	CPONE, CPMOVIE
;
; CALLS TO: 
;	CPDISK, CPDIFF, WINUP, CPTV
;
; INPUTS:
;	NORM_DIFF     : 0 means do a normal image
;                       1 means do a difference image
;	DONORTH       : just passing through, on it's way to CPTV
; 	DODISK        : just passing through, on it's way to CPTV
;	DORADII       : just passing through, on it's way to CPTV
;	DORADIAL      : just passing through, on it's way to CPTV
;	RULETIMES     : just passing through, on it's way to CPTV 
; 	RULECONDITION : just passing through, on it's way to CPTV
;	POSFAC        : just passing through, on it's way to CPDIFF 
;	NEGFAC        : just passing through, on it's way to CPDIFF 
;	SIN_MOV       : 0 means current display mode is for single images
;	           	1 means current display mode is for movies
;	PROCESSED     : just passing through, on it's way to CPTV
;	SAVEME 	      : just passing through, on it's way to CPTV
;	W 	      : just passing through, on it's way to CPTV
;	NFILES        : just passing through, on it's way to CPTV
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	OK : 0 means failed to successfully read the input file
;	     1 means successfully read the input file
;	PROCESSED : just passing through, on it's back to CPMOVIE
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS :
;	CPCOLORS to load color tables
;	COLORS to change values in R_CURR, G_CURR, B_CURR to desired table
;	CPDISPFILE for files to read and output device color condition
;	CPNEW_WINDOW  for window display variables
;	CPWIDGBASES for widget IDs to write out informational messages
;
; SIDE EFFECTS : 
;	Changes values in IDL common block COLORS
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Read in data file(s) and if a difference image do the subtraction.
;	Scale the data to the appropriate color table and load that table.
;	Call to CPTV to output the CP display.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;-

pro cpdisplay, norm_diff=norm_diff, donorth=donorth, $
	         dodisk=dodisk, doradii=doradii, doradial=doradial,  $
		 ruletimes=ruletimes, rulecondition=rulecondition, $
		 posfac=posfac, negfac=negfac, sin_mov=sin_mov, $
		 processed=processed, saveme=saveme, wmovie=w, $
		 nfiles=nfiles, ok=ok

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal
@cpcolors.common
;common cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset
@cpdispfiles.common
;common dispfiles, filename, diffname1, diffname2, onewin, colo
@cpnew_window.common
;common new_window, lastmark, sc, current_window, reuse, first_pass

common colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr


 if n_elements(processed) eq 0 then processed=0  ; define if no value 

 hk = bytarr(224, norm_diff+1)          ; house keeping, 2D for differences

 if norm_diff eq 0 then begin    	; normal image, not a difference
   cpdisk, data=data, hk=houskeep, file=filename, ok=ok, $
			imsiz=imsiz, sin_mov=sin_mov

   if not(ok) then begin
        explain = 'Error with file ' + filename 
        if sin_mov eq 0 then widget_control,comment,set_value=explain,/append $
		        else widget_control,news,set_value=explain,/append 
	return
   endif

   hk(0,0) = houskeep
   ih = ishft(hk(9,0) and 'f0'x, -4)
   iv = hk(9,0) and 'f'x                	; horizontal position parameter
   reso = ishft((hk(3,0) and '4'x),-2)  	; vertical position parameter
   if reso eq 0 then begin
	data=rebin(data, imsiz/2, imsiz/2)  	; high res to low res
	imsiz = imsiz/2
   endif


 endif else begin                   	; difference image
   ; go get the base image data
   cpdisk, data=data1, hk=houskeep, file=diffname1, ok=ok, $
				imsiz=imsiz,sin_mov=sin_mov
   if not(ok) then begin
        explain = 'Error with file ' + filename 
        if sin_mov eq 0 then widget_control,comment,set_value=explain,/append $
		        else widget_control,news,set_value=explain,/append 

        explain = 'Requires both images for a difference '
        if sin_mov eq 0 then widget_control,comment,set_value=explain,/append $
		        else widget_control,news,set_value=explain,/append 
	return
   endif

   hk(0,0) = houskeep                   ; base image's housekeeping info
   ih = ishft(hk(9,0) and 'f0'x, -4)
   iv = hk(9,0) and 'f'x                	; horizontal position parameter
   reso = ishft((hk(3,0) and '4'x),-2)    	; vertical position parameter
   if reso eq 0 then begin
	data1=rebin(data1, imsiz/2, imsiz/2)  	; high res to low
	imsiz = imsiz/2
   endif
   ; now get the subtraction image data
   cpdisk, data=data2, hk=houskeep, file=diffname2, ok=ok, $
				imsiz=imsiz, sin_mov=sin_mov
   if not(ok) then begin
        explain = 'Error with file ' + filename 
        if sin_mov eq 0 then widget_control,comment,set_value=explain,/append $
		        else widget_control,news,set_value=explain,/append 
        explain = 'Requires both images for a difference '
        if sin_mov eq 0 then widget_control,comment,set_value=explain,/append $
		        else widget_control,news,set_value=explain,/append 
	return
   endif
   hk(0,1) = houskeep             	; subtraction image's housekeeping info
   ih2 = ishft(hk(9,1) and 'f0'x, -4)
   iv2 = hk(9,1) and 'f'x                	; horizontal position parameter
   reso = ishft((hk(3,1) and '4'x),-2)    	; vertical position parameter
   if reso eq 0 then begin
	data2=rebin(data2, imsiz/2, imsiz/2)  	; high res to low
	imsiz = imsiz/2
   endif
 

   ; size testing, images must be same
   siz1 = size(data1)  &  siz2 = size(data2)
   if siz1(1) ne siz2(1) then begin
        explain = 'Base and Subtraction images are different sizes'
	if sin_mov eq 0 then widget_control,comment,set_value=explain,/append $
	                else widget_control,news,set_value=explain,/append 
 	return
   endif

   ; subtract "base image" from "subtraction image"
   data1 = fix(data1)  &  data2 = fix(data2)  
   data = data2 - data1 
 endelse
 

 housekp = 150.			; room to write out house keeping info
 x_siz=448.  &  y_siz = 448.	; width and height of image windowing area 
 xwinsiz = housekp*(norm_diff+1.) + x_siz	; width window w/ housekeeping 
 ywinsiz = y_siz				; height window w/ housekeeping 

 square = x_siz/4.              ; size of one of the 16 position squares

 case 1 of 
    (ih eq 7) or (ih eq 6) : xpos = square + housekp	; 1 square to right
    (ih eq 3) : xpos = square*2 + housekp               ; 2 squares to right
    else : xpos = 0 + housekp				; left edge w/ house
 endcase
 case 1 of
    (iv eq 14) or (iv eq 6) : ypos = square		; 1 square up 
    (iv eq 12) : ypos = square*2			; 2 sqaures up
    else : ypos = 0                                     ; bottom edge
 endcase

 case coltyp of
    0: begin	; normal display so use b/w, data values within 0 to +255

       if colreset then begin    ; reset to standard c/p normal color table
          r_curr=norm_r  &  g_curr=norm_g  &  b_curr=norm_b  
       endif

       data = (!p.color-2)/255. * data	; scale data to !p.color-2
                                        ;   2 values reserved for lines
       end                               

    1: begin    ; difference images, data values within -255 to +255

       if colreset then begin    ; reset to standard c/p difference color table
          r_curr=diff_r  &  g_curr=diff_g  &  b_curr=diff_b 
       endif

       cpdiff, data=data, posfac=posfac, negfac=negfac
       end
 endcase

 if sin_mov eq 0 and colo eq -1 then begin  
 ; if a single image (movies make their own) and if output going to screen

	   if reuse and not(first_pass) then begin	; reuse windows
		wdelete, current_window
	   endif else begin
		lastmark = 1 - lastmark			; alternate position
		first_pass = 0 				; now have at least 1
 	   endelse
		
           if not(norm_diff) then title = 'CP '+ filename $
		             else title = 'CP '+ diffname1 + ' / ' + diffname2
;           winup, /free, title=title, $
;	        xsiz=xwinsiz, ysiz=ywinsiz, retain=2, $
;		xpos=sc(0)-xwinsiz-10, $  
;		ypos=(sc(1)-ywinsiz) * lastmark + (lastmark * (-20) + 10)
           winup, /free, title=title, $
	        xsiz=xwinsiz, ysiz=ywinsiz, retain=2, $
		xpos=sc(0)-10, $  
		ypos=sc(1) * lastmark + (lastmark * (-20) + 10)

	   current_window = !d.window

 endif

 tvlct, r_curr, g_curr, b_curr   		; load some color table 

 cptv, data=data, hk=hk, xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz, $
		xwinsiz=xwinsiz, ywinsiz=ywinsiz, colo=colo, $
		imsiz=imsiz, donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, housekp=housekp, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, processed=processed, $
		filename=filename, diffname1=diffname1, diffname2=diffname2, $
	        saveme=saveme, wmovie=w, nfiles=nfiles

end



