;+ =========================================================================== +
;
; NAME: CPDISK
;
; PURPOSE: 
;	Read in a CP data file with housekeeping information and image.
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
; 	CPDISK, DATA=DATA, HK=HOUSKEEP, FILE=FILENAME, OK=OK, $
;		IMSIZ=IMSIZ, SIN_MOV=SIN_MOV
;
; CALLED BY: 
;	CPDISPLAY
;
; CALLS TO: 
;	none
;
; INPUTS:
;	FILENAME = Filename of CP data file to be read.
;	SIN_MOV  = 0 means current display mode is for single images
;	           1 means current display mode is for movies
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	DATA  = Byte array of size equal to CP image in pixels.
;	HK    = Byte array of size equal to one dimension of image array.
;	        Contain CP housekeeping information, padded with zeroes
;	OK    = 0 means failed to successfully read the input file
;	        1 means successfully read the input file
;	IMSIZ = Size of one dimension of CP image, CP images are square.  
;		 
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	CPWIDGBASES for widget IDs of informational output widgets.
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Locate and open direct access data file. The total size in bytes,
;	minus the header, squared is the size of the CP image in pixels.  
;	A VMS record length in multiples of 512 was tested for CPTAPE.PRO,
;	but not implemented.  Code for that option remains in these routines.
;
; MODIFICATION HISORY:
;	1991 - Elaine Einfalt (HSTX)
;
;       22 Feb 93 - Andrew L. Stanger (HAO/NCAR)
;	Modify so that header includes housekeeping & bitmap records,
;	since unaltered C/P image files contain two header records (hk & bitmap)
;	at the beginning of each file.
;       Hence, status.size = recl * (recl + 2), and
;       image array ('data') now starts at w (2) instead of w (1).
;
;- =========================================================================== -


pro cpdisk, data=data, hk=cphk, file=filename, ok=ok, imsiz=imsiz, $
	    sin_mov=sin_mov, imgndx=imgndx

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;               xcen, ycen, pixrs

   ii = imgndx

   if (strlen (filename (0)) eq 0) then filename = 'no_file_entered'
   a = findfile (filename)  
   ok = 0					; assume it will fail
   if (a (0) eq '') then begin
      explain = 'No file found: ' + filename +' in CPDISK'
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain, /append $
      else widget_control, news, set_value=explain, /append 
      return
   endif

   on_ioerror, BADREAD

   openr, unit, filename, /get_lun 
   status = fstat (unit)  &  imsiz = fix (sqrt (status.size))	; file size

   ; If not one of the valid C/P data file sizes.
   ; ex: 229888 = (448 lines data + 1 line header) * 512 record length
   ; or  201152 = (448 lines data + 1 line header) * 448 record length

   case status.size of
	201600 : begin imsiz = 448 & recl = 448 & end
	804608 : begin imsiz = 896 & recl = 896 & end
	50624  : begin imsiz = 224 & recl = 224 & end
	113568 : begin imsiz = 336 & recl = 336 & end
	452928 : begin imsiz = 672 & recl = 672 & end

	403200 : begin imsiz = 448 & recl = 448 & end
       1609216 : begin imsiz = 896 & recl = 896 & end
	101248 : begin imsiz = 224 & recl = 224 & end
	227136 : begin imsiz = 336 & recl = 336 & end
	905856 : begin imsiz = 672 & recl = 672 & end

	230400 : begin imsiz = 448 & recl = 512 & end
	919552 : begin imsiz = 896 & recl = 1024 & end
	115712 : begin imsiz = 224 & recl = 512 & end
	173056 : begin imsiz = 336 & recl = 512 & end
	690176 : begin imsiz = 672 & recl = 1024 & end
	else   : begin print, 'invalid image size:', status.size & return & end
   endcase

   hk = bytarr (224)
   readu, unit, hk					; read C/P housekeeping

   cphk = fix (hk and 'ff'x)				; convert to short int.
   cphk_decode, cphk=cphk, imgndx=imgndx		; Decode housekeeping.

   recl  = npx (ii)
   imsiz = npy (ii)

   if (nbypx (ii) eq 1)  then					$
      w = assoc (unit, bytarr (recl, imsiz + 2))		$
   else								$
      w = assoc (unit, intarr (recl, imsiz + 2))	

   v = w (0)
   data = v (*, 2:imsiz+1)				; Read image data.

   ok = 1						; success

   BADREAD: 
   free_lun, unit  

   return
end

;+ =========================================================================== +
