;+ =========================================================================== +
;
; NAME: 
;	CPHOUSE
;
; PURPOSE:  
;	Output housekeeping information.
;
; CATEGORY: 
;	CP diplay
;
; CALLING SEQUENCE: 
;	pro cphouse, hk=hk, ntime=ntime, colo=colo
;
; CALLED BY:  
;	CPTV, CPMOVIE
;
; CALLS TO:  
;	none
;
; INPUTS:
;	HK	: Array of size (224 x N) where N is 1 for single images
;		  and 2 for difference images. Contains the housekeeping
;		  information for the current image(s)
;	NTIME	:  0 means display the house keeping for the normal image 
;		     or base image of a difference display
;		   1 means use the subtraction image values in HK
;	COLO    : -1 means no hard copy allowed
;		   1 means hard copy device is color
;		   0 means hard copy device is color
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:  
;	none
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 
;	none
;
; SIDE EFFECTS: 
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Writes the house keeping information to the output device.
;	If a difference image, the base image house keeeping is on the
;	left and the subtraction image house keeping is written on the right.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;	1 March 1993 Andrew L. Stanger (HAO/NCAR)
;	   IRIX apparently requires font size 1/2 as large as for sunos.
;
;- =========================================================================== -

pro cphouse, hk=hk, ntime=ntime, colo=colo

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;               xcen, ycen, pixrs

@cp_date.common
;COMMON cp_date, iyear, idoy, imonth, iday, ihour, iminute, isecond

   ii = ntime

   ; Adjust location/increment parameters for 448 pixel height, not 512.

   x = 0.01 + (0.85 * ntime)		; starting x location.
   y = 0.97				; starting y location.
   textfac = 448 / 480.0		; image is 448x448. window height is 480
   down = 0.05 * textfac		; y increment for adjacent text.
   skip = 0.01 * textfac		; y increment for space between text.

   labcs = 1.2				; Set standard char size for labels.
   valcs = 2.0				; Set standard char size for values.

   ;case !version.os of
   ;   'IRIX'	: osfac = 0.5		; Reduce size by 1/2 for SGI/IRIX.
   ;   else	: osfac = 1.0		; Use standard char size for all others.
   ;endcase

   osfac = 1.0

   if (colo eq -1) then	$
      begin
         colofac = 1.00			; Display size factor.
	 if (!version.os eq 'IRIX') then osfac = 0.5  ; SGI:reduce display size.
      end		$
   else colofac = 0.80			; Print   size factor.

   labsiz = labcs * colofac * osfac	; Label font size for xyouts.
   valsiz = valcs * colofac * osfac	; Value font size for xyouts.

   c2 = !p.color			; color for labels.
   c1 = !p.color - 4			; color for data values.

   ; Year : DOY

   xyouts, x, y, 'DATE', /normal, size = labsiz, color=c2  &  y = y - down
   xyouts, x, y, strtrim (iyear (ii), 2) + ':' + string (idoy (ii), $
		 format = '(i3.3)'), /normal, size=valsiz, color=c1
   y = y - down - skip
                               
   xyouts, x, y, 'TIME', /normal, size = labsiz, color=c2  &  y = y - down
   time = string (fix (hk (20)), format='(i2.2)') + ':' + $
          string (fix (hk (21)), '(i2.2)') + ':' + $
	  string (fix (hk (24)), '(i2.2)')
   xyouts, x, y, time, /normal, size=valsiz, color=c1  &  y = y - down - skip

   ; Day, Month & Year.

   month = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', $
            'Oct', 'Nov', 'Dec']
   xyouts, x, y, string (iday (ii), format = '(i2.2)') + ' ' + $
		 month (imonth (ii)), /normal, size = valsiz, color=c1  
   y = y - down
   xyouts, x, y, strtrim (1900 + iyear (ii), 2), $
		 /normal, size = valsiz, color=c1
   y = y - down - skip 

   ; Spectral filter.

   xyouts, x, y, 'FILTER', /normal, size=labsiz, color=c2  &  y=y-down
   cfw = ['RED','H-alpha', 'OPEN', 'BLUE', 'WIDE BD', 'GREEN']
   xyouts, x, y, strtrim (cfw (ifw (ii)), 2), /normal, size=valsiz, color=c1
   y = y - down - skip

   ; Polaroid filter.

   xyouts, x, y, 'POLAROID', /normal, size=labsiz, color=c2  &  y = y-down
   cpw = ['CLR', '+60 deg',  '-60 deg', '0 deg']
   xyouts, x, y, strtrim (cpw (ipw (ii)), 2), /normal, size=valsiz, color=c1
   y = y - down - skip

   ; Sector mirror.

   xyouts, x, y, 'SECTOR', /normal, size=labsiz, color=c2  &  y = y-down
   csm = ['SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W', 'NESW']
   xyouts, x, y, strtrim (csm (ism (ii)), 2), /normal, size=valsiz, color=c1
   y = y-down-skip

   ; Exposure duration (seconds).

   xyouts, x, y, 'EXPOSURE', /normal, size=labsiz, color=c2  &  y = y-down
   xyouts, x, y, strtrim (strmid (iexp (ii),1,8), 2), $
		 /normal, size=valsiz, color=c1
   y = y - down - skip

   ; Image Resolution.

   xyouts, x, y, 'RESOLUTION', /normal, size=labsiz, color=c2  &  y = y - down
   cres = ['HIGH','LOW']
   xyouts, x, y, strtrim (cres (ires (ii)), 2), /normal, size=valsiz, color=c1
   y = y - down - skip

   ; Spacecraft Roll.

   xyouts, x, y, 'ROLL', /normal, size=labsiz, color=c2  &  y = y-down
   roll = iroll (ii)
   if (roll gt 180) then roll = roll - 360
   xyouts, x, y, strtrim (roll, 2), /normal, size=valsiz, color=c1
   y = y - down - skip

end 

;+ =========================================================================== +
