;+ =========================================================================== +
;
; file CPDISPLAY.PRO -- includes routines CPTV, CPDIFF, CPDISPLAY
;
;+ =========================================================================== +
;
; NAME:
;	CPTV
;
; PURPOSE:
; 	Send a single (not movie) image, housekeeping info and 
;	requested drawing option output to screen or printer.
;
; CATEGORY:  
;	CP display
;
; CALLING SEQUENCE:
;	CPTV, DATA=DATA, HK=HK, XPOS=XPOS, YPOS=YPOS, X_SIZ=X_SIZ, 
;		Y_SIZ=Y_SIZ, XWINSIZ=XWINSIZ, YWINSIZ=YWINSIZ, COLO=COLO, $
;		IMSIZ=IMSIZ, DONORTH=DONORTH, DODISK=DODISK, $
;		DORADII=DORADII, DORADIAL=DORADIAL, HOUSEKP=HOUSEKP, $
;		RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		SIN_MOV=SIN_MOV, NORM_DIFF=NORM_DIFF, PROCESSED=PROCESSED, $
;		FILENAME=FILENAME, DIFFNAME1=DIFFNAME1, DIFFNAME2=DIFFNAME2, $
;		SAVEME=SAVEME, WMOVIE=W, NFILES=NFILES
;
; CALLED BY:
;	CPDISPLAY
;
; CALLS TO:  
;	CPHOUSE, CPSUNCEN, PSPLOT, CPINTERANIMATE
;         
; INPUTS:
;	DATA  : Byte array of size equal to CP image.
;	HK	: Array of size (224 x N) where N is 1 for single images
;		  and 2 for difference images. Contains the housekeeping
;		  information for the current image(s)
;	XPOS  : X direction screen coordinate (device pixels), to place window
;	YPOS  : Y direction screen coordinate (device pixels), to place window
;	X_SIZ : Image X size
;	Y_SIZ : Image Y size
;	XWINSIZ :  X direction width of window w/ housekeeping
;	YWINSIZ :  Y direction width of window 
;	COLO  : -1 means no hard copy allowed; image to be displayed on screen.
;		 1 means hard copy device is color
;		 0 means hard copy device is B/W 
;	IMSIZ : Size of one dimension of CP image, CP images are square.  
;	DONORTH : 0 means don't draw solar north
;		  1 means do draw solar north on image
;	DODISK  : 0 means don't draw solar disk
;		  1 means do draw solar disk
;	DORADII : 0 means don't draw radius lines
;		  1 means do draw radius lines
;	DORADIAL : 0 means don't draw radial lines
;		   1 means do draw radial lines
;	HOUSEKP : X direction size reserved for house keeping information
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	NORM_DIFF : 0 means do a normal image
;                   1 means do a difference image
;	PROCESSED : for movies a number indicating the current movie frame
;	FILENAME  : string filename normal images	
;	DIFFNAME1 : string filename for base image 
;	DIFFNAME2 : string filename for subtraction image 
;	SAVEME    : for movies 0 means don't save movie to a file
;			       1 means do save movie to a file
;	W 	: associated record array for file output
;	NFILES  : for movies the number of frames in movie
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	PROCESSED : for movies a number indicating the next movie frame
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	CPWIDGBASES for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	If SAVEME is set then creates a file containing the images and
;	housekeeping information of a movie.
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:  
;	Output image to selected device and call CPSUNCEN to draw line options.
;	For movies to be saved, write necessary arrays to a previously-opened
;	file.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	1 April 1993 - Andrew L. Stanger (HAO/NCAR)
;	Add title area below image for colorbar & image logo.
;	Draw box around window & image.
;	Also display 2-byte/pixel images.
;	Do simple scaling of intentites for difference image.
;	Create postscript file based upon image filename.
;- =========================================================================== -


pro cptv, data=data, hk=hk, xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz, $
		xwinsiz=xwinsiz, ywinsiz=ywinsiz, colo=colo, $
		imsiz=imsiz, donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, $
		housekp=housekp, ylabel=ylabel, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, processed=processed, $
		filename=filename, diffname1=diffname1, diffname2=diffname2, $
		saveme=saveme, wmovie=w, nfiles=nfiles

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cpcolors.common
; COMMON cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;		   coltyp, colreset, maxcolor, colorbar

@cpdirnames.common
; common cpdirnames, dirname, dirname1, dirname2

@cp_date.common
;COMMON cp_date, iyear, idoy, imonth, iday, ihour, iminute, isecond

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;               xcen, ycen, pixrs

   if (sin_mov eq 1) then colo = -1		; can't print movies

   savpcolor = maxcolor				; data color max

   fxwinsiz = float (xwinsiz)
   fywinsiz = float (ywinsiz)

   ; Set postscript device & parameters.

   if (colo ge 0) then begin			; making a hardcopy
      set_plot, 'ps', /copy   &   !p.font = 0

      ; 8.5x11" paper is 21.6x28 cm.  A 1" border leaves an area ~16.5x23 cm.
      ; Postscript units below are (by default) in cm units.  2.54cm = 1 inch.
      ; xoffset & yoffset are always relative to lower left corner of page
      ; when viewed in portrait mode, which is the default orientation.

      if (norm_diff eq 0) then oxsiz = 18.7 $
			  else oxsiz = 22.0	; x-dim output size.
      oysiz =  oxsiz * (fywinsiz / fxwinsiz)	; compute y-dim size.
      xborder = (28.0 - oxsiz) / 2.0		; center image on paper.
      yoff = oxsiz + xborder		; yoff => x-axis in landscape mode.
      xoff = 2.54			; xoff => y-axis in landscape mode.

      ; Create postscript file name.

      case !version.os of
	'vms' : begin
		  if (colo eq 0) then  pfile = 'cpfbw.ps'	$
		    else  pfile = 'cpfcolor.ps'
		end

	else  : begin	; UNIX filesystems: ultrix, sunos, IRIX.
			; Create postscript filename, based upon image filename.
			; First, eliminate directory pathname.

		  if (norm_diff eq 0)  then	$
		    begin
		      imgindex = strlen (dirname)
		      imglength = strlen (filename) - imgindex
		      namimg = strmid (filename, imgindex, imglength)
		    end		$
		  else		$
		    begin
		      csm=['sw', 'ss', 'se', 'ee', 'ne', 'nn', 'nw', 'ww', '4q']

		      cyear   = string (iyear (1),   format='(i2.2)')
		      cdoy    = string (idoy  (1),   format='(i3.3)')
		      chour1  = string (ihour (1),   format='(i2.2)')
		      cmin1   = string (iminute (1), format='(i2.2)')
		      chour0  = string (ihour (0),   format='(i2.2)')
		      cmin0   = string (iminute (0), format='(i2.2)')

		      cyear  = strtrim (cyear,  1)
		      cdoy   = strtrim (cdoy,   1)
		      chour1 = strtrim (chour1, 1)
		      cmin1  = strtrim (cmin1,  1)
		      chour0 = strtrim (chour0, 1)
		      cmin0  = strtrim (cmin0,  1)

		      namimg = cyear + cdoy + '.' + $
				chour1 + cmin1 + '-' + chour0 + cmin0 + '.'

		      if (nbypx (1) eq 1)  then namimg = namimg + 'v'	$
		      else	$
		      if (nbypx (1) eq 2)  then namimg = namimg + 'c'	$
		      else	$
					    namimg = namimg + 'x'
		      namimg = namimg + csm (ism (1))
		    end

		  if (colo eq 0) then pfile = namimg + '.fbw.ps'	$
				 else pfile = namimg + '.fcolor.ps'
		end
      endcase

      ; print, 'cptv> oxsiz/oysiz:', oxsiz, oysiz, '  xoff/yoff:', xoff, yoff

      device, /landscape, bits_per_pixel=8, xsize=oxsiz, ysize=oysiz, $
	      xoffset=xoff, yoffset=yoff, color=colo, /helvetica, $
	      filename=pfile
   endif

   xd_hk = housekp					; save pixel units.
   yd_label = ylabel					; save pixel units.
   xd_siz = x_siz					; save pixel coord.
   yd_siz = y_siz					; save pixel coord.

   ; Convert variables from pixel to normalized values (0.0 - 1.0).
   ; It is necessary to use normalized coordinates for postscript output.

   xpos  = xpos  / fxwinsiz  &  ypos  = ypos  / fywinsiz   ; x & y origin
   x_siz = x_siz / fxwinsiz  &  y_siz = y_siz / fywinsiz   ; x & y size
   housekp = housekp / fxwinsiz				   ; housekeeping width
   ylabel  = ylabel  / fywinsiz				; logo/colorbar height

   ;if (colo ne 0) then erase, !p.color-5		; white background.

   ; Display image or create postscript file.

   tv, data, xpos, ypos, xsize=x_siz, ysize=y_siz, /normal, /order

   ; Write annotation. Do twice for a difference image.

   for ntime = 0, norm_diff do $
      cphouse, hk = hk (*, ntime), ntime=ntime, colo=colo

   ; Draw "+" at sun center.
   ; Draw "overlay" options:  If requested, draw sun circle, polar coord grid.

   cpsuncen, imsiz=imsiz, hk=hk (*,0), donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, colo=colo, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, savpcolor=savpcolor, $
		housekp=housekp, ylabel=ylabel, $
		xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz 

   ; Write image logo at bottom of image window.

   case !version.os of
      'IRIX'	: osfac = 0.5		; Reduce size by 1/2 for SGI/IRIX.
      else	: osfac = 1.0		; Use standard char size for all others.
   endcase

   if (colo eq -1) then	$
        csize = 1.50 * osfac	$		; Display character size.
   else csize = 1.10				; Print   character size.

   xtext = (xd_hk + 80) / fxwinsiz   &   ytext =  20 / fxwinsiz
   xyouts, xtext, ytext, 'CORONAGRAPH', /normal, size=csize, color=!p.color-2

   xtext =  xd_hk / fxwinsiz   &   ytext =  12 / fxwinsiz
   xyouts, xtext, ytext, 'HAO/SMM', /normal, size=csize, color=!p.color-2

   xtext = (xd_hk + 80) / fxwinsiz   &   ytext =   4 / fxwinsiz
   xyouts, xtext, ytext, 'POLARIMETER', /normal, size=csize, color=!p.color-2

   ; Write vertical line to separate annotation from image.

   xwl = 0.0						; window origin :
   ywb = 0.0						; lower left corner.
   xwr  = (xwinsiz - 1) / fxwinsiz 			; window right edge.
   ywt  = (ywinsiz - 1) / fywinsiz			; window top   edge.

   xil = (xd_hk -      1) / fxwinsiz			; left  of image.
   xir = (xd_hk + xd_siz) / fxwinsiz			; right of image.
   yib = (yd_label - 2)   / fywinsiz			; below image.

   ; Draw line around window perimeter.

   PLOTS, [xwl, xwr], [ywb, ywb], COLOR=!p.color-2, /normal
   PLOTS, [xwr, xwr], [ywb, ywt], COLOR=!p.color-2, /normal
   PLOTS, [xwr, xwl], [ywt, ywt], COLOR=!p.color-2, /normal
   PLOTS, [xwl, xwl], [ywt, ywb], COLOR=!p.color-2, /normal

   ; Draw lines separating annotation from image.

   PLOTS, [xwl, xwr], [yib, yib], COLOR=!p.color-2, /normal
   PLOTS, [xil, xil], [yib, ywt], COLOR=!p.color-2, /normal
   if (norm_diff NE 0) then $
      PLOTS, [xir, xir], [yib, ywt], COLOR = !p.color-2, /normal

   ; Write colorbar across bottom of image window.

   size_cb = size (colorbar)			; get size info for 'colorbar'.
   x_cbsiz = size_cb (1)			; colorbar width.
   y_cbsiz = size_cb (2)			; colorbar height.

   y_cbb = yd_label / 2				; colorbar bottom.
   y_cbt = y_cbb + y_cbsiz - 1 			; colorbar top.

   x_cbl = xd_hk + xd_siz - x_cbsiz - 1		; colorbar left edge.
   x_cbr = x_cbl + x_cbsiz - 1			; colorbar right edge.

   xcbsiz = x_cbsiz / fxwinsiz			; normalized colorbar width.
   ycbsiz = y_cbsiz / fywinsiz			; normalized colorbar height.

   ; print, 'cptv> x_cbl/y_cbb:', x_cbl/fxwinsiz, y_cbb/fywinsiz

   tv, colorbar, x_cbl/fxwinsiz, y_cbb/fywinsiz, $
		 xsize=xcbsiz, ysize=ycbsiz, /normal   ; normalized coords.

   ; Draw box around colorbar.

   xcbl = (x_cbl - 1) / fxwinsiz
   xcbr = (x_cbr + 1) / fxwinsiz
   ycbb = (y_cbb - 1) / fywinsiz
   ycbt = (y_cbt + 1) / fywinsiz

   PLOTS, [xcbl, xcbr], [ycbb, ycbb], COLOR = !p.color-2, /normal
   PLOTS, [xcbl, xcbr], [ycbt, ycbt], COLOR = !p.color-2, /normal
   PLOTS, [xcbl, xcbl], [ycbb, ycbt], COLOR = !p.color-2, /normal
   PLOTS, [xcbr, xcbr], [ycbb, ycbt], COLOR = !p.color-2, /normal

   ; colorbar tick mark coordinates.

   xcb000 = (x_cbl      ) / fxwinsiz
   xcb063 = (x_cbl +  63) / fxwinsiz
   xcb127 = (x_cbl + 127) / fxwinsiz
   xcb191 = (x_cbl + 191) / fxwinsiz
   xcb255 = (x_cbl + 255) / fxwinsiz

   xcb025 = (x_cbl + maxcolor * 0.25) / fxwinsiz
   xcb050 = (x_cbl + maxcolor * 0.50) / fxwinsiz
   xcb075 = (x_cbl + maxcolor * 0.75) / fxwinsiz
   xcb100 = (x_cbl + maxcolor       ) / fxwinsiz

   ycbbeg = (y_cbt +   2) / fywinsiz
   ycbend = (y_cbt +   4) / fywinsiz

   ; Draw tick marks above colorbar: mark levels 0, 63, 127, 191 & 255.

   PLOTS, [xcb000, xcb000], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb063, xcb063], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb127, xcb127], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb191, xcb191], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb255, xcb255], [ycbbeg, ycbend], COLOR = !p.color, /normal

   ycbbeg = (y_cbb -   2) / fywinsiz
   ycbend = (y_cbb -   6) / fywinsiz

   ; Draw tick marks below colorbar: mark 0, 25, 50, 75 & 100 % of maxcolor.

   PLOTS, [xcb000, xcb000], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb025, xcb025], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb050, xcb050], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb075, xcb075], [ycbbeg, ycbend], COLOR = !p.color, /normal
   PLOTS, [xcb100, xcb100], [ycbbeg, ycbend], COLOR = !p.color, /normal

   if (colo eq -1) then	$
        csize = 0.80 * osfac	$		; Display character size.
   else csize = 0.60				; Print   character size.

   ;if (colo eq -1) then	$
   ;     csize = 0.4	$			; Display character size.
   ;else csize = 0.8				; Print   character size.

   xyouts, xcb000, 2/fywinsiz, '0%',   size = csize, COLOR = !p.color, /normal
   xyouts, xcb025, 2/fywinsiz, '25%',  size = csize, COLOR = !p.color, /normal
   xyouts, xcb050, 2/fywinsiz, '50%',  size = csize, COLOR = !p.color, /normal
   xyouts, xcb075, 2/fywinsiz, '75%',  size = csize, COLOR = !p.color, /normal
   xyouts, xcb100, 2/fywinsiz, '100%', size = csize, COLOR = !p.color, /normal

   ; Close PS file & reset device to X-window server.
   ; Write message to text widget.

   if (colo ge 0) then begin
      device, /close  &  set_plot, 'x'  &  !p.font=-1 
      widget_control, comment, set_value='Wrote image to PS file: ' + pfile, $
		      /append

      if (colo eq 0) then $
         print, 'Wrote image to B/W postscript file: ' + pfile $
      else if (colo eq 1) then $
         print, 'Wrote image to color postscript file: ' + pfile

      ; Print PS image file.

;      if (colo eq 0) then spawn, 'print/setup=psi/delete/notify idl.ps' $
;                     else spawn, 'print/que=color$script/notify/delete idl.ps'

;25 Feb 93 ALS      psplot, filename='cp.ps', color=colo, /delete

   endif

   if (sin_mov ne 0) then begin          ; load a movie frame to movie widget
      cpinteranimate, frame=processed, window=!d.window

      if (saveme) then begin			
         ;
         ; Save a movie frame by writing it to a file.
         ;
         ndata = bytarr (imsiz,imsiz+2)	; 2 headers and image

         ; Write header(s) to file.

         for ntime = 0, norm_diff do $	; do twice if difference
            ndata (0, ntime) = [hk (*, ntime), nfiles-1b,  xpos*100b, ypos*100b]	
         ; Write information needed by CPSUNCEN.PRO once the movie is restored.

         ndata (227) = donorth
         ndata (228) = dodisk
         ndata (229) = doradii
         ndata (230) = doradial
         ndata (231) = colo
         ndata (232) = ruletimes
         ndata (233) = rulecondition
         ndata (234) = norm_diff		; normal / difference image
         ndata (235) = housekp*100		; percent space left for info
         ndata (236) = x_siz*100		; percent image X size
         ndata (237) = y_siz*100		; percent image Y size

         ; Add image of size IMSIZ x IMSIZ.

         ndata (*, 2:*) = data		
         w (processed) = ndata	; saving movie frame w/ headers to file
         ndata = 0b		; clear memory
      endif

      processed = processed + 1

      mes = 'Proccessed frame ' + strtrim (processed,2) + ': ' 
      if (norm_diff eq 0) then mes = mes + filename $
	    		  else mes = mes + diffname1 + ' sub ' + diffname2
      widget_control, news, set_value= mes, /append 

      erase
   endif

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	CPDIFF
;
; PURPOSE:
; 	Scale the difference images to align with the difference color table.
;
; CATEGORY:  
;	CP diplay
;
; CALLING SEQUENCE: 
;      	CPDIFF, DATA=DATA, POSFAC=POSFAC, NEGFAC=NEGFAC
;
; CALLED BY:  
;	CPDISPLAY
;
; CALLS TO: 
;	none
;
; INPUTS:
;	DATA  : Byte array of size equal to CP image.  Array values
;		range from -255 to +255.
;
; OPTIONAL INPUTS:  
;	none
;
; OUTPUTS:
;	DATA   : Byte array of size equal to CP image.  Array values
;		 range from 1 to the value of !D.N_COLORS-2
;	POSFAC : image red color scale factor
;	NEGFAC : image blue color scale factor
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 
;	none
;
; SIDE EFFECTS: 
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Align data array values to match the difference color table and
;	the users (or default) values for scaling the negative and positive
;	values resulting from the image subtraction.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;+ =========================================================================== +

pro cpdiff, data=data, posfac=posfac, negfac=negfac

@cpcolors.common
; COMMON cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;		   coltyp, colreset, maxcolor

   blk = (maxcolor) / 2.0		; middle value(s) of blue/red scale
   if ((blk mod 1) ne 0) then begin loblk = fix (blk)  &  hiblk = loblk+1
		   endif else begin loblk = blk        &  hiblk = blk  & endelse

       neg = where (data lt 0)  &  sizn = size (neg)   ; where are the negatives
       pos = where (data ge 0)  &  sizp = size (pos)   ; where are the positives

       ; Scale the negative from 0 to the faintest blue color.

       fntb = loblk - 1.0		; the faintest blue, 1 index from black
       if (sizn (0) ne 0) then data (neg) = (data (neg) * (fntb/negfac)) + loblk

       ; Scale the positives from faintest red color to maxcolor.

       fntr = hiblk + 1.0		; the faintest red, 1 index from black
       if (sizp (0) ne 0) then $
	  data (pos) = (data (pos) * ((maxcolor-fntr)/posfac)) + hiblk
                                            
       ; Truncate data values equal to zero to 1 (index 0 is a black line),
       ; and data values equal to !p.color to !p.color-6 (index !p.color is 
       ; a white line).

       data = 1 > byte (data) < !p.color-6 
     
end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	CPDISPLAY
;
; PURPOSE:
;	Setup image for display
;
; CATEGORY:  
;	CP display
;
; CALLING SEQUENCE:
; 	CPDISPLAY, NORM_DIFF=NORM_DIFF, DONORTH=DONORTH, $
;	         DODISK=DODISK, DORADII=DORADII, DORADIAL=DORADIAL,  $
;		 RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		 POSFAC=POSFAC, NEGFAC=NEGFAC, SIN_MOV=SIN_MOV, $
;		 PROCESSED=PROCESSED, SAVEME=SAVEME, WMOVIE=W, $
;		 NFILES=NFILES, OK=OK
;
; CALLED BY:
;	CPONE, CPMOVIE
;
; CALLS TO: 
;	CPDISK, CPDIFF, WINUP, CPTV
;
; INPUTS:
;	NORM_DIFF     : 0 means do a normal image
;                       1 means do a difference image
;	DONORTH       : just passing through, on it's way to CPTV
; 	DODISK        : just passing through, on it's way to CPTV
;	DORADII       : just passing through, on it's way to CPTV
;	DORADIAL      : just passing through, on it's way to CPTV
;	RULETIMES     : just passing through, on it's way to CPTV 
; 	RULECONDITION : just passing through, on it's way to CPTV
;	POSFAC        : just passing through, on it's way to CPDIFF 
;	NEGFAC        : just passing through, on it's way to CPDIFF 
;	SIN_MOV       : 0 means current display mode is for single images
;	           	1 means current display mode is for movies
;	PROCESSED     : just passing through, on it's way to CPTV
;	SAVEME 	      : just passing through, on it's way to CPTV
;	W 	      : just passing through, on it's way to CPTV
;	NFILES        : just passing through, on it's way to CPTV
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	OK : 0 means failed to successfully read the input file
;	     1 means successfully read the input file
;	PROCESSED : just passing through, on it's back to CPMOVIE
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS :
;	CPCOLORS to load color tables
;	COLORS to change values in R_CURR, G_CURR, B_CURR to desired table
;	CPDISPFILE for files to read and output device color condition
;	CPNEW_WINDOW  for window display variables
;	CPWIDGBASES for widget IDs to write out informational messages
;
; SIDE EFFECTS : 
;	Changes values in IDL common block COLORS
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Read in data file(s) and if a difference image do the subtraction.
;	Scale the data to the appropriate color table and load that table.
;	Call to CPTV to output the CP display.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;- =========================================================================== -

pro cpdisplay, norm_diff=norm_diff, donorth=donorth, $
	         dodisk=dodisk, doradii=doradii, doradial=doradial,  $
		 ruletimes=ruletimes, rulecondition=rulecondition, $
		 posfac=posfac, negfac=negfac, sin_mov=sin_mov, $
		 processed=processed, saveme=saveme, wmovie=w, $
		 nfiles=nfiles, ok=ok

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cpcolors.common
;common cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset

@cpdispfiles.common
;common dispfiles, filename, diffname1, diffname2, onewin, colo

@cpnew_window.common
;common new_window, lastmark, sc, current_window, reuse, first_pass

common colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr

@cpcolors.common
; COMMON cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;		   coltyp, colreset, maxcolor

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;               xcen, ycen, pixrs

  if (n_elements (processed) eq 0) then processed = 0	; define if no value 

  hk = bytarr (224, norm_diff+1)          ; housekeeping, 2D for differences

  ; --- Normal image ---

  if (norm_diff eq 0) then begin    	; normal image, not a difference

    ; Read image(s) from disk.

    cpdisk, data=data, hk=houskeep, file=filename, ok=ok, $
	    imsiz=imsiz, sin_mov=sin_mov, imgndx=0

    ; Write status message to text widget.

    if (not (ok)) then begin
        explain = 'Error with file ' + filename 
        if (sin_mov eq 0) then $
	     widget_control, comment, set_value=explain, /append $
	else widget_control, news,    set_value=explain, /append 
	return
    endif

    ; Extract IH, IV, & resolution from housekeeping.

    hk (0, 0) = houskeep
    ih = ishft (hk (9, 0) and 'f0'x, -4)
    iv = hk (9, 0) and 'f'x                	; horizontal position parameter
    reso = ishft ((hk (3, 0) and '4'x), -2)  	; vertical position parameter

    ; If image is High res, convert to Low.

    if (reso eq 0) then begin
	data = rebin (data, imsiz / 2, imsiz / 2)  	; high res to low res
	imsiz = imsiz / 2
    endif

;    if (nbypx (0) eq 1) then	$
;	data = bytscl (data, min=24, max=176, top=maxcolor)
;    if (nbypx (0) eq 2) then	$
;	data = bytscl (data, min= 0, max=512, top=maxcolor)

  ; --- Difference Image  ---

  endif else begin                   	; difference image

    ; Read base image data from disk.

    cpdisk, data=data1, hk=houskeep, file=diffname1, ok=ok, $
	    imsiz=imsiz, sin_mov=sin_mov, imgndx=0

    ; Write status message to text widget.

    if (not (ok)) then begin
        explain = 'Error with file ' + filename 
        if (sin_mov eq 0) then $
	     widget_control, comment, set_value=explain, /append $
        else widget_control, news,    set_value=explain, /append 

        explain = 'Requires both images for a difference '
        if (sin_mov eq 0) then $
	     widget_control, comment, set_value=explain, /append $
	else widget_control, news,    set_value=explain, /append 
	return
    endif

    ; Extract IH, IV, & resolution from housekeeping.

    hk (0, 0) = houskeep			; base image's housekeeping info
    ih = ishft (hk (9, 0) and 'f0'x, -4)
    iv = hk (9, 0) and 'f'x                	; horizontal position parameter
    reso = ishft ((hk (3, 0) and '4'x),-2)    	; vertical position parameter

    ; If image is High res, convert to Low.

    if (reso eq 0) then begin
	data1 = rebin (data1, imsiz / 2, imsiz / 2)  	; high res to low
	imsiz = imsiz / 2
    endif

    ; Now read subtraction image data from disk.

    cpdisk, data=data2, hk=houskeep, file=diffname2, ok=ok, $
	    imsiz=imsiz, sin_mov=sin_mov, imgndx=1

    ; Write status message to text widget.

    if (not (ok)) then begin
        explain = 'Error with file ' + filename 
        if (sin_mov eq 0) then $
	     widget_control, comment, set_value=explain, /append $
	else widget_control, news,    set_value=explain, /append 

        explain = 'Requires both images for a difference '
        if (sin_mov eq 0) then $
	     widget_control, comment, set_value=explain, /append $
	else widget_control, news,    set_value=explain, /append 
	return
    endif

    ; Extract IH, IV, & resolution from housekeeping.

    hk (0, 1) = houskeep             	; subtraction image's housekeeping info
    ih2 = ishft (hk (9, 1) and 'f0'x, -4)
    iv2 = hk (9, 1) and 'f'x                	; horizontal position parameter
    reso = ishft ((hk (3, 1) and '4'x), -2)    	; vertical position parameter

    ; If image is High res, convert to Low.

    if (reso eq 0) then begin
	data2 = rebin (data2, imsiz / 2, imsiz / 2)  	; high res to low
	imsiz = imsiz / 2
    endif

    ; Size testing: images must be same.

    siz1 = size (data1)  &  siz2 = size (data2)
    if (siz1 (1) ne siz2 (1)) then begin
        explain = 'Base and Subtraction images are different sizes'
	if (sin_mov eq 0) then $
	     widget_control, comment, set_value=explain, /append $
	else widget_control, news,    set_value=explain, /append 
 	return
    endif

    ; Subtract "base image" from "subtraction image".

    data1 = fix (data1)  &  data2 = fix (data2)  	; convert to short int.
    data = data2 - data1 
  endelse
 
  ; Compute size needed for image display.

  housekp = 96.0			; room to write out house keeping info
  ylabel = 32.0				; room to write colorbar + image name
  x_siz  = 448.0  &  y_siz = 448.0    ; width and height of image windowing area
  xwinsiz = housekp * (norm_diff + 1.0) + x_siz	; width window w/ housekeeping 
  ywinsiz = y_siz + ylabel			; height window w/ housekeeping 

  ; Compute x/y position for images less than full size (ih/iv < 'f'x).

  square = x_siz / 4.0			; size of one of the 16 position squares

  case 1 of 
    (ih eq 7) or (ih eq 6) : xpos = square + housekp	; 1 square to right
    (ih eq 3) : xpos = square*2 + housekp               ; 2 squares to right
    else : xpos = 0.0 + housekp				; left edge w/ house
  endcase

  case 1 of
    (iv eq 14) or (iv eq 6) : ypos = square + ylabel	; 1 square up 
    (iv eq 12) : ypos = square*2 + ylabel		; 2 sqaures up
    else : ypos = 0.0 + ylabel                          ; bottom edge
  endcase
  ypos = ypos - 1.0					; allow for image border

  ; Set appropriate color map.

  case coltyp of
    0: begin	; Normal display, so use b/w.  Data values within 0 to +255.

         if (colreset) then begin	;reset to standard cp normal color table
            r_curr = norm_r  &  g_curr = norm_g  &  b_curr = norm_b  
         endif

;         data = (maxcolor) / 255.0 * data	; scale data to maxcolor
						; 5 levels reserved for lines
         if (nbypx (0) eq 1) then	$
	   data = bytscl (data, min=24, max=176, top=maxcolor)
         if (nbypx (0) eq 2) then	$
	   data = bytscl (data, min= 0, max=512, top=maxcolor)

       end                               

    1: begin    ; Difference image.  Data values within -255 to +255.

         if (colreset) then begin	; reset to standard c/p diff color table
            r_curr = diff_r  &  g_curr = diff_g  &  b_curr = diff_b 
         endif

         difmin = min (data)
	 difmax = max (data)
	 if (abs (difmin) gt difmax)  then  difmax = -difmin	$
				      else  difmin = -difmax
         data = bytscl (data, min = difmin, max = difmax, top = maxcolor)

         ;cpdiff, data=data, posfac=posfac, negfac=negfac
       end
  endcase

  ; If a single image (movies make their own) and if output going to screen :

  if ((sin_mov eq 0) and (colo eq -1)) then begin  

    if (reuse and not (first_pass)) then begin	; reuse windows
      wdelete, current_window
    endif else begin
      lastmark = 1 - lastmark			; alternate position
      first_pass = 0 				; now have at least 1
    endelse
		
    if (not (norm_diff)) then title = 'CP '+ filename $
      else title = 'CP '+ diffname1 + ' / ' + diffname2

    xloc =  sc (0) - xwinsiz - 10
    yloc = (sc (1) - ywinsiz) * lastmark + (lastmark * (-20) + 10)

    winup, /free, title=title, $
           xsiz = xwinsiz, ysiz = ywinsiz, retain = 2, $
           xpos = xloc,  ypos = yloc  

    current_window = !d.window

    ; print, 'cpdisplay> xwinsiz:', xwinsiz, ' ywinsiz:', ywinsiz, $
    ;        '   xloc:', xloc, ' yloc:', yloc
  endif

  ; Load color map.

  tvlct, r_curr, g_curr, b_curr

  ; Display image (or write postscript file).

  cptv, data=data, hk=hk, xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz, $
		xwinsiz=xwinsiz, ywinsiz=ywinsiz, colo=colo, $
		imsiz=imsiz, donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, $
		housekp=housekp, ylabel=ylabel, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, processed=processed, $
		filename=filename, diffname1=diffname1, diffname2=diffname2, $
	        saveme=saveme, wmovie=w, nfiles=nfiles

end

;+ =========================================================================== +
