;+
; NAME:
;        IMGFMT
;
; PURPOSE:
;         Resize and repostion a solar disk image so that it is registered
;         with a specified image center & diameter.
;      
; METHOD: The image is resized and reposisitioned to match the first
;         (the TARGET).
;
; CALLING SEQUENCE:
;
; obuf = imgfmt (ibuf, MKTV=mktv					  $
;           ISIZE=[ixdim,iydim], ICENTER=[ixcen, iycen], IRADIUS=iradius, $
;           OSIZE=[oxdim,oydim], OCENTER=[oxcen, oycen], ORADIUS=oradius)
;                                                            
; PARAMETERS:
;
;	obuf		The rescaled output image.
;
;	ibuf		The input image.
;
;	ISIZE		input image size [xdim, ydim]	default: [512,512]
;	ICENTER		input [x,y] center		default: [255.5,255.5]
;	IRADIUS		input solar radius		default: 224
;
;	OSIZE		output image size [xdim, ydim]	default: [512,512]
;	OCENTER		output [x,y] center		default: [255.5,255.5]
;	ORADIUS		output solar radius		default: 224 
;
; KEYWORDS:
;
;	MKTV   		if you want to put obuf on a window set /mktv.
;
; CAUTION:
;
;	When working with fits images, the top few lines or so 
;	should be removed before input. These contain the 
;	header and gum things up.
;
; RETURN TYPE: Same as input.
; 
; HISTORY:
;	Andrew L. Stanger (ALS), HAO/NCAR, 6 May 1996.
;       9 May 1996: Add gen_loc call.
;-

FUNCTION imgfmt, ibuf, MKTV=mktv,				$
		 ISIZE=isize, ICENTER=icenter, IRADIUS=iradius,	$
		 OSIZE=osize, OCENTER=ocenter, ORADIUS=oradius

; --- Specify location and size of the target image (video frame).

IF (KEYWORD_SET (icenter)) THEN BEGIN
   ixcen = icenter (0)
   iycen = icenter (1)
ENDIF ELSE BEGIN			; Center of default image [512,512].
   ixcen = 255.5
   iycen = 255.5
ENDELSE

IF (KEYWORD_SET (iradius)) THEN BEGIN
   ixrad = iradius
   iyrad = iradius
ENDIF ELSE BEGIN			; Default radius.
   ixrad = 224.0
   iyrad = 224.0
ENDELSE

IF (KEYWORD_SET (isize)) THEN BEGIN
   ixdim = isize (0)
   iydim = isize (1)
ENDIF ELSE BEGIN			; Default output size.
   ixdim = 512
   iydim = 512
ENDELSE

print, 'imgfmt> ixcen,iycen: ', ixcen, iycen
print, 'imgfmt> ixrad,iyrad: ', iyrad, iyrad
print, 'imgfmt> ixdim,iydim: ', ixdim, iydim

IF (KEYWORD_SET (ocenter)) THEN BEGIN
   oxcen = ocenter (0)
   oycen = ocenter (1)
ENDIF ELSE BEGIN			; Center of default image [512,512].
   oxcen = 255.5
   oycen = 255.5
ENDELSE

IF (KEYWORD_SET (oradius)) THEN BEGIN
   oxrad = oradius
   oyrad = oradius
ENDIF ELSE BEGIN			; Default radius.
   oxrad = 224.0
   oyrad = 224.0
ENDELSE

IF (KEYWORD_SET (osize)) THEN BEGIN
   oxdim = osize (0)
   oydim = osize (1)
ENDIF ELSE BEGIN			; Default output size.
   ixdim = 512
   iydim = 512
ENDELSE

print, 'imgfmt> oxcen,oycen: ', oxcen, oycen
print, 'imgfmt> oxrad,oyrad: ', oyrad, oyrad
print, 'imgfmt> oxdim,oydim: ', oxdim, oydim

;--- Center image in array.

ixcen = fix (ixcen)
iycen = fix (iycen)
print, 'imgfmt> ixcen,iycen: ', ixcen, iycen

ixl = ixcen + 1				; x-axis: pixels left  of center. 
ixr = ixdim - ixcen - 1			; x-axis: pixels right of center.

iyb = iycen + 1				; y-axis: pixels below center. 
iyt = iydim - iycen - 1			; y-axis: pixels above center.

print, 'ixl,ixr,(ixl-ixr): ', ixl, ixr, ixl-ixr
print, 'iyb,iyt,(iyt-iyb): ', iyb, iyt, iyb-iyt

;--- Create new array in which solar disk is centered and has the
;    same FOV ratio as the specified output image.
;
;--- Make new dimension based upon Field-Of-View ratio (solar radii / dimension)

xfovout = oxdim / oxrad
yfovout = oydim / oyrad
cixdim = fix (xfovout * ixrad + 0.5)
ciydim = fix (yfovout * iyrad + 0.5)

print, 'xfovout,yfovout,ratio: ', xfovout, yfovout, xfovout/yfovout
print, 'cixdim,ciydim: ', cixdim, ciydim

cibuf = bytarr (cixdim, ciydim)

cixcen = cixdim / 2
ciycen = ciydim / 2
cixbeg = cixcen - ixcen
cixend = cixbeg + ixdim - 1
ciybeg = ciycen - iycen
ciyend = ciybeg + iydim - 1

print, 'size ( ibuf): ', size ( ibuf)
print, 'size (cibuf): ', size (cibuf)
print, 'cixcen,ciycen: ', cixcen, ciycen
print, 'cixbeg,cixend,ciybeg,ciyend: ', cixbeg, cixend, ciybeg, ciyend

cibuf (cixbeg : cixend, ciybeg : ciyend) =  ibuf (*, *)

; --- Resize the input image.

;obuf = congrid (cibuf, oxdim, oydim, 0, /cubic)

obuf = congrid (cibuf, oxdim, oydim, 0)

IF (KEYWORD_SET (mktv) NE 0) THEN TVSCL, obuf		; Plot on screen.

RETURN, obuf

END
