;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: initgeom.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 23, 1998
;
; Last Modified: Thu Feb 26 10:06:11 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO initgeom, im, minint, xc, yc, r, tv=tv, print=print, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       initgeom     (procedure)
;
; PURPOSE:
;       To find an initial estimate of the solar geometry of a full disk
;       image.
;
; CALLING SEQUENCE:
;
;       initgeom, im, minint, xc, yc, r [,help=help]
;
; INPUTS:
;       im - full disk solar image.
;   minint - the minimum intensity value at/near the solar limb.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       xc - x image coordinate of the solar disk center.
;       yc - y image coordinate of the solar disk center.
;        r - average radius of the solar disk.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;         /tv.  Will display fit to graphics window.
;      /print.  Will print results of ellipse fit to screen.
;
; CALLS:
;       delvarx.pro, fitellipse.pro.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written February 23, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 23, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'initgeom'
  RETURN
ENDIF

IF(N_PARAMS() NE 5) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  initgeom, im, minint, xc, yc, r [,help=help]'
  PRINT, ''
  RETURN
ENDIF

; Get image size information.

sz = size(im)

nx = sz(1) & ny = sz(2)


; Create bi-level image, based on the input minimum limb intensity.

wim = im GT minint


; Create sobel edge enhanced image of bi-level image.

sim = sobel(wim)

sim = sim GT 0                  ; make sobel image bi-level


; delete bi-level image to save some memory.

;delvarx, wim


; Avoid edge effects - set first 1% of edge pixels to zero.

dx = fix(nx/100.0) & dy = fix(ny/100.0)
 
sim(*, 0:dy) = 0.0           ; bottom
 
sim(0:dx, *) = 0.0           ; left
 
sim(nx-dx:nx-1, *) = 0.0     ; right
 
sim(*, ny-dy:ny-1) = 0.0     ; top


; Initialize storage arrays.

xpts = [-1.0] & ypts = [-1.0]


; Loop over all pixels in the sobel image.  If the pixel value is greater than
; zero, then record the pixel location in the image as an edge.

FOR i = 0, nx-1 DO BEGIN
   FOR j = 0, ny-1 DO BEGIN
      
      IF(sim(i, j) GT 0) THEN BEGIN
         xpts = [xpts, i]
         ypts = [ypts, j]
      ENDIF
      
   ENDFOR
ENDFOR

; delete sobel image to save some memory.
;delvarx, sim

IF(keyword_set(tv)) THEN tvscl, rebin(im,512,512)

; Transpose the edge coordinate arrays for input into fitellipse.pro.

xpts = transpose(xpts(where(xpts NE -1.0)))
ypts = transpose(ypts(where(ypts NE -1.0)))

scl = 512./nx

IF(keyword_set(tv)) THEN $
   FOR i = 0, n_elements(xpts)-1 DO plots,scl*[xpts(i)],scl*[ypts(i)], $
					  /dev,psym=3

; Perform a least-squares fit of the edge/limb points to an ellipse.

fitellipse, xpts, ypts, coeff, xc, yc, r1, r2, phi, err = err


; Print results.

IF(keyword_set(print)) THEN BEGIN
   print, 'XC', 'YC', 'RX', 'RY', 'PHI', format = '(5(8x, a))'
   print, xc, yc, r1, r2, !radeg*phi, format = '(5(3x, f7.2))'
ENDIF

; Average the ellipse radii.

r = (r1+r2)/2.0

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'initgeom.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
