/* acos_name.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create ACOS FITS output file name.
 * Use SOHO filename format:   OOOO_TTTTR_WS_YYYYMMDD_HHMMSS.fts
 * where OOOO  = Origin
 *       TTTT  = Telescope: DPMx, LOWK, CHIP
 *       R     = Reduction code: x,r,c,l,d,m
 *       W     = Wavelength code: x,h,c,m,w,a,b,r,k,s
 *       S     = Source code: x,s,l,o,d,b,c,f
 *
 *       YYYY  = year
 *       MM    = month
 *       DD    = day
 *
 *       HH    = hour
 *       MM    = minute
 *       SS    = second
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   13 February 1996 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "acos.h"
#include "acoslib.h"

char *
acos_name ()

{
   static char outnam [80]	= "OOOO_TTTTR_WS_YYYYMMDD_HHMMSS.fts";
   static char origin [8]	= "mlso" ;
   static char telescope [8]	= "TTTT" ;
   static char reduce [4]	= "x" ;
   static char wave [4]		= "x" ;
   static char source [4]	= "x" ;
   static char obs_date [12]	= "YYYYMMDD";
   static char obs_time [8]	= "HHMMSS";
   static char ext [8]		= "fts";
   int iyear ;
   int rcode ;
   int wcode ;
   int scode ;

   strcpy (origin, "mlso") ;

   switch (ACOSI->ttype)
      {
      case 0:	strcpy (telescope, "dpm~") ;	break ;
      case 1:	strcpy (telescope, "lowk") ;	break ;
      case 2:	strcpy (telescope, "chip") ;	break ;
      default:	strcpy (telescope, "ZZZZ") ;	break ;
      }

   /*** Set image type and miscellaneous information codes according to the
    *** ACOS image type parameter.
    ***/

   wcode =  ACOSI->dtype / 100 ;
   scode = (ACOSI->dtype - wcode * 100) / 10 ;
   rcode =  ACOSI->dtype - wcode * 100 - scode * 10 ;

   switch (rcode)
      {
      case 0:	strcpy (reduce, "x") ;	break ;		/* Undefined. */
      case 1:	strcpy (reduce, "r") ;	break ;		/* Raw data. */
      case 2:	strcpy (reduce, "c") ;	break ;		/* Calibrated data */
      case 3:	strcpy (reduce, "l") ;	break ;		/* Limb dark removed. */
      case 4:	strcpy (reduce, "d") ;	break ;		/* Ratio image. */
      case 5:	strcpy (reduce, "m") ;	break ;		/* Subtraction image. */
      }

   switch (wcode)
      {
      case 0:	strcpy (wave, "x") ;	break;		/* Undefined. */
      case 1:	strcpy (wave, "h") ;	break;		/* Helium-I: 1083.03nm*/
      case 2:	strcpy (wave, "c") ;	break;		/* Continuum. */
      case 3:	strcpy (wave, "m") ;	break;		/* Helium-I: 1082.91nm*/
      case 4:	strcpy (wave, "w") ;	break;		/* Water vapor. */
      case 5:	strcpy (wave, "a") ;	break;		/* Halpha: 656.2808nm */
      case 6:	strcpy (wave, "b") ;	break;		/* Blue Halpha wing. */
      case 7:	strcpy (wave, "r") ;	break;		/* Red  Halpha wing. */
      case 8:	strcpy (wave, "k") ;	break;		/* K-corona: 657.4 nm */
      case 9:	strcpy (wave, "s") ;	break;		/* Other spectrum. */
      }

   switch (scode)
      {
      case 0:	strcpy (source, "x") ;	break;		/* Undefined. */
      case 1:	strcpy (source, "s") ;	break;		/* Sun: full disk. */
      case 2:	strcpy (source, "l") ;	break;		/* Sun: limb. */
      case 3:	strcpy (source, "o") ;	break;		/* Opal. */
      case 4:	strcpy (source, "d") ;	break;		/* Diffuser. */
      case 5:	strcpy (source, "b") ;	break;		/* Bias (dark). */
      case 6:	strcpy (source, "c") ;	break;		/* Cal. optics. */
      case 7:	strcpy (source, "f") ;	break;		/* Flat field image. */
      }

   /* Ensure that year is a full 4-digit number. */

   iyear = ACOSI->year ;
   if (iyear < 1900)  iyear += 1900 ;

   /* Create date & time strings. */

   sprintf (obs_date, "%04d%02d%02d", iyear, ACOSI->month, ACOSI->day) ;
   sprintf (obs_time, "%02d%02d%02d",
	    ACOSI->hour, ACOSI->minute, ACOSI->second) ;

   /* Create output file name. */

   sprintf (outnam, "%s_%s%s_%s%s_%s_%s.fts",
	    origin, telescope, reduce, wave, source, obs_date, obs_time, ext) ;

   return (outnam) ;
}
