/* circle_perimh.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Circle perimeter: horizontal type.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate the x/y coordinates of the circle perimeter,
 * given the radius of the circle, and the center coordinates of the circle.
 * Each pair of coordinate points (xir,yir) & (xil,yil) correspond to the
 * intersection of a horizontal line passing through the circle.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 13 November 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

circle_perimh (rc, xc, yc, xir, yir, xil, yil, nperim)

int rc;				/* Circle radius (pixel units). */
int xc, yc;			/* Circle center coordinates (pixel units). */
int xir [];			/* X coordinates (right hemisphere). */
int yir [];			/* Y coordinates (right hemisphere). */
int xil [];			/* X coordinates (left hemisphere). */
int yil [];			/* Y coordinates (left hemisphere). */
int *nperim;			/* # of perimeter values in xir/yir/xil/yil. */
{
   int i;			/* Counter for nperim. */
   int xt = 0;			/* x coordinate (right hemisphere). */
   int xb = 0;			/* x coordinate (left hemisphere). */
   int yt = 0;			/* y coordinate (right hemisphere). */
   int yb = 0;			/* y coordinate (left hemisphere). */
   int xylim;			/* x/y loop limit. */

   double rsq;			/* radius squared. */
   double xsq;			/* x coordinate (circle locus) squared. */
   double ysq;			/* y coordinate (circle locus) squared. */
   double xd;			/* x-axis distance. */
   double yd;			/* y-axis distance. */

/*
 *   printf ("circle_perim> rc:%3d  xc:%3d   yc:%3d\n", rc, xc, yc);
 */
   xylim = (int) (((1.41421356 * rc) / 2.0) + 0.5);
   rsq = (double) (rc * rc);

   /*** Circle equation: r^2 = x^2 + y^2 ***/

   /*** Method of determining circle perimeter coordinates:
    ***
    *** Increment x-coordinate one pixel at a time, starting at 0 degrees
    *** and ending at 180 degrees, using a circle-centered coordinate system.
    *** Translate the local x-coordinate into image coordinates (xt).
    *** Compute corresponding y-coordinate (yt).
    *** Also compute the "mirror image" x/y coordinates (xb,yb).
    ***/

   i = -1;

   for (yt = xylim; yt >= -xylim; yt--)
      {
      i++;
      ysq = (double) (yt * yt);
      xsq = rsq - ysq;

      if (ysq != rsq)
         {
         xd = sqrt (xsq);
         xt = (int) (xd + 0.5);
         }
      else
         {
         xd = 0.0;
         xt = 0;
         }

      xir [i] =  xt + xc;	/* X coordinate: right hemisphere, img coord. */
      yir [i] =  yt + yc;	/* Y coordinate: right hemisphere, img coord. */

      xil [i] = -xt + xc;	/* X coordinate: left hemisphere, img.coord.*/
      yil [i] =  yt + yc;	/* Y coordinate: left hemisphere, img.coord.*/

/*
 *      printf ("i:%3d   xt:%3d   yt:%3d   xir:%3d  yir:%3d   xil:%3d   yil:%3d\n",
 *              i, xt, yt, xir [i], yir [i], xil [i], yil [i]);
 */
      }

   for (xt = xylim; xt >= 0; xt--)
      {
      i++;

      xsq = (double) (xt * xt);
      ysq = rsq - xsq;

      if (xsq != rsq)
         {
         yd = sqrt (ysq);
         yt = (int) (yd + 0.5);
         }
      else
         {
         yd = 0.0;
         yt = 0;
         }

      xir [i] =  xt + xc;	/* X coordinate: right hemisphere, img coord. */
      yir [i] =  yt + yc;	/* Y coordinate: right hemisphere, img coord. */

      xil [i] = -xt + xc;	/* X coordinate: left hemisphere, img.coord.*/
      yil [i] =  yt + yc;	/* Y coordinate: left hemisphere, img.coord.*/

/*
 *      printf ("i:%3d   xt:%3d   yt:%3d   xir:%3d  yir:%3d   xil:%3d   yil:%3d\n",
 *               i, xt, yt, xir [i], yir [i], xil [i], yil [i]);
 */
      } 

   for (xt = xylim; xt >= 0; xt--)
      {
      i++;

      xsq = (double) (xt * xt);
      ysq = rsq - xsq;

      if (xsq != rsq)
         {
         yd = sqrt (ysq);
         yt = (int) (yd + 0.5);
         }
      else
         {
         yd = 0.0;
         yt = 0;
         }

      xir [i] =  xt + xc;	/* X coordinate: right hemisphere, img coord. */
      yir [i] = -yt + yc;	/* Y coordinate: right hemisphere, img coord. */

      xil [i] = -xt + xc;	/* X coordinate: left hemisphere, img.coord.*/
      yil [i] = -yt + yc;	/* Y coordinate: left hemisphere, img.coord.*/

/*
 *      printf ("i:%3d   xt:%3d   yt:%3d   xir:%3d  yir:%3d   xil:%3d   yil:%3d\n",
 *              i, xt, yt, xir [i], yir [i], xil [i], yil [i]);
 */
      } 


   *nperim = i + 1;

   return (0);
}
