/* dpmlib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO Digital Prominence Monitor (DPM) header structure definition.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 February 1994
 * 10 Sep 1997: Add DPMI->wcode, DPMI->scode, DPMI->wcode.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*
 *	dpmon_header.h
 *
 *	Header structure for digital Prominence Monitor data records
 *
 *	A dPMon "Image Record" is an array of short integers (16 bit)
 *	1534 rows (x subscript rapidly varying) by 
 *	1030 columns (y subscript, slowly varying).
 *
 *	The first row, 3068 bytes, contains this header structure.
 *	The next 1029 rows each contain 1534 unsigned short integers
 *		of camera image.
 *
 *	The physical format on the tape:
 *		Image Records writen as 103 blocks each 30680 bytes
 *	(2 bytes/short integer) in length.
 *		Image records follow image records.
 *		Before every calibration sequence there is a filemark.
 *		At the end of the day there are two filemarks.
 *
 *	Image normalization:
 *		I recommend normalizing all data to 13 bits (0-8192).
 *	This is the best possible signal to noise achievable with a 
 *	10-bit camera summed 64 times to fill a 16 bit word.
 *	To do the normalization FIRST multiply each pixel value by 8
 *	then divide by the number of exposures summed,the .Nsum element
 *	of the header structure.  The Nsum value will normally be a power
 *	of two so that shifts can be used (check first).
 *
 *	David Elmore
 *
 *	Thu Jan 13 20:01:17 GMT 1994
 */

struct	dpm_header
	{
	int		Version;	/* Program version number */

	int		Year;		/* Date and time UT */
	int		Month;
	int		Day;
	int		Hour;
	int		Minute;
	int		Sec;

	int		Xdim;		/* # of pixels in X */
	int		Ydim;		/* # of pixels in Y */
	int		Nsum;		/* # of exposures summed to form one */
	int		Exposure;	/* Exposure time in hundredths of sec*/
	int		Gain;		/* Camera controller gain factor */
	int		Odisk;		/* O disk diameter in arc-seconds */
	int		Type;		/* DARK, DISK, LIMB, or FLAT */

	int		Optics[8];	/* Positions of all 8 optical devs */
					/* see dev_defs.h for definitions */

	float	Pangle;			/* P angle in degrees */
	float	Bangle;			/* B angle in degrees */
	float	Langle;			/* L angle in degrees */
	float	HA;			/* Hour angle in degrees */
	float	Elevation;		/* Solar elevation in degrees */
	float	Azimuth;		/* Solar azimuth in degrees */
	float	Amass;			/* Airmass */
	float	Radius;			/* Solar radius in arc-seconds 30*4*/

	char	dummy[3068-122];	/* set so that length <= 3068 */
	};


struct dpm_info		/* MLSO Digital Prominence Monitor housekeeping info. */
   {
   int version  ;		/* Version number.		*/
   int dtype	;		/* Data type [1-8]		*/
   int wcode	;		/* Wavelength code [0-9]	*/
   int scode	;		/* Source     code [0-7]	*/
   int rcode	;		/* Reduction  code [0-5]	*/
   int xdim	;		/* Image width			*/
   int ydim	;		/* Image height			*/
   int zdim	;		/* Image depth			*/
   int nbypx	;		/* # bytes/pixel		*/
   int dmin	;		/* Data minimum value		*/
   int dmax	;		/* Data maximum value		*/
   int smin	;		/* Display min value		*/
   int smax	;		/* Display max value		*/
   int year	;		/* Image year			*/
   int month	;		/* Image month			*/
   int day	;		/* Image day (of month)		*/
   int doy	;		/* Image day-of-year		*/
   int hour	;		/* Image hour			*/
   int minute	;		/* Image minute			*/
   int second	;		/* Image second			*/
   int cal_year ;		/* Calibration year.		*/
   int cal_month;		/* Calibration month.		*/
   int cal_day	;		/* Calibration day.		*/
   int nsum	;		/* # images summed (added)	*/
   int expdur	;		/* Exposure duration (sec)	*/
   int gain	;		/* Gain	factor			*/
   int odisk	;		/* Occulting disk (arcsec)	*/
   float bscale ;		/* Brightness scaling factor.	*/
   float bzero  ;		/* Brightness scaling offset.	*/
   float pixrs	;		/* # pixels/Rsun		*/
   float pangle	;		/* P-angle (degrees)		*/
   float bangle	;		/* B-angle (degrees)		*/
   float langle	;		/* L-angle (degrees)		*/
   float hangle	;		/* Hour-angle (degrees)		*/
   float nangle ;		/* Angle (CCW) of north w.r.t. +X axis. */
   float elev   ;		/* Solar elevation (degrees)	*/
   float azimuth;		/* Solar azimuth (degrees)	*/
   float airmass;		/* Airmass.			*/
   float rsun   ;		/* Solar radius (arcsec)	*/
   float xcen	;		/* X-axis sun center		*/
   float ycen	;		/* Y-axis sun center		*/
   float rmin   ;		/* Inner FOV radius (Rsun)      */
   float rmax   ;		/* Outer FOV radius (Rsun)      */
   char optdev [8][16];		/* Names of optical devices	*/
   char optpos [8][16];		/* Positions of optical devices	*/
   } ;

#ifdef DPM

   struct dpm_header          DPMHS ;
   struct dpm_header *DPMH = &DPMHS ;

   struct dpm_info          DPMIS ;
   struct dpm_info *DPMI = &DPMIS ;

#else

   extern struct dpm_header  DPMHS ;
   extern struct dpm_header *DPMH ;

   extern struct dpm_info  DPMIS ;
   extern struct dpm_info *DPMI ;

#endif

