/* makesmm.c
 * *****************************************************************************
 * Read SMM C/P image from disk to memory array.
 * *****************************************************************************
 * Andrew L. Stanger
 * 1 September 1987: Accomodate high resolution and 4-quad images.
 * 14 February 1990: Determine bppx from hskp byte # 103 for all images.
 * 12 April 1990: determine dtype from hskp bytes 221 & 222.
 *  2 May 1990: hk + bitmap reside in one record for floating point images.
 * 31 January 1991: refine xcen & ycen computation.
 * *****************************************************************************
 */

#include <stdio.h>
#include <math.h>
#include "iproc.ext"

#define	MAXHSK	224

char *malloc();

int	nres;		/* Resolution			*/

float	pixrs;		/* # Pixels per solar radius. */
float	rspix;		/* # Solar radii per pixel. */
float	degrad;		/* # Degrees per solar radius. */
float	raddeg; 	/* # Solar radii per degree. */


struct IMAGE *
makesmm (filename)

char *filename;
{
   FILE *imgfd, *fopen();
   struct IMAGE *ifcb;
   int imgsiz, nread;
   int nrechd;			/* # records for header. */
   long offset;
   static char itype[] = "SMM";
   static char errbuf [80];
	
   /*** Allocate space for the image file control block. ***/

   ifcb = (struct IMAGE *) malloc (sizeof (struct IMAGE));
   ifcb->name = (char *) malloc (strlen (filename) + 1);
   ifcb->type = (char *) malloc (strlen (itype) + 1);
   ifcb->addr = NULL;

   strcpy (ifcb->type, itype);
   strcpy (ifcb->name, filename);

   if ((imgfd = fopen (filename, "r")) == NULL)
      return (NULL);
   sprintf (errbuf, "makesmm> filename: %s", filename);
   disp_err (errbuf);

   /*** Decommutate Housekeeping. ***/

   if (smmhk (ifcb, imgfd))  return (NULL);

   /*** Offset of polar axis relative to screen horizontal (+x axis). ***/

   ifcb->degnor = 133.0 + (float) ifcb->roll;

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

   if (ifcb->bppx == 4)
      nrechd = 1;
   else
      nrechd = 2;
   offset = (long) (ifcb->dx * ifcb->bppx * nrechd);
   if (lseek (fileno(imgfd), offset, 0) == -1)
      {
      fprintf (stderr, "makesmm(): seek error.\n");  
      return (NULL); 
      }
   imgsiz = ifcb->dx * ifcb->dy * ifcb->bppx;
   sprintf (errbuf, "makesmm> dx: %d  dy: %d  bppx: %d  imgsiz: %d",
            ifcb->dx, ifcb->dy, ifcb->bppx, imgsiz);
   disp_err (errbuf);

   /*** Allocate space for the image. ***/

   if ((ifcb->addr = (unsigned char *) malloc (imgsiz)) == NULL)
      { 
      sprintf (errbuf, "makesmm(): memory allocation error.");
      disp_err (errbuf);
      return (NULL);
      }
   sprintf (errbuf, "makesmm> ifcb->addr: %d", ifcb->addr);
   disp_err (errbuf);

   /***
    *** Read image into memory.
    *** Swap bytes if image is 2 bytes/pixel and in DEC byte order.
    ***/

   if ((nread = read (fileno(imgfd), ifcb->addr, imgsiz)) < imgsiz)
      {
      sprintf (errbuf, "bytord: %d   imgsiz: %d   nread: %d\n",
	       ifcb->bytord, imgsiz, nread);
      disp_err (errbuf);
      if (ifcb->bppx == 2 && ifcb->bytord == 1)
	 swab (ifcb->addr, ifcb->addr, imgsiz);
      return (NULL); 
      }

   if (fclose(imgfd) == -1)
      { 
      fprintf (stderr, "makesmm> file close error.%s\n", ifcb->name); 
      return (NULL);
      }

   /*** Define the default display area on a 512x512 image display screen. ***/

   _GD.y  = ifcb->dy+64*nres-1;	/* initial y pixel on the display screen    */
   _GD.x  = ifcb->ir+64*nres-1;	/* initial x pixel on the display screen    */
   _GD.dx = ifcb->dx;	/* image extent in x pixels 		    */
   _GD.dy = ifcb->dy;	/* image extent in y pixels		    */
   sprintf (errbuf, "GD.x: %d  GD.y: %d  GD.dx: %d  GD.dy: %d",
            _GD.x, _GD.y, _GD.dx, _GD.dy);
   disp_err (errbuf);

   return ((struct IMAGE *) ifcb);
}


/* smmhk (ifc)		-a relative of cphk.c.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Routine to decode housekeeping and put result into
 * image file control structure ifc.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Written by Rainer Illing   13 December 1983
 * Revised by Andrew Stanger  28 December 1983
 * Updated to compute ip,ir,lp,lr and look up disk center:  27 January 1984
 * Add definition of scaling variables:  30 January 1984.
 * Open and close image locally:  22 February 1984.
 * Remove call to 'hkshow':  24 February 1984
 * Rename include files:  27 February 1984.
 * Read into character array, then copy to integer array: 20 March 1984.
 * Change open, read, close to  fopen, fread, fclose: 25 April 1984.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Revised by Kevin Cole      8 August   1984
 * Made this routine part of the image processing library (iproc.a). 
 * 22 February 1985: Add roll.
 * 25 March 1986: Change south & west centers in ww6cen for Joan Burkepile.
 * 20 October 1989: add bytord field to IMAGE structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

smmhk (ifc, imgfd)	/* SMM C/P Housekeeping Decoder */

struct IMAGE *ifc;	/* Image file name. */
FILE *imgfd;		/* Pointer to image file. */

{
FILE *tfd;
static char tname[80];
int dtype, ntype;
static int idt [12];
static	int	lih[4], liv[4];		/* Binary components of IH & IV. */

/* Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). */
/* (No geometric correction applied.) */

static	float	vidcen[9][2] = { 36.3, 499.8,	/* sw */
				213.1, 193.6,	/* s  */
				539.9,   3.3,	/* se */
				820.0,  87.0,	/* e  */
				892.4, 361.5,	/* ne */
				718.6, 679.3,	/* n  */
				386.7, 856.4,	/* nw */
				104.3, 792.1,	/* w  */
				447.5, 447.5};	/* n+e+s+w */

/* Disk centers (vs. Sector Mirror) for calibrated data (C34520). */
/* (Processed with geometric correction WW6COR). */

static	float	ww6cen[9][2] = { 47.9, 507.5,	/* sw */
				216.9, 205.6,	/* s   Joan: 25 Mar 86 */
				529.7,  40.3,	/* se */
				814.6,  85.4,	/* e  */
				884.0, 353.4,	/* ne */
				714.8, 671.1,	/* n  */
				393.2, 824.1,	/* nw */
				123.1, 802.4,	/* w   Joan: 25 Mar 86 */
				447.5, 447.5};	/* n+e+s+w */

static	float	secrad = 961.;	/* arcseconds per solar radius. */
static	float	secpix = 5.933; /* arcseconds per pixel (Hi res). */

float	pi = 3.141592654;
float	radrol, rollwt;

int	i, k, yrref, doyref;
int	kres, ihs, ivs, ih, iv, jsm;
int	lenimg, iext;
int	roll, roll1, roll2;
static	int	h[MAXHSK];	/* Housekeeping array (integer). */

char	*c;		/* Housekeeping array (character). */
char	*malloc();	/* system allocation routine. */
int	isign;		/* Sign bit for roll. */
int	scroll;		/* Roll to closest 10 degrees. */
/* .......................................................................... */

/*** Allocate storage for housekeeping array. ***/

   if ((c = (char *) malloc(MAXHSK)) == NULL) 
      {
      fprintf (stderr, "smmhk> malloc error.\n") ;
      fflush (stderr);
      (void) fclose (imgfd);
      return (NULL);
      }
   if (fread (c, sizeof(char), MAXHSK, imgfd) == NULL) 
      { 
      (void) fclose (imgfd); 
      free (c);
      return (NULL); 
   }

/*** Copy housekeeping from character*1 to integer*4 array. ***/

   for (i = 0; i < MAXHSK; i++)  *(h + i) = *(c + i) & 0377;

   free (c);

/*** Year and Day-of-Year. ***/

   ifc->year = *h;		/* year */
   ifc->doy = *(h+1) << 8;
   ifc->doy += *(h+2);		/* day-of-year */

/*** Polaroid. ***/

   ifc->ipw = (*(h+3) & 0300) >> 6;

/*** Resolution. ***/

   ifc->ires = (*(h+3) & 04) >> 2;

/*** IH,IV. ***/

   ih = (*(h+9) & 0360) >> 4;
   iv = (*(h+9) & 017);

   lih[0] = (ih & 010) >> 3;
   lih[1] = (ih &  04) >> 2;
   lih[2] = (ih &  02) >> 1;
   lih[3] = (ih &  01);

   liv[0] = (iv & 010) >> 3;
   liv[1] = (iv &  04) >> 2;
   liv[2] = (iv &  02) >> 1;
   liv[3] = (iv &  01);

/*** Exposure. ***/

   ifc->exp = (*(h+10) & 0377)<<4;
   ifc->exp += (*(h+11) & 0360)>>4;

/*** Roll. ***/

   roll1 = (int) (*(h+11) & 017);
   roll2 = (int) (*(h+12) & 0377);
   isign =  (roll1 & 8) / 8;

   /***... 1980: Least significant roll bit = .111906 degrees. ...***/

   if (ifc->year == 80)
      {
      roll = (int) ((roll1 & 7) * 256 + (roll2 & 0377));
      radrol = (float) roll;
      if (isign == 1)  radrol = - (float) (((~(roll)+1) & 03777));
      rollwt = 0.111906;
      }

   /***... 1984-89: Least significant roll bit = 7.16197 degrees. ...***/

   if (ifc->year > 80)
      {
      roll = (int) ((roll1 & 7) * 4 + (roll2 & 0300) / 64);
      radrol = (float) roll;
      if (isign == 1)  radrol = - (float) (((~(roll)+1) & 037));
      rollwt = 7.16197;
     }

   roll = (int) (radrol * rollwt);

/*** For 1984-89, set roll to closest 10 degrees. ***/

   scroll = roll;
   if (ifc->year > 80)
      {
      if (roll > 0)  scroll += 5;
      scroll = (scroll / 10) * 10;
      }
   ifc->roll = scroll;
   ifc->degnor = 133.0 + (float) scroll;

/*** Green line position and stow. ***/

   ifc->iglpos = *(h+13) & 0377;
   ifc->iglstw = (*(h+14) & 0300) >> 6;

/*** Sector mirror. ***/

   jsm = (*(h+14) & 074) >> 2;
   ifc->ism = (jsm-1)/2;		/* Decode to array index. */
   if (jsm == 0) ifc->ism = 8;		/* Special case for NESW. */

/*** Calibration mirror. ***/

   ifc->icm = (*(h+15) & 060) >> 4;

/*** Filter wheel. ***/

   ifc->ifw = ((*(h+15) & 016) >> 1) - 1;	/* Decode to array index. */

/*** Hours and minutes. ***/

   ifc->hour = *(h+20);
   ifc->minute = *(h+21);
   ifc->sec = *(h+24);

/*** Byte order: 0 -> Sun Microsystems, 1 -> DEC. ***/

   ifc->bytord = 0;
   if (*(h+52) > 0)  ifc->bytord = 1;

/*** Quantity. ***/

   ifc->quan = *(h+79) & 0377;

/*** Bytes per pixel. ***/

   ifc->bppx = *(h+103) / 8;

/*** Frame number (operational status) ***/

   ifc->frameno = ((*(h+30) & 0377) << 4) + ((*(h+31) & 0360) >> 4);

/*** Reformat date. ***/

   yrref = *(h+55);
   doyref = (*(h+62)) << 8;
   doyref += *(h+63);

   free (h);

/*** Compute start/end pixel/line coordinates for C/P image. ***/

   kres = ifc->ires + 1;
   ifc->ip = 0;
   ifc->lp = 895 / kres;
   ifc->ir = 0;
   ifc->lr = 895 / kres;
   ihs = 0;
   ivs = 0;

   for (i=0; i<4; i++)
      {
      if (lih[i] == 0 && ihs == 0)  ifc->ip += 224 / kres;
      if (liv[i] == 0 && ivs == 0)  ifc->ir += 224 / kres;
      if (lih[i] == 1)  ihs = 1;
      if (liv[i] == 1)  ivs = 1;
      if (lih[i] == 0 && ihs == 1)  ifc->lp -= 224 / kres;
      if (liv[i] == 0 && ivs == 1)  ifc->lr -= 224 / kres;
      }

   ifc->dx = ifc->dy = ifc->lp - ifc->ip + 1;

   nres = 1;
   if(ifc->ires == 0)  nres = 2;

/*** DTYPE: data type.
 * =1 raw vidicon data (DN units, 8 bits).
 * =2 Equivalent seconds * 5, 8 bits.
 * =3 Calibrated data (10^-11 B/Bsun units, 16 bits). 
 * Data type is inferred from the first letter of the filename extension.
 *
 * A C/P filename has the following format :
 * ydddhhmms.vxx :	type 1
 *          .txx :	     2
 *          .cxx :	     3
 * where y = year -1980.
 *       ddd = day-of-year, hh = hour, mm = minute, s = sec/10.
 *       x = arbitrary character.
 */

   lenimg = strlen(ifc->name);
   iext = -1;
   for (i=0; i<lenimg; i++) if(ifc->name[i] == '.') iext = i+1;

/* SMM pixel values come in three flavors; calibrated [b/bsun],
 * equivalent seconds and raw video data
 */

/*
 *   ifc->dtype = 3;
 *   if (ifc->name[iext] == 'v' || ifc->name[iext] == 'V')  ifc->dtype = 1;
 *   if (ifc->name[iext] == 't' || ifc->name[iext] == 'T')  ifc->dtype = 2;
 */
   if (ifc->dtype == 3)		/* calibrated intensity 	*/
      ifc->scale = 1.e-11;	/* scale factor B/BSUN	 	*/
   if (ifc->dtype == 2)		/* equivalent seconds		*/
      ifc->scale = 1.0;		/* scale factor 		*/
   if (ifc->dtype == 1)		/* raw video data    		*/
      ifc->scale = 1.0;		/* scale factor 		*/

   idt [0]   = *(h + 221) &  001;/* DN values (0-255). */
   idt [1]   = *(h + 221) &  002;/* Equivalent Second values (10-bit log). */
   idt [2]   = *(h + 221) &  004;/* Calibrated radiance (B/Bsun). */
   idt [3]   = *(h + 221) &  010;/* Density (elec/cm**2). */
   idt [4]   = *(h + 221) &  020;/* difference image (floating point). */
   idt [5]   = *(h + 221) &  040;/* Bk [K-corona] (B/Bsun). */
   idt [6]   = *(h + 221) & 0100;/* (Bcme-Bbkg)/Bk   (floating point ratio). */
   idt [7]   = *(h + 221) & 0200;/* (Bcme-Bbkg)/Bbkg (floating point ratio). */

   idt [8]   = *(h + 222) &  001;/* Polarization data (floating point 0-1.0). */
   idt [9]   = *(h + 222) &  002;/* pB (B/Bsun). */
   idt [10]  = *(h + 222) &  004;/* B radial + tangential (B/Bsun). */
   idt [11]  = *(h + 222) &  010;/* B tangential (B/Bsun). */
   idt [12]  = *(h + 222) &  020;/* B radial (B/Bsun). */
   idt [13]  = *(h + 222) &  040;/* undefined data type. */
   idt [14]  = *(h + 222) & 0100;/* undefined data type. */
   idt [15]  = *(h + 222) & 0200;/* undefined data type. */

   ifc->str =  *(h + 223) &  001;    /* Stray Light flag (=1: removed). */
   ifc->vig = (*(h + 223) &  002)/ 2;/* Vignetting flag (=1: removed). */
   ifc->dsk = (*(h + 223) &  004)/ 4;/* Disk regstration flag (=1: corrected).*/
   ifc->fc  = (*(h + 223) &  010)/ 8;/* F-corona flag (=1: removed). */

   ntype = 0;
   for (i=0; i<12; i++)  if (idt[i] != 0)
      {
      ntype++;
      dtype = i + 1; 
      }
/*
 *   if (ntype > 1)
 *      fprintf (stderr, "makesmm> more than one data type specified.\n"); 
 */
   ifc->dtype = dtype;
   if (*(h + 220) != 0)
      ifc->scale = pow ((double) 10.0, (double) (-1 * *(h + 220)));
   else
      ifc->scale = 1.0;

/*
   sprintf (tname, "%s.typ", ifc->name);
   tfd = fopen (tname, "w");
   for (i=0; i<224; i++)  fprintf (tfd, "smmhk> hskp[%d]: %d\n",
				   i, *(h + i));
   fprintf (tfd, "smmhk> hskp[220]: %d   hskp[221]: %d   hskp[222]: %d\n",
	    *(h + 220), *(h + 221), *(h + 222));
   fprintf (tfd, "smmhk> %s   dtype: %d   ifc->dtype: %d   ifc->scale: %f\n",
      ifc->name, dtype, ifc->dtype, ifc->scale);
   for (i=0; i<9; i++)  fprintf (tfd, "smmhk> idt[%d]: %d\n", i, idt[i]);
   fprintf (tfd, "smmhk> ntype: %d\n", ntype);
   fclose (tfd);
 */

/*
 *   if (ifc->year == 80)
 *      {
 *      ifc->bppx = 2;
 *      if (ifc->dtype == 1 || ifc->dtype == 2)  ifc->bppx = 1;
 *      }
 */

/*** Look up disk center location according to sector mirror. ***/

/*** Raw DN images : (no geometric distortion correction) ***/
   if (ifc->dtype == 1)
      {
      ifc->xcen = ((vidcen[ifc->ism][0] + 0.5) / (float) kres) - 0.5;
      ifc->ycen = ((vidcen[ifc->ism][1] + 0.5) / (float) kres) - 0.5;
      }

/* Equivalent Second and Calibrated images :
 * (SMMCOR or WW6COR geometric corrections)
 */

   if (ifc->dtype != 1)
      {
      ifc->xcen = ((ww6cen[ifc->ism][0] + 0.5) / (float) kres) - 0.5;
      ifc->ycen = ((ww6cen[ifc->ism][1] + 0.5) / (float) kres) - 0.5;
      }

/*** Set up scale variables. ***/

   raddeg = pi / 180.;
   degrad = 180. / pi;
   pixrs = secrad / (secpix * kres);
   if (ifc->ism == 8)  pixrs *= 0.5;
   ifc->pixsr = pixrs;
   rspix = 1.0 / pixrs;

   return (0);
}


/*------------------------------------------------------------------------------
 * G E T H K ()		- returns the housekeeping vector from an SMM image
 *			  or NULL if an error occurs.
 * 	This routine must be declared as 
 *		int *gethk()
 * 	in the calling program.
 *------------------------------------------------------------------------------
 */

char *gethk(fcb)

struct IMAGE *fcb;
{
   char *c;
   int i;
   FILE *fopen(), *imgfd;
   static char housekeeping[896*2];

   /*** Open image file. ***/

   if ((imgfd = fopen (fcb->name, "r")) == NULL)
      return (NULL);

   if ((c = (char *) malloc (2 * fcb->dx * fcb->bppx)) == NULL) 
      {
      fprintf (stderr,"smmhk> malloc error.\n") ;
      fflush (stderr);
      (void) fclose (imgfd);
      return (NULL);
      }
   if (fread (c, sizeof (char), 2 * fcb->dx * fcb->bppx, imgfd) == NULL) 
      { 
      (void) fclose (imgfd); 
      free (c);
      return (NULL); 
      }

/*** Copy housekeeping from character*1 to integer*4 array. ***/

   for (i = 0; i < 2 * fcb->dx * fcb->bppx; i++)
      *(housekeeping + i) = *(c + i) & 0377;

   free (c);
   fclose (imgfd);
   return (housekeeping);
}


/*------------------------------------------------------------------------------
 * G P U T F I L E ( )
 *------------------------------------------------------------------------------
 */

gputfile (icb, filename)

struct IMAGE *icb;
char *filename;
{
   char *hsk;
   FILE *fopen(), *fd;
	
   hsk = gethk (icb);

   if ((fd = fopen (filename,"w")) == NULL)
      {
      return (-1);
      }
   else
      {
      write (fileno (fd), hsk, 2 * icb->dx * icb->bppx);
      write (fileno(fd), icb->addr, icb->dx * icb->dy * icb->bppx);
      }
   fclose (fd);
}
