/* glmk3_north.c
/*-----------------------------------------------------------------------------
 * Draws a north pointer with center (screen coordinates) at (x,y),
 * circle radius cirrad, tip radius tiprad.
 * For MLSO Mark-III images.
 *
 * If the coordinates fall outside the displayable screen area, the 
 * coordinates are clipped to the screen boundary.
 *
 * externals required;
 *    mk3_coord ()	- Mark-III polar <-> cartesian coordinate conversion.
 *    gl_wvec ()	- draw a vector on the display screen.
 *    gMINROW, gMAXROW  - integers. Minimum and maximum vertical pixels.
 *    gMINCOL, gMAXCOL  - integers. Minimum and maximum horizontal pixels.
 *-----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   10 January 1994
 *-----------------------------------------------------------------------------
 */
#include <stdio.h>
#include "/home/stanger/lib/include/mk3lib.ext"
#include "/home/stanger/lib/include/img_scale.ext"

glmk3_north (x, y, cirrad, tiprad, cindex)

int x, y;			/* cross hair center position. */
float cirrad;			/* Circle radius (Rsun). */
float tiprad;			/* Pointer tip radius (Rsun). */
int cindex;			/* color LUT index. */
{
   float r, th;
   float xx, yy;
   int xg, yg;		/* Inner circle. */
   int xcn, ycn;	/* North of center. */
   int xcs, ycs;	/* South of center. */
   int xce, yce;	/* East of center. */
   int xcw, ycw;	/* West of center. */
   int xpn, ypn;	/* North pointer tip. */
   int xwe, ywe;	/* Left wing tip. */
   int xww, yww;	/* Right wing tip. */

   /*** North pointer tip coordinates. ***/

   r = tiprad;
   th = 0.0;
   mk3_coord (&r, &th, &xx, &yy, 1, 0);
   xpn = (xx + 0.0) + gMINROW;
   ypn = (yy + 0.0) + gMINCOL;

   /*** East wing tip coordinates. ***/

   r = tiprad;
   th = 90.0;
   mk3_coord (&r, &th, &xx, &yy, 1, 0);
   xwe = (xx + 0.0) + gMINROW;
   ywe = (yy + 0.0) + gMINCOL;

   r = cirrad;
   mk3_coord (&r, &th, &xx, &yy, 1, 0);
   xce = (xx + 0.0) + gMINROW;
   yce = (yy + 0.0) + gMINCOL;

   /*** West wing tip coordinates. ***/

   r = tiprad;
   th = 270.0;
   mk3_coord (&r, &th, &xx, &yy, 1, 0);
   xww = (xx + 0.0) + gMINROW;
   yww = (yy + 0.0) + gMINCOL;

   r = cirrad;
   mk3_coord (&r, &th, &xx, &yy, 1, 0);
   xcw = (xx + 0.0) + gMINROW;
   ycw = (yy + 0.0) + gMINCOL;

   /*** Draw dots surrounding disk center pixel. ***/

   r = 0.0;
   th = 0.0;
   mk3_coord (&r, &th, &xx, &yy, 1, 0);
   xg = (xx + 0.0) + gMINROW;
   yg = (yy + 0.0) + gMINCOL;
   if (xg  >= gMINCOL && xg  <= gMAXCOL && yg  >= gMINROW && yg  <= gMAXROW)
      {
      gl_wvec (xg-1, yg,   xg-1, yg,   cindex);
      gl_wvec (xg+1, yg,   xg+1, yg,   cindex);
      gl_wvec (xg,   yg-1, xg,   yg-1, cindex);
      gl_wvec (xg,   yg+1, xg,   yg+1, cindex);
      }

   /*** Draw circle around disk center. ***/

   r = cirrad;
   for (th=0.0; th<=360.0; th += 5.0)
      {
      mk3_coord (&r, &th, &xx, &yy, 1, 0);
      xg = (xx + 0.0) + gMINROW;
      yg = (yy + 0.0) + gMINCOL;
      if (xg  >= gMINCOL && xg  <= gMAXCOL && yg >= gMINROW && yg <= gMAXROW)
         gl_wvec (xg, yg, xg, yg, cindex);
      }

   /*** Draw north pointer. ***/

   /* North pointer tip to east wing tip. */

   if (xpn >= gMINCOL && xpn <= gMAXCOL && ypn >= gMINROW && ypn <= gMAXROW &&
       xwe >= gMINCOL && xwe <= gMAXCOL && ywe >= gMINROW && ywe <= gMAXROW)
      gl_wvec (xpn, ypn, xwe, ywe, cindex);

   /* Circle to east wing tip. */

   if (xce >= gMINCOL && xce <= gMAXCOL && yce >= gMINROW && yce <= gMAXROW &&
       xwe >= gMINCOL && xwe <= gMAXCOL && ywe >= gMINROW && ywe <= gMAXROW)
      gl_wvec (xce, yce, xwe, ywe, cindex);

   /* North pointer tip to west wing tip. */

   if (xpn >= gMINCOL && xpn <= gMAXCOL && ypn >= gMINROW && ypn <= gMAXROW &&
       xww >= gMINCOL && xww <= gMAXCOL && yww >= gMINROW && yww <= gMAXROW)
      gl_wvec (xpn, ypn, xww, yww, cindex);

   /* Circle to west wing tip. */

   if (xcw >= gMINCOL && xcw <= gMAXCOL && ycw >= gMINROW && ycw <= gMAXROW &&
       xww >= gMINCOL && xww <= gMAXCOL && yww >= gMINROW && yww <= gMAXROW)
      gl_wvec (xcw, ycw, xww, yww, cindex);

   return (0);
}
