/* lumadj.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Correction for solar luminance due to Earth's elliptical orbit about the sun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 12 February 1993
 *  1 August 1994: fix iy adjustment.
 *  3 August 1994: compute eccentric anomaly angle.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h> 

float
lumadj (iyear, idoy, ihour)

int iyear;		/* Year. */
int idoy;		/* Day of year. */
int ihour;		/* Hour. */

{
   int iy;

   double A = 1.0;		/* Semi-major axis of ellipse (AU units). */
   double ecc = 0.0167;		/* Eccentricity of Earth's orbit. */
   double G = 6.672e-11 ;	/* Gravitational constant: m^3/(kg * sec^2) */
   double E = 0.0 ;		/* Eccentric anomaly. */
   double Eang = 0.0 ;		/* degrees. */
   double E0, E1, E2 ;
   double dE0, dE1 ;
   double M = 0.0 ;		/* Mean anomaly. */
   double Mang = 0.0 ;
   double M0, M1 ;
   double Ms = 1.9891e30 ;	/* Mass of sun (kg). */
   double Me = 0.0 ;		/* Mass of earth (kg). */
   double a = 1.496e+11;	/* Semi-major axis of ellipse (meters). */
   double u  = 0.0 ;		/* u = G * (Mass[sun] + Mass[earth]) */
   double n   = 0.0 ;		/* mean angular velocity. */
   double period = 365.2596;	/* Earth's orbital period. */
   double PI = 3.14159265;	/* PI. */
   double radius;		/* Orbital radius (earth to sun). */
   double theta;		/* Angle of Earth-sun line w.r.t. perihelion. */
   double doyo;			/* Day-of-year of observation. */
   double doyp;			/* Perihelion day-of-year. */

   /* Perihelion DOY & hour. */

   /* year:                   80  81  82  83  84  85  86  87  88  89 */
   static float Pdoy  [10] = { 3,  2,  4,  2,  3,  3,  2,  4,  4,  1};
   static float Phour [10] = {15,  2, 11, 16, 22, 20,  5, 23,  0, 22};

   iy = iyear;
   if (iy > 1900)  iy -= 1900;
   if (iy >= 80)   iy -=   80;

/*
   fprintf (stdout, "lumadj> iyear:%2d  idoy:%3d  ihour:%2d  iy:%2d  Pdoy:%6.2f   Phour:%6.2f\n",
	            iyear, idoy, ihour, iy, Pdoy [iy], Phour [iy]) ;
 */

   doyp = (double) (Pdoy [iy] + Phour [iy] / 24.0);

   doyo = (double) (idoy + ihour / 24.0);

   fprintf (stdout, "lumadj> doyp:%6.2f  doyo:%6.2f", doyp, doyo) ;

   /*** Compute u = G * {Mass [sun] + Mass [earth]} */

   Me = Ms / 332946.0 ;
   u = G * (Ms + Me) ;				/* m^3 / sec^2 */

   /*** Compute n (mean angular velocity). */

   n = sqrt (u / (a * a * a)) ;			/* radians/sec */

/*
   fprintf (stdout, "lumadj> G:%8.2e  Ms:%8.2e  Me:%8.2e  u:%8.2e  a:%8.2e  n:%8.2e\n",
	    G, Ms, Me, u, a, n) ;
 */

   n *= 86400.0 ;				/* radians/day */

   /*** Compute M (mean anomaly). ***/

   M = n * (doyo - doyp) ;			/* radians */

   /*** Compute E (eccentric anomaly). */

   E0 = M ;
   M0 = E0 - ecc * sin (E0) ;
   dE0 = (M - M0) / (1.0 - ecc * cos (E0)) ;

   E1 = E0 + dE0 ;
   M1 = E1 - ecc * sin (E1) ;
   dE1 = (M - M1) / (1.0 - ecc * cos (E1)) ;

   E2 = E1 + dE1 ;

   E = E2 ;
   theta = E2 ;

/*
   fprintf (stdout, "lumadj> M:%8.2e  E0,1,2: %8.2e %8.2e %8.2e\n",
	    M, E0, E1, E2) ;
 */
   Eang = E * (180.0 / PI) ;
   Mang = M * (180.0 / PI) ;
   fprintf (stdout, "        M:%8.3f  E:%8.3f\n", Mang, Eang) ;

   /*** Angle w.r.t. perihelion. ***/

/*
   theta = (2.0 * PI * (doyo - doyp)) / period;
 */

   radius = (A * (1.0 - ecc*ecc)) / (1.0 + ecc * cos (theta));

   /*** Since luminosity diminishes as 1/r^2, correction is r^2. ***/

   return (radius * radius);
}
