/* sunrad.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Computes Solar radius size in arcseconds, correcting for elliptical orbit. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 1 November 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h> 

float
sunrad (iyear, idoy, ihour)

int iyear;		/* Year. */
int idoy;		/* Day of year. */
int ihour;		/* Hour. */

{
   int iy;

   double AU = 1.49597870e+8 ;	/* Astronomical Unit (km). */
   double A = 1.0;		/* Semi-major axis of ellipse (AU units). */
   double ecc = 0.01675;	/* Eccentricity of Earth's orbit. */
   double period = 365.2596;	/* Earth's orbital period. */
   double PI = 3.14159265;	/* PI. */
   double rsun;			/* Solar Radius (AU units). */
   double rsin;			/* Solar Radius (sine). */
   double rsunkm = 6.96e+5 ;	/* Solar Radius (km units). */
   double esdist;		/* Earth-Sun distance (AU units). */
   double esdistkm;		/* Earth-Sun distance (km units). */
   double theta;		/* Angle of Earth-sun line w.r.t. perihelion. */
   double doyo;			/* Day-of-year of observation. */
   double doyp;			/* Perihelion day-of-year. */


   /* Perihelion DOY & hour (as a function of year: 80-89). */

   static float Pdoy  [10] = { 3,  2,  4,  2,  3,  3,  2,  4,  4,  1};
   static float Phour [10] = {15,  2, 11, 16, 22, 20,  5, 23,  0, 22};

   /*** Convert year to array index. ***/

   iy = iyear;
   if (iyear > 1900)  iy -= 1900;
   if (iyear >= 80)   iy -=   80;

   doyp = (double) (Pdoy [iy] + Phour [iy] / 24.0);	/* perihelion DOY. */

   doyo = (double) (idoy + ihour / 24.0);		/* image DOY. */

   theta = (2.0 * PI * (doyo - doyp)) / period;	/* Angle w.r.t. perihelion. */

   esdist = (A * (1.0 - ecc*ecc)) / (1.0 + ecc * cos (theta));

   esdistkm = esdist * AU ;		/* Earth-Sun distance (km) */

   rsin = (rsunkm / esdistkm) ;		/* sine (E-S angle) radians */

   rsun = asin (rsin) * (180.0 / PI) * 3600.0 ;		/* Arcseconds */
   
   printf ("sunrad> iy:%2d  doyo:%8.4f  theta:%8.4f  esdist:%8.4f  rsun:%8.4f\n",
	    iy, doyo, theta, esdist, rsun);

   return ((float) rsun);
}
