;+ --------------------------------------------------------------------------- +
; NAME:  
;	MK3COLOR
;
; PURPOSE:   
;	Creates arrays for MK3 black and white and MK3 differcence color tables
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
;	MK3COLOR, MAXCOLOR=MAXCOLOR,
;		  DIFF_R=DIFF_R, DIFF_G=DIFF_G, DIFF_B=DIFF_B, $
;		  NORM_R=NORM_R, NORM_G=NORM_G, NORM_B=NORM_B
;
; CALLED BY:  
;	MK3.PRO
;
; CALLS TO:  
;	none
;
; INPUTS: 
;	none
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS: 
;	DIFF_R = red color array for difference images
;	DIFF_G = green array for difference images
;	DIFF_B = blue  array for difference images
;
;	NORM_R = red color array for normal (not difference) images
;	NORM_G = green color array for normal (not difference) images
;	NORM_B = blue color array for normal (not difference) images
;
; OPTIONAL OUTPUTS: none
;
; COMMON BLOCKS: 
;	none, but once the outputs are returned, they are in common MK3COLORS.
;
; SIDE EFFECTS: 
;	Temporarily creates a small IDL window. Users will see it "blip"
;	on the screen.  Necessary to obtain a corrrect value in !D.N_COLORS.
;
; RESTRICTIONS:
;	Must have X window system graphic device.
;
; PROCEDURE:
;	Create color array of size !D.N_COLORS. For normal (not difference)
;	images, the color table is linear black to white, with white at the
;	top index and red one index below that. For difference images,
;       the color table goes from blue to black (mid point index) to red
;       to yellow, with black at lowest index and white at maximun index.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;- --------------------------------------------------------------------------- -

pro mk3color, maxcolor=maxcolor, diff_r=diff_r, diff_g=diff_g, diff_b=diff_b, $
	     norm_r=norm_r, norm_g=norm_g, norm_b=norm_b

 ; Get # colors not used by the system

 window, /free, xsize=10, ysize=10 & wdelete, !d.window

 print, !d.n_colors
 maxcolor = !p.color - 5

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

 ; Normal color table, black and white with white line at max color index 
 ; and red line at max index minus 1

 norm_r = indgen (!d.n_colors)			; colors to max available
 norm_g = norm_r
 norm_b = norm_r

 ; Grey at max value.  Used for plotting labels.

 norm_r (!p.color) = 128
 norm_g (!p.color) = 128
 norm_b (!p.color) = 128

 ; Blue :

 norm_r (!p.color-1) =   0
 norm_g (!p.color-1) =   0
 norm_b (!p.color-1) = 255

 ; Green :

 norm_r (!p.color-2) =   0
 norm_g (!p.color-2) = 255
 norm_b (!p.color-2) =   0

 ; Yellow :

 norm_r (!p.color-3) = 255
 norm_g (!p.color-3) = 255
 norm_b (!p.color-3) =   0

 ; Red :

 norm_r (!p.color-4) = 255
 norm_g (!p.color-4) =   0
 norm_b (!p.color-4) =   0

 ; white at max image data value

 norm_r (!p.color-5) = 255
 norm_g (!p.color-5) = 255
 norm_b (!p.color-5) = 255

; savbwr=bwr  &  savbwg=bwg  &  savbwb=bwb           ; save b/w colors

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

 ; Difference color table, intense blue to black at mid-index value(s) to
 ; red to yellows. At max index is a white line.

 diff_r = intarr (!d.n_colors)		; colors to max available
 diff_g = diff_r
 diff_b = diff_r
   
 ; Grey at max value.  Used for plotting labels.

 diff_r (!p.color) = 128
 diff_g (!p.color) = 128
 diff_b (!p.color) = 128

 ; Blue :

 diff_r (!p.color-1) =   0
 diff_g (!p.color-1) =   0
 diff_b (!p.color-1) = 255

 ; Green :

 diff_r (!p.color-2) =   0
 diff_g (!p.color-2) = 255
 diff_b (!p.color-2) =   0

 ; Yellow :

 diff_r (!p.color-3) = 255
 diff_g (!p.color-3) = 255
 diff_b (!p.color-3) =   0

 ; Red :

 diff_r (!p.color-4) = 255
 diff_g (!p.color-4) =   0
 diff_b (!p.color-4) =   0

 ; white at max image data value

 diff_r (!p.color-5) = 255
 diff_g (!p.color-5) = 255
 diff_b (!p.color-5) = 255

 mid = (maxcolor) / 2.0			; middle value of blue/red scale

 ; if not integer, make two black at middle to center arrays

 if ((mid mod 1) ne 0) then $
    begin lomid = fix (mid)  &  himid = lomid+1  & endif $
    else $
    begin lomid = mid        &  himid = mid   & endelse

 reds = indgen (lomid) * 2		; array values from 0 to ~max
 blues = max (reds) - reds		; array values from ~max to 0
 reds = reds < 255 & blues = blues < 255
 diff_b (0) = blues  &  diff_r (himid+1) = reds    ; fill in color gun arrays

 greennum = 20				; top indices will be shades of yellow
 startgreen= fix (maxcolor - greennum)
 green = (indgen (greennum)+1) * (255.0/greennum)
 diff_g (startgreen) = green		; fill in color gun arrays

   mid    = lomid / 2
   numlev = mid
   for i = 1, mid do $
      begin
      diff_r (i) = 0
      diff_g (i) = (mid - i) * (255.0 / numlev)
      diff_b (i) = 255
      end

   mid = lomid / 2 + 1
   numlev = lomid - mid + 1
   for i = mid, lomid do $
      begin
      diff_r (i) = 0
      diff_g (i) = 0
      diff_b (i) = (lomid - i) * (255.0 / numlev)
      end

   numlev = (maxcolor - himid) / 2
   mid = himid + numlev - 1
   for i = himid, mid do $
      begin
      diff_r (i) = (i - himid) * (255.0 / numlev)
      diff_g (i) = 0
      diff_b (i) = 0
      end

   numlev = maxcolor - mid + 1
   for i = mid, maxcolor do $
      begin
      diff_r (i) = 255
      diff_g (i) = (i - mid) * (255.0 / numlev)
      diff_b  (i) = 0
      end

 ; white at max value

 diff_r (maxcolor) = 255   
 diff_g (maxcolor) = 255  
 diff_b (maxcolor) = 255 
 
 ; black line at min value

 diff_r (0) = 0
 diff_g (0) = 0
 diff_b (0) = 0 
 
; savbrr = brr  &  savbrg = brg  &  savbrb = brb	; save difference colors
end

;+ --------------------------------------------------------------------------- +
