;+ =========================================================================== +
;
; NAME: MK3DISK
;
; PURPOSE: 
;	Read in a Mark-III data file with tailer information and image.
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
; 	MK3DISK, DATA=DATA, TAIL=TAIL, FILE=FILENAME, OK=OK, $
;		IMSIZ=IMSIZ, SIN_MOV=SIN_MOV
;
; CALLED BY: 
;	MK3DISPLAY
;
; CALLS TO: 
;	none
;
; INPUTS:
;	FILENAME = Filename of MK3 data file to be read.
;	SIN_MOV  = 0 means current display mode is for single images
;	           1 means current display mode is for movies
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	DATA  = Byte array of size equal to MK3 image in pixels.
;	TAIL  = Byte array of size 40 bytes.
;	        Contains MK3 tailer information, padded with zeroes
;	OK    = 0 means failed to successfully read the input file
;	        1 means successfully read the input file
;	IMSIZ = Size of one dimension of MK3 image, MK3 images are square.  
;		 
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	MK3WIDGBASES for widget IDs of informational output widgets.
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Locate and open direct access data file.
;	The total size in bytes, minus the tailer, squared,
;	is the size of the MK3 image in pixels.  
;
; MODIFICATION HISORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	17 Mar 93 - Andrew L. Stanger (HAO/NCAR)
;	Adapted from "cpdisk" for MLSO Mark-III images.
;- =========================================================================== -


pro mk3disk, data=data, hk=mk3hk, file=filename, ok=ok, imsiz=imsiz, $
		sin_mov=sin_mov

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@mk3_img.common
; COMMON mk3_img, iyear, idoy, imonth, iday, ihour, iminute, isecond, $
;                 dtype, nbypx, npx, npy, channel, dmin, dmax, smin, smax, $
;                 scndir, scndur, pixrs, pangle

   ; Mark-III file "header" : 40 bytes.

;   mk3hdr = {mk3, $
;	flagch:		FIX  (0), $
;	juld:		FIX  (0), $
;	time:		LONG (0), $
;	phase:		FIX  (0), $
;	bright:		FIX  (0), $
;	pangle:		FIX  (0), $
;	nray:		FIX  (0), $
;	qmax:		FIX  (0), $
;	qmin:		FIX  (0), $
;	didxx:		FIX  (0), $
;	didxn:		FIX  (0), $
;	exdate:		FIX  (0), $
;	scndur:		FIX  (0), $
;	ipixscl:	LONG (0), $
;	ir0:		LONG (0), $
;	idr:		LONG (0)  $
;	}

   ; Mark-III pic "header" : 24 bytes.

   mk3pic = {pic, $
	id:	FIX (0), $
	lcom:	FIX (0), $
	type:	FIX (0), $
	dims:	FIX (0), $
	hpls:	FIX (0), $
	plns:	FIX (0), $
	rows:	FIX (0), $
	cols:	FIX (0), $
	bp  :	FIX (0), $
	sbp :	FIX (0), $
	xstrt:	FIX (0), $
	ystrt:	FIX (0)  $
	}

   if (strlen (filename (0)) eq 0) then filename = 'no_file_entered'
   a = findfile (filename)  
   ok = 0					; assume it will fail

   if (a (0) eq '') then begin
      explain = 'No file found: ' + filename +' in MK3DISK'
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain, /append $
      else widget_control, news, set_value=explain, /append 
      return
   endif

   on_ioerror, BADREAD

   openr, unit, filename, /get_lun 
   status = fstat (unit)  &  imsiz = fix (sqrt (status.size))	; file size

   npx = 512							; image width
   npy = 512							; image height
   if (imsiz > npx*npy*2)  then nbypx = 2  else  nbypx = 1

   recl  = npx
   imsiz = npy 

   case nbypx of
	1 :	w = assoc (unit, bytarr (recl, imsiz))  
	2 :	w = assoc (unit, intarr (recl, imsiz))
	else :	return
   endcase

   ; Read image data.

   v = w (0)
   data = v (*, 0:imsiz-1)

   ; Read Mark-III binary "header" (follows image data).

   mk3hk = INTARR (20)
   READU, unit, mk3hk 

;   print, 'mk3disk> mk3hk:', mk3hk

   ; Read Mark-III PIC (WHIP) "header" (follows Mark-III binary "header").

   READU, unit, mk3pic

   ; NOTE: there is also an ASCII tailer at the end of the file,
   ; but it is not accessed by this program.

   ok = 1							; success

   BADREAD: 
   free_lun, unit  

   return
end

;+ =========================================================================== +
