;+ =========================================================================== +
;
; file MK3DISPLAY.PRO -- includes routines MK3TV, MK3DIFF, MK3DISPLAY
;
;+ =========================================================================== +
;
; NAME:
;	MK3TV
;
; PURPOSE:
; 	Send a single (not movie) image, housekeeping info and 
;	requested drawing option output to screen or printer.
;
; CATEGORY:  
;	Mark-III display
;
; CALLING SEQUENCE:
;	MK3TV, DATA=DATA, HK=HK, XPOS=XPOS, YPOS=YPOS, X_SIZ=X_SIZ, 
;		Y_SIZ=Y_SIZ, XWINSIZ=XWINSIZ, YWINSIZ=YWINSIZ, COLO=COLO, $
;		IMSIZ=IMSIZ, DONORTH=DONORTH, DODISK=DODISK, $
;		DORADII=DORADII, DORADIAL=DORADIAL, HOUSEKP=HOUSEKP, $
;		RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		SIN_MOV=SIN_MOV, NORM_DIFF=NORM_DIFF, PROCESSED=PROCESSED, $
;		FILENAME=FILENAME, DIFFNAME1=DIFFNAME1, DIFFNAME2=DIFFNAME2, $
;		SAVEME=SAVEME, WMOVIE=W, NFILES=NFILES
;
; CALLED BY:
;	MK3DISPLAY
;
; CALLS TO:  
;	MK3HOUSE, MK3SUNCEN, PSPLOT, MK3INTERANIMATE
;         
; INPUTS:
;	DATA  : Byte array of size equal to MK3 image.
;	HK	: Array of size (40 x N) where N is 1 for single images
;		  and 2 for difference images. Contains the housekeeping
;		  information for the current image(s)
;	XPOS  : X direction screen coordinate (device pixels), to place window
;	YPOS  : Y direction screen coordinate (device pixels), to place window
;	X_SIZ : Image X size
;	Y_SIZ : Image Y size
;	XWINSIZ :  X direction width of window w/ housekeeping
;	YWINSIZ :  Y direction width of window 
;	COLO  : -1 means no hard copy allowed
;		 1 means hard copy device is color
;		 0 means hard copy device is B/W 
;	IMSIZ : Size of one dimension of MK3 image, MK3 images are square.  
;	DONORTH : 0 means don't draw solar north
;		  1 means do draw solar north on image
;	DODISK  : 0 means don't draw solar disk
;		  1 means do draw solar disk
;	DORADII : 0 means don't draw radius lines
;		  1 means do draw radius lines
;	DORADIAL : 0 means don't draw radial lines
;		   1 means do draw radial lines
;	HOUSEKP : X direction size reserved for house keeping information
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	NORM_DIFF : 0 means do a normal image
;                   1 means do a difference image
;	PROCESSED : for movies a number indicating the current movie frame
;	FILENAME  : string filename normal images	
;	DIFFNAME1 : string filename for base image 
;	DIFFNAME2 : string filename for subtraction image 
;	SAVEME    : for movies 0 means don't save movie to a file
;			       1 means do save movie to a file
;	W 	: associated record array for file output
;	NFILES  : for movies the number of frames in movie
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	PROCESSED : for movies a number indicating the next movie frame
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	MK3WIDGBASES for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	If SAVEME is set then creates a file containing the images and
;	housekeeping information of a movie.
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:  
;	Output image to selected device and call MK3SUNCEN to draw line options.
;	For movies to be saved, write necessary arrays to a previously-opened
;	file.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cptv" for MLSO Mark-III images.
;- =========================================================================== -


pro mk3tv, data=data, hk=hk, xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz, $
		xwinsiz=xwinsiz, ywinsiz=ywinsiz, colo=colo, $
		imsiz=imsiz, donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, housekp=housekp, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, processed=processed, $
		filename=filename, diffname1=diffname1, diffname2=diffname2, $
		saveme=saveme, wmovie=w, nfiles=nfiles

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

   if (sin_mov eq 1) then colo = -1			; can't print movies

   savpcolor = !p.color - 5				; windows color max

   if (colo ge 0) then begin			; making a hardcopy
      set_plot, 'ps', /copy   &   !p.font = 0
      if (norm_diff eq 0) then oxsiz = 21.0 else oxsiz = 23.0  ; output size
      oysiz =  oxsiz * (ywinsiz / xwinsiz)		; square up output
      xoff = 3.0 + norm_diff  &  yoff = 25.5

      device, /landscape, bits_per_pixel=8, xsize=oxsiz, ysize=oysiz, $
	      xoffset=xoff, yoffset=yoff, color=colo, /helvetica, $
	      filename='mk3.ps'
   endif

   xhk = housekp
   xd_siz = x_siz					; save device coord.
   yd_siz = y_siz					; save device coord.
   xpos  = xpos / xwinsiz   &  ypos  = ypos / ywinsiz	; convert to normalized
   x_siz = x_siz / xwinsiz  &  y_siz = y_siz / ywinsiz	; convert to normalized
   housekp = housekp / xwinsiz

   tv, data, xpos, ypos, /normal	;Display image | create file 'mk3.ps'.

   ; Write annotation twice if a difference.

   for ntime = 0, norm_diff do $
      mk3house, hk = hk (*, ntime), ntime=ntime, colo=colo

   mk3suncen, imsiz=imsiz, hk=hk(*,0), donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, colo=colo, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		housekp=housekp, $
		sin_mov=sin_mov, norm_diff=norm_diff, savpcolor=savpcolor, $
		xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz 

   ; Write vertical line to separate annotation from image.

   PLOTS, [xhk - 1, xhk - 1], [0, yd_siz - 1], COLOR = !p.color-2, /DEVICE
   if (norm_diff NE 0) then $
      PLOTS, [xhk + xd_siz + 1, xhk + xd_siz + 1], [0, yd_siz - 1], $
	    COLOR = !p.color-2, /DEVICE

   if (colo ge 0) then begin			; close and print image
      device, /close  &  set_plot, 'x'  &  !p.font=-1 
      widget_control, comment, set_value='Printing output', /append

;      if (colo eq 0) then spawn, 'print/setup=psi/delete/notify idl.ps' $
;                     else spawn, 'print/que=color$script/notify/delete idl.ps'

;25 Feb 93   psplot, filename='mk3.ps', color=colo, /delete

   if (colo eq 0) then $
      print, 'Wrote image to B/W postscript file: mk3.ps' $
   else if (colo eq 1) then $
      print, 'Wrote image to color postscript file: mk3.ps'

   endif

   if (sin_mov ne 0) then begin          ; load a movie frame to movie widget
      mk3interanimate, frame=processed, window=!d.window

      if (saveme) then begin			
         ;
         ; Save a movie frame by writing it to a file
         ;
         ndata = bytarr (imsiz, imsiz+2)	; 2 headers and image

         ; write header(s) to file 

         for ntime = 0, norm_diff do $	; do twice if difference
            ndata (0, ntime) = [hk (*, ntime), nfiles-1b,  xpos*100b, ypos*100b]	
         ; write information needed by MK3SUNCEN.PRO once the movie is restored.

         ndata (227) = donorth
         ndata (228) = dodisk
         ndata (229) = doradii
         ndata (230) = doradial
         ndata (231) = colo
         ndata (232) = ruletimes
         ndata (233) = rulecondition
         ndata (234) = norm_diff          ; normal / difference image
         ndata (235) = housekp*100	; percent space left for info
         ndata (236) = x_siz*100		; percent image X size
         ndata (237) = y_siz*100		; percent image Y size

         ; add image of size IMSIZ x IMSIZ

         ndata (*, 2:*) = data		
         w (processed) = ndata	; saving movie frame w/ headers to file
         ndata = 0b		; clear memory
      endif

      processed = processed + 1

      mes = 'Proccessed frame ' + strtrim (processed,2) + ': ' 
      if (norm_diff eq 0) then mes = mes + filename $
	    		  else mes = mes + diffname1 + ' sub ' + diffname2
      widget_control, news, set_value= mes, /append 

      erase
   endif

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3DIFF
;
; PURPOSE:
; 	Scale the difference images to align with the difference color table.
;
; CATEGORY:  
;	Mark-III diplay
;
; CALLING SEQUENCE: 
;      	MK3DIFF, DATA=DATA, POSFAC=POSFAC, NEGFAC=NEGFAC
;
; CALLED BY:  
;	MK3DISPLAY
;
; CALLS TO: 
;	none
;
; INPUTS:
;	DATA  : Byte array of size equal to MK3 image.  Array values
;		range from -255 to +255.
;
; OPTIONAL INPUTS:  
;	none
;
; OUTPUTS:
;	DATA   : Byte array of size equal to MK3 image.  Array values
;		 range from 1 to the value of !D.N_COLORS-2
;	POSFAC : image red color scale factor
;	NEGFAC : image blue color scale factor
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 
;	none
;
; SIDE EFFECTS: 
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Align data array values to match the difference color table and
;	the user's (or default) values for scaling the negative and positive
;	values resulting from the image subtraction.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cpdiff" for MLSO Mark-III images.
;+ =========================================================================== +

pro mk3diff, data=data, posfac=posfac, negfac=negfac

@mk3colors.common

   blk = (maxcolor) / 2.0		; middle value(s) of blue/red scale
   if ((blk mod 1) ne 0) then begin loblk = fix (blk)  &  hiblk = loblk+1
		   endif else begin loblk = blk        &  hiblk = blk  & endelse

       neg = where (data lt 0)  &  sizn = size (neg)   ; where are the negatives
       pos = where (data ge 0)  &  sizp = size (pos)   ; where are the positives

       ; Scale the negative from 0 to the faintest blue color.

       fntb = loblk - 1.0		; the faintest blue, 1 index from black
       if (sizn (0) ne 0) then data (neg) = (data (neg) * (fntb/negfac)) + loblk

       ; Scale the positives from faintest red color to maxcolor.

       fntr = hiblk + 1.0		; the faintest red, 1 index from black
       if (sizp (0) ne 0) then $
	  data (pos) = (data (pos) * ((maxcolor-fntr)/posfac)) + hiblk
                                            
       ; Truncate data values equal to zero to 1 (index 0 is a black line),
       ; and data values equal to maxcolor to maxcolor-1 (index maxcolor is 
       ; a white line).

       data = 1 > byte (data) < maxcolor-1 
     
end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3DISPLAY
;
; PURPOSE:
;	Setup image for display
;
; CATEGORY:  
;	Mark-III display
;
; CALLING SEQUENCE:
; 	MK3DISPLAY, NORM_DIFF=NORM_DIFF, DONORTH=DONORTH, $
;	         DODISK=DODISK, DORADII=DORADII, DORADIAL=DORADIAL,  $
;		 RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		 POSFAC=POSFAC, NEGFAC=NEGFAC, SIN_MOV=SIN_MOV, $
;		 PROCESSED=PROCESSED, SAVEME=SAVEME, WMOVIE=W, $
;		 NFILES=NFILES, OK=OK
;
; CALLED BY:
;	MK3ONE, MK3MOVIE
;
; CALLS TO: 
;	MK3DISK, MK3DIFF, WINUP, MK3TV
;
; INPUTS:
;	NORM_DIFF     : 0 means do a normal image
;                       1 means do a difference image
;	DONORTH       : just passing through, on it's way to MK3TV
; 	DODISK        : just passing through, on it's way to MK3TV
;	DORADII       : just passing through, on it's way to MK3TV
;	DORADIAL      : just passing through, on it's way to MK3TV
;	RULETIMES     : just passing through, on it's way to MK3TV 
; 	RULECONDITION : just passing through, on it's way to MK3TV
;	POSFAC        : just passing through, on it's way to MK3DIFF 
;	NEGFAC        : just passing through, on it's way to MK3DIFF 
;	SIN_MOV       : 0 means current display mode is for single images
;	           	1 means current display mode is for movies
;	PROCESSED     : just passing through, on it's way to MK3TV
;	SAVEME 	      : just passing through, on it's way to MK3TV
;	W 	      : just passing through, on it's way to MK3TV
;	NFILES        : just passing through, on it's way to MK3TV
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:
;	OK : 0 means failed to successfully read the input file
;	     1 means successfully read the input file
;	PROCESSED : just passing through, on it's back to MK3MOVIE
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS :
;	MK3COLORS to load color tables
;	COLORS to change values in R_CURR, G_CURR, B_CURR to desired table
;	MK3DISPFILE for files to read and output device color condition
;	MK3NEW_WINDOW  for window display variables
;	MK3WIDGBASES for widget IDs to write out informational messages
;
; SIDE EFFECTS : 
;	Changes values in IDL common block COLORS
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Read in data file(s) and if a difference image do the subtraction.
;	Scale the data to the appropriate color table and load that table.
;	Call to MK3TV to output the MK3 display.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cpdisplay" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3display, norm_diff=norm_diff, donorth=donorth, $
	         dodisk=dodisk, doradii=doradii, doradial=doradial,  $
		 ruletimes=ruletimes, rulecondition=rulecondition, $
		 posfac=posfac, negfac=negfac, sin_mov=sin_mov, $
		 processed=processed, saveme=saveme, wmovie=w, $
		 nfiles=nfiles, ok=ok

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@mk3colors.common
;common mk3colors, maxcolor, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset

@mk3dispfiles.common
;common dispfiles, filename, diffname1, diffname2, onewin, colo

@mk3_img.common
; COMMON mk3_img, iyear, idoy, imonth, iday, ihour, iminute, isecond, $
;                 dtype, nbypx, npx, npy, channel, dmin, dmax, smin, smax, $
;                 scndir, scndur, pixrs, pangle

@mk3new_window.common
;common new_window, lastmark, sc, current_window, reuse, first_pass

common colors, r_orig, g_orig, b_orig, r_curr, g_curr, b_curr


   if (n_elements (processed) eq 0) then processed = 0	; define if no value 

   hk = intarr (20, norm_diff+1)        ; housekeeping, 2D for differences

   ; --- Normal image ---

   if (norm_diff eq 0) then begin

      ; Read image(s) from disk.

      mk3disk, data=data, hk=houskeep, file=filename, ok=ok, $
			imsiz=imsiz, sin_mov=sin_mov

      ; Write status message to text widget.

      if (not (ok)) then begin
         explain = 'Error with file ' + filename 
         if (sin_mov eq 0) then $
	      widget_control, comment, set_value=explain, /append $
	 else widget_control, news,    set_value=explain, /append 
	 return
      endif

      hk (0, 0) = houskeep
   endif $

   ; --- Difference Image  ---

   else begin

      ; Read base image data from disk.

      mk3disk, data=data1, hk=houskeep, file=diffname1, ok=ok, $
		imsiz=imsiz, sin_mov=sin_mov

      ; Write status message to text widget.

      if (not (ok)) then begin
         explain = 'Error with file ' + filename 
         if (sin_mov eq 0) then $
	      widget_control, comment, set_value=explain, /append $
         else widget_control, news,    set_value=explain, /append 

         explain = 'Requires both images for a difference '
         if (sin_mov eq 0) then $
	      widget_control, comment, set_value=explain, /append $
	 else widget_control, news,    set_value=explain, /append 
	 return
      endif

      hk (0, 0) = houskeep                   ; base image's housekeeping info

      ; Now read subtraction image data from disk.

      mk3disk, data=data2, hk=houskeep, file=diffname2, ok=ok, $
		imsiz=imsiz, sin_mov=sin_mov

      ; Write status message to text widget.

      if (not (ok)) then begin
         explain = 'Error with file ' + filename 
         if (sin_mov eq 0) then $
	      widget_control, comment, set_value=explain, /append $
	 else widget_control, news,    set_value=explain, /append 

         explain = 'Requires both images for a difference '
         if (sin_mov eq 0) then $
	      widget_control, comment, set_value=explain, /append $
	 else widget_control, news,    set_value=explain, /append 
	 return
      endif

      ; Size testing: images must be same.

      siz1 = size (data1)  &  siz2 = size (data2)
      if (siz1 (1) ne siz2 (1)) then begin
         explain = 'Base and Subtraction images are different sizes'
	 if (sin_mov eq 0) then $
	      widget_control, comment, set_value=explain, /append $
	 else widget_control, news,    set_value=explain, /append 
 	 return
      endif

       hk (0, 1) = houskeep            	; subtraction image's housekeeping info

      ; Subtract "base image" from "subtraction image".

      data1 = fix (data1)  &  data2 = fix (data2)  	; convert to short int.
      data = data2 - data1 
   endelse
 
   ; Compute size needed for image display.

   housekp = 150.0			; room to write out house keeping info
   x_siz  = npx  &  y_siz = npy     ; width and height of image windowing area
   xwinsiz = housekp * (norm_diff+1.0) + x_siz	; width window w/ housekeeping 
   ywinsiz = y_siz				; height window w/ housekeeping 
   xpos = 0 + housekp
   ypos = 0

  case coltyp of
    0: begin	; Normal display, so use b/w.  Data values within 0 to +255.

         if (colreset) then begin      ;reset to standard mk3 normal color table
            r_curr = norm_r  &  g_curr = norm_g  &  b_curr = norm_b  
         endif

         data = (maxcolor) / 255.0 * data	; scale data to maxcolor
       end                               

    1: begin		; Difference image.  Data values within -255 to +255.

         if (colreset) then begin	; reset to standard c/p diff color table
            r_curr = diff_r  &  g_curr = diff_g  &  b_curr = diff_b 
         endif

         mk3diff, data=data, posfac=posfac, negfac=negfac
       end
  endcase

  if (sin_mov eq 0) and colo eq -1 then begin  

  ; If a single image (movies make their own) and if output going to screen :

    if (reuse and not (first_pass)) then begin	; reuse windows
      wdelete, current_window
    endif else begin
      lastmark = 1 - lastmark			; alternate position
      first_pass = 0 				; now have at least 1
    endelse
		
    if (not (norm_diff)) then title = 'MK3 '+ filename $
      else title = 'MK3 '+ diffname1 + ' / ' + diffname2

    xloc =  sc (0) - xwinsiz - 10
    yloc = (sc (1) - ywinsiz) * lastmark + (lastmark * (-20) + 10)

    winup, /free, title=title, $
           xsiz = xwinsiz, ysiz = ywinsiz, retain = 2, $
           xpos = xloc,  ypos = yloc  

    current_window = !d.window

  endif

  tvlct, r_curr, g_curr, b_curr   		; load some color table 

  mk3tv, data=data, hk=hk, xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz, $
		xwinsiz=xwinsiz, ywinsiz=ywinsiz, colo=colo, $
		imsiz=imsiz, donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, housekp=housekp, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, processed=processed, $
		filename=filename, diffname1=diffname1, diffname2=diffname2, $
	        saveme=saveme, wmovie=w, nfiles=nfiles

end

;+ =========================================================================== +
