;+ =========================================================================== +
;
; NAME: 
;	MK3HOUSE
;
; PURPOSE:  
;	Output housekeeping information.
;
; CATEGORY: 
;	Mark-III diplay
;
; CALLING SEQUENCE: 
;	pro mk3house, hk=hk, ntime=ntime, colo=colo
;
; CALLED BY:  
;	MK3TV, MK3MOVIE
;
; CALLS TO:  
;	none
;
; INPUTS:
;	HK	: Array of size (40 x N) where N is 1 for single images
;		  and 2 for difference images. Contains the housekeeping
;		  information for the current image(s)
;	NTIME	:  0 means display the house keeping for the normal image 
;		     or base image of a difference display
;		   1 means use the subtraction image values in HK
;	COLO    : -1 means no hard copy allowed
;		   1 means hard copy device is color
;		   0 means hard copy device is color
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS:  
;	none
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 
;	none
;
; SIDE EFFECTS: 
;	none
;
; RESTRICTIONS: 
;	none
;
; PROCEDURE:
;	Writes the house keeping information to the output device.
;	If a difference image, the base image house keeeping is on the
;	left and the subtraction image house keeping is written on the right.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;	1 March 1993 Andrew L. Stanger (HAO/NCAR)
;	   IRIX apparently requires font size 1/2 as large as for sunos.
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cphouse" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3house, hk=hk, ntime=ntime, colo=colo

@mk3_img.common
; COMMON mk3_img, iyear, idoy, imonth, iday, ihour, iminute, isecond, $
;                 dtype, nbypx, npx, npy, channel, dmin, dmax, smin, smax, $
;                 scndir, scndur, pixrs, pangle

   imask  = '177777'O
   lmask  = '37777777777'OL
   degrad = 57.29578			; degrees per radian.

   x = 0.01 + (0.84 * ntime)  &  y = 0.96  &  down = 0.05  &  skip = 0.01

   labcs = 1.2				; Set standard char size for labels.
   valcs = 2.0				; Set standard char size for values.

   case !version.os of
      'IRIX'	: osfac = 0.5		; Reduce size by 1/2 for SGI/IRIX.
      else	: osfac = 1.0		; Use standard char size for all others.
   endcase

   if (colo eq -1) then $
        colofac = 1.0   $		; Display size factor.
   else colofac = 0.8			; Print   size factor.

   labsiz = labcs * colofac * osfac	; Label font size for xyouts.
   valsiz = valcs * colofac * osfac	; Value font size for xyouts.

   c2 = !p.color			; color for labels.
   c1 = !p.color - 4			; color for data values.

   ;*** Decode flags in `filehd.flagch'. ***

   ; quickflag 	= 0
   ; pBflag	= 0
   ; qdflag	= 0
   ; udflag	= 0
   ; idflag	= 0
   ; posflag	= 0
   ; negflag	= 0

   hkflags = FIX (ISHFT (FIX (hk (0) AND imask), -3))
   mk3flags, hkflags, $
             quickflag, pBflag, qdflag, udflag, idflag, posflag, negflag

   xyouts, x, y, 'DATE', /normal, size = labsiz, color = c2  &  y = y-down
   hkdate = fix (hk (1)) AND imask

   mjdate, hkdate, imonth, iday, iyear

   daymon  = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]  ; normal year.
   daymonl = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]  ; leap year.

    ;leap = year%4 == 0 && year%100 != 0 || year%400 == 0 ;

    if (((iyear mod 4) EQ 0)    AND $
	((iyear mod 100) NE  0) OR (iyear mod 400) EQ 0)  $
	then  leap = 1   else leap = 0

   idoy = iday
   for i = 0, imonth-1 do $
   begin
      if (leap) then $
         idoy = idoy + daymonl (i) $
      else $
         idoy = idoy + daymon (i)
   end

   xyouts, x, y, strtrim (iyear - 1900, 2) + ':' + $
		 string (idoy, format = '(i3.3)'), $
	   /normal, size=valsiz, color = c1
   y = y - down - skip
                               
   xyouts, x, y, 'TIME', /normal, size = labsiz, color = c2  &  y = y - down
   hktime = LONG (hk (2) AND imask) * 65536 + LONG (hk (3) AND imask)
   mk3time, hktime, ihour, iminute, isecond
   time = string (fix (ihour), format='(i2.2)') + ':' + $
          string (fix (iminute), '(i2.2)') + ':' + $
	  string (fix (isecond), '(i2.2)')
   xyouts, x, y, time, /normal, size=valsiz, color = c1  &  y = y - down - skip

   ;*** Get the day and month from day-of-year and year. ***

   month = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', $
            'Oct', 'Nov', 'Dec']
   nday = intarr (12)
   nday ( [0,2,4,6,7,9,11] ) = 31  &  nday ( [3,5,8,10] ) = 30
   ly = float (iyear)/4.0
   if ((ly - fix (ly)) eq 0.0) then nday (1) = 29 else nday (1) = 28
   ndt = 0
   for i = 0, 11 do begin
      ndt = ndt + nday (i)  &  if (ndt ge idoy) then goto, label1
   endfor

label1:  mth = i  &  idm = idoy - ndt + nday (i)

   xyouts, x, y, string (idm, format = '(i2.2)') + ' ' + month (mth), $
	   /normal, size = valsiz, color = c1
   y = y - down
   xyouts, x, y, string (FIX (iyear)), /normal, size = valsiz, color = c1
   y = y - down - skip 

   ;*** Channel ***

   xyouts, x, y, 'TYPE : CHAN', /normal, size=labsiz, color = c2  &  y=y-down
   if (qdflag) then begin type = 'SRQ  ' & dtype = 2  & end
   if (udflag) then begin type = 'SRU  ' & dtype = 8  & end
   if (idflag) then begin type = 'SRI  ' & dtype = 5  & end
   if (pbflag) then begin type = 'SRpB ' & dtype = 11 & end
   channel = FIX (hk (0)) AND '000007'O		; least significant 3 bits.
   schan = type + STRING (channel, format='(i2)')
   xyouts, x, y, schan, /normal, size=valsiz, color=c1 & y = y-down-skip

   ;*** Data min/max. ***

   xyouts, x, y, 'MIN : MAX', /normal, size=labsiz, color = c2  &  y=y-down
   dmax = FIX (hk (8))					; Data max value.
   dmin = FIX (hk (9))					; Data min value.
   sminmax = STRING (FIX (dmin), format='(i4)') + ':' + $
	     STRING (FIX (dmax), format='(i4)')
   xyouts, x, y, sminmax, /normal, size=valsiz, color = c1  &  y=y-down-skip

   xyouts, x, y, 'SCAN DIR', /normal, size=labsiz, color = c2  &  y=y-down
   scndir = '*****'
   if (posflag) then scndir = 'CCW +' $
   else $
   if (negflag) then scndir = ' CW -'
   xyouts, x, y, scndir, /normal, size=valsiz, color = c1  &  y = y-down-skip

   xyouts, x, y, 'SCAN DUR', /normal, size=labsiz, color = c2  &  y=y-down
   scndur = FIX ((hk (13) AND imask) / 60)		; convert to seconds.
   sscndur = STRING (scndur, format='(i3)') + ' sec'	; Convert to string.
   xyouts, x, y, sscndur, /normal, size=valsiz, color = c1
   y = y - down - skip

   xyouts, x, y, 'PIXELS/Rsun', /normal, size=labsiz, color = c2
   y = y - down
   pixrs = (LONG (hk (14) AND imask) * 65536 + $
	    LONG (hk (15) AND imask)) / 10000.0			; # Pixels/Rsun.
   spixrs = STRING (pixrs, format='(f6.1)')
   xyouts, x, y, spixrs, /normal, size=valsiz, color = c1
   y = y - down - skip

   xyouts, x, y, 'P-ANGLE', /normal, size=labsiz, color = c2  &  y = y-down
   pangle = FLOAT (hk (6) / 60.0)	; Convert P-angle to degrees.
   spangle = STRING (pangle, format='(f6.1)') + ' deg'
   xyouts, x, y, spangle, /normal, size=valsiz, color = c1
   y = y - down - skip

end 

;+ =========================================================================== +
