/* mk3hskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO Mark-III housekeeping decoder.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   6 January 1994
 * 11 Feb 94: get dtype from rdmk3hk; set bscale & bzero.
 * 23 Feb 94: add phase to MK3I.
 * 24 Feb 94: add opc date & time.
 * 28 Feb 94: correct tmin/tmax for phase & pangle.
 * 25 Apr 94: dtype 23 is now 24 (avgrpB); dtype 23 is avgpB.
 *  9 Sep 94: use WHIP footer to get nbypx.
 * 18 Apr 95: Do NOT use ipixscl to determine pixrs, since it is incorrect
 *            for channel 1 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "/home/cordyn/mlsofits/lib/mk3/mk3lib.ext"

mk3hskp (namfil, pmode)

char namfil [] ;			/* Name of MLSO Mark-III image file. */
int pmode ;				/* Print mode. */

{
   static struct stat FS ;
   struct stat *fs = &FS;

   static char scndir [8];

   int Dtype;
   int channel ;
   int dmin, dmax ;
   int dtype ;
   int fd ;
   int filesize ;
   int flagch ;
   int quickflag ;
   int idflag, qdflag, udflag, pbflag ;
   int hour, minute, second ;
   int juld ;
   int month, day, year ;
   int nbypx ;
   int posflag, negflag ;
   int time ;
   int xdim, ydim, zdim ;

   float bscale, bzero ;		/* actual = data * bscale + bzero. */
   float pixrs ;
   float pangle ;
   float phase ;
   float scndur ;			/* in seconds. */

   /*** Read Mark-III housekeeping from image file. ***/

   dtype = rdmk3hk (namfil, pmode) ;

   /*** Decode flag element. ***/

   flagch = (int) (MK3H->flagch & 0177777);
   mk3flags (flagch, &quickflag, &pbflag, &qdflag, &udflag, &idflag,
		        &posflag, &negflag, &channel) ;

   if (pmode)
      {
      fprintf (stdout, "quickflag :	%d\n", quickflag) ;
      fprintf (stdout, "pbflag    :	%d\n", pbflag   ) ;
      fprintf (stdout, "qdflag    :	%d\n", qdflag   ) ;
      fprintf (stdout, "udflag    :	%d\n", udflag   ) ;
      fprintf (stdout, "idflag    :	%d\n", idflag   ) ;
      fprintf (stdout, "posflag   :	%d\n", posflag  ) ;
      fprintf (stdout, "negflag   :	%d\n", negflag  ) ;
      fprintf (stdout, "channel   :	%d\n", channel  ) ;
      }

   /*** Determine data type. ***/

   /* MK3 data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * q   --> cylindrical q data					 0
    * rq  --> rectangular q data, origin in SE				 1
    * srq --> scaled rectangular q data, origin in SE			 2
    * i   --> cylindrical i data					 3
    * ri  --> rectangular i data, origin in SE				 4
    * sri --> scaled rectangular i data, origin in SE			 5
    * u   --> cylindrical u data					 6
    * ru  --> rectangular u data, origin in SE				 7
    * sru --> scaled rectangular u data, origin in SE			 8
    * pB  --> cylindrical pB*1.e-10 data				 9
    * rpB  --> rectangular pB*1.e-10 data, origin in SE			10
    * srpB --> scaled rectangular pB data, origin in SE			11
    * Ne  --> cylindrical Ne data					12
    * rNe  --> rectangular Ne*1.e+33 data, origin in SE			13
    * srNe --> scaled rectangular Ne data, origin in SE			14
    * aba --> barrel angles as ascii in degrees *10			15
    * ba --> barrel angles (shorts) with 16384=360 degrees		16
    * ast --> scan times as ascii in seconds				17
    * st --> scan times (shorts) with 60=1 second (Hz)			18
    * a .calib file							19
    * a .tscp file							20
    * I  --> calibrated cylindrical I data				21
    * U  --> calibrated cylindrical U data				22
    * avgpB  --> avg'd cylindrical pB*1.e-10 data, origin PA 0, r0	23
    * avgrpB --> avg'd rectangular pB*1.e-10 data, origin in SE		24
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   MK3I->dtype = dtype ;

   /*** Determine nbypx, xdim, ydim based upon data type. ***/

   switch (dtype)
      {
      case  0 :							/* q	*/
      case  3 :							/* i	*/
      case  6 :							/* u	*/
      case  9 :							/* pB	*/
      case 23 :							/* pB	*/
                xdim = 128; ydim = MK3H->nray; nbypx = 2;
		bscale = 1.0e-10; bzero = 0.0;
                MK3I->smin = -1000 ;   MK3I->smax = 10000 ;
		break ;

      case  1 :							/* rq	*/
      case  4 :							/* ri	*/
      case  7 :							/* ru	*/
      case 10 :							/* rpB	*/
                xdim = 512; ydim = 512; nbypx = 2;
		bscale = 1.0e-10; bzero = 0.0;
                MK3I->smin = -1000 ;   MK3I->smax = 10000 ;
		break ;

      case  2 :							/* srq	*/
      case  5 :							/* sri	*/
      case  8 :							/* sru	*/
      case 11 :							/* srpB	*/
      case 14 :							/* srNe */
		xdim = 512; ydim = 512; nbypx = 1;
		bscale = 1.0; bzero = 0.0;
                MK3I->smin =     0 ;   MK3I->smax =   250 ;
		break ;

      case 12 :							/* Ne   */
      case 13 :							/* rNe  */
                xdim = 512; ydim = 512; nbypx = 2;
		bscale = 1.0e+12; bzero = 0.0;
                MK3I->smin =     0 ;   MK3I->smax = 10000 ;
		break ;

      case 15 : xdim = MK3H->nray; ydim = 1; nbypx = 1;		/* ASCII B.A. */
		bscale = 1.0; bzero = 0.0;
                MK3I->smin =  -180 ;   MK3I->smax =   180 ;
		break ;

      case 16 : xdim = MK3H->nray; ydim = 1; nbypx = 2;		/* SHORT B.A. */
		bscale = 360.0 / 16384.0; bzero = 0.0;
                MK3I->smin =  -180 ;   MK3I->smax =   180 ;
		break ;

      case 17 : xdim = MK3H->nray; ydim = 1; nbypx = 1;		/* ASCII STIME*/
		bscale = 1.0; bzero = 0.0;
                MK3I->smin =     0 ;   MK3I->smax =   250 ;
		break ;

      case 18 : xdim = MK3H->nray; ydim = 1; nbypx = 2;		/* SHORT STIME*/
		bscale = 1.0 / 60.0; bzero = 0.0;		/* Hertz->sec */
                MK3I->smin =     0 ;   MK3I->smax =   250 ;
		break ;

      case 19 : xdim = 0; ydim = 0; nbypx = 2;			/* calib      */
		bscale = 1.0; bzero = 0.0;
                MK3I->smin =     0 ;   MK3I->smax =   250 ;
		break ;

      case 20 : xdim = 0; ydim = 0; nbypx = 2;			/* tscp       */
		bscale = 1.0; bzero = 0.0;
                MK3I->smin =     0 ;   MK3I->smax =   250 ;
		break ;

      case 21 : xdim = 512; ydim = 512; nbypx = 2;		/* cal I      */
		bscale = 1.0e-10; bzero = 0.0;
                MK3I->smin = -1000 ;   MK3I->smax = 10000 ;
		break ;

      case 22 : xdim = 512; ydim = 512; nbypx = 2;		/* cal U      */
		bscale = 1.0e-10; bzero = 0.0;
                MK3I->smin = -1000 ;   MK3I->smax = 10000 ;
		break ;

      case 24 : xdim = 512; ydim = 512; nbypx = 2;		/* avg rpB    */
		bscale = 1.0e-10; bzero = 0.0;
                MK3I->smin = -1000 ;   MK3I->smax = 10000 ;
		break ;
      }

   if (MK3I->nbypx == 0)
      MK3I->nbypx = nbypx ;
   zdim = 1 ;
   MK3I->xdim = xdim ;
   MK3I->ydim = ydim ;
   MK3I->zdim = zdim ;

   /*** Compute Mark-III "julian" date. ***/

   juld = (int) (MK3H->juld & 0177777) ;
   mjdate (juld, &year, &month, &day) ;
   MK3I->year  = year ;
   MK3I->month = month ;
   MK3I->day   = day ;
   MK3I->doy   = day_of_year (year, month, day) ;

   /*** Convert month & day to day-of-year. */

   time = MK3H->time ;
   mk3time (time, &hour, &minute, &second) ;
   MK3I->hour   = hour ;
   MK3I->minute = minute ;
   MK3I->second = second ;

   /*** Compute Mark-III extraction date. ***/
   /*** For now, assume calibration date coincides with extraction date. ***/

   /*** Calibration date is set by "rdmkhk.c". ***/

/*
   juld = (int) (MK3H->exdate & 0177777) ;
   mjdate (juld, &year, &month, &day) ;
   if (MK3H->exdate < MK3H->juld || MK3H->exdate == 0)
      {
      year = month = day = 0;
      }
   MK3I->cal_year  = year ;
   MK3I->cal_month = month ;
   MK3I->cal_day   = day ;
 */

   /*** Image min/max values. ***/

   MK3I->dmin = MK3H->dmin ;
   MK3I->dmax = MK3H->dmax ;

   /**** Scan direction. ***/

   if (posflag)
      {
      strcpy (scndir, "CCW +") ;
      MK3I->scndir = 1 ;
      }
   else
   if (negflag)
      {
      strcpy (scndir, " CW -") ;
      MK3I->scndir = -1 ;
      }
   else
      {
      strcpy (scndir, "*****") ;
      MK3I->scndir =  0 ;
      }

   /*** Scan duration. ***/

   scndur = (float) (MK3H->scndur & 0177777) / 60.0 ;	/* Hertz -> seconds. */
   MK3I->scndur = scndur ;

   /*** Intensity scaling factor & offset. ***/

   MK3I->bscale = bscale ;
   MK3I->bzero  = bzero  ;

   /*** Geometric scale size: pixels/Rsun. ***/

/*
 *   pixrs = (float) (MK3H->ipixscl) / 10000.0 ;
 *   MK3I->pixrs = pixrs ;
 */

   /*** Phase: angular position of channel w.r.t. geo-centric north. ***/

   phase = (float) MK3H->phase * (180.0 / 32768.0 ) ;	/* Convert to degrees.*/
   MK3I->phase = phase ;

   /*** P-angle. ***/

   pangle = (float) MK3H->pangle / 60.0 ;	/* Convert to degrees. */
   MK3I->pangle = pangle;

   /*** N-angle: CCW angle of solar north w.r.t. +X axis. ***/

   MK3I->nangle = 90.0 ;

   /*** X/Y sun center. ***/

   MK3I->xcen = 255.5 ;
   MK3I->ycen = 255.5 ;

   /*** Inner/Outer FOV radii. ***/

   MK3I->rmin = ((float) MK3H->ir0) / 10000.0 ;
   MK3I->rmax = (((float) MK3H->idr) / 10000000.0) * 127.0 + MK3I->rmin ;

   /*** Min/max barrel angles: correct for phase & p-angle. ***/

   MK3I->tmin -= (phase + pangle) ;
   MK3I->tmax -= (phase + pangle) ;

   if (pmode)
      {
      fprintf (stdout, "MK3I->dtype     :  %7d\n",   MK3I->dtype) ;
      fprintf (stdout, "MK3I->channel   :  %7d\n",   MK3I->channel) ;
      fprintf (stdout, "MK3I->xdim      :  %7d\n",   MK3I->xdim) ;
      fprintf (stdout, "MK3I->ydim      :  %7d\n",   MK3I->ydim) ;
      fprintf (stdout, "MK3I->zdim      :  %7d\n",   MK3I->zdim) ;
      fprintf (stdout, "MK3I->nbypx     :  %7d\n",   MK3I->nbypx) ;
      fprintf (stdout, "MK3I->dmin      :  %7d\n",   MK3I->dmin) ;
      fprintf (stdout, "MK3I->dmax      :  %7d\n",   MK3I->dmax) ;
      fprintf (stdout, "MK3I->smin      :  %7d\n",   MK3I->smin) ;
      fprintf (stdout, "MK3I->smax      :  %7d\n",   MK3I->smax) ;
      fprintf (stdout, "MK3I->year      :  %7d\n",   MK3I->year) ;
      fprintf (stdout, "MK3I->month     :  %7d\n",   MK3I->month) ;
      fprintf (stdout, "MK3I->day       :  %7d\n",   MK3I->day) ;
      fprintf (stdout, "MK3I->doy       :  %7d\n",   MK3I->doy) ;
      fprintf (stdout, "MK3I->hour      :  %7d\n",   MK3I->hour) ;
      fprintf (stdout, "MK3I->minute    :  %7d\n",   MK3I->minute) ;
      fprintf (stdout, "MK3I->second    :  %7d\n",   MK3I->second) ;
      fprintf (stdout, "MK3I->cal_year  :  %7d\n",   MK3I->cal_year ) ;
      fprintf (stdout, "MK3I->cal_month :  %7d\n",   MK3I->cal_month) ;
      fprintf (stdout, "MK3I->cal_day   :  %7d\n",   MK3I->cal_day  ) ;
      fprintf (stdout, "MK3I->opc_year  :  %7d\n",   MK3I->opc_year ) ;
      fprintf (stdout, "MK3I->opc_month :  %7d\n",   MK3I->opc_month) ;
      fprintf (stdout, "MK3I->opc_day   :  %7d\n",   MK3I->opc_day  ) ;
      fprintf (stdout, "MK3I->opc_hour  :  %7d\n",   MK3I->opc_hour ) ;
      fprintf (stdout, "MK3I->opc_minute:  %7d\n",   MK3I->opc_minute) ;
      fprintf (stdout, "MK3I->opc_second:  %7d\n",   MK3I->opc_second) ;
      fprintf (stdout, "MK3I->scndir    :  %7d\n",   MK3I->scndir) ;
      fprintf (stdout, "MK3I->scndur    :  %7.3f\n", MK3I->scndur) ;
      fprintf (stdout, "MK3I->bscale    :  %7.3e\n", MK3I->bscale) ;
      fprintf (stdout, "MK3I->bzero     :  %7.3f\n", MK3I->bzero ) ;
      fprintf (stdout, "MK3I->pixrs     :  %7.3f\n", MK3I->pixrs) ;
      fprintf (stdout, "MK3I->phase     :  %7.3f\n", MK3I->phase) ;
      fprintf (stdout, "MK3I->pangle    :  %7.3f\n", MK3I->pangle) ;
      fprintf (stdout, "MK3I->nangle    :  %7.3f\n", MK3I->nangle) ;
      fprintf (stdout, "MK3I->xcen      :  %7.3f\n", MK3I->xcen) ;
      fprintf (stdout, "MK3I->ycen      :  %7.3f\n", MK3I->ycen) ;
      fprintf (stdout, "MK3I->rmin      :  %7.3f\n", MK3I->rmin) ;
      fprintf (stdout, "MK3I->rmax      :  %7.3f\n", MK3I->rmax) ;
      fprintf (stdout, "MK3I->tmin      :  %7.3f\n", MK3I->tmin) ;
      fprintf (stdout, "MK3I->tmax      :  %7.3f\n", MK3I->tmax) ;
      }

   return (0) ;
}
