/* fpc_sq.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Crop a list of MLSO FITS images [PICS or CHIP].
 *------------------------------------------------------------------------------
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction, scan time are obtained from the extension HDU.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   13 Nov 2003   fpc_sq.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/picard/d/sw/fits/cfitsio/fitsio.h"

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{

    extern unsigned char *crop_img (unsigned char *imginp,
                                    int xdiminp, int ydiminp, int nbypix,
                                    float xceninp, float yceninp,
                                    int xdimout, int ydimout);

    float get_frac_offset () ;
    int  usage () ;
    void printerror () ;

    char *w ;				/* Pointer to command line arguments */
    char *parg ;			/* Argument pointer. */
    static char comment   [80] ;
    static char cnaxis1   [80] ;
    static char cnaxis2   [80] ;
    static char ccrpix1   [80] ;
    static char ccrpix2   [80] ;
    static char ccrota1   [80] ;
    static char ccrota2   [80] ;
    static char ccoordnam [80] ;
    static char cdatamin  [80] ;
    static char cdatamax  [80] ;
    static char cdispmin  [80] ;
    static char cdispmax  [80] ;
    static char code_obs   [8] ;
    static char lcode_obs  [8] = "wsr";
    static char coordnam  [24] ;
    static char date_obs  [24] ;		/* Observation date. */
    static char fileinp   [80] ;
    static char fileout   [80] = "!new.fts";
    static char nampgm    [24] ;
    static char object    [24] ;
    static char origin    [24] ;
    static char sdir2     [4] = "ru" ;		/* Scan direction: rd or ru. */
    static char telescop  [24] ;
    static char tscope    [24] ;
    static char instrume  [24] ;
    static char type_obs  [24] ;
    static char time_obs  [24] ;		/* Observation time. */

    static int  extbuf [168] ;			/* FITS extension data. */
    static long naxes    [2] ;

    unsigned char *imgc ;
    int anynull ;
    int bitpix = 16 ;
    int hdutype = 0 ;
    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int ival ;
    int i, ix, iy ;
    int lscale = 0 ;
    int morekeys = 0 ;		/* Additional keywords to be reserved. */
    int naxis = 2 ;
    int nbypix = 2 ;
    int nbyflt = 4 ;
    int ncode = 0 ;
    int nelimg ;
    int nelext = 168 ;
    int nhdunum = 0 ;
    int status = 0 ;
    int xdim, ydim ;
    int xdimsq = 1024, ydimsq = 1024 ;

    long fpixel = 1 ;
    long npix ;
    long npixsq ;
    short smin, smax ;

    short *imgbuf ;			/* Pointer to image array. */
    short *imgsq ;			/* Pointer to square image. */
    short *img1, *img2, *img3, *img4, *img5, *img6, *img7 ;

    float crota1, crota2 ;		/* Coordinate rotation angles. */
    float crradius ;			/* Solar Radius (pixels). */
    float crpix1, crpix2 ;		/* Sun center (pixels). */
    float datamin, datamax ;
    float dispmin, dispmax ;
    float fmin, fmax ;
    float nullval = 0 ;			/* Don't check for null values in img.*/
    float radius ;
    float solar_p0 ;			/* P-angle. */
    float xcen, ycen ;
    float xcensq, ycensq ;
    float xshift, yshift ;		/* x/y shift for cropping. */
    float wavelen = 0.0 ;
    float wmin, wmax ;

    float *fbuf ;

    double scale = 1.0 ;
    double zero  = 0.0 ;

    fitsfile *fpinp, *fpout ;

    /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

    if (argc <= 1) { usage (argv [0]) ; exit (-1) ; }

    strcpy (nampgm, argv [0]) ;

    npixsq = xdimsq * ydimsq ;

    while (--argc)
      {
      if (*(parg = *++argv) == '-')
	{
	switch (parg [1])
	    {
	    case 's':	lscale = 1 ;
			--argc ; parg = *++argv ; sscanf (parg, "%f", &wmin) ;
			--argc ; parg = *++argv ; sscanf (parg, "%f", &wmax) ;
			break ;

	    default:	fprintf (stderr, "Unknown flag %s\n", *argv) ;
			usage (nampgm) ; exit (-1) ;
            }
        }
      else
        {
        strcpy (fileinp, parg) ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fpinp, fileinp, READONLY, &status) )
	    printerror ( status );

        if ( fits_get_num_hdus (fpinp, &nhdunum, &status) ) printerror (status) ;

	/*** Primary FITS header. ***/

	if (fits_read_key (fpinp, TINT, "NAXIS1", &xdim,
	    cnaxis1, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TINT, "NAXIS2", &ydim,
	    cnaxis2, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TINT, "BITPIX", &bitpix,
	    comment, &status))  printerror (status) ;
	nbypix = bitpix / 8 ;

	if (fits_read_key (fpinp, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	printf ("iyear: %d  imonth: %d  iday: %d\n", iyear, imonth, iday) ;

	if (fits_read_key (fpinp, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	printf ("ihour: %d  iminute: %d  isecond: %d\n",
	    ihour, iminute, isecond) ;

	if (fits_read_key (fpinp, TSTRING, "ORIGIN", origin,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TSTRING, "TELESCOP", telescop,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TSTRING, "INSTRUME", instrume,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TSTRING, "OBJECT", object,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TSTRING, "CODE-OBS", code_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TSTRING, "TYPE-OBS", type_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "CRPIX1", &crpix1,
	    ccrpix1, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "CROTA1", &crota1,
	    ccrota1, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "CRPIX2", &crpix2,
	    ccrpix2, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "CROTA2", &crota2,
	    ccrota2, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TSTRING, "COORDNAM", coordnam,
	    ccoordnam, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "DATAMIN",  &datamin,
	    cdatamin, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "DATAMAX",  &datamax,
	    cdatamax, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "DISPMIN",  &dispmin,
	    cdispmin, &status))  printerror (status) ;

	if (fits_read_key (fpinp, TFLOAT, "DISPMAX",  &dispmax,
	    cdispmax, &status))  printerror (status) ;

        ncode = strlen (code_obs) ;
	for (i = 0; i < ncode; i++)
	    {
	    ival = code_obs [i] ;
	    lcode_obs [i] = (char) tolower (ival) ;
	    }

	printf ("code_obs:   %s\n", code_obs) ;
	printf ("lcode_obs:  %s\n", lcode_obs) ;
	printf ("telescop:  [%s]\n", telescop) ;
        strcpy (tscope, "xxx") ;
        if (strncmp (telescop, "DPM",  3) == 0)  strcpy (tscope, "dpm");
        if (strncmp (telescop, "CHIP", 4) == 0)  strcpy (tscope, "chp");

        if (xdim == xdimsq && ydim == ydimsq && crota1 == 0.0 && crota2 == 0)
	    {
	    fprintf (stdout, "%s is already cropped & rotated.\n", fileinp) ;
	    if ( fits_close_file (fpinp, &status) )  printerror (status) ;
	    status = 0 ;
	    break ;
	    }

	/*** Read FITS image data. ***/

	/* Note that even though the FITS images contains unsigned integer */
	/* pixel values (or more accurately, signed integer pixels with    */
	/* a bias of 32768),  this routine is reading the values into a    */
	/* float array.   Cfitsio automatically performs the datatype      */
	/* conversion in cases like this.                                  */

	if (crpix1 >= 0.0)
	    xcen = crpix1 ;
	else
	    xcen = (xdim - 1) * 0.5 ;

	if (crpix2 >= 0.0)
	    ycen = crpix2 ;
	else
	    ycen = (ydim - 1) * 0.5 ;

	xcensq = (xdimsq - 1) * 0.5 ;
	ycensq = (ydimsq - 1) * 0.5 ;

	npix     = xdim * ydim ;
	fbuf     = (float *) malloc (npix * sizeof (float)) ;
	imgbuf   = (short *) malloc (npix * sizeof (short)) ;
	nullval  = 0 ;		/* Don't check for null values in the image. */

	if ( !fits_set_bscale (fpinp, scale, zero, &status) )
	    printerror (status) ;

	if ( fits_read_img (fpinp, TSHORT, fpixel, npix, &nullval, imgbuf,
			    &anynull, &status) )
	    printerror ( status ) ;

        smin =  32767 ;
        smax = -32767 ;
	for (i = 0; i < npix; i++)
	    {
            if ( imgbuf [i] < smin )
                smin = imgbuf [i];

	    if ( imgbuf [i] > smax )
                smax = imgbuf [i];
	    }
	printf ("smin/smax: %d  %d\n", smin, smax) ;

	/*** Crop image: [xdimsq, ydimsq]. ***/

        printf ("xdim/ydim: %4d %4d  nbypix: %d  xcen/ycen: %7.2f %7.2f\n",
                 xdim, ydim, nbypix, xcen, ycen) ;
        printf ("xdimsq/ydimsq: %4d %4d\n", xdimsq, ydimsq) ;
	printf ("crop image.\n");

        if (xdim > xdimsq || ydim > ydimsq)
	    {
            ix = (int) xcen ;
            iy = (int) ycen ;
            imgc = (unsigned char *) imgbuf ;
            printf ("imgc: %d \n", *(imgc + ix + iy*xdim)) ;
            imgsq = (short *) crop_img (imgc, xdim, ydim, nbypix,
				        xcen, ycen, xdimsq, ydimsq) ;
            }
        else
	    {
	    fprintf (stdout, "%s is already cropped.\n", fileinp) ;
	    imgsq = imgbuf ;
	    }

        smin =  32767 ;
        smax = -32767 ;
	for (i = 0; i < npixsq; i++)
	    {
            if ( *(imgsq + i) < smin )
                smin = *(imgsq + i);

	    if ( *(imgsq + i) > smax )
                smax = *(imgsq + i);
	    }
	printf ("smin/smax: %d  %d\n", smin, smax) ;
	fmin = (float) smin ;
	fmax = (float) smax ;

	/*** Create output file name. ***/

	sprintf (fileout, "!%04d%02d%02d.%02d%02d%02d.%s.%s.sq.fts",
		 iyear, imonth, iday, ihour, iminute, isecond,
		 tscope, lcode_obs) ;

	fprintf (stdout, "fileout: %s\n", fileout) ;

	/*** Create output file. ***/

/*
	if ( fits_create_template (&fpout, fileout, fileinp, &status) )
	    printerror ( status ) ;
 */

        if ( fits_create_file (&fpout, fileout, &status) )
	    printerror ( status ) ;

	if ( fits_copy_hdu (fpinp, fpout, 0, &status) )
	    printerror ( status ) ;

/*
        if ( fits_movabs_hdu (fpout, 1, NULL, &status) )
	    printerror ( status );
 */

	/*** Resize image (updates naxis1, naxis2, bitpix). ***/

	naxes [0] = xdimsq ;
	naxes [1] = ydimsq ;
	if (fits_resize_img (fpout, bitpix, naxis, naxes, &status) )
	    printerror ( status ) ;

/*
        if (fits_create_img (fpout, -32, naxis, naxes, &status) )
	    printerror ( status ) ;
 */

	/*** Write square, rotated image array into fits data array. ***/

	if ( !fits_set_bscale (fpout, scale, zero, &status) )
	    printerror (status) ;

	nelimg = xdimsq * ydimsq ;
	if ( fits_write_img (fpout, TSHORT, fpixel, nelimg, imgsq, &status) )
	    printerror ( status ) ;

	/*** Update sun center information. ***/

        if ( fits_update_key (fpout, TFLOAT, "CRPIX1", &xcensq, ccrpix1,
	     &status)) printerror ( status ) ;

        if ( fits_update_key (fpout, TFLOAT, "CRPIX2", &ycensq, ccrpix2,
	     &status)) printerror ( status ) ;

        if ( fits_update_key (fpout, TFLOAT, "DATAMIN", &fmin, cdatamin,
	     &status)) printerror ( status ) ;

        if ( fits_update_key (fpout, TFLOAT, "DATAMAX", &fmax, cdatamax,
	     &status)) printerror ( status ) ;

        if (lscale)
	    {
            if ( fits_update_key (fpout, TFLOAT, "DISPMIN", &wmin, cdispmin,
	         &status)) printerror ( status ) ;

            if ( fits_update_key (fpout, TFLOAT, "DISPMAX", &wmax, cdispmax,
	         &status)) printerror ( status ) ;
            }

        /*** Copy extension HDU. ***/

        if ( nhdunum >= 2)
	    {
            if ( fits_movabs_hdu (fpinp, 2, NULL, &status) )
	        printerror ( status );

/*
            if ( fits_movabs_hdu (fpout, 2, NULL, &status) )
	        printerror ( status );
 */

	    if ( fits_copy_hdu (fpinp, fpout, 0, &status) )
	        printerror ( status ) ;
            }

	/*** Close FITS output file. ***/

	if (fits_close_file (fpout, &status))  printerror ( status ) ;

	/*** Close FITS input file. ***/

	if ( fits_close_file (fpinp, &status) )  printerror (status) ;
	status = 0 ;
        }
   
      }
   exit (0) ;
}

/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */

/* usage (prog_name)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int
usage (prog_name)

char *prog_name ;
{
    static char help [84] ;

    sprintf (help, "%s [-s size] image1 [image2 image3 ...]",
	     prog_name) ;
    fprintf (stderr, "%s\n", help) ;
    fprintf (stderr, " Example:  fpc_sq  20010409.175320.dpm.asl.fts\n") ;

    return (0) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

