/* fts4jpg.c
 * -----------------------------------------------------------------------------
 * Convert FITS image(s) into JPEG format.
 * -----------------------------------------------------------------------------
 * Requires cfitsio and ImageMagick libraries.
 * -----------------------------------------------------------------------------
 * Tony Darnell   HAO/NCAR   5 Mar 2002
 *  2 Feb 2005 : [ALS] Add "-f" [fixed scaling] option.
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <getopt.h>
#include <sys/types.h>

#include "fitsio.h"
#include "magick/api.h"

int scaleimg  (int *img, int xsz, int ysz, int imin, int imax, int top);
int scalechip (int *img, int xsz, int ysz, int imin, int imax, int top,
               float crradius, float xcen, float ycen);

int main (int argc, char *argv [])
{
  FILE     *fptr;
  fitsfile *fileptr;
  int      *img;

  static char filename   [80];
  static char outname    [80];
  static char tmparr     [80];
  static char origin     [20];
  static char instrument [20];
  static char dateobs    [20];
  static char timeobs    [20];
  static char codeobs    [20];
  static char typeobs    [20];
  static char object     [20];
  static char typedata   [20];
  static char coordnam   [20];
  static char colorpath  [24] = "/home/cordyn/color/";
  static char *geometry;

  static int index [255], red [255], green [255], blue [255];
  static int ind, r, g, b;

  int anynull = -1;
  int colorflag;
  int cropflag=0;
  int fixscale = 0;
  int fpixel = 1;
  int i;
  int ii;
  int indx, c;
  int ix, iy;
  int ky;
  int loflag, hiflag, topflag;
  int mystat;
  int nelements;
  int newcalflag = 0;
  int newxsz, newysz;
  int nbyimg;
  int nullval;
  int scalelimb = 1;
  int scalemin, scalemax, top;
  int status = 0;
  int xpos, ypos;
  int xsz, ysz;

  int year, mon, day;
  int leap, doy = 0;
  int day_tab [2][13] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
	                  0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

  float crradius;
  float dispmin, dispmax;
  float xcen, ycen;

  unsigned char *imgbuf;
  unsigned char *newimgbuf;
  unsigned char *colorbuf;

  /* The following are structures specific to the ImageMagick library */

  Image         *jpegimg;
  ImageInfo     *img_info;
  DrawInfo      *info;
  ExceptionInfo exception;
  PixelPacket   textcolor;

  /*** Parse command line ***/

  loflag = hiflag = topflag = colorflag = 1;
  top    = 249;
  opterr =   0;
  fixscale = 0;

  while ((c = getopt (argc, argv, "fl:h:t:c:s:n")) != -1)
    switch (c)
      {
      case 'l':
        scalemin = atoi (optarg);
	loflag = 0;
        break;
      case 'h':
	scalemax = atoi (optarg);
	hiflag = 0;
	break;
      case 't':
	top = atoi (optarg);
	topflag = 0;
	break;
      case 'f':
        fixscale = 1;
        printf ("fixscale: %d\n", fixscale);
        break;
      case 'c':
        fptr = fopen (strcat (colorpath, optarg), "r");
        i=0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
	colorflag = 0;
	break;
      case 's':
	geometry = optarg;
	cropflag = 1;
	break;
      case 'n':
	newcalflag = 1;
	break;
      default:
	abort ();
      }

  /*** File Loop. ***/

  for (indx = optind; indx < argc; indx++)
    {

/*
 *    printf ("Start of loop\n");
 *    fflush (stdout);
 */

    /*** Open FITS file and get image info. ***/

    strcpy (filename, argv [indx]);

    fits_open_file (&fileptr, filename, READONLY, &status);

    fits_read_key  (fileptr, TINT,    "NAXIS1",   &xsz,        NULL, &status);
    fits_read_key  (fileptr, TINT,    "NAXIS2",   &ysz,        NULL, &status);
    fits_read_key  (fileptr, TSTRING, "ORIGIN",   origin,      NULL, &status);
    fits_read_key  (fileptr, TSTRING, "TELESCOP", instrument,  NULL, &status);
    fits_read_key  (fileptr, TSTRING, "DATE-OBS", dateobs,     NULL, &status);
    fits_read_key  (fileptr, TSTRING, "TIME-OBS", timeobs,     NULL, &status);
    fits_read_key  (fileptr, TSTRING, "CODE-OBS", codeobs,     NULL, &status);
    fits_read_key  (fileptr, TSTRING, "TYPE-OBS", typeobs,     NULL, &status);
    fits_read_key  (fileptr, TSTRING, "OBJECT",   object,      NULL, &status);
    fits_read_key  (fileptr, TSTRING, "COORDNAM", coordnam,    NULL, &status);
    fits_read_key  (fileptr, TFLOAT,  "DISPMIN",  &dispmin,    NULL, &status);
    fits_read_key  (fileptr, TFLOAT,  "DISPMAX",  &dispmax,    NULL, &status);

    if (strstr (instrument, "CHIP"))
      {
      fits_read_key  (fileptr, TSTRING, "TYPE-DP",  typedata, NULL, &status);
      fits_read_key  (fileptr, TFLOAT,  "CRRADIUS", &crradius, NULL, &status);
      fits_read_key  (fileptr, TFLOAT,  "CRPIX1",   &xcen,     NULL, &status);
      fits_read_key  (fileptr, TFLOAT,  "CRPIX2",   &ycen,     NULL, &status);
      }
  
    /*** Figure out the DOY from "dateobs". ***/

    fits_str2date (dateobs, &year, &mon, &day, &status);
    leap = year%4 == 0 && year%100 != 0 || year%400 == 0 ;
    doy = 0;
    for (ii = 1; ii < mon; ii++)  doy += day_tab [leap][ii];
    doy += day;

    /*
     *  printf ("instrument: %s\n", instrument) ;
     *  fflush (stdout) ;
     */

    if (loflag) { scalemin = (int) (dispmin + 0.5);}
    if (hiflag) { scalemax = (int) (dispmax + 0.5);}

    /*** Read image from file. ***/

    nelements = xsz * ysz;
    img = (int *) malloc (nelements * sizeof (int));

    /* fits_read_img (fileptr, TINT, fpixel, nelements, &nullval, img, 
     *                &anynull,&status);
     */

    if (fits_read_img (fileptr, TINT, fpixel, nelements, &nullval, img, 
		                &anynull, &status))
      fits_report_error (stderr, status);

    fits_close_file (fileptr, &status);

    /* fits_report_error (stderr, status); */

    /*** Assign color table according to instrument. ***/

    if (strstr (instrument, "CHIP"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/RED_TEMPERATURE.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }
      if (strstr (typedata, "VEL"))
        {
        textcolor.red   = MaxRGB;
        textcolor.green = MaxRGB;
        textcolor.blue  = MaxRGB;
        scalelimb       = MaxRGB;
        }
      else
        {
        textcolor.red   = 0;
        textcolor.green = 0;
        textcolor.blue  = 0;
        }
/*
 *      if (loflag) { scalemin = -1200;}
 *      if (hiflag) { scalemax =    50;}
 */
      if (fixscale) {scalemin = -1200; scalemax = 50;}
      }
    else
    if (strstr (instrument, "MK4"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/qual.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }

      textcolor.red   = MaxRGB;
      textcolor.green = MaxRGB;
      textcolor.blue  = MaxRGB;
/*
 *      if (loflag) { scalemin = -350; }
 *      if (hiflag) { scalemax = 1200; }
 */
      if (fixscale) {scalemin = - 350; scalemax = 1200;}
      }
    else
    if (strstr (instrument, "MK3"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/qual.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }

      textcolor.red   = MaxRGB;
      textcolor.green = MaxRGB;
      textcolor.blue  = MaxRGB;
/*
 *      if (loflag) { scalemin = -1100; }
 *      if (hiflag) { scalemax =  4000; }
 */
      if (fixscale) {scalemin = -1100; scalemax = 4000;}
      }
    else
    if (strstr (instrument, "DPM"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/RED_TEMPERATURE.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
          {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }

      textcolor.red   = MaxRGB;
      textcolor.green = MaxRGB;
      textcolor.blue  = MaxRGB;

/*
 *      if (loflag)
 *        {
 *        if      (strstr (object, "SUN"))   { scalemin = 1; } 
 *        else if (strstr (object, "LIMB"))  { scalemin = 1; }
 *        }
 *      if (hiflag)
 *        {
 *        if      (strstr (object, "SUN"))   { scalemax = 600; } 
 *        else if (strstr (object, "LIMB"))  { scalemax = 200; }
 *        }
 */
      if (fixscale)
        {
        printf ("fixscale: %d\n", fixscale);
        if (strstr (codeobs, "ASC" )) {scalemin =  250; scalemax =  750;}
        if (strstr (codeobs, "ASL" )) {scalemin =  800; scalemax = 1300;}
        if (strstr (object,  "LIMB")) {scalemin =    4; scalemax =   64;}
        printf ("scalemin/max: %d  %d\n", scalemin, scalemax);
        }
      }

    /*** Perform image scaling and create image buffers. ***/

    if ((strstr (instrument, "CHIP") && scalelimb) && (newcalflag))
      scalechip (img, xsz, ysz, scalemin, scalemax, top,
		 crradius, xcen, ycen);
    else 
      scaleimg (img, xsz, ysz, scalemin, scalemax, top);

    nbyimg = xsz * ysz * sizeof (unsigned char);

    imgbuf    = (unsigned char *) malloc (nbyimg);
    newimgbuf = (unsigned char *) malloc (nbyimg);
    colorbuf  = (unsigned char *) malloc (nbyimg * 3);

    /*** Convert int array to byte array. ***/

    for (i = 0; i < nelements; i++)  imgbuf [i] = (unsigned char) img [i];

    /*** Invert image top to bottom. ***/

    for (iy = 0; iy < ysz; iy++)
      {
      ky = ysz - iy - 1 ;
      for (ix = 0; ix < xsz; ix++)
	*(newimgbuf + ix + (iy*xsz)) = *(imgbuf + ix + (ky*xsz));
      }

    /*** RGB interlacing. ***/

    for (i = 0; i < nelements; i++)
      {
      colorbuf [i*3]   = (unsigned char) red   [newimgbuf [i]];
      colorbuf [i*3+1] = (unsigned char) green [newimgbuf [i]];
      colorbuf [i*3+2] = (unsigned char) blue  [newimgbuf [i]];
      }

    /*** Create ImageMagick image, annotate, apply filters and resize. ***/

    GetExceptionInfo (&exception);
    img_info = CloneImageInfo  ((ImageInfo *) NULL);
    jpegimg  = ConstituteImage (xsz, ysz, "RGB", CharPixel, colorbuf,
                                  &exception);
    info       = CloneDrawInfo   (img_info, (DrawInfo *) NULL);

    if (!(cropflag))
      {
      if (xsz == ysz)
        {
        newxsz = 512; newysz = 512;
        jpegimg = ScaleImage (jpegimg, 512, 512, &exception);
        xpos = 256; ypos = 256;
        }
      else
        {
        newxsz = xsz / 2;
        newysz = ysz / 2;

        jpegimg = MinifyImage (jpegimg, &exception);

        xpos = newxsz / 2;
        ypos = newysz / 2;
        }
      }
    else
      {
      printf ("Cropping image...\n"); 
      newxsz = 512; newysz = 512;
      xpos = 256; ypos = 256;
      TransformImage (&jpegimg, geometry, "512x512");
      }

  /*
    ContrastImage (jpegimg, 1);
    NormalizeImage (jpegimg);
    jpegimg = EnhanceImage (jpegimg, &exception);
    info.stroke = textcolor;
    GetDrawInfo (img_info, &info);
    info.fill = textcolor;
    info.box = textcolor;
    info.text_antialias = 1;
    info.pointsize = 18;
    info.border_color = textcolor;
    info.box = textcolor;
    info.primitive = tmparr;
    info.primitive = tmparr;
    info.primitive = tmparr;
    status = DrawImage (jpegimg, info);
    mystat = AnnotateImage (jpegimg, info);
    printf ("%d\n", mystat);
    info->pointsize = 18;
    info->font = "-*-clean-medium-r-*-*-16-*-*-*-*-*-*";
    info->font = "-*-fixed-medium-r-*-*-18-*-*-*-*-iso8859-1";
    info->font = "-*-courier-medium-r-*-*-17-*-*-*-*-*-iso8859-1";
   */

    info->fill.opacity = 255;
    info->fill = textcolor;

    sprintf (tmparr, "text 3,20 '%s %s' text 3,40 '%s' text 3,60 '%s' text 3,100 'DOY: %d' ",
             origin, instrument, typeobs, dateobs, doy);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimg, info); 

    sprintf (tmparr, "text 3, 80 '%s UT' text %d, 22 'N' text %d, %d 'W'", 
             timeobs, xpos, newxsz-24, ypos);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimg, info);

    sprintf (tmparr, "text %d,%d 'S' text 12,%d 'E'",
             xpos, newysz-12, ypos);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimg, info);

    sprintf (tmparr, "text 3,%d 'Scaling: %d to %d'",
	     newysz-10, scalemin, scalemax);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimg, info);

    sprintf (tmparr, "text %d,%d '%s'",
             newxsz-100, newysz-10, coordnam);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimg, info);

    /*** Save image as desired filetype. ***/

    strcpy (outname, filename);
    strcpy (strstr (outname, "fts"), "jpg");
    printf ("%s\n", outname);
    (void) strcpy (jpegimg->filename, outname);

    WriteImage (img_info, jpegimg);

    /*
     *  printf ("End of loop\n");
     *  fflush (stdout);
     */

    free (newimgbuf);
    free (imgbuf);
    free (colorbuf);
    free (img);
    }  /* File Loop */
}  /* Main */



/* -------------------------------------------------------------------------- */

/*
 * scaleimg applies a linear scaling to the pixel data
 * from supplied data min, max values as well as a top.
 * Works just like IDL's bytscl routine.
 *
 * 24 Jan 2005 [ALS]:
 * NOTE: ImageMagick apparently sets a pixel intensity of 0 (zero) to white.
 * Consequently, I have avoided using this pixel magnitude.
 */

int scaleimg (int *img, int xsz, int ysz, int imin, int imax, int top)

{
  int i; 
  int ipix;
  int nelements;
  int newpix;
  int omin   = 0;
  int omax   = top;
  int opix;
  int offset = imin;

  float bscale;
  float spix;

  /*
   *  printf ("scaleimg> xsz:%4d  ysz:%4d  imin:%5d  imax:%5d  top:%5d\n",
   *          xsz, ysz, imin, imax, top) ;
   */

  nelements = xsz * ysz;

  bscale = (float) (imax - imin) / (float) (omax - omin);
  for (i = 0; i < nelements; i++)
    {
    ipix = *(img + i);
    if (ipix <= -30000)
      {
      *(img + i) = 1;
      }
    else if (ipix == -3000)
      {
      *(img+i) = 1;
      }
    else if (ipix == 2000)
      {
      *(img + (i-1)) = omax;
      *(img +  i   ) = omax;
      *(img + (i+1)) = omax;
      }
    else
      {
      spix = (float) (ipix - offset) / bscale;  /* Adjusted pixel intensity. */
      if (spix <  0.0) newpix = (int) (spix - 0.5);	/* round down. */
      if (spix >= 0.0) newpix = (int) (spix + 0.5);	/* round up.   */
      opix = newpix;
      if (newpix < omin+1) { opix =    1;}		/* underflow. */
      if (newpix > omax  ) { opix = omax;}		/* overflow.  */
      *(img + i) = (int) opix;
/*
 *      if (*(img + i) < 0 || *(img + i) > top)
 *        printf (">>>>>>> scaleimg> i: %4d  img: %4d\n", i, *(img + i));
 */
      }
    }

  return (0);
}


/* -------------------------------------------------------------------------- */

/*
 * scalechip applies different scaling to the limb than it does to the
 * disk because we see the disk in absorption and the limb in emmission.
 * The scaling values are currently hard-coded in and may need to be
 * added to the command line at some point in the future.
 *
 */

int scalechip (int *img, int xsz, int ysz, int imin, int imax, int top, 
	       float crradius, float xcen, float ycen)

{
  int i;
  int ipix;
  int ix, iy; 
  int limb_imin   =   0;
  int limb_imax   = 350;
  int limb_offset = limb_imin;
  int nelements;
  int newpix;
  int offset = imin; 
  int omax   = top;
  int omin   = 0;
  int opix;

  float disc_bscale;
  float limb_bscale;
  float radius;
  float spix;
  float xdist, ydist;

  nelements = xsz * ysz;

  disc_bscale = (float) (imax - imin) / (float) (omax - omin);
  limb_bscale = (float) (limb_imax - limb_imin) / (float) (omax - omin);

  for (iy = 0; iy < ysz; iy++)
    for (ix = 0; ix < xsz; ix++)
      {
      ipix = *(img + ix + iy * xsz);
      xdist = (float) ix - xcen ;
      ydist = (float) iy - ycen ;
      radius = sqrt ((xdist * xdist) + (ydist * ydist));
      if ((radius / crradius) < 1.0)
	{
	if (ipix < imin) ipix = imin;
	if (ipix > imax) ipix = imax;
    
        spix = (float) (ipix - offset) / disc_bscale;
        if (spix <  0.0) newpix = (int) (spix - 0.5);
        if (spix >= 0.0) newpix = (int) (spix + 0.5);
        opix = newpix;
        if (newpix <    1) opix = 1;
        if (newpix > omax) opix = omax;
        *(img + ix + iy * xsz) = opix;
        }
      else
        {
	if (ipix == -1000.0)
	    {*(img + ix + iy * xsz) = limb_imax;}
        else
          {
          spix = (float) (ipix - limb_offset) / limb_bscale;
          if (spix <  0.0) newpix = (int) (spix - 0.5);
          if (spix >= 0.0) newpix = (int) (spix + 0.5);
          opix = newpix;
          if (newpix <    1) opix = 1;
          if (newpix > omax) opix = omax;
          *(img + ix + iy * xsz) = opix;
          }
      }
    }
  return (0);
}

/* -------------------------------------------------------------------------- */

