/*
 * FitEllipse.c
 *
 * Written April 7, 2000 by Randy Meisner, HAO/NCAR, Boulder, CO
 * (meisner@ucar.edu)
 *
 * PROJECT:  SunRISE/PSPT
 *
 * PURPOSE:  To fit a set of (x, y) data points to an ellipse, using SVD.
 *
 * INPUTS:   long   n - the number of (x, y) data points
 *           float *x - x[n] data points (indicies ranging from 1 to n).
 *           float *y - y[n] data points (indicies ranging from 1 to n).
 *
 * OUTPUTS:  float *xc - x coordinate of center of ellipse
 *           float *yc - y coordinate of center of ellipse
 *           float *rx - ellipse radius in x direction
 *           float *ry - ellipse radius in y direction
 *
 * RETURNS:  0 if ellipse fit is successful
 *           1 if ellipse fit fails
 *
 * CALLS:    SVD.c
 *
 *****************************************************************************/

#include "reduction.h"

void FitEllipse(long n, float *x, float *y,
		float *xc, float *yc, float *rx, float *ry){
  
  long i;
  
  float aa, bb, cc, dd, ee, ff, det, phi, aaa, bbb, fff;

  float *b, *xx, **a;


  /* setup a matrix and b vector for svd computation */

  *xc = 0.0;
  *yc = 0.0;
  *rx = 0.0;
  *ry = 0.0;

  b = vector(1, n);

  xx = vector(1, 5);

  a = matrix(1, n, 1, 5);


  for(i = 1; i <= n; i++){

    b[i] = (-1.0*(x[i]*x[i]))-(y[i]*y[i]);

    a[i][1] = (x[i]*x[i])-(y[i]*y[i]);
    a[i][2] = x[i]*y[i];
    a[i][3] = x[i];
    a[i][4] = y[i];
    a[i][5] = 1.0;

  }


  /* run the SVD calculation */

  SVD(n, 5, a, b, xx);


  /* Determine the center of the ellipse and the semi-axes. */

  aa = 1.0+xx[1];
  bb = 1.0-xx[1];
  cc = xx[2];
  dd = xx[3];
  ee = xx[4];
  ff = xx[5];

  printf("%f  %f  %f  %f  %f  %f\n", aa,bb,cc,dd,ee,ff);

  det = 4.0f*aa*bb-(cc*cc);

  if(det == 0){
    
    printf("Fit Ellipse:  Not a quadratic form!\n");
    
    *xc = -1.0;
    *yc = -1.0;
    *rx = -1.0;
    *ry = -1.0;

  } else {

    *xc = (cc * ee - 2.0 * bb * dd) / det;
    *yc = (cc * dd - 2.0 * aa * ee) / det;

    phi = (float) (0.5*atan(cc/(aa-bb+pow(1.,-32))));

    aaa = (float) (0.5*(aa+bb+(aa-bb)/cos(2.0*phi)));

    bbb = aa + bb - aaa;

    fff = aa*(*xc)*(*xc) + bb*(*yc)*(*yc) + cc*(*xc)*(*yc) - ff;

    if(fff/aaa < 0 || fff/bbb < 0){
      
      printf("Fit Ellipse:  Not an ellipse!\n");
      
      *rx = -1.0;
      *ry = -1.0;

    } else {

      *rx = (float) (sqrt(fff/aaa));
      *ry = (float) (sqrt(fff/bbb));
      
    }
    
  }

  free_vector(b, 1, n);
  free_vector(x, 1, 5);
  free_matrix(a, 1, n, 1, 5);

}
