/*
 * SVD.c
 *
 * Written April 7, 2000 by Randy Meisner, HAO/NCAR, Boulder, CO
 * (meisner@ucar.edu)
 *
 * PROJECT:  SunRISE/PSPT
 *
 * PURPOSE:  To provide a solution of linear algebraic equations, using
 *           singular value decomposition (Solves the equation A x = b for x).
 *
 * INPUTS:   long     m - dimension 1 of matrix A
 *           long     n - dimension 2 of matrix A
 *           float **A - matrix A (indicies ranging from 1 to m, n).
 *           float  *B - vector b (indicies ranging from 1 to m).
 *
 * OUTPUTS:  float  *X - solution vector x (indicies ranging from 1 to n).
 *
 * CALLS:    svdcmp.c, svbksb.c
 *
 *****************************************************************************/

#include "reduction.h"

void SVD(long m, long n, float **A, float *B, float *X){

  /* A[m][n]
   * B[m]
   * X[n]
   */

  float **aa, *bb, *xx;
  float **v, *w;

  float wmax = -10000.0, wmin = 10000.0;

  long i, j;


  /* copy A, B into a, b */

  aa = matrix(1, m, 1, n);

  bb = vector(1, m);

  xx = vector(1, n);

  for(i = 1; i <= m; i++){

    bb[i] = B[i];

    for(j = 1; j <= n; j++){

      aa[i][j] = A[i][j];

    }
  }


  /* perform the singular value decomposition */

  v = matrix(1, n, 1, n);

  w = vector(1, n);

  svdcmp(aa, m, n, w, v);

  for(i = 1; i <= n; i++){
    printf("w: %f\n",w[i]);
    if(w[i] > wmax) wmax = w[i];
  }

  wmin = wmax*1.0e-6;

  printf("wmin  %f     wmax  %f\n", wmin, wmax);

  for(i = 1; i <= n; i++) if(w[i] < wmin) w[i] = 0.0;


  /* obtain the solution vector, x, via backsubstitution */

  svbksb(aa, w, v, m, n, bb, xx);


  /* copy the solution vector, x, into the output solution vector, X */

  for(i = 1; i <= n; i++)
    X[i] = xx[i];


  /* free up memory */

  free_matrix(aa, 1, m, 1, n);
  free_matrix(v, 1, n, 1, n);

  free_vector(bb, 1, m);
  free_vector(xx, 1, n);
  free_vector(w, 1, n);

}
