/*
 * float **im - full disk solar image array
 * float minint - minimum intensity at the solar limb
 * long xdim - image size in x dimension
 * long ydim - image size in y dimension
 * float *xc - x center coordinate of solar disk
 * float *yc - y center coordinate of solar disk
 * float *rx - radius of solar disk in x dimension
 * float *ry - radius of solar disk in y dimension
 *
 */


#include "reduction.h"

float slope = 0.0, yint = 0.0;

void SolarGeom(float **im, float minint, long xdim, long ydim,
	       float *xc, float *yc, float *rx, float *ry){

  float threshold = 20.0, total = 0.0, dr = 5.0, rr = 0.0, amp = 0.0;

  float xacc = 0.0001;

  float **lim, **sim, *xdat, *ydat, *xdata, *ydata, *xvec, *yvec;

  float xxc = 0.0, yyc = 0.0, rrx = 0.0, rry = 0.0, r = 0.0;

  long i, j, k, ii, jj, dx, dy, count;


  /* GET INITIAL GEOMETRY ESTIMATE */
  /* ============================= */

  /* allocate sobel image array */

  sim = matrix(1, xdim, 1, ydim);

  /* make bi-level image based on minint */

  for(i = 1; i <= xdim; i++)
    for(j = 1; j <= ydim; j++)
      if(im[i][j] > minint){
	sim[i][j] = 1.0;
      } else {
	sim[i][j] = 0.0;
      }


  /* sobel filter the bi-level image */

  sobel(sim, xdim, ydim);


  /* get the sum of the sobel image so we know how many (x,y) points we
     will store for the ellipse fitting procedure */

  total = 0.0;

  for(i = 1; i <= xdim; i++)
    for(j = 1; j <= ydim; j++){
      if(sim[i][j] != 0) sim[i][j] = 1.0;
      total += sim[i][j];
    }


  /* allocate memory for (x,y) data vectors */

  xdat = vector(1, (long) total);
  ydat = vector(1, (long) total);

  /* store (x,y) data points of limb for ellipse fit */

  count = 1;

  for(i = 1; i <= xdim; i++)
    for(j = 1; j <= ydim; j++)
      if(sim[i][j] != 0.0){
	xdat[count] = (float) i;
	ydat[count] = (float) j;
	count++;
      }

  count--;

  /* free sobel image memory */

  free_matrix(sim, 1, xdim, 1, ydim);

  /* check that count and total are equal */

  if((float) count != total){
    printf("count = %ld\n", count);
    printf("total = %f\n", total);
    printf("ERROR!  count & total are not equal.\n");
    exit(1);
  }


  /* fit limb points to ellipse */

  FitEllipse(count, xdat, ydat, &xxc, &yyc, &rrx, &rry);

  r = (rrx+rry)/2.0;

  printf("INITGEOM\n--------\n");
  printf("XC:  %11.6f    YC:  %11.6f\n", xxc, yyc);
  printf("RX:  %11.6f    RY:  %11.6f\n", rrx, rry);
  printf(" R:  %11.6f    NP:  %5ld\n\n", r, count);

  /* free xdat and ydata vector memory */

  free_vector(xdat, 1, (long) total);
  free_vector(ydat, 1, (long) total);


  /* DETERMINE SOLAR GEOMETRY WITH LAPLACIAN IMAGE */
  /* ============================================= */


  /* allocate memory for Laplacian image */

  lim = matrix(1, xdim, 1, ydim);

  /* generate laplacian image */

  Laplacian(im, xdim, ydim, lim);

  /* determine 1% edges to avoid any dead pixels */

  dx = (long) (0.01*xdim);
  dy = (long) (0.01*ydim);


  /* search the laplacian image for zero crossings within +/- 10 pixels of
     initial geometry radius, and with amplitude of "threashold" or greater */

  xdata = vector(1, 20000);
  ydata = vector(1, 20000);

  xvec = vector(1, xdim);
  yvec = vector(1, ydim);

  ii = 1;

  for(i = dx; i <= (xdim-dx); i++)
    for(j = dy; j <= (ydim-dy); j++){

      rr = sqrt( ((i-xxc)*(i-xxc)) + ((j-yyc)*(j-yyc)) );

      if(fabs(r-rr) < dr){

	/* check x dimension for zero crossing */

	if(lim[i][j]*lim[i+1][j] < 0.0){
	  for(k = 1; k <= xdim; k++) xvec[k] = lim[k][j];
	  amp = local_amp(xvec, xdim, i);
	  if(amp >= threshold){
	    slope = lim[i+1][j]-lim[i][j];
	    yint = (-1.0*slope*i)+lim[i][j];
	    xdata[ii] = rtbis(lineq, (float) i, (float) (i+1), xacc);
	    ydata[ii] = (float) j;
	    ii++;
	  }/* endif */
	}/* endif */


	/* check y dimension for zero crossing */

	if(lim[i][j]*lim[i][j+1] < 0.0){
	  for(k = 1; k <= ydim; k++) xvec[k] = lim[i][k];
	  amp = local_amp(xvec, xdim, i);
	  if(amp >= threshold){
	    slope = lim[i][j+1]-lim[i][j];
	    yint = (-1.0*slope*j)+lim[i][j];
	    ydata[ii] = rtbis(lineq, (float) j, (float) (j+1), xacc);
	    xdata[ii] = (float) i;
	    ii++;
	  }/* endif */
	}/* endif */


      }/* endif */

    }/* endfor's */

  ii--;

  /* free xvec and yvec vector memory */

  free_matrix(lim, 1, xdim, 1, ydim);
  free_vector(xvec, 1, xdim);
  free_vector(yvec, 1, ydim);

  /* fit xdata, ydata to ellipse */

  FitEllipse(ii, xdata, ydata, xc, yc, rx, ry);

  r = (*rx + *ry)/2.0;

  printf("SOLARGEOM\n--------\n");
  printf("XC:  %11.6f    YC:  %11.6f\n", *xc, *yc);
  printf("RX:  %11.6f    RY:  %11.6f\n", *rx, *ry);
  printf(" R:  %11.6f    NP:  %5ld\n\n", r, ii);

} /* end SolarGeom.c */


float lineq(float x){

  return (slope*x+yint);

}
