/* fits_shift.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * "Shift" [translate] a FITS image by the amount specified.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Uses cfitsio library for FITS operations.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   12 March 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/opt/share/PUBLIC/include/fitsio.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern float *rd_fits () ;
    extern float blint () ;

    static char comment  [80] ;
    static char ifilename [80] ;
    static char ofilename [80] ;

    unsigned char *imginp ;
    unsigned char *imgout ;
    unsigned char *cibuf, *cobuf ;
    short *sibuf, *sobuf ;
    float *fibuf, *fobuf ;

    int bitpix ;
    int i, idot ;
    int ix, iy ;
    int morekeys = 0 ;
    int nbypix ;
    int status, nfound, anynull ;

    long xdim, ydim ;
    long fpixel = 1, npix ;

    float crpix1, crpix2 ;
    float fval ;
    float minint ;
    float nullval = 0 ;
    float xrad, yrad ;
    float xcen, ycen ;
    float xcen2, ycen2 ;
    float xshift, yshift ;
    float xx, yy ;

    fitsfile *fip ;	/* Pointer to the FITS file, defined in fitsio.h */
    fitsfile *fop ;


    /*** Get FITS file name from command line. ***/

    strcpy (ifilename, argv [1]) ;
    strcpy (&xshift,   argv [2]) ;
    strcpy (&yshift,   argv [3]) ;

    fprintf (stdout, "input filename: %s\n", ifilename) ;
    idot = -1 ;
    for (i = 0 ; i < strlen (ifilename) ; i++)
	{
	if (ifilename [i] == '.' && ifilename [i+1] == 'f') idot = i ;
	}
    if (idot == -1)
	{
	fprintf (stderr, "Invalid input file : %s\n", ifilename) ;
	exit (-1) ;
	}
    else
	{
	strncpy (ofilename, ifilename, idot) ;
	strcat (ofilename, ".cen.fts") ;
	fprintf (stdout, "output file name: %s\n", ofilename) ;
	}

    status = 0 ;

    /*** Open fits file. ***/

    if ( fits_open_file (&fip, ifilename, READWRITE, &status) )
         printerror ( status ) ;

    /*** Read the NAXIS1 and NAXIS2 keyword to get image size. ***/

    if ( fits_read_key (fip, TINT, "NAXIS1", &xdim, comment, &status) )
         printerror ( status ) ;
    if ( fits_read_key (fip, TINT, "NAXIS2", &ydim, comment, &status) )
         printerror ( status ) ;
    
    printf ("xdim: %4d  ydim: %4d\n", xdim, ydim) ;

    if ( fits_read_key (fip, TINT, "BITPIX", &bitpix, comment, &status) )
         printerror ( status ) ;
    nbypix = bitpix / 8 ;

    if ( fits_read_key (fip, TFLOAT, "CRPIX1", &crpix1, comment, &status) )
         printerror ( status ) ;
    if ( fits_read_key (fip, TFLOAT, "CRPIX2", &crpix2, comment, &status) )
         printerror ( status ) ;

    /*** If CRPIX1 & CRPIX2 are positive, disc center is already known. ***/

        printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
/*
    if (crpix1 > 0.0 && crpix2 > 0.0)
	{
        printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
        printf ("Disc center is already known.  No need to recompute & update.\n") ;
	exit (0) ;
	}
 */

    /* Read FITS image data. */

    /* Note that even though the FITS images contains unsigned integer */
    /* pixel values (or more accurately, signed integer pixels with    */
    /* a bias of 32768),  this routine is reading the values into a    */
    /* float array.   Cfitsio automatically performs the datatype      */
    /* conversion in cases like this.                                  */

    npix     = xdim * ydim ;
    imginp   = (unsigned char *) malloc (npix * nbypix) ;
    imgout   = (unsigned char *) malloc (npix * nbypix) ;
    fpixel   = 1 ;
    nullval  = 0 ;               /* Don't check for null values in the image. */

    if ( fits_read_img (fip, TFLOAT, fpixel, npix, &nullval,
                        imginp, &anynull, &status) )
         printerror ( status ) ;

    /*** Limit precision to 2 decimal places. ***/

    xcen = crpix1 ;   
    ycen = crpix2 ;
    xcen = ((int)(xcen * 100.0) ) / 100.0 ;
    ycen = ((int)(ycen * 100.0) ) / 100.0 ;

    printf ("xcen/ycen: %7.2f %7.2f\n", xcen, ycen) ;

    /*** Perform translation using bilinear interpolation. ***/

    if (nbypix == 1)
	{
	cibuf = (unsigned char *) imginp ;
	cobuf = (unsigned char *) imgout ;
        for (iy = 0 ; iy < ydim ; iy++)
	    {
            yy = (float) iy + yshift ;
	    for (ix = 0 ; ix < xdim ; ix++)
	        {
	        xx = (float) ix + xshift ;
	        fval = blint (cibuf, nbypix, xdim, ydim, xx, yy) ;
	        if (fval < 0)
	            *(cobuf + ix + iy*xdim) = 0 ;
                else
	            *(cobuf + ix + iy*xdim) = (unsigned char) fval ;
	        }
	    }
	}
    else
    if (nbypix == 2)
	{
	sibuf = (short *) imginp ;
	sobuf = (short *) imgout ;
        for (iy = 0 ; iy < ydim ; iy++)
	    {
            yy = (float) iy + yshift ;
	    for (ix = 0 ; ix < xdim ; ix++)
	        {
	        xx = (float) ix + xshift ;
	        fval = blint (sibuf, nbypix, xdim, ydim, xx, yy) ;
	        if (fval < 0)
	            *(sobuf + ix + iy*xdim) = 0.0 ;
                else
	            *(sobuf + ix + iy*xdim) = (short) fval ;
	        }
	    }
	}
    else
    if (nbypix == 4)
	{
	fibuf = (float *) imginp ;
	fobuf = (float *) imgout ;
        for (iy = 0 ; iy < ydim ; iy++)
	    {
            yy = (float) iy + yshift ;
	    for (ix = 0 ; ix < xdim ; ix++)
	        {
	        xx = (float) ix + xshift ;
	        fval = blint (fibuf, nbypix, xdim, ydim, xx, yy) ;
	        if (fval < 0)
	            *(fobuf + ix + iy*xdim) = 0.0 ;
                else
	            *(fobuf + ix + iy*xdim) = (short) fval ;
	        }
	    }
	}
    else
	{
	fprintf (stderr, "Invalid data type . . . bitpix = %d\n", bitpix) ;
	}


    if (fits_create_file (&fop, ofilename, &status))	/* create FITS file*/
         printerror ( status );		/* call printerror if error occurs */

    /*** Copy the primary array from the input file to the output file ***/

    morekeys = 0;	/* Don't reserve space for additional keywords */
    if ( fits_copy_hdu (fip, fop, morekeys, &status) )
         printerror ( status );

    /*** Update the CRPIX1 & CRPIX2 keywords with the modified center. ***/

    xcen2 = xcen + xshift ;
    ycen2 = ycen + yshift ;
    if ( fits_update_key (fop, TFLOAT, "CRPIX1", &xcen2, 0, &status) )
         printerror( status );

    if ( fits_update_key (fop, TFLOAT, "CRPIX2", &ycen2, 0, &status) )
         printerror( status );

    /*** Write the image array to the FITS file ***/

    if (nbypix == 1)
        if ( fits_write_img (fop, TBYTE, fpixel, npix, cobuf, &status) )
            printerror ( status );            

    if (nbypix == 2)
        if ( fits_write_img (fop, TSHORT, fpixel, npix, sobuf, &status) )
            printerror ( status );            

    if (nbypix == 4)
        if ( fits_write_img (fop, TFLOAT, fpixel, npix, fobuf, &status) )
            printerror ( status );            

    /*** Close FITS files. ***/

    if ( fits_close_file (fip, &status) )  printerror ( status ) ;
    if ( fits_close_file (fop, &status) )  printerror ( status ) ;

    exit (0) ;
}

