/* freg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find sun center & radius of a solar disc image in FITS format.
 * Shift image to co-align sun center with array center.
 *
 * Use "findshift" to locate sun center.
 * Use "doshift"   to translate image.
 *
 * Update CRPIX1 & CRIPIX2 values in FITS header.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Uses cfitsio library for FITS operations.
 * Uses findshift.c & doshift.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    6 Apr 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/opt/share/PUBLIC/include/fitsio.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern float *rd_fits () ;

    static char comment  [80] ;
    static char filename [80] ;

    int status, nfound, anynull ;
    int border = 6 ;
    int ix, iy ;

    short *imgc ;

    long xdim, ydim ;
    long xdimc, ydimc ;
    long fpixel, npix ;

    float crpix1, crpix2 ;
    float *imgbuf ;
    float minint ;
    float nullval ;
    float xrad, yrad ;
    float xcen, ycen ;
    float xc, yc ;
    float xshift, yshift ;
    float diam ;

    fitsfile *fptr ;	/* Pointer to the FITS file, defined in fitsio.h */

    if (argc <= 1)
	{
	fprintf (stderr, "usage: %s fits_file_name\n", argv [0]) ;
	exit (-1) ;
	}

    /*** Get FITS file name from command line. ***/

    strcpy (filename, argv [1]) ;

    printf ("filename: %s\n", filename) ;

    status = 0 ;

    /* Open fits file. */

    if ( fits_open_file (&fptr, filename, READWRITE, &status) )
         printerror ( status ) ;

    /* Read the NAXIS1 and NAXIS2 keyword to get image size. */

    if ( fits_read_key (fptr, TINT, "NAXIS1", &xdim, comment, &status) )
         printerror ( status ) ;
    if ( fits_read_key (fptr, TINT, "NAXIS2", &ydim, comment, &status) )
         printerror ( status ) ;
    
    printf ("xdim: %4d  ydim: %4d\n", xdim, ydim) ;

    if ( fits_read_key (fptr, TFLOAT, "CRPIX1", &crpix1, comment, &status) )
         printerror ( status ) ;
    if ( fits_read_key (fptr, TFLOAT, "CRPIX2", &crpix2, comment, &status) )
         printerror ( status ) ;



    /*** If CRPIX1 & CRPIX2 are positive, disc center is already known. ***/

        printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
/*
    if (crpix1 > 0.0 && crpix2 > 0.0)
	{
        printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
        printf ("Disc center is already known.  No need to recompute & update.\n") ;
	exit (0) ;
	}
 */


    /* Read FITS image data. */

    /* Note that even though the FITS images contains unsigned integer */
    /* pixel values (or more accurately, signed integer pixels with    */
    /* a bias of 32768),  this routine is reading the values into a    */
    /* float array.   Cfitsio automatically performs the datatype      */
    /* conversion in cases like this.                                  */

    npix     = xdim * ydim ;
    imgbuf   = (float *) malloc (npix * sizeof (float)) ;
    fpixel   = 1 ;
    nullval  = 0 ;               /* Don't check for null values in the image. */

    if ( fits_read_img (fptr, TFLOAT, fpixel, npix, &nullval,
                        imgbuf, &anynull, &status) )
         printerror ( status ) ;

    /*** Crop image. ***/

    xdimc = xdim - border * 2 ;
    ydimc = ydim - border * 2 ;
    imgc = (short *) malloc (xdimc * ydimc * sizeof (short)) ;

    for (iy = 0 ; iy < ydimc ; iy++)
	for (ix = 0; ix < xdimc; ix++)
	    {
	    *(imgc + ix + iy*xdimc) =
		(short) *(imgbuf + ix + border + (iy + border) * xdim) ;
	    }

    /*** Compute cropped image array center. ***/

    xc = (xdimc - 1) * 0.5 ;
    yc = (ydimc - 1) * 0.5 ;

    /*** Compute sun center & radius. ***/

    findshift (imgc, xdimc, ydimc, &xshift, &yshift, &diam) ;

    printf ("xshift/yshift: %7.2f  %7.2f   diam: %7.2f\n",
	     xshift, yshift, diam) ;
    xcen = xc - xshift + border ;
    ycen = yc - yshift + border ;
    xrad = diam * 0.5 ;
    yrad = diam * 0.5 ;

    /*** Translate image to align sun center with array center. ***/

    doshift (imgbuf, xdim, ydim, xshift, yshift) ;

    /*** Compute array center. ***/

    xc = (xdim - 1) * 0.5 ;
    yc = (ydim - 1) * 0.5 ;

    /*** Limit precision to 2 decimal places. ***/

    xcen = ((int)(xcen * 100.0) ) / 100.0 ;
    ycen = ((int)(ycen * 100.0) ) / 100.0 ;
    xrad = ((int)(xrad * 100.0) ) / 100.0 ;
    yrad = ((int)(yrad * 100.0) ) / 100.0 ;

    printf ("xcen/ycen: %7.2f %7.2f  xrad/yrad: %7.2f %7.2f\n",
	     xcen, ycen, xrad, yrad) ;

    /*** Update CRPIX1 & CRPIX2. ***/

    if ( fits_update_key (fptr, TFLOAT, "CRPIX1", &xc, comment, &status) )
	printerror (status) ;
    if ( fits_update_key (fptr, TFLOAT, "CRPIX2", &yc, comment, &status) )
	printerror (status) ;

    /*** Close FITS file. ***/

    if ( fits_close_file (fptr, &status) )  printerror ( status ) ;

    exit (0) ;
}

