/* fregs.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Shift image to co-align sun center with array center.
 *
 * Get sun center from FITS header.
 * Use "blint" to translate image.
 *
 * Update CRPIX1 & CRIPIX2 values in FITS header.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Uses cfitsio library for FITS operations.
 * Uses blint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    6 Apr 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/opt/share/PUBLIC/include/fitsio.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern float *rd_fits () ;
    extern unsigned char *translate_img () ;

    static char comment  [80] ;
    static char filename [80] ;

    int bitpix ;
    int status, nfound, anynull ;
    int border = 6 ;
    int ix, iy ;
    int nbypix ;

    long xdim, ydim ;
    long xdimc, ydimc ;
    long fpixel, npix ;

    float crpix1, crpix2 ;
    float *imgbuf ;
    float *imgc ;
    float minint ;
    float nullval ;
    float xrad, yrad ;
    float xcen, ycen ;
    float xc, yc ;
    float xsh, ysh ;
    float diam ;

    fitsfile *fptr ;	/* Pointer to the FITS file, defined in fitsio.h */

    if (argc <= 1)
	{
	fprintf (stderr, "usage: %s fits_file_name\n", argv [0]) ;
	exit (-1) ;
	}

    /*** Get FITS file name from command line. ***/

    strcpy (filename, argv [1]) ;

    printf ("filename: %s\n", filename) ;

    status = 0 ;

    /* Open fits file. */

    if ( fits_open_file (&fptr, filename, READWRITE, &status) )
         printerror ( status ) ;

    /*** Get BITPIX parameter. ***/

    if ( fits_read_key (fptr, TINT, "BITPIX", &bitpix, comment, &status) )
         printerror ( status ) ;
    
    /* Read the NAXIS1 and NAXIS2 keyword to get image size. */

    if ( fits_read_key (fptr, TINT, "NAXIS1", &xdim, comment, &status) )
         printerror ( status ) ;
    if ( fits_read_key (fptr, TINT, "NAXIS2", &ydim, comment, &status) )
         printerror ( status ) ;
    
    printf ("xdim: %4d  ydim: %4d\n", xdim, ydim) ;

    /*** Get sun center. ***/

    if ( fits_read_key (fptr, TFLOAT, "CRPIX1", &crpix1, comment, &status) )
         printerror ( status ) ;
    if ( fits_read_key (fptr, TFLOAT, "CRPIX2", &crpix2, comment, &status) )
         printerror ( status ) ;

    xcen = crpix1 ;
    ycen = crpix2 ;

    /*** Compute new array center. ***/

    xc = (xdim - 1) * 0.5 ;
    yc = (ydim - 1) * 0.5 ;

    /*** Compute shifts. ***/

    xsh = xc - xcen ;
    ysh = yc - ycen ;

    /*** If CRPIX1 & CRPIX2 are positive, disc center is already known. ***/

        printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;

    if (crpix1 < 0.0 || crpix2 < 0.0)
	{
        printf ("Invalid sun center: crpix1:%8.2f  crpix2:%8.2f\n",
		crpix1, crpix2) ;
	exit (0) ;
	}

    /* Read FITS image data. */

    /* Note that even though the FITS images contains unsigned integer */
    /* pixel values (or more accurately, signed integer pixels with    */
    /* a bias of 32768),  this routine is reading the values into a    */
    /* float array.   Cfitsio automatically performs the datatype      */
    /* conversion in cases like this.                                  */

    npix     = xdim * ydim ;
    imgbuf   = (float *) malloc (npix * sizeof (float)) ;
    fpixel   = 1 ;
    nullval  = 0 ;               /* Don't check for null values in the image. */

    if ( fits_read_img (fptr, TFLOAT, fpixel, npix, &nullval,
                        imgbuf, &anynull, &status) )
         printerror ( status ) ;

    nbypix = 4 ;		/* Float */

    /*** Translate image to align sun center with array center. ***/

    imgc = (float *) malloc (xdim * ydim * sizeof (short)) ;

    imgc = (float *) translate_img (imgbuf, xdim, ydim, nbypix, xsh, ysh) ;

    /*** Compute new array center. ***/

    xc = (xdim - 1) * 0.5 ;
    yc = (ydim - 1) * 0.5 ;

    /*** Limit precision to 2 decimal places. ***/

    xc = ((int)(xc * 100.0) ) / 100.0 ;
    yc = ((int)(yc * 100.0) ) / 100.0 ;

    printf ("xc/yc: %7.2f %7.2f\n", xc, yc) ;

    /*** Update CRPIX1 & CRPIX2. ***/

    if ( fits_update_key (fptr, TFLOAT, "CRPIX1", &xc, comment, &status) )
	printerror (status) ;
    if ( fits_update_key (fptr, TFLOAT, "CRPIX2", &yc, comment, &status) )
	printerror (status) ;

    /*** Write data into FITS image array. ***/

    if (fits_write_img (fptr, TFLOAT, fpixel, npix, imgc, &status) )
	printerror (status) ;

    /*** Close FITS file. ***/

    if ( fits_close_file (fptr, &status) )  printerror ( status ) ;

    exit (0) ;
}

