!
! NAME:  initgeom.f90
!
! PROJECT:  SunRISE/PSPT
!
! PURPOSE:  To find an initial estimate of the solar geometry of a full disk
!           solar image, given a value for the minimum disk intensity at the
!           solar limb.
!
! INPUTS:     im - the full solar disk image to be examined.
!         minint - the minimum disk intensity at the solar limb.
!           xdim - size of the input image in the x dimension.
!           ydim - size of the input image in the y dimension.
!
! OUTPUTS:  xc - x center coordinate of the solar disk.
!           yc - y center coordinate of the solar disk.
!            r - average radius (rx, ry) of the solar disk.
!
! LIBRARIES:  None.
!
! HISTORY:  Written January 14, 1999 by Randy Meisner, HAO/NCAR
!           (meisner@ucar.edu).
!
! COMPILE:  f90 -c -O initgeom.f90
!
!******************************************************************************

SUBROUTINE initgeom(im, minint, xdim, ydim, xc, yc, r)

  !USE im_data

  ! Input variables

  INTEGER, INTENT(in) :: xdim, ydim

  REAL, INTENT(in)    :: im(xdim, ydim), minint
  REAL, INTENT(inout) :: xc, yc, r

  ! Local variables

  INTEGER, PARAMETER :: nmax = 20000
  INTEGER :: i, j, n, dx, dy
  REAL    :: xdat(nmax), ydat(nmax), a(nmax, 5), b(nmax), krank, rnorm

  REAL, ALLOCATABLE :: tim(:,:)


  n = 1

  ! Generate bilevel image, based on minint

  ALLOCATE(tim(xdim, ydim))

  WHERE(im < minint) tim = 0.0
  WHERE(im >= minint) tim = 1.0


  ! Generate gradient image from bilevel image

  tim = (tim-CSHIFT(tim, 1, 1))**2 + (tim-CSHIFT(tim, 1, 2))**2


  ! Zero 1% of edges to avoid dead columns/rows

  dx = INT(0.01*xdim)
  dy = INT(0.01*ydim)

  tim(1:dx, 1:) = 0.0
  tim(1:, 1:dy) = 0.0

  tim(xdim-dx:xdim, 1:) = 0.0
  tim(1:, ydim-dy:ydim) = 0.0

  ! Loop over all pixels and save coordinates of where pixel value is
  ! greater than zero.  NOTE:  THESE ARE UNIT OFFSET ARRAYS, SUBTRACT ONE
  ! TO ATTAIN ZERO OFFSET ARRAY VALUES.

  DO i = 1, xdim
     DO j = 1, ydim

        IF(tim(i, j) > 0) THEN

           xdat(n) = REAL(i)
           ydat(n) = REAL(j)

           n = n+1

        END IF

     END DO
  END DO


  ! Deallocate tim array

  DEALLOCATE(tim)


  ! Fit (x, y) data points to an ellipse

  CALL fitqdr(n, nmax, a, b, xdat, ydat, krank, rnorm)


  ! Calculate center coordinates and radius

  AA = 1. + B(1)
  BB = 1. - B(1)
  CC = B(2)
  DD = B(3)
  EE = B(4)
  FF = B(5)

  DET = 4.*AA*BB - CC*CC
  IF(DET == 0.) STOP 'Not a quadratic form!'

  xc = (CC*EE - 2.*BB*DD) / DET
  yc = (CC*DD - 2.*AA*EE) / DET

  PHI = 0.5*ATAN( DBLE(CC)/(DBLE(AA)-DBLE(BB)+1E-32))
  AAA = 0.5*(AA+BB + (AA-BB)/COS(DBLE(2.)*DBLE(PHI)))
  BBB = AA+BB - AAA
  FFF = AA*xc*xc+BB*yc*yc+CC*xc*yc - FF

  IF(FFF/AAA < 0. .OR. FFF/BBB < 0.) STOP 'Not an ellipse!'

  RR1 = SQRT(FFF/AAA)
  RR2 = SQRT(FFF/BBB)

  r = (rr1+rr2)/2.0

END SUBROUTINE initgeom
