!
! NAME:  limb_int.f90
!
! PROJECT:  SunRISE/PSPT
!
! PURPOSE:  To find an approximate minimum value of the solar disk intensity at
!           the solar limb for defining an intensity threshold.
!
! INPUTS:   im - the full disk solar image to be examined.
!         xdim - the size of the image in the x dimension.
!         ydim - the size of the image in the y dimension.
!
! OUTPUTS:  minint - the intensity value of the solar disk limb.
!
! LIBRARIES:  None.
!
! COMPILE:  f90 -c -O limb_int.f90
!
!******************************************************************************

SUBROUTINE limb_int(im, xdim, ydim, minint)

  !USE im_data

  ! Input variables

  INTEGER, INTENT(in) :: xdim, ydim

  REAL, INTENT(in)    :: im(xdim, ydim)

  REAL, INTENT(inout) :: minint

  ! Local variables

  INTEGER :: nx2, ny2, i, dx, dy, ny4, ds
  REAL    :: mt(3), rtn, mxv

  REAL, ALLOCATABLE :: row(:), sim(:,:)


  ds = 5

  ! Generate a Sobel filtered image of the input image to detect the edges.

  ALLOCATE(sim(xdim, ydim))

  sim = im

  CALL sobel(sim, xdim, ydim)

  ! Normalize sobel image to its maximum value

  mxv = MAXVAL(sim)

  sim = sim/mxv

  ! Set normalized values below 0.03 equal to zero

  WHERE(sim < 0.03) sim = 0.0


  ! Zero edges of bilevel sobel image

  dx = INT(0.01*xdim)
  dy = INT(0.01*ydim)

  sim(1:dx+1, 1:) = 0.0
  sim(1:, 1:dy+1) = 0.0

  sim(xdim-dx-1:xdim, 1:) = 0.0
  sim(1:, ydim-dy-1:ydim) = 0.0

  ! Examine three rows of the Sobel image, looking for the first maximum of
  ! each row; this maximum should correspond to the limb; select an intensity
  ! value 2 pixels in toward disk center.

  ! Find first maximum from three rows of the normalized Sobel image; select
  ! an intensity value ds pixels toward sun center to use as limb value.

  nx2 = INT(xdim/2)
  ny2 = INT(ydim/2)
  ny4 = INT(ydim/8)

  ALLOCATE(row(nx2))

  row = sim(1:nx2, ny2)

  DO i = 2*dx, nx2
     IF(row(i+1) < row(i)) THEN
        mt(1) = im(i+ds, ny2)
        EXIT
     END IF
  END DO


  row = sim(1:nx2, ny2+ny4)

  DO i = 2*dx, nx2
     IF(row(i+1) < row(i)) THEN
        mt(2) = im(i+ds, ny2+ny4)
        EXIT
     END IF
  END DO

  row = sim(1:nx2, ny2-ny4)

  DO i = 2*dx, nx2
     IF(row(i+1) < row(i)) THEN
        mt(3) = im(i+ds, ny2-ny4)
        EXIT
     END IF
  END DO


  DEALLOCATE(sim, row)


  ! Select the median value of the 3 samples to return.

!  IF(mt(1) > mt(2) .AND. mt(1) < mt(3)) rtn = mt(1)
!  IF(mt(1) < mt(2) .AND. mt(1) > mt(3)) rtn = mt(1)
!  IF(mt(2) > mt(1) .AND. mt(2) < mt(3)) rtn = mt(2)
!  IF(mt(2) < mt(1) .AND. mt(2) > mt(3)) rtn = mt(2)
!  IF(mt(3) > mt(2) .AND. mt(3) < mt(1)) rtn = mt(1)
!  IF(mt(3) < mt(2) .AND. mt(3) > mt(1)) rtn = mt(1)
!  IF(mt(1) == 0 .AND. mt(2) == 0 .AND. mt(3) /= 0) rtn = mt(3)
!  IF(mt(1) == 0 .AND. mt(2) /= 0 .AND. mt(3) == 0) rtn = mt(2)
!  IF(mt(1) /= 0 .AND. mt(2) == 0 .AND. mt(3) == 0) rtn = mt(1)

  IF(mt(1) == 0 .AND. mt(2) == 0 .AND. mt(3) == 0) THEN

     PRINT*, 'ERROR:  limb_int can not determine minimum limb intensity.'
     PRINT*, '        setting to default value of 400.0'

     rtn = 400.0

  ELSE

     ! Special case handling for improperly summed images.
     
     rtn = SUM(mt)/3.0
     
     IF(rtn <= 10) rtn = 30.0

     IF(rtn >= 1000) rtn = 400.0
     
  END IF

  minint = rtn

  PRINT*, 'limb_int: ', mt(1), mt(2), mt(3), '     avg: ', minint

END SUBROUTINE limb_int
