/* mk4time.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Computes the time of a feature at a specified position angle,
 * based on the direction of the scan, and the scan duration.
 *
 * The direction and duration values are obtained from the FITS header(s).
 * The user need only supply the angle(s) of interest, 
 * and the image file name (FITS).
 *
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction, scan time are obtained from the extension HDU.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Author: Alice Lecinski   HAO/NCAR   findtime.c [Mk3]
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Update: Andrew L. Stanger   HAO/NCAR   mk4time.c   21 December 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <ctype.h>
#include "math.h"
#include "fitsio.h"

#define FOREVER 1
#define DONXT   -999
#define QUIT    0

char *file_name ;
int  Anglesflg, angleflg, cflag, rflag, srflag ;
int  quickflag, Neflag ;
unsigned Itime, Truetime ;

int idflag , udflag , qdflag , pBflag , posflag , negflag ;
int mon,day,year ;

/* -------------------------------------------------------------------------- */

main (an, av)

int an ;
char *av [] ;
{

    float get_frac_offset () ;
    void prsum () ;
    void printerror () ;

    char *w ;		/* pointer to command line arguments */
    static char comment [80] ;
    static char date_obs [24] ;		/* Observation date. */
    static char time_obs [24] ;		/* Observation time. */

    int bar_dir ;			/* Barrel scan direction. */
    int doit ;
    int phour, pminute, psecond ;	/* Position time. */
    int bhour, bminute, bsecond ;	/* Begin scan time. */
    int dhour, dminute, dsecond ;	/* Delta (offset) time. */
    int byear, bmonth,  bday    ;	/* Begin date. */
    int pyear, pmonth,  pday    ;	/* Position date. */
    int status ;

    float ang_range = 360.0 ;		/* Range of angles in scan. */
    float delta_time ;			/* Delta time (offset from start). */
    float duration ;			/* Scan duration (seconds). */
    float frac_time ;			/* Fraction of time for position. */
    float Phase ;			/* Phase angle (offset from Geo north.*/
    float solar_p0 ;			/* P-angle. */

    double PA ;
    double thetamin , thetamax , ThetaRng , fabs() ;

    FILE *infid ;
    fitsfile *fptr ;

    if ( an <= 2 ) { prsum () ; exit (-4) ; }
    while (--an)
	{
	if (*(w = *++av) == '-')	/* Check for flag arguments */
	    switch (w [1])
	    {
	    case 'A':  Anglesflg++ ; break ;
	    case 'a':  angleflg++ ;
	               --an ; w = *++av ;
		       sscanf (w, "%lf", &PA) ; break ;
	    default:   fprintf (stderr," unknown flag %s\n",*av) ;
		       prsum () ; exit (-1) ;
	    }

	else		/* Get the image filename. */
	    {

	    if (*w == '?') { prsum () ; exit (-2) ; }
	    if ( angleflg + Anglesflg == 0 )
		{
		fprintf (stderr,"One of -a or -A  MUST be given\n") ;
	        prsum () ; exit (-3) ;
		}

	    file_name = w ;
	    status = 0;

	    /*** Open FITS file. ***/

	    if ( fits_open_file (&fptr, file_name, READONLY, &status) )
		printerror ( status );

	    /*** Primary FITS header. ***/

	    if (fits_read_key (fptr, TSTRING, "DATE-OBS", date_obs,
		comment, &status))  printerror (status) ;
	    sscanf (date_obs, "%04d-%02d-%02d", &byear, &bmonth, &bday) ;
	    pyear  = byear ;
	    pmonth = bmonth ;
	    pday   = bday ;

	    if (fits_read_key (fptr, TSTRING, "TIME-OBS", time_obs,
		comment, &status))  printerror (status) ;
	    sscanf (time_obs, "%2d:%2d:%2d", &bhour, &bminute, &bsecond) ;

	    if (fits_read_key (fptr, TFLOAT, "SOLAR_P0", &solar_p0,
		comment, &status))  printerror (status) ;

	    /*** Move to extension header. ***/

	    if (fits_movabs_hdu (fptr, 2, NULL, &status))  printerror (status) ;

	    if (fits_read_key (fptr, TINT, "BAR_DIR", &bar_dir,
		comment, &status))  printerror (status) ;

	    if (fits_read_key (fptr, TFLOAT, "DURATION", &duration,
		comment, &status))  printerror (status) ;

	    if (fits_read_key (fptr, TFLOAT, "PHASE", &Phase,
		comment, &status))  printerror (status) ;

	    duration = duration / 60.0 ;	/* Convert from Hertz to sec. */

	    doit = FOREVER ;

	    if ( Anglesflg )
		fprintf (stdout, 
		"Enter %d as a position angle to get to the next image.\n", 
		DONXT ) ;

	    /*** Multiple position angle loop. ***/

	    while ( doit )
		{
		if ( Anglesflg )
		    {
		    fprintf(stdout, 
		       "\nEnter in the position angle of your feature --> ") ;
		    fscanf(stdin, "%lf", &PA) ;
		    if ( PA == DONXT ) break ;
		    }
		else doit=QUIT ;

		/*** Compute fractional time offset from beginning of scan. ***/

		frac_time = get_frac_offset ( bar_dir, ang_range,
					      solar_p0, Phase, PA ) ;

		delta_time = duration * frac_time ;
		dhour   = (int)  (delta_time / 3600.0 );
		dminute = (int) ((delta_time - dhour   * 3600) / 60.0)  ;
		dsecond = (int)  (delta_time - dminute *   60) ;

		psecond = bsecond + dsecond ;
		pminute = bminute + dminute ;
		phour   = bhour   + dhour ;

		/*** Adjust time fields for overflow. ***/

		if (psecond > 59)
		    {
		    pminute += 1 ;
		    psecond -= 60 ;
		    }
		if (pminute > 59)
		    {
		    phour += 1 ;
		    pminute -= 60 ;
		    }
		if (phour > 23)
		    {
		    pday += 1 ;
		    phour -= 24 ;
		    }

                fprintf (stdout, "%s  Position Angle: %7.2f  Date: %04d-%02d-%02d  Time: %02d:%02d:%02d\n",
		    file_name, PA, pyear, pmonth, pday, phour, pminute, psecond) ;
	        }

	    /*** Close FITS file. ***/

	    if ( fits_close_file (fptr, &status) )  printerror (status) ;
	    }
        }
   exit (0) ;
}

/* -------------------------------------------------------------------------- */

/* get_frac_offset ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute time offset from beginning of scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

float
get_frac_offset (dir, range, pangle, phase, position)

int dir ;
float range ; 
float pangle ;
float phase ;
float position ;
{

    float frac_offset ;
    float ref ;

    if ( dir > 0 )
	{
	ref = position - 180 + pangle + phase ;
        }
    if ( dir < 0 )
	{
	ref = 360 - ( position - 180 + pangle + phase  ) ;
        }

    if ( ref <   0 ) ref += 360 ;
    if ( ref > 360 ) ref -= 360 ;

    frac_offset = (ref / range) ;

/*
 *    printf ("ref: %7.2f  range: %7.2f   frac_offset: %8.3f\n",
 *	     ref, range, frac_offset) ;
 */

    return (frac_offset) ;
}

/* -------------------------------------------------------------------------- */

/* prsum ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void
prsum ()
{
    static char help [] = "\
findtime [-a position_angle] [-A]  imagename1 imagename2 imagename3 \n\
\n\
Example:              findtime -a 270 19_20.srq \n\
Example:              findtime -a 270 19_07.srq 19_20.srq \n\
Example:              findtime -A     19_07.srq 19_20.srq \n\
\n\
This program attempts to find the exact time for a feature at the given \n\
position angle.  \n\
See the file on hao --> /home/hao/cordyn/mk3C/Instructions/scan_directions\n\
to see how the computation is done.\n\
\n\
Flags	    Description\n\
-----	    -----------\n\
\n\
-[a angle]  the position angle of interest.\n\
-[A]        Get the program to prompt the user for many position angles.\n\
" ;

       fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

