!
! NAME:
!
! PROJECT:  SunRISE/PSPT
!
! PURPOSE:  To determine the solar geometry of a full disk solar image, using
!           the zero point crossings of the Laplacian of the image and least-
!           squares fitting of the points to determine the location of the limb
!           and disk center.
!
! INPUTS:    im - full disk solar image to be examined.
!        minint - minimum intensity disk value at/near the solar limb.
!          xdim - size of the image in the x dimension.
!          ydim - size of the image in the y dimension.
!
! OUTPUTS:  rx - x radius of the solar disk, in pixels.
!           ry - y radius of the solar disk, in pixels.
!           xc - x center coordinate of the solar disk.
!           yc - y center coordinate of the solar disk.
!
! LIBRARIES:  None
!
! HISTORY:  Written January 14, 1999 by Randy Meisner, HAO/NCAR
!           (meisner@ucar.edu).
!
! COMPILE:  f90 -c -O solargeom.f90
!
!******************************************************************************

SUBROUTINE solargeom(im, minint, xdim, ydim, rx, ry, xc, yc)

  !USE im_data

  ! Input variables

  INTEGER, INTENT(in) :: xdim, ydim

  REAL, INTENT(in)    :: im(xdim, ydim), minint
  REAL, INTENT(inout) :: rx, ry, xc, yc

  ! Local variables

  INTEGER, PARAMETER :: nmax = 15000
  REAL, ALLOCATABLE  :: lim(:,:)          ! Laplacian image
  REAL               :: kern(3,3)         ! Laplacian kernal
  REAL               :: xxc, yyc, r, zc(2), krank, rnorm, threshold
  REAL, ALLOCATABLE  :: xvec(:), yvec(:)
  INTEGER            :: i, j, dx, dy, count
  REAL               :: xdat(nmax), ydat(nmax), a(nmax,5), b(nmax)


  xdat = 0.0
  ydat = 0.0
  count = 1

  ! Get initial image geometry estimate

  CALL initgeom(im, minint, xdim, ydim, xxc, yyc, r)

  ! PRINT*, 'Init. XC, YC, R:  ', xxc, yyc, r
  ! PRINT*, ''

  ! Initialize the Laplacian kernal

  kern = -1.0
  kern(2, 2) = 8.0

  ! Create Laplacian image.

  ALLOCATE(lim(xdim, ydim))

  lim = 0.0

  DO i = 2, xdim-1
     DO j = 2, ydim-1

        lim(i,j) = (im(i-1, j-1)*kern(1,1) + &
             im(i-1, j)  *kern(1,2) + &
             im(i-1, j+1)*kern(1,3) + &
             im(i, j-1)  *kern(2,1) + &
             im(i, j)    *kern(2,2) + &
             im(i, j+1)  *kern(2,3) + &
             im(i+1, j-1)*kern(3,1) + &
             im(i+1, j)  *kern(3,2) + &
             im(i+1, j+1)*kern(3,3))

     END DO
  END DO


  ! Zero 1% of edges to avoid dead columns/rows & edge effects

  dx = INT(0.01*xdim)
  dy = INT(0.01*ydim)

  lim(1:dx, 1:) = 0.0
  lim(1:, 1:dy) = 0.0

  lim(xdim-dx:xdim, 1:) = 0.0
  lim(1:, ydim-dy:ydim) = 0.0


  ! Loop over each column and search for limb points

  ALLOCATE(yvec(ydim))

  threshold = 20

  DO i = 1, xdim

     ! extract a column

     yvec = lim(i, 1:)

     IF(MAXVAL(yvec) /= MINVAL(yvec)) THEN

        ! smooth the column to remove false zero crossings

        CALL smooth(yvec, ydim)


        ! check column for limb points

        CALL limbchk(yvec, ydim, i, xxc, yyc, r, threshold, -1, zc)


        ! add limb point coordinates to storage arrays

        IF(zc(1) > 0) THEN

           xdat(count) = REAL(i)
           ydat(count) = zc(1)
           count = count+1

        END IF

        IF(zc(2) > 0) THEN

           xdat(count) = REAL(i)
           ydat(count) = zc(2)
           count = count+1

        END IF

     END IF

  END DO

  DEALLOCATE(yvec)


  ! Loop over each row and search for limb points

  ALLOCATE(xvec(xdim))

  DO i = 1, ydim

     ! extract a row

     xvec = lim(1:, i)

     IF(MAXVAL(xvec) /= MINVAL(xvec)) THEN

        ! smooth the row to remove false zero crossings

        CALL smooth(xvec, xdim)

        ! check row for limb points

        CALL limbchk(xvec, xdim, i, xxc, yyc, r, threshold, 1, zc)

        ! add limb point coordinates to storage arrays

        IF(zc(1) > 0) THEN

           xdat(count) = zc(1)
           ydat(count) = REAL(i)
           count = count+1

        END IF

        IF(zc(2) > 0) THEN

           xdat(count) = zc(2)
           ydat(count) = REAL(i)
           count = count+1

        END IF

     END IF

  END DO

  DEALLOCATE(xvec)


  ! Free Laplacian image from memory

  DEALLOCATE(lim)

  count = count-1

  ! PRINT*, 'Laplacian Geometry:  ', count, ' points to fit.'


  ! Fit (x, y) data points to an ellipse

  CALL fitqdr(count, nmax, a, b, xdat, ydat, krank, rnorm)


  ! Calculate center coordinates and radius

  AA = 1. + B(1)
  BB = 1. - B(1)
  CC = B(2)
  DD = B(3)
  EE = B(4)
  FF = B(5)

  DET = 4.*AA*BB - CC*CC
  IF(DET == 0.) STOP 'Not a quadratic form!'

  xc = (CC*EE - 2.*BB*DD) / DET
  yc = (CC*DD - 2.*AA*EE) / DET

  PHI = 0.5*ATAN( DBLE(CC)/(DBLE(AA)-DBLE(BB)+1E-32))
  AAA = 0.5*(AA+BB + (AA-BB)/COS(DBLE(2.)*DBLE(PHI)))
  BBB = AA+BB - AAA
  FFF = AA*xc*xc+BB*yc*yc+CC*xc*yc - FF

  IF(FFF/AAA < 0. .OR. FFF/BBB < 0.) STOP 'Not an ellipse!'

  rx = SQRT(FFF/AAA)
  ry = SQRT(FFF/BBB)

  r = (rx+ry)/2.0

  ! Print the results

  ! PRINT 100, 'X0:', xc, ' Y0:', yc
  ! PRINT 100, 'R1:', rx, ' R2:', ry
  ! PRINT 100, ' R:',  r, 'Phi:', PHI

100 FORMAT(a, 2x, f9.3, 3x, a, 2x, f9.3)

  !PRINT*, ''

END SUBROUTINE solargeom

!##############################################################################

SUBROUTINE smooth(vec, dim)

  ! Input variables

  INTEGER, INTENT(in) :: dim
  REAL, INTENT(inout) :: vec(dim)

  ! Local variables

  INTEGER :: i
  REAL    :: t1, t2, t3


  DO i = 2, dim-1

     IF(i == 2) THEN

        t1 = vec(i-1)
        t2 = vec(i)
        t3 = vec(i+1)

     ELSE

        t1 = t2
        t2 = t3
        t3 = vec(i+1)

     END IF

     vec(i) = (t1+t2+t3)/3.0

  END DO

END SUBROUTINE smooth

!##############################################################################

SUBROUTINE limbchk(vec, dim, idx, xc, yc, r, thresh, dir, zcross)

  ! Input variables

  INTEGER, INTENT(in) :: dim, idx, dir
  REAL, INTENT(in)    :: vec(dim), xc, yc, r, thresh
  REAL, INTENT(inout) :: zcross(2)

  ! Local variables

  INTEGER :: i
  REAL    :: rc, amplitude, cc, zc, local_amp, zinterp

  ! Initialize zcross

  zcross = -1.0

  amplitude = 0.0


  ! Loop over all elements of vector and check for zero crossings

  DO i = 1, dim-1

     IF(vec(i)*vec(i+1) < 0) THEN

        cc = REAL(i)

        ! row processing

        IF(dir == 1) rc = SQRT( (cc-xc)*(cc-xc) + (idx-yc)*(idx-yc) )

        ! column processing

        IF(dir == -1) rc = SQRT( (idx-xc)*(idx-xc) + (cc-yc)*(cc-yc) )


        IF(ABS(r-rc) < 5) THEN

           amplitude = local_amp(vec, dim, i)

           IF(amplitude >= thresh) THEN

              zc = zinterp(vec, dim, i)

              IF(zcross(1) == -1) THEN
                 zcross(1) = zc
              ELSE
                 zcross(2) = zc
              END IF 

         END IF

      END IF

   END IF

END DO

END SUBROUTINE limbchk

!##############################################################################

REAL FUNCTION local_amp(vec, dim, idx)

  ! Input variables

  INTEGER, INTENT(in) :: dim, idx
  REAL, INTENT(in)    :: vec(dim)

  ! Local variables

  REAL    :: amp, left, right
  INTEGER :: i, j, rf, lf, itr

  REAL, ALLOCATABLE :: x(:)

  ALLOCATE(x(dim))

  x = ABS(vec)

  rf = 0
  lf = 0

  itr = 0

  ! Search for maxima on both sides of x

  i = idx
  j = idx

  DO WHILE( (rf == 0).OR.(lf == 0) )

     ! check for edges

     IF(i == 0) lf = 1
     IF(j == dim) rf = 1

     IF((i > 0).AND.(lf == 0)) THEN

        IF(x(i-1) < x(i)) THEN

           lf = 1
           left = vec(i)

        END IF

        i = i-1

     END IF


     IF((j < dim).AND.(rf == 0)) THEN

        IF(x(j+1) < x(j)) THEN

           rf = 1
           right = vec(j)

        END IF

        j = j+1

     END IF

     itr = itr+1

  END DO

  DEALLOCATE(x)

  amp = ABS(right-left)

  local_amp = amp

END FUNCTION local_amp

!##############################################################################

REAL FUNCTION zinterp(vec, dim, idx)

  ! Input variables

  INTEGER, INTENT(in) :: dim, idx
  REAL, INTENT(in)    :: vec(dim)

  ! Local variables

  INTEGER :: j, k, ok, iter, maxitr
  REAL, DIMENSION(20) :: ax, ay, ay2
  REAL :: tol, dx, xx, x1, func, s1, s2, zval


  k = 0
  ok = 0
  iter = 0
  maxitr = 1000
  tol = 1.0E-4
  dx = 0.5
  zval = -1.0

  ! Transfer surrounding 20 elements to temp array

  ay = vec(idx-9:idx+10)

  j = 1
  DO i = idx-9, idx+10
     ax(j) = REAL(i)
     j = j+1
  END DO

  ! Determine end-point slopes

  s1 = ay(2)-ay(1)
  s2 = ay(20)-ay(19)

  ! Set up spline calculation

  CALL spline(ax, ay, 20, s1, s2, ay2)

  xx = REAL(idx)
  x1 = xx

  ! Iterate to find zero crossing

  DO WHILE(ok == 0)

     ! check if max number of iterations has been reached

     IF(iter >= maxitr) THEN

        PRINT*, 'ZINTERP:  max. iter.'
        zval = -1.0
        EXIT

     END IF

     ! spline fit data at point xx

     CALL splint(ax, ay, ay2, 20, xx, func)

     ! check value against tolerance

     IF((ABS(func) < tol).OR.(dx < tol)) THEN

        ok = 1
        zval = xx
        EXIT

     ELSE

        ! add step to guess

        IF(func*vec(idx+1) < 0) THEN

           xx = xx+dx

        ELSE

           xx = xx-dx
           dx = dx/2.0
           xx = xx+dx

        END IF

     END IF

     iter = iter+1

  END DO

  zinterp = zval

END FUNCTION zinterp
