/* chip_ldark12.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Perform radial scans on a CHIP image.
 * Determine limb darkening correction by averaging radial scans.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   16 May 2000
 * 20 Sep 2000: Average equatorial radial scans to make limb darkening function.
 * 21 Sep 2000: Also output scandisk [angle][radius] array.
 *  4 Oct 2000: Compute and output an image containing radial distances.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#define ACOS

#include "/home/cordyn/fits/mechanism_defs.h"
#include "/home/cordyn/fits/acos.h"
#include "/home/cordyn/fits/chip.h"

#include "/home/cordyn/fits/device.h"
#include "/home/cordyn/fits/analog.h"
#include "/home/cordyn/fits/lcvr.h"
#include "/home/cordyn/fits/mic.h"

#include "/home/stanger/lib/src/acos/acoslib.h"
#include "chipscan.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern int acoshskp () ;		/* Function to read ACOS header. */
   extern unsigned short *rdacos () ;	/* Function to read ACOS image. */
   extern float lint () ;		/* Linear interpolation. */

   static char namimg [80] ;		/* CHIP input file name. */
   static char namldc [80] ;		/* Limb darken correction file name. */
   static char namlds [80] ;		/* scandisk output file name. */
   static char namout [80] ;		/* Corrected image file name. */
   static char namdc  [80] ;		/* Doppler correction image file name.*/
   static char namrad [80] ;		/* Radial distance file. */

   static float scanmax [MAXSCN] ;
   static float scanavg [MAXSCN] ;
   static float scan    [MAXSCN] ;
   static float scandx  [MAXSCN] ;
   float *dopcor ;
   float *dimg ;
   float *ldbuf, *ldsqr ;
   float *radbuf ;
   float *scandisk ;
   unsigned short *acosbuf ;             /* MLSO ACOS file  array pointer. */
   unsigned short *sbuf ;
   unsigned char *padbuf ;
   unsigned char *cbuf ;

   unsigned short sval ;

   float angle ;
   float cdelt1 ;
   float cord ;
   float degnorth = 0.0 ;
   float dopmax = 0.0 ;
   float eqdist ;
   float ffix ;
   float ldcor ;
   float lambda ;
   float limbdark ;
   float ldmin, ldmax ;
   float dcmin, dcmax ;
   float pangle ;
   float pixrs ;
   float rc ;
   float radius ;
   float radmin, radmax ;
   float rmin, rmax ;
   float rinc ;
   float savg ;
   float theta ;
   float tmin =  90.0 ;
   float tmax = 270.0 ;
   float tinc = 180.0 ;
   float xdopcor, ydopcor ;
   float xc, yc ;
   float xh ;
   float xcen, ycen ;
   float xcensq, ycensq ;
   float xx, yy ;

   int ipixrs ;
   int ir ;
   int it ;
   int ix, iy ;
   int ixsq, iysq ;
   int ixbeg, ixend ;
   int ixcen, iycen ;
   int iybeg, iyend ;
   int navg ;
   int nbypix ;
   int nbyimg ;
   int nang = 362 ;
   int nave = 11 ;
   int npad ;
   int npximg ;
   int nrad ;
   int nrs ;
   int ns ;
   int pmode  =   1 ;
   int scntyp =   1 ;
   int itmin  =   0 ;
   int itmax  = 361 ;
   int itinc  =   1 ;
   int wcode ;
   int xdim, ydim ;
   int xdimsq, ydimsq ;
   int xdif, ydif ;

   unsigned short sfix ;

   FILE *FD, *FDOUT, *FDCOR, *FDS, *FDC, *FDR ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s chip_filename\n", argv [0]) ;
      exit (-1) ;
      }

   fprintf (stdout, "namimg: %s\n", namimg) ;

   /* Read header. */

   acoshskp (namimg, 0) ;

   /* Get items from header. */

   xdim   = ACOSI->xdim ;
   ydim   = ACOSI->ydim ;
   nbypix = ACOSI->nbypx ;
   xcen   = (float) (ACOSH->Xdim - 1) * 0.5 ;
   ycen   = (float) (ACOSH->Ydim - 1) * 0.5 ;
   ixcen  = (int) xcen ;
   iycen  = (int) ycen ;
   cdelt1 = 2.29 ;				/* arcsec / pixel */
   pixrs  = ACOSI->rsun / cdelt1 ;		/* pixels / solar radius. */
   ipixrs = (int) pixrs ;
   pangle = ACOSI->pangle ;			/* rotation angle. */
   wcode  = ACOSH->Type / 100 ;			/* wavelength code. */

   printf ("xdim: %d  ydim: %d  nbypix: %d\n", xdim, ydim, nbypix) ;
   printf ("xcen: %7.2f ycen: %7.2f\n", xcen, ycen) ;
   printf ("pixrs: %7.2f  pangle: %7.2f\n", pixrs, pangle) ;

   xdimsq = 936 ;
   ydimsq = 936 ;
   xcensq = (xdimsq - 1) / 2.0 ;
   ycensq = (ydimsq - 1) / 2.0 ;
   printf ("xdimsq/ydimsq: %4d %4d\n", xdimsq, ydimsq) ;
   printf ("xcensq/ycensq: %7.2f  %7.2f\n", xcensq, ycensq) ;

   /*** Allocate storage for image array. ***/

   npximg  = xdim * ydim ;			/* # pixels in image. */
   nbyimg  = npximg * nbypix ;
   acosbuf = (unsigned short *) malloc (xdim * ydim * nbypix) ;
   sbuf    = (unsigned short *) malloc (xdim * ydim * nbypix) ;
   cbuf    = (unsigned char *)  sbuf ;
   ldbuf   = (float *) malloc (xdim   * ydim   * sizeof (float)) ;
   dopcor  = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;
   radbuf  = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;
   
   /*** Read image. ***/

/* acosbuf = rdacos (namimg, pmode) ; */

   FD = fopen (namimg, "r") ;   
   if (FD == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      exit (-2) ;
      }

   fread (acosbuf, nbypix, npximg, FD) ;

   fclose (FD) ;

   /*** Set first 3 lines of image to zero. ***/

   for (iy = 0; iy < 3; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(acosbuf + ix + iy * xdim) = 0 ;
	 }

   /*** Invert CHIP image top-to-bottom. ***/

   for (iy = 0; iy < ydim; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(sbuf + ix + iy * xdim) = *(acosbuf + ix + (ydim - iy - 1) * xdim) ;
	 }

   /*** Find average intensity near disk center. ***/

/*
 *   ixcen = xdim - 1 - (int) xcen ;
 *   iycen = ydim - 1 - (int) ycen ;
 */

   ixbeg = ixcen - 5 ;
   ixend = ixbeg + 10 ;
   iybeg = iycen - 5 ;
   iyend = iybeg + 10 ;

   savg = 0.0 ;
   navg = 0 ;
   for (iy = iybeg; iy <= iyend; iy++)
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 savg += (float) *(sbuf + ix + iy * xdim) ;
	 navg++ ;
	 }
   savg = (savg / navg) ;
   printf ("savg: %7.2f\n", savg) ;

   printf ("ixcen/iycen: %d %d  sbuf [center]: %d\n",
	   ixcen, iycen, *(sbuf + ixcen + iycen * xdim)) ;

   /*** Compute "doppler" correction ***/
   /*** Estimate max correction at limb to be 1.5% of central average. ***/

   fprintf (stdout, "wcode: %d   CWAVELENGTH_d: %d\n", wcode, CWAVELENGTH_d) ;

   if (wcode == CWAVELENGTH_d)
      {
      for (iy = 0; iy < ydimsq; iy++)
         {
         yy = (float) iy ;
         yc = (yy - ycensq) / pixrs;	/* Y distance from center [Rsun units]*/
	 if (fabs (yc) > 1.0)
	    {
	    lambda  = 0.0 ;
	    eqdist  = 0.0 ;
	    ydopcor = 0.0 ;
	    }
         else
	    {
            lambda  = asinf (yc) ;
	    cord    = cosf (lambda) * pixrs ;
            eqdist  = cosf (lambda) ;
            ydopcor = eqdist ; 
	    }

         dopmax = 0.015 * savg * ydopcor ;	/* 1.5% of central brightness */

/*
	 if (iy / 10 * 10 == iy)
	 fprintf (stdout, "iy: %4d  yc: %6.4f  lambda: %9.4f  eqdist: %9.4f, ydopcor: %9.4f\n",
		  iy, yc, lambda, eqdist, ydopcor) ;
 */

         for (ix = 0; ix < xdimsq; ix++)
	    {
	    xx = (float) ix ;
	    xc = (xx - xcensq) / pixrs ;
            xh = (xx - xcensq) / cord ; /* X distance from center [Rsun units]*/

            rc = sqrt (xc * xc + yc * yc) ;
            if (fabs (rc) > 1.0)
	       xdopcor = 0.0 ;
	    else
	       xdopcor = xh ;

	    *(dopcor + ix + iy*xdimsq)  = xdopcor * dopmax ;

	    if (ix / 10 * 10 == ix && (iy == 468 || iy == 675))
	    printf("ix: %4d  iy: %4d  xh: %6.4f  cord: %7.2f  xdopcor: %9.4f  dopcor: %7.4f\n",
		     ix, iy, xh, cord, xdopcor, *(dopcor + ix + iy*xdimsq)) ;
            }
         }
      }
   else
      for (iy = 0; iy < ydimsq; iy++)
         for (ix = 0; ix < xdimsq; ix++)
	    *(dopcor + ix + iy*xdimsq) = 1.0 ;

   dcmin =  1.0e+10 ;
   dcmax = -1.0e+10 ;
   for (iy = 0; iy < ydimsq; iy++)
      for (ix = 0; ix < xdimsq; ix++)
	 {
         if (*(dopcor + ix + iy*xdimsq) < dcmin) dcmin = *(dopcor + ix + iy*xdimsq);
         if (*(dopcor + ix + iy*xdimsq) > dcmax) dcmax = *(dopcor + ix + iy*xdimsq);
	 }

   fprintf (stdout, "dcmin, dcmax: %7.2f  %7.2f\n", dcmin, dcmax) ;

   sprintf (namdc, "%s.dc", namimg) ;
   fprintf (stdout, "namdc: %s\n", namdc) ;
   FDC = fopen (namdc, "w") ;
   fwrite (dopcor, xdimsq * ydimsq, sizeof (float), FDC) ;
   fclose (FDC) ;

   /*** Create radial distance file (helium image only). ***/

   if (wcode == CWAVELENGTH_h)
      {
      for (iy = 0; iy < ydimsq; iy++)
         {
         yy = (float) iy ;
         for (ix = 0; ix < xdimsq; ix++)
	    {
            xx = (float) ix ;
            crcoord (&radius, &angle, &xx, &yy, -1,
		     pangle, xcensq, ycensq, pixrs) ;
            *(radbuf + ix + iy * xdimsq) = radius ;
	    if (ix == iy)  printf ("ix/iy: %4d %4d  radius: %7.2f  angle: %7.2f\n", ix, iy, radius, angle) ;
	    }
         }

      sprintf (namrad, "%s.rad", namimg) ;
      FDR = fopen (namrad, "w") ;
      fwrite (radbuf, xdimsq * ydimsq, sizeof (float), FDR) ;
      fclose (FDR) ;
      }

   /*** For scandisk array, angle dimension needs to go 1.0 degree BEYOND
    *** the range of 0 -> 360 degrees (i.e., -1.0 & 361.0) for bilinear
    *** interpolation at the boundaries (0 degrees and 360 degrees).
    ***/

   nang = itmax - itmin + 1 ;

   rinc = 1.0 / pixrs ;			/* Radial increment: ~ 1 pixel. */
   rmin = 0.0 ;
   rmax = (ipixrs - 1) / pixrs ;
/*
 *   rmax = 1.00001 ;
 *   rmax = 1.0 + rinc ;
 *   rmax = (ipixrs - 1) / pixrs ;
 */
   nrad = (int) (((rmax - rmin) / rinc) + 0.0001) + 1 ;
   printf ("computed nrad: %d   computed nang: %d\n", nrad, nang) ;

   scandisk = (float *) malloc (nrad * nang * sizeof (float)) ;

   /*** Clear scan buffers. ***/

   for (ir = 0; ir < MAXSCN; ir++) scanavg [ir] = 0.0 ;

   for (it = itmin; it <= itmax; it++)
      for (ir = 0; ir < nrad; ir++)
         *(scandisk + ir + it * nrad) = 0.0 ;

   /*** Perform radial scans. ***/
   /*** Use P-angle so that scans are done in heliocentric coordinates. ***/

   for (it = itmin; it < itmax; it += 1.0)
      {
      theta = (float) it - 1.0 ;	/* Start at -1.0 degree. */

      crscan2 (cbuf, pangle, nbypix, xdim, xcen, ycen, pixrs,
	      rmin, rmax, rinc, theta, &scntyp, scan, scandx, &nrs, nave) ;

      if (nrs > nrad)
	 {
	 fprintf (stderr, "chip_ldark2> Error: nrs [%d] > nrad [%d]\n",
		  nrs, nrad) ;
	 exit (-1) ;
	 }

      /*** Save each radial scan in a 2-D array: (r,th). ***/
      /*** Accumulate sum of radial scans. ***/

      for (ir = 0; ir < nrs; ir++)
	 {
	 *(scandisk + ir + it * nrad) = scan [ir] ;
	 scanavg [ir] = scanavg [ir] + scan [ir] ;

/*         scandisk [it][ir] = scan [ir] ; */
/*
 *	 printf ("ir: %4d  it: %4d  scandisk: %7.2f\n", ir, it, scandisk [it][ir]) ;
 */
	 }
      }

   fprintf (stdout, "rinc: %10.4f   nrs: %d  nrad: %d   nang: %d   nave: %d\n",
	    rinc, nrs, nrad, nang, nave) ;
   fflush (stdout) ;

   /*** Write radial scans to a disk file. ***/

   sprintf (namlds, "%s.lds", namimg) ;
   fprintf (stdout, "namlds: %s\n", namlds) ;
   FDS = fopen (namlds, "w") ;
   fwrite (scandisk, nrad * nang, sizeof (float), FDS) ;
   fclose (FDS) ;

/* -------------------------------------------------------------------------- */
   /*** Perform equatorial radial scans. ***/

   for (ir = 0; ir < MAXSCN; ir++)
       {
       scanavg [ir] = 0.0 ;
       scanmax [ir] = 0.0 ;
       }

   rmin = 0.0 ;
   rmax = 1.00001 ;
   rmax = (ipixrs - 1) / pixrs ; /* Make rmax slightly smaller than 1.0 Rsun  */
   rinc = 1.0 / pixrs ;
   nang = 0 ;

   itmin =  90 ;
   itmax = 270 ;
   itinc = 180 ;
   for (it = itmin; it <= itmax; it += itinc)
      {
      theta = it ;
      nang = nang + 1 ;
      fprintf (stdout, "theta: %7.2f\n", theta) ;

      crscan2 (acosbuf, pangle, nbypix, xdim, xcen, ycen, pixrs,
	      rmin, rmax, rinc,
	      theta, &scntyp, scan, scandx, &ns, nave) ;

      for (ir = 0; ir < ns; ir++)
	 {
	 scanavg [ir] = scanavg [ir] + scan [ir] ;
	 if (scan [ir] > scanmax [ir])  scanmax [ir] = scan [ir] ;
	 }
      }

   /*** Compute average radial scan. ***/

   for (ir = 0; ir < ns; ir++)
      scanavg [ir] /= (float) nang ;

   /*** Remove limb darkening. ***/

   ldmin  =  1.0e+6 ;
   ldmax  = -1.0e+6 ;
   radmin =  10.0 ;
   radmax = -10.0 ;

   for (iy = 1; iy < ydim; iy++)
      {
      yy = (float) iy ;
      for (ix = 0; ix < xdim; ix++)
	 {
	 xx = (float) ix ;

	 /*** Get solar radius. ***/

         crcoord (&radius, &angle, &xx, &yy, -1, degnorth, xcen, ycen, pixrs) ;

/*
         if (iy == 515)
	    printf ("ix/iy: %4d  xx/yy: %7.2f %7.2f  radius: %5.2f  sbuf: %6d\n",
		     ix, xx, yy, radius, *(sbuf + ix + iy * xdim)) ;
 */

         if (radius < radmin) radmin = radius ;
	 if (radius > radmax) radmax = radius ;

         ldcor = 0.0 ;
         if (radius < 0.99)
	    {
	    limbdark = lint (radius, scandx, scanmax, ns) ;
	    ldcor = savg / limbdark ;
	    if (ldcor < 0.0) ldcor = 0.0 ;
	    sval = *(sbuf + ix + iy * xdim) ;
	    ffix = (*(sbuf + ix + iy * xdim) * ldcor + 0.5) ;
	    sfix = (unsigned short) ffix ;
	    *(sbuf + ix + iy * xdim) = sfix ;

/*
         if (iy == 515)
	    printf ("radius: %5.2f  angle: %7.2f  limbdark: %7.2f  ldcor: %5.2f  sval: %6d\n",
		     radius, angle, limbdark, ldcor, sval) ;
 */

/*
	    printf ("xx/yy: %7.2f %7.2f  r/th: %7.2f %7.2f sbuf: %5d  sfix: %5d\n", xx, yy, radius, angle, *(sbuf + ix + iy * xdim), sfix) ;
 */
	    }

         *(ldbuf + ix + iy * xdim) = ldcor ;
         if (ldcor < ldmin) ldmin = ldcor ;
         if (ldcor > ldmax) ldmax = ldcor ;
	 }
      }

   printf ("ldmin:  %8.2f  ldmax:  %8.2f\n", ldmin, ldmax) ;
   printf ("radmin: %8.2f  radmax: %8.2f\n", radmin, radmax) ;

   /*** Write corrected image to disk. ***/

   npad = xdim * nbypix - sizeof (ACOSHS) ;
   padbuf = (unsigned char *) malloc (npad) ;

   sprintf (namout, "%s.ff", namimg) ;
   fprintf (stdout, "namout: %s\n", namout) ;
   FDOUT = fopen (namout, "w") ;
   fwrite ((char *) &ACOSHS, sizeof (ACOSHS), 1, FDOUT) ;
   fwrite (padbuf, npad, sizeof(unsigned char), FDOUT) ;
   fwrite (sbuf,   xdim * ydim, sizeof (unsigned short), FDOUT) ;
   fclose (FDOUT) ;

   /*** Write limb darkening correction "image" to disk. ***/

/*
 *   ixbeg = (int) ((ixcen - xdimsq / 2) + 0.5) ;
 *   ixbeg = (int) ((ixcen - xdimsq / 2)) ;
 */
   ixbeg = (int) ((xcen - xcensq) + 0.5) ;
   ixend = ixbeg + xdimsq - 1 ;

/*
 *   iybeg = (int) ((iycen - ydimsq / 2) + 0.5) ;
 *   iybeg = (int) ((iycen - ydimsq / 2)) ;
 */
   iybeg = (int) ((ycen - ycensq) + 0.5) ;
   iyend = iybeg + ydimsq - 1 ;

   xdif = (int) (((xdim - xdimsq) / 2) + 0.5) ;
   ydif = (int) (((ydim - ydimsq) / 2) + 0.5) ;

   printf ("ixbeg,ixend: %4d %4d  iybeg,iyend: %4d %4d\n",
            ixbeg, ixend, iybeg, iyend) ;

   ldsqr = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;

   for (iy = iybeg; iy <= iyend; iy++)
      {
      iysq = iy - ydif ;
      if (iysq < 0 || iysq > ydimsq-1)
      printf ("ix: %4d  iy: %4d  ixsq: %4d\n", ix, iy, ixsq) ;
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 ixsq = ix - xdif ;
	 if (ixsq < 0 || ixsq > xdimsq-1)
	 printf ("ix: %4d  iy: %4d  ixsq: %4d\n", ix, iy, ixsq) ;
	 *(ldsqr + ixsq + iysq * xdimsq) = *(ldbuf  + ix + iy * xdim) ;
	 }
      }

   sprintf (namldc, "%s.ldcor", namimg) ;
   fprintf (stdout, "namldc: %s\n", namldc) ;

   FDCOR = fopen (namldc, "w") ;
   fwrite (ldsqr, xdimsq * ydimsq, 4, FDCOR) ;
   fclose (FDCOR) ;

   free (padbuf) ;
   free (sbuf) ;
   free (acosbuf) ;
   free (ldsqr) ;
   free (ldbuf) ;
   free (dopcor) ;
   free (scandisk) ;

   exit (0) ;
}
