/* chip_ldark2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Perform radial scans on a CHIP image.
 * Determine limb darkening correction via bilinear interpolation of
 * the radial scans.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   16 May 2000
 *  9 Aug 2000 : Use individual radial scans to form an (r,theta) limb
 *              darkening grid over which to interpolate.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#define ACOS

#include "/home/cordyn/mlsofits/mechanism_defs.h"
#include "/home/cordyn/mlsofits/acos.h"
#include "/home/cordyn/mlsofits/chip.h"

#include "/home/cordyn/mlsofits/device.h"
#include "/home/cordyn/mlsofits/analog.h"
#include "/home/cordyn/mlsofits/lcvr.h"
#include "/home/cordyn/mlsofits/mic.h"

#include "/home/stanger/lib/src/acos/acoslib.h"
#include "chipscan.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern int acoshskp () ;		/* Function to read ACOS header. */
   extern unsigned short *rdacos () ;	/* Function to read ACOS image. */
   extern float lint () ;		/* Linear interpolation. */
   extern float blint () ;		/* Bilinear interpolation. */

   static char namfts [80] ;		/* FITS file name. */
   static char namimg [80] ;		/* ACOS file name. */
   static char namout [80] ;		/* Limb Darkening corrected image. */
   static char namldc [80] ;		/* Limb Darkening correction name. */
   static char namlds [80] ;		/* Limb Darkening scan name. */

   float *scandisk ;
   static float scanavg [MAXSCN] ;
   static float scan    [MAXSCN] ;
   static float scandx  [MAXSCN] ;
   float *ldbuf, *ldsqr ;
   unsigned short *acosbuf ;             /* MLSO ACOS file  array pointer. */
   unsigned short *sbuf ;
   unsigned char *padbuf ;
   unsigned char *cbuf ;

   unsigned short sval ;

   float angle ;
   float cdelt1 ;
   float degnorth = 0.0 ;
   float ffix ;
   float ldcor ;
   float limbdark ;
   float ldmin, ldmax ;
   float pangle ;
   float pang ;
   float pixrs ;
   float radius ;
   float radmin, radmax ;
   float rmin, rmax ;
   float rinc ;
   float rindex ;
   float tindex ;
   float cenavg ;
   float theta ;
   float xcen, ycen ;
   float xx, yy ;

   int ipixrs ;
   int ir ;
   int it ;
   int ix, iy ;
   int ixsq, iysq ;
   int ixbeg, ixend ;
   int ixcen, iycen ;
   int iybeg, iyend ;
   int navg ;
   int nbypix ;
   int nbyimg ;
   int nang = 362 ;		/* Angle range: -1.0 --> 361.0 degrees. */
   int nave = 11 ;
   int npad ;
   int npximg ;
   int nrad = 450 ;		/* Beyond largest radius the sun can become. */
   int nrs ;			/* Number of measurements in radial scan. */
   int ns ;
   int pmode = 1 ;
   int scntyp = 1 ;
   int irmin =   0 ;
   int irmax = 449 ;
   int itmin =   0 ;
   int itmax = 361 ;
   int xdim, ydim ;
   int xdimsq, ydimsq ;
   int xdif, ydif ;

   unsigned short sfix ;

   FILE *FD, *FDOUT, *FDCOR ;
   FILE *FDS ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s chip_filename\n", argv [0]) ;
      exit (-1) ;
      }

   sprintf (namout, "%s.ld", namimg) ;

   /*** Read header. ***/

   acoshskp (namimg, 0) ;

   /*** Get items from header. ***/

   xdim = ACOSI->xdim ;
   ydim = ACOSI->ydim ;
   nbypix = ACOSI->nbypx ;
   xcen = (float) (ACOSH->Xdim - 1) * 0.5 ;
   ycen = (float) (ACOSH->Ydim - 1) * 0.5 ;
   ixcen = (int) xcen ;
   iycen = (int) ycen ;
   cdelt1 = 2.29 ;				/* arcsec / pixel */
   pixrs = ACOSI->rsun / cdelt1 ;		/* pixels / solar radius. */
   ipixrs = (int) pixrs ;
   pangle = ACOSI->pangle ;			/* rotation angle. */
   pang   = 0.0 ;
   printf ("xdim: %d  ydim: %d\n", xdim, ydim) ;
   printf ("xcen: %7.2f ycen: %7.2f\n", xcen, ycen) ;
   printf ("pixrs: %7.2f  pangle: %7.2f\n", pixrs, pangle) ;

   /*** Allocate storage for image array. ***/

   npximg = ACOSH->Xdim * ACOSH->Ydim ;         /* # pixels in image. */
   nbypix = sizeof (unsigned short) ;           /* # bytes  in image. */
   nbyimg = npximg * nbypix ;

   acosbuf = (unsigned short *) malloc (npximg * nbypix) ;
   sbuf    = (unsigned short *) malloc (npximg * nbypix) ;
   ldbuf   = (float *)          malloc (npximg * sizeof (float)) ;
   
   /*** Read image data. ***/

/*
 *   acosbuf = rdacos (namimg, pmode) ;
 */

   FD = fopen (namimg, "r") ;   
   if (FD == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      exit (-2) ;
      }

   fread (acosbuf, nbypix, npximg, FD) ;
   fclose (FD) ;

   /*** Set first 3 lines of image to zero (eliminate binary header). ***/

   for (iy = 0; iy < 3; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(acosbuf + ix + iy * xdim) = 0 ;
	 }

   /*** Invert image top-to-bottom. ***/

   for (iy = 0; iy < ydim; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(sbuf + ix + iy * xdim) = *(acosbuf + ix + (ydim - iy - 1) * xdim) ;
	 }

   cbuf = (unsigned char *) sbuf ;

   /*** Find average intensity near disk center. ***/

   ixbeg = ixcen - 5 ;
   ixend = ixbeg + 10 ;
   iybeg = iycen - 5 ;
   iyend = iybeg + 10 ;

   cenavg = 0.0 ;
   navg =   0 ;
   for (iy = iybeg; iy <= iyend; iy++)
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 cenavg += (float) *(sbuf + ix + iy * xdim) ;
	 navg++ ;
	 }
   cenavg = (cenavg / navg) ;

   ix = (int) xcen ;
   iy = (int) ycen ;

   printf ("sbuf [center]: %d   cenavg: %7.2f\n",
	   *(sbuf + ix + iy * xdim), cenavg) ;

   /*** For scandisk array, angle dimension needs to go 1.0 degree BEYOND
    *** the range of 0 -> 360 degrees (i.e., -1.0 & 361.0) for bilinear
    *** interpolation at the boundaries (0 degrees and 360 degrees).
    ***/

   itmin =   0 ;
   itmax = itmin + nang ;

   rinc = 1.0 / pixrs ;			/* Radial increment: ~ 1 pixel. */
   rmin = 0.0 ;
   rmax = (ipixrs - 1) / pixrs ;
/*
 *   rmax = 1.00001 ;
 *   rmax = 1.0 + rinc ;
 *   rmax = (ipixrs - 1) / pixrs ;
 */
   nrad = (int) (((rmax - rmin) / rinc) + 0.0001) + 1 ;
   printf ("computed nrad: %d   computed nang: %d\n", nrad, nang) ;

   scandisk = (float *) malloc (nrad * nang * sizeof (float)) ;

   /*** Clear scan buffers. ***/

   for (ir = 0; ir < MAXSCN; ir++) scanavg [ir] = 0.0 ;

   for (it = 0; it < nang; it++)
      for (ir = 0; ir < nrad; ir++)
         *(scandisk + ir + it * nrad) = 0.0 ;

   /*** Perform radial scans. ***/
   /*** Set P-angle to 0 so that scans are done in geocentric coordinates. ***/

   for (it = itmin; it < itmax; it += 1.0)
      {
      theta = (float) it - 1.0 ;	/* Start at -1.0 degree. */

      crscan2 (cbuf, pang, nbypix, xdim, xcen, ycen, pixrs,
	      rmin, rmax, rinc, theta, &scntyp, scan, scandx, &nrs, nave) ;

      if (nrs > nrad)
	 {
	 fprintf (stderr, "chip_ldark2> Error: nrs [%d] > nrad [%d]\n",
		  nrs, nrad) ;
	 exit (-1) ;
	 }

      /*** Save each radial scan in a 2-D array: (r,th). ***/
      /*** Accumulate sum of radial scans. ***/

      for (ir = 0; ir < nrs; ir++)
	 {
	 *(scandisk + ir + it * nrad) = scan [ir] ;
	 scanavg [ir] = scanavg [ir] + scan [ir] ;

/*         scandisk [it][ir] = scan [ir] ; */
/*
 *	 printf ("ir: %4d  it: %4d  scandisk: %7.2f\n", ir, it, scandisk [it][ir]) ;
 */
	 }
      }

   fprintf (stdout, "rinc: %10.4f   nrs: %d  nrad: %d   nang: %d   nave: %d\n",
	    rinc, nrs, nrad, nang, nave) ;
   fflush (stdout) ;

   /*** Write radial scans to a disk file. ***/

   sprintf (namlds, "%s.lds", namimg) ;
   FDS = fopen (namlds, "w") ;
   fwrite (scandisk, nrad * nang, sizeof (float), FDS) ;
   fclose (FDS) ;

   /*** Compute average radial scan. ***/

   for (ir = 0; ir < nrad; ir++)
      scanavg [ir] /= (float) nang ;

   /*** Remove limb darkening. ***/
   /*** Use bilinear interpolation. ***/

   ldmin  =  1.0e+6 ;
   ldmax  = -1.0e+6 ;
   radmin =  10.0 ;
   radmax = -10.0 ;

   for (iy = 1; iy < ydim; iy++)
      {
      yy = (float) iy ;
      for (ix = 0; ix < xdim; ix++)
	 {
	 xx = (float) ix ;

	 /*** Get solar radius. ***/

         crcoord (&radius, &angle, &xx, &yy, -1, degnorth, xcen, ycen, pixrs) ;

/*
 *         if (iy == 515)
 *	    printf ("ix/iy: %4d  xx/yy: %7.2f %7.2f  radius: %5.2f  sbuf: %6d\n",
 *		     ix, xx, yy, radius, *(sbuf + ix + iy * xdim)) ;
 */

         if (radius < radmin) radmin = radius ;
	 if (radius > radmax) radmax = radius ;

         ldcor = 0.0 ;
         if (radius < 0.995)
	    {
/*
 *            limbdark = lint (radius, scandx, scanavg, nrad) ;
 */
	    rindex = radius / rinc ;
	    tindex = angle + 1.0 ;	/* Since angle axis range: -1 to 361. */
	    limbdark = blint (scandisk, 4, nrad, nang, rindex, tindex) ;

            if (limbdark != 0.0)
	       ldcor = cenavg / limbdark ;
            else
	       ldcor = 0.0 ;

	    if (ldcor < 0.0) ldcor = 0.0 ;
	    sval = *(sbuf + ix + iy * xdim) ;
	    ffix = (*(sbuf + ix + iy * xdim) * ldcor + 0.5) ;
	    sfix = (unsigned short) ffix ;
	    *(sbuf + ix + iy * xdim) = sfix ;

         if (iy == 515)
	    printf ("rindex: %5.2f radius:% 6.4f angle: %7.2f limbdark: %6.1f ldcor: %5.2f\n",
		     rindex, radius, angle, limbdark, ldcor) ;

/*
	    printf ("xx/yy: %7.2f %7.2f  rindex: %7.2f  r/th: %7.2f %7.2f sbuf: %5d\n", xx, yy, rindex, radius, angle, *(sbuf + ix + iy * xdim)) ;
 */
	    }

         *(ldbuf + ix + iy * xdim) = ldcor ;
         if (ldcor < ldmin) ldmin = ldcor ;
         if (ldcor > ldmax) ldmax = ldcor ;
	 }
      }

   printf ("ldmin:  %8.2f  ldmax:  %8.2f\n", ldmin, ldmax) ;
   printf ("radmin: %8.2f  radmax: %8.2f\n", radmin, radmax) ;

   /*** Write corrected image to disk. ***/

   npad = xdim * nbypix - sizeof (ACOSHS) ;
   padbuf = (unsigned char *) malloc (npad) ;
   FDOUT = fopen (namout, "w") ;
   fwrite ((char *) &ACOSHS, sizeof (ACOSHS), 1, FDOUT) ;
   fwrite (padbuf, npad, sizeof(unsigned char), FDOUT) ;
   fwrite (sbuf,   xdim * ydim, sizeof (unsigned short), FDOUT) ;
   fclose (FDOUT) ;

   /*** Write limb darkening correction "image" to disk. ***/

   printf ("ixcen,iycen: %4d %4d\n", ixcen, iycen) ;
   xdimsq = 936 ;
   ydimsq = 936 ;

   ixbeg = (int) ((ixcen - xdimsq / 2) + 0.5) ;
   ixend = ixbeg + xdimsq - 1 ;
   iybeg = (int) ((iycen - ydimsq / 2) + 0.5) ;
   iyend = iybeg + ydimsq - 1 ;

   xdif = (int) (((xdim - xdimsq) / 2) + 0.5) ;
   ydif = (int) (((ydim - ydimsq) / 2) + 0.5) ;

   printf ("ixbeg,ixend: %4d %4d  iybeg,iyend: %4d %4d\n",
            ixbeg, ixend, iybeg, iyend) ;

   ldsqr = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;

   for (iy = iybeg; iy <= iyend; iy++)
      {
      iysq = iy - ydif + 1 ;
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 ixsq = ix - xdif + 1 ;
	 *(ldsqr + ixsq + iysq * xdimsq) = *(ldbuf + ix + iy * xdim) ;
	 }
      }

   /*** Write FITS file. ***/

/*
 *   sprintf (namfts, "%s.ldcor.fts", namimg) ;
 *   img2fts (namfts, ldsqr, 4, xdimsq, ydimsq) ;
 */

   sprintf (namldc, "%s.ldcor", namimg) ;
   FDCOR = fopen (namldc, "w") ;
   fwrite (ldsqr, xdimsq * ydimsq, 4, FDCOR) ;
   fclose (FDCOR) ;

   free (padbuf) ;
   free (sbuf) ;

   exit (0) ;
}
