/* chip_ldark3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Perform radial scans on a CHIP image.
 * Determine limb darkening correction by averaging radial scans.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   16 May 2000
 * 9 Aug 2000 : Use individual radial scans to form an (r,theta) limb
 *              darkening grid over which to interpolate.
 * 11 Aug 2000 : Read input images in FITS format, thereby eliminating need
 *               to invert the image top-to-bottom, and using disk center info.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/fitslib.h"
#include "chipscan.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern unsigned char *rdfits () ;	/* Read a FITS file. */
   extern int acoshskp () ;		/* Function to read ACOS header. */
   extern unsigned short *rdacos () ;	/* Function to read ACOS image. */
   extern float lint () ;		/* Linear interpolation. */
   extern float blint () ;		/* Bilinear interpolation. */

   static char namfts [80] ;		/* FITS file name. */
   static char namimg [80] ;		/* ACOS file name. */
   static char namout [80] ;		/* Limb Darkening corrected image. */
   static char namldc [80] ;		/* Limb Darkening correction name. */

/*   static float scanrad [360][MAXSCN] ; */
   float *scanrad ;
   static float scanavg [MAXSCN] ;
   static float scan    [MAXSCN] ;
   static float scandx  [MAXSCN] ;
   float *ldbuf, *ldsqr ;
   unsigned short *sbuf ;
   unsigned char *cbuf ;

   unsigned short sval ;

   float angle ;
   float cdelt1 ;
   float degnorth = 0.0 ;
   float ffix ;
   float ldcor ;
   float limbdark ;
   float ldmin, ldmax ;
   float pangle ;
   float pang ;
   float pixrs ;
   float radius ;
   float radmin, radmax ;
   float rmin, rmax ;
   float rinc ;
   float rindex ;
   float tindex ;
   float savg ;
   float theta ;
   float xcen, ycen ;
   float xx, yy ;

   int ipixrs ;
   int ir ;
   int it ;
   int ix, iy ;
   int ixsq, iysq ;
   int ixbeg, ixend ;
   int ixcen, iycen ;
   int iybeg, iyend ;
   int navg ;
   int nbypix ;
   int nbyimg ;
   int nang ;
   int nave = 15 ;
   int npximg ;
   int ns ;
   int nrad ;
   int nth ;
   int pmode = 1 ;
   int scntyp = 1 ;
   int  itmin =   0 ;
   int  itmax = 360 ;
   int xdim, ydim ;
   int xdimsq, ydimsq ;
   int xdif, ydif ;

   unsigned short sfix ;

   FILE *FD, *FDOUT, *FDCOR ;
   FILE *FDS ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s chip_filename\n", argv [0]) ;
      exit (-1) ;
      }

   sprintf (namout, "%s.ld", namimg) ;

   /*** Read FITS image. ***/

   cbuf = rdfits (namimg) ;
   sbuf = (unsigned short *) cbuf ;

   /*** Get items from header. ***/

   xdim   = naxis1 ;
   ydim   = naxis2 ;
   nbypix = bitpix / 8 ;
   xcen   = crpix1 ;
   ycen   = crpix2 ;
   if (xcen < 0.0)  xcen = (xdim  / 2.0) - 0.5 ;
   if (ycen < 0.0)  ycen = (ydim  / 2.0) - 0.5 ;

   ixcen = (int) xcen ;
   iycen = (int) ycen ;
   pixrs = crradius ;				/* pixels / solar radius. */
   ipixrs = (int) pixrs ;
   pangle = solar_p0 ;				/* P-angle. */
   pang   = 0.0 ;

   printf ("xdim: %d  ydim: %d\n", xdim, ydim) ;
   printf ("xcen: %7.2f ycen: %7.2f\n", xcen, ycen) ;
   printf ("pixrs: %7.2f  pangle: %7.2f\n", pixrs, pangle) ;

   /*** Allocate storage for image array. ***/

   npximg = xdim * ydim ;
   nbyimg = npximg * nbypix ;
   ldbuf = (float *) malloc (npximg * sizeof (float)) ;
   
   /*** Set last 3 lines of image to zero (image is inverted top-to-bottom) ***/

   for (iy = ydim-3; iy < ydim; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(sbuf + ix + iy * xdim) = 0 ;
	 }

   /*** Find average intensity near disk center. ***/

   ixbeg = ixcen - 5 ;
   ixend = ixbeg + 10 ;
   iybeg = iycen - 5 ;
   iyend = iybeg + 10 ;

   savg = 0.0 ;
   navg =   0 ;
   for (iy = iybeg; iy <= iyend; iy++)
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 savg += (float) *(sbuf + ix + iy * xdim) ;
	 navg++ ;
	 }
   savg = (savg / navg) ;
   printf ("savg: %7.2f\n", savg) ;

   ix = (int) xcen ;
   iy = (int) ycen ;
   printf ("sbuf [center]: %d\n", *(sbuf + ix + iy * xdim)) ;

   /*** Clear scan buffers. ***/

   /*** For scanrad array, angle dimension needs to go 1.0 degree BEYOND
    *** the range of 0 -> 360 degrees for bilinear interpolation at the
    *** boundaries (0 degrees and 360 degrees).
    ***/

   itmin =   0 ;
   itmax = 362 ;		/* angle range needs to be -1.0 to 361.0 deg. */

   rmin = 0.0 ;
   rmax = 1.00001 ;
   rmax = (ipixrs - 1) / pixrs ; /* Make rmax smaller than 1.0 Rsun  */
   rinc = 1.0 / pixrs ;
   rmax = 1.0 + rinc ;
   nth  = 362 ;
   nrad = (int) ((rmax - rmin) / rinc) + 1 ;
   printf ("computed nrad: %d   computed nth: %d\n", nrad, nth) ;

   scanrad = (float *) malloc (nrad * nth * sizeof (float)) ;

   for (ir = 0; ir < MAXSCN; ir++) scanavg [ir] = 0.0 ;

   for (it = 0; it < nth; it++)
      for (ir = 0; ir < nrad; ir++)
/*         scanrad [it][ir] = 0.0 ; */
         *(scanrad + ir + it * nrad) = 0.0 ;

   /*** Perform radial scans. ***/
   /*** Set P-angle to 0 so that scans are done in geocentric coordinates. ***/

   nang = 0 ;
   for (it = itmin; it < itmax; it += 1.0)
      {
      theta = (float) it - 1.0 ;
      nang = nang + 1 ;

      crscan2 (cbuf, pang, nbypix, xdim, xcen, ycen, pixrs,
	      rmin, rmax, rinc,
	      theta, &scntyp, scan, scandx, &nrad, nave) ;

      /*** Save each radial scan in a 2-D array: (r,th). ***/
      /*** Accumulate sum of radial scans. ***/

      for (ir = 0; ir < nrad; ir++)
	 {
/*         scanrad [it][ir] = scan [ir] ; */
	 *(scanrad + ir + it * nrad) = scan [ir] ;
	 scanavg [ir] = scanavg [ir] + scan [ir] ;
/*
 *	 printf ("ir: %4d  it: %4d  scanrad: %7.2f\n", ir, it, scanrad [it][ir]) ;
 */
	 }
      }

   fprintf (stdout, "rinc: %10.4f   nrad: %d   nang: %d\n", rinc, nrad, nang) ;
   fflush (stdout) ;

   FDS = fopen ("scanrad.tab", "w") ;
   fwrite (scanrad, nrad * nang, sizeof (float), FDS) ;
   fclose (FDS) ;

   /*** Compute average radial scan. ***/

   for (ir = 0; ir < nrad; ir++)
      scanavg [ir] /= (float) nang ;

   /*** Remove limb darkening. ***/
   /*** Use bilinear interpolation. ***/

   ldmin  =  1.0e+6 ;
   ldmax  = -1.0e+6 ;
   radmin =  10.0 ;
   radmax = -10.0 ;

   for (iy = 1; iy < ydim; iy++)
      {
      yy = (float) iy ;
      for (ix = 0; ix < xdim; ix++)
	 {
	 xx = (float) ix ;

	 /*** Get solar radius. ***/

         crcoord (&radius, &angle, &xx, &yy, -1, degnorth, xcen, ycen, pixrs) ;

/*
         if (iy == 515)
	    printf ("ix/iy: %4d  xx/yy: %7.2f %7.2f  radius: %5.2f  sbuf: %6d\n",
		     ix, xx, yy, radius, *(sbuf + ix + iy * xdim)) ;
 */

         if (radius < radmin) radmin = radius ;
	 if (radius > radmax) radmax = radius ;

         ldcor = 0.0 ;
         if (radius < 1.00)
	    {
/*
 *            limbdark = lint (radius, scandx, scanavg, nrad) ;
 */
	    rindex = radius / rinc ;
	    tindex = angle + 1.0 ;	/* Since angle axis range: -1 to 361. */
	    limbdark = blint (scanrad, 4, nrad, nang, rindex, tindex) ;

            if (limbdark != 0.0)
	       ldcor = savg / limbdark ;
            else
	       ldcor = 0.0 ;

	    if (ldcor < 0.0) ldcor = 0.0 ;
	    sval = *(sbuf + ix + iy * xdim) ;
	    ffix = (*(sbuf + ix + iy * xdim) * ldcor + 0.5) ;
	    sfix = (unsigned short) ffix ;
	    *(sbuf + ix + iy * xdim) = sfix ;

         if (iy == 515)
	    printf ("rindex: %5.2f radius:% 6.4f angle: %7.2f limbdark: %6.1f ldcor: %5.2f\n",
		     rindex, radius, angle, limbdark, ldcor) ;

/*
	    printf ("xx/yy: %7.2f %7.2f  rindex: %7.2f  r/th: %7.2f %7.2f sbuf: %5d\n", xx, yy, rindex, radius, angle, *(sbuf + ix + iy * xdim)) ;
 */
	    }

         *(ldbuf + ix + iy * xdim) = ldcor ;
         if (ldcor < ldmin) ldmin = ldcor ;
         if (ldcor > ldmax) ldmax = ldcor ;
	 }
      }

   printf ("ldmin:  %8.2f  ldmax:  %8.2f\n", ldmin, ldmax) ;
   printf ("radmin: %8.2f  radmax: %8.2f\n", radmin, radmax) ;

   /*** Write corrected image to disk. ***/

   FDOUT = fopen (namout, "w") ;
   fwrite (sbuf,   xdim * ydim, sizeof (unsigned short), FDOUT) ;
   fclose (FDOUT) ;

   /*** Write limb darkening correction "image" to disk. ***/

   fprintf (stdout, "ixcen,iycen: %4d %4d\n", ixcen, iycen) ;
   xdimsq = 936 ;
   ydimsq = 936 ;

   ixbeg = (int) ((xcen - xdimsq / 2) + 0.5) ;
   ixend = ixbeg + xdimsq - 1 ;
   iybeg = (int) ((ycen - ydimsq / 2) + 0.5) ;
   iyend = iybeg + ydimsq - 1 ;

   xdif = (int) (((xdim - xdimsq) / 2) + 0.5) ;
   ydif = (int) (((ydim - ydimsq) / 2) + 0.5) ;

   fprintf (stdout, "xdif/ydif: %4d %4d\n", xdif, ydif) ;
   fprintf (stdout, "ixbeg,ixend: %4d %4d  iybeg,iyend: %4d %4d\n",
                    ixbeg, ixend, iybeg, iyend) ;

   ldsqr = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;

   for (iy = iybeg; iy <= iyend; iy++)
      {
      iysq = iy - ydif + 1 ;
      if (iysq < 0 || iysq > ydimsq-1)
	 {
	 printf ("iysq invalid: %d\n", iysq) ;
	 exit (-1) ;
	 }
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 ixsq = ix - xdif + 1 ;
         if (ixsq < 0 || ixsq > xdimsq-1)
	    {
	    printf ("ixsq invalid: %d\n", ixsq) ;
	    exit (-2) ;
	    }
	 *(ldsqr + ixsq + iysq * xdimsq) = *(ldbuf + ix + iy * xdim) ;
	 }
      }

   /*** Write FITS file. ***/

/*
 *   sprintf (namfts, "%s.ldcor.fts", namimg) ;
 *   img2fts (namfts, ldsqr, 4, xdimsq, ydimsq) ;
 */

   sprintf (namldc, "%s.ldcor", namimg) ;
   FDCOR = fopen (namldc, "w") ;
   fwrite (ldsqr, xdimsq * ydimsq, 4, FDCOR) ;
   fclose (FDCOR) ;

   free (sbuf) ;

   exit (0) ;
}
