/* chip_ldcor5.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Determine limb darkening correction via bilinear interpolation of
 * the azimuthal scans.
 *
 * Use chip_ldark5.c to generate the polar coordinate array of azimuthal scans
 * (one azimuthal scan for every radial element).
 * Use IDL code (do_fit5.pro) to fit an equation to the azimuthal scans.
 * Use this program to perform the bilinear interpolation to create an image
 * of limb-darkening correction factors.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 Aug 2000
 * 31 August 2000 : Adapted from chip_ldcor.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#define ACOS

#include "/home/cordyn/mlsofits/mechanism_defs.h"
#include "/home/cordyn/mlsofits/acos.h"
#include "/home/cordyn/mlsofits/chip.h"

#include "/home/cordyn/mlsofits/device.h"
#include "/home/cordyn/mlsofits/analog.h"
#include "/home/cordyn/mlsofits/lcvr.h"
#include "/home/cordyn/mlsofits/mic.h"

#include "/home/stanger/lib/src/acos/acoslib.h"
#include "chipscan.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern int acoshskp () ;		/* Function to read ACOS header. */
   extern unsigned short *rdacos () ;	/* Function to read ACOS image. */
   extern float lint () ;		/* Linear interpolation. */
   extern float blint () ;		/* Bilinear interpolation. */
   extern unsigned short *srotimg () ;  /* Rotation function. */

   static char namimg [80] ;		/* ACOS file name. */
   static char namout [80] ;		/* Limb Darkening corrected image. */
   static char namldc [80] ;		/* Limb Darkening corrected name. */
   static char namldf [80] ;		/* Limb Darkening factor name. */

   float *scanrad ;
   float *ldfactab ;
   static float scanavg [MAXSCN] ;
   static float scan    [MAXSCN] ;
   static float scandx  [MAXSCN] ;
   float *ldbuf, *ldsqr ;
   unsigned short *acosbuf ;             /* MLSO ACOS file  array pointer. */
   unsigned short *sbuf ;
   unsigned short *srot ;
   unsigned char *padbuf ;
   unsigned char *cbuf ;

   unsigned short sval ;

   float angle ;
   float cdelt1 ;
   float degnorth = 0.0 ;
   float ffix ;
   float ldcor ;
   float limbdark ;
   float ldmin, ldmax ;
   float pangle ;
   float pang ;
   float pixrs ;
   float radius ;
   float radmin, radmax ;
   float rmin, rmax ;
   float rinc ;
   float rindex ;
   float tindex ;
   float savg ;
   float theta ;
   float xcen, ycen ;
   float xx, yy ;

   int ipixrs ;
   int ir ;
   int it ;
   int ix, iy ;
   int ixsq, iysq ;
   int ixbeg, ixend ;
   int ixcen, iycen ;
   int iybeg, iyend ;
   int navg ;
   int nbypix ;
   int nbyimg ;
   int nang ;
   int nave = 11 ;
   int npad ;
   int npximg ;
   int ns ;
   int nrad ;
   int pmode = 1 ;
   int scntyp = 1 ;
   int  itmin =   0 ;
   int  itmax = 360 ;
   int xdim, ydim ;
   int xdimsq, ydimsq ;
   int xdif, ydif ;

   unsigned short sfix ;

   FILE *FD, *FDOUT, *FDCOR ;
   FILE *FDS ;
   FILE *FDC ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s chip_filename\n", argv [0]) ;
      exit (-1) ;
      }

   sprintf (namout, "%s.ld", namimg) ;

   /*** Read header. ***/

   acoshskp (namimg, 0) ;

   /*** Get items from header. ***/

   xdim = ACOSI->xdim ;
   ydim = ACOSI->ydim ;
   nbypix = ACOSI->nbypx ;
   xcen = (float) (ACOSH->Xdim - 1) * 0.5 ;
   ycen = (float) (ACOSH->Ydim - 1) * 0.5 ;
   ixcen = (int) xcen ;
   iycen = (int) ycen ;
   cdelt1 = 2.29 ;				/* arcsec / pixel */
   pixrs = ACOSI->rsun / cdelt1 ;		/* pixels / solar radius. */
   ipixrs = (int) pixrs ;
   pangle = ACOSI->pangle ;			/* rotation angle. */
   pang   = 0.0 ;
   printf ("xdim: %d  ydim: %d\n", xdim, ydim) ;
   printf ("xcen: %7.2f ycen: %7.2f\n", xcen, ycen) ;
   printf ("pixrs: %7.2f  pangle: %7.2f\n", pixrs, pangle) ;

   /*** Allocate storage for image array. ***/

   npximg = ACOSH->Xdim * ACOSH->Ydim ;         /* # pixels in image. */
   nbypix = sizeof (unsigned short) ;           /* # bytes  in image. */
   nbyimg = npximg * nbypix ;

   acosbuf = (unsigned short *) malloc (npximg * nbypix) ;
   sbuf    = (unsigned short *) malloc (npximg * nbypix) ;
   srot    = (unsigned short *) malloc (npximg * nbypix) ;
   ldbuf   = (float *)          malloc (npximg * sizeof (float)) ;
   
   /*** Read image data. ***/

   FD = fopen (namimg, "r") ;   
   if (FD == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      exit (-2) ;
      }

   fread (acosbuf, nbypix, npximg, FD) ;
   fclose (FD) ;

   /*** Set first 3 lines of image to zero (eliminate binary header). ***/

   for (iy = 0; iy < 3; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(acosbuf + ix + iy * xdim) = 0 ;
	 }

   /*** Invert image top-to-bottom. ***/

   for (iy = 0; iy < ydim; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(sbuf + ix + iy * xdim) = *(acosbuf + ix + (ydim - iy - 1) * xdim) ;
	 }

   /*** Rotate image to create image in heliocentric coordinates. ***/

   srot = srotimg (sbuf, xdim, ydim, ixcen, iycen, -pangle) ;
   free (sbuf) ;
   sbuf = srot ;

   itmin =   0 ;
   itmax = 362 ;		/* angle range needs to be -1.0 to 361.0 deg. */

   rinc = 1.0 / pixrs ;
   rmin = 0.0 ;
   rmax = (ipixrs - 1) / pixrs ; /* Make rmax smaller than 1.0 Rsun  */
   nang = 362 ;
   nrad = (int) (((rmax - rmin) / rinc) + 0.5) ;
   printf ("computed nrad: %d   computed nang: %d\n", nrad, nang) ;

   ldfactab = (float *) malloc (nrad * nang * sizeof (float)) ;

   /*** Read polar coordinate radial scan table. ***/

   sprintf (namldf, "%s.lds_fac", namimg) ;

   FDC = fopen (namldf, "r") ;
   fread  (ldfactab, nrad * nang, sizeof (float), FDC) ;
   fclose (FDC) ;

   /*** Remove limb darkening. ***/
   /*** Use bilinear interpolation. ***/

   ldmin  =  1.0e+6 ;
   ldmax  = -1.0e+6 ;
   radmin =  10.0 ;
   radmax = -10.0 ;

   for (iy = 1; iy < ydim; iy++)
      {
      yy = (float) iy ;
      for (ix = 0; ix < xdim; ix++)
	 {
	 xx = (float) ix ;

	 /*** Get solar radius. ***/

         crcoord (&radius, &angle, &xx, &yy, -1, degnorth, xcen, ycen, pixrs) ;

/*
         if (iy == 515)
	    printf ("ix/iy: %4d  xx/yy: %7.2f %7.2f  radius: %5.2f  sbuf: %6d\n",
		     ix, xx, yy, radius, *(sbuf + ix + iy * xdim)) ;
 */

         if (radius < radmin) radmin = radius ;
	 if (radius > radmax) radmax = radius ;

         ldcor = 0.0 ;
         if (radius < 0.995)
	    {
	    rindex = radius / rinc ;
	    tindex = angle + 1.0 ;	/* Since angle axis range: -1 to 361. */
	    ldcor = blint (ldfactab, 4, nrad, nang, rindex, tindex) ;

	    if (ldcor < 0.0) ldcor = 0.0 ;

	    sval = *(sbuf + ix + iy * xdim) ;
	    ffix = (*(sbuf + ix + iy * xdim) * ldcor + 0.5) ;
	    sfix = (unsigned short) ffix ;
	    *(sbuf + ix + iy * xdim) = sfix ;

         if (iy == 515)
	    printf ("rindex:%5.2f  radius:%6.4f  angle:%7.2f  ldcor:%5.2f\n",
		     rindex, radius, angle, ldcor) ;

/*
	    printf ("xx/yy: %7.2f %7.2f  rindex: %7.2f  r/th: %7.2f %7.2f sbuf: %5d\n", xx, yy, rindex, radius, angle, *(sbuf + ix + iy * xdim)) ;
 */
	    }

         *(ldbuf + ix + iy * xdim) = ldcor ;
         if (ldcor < ldmin) ldmin = ldcor ;
         if (ldcor > ldmax) ldmax = ldcor ;
	 }
      }

   printf ("ldmin:  %8.2f  ldmax:  %8.2f\n", ldmin, ldmax) ;
   printf ("radmin: %8.2f  radmax: %8.2f\n", radmin, radmax) ;

   /*** Write corrected image to disk. ***/

   npad = xdim * nbypix - sizeof (ACOSHS) ;
   padbuf = (unsigned char *) malloc (npad) ;
   FDOUT = fopen (namout, "w") ;
   fwrite ((char *) &ACOSHS, sizeof (ACOSHS), 1, FDOUT) ;
   fwrite (padbuf, npad, sizeof (unsigned char), FDOUT) ;
   fwrite (sbuf,   xdim * ydim, sizeof (unsigned short), FDOUT) ;
   fclose (FDOUT) ;

   /*** Write limb darkening correction "image" to disk. ***/

   printf ("ixcen,iycen: %4d %4d\n", ixcen, iycen) ;
   xdimsq = 936 ;
   ydimsq = 936 ;

   ixbeg = (int) ((ixcen - xdimsq / 2) + 0.5) ;
   ixend = ixbeg + xdimsq - 1 ;
   iybeg = (int) ((iycen - ydimsq / 2) + 0.5) ;
   iyend = iybeg + ydimsq - 1 ;

   xdif = (int) (((xdim - xdimsq) / 2) + 0.5) ;
   ydif = (int) (((ydim - ydimsq) / 2) + 0.5) ;

   printf ("ixbeg,ixend: %4d %4d  iybeg,iyend: %4d %4d\n",
            ixbeg, ixend, iybeg, iyend) ;

   ldsqr = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;

   for (iy = iybeg; iy <= iyend; iy++)
      {
      iysq = iy - ydif + 1 ;
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 ixsq = ix - xdif + 1 ;
	 *(ldsqr + ixsq + iysq * xdimsq) = *(ldbuf + ix + iy * xdim) ;
	 }
      }

   /*** Write FITS file. ***/

/*
 *   sprintf (namfts, "%s.ldcor.fts", namimg) ;
 *   img2fts (namfts, ldsqr, 4, xdimsq, ydimsq) ;
 */

   sprintf (namldc, "%s.ldcor", namimg) ;
   FDCOR = fopen (namldc, "w") ;
   fwrite (ldsqr, xdimsq * ydimsq, 4, FDCOR) ;
   fclose (FDCOR) ;

   free (padbuf) ;
   free (sbuf) ;

   exit (0) ;
}
