/* selip.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Returns the kth smallest value in the array arr [1 ... n].
 * The input array is NOT altered.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * "Numerical Recipes: The Art of Scientific Computing" 2nd Edition
 * William H. Press, William T. Vetterling, Saul A. Teukolsky, Brian P. Flannery
 * page 343
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   22 May 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "nrutil.h"
#define M 64
#define BIG 1.0e+30
#define FREEALL free_vector(sel,1,M+2);free_lvector(isel,1,M+2);

float
selip (k, n, arr)

unsigned long k ;
unsigned long n ;
float arr [] ;

{
    void shell (unsigned long n, float a []) ;
    unsigned long i, j ;
    unsigned long jl, jm, ju ;
    unsigned long kk ;
    unsigned long mm ;
    unsigned long nlo, nxtmm ;
    unsigned long *isel ;
    float ahi, alo ;
    float sum ;
    float *sel ;

    if (k < 1 || k > n || n <= 0)  nrerror ("bad input to selip") ;
    isel = lvector (1, M+2) ;
    sel = vector (1, M+2) ;
    kk = k ;
    ahi = BIG ;
    alo = - BIG ;
    for ( ; ; )   /* Main iteration loop (until desired element is isolated). */
	{
	mm = nlo = 0 ;
	sum = 0.0 ;
	nxtmm = M + 1 ;
	for (i = 1; i <= n; i++)	/* Make a pass thru the entire array. */
	    {
	    if (arr [i] >= alo && arr [i] <= ahi)
		{	/* Consider only elements in the current brackets. */
		mm++ ;
		if (arr [i] == alo)  nlo++ ;/* In case of ties for low bracket*/

		/* Now use statistical procedure for selecting m in-range
		 * elements with equal probability, even without knowing
		 * in advance how many there are !
		 */

		if (mm < M)  sel [mm] = arr [i] ;
		else
		if (mm == nxtmm)
		    {
		    nxtmm = mm + mm / M ;
		    sel [1 + ((i + mm + kk) % M)] = arr [i] ; /* % operation
							       * provides a
							       * somewhat random
							       * number.
							       */
		    }
                sum += arr [i] ;
		}
	    }

        if (kk <= nlo)		/* Desired element is tied for lower bound;
				 * return it.
				 */
	    {
	    FREEALL
	    return (alo) ;
	    }
        else
	if (mm <= M)		/* All in-range elements were kept.
				 * So return answer by direct method.
				 */
	    {
	    shell (mm, sel) ;
	    ahi = sel [kk] ;
	    FREEALL
	    return (ahi) ;
	    }
        sel [M + 1] = sum / mm ;	/* Augment selected set by mean value
					 * (fixes degeneracies),
					 * and sort it.
					 */
	shell (M + 1, sel) ;
	sel [M + 2] = ahi ;
	for (j = 1; j <= M+2; j++)  isel [j] = 0 ;	/* Zero count array. */
	for (i = 1; i <= n; i++)	/* Make another pass through array. */
	    {
	    if (arr [i] >= alo && arr [i] <= ahi)
		{			/* For each in-range element ... */
		jl = 0 ;
		ju = M + 2 ;
		while (ju - jl > 1)	/* ... find its position among the 
					 *     select by bisection ...
					 */
		    {
		    jm = (ju + jl) / 2 ;
		    if (arr [i] >= sel [jm])
			jl = jm ;
		    else
			ju = jm ;
		    }
                isel [ju]++ ;		/* ... and incrment the counter. */
		}
	    }
        j = 1 ;				/* Now we can narrow the bounds to just
					 * one bin; that is, by a factor or
					 * order m.
					 */
	while (kk > isel [j])
	    {
	    alo = sel [j] ;
	    kk -= isel [j++] ;
	    }
        ahi = sel [j] ;
	}
}
