/*
 *  acos_header.h
 *
 *  Header structure for digital Prominence Monitor, LowK, and CHIP data records
 *  This header is contained in the first row of the images.
 *
 *  dPMon:
 *      1534 columns by 1030 rows.
 *	    206 blocks of 15340 bytes each
 *      cols: x subscript rapidly varying starting in the West geocentric.
 *      rows: y subscript, slowly varying starting in the South geocentric.
 *      All image data are shorts (possibly slightly negative)
 *
 *  LowK
 *      1534 columns by 1030 rows.
 *      206 blocks of 15340 bytes each
 *      cols: x subscript rapidly varying starting in the West geocentric.
 *      rows: y subscript, slowly varying starting in the South geocentric.
 *      Lowk data consist of four polarization states in quadrants 
 *      of the full image, each state being 767 x 515.
 *      Their orientations are the same.
 *      The top and bottom images are oppositely encoded 
 *      polarization states.  Images contain modulation states
 *      1 and 2 (left and right) or 3 and 4.
 *      To get stokes parameters.  Bias correct, gain correct, and
 *      register all 8 images (states 1,2,3 & 4, top and bottom ).
 *      I = sum of all 8
 *      Q = +1top -1bot +2top -2bot -3top +3bot -4top +4bot
 *      U = +1top -1bot -2top +2bot -3top +3bot +4top -4bot
 *      Coordinates are plane of the sky.  
 *      All image data are shorts (possibly slightly negative)
 *
 *  CHIP
 *      1534 columns by 1030 rows.
 *      206 blocks of 15340 bytes each
 *      cols: x subscript rapidly varying starting in the East geocentric.
 *      rows: y subscript, slowly varying starting in the North geocentric.
 *      All image data are shorts (possibly slightly negative)
 *
 *
 *  David Elmore
 *
 *  Thu Nov  2 08:25:08 MST 1995
 */

struct	acos_header
    {
    /* Telescope */
    int		Version;	/* IS_CHIP, IS_DPMON, IS_LOWK */

    /* Date and Time */
    int		Year;		/* Date and time UT */
    int		Month;
    int		Day;
    int		Hour;
    int		Minute;
    int		Sec;

    /* Image information */
    int		Xdim;		/* # of pixels in X */
    int		Ydim;		/* # of pixels in Y */
    int		Nsum;		/* # of exposures summed to form one */
    int		Exposure;	/* Exposure time in hundredths of sec*/
    int		Gain;		/* Camera controller gain factor */
    int		Odisk;		/* O disk diameter in arc-seconds */
    int		Type;		/* see acos.h for definitions */
    int		Wave;		/* Wavelength of observation */

    /* Ephemeris */
    float	Pangle;		/* P angle in degrees */
    float	Bangle;		/* B angle in degrees */
    float	Langle;		/* L angle in degrees */
    float	HA;			/* Hour angle in degrees */
    float	Elevation;	/* Solar elevation in degrees */
    float	Azimuth;	/* Solar azimuth in degrees */
    float	Amass;		/* Airmass */
    float	Radius;		/* Solar radius in arc-seconds 30*4*/

    /* Real-time image information */
    float	Xshift[16];	/* Amount of shift (arc-sec) applied to exposures */
						/* which were summed to make this image */
    float	Yshift[16];	/* Amount of shift (arc-sec) applied to exposures */
						/* which were summed to make this image */
    float	Diam[16];	/* Measured diameters (arc-sec) of exposures */
						/* which were summed to make this image */

    /* Instrument Housekeeping */
    int	Pos[32];		/* Positions of all 32 optical devs */
						/* see dev_defs.h for definitions */
    int	Drive[32];		/* Drive requests for all 32 opticsl devices */
    float	Analog[16];	/* The sixteen analog readings, interpreted */
    float	Lcvr[8];	/* The eight liquid crystal millivoltages */
    float	Mic[6];		/* The six motor micrometer readings */

    };

#ifdef ACOS
	struct	acos_header	*Image_header;
#else
	extern	struct	acos_header	*Image_header;
#endif
