/* acos3fits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert a binary MLSO/ACOS image to FITS format.
 *
 * ACOS : Advanced Coronal Observing System, includes these instruments :
 *        DPM: Digital Prominence Monitor
 *        LowK: Low K-corona Polarimeter
 *        CHIP: Chromospheric Helium-1 Imaging Photometer
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   9 November 1995
 * 29 Nov 1995 : Add WAVELEN.
 * 30 Nov 1995 : Add Housekeeping Extension.
 *  1 Dec 1995 : Add units to housekeeping items.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "acos.h"
#include "acoslib.ext"
#define MAXLIN 180		/* Maximum # lines allocated for FITS header. */
char fitbuf   [MAXLIN][80] ;	/* FITS header buffer. */
char outbuf   [84] ;

acostofits (namimg, namext, hduflag, pmode)

   char namimg [] ;		/* Name of primary file. */

   char namext [] ;		/* Name of extension file. */

   int hduflag ;		/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */

   int pmode ;			/* Print mode. */


{
   extern char *malloc () ;		/* Function to allocate memory. */
   extern unsigned char *rdacos () ;	/* Function to read ACOS image. */
   char *padbuf ;			/* Pointer to padding buffer. */
   unsigned char *acosbuf ;		/* MLSO ACOS file  array pointer. */
   unsigned char *imgbuf ;		/* MLSO ACOS image array pointer. */
   unsigned char *hkbuf ;		/* MLSO ACOS housekeeping pointer.*/
   unsigned char *cibuf, *cobuf ;	/* unsigned char image pointers. */

   unsigned char cmin, cmax ;		/* Data min/max values. */
   unsigned char cval ;			/* Data value. */


   static char kairmass  [12] = "AIRMASS " ;
   static char kazimuth  [12] = "AZIMUTH " ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcode_obs [12] = "CODE-OBS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kocc_disc [12] = "OCC-DISC" ;
   static char kdisp_dir [12] = "DISP-DIR" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kacosver  [12] = "ACOSVER " ;
   static char kelev     [12] = "ELEV    " ;
   static char kend      [12] = "END     " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kgain_det [12] = "GAIN-DET" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char khour_ang [12] = "HOUR-ANG" ;
   static char kinstrume [12] = "INSTRUME" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char knorthang [12] = "NORTHANG" ;
   static char kobject   [12] = "OBJECT  " ;
   static char koptdev1  [12] = "OPTDEV1 " ;
   static char koptdev2  [12] = "OPTDEV2 " ;
   static char koptdev3  [12] = "OPTDEV3 " ;
   static char koptdev4  [12] = "OPTDEV4 " ;
   static char koptdev5  [12] = "OPTDEV5 " ;
   static char koptdev6  [12] = "OPTDEV6 " ;
   static char koptdev7  [12] = "OPTDEV7 " ;
   static char koptdev8  [12] = "OPTDEV8 " ;
   static char koptpos1  [12] = "OPTPOS1 " ;
   static char koptpos2  [12] = "OPTPOS2 " ;
   static char koptpos3  [12] = "OPTPOS3 " ;
   static char koptpos4  [12] = "OPTPOS4 " ;
   static char koptpos5  [12] = "OPTPOS5 " ;
   static char koptpos6  [12] = "OPTPOS6 " ;
   static char koptpos7  [12] = "OPTPOS7 " ;
   static char koptpos8  [12] = "OPTPOS8 " ;
   static char korigin   [12] = "ORIGIN  " ;
   static char knimavg   [12] = "NIMAVG  " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char krsun     [12] = "RSUN    " ;
   static char ksolar_b0 [12] = "SOLAR_B0" ;
   static char ksolar_l0 [12] = "SOLAR_L0" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ktelescop [12] = "TELESCOP" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char kversion  [12] = "VERSION " ;
   static char kwavelen  [12] = "WAVELEN " ;
   static char kwavewid  [12] = "WAVEWID " ;

   static char atype1   [24] ;			/* Coord type axis 1. */
   static char atype2   [24] ;			/* Coord type axis 2. */
   static char bunit    [32] ;			/* Brightness unit. */
   static char ctime    [12] = "HH:MM:SS" ;	/* File creation time. */
   static char ctype1   [24] ;			/* Coord type axis 1. */
   static char ctype2   [24] ;			/* Coord type axis 2. */
   static char ctypenam [24] = "GEOCENTRIC";	/* Coordinate system name. */
   static char cunit1   [24] ;			/* Coord units axis 1. */
   static char cunit2   [24] ;			/* Coord units axis 2. */
   static char datcom   [72] ;			/* Data min/max comment. */
   static char cdate    [12] = "DD/MM/YY" ;	/* File creation date. */
   static char date_cal [12] = "DD/MM/YY" ;	/* Calibration date. */
   static char date_obs [12] = "DD/MM/YY" ;	/* Observation date. */
   static char eq       [4]  = "= " ;
   static char instrume [12] = "CCD" ;		/* Instrument name. */
   static char namfit   [80] ;			/* FITS file name. */
   static char object   [12] = "SUN" ;		/* Object. */
   static char origin   [12] = "HAO" ;		/* Origin. */
   static char pixcom   [48] ;			/* Data type comment. */
   static char refcom   [24] ;			/* Coord ref comment. */
   static char disp_dir [ 4] ;		/* Display direction: R/L, U/D, O/I. */
   static char telescop [12] = "ACOS" ;		/* Telescope name. */
   static char time_obs [12] = "HH:MM:SS" ;	/* Observation time. */
   static char type_obs [24] ;			/* Observation type. */
   static char version  [24] ;
   static char tscopnam [3][8] = {"DPM", "LOWK", "CHIP"} ;

   static char tscopcom [3][44] =
      {"DIGITAL PROMINENCE MONITOR",
       "LOW K-CORONA CORONAGRAPH",
       "CHROMOSPHERIC HELIUM-1 IMAGING PHOTOMETER",
      } ;
   static char instcom [3][24] =
      {"Kodak Megaplus 1.6 CCD",
       "Kodak Megaplus 1.6 CCD",
       "Kodak Megaplus 1.6 CCD"
      } ;
/*
 *   static char optnam [13][12] =
 *      {"FILTER-STAGE","O2-STAGE","CHIP-COVER","CHIP-DIFFUSER","CHIP-STROBE",
 *       "PMLK-STROBE","OCCULT-LIMITS","PMLK-COVER","POL-STAGE","FILTER-WHEEL",
 *       "POL-WHEEL","OBJ-WHEEL","OCC-WHEEL"} ;
 */
   static char optnam [13][12] =
      {"FILSTAGE","O2-STAGE","COVER","DIFFUSER","STROBE",
       "STROBE","OCC-LIM","COVER","POLSTAGE","FIL-WHL",
       "POL-WHL","OBJ-WHL","OCC-WHL"} ;

   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */

   short *sibuf, *sobuf ;		/* Short image pointers. */
   short smin, smax ;			/* Data min/max values. */
   short sval = 0 ;			/* Data value. */

   int bitpix ;				/* # bits per pixel. */
   int doy_obs ;			/* Observation day-of-year. */
   int gcount = 0 ;			/* gcount. */
   int iyear ;				/* year - 1900. */
   int i, ix, iy ;			/* Indices. */
   int iynum ;				/* # lines in FITS header. */
   int nhdu ;				/* # HDU blocks to write. */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* Integer value. */
   int kx, ky ;				/* Image indices. */
   int ndim   = 0 ;			/* # dimensions for image (2=XY). */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyfits = 2880 ;			/* # bytes per FITS logical record. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int pcount  = 0 ;			/* pcount. */
   int nrec ;				/* # fits records for image. */
   int nelwr   = 0 ;
   int year_obs ;			/* Observation year. */

   float *fibuf, *fobuf ;		/* Float image pointers. */

   float northdeg =  90.0 ;		/* Solar north w.r.t. +X axis. */
   float bscale   =   1.0 ;		/* Brightness intensity scale factor. */
   float bzero    =   0.0 ;		/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float crpix1, crpix2 ;		/* Reference pixel location. */
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crradius = 400.84 ;		/* Solar radius [pixels] = 960/2.395. */
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin, dmax ;			/* Data min/max values. */
   float swver = 1.00 ;			/* ACOS FITS s/w version number. */
   float fmin, fmax ;			/* Data min/max values. */
   float fval ;				/* Data value. */
   float vmin, vmax ;			/* Data min/max values. */
   float wmin =    0.0 ;		/* Display min value. */
   float wmax = 4000.0 ;		/* Display max value. */
   FILE *FF ;				/* FITS file ID. */

   sprintf (namfit, "%s.fts", namimg) ;

   if (pmode)
      {
      fprintf (stdout, "acostofits> namimg: %s  namext: %s   hduflag: %d\n",
	       namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- acostofits> %s -> %s -------\n\n",
	       namimg, namfit) ;
      }

   /*** Read MLSO/ACOS image/data from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2)
      acosbuf = rdacos (namimg, pmode) ;/* Read ACOS dataset into memory. */
   else
      acosbuf = rdacos (namext, pmode) ;/* Read ACOS dataset into memory. */

   nx = ACOSI->xdim ;			/* X dimension. */
   ny = ACOSI->ydim ;			/* Y dimension. */
   npximg = nx * ny ;			/* # pixels in image. */
   nbyimg = npximg * ACOSI->nbypx ;	/* # bytes in image. */
   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			nx, ny, ACOSI->nbypx, npximg, nbyimg) ;

   /*** Image origin is at upper right (effectively rotated 180 degrees). ***/
   /*** Flip image about both x & y axes to put image origin at lower left. ***/

   imgbuf = (unsigned char *) malloc (nbyimg) ;

   switch (ACOSI->nbypx)
      {
      case 1:
	 cobuf = (unsigned char *) imgbuf ;
	 cibuf = (unsigned char *) acosbuf ;
	 cmin  = 255 ;
	 cmax  =   0 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       kx = nx - ix - 1 ;
	       cval =
               *(cobuf + ix + iy*nx) = *(cibuf + kx + ky*nx) ;
               if (cval < cmin)  cmin = cval ;
               if (cval > cmax)  cmax = cval ;
	       }
	    }
         ACOSI->dmin = (int) cmin ;
         ACOSI->dmax = (int) cmax ;
	 vmin = (float) cmin ;
	 vmax = (float) cmax ;
         break ;

      case 2:
	 sobuf = (short *) imgbuf ;
	 sibuf = (short *) acosbuf ;
	 smin  =  32767 ;
	 smax  = -32767 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       kx = nx - ix - 1 ;
	       sval =
               *(sobuf + ix + iy*nx) = *(sibuf + kx + ky*nx) ;
               if (sval < smin)  smin = sval ;
               if (sval > smax)  smax = sval ;
	       }
	    }
         ACOSI->dmin = (int) smin ;
         ACOSI->dmax = (int) smax ;
	 vmin = (float) smin ;
	 vmax = (float) smax ;
         break ;

      case 4:
	 fobuf = (float *) imgbuf ;
	 fibuf = (float *) acosbuf ;
	 fmin  =  1.0e+30 ;
	 fmax  = -1.0e+30 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       kx = nx - ix - 1 ;
               *(fobuf + ix + iy*nx) = *(fibuf + kx + ky*nx) ;
               if (fval < fmin)  fmin = fval ;
               if (fval > fmax)  fmax = fval ;
	       }
	    }
         ACOSI->dmin = (int) fmin ;
         ACOSI->dmax = (int) fmax ;
	 vmin = fmin ;
	 vmax = fmax ;
         break ;

      default:
	 fprintf (stderr, "Invalid ACOS image . . . nbypx: %d\n", ACOSI->nbypx) ;
	 exit (-1) ;
      }

   iyear = ACOSI->year ;
   if (ACOSI->year > 1900)  iyear = ACOSI->year - 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, ACOSI->month, ACOSI->day) ;

   sprintf (date_obs, "%02d/%02d/%02d", ACOSI->day, ACOSI->month, iyear) ;
   sprintf (time_obs, "%02d:%02d:%02d", ACOSI->hour, ACOSI->minute, ACOSI->second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   sprintf (cdate, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (ctime, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   iyear = ACOSI->cal_year ;
   if (ACOSI->cal_year > 1900)  iyear = ACOSI->cal_year - 1900 ;
   sprintf (date_cal, "%02d/%02d/%02d", ACOSI->cal_day, ACOSI->cal_month, iyear) ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (ACOSI->xdim > 0)
      ndim++ ;
   if (ACOSI->ydim > 1)
      ndim++ ;

   /*
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Set parameters.
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;
   strcpy (origin,   "HAO") ;
   sprintf (telescop, "ACOS/%s", &tscopnam [ACOSI->ttype][0]) ;
   strcpy (instrume, "CCD") ;
   strcpy (datcom,   "CCD") ;
   strcpy (atype1,   "X [EAST->WEST ]") ;
   strcpy (atype2,   "Y [SOUTH->NORTH]") ;
   strcpy (ctypenam, "GEOCENTRIC") ;
   strcpy (cunit1,   "ARCSEC") ;
   strcpy (cunit2,   "ARCSEC") ;
   strcpy (refcom,   "SUN CENTER") ;
   strcpy (disp_dir,  "RU") ;		/* Display direction: right, up. */

   crpix1 = (float) ((nx - 1) / 2) + 0.5 ;	/* Sun center (X pixels). */
   crpix2 = (float) ((ny - 1) / 2) + 0.5 ;	/* Sun center (Y pixels). */
   crval1 = 0.0 ;				/* Sun center (X arcsec). */
   crval2 = 0.0 ;				/* Sun center (Y arcsec). */
   cdelt1 = 2.395 ;				/* arcsec/pixel. */
   cdelt2 = 2.395 ;				/* arcsec/pixel. */
   crradius = ACOSI->rsun / cdelt1 ;		/* Solar radius (pixels). */

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * ACOS data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   /*** Set object.
    *** Set coordinate type(s).
    *** Set bitpix.
    *** Set display scaling.
    ***/

   switch (ACOSI->dtype)
      {
      case  0: /* DPM other */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_OTHER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  1: /* DPM disk */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_DISK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  2: /* DPM limb */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_LIMB") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  3: /* DPM disc_diffuser */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_DISC_DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  4: /* DPM limb_diffuser */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_LIMB_DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  5: /* DPM disk dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_DISK_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  6: /* DPM limb dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_LIMB_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 15: /* DPM last */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DPM_LAST") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 16: /* LowK I */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_I") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 17: /* LowK Q */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_Q") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 18: /* LowK U */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_U") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 19: /* LowK I Gain */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_I_GAIN") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 20: /* LowK Q Gain */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_Q_GAIN") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 21: /* LowK U Gain */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_U_GAIN") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 22: /* LowK I Dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_I_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 23: /* LowK Q Dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_Q_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 24: /* LowK U Dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_U_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 25: /* LowK I Pol */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_I_POL") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 26: /* LowK Q Pol */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_Q_POL") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 27: /* LowK U Pol */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_U_POL") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 28: /* LowK other */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_OTHER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 31: /* LowK last */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "LOWK_LAST") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 32: /* chip_major */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "CHIP_MAJOR") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 33: /* chip_minor */
               strcpy (object,   "SUN") ;   
               strcpy (type_obs, "CHIP_MINOR") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 1500.0 ;
               break ;
      case 34: /* chip_continuum */
               strcpy (object,   "SUN") ;   
               strcpy (type_obs, "CHIP_CONTINUUM") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 35: /* chip_water */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "CHIP_WATER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 36: /* chip_major_dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "CHIP_MAJOR_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case 37: /* chip_minor_dark */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "CHIP_MINOR_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 38: /* chip_continuum_dark */
               strcpy (object,   "SUN") ;   
               strcpy (type_obs, "CHIP_CONTINUUM_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "Y") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 39: /* chip_water_dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "CHIP_WATER_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 40: /* chip_major_diffuser */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "CHIP_MAJOR_DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 41: /* chip_minor_diffuser */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "CHIP_MINOR_DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 42: /* chip_continuum_diffuser */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "CHIP_CONTINUUM_DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 43: /* chip_water_diffuser */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "CHIP_WATER_DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case 47: /* chip_last_diffuser */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "CHIP_LAST") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      default:
               strcpy (object,   "UNKNOWN") ;   
	       strcpy (type_obs, "UNKNOWN") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
	       fprintf (stderr, "Invalid dtype: %d\n", ACOSI->dtype) ;
	       exit (-1) ;
      }

   /*** Create FITS file. ***/

   if (hduflag == 3)				/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- PRIMARY HDU ---\n") ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   sprintf (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: MLSO/ACOS") ;
   else
   sprintf (&fitbuf [iy++][0],  "XTENSION= 'ACOS    '           / MLSO/ACOS %s EXTENSION", type_obs) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, ny, atype2) ;
   else
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;

   /*** Extension information. ***/

/*
 *   if (hduflag == 1)
 *      {
 *      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
 *      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
 *      }
 *   else
 *   if (hduflag == 2)
 *      {
 *      sprintf (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
 *      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
 *      }
 *   else
 */

   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;
      }
   else
      {
      sprintf (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      }

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ Mauna Loa Solar Observatory (MLSO)") ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ High Altitude Observatory (HAO), DIV. OF NCAR") ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-9s'",     ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ MLSO %s", &tscopcom [ACOSI->ttype][0]) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kinstrume, eq, instrume) ;
   sprintf (&fitbuf [iy++][31], "/ %s", &instcom [ACOSI->ttype][0]) ;


   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
				doy_obs, year_obs) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS [UT]") ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s %s", atype1, ctypenam) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [INDEX]",
			      kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1) ;

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s %s", atype2, ctypenam) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [INDEX]",
			           kcrpix2, eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2  , eq, cdelt2, atype2, cunit2) ;
      }
   else
      {
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      }

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM NAME") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f", kcrradius, eq, crradius) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR RADIUS [PIXELS]") ;

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR RADIUS [ARCSEC]",
			        krsun    , eq, ACOSI->rsun) ;

   if (ACOSI->ttype == IS_DPMON || ACOSI->ttype == IS_LOWK)
      {
      sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / OCCULTING DISC DIAMETER [ARCSECONDS]",
				   kocc_disc, eq, ACOSI->odisk) ;
      }
   else
   if (ACOSI->ttype == IS_CHIP)
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
			      ksolar_p0, eq, ACOSI->pangle) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
			      ksolar_b0, eq, ACOSI->bangle) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE",
			      ksolar_l0, eq, ACOSI->langle) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR HOUR ANGLE [DEGREES] (W.R.T.ZENITH E- W+)",
			      khour_ang, eq, ACOSI->hangle) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR ELEVATION [DEGREES]",
			      kelev    , eq, ACOSI->elev) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SOLAR AZIMUTH [DEGREES]",
			      kazimuth , eq, ACOSI->azimuth) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / AIRMASS [ATMOSPHERES]",
			      kairmass , eq, ACOSI->airmass) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
			      kcode_obs, eq, ACOSI->dtype) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [NUMERICAL CODE] ") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
			      kwavelen,  eq, ACOSI->wavelen) ;
   sprintf (&fitbuf [iy++][31], "/ CENTRAL WAVE LENGTH OF OBSERVATION [NM]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
			      kwavewid,  eq, ACOSI->wavewid) ;
   sprintf (&fitbuf [iy++][31], "/ BANDPASS OF FILTER [NM]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
			      kgain_det, eq, ACOSI->gain) ;
   sprintf (&fitbuf [iy++][31], "/ DETECTOR GAIN FACTOR") ;

/* -------------------------------------------------------------------------- */

   /*** Number of Images Averaged. ***/

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20d / NUMBER OF IMAGES AVERAGED",
			      knimavg  , eq, ACOSI->navg) ;
   /*** Exposure Duration. ***/

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / EXPOSURE DURATION [SECONDS]",
			      kexpdur  , eq, ACOSI->expdur) ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'",     kbunit, eq, bunit) ;
   sprintf (&fitbuf [iy++][31], "/ MAGNITUDE UNITS") ;

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / PHYSICAL = DATA * BSCALE + BZERO",
			       kbscale, eq, ACOSI->bscale) ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kbzero , eq, ACOSI->bzero ) ;
   sprintf (&fitbuf [iy++][31], "/ PHOTOMETRIC SCALING OFFSET [SEE BSCALE]") ;

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / MINIMUM DATA VALUE [%s]",
			       kdatamin , eq, vmin, datcom) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / MAXIMUM DATA VALUE [%s]",
			       kdatamax , eq, vmax, datcom) ;

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / MIN VALUE FOR DISPLAY [UNSCALED]",
			      kdispmin , eq, wmin) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / MAX VALUE FOR DISPLAY [UNSCALED]",
			      kdispmax , eq, wmax) ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", kdisp_dir,  eq, disp_dir) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION (AXIS 1 2 3): R/L  U/D  O/I") ;

/* -------------------------------------------------------------------------- */
/* Optical Devices                                                            */
/* -------------------------------------------------------------------------- */

/*
   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   if (ACOSI->ttype == IS_DPMON || ACOSI->ttype == IS_LOWK)
   {
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [0][0],  eq, &ACOSI->optpos [0][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK FILTER STAGE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [1][0],  eq, &ACOSI->optpos [1][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK O2 STAGE") ;
   }

   if (ACOSI->ttype == IS_CHIP)
   {
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [2][0],  eq, &ACOSI->optpos [2][0]) ;
   sprintf (&fitbuf [iy++][31],"/ CHIP COVER") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [3][0],  eq, &ACOSI->optpos [3][0]) ;
   sprintf (&fitbuf [iy++][31],"/ CHIP DIFFUSER") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [4][0],  eq, &ACOSI->optpos [4][0]) ;
   sprintf (&fitbuf [iy++][31],"/ CHIP STROBE") ;
   }

   if (ACOSI->ttype == IS_DPMON || ACOSI->ttype == IS_LOWK)
   {
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [5][0],  eq, &ACOSI->optpos [5][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK STROBE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [6][0],  eq, &ACOSI->optpos [6][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK OCCULTING LIMITS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [7][0],  eq, &ACOSI->optpos [7][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK COVER") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [8][0],  eq, &ACOSI->optpos [8][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK POLARIZER STAGE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [9][0],  eq, &ACOSI->optpos [9][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK FILTER WHEEL") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [10][0], eq, &ACOSI->optpos [10][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK POLARIZER WHEEL") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [11][0], eq, &ACOSI->optpos [11][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK OBJECTIVE WHEEL") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", &ACOSI->optdev [12][0], eq, &ACOSI->optpos [12][0]) ;
   sprintf (&fitbuf [iy++][31],"/ DPM/LOWK OCCULTING WHEEL") ;
   }
 */

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   sprintf (version, "ACOS v%4.2f", swver) ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", kversion, eq, version) ;
   sprintf (&fitbuf [iy++][31],"/ ACOS FITS S/W VERSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", kdate,     eq, cdate) ;
   sprintf (&fitbuf [iy++][31], "/ FITS FILE GENERATION DATE: DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'",     kdate_cal, eq, date_cal) ;
   sprintf (&fitbuf [iy++][31],"/ ACOS CALIBRATION DATE: DD/MM/YY [UT]") ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   sprintf (&fitbuf [iy++][0],  "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;
   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   THE ADVANCED CORONAL OBSERVING SYSTEM (ACOS) CONSISTS OF 3 SOLAR") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INSTRUMENTS (ALL LOCATED AT MLSO):") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   DIGITAL PROMINENCE MONITOR (DPM), LOW K-CORONA POLARIMETER (LOWK),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   & CHROMOSPHERIC HELIUM-1 IMAGING PHOTOMETER (CHIP).") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   EACH ACOS INSTRUMENT USES A KODAK MEGAPLUS 1.6 CCD DETECTOR.") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.") ;
   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (imgbuf, ACOSI->nbypx, npximg, FF) ;
   free (acosbuf) ;
   free (imgbuf) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FF) ;
      free (padbuf) ;
      }
   if (pmode)
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg) ;

   /* Add real-time & housekeeping info in extension. */

   acosext (FF, type_obs, date_obs, time_obs, pmode) ;

   close (FF) ;
   return (0) ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* acosext: FITS extension containing real-time & housekeeping information.
 */

acosext (FD, type_obs, date_obs, time_obs, pmode)

FILE *FD ;
char *type_obs ;
char *date_obs ;
char *time_obs ;
int pmode ;

{
   static char kanalog   [12] = "ANALOG" ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kdiam     [12] = "DIAM" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char klcvr     [12] = "LCVR" ;
   static char kmic      [12] = "MIC" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char koptdev   [12] = "OPTDEV" ;
   static char koptdrv   [12] = "OPTDRV" ;
   static char koptpos   [12] = "OPTPOS" ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char kxshift   [12] = "XSHIFT" ;
   static char kyshift   [12] = "YSHIFT" ;

   static char pixcom    [48] ;			/* Data type comment. */
   static char eq        [4]  = "= " ;

/*
 *   static char optkey [13][12] =
 *      {"FILSTAGE","O2-STAGE","CHIPCOVR","DIFFUSER","CHIPSTRB",
 *       "PMLKSTRB","OCC-LIM","PMLKCOVR","POLSTAGE","FIL-WHL",
 *       "POL-WHL","OBJ-WHL","OCC-WHL"} ;
 */

   static char optnam [13][16] =
      {
      "FILTER-STAGE",   "O2-STAGE",       "CHIP-COVER",    "CHIP-DIFFUSER",
      "CHIP-STROBE",    "DPM/LOWK-STROBE","OCCULT-LIMITS", "DPM/LOWK-COVER",
      "POLARIZER-STAGE","FILTER-WHEEL",   "POLARIZER-WHEEL","OBJECTIVE-WHEEL",
      "OCCULTING-WHEEL"
      } ;

   static char analogname [16][28] =
      {
      "DPM/LowK Plate Temp    (C)",
      "DPM/LowK Air Temp      (C)",
      "DPM/LowK Filter Temp   (C)",
      "DPM/LowK Spare Temp    (C)",
      "LowK PBS Temp          (C)",
      "LowK Modulator Temp    (C)",
      "Sky Transmission     (ARB)",
      "Spare Voltage          (V)",
      "CHIP Filter Temp       (C)",
      "CHIP Plate Temp        (C)",
      "CHIP Air Temp          (C)",
      "CHIP Spare Temp        (C)",
      "Guider X Error       (ARB)",
      "Guider Y Error       (ARB)",
      "O1 X Error           (ARB)",
      "O1 Y Error           (ARB)"
      } ;

   static char lcvrname [8][28] =
      {
      "CHIP Filter Stage 4    (V)",
      "CHIP Filter Stage 3    (V)",
      "CHIP Filter Stage 2    (V)",
      "CHIP Filter Stage 1    (V)",
      "Spare LCVR 3           (V)",
      "Spare LCVR 2           (V)",
      "Spare LCVR 1           (V)",
      "LowK Modulator Stage 1 (V)"
      } ;

   static char micname [6][28] =
      {
      "CHIP Camera Z    (MICRONS)",
      "CHIP Camera X    (MICRONS)",
      "CHIP Camera Y    (MICRONS)",
      "DPM/LowK C1 X    (MICRONS)",
      "DPM/LowK C1 Y    (MICRONS)",
      "DPM/LowK C1 Z    (MICRONS)"
      } ;

   int gcount = 0 ;				/* gcount. */
   int pcount  = 0 ;				/* pcount. */
   int nhdu = 0 ;				/* # FITS header blocks. */
   int i ;
   int ix = 0, iy = 0 ;
   int iyear ;
   int iynum = 0 ;
   int ival = 0 ;
   int bitpix = 16 ;
   int ndim = 0 ;
   int nelwr = 0 ;
   int year_obs ;
   int doy_obs ;

   /*** Determine day-of-year. ***/

   iyear = ACOSI->year ;
   if (ACOSI->year > 1900)  iyear = ACOSI->year - 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, ACOSI->month, ACOSI->day) ;

   strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   iy = 0 ;

   sprintf (&fitbuf [iy++][0],  "XTENSION= 'ACOS_HSKP'          / MLSO/ACOS HOUSEKEEPING EXTENSION") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
				doy_obs, year_obs) ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS [UT]") ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-6s%02d%2s%20.4f / XSHIFT [%2d] [pixels]",
				  kxshift, i, eq, ACOSH->Xshift [i], i) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-6s%02d%2s%20.4f / YSHIFT [%2d] [pixels]",
				  kyshift, i, eq, ACOSH->Yshift [i], i) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-4s%02d  %2s%20.4f / DIAM [%2d] [pixels]",
				  kdiam, i, eq, ACOSH->Diam [i], i) ;

   /*** Optical devices & positions. ***/

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 13; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptdev, i, eq,
				&ACOSI->optdev [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %s NAME", &optnam [i][0]) ;
   }

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 13; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptpos, i, eq,
				&ACOSI->optpos [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %s POSITION", &optnam [i][0]) ;
   }

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 13; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptdrv, i, eq,
				&ACOSI->optdrv [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %s DRIVE", &optnam [i][0]) ;
   }

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-6s%02d%2s%20.4f / %s", kanalog, i, eq,
                                ACOSH->Analog [i], &analogname [i][0]) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 8; i++)
   sprintf (&fitbuf [iy++][0],  "%-4s%02d  %2s%20.4f / %s", klcvr, i, eq,
				ACOSH->Lcvr [i], &lcvrname [i][0]) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   for (i = 0; i < 6; i++)
   sprintf (&fitbuf [iy++][0],  "%-3s%02d   %2s%20.4f / %s", kmic, i, eq, 
				ACOSH->Mic [i], &micname [i][0]) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*2880, FD) ;

   return (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

month_day (year, doy, pmonth, pday)	/* Set month & day from day-of-year. */

int year, doy, *pmonth, *pday ;

{
   static int day_tab [2][13] =
      {
      {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      } ;
   int i, leap ;

   leap = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0) ;

   for (i = 1 ; doy > day_tab [leap][i] ; i++)
      doy -= day_tab [leap][i] ;

   *pmonth = i ;
   *pday = doy ;

   return (0) ;
}
