/* limb_dark.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute limb darkening fit for a solar disk image.
 *------------------------------------------------------------------------------
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction, scan time are obtained from the extension HDU.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   21 June 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/riker/d/sw/fits/cfitsio/fitsio.h"

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{
    int  usage () ;
    void printerror () ;

    char *w ;				/* pointer to command line arguments */
    static char comment   [80] ;
    static char cdatamin  [80] ;
    static char cdatamax  [80] ;
    static char cdispmin  [80] ;
    static char cdispmax  [80] ;
    static char cnaxis1   [80] ;
    static char cnaxis2   [80] ;
    static char cobject   [80] ;
    static char ccrpix1   [80] ;
    static char ccrpix2   [80] ;
    static char ccrota1   [80] ;
    static char ccrota2   [80] ;
    static char ccoordnam [80] ;
    static char code_obs   [8] ;
    static char code_obs1  [8] ;
    static char code_obs2  [8] ;
    static char code_obs3  [8] ;
    static char code_obs4  [8] ;
    static char code_obs5  [8] ;
    static char code_obs6  [8] ;
    static char code_obs7  [8] ;
    static char lcode_obs  [8] ;
    static char coordnam  [24] ;
    static char date_obs  [24] ;		/* Observation date. */
    static char file      [80] ;
    static char fileldf   [80] ;
    static char filerad   [80] ;
    static char nampgm    [24] ;
    static char object    [24] ;
    static char origin    [24] ;
    static char sdir2     [4] = "ru" ;		/* Scan direction: rd or ru. */
    static char telescop  [24] ;
    static char instrume  [24] ;
    static char type_obs  [24] ;
    static char time_obs  [24] ;		/* Observation time. */

    static int  extbuf [168] ;			/* FITS extension data. */
    static long naxes    [2] ;
    static long npts [20][3] ;			/* dim = [3][niter] */

    static float coef [20][8] ;			/* coefficients. */
    static float ex   [] = {1, 3} ;
    static float rms  [20] ;			/* dimension = niter */

    unsigned char *imgc ;
    int anynull ;
    int bitpix = 16 ;
    int dimcoef = 8 ;
    int hdutype = 0 ;
    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int ival ;
    int i, ix, iy ;
    int m1, m2 ;
    int morekeys = 0 ;		/* Additional keywords to be reserved. */
    int naxis = 2 ;
    int nbypix = 2 ;
    int nbyflt = 4 ;
    int ncode = 0 ;
    int nelimg ;
    int nelext = 168 ;
    int nex = 2 ;			/* # elements in ex []. */
    int niter = 20 ;			/* # iterations for flmbiter.f */
    int status = 0 ;
    int xdim, ydim ;

    long fpixel = 1 ;
    long npix ;

    float *imgbuf ;			/* Pointer to image array. */
    float *imgldf ;			/* Pointer to limb-dark fit image. */

    short *img1, *img2, *img3, *img4, *img5, *img6, *img7 ;

    float crota1, crota2 ;		/* Coordinate rotation angles. */
    float crradius ;			/* Solar Radius (pixels). */
    float crpix1, crpix2 ;		/* Sun center (pixels). */
    float datamin, datamax ;
    float dispmin, dispmax ;
    float fmin, fmax ;
    float nullval = 0 ;			/* Don't check for null values in img.*/
    float radius ;
    float radius_lim ;
    float rms_hi ;
    float rms_lo ;
    float solar_p0 ;			/* P-angle. */
    float xcen, ycen ;
    float xshift, yshift ;		/* x/y shift for cropping. */
    float wavelen = 0.0 ;

    float *fbuf ;
    float *fit ;
    float *weight ;

    fitsfile *fp, *fpldf ;

    if (argc <= 1) { usage (argv [0]) ; exit (-1) ; }

    strcpy (nampgm, argv [0]) ;

    while (--argc)
	{
	strcpy (file, argv [argc]) ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fp, file, READONLY, &status) )
	    printerror ( status );

	/*** Primary FITS header. ***/

	if (fits_read_key (fp, TINT, "NAXIS1", &xdim,
	    cnaxis1, &status))  printerror (status) ;

	if (fits_read_key (fp, TINT, "NAXIS2", &ydim,
	    cnaxis2, &status))  printerror (status) ;

	if (fits_read_key (fp, TINT, "BITPIX", &bitpix,
	    comment, &status))  printerror (status) ;
	nbypix = bitpix / 8 ;

	if (fits_read_key (fp, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	printf ("iyear: %d  imonth: %d  iday: %d\n", iyear, imonth, iday) ;

	if (fits_read_key (fp, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	printf ("ihour: %d  iminute: %d  isecond: %d\n",
	    ihour, iminute, isecond) ;

	if (fits_read_key (fp, TSTRING, "ORIGIN", origin,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "TELESCOP", telescop,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "INSTRUME", instrume,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "OBJECT", object,
	    cobject, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "TYPE-OBS", type_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "CODE-OBS", code_obs,
	    comment, &status))  printerror (status) ;

	ncode = strlen (code_obs) ;
	if (ncode > 7)  ncode = 7 ;
	for (i = 0; i < ncode; i++)
	    {
	    ival = code_obs [i] ;
	    lcode_obs [i] = (char) tolower (ival) ;
	    }

	if (fits_read_key (fp, TFLOAT, "CRPIX1", &crpix1,
	    ccrpix1, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CROTA1", &crota1,
	    ccrota1, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRPIX2", &crpix2,
	    ccrpix2, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CROTA2", &crota2,
	    ccrota2, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "WAVELEN", &wavelen,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "COORDNAM", coordnam,
	    ccoordnam, &status))  printerror (status) ;

	fprintf (stdout, "file: %s  code_obs: %s  wavelen: %8.2f\n",
		 file, code_obs, wavelen) ;

	if (fits_read_key (fp, TFLOAT, "DATAMIN", &datamin,
	    cdatamin, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "DATAMAX", &datamax,
	    cdatamax, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "DISPMIN", &dispmin,
	    cdispmin, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "DISPMAX", &dispmax,
	    cdispmax, &status))  printerror (status) ;

	/* Read FITS image data. */

	/* Note that even though the FITS images contains unsigned integer */
	/* pixel values (or more accurately, signed integer pixels with    */
	/* a bias of 32768),  this routine is reading the values into a    */
	/* float array.   Cfitsio automatically performs the datatype      */
	/* conversion in cases like this.                                  */

	if (crpix1 >= 0.0)
	    xcen = crpix1 ;
	else
	    xcen = (xdim - 1) * 0.5 ;

	if (crpix2 >= 0.0)
	    ycen = crpix2 ;
	else
	    ycen = (ydim - 1) * 0.5 ;


	npix     = xdim * ydim ;
	imgbuf   = (float *) malloc (xdim * ydim * sizeof (float)) ;
	nullval  = 0 ;		/* Don't check for null values in the image. */

	if ( fits_read_img (fp, TFLOAT, fpixel, npix, &nullval, imgbuf,
			    &anynull, &status) )
	    printerror ( status ) ;

	fmin =  1.0e+30 ;
	fmax = -1.0e+30 ;
	for (iy = 0; iy < ydim; iy++)
	   for (ix = 0; ix < xdim; ix++)
	       {
	       if (*(imgbuf + ix + iy*xdim) < fmin)
		   fmin = *(imgbuf + ix + iy*xdim) ;
	       if (*(imgbuf + ix + iy*xdim) > fmax)
		   fmax = *(imgbuf + ix + iy*xdim) ;
	       }
	fprintf (stdout, "fmin/fmax: %7.2f  %7.2f\n", fmin, fmax) ;

	/*** Compute Limb darkening fit (Peter Bandurian's Fortran code). ***/

	fit    = (float *) malloc (xdim * ydim * 2 * sizeof (float)) ;
	weight = (float *) malloc (xdim * ydim * sizeof (float)) ;
	radius_lim = crradius - 4.0 ;

	switch (code_obs [0])
	    {
	    case 'D':
	    case 'd':  rms_lo = -1.0  ;  rms_hi = 2.0 ;  break ;
	    case 'C':
	    case 'c':  rms_lo = -1.0  ;  rms_hi = 2.0 ;  break ;
	    case 'B':
	    case 'b':  rms_lo = -0.25 ;  rms_hi = 3.0 ;  break ;
	    case 'H':
	    case 'h':  rms_lo = -0.25 ;  rms_hi = 3.0 ;  break ;
	    case 'R':
	    case 'r':  rms_lo = -0.25 ;  rms_hi = 3.0 ;  break ;
	    case 'S':
	    case 's':  rms_lo = -1.0  ;  rms_hi = 2.0 ;  break ;
	    case 'T':
	    case 't':  rms_lo = -1.0  ;  rms_hi = 2.0 ;  break ;
	    default :  rms_lo = -1.0  ;  rms_hi = 2.0 ;
	    }

        m1 = 2 ;
	m2 = 2 ;

/*
	flmbiter_ (&xcen, &ycen, &crradius, &radius_lim, imgbuf, &xdim, &ydim,
		   weight, &m1, &m2, ex, &nex, fit, &niter,
		   rms, &rms_lo, &rms_hi, coef, &dimcoef, npts) ;
 */

	flmbext_ (&xcen, &ycen, &crradius, &radius_lim, imgbuf, &xdim, &ydim,
		   weight, &m1, &m2, ex, &nex, fit, &niter,
		   rms, &rms_lo, &rms_hi, coef, &dimcoef, npts) ;

	imgldf = fit ;

	fmin =  1.0e+30 ;
	fmax = -1.0e+30 ;
	for (iy = 0; iy < ydim; iy++)
	   for (ix = 0; ix < xdim; ix++)
	       {
	       if (*(fit + ix + iy*xdim) < fmin) fmin = *(fit + ix + iy*xdim) ;
	       if (*(fit + ix + iy*xdim) > fmax) fmax = *(fit + ix + iy*xdim) ;
	       }
	fprintf (stdout, "fmin/fmax: %7.2f  %7.2f\n", fmin, fmax) ;

	/*** Create output file name. ***/

	sprintf (fileldf, "%04d%02d%02d.%02d%02d%02d.chp.%s.ldf.fts",
		 iyear, imonth, iday, ihour, iminute, isecond, lcode_obs) ;

	fprintf (stdout, "fileldf: %s\n", fileldf) ;

	/*** Create output file. ***/

	if ( fits_create_template (&fpldf, fileldf, file, &status) )
	    printerror ( status ) ;

	/*** Write L-D fit image array into fits data array. ***/

	nelimg = xdim * ydim ;
	if ( fits_write_img (fpldf, TFLOAT, fpixel, nelimg, imgldf, &status) )
	    printerror ( status ) ;

	/*** Resize image (updates naxis1, naxis2, bitpix). ***/

	naxes [0] = xdim ;
	naxes [1] = ydim ;
	if (fits_resize_img (fpldf, bitpix, naxis, naxes, &status) )
	    printerror ( status ) ;

	/*** Update sun center information. ***/

	strcpy (object, "LIMB DARK FIT") ;
        if ( fits_update_key (fpldf, TSTRING, "OBJECT", &object, cobject,
	     &status)) printerror ( status ) ;

	datamin = fmin ;
        if ( fits_update_key (fpldf, TFLOAT, "DATAMIN", &datamin, cdatamin,
	     &status)) printerror ( status ) ;

	datamax = fmax ;
        if ( fits_update_key (fpldf, TFLOAT, "DATAMAX", &datamax, cdatamax,
	     &status)) printerror ( status ) ;

	dispmin = fmin ;
        if ( fits_update_key (fpldf, TFLOAT, "DISPMIN", &dispmin, cdispmin,
	     &status)) printerror ( status ) ;

	dispmax = fmax ;
        if ( fits_update_key (fpldf, TFLOAT, "DISPMAX", &dispmax, cdispmax,
	     &status)) printerror ( status ) ;

	if (fits_close_file (fpldf, &status))  printerror ( status ) ;

	/*** Close FITS file. ***/

	if ( fits_close_file (fp, &status) )  printerror (status) ;
	status = 0 ;
        }

   
   exit (0) ;
}

/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */

/* usage (prog_name)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int
usage (prog_name)

char *prog_name ;
{
    static char help [84] ;

    sprintf (help, "%s image1 image2 image3 image4 image5 image6 image7",
	     prog_name) ;
    fprintf (stderr, "%s\n", help) ;
    fprintf (stderr, " Example:  chipcal  dsr.fts csr.fts bsr.fts hsr.fts\n") ;
    fprintf (stderr, "                    rsr.fts ssr.fts tsr.fts\n") ;

    return (0) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

