/* acos_name.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create ACOS FITS output file name.
 * Use SOHO filename format:   OOOO_TTTTR_WS_YYYYMMDD_HHMMSS.fts
 * where OOOO  = Origin
 *       TTTT  = Telescope: DPMx, LOWK, CHIP
 *       R     = Reduction code: x,r,c,l,d,m
 *       W     = Wavelength code: x,h,c,m,w,a,b,r,k,s
 *       S     = Source code: x,s,l,o,d,b,c,f
 *
 *       YYYY  = year
 *       MM    = month
 *       DD    = day
 *
 *       HH    = hour
 *       MM    = minute
 *       SS    = second
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   13 February 1996 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/cordyn/mlsofits/mechanism_defs.h"
#include "/home/cordyn/mlsofits/lib/acos/acos.h"
#include "/home/cordyn/mlsofits/lib/acos/acos_info.h"
#include "/home/cordyn/mlsofits/lib/acos/acos_header.h"
#include "/home/cordyn/mlsofits/lib/acos/room.h"
#include "/home/cordyn/mlsofits/device.h"
#include "/home/cordyn/mlsofits/analog.h"


char *
acos_name ()

{
   static char outnam [80]	= "OOOO_TTTTR_WS_YYYYMMDD_HHMMSS.fts";
   static char origin [8]	= "mlso" ;
   static char telescope [8]	= "TTTT" ;
   static char reduce [4]	= "x" ;
   static char wave [4]		= "x" ;
   static char source [4]	= "x" ;
   static char obs_date [12]	= "YYYYMMDD";
   static char obs_time [8]	= "HHMMSS";
   static char ext [8]		= "fts";
   int iyear ;
   int rcode ;
   int wcode ;
   int scode ;

   strcpy (origin, "mlso") ;

   switch (ACOSH->Version)
      {
      case IS_DPMON:	strcpy (telescope, "dpmn") ;	break ;
      case IS_LOWK:	strcpy (telescope, "lowk") ;	break ;
      case IS_CHIP:	strcpy (telescope, "chip") ;	break ;
      default:	        strcpy (telescope, "ZZZZ") ;	break ;
      }

   /*** Set image type and miscellaneous information codes according to the
    *** ACOS image type parameter.
    ***/

   wcode =  ACOSH->Type / 100 ;
   scode = (ACOSH->Type - wcode * 100) / 10 ;
   rcode =  ACOSH->Type - wcode * 100 - scode * 10 ;

   switch (rcode)
      {
      case REDUCTION_x:	strcpy (reduce, "x") ;	break ;/* Undefined. */
      case REDUCTION_r:	strcpy (reduce, "r") ;	break ;/* Raw data. */
      case REDUCTION_c:	strcpy (reduce, "c") ;	break ;/* Calibrated data */
      case REDUCTION_l:	strcpy (reduce, "l") ;	break ;/* Limb dark removed. */
      case REDUCTION_d:	strcpy (reduce, "d") ;	break ;/* Ratio image. */
      case REDUCTION_m:	strcpy (reduce, "m") ;	break ;/* Subtraction image. */
      case REDUCTION_h:	strcpy (reduce, "h") ;	break ;/* limb-d, heliocentric*/
      }

   switch (wcode)
      {
      case WAVELENGTH_x: strcpy (wave, "x") ;	break; /* Undefined. */
      case WAVELENGTH_h: strcpy (wave, "h") ;	break; /* Helium-I: 1083.03nm*/
      case WAVELENGTH_c: strcpy (wave, "c") ;	break; /* Continuum. */
      case WAVELENGTH_m: strcpy (wave, "m") ;	break; /* Helium-I: 1082.91nm*/
      case WAVELENGTH_w: strcpy (wave, "w") ;	break; /* Water vapor. */
      case WAVELENGTH_a: strcpy (wave, "a") ;	break; /* Halpha: 656.2808nm */
      case WAVELENGTH_b: strcpy (wave, "b") ;	break; /* Blue Halpha wing. */
      case WAVELENGTH_r: strcpy (wave, "r") ;	break; /* Red  Halpha wing. */
      case WAVELENGTH_k: strcpy (wave, "k") ;	break; /* K-corona: 657.4 nm */
      case WAVELENGTH_s: strcpy (wave, "s") ;	break; /* Other spectrum. */
      }

   switch (scode)
      {
      case SOURCE_x: strcpy (source, "x") ; break; 	/* Undefined. */
      case SOURCE_s: strcpy (source, "s") ; break; 	/* Sun: full disk. */
      case SOURCE_l: strcpy (source, "l") ; break; 	/* Sun: limb. */
      case SOURCE_o: strcpy (source, "o") ; break; 	/* Opal. */
      case SOURCE_d: strcpy (source, "d") ; break; 	/* Diffuser. */
      case SOURCE_b: strcpy (source, "b") ; break; 	/* Bias (dark). */
      case SOURCE_c: strcpy (source, "c") ; break; 	/* Cal. optics. */
      case SOURCE_f: strcpy (source, "f") ; break; 	/* Flat field image. */
      }

   /* Ensure that year is a full 4-digit number. */

   iyear = ACOSI->year ;
   if (iyear < 1900)  iyear += 1900 ;

   /* Create date & time strings. */

   sprintf (obs_date, "%04d%02d%02d", iyear, ACOSH->Month, ACOSH->Day) ;
   sprintf (obs_time, "%02d%02d%02d",
	    ACOSH->Hour, ACOSH->Minute, ACOSH->Sec) ;

   /* Create output file name. */

   sprintf (outnam, "%s_%s%s_%s%s_%s_%s.fts",
	    origin, telescope, reduce, wave, source, obs_date, obs_time, ext) ;

   return (outnam) ;
}
/* acos_yrddoy_name.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 Just like Andy's routine above, except uses yrddoy.
 We'll use this for a while... ?
 */

char *
acos_yrddoy_name ()

{
   static char outnam [80]	= "OOOO_TTTTR_WS_YYdDOY_HHMMSS.fts";
   static char origin [8]	= "mlso" ;
   static char telescope [8]	= "TTTT" ;
   static char reduce [4]	= "x" ;
   static char wave [4]		= "x" ;
   static char source [4]	= "x" ;
   static char obs_date [12]	= "YYdDOY"; /* like 96d001 for Jan 1
					       and  96d356 for Dec 31
					       in a non-leap year. */
   static char obs_time [8]	= "HHMMSS";
   static char ext [8]		= "fts";
   int iyear ;
   int rcode ;
   int wcode ;
   int scode ;

   int different_spot_for_reduction = 1;
   /*  Set this to 1 if you want
		   "OOOO_TTTT_WSR_YYdDOY_HHMMSS.fts";
       instead of 
		   "OOOO_TTTTR_WS_YYdDOY_HHMMSS.fts";
   */

   strcpy (origin, "mlso") ;

   switch (ACOSH->Version)
      {
      case IS_DPMON:	strcpy (telescope, "dpmn") ;	break ;
      case IS_LOWK:	strcpy (telescope, "lowk") ;	break ;
      case IS_CHIP:	strcpy (telescope, "chip") ;	break ;
      default:	        strcpy (telescope, "ZZZZ") ;	break ;
      }

   /*** Set image type and miscellaneous information codes according to the
    *** ACOS image type parameter.
    ***/

   wcode =  ACOSH->Type / 100 ;
   scode = (ACOSH->Type - wcode * 100) / 10 ;
   rcode =  ACOSH->Type - wcode * 100 - scode * 10 ;

   switch (rcode)
      {
      case REDUCTION_x:	strcpy (reduce, "x") ;	break ;/* Undefined. */
      case REDUCTION_r:	strcpy (reduce, "r") ;	break ;/* Raw data. */
      case REDUCTION_c:	strcpy (reduce, "c") ;	break ;/* Calibrated data */
      case REDUCTION_l:	strcpy (reduce, "l") ;	break ;/* Limb dark removed. */
      case REDUCTION_d:	strcpy (reduce, "d") ;	break ;/* Ratio image. */
      case REDUCTION_m:	strcpy (reduce, "m") ;	break ;/* Subtraction image. */
      case REDUCTION_h:	strcpy (reduce, "h") ;	break ;/* limb-d, heliocentric*/
      }

   switch (wcode)
      {
      case WAVELENGTH_x: strcpy (wave, "x") ;	break; /* Undefined. */
      case WAVELENGTH_h: strcpy (wave, "h") ;	break; /* Helium-I: 1083.03nm*/
      case WAVELENGTH_c: strcpy (wave, "c") ;	break; /* Continuum. */
      case WAVELENGTH_m: strcpy (wave, "m") ;	break; /* Helium-I: 1082.91nm*/
      case WAVELENGTH_w: strcpy (wave, "w") ;	break; /* Water vapor. */
      case WAVELENGTH_a: strcpy (wave, "a") ;	break; /* Halpha: 656.2808nm */
      case WAVELENGTH_b: strcpy (wave, "b") ;	break; /* Blue Halpha wing. */
      case WAVELENGTH_r: strcpy (wave, "r") ;	break; /* Red  Halpha wing. */
      case WAVELENGTH_k: strcpy (wave, "k") ;	break; /* K-corona: 657.4 nm */
      case WAVELENGTH_s: strcpy (wave, "s") ;	break; /* Other spectrum. */
      }

   switch (scode)
      {
      case SOURCE_x: strcpy (source, "x") ; break; 	/* Undefined. */
      case SOURCE_s: strcpy (source, "s") ; break; 	/* Sun: full disk. */
      case SOURCE_l: strcpy (source, "l") ; break; 	/* Sun: limb. */
      case SOURCE_o: strcpy (source, "o") ; break; 	/* Opal. */
      case SOURCE_d: strcpy (source, "d") ; break; 	/* Diffuser. */
      case SOURCE_b: strcpy (source, "b") ; break; 	/* Bias (dark). */
      case SOURCE_c: strcpy (source, "c") ; break; 	/* Cal. optics. */
      case SOURCE_f: strcpy (source, "f") ; break; 	/* Flat field image. */
      }

   /* Ensure that year is a full 4-digit number. */

   iyear = ACOSI->year ;
   if (iyear > 2000)  iyear -= 2000 ;  /* Adjust for 2000 -- Alice */
   if (iyear > 1900)  iyear -= 1900 ;

   /* Create date & time strings. */

   sprintf(obs_date,"%02dd%03d", iyear, 
				 day_of_year (iyear, ACOSH->Month, ACOSH->Day));
   sprintf (obs_time, "%02d%02d%02d",
	    ACOSH->Hour, ACOSH->Minute, ACOSH->Sec) ;

   /* Create output file name. */

	    /*  Ooops, Andy only put int 7%s's but gave 8 variables...
		I fixed it -- room for ext in "fts"  */
   if(different_spot_for_reduction)
       sprintf (outnam, "%s_%s_%s%s%s_%s_%s.%s",
	        origin,telescope,wave,source,reduce,obs_date,obs_time,ext);
   else
       sprintf (outnam, "%s_%s%s_%s%s_%s_%s.%s",
	        origin,telescope,reduce,wave,source,obs_date,obs_time,ext);

   return (outnam) ;
}
