/* acostofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert a binary MLSO/ACOS image to FITS format.
 *
 * ACOS : Advanced Coronal Observing System, includes these instruments :
 *        DPM: Digital Prominence Monitor
 *        LowK: Low K-corona Polarimeter
 *        CHIP: Chromospheric Helium-1 Imaging Photometer
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   9 November 1995
 * 29 Nov 1995 : Add WAVELEN.
 * 30 Nov 1995 : Add Housekeeping Extension.
 *  1 Dec 1995 : Add units to housekeeping items.
 * 15 Feb 1996 : Change code_obs from type integer to character.
 *               Add code_dp, solar_ra, solardec.
 *  8 Mar 1996 : origin: MLSO, telescop: ACOS / DPM, LOWK, CHIP, MK3.
 * 18 Mar 1996 : Remove image orientation (180 degree rotation) logic.
 *               Change pixel resolution from 2.395 to 2.290.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/cordyn/mlsofits/mechanism_defs.h"
#include "/home/cordyn/mlsofits/lib/acos/acos.h"
#include "/home/cordyn/mlsofits/lib/acos/acos_info.h"
#include "/home/cordyn/mlsofits/lib/acos/acos_header.h"
#include "/home/cordyn/mlsofits/lib/acos/room.h"
#include "/home/cordyn/mlsofits/device.h"
#include "/home/cordyn/mlsofits/analog.h"
#include "/home/cordyn/mlsofits/lcvr.h"
#include "/home/cordyn/mlsofits/mic.h"

#define MAXLIN 180              /* Maximum # lines allocated for FITS header. */
char fitbuf   [MAXLIN][80] ;    /* FITS header buffer. */
char outbuf   [84] ;



static char korigin   [12] = "ORIGIN  " ;
static char  origin   [12] = "MLSO" ;
static char origincom [28] = "MAUNA LOA SOLAR OBSERVATORY" ;
static char ktelescop [12] = "TELESCOP" ;
static char telescop [12] =  "ACOS" ;           /* Telescope name. */
static char tscopcom [4][44] = {"DIGITAL PROMINENCE MONITOR",
			       "LOW K-CORONA CORONAGRAPH",
			       "CHROMOSPHERIC HELIUM-1 IMAGING PHOTOMETER",
			       "MARK-III K-CORONAMETER" } ; 
static char tscopnam [4][8] = {"DPM", "LOWK", "CHIP", "MK3"} ;



int
acostofits (namimg, namext, hduflag, pmode , nodel )

   char namimg [] ;             /* Name of primary file. */

   char namext [] ;             /* Name of extension file. */

   int hduflag ;                /* 1: primary image ; no extension.        */
                                /* 2: primary image ; extension exists.    */
                                /* 3: extension image ; append to namfits. */

   int pmode ;                  /* Print mode. */

   int nodel ;                  /* Whether or no to delete the file after 
                                 * conversion to fits. */


{
   extern char *malloc () ;           /* Function to allocate memory. */
   extern unsigned char *rdacos () ;  /* Function to read ACOS image. */
   char *padbuf ;                     /* Pointer to padding buffer. */
   unsigned char *acosbuf ;           /* MLSO ACOS file  array pointer. */
   unsigned char *imgbuf ;            /* MLSO ACOS image array pointer. */
   unsigned char *cibuf ;             /* unsigned char image pointers. */

   unsigned char cmin, cmax ;         /* Data min/max values. */
   unsigned char cval ;               /* Data value. */

   int do_yrddoy = 1;   /* Set this to 1 if you want yrddoy names,
                           Set it to   0 if you do not. 
                        */
   int do_flipTB = 1;   /* Set this to 1 if you need to flip an image
                                         Top to Bottom.   (CHIP)
                           Set it to   0 if you do not. 
                        */
   int do_rot180 = 1;   /* Set this to 1 if you need to rotate an image
                                         by 180 degrees.  (dPmon)
                           Set it to   0 if you do not. 
                        */


   static char kairmass  [12] = "AIRMASS " ;
   static char kazimuth  [12] = "AZIMUTH " ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcode_obs [12] = "CODE-OBS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kocc_disc [12] = "OCC-DISC" ;
   static char kdisp_dir [12] = "DISP-DIR" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kacosver  [12] = "ACOSVER " ;
   static char kelev     [12] = "ELEV    " ;
   static char kend      [12] = "END     " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kgain_det [12] = "GAIN-DET" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char khour_ang [12] = "HOUR-ANG" ;
   static char kinstrume [12] = "INSTRUME" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char knorthang [12] = "NORTHANG" ;
   static char kobject   [12] = "OBJECT  " ;
   static char knimsum   [12] = "NIMSUM  " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char krsun     [12] = "RSUN    " ;
   static char ksolar_b0 [12] = "SOLAR_B0" ;
   static char ksolar_l0 [12] = "SOLAR_L0" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ksolar_ra [12] = "SOLAR_RA" ;
   static char ksolardec [12] = "SOLARDEC" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char ktype_dp  [12] = "TYPE-DP " ;
   static char kversion  [12] = "VERSION " ;
   static char kwavelen  [12] = "WAVELEN " ;
   static char kwavewid  [12] = "WAVEWID " ;
   static char krcode    [4]  = "?" ;
   static char kscode    [4]  = "?" ;
   static char kwcode    [4]  = "?" ;

   static char atype1   [24] ;                    /* Coord type axis 1. */
   static char atype2   [24] ;                    /* Coord type axis 2. */
   static char bunit    [32] ;                    /* Brightness unit. */
   static char ctime    [12] = "HH:MM:SS" ;       /* File creation time. */
   static char ctype1   [24] ;                    /* Coord type axis 1. */
   static char ctype2   [24] ;                    /* Coord type axis 2. */
   static char ctypenam [24] = "GEOCENTRIC";      /* Coordinate system name. */
   static char cunit1   [24] ;                    /* Coord units axis 1. */
   static char cunit2   [24] ;                    /* Coord units axis 2. */
   static char datcom   [72] ;                    /* Data min/max comment. */
   static char cdate    [12] = "DD/MM/YY" ;       /* File creation date. */
   static char code_obs [12] = "???     " ;       /* Observation code. */
   static char date_cal [12] = "DD/MM/YY" ;       /* Calibration date. */
   static char date_obs [12] = "DD/MM/YY" ;       /* Observation date. */
   static char eq       [4]  = "= " ;
   static char instrume [12] = "CCD" ;            /* Instrument name. */
   static char namfit   [80] ;                    /* FITS file name. */
   static char object   [12] = "SUN" ;            /* Object. */
   static char pixcom   [48] ;                    /* Data type comment. */
   static char refcom   [24] ;                    /* Coord ref comment. */
   static char disp_dir [ 4] ;           /* Display direction: R/L, U/D, O/I. */
   static char time_obs [12] = "HH:MM:SS" ;       /* Observation time. */
   static char type_obs [24] ;                    /* Observation type. */
   static char type_dp  [24] ;                    /* Data Processing type. */
   static char version  [24] ;
   static char instnam  [4][8] = {"CCD", "CCDK", "CCD", "RETICON"} ;
   static char wave [12] ;
   static char source [12] ;
   static char reduce [12] ;
   static char tscope [12] ;

   int nimsum ;


   static char instcom [4][28] =
      {"Kodak Megaplus 1.6 CCD",
       "Kodak Megaplus 1.6 CCD",
       "Kodak Megaplus 1.6 CCD",
       "128-element Linear Reticon"
      } ;

   static int ic [7] ;                  /* Array containing date & time:
                                         * year, doy, day, month,
                                         * hour, minute, second.
                                         */

   short *sibuf ;                      /* Short image pointers. */
   short smin, smax ;                  /* Data min/max values. */
   short sval = 0 ;                    /* Data value. */

   int bitpix ;                        /* # bits per pixel. */
   int doy_obs ;                       /* Observation day-of-year. */
   int gcount = 0 ;                    /* gcount. */
   int iyear ;                         /* year - 1900. */
   int i, ii , ix, iy ;                /* Indices. */
   int iynum ;                         /* # lines in FITS header. */
   int nhdu ;                          /* # HDU blocks to write. */
   int icyear ;                        /* year. */
   int icdoy ;                         /* day-of-year. */
   int icday ;                         /* day-of-month. */
   int icmonth ;                       /* month. */
   int ichour ;                        /* hour. */
   int icmin ;                         /* minute. */
   int icsec ;                         /* second. */
   int ival ;                          /* Integer value. */
   int kx, ky ;                        /* Image indices. */
   int ndim   = 0 ;                    /* # dimensions for image (2=XY). */
   int nx, ny ;                        /* # pixels for X & Y axes. */
   int nbyimg ;                        /* # bytes in image array. */
   int nbyfits = 2880 ;                /* # bytes per FITS logical record. */
   int npximg ;                        /* # pixels in image. */
   int npad ;                          /* # bytes to pad file. */
   int pcount  = 0 ;                   /* pcount. */
   int nrec ;                          /* # fits records for image. */
   int nelwr   = 0 ;
   int rcode ;
   int scode ;
   int wcode ;
   int year_obs ;                      /* Observation year. */

   float *fibuf ;                      /* Float image pointers. */

   float northdeg =  90.0 ;            /* Solar north w.r.t. +X axis. */
   float bscale   =   1.0 ;            /* Brightness intensity scale factor. */
   float bzero    =   0.0 ;            /* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;              /* Physical coordinate increments. */
   float crpix1, crpix2 ;              /* Reference pixel location. */
   float crota1, crota2 ;              /* Physical coordinate rotation angle.*/
   float crradius = 419.21 ;           /* Solar radius [pixels] = 960/2.29. */
   float crval1, crval2 ;              /* Physical coordinate values. */
   float dmin, dmax ;                  /* Data min/max values. */
   float swver = 1.00 ;                /* ACOS FITS s/w version number. */
   float fmin, fmax ;                  /* Data min/max values. */
   float fval ;                        /* Data value. */
   float xcenter, ycenter, diam ;      /* X/Y centers + disk diameter. */
   float vmin, vmax ;                  /* Data min/max values. */
   float wmin =    0.0 ;               /* Display min value. */
   float wmax = 4000.0 ;               /* Display max value. */
   int   mx;
   FILE *ffid ;                        /* FITS file ID. */

   int duplicate_hdr;   /* Some times the header does not get refreshed, ick. */


   /*** Read MLSO/ACOS image/data from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2)
      acosbuf = rdacos (namimg, pmode) ;
   else
      acosbuf = rdacos (namext, pmode) ;

   strcpy (namfit, ((do_yrddoy) ? acos_yrddoy_name() : acos_name ())) ;

   if (pmode)
      {
      fprintf (stdout, "acostofits> namimg: %s  namext: %s   hduflag: %d\n",
               namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- acostofits> %s -> %s -------\n\n",
               namimg, namfit) ;
      }

   duplicate_hdr = ckifexists(namimg,namfit);

   if((duplicate_hdr==0) && ( nodel==0 )) remove_file(namimg);
   if( duplicate_hdr==1) {
	fprintf(stderr,"\tNo fits file will be produced.\n");
	return(1);
   }

   nx = ACOSH->Xdim ;                        /* X dimension. */
   ny = ACOSH->Ydim ;                        /* Y dimension. */
   npximg = nx * ny ;                        /* # pixels in image. */
   nbyimg = npximg * ACOSI->nbypx ;          /* # bytes in image. */
   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
                        nx, ny, ACOSI->nbypx, npximg, nbyimg) ;

   wcode =  ACOSH->Type / 100 ;                            /* Wavelength */
   scode = (ACOSH->Type - wcode * 100) / 10 ;              /* Source     */
   rcode =  ACOSH->Type - wcode * 100 - scode * 10 ;       /* Reduction  */

   /*** Find min/max data values. ***/
   /*** Find disk center for short integer images. ***/

   imgbuf = acosbuf + nx * ACOSI->nbypx ;  /* make the image start past
                                              the header.   */



   switch (ACOSI->nbypx)
      {
      case 1:
         cibuf = (unsigned char *) imgbuf ;
         cmin  = 255 ;
         cmax  =   0 ;
         for (iy = 0 ; iy < ny ; iy++)
            {
            for (ix = 0 ; ix < nx ; ix++)
               {
               cval = *(cibuf + ix + iy*nx) ;
               if (cval < cmin)  cmin = cval ;
               if (cval > cmax)  cmax = cval ;
               }
            }
         ACOSI->dmin = (int) cmin ;
         ACOSI->dmax = (int) cmax ;
         vmin = (float) cmin ;
         vmax = (float) cmax ;
         break ;

      case 2:
         sibuf = (short *) imgbuf ;

         if((ACOSH->Version == IS_CHIP) && do_flipTB && (rcode!=REDUCTION_h)) 
	     flipTB(sibuf,nx,ny-1); /*Flip the image top to bottom.*/

         if((ACOSH->Version == IS_DPMON) && do_rot180 && (rcode!=REDUCTION_h))
	     rot180(sibuf,nx,ny-1); /*Rotate the image by 180 degrees. */

         smin  =  32767 ;
         smax  = -32767 ;
	 nimsum = ACOSH->Nsum ; mx=1023*nimsum;
				mx=(mx < 32767)? mx : 32767 ;

         /* Trim the image by 100 on top and bottom to avoid noisy bits... */
         sibuf += nx*100;
         ix = nx*ny - nx*200;
         for(ii=0;ii<ix;ii++){
            smin = ( *sibuf < smin && *sibuf >  -1000 ) ? *sibuf : smin ;
            smax = ( *sibuf > smax && *sibuf <     mx ) ? *sibuf : smax ;
            sibuf++;
         }

         ACOSI->dmin = (int) smin ;
         ACOSI->dmax = (int) smax ;
         vmin = (float) smin ;
         vmax = (float) smax ;

         break ;

      case 4:
         fibuf = (float *) imgbuf ;
         fmin  =  1.0e+30 ;
         fmax  = -1.0e+30 ;
         for (iy = 0 ; iy < ny ; iy++)
            {
            for (ix = 0 ; ix < nx ; ix++)
               {
               fval = *(fibuf + ix + iy*nx) ;
               if (fval < fmin)  fmin = fval ;
               if (fval > fmax)  fmax = fval ;
               }
            }
         ACOSI->dmin = (int) fmin ;
         ACOSI->dmax = (int) fmax ;
         vmin = fmin ;
         vmax = fmax ;
         break ;

      default:
         fprintf (stderr, 
	    "Invalid ACOS image . . . (numnber of bytes per pixel) nbypx: %d\n",
	    ACOSI->nbypx) ;
         exit (-1) ;
      }

   iyear = ACOSI->year ;
   if (ACOSI->year > 1900)  iyear = ACOSI->year - 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, ACOSH->Month, ACOSH->Day) ;

   sprintf (date_obs, "%02d/%02d/%02d", ACOSH->Day, ACOSH->Month, iyear) ;
   sprintf (time_obs, "%02d:%02d:%02d", ACOSH->Hour, ACOSH->Minute, ACOSH->Sec) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear       = ic [0] ;
   icdoy        = ic [1] ;
   icday        = ic [2] ;
   icmonth      = ic [3] ;
   ichour       = ic [4] ;
   icmin        = ic [5] ;
   icsec        = ic [6] ;
   sprintf (cdate, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (ctime, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   iyear = ACOSI->cal_year ;
   if (ACOSI->cal_year > 1900)  iyear = ACOSI->cal_year - 1900 ;
   sprintf (date_cal, "%02d/%02d/%02d", ACOSI->cal_day, ACOSI->cal_month, iyear) ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (ACOSH->Xdim > 0)
      ndim++ ;
   if (ACOSH->Ydim > 1)
      ndim++ ;

   /*
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Set parameters.
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;
   strcpy (origin,   "MLSO") ;
   sprintf (telescop, "%s", &tscopnam [ACOSH->Version][0]) ;
   sprintf (instrume, "%s", &instnam [ACOSH->Version][0]) ;
   sprintf (datcom,   "%s", &instnam [ACOSH->Version][0]) ;
   strcpy (atype1,   "X [EAST->WEST ]") ;
   strcpy (atype2,   "Y [SOUTH->NORTH]") ;
   strcpy (ctypenam, "GEOCENTRIC") ;
   strcpy (cunit1,   "ARCSEC") ;
   strcpy (cunit2,   "ARCSEC") ;
   strcpy (refcom,   "SUN CENTER") ;
   strcpy (disp_dir,  "RU") ;                /* Display direction: right, up. */

   crpix1 = (float) ((nx - 1) / 2) + 0.5 ;   /* Sun center (X pixels). */
   crpix2 = (float) ((ny - 1) / 2) + 0.5 ;   /* Sun center (Y pixels). */
   crval1 = 0.0 ;                            /* Sun center (X arcsec). */
   crval2 = 0.0 ;                            /* Sun center (Y arcsec). */
   cdelt1 = 2.290 ;                          /* arcsec/pixel. */
   cdelt2 = 2.290 ;                          /* arcsec/pixel. */

   if (ACOSH->Version == IS_DPMON) {
       switch (scode) {
	  /* I (alice) measured 2.90 for the disk and 2.95 for the limb
	     images based on the occ. disk size of 1961arcseconds.
	     David beleives they are both the same, or very close,
	     so I've made both 2.90 --Alice
	  */
	  case  SOURCE_s:  cdelt1 = 2.90 ;  /* arcsec/pixel. */
			   cdelt2 = 2.90 ;  /* arcsec/pixel. */
			   break ;
	  case  SOURCE_l:  cdelt1 = 2.90 ;  /* arcsec/pixel. */
			   cdelt2 = 2.90 ;  /* arcsec/pixel. */
			   break ;
	  default :        cdelt1 = 2.90 ;  /* arcsec/pixel. */
			   cdelt2 = 2.90 ;  /* arcsec/pixel. */
       }
   }

   crradius = ACOSH->Radius / cdelt1 ;         /* Solar radius (pixels). */

    /*** Find disk center.  Exclude housekeeping line. ***/
    /*** Alice put the housekeeping line back in -- just to keep
         things consistant with the calibration code. */
    /*** It looks like after Jan 1 , 1997, the dPmon disk images are
	 centered on the fly at MLSO when they are captured.
	 So, the findcen below is skipped for these images,
	 and the image center set to 767.5,514.5                   */
    /*** Ha! But it turns out that the on the fly code fails on sharp dust
	 bit, so I have to redo it and try to do it better anyway! */

	 if(rcode != REDUCTION_h) {
	     sibuf = (short *) imgbuf ;
	     if(ACOSH->Version == IS_CHIP ) diam = 840.;
	     if(ACOSH->Version == IS_DPMON) diam = crradius*2. ;

	     findcen (sibuf, nx, ny, &xcenter, &ycenter, &diam, scode) ;
	 }
	 else{
	    diam=0.;
	    for(ii=0;ii<ACOSH->Nsum;ii++)  diam += ACOSH->Diam[ii];
	    diam /= (float)(ACOSH->Nsum);
	    xcenter = (float)(ACOSH->Xdim - 1) * 0.5;
	    ycenter = (float)(ACOSH->Ydim - 1) * 0.5;
	 }
    /********************************************************************/

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * ACOS data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   /*** Set object.
    *** Set coordinate type(s).
    *** Set bitpix.
    *** Set display scaling.
    ***/

   /*** Default designations. ***/

   strcpy (object,   "UNKNOWN") ;   
   strcpy (type_obs, "UNKNOWN") ;
   strcpy (type_dp , "UNKNOWN") ;
   strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
   strcpy (ctype1,   "X") ;
   strcpy (ctype2,   "Y") ;
   strcpy (bunit,    "CCD") ;
   bitpix =  16 ;                        /* short integer image. */
   wmin = 0.0 ;
   wmax = 4000.0 ;

   /*** Set image type and miscellaneous information codes according to the
    *** ACOS image type parameter.
    ***/

   switch (wcode)
      {
      case  WAVELENGTH_x:  strcpy (wave, "UNDEF") ;      strcpy (kwcode, "x") ;
			   break ;
      case  WAVELENGTH_h:  strcpy (wave, "HELIUM-I") ;   strcpy (kwcode, "h") ;
			   break ;
      case  WAVELENGTH_c:  strcpy (wave, "HE-CONT") ;    strcpy (kwcode, "c") ;
			   break ;
      case  WAVELENGTH_m:  strcpy (wave, "HE-MINOR") ;   strcpy (kwcode, "m") ;
			   break ;
      case  WAVELENGTH_w:  strcpy (wave, "H2O") ;        strcpy (kwcode, "w") ;
			   break ;
      case  WAVELENGTH_a:  strcpy (wave, "HALPHA") ;     strcpy (kwcode, "a") ;
			   break ;
      case  WAVELENGTH_b:  strcpy (wave, "HA-BLUE") ;    strcpy (kwcode, "b") ;
			   break ;
      case  WAVELENGTH_r:  strcpy (wave, "HA-RED") ;     strcpy (kwcode, "r") ;
			   break ;
      case  WAVELENGTH_k:  strcpy (wave, "KCORONA") ;    strcpy (kwcode, "k") ;
			   break ;
      case  WAVELENGTH_s:  strcpy (wave, "OTHER") ;      strcpy (kwcode, "s") ;
			   break ;
      default:             strcpy (wave, "UNDEF") ;      strcpy (kwcode, "?") ;
      }

   switch (scode)
      {
      case  SOURCE_x:  strcpy (source, "UNDEF") ;    strcpy (kscode, "x") ;
		       break ;
      case  SOURCE_s:  strcpy (source, "SUN") ;      strcpy (kscode, "s") ;
		       break ;
      case  SOURCE_l:  strcpy (source, "LIMB") ;     strcpy (kscode, "l") ;
		       break ;
      case  SOURCE_o:  strcpy (source, "OPAL") ;     strcpy (kscode, "o") ;
		       break ;
      case  SOURCE_d:  strcpy (source, "DIFFUSER") ; strcpy (kscode, "d") ;
		       break ;
      case  SOURCE_b:  strcpy (source, "DARK") ;     strcpy (kscode, "b") ;
		       break ;
      case  SOURCE_c:  strcpy (source, "CAL-OPT") ;  strcpy (kscode, "c") ;
		       break ;
      case  SOURCE_f:  strcpy (source, "FLAT-FLD") ; strcpy (kscode, "f") ;
		       break ;
      default:         strcpy (source, "UNKNOWN") ;  strcpy (kscode, "?") ;
      }

   switch (rcode)
      {
      case  REDUCTION_x:  strcpy (reduce, "UNDEF") ;    strcpy (krcode, "x") ;
                strcpy (bunit,  "CCD") ;       break ;
      case  REDUCTION_r:  strcpy (reduce, "RAW") ;      strcpy (krcode, "r") ;
                strcpy (bunit,  "CCD") ;       break ;
      case  REDUCTION_c:  strcpy (reduce, "CAL") ;      strcpy (krcode, "c") ;
                strcpy (bunit,  "PB") ;        break ;
      case  REDUCTION_l:  strcpy (reduce, "LD-COR") ;   strcpy (krcode, "l") ;
                strcpy (bunit,  "PB") ;        break ;
      case  REDUCTION_d:  strcpy (reduce, "RATIO") ;    strcpy (krcode, "d") ;
                strcpy (bunit,  "NONE") ;      break ;
      case  REDUCTION_m:  strcpy (reduce, "CONTRAST") ; strcpy (krcode, "m") ;
                strcpy (bunit,  "PB") ;        break ;
      case  REDUCTION_h:  strcpy (reduce, "LD-COR") ;   strcpy (krcode, "h") ;
                strcpy (bunit,  "PB") ;        
	        strcpy (ctypenam, "HELIOCENTRIC") ;
		break ;
      default:  strcpy (reduce, "UNKNOWN") ;  strcpy (krcode, "?") ;
                strcpy (bunit,  "CCD") ;
      }

/*
 *   if ((ACOSH->Version==IS_LOWK) || (ACOSH->Version==IS_MK3))
 *      {
 *      switch (rcode)
 *         {
 *         case  0:        strcpy (bunit, "CCD") ;     break ;
 *         case  1:        strcpy (bunit, "CCD") ;     break ;
 *         case  2:        strcpy (bunit, "PB") ;      break ;
 *         case  3:        strcpy (bunit, "PB") ;      break ;
 *         case  4:        strcpy (bunit, "NONE") ;    break ;
 *         case  5:        strcpy (bunit, "PB") ;      break ;
 *         default:        strcpy (bunit, "CCD") ;
 *         }
 *      }
 */

   if (ACOSH->Version == IS_DPMON)
      {
      strcpy (ctypenam, "HELIOCENTRIC") ; /* Still heliocentric --
					   * will change though... 6/6/96 */
      switch (rcode)
         {
         case  REDUCTION_x:  strcpy (bunit, "CCD") ;         break ;
         case  REDUCTION_r:  strcpy (bunit, "CCD") ;         break ;
         case  REDUCTION_c:  strcpy (bunit, "BSUN") ;        break ;
         case  REDUCTION_l:  strcpy (bunit, "BSUN") ;        break ;
         case  REDUCTION_d:  strcpy (bunit, "NONE") ;        break ;
         case  REDUCTION_m:  strcpy (bunit, "BSUN") ;        break ;
         case  REDUCTION_h:  strcpy (bunit, "BSUN") ;        break ;
         default:            strcpy (bunit, "CCD") ;
         }
      }

   if (ACOSH->Version == IS_CHIP)
      {
      switch (rcode)
         {
         case  REDUCTION_x:  strcpy (bunit, "CCD") ;         break ;
         case  REDUCTION_r:  strcpy (bunit, "CCD") ;         break ;
         case  REDUCTION_c:  strcpy (bunit, "BSUN") ;        break ;
         case  REDUCTION_l:  strcpy (bunit, "BSUN") ;        break ;
         case  REDUCTION_d:  strcpy (bunit, "NONE") ;        break ;
         case  REDUCTION_m:  strcpy (bunit, "BSUN") ;        break ;
         case  REDUCTION_h:  strcpy (bunit, "BSUN") ;        break ;
         default:            strcpy (bunit, "CCD") ;
         }
      }

   nimsum = ACOSH->Nsum ;

   if (ACOSH->Version == IS_DPMON) {
      wmin =    0.0 ;
      switch ( scode ) {
	  case SOURCE_s: wmax= 1024.;                        break;
	  case SOURCE_l: wmax= 1024.;                        break;
	  case SOURCE_d: wmax=  512.;                        break;
	  case SOURCE_b: wmax=   64.; wmin= -wmax;           break;
	  case SOURCE_f: wmax= 1024.;                        break;
	  default      : wmax= 1024.;
      }
      wmin *= (float)nimsum;
      wmax *= (float)nimsum;
      /* The above is not reliable. Try this instead. */
      wmax = vmax  + 0.2 *vmax ;
   }


   if (ACOSH->Version == IS_CHIP) {
      wmin =    0.0 ;
      switch ( scode ) {
	  case SOURCE_s: switch(wcode) {
			    case WAVELENGTH_w: wmax= 512.;   break;
			    default:           wmax= 512.;
			 } 
			 break;
	  case SOURCE_b: wmax=  128.; wmin= -wmax;           break;
	  case SOURCE_f: wmax= 1024.;                        break;
	  default      : wmax= 1024.;
      }
      wmin *= (float)nimsum;
      wmax *= (float)nimsum;
      /* The above is not reliable. Try this instead. */
      wmax = vmax  + 0.2 *vmax ;
   }
   strcpy (tscope, &tscopnam [ACOSH->Version][0]) ;

   sprintf (code_obs, "%s%s%s", kwcode, kscode, krcode) ;
   strcpy (object, source) ;
   strcpy (type_obs, wave) ;
   strcpy (type_dp, reduce) ;

   /*** Create FITS file. ***/

   if (hduflag == 3)                                /* Extension dataset. */
      {
      if ((ffid = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else                                                /* Primary dataset. */
      {
      if ((ffid = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- PRIMARY HDU ---\n") ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   sprintf (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: MLSO/ACOS") ;
   else
   sprintf (&fitbuf [iy++][0],  "XTENSION= 'ACOS    '           / MLSO/ACOS %s EXTENSION", type_obs) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
                                knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
                                knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
                                   knaxis2, eq, ny, atype2) ;
   else
      sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;

   /*** Extension information. ***/

   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
                                  kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
                                  kgcount, eq, gcount) ;
      }
   else
      {
      sprintf (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
      }

   /*** Date & Time of observation. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
                                doy_obs, year_obs) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS [UT]") ;

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ %s", origincom) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ %s", &tscopcom [ACOSH->Version][0]) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kinstrume, eq, instrume) ;
   sprintf (&fitbuf [iy++][31], "/ %s", &instcom [ACOSH->Version][0]) ;

   /*** Observation source & type. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",
                                kcode_obs, eq, code_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [CHARACTER CODE] ") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ SOURCE OBJECT") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", ktype_dp,   eq, type_dp) ;
   sprintf (&fitbuf [iy++][31], "/ DATA PROCESSING TYPE") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f",
                                kwavelen,  eq, ACOSH->Wave) ;
   sprintf (&fitbuf [iy++][31], "/ CENTRAL WAVE LENGTH OF OBSERVATION [NM]") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f",
                                kwavewid,  eq, ACOSI->wavewid) ;
   sprintf (&fitbuf [iy++][31], "/ BANDPASS OF FILTER [NM]") ;


   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s %s", atype1, ctypenam) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [INDEX:PIXELS]",
                                kcrpix1  , eq, xcenter, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
                                kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
                                kcdelt1  , eq, cdelt1, atype1, cunit1) ;

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s %s", atype2, ctypenam) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [INDEX:PIXELS]",
                                   kcrpix2, eq, ycenter, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
                                   kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
                                   kcdelt2  , eq, cdelt2, atype2, cunit2) ;
      }
   else
      {
      sprintf (&fitbuf  [iy++][0],  "COMMENT                                                                         ") ;
      sprintf (&fitbuf  [iy++][0],  "COMMENT                                                                         ") ;
      sprintf (&fitbuf  [iy++][0],  "COMMENT                                                                         ") ;
      sprintf (&fitbuf  [iy++][0],  "COMMENT                                                                         ") ;
      }

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM NAME") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f", kcrradius, eq, crradius) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR RADIUS [PIXELS]") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR RADIUS [ARCSEC]",
                                krsun    , eq, ACOSH->Radius) ;

   if (ACOSH->Version == IS_DPMON || ACOSH->Version == IS_LOWK)
      {
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / OCCULTING DISC DIAMETER [ARCSECONDS]",
                                   kocc_disc, eq, ACOSI->odisk) ;
      }
   else
   if (ACOSH->Version == IS_CHIP)
      sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;


   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
                                ksolar_p0, eq, ACOSH->Pangle) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
                                ksolar_b0, eq, ACOSH->Bangle) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE",
                                ksolar_l0, eq, ACOSH->Langle) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR RIGHT ASCENSION [DEGREES]",
                                ksolar_ra, eq, ACOSH->RA) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR DECLINATION [DEGREES]",
                                ksolardec, eq, ACOSH->Dec) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR HOUR ANGLE [DEGREES] (W.R.T.ZENITH E- W+)",
                                khour_ang, eq, ACOSH->HA) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR ELEVATION [DEGREES]",
                                kelev    , eq, ACOSH->Elevation) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR AZIMUTH [DEGREES]",
                                kazimuth , eq, ACOSH->Azimuth) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / AIRMASS [ATMOSPHERES]",
                                kairmass , eq, ACOSH->Amass) ;


   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.3f",
                                kgain_det, eq, ACOSI->gain) ;
   sprintf (&fitbuf [iy++][31], "/ DETECTOR GAIN FACTOR") ;

/* -------------------------------------------------------------------------- */

   /*** Number of Images Summed  . ***/

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF IMAGES SUMMED  ",
                                knimsum  , eq, ACOSH->Nsum) ;
   /*** Exposure Duration. ***/

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / EXPOSURE DURATION [SECONDS]",
                              kexpdur  , eq, ACOSI->expdur) ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kbunit, eq, bunit) ;
   sprintf (&fitbuf [iy++][31], "/ PIXEL MAGNITUDE UNITS") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PHYSICAL = DATA * BSCALE + BZERO",
                                kbscale, eq, ACOSI->bscale) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.3f", kbzero , eq, ACOSI->bzero ) ;
   sprintf (&fitbuf [iy++][31], "/ PHOTOMETRIC SCALING OFFSET [SEE BSCALE]") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MINIMUM DATA VALUE [%s]",
                                kdatamin , eq, vmin, datcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MAXIMUM DATA VALUE [%s]",
                                kdatamax , eq, vmax, datcom) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MIN VALUE FOR DISPLAY [UNSCALED]",
                                kdispmin , eq, wmin) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MAX VALUE FOR DISPLAY [UNSCALED]",
                                kdispmax , eq, wmax) ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kdisp_dir,  eq, disp_dir) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION (AXIS 1 2 3): R/L  U/D  O/I") ;


   sprintf (version, "ACOS v%4.2f", swver) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kversion, eq, version) ;
   sprintf (&fitbuf [iy++][31], "/ ACOS FITS S/W VERSION") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kdate,     eq, cdate) ;
   sprintf (&fitbuf [iy++][31], "/ FITS FILE GENERATION DATE: DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_cal, eq, date_cal) ;
   sprintf (&fitbuf [iy++][31], "/ ACOS CALIBRATION DATE: DD/MM/YY [UT]") ;


   sprintf (&fitbuf [iy++][0],  "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   THE ADVANCED CORONAL OBSERVING SYSTEM (ACOS) CONSISTS OF 3 SOLAR") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INSTRUMENTS (ALL LOCATED AT MLSO):") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT      DIGITAL PROMINENCE MONITOR (DPM), LOW K-CORONA POLARIMETER (LOWK),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT      & CHROMOSPHERIC HELIUM-1 IMAGING PHOTOMETER (CHIP).") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.") ;
   sprintf (&fitbuf [iy++][0],  "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         {
         if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
         ival = (int) fitbuf [iy][ix] ;
         fitbuf [iy][ix] = (char) toupper (ival) ;
         }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*2880, ffid) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (acosbuf, ACOSI->nbypx, npximg, ffid) ;
   free (acosbuf) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, ffid) ;
      free (padbuf) ;
      }
   if (pmode)
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg) ;

   /* Add real-time & housekeeping info in extension. */

   acosext (ffid, object, type_obs, date_obs, time_obs, pmode) ;

   close (ffid) ;
   return (0) ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

/* acosext: FITS extension containing real-time & housekeeping information.
 */

acosext (FD, object, type_obs, date_obs, time_obs, pmode)

FILE *FD ;
char *object   ;
char *type_obs ;
char *date_obs ;
char *time_obs ;
int pmode ;

{
   static char kanalog   [12] = "ANALOG" ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kdiam     [12] = "DIAM" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char klcvr     [12] = "LCVR" ;
   static char kmic      [12] = "MIC" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char kobject   [12] = "OBJECT  " ;
   static char koptdev   [12] = "OPTDEV" ;
   static char koptdrv   [12] = "OPTDRV" ;
   static char koptpos   [12] = "OPTPOS" ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char kxshift   [12] = "XSHIFT" ;
   static char kyshift   [12] = "YSHIFT" ;


   static char pixcom    [48] ;                        /* Data type comment. */
   static char eq        [4]  = "= " ;
   static char cposition [10] = "POSITION" ;
   static char cdrive    [10] = "DRIVE" ;


   static char analog_units [NUM_A2D][8] =
      { "(C)", "(C)", "(C)", "(C)", "(C)", "(C)", "(ARB)",
	"(V)", "(C)", "(C)", "(C)", "(C)",
	"(ARB)", "(ARB)", "(ARB)", "(ARB)"
      } ;

   static char lcvr_units [NUM_LCVR][8] =
      { "(V)", "(V)", "(V)", "(V)", "(V)", "(V)", "(V)", "(V)"
      } ;

   static char mic_units [NUM_MIC][10] =
      { "(MICRONS)", "(MICRONS)", "(MICRONS)", 
        "(MICRONS)", "(MICRONS)", "(MICRONS)"
      } ;

   int gcount = 0 ;                                /* gcount. */
   int pcount  = 0 ;                                /* pcount. */
   int nhdu = 0 ;                                /* # FITS header blocks. */
   int i ;
   int ix = 0, iy = 0 ;
   int iyear ;
   int iynum = 0 ;
   int ival = 0 ;
   int bitpix = 16 ;
   int ndim = 0 ;
   int nelwr = 0 ;
   int year_obs ;
   int doy_obs ;
   struct a2d *mya2d;

   sprintf (telescop, "%s", &tscopnam [ACOSH->Version][0]) ;

   /*** Determine day-of-year. ***/

   iyear = ACOSI->year ;
   if (ACOSI->year > 1900)  iyear = ACOSI->year - 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, ACOSH->Month, ACOSH->Day) ;

   strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         fitbuf [iy][ix] = ' ' ;

   iy = 0 ;

   sprintf (&fitbuf [iy++][0],  "XTENSION= 'ACOS_HSKP'          / MLSO/ACOS HOUSEKEEPING EXTENSION") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
                                knaxis, eq, ndim) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
                                kpcount, eq, pcount) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
                                kgcount, eq, gcount) ;

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ %s", origincom) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ %s", &tscopcom [ACOSH->Version][0]) ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ SOURCE OBJECT") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
                                doy_obs, year_obs) ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS [UT]") ;

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-6s%02d%2s%20.4f / XSHIFT [%2d] [pixels]",
                                kxshift, i, eq, ACOSH->Xshift [i], i) ;

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-6s%02d%2s%20.4f / YSHIFT [%2d] [pixels]",
                                kyshift, i, eq, ACOSH->Yshift [i], i) ;

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < 16; i++)
   sprintf (&fitbuf [iy++][0],  "%-4s%02d  %2s%20.4f / DIAM [%2d] [pixels]",
                                kdiam, i, eq, ACOSH->Diam [i], i) ;

   /*** Optical devices & positions. ***/

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < NUM_DEV; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptdev, i, eq,
                                &ACOSI->optdev [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %-6s%02d NAME", koptdev, i ) ;
   }

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < NUM_DEV; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptpos, i, eq,
                                &ACOSI->optpos [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %-23s%10s", &ACOSI->optdev [i][0],cposition);
   }

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < NUM_DEV; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptdrv, i, eq,
                                &ACOSI->optdrv [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %-23s%10s", &ACOSI->optdev [i][0],cdrive) ;
   }

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < NUM_A2D; i++)  {
       mya2d = &analog[i];
       sprintf (&fitbuf [iy++][0],  "%-6s%02d%2s%20.4f / %-23s%10s",
				    kanalog, i, eq,
				    ACOSH->Analog [i], 
                                    mya2d->name,
				    &analog_units [i][0]) ;
   }

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < NUM_LCVR; i++) {
       sprintf (&fitbuf [iy++][0],  "%-4s%02d  %2s%20.4f / %-23s%10s",
				    klcvr, i, eq,
				    ACOSH->Lcvr [i], 
				    lcvr_name [i],
				    &lcvr_units [i][0]) ;
   }

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   for (i = 0; i < NUM_MIC; i++) {
       sprintf (&fitbuf [iy++][0],  "%-3s%02d   %2s%20.4f / %-23s%10s",
				    kmic, i, eq, 
				    (ACOSH->Mic [i] < -9999999. )
					?
					-9999999.
					:   (ACOSH->Mic [i] >  9999999. )
					    ?
					    9999999. 
					    :
					    ACOSH->Mic [i]
					, 
				    mic_name [i],
				    &mic_units [i][0]) ;
   }

   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   sprintf (&fitbuf [iy++][0],  "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         {
         if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
         ival = (int) fitbuf [iy][ix] ;
         fitbuf [iy][ix] = (char) toupper (ival) ;
         }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*2880, FD) ;

   return (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

month_day (year, doy, pmonth, pday)        /* Set month & day from day-of-year. */

int year, doy, *pmonth, *pday ;

{
   static int day_tab [2][13] =
      {
      {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      } ;
   int i, leap ;

   leap = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0) ;

   for (i = 1 ; doy > day_tab [leap][i] ; i++)
      doy -= day_tab [leap][i] ;

   *pmonth = i ;
   *pday = doy ;

   return (0) ;
}
int
ckifexists(tape_filenm,  filenm )
    char  *tape_filenm, *filenm ;
{
   
    int ii;
    char lsit[200], line[200], rmit[200];
    FILE *pfid;

    sprintf(line,"\0");
    sprintf(lsit,"ls %s\n",filenm);         pfid=popen(lsit,"r+");
    fgetl  (line,sizeof(line),pfid);        ii=pclose(pfid);
 
    /* fprintf(stderr,"%d strlenline\n",strlen(line)); */
    if(strlen(line) > 1) {  /* A file with the same filename already exists. */
	fprintf(stderr,
	    "\tFile  %s  would have had the same filename as: \t %s \n",
	    tape_filenm,filenm);
	return(1);
    }
    else {
	return(0);
    }
}
int
remove_file(tape_filenm)
    char   *tape_filenm;
{
    int ii;
    char line[200], rmit[200];
    FILE *pfid;

    sprintf(rmit,"/bin/rm %s\n",tape_filenm);
    pfid=popen(rmit,"r+"); fgetl(line,sizeof(line),pfid);
    ii=pclose(pfid);

}

int
flipTB(img,xxdim,yydim)
    short *img;
    int   xxdim,yydim;
{
    extern char *malloc () ;
    short *rimg, *qq, *rq;
    int ii, jj, imsz;

    imsz=xxdim*yydim;

    rimg = (short *)malloc(imsz*sizeof(short));

    /* Flip top to bottom */
    qq=img; rq=rimg+imsz - xxdim; 
    for(jj=0;jj<yydim;jj++) {
        for(ii=0;ii<xxdim;ii++) *rq++ = *qq++ ;
        rq -= 2*xxdim;
    }

    /* Copy it back. */
    qq=img; rq=rimg;        for(jj=0;jj<imsz;jj++)  *qq++ = *rq++ ;


    free((char *)rimg);

}
int
rot180(img,xxdim,yydim)
    short *img;
    int   xxdim,yydim;
{
    extern char *malloc () ;
    short *rimg, *qq, *rq;
    int ii, jj, imsz;

    imsz=xxdim*yydim;

    rimg = (short *)malloc(imsz*sizeof(short));

    /* Rotate the image by 180 degrees. */
    qq=img; rq=rimg+imsz-1; for(jj=0;jj<imsz;jj++)  *rq-- = *qq++ ;

    /* Copy it back. */
    qq=img; rq=rimg;        for(jj=0;jj<imsz;jj++)  *qq++ = *rq++ ;

    free((char *)rimg);

}
