/* mk4_cpbrot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate a list of MLSO/Mk4 cpb images.
 *------------------------------------------------------------------------------
 * Uses the "cfitsio" library to extract information from the FITS headers.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   28 Aug 2003
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/picard/d/sw/fits/cfitsio/fitsio.h"

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{

    int  usage () ;
    void printerror () ;
    short *rotate_cpbimg () ;

    char *w ;				/* pointer to command line arguments */
    static char comment   [80] ;
    static char cbitpix   [80] ;
    static char cnaxis1   [80] ;
    static char cnaxis2   [80] ;
    static char ccrpix1   [80] ;
    static char ccrpix2   [80] ;
    static char ccrota1   [80] ;
    static char ccrota2   [80] ;
    static char ccoordnam [80] ;
    static char crotang   [80] ;
    static char coordnam  [24] ;
    static char date_obs  [24] ;		/* Observation date. */
    static char dataform  [24] ;		/* Data form. */
    static char file      [80] ;
    static char filerot   [80] ;
    static char filerad   [80] ;
    static char nampgm    [24] ;
    static char object    [24] ;
    static char origin    [24] ;
    static char rotdir     [8] ;
    static char sdir2     [4] = "ru" ;		/* Scan direction: rd or ru. */
    static char telescop  [24] ;
    static char instrume  [24] ;
    static char type_obs  [24] ;
    static char time_obs  [24] ;		/* Observation time. */

    static int  extbuf [168] ;			/* FITS extension data. */
    static long naxes    [2] ;

    unsigned char *imgc ;
    int anynull ;
    int argnum ;
    int bitpix = 16 ;
    int hdutype = 0 ;
    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int ival ;
    int i, ix, iy ;
    int morekeys = 0 ;		/* Additional keywords to be reserved. */
    int naxis = 2 ;
    int nbypix = 2 ;
    int nbyflt = 4 ;
    int ncode = 0 ;
    int nelimg ;
    int nelext = 168 ;
    int status = 0 ;
    int xdim, ydim ;
    int nmax ;

    long fpixel = 1 ;
    long npix ;

    short *simg ;			/* Pointer to image array. */
    short *rimg ;			/* Pointer to rotated image. */
    short *img1, *img2, *img3, *img4, *img5, *img6, *img7 ;
    short smin, smax ;

    float bscale, bzero ;		/* Photometric scaling parameters. */
    float crota1, crota2 ;		/* Coordinate rotation angles. */
    float crradius ;			/* Solar Radius (pixels). */
    float crpix1, crpix2 ;		/* Sun center (pixels). */
    float dmin, dmax ;
    float fmin, fmax ;
    float fval ;
    float nullval = 0 ;			/* Don't check for null values in img.*/
    float protang ;			/* Positive angle to rotate image. */
    float radius ;
    float rotang = 0.0 ;		/* Angle CCW to rotate image. */
    float solar_p0 ;			/* P-angle. */
    float xcen, ycen ;
    float xshift, yshift ;		/* x/y shift for cropping. */
    float wavelen = 0.0 ;

    float *fimg ;

    fitsfile *fp, *fprot ;

    if (argc <= 2) { usage (argv [0]) ; exit (-1) ; }

    strcpy (nampgm, argv [0]) ;		/* Get program name. */
    sscanf (argv [1], "%f", &rotang) ;
    argc--;				/* Skip to next argument. */
    argnum = 1 ;
/*
 *    printf ("argc: %d  argv [1]: %s\n", argc, argv [1]) ;
 *    printf ("rotang: %6.1f\n", rotang);
 *    printf ("argv [argnum]:   %s\n", argv [argnum]) ;
 *    printf ("argv [argnum+1]: %s\n", argv [argnum+1]) ;
 */

    if (rotang == 0.0)
	{
	fprintf (stderr, "rotation angle missing or else set to zero.\n");
	exit (0) ;
	}

    /*** Create a comment for the ROLLADJ keyword. ***/

    protang = fabsf (rotang) ;		/* Absolute value of rotation angle. */
    if (rotang < 0.0)
	strcpy (rotdir, "CW") ;
    else
	strcpy (rotdir, "CCW") ;
    sprintf (crotang, "IMAGE ROTATED %6.1f DEGREES %s.", protang, rotdir) ;

    while (--argc)
	{
	argnum++ ;
	strcpy (file, argv [argnum]) ;
	fprintf (stdout, "file: %s\n", file) ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fp, file, READONLY, &status) )
	    printerror ( status );

	/*** Primary FITS header. ***/

	if (fits_read_key (fp, TINT, "BITPIX", &bitpix,
	    cbitpix, &status))  printerror (status) ;
	nbypix = bitpix / 8 ;

	fprintf (stdout, "nbypix: %d\n", nbypix) ;

	if (fits_read_key (fp, TINT, "NAXIS1", &xdim,
	    cnaxis1, &status))  printerror (status) ;

	if (fits_read_key (fp, TINT, "NAXIS2", &ydim,
	    cnaxis2, &status))  printerror (status) ;

	fprintf (stdout, "xdim/ydim: %4d %4d\n", xdim, ydim) ;

	if (fits_read_key (fp, TSTRING, "ORIGIN", origin,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "TELESCOP", telescop,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "INSTRUME", instrume,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	printf ("iyear: %d  imonth: %d  iday: %d\n", iyear, imonth, iday) ;

	if (fits_read_key (fp, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	printf ("ihour: %d  iminute: %d  isecond: %d\n",
	    ihour, iminute, isecond) ;

	if (fits_read_key (fp, TSTRING, "OBJECT", object,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "TYPE-OBS", type_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TSTRING, "DATAFORM", dataform,
	    comment, &status))  printerror (status) ;

	if (strcmp (dataform, "POLAR") != 0)
	    {
	    fprintf (stderr, "Image is not in polar coordinate form.\n") ;
	    break ;
	    }

	if (fits_read_key (fp, TSTRING, "COORDNAM", coordnam,
	    ccoordnam, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRPIX1", &crpix1,
	    ccrpix1, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CROTA1", &crota1,
	    ccrota1, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRPIX2", &crpix2,
	    ccrpix2, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CROTA2", &crota2,
	    ccrota2, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "BSCALE", &bscale,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fp, TFLOAT, "BZERO", &bzero,
	    comment, &status))  printerror (status) ;

	fprintf (stdout, "bscale/bzero: %12.4e  %12.4e\n", bscale, bzero) ;

	/* Read FITS image data. */

	/* Note that even though the FITS images contains unsigned integer */
	/* pixel values (or more accurately, signed integer pixels with    */
	/* a bias of 32768),  this routine is reading the values into a    */
	/* float array.   Cfitsio automatically performs the datatype      */
	/* conversion in cases like this.                                  */

	if (crpix1 >= 0.0)
	    xcen = crpix1 ;
	else
	    xcen = (xdim - 1) * 0.5 ;

	if (crpix2 >= 0.0)
	    ycen = crpix2 ;
	else
	    ycen = (ydim - 1) * 0.5 ;

	npix = xdim * ydim ;
	fimg = (float *) malloc (npix * sizeof (float)) ;
	simg = (short *) malloc (npix * sizeof (short)) ;
	nullval  = 0 ;		/* Don't check for null values in the image. */

/*
	if ( fits_read_img (fp, TFLOAT, fpixel, npix, &nullval, fimg,
			    &anynull, &status) )
	    printerror ( status ) ;
 */


	if ( fits_read_img (fp, TSHORT, fpixel, npix, &nullval, simg,
			    &anynull, &status) )
	    printerror ( status ) ;


	/*** Close input FITS file. ***/

	if ( fits_close_file (fp, &status) )  printerror (status) ;
	status = 0 ;

        dmin =  1.0e+30 ;
        dmax = -1.0e+30 ;
	for (i = 0; i < npix; i++)
	    {
            if ( fimg [i] < dmin )
                dmin = fimg [i];

	    if ( fimg [i] > dmax )
                dmax = fimg [i];
	    }
	printf ("fimg dmin/dmax: %9.2e  %9.2e\n", dmin, dmax) ;

        for (i = 0; i < npix; i++)
	    {
	    fval = (short) ((fimg [i] - bzero) / bscale) ;
	    if (fval >  16384)  fval =  16384 ;
	    if (fval < -16384)  fval = -16384 ;
	    simg [i] = fval ;
	    }

        smin =  32000 ;
        smax = -32000 ;
	for (i = 0; i < npix; i++)
	    {
            if ( simg [i] < smin )
                smin = simg [i];

	    if ( simg [i] > smax )
                smax = simg [i];
	    }
	printf ("simg smin/smax: %9d  %9d\n", smin, smax) ;

        nmax = 0;
	for (i = 0; i < npix; i++)
	    {
	    if (simg [i] < -32000) simg [i] = -30000 ;
	    if (simg [i] >  32000) simg [i] =  30000 ;
	    if (simg [i] > 10000)  nmax++;
	    }
	fprintf (stdout, "nmax: %d\n", nmax);

	/*** Rotate image to correct for Mk4 barrel angle error. ***/

	if (rotang != 0.0)
	    rimg = (short *) rotate_cpbimg (simg, xdim, ydim, rotang) ;

	for (iy = 0; iy < ydim; iy++)
	    for (ix = 0; ix < xdim; ix++)
		{
		*(rimg + ix + iy*xdim) = (short) (ix * 8) ;
		}

        smin =  32765 ;
        smax = -32765 ;
	for (i = 0; i < npix; i++)
	    {
            if ( rimg [i] < smin )
                smin = rimg [i];

	    if ( rimg [i] > smax )
                smax = rimg [i];
	    }
	printf ("rimg smin/smax: %9d  %9d\n", smin, smax) ;

	/*** Create output file name. ***/

	sprintf (filerot, "%04d%02d%02d.%02d%02d%02d.mk4.cpb.rot.fts",
		 iyear, imonth, iday, ihour, iminute, isecond) ;

	fprintf (stdout, "filerot: %s\n", filerot) ;

	/*** Create output file. ***/

	if ( fits_create_template (&fprot, filerot, file, &status) )
	    printerror ( status ) ;

	if ( fits_update_key (fprot, TINT, "BITPIX", &bitpix, cbitpix,
	                     &status) ) printerror ( status ) ;

	/*** Add ROLLADJ keyword. ***/

	if ( fits_write_key (fprot, TFLOAT, "ROLLADJ", &rotang, crotang,
	                     &status) ) printerror ( status ) ;

	fprintf (stdout, "before fitw_write_img\n") ;
	fflush  (stdout) ;

	/*** Write rotated image array into fits data array. ***/

	nelimg = xdim * ydim ;
	if ( fits_write_img (fprot, TSHORT, fpixel, nelimg, simg, &status) )
	    printerror ( status ) ;

	fprintf (stdout, "after  fitw_write_img\n") ;
	fflush  (stdout) ;

	if (fits_close_file (fprot, &status))  printerror ( status ) ;
        }

   
   exit (0) ;
}

/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */

/* usage (prog_name)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int
usage (prog_name)

char *prog_name ;
{
    static char help [84] ;

    sprintf (help, "%s rotation_angle image1 image2 image3 ...",
	     prog_name) ;
    fprintf (stderr, "%s\n", help) ;
    fprintf (stderr, " Example: %s  9.5  img1.mk4.cpb.fts img2.mk4.cpb.fts\n",
	     prog_name) ;

    return (0) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

