/* mk4tofitsv2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Mark-4 image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   10 August 1999
 * 23 Sep 1999: Add extension which includes mk4 binary header.
 * 19 Jan 2000: Updated items: INSTRUME='DALSA", 
 *  7 Mar 2000: Version 1.10: Modified for revised Mk4 calibration.
 *  8 Mar 2000: Add reffile & drftfile.
 * 10 Mar 2000: Add xscale & yscale, update extension.
 * 15 Mar 2000: Increase storage for mk4buf [1024x1024].
 * 16 Jun 2000: Use mk4_name () to create output FITS file name.
 * 28 Sep 2000: cdelt1/cdelt2 = MK4F->rsun / MK4F->pixrs.
 * 20 Dec 2000: Oops!  bitpix was set to 1 in extension.  Changed it to 8.
 *  8 Jan 2001: Also need EXTEND  = T in primary header.
 * 19 Oct 2001: Add Instrument Description items for Mk4 version 2 calibration.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <string.h>
#include "mk4v2.h"
#include "mk4headv2.h"
#include "mk4fitsv2.h"
#include "img_scale.ext"
#define MAXLIN 180		/* Maximum # lines allocated for FITS header. */
char fitbuf   [MAXLIN][80] ;    /* FITS header buffer. */
extern int nbypix ;		/* Output image: #bytes/pixel */

mk4tofits (namimg, namext, hduflag, pmode)

   char namimg [] ;		/* Name of primary file. */

   char namext [] ;		/* Name of extension file. */

   int hduflag ;		/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */

   int pmode ;			/* Print mode. */

{
   extern char *malloc () ;		/* Allocates memory. */
   extern int rdmk4 () ;		/* Reads Mark-4 image. */
   extern char *mk4_name () ;		/* Creates output FITS file name. */

   static char str [80] ;
   char *sptr;
   char *padbuf ;			/* Pointer to padding buffer array. */
   unsigned char *bbuf ;		/* pointer to a byte buffer. */
   unsigned char *cbuf ;		/* pointer to a byte buffer. */
   static unsigned char mk4buf [1024*1024*sizeof(short)] ;/* mk4 image buffer.*/

   static char kangnorth [12] = "ANGNORTH" ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdataform [12] = "DATAFORM" ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kdiraxis1 [12] = "DIRAXIS1" ;
   static char kdiraxis2 [12] = "DIRAXIS2" ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdrftfile [12] = "DRFTFILE" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char kobject   [12] = "OBJECT  " ;
   static char kphase    [12] = "PHASE   " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char kreffile  [12] = "REFFILE " ;
   static char krmin     [12] = "RMIN    " ;
   static char krmax     [12] = "RMAX    " ;
   static char krsun     [12] = "RSUN    " ;
   static char kscan_dur [12] = "SCAN_DUR" ;
   static char kscan_dir [12] = "SCAN_DIR" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ksolar_b0 [12] = "SOLAR_B0" ;
   static char ktscpfile [12] = "TSCPFILE" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char ktmin     [12] = "TMIN    " ;
   static char ktmax     [12] = "TMAX    " ;
   static char kversion  [12] = "VERSION " ;

   static char atype1 [24] ;			/* Coord type axis 1. */
   static char atype2 [24] ;			/* Coord type axis 2. */
   static char atype1com [24] ;			/* Coord type axis 1 comment. */
   static char atype2com [24] ;			/* Coord type axis 2 comment. */
   static char bunit    [32] ;			/* Brightness unit. */
   static char bunitcom [32] ;			/* Brightness unit comment. */
   static char ctype1   [24] ;			/* Coord type axis 1. */
   static char ctype2   [24] ;			/* Coord type axis 2. */
   static char cunit1   [24] ;			/* Coord units axis 1. */
   static char cunit2   [24] ;			/* Coord units axis 2. */
   static char outbuf   [84] ;			/* output buffer. */
   static char datcom   [24] ;			/* data scale comment. */
   static char cdate    [12] = "CCYY-MM-DD" ;	/* File creation date. */
   static char ctypenam [24] = "GEO-CENTRIC" ;	/* Coordinate System Name. */
   static char date_cal [12] = "CCYY-MM-DD" ;	/* Date image calibrated. */
   static char date_obs [12] = "CCYY-MM-DD" ;	/* Observation date. */
   static char dform    [32] ;			/* Data form. */
   static char dformcom [32] = "";		/* Data form comment. */
   static char diraxis1 [12] = "RIGHT" ;	/* Direction of axis 1: right.*/
   static char diraxis2 [12] = "UP" ;		/* Direction of axis 2: up. */
   static char eq [4] = "= " ;			/* Equal symbol. */
   static char namfit [80] ;			/* FITS file name. */
   static char object [48] ;			/* Object. */
   static char objcom [48] ;			/* Object comment. */
   static char typcom [48] ;			/* type_obs comment. */
   static char pixcom [48] ;			/* data type comment. */
   static char refcom1 [48] ;			/* coord ref comment. */
   static char refcom2 [48] ;			/* coord ref comment. */
   static char scan_dir [24] ;			/* Scan direction. */
   static char ctime    [12] = "HH:MM:SS" ;	/* File creation time. */
   static char time_obs [12] ;			/* Observation time. */
   static char type_obs [72] ;			/* Time of opal cal 0 img.*/
   static char version  [24] ;			/* MK4 version. */

   short *sbuf ;				/* short integer pointer. */
   short sval ;

   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */

   int bitpix ;				/* # bits per pixel. */
   int cimax = 0;			/* Clip input  maximum. */
   int cimin = 0;			/* Clip input  minimum. */
   int dbmax = 0;			/* Data input  maximum [byte]. */
   int dbmin = 0;			/* Data input  minimum [byte]. */
   int dimax = 0;			/* Data input  maximum. */
   int dimin = 0;			/* Data input  minimum. */
   int drftlen = 0;			/* Drift file string length. */
   int dmin = 0 ;			/* Data minimum value. */
   int dmax = 0 ;			/* Data maximum value. */
   int doyflag = 0 ;			/* =1 use DOY, else use month & day. */
   int gcount = 1 ;			/* Extension gcount. */
   int iyear ;				/* year - 1900. */
   int i, ix, iy ;			/* index values. */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* integer value. */
   int iynum ;				/* # lines in FITS header. */
   int ndim = 0 ;			/* # dimensions for image (2=XY). */
   int nhdu ;				/* # HDU blocks to write. */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyrec = 2880 ;			/* # bytes per FITS logical record. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int nrec ;				/* # fits records for image. */
   int nelwr  = 0 ;			/* # elements written by fwrite. */
   int pcount = 0 ;			/* Extension pcount. */
   int reflen = 0;			/* Reference file string length. */
   int swrite = 0 ;
   int tscplen = 0;			/* Telescope file string length. */
   int wmaxo = 0;			/* Display output maximum. */
   int wmino = 0;			/* Display output minimum. */

   float angnorth =  0.0 ;		/* solar north w.r.t. +Y axis. */
   float bscale = 1.0e-10 ;		/* Brightness intensity scale factor. */
   float bzero  =     0.0 ;		/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float crpix1, crpix2 ;		/* Reference pixel location.*/
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dsmin  = 0.0 ;			/* Data minimum value [scaled]. */
   float dsmax  = 0.0 ;			/* Data maximum value [scaled]. */
   float fval  = 0.0 ;			/* floating point value. */
   float phase, pangle ;		/* MK4 phase angle & p-angle. */
   float r0, dr ;			/* pixel 0 radius, r between pixels. */
   float dt ;				/* angle between rays. */
   float rad0, ang0 ;			/* radius/theta center coordinates. */
   float slope = 1.0 ;			/* Slope. */
   float soff = 0.0 ;			/* Offset. */
   float wmin = 0.0 ;			/* Display min value. */
   float wmax = 0.0 ;			/* Display max value. */
   float wsmin = 0.0 ;			/* Display min value [scaled]. */
   float wsmax = 0.0 ;			/* Display max value [scaled]. */
   FILE *FF ;				/* FITS file ID. */

   swrite = 0 ;
   sprintf (namfit, "%s.fts", namimg) ;

   if (pmode)
      {
      fprintf (stdout, "mk4tofits> namimg: %s  namext: %s   hduflag: %d\n",
	       namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- mk4tofits> %s -> %s -------\n\n",
	       namimg, namfit) ;
      }

   /*** Read MLSO Mark-4 image/data from disk into memory array. ***/

   sbuf = (short *) mk4buf ;
   if (hduflag == 1 || hduflag == 2)
      rdmk4 (namimg, mk4h, sbuf, pmode) ; /* Read Mk4 dataset into memory. */
   else
      rdmk4 (namext, mk4h, sbuf, pmode) ; /* Read Mk4 dataset into memory. */

   strcpy (namfit, mk4_name (doyflag)) ;
   fprintf (stdout, "%s\n", namfit) ;

   nx = MK4F->xdim ;			/* X dimension. */
   ny = MK4F->ydim ;			/* Y dimension. */
   npximg = nx * ny ;			/* # pixels in image. */
   nbyimg = npximg * MK4F->nbypx ;	/* # bytes in image. */
   bscale = MK4F->bscale ;
   bzero  = MK4F->bzero  ;

   /*** Compute the min/max intensity values in the image/array. ***/

   if (MK4F->dmin == 0 & MK4F->dmax == 0)
      {
      dimin =  65535 ;
      dimax = -65535 ;
      for (i = 0; i < npximg; i++)
	 {
	 ival = *(sbuf + i) ;
         if (ival < dimin) dimin = ival ;
         if (ival > dimax) dimax = ival ;
	 }
      MK4F->dmin = dimin ;
      MK4F->dmax = dimax ;
      }

   dmin = MK4F->dmin ;			/* Default data minimum value. */
   dmax = MK4F->dmax ;			/* Default data minimum value. */
   wmino = MK4F->smin ;			/* Default display minimum value. */
   wmaxo = MK4F->smax ;			/* Default display maximum value. */

   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			nx, ny, MK4F->nbypx, npximg, nbyimg) ;

   iyear = MK4F->year ;
/*   if (MK4F->year > 1900)  iyear = MK4F->year - 1900 ; */
   if (MK4F->year < 1900)
      {
      if (MK4F->year >= 79)  iyear += 1900 ;
      else		     iyear += 2000 ;
      MK4F->year = iyear ;
      }

   sprintf (date_obs, "%04d-%02d-%02d", iyear, MK4F->month, MK4F->day) ;
   sprintf (time_obs, "%02d:%02d:%02d", MK4F->hour, MK4F->minute, MK4F->second) ;

   iyear = MK4F->cal_year ;
   if (MK4F->cal_year < 1900 && MK4F->cal_year != 0)
      {
      if (MK4F->cal_year >= 79)  iyear += 1900 ;
      else	 	         iyear += 2000 ;
      MK4F->cal_year = iyear ;
      }
   sprintf (date_cal, "%04d-%02d-%02d",
	    iyear, MK4F->cal_month, MK4F->cal_day) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   if (icyear < 1900 && icyear >= 79)  icyear += 1900 ;
   else				       icyear += 2000 ;
   if (pmode)
      fprintf (stdout, "ic [0]: %d  icyear: %d\n", ic [0], icyear) ;
   sprintf (cdate, "%04d-%02d-%02d", icyear, icmonth, icday) ;
   sprintf (ctime, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   if (MK4F->scndir == -1)
      strcpy (scan_dir, "CW") ;
   else
   if (MK4F->scndir ==  1)
      strcpy (scan_dir, "CCW") ;
   else
      strcpy (scan_dir, "") ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (MK4F->xdim > 0)
      ndim++ ;
   if (MK4F->ydim > 1)
      ndim++ ;

   /*** Set pixel data structure comment to blanks. ***/

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;

   /*** Set default (Cartesian) coordinate parameters. ***/

   strcpy (bunitcom, "") ;
   strcpy (object,   "CORONA") ;   
   strcpy (objcom,   "WHITE LIGHT [700-1080 NANOMETERS]") ;
   strcpy (type_obs, "W.LIGHT") ;   
   strcpy (typcom,   "") ;   
   strcpy (datcom, "[BSUN]") ;		/* DATAMIN/DATAMAX UNITS. */
   crpix1 = MK4F->xcen ;		/* X sun center [pixels]. */
   crpix2 = MK4F->ycen ;		/* Y sun center [pixels]. */

/*   crpix1 = (float) ((nx - 1) / 2) + 0.5 ; */	/* X sun center [pixels]. */
/*   crpix2 = (float) ((ny - 1) / 2) + 0.5 ; */	/* Y sun center [pixels]. */
   crval1 = 0.0 ;			/* X sun center [Rsun]. */
   crval2 = 0.0 ;			/* Y sun center [Rsun]. */
   cdelt1 = 1.0 / MK4F->pixrs ;		/* # Rsun per pixel in x-axis. */
   cdelt2 = 1.0 / MK4F->pixrs ;		/* # Rsun per pixel in y-axis. */
   cdelt1 = 1.0 / MK4F->xscale ;	/* X-axis scale. */
   cdelt2 = 1.0 / MK4F->yscale ;	/* Y-axis scale. */
   cdelt1 = MK4F->rsun / MK4F->pixrs;	/* arcsec/pixel. */
   cdelt2 = MK4F->rsun / MK4F->pixrs;	/* arcsec/pixel. */
   crota1 = 0.0 ;			/* Rotation of x-axis w.r.t. ctype1. */
   crota2 = 0.0 ;			/* Rotation of y-axis w.r.t. ctype2. */

   /*** Reference point:
    *** Cartesian coordinates: sun center.
    *** Polar coordinates: radius = 0, theta = 0. 
    ***/

   strcpy (refcom1, "SUN CENTER") ;
   strcpy (refcom2, "SUN CENTER") ;
   strcpy (atype1com, "") ;
   strcpy (atype2com, "") ;

   phase  = (float) MK4F->phase ;
   pangle = (float) MK4F->pangle ;
   r0 = (float) mk4h->r0 * 1.0e-4 ;	/* Pixel 0 radius (Rsun). */
   dr = (float) mk4h->dr * 1.0e-7 ;	/* Pixel increment (Rsun). */
   dt = 17.0 * ba2deg ;			/* angular increment (deg) for rays. */
   rad0 = - r0 / dr ;			/* Sun center w.r.t. pixel #0 [index].*/
   if (MK4F->tmin < 0)
      ang0 = (  0.0 - MK4F->tmin) / dt ;	/* Theta = 0 in index units. */
   else
      ang0 = (360.0 - MK4F->tmin) / dt ;	/* Theta = 0 in index units. */

   strcpy (cunit1, "ARCSEC") ;
   strcpy (cunit2, "ARCSEC") ;
   sprintf (version, "MK4 1.10") ;

   /*** Define various parameters, based upon MK4 data type:
     *  object
     *  type_obs
     *  brightness units
     *  coordinate type, reference, increment
     *  bitpix
     *  display scaling
     */

   if (MK4F->ptype == IS_cal_rect_byte)
      {
      bitpix = 8 ;			/* unsigned char */
      strcpy (pixcom,   "1-BYTE UNSIGNED INTEGERS") ;
      }
   else
      {
      bitpix =  16 ;			/* short integer */
      strcpy (pixcom,   "2-BYTE SIGNED INTEGERS") ;
      }

   /*** Set Observation Type. ***/

   switch (MK4F->ptype)
      {
      case IS_raw          :
      case IS_raw_spln     :
      case IS_raw_rect     :
      case IS_raw_avg      :
      case IS_raw_avg_spln :
      case IS_raw_avg_rect :
			switch (MK4F->dtype)
			   {
			   case IS_i  : strcpy (type_obs, "I [RAW]") ;
					strcpy (typcom,   "STOKES I VECTOR") ;
	       				strcpy (bunit,    "COUNTS") ;
	       				strcpy (bunitcom, "[NOT CALIBRATED]") ;
               				wmino = 2500 ;  wmaxo = 5000 ;
					break ;
			   case IS_q  : strcpy (type_obs, "Q [RAW]") ;
					strcpy (typcom,   "STOKES Q VECTOR") ;
	       				strcpy (bunit,    "COUNTS") ;
	       				strcpy (bunitcom, "[NOT CALIBRATED]") ;
               				wmino = -250 ;  wmaxo = 2000 ;
					break ;
			   case IS_u  : strcpy (type_obs, "U [RAW]") ;
					strcpy (typcom,   "STOKES U VECTOR") ;
	       				strcpy (bunit,  "COUNTS") ;
	       				strcpy (bunitcom, "[NOT CALIBRATED]") ;
               				wmino = -125 ;  wmaxo = 125 ;
					break ;
			   case IS_ba : strcpy (type_obs, "BA") ;
					strcpy (typcom,   "BARREL ANGLES");
	       				strcpy (object,   "BARREL ANGLES") ;
	       				strcpy (objcom,   "AZIMUTHAL POSITION");
	       				strcpy (bunit,    "DEGREES") ;
					wmino = MK4F->dmin ;
					wmaxo = MK4F->dmax ;
					break ;
			   case IS_st : strcpy (type_obs, "ST") ;
					strcpy (typcom,   "SCAN TIMES") ;
	       				strcpy (object,   "SCAN TIMES") ;
	       				strcpy (objcom,   "TIMES FOR EACH RADIAL SCAN") ;
	       				strcpy (bunit,    "TIME") ;
					wmino = MK4F->dmin ;
					wmaxo = MK4F->dmax ;
					break ;
			   case IS_sk : strcpy (type_obs, "SK") ;
					strcpy (typcom,   "SKY BRIGHTNESS") ;
	       				strcpy (object,   "SKY BRIGHTNESS") ;
	       				strcpy (objcom,   "SKY TRANSMISSION LEVEL") ;
	       				strcpy (bunit,    "COUNTS") ;
	       				strcpy (bunitcom, "[NOT CALIBRATED]") ;
					wmino = MK4F->dmin ;
					wmaxo = MK4F->dmax ;
					break ;
			   default    : strcpy (type_obs, "UNKNOWN") ;
					strcpy (typcom,   "") ;
	       				strcpy (object,   "UNKNOWN") ;
	       				strcpy (objcom,   "") ;
	       				strcpy (bunit,    "UNKNOWN") ;
			   }
			break ;

      case IS_cal               :
      case IS_cal_slit          :
      case IS_cal_avg           :
      case IS_cal_avg_slit      :
      case IS_cal_rect          :
      case IS_cal_rect_slit     :
      case IS_cal_rect_byte     :
      case IS_cal_avg_rect      :
      case IS_cal_avg_rect_slit :
			switch (MK4F->dtype)
			   {
			   case IS_i   : strcpy (type_obs, "I [CAL]") ;
					 strcpy (typcom,   "STOKES I VECTOR") ;
	       				 strcpy (bunit,    "BSUN") ;
               				 wmino = 0 ;  wmaxo = 2000 ;
					 break ;
			   case IS_q   : strcpy (type_obs, "pB") ;
					 strcpy (typcom,   "POLARIZATION BRIGHTNESS") ;
	       				 strcpy (bunit,    "BSUN") ;
               				 wmino =  -250 ;  wmaxo = 6000 ;

					 if (MK4F->ptype == IS_cal_slit ||
					     MK4F->ptype == IS_cal_avg_slit)
					    {
					    strcpy (type_obs, "pB-VIGNET") ;
					    strcat (bunit,    "*VIGNET") ;
	       				    strcpy (bunitcom, "[QUALITATIVE]") ;
					    wmino = -100 ;  wmaxo = 650 ;
					    }
					 if (MK4F->ptype == IS_cal_rect)
					    {
					    strcpy (type_obs, "pB") ;
               				    wmino = -250 ;  wmaxo = 6000 ;
					    }
					 if (MK4F->ptype == IS_cal_rect_slit)
					    {
					    strcpy (type_obs, "pB-VIGNET") ;
					    strcat (bunit,    "*VIGNET") ;
	       				    strcpy (bunitcom, "[QUALITATIVE]") ;
					    wmino =    0 ;  wmaxo = 10000 ;
               				    wmino = -250 ;  wmaxo = 10000 ;
					    wmino = -100 ;  wmaxo = 650 ;
					    }
					 if (MK4F->ptype == IS_cal_avg_rect_slit)
					    {
					    strcpy (type_obs, "pB-VIGNET") ;
					    strcat (typcom,   " [VIGNETTED]") ;
					    strcat (bunit,    "*VIGNET") ;
	       				    strcpy (bunitcom, "[QUALITATIVE]") ;
               				    wmino =    0 ;  wmaxo = 6000 ;
					    wmino = -100 ;  wmaxo = 650 ;
					    }
					 break ;
			   case IS_u   : strcpy (type_obs, "U [CAL]") ;
					 strcpy (typcom,   "STOKES U VECTOR") ;
	       				 strcpy (bunit,    "BSUN") ;
               				 wmino = -250 ;  wmaxo = 1000 ;
					 break ;
			   case IS_tscp: strcpy (type_obs, "TSCP") ;
					 strcpy (typcom,   "TELESCOPE CALIBRATION MATRIX") ;
	       				 strcpy (object,   "TSCP INFO") ;
	       				 strcpy (objcom,   "TELESCOPE CALIBRATION MATRIX") ;
	       				 strcpy (bunit,    "NONE") ;
					 break ;
			   default    :  strcpy (type_obs, "UNKNOWN") ;
					 strcpy (typcom,   "") ;
	       				 strcpy (object,   "UNKNOWN") ;
	       				 strcpy (objcom,   "") ;
	       				 strcpy (bunit,    "UNKNOWN") ;
			   }
			break ;

      default :		strcpy (type_obs, "UNKNOWN") ;
	       		strcpy (bunit,    "UNKNOWN") ;
      }

   /*** For average images, append "avg" to type_obs. ***/

   switch (MK4F->ptype)
      {
      case IS_raw_avg		:
      case IS_raw_avg_spln	:
      case IS_raw_avg_rect	:
      case IS_cal_avg_rect      :
      case IS_cal_avg_rect_slit :
				strcat (type_obs, "-AVG") ;
				strcat (typcom,   "[AVERAGE]") ;
      default : ;
      }

   /*** Set coordinate type name. ***/

   switch (MK4F->ptype)
      {
      /* Polar Coordinate Data, Non-Spline Fitted. */

      case IS_raw         	:
      case IS_raw_avg     	:
         switch (MK4F->dtype)
	   {
           case IS_i  :
	   case IS_q  :
	   case IS_u  :
		strcpy (ctypenam, "GEO-CENTRIC") ;
		strcpy (dform, "POLAR") ;
		strcpy (atype1, "POSITION ANGLE") ;
		strcpy (atype1com, "[CCW]") ;
		strcpy (atype2, "RADIUS") ;
		strcpy (refcom1, "GEO NORTH") ;
		strcpy (refcom2, "SUN CENTER") ;
/*
 *	       	crpix1 = rad0 ;
 */
/*		crpix2 = ang0 ; */ /* Sun center location. */
	       	crval1 = 0.0 ;
		crval2 = 0.0 ;  /* coord value at sun center. */
 	       	cdelt1 = 0.5 ;	/* degrees/pixel */
/*
 *	       	cdelt2 = dr ;
 */
               	strcpy (cunit1,   "DEGREES") ;
               	strcpy (cunit2,   "ARCSEC") ;
	       	crota1 = - phase - pangle ;
		crota2 = - phase - pangle ;
		break ;
	   case IS_ba :
	        strcpy (ctypenam, "LINEAR") ;
		strcpy (dform, "TABLE") ;
		crpix1 = 0.0 ;
		crpix2 = 0.0 ;
		crval1 = 0.0 ;
		crval2 = 0.0 ;
		cdelt1 = 1.0 ;
/*
 *		cdelt2 = dt ;
 */
		strcpy (atype1, "SCAN POSITION") ;
		strcpy (atype2, "") ;
               	strcpy (cunit1, "INDEX") ;
               	strcpy (cunit2, "") ;
		break ;
	   case IS_st :
	        strcpy (ctypenam, "LINEAR") ;
		strcpy (dform, "TABLE") ;
		crpix1 = 0.0 ;
		crpix2 = 0.0 ;
		crval1 = 0.0 ;
		crval2 = 0.0 ;
		cdelt1 = 1.0 ;
		cdelt2 = 1.0 ;
		strcpy (atype1, "") ;
		strcpy (atype2, "SCAN TIME") ;
               	strcpy (cunit1, "INDEX") ;
               	strcpy (cunit2, "") ;
		break ;
	   case IS_sk :
	        strcpy (ctypenam, "LINEAR") ;
		strcpy (dform, "TABLE") ;
		crpix1 = 0.0 ;
		crpix2 = 0.0 ;
		crval1 = 0.0 ;
		crval2 = 0.0 ;
		cdelt1 = 1.0 ;
		cdelt2 = 1.0 ;
		strcpy (atype1, "SKY TRANSMISSION") ;
		strcpy (atype2, "") ;
               	strcpy (cunit1, "INDEX") ;
               	strcpy (cunit2, "") ;
		break ;
            }
         break ;

      /* Polar Coordinate Data, Spline Fitted. */

      case IS_raw_spln    	:
      case IS_raw_avg_spln	:

      case IS_cal               :
      case IS_cal_slit          :
      case IS_cal_avg           :
      case IS_cal_avg_slit      :
	switch (MK4F->dtype)
	    {
	    case IS_i  :
	    case IS_q  :
	    case IS_u  :
		strcpy (ctypenam, "HELIO-CENTRIC") ;
		strcpy (dform, "POLAR") ;
		strcpy (atype1, "POSITION ANGLE") ;
		strcpy (atype1com, "[CCW]") ;
		strcpy (atype2, "RADIUS") ;
		strcpy (refcom1, "SOLAR NORTH") ;
		strcpy (refcom2, "SUN CENTER") ;

		crval1 = 0.0 ;
		crval2 = 0.0 ;  /* coord value at sun center. */
 	       	cdelt1 = 0.5 ;	/* degree/pixel */
		strcpy (cunit1,   "DEGREES") ;
		strcpy (cunit2,   "ARCSEC") ;
		crota1 = 0.0 ;
		crota2 = 0.0 ;
		break ;

	    case IS_ba :
		strcpy (ctypenam, "GEO-LINEAR") ;
		strcpy (dform, "TABLE") ;
		crpix1 = 0.0 ;
		crpix2 = 0.0 ;
		crval1 = 0.0 ;
		crval2 = 0.0 ;
		cdelt1 = 1.0 ;
/*
 *		cdelt2 = dt ;
 */
		strcpy (atype1, "SCAN POSITION") ;
		strcpy (atype2, "") ;
		strcpy (cunit1, "INDEX") ;
		strcpy (cunit2, "") ;
		break ;

	    case IS_st :
		strcpy (ctypenam, "GEO-LINEAR") ;
		strcpy (dform, "TABLE") ;
		crpix1 = 0.0 ;
		crpix2 = 0.0 ;
		crval1 = 0.0 ;
		crval2 = 0.0 ;
		cdelt1 = 1.0 ;
		cdelt2 = 1.0 ;
		strcpy (atype1, "SCAN TIME") ;
		strcpy (atype2, "") ;
		strcpy (cunit1, "INDEX") ;
		strcpy (cunit2, "") ;
		break ;
	    case IS_sk :
		strcpy (ctypenam, "GEO-LINEAR") ;
		strcpy (dform, "TABLE") ;
		crpix1 = 0.0 ;
		crpix2 = 0.0 ;
		crval1 = 0.0 ;
		crval2 = 0.0 ;
		cdelt1 = 1.0 ;
		cdelt2 = 1.0 ;
		strcpy (atype1, "SKY TRANSMISSION") ;
		strcpy (atype2, "") ;
		strcpy (cunit1, "INDEX") ;
		strcpy (cunit2, "") ;
		break ;
	    default    : ;
	    }
	break ;

      case IS_raw_rect    	:
      case IS_raw_avg_rect	:
      case IS_cal_rect          :
      case IS_cal_rect_slit     :
      case IS_cal_rect_byte     :
      case IS_cal_avg_rect      :
      case IS_cal_avg_rect_slit :
		strcpy (ctypenam, "HELIO-CENTRIC") ;
		switch (MK4F->dtype)
		   {
		   case IS_i  :
		   case IS_q  :
		   case IS_u  :
			strcpy (dform, "CARTESIAN") ;
			strcpy (atype1, "X") ;
			strcpy (atype2, "Y") ;
			break ;
		   case IS_ba :
			strcpy (dform, "TABLE") ;
			strcpy (atype1, "BARREL ANGLE") ;
			strcpy (atype2, "") ;		break ;
		   case IS_st :
			strcpy (dform, "TABLE") ;
			strcpy (atype1, "SCAN TIME") ;
			strcpy (atype2, "") ;		break ;
		   case IS_sk :
			strcpy (dform, "TABLE") ;
			strcpy (atype1, "SKYTX") ;
			strcpy (atype2, "") ;		break ;
		   default    :
			strcpy (dform, "UNKNOWN");
		   }
		break ;

      default :			strcpy (ctypenam, "UNKNOWN") ;
				strcpy (dform, "UNKNOWN");
      }

   sprintf (ctype1, "%s", atype1) ;
   sprintf (ctype2, "%s", atype2) ;

   /*** Convert to 8-bits/pixel. ***/

   if (nbypix == 1)
      {
      fprintf (stdout, "Image converted to 8-bits/pixel.\n") ;
      strcpy (pixcom,   "1-BYTE UNSIGNED PIXELS") ;
      strcpy (bunit,    "DATA") ;
      bitpix =  8 ;
      MK4F->nbypx = 1 ;
      nbyimg = npximg * MK4F->nbypx ;	/* # bytes in image. */

      /*** Convert from short to unsigned char. ***/

      bscale = 1.0 ;
      bzero  = 0.0 ;
      dbmin =    0 ;
      dbmax =  249 ;
      wmin = dbmin ;
      wmax = dbmax ;

      if (lscale == 1)
	 {
	 dimin = foldmin ;
	 dimax = foldmax ;
	 }
      else
	 {
         dimin = wmino ;
         dimax = wmaxo ;
	 }
      dmin = dimin ;
      dmax = dimax ;

      /*** Perform intensity scaling. ***/

      bbuf = mk4buf ;
      cbuf = (unsigned char *) malloc (npximg * sizeof (unsigned char)) ;

      soff  = dimin ;
      slope = (float) (dbmax - dbmin) / (float) (dimax - dimin) ;

/*
 *      dmin =  32767 ;
 *      dmax = -32767 ;
 */
      for (i = 0; i < npximg; i++)
	 {
         fval = ((float) *(sbuf + i) - soff) * slope ;
	 sval = (short) fval ;
         if (sval < dbmin)  sval = dbmin ;
         if (sval > dbmax)  sval = dbmax ;
/*
 *	 if (sval < dmin)   dmin = sval ;
 *	 if (sval > dmax)   dmax = sval ;
 */
	 *(cbuf + i) = (unsigned char) sval ;
	 *(sbuf + i) = 0 ;
	 }

      for (i = 0; i < npximg; i++)
	 *(bbuf + i) = *(cbuf + i) ;	/* Copy cbuf into mk4buf. */

      free (cbuf) ;			/* Discard storage for cbuf. */
      }
   else
      {
      wmin = wmino ;
      wmax = wmaxo ;
      }

   MK4F->dmin = dmin ;
   MK4F->dmax = dmax ;
   dsmin  = MK4F->dmin * MK4F->bscale + MK4F->bzero ;
   dsmax  = MK4F->dmax * MK4F->bscale + MK4F->bzero ;

   MK4F->smin = wmin ;
   MK4F->smax = wmax ;
   wsmin = (float) wmin * bscale + bzero ;
   wsmax = (float) wmax * bscale + bzero ;

   MK4F->bscale = bscale ;
   MK4F->bzero  = bzero ;

   /*** Create FITS file. ***/

   if (hduflag == 3)				/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- FIRST HDU ---\n") ;

   /*** Fill FITS header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   strcpy  (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: MLSO MARK-4 K-CORONAMETER") ;
   else
   sprintf (&fitbuf [iy++][0],  "XTENSION= 'IMAGE   '           / MLSO MARK-4 %s EXTENSION", dform) ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, ny, atype2) ;

   strcpy  (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;

   /*** Extension information. ***/

   if (hduflag == 2)
      {
      strcpy  (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      }
   else
   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;
      }

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "ORIGIN  = 'HAO'") ;
   sprintf (&fitbuf [iy++][31], "/ HIGH ALTITUDE OBSERVATORY (HAO)") ;
   strcpy  (&fitbuf [iy  ][0],  "TELESCOP= 'MK4'") ;
   strcpy  (&fitbuf [iy++][31], "/ MARK-4 K-CORONAMETER (MK4)") ;
   strcpy  (&fitbuf [iy  ][0],  "INSTRUME= 'DALSA'") ;
   strcpy  (&fitbuf [iy++][31], "/ ROTATING 2048-ELEMENT DALSA LINEAR ARRAY") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: CCYY-MM-DD [%04d DOY %03d]",
				MK4F->year, MK4F->doy) ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", ktime_obs, eq, time_obs) ;
   strcpy  (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ %s", objcom) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ %s", typcom) ;

   /*** Data Format. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kdataform, eq, dform) ;
   sprintf (&fitbuf [iy++][31], "/ MK4 DATA FORM %s", dformcom) ;

   /*** Coordinate System Name: geocentric or heliocentric, etc. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s %s", atype1, atype1com) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / %s %s [PIXEL]",
			      kcrpix1  , eq, crpix1, atype1, refcom1) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom1, cunit1) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6g / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s ROTATION [DEG.] FROM REFERENCE", 
				kcrota1, eq, crota1, atype1) ;

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s %s", atype2, atype2com) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [PIXEL]",
			           kcrpix2, eq, crpix2, atype2, refcom2) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom2, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.6g / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2, eq, cdelt2, atype2, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s ROTATION [DEG.] FROM REFERENCE", 
				   kcrota2, eq, crota2, atype2) ;
      }

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / PIXELS PER SOLAR RADIUS", 
				 kcrradius, eq, MK4F->pixrs) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR RADIUS  [ARCSEC]",
                                krsun, eq, MK4F->rsun) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
                                ksolar_p0, eq, MK4F->pangle) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
                                ksolar_b0, eq, MK4F->bangle) ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'",     kbunit, eq, bunit) ;
   sprintf (&fitbuf [iy++][31], "/ MAGNITUDE UNITS %s", bunitcom) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6e / PHYSICAL = DATA * BSCALE + BZERO",
			       kbscale, eq, bscale) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6e", kbzero , eq, bzero ) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6e / MINIMUM DATA VALUE (%6d)",
			       kdatamin , eq, dsmin, MK4F->dmin) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6e / MAXIMUM DATA VALUE (%6d)",
			       kdatamax , eq, dsmax, MK4F->dmax) ;

   /*** All keywords after this point are unique to the MLSO MK4. ***/

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6e / MIN. DISPLAY VALUE (%6d)",
			        kdispmin , eq, wsmin, MK4F->smin, datcom) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6e / MAX. DISPLAY VALUE (%6d)",
			        kdispmax , eq, wsmax, MK4F->smax, datcom) ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kdiraxis1, eq, diraxis1) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 1 [X]") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kdiraxis2, eq, diraxis2) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 2 [Y]") ;

/*
 *   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'",     kscan_dir,   eq, scan_dir) ;
 *   strcpy  (&fitbuf [iy++][31], "/ SCAN DIRECTION: CW=CLOCKWISE, CCW=COUNTER-CW.") ;

 *   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / SCAN DURATION [SECONDS]",
 *			        kscan_dur, eq, MK4F->scndur) ;
 */

   switch (MK4F->ptype)
      {				/* Polar Coordinate Images. */

      case 10:			/* IS_raw          */
      case 11:			/* IS_raw_spln     */
      case 13:			/* IS_cal          */
      case 14:			/* IS_cal_slit     */
      case 18:			/* IS_cal_avg      */
      case 19:			/* IS_cal_avg_slit */
      case 24:			/* IS_raw_avg      */
      case 25:			/* IS_raw_avg_spln */

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / START BARREL ANGLE [DEGREES]",
		                ktmin, eq, MK4F->tmin) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.2f / STOP  BARREL ANGLE [DEGREES]",
		                ktmax, eq, MK4F->tmax) ;
               break ;
      default:
	       break ;
      }

   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6g / INNER FIELD-OF-VIEW LIMIT [RSUN]",
			        krmin,     eq, MK4F->rmin) ;
   sprintf (&fitbuf [iy++][0], "%-8s%2s%20.6g / OUTER FIELD-OF-VIEW LIMIT [RSUN]",
			        krmax,     eq, MK4F->rmax) ;

   switch (MK4F->ptype)		/* Calibrated data. */

      {
      case IS_cal               :
      case IS_cal_slit          :
      case IS_cal_rect          :
      case IS_cal_rect_slit     :
      case IS_cal_rect_byte     :
      case IS_cal_avg           :
      case IS_cal_avg_slit      :
      case IS_cal_avg_rect      :
      case IS_cal_avg_rect_slit :

         tscplen = strlen (MK4F->tscpfile) + 13 ;
         sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",
		  ktscpfile, eq, MK4F->tscpfile ) ;
         if (tscplen > 31)
   	   strcpy  (&fitbuf [iy++][tscplen], "/ TSCP FILE") ;
         else
   	   strcpy  (&fitbuf [iy++][31], "/ TSCP FILE") ;

         reflen = strlen (MK4F->reffile) + 13 ;
         sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",
		  kreffile, eq, MK4F->reffile ) ;
         if (reflen > 31)
   	   strcpy  (&fitbuf [iy++][reflen], "/ REF. FILE") ;
         else
   	   strcpy  (&fitbuf [iy++][31], "/ REFERENCE FILE") ;

         drftlen = strlen (MK4F->drftfile) + 13 ;
         sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",
		  kdrftfile, eq, MK4F->drftfile ) ;
         if (drftlen > 31)
   	   strcpy  (&fitbuf [iy++][drftlen], "/ DRIFT FILE") ;
         else
   	   strcpy  (&fitbuf [iy++][31], "/ DRIFT FILE") ;

         sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kdate_cal, eq, date_cal) ;
         strcpy  (&fitbuf [iy++][31],
		  "/ MK4 CALIBRATION DATE :       CCYY-MM-DD") ;
         break ;

      default :
         break ;
      }

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kdate,     eq, cdate) ;
   strcpy  (&fitbuf [iy++][31], "/ DATE OF FITS FILE CREATION : CCYY-MM-DD") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kversion,  eq, version) ;
   strcpy  (&fitbuf [iy++][31], "/ MK4 VERSION NUMBER") ;

   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;


   switch (MK4F->dtype)
      {
      case IS_i:
      case IS_q:
      case IS_u:

         strcpy  (&fitbuf [iy++][0],  "COMMENT   FOR IMAGES, PIXELS ARE DISPLAYED WITH THE ORIGIN AT THE LOWER LEFT") ;
         strcpy  (&fitbuf [iy++][0],  "COMMENT   CORNER; I.E., HORIZONTAL SCANS (LEFT TO RIGHT), WITH THE FIRST SCAN") ;
         strcpy  (&fitbuf [iy++][0],  "COMMENT   AT THE BOTTOM OF THE IMAGE, & THE LAST SCAN AT THE TOP.") ;

         switch (MK4F->ptype)
            {
            /* Cartesian Coordinate Images. */

            case 12:			/* IS_raw_rect          */
            case 15:			/* IS_cal_rect          */
            case 16:			/* IS_cal_rect_slit     */
            case 17:			/* IS_cal_rect_byte     */
            case 20:			/* IS_cal_avg_rect      */
            case 21:			/* IS_cal_avg_rect_slit */
            case 26:			/* IS_raw_avg_rect      */

               strcpy  (&fitbuf [iy++][0],  "COMMENT   AXIS 1 IS EAST-WEST AND AXIS 2 IS NORTH-SOUTH.") ;
               strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
	       break ;
         
            /* Polar Coordinate Images. */
      
            case 10:			/* IS_raw          */
            case 11:			/* IS_raw_spln     */
            case 13:			/* IS_cal          */
            case 14:			/* IS_cal_slit     */
            case 18:			/* IS_cal_avg      */
            case 19:			/* IS_cal_avg_slit */
            case 24:			/* IS_raw_avg      */
            case 25:			/* IS_raw_avg_spln */
      
               strcpy  (&fitbuf [iy++][0],  "COMMENT   AXIS 1 IS THE AZIMUTHAL AXIS AND AXIS 2 IS THE RADIAL AXIS.") ;
      
               strcpy  (&fitbuf [iy++][0],  "COMMENT   TMIN & TMAX INDICATE THE START/STOP ANGLES FOR THE AZIMUTHAL AXIS.") ;
      
               strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
               break ;
            default:
	       break ;
            }

         switch (MK4F->ptype)		/* Calibrated images. */
            {
            case IS_cal:
            case IS_cal_slit:
            case IS_cal_rect:
            case IS_cal_rect_slit:
            case IS_cal_avg:
            case IS_cal_avg_slit:
            case IS_cal_avg_rect:
            case IS_cal_avg_rect_slit:
      
               strcpy  (&fitbuf [iy++][0],  "COMMENT   CALIBRATED BRIGHTNESS MEASUREMENTS IN THE MARK-4 DATA ARE SPECIFIED") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   IN UNITS OF BSUN, WHICH IS A RATIO OF BRIGHTNESS OF THE CORONA TO THE") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   MEAN BRIGHTNESS OF THE SOLAR DISK.  TO CONVERT INTEGER PIXEL VALUES") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   TO BSUN UNITS, MULTIPLY EACH PIXEL VALUE BY BSCALE & ADD BZERO.") ;
               strcpy  (&fitbuf [iy++][0],  "COMMENT   INTENSITY DATA IS *QUANTITATIVELY* USEFUL ONLY WHEN BUNIT='BSUN'.     ") ;
               strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
      	       break ;
            default:
      	       break ;
            }

         break ;

      case IS_ba:
         strcpy  (&fitbuf [iy++][0],  "COMMENT   AXIS 1 IS THE BARREL ANGLE AXIS.") ;
	 break ;

      case IS_sk:
         strcpy  (&fitbuf [iy++][0],  "COMMENT   AXIS 1 IS THE SKY TRANSMISSION AXIS.") ;
	 break ;

      case IS_st:
         strcpy  (&fitbuf [iy++][0],  "COMMENT   AXIS 1 IS THE SCAN TIMES AXIS.") ;
	 break ;

      default:
         break ;
      }


   strcpy  (&fitbuf [iy++][0],  "COMMENT   THE MARK-4 K-CORONAMETER (MK4) OBSERVES THE LOWER CORONA [700-950 NM]") ;
/*
 *   sprintf (&fitbuf [iy++][0],  "COMMENT   FROM %4.2f to %4.2f SOLAR RADII [RSUN], USING A 2048-ELEMENT LINEAR", MK4F->rmin, MK4F->rmax) ;
*/
   sprintf (&fitbuf [iy++][0],  "COMMENT   FROM ~1.07 to ~2.85 SOLAR RADII [RSUN], USING A 2048-ELEMENT LINEAR") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   DALSA ARRAY (SAMPLED TO 384 PIXELS) ALIGNED ALONG A RADIAL DIRECTION") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   OUTWARD FROM THE CENTER OF THE SUN.  THE DETECTOR IS ROTATED OVER A") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   3-MINUTE PERIOD TO ACCUMULATE A SET OF RADIAL RAYS, WHICH MAY BE") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   CONVERTED FROM POLAR TO CARTESIAN (I.E., RECTANGULAR) COORDINATES.") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;

   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/

   iynum = iy ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
/*	 fitbuf [iy][ix] = (char) toupper (ival) ;*/ /* Convert to upper case.*/
	 fitbuf [iy][ix] = (char) ival ;
	 }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*nbyrec, FF) ;
   swrite += nelwr ;
   if (pmode)
      fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (mk4buf, MK4F->nbypx, npximg, FF) ;
   swrite += nelwr ;
   if (pmode)
      fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;
   free (mk4buf) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyrec ;
   if ((nrec * nbyrec) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyrec) - nbyimg ;
      if (pmode)
         fprintf (stdout, "npad:%4d  nrec:%4d  nbyrec:%4d  nbyimg:%6d\n",
                  npad, nrec, nbyrec, nbyimg) ;
      fflush (stdout) ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FF) ;
      swrite += nelwr ;
      if (pmode)
         fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;
      free (padbuf) ;
      }

   mk4ext (FF, object, type_obs, date_obs, time_obs, pmode) ;
   
   if (pmode)
      {
      printf ("End of mk4tofits\n") ;
      fflush (stdout) ;
      }
   fclose (FF) ;
   return (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

mk4ext (FD, object, type_obs, date_obs, time_obs, pmode)

FILE *FD ;
char *object   ;                /* object description. */
char *type_obs ;                /* observation type. */
char *date_obs ;                /* observation date. */
char *time_obs ;                /* observation time. */
int pmode ;                     /* print mode.  0:silent, 1:verbose. */

{
   extern char *malloc () ;		/* Function to allocate memory. */

   static char outbuf [84] ;			/* output buffer. */

   static char eq        [4] = "= "      ;
   static char kpcount   [12] = "PCOUNT"   ;

   static char kbitpix   [12] = "BITPIX  " ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char kobject   [12] = "OBJECT  " ;
   static char ktype_obs [12] = "TYPE-OBS" ;

   static char ki_year   [12] = "I_YEAR"   ;
   static char ki_month  [12] = "I_MONTH"  ;
   static char ki_day    [12] = "I_DAY"    ;
   static char ki_hour   [12] = "I_HOUR"   ;
   static char ki_minute [12] = "I_MINUTE" ;
   static char ki_censec [12] = "I_CENSEC" ;
   static char ki_juld   [12] = "I_JULD"   ;
   static char ki_doy    [12] = "I_DOY"    ;
   static char ki_time   [12] = "I_TIME"   ;

   static char kxdim     [12] = "XDIM"     ;
   static char kydim     [12] = "YDIM"     ;
   static char kduration [12] = "DURATION" ;
   static char kfill1_0  [12] = "FILL1_0"  ;
   static char kfill1_1  [12] = "FILL1_1"  ;
   static char kfill1_2  [12] = "FILL1_2"  ;
   static char kfill1_3  [12] = "FILL1_3"  ;

   static char kpangle   [12] = "PANGLE"   ;
   static char kbangle   [12] = "BANGLE"   ;
   static char klangle   [12] = "LANGLE"   ;
   static char kra       [12] = "RA"       ;
   static char kdec      [12] = "DEC"      ;
   static char kha       [12] = "HA"       ;
   static char kelev     [12] = "ELEV"     ;
   static char kazimuth  [12] = "AZIMUTH"  ;
   static char kamass    [12] = "AMASS"    ;
   static char kradius   [12] = "RADIUS"   ;
   static char kskytx    [12] = "SKYTX"    ;
   static char kmk4phase [12] = "MK4PHASE" ;
   static char kspare1   [12] = "SPARE1"   ;
   static char kdatascl  [12] = "DATASCL"  ;
   static char kdataoff  [12] = "DATAOFF"  ;
   static char kpbrefscl [12] = "PBREFSCL" ;
   static char korghair0 [12] = "ORGHAIR0" ;
   static char korghair1 [12] = "ORGHAIR1" ;
   static char khair0    [12] = "HAIR0"    ;
   static char khair1    [12] = "HAIR1"    ;
   static char kxcen     [12] = "XCEN"     ;
   static char kycen     [12] = "YCEN"     ;
   static char kpixscl   [12] = "PIXSCL"   ;
   static char kr0       [12] = "R0"       ;
   static char kdr       [12] = "DR"       ;
   static char krmaxd    [12] = "RMAXD"    ;
   static char kth_min   [12] = "TH_MIN"   ;
   static char kth_max   [12] = "TH_MAX"   ;
   static char kraw_tmin [12] = "RAW_TMIN" ;
   static char kraw_tmax [12] = "RAW_TMAX" ;
   static char kbacklash [12] = "BACKLASH" ;
   static char kphase    [12] = "PHASE"    ;

   static char kobstype  [12] = "OBSTYPE"  ;
   static char kdatatype [12] = "DATATYPE" ;
   static char kprocstep [12] = "PROCSTEP" ;
   static char kbar_dir  [12] = "BAR_DIR"  ;
   static char kpixels   [12] = "PIXELS"   ;
   static char knray     [12] = "NRAY"     ;
   static char kfill2    [12] = "FILL2"    ;
   static char kc_year   [12] = "C_YEAR"   ;
   static char kc_month  [12] = "C_MONTH"  ;
   static char kc_day    [12] = "C_DAY"    ;
   static char kc_hour   [12] = "C_HOUR"   ;
   static char kc_minute [12] = "C_MINUTE" ;
   static char kc_censec [12] = "C_CENSEC" ;
   static char kc_juld   [12] = "C_JULD"   ;
   static char kc_doy    [12] = "C_DOY"    ;
   static char kc_time   [12] = "C_TIME"   ;

   static char ktscpfile [12] = "TSCPFILE" ;
   static char kreffile  [12] = "REFFILE"  ;
   static char kdrftfile [12] = "DRFTFILE" ;
   static char kavgtime1 [12] = "AVGTIME1" ;
   static char kavgtime2 [12] = "AVGTIME2" ;
   static char kavgtime3 [12] = "AVGTIME3" ;
   static char kavgtime4 [12] = "AVGTIME4" ;
   static char kavgtime5 [12] = "AVGTIME5" ;
   static char kavgtime6 [12] = "AVGTIME6" ;
   static char kavgtime7 [12] = "AVGTIME7" ;
   static char kavgtime8 [12] = "AVGTIME8" ;

   /*** InstrDescrip parameters ***/

   static char kjuldate0 [12] = "JULDATE0" ;
   static char kjuldate1 [12] = "JULDATE1" ;
   static char kkappa    [12] = "KAPPA"    ;
   static char kbaroff   [12] = "BAROFF"   ;
   static char khairspt0 [12] = "HAIRSPT0" ;
   static char khairspt1 [12] = "HAIRSPT1" ;
   static char kr0_id    [12] = "R0_ID"    ;
   static char kdr_id    [12] = "DR_ID"    ;
   static char kgoodrmin [12] = "GOODRMIN" ;
   static char kgoodrmax [12] = "GOODRMAX" ;
   static char kbiasspt0 [12] = "BIASSPT0" ;
   static char kbiasspt1 [12] = "BIASSPT1" ;
   static char kzerosky  [12] = "ZEROSKY"  ;
   static char kvenusfac [12] = "VENUSFAC" ;
   static char kextrafac [12] = "EXTRAFAC" ;
   static char kdiff_fac [12] = "DIFF_FAC" ;
   static char kdiff_pix [12] = "DIFF_PIX" ;
   static char kphipix   [12] = "PHIPIX"   ;
   static char kpolckpix [12] = "POLCKPIX" ;
   static char knegpix0  [12] = "NEGPIX0"  ;
   static char knegpix1  [12] = "NEGPIX1"  ;
   static char kwrappix0 [12] = "WRAPPIX0" ;
   static char kwrappix1 [12] = "WRAPPIX1" ;
   static char kringpix0 [12] = "RINGPIX0" ;
   static char kringpix1 [12] = "RINGPIX1" ;
   static char kringpixr [12] = "RINGPIXR" ;
   static char kcorrect0 [12] = "CORRECT0" ;
   static char kcorrect1 [12] = "CORRECT1" ;

   static char avgtime1  [24] ;
   static char avgtime2  [24] ;
   static char avgtime3  [24] ;
   static char avgtime4  [24] ;
   static char avgtime5  [24] ;
   static char avgtime6  [24] ;
   static char avgtime7  [24] ;
   static char avgtime8  [24] ;
   static char pixcom    [40] ;

   int pcount = 0 ;             /* pcount. */
   int gcount = 1 ;             /* gcount. */
   int nhdu = 0 ;               /* # FITS header blocks. */

   int bitpix = 8 ;
   int ix, iy ;
   int iyear ;
   int iynum ;
   int ival ;
   int i ;
   int nelwr ;
   int nbyhd ;
   int nbyrec = 2880 ;
   int ndim = 1 ;
   int nrec ;
   int npad ;
   int naxis1 = 1024 ;
   int swrite ;
   int tscplen ;
   int reflen ;
   int drftlen ;

   char *padbuf ;

   strncpy (avgtime1, &mk4h->AvgTimes [  0], 20) ;
   strncpy (avgtime2, &mk4h->AvgTimes [ 20], 20) ;
   strncpy (avgtime3, &mk4h->AvgTimes [ 40], 20) ;
   strncpy (avgtime4, &mk4h->AvgTimes [ 60], 20) ;
   strncpy (avgtime5, &mk4h->AvgTimes [ 80], 20) ;
   strncpy (avgtime6, &mk4h->AvgTimes [100], 20) ;
   strncpy (avgtime7, &mk4h->AvgTimes [120], 20) ;
   strncpy (avgtime8, &mk4h->AvgTimes [140], 20) ;

   strcpy (pixcom,   "MIXTURE OF CHAR[C], INT[I], FLOAT[F] VARIABLES") ;

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         fitbuf [iy][ix] = ' ' ;

   iy = 0 ;


   sprintf (&fitbuf [iy++][0],  "XTENSION= 'MK4_HSKP'          / MLSO/MK4 BINARY HEADER EXTENSION") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
                                knaxis, eq, ndim) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF DIMENSION # 1",
                                knaxis1, eq, naxis1) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
                                kpcount, eq, pcount) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
                                kgcount, eq, gcount) ;

/*
   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT DATA ARRAY CONTAINS THE MK4 BINARY HEADER STRUCTURE.                    ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT DATA TYPES: CHARACTERS [C], INTEGERS [I], FLOATING POINT [F].           ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT DECODED HEADER IS PROVIDED BELOW:                                       ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
 */

   sprintf (date_obs, "%04d-%02d-%02d", iyear, MK4F->month, MK4F->day) ;
   sprintf (time_obs, "%02d:%02d:%02d", MK4F->hour, MK4F->minute, MK4F->second) ;

   sprintf (&fitbuf [iy++][0],  "COMMENT  KEYWORDS & PARAMETERS BELOW DESCRIBE THE MK4 BINARY HEADER CONTENTS:   ") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_year,   eq, mk4h->Image.Year) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES   0-  3 [I]: IMAGE YEAR: CCYY") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_month,  eq, mk4h->Image.Month) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES   4-  7 [I]: IMAGE MONTH: 1-12") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_day,    eq, mk4h->Image.Day) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES   8- 11 [I]: IMAGE DAY: 1-31") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_hour,   eq, mk4h->Image.Hour) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  12- 15 [I]: IMAGE HOUR: 0-23" ) ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_minute, eq, mk4h->Image.Minute) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  16- 19 [I]: IMAGE MINUTE: 0-59") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_censec, eq, mk4h->Image.CentiSec) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  20- 23 [I]: IMAGE CENTISEC: seconds*100") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_juld,   eq, mk4h->Image.Juld) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  24- 27 [I]: IMAGE JULIAN DATE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_doy,    eq, mk4h->Image.Doy) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  28- 31 [I]: IMAGE DAY-OF-YEAR: 1-366") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ki_time,   eq, mk4h->Image.Time) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  32- 35 [I]: IMAGE TIME: HERTZ UNITS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kxdim,     eq, mk4h->Xdim) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  36- 39 [I]: X-AXIS DIMENSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kydim,     eq, mk4h->Ydim) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  40- 43 [I]: Y-AXIS DIMENSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kduration, eq, mk4h->Duration) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  44- 47 [I]: SCAN DURATION [HZ]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kjuldref,  eq, mk4h->JuldRef) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  48- 51 [I]: FILL BYTES") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfill1_0,  eq, mk4h->Fill1 [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  52- 55 [I]: FILL BYTES") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfill1_1,  eq, mk4h->Fill1 [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  56- 59 [I]: FILL BYTES") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfill1_2,  eq, mk4h->Fill1 [2]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  60- 63 [I]: FILL BYTES") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kpangle,   eq, mk4h->Pangle) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  64- 67 [F]: SOLAR P-ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kbangle,   eq, mk4h->Bangle) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  68- 71 [F]: SOLAR B-ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", klangle,   eq, mk4h->Langle) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  72- 75 [F]: SOLAR L-ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kra,       eq, mk4h->RA) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  76- 79 [F]: RIGHT ASCENSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kdec,      eq, mk4h->Dec) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  80- 83 [F]: DECLINATION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kha,       eq, mk4h->HA) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  84- 87 [F]: SOLAR HOUR-ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kelev,     eq, mk4h->Elevation) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  88- 91 [F]: SOLAR ELEVATION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kazimuth,  eq, mk4h->Azimuth) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  92- 95 [F]: SOLAR AZIMUTH") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kamass,    eq, mk4h->Amass) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES  96- 99 [F]: AIR MASS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kradius,   eq, mk4h->Radius) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 100-103 [F]: SOLAR RADIUS [ARCSEC]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kskytx,    eq, mk4h->SkyTx) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 104-107 [F]: SKY TRANSMISSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kmk4phase, eq, mk4h->InstrPhs) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 108-111 [F]: MK4 COORD FRAME PHASE [DEG]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kspare1,   eq, mk4h->spare1) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 116-119 [F]: SPARE STORAGE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g",
	    kdatascl,  eq, mk4h->data_scale) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 120-123 [F]: DATA SCALE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g",
	    kdataoff,  eq, mk4h->data_offset) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 124-127 [F]: DATA OFFSET") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    kspare2,   eq, mk4h->spare2) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 128-131 [F]: SPARE 2") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    korghair0, eq, mk4h->orig_hairspot [0]) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 132-135 [F]: ORIGINAL HAIRLINE LOC [0]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    korghair1, eq, mk4h->orig_hairspot [1]) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 136-139 [F]: ORIGINAL HAIRLINE LOC [1]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    khair0, eq, mk4h->hairspot [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 140-143 [F]: HAIRLINE LOCATION [0]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    khair1, eq, mk4h->hairspot [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 144-147 [F]: HAIRLINE LOCATION [1]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    kxcen,     eq, mk4h->Xcen) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 112-115 [F]: X-AXIS SUN CENTER [PIXELS]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kycen,     eq, mk4h->Ycen) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 148-151 [F]: Y-AXIS SUN CENTER [PIXELS]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kpixscl,   eq, mk4h->pixscl) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 152-155 [F]: SCALE [CARTESIAN:PIX/RSUN]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g", kr0,       eq, mk4h->r0) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 156-159 [F]: RADIUS LOWEST PIXEL [RSUN]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g", kdr,       eq, mk4h->dr) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 160-163 [F]: RADIAL PIXEL INC. [RSUN]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g", krmaxd,    eq, mk4h->rmax) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 164-167 [F]: RADIUS OF HIGHEST PIXEL") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    kth_min,   eq, mk4h->theta_min) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 168-171 [F]: MINIMUM SCAN ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    kth_max,   eq, mk4h->theta_max) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 172-175 [F]: MAXIMUM SCAN ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    kraw_tmin, eq, mk4h->raw_theta_min) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 176-179 [F]: RAW MIN.SCAN ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f",
	    kraw_tmax, eq, mk4h->raw_theta_max) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 180-183 [F]: RAW MAX.SCAN ANGLE [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kbacklash, eq, mk4h->backlash) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 184-187 [F]: BACKLASH ADJ APPLIED [DEG]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kphase,    eq, mk4h->Phase) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 188-191 [F]: PHASE OFFSET [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kobstype,  eq, mk4h->ObsType) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 192-195 [I]: OBS: CAL0,CAL15,DARK,CORONA") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdatatype, eq, mk4h->DataType) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 196-199 [I]: DATA TYPE: I, Q, U");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kprocstep, eq, mk4h->ProcSteps) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 200-203 [I]: DP: RAW,CAL,AVG,RECT,SLIT") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kbar_dir,  eq, mk4h->Direction) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 204-207 [I]: SCAN DIRECTION:-1=REV +1=FWD");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", knray,     eq, mk4h->Nray) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 208-211 [I]: NUMBER OF RADIAL RAYS");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpixels,   eq, mk4h->Pixels) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 212-215 [I]: NUMBER OF PIXELS");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfill2,    eq, mk4h->Fill2) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 216-219 [I]: FILL BYTES");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_year,   eq, mk4h->Date_calibrated.Year) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 220-223 [I]: CAL YEAR: CCYY") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_month,  eq, mk4h->Date_calibrated.Month) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 224-227 [I]: CAL MONTH: 1-12") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_day,    eq, mk4h->Date_calibrated.Day) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 228-231 [I]: CAL DAY: 1-31") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_hour,   eq, mk4h->Date_calibrated.Hour) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 232-235 [I]: CAL HOUR: 0-23" ) ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_minute, eq, mk4h->Date_calibrated.Minute) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 236-239 [I]: CAL MINUTE: 0-59") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_censec, eq, mk4h->Date_calibrated.CentiSec) ;
   sprintf (&fitbuf [iy++][31],
	    "/ BYTES 240-243 [I]: CAL CENTISEC: seconds * 10") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_juld,   eq, mk4h->Date_calibrated.Juld) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 244-247 [I]: CAL JULIAN DATE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_doy,    eq, mk4h->Date_calibrated.Doy) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 248-251 [I]: CAL DAY-OF-YEAR: 1-366") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d",
	    kc_time,   eq, mk4h->Date_calibrated.Time) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 252-255 [I]: CAL TIME: HERTZ UNITS") ;

   tscplen = strlen (MK4F->tscpfile) + 13 ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", ktscpfile, eq, mk4h->tscpfile) ;
   if (tscplen > 31)
   	strcpy  (&fitbuf [iy++][tscplen],
		 "/ BYTES 256-319 [C]: TSCP FILE") ;
   else
   	strcpy  (&fitbuf [iy++][31], "/ BYTES 256-319 [C]: TSCP FILE") ;

   reflen = strlen (MK4F->reffile) + 13 ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kreffile, eq, mk4h->reffile) ;
   if (reflen > 31)
   	strcpy  (&fitbuf [iy++][reflen],
		 "/ BYTES 320-383 [C]: REFERENCE FILE") ;
   else
   	strcpy  (&fitbuf [iy++][31],
		 "/ BYTES 320-383 [C]: REFERENCE FILE") ;

   verlen = strlen (MK4F->version) + 13 ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kversion,  eq, mk4h->version) ;
   if (verlen > 31)
   	strcpy  (&fitbuf [iy++][verlen],
		 "/ BYTES 384-447 [C]: VERSION") ;
   else
   	strcpy  (&fitbuf [iy++][31],
		 "/ BYTES 384-447 [C]: VERSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime1, eq, avgtime1) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 448-467 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime2, eq, avgtime2) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 468-487 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime3, eq, avgtime3) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 488-507 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime4, eq, avgtime4) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 508-527 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime5, eq, avgtime5) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 528-547 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime6, eq, avgtime6) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 548-567 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime7, eq, avgtime7) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 568-587 [C]: IMAGE TIMES FOR AVERAGE");
   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%-20s'", kavgtime8, eq, avgtime8) ;
   sprintf (&fitbuf [iy++][33], "/ BYTES 588-607 [C]: IMAGE TIMES FOR AVERAGE");

   /*** InstrDescrip parameters ***/

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g", kjuldate0, eq, mk4h->ID.JuldRange [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 608-615 [D]: START DATE FOR INST. PARAM.") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g", kjuldate1, eq, mk4h->ID.JuldRange [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 616-623 [D]: END   DATE FOR INST. PARAM.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.6g", kkappa, eq, mk4h->ID.kappa) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 624-627 [F]: B=KAPPA*(I^(KAPPA-1)") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kbaroff, eq, mk4h->ID.BarrelOffset) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 628-631 [F]: BARREL OFFSET [DEG].") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", khairspot0, eq, mk4h->ID.hairspot [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 632-635 [F]: HAIR SPOT 0.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", khairspot1, eq, mk4h->ID.hairspot [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 636-639 [F]: HAIR SPOT 1.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kr0_id, eq, mk4h->ID.r0) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 640-643 [F]: R0 [Rsun].") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kdr_id, eq, mk4h->ID.dr) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 644-647 [F]: PIXEL INCREMENT [Rsun].") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kgoodrmin, eq, mk4h->ID.goodRmin) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 648-651 [F]: MINIMUM HEIGHT [Rsun].") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kgoodrmin, eq, mk4h->ID.goodRmin) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 652-655 [F]: MAXIMUM HEIGHT [Rsun].") ;


   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kbiasspt0, eq, mk4h->ID.biasspot [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 656-659 [F]: LOWEST  PIXEL FOR BIAS [Rsun].") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kbiasspt1, eq, mk4h->ID.biasspot [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 660-663 [F]: HIGHEST PIXEL FOR BIAS [Rsun].") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kzerosky, eq, mk4h->ID.ZeroSky) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 664-667 [F]: SKY TRANSMISSION MAXIUMUM.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kvenusfac, eq, mk4h->ID.VenusCalFac) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 668-671 [F]: VENUS CALIBRATION FACTOR.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kextrafac, eq, mk4h->ID.ExtraCalFac) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 672-675 [F]: VENUS CALIBRATION FACTOR.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20.2f", kdiff_fac, eq, mk4h->ID.DiffractFac) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 676-679 [F]: DIFFRACTION CAL. FACTOR.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdiff_pix, eq, mk4h->ID.DiffractPix) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 680-683 [F]: HIGHEST PIXEL FOR DIFFRACTION.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kphipix, eq, mk4h->ID.PhiPix) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 684-687 [F]: HIGHEST PIXEL FOR DIFFRACTION.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpolpix, eq, mk4h->ID.PolarityCkPix) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 688-691 [F]: REV. POLARITY PIXEL OFFSET.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", knegpix0, eq, mk4h->ID.NegPix [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 692-695 [F]: LOWEST  PIXEL FOR NEG. FIX.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", knegpix1, eq, mk4h->ID.NegPix [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 696-699 [F]: HIGHEST PIXEL FOR NEG. FIX.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kwrappix0, eq, mk4h->ID.WrapPix [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 700-703 [F]: LOWEST  PIXEL FOR NEG. WRAP.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kwrappix1, eq, mk4h->ID.WrapPix [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 704-707 [F]: HIGHEST PIXEL FOR NEG. WRAP.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kringpix0, eq, mk4h->ID.RingSearch [0]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 708-711 [F]: LOWEST  PIXEL FOR RING SEARCH.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kringpix1, eq, mk4h->ID.RingSearch [1]) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 712-715 [F]: HIGHEST PIXEL FOR RING SEARCH.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kringpixr, eq, mk4h->ID.RingFixPix) ;
   sprintf (&fitbuf [iy++][31], "/ BYTES 716-719 [F]: RING REFERENCE HEIGHT.") ;


   cor0len = strlen (MK4F->CorrectFile0) + 13 ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kcorrect0, eq, mk4h->CorrectFile0) ;
   if (cor0len > 31)
   	strcpy  (&fitbuf [iy++][cor0len],
		 "/ BYTES 720-783 [C]: CORRECT #0 FILE.") ;
   else
   	strcpy  (&fitbuf [iy++][31],
		 "/ BYTES 720-783 [C]: CORRECT #0 FILE") ;

   cor1len = strlen (MK4F->CorrectFile0) + 13 ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%s'", kcorrect1, eq, mk4h->CorrectFile1) ;
   if (cor1len > 31)
   	strcpy  (&fitbuf [iy++][cor1len],
		 "/ BYTES 784-847 [C]: CORRECT #1 FILE.") ;
   else
   	strcpy  (&fitbuf [iy++][31],
		 "/ BYTES 784-847 [C]: CORRECT #1 FILE") ;

   sprintf (&fitbuf [iy++][0],  "COMMENT                        / BYTES 848-1023 [C]: FILL BYTES                 ") ;

   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
/*	 fitbuf [iy][ix] = (char) toupper (ival) ;*/ /* Convert to upper case.*/
	 fitbuf [iy][ix] = (char) ival ;
	 }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*nbyrec, FD) ;
   swrite += nelwr ;
   if (pmode)
      fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;

   /*** Write binary header data. ***/

   nbyhd = sizeof (mk4head) ;
   nelwr = fwrite (mk4h, nbyhd, 1, FD) ;
   swrite += nelwr ;
   if (pmode)
      fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;
/*   free (mk4buf) */ ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyhd / nbyrec ;
   if ((nrec * nbyrec) < nbyhd)
      {
      nrec++ ;
      npad = (nrec * nbyrec) - nbyhd ;
      if (pmode)
         fprintf (stdout, "npad:%4d  nrec:%4d  nbyrec:%4d  nbyhd:%6d\n",
                  npad, nrec, nbyrec, nbyhd) ;
      fflush (stdout) ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FD) ;
      swrite += nelwr ;
      if (pmode)
         fprintf (stdout, "nelwr: %7d swrite: %7d\n", nelwr, swrite) ;
      free (padbuf) ;
      }

}

