/* fmk4syn_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display MLSO Mark-IIV synoptic map in SGI/GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Assumes that a 512x512x8-bit window has been opened using winopen ().
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   mk4syn_view.c
 *  3 Dec 1993
 * 10 Dec 1993: Display BOTH east & west limbs in one window.
 * 24 Jan 1994: begdoye, enddoye, begdoyw, enddoyw changed from short to int.
 * 19 Aug 1996: rasterfile output name format: mk3.CCCC.rXX, CCCC=C.R., XX=Rsun.
 * 21 Aug 1996: multiple rotation version [mk3syn4.c].
 * 13 Mar 2002: Adapt for Mk4.
 * 17 Jun 2002: smax=32000 if R < 1.2 Rsun.
 *  5 Nov 2002: Revise display ranges after modifying "mk4syn.c".
 *  6 Nov 2002: Adapt for floating point synoptic map files.
 * 18 Nov 2002: Change input name syntax: mk4.eCCCC.rRRR.df and
 *		mk4.wCCCC.rRRR.df,
 *		where CCCC = Carrington Rotation and RRR = radius * 100.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/glcp_color.h"
#include "mk4syn.h"

fmk4syn_view (wxsize, wysize, begrot, endrot, solar_radius)

int wxsize, wysize ;			/* Window size. */
int begrot, endrot ;			/* Beginning & ending rotations. */
float solar_radius ;			/* Radius (Rsun). */

{
   extern int pmode ;
   static char slimb [4] ;		/* Solar limb [e,w]. */
   static char limb ;			/* Solar limb [e,w]. */
   static char eastfile [80] ;		/* East limb file name. */
   static char westfile [80] ;		/* West limb file name. */
   static char namout [80] ;
   static float fbufe [181][360] ;	/* Synoptic east map buffer. */
   static float fbufw [181][360] ;	/* Synoptic west map buffer. */
   unsigned short *smape, *smapw ;
   float fmaxe =     0 ;		/* Data maximum. */
   float fmine= 65535 ;			/* Data minimum. */
   float fmaxw =     0 ;		/* Data maximum. */
   float fminw= 65535 ;			/* Data minimum. */
   float fvale, fvalw ;
   short imine, imaxe ;
   short iminw, imaxw ;
   unsigned short soff = 512 ;		/* Offset for LUT #2. */
   short svale, svalw ;			/* Short integer. */
   float offset = 0 ;			/* Data offset. */
   float radius ;

   int begdoye, begyre ;		/* East begin DOY, year. */
   int enddoye, endyre ;		/* East end   DOY, year. */
   int begdoyw, begyrw ;		/* West begin DOY, year. */
   int enddoyw, endyrw ;		/* West end   DOY, year. */
   int crot ;				/* Carrington rotation. */
   int i ;
   int irot ;				/* Rotation index. */
   int ix, iy, ixo ;			/* indices. */
   int nlat  = 181 ;			/* # latitudes. */
   int nlong = 360 ;			/* # longitudes. */
   int nlongo ;				/* # longitude pixels output. */
   int nrot ;				/* # rotations to map. */
   int ipass = 0 ;			/* Loop counter. */
   int iradius ;
   int xlle0, ylle0, xure0, yure0 ;	/* rectwrite coordinates. */
   int xllw0, yllw0, xurw0, yurw0 ;	/* rectwrite coordinates. */
   int xlle, ylle, xure, yure ;		/* rectwrite coordinates. */
   int xllw, yllw, xurw, yurw ;		/* rectwrite coordinates. */

   float fval ;
   float slope ;			/* Scaling factor. */

   FILE *FD ;

   radius = solar_radius ;
   printf ("fmk4syn_view> begrot: %4d  endrot: %4d  radius:%7.2f\n",
	   begrot, endrot, radius) ;

   gl_wclear (glcp_white) ;

   nrot = endrot - begrot + 1 ;
   nlongo = nlong / nrot ;
   smape = (unsigned short *) malloc (nlat * nlongo * sizeof (unsigned short)) ;
   smapw = (unsigned short *) malloc (nlat * nlongo * sizeof (unsigned short)) ;
   if (pmode)
      {
      printf ("fmk4syn_view> nlat: %d   nlong : %d\n", nlat, nlong) ;
      printf ("fmk4syn_view> nrot: %d   nlongo: %d\n", nrot, nlongo) ;
      }

   /*** Create output name for sun rasterfile. ***/

   if (begrot == endrot)
      sprintf (namout, "mk4.ew%04d.r%03d.sun",
	       begrot, (int) (radius*100.0 + 0.5)) ;
   else
      sprintf (namout, "mk4.ew%04d-%04d.r%03d.sun",
	       begrot, endrot,  (int) (radius*100.0 + 0.5)) ;

   /*** Define lower left and upper right corner pixel coordinates
    *** for synoptic array placement in map frame buffer.
    ***/

      xllw0 = (wxsize - nlong) / 2 ;
      yllw0 = bfy * 3 ;
/*
 *      yllw0 = (wysize/2 - nlat) / 2 ;
 */
      xurw0 = xllw0 + nlong - 1 ;
      yurw0 = yllw0 + nlat - 1 ;

      xlle0 = (wxsize - nlong) / 2 ;
      ylle0 = yurw0 + bfy * 2 ;
/*
 *      ylle0 = (wysize/2 - nlat) / 2 + wysize/2 ;
 */
      xure0 = xlle0 + nlong - 1 ;
      yure0 = ylle0 + nlat - 1 ;

      if (pmode)
	 {
         printf ("fmk4syn_view> xlle0/ylle0: %4d %4d   xure0/yure0: %4d %4d\n",
                  xlle0, ylle0, xure0, yure0) ;
         printf ("fmk4syn_view> xllw0/yllw0: %4d %4d   xurw0/yurw0: %4d %4d\n",
                  xllw0, yllw0, xurw0, yurw0) ;
         }

   ipass = 0 ;

   for (irot = begrot; irot <= endrot; irot++)
      {
      ipass++;

      /*** Adjust lower left & upper right coordinates for each rotation. ***/

      xlle = xlle0 + (nrot-ipass)   * nlongo ;
      ylle = ylle0 ;
      xure = xlle0 + (nrot-ipass+1) * nlongo - 1 ;
      yure = yure0 ;
      xllw = xllw0 + (nrot-ipass)   * nlongo ;
      yllw = yllw0 ;
      xurw = xllw0 + (nrot-ipass+1) * nlongo - 1 ;
      yurw = yurw0 ;

      if (pmode)
	 {
         printf ("fmk4syn_view> xlle/ylle: %4d %4d   xure/yure: %4d %4d\n",
                  xlle, ylle, xure, yure) ;
         printf ("fmk4syn_view> xllw/yllw: %4d %4d   xurw/yurw: %4d %4d\n",
                  xllw, yllw, xurw, yurw) ;
         }

      /*** Define input binary file name for East & West synoptic maps. ***/

      sprintf (eastfile, "%04de%4.2f", irot, radius) ;
      sprintf (westfile, "%04dw%4.2f", irot, radius) ;

      sprintf (eastfile, "fmk4.cr%04d.e%d", irot, (int) (radius * 100)) ;
      sprintf (westfile, "fmk4.cr%04d.w%d", irot, (int) (radius * 100)) ;

      sprintf (eastfile, "mk4.e%04d.r%d.df", irot, (int) (radius * 100)) ;
      sprintf (westfile, "mk4.w%04d.r%d.df", irot, (int) (radius * 100)) ;

      if (pmode)
         printf ("fmk4syn_view> eastfile: %s   westfile: %s\n", eastfile, westfile) ;
/*
      sscanf (eastfile, "%4d%1s%4.2f", &crot, &slimb, &radius) ;
      sscanf (&eastfile [5], "%f", &radius) ;

      sscanf (eastfile, "fmk4.cr%4d%.1s%d", &crot, &slimb, &iradius) ;
      sscanf (&eastfile [5], "%f", &radius) ;
      limb = (char) slimb [0] ;
      if (pmode)
         fprintf (stdout, "fmk4syn_view> crot:%4d   limb:%c   radius:%4.2f\n",
	          crot, limb, radius) ;

      sscanf (westfile, "%4d%1s%4.2f", &crot, &slimb, &radius) ;
      sscanf (&westfile [5], "%f", &radius) ;
      limb = (char) slimb [0] ;
 */
      if (pmode)
         fprintf (stdout, "fmk4syn_view> crot:%4d   limb:%c   radius:%4.2f\n",
                  crot, limb, radius) ;

      /*** Open East synoptic data file. ***/

      if ((FD = fopen (eastfile, "r")) == NULL)
         {
         fprintf (stderr, "fmk4syn_view> Unable to open file: %s\n", eastfile) ;
         exit (-1) ;
         }

      /*** Read data from synoptic file. ***/

      fread (fbufe, sizeof (float), nlat*nlong, FD) ;
      fread (&begdoye, sizeof (int), 1, FD) ;
      fread (&begyre,  sizeof (int), 1, FD) ;
      fread (&enddoye, sizeof (int), 1, FD) ;
      fread (&endyre,  sizeof (int), 1, FD) ;
      fclose (FD) ;
      if (pmode)
         printf ("fmk4syn_view> begdoye:%6d   begyre:%4d   enddoye:%6d   endyre:%4d\n",
	         begdoye, begyre, enddoye, endyre) ;

      /*** Open West synoptic data file. ***/

      if ((FD = fopen (westfile, "r")) == NULL)
         {
         fprintf (stderr, "fmk4syn_view> Unable to open file: %s\n", westfile) ;
         exit (-1) ;
         }

      /*** Read data from synoptic file. ***/

      fread (fbufw, sizeof (float), nlat*nlong, FD) ;
      fread (&begdoyw, sizeof (int), 1, FD) ;
      fread (&begyrw,  sizeof (int), 1, FD) ;
      fread (&enddoyw, sizeof (int), 1, FD) ;
      fread (&endyrw,  sizeof (int), 1, FD) ;
      fclose (FD) ;
      if (pmode)
         printf ("fmk4syn_view> begdoyw:%6d   begyrw:%4d   enddoyw:%6d   endyrw:%4d\n",
	         begdoyw, begyrw, enddoyw, endyrw) ;

      /*** Find data min/max. ***/

      fmaxe = -1.0e-30 ;
      fmine =  1.0e+30 ;
      fmaxw = -1.0e+30 ;
      fminw =  1.0e+30 ;
      for (iy = 0 ; iy < nlat ; iy++)
         for (ix = 0 ; ix < nlong ; ix++)
	    {
	    if (fmaxe < fbufe [iy][ix])  fmaxe = fbufe [iy][ix] ;
	    if (fmine > fbufe [iy][ix])  fmine = fbufe [iy][ix] ;
	    if (fmaxw < fbufw [iy][ix])  fmaxw = fbufw [iy][ix] ;
	    if (fminw > fbufw [iy][ix])  fminw = fbufw [iy][ix] ;
	    }
/*
      imine = fmine ;
      imaxe = fmaxe ;
      iminw = fminw ;
      imaxw = fmaxw ;
 */

      printf ("fmk4syn_view> fmine:%8.2e4   fmaxe:%8.2e  fminw:%8.2e  fmaxw:%8.2e\n",
	       fmine, fmaxe, fminw, fmaxw) ;
      fmine =     0.0 ;
      fminw =     0.0 ;

      if (radius <=  1.25)
         {
	 fmine =      0 ;
	 fminw =      0 ;
	 fmaxe = 1.0e-6 ;
	 fmaxw = 1.0e-6 ;
         }
      else
      if (radius > 1.25 && radius <= 1.40)
	 {
	 fmine =       0 ;
	 fminw =       0 ;
         fmaxe =  5.0e-7 ;
         fmaxw =  5.0e-7 ;
	 }
      else
      if (radius > 1.40 && radius <= 1.55)
	 {
	 fmine =       0 ;
	 fminw =       0 ;
	 fmaxe =  2.5e-7 ;
	 fmaxw =  2.5e-7 ;
	 }
      else
      if (radius > 1.55 && radius <= 1.80)
         {
	 fmine =       0 ;
	 fminw =       0 ;
	 fmaxe =  1.0e-7 ;
	 fmaxw =  1.0e-7 ;
         }
      else
      if (radius > 1.80 && radius <= 2.05)
	 {
	 fmine = -5.0e-9 ;
	 fminw = -5.0e-9 ;
         fmaxe =  5.0e-8 ;
         fmaxw =  5.0e-8 ;
	 }
      else
      if (radius > 2.05 && radius <= 2.30)
	 {
	 fmine = -5.0e-9 ;
	 fminw = -5.0e-9 ;
         fmaxe =  2.0e-8 ;
         fmaxw =  2.0e-8 ;
	 }
      else
         {
	 fmine = -5.0e-9 ;
	 fminw = -5.0e-9 ;
         fmaxe =  1.0e-8 ;
         fmaxw =  1.0e-8 ;
         }

/*      if (fmine < 0)  fmine = 0 ; */
/*      if (fminw < 0)  fminw = 0 ; */

      if (fmaxe == 0)
         {
         fprintf (stderr, "fmk4syn_view> Max E pixel value is 0.\n") ;
         exit (-1) ;
         }

      if (fmaxw == 0)
         {
         fprintf (stderr, "fmk4syn_view> Max W pixel value is 0.\n") ;
         exit (-1) ;
         }

/*
      fmine = imine ;
      fmaxe = imaxe ;
      fminw = iminw ;
      fmaxw = imaxw ;
 */

      fmine  = 0.0 ;
      fminw  = 0.0 ;
      offset = 0 ;
      offset = fmine ;
      slope  = 250.0 / (float) (fmaxe - fmine) ;

      if (pmode)
         {
         printf ("fmk4syn_view> fmine:  %8.2e   fmaxe: %8.2e\n", fmine, fmaxe) ;
         printf ("fmk4syn_view> offset: %8.2e   slope: %8.2e\n", offset, slope);
         }

      /*** Apply scaling factor & add display offset. ***/
      /*** Also reverse image top to bottom. ***/

      for (iy = 0 ; iy < nlat ; iy++)
	 {
	 ixo = 0 ;
         for (ix = 0 ; ix < nlong ; ix = ix + nrot)
	    {
	    /*** East. ***/

	    fvale = (float) fbufe [nlat - iy - 1][ix] ;
/*
	    if (fvale < fmine) fvale = fmine ;
	    if (fvale > fmaxe) fvale = fmaxe ;
 */
	    svale = (short) ((fvale - offset) * slope) ;
/*
	    if (iy == 90) printf ("fmk4syn_view> ix: %3d fvale: %8.2e  svale: %3d\n",
				   ix, fvale, svale) ;
 */
	    if (svale <= 0)   svale =   0 ; 
	    if (svale > 249)  svale = 249 ; 
	    *(smape + ixo +iy*nlongo) = (unsigned short) (svale + soff) ;

	    /*** West. ***/

	    fvalw = (float) fbufw [nlat - iy - 1][ix] ;
	    if (fvalw < fminw) fvalw = fminw ;
	    if (fvalw > fmaxw) fvalw = fmaxw ;
	    svalw = (short) ((fvalw - offset) * slope) ;
	    if (svalw <= 0)   svalw =   0 ; 
	    if (svalw > 249)  svalw = 249 ; 
	    *(smapw + ixo + iy*nlongo) = (unsigned short) (svalw + soff) ;
/*
 *            if (iy == 0)
 *            printf ("ixo/iy: %3d %3d   ix/iy: %3d %3d\n", ixo, iy, ix, iy) ;
 */
	    ixo++ ;
            }
         }
      rectwrite (xlle, ylle, xure, yure, smape) ;
      rectwrite (xllw, yllw, xurw, yurw, smapw) ;
      }
   free (smape) ;
   free (smapw) ;

   printf ("fmk4syn_view> radius: %7.2f\n", radius) ;

   mk4syn_label (wxsize, wysize,
                 xlle0, ylle0, xure0, yure0,
                 xllw0, yllw0, xurw0, yurw0,
                 begyre, begdoye, endyre, enddoye,
                 begyrw, begdoyw, endyrw, enddoyw,
                 begrot, radius) ;

   mk4syn_save (namout) ;

   return (0) ;
}
