/* filters.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Collection of filter routines.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Author: Alice Lecinski   HAO/NCAR   Filters.c   21 Mar 2000
 * Updates: Andrew L. Stanger   HAO/NCAR
 * 16 May 2001: cosmetic changes only.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

#include "mk4v2.h"

float  Copy [2*VECTOR] ;
float  Mean   [VECTOR] ;
float  MeanSq [VECTOR] ;
float  Var_X  [VECTOR] ;

/* -------------------------------------------------------------------------- */

/*** Median Filter. ***/
/*** This can do a wrap around to handle the edges. ***/

int
MedianFilt ( Vect, neigh, npts, wrap, doprint ) 

    float *Vect ;
    int   neigh ;
    int   npts ;
    int   wrap ;
    int   doprint ;
{
    int ii ;
    int hsz ;
    int mpos ;
    float *pV, *pC ;
    float stddev ;


    hsz = (neigh - 1) / 2 ;

    pV = Vect ; pC=Copy ;   for (ii = 0 ; ii < npts ; ii++){ *pC++ = *pV++ ; }
    pV = Vect ;             for (ii = 0 ; ii < npts ; ii++){ *pC++ = *pV++ ; }

    if (wrap==1)
	{
	pV = Vect + hsz ;
	pC = Copy ;
	for (ii = hsz ; ii < npts ; ii++)
	    {
	    fmedian (pC, neigh, pV, &mpos, doprint) ;
	    pV++ ;
	    pC++ ;
	    }
	pV = Vect ;
	for (ii = 0 ; ii < hsz ; ii++)
	    {
	    fmedian (pC, neigh, pV, &mpos, doprint) ;
	    pV++ ;
	    pC++ ;
	    }
        }
    else
	{
	pV = Vect + hsz ;
	pC = Copy ;
	for (ii = hsz ; ii < npts - hsz ; ii++)
	    {
	    fmedian (pC, neigh, pV, &mpos, doprint) ;
	    pV++ ;
	    pC++ ;
	    }
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

/*** Lee Filter ***/

int
LeeFilt ( Vect, npts, wrap )

    float *Vect ;
    int   npts ;
    int   wrap ;
{
    int ii ;
    int neigh ;
    float *pV , *pC, *pM, *pMS, *pX ;
    float sigma, SigSq, ovSigSqP1 ;
    float ftemp, tmp ;

    neigh = 5 * 2 + 1 ;
    sigma = 5.0 ;
    SigSq = sigma * sigma ;
    ovSigSqP1 = 1.0 / (sigma * sigma + 1.0) ;

    pM=Mean ; pV=Vect ;   for (ii = 0 ; ii < npts ; ii++) { *pM++ = *pV++ ;  }

    if(wrap==1) Smooth_Wrap   (Mean, neigh, npts) ;
    else        Smooth_NoWrap (Mean, neigh, npts) ;

    pMS = MeanSq ;  pM = Mean ;   
    for (ii = 0 ; ii < npts ; ii++)
	{ 
	ftemp = *pM * (*pM) ;
	*pMS  = ftemp ;
	pMS++ ;
	pM++ ;
        }
    
    pV = Vect ;    pM = Mean ;
    for (ii = 0 ; ii < npts ; ii++)
	{ 
	ftemp = *pV - *pM ;
	*pV   = ftemp ;
	pV++ ;
	pM++ ;
        }

    pX = Var_X ;   pV = Vect ;
    for (ii = 0 ; ii < npts ; ii++)
	{
	ftemp = *pV * (*pV) ;
	*pX   = ftemp ;
	pX++ ;
	pV++ ;
        }

    if (wrap == 1) Smooth_Wrap (Var_X, neigh, npts) ;
    else         Smooth_NoWrap (Var_X, neigh, npts) ;

    pX = Var_X ;    pMS = MeanSq ;
    for (ii = 0 ; ii < npts ; ii++)
	{ 
	ftemp  = *pX + *pMS ;
	ftemp *= ovSigSqP1 ;
	ftemp -= *pMS ;
	*pX    = ftemp ;
	pX++ ;
	pMS++ ;
        }

    pX = Var_X ;    pMS = MeanSq ;  pM = Mean ;     pV = Vect ;
    for (ii = 0 ; ii < npts ; ii++)
	{ 
	ftemp  = *pMS * SigSq ;
	ftemp += *pX ;
	tmp = (ftemp != 0.) ? *pX / ftemp : 0. ;
	tmp *= *pV ;
	tmp += *pM ;
	*pV  = tmp ;
	pX++ ;
	pMS++ ;
	pM++ ;
	pV++ ;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int
DSmooth_NoWrap ( Vect, neigh, npts )

    double *Vect ;
    int   neigh ;
    int   npts ;
{
    int ii ;
    int hsz ;
    double *pV ;
    float  *pC ;
    float mean , stddev ;

    hsz = (neigh - 1) / 2 ;

    pV = Vect ; pC = Copy ;
    for (ii = 0 ; ii < npts ; ii++) { *pC++ = (float)(*pV) ; pV++ ; }

    pV = Vect + hsz ;
    pC = Copy ;
    for (ii = hsz ; ii < npts - hsz ; ii++)
	{
	get_mean (pC, neigh, &mean, &stddev) ;
	*pV = (double)mean ;
	pV++ ;
	pC++ ;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int
Smooth_NoWrap ( Vect, neigh, npts )

    float *Vect ;
    int   neigh ;
    int   npts ;
{
    int ii ;
    int hsz ;
    float *pV, *pC ;
    float mean , stddev ;

    hsz = (neigh - 1) / 2 ;

    pV = Vect ; pC = Copy ;  for (ii = 0 ; ii < npts ; ii++) { *pC++ = *pV++ ; }
    pV = Vect + npts - 1 ;   for (ii = 0 ; ii < npts ; ii++) { *pC++ = *pV-- ; }

    pV = Vect + hsz ;
    pC = Copy ;
    for (ii = hsz ; ii < npts - hsz ; ii++)
	{
	get_mean (pC, neigh, &mean, &stddev) ;
	*pV = mean ;
	pV++ ;
	pC++ ;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int
Smooth_Wrap ( Vect, neigh, npts )

    float *Vect ;
    int   neigh ;
    int   npts ;
{
    int ii ;
    int hsz ;
    float *pV, *pC ;
    float stddev ;

    hsz = (neigh - 1) / 2 ;

    pV = Vect ; pC = Copy ;  for (ii = 0 ; ii < npts ; ii++) { *pC++ = *pV++ ; }
    pV = Vect ;              for (ii = 0 ; ii < npts ; ii++) { *pC++ = *pV++ ; }

    pV = Vect + hsz ;
    pC = Copy ;
    for (ii = hsz ; ii < npts ; ii++)
	{
	get_mean (pC, neigh, pV, &stddev) ;
	pV++ ;
	pC++ ;
        }
    pV = Vect ;
    for (ii = 0 ; ii < hsz ; ii++)
	{
	get_mean (pC, neigh, pV, &stddev) ;
	pV++ ;
	pC++ ;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int
smoothie ( V, npts, smoolen )

    float *V ;
    int  npts ;
    int  smoolen ;
{
    int wrap ;
    int doprint ;
    wrap = 0 ;
    doprint = 0 ;
    MedianFilt (V, smoolen, npts, wrap, doprint) ;
    LeeFilt    (V,          npts, wrap) ;

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int
replace_bad ( V, npts, limit, replacement )

    float *V ;
    int npts ;
    float limit ;
    float replacement ;
{
    int ii ;
    float *pV ;
    float  ftmp, ftmp2 ;
    double dtmp, dlimit ;
    int doprint = 0 ;

    dlimit = (double) (limit) ;

    if (doprint == 1)
	fprintf (stderr, "limit %f  replacement %f\n", limit, replacement) ;

    pV = V ;
    for (ii = 0 ; ii < npts ; ii++)
	{
	dtmp = (double) *pV ;
	ftmp = ((fabs (dtmp) > dlimit) || (isnan (dtmp) == 1))
	     ? replacement : *pV ;
	*pV  = ftmp ;
	pV++ ;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

DSmooth_LeastSq ( X, Y, npts, Rlimit )

  double *X ;
  double *Y ;
  int npts ;
  float Rlimit ;
{
    double *pX,  *pY ;
    float  *pGX, *pGY ;
    float  a, b, R ;
    float  testa, testb ;
    int ii, numgood ;

    /*** Rebuild the arrays to use, skipping those points flagged by a -1. ***/

    numgood = 0 ;
    pX = X ;
    pY = Y ;
    pGX = Var_X ;
    pGY = Mean ;
    for (ii = 0 ; ii < npts ; ii++)
	{
	if ( *pX >= 0.0 )
	    {
	    numgood++ ;
	    *pGX = (float) (*pX) ;
	    *pGY = (float) (*pY) ;
	    pGX++ ;
	    pGY++ ;
	    }
	pX++ ;
	pY++ ;
        }

    pGX = Var_X ; 
    pGY = Mean ;
    do_lst (&a, &b, &R, pGX, pGY, numgood, 1) ;
    testa = ( a >= 0.0 ) ? a : -1.0 * a ;
    testb = ( b >= 0.0 ) ? b : -1.0 * b ;

    if (R > Rlimit)
	{
	pY = Y ;
	for (ii = 0 ; ii < npts ; ii++)
	    {
	    *pY = (double) ( a + b *(float)ii ) ;
	    pY++ ;
	    }
	fprintf (stderr, "\t\tGood correlation and can replace.\n") ;
	return (1) ;
        }
    else if ((testb * 100.0 < testa) && (R > 0.9 * Rlimit))
	{
	pY = Y ;
	for (ii = 0 ; ii < npts ; ii++)
	    {
	    *pY = (double)( a + b *(float)ii ) ;
	    pY++ ;
	    }
	fprintf (stderr, "\t\tAcceptable correlation and can replace.\n") ;
	return (0) ;
        }
    else
	{
	fprintf (stderr, "\t\tBAD correlation and can NOT replace.\n") ;
	return (-1) ;
        }
}

/* -------------------------------------------------------------------------- */

int
zeroLO ( im, xdim, spot )

float *im ; int xdim ;
int spot ;
{
    int ii ;
    float *pf ;

    fprintf (stderr, "\t\tZeroing LOW\n") ;

    pf  = im ;
    for (ii = 0 ; ii < spot * xdim ; ii++)
	{
	*pf = 0.0 ;
	pf++ ;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int
zeroHI ( im, xdim, ydim, spot )

float *im ; int xdim ; int ydim ; int spot ;
{
    int ii, imsz ;
    float *pf ;

    fprintf (stderr, "\t\tZeroing HIGH\n") ;

    imsz = xdim * ydim ;

    pf  = im + spot * xdim ;
    for (ii = spot * xdim ; ii < imsz ; ii++)
	{
	*pf = 0.0 ;
	pf++ ;
        }

    return (0) ;
}

