/* crop_img.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Crop image.
 * The result of the function is a pointer to the output cropped image.
 *
 * usage: imgout = crop_img (img, nx, ny, nbypix, xcen, ycen, nxout, nyout)
 *
 * The sun center of the output image coincides with the array center.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   26 June 2001
 * 15 Dec 2004: Revised logic for computing xo & yo.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <stdlib.h>

unsigned char *
crop_img (unsigned char *imginp, int xdiminp, int ydiminp, int nbypix, float xceninp, float yceninp, int xdimout, int ydimout)

/*
 * unsigned char *imginp ;		Input image.
 * int xdiminp, ydiminp ;		Input image dimensions.
 * int nbypix ;				Pixel depth of input image.
 * float xceninp, yceninp ;		Sun center  of input image.
 * int xdimout, ydimout ;		Output image dimensions.
 */

{
    float blint (unsigned char *imgbuf, int nbypix, int xdim, int ydim,
                 float x, float y);

    unsigned char *imgout ;   
    unsigned char *cibuf ;
    unsigned char *cobuf ;

    short *sibuf ;
    short *sobuf ;

    float *fobuf ;

    int ix, iy ;
    float xo, yo ;

    float fval ;
    float xcenout, ycenout ;		/* Sun center of ouptut image. */
    float xx, yy ;

    /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

    /*** Define the output sun center to coincide with the array center. ***/

    xcenout = (xdimout - 1) * 0.5 ;	/* X center of output image. */
    ycenout = (ydimout - 1) * 0.5 ;	/* Y center of output image. */

    /***
      *  Compute the X & Y origin (lower left corner) of the
      *  subset [cropped] image within the original image.
      *.
      *  Using the center of the input image as a reference,
      *  subtract the center of the output image to get
      *  the starting coordinates for the subset image origin.
     ***/

    xo = xceninp - xcenout ;
    yo = yceninp - ycenout ;

    printf ("crop_img> xceninp/yceninp: %7.2f  %7.2f\n", xceninp, yceninp);
    printf ("crop_img> xcenout/ycenout: %7.2f  %7.2f\n", xcenout, ycenout);
    printf ("crop_img> xo/yo: %7.2f  %7.2f\n", xo, yo) ;
    ix = (int) xceninp ;
    iy = (int) yceninp ;
    printf ("crop_img> imginp: %d\n", *(imginp + ix + iy*xdiminp));

    cibuf  = (unsigned char *) imginp ;
    imgout = (unsigned char *) malloc (xdimout * ydimout * nbypix) ;

    /*** Perform translation using bilinear interpolation. ***/

    if (nbypix == 1)
	{
	cobuf = (unsigned char *) imgout ;
        for (iy = 0 ; iy < ydimout ; iy++)
	    {
            yy = (float) iy + yo ;
	    for (ix = 0 ; ix < xdimout ; ix++)
	        {
	        xx = (float) ix + xo ;
                if (yy < 0 || yy >= ydimout ||
                    xx < 0 || xx >= xdimout)
                    fval = 0.0 ;
                else
                    {
	            fval = blint (cibuf, nbypix, xdiminp, ydiminp, xx, yy) ;
	            if (fval < 0)
	                fval = 0.0 ;
                    }
	        *(cobuf + ix + iy*xdimout) = (unsigned char) fval ;
	        }
	    }
	}
    else
    if (nbypix == 2)
	{
	sibuf = (short *) imginp ;
	sobuf = (short *) imgout ;
        for (iy = 0 ; iy < ydimout ; iy++)
	    {
            yy = (float) iy + yo ;
	    for (ix = 0 ; ix < xdimout ; ix++)
	        {
	        xx = (float) ix + xo ;
                if (yy < 0 || yy >= ydiminp ||
                    xx < 0 || xx >= xdiminp)
                    fval = 0.0 ;
                else
                    {
                    fval = *(sibuf + (int) xx + (int) yy * xdiminp) ;
	            fval = blint (imginp, nbypix, xdiminp, ydiminp, xx, yy) ;
	            if (fval < 0)
	                fval = 0.0 ;
/*
                    printf ("xx/yy: %7.2f  %7.2f  fval: %7.2f\n", xx, yy, fval);
                    fflush (stdout) ;
 */
                    }
/*
                printf ("ix/iy %4d  %4d\n", ix, iy) ;
                fflush (stdout) ;
 */
	        *(sobuf + ix + iy*xdimout) = (short) fval ;
	        }
	    }
	}
    else
    if (nbypix == 4)
	{
	fobuf = (float *) imgout ;
        for (iy = 0 ; iy < ydimout ; iy++)
	    {
            yy = (float) iy + yo ;
	    for (ix = 0 ; ix < xdimout ; ix++)
	        {
	        xx = (float) ix + xo ;
                if (yy < 0 || yy >= ydimout ||
                    xx < 0 || xx >= xdimout)
                    fval = 0.0 ;
                else
                    {
	            fval = blint (cibuf, nbypix, xdiminp, ydiminp, xx, yy) ;
	            if (fval < 0)
	                fval = 0.0 ;
                    }
	        *(fobuf + ix + iy*xdimout) = (short) fval ;
	        }
	    }
	}
    else
	{
	fprintf (stderr, "Invalid data type . . . nbypix = %d\n", nbypix) ;
	return (NULL) ;
	}

    return (imgout) ;
}
