/* picscal_nodiffs.c
 * -----------------------------------------------------------------------------
 * Remove bright pixels from a solar PICS image.
 * -----------------------------------------------------------------------------
 * Author: Tony Darnell   HAO/NCAR   7 Feb 2003
 * Updates:
 * [ALS] 10 Oct 2003: CODE_OBS = 'ASC' for disc - bias images.
 * [TD]  10 Dec 2004: Remove diffuser image code.
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include "fitsio.h"
#include "libacos.h"

main (int argc, char *argv [])
{
  char datestr [11];
  char *rawname = "", *outname = "";

  int anynull = -1;
  int bitpix;
  int i;
  int ix, iy;
  int nullval;
  int status = 0;
  int year, month, day;

  short *calimg;

  long fpixel = 1, nelements;
  long xsz, ysz;
  long naxes [2] = {0,0}, naxis = 2;

  float *rawimg;
  float avg, adev;
  float brtavg;
  float brtpct = 0.05;
  float crradius;
  float curt;
  float radius;
  float sdev;
  float skew;
  float var;
  float xcen, ycen;
  float xdist, ydist;

  fitsfile *rawptr, *outfptr;

  /* ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ */

  if (argc != 3)
    {
    printf ("usage:  picscal_nodiffs raw_image outputname\n");
    exit (1);
    }

  rawname  = argv [1];
  outname  = argv [2];

  fits_open_file (&rawptr, rawname, READONLY, &status);
  
  /*** Get today's date. ***/

  fits_get_system_date (&day, &month, &year, &status);
  fits_date2str (year, month, day, datestr, &status);

  fits_read_key (rawptr, TINT,   "NAXIS1",   &xsz,      NULL, &status);
  fits_read_key (rawptr, TINT,   "NAXIS2",   &ysz,      NULL, &status);
  fits_read_key (rawptr, TINT,   "BITPIX",   &bitpix,   NULL, &status);
  fits_read_key (rawptr, TFLOAT, "CRRADIUS", &crradius, NULL, &status);
  fits_read_key (rawptr, TFLOAT, "CRPIX1",   &xcen,     NULL, &status);
  fits_read_key (rawptr, TFLOAT, "CRPIX2",   &ycen,     NULL, &status);

  naxes [0] = xsz;
  naxes [1] = ysz;

  if (xcen < 0) xcen = (xsz / 2.0) - 0.5;
  if (ycen < 0) ycen = (ysz / 2.0) - 0.5;

  if (status)
    {
    fits_report_error (stderr, status); /* print error message */
    return (status);
    }

  nelements = xsz * ysz;
  rawimg    = (float *) malloc (nelements * sizeof (float));
  calimg    = (short *) malloc (nelements * sizeof (short));

  fits_read_img (rawptr, TFLOAT, fpixel, nelements, &nullval, rawimg,
                         &anynull, &status);

  fits_close_file (rawptr, &status);

  /* Clean up raw image of hot pixels and negative values
   * due to wraparound of hot pixels.
   */

  acos_moment (rawimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);

  for (i = 0; i < nelements; i++)
    {
    if (rawimg [i] < 0)         rawimg [i] = 0;
    if (rawimg [i] > 7 * sdev)  rawimg [i] = 0;
    }

  acos_moment (rawimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);

  for (i = 0; i < nelements; i++)
    calimg [i] = (short) rawimg [i];

  /*** Get rid of outfile if it already exists ***/
  /* acos_remove_file(outname); */

  fits_create_template (&outfptr, outname, rawname, &status);
  fits_update_key (outfptr, TINT, "BITPIX", &bitpix, "2-byte short", &status);
  fits_set_hdustruc (outfptr, &status);

  fits_update_key (outfptr, TSTRING, "DATE-CAL", &datestr, NULL, &status); 
  fits_update_key (outfptr, TSTRING, "CODE-OBS", "ASC",    NULL, &status); 

  /*** Write the corrected image to disk ***/

  fits_write_img (outfptr, TSHORT, fpixel, nelements, calimg, &status);
  
  fits_close_file (outfptr, &status);

  exit (0);
}
