/* picsldc3.cpp
 * -----------------------------------------------------------------------------
 * Remove limb darkening from a PICS disc image.
 * -----------------------------------------------------------------------------
 * Author: Tony Darnell   HAO/NCAR   26 Aug 2003
 * Revisions:
 * [ALS] 10 Oct 2003: CODE_OBS = 'ASL' for disc, limb dark corr.
 * [ALS]  9 Dec 2004: Update sun center (CRPIX1/CRPIX2) in ASC image.
 * [ALS]  9 Jan 2005: Do NOT perform image rotation.
 * -----------------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <values.h>

#include "SciCPP.hpp"
#include "fitsio.h"
#include "libacos.h"
#include "libldc.hpp"

/*
 * extern short *
 * acos_frotimg (float *fimg, int xdim, int ydim, float xcen, float ycen,
 *                            float angdeg, float oval);
 * extern void
 * acos_shift (short *simg, float *fimg, int xdim, int ydim,
 *                          float xshift, float yshift);
 * void acos_sharpen (short *simg, int xdim, int ydim, int sharpen)
 */

main (int argc, char *argv [])
{
  extern unsigned char *crop_img (unsigned char *imginp,
                                  int xdiminp, int ydiminp, int nbypix,
                                  float xceninp, float yceninp,
                                  int xdimout, int ydimout);
  char asc_name     [80];
  char asl_name     [80];
  char code_obs     [ 8] = "ASL";
  char codeobs_com  [50] = "type of observation [character code]";
  char coordnam     [20] = "Heliocentric";
  char coord_com    [50] = "Coordinate System name";
  char crota1_com   [50] = "X [East->West]   rotation [deg] from reference";
  char crota2_com   [50] = "Y [South->North] rotation [deg] from reference";
  char datestr      [12];
  char fasl_name    [80];
  char type_dp      [20] = "LDC";
  char typedp_com   [50] = "Limb Darkening Correction applied";

  short smin = 0;
  short smax = 0;
  short sval = 0;

  int bitpix  = 16;
  int count   = 0;
  int day     = 0;
  int month   = 0;
  int year    = 0;
  int dmin    = 10000;
  int dmax    = 0;
  int i       = 0;
  int ii      = 0;
  int ip      = 0;
  int j       = 0;
  int k       = 0;
  int ix      = 0;
  int iy      = 0;
  int jj      = 0;
  int nbypix  = 2;
  int nc      = 0;
  int pmax    = 5000;
  int status  = 0;
  int xdim    = 0;
  int xdimsq  = 1024;
  int ydim    = 0;
  int ydimsq  = 1024;

  long naxes [2], fpixel = 1, nelements, naxis = 2;
  
  float avg      = 0.0;
  float bg_amp   = 0.0;
  float crradius = 0.0;
  float datamin  = 0.0;
  float datamax  = 0.0;
  float dispmin  = 0.0;
  float dispmax  = 0.0;
  float fitmax   = 0.0;
  float fitref   = 1000.0 ;
  float fmin     = 10000.0 ;
  float fmax     = 0.0;
  float hours    = 0.0;
  float mins     = 0.0;
  float pangle   = 0.0;
  float radius   = 0.0;
  float rotang   = 0.0;
  float rx       = 0.0;
  float ry       = 0.0;
  float secs     = 0.0;
  float xcen     = 0.0;
  float xcensq   = 0.0;
  float xdist    = 0.0;
  float xshft    = 0.0;
  float ycen     = 0.0;
  float ycensq   = 0.0;
  float ydist    = 0.0;
  float yshft    = 0.0;

  double seconds;

  float		*fimg;
  short		*simg;
  short		*sbuf;
  unsigned char *cimg;

  fitsfile	*fasc, *fasl ;

  Matrix	asc_img;
  Matrix        fit_img;
  Matrix        flat_img;
  Matrix        mask;
  Vector	dr;
  Vector        di;
  Vector        dmu;
  Vector        MedianCoef;
  CoefLabel	*coef;
  time_t	start;
  time_t        stop;

  /* ------------------------------------------------------------------------ */

  if (argc < 2)
    {
    printf ("\nusage:  picsldc image1 [image2 image3 ...]\n\n");
    exit (1);
    }

  printf ("\nPICSLDC:  There are %d file(s) to process\n\n", argc - 1);

  /*** File processing loop. ***/

  for (ip = 1; ip < argc; ip++)
    {
    strcpy (asc_name, argv [ip]);
    printf ("\nProcessing %s\n\n", asc_name);
    start = time (NULL);

    /*** Create output file name. ***/

    strcpy (asl_name, asc_name);
    strcpy (strstr (asl_name, "sc"), "sl.fts");		/* asc.fts -> asl.fts */
    sprintf (fasl_name, "!%s", asl_name);

    /*** Read image array. ***/

    asc_img = readfits (asc_name);

    xdim = asc_img.Rows ();
    ydim = asc_img.Columns ();

    /*** Open FITS image. ***/

    fits_open_file (&fasc, asc_name, READWRITE, &status);

    /*** Get solar radius [pixels] and rotation angle. ***/

    fits_read_key  ( fasc, TINT,    "NAXIS1",   &xdim,     NULL,       &status);
    fits_read_key  ( fasc, TINT,    "NAXIS2",   &ydim,     NULL,       &status);
    fits_read_key  ( fasc, TFLOAT,  "CRRADIUS", &crradius, NULL,       &status);
    fits_read_key  ( fasc, TFLOAT,  "CROTA1",   &pangle,   crota1_com, &status);
    fits_read_key  ( fasc, TFLOAT,  "CROTA2",   &pangle,   crota2_com, &status);
    fits_read_key  ( fasc, TFLOAT,  "DATAMIN",  &datamin,  NULL,       &status);
    fits_read_key  ( fasc, TFLOAT,  "DATAMAX",  &datamax,  NULL,       &status);

    dispmin = datamax * 0.25;
    dispmax = datamax * 0.90;

    /*** Find sun center and disk radius. ***/

    solargeom (asc_img, &xcen, &ycen, &rx, &ry);

    /*** Update sun center information in FITS header. ***/

    fits_update_key (fasc, TFLOAT,  "CRPIX1",   &xcen,     NULL, &status);
    fits_update_key (fasc, TFLOAT,  "CRPIX2",   &ycen,     NULL, &status);
    fits_update_key (fasc, TFLOAT,  "DATAMIN",  &datamin,  NULL, &status);
    fits_update_key (fasc, TFLOAT,  "DATAMAX",  &datamax,  NULL, &status);
    fits_update_key (fasc, TFLOAT,  "DISPMIN",  &dispmin,  NULL, &status);
    fits_update_key (fasc, TFLOAT,  "DISPMAX",  &dispmax,  NULL, &status);
    fits_update_key (fasc, TFLOAT,  "XCEN_RAW", &xcen,     NULL, &status);
    fits_update_key (fasc, TFLOAT,  "YCEN_RAW", &ycen,     NULL, &status);
    fits_close_file (fasc, &status);

    /*** Compute average intensity in a background region. ***/

    count  = 0;
    bg_amp = 0.0;

    for (ii = 100; ii <= 200; ii++)
        for (jj = 100; jj <= 200; jj++)
	    {
	    bg_amp += asc_img (ii, jj);
	    count++;
            }

    bg_amp /= count;
    if (bg_amp < 0.0)
        bg_amp = 1.0;
    printf ("Average background intensity:  %f  (%d)\n", bg_amp, count);

    /*** Compute average intensity at disk center. ***/

    avg   = 0.0;
    count = 0;
    for (ii = xcen-10; ii <= xcen+9; ii++)
        for (jj = ycen-10; jj <= ycen+9; jj++)
	    {
	    avg += asc_img (ii, jj);
	    count++;
            }
    avg /= count;
    printf ("Disk center average intensity:  %f  (%d)\n", avg, count);

    /*** Compute limb darkening profile. ***/

    mask = quartile_profile   (asc_img,    xcen, ycen, Min (rx, ry),
                                           1.2, -1.2, di, dr);
    coef = legendre_fourier   (asc_img,    xcen, ycen, Min (rx, ry),
                                           mask, 1, 4, &nc);
    fit_img  = make_fit_image (xdim, ydim, xcen, ycen, Min (rx, ry),
                                           1, 4, coef);
    fitmax = fit_img.Max ();
    
    writefits         ("quartile_fit.fts", fit_img);
    writefits         ("mask.fts",         mask);
    ChangeDimensions  (xdim, ydim,         &flat_img);

    /*** Remove limb darkening profile. ***/

    for (j = 1; j <= ydim; j++)
        for (i = 1; i <= xdim; i++)
	    {
	    xdist = (float) i - xcen;
	    ydist = (float) j - ycen;
	    radius = sqrt ((xdist * xdist) + (ydist * ydist)) ;
	    radius /= crradius ;

	    if (radius <= 1.0)
	        flat_img (i,j) = (asc_img (i,j) / fit_img (i,j)) * fitref;
            else
	        flat_img (i,j) = asc_img (i,j);
            }

    naxes [0] =         flat_img.Rows ();
    naxes [1] =         flat_img.Columns ();
    datamin   = (float) flat_img.Min ();
    datamax   = (float) flat_img.Max ();
    dispmin   =  800.0;
    dispmax   = 1300.0;
    nelements = naxes [0] * naxes [1];
    fimg      = new float [nelements];

    /*** Convert image to float. ***/

    k = 0;
    for (j = 1; j <= naxes [1]; j++)
        for (i = 1; i <= naxes [0]; i++)
	    {
            fimg [k] = flat_img (i, j);
            k++;
            }

    /*** Get current date. ***/

    fits_get_system_date  (&day, &month, &year, &status);
    fits_date2str         (year, month, day, datestr, &status);


    /*** Copy image into a short array. ***/

    simg = (short *) malloc (xdim * ydim * sizeof (short)) ;
    for (iy = 0; iy < ydim; iy++)
        for (ix = 0; ix < xdim; ix++)
            {
            *(simg + ix + iy*xdim) = (short) *(fimg + ix + iy*xdim) ;
            }

    /*** Rotate the image: geocentric -> heliocentric orientation. ***/

/*
 *    xshft = (xdim / 2.0) - 0.5 - xcen;
 *    yshft = (ydim / 2.0) - 0.5 - ycen;
 *    printf ("Rotating for a P-angle of:  %f\n", pangle);
 *
 *    simg = (short *) acos_frotimg (fimg, xdim, ydim, xcen, ycen, pangle, 0.0);
 *
 *    rotang = 0.00;
 */

    /*** Find image min/max. ***/

    smin = 100000;
    smax =      0;
    for (iy = 0; iy < ydim; iy++)
        for (ix = 0; ix < xdim; ix++)
            {
            sval = *(simg + ix + iy*xdim);
            if (smin > sval)  smin = sval;
            if (smax < sval)  smax = sval;
            }
    datamin = (float) smin;
    datamax = (float) smax;

    printf ("smin: %d  smax: %d\n", smin, smax);

    /*** Shift the image: make sun center = array center. ***/
/*
 *    acos_shift (simg, fimg, xdim, ydim, xshft, yshft);
 * 
 *    printf ("xdim/ydim: %4d %4d  nbypix: %d  xcen/ycen: %7.2f %7.2f\n",
 *             xdim, ydim, nbypix, xcen, ycen) ;
 *    fflush (stdout) ;
 */

    /*** Crop the image: [xdimsq, ydimsq] with sun center = array center. ***/

    sbuf = simg ;
/*
 *    xcensq = (xdimsq / 2) - 0.5;
 *    ycensq = (ydimsq / 2) - 0.5;
 *    cimg = (unsigned char *) simg;
 *    sbuf = (short *) crop_img (cimg, xdim, ydim, nbypix, xcen, ycen,
 *                               xdimsq, ydimsq) ;
 *    ix = (int) xcensq ;
 *    iy = (int) ycensq ;
 *    printf ("sbuf: %d\n", *(sbuf + ix + iy*xdimsq));
 *
 *    printf ("crop_img done.\n");
 *    fflush (stdout);
 */

    /*** Sharpen the image. ***/

    // acos_sharpen (sbuf, gxdim, ydim, 50);

    /*** Get rid of outfile if it already exists ***/

/*
 *    fits_open_file (&fasl, asl_name, READWRITE, &status);
 *    if (status == 0)
 *        {
 *        printf ("%s exists.  Deleting...\n",asl_name);
 *        fits_delete_file (fasl, &status);
 *        }
 *    else
 *        {
 *        status = 0;
 *        printf ("%s doesn't exist.\n", asl_name);
 *        }
 */
    printf ("Writing file:  %s\n", asl_name);

    /*** Create output FITS file. ***/

    fits_create_template (&fasl, fasl_name, asc_name, &status);
/*
 *    naxes [0] = xdimsq ;
 *    naxes [1] = ydimsq ;
 *    fits_resize_img (fasl, bitpix, naxis, naxes, &status);
 */

    fits_update_key (fasl, TINT,    "BITPIX",   &bitpix,  NULL, &status);
    fits_update_key (fasl, TINT,    "NAXIS1",   &xdim,     NULL, &status);
    fits_update_key (fasl, TINT,    "NAXIS2",   &ydim,     NULL, &status);
    fits_update_key (fasl, TFLOAT,  "CRPIX1",   &xcen,     NULL, &status);
    fits_update_key (fasl, TFLOAT,  "CRPIX2",   &ycen,     NULL, &status);
/*
 *    fits_update_key (fasl, TINT,    "NAXIS1",   &xdimsq,  NULL, &status);
 *    fits_update_key (fasl, TINT,    "NAXIS2",   &ydimsq,  NULL, &status);
 *    fits_update_key (fasl, TFLOAT,  "CRPIX1",   &xcensq,  NULL, &status);
 *    fits_update_key (fasl, TFLOAT,  "CRPIX2",   &ycensq,  NULL, &status);
 */
    fits_update_key (fasl, TFLOAT,  "XCEN_RAW", &xcen,    NULL, &status);
    fits_update_key (fasl, TFLOAT,  "YCEN_RAW", &ycen,    NULL, &status);
/*
    fits_update_key (fasl, TFLOAT,  "CROTA1",   &rotang,  NULL, &status);
    fits_update_key (fasl, TFLOAT,  "CROTA2",   &rotang,  NULL, &status);
 */
    fits_update_key (fasl, TSTRING, "TYPE-DP",  type_dp,  typedp_com, &status);
    fits_update_key (fasl, TSTRING, "CODE-OBS", code_obs, NULL, &status);
    fits_update_key (fasl, TFLOAT,  "CRRADIUS", &rx,      NULL, &status);
    fits_update_key (fasl, TSTRING, "DATE-CAL", datestr,  NULL, &status);
    fits_update_key (fasl, TSTRING, "COORDNAM", coordnam, NULL, &status);
    fits_update_key (fasl, TFLOAT,  "DATAMIN",  &datamin, NULL, &status);
    fits_update_key (fasl, TFLOAT,  "DATAMAX",  &datamax, NULL, &status);
    fits_update_key (fasl, TFLOAT,  "DISPMIN",  &dispmin, NULL, &status);
    fits_update_key (fasl, TFLOAT,  "DISPMAX",  &dispmax, NULL, &status);

    fits_set_hdustruc (fasl, &status);

    /*** Write the output image array. ***/

    fits_write_img (fasl, TSHORT, fpixel, nelements, sbuf, &status);

    /*** Close the output FITS file. ***/

    fits_close_file (fasl, &status);
    fits_report_error (stderr, status);

    /*** Elapsed time to perform the limb darkening correction. ***/

    stop = time (NULL);
    seconds = difftime (stop, start);
    hours = seconds / 3600.0;
    if (hours < 1) {hours = 0.0;}
    mins = seconds / 60.0;
    if (mins < 1) { mins = 0.0;}
    secs = (int) seconds % 60;
    printf ("\n\tTime Elapsed: %02.0f:%02.0f:%05.2f\n\n", hours, mins, secs);

 } //endfor

 printf ("<========= Finished ==========>\n\n");
 exit (0) ;
}
