/* rdmk3hk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read MLSO Mark-III housekeeping and return it as a structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   6 January 1994
 * 11 Feb 94: add ability to read datatype, tmin/tmax, & caldate
 *              from ASCII footer.
 * 24 Feb 94: add opc date & time.
 * 25 Feb 94: modify logic to read "date_calibration_created" in ASCII footer.
 * 25 Apr 94: dtype 23 is now 24 (avgrpB);  dtype 23 is avgpB.
 *  9 Sep 94: use WHIP footer to get nbypx.
 * 18 Apr 95: get pixrs from ASCII footer, since ipixscl is incorrect for
 *            channel 1 images.
 * 30 May 97: Increase # characters backed up for footer search from 400 to 800.
 *            This was necessary due to extraneous characters inserted at end
 *            of tailer by slit conversion program.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/cordyn/mlsofits/lib/mk3/whip.h"
#include "/home/cordyn/mlsofits/lib/mk3/mk3lib.ext"
#define BKWDS 1200

rdmk3hk (namfil, pmode)

char namfil [] ;			/* Image filename. */
int pmode ;				/* Print mode. */
{
   FILE *FD ;				/* File pointer. */
   struct pic_hdr WH ;			/* WHIP header. */
   struct pic_hdr *wh = &WH ;		/* WHIP header. */

   int cal_year, cal_month, cal_day;	/* Calibration date. */
   int opc_year, opc_month, opc_day;	/* Calibration date. */
   int opc_hour, opc_minute, opc_second;	/* Calibration date. */
   int dtype;				/* Numerical data type [0-24]. */
   int eof = 0;				/* =1 : EOF encountered. */
   int eofloc;				/* EOF location (bytes). */
   int footloc, asciiloc;		/* footer locations: binary, ascii. */
   int whiploc ;			/* WHIP footer location. */
   int isec, fsec ;			/* seconds integral & fractional parts*/
   int sizeHS ;				/* Size of MK3HS structure. */
   float tmin, tmax;			/* Begin/end barrel angles. */
   float fpixrs ;			/* pixels per solar radius. */
   static char caldate [16] ;		/* ASCII representation of cal date. */
   static char opcdate [16] ;		/* ASCII representation of opc date. */
   static char opctime [16] ;		/* ASCII representation of opc time. */
   static char datatype [8] ;		/* ASCII representation of data type. */
   static char ss [40];			/* General purpose character string. */ 
   static char afoot [2000];		/* ASCII footer. */
   static short whipfoot [12] ;		/* WHIP footer. */

   float csec ;

   sizeHS = sizeof (MK3HS) ;

   if (pmode) fprintf (stdout, "rdmk3hk\n") ;

   /*** Open Mark-III image file. ***/

   if ((FD = fopen (namfil, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namfil) ;
      return (-1) ;
      }
   if (pmode) fprintf (stdout, "rdmk3hk> open file: %s\n", namfil) ;

   /*** Get footer locations from ASCII footer. 
    ***
    *** MK3 file form:
    *** binary data (512x512 short ints/128xnrays short ints/nrays short ints)
    *** binary information (housekeeping data = 40 bytes)
    *** WHIP header info (12 short ints = 24 bytes)
    *** ASCII footer is at end of the file
    ***/

   fseek (FD, 0, 2);	/* Skip to end of file. */
   if (pmode) fprintf (stdout, "rdmk3hk> skip to end of file\n") ;
   eofloc = ftell (FD) ;
   fseek (FD, -800, 2);	/* Skip several 80-column lines back in ASCII footer.*/
   if (pmode) fprintf (stdout, "rdmk3hk> skip 400 char before end of file\n") ;

   /* Binary footer location. */

   fscanf (FD, "%s", ss);
   while ( strcmp (ss, "entire") != 0 )
      fscanf (FD, "%s", ss);
   fscanf (FD, "%*s%ld", &footloc);
   if (pmode)
      fprintf (stdout, "footloc %d\n", footloc);
   fflush (stdout) ;

   /* WHIP   footer location. */

   whiploc = 0 ;
   fscanf (FD, "%s", ss);
   while ( strcmp (ss, "WHIPS") != 0 )
      fscanf (FD, "%s", ss);
   fscanf (FD, "%*s%ld", &whiploc);
   if (pmode)
      fprintf (stdout, "whiploc %d\n", whiploc);
   fflush (stdout) ;

   /*** Read WHIP  footer. ***/

   if (whiploc >= 0)
      {
      fseek (FD, whiploc, SEEK_SET);
      fread (wh,  sizeof (WH), 1, FD);
      if (wh->id != 0)
         switch (wh->id)
           {
           case BYTE  :   MK3I->nbypx = 1 ;  break ;
           case SHORT :   MK3I->nbypx = 2 ;  break ;
           case FLOAT :   MK3I->nbypx = 4 ;  break ;
           }
         else
            MK3I->nbypx = 0 ;
      }
   else MK3I->nbypx = 0 ;

   /* ASCII footer location. */

   fscanf (FD, "%s", ss);
   while ( strcmp (ss, "ascii") != 0 )
      fscanf (FD, "%s", ss);
   fscanf (FD, "%*s%ld", &asciiloc) ;
   if (pmode)
      fprintf (stdout, "asciiloc %d\n", asciiloc);
   fflush (stdout) ;

   /*** Read ASCII footer. ***/

   fseek (FD, asciiloc, SEEK_SET);
   fread (afoot,   1,  eofloc - asciiloc + 1  , FD);
   if (pmode)
      fputs (afoot, stdout);
   fseek (FD, asciiloc, SEEK_SET);
   fscanf (FD, "%s", ss);

   /*** Get Scale Size. ***/

   while ( strcmp (ss, "scale_size") != 0 )
      {
      fscanf (FD, "%s", ss);
      }
   fscanf (FD, "%f", &fpixrs) ;
   MK3I->pixrs = fpixrs ;

   /*** Get data ASCII type. ***/

   while ( strcmp (ss, "type") != 0 )
      {
      fscanf (FD, "%s", ss);
      }
   fscanf (FD, "%s", datatype) ;

   /*** Get min/max barrel angles. ***/

   fscanf (FD, "%s", ss);
   while ( strcmp (ss, "from") != 0 )
      {
      fscanf (FD, "%s", ss);
      }
   fscanf (FD, "%f", &tmin) ;
   fscanf (FD, "%s", ss) ;
   fscanf (FD, "%f", &tmax) ;

   MK3I->tmin = tmin ;
   MK3I->tmax = tmax ;

   /*** Get calibration date. ***/

   eof = 0 ;
   strcpy (caldate, "00/00/00") ;
   while ( strcmp (ss, "date_calibration_created") != 0 )
      {
      if (fscanf (FD, "%s", ss) == EOF)
	 {
	 fprintf (stderr,
		  "'date_calibration_created' not found in ASCII footer.\n") ;
         eof = 1 ;
         fseek (FD, asciiloc, SEEK_SET);
	 break ;
	 }
      }
   if (!eof)
      fscanf (FD, "%s", caldate) ;		/* Calibration creation date. */

   /*** Get Opal Calibration 0 date. ***/

   eof = 0 ;
   strcpy (opcdate, "00/00/00") ;
   while ( strcmp (ss, "calibration") != 0 )
      {
      if (fscanf (FD, "%s", ss) == EOF)
	 {
	 fprintf (stderr,
		  "'calibration date' not found in ASCII footer.\n") ;
         eof = 1 ;
         fseek (FD, asciiloc, SEEK_SET);
	 break ;
	 }
      }

   while ( strcmp (ss, "date") != 0 )
      {
      if (fscanf (FD, "%s", ss) == EOF)
	 {
	 fprintf (stderr,
		  "'calibration date' not found in ASCII footer.\n") ;
         eof = 1 ;
         fseek (FD, asciiloc, SEEK_SET);
	 break ;
	 }
      }
   if (!eof)
      {
      fscanf (FD, "%s", opcdate) ;		/* Cal 0 (opal only) date. */
      }

   /*** Get Opal Calibration 0 time. ***/

   eof = 0 ;
   strcpy (opctime, "00_00_00.00") ;
   while ( strcmp (ss, "calibration") != 0 )
      {
      if (fscanf (FD, "%s", ss) == EOF)
	 {
	 fprintf (stderr,
		  "'calibration time' not found in ASCII footer.\n") ;
         eof = 1 ;
         fseek (FD, asciiloc, SEEK_SET);
	 break ;
	 }
      }

   while ( strcmp (ss, "time") != 0 )
      {
      if (fscanf (FD, "%s", ss) == EOF)
	 {
	 fprintf (stderr,
		  "'calibration time' not found in ASCII footer.\n") ;
         eof = 1 ;
         fseek (FD, asciiloc, SEEK_SET);
	 break ;
	 }
      }
   if (!eof)
      {
      fscanf (FD, "%s", opctime) ;		/* Cal 0 (opal only) time. */
      }

   sscanf (caldate, "%2d/%2d/%2d", &cal_month, &cal_day, &cal_year) ;

   sscanf (opcdate, "%2d/%2d/%2d", &opc_month, &opc_day, &opc_year) ;

   sscanf (opctime, "%2d_%2d_%2d", &opc_hour, &opc_minute, &isec) ;

   opc_second = (int) (isec) ;

   if (cal_year == 0)
      cal_year = cal_month = cal_day = 0 ;

   MK3I->cal_year   = cal_year ;		/* Date image calibrated. */
   MK3I->cal_month  = cal_month ;
   MK3I->cal_day    = cal_day ;

   MK3I->opc_year   = opc_year ;		/* Date of cal 0 (opal only). */
   MK3I->opc_month  = opc_month ;
   MK3I->opc_day    = opc_day ;

   MK3I->opc_hour   = opc_hour ;		/* Time of cal 0 (opal only). */
   MK3I->opc_minute = opc_minute ;
   MK3I->opc_second = opc_second ;

   /* MK3 data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * q   --> cylindrical q data					 0
    * rq  --> rectangular q data, origin in SE				 1
    * srq --> scaled rectangular q data, origin in SE			 2
    * i   --> cylindrical i data					 3
    * ri  --> rectangular i data, origin in SE				 4
    * sri --> scaled rectangular i data, origin in SE			 5
    * u   --> cylindrical u data					 6
    * ru  --> rectangular u data, origin in SE				 7
    * sru --> scaled rectangular u data, origin in SE			 8
    * pB  --> cylindrical pB*1.e-10 data				 9
    * rpB  --> rectangular pB*1.e-10 data, origin in SE			10
    * srpB --> scaled rectangular pB data, origin in SE			11
    * Ne  --> cylindrical Ne data					12
    * rNe  --> rectangular Ne*1.e+33 data, origin in SE			13
    * srNe --> scaled rectangular Ne data, origin in SE			14
    * aba --> barrel angles as ascii in degrees *10			15
    * ba --> barrel angles (shorts) with 16384=360 degrees		16
    * ast --> scan times as ascii in seconds				17
    * st --> scan times (shorts) with 60=1 second (Hz)			18
    * a .calib file							19
    * a .tscp file							20
    * I  --> calibrated cylindrical I data				21
    * U  --> calibrated cylindrical U data				22
    * avgpB  --> avg'd cylindrical pB*1.e-10 data, origin PA 0, r0	23
    * avgrpB --> avg'd rectangular pB*1.e-10 data, origin in SE		24
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   dtype = -1 ;
   if (strcmp (datatype, "q") == 0)		dtype = 0;
   if (strcmp (datatype, "rq") == 0)		dtype = 1;
   if (strcmp (datatype, "srq") == 0)		dtype = 2;
   if (strcmp (datatype, "i") == 0)		dtype = 3;
   if (strcmp (datatype, "ri") == 0)		dtype = 4;
   if (strcmp (datatype, "sri") == 0)		dtype = 5;
   if (strcmp (datatype, "u") == 0)		dtype = 6;
   if (strcmp (datatype, "ru") == 0)		dtype = 7;
   if (strcmp (datatype, "sru") == 0)		dtype = 8;
   if (strcmp (datatype, "pB") == 0)		dtype = 9;
   if (strcmp (datatype, "rpB") == 0)		dtype = 10;
   if (strcmp (datatype, "srpB") == 0)		dtype = 11;
   if (strcmp (datatype, "Ne") == 0)		dtype = 12;
   if (strcmp (datatype, "rNe") == 0)		dtype = 13;
   if (strcmp (datatype, "srNe") == 0)		dtype = 14;
   if (strcmp (datatype, "aba") == 0)		dtype = 15;
   if (strcmp (datatype, "ba") == 0)		dtype = 16;
   if (strcmp (datatype, "ast") == 0)		dtype = 17;
   if (strcmp (datatype, "st") == 0)		dtype = 18;
   if (strcmp (datatype, "calib") == 0)		dtype = 19;
   if (strcmp (datatype, "tscp") == 0)		dtype = 20;
   if (strcmp (datatype, "I") == 0)		dtype = 21;
   if (strcmp (datatype, "U") == 0)		dtype = 22;
   if (strcmp (datatype, "avgpB") == 0)		dtype = 23;
   if (strcmp (datatype, "avgrpB") == 0)	dtype = 24;
   if (strcmp (datatype, "avg") == 0)		dtype = 24;	/* old tailer.*/

   if (pmode)
      fprintf (stdout, "rdmk3hk> datatype: %s   dtype: %d\n", datatype, dtype) ;

   /*** Position file pointer to start of housekeeping information. ***/

   fseek (FD, footloc, SEEK_SET) ;

   /*** Read housekeeping information into structure. ***/

   if (fread ((char *) &MK3HS, sizeHS, 1, FD) == NULL)
      {
      fprintf (stderr, "Unable to read 1st record in file %s.\n", namfil) ;
      return (-1) ;
      }

   fclose (FD) ;

   if (pmode)
      {
      fprintf (stdout, "flagch : %d\n", MK3HS.flagch ) ;
      fprintf (stdout, "juld   : %d\n", MK3HS.juld   ) ;
      fprintf (stdout, "time   : %d\n", MK3HS.time   ) ;
      fprintf (stdout, "phase  : %d\n", MK3HS.phase  ) ;
      fprintf (stdout, "bright : %d\n", MK3HS.bright ) ;
      fprintf (stdout, "pangle : %d\n", MK3HS.pangle ) ;
      fprintf (stdout, "nray   : %d\n", MK3HS.nray   ) ;
      fprintf (stdout, "dmax   : %d\n", MK3HS.dmax   ) ;
      fprintf (stdout, "dmin   : %d\n", MK3HS.dmin   ) ;
      fprintf (stdout, "didxx  : %d\n", MK3HS.didxx  ) ;
      fprintf (stdout, "didxn  : %d\n", MK3HS.didxn  ) ;
      fprintf (stdout, "exdate : %d\n", MK3HS.exdate ) ;
      fprintf (stdout, "scndur : %d\n", MK3HS.scndur ) ;
      fprintf (stdout, "ipixscl: %d\n", MK3HS.ipixscl) ;
      fprintf (stdout, "ir0    : %d\n", MK3HS.ir0    ) ;
      fprintf (stdout, "idr    : %d\n", MK3HS.idr    ) ;
      fprintf (stdout, "\n");
      }

   return (dtype) ;
}
