;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: rtsafe.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, September 2, 1997
;
; Last Modified: Tue Sep  2 13:27:16 1997 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION rtsafe, x, y, x1, x2, acc=acc, silent=silent, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       rtsafe     (function)
;
; PURPOSE:
;       To find roots of tabulated data using a combination of Newton-Raphson
;       and bisection methods.  This code is based on the C code in Numerical
;       Recipes in C, 2nd edition for rtsafe.c.
;
; CALLING SEQUENCE:
;
;       result = rtsafe(x, y, x1, x2 [,help=help])
;
; INPUTS:
;       x - independent variable of the tabulated data.
;       y - dependent variable of the tabulated data.
;   x1,x2 - independent variable values which bracket the root.
;
; OPTIONAL INPUTS: 
;       acc - accuracy for finding root (default is 1.0e-6)
;
; OUTPUTS:
;       Returns the independent variable value where the tabulated function
;       data (y) has a value of zero.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;       /silent.  Will not print error messages.
;
; CALLS:
;       None.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE image processing software.
;
; PREVIOUS HISTORY:
;       Written September 2, 1997, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, September 2, 1997
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'rtsafe'
  RETURN, -999
ENDIF

IF(N_PARAMS() NE 4) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  result = rtsafe(x, y, x1, x2 [,acc=acc, silent=silent, ' + $
	 'help=help])'
  PRINT, ''
  RETURN, -999
ENDIF

IF(n_elements(acc) EQ 0) THEN acc = 1.0e-06

maxit = 100


fl = spline(x, y, x1)
fh = spline(x, y, x2)

dfx = deriv(x, y)

IF((fl GT 0.0 AND fh GT 0.0) OR (fl LT 0.0 AND fh LT 0.0)) THEN BEGIN
   IF(NOT(keyword_set(silent))) THEN BEGIN
      print, '% RTSAFE:  Root must be bracketed.', format = '(/, a, /)'
      print, x1, x2
   ENDIF
   return, min(abs([x1, x2]))
ENDIF

IF(fl EQ 0.0) THEN return, x1
IF(fh EQ 0.0) THEN return, x2


IF(fl LT 0.0) THEN BEGIN
   xl = x1
   xh = x2
ENDIF ELSE BEGIN
   xh = x1
   xl = x2
ENDELSE


rts = 0.5*(x1+x2)

dxold = abs(x2-x1)

dx = dxold

f = spline(x, y, rts)
df = spline(x, dfx, rts)


FOR i = 0, maxit-1 DO BEGIN

   IF((((rts-xh)*df-f)*((rts-xl)*df-f) GE 0.0) OR $
      (abs(2.0*f) GT abs(dxold*df))) THEN BEGIN
      
      dxold = dx
      dx = 0.5*(xh-xl)
      rts = xl+dx
      
      IF(xl EQ rts) THEN return, rts
      
   ENDIF ELSE BEGIN
      
      dxold = dx
      dx = f/df
      temp = rts
      rts = rts-dx
      
      IF(temp EQ rts) THEN return, rts
      
   ENDELSE
   
   IF(abs(dx) LT acc) THEN return, rts
   
   f = spline(x, y, rts)
   
   df = spline(x, dfx, rts)
   
   IF(f LT 0.0) THEN xl = rts ELSE xh = rts
   
ENDFOR

IF(NOT(keyword_set(silent))) THEN BEGIN
   print, '% RTSAFE:  Maximum number of iterations exceeded.'
   plot,x,y
ENDIF

RETURN, rts

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'rtsafe.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
