;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: rtspline.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 18, 1998
;
; Last Modified: Thu Feb 19 09:59:37 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
FUNCTION rtspline, x, fcn, x1, x2, tolx=tolx, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       rtspline     (function)
;
; PURPOSE:
;       To find the zero crossing (root) of a tabulated set of data, using a
;       spline fit to the input data and a simple search method.
;
; CALLING SEQUENCE:
;
;       result = rtspline(x, fcn, x1, x2 [,help=help])
;
; INPUTS:
;       x - tabulated independent variable of the function.
;     fcn - tabulated dependent variable data.  Must be the same size as x.
;      x1 - left bracketing point of root.
;      x2 - right bracketing point of root.
;
; OPTIONAL INPUTS: 
;       tolx - tolerance for finding root (default=1.0e-04).
;
; OUTPUTS:
;       Returns x value location where the zero crossing occurs, to within the
;       tolerance of the search, according to a spline fit of the data.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;
; CALLS:
;       None.
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       SunRISE/PSPT general use.
;
; PREVIOUS HISTORY:
;       Written February 18, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 18, 1998
;-
;
ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'rtspline'
  RETURN, -999
ENDIF

IF(N_PARAMS() NE 4) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  result = rtspline(x, fcn, x1, x2 [,help=help])'
  PRINT, ''
  RETURN, -999
ENDIF

; Check for input tolerance, else set default value.

IF(n_elements(tolx) EQ 0) THEN tolx = 1.0e-04

; Start from left edge bracket.

xx = x1

; Calculate the spline fit of the data at the starting point.

fold = spline(x, fcn, xx) & fold = fold(0)

; Set the initial step size.

dx = 0.5

; Keep track of the number of iteration.

iter = 0L


loopit:

iter = iter+1

; Increment the check point by the step size.

xx = xx+dx

; Calculate the spline fit of the check point.

func = spline(x, fcn, xx) & func = func(0)


IF(iter GT 500) THEN print, iter, xx, dx, func


; Check if we stepped over the zero crossing.

IF(fold*func LT 0) THEN BEGIN
   
   xx = xx-dx                   ; Go back to the previous check point.
   
   dx = dx/2.0                  ; Decrease the step size by half.
   
ENDIF

; Set upper limit of 1000 iterations on search.

IF(iter GT 1000) THEN return, xx

print,'ZEROa =',func


; Check if step size is within tolerance, if not, do the loop again.

IF(abs(dx) GT tolx) THEN GOTO, loopit

print,'ZEROb =',func

RETURN, xx

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'rtspline.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
