/* polyfit.c
 *------------------------------------------------------------------------------
 * Make a least-squares fit to data with a polynomial curve:
 *   y = a [0] + a [1] * x + a [2] * x**2 + a [3] * x**3 + . . .
 *------------------------------------------------------------------------------
 * usage :
 *    x		array of data points for independent values
 *    y		array of data points for dependent values
 *    sigmay	array of standard deviations for y data points.
 *    npts	number of pairs of data points
 *    nterms	number of coefficients (degree of polynomial + 1)
 *    mode	determines method of weighting least-squares fit
 *		+1 (instrumental) weight(i) = 1. / sigmay(i)**2
 *		 0 (no weighting) weight(i) = 1.
 *		-1 (statistical)  weight(i) = 1./y(i)
 *    a		array of coefficients of polynomial
 *    chisqr	reduced chi square for fit
 *------------------------------------------------------------------------------
 * required function :
 *    determ (array, norder)
 *       evaluates the determinant of a symmetic 2-dimensional
 *       matrix of order `norder'.
 *------------------------------------------------------------------------------
 * note: dimension statement valid for `nterms' up to 10.
 *------------------------------------------------------------------------------
 * Bevington, "Data Reduction and Error Analysis for the Physical Sciences"
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 2 September 1988
 * 27 March 1991: converted from Fortran to C.
 *------------------------------------------------------------------------------
 */

polyfit (x, y, sigmay, npts, nterms, mode, a, chisqr)

float x [], y [];
float sigmay [];
int npts, nterms, mode;
float a [];
float *chisqr;

{
   extern float determ ();
   extern double fabs ();

   double sumx [19], sumy [10], array [10][10];
   double xterm, yterm, chisq;

   float delta, free;
   float weight;
   float xi, yi;

   int i, j, k, l, n;
   int nmax;

   /*** Accumulate weighted sums. ***/

   nmax = 2 * nterms - 1;

   for (n = 0; n < nmax; n++)    sumx [n] = 0.0;
   for (j = 0; j < nterms; j++)  sumy [j] = 0.0;

   chisq = 0.0;
   for (i = 0; i < npts; i++)
      {
      xi = x [i];
      yi = y [i];

      if (mode == 0)
         weight = 1.0;

      else
      if (mode < 0)
         {
         if (yi == 0.0)
            weight = 1.0;
         else
            weight = 1.0 / (float) fabs ((double) yi);
         }

      else
      if (mode > 0)
         weight = 1.0 / (sigmay [i] * sigmay [i]);

      xterm = weight;

      for (n = 0; n < nmax; n++)
         {
         sumx [n] += xterm;
         xterm *= xi;
         }

      yterm = weight * yi;

      for (n = 0; n < nterms; n++)
         {
         sumy [n] += yterm;
         yterm *= xi;
         }

      chisq += weight * yi * yi;
      }

   /*** Construct matrices and calulate coefficients. ***/

   for (j = 0; j < nterms; j++)
      for (k = 0; k < nterms; k++)
         {
         n = j + k;
         array [k][j] = sumx [n];
         }

   delta = determ (array, nterms);
   if (delta == 0)
      {
      *chisqr = 0.0;
      for (j = 0; j < nterms; j++)  a [j] = 0.0;
      return (0);
      }

   for (l = 0; l < nterms; l++)
      {
      for (j = 0; j < nterms; j++)
         {
         for (k = 0; k < nterms; k++)
            {
            n = j + k;
            array [k][j] = sumx [n];
            }
         array [l][j] = sumy [j];
         }
      a [l] = determ (array, nterms) / delta;
      }

   /*** Calculate chi-square. ***/

   for (j = 0; j < nterms; j++)
      {
      chisq -= 2.0 * a [j] * sumy [j];

      for (k = 0; k < nterms; k++)
         {
         n = j + k;
         chisq += a [j] * a [k] * sumx [n];
         }
      }

   free = npts - nterms;

   *chisqr = chisq / free;

   return (0);
}


/*
 *------------------------------------------------------------------------------
 * function determ
 *------------------------------------------------------------------------------
 * Calculate the determinant of a square matrix
 *------------------------------------------------------------------------------
 * usage :
 *   det = determ (array, norder)
 *------------------------------------------------------------------------------
 * description of parameters :
 *    array	matrix
 *    norder	order of determinant (degree of matrix)
 *------------------------------------------------------------------------------
 * This routine destroys the input matrix array.
 * Dimension statement valid for norder up to 10.
 *------------------------------------------------------------------------------
 * Bevington, "Data Reduction and Error Analysis for the Physical Sciences"
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 2 September 1988
 * 27 March 1991: converted from Fortran to C.
 *------------------------------------------------------------------------------
 */

float
determ (array, norder)

double array [10][10];
int norder;

{
   double save;
   float det;
   int i, j, k, k1;

   det = 1.0;

   for (k = 0; k < norder; k++)
      {

      /*** Interchange columns if diagonal element is zero. ***/

      if (array [k][k] == 0)
         {
         for (j = k; j < norder; j++)
            if (array [j][k] != 0.0)  goto nonzero; 
         det = 0.0;
         return (det);

nonzero:
         for (i = k; i < norder; i++)
            {
            save = array [j][i];
            array [j][i] = array [k][i];
            array [k][i] = save;
            }
         det = - det;
         }

   /*** Subtract row k from lower rows to get diagonal matrix. ***/


      det *= array [k][k];
      if (k < (norder - 1))
         {
         k1 = k + 1;
         for (i = k1; i < norder; i++)
            for (j = k1; j < norder; j++)
               {
               array [j][i] -= (array [k][i] * array [j][k]) / array [k][k];
               }
         }
      }

   return (det);
}
