/* addhk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Add information to C/P housekeeping record.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * UNIX 4.2 BSD   C programming language
 * 25 April 1984: creation date.
 * 6 June 1985: Increase file ID from 13 to 14 characters (bytes 144-157).
 *		Move numerically-coded comments from byte 157 to byte 221.
 * 13 March 1988: Add processor designation to byte #70.
 * 21 March 1988: set bit #5 in byte #70 to indicate Vax byte order.
 * 15 November 1988: set byte #70 explicitly to the value 8 (Vax byte order).
 * 19 October 1989: Do not change byte #70.  Set byte order in byte #52.
 * 17 August 2001: icyear is specified as an offset from the year 1900.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

addhk (h, byte_order)

char	*h;		/* Pointer to housekeeping array. */
int byte_order;		/* Byte order: 0 -> Sun-4;  1-> DEC. */

{
   int i;
   int nbtpix;	/* # bits per pixel (=16). */

   static int ic [7]; /* Array containing year,doy,day,month,hour,minute,sec. */
   int icyear;		/* Year image was created. */
   int icdoy;		/* Day-of-Year when image was created. */
   int icday;		/* Day-of-Month. */
   int icmonth;		/* Month. */
   int ichour;		/* Hour. */
   int icmin;		/* Minute. */
   int icsec;		/* Second. */

   int	ifwmag;		/* Filter magnification (*10,000). */
   int	ipcorx,ipcory;	/* Polaroid shift correction (*10). */
   int	iscorx, iscory; /* FW/SM shift; no disk registration (*10). */
   int	icxcen, icycen; /* Computed disk center (x,y) (*10). */
   int	idxcen, idycen; /* Standard disk center (x,y) (*10). */
   int	isxcen, isycen; /* Standard sun center (x,y) (*10). */
   int	iescor;		/* Equivalent Second filter correction (*10,000). */
   int	ifwrad;		/* Radiance conversion factor (*10^11). */
   int	icexp;		/* Scaling factor for brightness; power of 10 (=11). */

/* ICODE: numerically-coded comments.
 *   bit#0 = least significant bit.
 *
 *   bit#0 set: raw DN values from vidicon.
 *   bit#1 set: Equivalent second values (log).
 *   bit#2 set: calibrated in units of 10^-11 B/Bsun.
 *   bit#3 set: stray light removed.
 *   bit#4 set: vignetting removed.
 *   bit#5 set: disk registration has been done.
 *   bit#6 set: not used.
 *   bit#7 set: not used.
 */

   int icode = 4;	/* Indicates calibrated image, coef of 10^-11 B/Bsun.*/

   extern int np;          /* # pixels per vidicon line. */
   extern int icfpos;      /* File position of image in volume. */
   extern int lstr;        /* Stray light flag. */
   extern int lvig;        /* Vignetting flag. */
   extern int lreg;        /* Disk registration flag. */

   extern float fwmag;         /* Filter magnification. */
   extern float pwcorx, pwcory;/* Polaroid shift correction; no disk registration. */
   extern float smcorx, smcory;/* FW/SM shift correction; no disk registration. */
   extern float cxcen, cycen;   /* Computed disk center (x,y). */
   extern float dxcen, dycen;   /* Standard disk center (x,y). */
   extern float sxcen, sycen;   /* Standard sun center (x,y). */
   extern float escorr;         /* Equivalent Second filter correction. */
   extern float radfac;         /* Radiance conversion factor. */

   extern char namvol [];   /* Storage medium name. */
   extern char namset [];   /* Name of dataset containing calibrated image. */
   extern char namcal [];   /* Name of calibrated image file. */
   extern char nama0 [];    /* Name of a0 parameter file. */
   extern char nama1 [];    /* Name of a1 parameter file. */
   extern char nama2 [];    /* Name of a2 parameter file. */
   extern char nama3 [];    /* Name of a3 parameter file. */
   extern char namgeo [];   /* Name of geometric distortion correction file. */
   extern char namstr [];   /* Name of Stray light file. */
   extern char namvig [];   /* Name of Vignetting file. */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   /*** Get current date and time. ***/

   cdatim (ic);
   icyear = ic [0];
   icdoy  = ic [1];
   icday  = ic [2];
   icmonth= ic [3];
   ichour = ic [4];
   icmin  = ic [5];
   icsec  = ic [6];
   printf ("current date: %04d:%03d:%02d:%02d:%02d\n",
           icyear + 1900, icdoy, ichour, icmin, icsec);

   /*** Store byte order: 0-> Sun-4;  1 -> DEC. ***/

   *(h+52) = byte_order;		/* Byte order. */
   
/*
 * Store computer used to produce calibrated image.
 *
 * bit 0: pdp-11/34
 * bit 1: IPD (Univac 1108)
 * bit 2: Cray
 * bit 3: Vax-11/750 ---> pretend image was calibrated on Vax.
 * bit 4: Sun-4/280
 */

/***   *(h+70) = (char) ((((int) *(h+70)) & 07) | 010) & 0377; ***/
/***   *(h+70) = (char) (8 & 0377); ***/

   /* Store # bits/pixel and # pixels/line. */

   nbtpix = 16;
   *(h+103) = nbtpix;

   *(h+110) = (np & 0177400) >> 8;
   *(h+111) = (np & 0377);

   /*** Store information about calibrated volume in housekeeping. ***/

   for (i=0;i<6;i++) *(h+i+128)= *(namvol+i);	/* Volume name. */
   for (i=0;i<6;i++) *(h+i+134)= *(namset+i);	/* Dataset name. */
   *(h+149) = icyear;				/* Year image calibrated. */
   *(h+150) = (icdoy & 0177400) >> 8;		/* DOY image calibrated (msb).*/
   *(h+151) = (icdoy & 0377);			/* DOY image calibrated (lsb).*/

   /*** Store image file name (ydddhhmmss.cxx) in housekeeping. ***/

   for (i=0;i<18;i++) *(h+i+119)= *(namcal+i);

   /*** Store volume names in housekeeping. ***/

   for (i=0;i<6;i++) *(h+i+152)= *(nama0+i);	/* A0 parameter. */
   for (i=0;i<6;i++) *(h+i+158)= *(nama1+i);	/* A1 parameter. */
   for (i=0;i<6;i++) *(h+i+164)= *(nama2+i);	/* A2 parameter. */
   for (i=0;i<6;i++) *(h+i+170)= *(nama3+i);	/* A3 parameter. */
   for (i=0;i<6;i++) *(h+i+176)= *(namgeo+i);	/* Geometric correction. */
   for (i=0;i<6;i++) *(h+i+182)= *(namstr+i);	/* Stray Light. */
   for (i=0;i<6;i++) *(h+i+188)= *(namvig+i);	/* Vignetting. */

   /*** Scale calibration parameters. ***/

   ifwmag = (int) (fwmag * 10000. + 0.5);	/* Filter magnification. */
   ipcorx = (int) (pwcorx * 100. + 0.5);	/* Polaroid shift, x-axis. */
   ipcory = (int) (pwcory * 100. + 0.5);	/* Polaroid shift, y-axis. */
   iscorx = (int) (smcorx * 100. + 0.5);	/* FW/SM shift, x-axis. */
   iscory = (int) (smcory * 100. + 0.5);	/* FW/SM shift, y-axis. */
   icxcen = (int) (cxcen * 10. + 0.5);		/* Computed disk center (X). */
   icycen = (int) (cycen * 10. + 0.5);		/* Computed disk center (Y). */
   idxcen = (int) (dxcen * 10. + 0.5);		/* Standard disk center (X). */
   idycen = (int) (dycen * 10. + 0.5);		/* Standard disk center (Y). */	
   isxcen = (int) (sxcen * 10. + 0.5);		/* Sun center, x-axis. */
   isycen = (int) (sycen * 10. + 0.5);		/* Sun center, y-axis. */
   iescor = (int) (escorr * 100. + 0.5);   /* Equiv.Sec. color correction. */
   ifwrad = (int) (radfac + 0.5);	   /* FW calflux radiance factor. */
   icexp = 11;			/* Brightness scale factor: exponent of 10. */

   /*** Store calibration parameters in housekeeping. ***/

   *(h+194) = (ifwmag & 0177400) >> 8;
   *(h+195) = (ifwmag & 0377);
   *(h+196) = (ipcorx & 0177400) >> 8;
   *(h+197) = (ipcorx & 0377);
   *(h+198) = (ipcory & 0177400) >> 8;
   *(h+199) = (ipcory & 0377);
   *(h+200) = (iscorx & 0177400) >> 8;
   *(h+201) = (iscorx & 0377);
   *(h+202) = (iscory & 0177400) >> 8;
   *(h+203) = (iscory & 0377);
   *(h+204) = (icxcen & 0177400) >> 8;
   *(h+205) = (icxcen & 0377);
   *(h+206) = (icycen & 0177400) >> 8;
   *(h+207) = (icycen & 0377);
   *(h+208) = (idxcen & 0177400) >> 8;
   *(h+209) = (idxcen & 0377);
   *(h+210) = (idycen & 0177400) >> 8;
   *(h+211) = (idycen & 0377);
   *(h+212) = (isxcen & 0177400) >> 8;
   *(h+213) = (isxcen & 0377);
   *(h+214) = (isycen & 0177400) >> 8;
   *(h+215) = (isycen & 0377);
   *(h+216) = (iescor & 0177400) >> 8;
   *(h+217) = (iescor & 0377);
   *(h+218) = (ifwrad & 0177400) >> 8;
   *(h+219) = (ifwrad & 0377);
   *(h+220) = icexp;

   /*
    * Numerically-coded comments.
    *
    * bit #0 set: raw DN intensity units.
    * bit #1 set: equivalent second units.
    * bit #2 set: B/Bsun units.
    * bit #3 set: stray light removed.
    * bit #4 set: vignetting removed.
    * bit #5 set: disk registration has been performed.
    * bit #6 set: currently unused.
    * bit #7 set: currently unused.
    */

   icode = 4;
   icode = icode | (lstr * power(2,3));
   icode = icode | (lvig * power(2,4));
   icode = icode | (lreg * power(2,5));
   *(h+221) = icode;

   /*** Fill spare bytes with zeroes. ***/

   *(h+222) = 0;
   *(h+223) = 0;
}

/* cdatim (ic)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fetches current date and time.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Uses current date and time to return decimal values for
 * Day-of-Year, day, month, and year-1900.
 * Also returns values for hours, minutes, and seconds.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * 12 January 1984
 * Andrew L. Stanger
 * SunOS 4.0.3  (UNIX 4.2 BSD)   C programming language
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#include <sys/types.h>
#include <time.h>

cdatim (ic)

struct { int iyear; int idoy; int iday; int imonth;
         int ihour; int imin; int isec; } *ic;
{
   struct tm *localtime (), *lclt;
   long time (), t;

   t = time (0);
   lclt = localtime (&t);
   ic->iyear = lclt->tm_year;
   ic->idoy =  lclt->tm_yday + 1;
   ic->iday =  lclt->tm_mday;
   ic->imonth =  lclt->tm_mon + 1;
   ic->ihour = lclt->tm_hour;
   ic->imin =  lclt->tm_min;
   ic->isec =  lclt->tm_sec;
}
