/* vigadj.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P Vignetting
 * Compensate for the actual Earth-Sun distance
 * due to Earth's elliptical orbit about the sun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR   1 November 1996
 * 2 March 1999: adapted from distadj.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h> 

float
vigadj (radius, iyear, idoy, ihour)

int iyear ;		/* Year. */
int idoy ;		/* Day of year. */
int ihour ;		/* Hour. */

{
   int iy ;

   double A = 0.0345 ;		/* Amplitude of the semi-annual correction. */
   double AU = 1.0 ;		/* Semi-major axis of ellipse (AU units). */
   double dist ;		/* Earth-Sun distance (AU units).
   double ecc = 0.01675 ;	/* Eccentricity of Earth's orbit. */
   double period = 365.2596 ;	/* Earth's orbital period. */
   double PI = 3.14159265 ;	/* PI. */
   double r ;
   double radius ;		/* Orbital radius (earth to sun). */
   double th ;
   double theta ;		/* Angle of Earth-sun line w.r.t. perihelion. */
   double doyo ;		/* Day-of-year of observation. */
   double doyp ;		/* Perihelion day-of-year. */
   float  vignet ;

   /* Perihelion DOY & hour (as a function of year: 80-89). */

   static float Pdoy  [10] = { 3,  2,  4,  2,  3,  3,  2,  4,  4,  1} ;
   static float Phour [10] = {15,  2, 11, 16, 22, 20,  5, 23,  0, 22} ;

   /* polynomial constants for a 9th order polynomial fit to the SMM C/P
    * vignetting curve.
    */

   static float pc [] = { 23995.0, -72335.0, 95648.0, -72744.000, 35058.000,
			 -11104.0,   2312.1,  -305.3,     23.211,    -0.77445} ;

   /*** Convert year to array index. ***/

   iy = iyear ;
   if (iyear > 1900)  iy -= 1900 ;
   if (iyear >= 80)   iy -=   80 ;
   if (iy < 0 || iy > 9)
      {
      fprintf (stderr, "Year not between 1980 & 1989.\n") ;
      exit (-1.0) ;
      }

   doyp = (double) (Pdoy [iy] + Phour [iy] / 24.0) ;	/* perihelion DOY. */

   doyo = (double) (idoy + ihour / 24.0) ;		/* image DOY. */

   /*** Compute angle w.r.t. perihelion. ***/

   theta = (2.0 * PI * (doyo - doyp) / period) +
	 2.0 * ecc * sin (2.0 * PI * (doyo - doyp) / period) ;

   dist = (A * (1.0 - ecc*ecc)) / (1.0 + ecc * cos (theta)) ;

   r = radius + A * cos (theta) * radius ;	/* Adjusted coronal distance */
   r2 = r * 2 ;
   r3 = r * r2 ;
   r4 = r * r3 ;
   r5 = r * r4 ;
   r6 = r * r5 ;
   r7 = r * r6 ;
   r8 = r * r7 ;
   r9 = r * r8 ;

   vignet = (float)      (pc [0] +
	    pc [1] * r  + pc [2] * r2 + pc [3] * r3 + pc [4] * r4 +
	    pc [5] * r5 + pc [6] * r6 + pc [7] * r7 + pc [8] * r8 + pc [9] * r9)

   fprintf (stderr,
      "vigadj> iy: %2d  doyo: %7.4f  theta: %7.4f  radius:%7.4f vignet:%7.4\n",
      iy, doyo, theta, radius, vignet) ;

   return (vignet) ;
}
