/* lumcor.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Luminance correction for SMM C/P image to compensate for
 * variation in solar flux due to the ellipticity of the Earth's orbit.
 * Currently works only with 1984 and 1985 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * author: Michele De La Pena  1 July 1985
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <strings.h>
#include "/usr/people/smm/lib/include/iproc.h"

static double period = 365.2596 ;	/* Orbital period of Earth. */
static double pi = 3.14159265 ;
static double epsilon = 0.01675 ;	/* Ellipticity of Earth's orbit. */
static double semimaj = 1.0 ;		/* semi-major axis of orbit in A.U. */

main (argc, argv)

int argc ;
char *argv [] ;

{
   struct IMAGE *inform ;
   struct IMAGE *outform ;

   char infile [80] ;
   char outfile [80] ;
   unsigned short int *indat, *outdat ;
   int dx, dy;
   int i, j, dot ;
   int year, doy, hour, min;
   double angle1, angle2, denom, dist, correct ;

   /*** Fetch input file name. ***/

   if (argc == 2)
      {
      strcpy (infile, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s input_CP_filename\n", argv [0]) ;
      exit (0) ;
      }

   for (i = 0; i < strlen (infile); i++)
      if (infile [i] == '.')  dot = i ;

   strcpy (outfile, infile) ;
   outfile [dot+3] = 'R' ;
   outfile [strlen (infile)] = '\0' ;

   /*** "make" opens the file and sets up C/P IMAGE data structure. ***/

   inform  = make (infile,  "SMM") ;	/* Input image. */
   outform = make (outfile, "SMM") ;	/* Output image. */

   fprintf (stdout, "File %s has been opened for reading.\n", infile);
   fprintf (stdout, "File %s has been opened for writing.\n", outfile);

   indat  = (unsigned short int *) inform->addr ;
   outdat = (unsigned short int *) outform->addr ;

   /*** Write information about image to terminal screen. ***/

   year = inform->year ;
   doy  = inform->doy ;
   hour = inform->hour ;
   min  = inform->minute ;

   fprintf (stdout, "Year: %2d  DOY: %3d  Time: %02d:%02d\n",
            year, doy, hour, min);

   if (year != 84 || year != 85)
      {
      fprintf (stderr, "Program %s works only for 1984/1985 C/P images!\n");
      exit (-1);
      }

   /*** Compute correction, which needs to be applied to image data.
    *** Correction is based upon date (year & DOY) of the image.
    ***/

   if (year == 84)
      angle1 = ((doy - 3.9167) * 2.0 * pi) / period ;
   else
      angle1 = ((doy - 3.8333) * 2.0 * pi) / period ;

   angle2 = 2.0 * epsilon * sin (angle1) ;
   denom = 1.0 + epsilon * cos (angle2 + angle2) ;
   dist = (semimaj * (1.0 - epsilon * epsilon)) / denom ;
   correct = dist * dist;

   fprintf (stdout, "Correction: %g\n", correct) ;

   dx = inform->dx ;
   dy = inform->dy ;

   printf ("Image is being processed.  Please be patient.\n") ;

   for (j = 0; j < dy; j++)
      for (i = 0; i < dx; i++)
         *(outdat + i + j*dx) = *(indat + i + j*dx) * correct;

   /*** Write output file to disk. ***/

   gputfile (outform, outfile) ;

   exit (0);
}
