/* nlftan.c
 * ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 * non-linear least-squares fit routines for C/P tangent equation.
 * ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
 * Andrew L. Stanger
 * Converted to C programming language from Fortran
 * 17 March 1987
 */
#include <stdio.h>
#include <math.h>

/*
 * *****************************************************************************
 * Tangent function for SMM C/P transfer curve.
 * *****************************************************************************
 */
float
func (dn, c, npar)

float dn;	/* Vidicon DN value. */
float c [];	/* Tangent equation parameters. */
int npar;	/* # parameters. */
{
float alpha, beta, gamma;
float delta, tau;

      alpha = c[0];
      beta  = c[1];
      gamma = c[2];

      delta  = (dn -  gamma) / beta;
      tau    = alpha * (float) tan ((double) delta);

fprintf (stderr, "func> dn:%f  tau:%f\n", dn, tau);
      return (tau);
}

/* *****************************************************************************
 * deriv: derivative of tangent equation.
 * *****************************************************************************
 */

deriv (dn, c, npar, f, df)

float *dn;	/* Vidicon DN value. */
float c[];	/* Parameters for tangent equation. */
int *npar;	/* # parameters. */
float *f;	/* Tangent function. */	
float df[];	/* Derivative of tangent function. */
{
float alpha, beta, gamma;
double delta;
float tdel;
float tau;
float cosq;

      alpha = c[0];
      beta  = c[1];
      gamma = c[2];

      delta  = (*dn - gamma) / beta;
      cosq   = (float) (cos (delta) * cos (delta));
      tdel   = (float) tan (delta);
      *f = tau = alpha * tdel;

      df[0]  = tdel;
      df[1]  = - ((alpha * delta) / (beta * cosq));
      df[2]  = - (alpha / (beta * cosq));

fprintf (stderr, "deriv> dn:%f  f:%f   df:%f   %f   %f\n", *dn, *f, df[0],df[1],df[2]);
      return(0);
}


/* *****************************************************************************
 * nllst2: non-linear least squares fitting routine.
 * *****************************************************************************
 * nllst2 (x, y, c, sig, fix, npar, nval, chisq, ier)
 *
 * Non-linear least squares fitting.
 *
 * Input :
 *    x         = array of independent quantities
 *    y         = array of dependent quantities
 *    c         = estimates of the  fitting parameters
 *    fix       - logical array
 *              if fix(i) = .true. then c(i) is not corrected
 *    npar      = number of parameters in fit (including those fixed)
 *    nval      = number of data points
 *
 * User supplied subroutines :
 * func(x,c,na)        func = value at x given parameters c
 * deriv(x,c,na,f,df)  f = value at x given parameters c
 *                     df(i) = df/dc(i)
 *
 * output :
 *   c         = least squares fitted parameters
 *   sig       = estimate of standard error in c.
 *   chisq     = estimate of unit weight variance of fit
 *   ier       = 1 - - > fit is ok
 *             = 2 - - > fit is suspicious
 *             = 3 - - > more than 25 iterations required
 *             = 4 - - > disaster, no fit found
 *
 * calls: symred, symsol, syminv
 *
 * comments:   1) if more than 6 parameters are used be sure to change
 *                the dimension statements.
 *             2) when debugging compile with the /de switch to print
 *                convergence information.
 *             3) to change the convergence criterion alter tol in data.
 *
 * modified 19 Aug 1979 so that argument of sqrt is always > 0.
 *
 */
#define nmax 6
#define itrmax 25
#define tol 1.e-4

nllst2 (x, y, c, sig, fix, npar, nval, chisq, ier)

float x[];	/* DN independent array (m values). */
float y[];	/* Exposure dependent array (m values). */
float c[];	/* Parameter array (npar values). */
float sig[];	/* sigma array (npar*2 values). */
int fix[];	/* Logical array (npar values): =0 fix; =1 no fix. */
int *npar;	/* # parameters. */
int *nval;	/* # data points in x & y arrays. */
float *chisq;	/* Chi-square. */
int *ier;	/* error flag. */

{
extern float func();
extern int deriv();
extern int symred();
extern int symsol();
extern int syminv();
extern char *malloc();

static float a[nmax][nmax], a0[nmax][nmax];
static float b[nmax], dc[nmax], c1[nmax];
static int indx[nmax];
int sing, conv;
int i, j, k, n;
int iter;
float fac;
float chi, chi0, chi1;
static float f;
float *df;
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
fprintf (stderr, "nllst2\n");
fprintf (stderr, "*npar: %d   *nval: %d\n", *npar, *nval);
for (i=0; i<*npar; i++)
   {
   fprintf (stderr, "c[%d]: %f   fix[%d]: %d\n", i, c[i], i, fix[i]);
   }
for (i=0; i<*nval; i++)
   {
   fprintf (stderr, "x[%d]: %f   y[%d]: %f\n", i, x[i], i, y[i]);
   }
   df = (float *) malloc (*npar * sizeof(float));

/*  Is there sufficient scratch space available ? */

   *ier = 4;
   if(*npar > nmax) return(*ier);
   *ier = 1;

/*  Non - linear least squares. */

   fac = 1.0e-3;
   n = 0;
   iter = 0;
   for (i=0; i<*npar; i++)
      {
      c1[i] = c[i];
      if(fix[i] == 0)
         {
         indx[n] = i;
         n++;
         }
      }

fprintf (stderr, "nllst2> n: %d\n", n);
      if (n == 0)  return(*ier);

/* Set up the linearized matrix. */

s100:
fprintf (stderr, "nllst2> 100 iter: %d\n", iter);
   for (i=0; i<n; i++)
      {
      for (j=0; j<n; j++)  a0[j][i] = 0.0;
      b[i] = 0.0;
      }

   chi0 = 0.0;
   for (k=0; k<*nval; k++)
      {
      deriv (&x[k], c, npar, &f, df);	/* Derivative of tangent equation. */
      for (i=0; i<n; i++)
         {
         j = indx[i];
         df[i] = df[j];
         b[i] += (y[k] - f) * df[i];
         for (j=i; j<n; j++)  a0[j][i] +=  df[i] * df[j];
         }

      chi0 += (y[k] - f) * (y[k] - f);
      }

/*
 * fprintf (stderr,"Iteration:%d   Sum of residuals squared: %g\n", iter, chi0);
 */

/* Apply a correction factor to avoid roundoff problems in test against chi1. */

   chi0 = 1.001 * chi0;

/* Multiply the diagonal by a scale factor. */

s200:
fprintf (stderr, "nllst2> 200 fac: %f\n", fac);
   for (i=0; i<n; i++)
      {
      for (j=i; j<n; j++)  a[j][i] = a0[j][i];
      a[i][i] *= (1.0 + fac);
      }

/* Find the corrections to c. */

   symred (a, &sing, n, nmax);

   if (sing == 1)
      {
      fprintf (stderr, "Matrix is singular; abandon all hope.\n");
      *ier = 4;
      return(*ier);
      }

   symsol (a, b, dc, n, nmax);
   conv = 1;
   for (i=0; i<n; i++)
      {
      j = indx[i];
      c1[j] = c[j] + dc[i];
      conv = conv && (abs((double) dc[i]) <= tol * (1.+ abs((double) (c1[j]))));
      }
fprintf (stderr, "nllst2> conv: %d\n", conv);

/* Evaluate new chi squared. */

   chi1 = 0.0;
   for (k=0; k<*nval; k++)
      {
      chi = (y[k] - func(x[k],c1,npar));
      chi1 += chi * chi;
      }

   for (i=0; i<n; i++)  fprintf (stderr, "dc: %g\n", dc[i]);

   if(chi1 >= chi0)		 /* Increase the damping factor. */
      {
      fac *= 10.0;
      if(fac <= 1.0e+5) goto s200;
      }

/* Last values were ok or there is no reason for increasing 'fac'. */

   fac *= 0.1;
   for (i=0; i<*npar; i++)  c[i] = c1[i];

   if (conv == 0)
      {
      iter++;
      if(iter < itrmax) goto s100;
      if(fac > 1.0e-3) *ier = 2;
      if(iter >= itrmax) *ier = 3;
      }

   *chisq = chi1 / (float) (*nval - n);
   symred (a0, &sing, n, nmax);
   syminv (a0, n, nmax);

   for (i=0; i<*npar; i++)  sig[i] = 0.0;

   for (j=0; j<n; j++)
      {
      i = indx[j];
      sig[i] = (float) sqrt( ( abs( (double) (*chisq *a0[j][j]) ) ) );
fprintf (stderr, "nllst2> sig[%d]: %f\n", i, sig[i]);
      }

fprintf (stderr, "nllst2> *chisq: %f   *ier: %d\n", *chisq, *ier);
      return(*ier);
}


/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
symred (a, sing, n, nr)

float a[nmax][nmax];
int *sing;
int n;
int nr;

{
int i, j, k, kk;
float x, y, z;

for (i=0; i<nmax; i++)
fprintf (stderr, "symred> a[%d][%d]: %f\n", i, i, a[i][i]);

   for (i=0; i<n; i++)
      {
      for (j=0; j<=i; j++)
         {
         x = a[i][j];
         if (i == j)
            {
            for (kk=1; kk<=j; kk++)
               {
               k = j + 1 - kk;
               y = a[k][i];
               z = y * a[k][k];
               a[k][i] = z;
               x -= y * z;
               }

            if(x == 0.0)
               {
               *sing = 1;
               return(0);
               }
            a[i][i] = 1.0 / x;
            }
         else
            {
            if (j > 0)
            for (kk=1; kk<=j; kk++)
               {
               k = j + 1 - kk;
               x -= a[k][i] * a[k][j];
               }
            a[j][i] = x;
            }
         }
      }

   *sing = 0;
   return(0);
}
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
symsol (a, b, x, n, nr)

float a[nmax][nmax];
float b[nmax];
float x[nmax];
int n;
int nr;

{
int i, ii;
int k, kk;
int ip1;
float y;

for (i=0; i<nmax; i++)
fprintf (stderr, "symsol> i:%d   a:%f   b:%f  x:%f\n", i, a[i][i], b[i], x[i]);

   for (i=0; i<n; i++)
      {
      y = b[i];
      if (i > 0)
         for (kk = 1; kk<=i; kk++)
            {
            k = i + 1 - kk;
            y = y - a[k][i] * x[k];
            }
      x[i] = y;
      }

   for (ii=0; ii<n; ii++)
      {
      i = n + 1 - ii;
      y = x[i] * a[i][i];
      if(i != n)
         {
         ip1 = i + 1;
         for (k=ip1; k<n; k++)  y -= a[i][k] * x[k];
         }
      x[i] = y;
      }

   return(0);
}

/* :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::: */
syminv (a, n, nr)

float a[nmax][nmax];
int n;
int nr;

{
int i, j, k;
int im1;
int ip1, jp1;
float x, z;

for (i=0; i<nmax; i++)
fprintf (stderr, "syminv> a[%d][%d]:%f\n", i, i, a[i][i]);

   for (i=1; i<n; i++)
      {
      im1 = i - 1;

      for (j=0; j<=im1; j++)
         {
         x = - a[j][i];
         if(j != im1)
            {
            jp1 = j + 1;
            for (k=jp1; k<=im1; k++)  x -= a[k][i] * a[j][k];
            }
         a[j][i] = x;
         }
      }

   for (j=0; j<n; j++)
      {
      for (i=j; i<n; i++)
         {
         x = a[j][i];
         if (i != n)
            {
            ip1 = i + 1;
            if(i == j)
               for (k=ip1; k<n; k++)
                  {
                  z = a[j][k];
                  a[j][k] = z * a[k][k];
                  x += z * a[j][k];
                  }
            else
               for (k=ip1; k<n; k++) x += a[i][k] * a[j][k];
            a[j][i] = x;
            }
         } /* End i-loop. */
      }	/* End j-loop. */

   return(0);
}
