c nlltana.f : non-linear least-squares fit routines for C/P tangent equation.
c**************************************************************
c
      function func (x, c, n)
c
      real c (n)
      real alpha, beta, gamma, eps, x, f
c
      alpha  = c(1)
      beta   = c(2)
      gamma  = c(3)
      eps    = c(4)

      f      = eps + alpha * tan ( (x - gamma) / beta)
      func   = f
c
      return
      end
c
c*******************************************************
c
      subroutine deriv (x, c, n, f, df)
c
      real c (n), df (n)
      real alpha, beta, gamma, eps, x
      real f, t, w
c
      alpha  = c(1)
      beta   = c(2)
      gamma  = c(3)
      eps    = c(4)
      w  = (x - gamma) / beta
      cosq   = cos (w) **2
      t      = tan (w)
      f      = eps + alpha * t
      df(1)  = t
      df(2)  = - ((alpha * w) / (beta * cosq))
      df(3)  = - (alpha / (beta * cosq))
      df(4)  = 1
      return
      end
c
c*******************************************************
c
      subroutine nllst2 (x, y, c, sig, fix, na, m, chisq, ier)
c
c*** Non-linear least squares fitting.
c
c     Input :
c        x         = array of independent quantities
c        y         = array of dependent quantities
c        c         = estimates of the  fitting parameters
c        fix       - logical array
c                  if fix(i) = .true. then c(i) is not corrected
c        na        = number of parameters in fit (including those fixed)
c        m         = number of data points
c
c     User supplied subroutines :
c        func(x,c,na)        func = value at x given parameters c
c        deriv(x,c,na,f,df)  f = value at x given parameters c
c                            df(i) = df/dc(i)
c
c     output :
c        c         = least squares fitted parameters
c        sig       = estimate of standard error in c.
c        chisq     = estimate of unit weight variance of fit
c        ier       = 1 - - > fit is ok
c                  = 2 - - > fit is suspicious
c                  = 3 - - > more than 25 iterations required
c                  = 4 - - > disaster, no fit found
c
c     calls: symred, symsol, syminv
c
c     comments:   1) if more than 6 parameters are used be sure to change
c                    the dimension statements.
c                 2) when debugging compile with the /de switch to print
c                    convergence information.
c                 3) to change the convergence criterion alter tol in data.
c
c     modified 19 Aug 1979 so that argument of sqrt is always > 0.
c
      dimension c(na),fix(na),x(m),y(m),a(6,6),a0(6,6),b(6),df(6),dc(6),
     . c1(6),indx(6),sig(6)
      logical fix,sing,conv
      data nmax, itrmax, tol / 6,  25,  1.0e - 4/
c
c---   is there sufficient scratch?
c
      ier = 4
      if(na.gt.nmax) return
      ier = 1
c
c---   non - linear least squares.
c
      fac = 1.0e - 3
      n = 0
      iter = 0
      do 99 i = 1, na
         c1(i) = c(i)
         if(fix(i)) go to 99
         n = n + 1
         indx(n) = i
   99 continue
c
      if(n.eq.0) return
c
c---   set up the linearized matrix.
c
  100 do 2 i = 1, n
         do 1 j = i, n
            a0(i,j) = 0.0
    1    continue
c
         b(i) = 0.0
    2 continue
c
      chi0 = 0.0
      do 5 k = 1, m
         call deriv(x(k),c,na,f,df)
         do 6 i = 1, n
            j = indx(i)
            df(i) = df(j)
    6    continue
c
         do 4 i = 1, n
            b(i) = b(i) + (y(k) - f)*df(i)
            do 3 j = i, n
               a0(i,j) = a0(i,j) + df(i)*df(j)
    3       continue
    4    continue
c
         chi0 = chi0 + (y(k) - f)**2
    5 continue
c
cd     print 500, iter, chi0
cd 500 format(' iteration ',i3,' sum of residuals squared ',1pe11.4)
c
c---  apply a correction factor to avoid roundoff problems in test against chi1.
      chi0 = 1.001*chi0
c     multiply the diagonal by a scale factor.
c
  200 do 12 i = 1, n
         do 11 j = i, n
            a(i,j) = a0(i,j)
   11    continue
c
         a(i,i) = a(i,i)*(1.0 + fac)
   12 continue
c
c---   find the corrections to c.
c
      call symred(a,sing,n,6)
      if(.not.sing) go to 113
cd     print 112
  112 format(' matrix is singular, abandon all hope')
      ier = 4
      return
c
  113 call symsol(a,b,dc,n,6)
      conv = .true.
      do 13 i = 1, n
         j = indx(i)
         c1(j) = c(j) + dc(i)
         conv = conv.and.(abs(dc(i)).le.tol*(1. + abs(c1(j))))
   13 continue
c
c---   evaluate new chi squared.
c
      chi1 = 0.0
      do 14 k = 1, m
         chi1 = chi1 + (y(k) - func(x(k),c1,na))**2
   14 continue
c
cd     print 501, (dc(i),i = 1,n)
cd 501 format(' dc ',1p6e11.4)
      if(chi1.lt.chi0) go to 300
c
c---   increase the damping factor.
c
      fac = 10.0*fac
      if(fac.le.1.0e5) go to 200
c
c---   last values were ok or there is no reason for increasing 'fac'.
c
  300 fac = 0.1*fac
      do 21 i = 1, na
         c(i) = c1(i)
   21 continue
      if(conv) go to 318
      iter = iter + 1
      if(iter.lt.itrmax) go to 100
      if(fac.gt.1.0e - 3) ier = 2
      if(iter.ge.itrmax) ier = 3
c
  318 chisq = chi1/float(m - n)
      call symred(a0,sing,n,6)
      call syminv(a0,n,6)
c
      do 320 i = 1, na
      sig(i) = 0.0
  320 continue
c
      do 330 j = 1, n
         i = indx(j)
         sig(i) = sqrt((abs(chisq*a0(j,j))))
  330 continue
c
      return
      end
c::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      subroutine symred(a,sing,n,nr)
c
      logical sing
c
      dimension a(nr,nr)
      do 7 i = 1, n
         do 6 j = 1, i
            x = a(j,i)
            if(i.ne.j) go to 3
            if(j.eq.1) go to 2
            do 1 kk = 2, j
               k = j + 1 - kk
               y = a(i,k)
               z = y*a(k,k)
               a(i,k) = z
               x = x - y*z
    1       continue
c
    2       if(x.eq.0.0e0) go to 8
            a(i,i) = 1.0e0/x
            go to 6
c
    3       if(j.eq.1) go to 5
            do 4 kk = 2, j
               k = j + 1 - kk
               x = x - a(i,k)*a(j,k)
    4       continue
c
    5       a(i,j) = x
    6    continue
    7 continue
c
      sing = .false.
      return
c
    8 sing = .true.
c
      return
      end
c::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      subroutine symsol(a,b,x,n,nr)
c
      dimension a(nr,nr),b(nr),x(nr)
c
      do 3 i = 1, n
         y = b(i)
         if(i.eq.1) go to 2
         do 1 kk = 2, i
            k = i + 1 - kk
            y = y - a(i,k)*x(k)
    1    continue
    2    x(i) = y
    3 continue
c
      do 6 ii = 1, n
         i = n + 1 - ii
         y = x(i)*a(i,i)
         if(i.eq.n) go to 5
         ip1 = i + 1
c
         do 4 k = ip1, n
            y = y - a(k,i)*x(k)
    4    continue
    5    x(i) = y
    6 continue
c
      return
      end
c:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
      subroutine syminv(a,n,nr)
c
      dimension a(nr,nr)
c
      do 4 i = 2, n
         im1 = i - 1
c
         do 3 j = 1, im1
            x = - a(i,j)
            if(j.eq.im1) go to 2
            jp1 = j + 1
c
            do 1 k = jp1, im1
               x = x - a(i,k)*a(k,j)
    1       continue
c
    2       a(i,j) = x
    3    continue
    4 continue
c
      do 16 j = 1, n
         do 15 i = j, n
            x = a(i,j)
            if(i.eq.n) go to 14
            ip1 = i + 1
            if(i.eq.j) go to 12
c
            do 11 k = ip1, n
               x = x + a(k,i)*a(k,j)
   11       continue
c
            go to 14
   12       do 13 k = ip1, n
               z = a(k,j)
               a(k,j) = z*a(k,k)
               x = x + z*a(k,j)
   13       continue
   14       a(i,j) = x
   15    continue
c
   16 continue
c
      return
      end
