c pf.f
c------------------------------------------------------------------------------
c Make a least-squares fit to data with a polynomial curve:
c   y = a(i) + a(2) * x + a(3) * x**2 + a(4) * x**3 + . . .
c------------------------------------------------------------------------------
c usage :
c    x		array of data points for independent values
c    y		array of data points for dependedent values
c    sigmay	array of standard deviations for y data points.
c    npts	number of pairs of data points
c    nterms	number of coefficients (degree of polynomial + 1)
c    mode	determines method of weighting least-squares fit
c		+1 (instrumental) weight(i) = 1. / sigmay(i)**2
c		 0 (no weighting) weight(i) = 1.
c		-1 (statistical)  weight(i) = 1./y(i)
c    a		array of coefficients of polynomial
c    chisqr	reduced chi square for fit
c------------------------------------------------------------------------------
c subroutines and functions required :
c    determ (array, norder)
c       evaluates the determinant of a symmetic 2-dimensional
c       matrix of order `norder'.
c------------------------------------------------------------------------------
c note: dimension statement valid for `nterms' up to 10.
c------------------------------------------------------------------------------
c Bevington, "Data Reduction and Error Analysis for the Physical Sciences"
c-------------------------------------------------------------------------------
c 2 September 1988
c------------------------------------------------------------------------------

      subroutine pf (x, y, sigmay, npts, nterms, mode, a, chisqr)

      double precision sumx(19), sumy(10), array(10,10)
      double precision xterm, yterm, chisq
      real x(1), y(1), sigmay(1), a(1)
c
c--- Accumulate weighted sums.
c
   11 nmax = 2*nterms - 1
      do 13 n = 1, nmax
         sumx(n) = 0.
   13 continue

      do 15 j = 1, nterms
         sumy(j) = 0.
   15 continue

      chisq = 0.
   21 do 50 i = 1, npts
         xi = x(i)
         yi = y(i)
   31    if (mode) 32, 37, 39
   32    if (yi)  35, 37, 33
   33    weight = 1.0 / yi
         go to 41
   35    weight = 1.0 / (-yi)
         go to 41
   37    weight = 1.0
         go to 41
   39    weight = 1.0 / sigmay(i)**2
   41    xterm = weight

         do 44 n = 1, nmax
            sumx(n) = sumx(n) + xterm
            xterm = xterm * xi
   44    continue

   45    yterm = weight * yi

         do 48 n = 1, nterms
            sumy(n) = sumy(n) + yterm
            yterm = yterm * xi
   48    continue

   49    chisq = chisq + weight * yi**2

   50 continue
c
c--- Construct matrices and calulate coefficients.
c
   51 do 54 j = 1, nterms
      do 54 k = 1, nterms
         n = j + k - 1
         array(j,k) = sumx(n)
   54 continue   
      delta = determ (array, nterms)
      if (delta)  61, 57, 61
   57 chisqr = 0.0
      do 59 j = 1, nterms
         a(j) = 0.0
   59 continue

      go to 80

   61 do 70 l = 1, nterms
   62    do 66 j = 1, nterms
            do 65 k = 1, nterms
               n = j + k - 1
               array(j,k) = sumx(n)
   65       continue
            array(j,l) = sumy(j)
   66    continue
         a(l) = determ (array, nterms) / delta
   70 continue
c
c--- Calculate chi-square.
c
   71    do 75 j = 1, nterms
         chisq = chisq - 2.0 * a(j) * sumy(j)

         do 75 k = 1, nterms
            n = j + k - 1
            chisq = chisq + a(j) * a(k) * sumx(n)
   75    continue

   76    free = npts - nterms

   77    chisqr = chisq / free

   80 return
      end



c-------------------------------------------------------------------------------
c function determ
c-------------------------------------------------------------------------------
c Calculate the determinant of a square matrix
c-------------------------------------------------------------------------------
c usage :
c   det = determ (array, norder)
c-------------------------------------------------------------------------------
c description of parameters :
c    array	matrix
c    norder	order of determinant (degree of matrix)
c-------------------------------------------------------------------------------
c This routine destroys the input matrix array.
c Dimension statement valid for norder up to 10.
c-------------------------------------------------------------------------------
c Bevington, "Data Reduction and Error Analysis for the Physical Sciences"
c-------------------------------------------------------------------------------
c 2 September 1988
c-------------------------------------------------------------------------------

      function determ (array, norder)

      double precision array(10,10), save

   10 determ = 1.0

   11 do 50 k = 1, norder
c
c--- Interchange columns if diagonal element is zero.
c
         if (array(k,k))  41, 21, 41
   21    do 23 j = k, norder
            if (array(k,j))  31, 23, 31
   23    continue

         determ = 0.0
         go to 60

   31    do 34 i = k, norder
            save = array(i,j)
            array(i,j) = array(i,k)
            array(i,k) = save
   34    continue
         determ = - determ
c
c--- Subtract row k from lower rows to get diagonal matrix.
c

   41    determ = determ * array(k,k)
         if (k - norder)  43, 50, 50
   43    k1 = k + 1
         do 46 i = k1, norder
         do 46 j = k1, norder
            array(i,j) = array(i,j) - array(i,k) * array(k,j)
     .                   / array(k,k)
   46    continue

   50 continue

   60 return
      end
