/* pf.c
 *------------------------------------------------------------------------------
 * Make a least-squares fit to data with a polynomial curve:
 *   y = a(i) + a(2) * x + a(3) * x**2 + a(4) * x**3 + . . .
 *------------------------------------------------------------------------------
 * usage :
 *    x		array of data points for independent values
 *    y		array of data points for dependedent values
 *    sigmay	array of standard deviations for y data points.
 *    npts	number of pairs of data points
 *    nterms	number of coefficients (degree of polynomial + 1)
 *    mode	determines method of weighting least-squares fit
 *		+1 (instrumental) weight(i) = 1. / sigmay(i)**2
 *		 0 (no weighting) weight(i) = 1.
 *		-1 (statistical)  weight(i) = 1./y(i)
 *    a		array of coefficients of polynomial
 *    chisqr	reduced chi square for fit
 *------------------------------------------------------------------------------
 * subroutines and functions required :
 *    determ (array, norder)
 *       evaluates the determinant of a symmetic 2-dimensional
 *       matrix of order `norder'.
 *------------------------------------------------------------------------------
 * note: dimension statement valid for `nterms' up to 10.
 *------------------------------------------------------------------------------
 * Bevington, "Data Reduction and Error Analysis for the Physical Sciences"
 *------------------------------------------------------------------------------
 * Andrew L. Stanger
 * 2 September 1988: converted to C
 *------------------------------------------------------------------------------
 */

pf (x, y, sigmay, npts, nterms, mode, a, chisqr)

float x[];
float y[];
float sigmay[];
int npts;
int nterms;
int mode;
float a[];
float chisqr;

{
float sumx, sumy, xterm, yterm, array, chisq;
float x[1], y[1], sigmay[1], a[1];
float sumx[19], sumy[10], array[10][10];

/* Accumulate weighted sums. */

   11 nmax = 2*nterms - 1
      do 13 n = 1, nmax
         sumx(n) = 0.
   13 continue

      do 15 j = 1, nterms
         sumy(j) = 0.
   15 continue

      chisq = 0.
   21 do 50 i = 1, npts
         xi = x(i)
         yi = y(i)
   31    if (mode) 32, 37, 39
   32    if (yi)  35, 37, 33
   33    weight = 1.0 / yi
         go to 41
   35    weight = 1.0 / (-yi)
         go to 41
   37    weight = 1.0
         go to 41
   39    weight = 1.0 / sigmay(i)**2
   41    xterm = weight

         do 44 n = 1, nmax
            sumx(n) = sumx(n) + xterm
            xterm = xterm * xi
   44    continue

   45    yterm = weight * yi

         do 48 n = 1, nterms
            sumy(n) = sumy(n) + yterm
            yterm = yterm * xi
   48    continue

   49    chisq = chisq + weight * yi**2

   50 continue

/* Construct matrices and calulate coefficients. */

   51 do 54 j = 1, nterms
      do 54 k = 1, nterms
         n = j + k - 1
         array(j,k) = sumx(n)
   54 continue   
      delta = determ (array, nterms)
      if (delta)  61, 57, 61
   57 chisqr = 0.0
      do 59 j = 1, nterms
         a(j) = 0.0
   59 continue

      go to 80

   61 do 70 l = 1, nterms
   62    do 66 j = 1, nterms
            do 65 k = 1, nterms
               n = j + k - 1
               array(j,k) = sumx(n)
   65       continue
            array(j,l) = sumy(j)
   66    continue
         a(l) = determ (array, nterms) / delta
   70 continue

/* Calculate chi-square. */

   71    do 75 j = 1, nterms
         chisq = chisq - 2.0 * a(j) * sumy(j)

         do 75 k = 1, nterms
            n = j + k - 1
            chisq = chisq + a(j) * a(k) * sumx(n)
   75    continue

   76    free = npts - nterms

   77    chisqr = chisq / free

   80 return(0);
}



/*------------------------------------------------------------------------------
 * function determ
 *------------------------------------------------------------------------------
 * Calculate the determinant of a square matrix
 *------------------------------------------------------------------------------
 * usage :
 *   det = determ (array, norder)
 *------------------------------------------------------------------------------
 * description of parameters :
 *    array	matrix
 *    norder	order of determinant (degree of matrix)
 *------------------------------------------------------------------------------
 * This routine destroys the input matrix array.
 * Dimension statement valid for norder up to 10.
 *------------------------------------------------------------------------------
 * Bevington, "Data Reduction and Error Analysis for the Physical Sciences"
 *------------------------------------------------------------------------------
 * Andrew L. Stanger
 * 2 September 1988: convert to C.
 *------------------------------------------------------------------------------
 */

float
determ (array, norder)

float array[];
int norder;
{
      real array, save
      real array (10,10)

   10 determ = 1.0

   11 do 50 k = 1, norder
c
c--- Interchange columns if diagonal element is zero.
c
         if (array(k,k))  41, 21, 41
   21    do 23 j = k, norder
            if (array(k,j))  31, 23, 31
   23    continue

         determ = 0.0
         go to 60

   31    do 34 i = k, norder
            save = array(i,j)
            array(i,j) = array(i,k)
            array(i,k) = save
   34    continue
         determ = - determ
c
c--- Subtract row k from lower rows to get diagonal matrix.
c

   41    determ = determ * array(k,k)
         if (k - norder)  43, 50, 50
   43    k1 = k + 1
         do 46 i = k1, norder
         do 46 j = k1, norder
            array(i,j) = array(i,j) - array(i,k) * array(k,j)
     .                   / array(k,k)
   46    continue

   50 continue

   60 return();
}
