/* cpcen.c
 * usage: cpcen [image_filename | @filename_list] [outfile]
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate C/P disk center by fitting an arc
 * to second diffraction ring (1.7 Rsun).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Works only for calibrated (short integer) C/P images.
 * Generates its own greyscale color table (saves & restores pre-existing LUT).
 * Automatically scales pixel values by dividing them by 2.
 * Opens a temporary GL window for image display.
 * Writes a "dotted" arc using overlay planes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 31 July 1984
 * 25 March 1991: modify for use on SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>

/* Global variables. */

#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/grscale.h"
#define Red 1
#define Green 2
#define Blue 3
#define ESC 27

int GRN = 1 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc ;
char *argv [] ;
{

extern char *rdimg () ;		/* Read image from disk to memory. */

FILE   *LIST, *LOG ;
char   *imgbuf ;			/* Pointer to image array. */
char   key ;
static char flag [8] ;
static char inline [80] ;	/* Input line buffer. */
static char namimg [80] ;	/* Image file name. */
static char namlst [80] ;	/* Name of file containing image list. */
static char namlog [80] ;	/* Output file name. */
static char nampgm [80] ;	/* Program file name. */
static char answer [2] ;		/* Storage for y/n response. */

static short r [512], g [512], b [512] ;
short icol ;
int cval ;
int i, ioff ;
float offset, slope ;

static float scan [MAXSCN] ;	/* Pointer to independent scan array. */
static float scandx [MAXSCN] ;	/* Pointer to dependent scan array. */
static char namsmf [8][3] = {"sw","ss","se","ee","ne","nn","nw","ww"} ;
static float tmin [8] = {140.,100., 50., 35.,320.,285.,230.,210.} ;
static float tmax [8] = {310.,260.,230.,155.,130., 85., 40.,330.} ;

float  thmin, thmax ;	/* Theta limits (deg.). */
float  thinc = 5.0 ;	/* Theta increment (deg.). */
float  radius = 1.7 ;	/* Solar radius (Rsun) for 2nd diffraction ring. */
float  xxcen, yycen ;	/* Hi resolution center coordinates. */
float  dx, dy ;		/* Delta x & y. */

int ierr ;		/* Error variable. */
int imgcnt ;		/* Image counter. */
int it ;			/* Theta index variable. */
int log = 0 ;		/* Flag to indicate existence of output log file. */
int nval ;		/* # values in scan. */
int oldres = -1 ;	/* Save old resolution value. */
int oldsm = -1 ;		/* Save old sector mirror value. */
int seq = 0 ;		/* Flag to indicate sequence of images. */
int xl = 760 ;		/* Specification of GL window position & size. */
int yl = 480 ;
int xu = 1271 ;
int yu = 991 ;

   /*** Process input parameter(s). ***/

   strcpy (nampgm, argv [0]) ;
   sprintf (namlog, "%s.log", nampgm) ;

   fprintf (stderr, "%s: locate disk center by visually fitting an arc\n",
            nampgm) ;
   fprintf (stderr, "         to the second diffraction ring.\n") ;
   fprintf (stderr, "Origin for SMM C/P image is the upper left corner.\n") ;
   fprintf (stderr, "Direction for +x is to the right ;  +y is down.\n");

   if (argc == 1)		/* No arguments. */
      {
      fprintf (stderr, "usage: %s [image_filename | @filename_list] [outfile]\n", argv [0]) ;
      fprintf (stderr, "Enter C/P image file name: ") ;
      fscanf  (stdin,  "%s", namimg) ;
      fprintf (stderr, "Enter output filename ['none' to omit]: ") ;
      fscanf  (stdin, "%s", namlog) ;
      if (strncmp (namlog, "none") == 0)
         log = 0 ;
      else
         log = 1 ;
      }
   else
   if (argc == 2)		/* 1 argument: must be image filename. */
      {
      strcpy (namimg, argv [1]) ;
      log = 0 ;
      }
   else
   if (argc == 3)		/* 2 arguments: 1st must be image filename. */
      {
      strcpy (namimg, argv [1]) ;		/* Input image file. */
      strcpy (namlog, argv [2]) ;		/* Output file name. */
      log = 1 ;
      }

   if (namimg [0] == '@')
      {
      seq = 1 ;
      strcpy (namlst, &namimg [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst) ;
         exit (-1) ;
         }
      }

   fprintf (stderr, "namlog: %s\n", namlog) ;

   if (strncmp (namlog, "none") == 0)
      log = 0 ;
   else
      {
      log = 1 ;
      if ((LOG = fopen (namlog, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create output file: %s\n", namlog) ;
         exit (-2) ;
         }
      }

   igr = 1 ;
   igterm = 1 ;
   lsv = 0 ;
   lfc = 0 ;
   imgcnt = 0 ;

   /*** GL window initialization. ***/

   foreground () ;
   prefposition (xl, xu, yl, yu) ;
   winopen (nampgm) ;
   gl_clear (BLACK) ;
   onemap () ;  gconfig ();	/* Set single-map mode for color table. */
   get_cmap () ;			/* Save color map. */

   /*** Write grey-scale color table. ***/

   icol = 0 ;
   ioff = 20 ;
   for (i=0 ; i<ioff; i++)  mapcolor (i+512, icol, icol, icol);

   offset = (float) ioff ;
   slope = 256.0 / (256.0-offset) ;
   for (i=ioff ; i<256; i++)
      {
      cval = 512 + i ;
      icol = (short) ( ((float) i - offset) * slope) ;
      mapcolor (cval, icol, icol, icol) ;
      }

   multimap () ;  gconfig ();	/* Set multi-map mode for color table. */
   setmap (2) ;			/* Select color map #2. */

newimg:
   drawmode (NORMALDRAW) ;		/* Set normal mode for drawing. */
   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst) ;
         goto quit ;
         }

   imgcnt++ ;

   /*** Display C/P image in GL window. ***/

   if ((ierr = cpwgl (namimg)) != 0)
      {
      fprintf (stderr, "Error (%d) in cpwgl.\n", ierr) ;
      exit (-3) ;
      }

   /*** Overlay set-up. ***/

   overlay (2) ;  gconfig ();	/* Establish two bitplanes for overlay. */

   drawmode (OVERDRAW) ;		/* Set overlay mode for drawing. */
   mapcolor (0,   0,   0,   0) ;	/* color 0 -> black. */
   mapcolor (1, 255,   0,   0) ;	/* color 0 -> red.   */
   mapcolor (2,   0, 255,   0) ;	/* color 0 -> green. */
   mapcolor (3,   0,   0, 255) ;	/* color 0 -> blue.  */
   gl_clear (0) ;

   /*** Open input image disk file and read housekeeping record. ***/

   fprintf (stderr, "Reading image %s into memory.\n", namimg) ;

   imgbuf = rdimg (namimg) ;		/* Read image into memory array. */

   if (imgcnt != 1 && oldres == H.ires && oldsm == H.ism)
      {
      xcen = xxcen / (H.ires+1) ;
      ycen = yycen / (H.ires+1) ;
      }
   oldres = H.ires ;
   oldsm  = H.ism ;
   xxcen = xcen * (H.ires+1) ;
   yycen = ycen * (H.ires+1) ;
   fprintf (stdout,"     G M T          Q(%%)  IMG  IH-IV RES  EXP  FW     PW  5303  SM     CM  ROLL\n") ;
   hkline (stdout) ;

   /*** Theta scan limits. ***/

   thmin = tmin[H.ism] - (float) scroll ;
   thmax = tmax[H.ism] - (float) scroll ;
   printf ("ism: %d   thmin: %f   thmax: %f\n", H.ism, thmin, thmax) ;

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red) ;

   /*** Request new instructions from user: (deltaX, deltaY), next or quit. ***/

next:
   fflush  (stdin) ;
   fprintf (stderr, "disk center . . . x: %6.2f   y: %6.2f\n", xxcen, yycen) ;
   fprintf (stderr, "%s> ", nampgm) ;

   fprintf(stderr, "Action: delta x,y (hi res) | n (next image) | q (quit) : ") ;
   fgets (inline, 80, stdin) ;
   key = inline [0] ;

   if (inline [0] == 'n')			/* Process next image. */
      {
      if (log)
         {
         fprintf (LOG, "%s %3d %3d\n", namimg, (int) xxcen, (int) yycen) ;
         }
      goto newimg ;
      }
   else
   if (inline [0] == 'q' || inline [0] == ESC)	/* Quit program. */
      {
      if (log)
         {
         fprintf (LOG, "%s %3d %3d\n", namimg, (int) xxcen, (int) yycen) ;
         fclose (LOG) ;
         }
      fclose (LIST) ;
      goto quit ;
      }
   else
   switch (key)					/* delta X & delta Y values ? */
      {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
      case '-':
      case '+':
         break ;
      default:
         fprintf (stderr, "Invalid response.\n") ;
         goto next ;
      }

   sscanf (inline, "%f,%f", &dx, &dy) ;
   xxcen += dx ;
   yycen += dy ;
   xcen = xxcen / (float) (H.ires+1) ;
   ycen = yycen / (float) (H.ires+1) ;
   gl_clear (0) ;

   /*** Perform theta scan. ***/

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red) ;

   goto next ;

   /*** Relinquish storage for image array. ***/

quit:

   free (imgbuf) ;

   /*** Clean up overlay bitplanes. ***/
   
   drawmode (OVERDRAW) ;
   gl_clear (0) ;

   drawmode (NORMALDRAW) ;
   onemap () ;  gconfig ();	/* Set single-map mode for color table. */

   restore_cmap () ;		/* and restore the color map      */

   greset () ;
   gexit () ;
   exit (0) ;
}


/* gl_clear
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Clear GL window to color specified by `level'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_clear (level)

Colorindex level ;

{
   color (level) ;
   clear () ;

   return (0) ;
}

/* get_cmap
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Save current color map.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#define MAXCOLI 4095 

static short CarrayR [MAXCOLI+1], CarrayG [MAXCOLI+1], CarrayB [MAXCOLI+1] ;
unsigned short icm ;

get_cmap ()

{
   short rcomp, gcomp, bcomp ;

   for (icm = 0 ; icm <= MAXCOLI; icm++)
      {
      getmcolor (icm, &rcomp, &gcomp, &bcomp) ;
      CarrayR [icm] = rcomp ;
      CarrayG [icm] = gcomp ;
      CarrayB [icm] = bcomp ;
    }
}

/* restore_cmap
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Restore saved color map.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
restore_cmap ()
{

   for (icm = 0 ; icm <= MAXCOLI; icm++) 
      mapcolor (icm, CarrayR [icm], CarrayG [icm], CarrayB [icm]) ;
}
