/* qcen.c
 * usage: cpcen [image_filename | @filename_list] [outfile]
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate C/P disk center by fitting an arc
 * to second diffraction ring (1.7 Rsun).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Works only for calibrated (short integer) C/P images.
 * Generates its own greyscale color table (saves & restores pre-existing LUT).
 * Automatically scales pixel values by dividing them by 2.
 * Opens a temporary GL window for image display.
 * Writes a "dotted" arc using overlay planes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 31 July 1984
 * 25 March 1991: modify for use on SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>

/* Global variables. */

#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/grscale.h"
#define Red 1
#define Green 2
#define Blue 3
#define ESC 27

int GRN = 1 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc ;
char *argv [] ;
{

int    cpview() ;		/* Display image on Grinnell. */
char   *rdimg() ;		/* Read image from disk to memory. */

FILE   *LIST, *LOG ;
char   *imgbuf ;			/* Pointer to image array. */
char   key ;
static char flag [8] ;
static char inline [80] ;	/* Input line buffer. */
static char namimg [80] ;	/* Image file name. */
static char namlst [80] ;	/* Name of file containing image list. */
static char namlog [80] ;	/* Output file name. */
static char nampgm [80] ;	/* Program file name. */
static char answer [2] ;		/* Storage for y/n response. */

static short r [512], g [512], b [512] ;
short icol ;
int index ;
int i, ioff ;
float offset, slope ;

static float scan [MAXSCN] ;	/* Pointer to independent scan array. */
static float scandx [MAXSCN] ;	/* Pointer to dependent scan array. */
static char namsmf [8][3] = {"sw","ss","se","ee","ne","nn","nw","ww"} ;
static float tmin [9] = {135.,100., 45., 35.,315.,285.,230.,210.,  0.} ;
static float tmax [9] = {315.,260.,235.,155.,135., 85., 45.,330.,360.} ;

/*** Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). ***/
/* (No geometric correction applied.) */

static	float	vidcen [9][2] = { 36.0, 500.0,	/* sw */
				 214.0, 194.0,	/* s  */
				 540.0,   4.0,	/* se */
				 820.0,  88.0,	/* e  */
				 892.0, 362.0,	/* ne */
				 718.0, 680.0,	/* n  */
				 386.0, 856.0,	/* nw */
				 104.0, 792.0,	/* w  */
				 447.5, 447.5} ;	/* n+e+s+w */

/*** Disk centers (vs. Sector Mirror) for calibrated data (C34520). ***/
/* (Processed with geometric correction WW6COR). */

static	float	ww6cen [9][2] = { 48.0, 508.0,	/* sw */
				 222.0, 206.0,	/* s  */
				 530.0,  40.0,	/* se */
				 814.0,  86.0,	/* e  */
				 884.0, 354.0,	/* ne */
				 714.0, 672.0,	/* n  */
				 394.0, 824.0,	/* nw */
				 114.0, 786.0,	/* w  */
				 447.5, 447.5} ;	/* n+e+s+w */

float  thmin, thmax ;	/* Theta limits (deg.). */
float  thinc = 5.0 ;	/* Theta increment (deg.). */
float  radius = 1.7 ;	/* Solar radius (Rsun) for 2nd diffraction ring. */
float  xxcen, yycen ;	/* Hi resolution center coordinates. */
float  dx, dy ;		/* Delta x & y. */
static	float	secrs = 961. ;	/* arcseconds per solar radius. */
static	float	secpix = 5.933 ; /* arcseconds per pixel (Hi res). */

int ierr ;		/* Error variable. */
int imgcnt ;		/* Image counter. */
int it ;			/* Theta index variable. */
int log = 0 ;		/* Flag to indicate existence of output log file. */
int nval ;		/* # values in scan. */
int oldres = -1 ;	/* Save old resolution value. */
int oldsm = -1 ;		/* Save old sector mirror value. */
int seq = 0 ;		/* Flag to indicate sequence of images. */
int xlhi = 248 ;		/* Specification of GL window position & size. */
int ylhi =   0 ;
int xuhi = 1271 ;
int yuhi = 1023 ;
int xllo = 760 ;		/* Specification of GL window position & size. */
int yllo = 480 ;
int xulo = 1271 ;
int yulo = 991 ;

   strcpy (nampgm, argv [0]) ;
   sprintf (namlog, "%s.log", nampgm) ;

   fprintf (stderr, "%s: locate disk center by visually fitting an arc\n",
            nampgm) ;
   fprintf (stderr, "         to the second diffraction ring.\n") ;
   fprintf (stderr, "Origin for SMM C/P image is the upper left corner.\n") ;
   fprintf (stderr, "Direction for +x is to the right ;  +y is down.\n");


   if (argc == 1)		/* No arguments. */
      {
      fprintf (stderr, "usage: %s [image_filename | @filename_list] [outfile]\n", argv [0]) ;
      fprintf (stderr, "Enter C/P image file name: ") ;
      fscanf  (stdin,  "%s", namimg) ;
      fprintf (stderr, "Enter output filename ['none' to omit]: ") ;
      fscanf  (stdin, "%s", namlog) ;
      if (strncmp (namlog, "none") == 0)
         log = 0 ;
      else
         log = 1 ;
      }
   else
   if (argc == 2)		/* 1 argument: must be image filename. */
      {
      strcpy (namimg, argv [1]) ;
      log = 0 ;
      }
   else
   if (argc == 3)		/* 2 arguments: 1st must be image filename. */
      {
      strcpy (namimg, argv [1]) ;		/* Input image file. */
      strcpy (namlog, argv [2]) ;		/* Output file name. */
      log = 1 ;
      }

   if (namimg [0] == '@')
      {
      seq = 1 ;
      strcpy (namlst, &namimg [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst) ;
         exit (-1) ;
         }
      }

   if (strncmp (namlog, "none") == 0)
      log = 0 ;
   else
      {
      log = 1 ;
      if ((LOG = fopen (namlog, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create output file: %s\n", namlog) ;
         exit (-2) ;
         }
      }

   igr = 1 ;
   igterm = 1 ;
   lsv = 0 ;
   lfc = 0 ;
   imgcnt = 0 ;

   /*** Save current color map. ***/

/*
   for (i=0 ; i<256; i++)
      {
      index = i + 512 ;
      getmcolor (index, r[i], g[i], b[i]) ;
      }
 */

   /*** GL window initialization. ***/

   fscanf (LIST, "%s", namimg) ;
   cphskp (namimg) ;
   fclose (LIST) ;
   LIST = fopen (namlst, "r") ;

   foreground () ;
   if (H.ires == 0)
      prefposition (xlhi, xuhi, ylhi, yuhi) ;
   else
      prefposition (xllo, xulo, yllo, yulo) ;
   winopen (nampgm) ;
   gl_clear (BLACK) ;
   onemap () ;			/* Set single-map mode for color table. */
   get_cmap () ;			/* Save color map. */


   /*** Write grey-scale color table. ***/

/*
   icol = 0 ;
   ioff = 20 ;
   for (i=0 ; i<ioff; i++)  mapcolor (i+512, icol, icol, icol);

   offset = (float) ioff ;
   slope = 256.0 / (256.0-offset) ;
   for (i=ioff ; i<256; i++)
      {
      index = 512 + i ;
      icol = (short) ( ((float) i - offset) * slope) ;
      mapcolor (index, icol, icol, icol) ;
      }
 */

   multimap () ;			/* Set multi-map mode for color table. */
   gconfig () ;
   setmap (2) ;			/* Select color map #2. */

newimg:
   drawmode (NORMALDRAW) ;		/* Set normal mode for drawing. */
   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst) ;
         goto quit ;
         }
   imgcnt++ ;
/*   fprintf (stderr, "Displaying image %s\n", namimg) ; */
   if ((ierr = cpwgl (namimg)) != 0)
      {
      fprintf (stderr, "Error (%d) in cpwfb.\n", ierr) ;
      exit (-3) ;
      }

   /*** Overlay set-up. ***/

   overlay (2) ;			/* Establish two bitplanes for overlay. */
   gconfig () ;

   drawmode (OVERDRAW) ;		/* Set overlay mode for drawing. */
   mapcolor (0,   0,   0,   0) ;	/* color 0 -> black. */
   mapcolor (1, 255,   0,   0) ;	/* color 0 -> red.   */
   mapcolor (2,   0, 255,   0) ;	/* color 0 -> green. */
   mapcolor (3,   0,   0, 255) ;	/* color 0 -> blue.  */
   gl_clear (0) ;

   /*** Open input image disk file and read housekeeping record. ***/

   fprintf (stderr, "Reading image %s into memory.\n", namimg) ;
   imgbuf = rdimg (namimg) ;		/* Read image into memory array. */

   pixrs = secrs / (secpix * (H.ires + 1)) ;
   if (H.ism == 8)  pixrs *= 0.5 ;
   H.pixrs = pixrs ;
   rspix = 1.0 / pixrs ;

   if (idtype == 1)
      {
      xcen = vidcen [H.ism][0] / (float) (H.ires + 1) ;
      ycen = vidcen [H.ism][1] / (float) (H.ires + 1) ;
      }
   else
      {
      xcen = ww6cen [H.ism][0] / (float) (H.ires + 1) ;
      ycen = ww6cen [H.ism][1] / (float) (H.ires + 1) ;
      }

   if (imgcnt != 1 && oldres == H.ires && oldsm == H.ism)
      {
      xcen = xxcen / (H.ires+1) ;
      ycen = yycen / (H.ires+1) ;
      }
   oldres = H.ires ;
   oldsm  = H.ism ;
   xxcen = xcen * (H.ires+1) ;
   yycen = ycen * (H.ires+1) ;
   fprintf (stdout,"     G M T          Q(%%)  IMG  IH-IV RES  EXP  FW     PW  5303  SM     CM  ROLL\n") ;
   hkline (stdout) ;

/*
 * printf("ih,iv: %d %d\n", H.ih, H.iv) ;
 * printf("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr) ;
 * printf("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype) ;
 */

   /*** Theta scan limits. ***/

   thmin = tmin[H.ism] - (float) scroll ;
   thmax = tmax[H.ism] - (float) scroll ;
   printf ("ism: %d   thmin: %f   thmax: %f\n", H.ism, thmin, thmax) ;

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red) ;

next:
   fflush  (stdin) ;
   fprintf (stderr, "disk center . . . x: %6.2f   y: %6.2f\n", xxcen, yycen) ;
   fprintf (stderr, "%s> ", nampgm) ;
   fprintf(stderr, "Action: delta x,y (hi res) | n (next image) | q (quit) : ") ;
   fgets (inline, 80, stdin) ;
   key = inline [0] ;
   if (inline [0] == 'n')
      {
      if (log)
         {
         fprintf (LOG, "%s %3d %3d\n", namimg, (int) xxcen, (int) yycen) ;
         }
      goto newimg ;
      }
   else
   if (inline [0] == 'q' || inline [0] == ESC)
      {
      if (log)
         {
         fprintf (LOG, "%s %3d %3d\n", namimg, (int) xxcen, (int) yycen) ;
         fclose (LOG) ;
         }
      fclose (LIST) ;
      goto quit ;
      }
   else
   switch (key)
      {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
      case '-':
      case '+':
         break ;
      default:
         fprintf (stderr, "Invalid response.\n") ;
         goto next ;
      }

   sscanf (inline, "%f,%f", &dx, &dy) ;
   xxcen += dx ;
   yycen += dy ;
   xcen = xxcen / (float) (H.ires+1) ;
   ycen = yycen / (float) (H.ires+1) ;
   gl_clear (0) ;

   /*** Perform theta scan. ***/

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red) ;

   goto next ;

   /*** Relinquish storage for image array. ***/

quit:

   free (imgbuf) ;

   /*** Clean up overlay bitplanes. ***/
   
   drawmode (OVERDRAW) ;
   gl_clear (0) ;

   drawmode (NORMALDRAW) ;
   onemap () ;			/* Set single-map mode for color table. */
   gconfig () ;
   restore_cmap() ;		/* and restore the color map      */
   greset() ;

   gconfig () ;
   gexit() ;
   exit (0) ;
}


gl_clear (level)

Colorindex level ;

{
   color (level) ;
   clear () ;

   return (0) ;
}

#define MAXCOLI 4095 

static short CarrayR[MAXCOLI+1], CarrayG[MAXCOLI+1], CarrayB[MAXCOLI+1] ;
unsigned short index ;

get_cmap ()

{
   short rcomp, gcomp, bcomp ;

   for (index = 0 ; index <= MAXCOLI; index++)
      {
      getmcolor (index, &rcomp, &gcomp, &bcomp) ;
      CarrayR [index] = rcomp ;
      CarrayG [index] = gcomp ;
      CarrayB [index] = bcomp ;
    }
}

restore_cmap ()
{

   for (index = 0 ; index <= MAXCOLI; index++) 
      mapcolor (index, CarrayR [index], CarrayG [index], CarrayB [index]) ;
}
