/*-----------------------------------------------------------------------
 * This function provides a method for averaging all the points in a scan.
 * Each scan (x, y and z) is composed of an array of "parallel" scans,
 * a certain number, given by SCANS, on one side, and the same amount on
 * the other side of the true scan axis.  In this way, if the values on the
 * actual scan axis are erroroneous, we can still make an accurate judgement
 * by averaging those around it.  Average() simply looks through each scan
 * until it finds the BRLIMIT.  After it is through finding the position of
 * the BRLIMIT in each scan, it uses an averaging scheme to determine the
 * "best" position.  
 *------------------------------------------------------------------------
 * Ron Decker ASAP/CU  SMM C/P March 2, 1992
 * Initial Creation.
 */ 

#include <stdio.h>
#include <math.h>
#include "center.h"
#include "/usr/people/smm/lib/include/cplib.ext"

extern struct hskp H;

int
average(scan, scantype)
int scan[][RADELTA*2];                     /*- x, y or z scan array */
char *scantype;     			   /*- string for printouts */
{

   int lim_array[(SCANS*2)+1];             /*- BRLIMIT position go here */
   int w, x, y, z;			   /*- loop variables */
   int median;    			   /*- find the median br position */
   int scans;                              /*- used in averaging loop */
   int diff;                		   /*- diff of median and br position*/ 
   int brlimit;                            /*- calculate brlimit */

   /*-
    * If this is a clear image, adjust the brlimit.
    */
   if (strncmp("CLR", H.pol, 3) == 0) {
      brlimit = BRLIMIT + CLRBRFIX;
   }
   else {
      brlimit = BRLIMIT;
   }

   /*-
    * Initialize the brlimit position array.
    */
   for (x=0; x<=(SCANS*2); x++) {
      lim_array[x] = INIT;
   }
 
   /*-
    * Search through each scan looking for the first brlimit value.  When found
    * record it in the lim_array.
    */
   for (x=0; x<=(SCANS*2); x++) {

      /*- 
       * Find the first brlimit.
       */
      y = 0;
      while ((scan[x][y] < brlimit) && (y < RADELTA*2)) {
	 y++;
      }

      /*-
       * If it wasn't found, report it and take the highest value found.
       */
      if (scan[x][y] < brlimit) {
	 /*fprintf(stderr,"average: %s: limit not reached\n", scantype);*/
	 y--;
      }
  
      /*-
       * Insert the value found into the array at its sorted position.
       * Lowest position values are inserted first, highest ones last.
       */
      z = (SCANS*2);
      while ((y < lim_array[z]) && (z > 0)) {
      z--;
      }
      if (lim_array[z] != INIT) {
         for (w=0; w < z; w++) {
            lim_array[w] = lim_array[w+1];
         }
      }
      lim_array[z] = y;
   }
/*
   for (x=0; x<= (SCANS*2); x++) {
      lim_array[x]++;
   }
   */

   /*-
    * This performs the actual averaging routine.  Because all of the position
    * values are sorted in the lim_array, we simply take the middle value in
    * the array, the median, and average it with values on either side in an
    * alternating fashion. i.e. average the median  with the one below it, then
    * with the one above it, then the one two below it, then two above it, etc.
    * Only values which are within AVGDELTA of the median are averaged in,
    * in an attempt to avoid averaging with an erroroneous value which could
    * destroy the whole process.
    */ 
   median = lim_array[SCANS];
   scans = SCANS;
   y = -1;

   for (x=0; x <= SCANS*2; x++) {
      y = -y;
      scans += y*x;
      diff = abs(median-lim_array[scans]);
      if ((0 <= diff) && (diff <= AVGDELTA)) {
	 median = ((median + lim_array[scans])/2);
      }
   }

   return(median);
}



	    

