/* csj.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Intensity scans (radial & theta): bilinear interpolation algorithm.
 * Reverse order in applying corrections: F-corona, vignet, stray, Munro.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 * 24 March 1987
 * 2 April 1987: change polaroid transmission factor (0.38 to 0.76).
 * 3 April 1987: add point-point scan.
 * 27 March 1990: convert to NCAR GKS graphics.
 * 10 June 1991: adapt for SGI/IRIS GL windows.
 * 27 August 1993: use Jupiter calibration correction.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/local/home/stanger/lib/include/cplib.h"
#include "/local/home/stanger/lib/include/img_scale.h"
#include "/local/home/stanger/lib/include/gl_color.h"
#include "/local/home/stanger/lib/include/ov_color.h"

/* Standard C system library and Math functions library. */

#include <stdio.h>
#include <math.h>

int DSP = 0;
int pcount = 0;
float polcor = 1.;		/* Polaroid transmission correction factor. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

extern	int prscan();	/* Radial intensity scan function. */
extern	int ptscan();	/* Theta intensity scan function. */
extern  int hkline();	/* Housekeeping information display on terminal. */
extern  int xyplot();	/* XY plot using NCAR system plot package. */
extern  int xysplot();	/* XY plot using NCAR system plot package. */
extern char *rdimg();	/* Read image from disk to memory. */

char	*image;			/* Pointer to image array. */
static char namdat [80];	/* Scan data file name. */
static char namimg [80];	/* Image file name. */
static char nampgm [80];	/* Program name. */
static char answer [4];		/* Storage for y/n response. */
static char action [12];	/* Storage for action response. */
static char ovrcolor [2];	/* Storage for overlay color. */
static char namemc [80]; /* Metacode file name. */
static char labx [40];	/* Label for X-axis. */
static char laby [40];	/* Label for Y-axis. */
static char title [60];	/* Graph title. */
static char labtyp [30];/* Data type description. */
static char gksnam [] = "cs.cgm";
static char gksout [80];
static char buff [80];

static float scan[MAXSCN];	/* Scan magnitude array. */
static float scandx[MAXSCN];	/* Scan index array. */
/*
 * Polaroid wheel :
 * p0 : clear			    telemetry value = 0
 * p1 : +60 degrees (w.r.t. pylon)  telemetry value = 1
 * p2 :   0 degrees		    telemetry value = 3
 * p3 : -60 degrees		    telemetry value = 2
 */
static float pctab[4] = {1.,0.4077,0.3942,0.4214}; /* order: p0, p1, p3, p2. */

float	rmin, rmax;	/* Radial limits (Rsun). */
float	rinc;		/* Radial increment. */
float	theta;		/* Solar position angle (degrees). */
float	th;		/* Solar position angle (degrees). */
float	tmin, tmax;	/* Angular limits (degrees). */
float	tinc;		/* Angular increment. */
float	radius;		/* Radius (R/Rsun). */
float	fymin, fymax;	/* Min/max values for y-axis plot scaling. */
float	symin = 0.0;	/* Min value for y-axis plot scaling. */
float	symax = 0.0;	/* Max value for y-axis plot scaling. */

int	scntyp = 2;	/* Scan type: 1=radial, 2=line. */
int	nval;		/* # values in scan. */
int	ndata;
int	i;
int	ierr;
int	ip1, ir1;
int	ip2, ir2;
int	idot;		/* Index pointer to "." in image filename.*/
int	ixscale = 0;	/* x-axis scaling type; 0=linear. */
int	iyscale = 0;	/* y-axis scaling type; 0=linear. */
int	ibkg = 1;	/* Background flag; 0=tick marks only. */
int	ismooth = 0;
int	plotit = 1;
int	clip = 0;
int	lunerr = 7;
int	isiz = 0;
int	wksid = 1;
int	lunit = 8;
int	wkstype = 1;

FILE	*datfd;		/* File descriptor for scan data file. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   fprintf (stderr, "cs: C/P intensity scans.\n");
   fprintf (stderr, "Note: action R requests multiple radial scans.\n");
   fprintf (stderr, "Note: action T requests multiple theta  scans.\n");
   fprintf (stderr, "Note: adding `xl' to an action character => log (x).\n");
   fprintf (stderr, "Note: adding `yl' to an action character => log (y).\n");
   fprintf (stderr,
            "Note: adding `yf' to an action character => set y limits.\n");
   strcpy (nampgm, argv [0]);

   if (argc < 2)
      {
      fprintf (stderr, "usage: %s [image_file_name]\n", argv [0]);
      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);
      }
   else
      {
      strcpy (namimg, argv [1]);
      }

   DSP = 1;
   igr = 1;
   ngr = 0;
   imgch = 0;
   ovrch = 10;

   lcc  = 0;
   lstr = 0;
   lvig = 0;
   lfc  = 0;

   /*** Create file names for scan data and plot. ***/

   for (i=0; i<strlen(namimg); i++)  if (namimg[i] == '.')  idot = i;
   strncpy (namdat, namimg, idot);
   strncpy (namemc, namimg, idot);
   strcat (namdat, ".dat");
   strcat (namemc, ".cgm");
   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
   putenv (gksout);

   fprintf (stderr, "data file:  %s\n", namdat);
   fprintf (stderr, "metacode plot file: %s\n", namemc);
   datfd = fopen (namdat, "w");


/*
   fprintf (stderr, "Are you using a graphics terminal [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   igterm = 0;
 */
    
/*   if (strncmp("y",answer,1) == 0) */
      {
      igterm = 1;
/*      gksini_ (&plotit, gksnam); */
      opngks_ ();

/*      gsclip_ (&clip); */
      }

   /*** Open input image disk file and read housekeeping record. ***/

   image = rdimg (namimg);
   hkline (stdout);
   if (idtype == 1)   strcpy (labtyp, "vidicon DN value   ");
   if (idtype == 3)   strcpy (labtyp, "brightness(B/Bsun) ");
   if (idtype == 4)   strcpy (labtyp, "density (elec/cm2) ");
   if (idtype == 5)   strcpy (labtyp, "B2 - B1   (B/Bsun) ");
   if (idtype == 6)   strcpy (labtyp, "     Bk   (B/Bsun) ");
   if (idtype == 7)   strcpy (labtyp, "(Be-Bb)/Bk (B/Bsun)");
   if (idtype == 8)   strcpy (labtyp, "(Be-Bb)/Bb (B/Bsun)");
   if (idtype == 9)   strcpy (labtyp, " polarization frac.");
   if (idtype == 10)  strcpy (labtyp, "      pB  (B/Bsun) ");
   if (idtype == 11)  strcpy (labtyp, "     B[r+t] (B/Bsun) ");
   if (idtype == 12)  strcpy (labtyp, "     B[tan] (B/Bsun) ");
   if (idtype == 13)  strcpy (labtyp, "     B[rad] (B/Bsun) ");
   if (idtype == 14)  strcpy (labtyp, "     B[f+s] (B/Bsun) ");


   /* Display works only for Low Resolution images. */
/*
   if (H.ires == 0)
      {
      igr = 0;
      fprintf (stderr, "High resolution images are not displayable.\n");
      }
 */

   printf ("Enter # points to average in scans : ");
   scanf  ("%d", &nave);

   /*** Display initialization. ***/

   if (igr == 1)
      {
      foreground ();

      if (H.ires == 0)
      prefposition (266, 1279,   0, 1023);
      else

      prefposition (760, 1271, 480, 991);/*512x512 window: upper right corner.*/
      fminit ();			/* Initialize SGI font manager. */

      winopen (nampgm);
      gl_wclear (BLACK);
      onemap ();
      gconfig ();
      pixmode (PM_ADD24, 512);		/* i.e., effectively use color map #2.*/

      glcp_view (namimg, 1);

      /*** Overlay set-up. ***/

      overlay (2);		/* Establish two bitplanes for overlay. */
      gconfig ();

      drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
      mapcolor (ov_black,   0,   0,   0);	/* color 0 -> black. */
      mapcolor (ov_red,   255,   0,   0);	/* color 0 -> red.   */
      mapcolor (ov_green,   0, 255,   0);	/* color 0 -> green. */
      mapcolor (ov_blue,    0,   0, 255);	/* color 0 -> blue.  */
      gl_wclear (ov_black);
      }

   /*** For calibrated images, which (if any) corrections need to be made ? ***/

   if (idtype ==  3 || idtype ==  5 || idtype ==  6 || idtype == 10 ||
       idtype == 11 || idtype == 12 || idtype == 13 || idtype == 14)
      {
      printf ("Apply Jupiter calibration correction [y,n] ? ");
      scanf  ("%s", answer);
      if (strncmp("y", answer, 1) == 0)  lcc = 1;
      else lcc = 0;
      }

   if (idtype ==  3 || idtype ==  5 ||                 idtype ==  10 ||
       idtype == 11 || idtype == 12 || idtype == 13 || idtype == 14)
      {
      printf ("Remove stray light [y,n] ? ");
      scanf  ("%s", answer);
      if (strncmp("y", answer, 1) == 0)  lstr = 1;
      else lstr = 0;
      }

   if (idtype != 7 && idtype != 8 && idtype != 9 && idtype != 15)
      {
      printf ("Remove vignetting [y,n] ? ");
      scanf  ("%s", answer);
      if (strncmp("y", answer, 1) == 0)  lvig = 1;
      else lvig = 0;
      }

   if (idtype ==  3 || idtype == 11 || idtype == 12 || idtype == 13 ||
       idtype == 14)
      {
      printf ("Remove F-corona [y,n] ? ");
      scanf  ("%s", answer);
      if (strncmp("y", answer, 1) == 0)  lfc = 1;
      else lfc = 0;
      }

   if (H.ipw != 0 && idtype == 3)
      {
      printf ("Apply polaroid transmission correction [y,n] ? ");
      scanf  ("%s", answer);
      if ((strncmp("y", answer, 1) == 0))
         polcor = 1.0 / (2.0 * pctab[H.ipw]);
      else polcor = 1.;
      fprintf (stdout, "polcor: %f\n", polcor);
      }

/******* Scan loop. *******/

next:
   scntyp = 0;
   symin = symax = 0.0;

   fprintf (stderr, "Action: radial, theta, point, quit [r,t,p,q] ? ");
   fscanf  (stdin,  "%s", action);

   if (strncmp ("q", action, 1) == 0)
      {
      fclose (datfd);
      free (image);

      /*** Clean up overlay bitplanes. ***/
   
      if (igr == 1)
         {
         drawmode (OVERDRAW);
         gl_wclear (ov_black);

         drawmode (NORMALDRAW);
         onemap ();		/* Set single-map mode for color table. */
         gconfig ();
         pixmode (PM_ADD24, 0);	/* i.e., effectively use color map #0.*/
         gexit();
         }

      if (igterm)
	 {
	 clsgks_ ();
/*         unlink (gksnam); */
	 }
      exit(0);
      }

   else if (action [0] == 'r' || action [0] == 'R')
      scntyp = 1;
   else if (action [0] == 'p' || action [0] == 'P')
      scntyp = 2;
   else if (action [0] == 't' || action [0] == 'T')
      scntyp = 3;

   if (action [1] == 'x' && action [2] == 'l')  ixscale = 1;  else ixscale = 0;
   if (action [1] == 'y' && action [2] == 'l')  iyscale = 1;  else iyscale = 0;
   if ((action [1] == 'y' && action [2] == 'f') || iyscale == 1)
      {
      fprintf (stderr, "Enter limits for Y-axis scale (min,max): ");
      fscanf  (stdin,  "%f,%f", &symin, &symax);
      fprintf (stdout, "symin: %g   symax: %g\n", symin, symax);
      }
   printf ("ixscale: %d   iyscale: %d\n", ixscale, iyscale);

   /*** Check calibration mirror telemetry. ***/

   if (H.icm == 1)	/* Calibration Mirror is OUT. */
      {
      if (lsv == 1)
         {
         printf ("Remove F-corona (y/n) ? ");
         scanf ("%s", answer);
         if (strncmp("y", answer, 1) == 0)  lfc = 1;
         else lfc = 0;
         }
      }

   else			/* Assume calibration mirror is IN. */
      {
      fprintf (stdout,"*** calibration mirror IN: no removal of stray,vignet, or F-corona. ***\n");
      lsv = 0;
      lfc = 0;
      }

   if (nbypix == 1)
   sprintf (laby, "%s pc=%4.2f", labtyp, polcor);
   else
   if (nbypix == 2)
   sprintf (laby, "%s pc=%4.2f", labtyp, polcor);
   if (nbypix == 4)
   sprintf (laby, "%s", labtyp);

   fprintf (datfd, "Image file name: %s\n", namimg);
   if (idtype == 3 && lcc == 1)
   fprintf (datfd, "*** WARNING *** Jupiter calibration correction has been applied.\n");
   if (H.icm != 1)
   fprintf (datfd,"*** calibration mirror IN: no removal of stray,vignet, or F-corona. ***\n");
   if (lstr == 1)  fprintf (datfd,"*** Stray light removed. ***\n");
   if (lvig == 1)  fprintf (datfd,"*** Vignetting removed. ***\n");
   if (lfc == 1)   fprintf (datfd,"*** F-corona removed. ***\n");

   /*** Radial scan. ***/

   if (scntyp == 1)
      {
      if (action [0] == 'R')
         {
         fprintf (stderr,
            "Enter begin/end/inc solar position angles for radial scan (deg.): ");
         fscanf  (stdin,  "%f,%f,%f", &tmin, &tmax, &tinc);
         }
      else
         {
         fprintf (stderr, "Enter solar position angle for radial scan (deg.): ");
         fscanf  (stdin,  "%f", &theta);
         tmin = tmax = theta;
         tinc = 1.0;
         }

      fprintf (stderr, "Enter solar radius min, max, & increment (Rsun): ");
      fscanf  (stdin,  "%f,%f,%f", &rmin, &rmax, &rinc);
      gets (buff);

      for (theta = tmin; theta <= tmax; theta += tinc)
         {
         prscan (namimg, image, rmin, rmax, rinc, theta, &scntyp,
                                scan, scandx, &nval);

         /* Write scan values to output data file. */

         fprintf (datfd, "Radial scan  %6.2f to %6.2f Rsun  %7.2f degrees\n",
                  rmin, rmax, theta);
         fprintf (datfd, "index    radius   pos.ang.  %s\n", labtyp) ;
         for (i=0; i<nval; i++)
            {
            fprintf (datfd,"%5d   %7.3f   %7.2f   %9.3e\n",
                     i, scandx [i], theta, scan [i]);
            }


         /* Plot scan. */
 
         if (igterm == 1)
            {
            if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s +jsvf", theta, namimg);
            else
            if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -s",theta,namimg);
            else
            if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -v",theta,namimg);
            else
            if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -f",theta,namimg);
            else
            if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -sv",theta,namimg);
            else
            if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -vf",theta,namimg);
            else
            if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -sf",theta,namimg);
            else
            if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -svf",theta,namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -j",theta,namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -js",theta,namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -jv",theta,namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -jf",theta,namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 0)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -jsv",theta,namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -jsf",theta,namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -jvf",theta,namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 1)
            sprintf (title,"Radial Scan %5.1f deg. %20.20s -jsvf",theta,namimg);
            else
            sprintf (title,"Radial Scan %5.1f deg. %20.20s ????",theta,namimg);
            sprintf (labx,"solar radius (Rsun)");

            if (symin != symax)
               {
               fymin = symin;
               fymax = symax;
               }
            
            xysplot (scandx, scan, rmin, rmax, fymin, fymax, nval,
                     namemc, title, labx, laby, ixscale, iyscale, ibkg);
            }
/*         ezxy_ (scandx, scan, &nval, title, strlen(title)); */
         }
      }

   else
   if (scntyp == 2)			/* Line scan. */

      {
      fprintf (stderr, "Enter starting pixel & line (x,y): ");
      fscanf  (stdin,  "%d,%d", &ip1, &ir1);
      fprintf (stderr, "Enter ending pixel & line (x,y): ");
      fscanf  (stdin,  "%d,%d", &ip2, &ir2);
      gets (buff);

      if ((ierr=plscan(image, ip1, ir1, ip2, ir2, &scntyp, scan, scandx, &nval))!= 0)
         fprintf (stderr, "cs> linscn error: %d\n", ierr);

      /* Write scan values to output data file. */

      fprintf (datfd, "Line Scan  (%d,%d) - (%d,%d)\n",
               ip1, ir1, ip2, ir2);
      fprintf (datfd, "index  dist:Rsun  %s\n", labtyp) ;
      for (i=0; i<nval; i++)
         {
         fprintf (datfd, "%5d   %7.2f   %9.2e\n",
	          i, scandx [i], scan [i]);
         }


      /* Plot scan. */
 
      if (igterm == 1)
         {
         sprintf (title,"Scan %d,%d-%d,%d %20.20s", ip1,ir1,ip2,ir2,namimg);

         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 0)
         strcat (title, " +jsvf");
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 0)
         strcat (title, " -s");
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 0)
         strcat (title, " -v");
         else
         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 1)
         strcat (title, " -f");
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 0)
         strcat (title, " -sv");
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 1)
         strcat (title, " -vf");
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 1)
         strcat (title, " -sf");
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 1)
         strcat (title, " -svf");
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 0)
         strcat (title, " -j");
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 0)
         strcat (title, " -js");
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 0)
         strcat (title, " -jv");
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 1)
         strcat (title, " -jf");
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 0)
         strcat (title, " -jsv");
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 1)
         strcat (title, " -jsf");
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 1)
         strcat (title, " -jvf");
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 1)
         strcat (title, " -jsvf");
         else
         strcat (title, " ????");

         sprintf (labx, "distance along line (Rsun)");

         if (symin != symax)
            {
            fymin = symin;
            fymax = symax;
            xysplot (scandx, scan, scandx [0], scandx [nval-1], fymin, fymax,
                     nval, namemc, title, labx, laby, ixscale, iyscale, ibkg);
            }
         else    
            xyplot (scandx, scan, nval, namemc, title, labx, laby,
                    ixscale, iyscale, ibkg);
         }
/*      ezxy_ (scandx, scan, &nval, title, strlen(title)); */
      }

   else

   if (scntyp == 3)			/* Theta scan. */
      {
      if (action [0] == 'T')
         {
         fprintf (stderr,
            "Enter begin/end/inc solar radii for theta scan (Rsun): ");
         fscanf  (stdin,  "%f,%f,%f", &rmin, &rmax, &rinc);
         }
      else
         {
         fprintf (stderr, "Enter solar radius for theta scan (Rsun): ");
         fscanf  (stdin,  "%f", &radius);
         rmin = rmax = radius;
         rinc = 0.1;
         }

      fprintf (stderr,"Enter position angle min, max, & increment (degrees): ");
      fscanf  (stdin, "%f,%f,%f", &tmin, &tmax, &tinc);
      gets (buff);

      for (radius = rmin; radius <= rmax; radius += rinc)
         {
         ierr = ptscan (namimg, image, tmin, tmax, tinc, radius, scan, scandx, &nval);
	 if (ierr != 0)
	    {
	    fprintf (stderr, "ptscan error: %d\n", ierr);
	    exit (ierr);
	    }

         /* Print scan values. */


         fprintf (datfd, "Theta scan  %6.2f to %6.2f degrees  %7.2f Rsun\n",
                  tmin, tmax, radius);
         fprintf (datfd, "index    radius   pos.ang.  %s\n", labtyp) ;
         for (i=0; i<nval; i++)
            {
            th = scandx [i];
            if (th > 360.0)  th -= 360.0;
               fprintf (datfd,"%5d   %7.3f   %7.2f   %9.2g\n",
                        i, radius, th, scan [i]);
            }
         printf ("nval:%3d\n", nval);

         /* Plot scan. */
 
         if (igterm == 1)
            {
            if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 0)
            sprintf (title,
               "Theta Scan %5.2f Rsun %20.20s +jsvf", radius, namimg);
            else
            if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -s", radius, namimg);
            else
            if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -v", radius, namimg);
            else
            if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -f", radius, namimg);
            else
            if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -sv", radius, namimg);
            else
            if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -vf", radius, namimg);
            else
             if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -sf", radius, namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -j", radius, namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -js", radius, namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -jv", radius, namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -jf", radius, namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 0)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -jsv", radius, namimg);
            else
            if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -jvf", radius, namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -jsf", radius, namimg);
            else
            if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 1)
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s -jsvf", radius, namimg);
            else
            sprintf (title,"Theta Scan %5.2f Rsun %20.20s ????", radius, namimg);

            sprintf (labx,"position angle (degrees)");

            if (symin != symax)
               {
               fymin = symin;
               fymax = symax;
               printf ("xysplot   fymin: %g   fymax: %g\n", fymin, fymax);
               xysplot (scandx, scan, tmin, tmax, fymin, fymax, nval,
                        namemc, title, labx, laby, ixscale, iyscale, ibkg);
               }
            else    
               xyplot (scandx, scan, nval, namemc, title, labx, laby,
                       ixscale, iyscale, ibkg);
            }
/*         ezxy_ (scandx, scan, &nval, title, strlen(title)); */
         }
      }

   fprintf (datfd, "\n");
   goto next;
}
