/* rollcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute roll in degrees and store in bytes #88 & #89.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 12 September 1989
 * 20 December 1989: remove line "if (scroll > 0)  scroll = roll + 7;"
 *                 in logic to round off roll to closest 10 degrees for 1984-89.
 *                 Add +5 to positive roll, -5 to negative roll before
 *                 dividing and multiplying by 10.
 *
 *    By default, extract roll information (scaled radian units)
 *    from bytes 11 & 12.
 *
 *    However, read bytes 88 & 89 instead if NONE of the following is true :
 *       if both=127 or 128;
 *       if one=127, one=128;
 *       if both=0
 *    The above conditions indicate that bytes 88 & 89 simply contain typical
 *    values characterized by spare bytes in the telemetry stream.
 *    In other words, if bytes 88 & 89 together form an integer whose value
 *    is in the expected range of 1-360, then this value is valid and
 *    will simply be rewritten with the range 0-359.
 *    Note that program `fixroll.c' will write an integer roll value into
 *    bytes 88 & 89 in the range 1-360 degrees.
 * 12 January 1990: For rolla<0, scrolla = (rolla*10)/10 (do not subtract 5).
 * 22 January 1990: When scrollb > 0, rolldif = scrollaa - scrollb.
 *  6 July 1990: round off scroll (1984-89) by adding 7, dividing by 10.
 * 23 July 1990: round upward positive rolls for 1980 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

rollcp (cbuf)

char cbuf [];

{
   double froll;
   double rollwt;
   double radrol;
   int isign;
   int roll;
   int rolla, rollb;
   int rolldif;
   int roll1, roll2;
   int scroll;
   int scrolla, scrollaa, scrollb;
   int lyear;

   lyear = cbuf [0];

/******* Compute roll using bytes 11 & 12. *******/

   /*--- Roll (degrees).
    * 6 m.s.bits of roll register are the 6 m.s.bits of S/C roll (18 bits)
    * 6 l.s.bits of roll register are the 6 m.s.bits of S/C time.
    */

   roll1 = (int) (cbuf [11] & 0377);
   roll2 = (int) (cbuf [12] & 0377);
   isign =  (roll1 & 8) / 8;

   /*... Least significant roll bit (1984-89) = 7.16197 degrees. ...*/
   /*... Truncate for negative rolls, round up for positive rolls. ...*/

   if (lyear > 80)
      {
      roll = (int) ((roll1 & 7) * 4 + (roll2 & 0300) / 64);
      radrol = (double) roll;
      if (isign == 1)  radrol = - (double) (((~(roll)+1) & 037));
      rollwt = 7.161972438;
      rolla = (int) (radrol * rollwt);
      }

   /*... Least significant roll bit (1980) = 0.111906 degrees. ...*/

   if (lyear == 80)
      {
      roll = (int) ((roll1 & 7) * 256 + (roll2 & 0377));
      radrol = (double) roll;
      if (isign == 1)  radrol = - (double) (((~(roll)+1) & 03777));
      rollwt = 0.111905819;
      froll = (radrol * rollwt);
      if (froll > 0.0)  rolla = (int) (froll + 0.5);  else rolla = (int) froll;
      }

/*printf ("radrol: %f  rollwt: %f   roll [rad]: %d\n", radrol, rollwt, roll);*/


/*   printf ("roll: %d\n", rolla); */

/*** For 1984-1989, round off roll to closest 10 degrees. ***/

   if (lyear == 80)
      scrolla = rolla;
   else
   if (lyear > 80)
      {
      if (isign == 1)
	 scrolla = rolla;
      else
         scrolla = rolla + 7;
      scrolla = (scrolla / 10) * 10;
      }
/*   if (rolla == -7) scrolla = 0; */	/* Special case. */

   if (scrolla <=  0)  scrolla += 360;
   if (scrolla > 360)  scrolla -= 360;

/******* Compute roll using bytes 88 & 89. *******/

   roll1 = (int) (cbuf [88] & 0377);
   roll2 = (int) (cbuf [89] & 0377);
   rollb = roll1 * 256 + roll2;
   if (rollb > -360 && rollb < 361 && rollb != 0)
      {
      scrollb = rollb;
      if (scrollb >= 360)  scrollb -= 360;
      if (scrollb <    0)  scrollb += 360;
      }
   else
      scrollb = -1;

/******* Compare rolla and rollb. *******/

   if (scrollb <= 0)		/* Invalid value in bytes 88 & 89. */
      scroll = scrolla;
   else				/* Valid value in bytes 88 & 89. */
      {
      scrollaa = rolla;
      if (scrollaa <   0)  scrollaa += 360;
      if (scrollaa > 360)  scrollaa -= 360;

      scroll = scrollb;

      rolldif = scrollaa - scrollb;
      if (rolldif < 0)  rolldif = - rolldif;
      if (rolldif > 10)
         {
         fprintf (stderr, "roll (bytes 11&12): %d disagrees with roll (bytes 88&89): %d\n",
		  scrollaa, scrollb);
	 fprintf (stderr, "Check records for correct roll, then run fixroll.\n");
	 exit (-1);
         }
      }

   /*** If roll is negative, convert to positive value. ***/

   if (scroll < 0)  scroll += 360;

/*
 *   fprintf (stdout, "rollcp> roll [degrees]: %d [decimal]   %o [octal]\n",
 *	    scroll, scroll);
 */

   /*** Store roll [integer degrees] into bytes #88 & 89 of housekeeping. ***/

   cbuf [88] = (char) ((scroll &0177400) >> 8);
   cbuf [89] = (char)  (scroll &   0377);

   return (0);
}
