/* cptofits_v304.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert SMM C/P image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR   27 October 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Modification History:
 *
 *  3 Feb 1994: Change order of data units: image, hk+bm, colormap.
 * 28 Mar 1994: reverse image top-to-bottom.
 * 18 Aug 1994: change CPVER to VERSION.
 *
 * 19 Mar 1996: VERSION = 'SMM C/P 1.02'
 *		change cptilt from +43 to -43 degrees. CROTA1=cptilt+scroll.
 *		crpix2 = ny - ycen, since image is reversed top to bottom.
 * 25 Mar 1996: change wmin/wmax to SCALED values.
 *
 * 10 Jul 1996: VERSION = "SMM C/P 1.03'
 *               Add DIRAXIS1='RIGHT' and DIRAXIS2='UP'.
 *  4 Mar 1997: CDELT1 & CDELT2 changed from Rsun to Arcsecond units.
 *  5 Mar 1997: Add CRRADIUS (pixels/solar radius).  Change RSUN units
 *                from pixels/solar radius to arcsec/solar radius.
 *  2 Sep 1997: Corrected 5303 filter label from FE-IV to FE-XIV.
 * 13 Nov 1997: Subtract SCROLL from CPTILT to get orientation.
 * 13 Nov 1997: Change cptilt to +43 degrees.  CROTA1 = - (cptilt+scroll).
 * 23 Feb 1998: Object='corona' or 'calib', depending upon calmir positon.
 *
 * 29 Mar 1999:	VERSION = 'SMM C/P 1.04'
 *		Change COORDNAM from "GEOCENTRIC" to "HELIOCENTRIC",
 *		since the SMM S/C zero degree roll axis is aligned with
 *		solar north.
 *		Add bunitcom to better explain the "BUNIT" units.
 *  2 Apr 1999: RSUN value is now computed by pb0r & stored in "H.rsun".
 * 13 May 1999: Define data type #16 as Mass (grams).
 * 15 Jun 1999: "smmcp." appended as a prefix to the FITS filename.
 *  4 Sep 2002: Change output filename format.  Previously, it was
 *              the same as the native file, with ".fts" appended.
 *		Now, it is: YYYYMMDD.HHMMSS.smmcp.TQQ.fts
 *		where YYYYMMDD = date [year month day],
 *		HHMMSS = time [hours minutes seconds],
 *		T = data type: v [raw], c [cal],
 *		QQ=quadrant: nn, ne, ee, se, ss, sw, ww, nw, 4q [n+e+s+w].
 *
 * 19 Sep 2002: VERSION = 'SMM C/P 1.05'
 *		DATE-OBS changed from old format [DD/MM/YY] to new: YYYY-MM-DD.
 *		Also, added TIMESYS keyword, with value 'UTC'.
 * 27 Sep 2002: VERSION = 'SMM C/P 2.00'
 *		Extension header modified to include ALL hk items.
 * 16 Dec 2005: VERSION = 'SMM C/P 3.01'
 *		Modify output file name: YYYYMMDD.HHMMSS.smmcp.wsr.SM.fsv.fts
 *              where YYYY=year, MM=month, DD=day, HH=hour, SS=seconds,
 *		WSR=wavelength/source/reduction code, SM=sector mirror,
 * 16 Jan 2006: VERSION = 'SMM C/P 3.02'
 *		Added LCC keyword (Jupiter calibration correction applied).
 * 17 Jan 2006: VERSION = 'SMM C/P 3.03'
 *		Adjust dispmax if CPH.lcc = 1.
 * 24 Jan 2006: VERSION = 'SMM C/P 3.04'
 *		Use idtype to specify 'reduce' names.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/smmcplib.h"
#define MAXLIN 180		/* Maximum # lines allocated for FITS header. */
char fitbuf   [MAXLIN][80] ;	/* FITS header buffer. */
int nbyfits = 2880 ;			/* # bytes per FITS logical record. */

cptofits3 (namimg, pmode)

   char namimg [] ;		/* Name of primary file. */

   int pmode ;			/* Print mode. */


{
   extern unsigned char *rdfimg () ;
   extern char *malloc () ;
   char *padbuf ;
   static char namfit [80] ;			/* FITS file name. */
   static char outbuf [84] ;
   static char pixcom [48] ;
   static char eq [4] = "= " ;
   static char diraxis1  [12] = "RIGHT" ;	/* Direction of axis 1: right.*/
   static char diraxis2  [12] = "UP" ;		/* Direction of axis 2: up. */

   static char kbitpix   [12] = "BITPIX  " ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;

   static char kangnorth [12] = "ANGNORTH" ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kcalmir   [12] = "CALMIR  " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcolorfil [12] = "COLORFIL" ;
   static char kcomment  [12] = "COMMENT " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kcptilt   [12] = "CPTILT  " ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kdataform [12] = "DATAFORM" ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdiraxis1 [12] = "DIRAXIS1" ;
   static char kdiraxis2 [12] = "DIRAXIS2" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kend      [12] = "END     " ;
   static char kframe    [12] = "FRAME   " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kexpunits [12] = "EXPUNITS" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char kglfilter [12] = "GLFILTER" ;
   static char khistory  [12] = "HISTORY " ;
   static char kinstrume [12] = "INSTRUME" ;
   static char korigin   [12] = "ORIGIN  " ;
   static char kobject   [12] = "OBJECT  " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char kpolaroid [12] = "POLAROID" ;
   static char kquantity [12] = "QUANTITY" ;
   static char krsun     [12] = "RSUN    " ;
   static char kscroll   [12] = "SCROLL  " ;
   static char ksector   [12] = "SECTOR  " ;
   static char ktelescop [12] = "TELESCOP" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktimesys  [12] = "TIMESYS " ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char kversion  [12] = "VERSION " ;

   static char atype1   [12] = "X" ;
   static char atype2   [12] = "Y" ;
   static char bunit    [24] ;
   static char bunitcom [48]  = "MAGNITUDE UNITS";
   static char calmir   [12] ;
   static char colorfil [12] ;
   static char ctype1   [12] ;
   static char ctype2   [12] ;
   static char ctypenam [16] = "HELIOCENTRIC" ;
   static char cunit1   [24] ;			/* Coord units axis 1. */
   static char cunit2   [24] ;			/* Coord units axis 2. */
   static char date     [12] ;
   static char date_obs [12] ;
   static char datcom   [24] ;			/* data scale comment. */
   static char dform    [24] ;
   static char glfilter [12] ;
   static char instrume [12] = "VIDICON" ;
   static char object   [12] = "CORONA" ;
   static char origin   [12] = "HAO" ;
   static char polaroid [12] ;
   static char refcom   [12] = "SUN CENTER" ;
   static char sector   [12] ;
   static char telescop [12] = "SMM C/P" ; 
   static char time     [12] ;
   static char time_obs [12] ;
   static char timesys  [ 8] = "UTC" ;
   static char type_obs [16] = "WHITE LIGHT" ;
   static char namsrc   [4] = {'u', '1', '3', '2'} ;	/* Polaroid Filter. */
   static char namwave  [4] ;
   static char namfw    [6] = {'r', 'a', 'o', 'b', 'w', 'g'};
   static char namfil [6][12] = {"RED", "H-ALPHA", "OPEN", "BLUE", "WIDEBAND",
				 "GREEN"} ;
   static char namcm  [4][8]  = {"MOVE", "OUT", "IN", "BAD"} ;
   static char namgls [4][8]  = {"MOVE", "FE-XIV", "STOW", "FAIL"} ;
   static char namobj [4][8]  = {"UNKNOWN", "CORONA", "CALIB", "UNKNOWN"};
   static char nampol [4][8]  = {"CLEAR", "+60 DEG", "-60 DEG", "  0 DEG"} ;
/*   static char nampol [4][8]  = {"P0", "P1", "P3", "P2"} ; */
   static char namsec [9][3] = {"SW","S", "SE","E", "NE","N", "NW","W", "4Q"} ;
   static char namsmf [9][3] = {"sw","ss","se","ee","ne","nn","nw","ww","4q"} ;
   static char namtype  [ 4] = {'v', 't', 'c', '\0'} ;
   static char namredlo [16] = {'r', 'e', 'i', 'n', 'd', 'k', 'h', 'g', 'f', 'p', 's', 't', 'r', 'b', 'q', 'm'} ;
   static char namredhi [16] = {'R', 'E', 'I', 'N', 'D', 'K', 'H', 'G', 'F', 'P', 'S', 'T', 'R', 'B', 'Q', 'M'} ;
   static char namcor   [ 8];
   static char version  [24] ;			/* SMM C/P version. */
   static char reduce   [16] ;

   unsigned char *cpbuf ;		/* SMM C/P file  array pointer. */
   unsigned char *imbuf ;		/* SMM C/P image array pointer. */
   unsigned char *cbuf ;
   short *sbuf ;
   float *fbuf ;

   char  cmin, cmax ;			/* min/max values. */
   short smin, smax ;			/* min/max values. */
   float fmin, fmax ;			/* min/max values. */

   int bitpix ;				/* # bits per pixel. */
   int bphk = 8 ;			/* bits/pixel for hskp & bitmap. */
   int gcount = 1 ;			/* Extension gcount. */
   int iday, imonth, iyear ;		/* date. */
   int i, ix, iy ;			/* indices. */
   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* Integer value. */
   int iynum ;				/* # lines in FITS header. */
   int len ;				/* String length. */
   int nbypx ;				/* # bytes per pixel. */
   int ndim = 2 ;			/* # dimensions for image (2=XY). */
   int nhdu ;				/* # HDU blocks to write. */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyext ;				/* # bytes in extension (hk + bitmap) */
   int nbyhk ;				/* # bytes in hskp & bitmap records. */
   int nbywrt = 0 ;			/* # bytes in dataset. */
   int nrechd = 2 ;			/* # records for hskp & bitmap. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int nrec ;				/* # fits records for image. */
   int nwrite ;				/* # bytes written to disk. */
   int pcount = 0 ;			/* Extension pcount. */
   int swrite ;				/* # bytes written to disk. */

   float angnorth ;			/* angle of solar north. */
   float bscale = 1.0 ;			/* Brightness intensity scale factor. */
   float bzero  = 0.0 ;			/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float cptilt =  43.0 ;		/* Axis tilt for S/C roll = 0. */
   float crpix1, crpix2 ;		/* Reference pixel location.*/
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin = 0.0 ;			/* Data minimum value. */
   float dmax = 0.0 ;			/* Data maximum value. */
   float expdur ;			/* Exposure duration (seconds). */
   float htilt  =  43.0 ;		/* Horiz.Axis tilt including S/C roll.*/
   float secrs  = 960.0 ;		/* Arcseconds / solar radius. */
   float vtilt  =  43.0 ;		/* Vert. Axis tilt including S/C roll.*/
   float wmin = 0.0 ;			/* Display min value. */
   float wmax = 0.0 ;			/* Display max value. */

   FILE *FF ;				/* FITS file ID. */

   nwrite = 0 ;
   swrite = 0 ;


   /*** Read SMM C/P image from disk into memory array. ***/

   cpbuf = rdfimg (namimg) ;		/* Read C/P image into memory. */

   cphskp3 (namimg) ;			/* Decode ALL HK items. */

   if (nbypix == 4)
      nrechd = 1 ;
   else
      nrechd = 2 ;

   /*** Create output FITS file name. ***/

   if (H.year < 1900)
      iyear = 1900 + H.year ;
   else
      iyear = H.year ;

   /*** Set namwave field to "i" if FW=open and GL is in optical path. ***/

   strcpy (namwave, &namfw [H.ifw]);
   if (H.ifw == 2 && H.iglstw == 1)
      {
      strcpy (namwave, "i");
      }
   printf ("H.ifw: %d  namwave: %s\n", H.ifw, namwave) ;

   /*** Create the "corrections" string. ***/

   sprintf (namcor, "%s", "xxxx");
   if (H.cc  == 1)  namcor [0] = 'c';
   if (H.str == 1)  namcor [1] = 's';
   if (H.vig == 1)  namcor [2] = 'v';
   if (H.f   == 1)  namcor [3] = 'f';

   /*
   if (CPH.lcc  == 0)  namcor [0] = 'c';
   if (CPH.lstr == 0)  namcor [1] = 's';
   if (CPH.lvig == 0)  namcor [2] = 'v';
   if (CPH.lfc  == 0)  namcor [3] = 'f';
    */

   if (H.ires == 1)		/* Low Resolution. */
      {
      for (i = 0; i < 4; i++)  reduce [i] = namredlo [i] ;
      if (H.cc == 1 & H.str == 1 && H.vig == 1)  reduce [2] = 'b' ;
      if (H.f == 1)  reduce [2] = 'k' ;
      }

   if (H.ires == 0)		/* High Resolution. */
      {
      for (i = 0; i < 4; i++)  reduce [i] = namredhi [i] ;
      if (H.cc == 1 & H.str == 1 && H.vig == 1)  reduce [2] = 'B' ;
      if (H.f == 1)  reduce [2] = 'K' ;
      }

   if (idtype > 0 && idtype < 4)
       {

      /*
       *  sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%s.fts",
       *	       iyear, H.month, H.day, H.hour, H.minute, H.second,
       *	       namtype [H.idtype-1], &namsmf [H.ism][0]) ;
       */

/*
   if (H.ires == 1)
         sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.%s.fts",
        	       iyear, H.month, H.day, H.hour, H.minute, H.second,
        	       namwave [0], namsrc [H.ipw], namredlo [H.idtype-1],
		       &namsmf [H.ism][0], namcor);

   if (H.ires == 0)
         sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.%s.fts",
        	       iyear, H.month, H.day, H.hour, H.minute, H.second,
        	       namwave [0], namsrc [H.ipw], namredhi [H.idtype-1],
		       &namsmf [H.ism][0], namcor);
 */

   sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.%s.fts",
            iyear, H.month, H.day, H.hour, H.minute, H.second,
  	    namwave [0], namsrc [H.ipw], reduce [H.idtype-1],
            &namsmf [H.ism][0], namcor);

/*
         sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.%s.fts",
        	       iyear, H.month, H.day, H.hour, H.minute, H.second,
        	       namwave [0], namsrc [H.ipw], reduce [0], &namsmf [H.ism][0], namcor);
 */



       /*
        * if (CPH.lvig == 0)
        *  sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.vig.fts",
	*           iyear, H.month, H.day, H.hour, H.minute, H.second,
	*           namwave [0], namsrc [0], namred [H.idtype-1], &namsmf [H.ism][0]);
        * else
        *   sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.fts",
	*           iyear, H.month, H.day, H.hour, H.minute, H.second,
	*           namwave [0], namsrc [0], namred [H.idtype-1], &namsmf [H.ism][0]);
	*/
      }
   else
      sprintf (namfit, "smmcp.%s.fts", namimg) ;

   printf ("H.idtype: %d  namred: %c\n", H.idtype, namredlo [H.idtype-1]);

   fprintf (stdout,
	    "\n------- cptofits> %s -> %s -------\n\n", namimg, namfit) ;
   fprintf (stdout, "--- PRIMARY   HDU ---\n") ;

   nbyhk = npix ;				/* Size of hskp data. */
   nbyext = npixl * nbypix * nrechd ;		/* Size of hk + bitmap. */
   imbuf = cpbuf + npixl * nbypix * nrechd ;	/* Start of pixel data. */
   for (i = 224; i < npix; i++) *(cpbuf + i) = 0 ; /* Clear unused part of HK */

   nx = ny = npixl ;			/* # pixels for X & Y axes. */
   nbypx = nbypix ;			/* # bytes per pixel. */
   npximg = nx * ny ;		/* # pixels in image. */
   nbyimg = npximg * nbypix ;	/* # bytes in image. */

   /*** Set object field. ***/

   strcpy (object, &namobj [H.icm][0]) ;

   /*** Determine month & day-of-month. ***/

   iyear = H.year ;
/*   if (H.year > 1900)  iyear = H.year - 1900 ; */
   if (H.year < 1900)  iyear = H.year + 1900 ;
   month_day (H.year, H.doy, &imonth, &iday) ;

   /*** Format observation date & time parameters. ***/

/*   sprintf (date_obs, "%02d/%02d/%02d", iday, imonth, iyear) ; */
   sprintf (date_obs, "%04d-%02d-%02d", iyear, imonth, iday) ;
   sprintf (time_obs, "%02d:%02d:%02d", H.hour, H.minute, H.second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   sprintf (date, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (time, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   /*** Set solar north angle. ***/

   htilt    = - (cptilt + scroll) ;	/* Angle of north w.r.t. +Y axis. */
   vtilt    = - (cptilt + scroll) ;	/* Angle of north w.r.t. +Y axis. */
   angnorth = - (cptilt + scroll) ;	/* Angle of north w.r.t. +Y axis. */

   /*** Coordinate system. ***/

   sprintf (ctype1, "%s", atype1) ;
   sprintf (ctype2, "%s", atype2) ;
   crpix1 = H.xcen ;				/* X sun center: pixels. */
   crpix2 = (float) 447*(2-H.ires) - H.ycen ;	/* Y sun center: pixels. */
   crval1 = crval2 = 0.0 ;			/* Heliocentric sun center. */
   cdelt1 = cdelt2 = 1.0 / H.pixrs * H.rsun ;	/* Arcseconds / pixel. */
   strcpy (cunit1, "ARCSECONDS") ;
   strcpy (cunit2, "ARCSECONDS") ;
   sprintf (version, "SMM C/P 3.04") ;

   /*** Set brightness unit. ***/
   /*** Set data form. ***/
   /*** Set display intensity min/max. ***/
   /*** Set bscale & bzero.  ***/

   switch (H.idtype)
      {
      case 1:	strcpy (bunit, "DN") ;
		strcpy (bunitcom, "Raw vidicon Digital Number") ;
		strcpy (dform, "Raw Video") ;
		wmin = 0.0 ;	wmax = 255.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 2:	strcpy (bunit, "1.024 Sec") ;
		strcpy (bunitcom, "Equivalent Seconds Exposure") ;
		strcpy (dform, "Equiv.Sec") ;
		wmin = 0.0 ;	wmax = 30.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 3:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B-Total") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 4:	strcpy (bunit, "Nelec/cm2") ;
		strcpy (bunitcom, "# Electrons per square centimeter") ;
		strcpy (dform, "ELEC DEN") ;
		wmin = 0.0 ;	wmax = 5.0e+15 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 5:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "BRT DIFF") ;
		wmin = -2.0e-9 ;wmax = 2.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 6:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B [K]") ;
		wmin = 0.0 ;	wmax = 2.5e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 7:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B2-B1/BK") ;
		wmin = -1.0 ;	wmax = 1.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 8:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B2-B1/B1") ;
		wmin = -1.0 ;	wmax = 1.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 9:	strcpy (bunit, "Fraction") ;
		strcpy (bunitcom, "Polarization Fraction") ;
		strcpy (dform, "Pol.Frac") ;
		wmin =  0.0 ;	wmax = 1.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 10:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Pol.Fraction * Brightness") ;
		strcpy (dform, "Pol.Brt.") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 11:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "Btan+Brad") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 12:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B [tan] ") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 13:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B [rad] ") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 14:	strcpy (bunit, "Bsun") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "BF*V + S") ;
		wmin = 0.0 ;	wmax = 2.5e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 15:	strcpy (bunit, "Ratio") ;
		strcpy (bunitcom, "Brightness relative to solar disk") ;
		strcpy (dform, "B1/B2") ;
		wmin = 0.0 ;	wmax = 2.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 16:
		strcpy (bunit, "Mass") ;
		strcpy (bunitcom, "Mass (grams)") ;
		strcpy (dform, "Mass") ;
		wmin = 0.0 ;    wmax = 1.0e+13 ;
		bzero = 0.0 ;   bscale = 1.0 ;
		break ;
      default:	strcpy (bunit, "B") ;
		strcpy (bunitcom, "Brightness") ;
		strcpy (dform, "B") ;
		wmin = 0.0 ;	wmax = 2.0e-9 ;
		if (CPH.lcc ==1) wmax *= 2.0 ;  if (CPH.lvig == 1) wmax *= 3.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
      }
   sprintf (datcom, "%s", bunit) ;
/*   squeeze (datcom, (int) 047) ; */		/* Eliminate "'" from string. */
/*   squeeze (datcom, (int) 040) ; */		/* Eliminate " " from string. */

   /*** Mechanism status names. ***/

   sprintf (polaroid, "%s", &nampol [H.ipw][0]) ;
   sprintf (calmir,   "%s", &namcm  [H.icm][0]) ;
   sprintf (sector,   "%s", &namsec [H.ism][0]) ;
   sprintf (colorfil, "%s", &namfil [H.ifw][0]) ;

   if (H.iglstw == 1)				/* Active Region. */
      sprintf (glfilter, "%s:%03d", &namgls [H.iglstw][0], H.iglpos) ;
   else						/* Move, Stow or Fail. */
      sprintf (glfilter, "%s", &namgls [H.iglstw][0]) ;

   /*** Set number of bits per pixel. ***/

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;

   switch (nbypx)
      {
      case 1:	bitpix =   8 ;			/* byte image. */
		strcpy (pixcom, "1 BYTE [8 BIT] UNSIGNED INTEGER PIXELS") ;
		cbuf = imbuf ;
		cmin = 255 ;
		cmax =   0 ;
                for (i = 0 ; i < npximg; i++)
                   {
                   if (*(cbuf + i) < cmin)  cmin = *(cbuf + i) ;
                   if (*(cbuf + i) > cmax)  cmax = *(cbuf + i) ;
                   }
                dmin = (float) cmin * bscale + bzero ;
                dmax = (float) cmax * bscale + bzero ;
		fprintf (stdout, "cmin/cmax: %3d %3d\n", cmin, cmax) ;
		break ;

      case 2:	bitpix =  16 ;			/* short integer image. */
		strcpy (pixcom, "2-BYTE SIGNED PIXELS.") ;
		sbuf = (short *) imbuf ;
		smin =  32767 ;
		smax = -32767 ;
                for (i = 0 ; i < npximg; i++)
                   {
                   if (*(sbuf + i) < smin)  smin = *(sbuf + i) ;
                   if (*(sbuf + i) > smax)  smax = *(sbuf + i) ;
                   }
                dmin = (float) smin * bscale + bzero ;
                dmax = (float) smax * bscale + bzero ;
		fprintf (stdout, "smin/smax: %4d %4d\n", smin, smax) ;
		break ;

      case 4:	bitpix = -32 ;			/* floating point image. */
		strcpy (pixcom, "4-BYTE FLOATING POINT PIXELS.") ;
		fbuf = (float *) imbuf ;
		fmin =  1.e+30 ;
		fmax = -1.e+30 ;
                for (i = 0 ; i < npximg; i++)
                   {
                   if (*(fbuf + i) < fmin)  fmin = *(fbuf + i) ;
                   if (*(fbuf + i) > fmax)  fmax = *(fbuf + i) ;
                   }
                dmin = (float) fmin * bscale + bzero ;
                dmax = (float) fmax * bscale + bzero ;
		fprintf (stdout, "fmin/fmax: %8.2g %8.2g\n", fmin, fmax) ;
		break ;

      default:	fprintf (stderr, "Invalid nbypx: %d\n", nbypx) ;  exit (-1) ;
      }

   fprintf (stdout, "nx/ny: %3d %3d   npximg: %6d  dmin/dmax: %8.2g   %8.2g\n",
	    nx, ny, npximg, dmin, dmax) ;

   /*** Create FITS file. ***/

   if ((FF = fopen (namfit, "w")) == NULL)
      {
      fprintf (stderr, "Unable to create file: %s\n", namfit) ;
      exit (-2) ;
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   strcpy  (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: SMM C/P IMAGE") ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                         knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                         knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                            knaxis2, eq, ny, atype2) ;
   else
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;

   /*** Extension information. ***/

      strcpy  (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  kobject, eq, object) ;
   strcpy  (&fitbuf [iy++][31], "/ FIELD-OF-VIEW: APPROX. 1.5 - 5.5 SOLAR RADII") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  ktype_obs, eq, type_obs) ;
   strcpy  (&fitbuf [iy++][31], "/ SEE 'COLORFIL' FOR SPECTRAL FILTER INFORMATION") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  korigin, eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ HIGH ALTITUDE OBSERVATORY (HAO)") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR MAXIMUM MISSION CORONAGRAPH/POLARIMETER") ;
   strcpy  (&fitbuf [iy  ][0],  "INSTRUME= 'VIDICON'") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  kinstrume, eq, instrume) ;
   strcpy  (&fitbuf [iy++][31], "/ WESTINGHOUSE SEC VIDICON: 896x896 SQUARE PIXELS") ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: YEAR-MONTH-DAY [DOY %3d %04d]",
				 H.doy, iyear) ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HOUR/MINUTE/SECOND") ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", ktimesys, eq, timesys) ;
   sprintf (&fitbuf [iy++][31], "/ DATE/TIME SYSTEM: UNIVERSAL COORDINATED TIME") ;

   /*** Coordinate system. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s", atype1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [PIXELS]",
				 kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [%s]",
				 kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s COORD INCREMENT [%s/PIXEL]",
				 kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   if (htilt < 0)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / C/P +X = %7.2f DEGREES CCW FROM ARRAY +X AXIS", 
				 kcrota1, eq, htilt, -htilt) ;
   else
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / C/P +X = %7.2f DEGREES CW FROM ARRAY +X AXIS", 
				 kcrota1, eq, htilt, htilt) ;

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s", atype2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [PIXELS]",
			            kcrpix2, eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [%s]",
			            kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s COORD INCREMENT [%s/PIXEL]",
			            kcdelt2, eq, cdelt2, atype2, cunit2) ;

      if (vtilt < 0)
         sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / C/P +Y = %7.2f DEGREES CCW FROM ARRAY +Y AXIS", 
				    kcrota2, eq, vtilt, -vtilt) ;
      else
         sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / C/P +Y = %7.2f DEGREES CW FROM ARRAY +Y AXIS", 
				    kcrota2, eq, vtilt, vtilt) ;
      }

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",     kbunit, eq, bunit) ;
   sprintf (&fitbuf [iy++][31], "/ %s", bunitcom) ;
   if (H.idtype == 1)
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s = DATA * BSCALE + BZERO",
			        kbscale, eq, bscale, datcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f", kbzero , eq, bzero ) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / MINIMUM DATA VALUE [%s]",
			         kdatamin , eq, dmin, datcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / MAXIMUM DATA VALUE [%s]",
			         kdatamax , eq, dmax, datcom) ;

      /*** All keywords after this point are unique to SMM C/P. ***/

      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / MIN VALUE FOR DISPLAY [%s]",
			         kdispmin , eq, wmin, datcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / MAX VALUE FOR DISPLAY [%s]",
			         kdispmax , eq, wmax, datcom) ;
      }
   else
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / %s = DATA * BSCALE + BZERO",
			        kbscale, eq, bscale, datcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e", kbzero , eq, bzero ) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MINIMUM DATA VALUE [%s]",
			         kdatamin , eq, dmin, datcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MAXIMUM DATA VALUE [%s]",
			         kdatamax , eq, dmax, datcom) ;

      /*** All keywords after this point are unique to SMM C/P. ***/

      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MIN VALUE FOR DISPLAY [%s]",
			         kdispmin , eq, wmin, datcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MAX VALUE FOR DISPLAY [%s]",
			         kdispmax , eq, wmax, datcom) ;
      }

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM NAME") ;

   /*** Coronagraph tilt with respect to the SMM spacecraft. ***/
   /*** C/P North is tilted 43 degrees CCW from SMM spacecraft axis. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s%20.3f", kcptilt, eq, cptilt) ;
   sprintf (&fitbuf [iy++][31], "/ CORONAGRAPH AXIS W.R.T. SMM SPACECRAFT [DEG]") ;

   /*** Solar Radius size (pixels). ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / SOLAR RADIUS [pixels]", 
				 kcrradius, eq, H.pixrs) ;

   /*** Solar Radius size (arcseconds). ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / SOLAR RADIUS [ARCSEC]",
			         krsun, eq, H.rsun ) ;

   /*** Display Orientation. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", kdiraxis1, eq, diraxis1) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 1 [X]") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", kdiraxis2, eq, diraxis2) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 2 [Y]") ;

   /*** Type of data values. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", kdataform, eq, dform) ;
   strcpy  (&fitbuf [iy++][31], "/ SMM C/P DATA FORM") ;

   /*** Instrument configuration. ***/

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%s'", kcolorfil,   eq, colorfil) ;
   strcpy  (&fitbuf [iy++][31], "/ COLORS: RED,H-ALPHA,OPEN,BLUE,WIDEBAND,GREEN") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%s'", kglfilter,   eq, glfilter) ;
   strcpy  (&fitbuf [iy++][31], "/ GREEN LINE FILTER POSITION") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%s'", kpolaroid,   eq, polaroid) ;
/*
 *   strcpy  (&fitbuf [iy++][31], "/ POLAROID FILTER: P0=CLR,P1=+60,P2=0,P3=-60 DEG") ;
 */
   strcpy  (&fitbuf [iy++][31], "/ POLAROID FILTER: CLEAR,+60 DEG,0 DEG,-60 DEG") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%s'", ksector,     eq, sector) ;
   strcpy  (&fitbuf [iy++][31], "/ SECTOR MIRROR POSITION: SW,S,SE,E,NE,N,NW,W,4Q") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%s'", kcalmir,     eq, calmir) ;
   strcpy  (&fitbuf [iy++][31], "/ CALIBRATION MIRROR : IN=CAL.PATH, OUT=CORONA") ;
   expdur = (float) H.exp * 1.024 ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20.3f", kexpdur,   eq, expdur) ;
   strcpy  (&fitbuf [iy++][31], "/ EXPOSURE DURATION [SECONDS]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d", kscroll,   eq, H.roll) ;
   strcpy  (&fitbuf [iy++][31], "/ SPACECRAFT ROLL [DEGREES]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d", kframe,    eq, H.frame) ;
   strcpy  (&fitbuf [iy++][31], "/ IMAGE FRAME COUNTER [0 -> 4095]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d", kquantity, eq, H.quan) ;
   strcpy  (&fitbuf [iy++][31], "/ IMAGE QUANTITY [% captured]") ;

   /*** FITS Conversion: software version. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%s'", kversion,  eq, version) ;
   strcpy  (&fitbuf [iy++][31], "/ SMM C/P FITS SOFTWARE VERSION") ;

/*
 *   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
 */
   sprintf (&fitbuf [iy++][0] , "COMMENT   The High Altitude Observatory (HAO) is a division of the National") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   Center for Atmospheric Research (NCAR), located in Boulder, Colorado.") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   The HAO Coronagraph/Polarimeter (C/P) telescope operated aboard the") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   NASA Solar Maximum Mission (SMM) satellite, which orbited the Earth") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   at an altitude of ~500 km from 14 Feb 1980 until 2 December 1989.") ; 
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   The HAO/SMM C/P telescope is externally occulted and uses a") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   Westinghouse SEC vidicon detector with 896x896 8-bit square pixels") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   (high resolution).  A full-view, low resolution mode yields a") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   448x448 image by performing a 2x2 pixel average.                       ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   A calibration mirror is used to view either a calibration reticle") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   target or the solar corona.  Image origin: lower left corner.") ;
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   The sector mirror reflects approximately one quadrant of the corona") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   at a time onto the vidicon.  Sector positions are labeled N, NE, E,") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   SE, S, SW, W, NW, and indicate the coronal region contained in a") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   quadrant for the nominal spacecraft roll orientation (0 degrees).") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   A positive roll moves the corona counter-clockwise (CCW).") ;
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   Spectral filter wheel positions are: Red, Green, Blue, Wideband,") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   H-alpha, and Open (which is occasionally used with a separate,") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   tiltable Green-Line [530.3 nm = FE-XIV] filter).") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   Polaroid wheel positions: Clear, +60, 0, -60 degrees.") ;

   /*** Terminate FITS HDU. ***/

   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   iynum = iy ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
/*
 *	 ival = (int) fitbuf [iy][ix] ;
 *	 fitbuf [iy][ix] = (char) toupper (ival) ;
 */
	 }

   fprintf (stdout, "iynum: %d  nhdu: %d\n", iynum, nhdu) ;

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write FITS header to disk file. ***/

   nwrite = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;
   swrite += nwrite ;
   fprintf (stdout, "header swrite: %d\n", swrite) ;

   nwrite = 0 ;

   /*** Write pixel data. ***/
   /*** Reverse image top-to-bottom. ***/

   for (iy = 0 ; iy < ny ; iy++)
      {
      nwrite  = fwrite (imbuf + (ny - iy - 1) * nx * nbypx, nbypx, nx, FF) ;
      swrite += nwrite ;
      }
   nbywrt = nx * ny * nbypx ;

   fprintf (stdout, "nx/ny: %3d %3d  nbypx: %d  nbyimg: %6d\n",
	    nx, ny, nbypx, nbyimg) ;
   fprintf (stdout, "data swrite: %d\n", swrite) ;

   /*** If data size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbywrt / nbyfits ;
   if ((nrec * nbyfits) < nbywrt)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbywrt ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nwrite  = fwrite (padbuf, 1, npad, FF) ;
      swrite += nwrite ;
      free (padbuf) ;
      }

   fprintf (stdout, "pad  swrite: %d\n", swrite) ;

   cpext (FF, cpbuf, nrechd, pmode) ;		/* Write extension. */

   free (cpbuf) ;
   fclose (FF) ;
   return (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

cpext (FF, cpbuf, nrechd, pmode)

FILE *FF ;
unsigned char *cpbuf ;
int nrechd ;
int pmode ;

{
   extern char *malloc () ;
   char *padbuf ;

   static char outbuf    [84] ;
   static char eq        [4] = "= " ;
   static char pixcom    [48] = "BITS/PIXEL" ;
   static char atype1    [12] = "X" ;
   static char atype2    [12] = "Y" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char kpcount   [12] = "PCOUNT  " ;

   static char kbitpix   [12] = "BITPIX  " ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;

   static char kpwpos    [12] = "PWPOS" ;
   static char khvstat   [12] = "HVSTAT" ;
   static char ksmmid    [12] = "SMMID" ;
   static char kresolut  [12] = "RESOLUT" ;
   static char kdatamode [12] = "DATAMODE" ;
   static char khkfrozen [12] = "HKFROZEN" ;
   static char ksegcnt   [12] = "SEGCNT" ;
   static char kexpinh   [12] = "EXPINH" ;
   static char kreadinh  [12] = "READINH" ;
   static char kprexpinh [12] = "PREXPINH" ;
   static char kmet      [12] = "MET" ;
   static char kih       [12] = "IH" ;
   static char kiv       [12] = "IV" ;
   static char kexpose   [12] = "EXPOSE" ;
   static char kglpos    [12] = "GLPOS" ;
   static char kglstow   [12] = "GLSTOW" ;
   static char ksmpos    [12] = "SMPOS" ;
   static char ksmmrollr [12] = "SMMROLLR" ;
   static char kispos    [12] = "ISPOS" ;
   static char kadpos    [12] = "ADPOS" ;
   static char kcmpos    [12] = "CMPOS" ;
   static char kfwpos    [12] = "FWPOS" ;

   static char kpes1a    [12] = "PES1A" ;
   static char kpes2a    [12] = "PES2A" ;
   static char kpes3a    [12] = "PES3A" ;
   static char kpes4a    [12] = "PES4A" ;

   static char khour     [12] = "HOUR" ;
   static char kminute   [12] = "MINUTE" ;
   static char kpcsyoff  [12] = "PCSYOFF" ;
   static char kpcszoff  [12] = "PCSZOFF" ;
   static char ksecond   [12] = "SECOND" ;
   static char kmetconv  [12] = "METCONV" ;
   static char kpcsth    [12] = "PCSTH" ;
   static char kdasth    [12] = "DASTH" ;
   static char kpcsovr   [12] = "PCSOVR" ;
   static char kdasovr   [12] = "DASOVR" ;
   static char kpcsthexc [12] = "PCSTHEXC" ;
   static char kdasthexc [12] = "DASTHEXC" ;
   static char kstat8v   [12] = "STAT8V" ;
   static char kpcsen    [12] = "PCSEN" ;
   static char ksecerase [12] = "SECERASE" ;
   static char ksecexpos [12] = "SECEXPOS" ;
   static char ksecread  [12] = "SECREAD" ;
   static char kimsizovr [12] = "IMSIZOVR" ;
   static char kflood    [12] = "FLOOD" ;
   static char kmebseq   [12] = "MEBSEQ" ;
   static char kisredun  [12] = "ISREDUN" ;
   static char kadredun  [12] = "ADREDUN" ;
   static char kcmredun  [12] = "CMREDUN" ;
   static char kglredun  [12] = "GLREDUN" ;
   static char kflpower  [12] = "FLPOWER" ;
   static char kimgcnt   [12] = "IMGCNT" ;
   static char kpes1b    [12] = "PES1B" ;
   static char kpes2b    [12] = "PES2B" ;
   static char kpes3b    [12] = "PES3B" ;
   static char kpes4b    [12] = "PES4B" ;
   static char krefvolid [12] = "REFVOLID" ;
   static char krefdsid  [12] = "REFDSID" ;
   static char kpes1c    [12] = "PES1C" ;
   static char kpes2c    [12] = "PES2C" ;
   static char kpes3c    [12] = "PES3C" ;
   static char kpes4c    [12] = "PES4C" ;
   static char kbytorder [12] = "BYTORDER" ;
   static char kscbvp15  [12] = "SCBVP15" ;
   static char kscbvm15  [12] = "SCBVM15" ;
   static char krefyear  [12] = "REFYEAR" ;
   static char kscbvp10  [12] = "SCBVP10" ;
   static char kscbvp12  [12] = "SCBVP12" ;
   static char kpcbvp15  [12] = "PCBVP15" ;
   static char kpcbvm15  [12] = "PCBVM15" ;
   static char kscbvp22  [12] = "SCBVP22" ;
   static char kscbvm22  [12] = "SCBVM22" ;
   static char krefdoy   [12] = "REFDOY" ;
   static char kpes1d    [12] = "PES1D" ;
   static char kpes2d    [12] = "PES2D" ;
   static char kpes3d    [12] = "PES3D" ;
   static char kpes4d    [12] = "PES4D" ;
   static char kpcsypos  [12] = "PCSYPOS" ;
   static char kpcszpos  [12] = "PCSZPOS" ;
   static char ksrc_cpu  [12] = "SRC_CPU" ;
   static char kmcbvp5   [12] = "MCBVP5" ;
   static char kdasypos  [12] = "DASYPOS" ;
   static char kdaszpos  [12] = "DASZPOS" ;
   static char kmebvp15  [12] = "MEBVP15" ;
   static char kmebvm15  [12] = "MEBVM15" ;
   static char kmebvp12  [12] = "MEBVP12" ;
   static char kmebvp5   [12] = "MEBVP5" ;
   static char kmebvp64  [12] = "MEBVP64" ;
   static char kimgquan  [12] = "IMGQUAN" ;
   static char kpes1e    [12] = "PES1E" ;
   static char kpes2e    [12] = "PES2E" ;
   static char kpes3e    [12] = "PES3E" ;
   static char kpes4e    [12] = "PES4E" ;
   static char ksdibvp15 [12] = "SDIBVP15" ;
   static char ksdibvm15 [12] = "SDIBVM15" ;
   static char ksdibvp5  [12] = "SDIBVP5" ;
   static char ksectemp  [12] = "SECTEMP" ;
   static char ksmmrolld [12] = "SMMROLLD" ;
   static char kvpbvp64  [12] = "VPBVP64" ;
   static char kvpbvp26  [12] = "VPBVP26" ;
   static char kvpbvp20  [12] = "VPBVP20" ;
   static char kvpbvm10  [12] = "VPBVM10" ;
   static char kfilvolt  [12] = "FILVOLT" ;
   static char kfilcur   [12] = "FILCUR" ;
   static char kpes1f    [12] = "PES1F" ;
   static char kpes2f    [12] = "PES2F" ;
   static char kpes3f    [12] = "PES3F" ;
   static char kpes4f    [12] = "PES4F" ;
   static char kpcathv   [12] = "PCATHV" ;
   static char kg1v      [12] = "G1V" ;
   static char kg2v      [12] = "G2V" ;
   static char knbitpix  [12] = "NBITPIX" ;
   static char kcathvh   [12] = "CATHVH" ;
   static char kcathvl   [12] = "CATHVL" ;
   static char kg3vh     [12] = "G3VH" ;
   static char kg3vl     [12] = "G3VL" ;
   static char kg4vh     [12] = "G4VH" ;
   static char kg4vl     [12] = "G4VL" ;
   static char knpixrow  [12] = "NPIXROW" ;
   static char kpes1g    [12] = "PES1G" ;
   static char kpes2g    [12] = "PES2G" ;
   static char kpes3g    [12] = "PES3G" ;
   static char kpes4g    [12] = "PES4G" ;
   static char ksecalcv  [12] = "SECALCV" ;
   static char ksecalch  [12] = "SECALCH" ;
   static char ksecalv   [12] = "SECALV" ;
   static char kfileid   [12] = "FILEID" ;
   static char kcalvolid [12] = "CALVOLID" ;
   static char kcaldsid  [12] = "CALDSID" ;
   static char kcalyear  [12] = "CALYEAR" ;
   static char kcaldoy   [12] = "CALDOY" ;
   static char knamea0   [12] = "NAMEA0" ;
   static char knamea1   [12] = "NAMEA1" ;
   static char knamea2   [12] = "NAMEA2" ;
   static char knamea3   [12] = "NAMEA3" ;
   static char knamecor  [12] = "NAMECOR" ;
   static char knamestr  [12] = "NAMESTR" ;
   static char knamevig  [12] = "NAMEVIG" ;
   static char kfwmag    [12] = "FWMAG" ;
   static char kpwcorx   [12] = "PWCORX" ;
   static char kpwcory   [12] = "PWCORY" ;
   static char ksmcorx   [12] = "SMCORX" ;
   static char ksmcory   [12] = "SMCORY" ;
   static char kfwcorx   [12] = "FWCORX" ;
   static char kfwcory   [12] = "FWCORY" ;
   static char kcxcen    [12] = "CXCEN" ;
   static char kcycen    [12] = "CYCEN" ;
   static char kdxcen    [12] = "DXCEN" ;
   static char kdycen    [12] = "DYCEN" ;
   static char ksxcen    [12] = "SXCEN" ;
   static char ksycen    [12] = "SYCEN" ;
   static char kescorr   [12] = "ESCORR" ;
   static char kradfac   [12] = "RADFAC" ;
   static char kcalexp   [12] = "CALEXP" ;
   static char kdatatype [12] = "DATATYPE" ;
   static char klcc      [12] = "LCC" ;
   static char klfc      [12] = "LFC" ;
   static char klreg     [12] = "LREG" ;
   static char klvig     [12] = "LVIG" ;
   static char klstr     [12] = "LSTR" ;

   int bitpix = 8 ;
   int gcount = 1 ;                     /* Extension gcount. */
   int i ;
   int ix, iy ;
   int ival ;
   int iynum ;
   int nx, ny ;
   int nbyext ;                         /* # bytes in extension (hk + bitmap) */
   int nbypx ;
   int nbyimg ;
   int nbywrt ;
   int ndim = 2 ;
   int nhdu ;
   int npad ;
   int npximg ;
   int nrec ;
   int nwrite ;
   int pcount = 0 ;                     /* Extension pcount. */
   int swrite ;

   nx = npix ;
   ny = 2 ;			/* 1st record is hskp, 2nd is bitmap. */
   nbypx = 1 ;			/* # bytes per element. */
   nbyimg  = nx * ny ;		/* # bytes in "image". */
   npximg  = 0 ;
   nbyext = npixl * nbypix * nrechd ;           /* Size of hk + bitmap. */

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   fprintf (stdout, "--- EXTENSION HDU ---\n") ;

   iy = 0 ;

   sprintf (&fitbuf [iy++][0],  "XTENSION= 'SMMCP_HK'           / SMM C/P HOUSEKEEPING + BITMAP EXTENSION") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
	                         knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
	                         knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION [HOUSEKEEPING, BITMAP]",
	                            knaxis2, eq, ny, atype2) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
	                        kpcount, eq, pcount) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
				kgcount, eq, gcount) ;

   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
/*
   strcpy  (&fitbuf [iy++][0], "COMMENT   1ST ROW: HOUSEKEEPING DATA ;  2ND ROW: SEGMENT [112 PIXELS] BITMAP.") ;
 */

   strcpy  (&fitbuf [iy++][0], "COMMENT   Keywords & Parameters below describe the C/P Housekeepign data:") ;

   /*** C/P Housekeeping Information. ***/

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpwpos, eq, CPH.pwpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 003 BIT 7-6 POLAROID WHEEL POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", khvstat, eq, CPH.hvstat) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 003 BIT 5   HV COMMAND STATUS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksmmid, eq, CPH.smmid) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 003 BIT 4-3 SMM INSTRUMENT ID") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kresolut, eq, CPH.resolut) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 003 BIT 2   RESOLULTION [0=HI, 1=LO]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdatamode, eq, CPH.datamode) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 003 BIT 1   DATA MODE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", khkfrozen, eq, CPH.hkfrozen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 003 BIT 0   HOUSEKEEPING FROZEN") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksegcnt, eq, CPH.segcnt) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 004 BIT 7-0 BYTE 005 BIT 7-3 SEGMENT COUNT") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kexpinh, eq, CPH.expinh) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 005 BIT 2   EXPOSURE INHIBIT") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kreadinh, eq, CPH.readinh) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 005 BIT 1   READ INHIBIT") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kprexpinh, eq, CPH.prexpinh) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 005 BIT 0   PREMATURE EXPOSE INHIBIT") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmet, eq, CPH.met) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 006-008 MISSION ELAPSED TIME [~1.024 sec]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kih, eq, CPH.ih) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 009 BIT 7-0 HORIZ IMAGE SEGMENT CODE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kiv, eq, CPH.iv) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 009 BIT 7-0 VERT  IMAGE SEGMENT CODE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kexpose, eq, CPH.expose) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 010 BIT 7-0 BYTE 011 BIT 7-4 EXPOSURE DUR.") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksmmrollr, eq, CPH.smmrollr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 011 BIT 3-0 SMM ROLL [0.1119 DEG. 1980]") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 012 BIT 7-6 SMM ROLL [7.1620 DEG. 1984-89]") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 012 BIT 5-0 SPACECRAFT ROLL    [1980]");
   strcpy  (&fitbuf [iy++][31], "/ BYTE 012 BIT 5-0 6 HIGHEST MET BITS [1984-89]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kglpos, eq, CPH.glpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 013 BIT 7-0 5303 FILTER POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kglstow, eq, CPH.glstow) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 014 BIT 7-6 5303 FILTER STOW") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksmpos, eq, CPH.smpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 014 BIT 5-2 SECTOR MIRROR      POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kispos, eq, CPH.ispos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 014 BIT 1-0 INTERNAL SHUTTER   POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kadpos, eq, CPH.adpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 015 BIT 7-6 APERTURE DOOR      POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcmpos, eq, CPH.cmpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 015 BIT 5-4 CALIBRATION MIRROR POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfwpos, eq, CPH.fwpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 015 BIT 3-1 FILTER WHEEL       POSITION") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 015 BIT 0   NOT USED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1a, eq, CPH.pes1a) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 016 BIT 7-0 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2a, eq, CPH.pes2a) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 017 BIT 7-0 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3a, eq, CPH.pes3a) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 018 BIT 7-0 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4a, eq, CPH.pes4a) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 019 BIT 7-0 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", khour, eq, CPH.hour) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 020 BIT 7-0 IMAGE HOUR") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kminute, eq, CPH.minute) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 021 BIT 7-0 IMAGE MINUTE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcsyoff, eq, CPH.pcsyoff) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 022 BIT 7-2 PCS Y OFFSET") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 022 BIT 1-0 NOT USED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcszoff, eq, CPH.pcszoff) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 023 BIT 7-2 PCS Z OFFSET") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 023 BIT 1-0 NOT USED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecond, eq, CPH.second) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 024 BIT 7-0 IMAGE SECOND") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmetconv, eq, CPH.metconv) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 025 BIT 7-0 MET -> UTC CONVERSION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcsth, eq, CPH.pcsth) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 026 BIT 7-4 PCS THRESHOLD") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdasth, eq, CPH.dasth) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 026 BIT 3-0 DAS THRESHOLD") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcsovr, eq, CPH.pcsovr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 7   PCS OVERRIDE STATUS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdasovr, eq, CPH.dasovr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 6   DAS OVERRIDE STATUS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcsthexc, eq, CPH.pcsthexc) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 5   PCS THRESHOLD EXCEEDED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdasthexc, eq, CPH.dasthexc) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 4   DAS THRESHOLD EXCEEDED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kstat8v, eq, CPH.stat8v) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 3   8-VOLT STATUS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcsen, eq, CPH.pcsen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 2   PCS ENABLE/DISABLE STATUS");
   strcpy  (&fitbuf [iy++][31], "/ BYTE 027 BIT 1-0 NOT USED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecerase, eq, CPH.secerase) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 7   SEC ERASE MODE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecexpos, eq, CPH.secexpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 6   SEC EXPOSE MODE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecread, eq, CPH.secread) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 5   SEC READ  MODE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kimsizovr, eq, CPH.imsizovr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 4   IMAGE SIZE OVERRIDE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kflood, eq, CPH.flood) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 3   FLOOD STATUS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmebseq, eq, CPH.mebseq) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 2   MEB SEQUENCER RESET") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 028 BIT 1-0 NOT USED") ;

   strcpy  (&fitbuf [iy++][31], "/ BYTE 029 BIT 7-5 NOT USED") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kisredun, eq, CPH.isredun) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 029 BIT 4   INT. SHUTTER  REDUNDANT ENABLE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kadredun, eq, CPH.adredun) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 029 BIT 3   APERTURE DOOR REDUNDANT ENABLE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcmredun, eq, CPH.cmredun) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 029 BIT 2   CAL. MIRROR   REDUNDANT ENABLE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kglredun, eq, CPH.glredun) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 029 BIT 1   5303 MECH.    REDUNDANT ENABLE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kflpower, eq, CPH.flpower) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 029 BIT 0   FLOOD LAMP POWER") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kimgcnt, eq, CPH.imgcnt) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 030 BIT 7-0 BYTE 031 BYTE 7-4 IMAGE COUNT") ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 031 BIT 3-0 NOT USED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1b, eq, CPH.pes1b) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 032 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2b, eq, CPH.pes2b) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 033 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3b, eq, CPH.pes3b) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 034 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4b, eq, CPH.pes4b) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 035 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", krefvolid, eq, CPH.refvolid) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 036-041 REFORMATTED VOLUME  ID") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", krefdsid, eq, CPH.refdsid) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 042-047 REFORMATTED DATASET ID") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1c, eq, CPH.pes1c) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 048 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2c, eq, CPH.pes2c) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 049 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3c, eq, CPH.pes3c) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 050 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4c, eq, CPH.pes4c) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 051 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kbytorder, eq, CPH.bytorder) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 052 BYTE ORDER [0=SUN,SGI,MAC, 1=DEC,IBM]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kscbvp15, eq, CPH.scbvp15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 053 SCB +15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kscbvm15, eq, CPH.scbvm15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 054 SCB -15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", krefyear, eq, CPH.refyear) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 055 YEAR-1900 DATA REFORMATTED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kscbvp10, eq, CPH.scbvp10) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 056 SCB +10V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kscbvp12, eq, CPH.scbvp12) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 057 SCB +12V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcbvp15, eq, CPH.pcbvp15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 058 PCB +15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcbvm15, eq, CPH.pcbvm15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 059 PCB -15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kscbvp22, eq, CPH.scbvp22) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 060 SCB +22V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kscbvm22, eq, CPH.scbvm22) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 061 SCB -22V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", krefdoy, eq, CPH.refdoy) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 062-063 DAY-OF-YEAR REFORMATTED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1d, eq, CPH.pes1d) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 064 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2d, eq, CPH.pes2d) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 065 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3d, eq, CPH.pes3d) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 066 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4d, eq, CPH.pes4d) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 067 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcsypos, eq, CPH.pcsypos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 068 PCS Y POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcszpos, eq, CPH.pcszpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 069 PCS Z POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksrc_cpu, eq, CPH.src_cpu) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 070 SRC [1=PDP,2=IPD,4=CRAY,8=VAX,16=SUN]");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmcbvp5, eq, CPH.mcbvp5) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 071 MCB +5V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdasypos, eq, CPH.dasypos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 072 DAS Y POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdaszpos, eq, CPH.daszpos) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 073 DAS Z POSITION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmebvp15, eq, CPH.mebvp15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 074 MEB +15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmebvm15, eq, CPH.mebvm15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 075 MEB -15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmebvp12, eq, CPH.mebvp12) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 076 MEB +12V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmebvp5, eq, CPH.mebvp5) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 077 MEB +5V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kmebvp64, eq, CPH.mebvp64) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 078 MEB +6.4V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kimgquan, eq, CPH.imgquan) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 079 IMAGE QUANTITY CAPTURED [0-100%]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1e, eq, CPH.pes1e) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 080 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2e, eq, CPH.pes2e) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 081 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3e, eq, CPH.pes3e) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 082 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4e, eq, CPH.pes4e) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 083 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksdibvp15, eq, CPH.sdibvp15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 084 SDIB +15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksdibvm15, eq, CPH.sdibvm15) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 085 SDIB -15V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksdibvp5, eq, CPH.sdibvp5) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 086 SDIB +5V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksectemp, eq, CPH.sectemp) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 087 SEC TEMP") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksmmrolld, eq, CPH.smmrolld) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 088-089 SMM ROLL [DEGREES]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kvpbvp64, eq, CPH.vpbvp64) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 090 VPB +6.4V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kvpbvp26, eq, CPH.vpbvp26) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 091 VPB +26V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kvpbvp20, eq, CPH.vpbvp20) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 092 VPB +20V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kvpbvm10, eq, CPH.vpbvm10) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 093 VPB -10V") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfilvolt, eq, CPH.filvolt) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 094 FILAMENT VOLTS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfilcur, eq, CPH.filcur) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 095 FILAMENT CURRENT") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1f, eq, CPH.pes1f) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 096 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2f, eq, CPH.pes2f) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 097 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3f, eq, CPH.pes3f) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 098 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4f, eq, CPH.pes4f) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 099 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpcathv, eq, CPH.pcathv) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 100 PHOTOCATHODE VOLTS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kg1v, eq, CPH.g1v) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 101 G1 VOLTS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kg2v, eq, CPH.g2v) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 102 G2 VOLTS") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", knbitpix, eq, CPH.nbitpix) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 103 #BITS/PIXEL [8, 16, 32]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcathvh, eq, CPH.cathvh) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 104 CATHODE VOLTS [HIGH]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcathvl, eq, CPH.cathvl) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 105 CATHODE VOLTS [LOW]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kg3vh, eq, CPH.g3vh) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 106 G3 VOLTS [HIGH]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kg3vl, eq, CPH.g3vl) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 107 G3 VOLTS [LOW]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kg4vh, eq, CPH.g4vh) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 108 G4 VOLTS [HIGH]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kg4vl, eq, CPH.g4vl) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 109 G4 VOLTS [LOW]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", knpixrow, eq, CPH.npixrow) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 110-111 #PIXELS/LINE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes1g, eq, CPH.pes1g) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 112 POINTING ERROR SENSOR #1") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes2g, eq, CPH.pes2g) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 113 POINTING ERROR SENSOR #2") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes3g, eq, CPH.pes3g) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 114 POINTING ERROR SENSOR #3") ;
   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpes4g, eq, CPH.pes4g) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 115 POINTING ERROR SENSOR #4") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecalcv, eq, CPH.secalcv) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 116 SEC ALIGNMENT CURRENT [VERT]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecalch, eq, CPH.secalch) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 117 SEC ALIGNMENT CURRENT [HORIZ]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksecalv, eq, CPH.secalv) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 118 SEC ALIGNMENT +15v") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kfileid, eq, CPH.fileid) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 119-136 REFORMATTED FILE ID [CHAR]");

   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   strcpy  (&fitbuf [iy++][0], "COMMENT   Calibration Information:") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kcalvolid, eq, CPH.calvolid) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 137-142 CAL VOLUME  ID [CHAR]");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", kcaldsid, eq, CPH.caldsid) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 143-148 CAL DATASET ID [CHAR]");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcalyear, eq, CPH.calyear) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 149     CAL YEAR [-1900]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcaldoy, eq, CPH.caldoy) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 150-151 CAL DAY-OF-YEAR ") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamea0, eq, CPH.namea0) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 152-157 A0 ARRAY NAME [CHAR]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamea1, eq, CPH.namea1) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 158-163 A1 ARRAY NAME [CHAR]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamea2, eq, CPH.namea2) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 164-169 A2 ARRAY NAME [CHAR]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamea3, eq, CPH.namea3) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 170-175 A3 ARRAY NAME [CHAR]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamecor, eq, CPH.namecor) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 176-181 GEOMETRIC COR. NAME") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamestr, eq, CPH.namestr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 182-187 STRAY LIGHT MASK NAME");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s'%s'", knamevig, eq, CPH.namevig) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 188-193 VIGNETTING  MASK NAME");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kfwmag, eq, CPH.fwmag) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 194-195 FILT. MAG FACTOR [x10,000]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpwcorx, eq, CPH.pwcorx) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 196-197 POLAROID X SHIFT [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kpwcory, eq, CPH.pwcory) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 198-199 POLAROID Y SHIFT [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksmcorx, eq, CPH.smcorx) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 200-201 SECTOR X CORR. [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksmcory, eq, CPH.smcory) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 202-203 SECTOR Y CORR. [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcxcen, eq, CPH.cxcen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 204-205 MEASURED DISK CENTER X [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcycen, eq, CPH.cycen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 206-207 MEASURED DISK CENTER Y [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdxcen, eq, CPH.dxcen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 208-209 STANDARD DISK CENTER X [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdycen, eq, CPH.dycen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 210-211 STANDARD DISK CENTER Y [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksxcen, eq, CPH.sxcen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 212-213 STANDARD SUN CENTER X [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", ksycen, eq, CPH.sycen) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 214-215 STANDARD SUN CENTER Y [x10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kescorr, eq, CPH.escorr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 216-217 EQUIV.SEC COLOR COR.[x1.0E+4]");

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kradfac, eq, CPH.radfac) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 218-219 RADIANCE FACTOR [x1.0E+11]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kcalexp, eq, CPH.calexp) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 220     CAL. SCALE FACTOR [exponent of 10]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", kdatatype, eq, CPH.datatype) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 221-222 DATA TYPE [1-16: BIT ENCODING]") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", klcc,  eq, CPH.lcc) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 223 BIT 7 CALIBRATION CORRECTION") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", klfc, eq, CPH.lfc) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 223 BIT 3 F-CORONA REMOVED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", klreg, eq, CPH.lreg) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 223 BIT 2 DISK REGISTRATION DONE") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", klvig, eq, CPH.lvig) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 223 BIT 1 VIGNETTING  REMOVED") ;

   sprintf (&fitbuf [iy  ][0], "%-8s%2s%20d", klstr, eq, CPH.lstr) ;
   strcpy  (&fitbuf [iy++][31], "/ BYTE 223 BIT 0 STRAY LIGHT REMOVED") ;

   /*** Terminate FITS HDU. ***/

   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;

   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
/* 
 *	 ival = (int) fitbuf [iy][ix] ;
 *	 fitbuf [iy][ix] = (char) toupper (ival) ;
 */
	 }

   fprintf (stdout, "iynum: %d  nhdu: %d\n", iynum, nhdu) ;

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write FITS extension header to disk file. ***/

   swrite  = 0 ;
   nwrite  = 0 ;
   nwrite  = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;
   swrite += nwrite ;
   fprintf (stdout, "header swrite: %d\n", swrite) ;

   /*** Write extension data: housekeeping + bitmap. ***/

   /*** For raw datatype only, set datatype=1, and clear remainder of hk. */

   if (CPH.datatype == 1)		
      {
      cpbuf [221] = 1 ;			/* Set datatype to 1. */
      for (i = 222; i < nx; i++)
	 cpbuf [i] = 0 ;		/* Set remainder of hk to zero. */
      }

   nwrite  = 0 ;
   nwrite  = fwrite (cpbuf, sizeof (char), nbyext, FF) ;
   swrite += nwrite ;
   nbywrt  = nbyext * sizeof (char) ;

   fprintf (stdout, "segcnt: %4d  nx/ny: %3d %3d  nbypx: %d  nbyimg: %6d\n",
            CPH.segcnt, nx, ny, nbypx, nbyimg) ;
   fprintf (stdout, "data swrite: %d\n", swrite) ;

   /*** If data size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbywrt / nbyfits ;
   if ((nrec * nbyfits) < nbywrt)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbywrt ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nwrite  = fwrite (padbuf, 1, npad, FF) ;
      swrite += nwrite ;
      free (padbuf) ;
      }

   fprintf (stdout, "pad  swrite: %d\n", swrite) ;

   fclose (FF) ;

   return (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

month_day (year, doy, pmonth, pday)	/* Set month & day from day-of-year. */

int year, doy, *pmonth, *pday ;

{
   static int day_tab [2][13] =
      {
      {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      } ;
   int i, leap ;

   leap = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0) ;

   for (i = 1 ; doy > day_tab [leap][i] ; i++)
      doy -= day_tab [leap][i] ;

   *pmonth = i ;
   *pday = doy ;

   return (0) ;
}

