/* shiftc.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Shift calibrated images by moving measured center to a "standard" center.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 19 November 1984 : Low resolution, 6 Rsun images only.
 * 25 November 1984 : Output filename is ydddhhmms(s).dxx.
 * 4 February 1985: Remove default pathname from in/out files.
 * 31 October 1990: write bitmap only once (previously written 4 times).
 *  7 November 1990: Set disk registration bit in housekeeping byte #223.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#define NPIX 448

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
main (argc, argv)

int argc;
char *argv [];
{
   char	*outname ();	/* Function: converts input name into output name. */
   FILE	*LIST;		/* File handle for list of images. */
   int	fd;		/* File descriptor for input image. */
   static short	cpin [NPIX][NPIX];
   static short	cpout [NPIX][NPIX];
   static short	hskp [NPIX], bitmap [NPIX];
   static char	namlst [80];	/* Name of file containing image filenames.*/
   static char	namimg [80];	/* Name of input image. */
   static char	namout [80];	/* Name of output image. */
   char	*hk;

   register i, n, stat;
   int	dx, dy;		/* Delta x & y (amount of image shift. */
   int	ism;		/* Sector mirror array index. */
   int	nbypix = 2;	/* # bytes per pixel. */
   int	nbyimg;		/* # bytes per image. */
   int	np;		/* # bytes per line. */
   int	nread;		/* # bytes read from disk. */
   int	x, y;		/* coordinates for input image. */
   int	xmin, xmax;	/* minimum & maximum pixels for input image. */
   int	ymin, ymax;	/* minimum & maximum lines for input image. */
   int	xx, yy;		/* coordinates for output image. */

   /* Disk centers (vs. Sector Mirror) for calibrated data (C34520). */
   /* (Processed with geometric correction WW6COR). */

static	float	ww6cen [9][2] = { 48.0, 508.0,	/* sw */
				 216.0, 206.0,	/* s  */
				 530.0,  40.0,	/* se */
				 814.0,  86.0,	/* e  */
				 884.0, 354.0,	/* ne */
				 714.0, 672.0,	/* n  */
				 394.0, 824.0,	/* nw */
				 114.0, 786.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

   int	cx = 0;		/* Standard X-coordinate for center. */
   int	cy = 0;		/* Standard Y-coordinate for center. */

   int	cxm = 0;	/* Measured X-coordinate for center. */
   int	cym = 0;	/* Measured Y-coordinate for center. */
   int	pmin;		/* Minimum for pixel/line coordinate. */
   int	pmax;		/* Maximum for pixel/line coordinate. */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   pmin = 0;
   pmax = NPIX - 1;

   if (argc == 2)
      {
      strcpy (namlst, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [filename_list]\n", argv [0]);
      fprintf (stderr, "format of filename_list: image_name xcen ycen (Hi res coords)\n");
      exit (0);
      }

   /*** Open file containing list of images. ***/

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s\n", namlst);
      exit (-1);
      }


   /*** Read filenames: order per line is : image, x-center, y-center. ***/

   while (fscanf (LIST, "%s %d %d", namimg, &cxm, &cym) != EOF)
      {
      cxm = (int) (cxm * 0.5);
      cym = (int) (cym * 0.5);

      /*** Append directory pathname onto filenames. ***/

      np = NPIX * nbypix;
      nbyimg = NPIX * NPIX * nbypix;

      /*** Open image file. ***/

      printf ("Image: %s\n", namlst);
      if ((fd = open (namimg, 0)) < 0)
         {
         fprintf (stderr, "Cannot open %s\n", namimg);
         exit (-2);
         }

      if ((stat = read (fd, hskp, np)) != np)
         {
         if (stat == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", namimg);
         else
         if (stat > 0)
            fprintf (stderr, "read: only %d bytes read\n", stat);
         else
            perror ("read");
         exit (-3);
         }

      if ((stat = read (fd, bitmap, np)) != np)
         {
         if (stat == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", namimg);
         else
         if (stat > 0)
            fprintf (stderr, "read: only %d bytes read\n", stat);
         else
            perror ("read");
         exit (-4);
         }

      /*** Decode sector position. ***/

      hk = (char *) hskp;
      *(hk + 223) |= 4;		/* Set disk registration bit. */
      ism = (*(hk+14) & 074)>>2;
      ism = (ism-1)/2;
      cx = (int) ((ww6cen [ism][0] + 0.5) * 0.5);
      cy = (int) ((ww6cen [ism][1] + 0.5) * 0.5);
      printf ("ism: %d\n", ism);
      printf ("cx:%d cy:%d cxm:%d cym:%d\n", cx, cy, cxm, cym);

      /*** Read image into memory array 'cpin'. ***/

      nread = read (fd, cpin, nbyimg);

      xmin = pmin;
      xmax = pmax;
      ymin = pmin;
      ymax = pmax; 
      dx = cxm - cx;
      dy = cym - cy;

printf ("xmin: %d  xmax: %d  ymin: %d  ymax: %d\n", xmin, xmax, ymin, ymax);
printf ("dx: %d  dy: %d\n", dx, dy);

      for (yy = ymin; yy <= ymax; yy++)
         {
         y = yy + dy;
         for (xx = xmin; xx <= xmax; xx++)
            {
            x = xx + dx;
            if ((x < pmin || x > pmax) ||
                (y < pmin || y > pmax))  cpout [yy][xx] = 0;
            else
               cpout [yy][xx] = cpin [y][x];
            }
         }
      printf ("xx: %d  yy: %d\n", xx, yy);

      close (fd);

      /*** Write shifted image to a disk file. ***/

over:
      strcpy (namout, outname (namimg));
      printf ("shifted image: %s\n", namout);
      if ((fd = creat (namout, 0664)) < 0)
         {
         fprintf (stderr, "Cannot create %s\n", namout);
         exit (-5);
         }

      if (write (fd, hskp, np) != np)
         {
         perror ("write");
         exit (-6);
         }
      if (write (fd, bitmap, np) != np)
         {
         perror ("write");
         exit (-7);
         }

      n = NPIX * NPIX * nbypix;
      if (write (fd, cpout, n) != n)
         {
         perror ("write (cpout)");
         exit (-8);
         }
      close (fd);
      }		/*end of while names	*/
   exit (0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
outname (namvid)

char *namvid;

{
int lenvid;
int idot;
int i;
char paknam[80];

   lenvid = strlen (namvid);
   idot = -1;
   for (i=0; i<lenvid; i++) 
      {
      if (namvid [i] == '.')  idot = i;
      paknam [i] = namvid [i];
      }
   if (idot == -1)
      {
      printf ("outname: invalid file name: %s\n", namvid);
      exit (-9);
      }
   paknam [idot+1] = 'd';
   paknam [lenvid] = '\0';

   return (paknam);
}
