/* cphskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decodes housekeeping and puts result into stucture H as defined
 * in file "hskp.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Originally written by Rainer Illing   13 December 1983
 * Revised by Andrew Stanger  28 December 1983
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Updated to compute ip,ir,lp,lr and look up disk center:  27 January 1984
 * Add definition of scaling variables:  30 January 1984.
 * Open and close image locally:  22 February 1984.
 * Remove call to 'hkshow':  24 February 1984
 * Rename include files:  27 February 1984.
 * Read into character array, then copy to integer array: 20 March 1984.
 * Change open, read, close to  fopen, fread, fclose: 25 April 1984.
 * Revise housekeeping structure: 25 October 1984.
 * 28 November 1984: Add roll.
 * 3 December 1984: Modify green line configuration (H.glc).
 *                  NESW sector designation.
 * 20 March 1985: add extension to 'H.filename'.
 * 21 May 1985: extract icyear and icdoy and add to file 'cpsize.h'.
 * 3 June 1985: Use "malloc" to allocate temporary storage for 'h[]'.
 *              Execute "free (c)" after 'c[]' is copied to 'h[]'.
 * 5 June 1985: If icyear < 80, set icyear = 0, icdoy = 0;
 * 7 November 1986: Determine nbypix from nbtpix (h[103]).
 * 11 November 1986: Extract 'npixl' from bytes 110 & 111.
 *                   Adjust disk center and geometric scale factor accordingly.
 * 14 November 1986: Add 'gres' for display of non-standard size images.
 * 17 November 1986: Add 'pscale': pixel magnitude scale factor.
 * 20 November 1986: idtype = 4 represents electron density data.
 * 5 December 1986: Compute 'scroll': roll adjusted to closest 10 degrees.
 * 11 December 1986: Set variables in 'cpgraf.h' by decoding hskp byte 221.
 * 17 December 1986: idtype=4 => floating point data.  lden=2 => density units.
 * 18 December 1986: For CRAY-calibrated data, set idtype from file name ext.
 *  6 September 1989: H.filename: "yydddhhmm.vxx".
 * 19 September 1989: Roll is computed differently for 1980 & 1984-89.
 * 16 November 1989: Use bytes 88 and 89 for roll (if both are not 127);
 * 14 December 1989: Do NOT use roll bytes 88 & 89.
 * 19 December 1989: remove line "if (scroll > 0)  scroll = roll + 7;"
 *                 in logic to round off roll to closest 10 degrees for 1984-89.
 *                 Add +5 to positive roll, -5 to negative roll before
 *                 dividing and multiplying by 10.
 * 20 December 1989: Shorten mechanism names for Housekeeping structure.
 * 31 January 1990: For scroll, do NOT add 5 for negative rolls.
 * 13 June 1990: Always determine nbypix from byte #103 unless != 8, 16, or 32.
 * 18 June 1990: Decode hskp bytes 221, 222 & 223 according to memo (20 Apr 90).
 * 20 June 1990: hskp structure: add str, vig, reg, fc, degnor, pixrs, pscale.
 * 23 July 1990: 1980 positive roll: round up.  1984-89 positive roll: +7 / 10.
 * 31 October 1990: scroll derived from either bytes 11&12 or 88&89.
 *  7 November 1990: define data type #14 (Bf + Bs).
 * 24 January 1991: define data type #15 (B1 / B2).
 * 25 January 1991: round off disk centers to closest even number.
 * 16 June 1992: idtype = 16 allowed for miscellaneous image type.
 *               bit 4 of byte 223 indicates K-corona removed.
 *               pscale always determined by byte 220.
 *  5 August 1992: add/change comments, remove pscale from disk center loop.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define	MAXHSK	224
#define	ZC	'\0'

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

cphskp (namimg)		/* SMM C/P Housekeeping Decoder */

char	namimg [];	/* Image file name. */

{

   /*** Arrays for housekeeping entries. ***/

   extern float pow10();
   static char  namext [80];
   static char  namres [] = "H\0L\0";
   static char  namfil [] = "r \0Ha\0op\0b \0wb\0g \0";
   static char  namsec [] = "sw\0s \0se\0e \0ne\0n \0nw\0w \0";
   static char  namsmf [8][3] = {"sw","ss","se","ee","ne","nn","nw","ww"};
   static char  nampol [] = "CLR\0+60\0-60\00dg\0";
   static char  namgls [] = "m\0a\0s\0f\0";
   static char  namcal [] = "m\0o\0i\0b\0";
   static char  cglstw [2];	    /* Storage for green line stow status. */
   static int   lih [4], liv [4];   /* Binary components of IH & IV. */
   static char  namtype [4] = {'v', 't', 'c', '\0'};

   /*** Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). ***/
   /* (No geometric correction applied.) */

   static float vidcen [9][2] = { 36.0, 500.0,	/* sw */
				 214.0, 194.0,	/* s  */
				 540.0,   4.0,	/* se */
				 820.0,  88.0,	/* e  */
				 892.0, 362.0,	/* ne */
				 718.0, 680.0,	/* n  */
				 386.0, 856.0,	/* nw */
				 104.0, 792.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

   /*** Disk centers (vs. Sector Mirror) for calibrated data (C34520). ***/
   /* (Processed with geometric correction WW6COR). */

   static float ww6cen [9][2] = { 48.0, 508.0,	/* sw */
				 222.0, 206.0,	/* s  */
				 530.0,  40.0,	/* se */
				 814.0,  86.0,	/* e  */
				 884.0, 354.0,	/* ne */
				 714.0, 672.0,	/* n  */
				 394.0, 824.0,	/* nw */
				 114.0, 786.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

   static float secrs = 961.;	/* arcseconds per solar radius. */
   static float secpix = 5.933; /* arcseconds per pixel (Hi res). */

   double PI = 3.141592654;
   double froll, radrol, rollwt;

   int i;
   int icode1, icode2, icode3;
   int idot;
   int iext;
   int iexp;
   int imgoff;
   int ires, ipw, ifw, ism, icm, iglstw, iglpos;
   int isign;
   int jsm;
   int kres, ihs, ivs;
   int lenimg;
   int lres;
   int nbtpix;
   int roll1, roll2, cproll, iroll, troll, tlmroll, rolldif;

   short *whiphd;

   int  *h;		/* Pointer to housekeeping array (integer).*/
   char *c;		/* Pointer to housekeeping array (character). */
   char *calloc ();	/* system allocation routine. */
   char *malloc ();	/* system allocation routine. */

   FILE *imgfd;		/* File pointer for image disk file. */

   for (i = 0; i < strlen (namimg); i++)
      {
      if (namimg [i] == '.')  idot = i;
      }
   strcpy (namext, &namimg [idot+1]);

   if (strcmp (namext, "pic") == 0)
      {
      imgoff = 24;
      }
   else
      imgoff = 0;

   /*** Open image file. ***/

   if ((imgfd = fopen (namimg,"r")) == NULL)  { fclose (imgfd);  return (-1); }

   /*** Allocate storage for housekeeping array. ***/

   if (((h = (int *) calloc (MAXHSK, sizeof (int))) == NULL) ||
      (c = malloc (MAXHSK)) == NULL) 
      {
      fprintf (stderr,"cphskp> malloc error.\n") ;
      fflush (stderr);
      fclose (imgfd);
      return (-2);
      }

   if (imgoff > 0)
      {
      whiphd = (short *) malloc (imgoff);
      if (fread (whiphd, sizeof (short), imgoff/2, imgfd) == NULL)
         {
         fclose (imgfd);
         free (c);
         free ((char *) h);
         return (-3);
         }
      }

   /*** Read housekeeping from C/P image file into character array 'c'. ***/

   if (fread (c, sizeof (char), MAXHSK, imgfd) == NULL)
      {
      fclose (imgfd);
      free (c);
      free ((char *) h);
      if (imgoff > 0)  free (whiphd);
      return (-3);
      }
   fclose (imgfd);

   /*** Copy housekeeping from `char' to `int' array. ***/

   for (i = 0; i < MAXHSK; i++)  *(h + i) = (int) *(c + i) & 0377;
   free (c);

   /* IDTYPE: data type.
    *		=1 raw vidicon data (DN units, 8 bits).
    *		=2 Equivalent seconds * 5, 8 bits.
    *		=3 Calibrated data (10^-11 B/Bsun units, 16 bits). 
    * Data type is inferred from the first letter of the filename extension.
    *
    * A C/P filename has the following format :
    * yydddhhmm.vxx :	type 1
    *          .txx :	     2
    *          .cxx :	     3
    *          .dxx :        4
    * where yy = year - 1900.
    *       ddd = day-of-year, hh = hour, mm = minute.
    *       xx = sector mirror position (typically).
    */

   lenimg = strlen (namimg);
   iext = -1;
   for (i=0; i<lenimg; i++)
      {
      if (namimg [i] == '.') iext = i+1;
      }

   nbypix = 2;
   idtype = 3;
   if (namimg [iext] == 'v' || namimg [iext] == 'V')  {idtype = 1; nbypix=1;}
   if (namimg [iext] == 't' || namimg [iext] == 'T')  {idtype = 2; nbypix=2;}
   if (namimg [iext] == 'd' || namimg [iext] == 'D')  {idtype = 4; nbypix=4;}

   /*** Year and Day-of-Year. ***/

   H.year = *h;		/* year */

   H.doy = *(h+1) << 8;
   H.doy += *(h+2);	/* day-of-year */

   /*** Polaroid Filter: p0=none, p1=+60, p2=0 degrees, p3=-60. ***/

   ipw = (*(h+3) & 0300) >> 6;
   H.ipw = ipw;
   for (i=0; i<4; i++)  *(H.pol+i) = *(nampol+4*ipw+i);

   /*** Resolution: high or low. ***/

   ires = (*(h+3) & 04) >> 2;
   H.ires = ires;
   for (i=0; i<3; i++) *(H.res+i) = *(namres+2*ires+i);

   /*** IH,IV: image format parameters (horizontal, vertical). ***/

   H.ih = (*(h+9) & 0360) >> 4;
   H.iv = (*(h+9) & 017);

   lih[0] = (H.ih & 010) >> 3;
   lih[1] = (H.ih &  04) >> 2;
   lih[2] = (H.ih &  02) >> 1;
   lih[3] = (H.ih &  01);

   liv[0] = (H.iv & 010) >> 3;
   liv[1] = (H.iv &  04) >> 2;
   liv[2] = (H.iv &  02) >> 1;
   liv[3] = (H.iv &  01);

   /*** Compute start & end of pixel & line (row) coordinates for C/P image.***/

   /*** resolution	ires	kres	nres	lres		gres
    *** high		0	1	2	npix/npixl	2/lres
    *** low		1	2	1	npix/npixl	1/lres
    ***/

   if (ires == 0)
      nres = 2;
   else
      nres = 1;
   kres = ires + 1;
   ip = 0;
   lp = 895 / kres;
   ir = 0;
   lr = 895 / kres;
   ihs = 0;
   ivs = 0;

   for (i = 0; i < 4; i++)
      {
      if (lih [i] == 0 && ihs == 0)  ip += 224 / kres;
      if (liv [i] == 0 && ivs == 0)  ir += 224 / kres;
      if (lih [i] == 1)  ihs = 1;
      if (liv [i] == 1)  ivs = 1;
      if (lih [i] == 0 && ihs == 1)  lp -= 224 / kres;
      if (liv [i] == 0 && ivs == 1)  lr -= 224 / kres;
      }

   H.ip = ip;
   H.ir = ir;
   H.lp = lp;
   H.lr = lr;

   npix = lp - ip + 1;
   lres = -1;
   gres = -1.0;

   if (npixl < 1 || npixl > 896)  npixl = npix;
   if (npixl > 0)
      {
      lres = npix / npixl;
      if (lres > 0)
         gres = (float) nres / (float) lres;
      else
         {
         gres = 1.0;
         lres = 1;
         }
      }

   /*** Exposure duration (units of 1.024 seconds). ***/

   H.exp  = (*(h+10) & 0377) << 4;
   H.exp += (*(h+11) & 0360) >> 4;

   /*** Spacecraft roll. ***/

   /*** Compute roll using bytes 11 & 12 (from SMM on-board computer [OBC]).***/

   roll1 = (int) (*(h+11) & 017);
   roll2 = (int) (*(h+12) & 0377);
   isign =  (roll1 & 8) / 8;		/* Sign bit. */

   /***... 1980: Least significant roll bit = 0.111905819 degrees. ...***/

   if (H.year == 80)
      {
      iroll = (int) ((roll1 & 7) * 256 + (roll2 & 0377));
      if (isign == 1)
	 radrol = - (float) (((~(iroll)+1) & 03777));
      else
         radrol = (float) iroll;

      rollwt = 0.111905819;
      froll = radrol * rollwt;
      if (froll > 0)
	 tlmroll = (int) (froll + 0.5);
      else
	 tlmroll = (int) froll;
      }

   /***... 1984-89: Least significant roll bit = 7.161972438 degrees. ...***/

   if (H.year > 80)
      {
      iroll = (int) ((roll1 & 7) * 4 + (roll2 & 0300) / 64);
      if (isign == 1)
	 radrol = - (float) (((~(iroll)+1) & 037));
      else
         radrol = (float) iroll;

      rollwt = 7.161972438;
      tlmroll = (int) (radrol * rollwt);
      }

   H.roll = tlmroll;

   /*** troll is ``best-guess'' roll in degrees derived from C/P telemetry. ***/

   if (H.year == 80)
      {
      troll = tlmroll;
      }
   else
   if (H.year > 80)	/* For 1984-1989, set scroll to closest 10 degrees. */
      {
      if (isign == 1)
         troll = tlmroll;
      else
         troll = tlmroll + 7;
      troll = (troll / 10) * 10;
      }
   else
   troll = 0;
   if (troll > 360)  troll -= 360;
   if (troll <   0)  troll += 360;
   if (troll > 180)  troll -= 360;

   /*** Compute roll using bytes 88 & 89 (valid if between 1 & 360). ***/

   roll1 = (int) (*(h+88) & 0377);
   roll2 = (int) (*(h+89) & 0377);
   cproll = roll1 * 256 + roll2;
   if (cproll > -360 && cproll < 361 && cproll != 0)
      {
      if (cproll > 360)  cproll -= 360;
      if (cproll <   0)  cproll += 360;
      if (cproll > 180)  cproll -= 360;
      }
   else
      cproll = -1;

   /*** Compare troll and cproll. ***/

   if (cproll <= 0)		/* Invalid value in bytes 88 & 89. */
      scroll = troll;
   else
      {
      iroll = troll;
      if (iroll > 360)  iroll -= 360;
      if (iroll <   0)  iroll += 360;
      if (iroll > 180)  iroll -= 360;
      scroll = cproll;

      rolldif = iroll - cproll;
      if (rolldif < 0)  rolldif = - rolldif;
      if (rolldif > 10)
	 {
	 fprintf (stderr,
	    "roll (bytes 11&12): %d disagrees with roll (bytes 88&89): %d\n",
            iroll, cproll);
         fprintf (stderr,"Check records for correct roll, then run fixroll.\n");
	 exit (-1);
	 }
      }

   H.degnor = 133. + (float) scroll; /* North is 133+scroll deg. from x-axis. */

   /*** Green line flter tilt position and stow status. ***/

   iglpos = *(h+13) & 0377;
   H.iglpos = iglpos;

   iglstw = (*(h+14) & 0300) >> 6;
   H.iglstw = iglstw;
   for (i=0; i<2; i++) *(cglstw+i) = *(namgls + 2*iglstw + i);

   /*** Sector mirror position: sw, s, se, e, ne, n, nw, w. ***/

   jsm = (*(h+14) & 074) >> 2;
   ism = (jsm-1)/2;		/* Decode to array position. */
   if (jsm == 0)  ism = 8;	/* Special case for NESW. */
   H.ism = ism;
   for (i=0; i<6; i++) *(H.sec+i)= *(namsec+3*ism+i);

   /*** Calibration mirror position: move, out, in, between. ***/

   icm = (*(h+15) & 060) >> 4;
   H.icm = icm;
   for (i=0; i<4; i++)  *(H.cal+i) = *(namcal+2*icm+i);

   /*** Color filter wheel position: red|H-alpha|open|blue|wideband|green. ***/

   ifw = (*(h+15) & 016) >> 1;
   ifw = ifw - 1;			/* Decode to array position. */
   H.ifw = ifw;
   for (i=0; i<7; i++)  *(H.fil+i) = *(namfil + 3*ifw + i);

   /*** Hours and minutes [at beginning of exposure]. ***/

   H.hour   = *(h+20);
   H.minute = *(h+21);
   H.second = *(h+24);

   /*** Image sequence counter (aka: 12-bit operational status). ***/

   H.frame = *(h+30) * 16 + (*(h+31) & 0360) / 16;

   /*** Byte order: 0 -> Sun-4, SGI;  1 -> DEC. ***/

   bytord = 0;
   if (*(h+52) > 0)  bytord = 1;

   /*** Quantity: % image segments captured (i.e., NOT zero-filled). ***/

   H.quan = *(h+79) & 0377;

   /*** Bits per pixel. ***/

   nbtpix = *(h+103);
   if (nbtpix == 8 || nbtpix == 16 || nbtpix == 32)
      nbypix = nbtpix / 8;
   H.nbypix = nbypix;

   /*** Actual # pixels per line. ***/

   npixl = (*(h+110) & 0377) * 256 + (*(h+111) & 0377);

   /*** Look up disk center location according to sector mirror. ***/

   /*** Raw DN images : (no geometric distortion correction) ***/

   if (idtype == 1)
      {
      xcen = vidcen [ism][0] / (float) (kres*lres);
      ycen = vidcen [ism][1] / (float) (kres*lres);
      }

   /*** Equivalent Second and Calibrated images :
    *** (SMMCOR or WW6COR geometric corrections)
    ***/

   else
   if (idtype > 1 && idtype < 17)
      {
      xcen = ww6cen [ism][0] / (float) (kres*lres);
      ycen = ww6cen [ism][1] / (float) (kres*lres);

      icyear = *(h+149);
      icdoy = (*(h+150)) * 256 + *(h+151);
      if (icyear < 80)
         {
         icyear = 0;
         icdoy = 0;
         }
      }
   H.xcen = xcen;
   H.ycen = ycen;

   /*** Photometric scaling factor. ***/

   iexp = (int) (*(h+220) & 0377);

   /*** Decipher numerical codes. ***/

   H.str  = 0;
   H.vig  = 0;
   H.reg  = 0;
   H.f    = 0;
   H.k    = 0;
   H.bit5 = 0;
   H.bit6 = 0;
   H.cc   = 0;

   icode1 = *(h+221) & 0377;
   icode2 = *(h+222) & 0377;
   icode3 = *(h+223) & 0377;

   if (namimg [iext] != 'v' && namimg [iext] != 'V')
      {
      if (icode1 & 1)   idtype = 1;	/* raw vidicon DN values (0-255). */
      if (icode1 & 2)   idtype = 2;	/* equivalent second (10-bit log). */
      if (icode1 & 4)   idtype = 3;	/* calibrated radiance (B/Bsun). */
      if (icode1 & 8)   idtype = 4;	/* electron density data (elec/cm2). */
      if (icode1 & 16)  idtype = 5;	/* B1 - B2 (deltab B). */
      if (icode1 & 32)  idtype = 6;	/* Bk (K-corona). */
      if (icode1 & 64)  idtype = 7;	/* (Bcme - Bbkg) / Bk. */
      if (icode1 & 128) idtype = 8;	/* (Bcme - Bbkg) / Bbkg. */

      if (icode2 & 1)   idtype = 9;	/* polarization fraction. */
      if (icode2 & 2)   idtype = 10;	/* pB. */
      if (icode2 & 4)   idtype = 11;	/* B radial+tangential. */
      if (icode2 & 8)   idtype = 12;	/* Btangential. */
      if (icode2 & 16)  idtype = 13;	/* Bradial. */
      if (icode2 & 32)  idtype = 14;	/* Bf + Bs. */
      if (icode2 & 64)  idtype = 15;	/* B1 / B2. */
      if (icode2 & 128) idtype = 16;	/* Miscellaneous. */

      if (icode3 & 1)   H.str  = 1;	/* stray light removed. */
      if (icode3 & 2)   H.vig  = 1;	/* vignetting removed. */
      if (icode3 & 4)   H.reg  = 1;	/* disk registered. */
      if (icode3 & 8)   H.f    = 1;	/* F-corona removed. */
      if (icode3 & 16)  H.k    = 1;	/* K-corona removed. */
      if (icode3 & 32)  H.bit5 = 1;	/* not used. */
      if (icode3 & 64)  H.bit6 = 1;	/* not used. */
      if (icode3 & 128) H.cc   = 1;	/* Munro calib. correction applied. */
      if (lstr == 2 && lvig == 2)  lsv = 2;
      }
   H.idtype = idtype;

   /*** Extract reference image year, DOY, hour, minute & second. ***/

   switch (idtype)
      {
      case 5:				/* B2 - B1 */
      case 7:				/* (Bk   - Bref) / Bk   */
      case 8:				/* (Bcme - Bref) / Bref */
      case 15:				/* B2 / B1 */
               year_zero   =  *(h+143);
               doy_zero    = (*(h+144) << 8) + *(h+145);
               hour_zero   =  *(h+146);
               minute_zero =  *(h+147);
               second_zero =  *(h+148);
      default:
               break;
      }

/*
 *   fprintf (stderr, "cphskp> icode1/2/3: %3.3o  %3.3o   %3.3o   idtype: %d\n",
 *      icode1, icode2, icode3, idtype);
 */

   /*** Convert Green Line Filter position & stow status into characters. ***/

   sprintf (H.glc, "%03d%s", iglpos, cglstw);


   /*** Convert date & time into characters [yydddhhmm]. ***/

   sprintf (H.filename, "%02d%03d%02d%02d",
            H.year, H.doy, H.hour, H.minute);

   /*** Image filename. ***/

   strncat (H.filename, ".", 1);
   strncat (H.filename, &namtype [idtype-1], 1);
   if (icm == 1)  strncat (H.filename, &namsmf [ism][0], 2);
   else
   if (icm == 2)  strcat (H.filename, "cm");
   else
      strcat (H.filename, "XX");
   lenimg = strlen (H.filename);
   H.filename [lenimg] = '\0';

   /*** Photometric scaling. ***/

   if (nbypix == 2)
      H.pscale = pscale = pow10 (-1 * iexp);
   else
      {
      if (iexp == 0)
         H.pscale = pscale = 1.0;
      else
         H.pscale = pscale = (float) iexp;
      }

   /*** Define values for various constants. ***/

   raddeg = PI / 180.0;				/* radians / degree. */
   degrad = 180.0 / PI;				/* degrees / radian. */
   pixrs = secrs / (secpix * kres * lres);	/* pixels  / solar radius. */
   if (ism == 8)  pixrs *= 0.5;			/* 4-quadrant composite image.*/
   H.pixrs = pixrs;
   rspix = 1.0 / pixrs;				/* solar radii / pixel. */

   free (h);					/* Discard storage for `h'. */
   if (imgoff > 0)  free (whiphd);
   return (0);
}
