/* prscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Radial Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984
 * 18 July 1984: added option to remove stray light & vignetting.
 * 26 July 1984: added calibration correction.
 *               added option to remove F-corona.
 * 7 February 1985: change double to float.
 * 10 December 1985: Apply fixcal to all calibrated images.
 * 15 May 1986: Use ATM/WLC F-corona function.
 * 14 November 1986: use 'gres' instead of 'nres' for display.
 * 25 November 1986: use 'rcoord' instead of 'coord' for roll compensation.
 * 24 March 1987: use separate flags for corrections: cal,stray,vignet,F-corona.
 * 26 March 1987: add polarization filter corrections.
 * 10 June 1991: adapt for SGI/IRIS GL windows.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/img_scale.ext"
#include "/usr/people/smm/lib/include/ov_color.h"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

prscan(namimg, image, rmin, rmax, rinc, theta, scntyp, scan, scandx, ns)

char	namimg[];	/* Image file name. */
char	*image;		/* Pointer to image array. */
float	rmin, rmax;	/* Solar radius limits for radial scan. */
float	rinc;		/* Radial increment for radial scan. */
float	theta;		/* Theta (degrees). */
int	*scntyp;	/* Pointer to scan type (1=radial,2=line). */
float	scan[];		/* Dependent axis scan array. */
float	scandx[];	/* Independent axis scan array. */
int	*ns;		/* Number of values in scan. */

{
extern	float calcor();	/* Munro calibration correction. */
extern	float cpstray();/* Stray light function. */
extern	float vignet();	/* Vignetting function. */
extern	float fcorona();/* F-corona function. */
extern float blint();
extern float polcor;	/* Polaroid transmission correction factor. */
float	dist, dstinc, dstsun;
float	radinc;
float   Bexp;		/* "Generic" magnitude scaling factor. */
float	cc, str, vig, fcc, pc;
float	cval;
float	x, y;
float	r;
float	th;

register   int	ir;
int	ixg, iyg;
int	ierr;
int	nscan;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Set scale factor. ***/

   Bexp = pscale;
   str = 0.0;
   vig = 1.0;
   fcc = 0.0;
   cc = 1.0;
   pc = 1.0;
/*
 *   if (polcor == 1.0)  pc = 1.;
 *   else pc = 0.5;
 */

   /*** Radial increment: set to 0.1 Rsun if 'rinc' <= 0. ***/

   radinc = rinc;
   if(rinc <= 0.)  radinc = 0.1;
   th = theta;

/* if (*scntyp == 2) */
      {
      dist = rmax - rmin;
      dstsun = -radinc;
      }

   /* ****** Radial Scan ****** */

   for (ir = 0; ir < MAXSCN; ir++)
      *(scan + ir) = *(scandx + ir) = 0.0;
 
   /* ....... Radial scan loop ....... */

   *ns = 0;
   ir = -1;
   for (r = rmin; r <= rmax; r += radinc)
      {
      ir++;
 
      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = rcoord (&r, &th, &x, &y, 1, scroll)) != 0)
         return (ierr);

      /*** Compute stray light, vignetting & F-corona values. ***/

      if (idtype == 3)
         {
         if (lcc == 1)  cc = calcor (r);
         if (lstr == 1)  str = cpstray (r) * pc;
         if (lfc == 1)   fcc = fcorona (r) * pc;
         if (lvig == 1)  vig = vignet (r);
         }

      /*** Draw "X" to mark scan point on IRIS display. ***/
 
      if (igr > 0)
         {
         ixg = (int) x/gres + gMINCOL;
         iyg = gMAXROW - (int) y/gres;
         gl_wvec (ixg, iyg, ixg+1, iyg+1, ov_red);
         gl_wvec (ixg+1, iyg, ixg, iyg+1, ov_red);
         }
      /* 
       * Perform bilinear interpolation to compute scan point value from image.
       * Use 4 nearest neighbors.
       */
      dstsun += radinc;
      if (*scntyp == 1)  *(scandx + ir) = (float) r;
      else if (*scntyp == 2)  *(scandx + ir) = (float) dstsun;

      cval = (blint (image, &x, &y) * Bexp) * cc;
      *(scan + ir) = (cval - str) * vig - fcc;

      /*
       * Divide scan values by 5 for equivalent second data type.
       * (numbers in file are 5 * equivalent second units.)
       */
 
      if (idtype == 2) *(scan + ir) /= 5.0;
      }

count:   *ns = ir + 1;

   /*** Smooth data using 'nave' point average. ***/
 
   faverage (nave, scan, *ns);
 
   nscan = *ns;
   if (igterm == 0)
	for (ir=0; ir<*ns; ir++)
           fprintf (stdout, "r:%7.3f  th:%7.2f  scan:%9.2e\n",
                    scandx [ir], th, scan [ir]);
   return (0);
}
