/* ptscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Theta Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984: Re-created on Vax-11/750; C language; UNIX 4.2 bsd.
 * 18 July 1984: Add stray light & vignetting removal capabilities.
 * 26 July 1984: Add calibration correction.  Add F-corona removal option.
 * 8 February 1985: Add average routine.
 * 6 December 1985: Apply fixcal to all calibrated images.
 * 16 December 1985: Replace 'coord' with 'rcoord'.
 * 17 November 1986: Replace 'nres' by 'gres' for display.
 * 26 March 1987: add polarization filter corrections.
 * 10 June 1991: adapt for SGI/IRIS GL windows.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/img_scale.ext"
#include "/usr/people/smm/lib/include/ov_color.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

ptscan (namimg, image, thmin, thmax, thinc, radius, scan, scandx, ns)

char	namimg[];	/* Image file name. */
char	*image;		/* Pointer to image array. */

float	thmin, thmax;	/* Solar position angle limits for theta scan. */
float	thinc;		/* Angular increment for theta scan. */
float	radius;		/* Radius (R/Rsun units). */
float	scan[];		/* Dependent axis scan array. */
float	scandx[];	/* Independent axis scan array. */

int	*ns;		/* Number of values in scan. */

{
extern float calcor();	/* Munro Calbiration correction function. */
extern float cpstray();	/* Stray light function. */
extern float vignet();	/* Vignetting function. */
extern float fcorona();	/* F-corona function. */
extern int faverage();	/* Scan average routine. */
extern float blint();	/* Bilinear interpolation function. */
extern int rcoord();	/* Coordinate transformation: polar <--> cartesian. */
extern float polcor;	/* Polaroid transmission correction factor. */

float	x, y;
float	r, th;
float	tinc;
float	Cexp = 1.e-11;		/* Scale factor for calibrated images. */
register float Bexp;		/* "Generic" scale factor. */
float	cc, str, vig, fcc, pc;
float	cval;
register int it;
int	ixg, iyg;
int	ierr;
int	nscan;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   /* Theta increment: set default to angle separating two adjacent pixels. */

   tinc = thinc;
   if (thinc <= 0.)  tinc = degrad / (radius*pixrs);
   r = radius;

   /*** Set scale factor. ***/

   Bexp = pscale;
   str = 0.0;
   vig = 1.0;
   fcc = 0.0;
   cc = 1.0;
   pc = 1.0;
   if (idtype == 3 && lcc == 1)  cc = calcor (r);
/*
 *  if (polcor == 1.0)  pc = 1.;
 *  else pc = 0.5;
 */

   /*** Clear scan arrays. ***/

   for (it = 0; it < MAXSCN; it++)
      *(scan + it) = *(scandx+ it) = 0.;
 
   /* ******* Theta scan loop ******* */
 
   if (thmax < thmin)  thmax += 360.0;

   /*** Compute stray light, vignetting & F-corona values. ***/

   if (idtype == 3)
      {
      if (lstr == 1)  str = cpstray (r) * pc;
      if (lfc == 1)   fcc = fcorona (r) * pc;
      if (lvig == 1)  vig = vignet (r);
      }

   *ns = 0;
   it = -1;
   for (th = thmin; th <= thmax; th += tinc)
      {
      it++;
 
      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = rcoord (&r, &th, &x, &y, 1, scroll)) != 0)
         return (ierr);

      /*** Draw "X" to mark theta scan point on display. ***/
 
      if (igr > 0)
         {
         ixg = (int) x / gres + gMINCOL;
         iyg = gMAXROW - (int) y / gres;
         gl_wvec (ixg,   iyg, ixg+1, iyg+1, ov_red);
         gl_wvec (ixg+1, iyg, ixg,   iyg+1, ov_red);
         }
 

      /*** Perform bilinear interpolation to compute scan point from image. ***/

      *(scandx + it) = (float) th;

      cval = (blint (image, &x, &y) * Bexp) * cc;
      *(scan + it) = (cval - str) * vig - fcc;

      }
   *ns = it + 1;

   /*** Smooth data using 'nave' point average. ***/
 
   faverage (nave, scan, *ns);
 
   /* Apply calibration correction. */
   /* Remove stray light & vignetting. */
   /* Remove F-corona. */
 
   nscan = *ns;

   /* Divide scan values by 5 for equivalent second data type.
    *(numbers in file are 5 * equivalent second units.)
    */
 
   if (idtype == 2)  for (it = 0; it < *ns; it++)
      scan [it] /= 5.0;
 
   if (igterm == 0)
      for (it=0; it<*ns; it++)
         fprintf (stdout, "r:%7.3f  th:%7.2f  scan:%9.2e\n",
                  r, scandx[it], scan[it]);
   fprintf (stdout, "ptscan> ns: %d\n", *ns);
   return (0);
}
