/* smmhk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decodes housekeeping and puts result into stucture H as defined
 * in file "smmhk.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Originally written by Rainer Illing   13 December 1983
 * Revised by Andrew Stanger  28 December 1983
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Updated to compute ip,ir,lp,lr and look up disk center:  27 January 1984
 * Add definition of scaling variables:  30 January 1984.
 * Open and close image locally:  22 February 1984.
 * Remove call to 'hkshow':  24 February 1984
 * Rename include files:  27 February 1984.
 * Read into character array, then copy to integer array: 20 March 1984.
 * Change open, read, close to  fopen, fread, fclose: 25 April 1984.
 * Revise housekeeping structure: 25 October 1984.
 * 28 November 1984: Add roll.
 * 3 December 1984: Modify green line configuration (H.glc).
 *                  NESW sector designation.
 * 20 March 1985: add extension to 'H.filename'.
 * 18 April 1985: Revise housekeeping structure.
 * 4 October 1985: copy actual image filename into 'H.filename'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define	MAXHSK	128
#define	ZC	'\0'

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.ext"
extern char pei;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

smmhk (namimg)	/* SMM C/P Housekeeping Decoder */

char	namimg[];	/* Image file name. */

{

/* Character arrays for housekeeping entries. */

static	char	namres[] = "H\0L\0";
static	char	namfil[] = "r \0Ha\0cl\0b \0wb\0g \0";
static	char	namsec[9][3] =
		{"sw","s ","se","e ","ne","n ","nw","w ","4q"};
static	char	namsmf[9][3] = {"sw","ss","se","ee","ne","nn","nw","ww","4q"};
static	char	nampol[] = "clr\0p1 \0p3 \0p2 \0";
static	char	namgls[] = "m\0a\0s\0f\0";
static	char	namcal[] = "m\0o\0i\0b\0";
static	char	cglstw[2];	/* Storage for green line stow status. */
static	int	lih[4], liv[4];	/* Binary components of IH & IV. */
static	char	namtype[4] = {'v', 't', 'c', '\0'};

/* Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). */
/* (No geometric correction applied.) */

static	float	vidcen[9][2] = { 36.3, 499.8,	/* sw */
				213.1, 193.6,	/* s  */
				539.9,   3.3,	/* se */
				820.0,  87.0,	/* e  */
				892.4, 361.5,	/* ne */
				718.6, 679.3,	/* n  */
				386.7, 856.4,	/* nw */
				104.3, 792.1,	/* w  */
				447.0, 447.0};	/* n+e+s+w */

/* Disk centers (vs. Sector Mirror) for calibrated data (C34520). */
/* (Processed with geometric correction WW6COR). */

static	float	ww6cen[9][2] = { 47.9, 507.5,	/* sw */
				222.2, 205.6,	/* s  */
				529.7,  40.3,	/* se */
				814.6,  85.4,	/* e  */
				884.0, 353.4,	/* ne */
				714.8, 671.1,	/* n  */
				393.2, 824.1,	/* nw */
				113.6, 786.4,	/* w  */
				447.0, 447.0};	/* n+e+s+w */

static	float	secrad = 961.;	/* arcseconds per solar radius. */
static	float	secpix = 5.933; /* arcseconds per pixel (Hi res). */

float	pi = 3.141592654;
float	radrol, rollwt;

int	i, k;
int	ires, ipw, ifw, ism, icm, iglstw, iglpos;
int	roll1, roll2, roll, isign;
int	jsm;
int	kres, ihs, ivs;
int	lenimg, iext;
int	h [MAXHSK];	/* Housekeeping array (integer). */
int	ipei;		/* Pre-exposure inhibit. */

char	*c;		/* Housekeeping array (character). */
char	*malloc();	/* system allocation routine. */

FILE	*imgfd;		/* File pointer for image disk file. */


/* Open image file and read housekeeping information. */

   if((imgfd = fopen(namimg,"r")) == NULL)  { fclose(imgfd); return(-1); }

/* Allocate storage for housekeeping array. */

   if((c = malloc(MAXHSK)) == NULL) 
	{
	fprintf(stderr,"cphskp> malloc error.\n") ;
	fflush(stderr);
        fclose(imgfd);
        return(-2);
	}
   if(fread(c, sizeof(char), MAXHSK, imgfd) == NULL) { fclose(imgfd); return(-1); }
   fclose(imgfd);

/* Copy housekeeping from character*1 to integer*4 array. */

   for (i = 0; i < MAXHSK; i++)  *(h + i) = *(c + i) & 0377;
   free (c);

/* Year and Day-of-Year. */

	H.year = *h;		/* year */

	H.doy = *(h+1)<<8;
	H.doy += *(h+2);	/* day-of-year */

/* Polaroid. */

	ipw = (*(h+3) & 0300)>>6;
	H.ipw = ipw;
	for (i=0;i<4;i++) *(H.pol+i)= *(nampol+4*ipw+i);

/* Resolution. */

	ires = (*(h+3) & 04)>>2;
	H.ires = ires;
	for (i=0;i<2;i++) *(H.res+i)= *(namres+2*ires+i);

/* Pre-Exposure Inhibit. */

	ipei = *(h+5) & 1;
	if (ipei == 1)  pei = '*';
	else pei = '.';

/* IH,IV. */

	H.ih = (*(h+9) & 0360)>>4;
	H.iv = (*(h+9) & 017);

	lih[0] = (H.ih & 010) >> 3;
	lih[1] = (H.ih &  04) >> 2;
	lih[2] = (H.ih &  02) >> 1;
	lih[3] = (H.ih &  01);

	liv[0] = (H.iv & 010) >> 3;
	liv[1] = (H.iv &  04) >> 2;
	liv[2] = (H.iv &  02) >> 1;
	liv[3] = (H.iv &  01);

/* Exposure. */

	H.exp = (*(h+10) & 0377)<<4;
	H.exp += (*(h+11) & 0360)>>4;

/*** S/C roll. ***/

   roll1 = (int) (*(h+11) & 017);
   roll2 = (int) (*(h+12) & 0377);
   isign =  (roll1 & 8) / 8;

   /***... 1980: Least significant roll bit = .111906 degrees. ...***/

   if (H.year == 80)
      {
      roll = (int) ((roll1 & 7) * 256 + (roll2 & 0377));
      radrol = (float) roll;
      if (isign == 1)  radrol = - (float) (((~(roll)+1) & 03777));
      rollwt = 0.111906;
      }

   /***... 1984-89: Least significant roll bit = 7.16197 degrees. ...***/

   if (H.year > 80)
      {
      roll = (int) ((roll1 & 7) * 4 + (roll2 & 0300) / 64);
      radrol = (float) roll;
      if (isign == 1)  radrol = - (float) (((~(roll)+1) & 037));
      rollwt = 7.16197;
     }

   roll = (int) (radrol * rollwt);
   H.roll = roll;
   scroll = roll;

/* Green line position and stow. */

	iglpos = *(h+13) & 0377;
	H.iglpos = iglpos;

	iglstw = (*(h+14) & 0300)>>6;
	H.iglstw = iglstw;
	for (i=0;i<2;i++) *(cglstw+i)= *(namgls+2*iglstw+i);

/* Sector mirror. */

	jsm = (*(h+14) & 074)>>2;
	ism = (jsm-1)/2;		/* Decode to array position. */
	if (jsm == 0)  ism = 8;		/* Special case for NESW. */
	H.ism = ism;
/*
 *	for (i=0;i<3;i++) *(H.sec+i)= *(namsec+3*ism+i);
 */
	strcpy (H.sec, &namsec[ism][0]);

/* Calibration mirror. */

	icm = (*(h+15) & 060)>>4;
	H.icm = icm;
	for (i=0;i<2;i++) *(H.cal+i)= *(namcal+2*icm+i);

/* Filter wheel. */

	ifw = (*(h+15) & 016)>>1;
	ifw = ifw - 1;			/* Decode to array position. */
	H.ifw = ifw;
	for (i=0;i<3;i++) *(H.fil+i)= *(namfil+3*ifw+i);

/* Hours and minutes. */

	H.hour = *(h+20);
	H.minute = *(h+21);
	H.second = *(h+24);

/* Image count (operational status). */

	H.frame = *(h+30) * 16 + (*(h+31) & 0360) / 16;

/* Quantity. */

	H.quan = *(h+79) & 0377;

	free(h);

/* Convert into characters. */

	sprintf(H.time,"19%2d  %3d: %02d: %02d: %02d",
		H.year,H.doy,H.hour,H.minute,H.second);
/*
 *	sprintf(H.filename,"%1d%03d%02d%02d%02d",
 *		H.year-80,H.doy,H.hour,H.minute,H.second);
 */
	sprintf(H.glc,"%03d%s",iglpos,cglstw);

/* Compute start/end pixel/line coordinates for C/P image. */

	nres = 1;
	if(ires == 0)  nres = 2;
	kres = ires + 1;
	ip = 0;
	lp = 895 / kres;
	ir = 0;
	lr = 895 / kres;
	ihs = 0;
	ivs = 0;

	for (i=0; i<4; i++)
	{
	if(lih[i] == 0 && ihs == 0)  ip += 224 / kres;
	if(liv[i] == 0 && ivs == 0)  ir += 224 / kres;
	if(lih[i] == 1)  ihs = 1;
	if(liv[i] == 1)  ivs = 1;
	if(lih[i] == 0 && ihs == 1)  lp -= 224 / kres;
	if(liv[i] == 0 && ivs == 1)  lr -= 224 / kres;
	}

	H.ip = ip;
	H.ir = ir;
	H.lp = lp;
	H.lr = lr;
	npix = lp - ip + 1;

/* IDTYPE: data type.
		=1 raw vidicon data (DN units, 8 bits).
		=2 Equivalent seconds * 5, 8 bits.
		=3 Calibrated data (10^-11 B/Bsun units, 16 bits). 
 * Data type is inferred from the first letter of the filename extension.
 *
 * A C/P filename has the following format :
 * ydddhhmms.vxx :	type 1
 *          .txx :	     2
 *          .cxx :	     3
 * where y = year -1980.
 *       ddd = day-of-year, hh = hour, mm = minute, s = sec/10.
 *       x = arbitrary character.
 */

	lenimg = strlen(namimg);
	iext = -1;
        for (i=0; i<lenimg; i++) if(namimg[i] == '.') iext = i+1;

	idtype = 3;
	if(namimg[iext] == 'v' || namimg[iext] == 'V')  idtype = 1;
	if(namimg[iext] == 't' || namimg[iext] == 'T')  idtype = 2;
	H.idtype = idtype;

	nbypix = 2;
	if(idtype == 1 || idtype == 2)  nbypix = 1;
	H.nbypix = nbypix;

/*
 *	strncat (H.filename, ".", 1);
 *	strncat (H.filename, &namtype[idtype-1], 1);
 *	strncat (H.filename, &namsmf[ism][0], 2);
 *	lenimg = strlen (H.filename);
 *	H.filename[lenimg] = '\0';
 */
	strcpy (H.filename, namimg);

/* Look up disk center location according to sector mirror. */

/* Raw DN images : (no geometric distortion correction) */

	if(idtype == 1)
	{
	xcen = vidcen[ism][0] / (float) kres;
	ycen = vidcen[ism][1] / (float) kres;
	}

/* Equivalent Second and Calibrated images :
  (SMMCOR or WW6COR geometric corrections) */

	if(idtype == 2 || idtype == 3)
	{
	xcen = ww6cen[ism][0] / (float) kres;
	ycen = ww6cen[ism][1] / (float) kres;
	}
	H.xcen = xcen;
	H.ycen = ycen;

/* Set up scale variables. */

	raddeg = pi / 180.;
	degrad = 180. / pi;
	pixrs = secrad / (secpix * kres);
	if (ism == 8)  pixrs *= 0.5;
	rspix = 1. / pixrs;

   return(0);
}
