/* circle_perim.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Circle perimeter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate the x/y coordinates of the circle perimeter,
 * given the radius of the circle, and the center coordinates of the circle.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 13 November 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

circle_perim (rc, xc, yc, xit, yit, xib, yib, nperim)

int rc;				/* Circle radius (pixel units). */
int xc, yc;			/* Circle center coordinates (pixel units). */
int xit [];			/* X coordinates (top hemisphere). */
int yit [];			/* Y coordinates (top hemisphere). */
int xib [];			/* X coordinates (bottom hemisphere). */
int yib [];			/* Y coordinates (bottom hemisphere). */
int *nperim;			/* # of perimeter values in xit/yit/xib/yib. */
{
   int i;			/* Counter for nperim. */
   int xt = 0;			/* x coordinate (top hemisphere). */
   int xb = 0;			/* x coordinate (bottom hemisphere). */
   int yt = 0;			/* y coordinate (top hemisphere). */
   int yb = 0;			/* y coordinate (bottom hemisphere). */
   int xylim;			/* x/y loop limit. */

   double rsq;			/* radius squared. */
   double xsq;			/* x coordinate (circle locus) squared. */
   double ysq;			/* y coordinate (circle locus) squared. */
   double xd;			/* x-axis distance. */
   double yd;			/* y-axis distance. */

/*
   printf ("circle_perim> rc:%3d  xc:%3d   yc:%3d\n", rc, xc, yc);
 */

   xylim = (int) (((1.41421356 * rc) / 2.0) + 0.5);
   rsq = (double) (rc * rc);

   /*** Circle equation: r^2 = x^2 + y^2 ***/

   /*** Method of determining circle perimeter coordinates:
    ***
    *** Increment x-coordinate one pixel at a time, starting at 0 degrees
    *** and ending at 180 degrees, using a circle-centered coordinate system.
    *** Translate the local x-coordinate into image coordinates (xt).
    *** Compute corresponding y-coordinate (yt).
    *** Also compute the "mirror image" x/y coordinates (xb,yb).
    ***/

   i = -1;

   for (yt = xylim; yt >= -xylim; yt--)
      {
      i++;
      ysq = (double) (yt * yt);
      xsq = rsq - ysq;

      if (ysq != rsq)
         {
         xd = sqrt (xsq);
         xt = (int) (xd + 0.5);
         }
      else
         {
         xd = 0.0;
         xt = 0;
         }

      xit [i] =  xt + xc;	/* X coordinate: top hemisphere, image coord. */
      yit [i] =  yt + yc;	/* Y coordinate: top hemisphere, image coord. */

      xib [i] = -xt + xc;	/* X coordinate: bottom hemisphere, img.coord.*/
      yib [i] = -yt + yc;	/* Y coordinate: bottom hemisphere, img.coord.*/

/*
      printf ("i:%3d   xt:%3d   yt:%3d   xit:%3d  yit:%3d   xib:%3d   yib:%3d\n",
              i, xt, yt, xit [i], yit [i], xib [i], yib [i]);
 */
      }

   for (xt = xylim; xt >= -xylim; xt--)
      {
      i++;

      xsq = (double) (xt * xt);
      ysq = rsq - xsq;

      if (xsq != rsq)
         {
         yd = sqrt (ysq);
         yt = (int) (yd + 0.5);
         }
      else
         {
         yd = 0.0;
         yt = 0;
         }

      xit [i] =  xt + xc;	/* X coordinate: top hemisphere, image coord. */
      yit [i] =  yt + yc;	/* Y coordinate: top hemisphere, image coord. */

      xib [i] = -xt + xc;	/* X coordinate: bottom hemisphere, img.coord.*/
      yib [i] = -yt + yc;	/* Y coordinate: bottom hemisphere, img.coord.*/

/*
      printf ("i:%3d   xt:%3d   yt:%3d   xit:%3d  yit:%3d   xib:%3d   yib:%3d\n",
              i, xt, yt, xit [i], yit [i], xib [i], yib [i]);
 */
      } 
   *nperim = i + 1;

   return (0);
}
