/* clscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Line Scan procedure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extracts values from an image along a line between two arbitrary end points.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 20 April 1984
 * 18 July 1984: add option to remove stray light & vignetting.
 * 31 July 1984: add option to remove F-corona; remove printf for scan values.
 * 7 February 1985: change double to float.
 * 7 November 1986: fix calibration correction logic.
 * 5 December 1986: use Saito F-corona.
 * 10 June 1991: adapt for SGI/IRIS GL windows.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>

#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/img_scale.ext"
#include "/usr/people/smm/lib/include/ov_color.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

clscan (ibuf, nbypx, ip1, ir1, ip2, ir2, scanpix, scanx, scany, ns)

unsigned char *ibuf;		/* Pointer to 1 byte image array. */
int     nbypx;			/* # bytes per pixel. */
int	ip1, ir1, ip2, ir2;	/* Pixel,line coordinate scan limits. */
float	scanpix [];		/* scan pixel intensity. */
int	scanx [];		/* x-axis scan position. */
int	scany [];		/* y-axis scan position. */
int	*ns;			/* Number of values in scan. */

{

extern float polcor;		/* Polaroid transmission correction factor. */
extern float cpstray ();	/* Stray Light function. */
extern float vignet ();		/* Vignetting function. */
extern float fcorona ();	/* Function to provide F-corona value. */
extern float calcor ();		/* Munro calibation correction factor. */

unsigned char  *imc;		/* Pointer to 1 byte image array. */
unsigned short *ims;		/* Pointer to 2 byte image array. */

float x, y, slope, yoff;
float x1, x2, y1, y2;
float r1, r2, th1, th2;
float r, th;
float pix1, rec1, pix2, rec2;

int i;
int ierr;			/* Error. */
int ival1, ival2;
int ix, ixinc, iy, iyinc;
int ix1, ix2, iy1, iy2;
int ixg, iyg;
int nx, nxdif;
int ny, nyinc, nydif;

   imc = (unsigned char  *) ibuf;
   ims = (unsigned short *) ibuf;

   /*** Clear scan arrays. ***/

   for (i = 0; i < *ns; i++)  scanx [i] = scany [i] = scanpix [i] = 0.0;

   /*** Check scan size. ***/

   ierr = 0;
   *ns = max (abs (ip2 - ip1), abs (ir2 - ir1)) + 1;

   /*** Orient start/end points so that start point is closest to disk. ***/
   /*   (radial scans only) */

   pix1 = (float) ip1;
   rec1 = (float) ir1;
   pix2 = (float) ip2;
   rec2 = (float) ir2;

   /*** Leave start/end points as given. ***/
 
   ix1 = ip1;
   ix2 = ip2;
   iy1 = ir1;
   iy2 = ir2;

   /*** Equation of a line:   y = slope * x + yoff. ***/

   x1 = (float) ix1;
   x2 = (float) ix2;
   y1 = (float) iy1;
   y2 = (float) iy2;
   slope = -1.0e+9;
   yoff = 0.0;
   if (ix1 != ix2)  slope = (y2 - y1) / (x2 - x1);
   if (ix1 != ix2)  yoff  = (x2*y1 - x1*y2) / (x2 - x1);

/*
   printf("ix1,iy1,ix2,iy2: %d %d %d %d\n", ix1, iy1, ix2, iy2);
   printf("slope,yoff: %f %f\n", slope, yoff);
 */

   if (fabs ((float) slope) < 1.0)
      {
/*      printf ("column-driven scan\n"); */

      /* ......column-driven loop : */

      nxdif = ix2 - ix1;
      if (nxdif < 0)  nxdif = -nxdif;
      nx = nxdif + 1;
      if (nx > *ns)
         {
         fprintf (stderr, "clscan> error...# scan values:%d > ns:%d\n",
                  nx, *ns);
         return (-1);
         }

      ix = ix1 - 1;
      ixinc = 1;
      if (ix1 > ix2)
         {
         ix = ix1 + 1;
         ixinc = -1;
	 }

      for (i = 0; i < nx; i++)
         {
         ix = ix + ixinc;
         x = (float) ix;
         y = (float) iy1;
         if (iy1 != iy2)  y = slope * x + yoff;
         iy = (int) y;

         /* 1 byte per pixel:
          *  clear all high order bits above low-order byte.
          */

         if (nbypx == 1)
            {
            ival1 = (*(imc + (ix-ip) + (iy-ir)   * npix) & 0377);
            ival2 = (*(imc + (ix-ip) + (iy-ir+1) * npix) & 0377);
            }
         else

         /*** 2 bytes per pixel. ***/

         if (nbypx == 2)
            {
            ival1 = *(ims + (ix-ip) + (iy-ir)   * npix) & 0177777;
            ival2 = *(ims + (ix-ip) + (iy-ir+1) * npix) & 0177777;
            }

         /*** Linear interpolation in vertical axis. ***/

         scanpix [i] = (float) (ival1 + (ival2-ival1) * (y-iy));
         scanx [i] = ix;
         scany [i] = iy;

/*
 *         if (igterm == 0)
 *            {
 *	      if (idtype == 3)
 *	      printf ("ix:%3d iy:%3d r:%7.3f th:%7.2f length(Rsun):%7.3f Bsun:%9.2e\n",
 *                    ix, iy, r, th, dstsun, scanpix [i]);
 *	      else
 *	      printf ("ix:%3d iy:%3d r:%7.3f th:%7.2f length(Rsun):%7.3f intensity:%7.2f\n",
 *                    ix, iy, r, th, dstsun, scanpix [i]);
 *	      }
 */

         /*** Plot 2 nearest points on IRIS in graphics overlay. ***/

/*
         if (igr == 1)
            {
            ixg = ix/nres + 63;
            iyg = 510 - iy/nres;
	    gl_wvec (ixg, iyg, ixg, iyg+1, ov_red);
	    }
 */
         }
      *ns = nx;
      faverage (nave, scanpix, *ns);
      }

   else

      {

      /* ...... row-driven loop : */

/*       printf ("row-driven scan\n"); */

      nydif = iy2 - iy1;
      if (nydif < 0)  nydif = -nydif;
      ny = nydif + 1;
      if (ny > *ns)
         {
         fprintf (stderr, "clscan> error...# scan values:%d > ns:%d\n",
                  ny, *ns);
         return (-2);
         }

      iy = iy1 - 1;
      iyinc = 1;
      if (iy1 > iy2)
         {
         iy = iy1 +1;
         iyinc = -1;
         }

      for (i = 0; i < ny; i++)
         {
         iy = iy + iyinc;
         x = (float) ix1;
         y = (float) iy;
         if (ix1 != ix2)  x = (y-yoff) / slope;
         ix = (int) x;

         /*** Extract 2 adjacent pixels (in same line) from image. ***/

         /* 1 byte per pixel:  clear all high order bits above low-order byte.*/

         if (nbypx == 1)
            {
            ival1 = (*(imc + (ix-ip)   + (iy-ir) * npix) & 0377);
            ival2 = (*(imc + (ix-ip+1) + (iy-ir) * npix) & 0377);
            }
         else

         /* 2 bytes per pixel. */

         if (nbypx == 2)
            {
            ival1 = *(ims + (ix-ip)   + (iy-ir) * npix) & 0177777;
            ival2 = *(ims + (ix-ip+1) + (iy-ir) * npix) & 0177777;
            }

         /*** Linear interpolation in horizonal axis. ***/

         scanpix [i] = (float) (ival1 + (ival2-ival1) * (x-ix)) ;
         scanx [i] = ix;
         scany [i] = iy;

/*
 *         if (igterm == 0)
 *            {
 *            if (idtype == 3)
 *               printf ("ix:%3d iy:%3d r:%6.2f th:%7.2f length(Rsun):%6.2f Bsun:%9.2e\n",
 *                       ix, iy, r, th, dstsun, scanpix [i]);
 *            else
 *               printf("ix:%3d iy:%3d r:%6.2f th:%7.2f length(Rsun):%7.2f intensity:%7.2f\n",
 *                       ix, iy, r, th, dstsun, scanpix [i]);
 *	      }
 */

         /*** Plot 2 nearest points on display in graphics overlay. ***/

/*
         if (igr == 1)
            {
            ixg = ix/nres + 63;
            iyg = 510 - iy/nres;
            gl_wvec (ixg, iyg, ixg+1, iyg, ov_red);
            }
 */
         }
      *ns = ny;
      faverage (nave, scanpix, *ns);

      }
   return (ierr);
}
