/* scloc.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate position of calibration reticle scratch mark.
 * First, extract a nearly diagonal image segment (containing the left
 * scratch mark) into a rectangular buffer consisting of horizontal stair steps.
 * Then, examine each horizontal line, and locate the brightest pixel in
 * each line.
 * The line in which the 4th, 5th & 6th pixels to the right of the brightest
 * pixel should be the line representing the middle flag in the "F" scratch
 * mark pattern.
 * This should then identify the position of this point on the scratch pattern.
 * Then the coordinates used to interpolate over the scratch mark may be
 * adjusted to correct for registration errors.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  9 October 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

scloc (imgbuf, xsize, ysize, scx, scy)

unsigned short *imgbuf;	/* Image buffer. */
int xsize, ysize;	/* Image dimensions. */
int *scx, *scy;		/* Location of "F" scratch small flag intersection. */

{
   int dx = 20, dy = 30;	/* Dimensions of subset image buffer. */
   int ix, iy;			/* Indices. */
   int ixmax = 0, iymax = 0;
   int maxval;
   int pixval;
   int x = 311, y = 189;	/* Lower left corner of "diagonal" box. */
   int xx, yy;			/* Index. */
   int xxmax, yymax;
   int ival;
   int ierr;
   unsigned short *dimg;	/* Character pointer to diagonal image array. */
   float val0;
   float val5, val6, val7;
   float rat5, rat6, rat7;
   float slope = 0.85;		/* Approximate slope of scratch mark. */
   float thresh = 0.50;		/* Threshold for pixel intensity ratio. */

   ixmax = iymax = xxmax = yymax = 0;
   *scx = *scy = 0;

   /* Allocate storage for rectangular area of image.
    * Read from `imgbuf' into memory array `dimg'.
    */

   dimg = (unsigned short *) malloc (dx * dy * sizeof (short));

   /*** Read a r->l, d->u diagonal image segment into a rectangular array. ***/

   yy = y - 1;

   for (iy = 0; iy < dy; iy++)
      {
      yy += 1;
      xx = x + iy - 1;
      maxval = 0;

      for (ix = 0; ix < dx; ix++)
         {
         xx = (int) ((float) x + (float) ix / slope);
         pixval = *(dimg + ix + iy*dx) = *(imgbuf + xx + yy * xsize) - 512;

/*
 *         printf ("scloc> ix: %3d   iy: %3d   xx: %3d  yy: %3d\n",
 *                 ix, iy, xx, yy);
 */

         /*** Find brightest pixel in each line. ***/

         if (pixval > maxval)
            {
            maxval = pixval;
            ixmax = ix;  iymax = iy;
            xxmax = xx;  yymax = yy;
            }
         }
      printf ("\nmaxval: %3d  ixmax:%3d  iymax:%3d  xxmax:%3d  yymax:%3d\n",
              maxval, ixmax, iymax, xxmax, yymax);

      /*** Check magnitude of pixels to the right of the brightest pixel. ***/
       
      val0 = (float) maxval;
      val5 = (float) *(dimg + ixmax + 5 + iy*dx);
      val6 = (float) *(dimg + ixmax + 6 + iy*dx);
      val7 = (float) *(dimg + ixmax + 7 + iy*dx);
      rat5 = val5 / val0;
      rat6 = val6 / val0;
      rat7 = val7 / val0;
      if (rat5 > thresh && rat6 > thresh && rat7 > thresh)
         {
         printf ("F intersect: ixmax:%3d   iymax:%3d   xxmax:%3d   yymax:%3d\n",
                 ixmax, iymax, xxmax, yymax);
         printf ("Pixel values: val0:%5.2f  val5:%5.2f  val6:%5.2f  val7:%5.2f\n",
                 val0, val5, val6, val7);
         printf ("ratios: pix5: %5.2f  pix6: %5.2f  pix7: %5.2f\n",
                 rat5, rat6, rat7);
      *scx = xxmax;
      *scy = yymax;
         }
      }


   free (dimg);
   return (0);
}
