/* cpmass.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * ****** Absolute/Excess mass calculation ******* ******* ******* *******
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * NCAR/HAO   SMM C/P
 * 26 September 1984
 * 21 December 1984: Add coronal brightness threshold.
 * 31 October 1985: Apply Munro calibration correction.
 *  1 November 1985: Use cphskp & hkline.
 * 12 July 1990: Remove stray light, vignetting, & F-corona (if required).
 * 20 July 1990: For excess mass, no F-corona subtraction is needed.
 * 15 November 1991: Eliminate cbthresh.
 *- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/whip/ov_color.ext"
#include "cpwedge.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

cpmass (namref, namcmp, mx, cbth, pgmout)

char	namref [];	/* Background image file name. */
char	namcmp [];	/* Comparison image file name. */
int	mx;		/* Mass: =1: CMP-REF; otherwise, Absolute: CMP. */
float	cbth;		/* Coronal brightness threshold (B/Bsun).*/
FILE *pgmout;		/* Output destination. */

{
#define	MAXPIX	896
#define RSUN 6.96e+10	/* Solar radius (cm). */
extern  int    cphskp(), hkline();/* Housekeeping decommutation & display. */
extern	int    min (), max ();	/* Integer min & max functions. */
extern	float  xmin (), xmax ();/* Specialized float min & max functions. */
extern	float  stray ();	/* Declare stray light function. */
extern	float  vignet ();	/* Declare vignetting function. */
extern	float  fcorona ();	/* F-corona function. */
extern	float  calcor ();	/* Munro calibration correction function. */
extern	double medsun ();	/* Mass density (grams/cm2) per 1.0 Bsun. */

static char	answer[2];	/* Storage for y/n response. */
char	*rbuf, *cbuf;

static short	irbuf [MAXPIX];	/* Line buffer for reference image. */
static short	icbuf [MAXPIX];	/* Line buffer for comparison image. */

int	cmpfd, reffd;	/* File descriptors for CMP & REF images. */
int	ctype;		/* Data type for comparison image. */
int	i;		/* General loop index. */
int	iix, iiy;	/* Index variables for pixel and line. */
int	iixb;		/* X-axis index for REF image. */
int	ipc,irc,lpc,lrc;/* Limits for CMP image. */
int	ipb,irb,lpb,lrb;/* Limits for REF image. */
int	irad;		/* Index for elec array. */
int	ixs, iys;	/* Sun center shift of REF image relative to CMP. */
int	ixcm, iycm;	/* Center of mass coordinates. */
int	ixcmg, iycmg;	/* Center of mass coordinates for display. */
int	ixg, iyg;	/* Display coordinates of integrated pixel. */
int	ix11,ix12,ix21,ix22;	/* Pixel limits for wedge. */
int	iy11,iy12,iy21,iy22;	/* Line limits for wedge. */
int	maxdat;
int	mv;		/* Vignetting flag: =1 vignetting has been removed. */
int	n;		/* temporary debug variable. */
int	nbyrec;		/* Number of bytes per record. */
int	num, numun;	/* Number of values in kmass & unmass. */
int	numneg;		/* Number of negative values in integration. */
int	numth;	       /* Number of values above 'cpth' brightness threshold. */
int	rtype;		/* Data type for reference image. */
int	xbeg, ybeg;	/* Search box pixel & record limits. */
int	xc1box, yc1box;	/* Initial pixel,record for CMP search box. */
int	xc2box, yc2box;	/* Last pixel,record for CMP search box. */
int	xr1box, yr1box; /* Initial pixel,record for REF search box. */
int	xr2box, yr2box; /* Last pixel,record for REF search box. */
int	xend, yend;	/* Search box pixel & record limits. */
int	xocp = 63;	/* C/P x-coordinate origin. */
int	yocp = 510;	/* C/P y-coordinate origin. */

static float	xbbuf [MAXPIX*MAXPIX];	/* Data buffer: coronal brightness. */
static float	xmbuf [MAXPIX*MAXPIX];	/* Data buffer: K-corona mass. */
float	bcmp;		/* Brightness of comparison image pixel. */
float	bref;		/* Brightness of reference  image pixel. */
float	cbrite;		/* Coronal mass. */
float	cbsum;		/* Integrated coronal mass. */
float	cbthresh;	/* Threshold level (B/Bsun). */
float	cc;		/* Munro calibration correction factor. */
float	cm2pix;		/* Area (cm**2) per pixel. */
float	cmpmass;	/* Comparison image mass. */
float	cxcen, cycen;	/* Sun center for comparison image. */
float	fx, fy;		/* Floating variables for iix & iiy. */
float	ibcmp;		/* Integrated coronal mass above a threshold level. */
float	ibref;		/* Integrated coronal mass above a threshold level. */
float	kbrite;		/* Integrated absolute or excess brightness value. */
float	kmass;		/* Integrated absolute or excess mass value. */
float	masspct;	/* % mass change. */
float	negmass;	/* Negative mass. */
float	negmassp;	/* Negative mass percentage. */
float	negpct;		/* % Mass which is negative. */
float	p11,p12,p21,p22;	/* Pixel limits for wedge. */
float	r11,r12,r21,r22;	/* Line limits for wedge. */
float	r, theta;	/* Radius & theta. */
float	rcm, thcm;	/* (r,th) coordinates of center-of-mass. */ 
float	refmass;	/* Reference image mass. */
float	rxcen, rycen;	/* Sun center for reference image. */
float	str, vig, fc;	/* Stray light, vignetting & F-corona  values. */
float	xcm, ycm;	/* (x,y) coordinates of center-of-mass. */
float	unmass;		/* Mass from unreliably low differences. */
float	unmassp;	/* % Mass from unreliably low differences. */
float	unpct;		/* Percentage from unreliably low differences. */
float	wtheta;		/* Wedge theta coordinate. */

double	gcm2;		/* grams per cm**2 along line of sight. */
double	Ne;		/* # electrons. */

static float difunr = 5.0e-10;	/* Threshold for unreliable differences. */
static double emass = 2.0e-24;	/* Mass per light-scattering electron. */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   rbuf = (char *) irbuf;
   cbuf = (char *) icbuf;
   maxdat = MAXPIX * MAXPIX;
   cbthresh = cbth;
   ixs = 0;
   iys = 0;

   /*** Open reference image file. ***/

   if (mx > 0)
      {
      if ((reffd = open (namref, 0)) < 0)
         { fprintf (stderr,"cpmass> Can't open file %s\n", namref);  return (-1); }

      /*** Decommutate housekeeping for reference image. ***/

      cphskp (namref);
      rxcen = xcen;
      rycen = ycen;
      rtype = idtype;
      nbyrec = npix * nbypix;
      ipb = ip;
      irb = ir;
      lpb = lp;
      lrb = lr;
      }

   /*** Decommutate housekeeping for comparison image. ***/

   cphskp (namcmp);

   cxcen = xcen;
   cycen = ycen;
   ctype = idtype;
   nbyrec = npix * nbypix;
   ipc = ip;
   irc = ir;
   lpc = lp;
   lrc = lr;

/*
 *   fprintf(stderr, "cxcen:%-6.2f cycen:%-6.2f nbypix:%1d npix:%3d\n",
 *                   cxcen, cycen, nbypix, npix);
 */

   /*** Verify that the image has been calibrated. ***/

   if (((mx > 1) && (rtype != 3)) || (ctype != 3 && ctype !=6))
      {
      fprintf (stderr,
              "cpmass> Image(s) not calibrated ... excess mass not done.\n");
      fprintf (pgmout,
              "cpmass> Image(s) not calibrated ... excess mass not done.\n");
      return (-2);
      }

   /*** Request sun center coordinates for comparison image. ***/

   if (cxcen <= 0.0 || cycen <= 0.0)
     {
      fprintf (stderr, "Enter sun center coordinates (hi res) for CMP image : ");
      fscanf  (stdin,  "%f,%f", &cxcen, &cycen);
      cxcen = cxcen / nres;
      cycen = cycen / nres;
     }

   /*** Request sun center coordinates for reference image. ***/

   if (mx > 0)
      {
      if (rxcen <= 0.0 || rycen <= 0.0)
        {
         fprintf (stderr,
                  "Enter sun center coordinates (hi res) for REF image : ");
         fscanf  (stdin,  "%f,%f", &rxcen, &rycen);
         rxcen = rxcen / nres;
         rycen = rycen / nres;
         if (rxcen <= 0.0)  rxcen = cxcen;
         if (rycen <= 0.0)  rycen = cycen;
        }
      ixs = cxcen - rxcen;
      iys = cycen - rycen;
      }

   /*** Stray Light & Vignetting removal required ? ***/

   if (lstr == 2 && lvig == 2)  mv = 1;  else mv = 0;

   fprintf (stderr, "Please wait ... mass computation in progress.\n");

   /*** Open comparison image file. ***/

   if ((cmpfd = open (namcmp, 0)) < 0)
      { fprintf (stderr,"cpmass> Can't open file %s\n", namcmp);  return (-3); }

   /*** Compute the cross-sectional area (square cm) for one pixel. ***/

   cm2pix = (RSUN*rspix) * (RSUN*rspix);

   /*** Convert wedge corners to cartesian coordinates. ***/

   if (rcoord (&wrmin, &wthmin, &p11, &r11, 1, scroll) != 0)  goto geoerr;
   if (rcoord (&wrmin, &wthmax, &p12, &r12, 1, scroll) != 0)  goto geoerr;
   if (rcoord (&wrmax, &wthmin, &p21, &r21, 1, scroll) != 0)  goto geoerr;
   if (rcoord (&wrmax, &wthmax, &p22, &r22, 1, scroll) != 0)  goto geoerr;

   /*** Define a rectangular box which contains the wedge. ***/

   xbeg = (int) xmin (p11, p12, p21, p22);
   xend = (int) xmax (p11, p12, p21, p22);
   ybeg = (int) xmin (r11, r21, r12, r22);
   yend = (int) xmax (r11, r21, r12, r22);

   /*** Orient the starting/ending pixels/records in increasing order. ***/

   /*** Comparison image. ***/

   xc1box = min (xbeg, xend);
   yc1box = min (ybeg, yend);
   xc2box = max (xbeg, xend);
   yc2box = max (ybeg, yend);

   /*** Background image. ***/

   if (mx > 0)
     {
      xr1box = xc1box - ixs;
      yr1box = yc1box - iys;
      xr2box = xc2box - ixs;
      yr2box = yc2box - iys;

/*
 *      fprintf (stdout,
 *               "ipb:%3d irb:%3d lpb:%3d lrb:%3d\n", ipb, irb, lpb, lrb);
 *      fprintf (stdout, "xr1box:%3d yr1box:%3d xr2box:%3d yr2box:%3d\n",
 *                        xr1box, yr1box, xr2box, yr2box);
 */

   /*** Verify that REF search box limits do not lie outside REF image. ***/

      if (xr1box < ipb || xr1box > lpb || xr2box < ipb || xr2box > lpb)
         {
         fprintf (stderr, "cpmass> wedge outside REF image boundary (x).\n");
         return (-4);
         }

      if (yr1box < irb || yr1box > lrb || yr2box < irb || yr2box > lrb)
         {
         fprintf (stderr, "cpmass> wedge outside REF image boundary (y).\n");
         return (-5);
         }
      }

   /*** Initialize variables. ***/

   cc    = pscale;
   fc    = 0.0;
   str   = 0.0;
   vig   = 1.0;
   Ne    = 0.0;
   cmpmass = 0.0;
   kbrite = 0.0;
   kmass  = 0.0;
   negmass = 0.0;
   refmass = 0.0;
   unmass = 0.0;
   xcm   = 0.0;
   ycm   = 0.0;

   num   = 0;
   numneg = 0;
   numun = 0;
   numth = 0;

   fprintf (stdout, "ixs: %3d iys: %3d\n", ixs, iys);
   fprintf (stdout, "wrmin:%7.2f  wrmax:%7.2f  wthmin:%7.2f  wthmax:%7.2f\n",
	    wrmin, wrmax, wthmin, wthmax);
   fprintf (stdout, "p11:%7.2f  p12:%7.2f  p21:%7.2f  p22:%7.2f\n", p11, p12, p21, p22);
   fprintf (stdout, "r11:%7.2f  r12:%7.2f  r21:%7.2f  r22:%7.2f\n", r11, r12, r21, r22);
   fprintf (stdout, "xc1box:%3d  xc2box:%3d  yc1box:%3d  yc2box:%3d\n",
            xc1box, xc2box, yc1box, yc2box);

   /*** Clear data buffers. ***/

   for (i=0; i<MAXPIX; i++)
      {
      irbuf [i] = 0;
      icbuf [i] = 0;
      }
   for (i=0; i<maxdat; i++)  xbbuf [i] = xmbuf [i] = 0.0;

   /*** Position image file pointer to beginning of box containing wedge. ***/

   if (mx > 0)
      {
      if (lseek (reffd, (long) (npix*nbypix*(yr1box+2)), 0) == -1)
         { fprintf (stderr, "cpmass> seek error.\n");  return (-6); }
      }

   if (lseek (cmpfd, (long) (npix*nbypix*(yc1box+2)), 0) == -1)
      { fprintf (stderr, "cpmass> seek error.\n");  return (-7); }

   /* ******* Record Loop ******* */

   i = -1;
   num = 0.;
   numth = 0;
   cbsum = 0.0;
   ibref = 0.0;
   ibcmp = 0.0;
   for (iiy = yc1box; iiy <= yc2box; iiy++)
      {
/*      fprintf (stderr, "cpmass...line # %3d\n", iiy); */
      fy = (float) iiy;

      if (mx > 0)
         {
         if (read (reffd, rbuf, nbyrec) <= 0)
            { fprintf (stderr,"cpmass> REF image read error.\n");  return (-8); }
         if (bytord)  srevbyt (rbuf, 2, npix);
         }

      if (read (cmpfd, cbuf, nbyrec) <= 0)
         { fprintf (stderr, "cpmass> CMP image read error.\n");  return (-9); }
      if (bytord)  srevbyt (cbuf, 2, npix);

      /* ******* Pixel Loop ******* */

      for (iix = xc1box; iix <= xc2box; iix++)
         {
         fx = (float) iix;
         if (rcoord (&r, &theta, &fx, &fy, -1, scroll) != 0)  goto geoerr;

/* fprintf (stdout,
 *          "fx:%6.2f fy:%6.2f  r:%6.2f theta:%6.2f\n", fx, fy, r, theta);
 */

         if (wthmax > 360.0 && theta < wthmin)
            wtheta = theta + 360.0;
         else
            wtheta = theta;

         if (r < wrmin || r > wrmax || wtheta < wthmin || wtheta > wthmax)
            goto pixend;

         /*** Draw dot to indicate pixel has been integrated. ***/

         ixg = (int) (fx/nres + xocp);
         iyg = (int) (yocp - fy/nres);
/*         gl_wvec (ixg, iyg, ixg, iyg, ov_blue); */

/* fprintf (stdout, "fx:%6.2f fy:%6.2f . . . passed r,th test.\n", fx, fy); */

         i++;
         num++;
         xbbuf [i] = xmbuf [i] = 0.0;

         irad = (int) ((r - 1.25) / 0.25);
	 gcm2 = medsun (r);

         /*** Use external stray/vignet functions ***/

         if (!H.vig)  vig = vignet (r);
         if (!H.str)  str = stray (r);
	 if (!H.fc )  fc  = fcorona (r);
         if (lcc)     cc  = calcor (r) * pscale;

         /*** Absolute mass: Remove stray light from comparison image. ***/

         if (mx  ==  0)
            {
            cbrite = (((float) icbuf [iix] * cc - str) * vig ) - fc;
/*
 *          fprintf (stdout,
 *             "%3d   icbuf[%3d]:%3d   str:%8.2e   vig:%8.2e   cbrite: %8.2e\n",
 *             iiy, iix, icbuf [iix], str, vig, cbrite);
 */
	    cbsum += cbrite;
/*
 *	    if (cbrite > cbthresh)
 */
               {
               numth++;
               ibcmp += cbrite;
               cmpmass += cbrite * gcm2 * cm2pix * cbrite;
               }
            xbbuf [i] = cbrite;
            xmbuf [i] = (float) (cbrite * gcm2 * cm2pix);
/*
 *	    fprintf (stdout,
 *           "gcm2: %8.2e   cm2pix: %8.2e   cbrite: %8.2e   xmbuf [%d]: %8.2e\n",
 *		     gcm2, cm2pix, cbrite, i, xmbuf [i]);
 */
            }

         /*** Excess mass: compute pixel coordinate for reference image. ***/

         else
            {
            iixb = iix - ixs;
            cbrite = (float) (icbuf [iix] - irbuf [iixb]) * cc * vig;
	    cbsum += cbrite;
/*
 *	    if (cbrite > cbthresh)
 */
               {
               numth++;
               bref = (((float) irbuf [iixb] * cc - str) * vig) - fc;
               bcmp = (((float) icbuf [iix]  * cc - str) * vig) - fc;
               ibref += bref;
               ibcmp += bcmp;
               refmass += bref * gcm2 * cm2pix;
               cmpmass += bcmp * gcm2 * cm2pix;
               }
            xbbuf [i] = cbrite;
            xmbuf [i] = (float) (cbrite * gcm2 * cm2pix);
            }

/* fprintf (stdout, "gcm2:%g cm2pix:%g v:%g\n", gcm2, cm2pix, v); */
/* fprintf (stdout, "icbuf:%-5d irbuf:%-5d\n", icbuf [iix], irbuf [iixb]); */
/* fprintf (stdout, "line:%d xmbuf [%d]: %g\n", iiy, iix, xmbuf [i]); */

         kbrite += xbbuf [i];
         kmass  += xmbuf [i];

         if (mx > 0)
            {
            if (xbbuf [i] < difunr && xbbuf [i] > -difunr)
               {
               unmass += xmbuf [i];
               numun++;
               }
            if (xmbuf [i] < 0.0)
               {
               negmass += xmbuf [i];
               numneg++;
               }
            }

         ycm += xmbuf [i] * fy;
         xcm += xmbuf [i] * fx;

pixend:    n++;		/* fprintf (stdout, "pixend...iix%3d\n", iix); */

         } /* End pixel loop. */

recend: n++;		/* fprintf (stdout, "recend...iiy:%3d\n", iiy); */

      } /* End record loop. */

   Ne = (double) kmass / emass;

   /*** Center of mass. ***/

   if (kmass  !=  0.0)
      {
      ixcm = (int) (xcm / kmass);
      iycm = (int) (ycm / kmass);
      }

   xcm = (float) ixcm;
   ycm = (float) iycm;

   if (rcoord (&rcm, &thcm, &xcm, &ycm, -1, scroll) != 0)  goto geoerr;
   fprintf (stdout, "rcm: %7.2f   thcm: %7.2f   xcm: %7.2f   ycm: %7.2f   scroll: %d\n",
	            rcm, thcm, xcm, ycm, scroll);

   /*** Draw "+" on display to indicate center of mass location. ***/
   /*** Draw lines connecting 4 wedge corners. ***/

   if (igr == 1)
      {
      ixcmg = ixcm/nres + xocp;
      iycmg = yocp - iycm/nres;
      gl_wvec (ixcmg-5, iycmg, ixcmg+5, iycmg, ov_red);
      gl_wvec (ixcmg, iycmg-5, ixcmg, iycmg+5, ov_red);

      ix11 = (int) p11 / nres + xocp;
      ix12 = (int) p12 / nres + xocp;
      ix21 = (int) p21 / nres + xocp;
      ix22 = (int) p22 / nres + xocp;
      iy11 = yocp - (int) r11 / nres;
      iy12 = yocp - (int) r12 / nres;
      iy21 = yocp - (int) r21 / nres;
      iy22 = yocp - (int) r22 / nres;
      }

/*   fprintf (pgmout, "F-corona has NOT been removed.\n"); */
   if (lcc)
      fprintf (pgmout,
              ">>> NOTE: Munro calibration correction has been applied. <<<\n");
   fprintf (pgmout,
      "Wedge area : %4.2f - %4.2f Rsun & %6.2f - %-6.2f degrees",
          wrmin, wrmax, wthmin, wthmax);
   fprintf (pgmout, "   # pixels : %d\n\n", num);

   if (mx)
      {
      masspct = ((cmpmass - refmass) / refmass) * 100.0;
      fprintf (pgmout, "Comparison image mass :   %9.2e grams\n",
               cmpmass);
      fprintf (pgmout, "Reference  image mass :   %9.2e grams\n",
               refmass);
   fprintf (pgmout,
      "[Integrated excess mass] :%9.2e grams <->%9.2e electrons: change %6.2f%%\n\n",
       kmass, Ne, masspct);
      }
   else
   fprintf (pgmout,
      "[Total integrated absolute mass] :%9.2e grams <->%9.2e electrons.\n\n",
       kmass, Ne);

   if (mx == 0)
      {
/*
 *      if (cbthresh > 0.0)
 */
         fprintf (pgmout,
                  "[Brightness > %9.2e B/Bsun] (%d) :  %9.2e B/Bsun\n\n",
                  cbthresh, numth, ibcmp);
      fprintf (pgmout,
               "[Integrated absolute brightness] : %9.2e B/Bsun\n\n",
               cbsum);
      }

   if (mx  > 0)
      {
      fprintf (pgmout, "[Cmp. brightness] (%d pixels) : %9.2e B/Bsun\n",
              numth, ibcmp);
      fprintf (pgmout, "[Bkg. brightness] (%d pixels) : %9.2e B/Bsun\n",
              numth, ibref);
      fprintf (pgmout,
               "[Integrated excess brightness] :   %9.2e B/Bsun\n\n",
               cbsum);

      unpct = ( (float) numun / (float) num ) * 100.0;
      negpct = ( (float) numneg / (float) num) * 100.0;
      unmassp = (unmass / kmass) * 100.0;
      negmassp = (negmass / kmass) * 100.0;
      fprintf (pgmout,
              "Unreliably low points [%5.2f%% (%d/%d) with -%9.2e < B <%9.2e]\n",
               unpct, numun, num, difunr, difunr);
      fprintf (pgmout, "contributed %9.2e grams [%5.2f%%] to the total.\n\n",
               unmass, unmassp);

      fprintf (pgmout,
              "Negative mass points [%5.2f%% (%d/%d) with B < 0.0]\n",
               negpct, numneg, num);
      fprintf (pgmout, "contributed %9.2e grams [%5.2f%%] to the total.\n\n",
               negmass, negmassp);

      }

   fprintf (pgmout, "The center of mass within this area is at %5.2f Rsun   %7.2f degrees.\n",
      rcm, thcm);
   fprintf (pgmout, "--------------------------------------------------------------------------------\n");

   if (mx > 0)  close (reffd);
   close (cmpfd);
   return (0);

geoerr: fprintf (stderr, "Invalid range: radius or theta");
        return (-10);
}

/* xmin (x1, x2, x3, x4)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find minimum of 4 floating point variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * 1 March 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

float
xmin (x1, x2, x3, x4)

float x1, x2, x3, x4;

{
float x;

   x = x1;
   if (x > x2)  x = x2;
   if (x > x3)  x = x3;
   if (x > x4)  x = x4;

   return (x);
}

/* xmax (x1, x2, x3, x4)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find maximum of 4 floating point variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * 1 March 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

float
xmax (x1, x2, x3, x4)

float x1, x2, x3, x4;

{
float x;

   x = x1;
   if (x < x2)  x = x2;
   if (x < x3)  x = x3;
   if (x < x4)  x = x4;

   return (x);
}
