/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * polyfill : fill polygon.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * A set of input (x,y) coordinates (polygon vertices) is used to generate
 * a perimeter boundary, sorted according to increasing y coordinate,
 * and filled with a specified overlay color.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * WHIP Region software is used to perform the following functions:
 * 1. makeov: scan conversion.
 * 2. drawov: draw perimeter into overlay in WHIP frame buffer.
 * 3. yxsort: sorts coordinates (x,y) according to ascending y order.
 * 4. filtb:  eliminates singularities & redundant points.
 * 5. fill:   fills bounded area with a specified color index value.
 *            also used to store coordinates of all interior pixels into a file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 18 Nov 1991
 * 19 June 1992: add `filtb' command.
 *  7 August 1992: change namcpv extension to "pvX", where X = region number.
 * 10 August 1992: add error check for system calls (makeov, drawov, etc).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
extern int regnum;			/* Region index number. */

short *
polyfill (namimg, xocp, yocp, poly, nver, cmap, nwp, ierr)

char namimg [];			/* Program name. */
int xocp, yocp;			/* C/P image origin. */
int poly [][2];			/* Polygon vertex array. */
int nver;			/* # vertices. */
int cmap;			/* Color map index. */
int *nwp;			/* # pixels inside polygon. */
int ierr;			/* Error status. */

{
   short *wpos;			/* pixel array pointer. */
   static char namcpv [80];	/* C/P vertex file. */
   static char namver [80];	/* Window vertex file. */
   static char namper [80];	/* Perimeter file. */
   static char namyx  [80];	/* yxsort file. */
   static char namfil [80];	/* Fill file. */
   static char op [80];		/* String for system operation. */

   int xcp, ycp;		/* X/Y coordinates in C/P format. */
   int xwin, ywin;		/* X/Y coordinates in window format. */
   int i;			/* General purpose index. */
   int ipix;			/* Pixel index. */

   FILE *fd;			/* File descriptor. */
   FILE *fdver;			/* Vertex file descriptor. */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   sprintf (namcpv, "%s.pv%d", namimg, regnum);
   fprintf (stderr, "namcpv: %s   regnum: %d\n", namcpv, regnum);

   while ((fd = fopen (namcpv, "r")) != NULL)
      {
      fprintf (stderr, "file : %s exists.  fd: %s\n\n", namcpv, fd);
      fclose (fd);
      regnum++;
      sprintf (namcpv, "%s.pv%d", namimg, regnum);
      fprintf (stderr, "namcpv: %s   regnum: %d\n", namcpv, regnum);
      }

   sprintf (namver, "%s.ver%d.ov", namimg, regnum);
   sprintf (namper, "%s.per%d.ov", namimg, regnum);
   sprintf (namyx,  "%s.yx%d.ov",  namimg, regnum);
   sprintf (namfil, "%s.fil%d.ov", namimg, regnum);

   unlink (namcpv);
   unlink (namver);
   unlink (namper);
   unlink (namyx);
   unlink (namfil);

   color (cmap);		/* Set current color for drawing. */

/*   Fill polygon using SGI GL function (v2i).
 *
 *   bgnpolygon ();
 *      for (i=0; i<nver; i++)  v2i (poly [i]);
 *      v2i (poly [0]);
 *   endpolygon ();
 */

   if ((fd = fopen (namcpv, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namcpv);
      return (NULL);
      }

   if ((fdver = fopen (namver, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namver);
      return (NULL);
      }

   /*** Write polygon vertices to vertex file. ***/

   for (i = 0; i < nver; i++)
      {
      xcp = poly [i][0];
      ycp = poly [i][1];

      xwin = xcp + xocp;
      ywin = yocp - ycp;

      fprintf (fd,    "%3d %3d\n", xcp,  ycp);
      fprintf (fdver, "%3d %3d\n", xwin, ywin);
      }

   xcp = poly [0][0];
   ycp = poly [0][1];

   xwin = xcp + xocp;
   ywin = yocp - ycp;

   fprintf (fd,    "%3d %3d\n", xcp,  ycp);	/* Repeat first vertex. */
   fprintf (fdver, "%3d %3d\n", xwin, ywin);	/* Repeat first vertex. */

   fclose (fd);
   fclose (fdver);

   /*** Open/close perimeter file. ***/

   if ((fd = fopen (namper, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namper);
      return (NULL);
      }
   fclose (fd);

   /*** Open/close yxsort file. ***/

   if ((fd = fopen (namyx, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namyx);
      return (NULL);
      }
   fclose (fd);

   /*** Open/close fill file. ***/

   if ((fd = fopen (namfil, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namfil);
      return (NULL);
      }
   fclose (fd);

   /*** Execute "makeov" command (WHIP): scan conversion between vertices. ***/

   sprintf (op, "makeov < %s > %s", namver, namper);
   ierr = system (op);
   fprintf (stdout, "makeov done\n");
   if (ierr)
      {
      fprintf (stderr, "*** makeov error: %d\n", ierr);
      regnum--;
      remfil (namcpv, namver, namper, namyx, namfil);
      return (NULL);
      }

   /*** Execute "drawov" command (WHIP): draws perimeter overlay into FB. ***/

   sprintf (op, "drawov -g %d %s", cmap, namper);
   ierr = system (op);
   fprintf (stdout, "drawov done\n");
   if (ierr)
      {
      fprintf (stderr, "*** drawov error: %d\n", ierr);
      regnum--;
      remfil (namcpv, namver, namper, namyx, namfil);
      return (NULL);
      }

   /*** Execute "yxsort" command (WHIP): sorts coordinates by ascending Y.***/

   sprintf (op, "yxsort %s %s", namper, namyx);
   ierr = system (op);
   fprintf (stdout, "yxsort done\n");
   if (ierr)
      {
      fprintf (stderr, "*** yxsort error: %d\n", ierr);
      regnum--;
      remfil (namcpv, namver, namper, namyx, namfil);
      return (NULL);
      }

   /*** Execute "filtb" command (WHIP): eliminates singularities. ***/

/*
 *   sprintf (op, "filtb %s", namyx);
 *   system (op);
 *   fprintf (stdout, "filtb done\n");
 *   if (ierr)
 *      {
 *      fprintf (stderr, "*** filltb error: %d\n", ierr);
 *      regnum--;
 *      remfil (namcpv, namver, namper, namyx, namfil);
 *      return (NULL);
 *      }
 */

   /*** Execute "fill" command (WHIP): fills bounded area with a color. ***/
   /*** Also saves coordinates of polygon interior pixels into a file.  ***/

   sprintf (op, "fill -xy -g %d %s > %s", cmap, namyx, namfil);
   ierr = system (op);
   fprintf (stdout, "fill   done\n");
   if (ierr)
      {
      fprintf (stderr, "fill error: %d\n", ierr);
      regnum--;
      remfil (namcpv, namver, namper, namyx, namfil);
      return (NULL);
      }

   /*** Read "fill" file to count the number of coordinate pairs. ***/

   fd = fopen (namfil, "r");
   *nwp = 0;
   while (fscanf (fd, "%d %d", &xwin, &ywin) != EOF) (*nwp)++;
   fclose (fd);
   fprintf (stdout, "polyfill> nver: %d   nwp: %d\n", nver, *nwp);

   /*** Reopen "fill" file and read pixel coordinates into array "wpos". ***/

   wpos = (short *)  malloc (2 * *nwp * sizeof (short));
   fd = fopen (namfil, "r");
   for (ipix =  0; ipix < *nwp; ipix++)
      {
      fscanf (fd, "%d %d", &xwin, &ywin);
      *(wpos + ipix * 2) = (short) xwin;
      *(wpos + ipix * 2 + 1) = (short) ywin;
      }
   fclose (fd);

   /*** Delete temporary files used for WHIP Region utilities. ***/

bye:
   unlink (namver);
   unlink (namper);
   unlink (namyx);
   unlink (namfil);

   return (wpos);
}


remfil (name1, name2, name3, name4, name5)

char *name1, *name2, *name3, *name4, *name5;

{
   unlink (name1);
   unlink (name2);
   unlink (name3);
   unlink (name4);
   unlink (name5);

   return (0);
}
