/* sdpmmkcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Merge Solar Disk, Prominence Monitor, Mark-III & SMM C/P images together.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C/P image must be low resolution, and calibrated.
 * Mark III image is of the type "srq".
 * Pmon image is a Grinnell "chrestore" format file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 *  2 June 1989: Use Eikonix-digitized images (512x512 byte array).
 *  3 August 1989: Use Eikonix-digitized images (with header).
 * 13 December 1990: change Mark-III geometric scale to 104.4435 pixels/Rsun.
 *  2 January 1991: add prompt for Pmon intensity rescaling.
 * 29 March 1991: adapt for SGI IRIS.
 * 12 August 1991: Change pmscale to rescale.
 * 13 August 1991: Allow user to change Mark-III scaling parameters.
 * 10 February 1992: add solar disk image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/grscale.h"
#include "/usr/people/smm/lib/include/cp_color.h"

main (argc, argv)

int argc;
char *argv [];
{

unsigned short *rdpmon ();
unsigned short *rdmk3 ();
unsigned short *rdcp ();
unsigned short *rotate_img ();
unsigned short *magimg ();
unsigned short *merge2s ();
unsigned short *merge3s ();
unsigned short *merge4s ();

int mskimg ();
int rescale ();
int wdisk ();

static char answer [80];
static char namesd [80];		/* Solar disk image file name. */
static char namepm [80];		/* Pmon image file name. */
static char namemk [80];		/* Mark-III image file name. */
static char namecp [80];		/* C/P image file name. */
static char cptime   [6] = "HH:MM";
static char mk3time  [6] = "HH:MM";
static char pmontime [6] = "HH:MM";
static char sdtime   [6] = "HH:MM";
static char nampgm [80];
static char namout [80] = "cor.img";

unsigned short *sdims;	/* Pointer to Solar Disk image array. */
unsigned short *pmims;	/* Pointer to Prominence Monitor image array. */
unsigned short *mkims;	/* Pointer to Mark-III image array. */
unsigned short *cpims;	/* Pointer to SMM C/P image array. */

unsigned short *sdimr;	/* Pointer to Solar Disk image array. */
unsigned short *pmimr;	/* Pointer to Prominence Monitor image array. */
unsigned short *mkimr;	/* Pointer to Mark-III image array. */
unsigned short *cpimr;	/* Pointer to SMM C/P image array. */

unsigned short *imgnew;	/* Pointer to composite image array. */
unsigned char  *cimg;	/* Pointer to output image array. */

int dispsd = 0;
int disppm = 0;
int dispmk = 0;
int dispcp = 0;
int i;
int nbypx = 2;
int nimg = 0;
int npximg;
int ybeg = 2;
int yinc = 18;
int ypos = 0;

int xl = 760;		/* Specification of GL window position & size. */
int yl = 480;
int xu = 1271;
int yu = 991;

static int cpdx = 448, cpdy = 448, cpnx = 448, cpny = 448;
static int cpcx, cpcy, icpdsk, icprad;

static int mkdx, mkdy, mknx, mkny;
static int mkcx, mkcy, imkdsk, imkrad;

static int pmdx, pmdy, pmnx, pmny;
static int pmcx, pmcy, ipmdsk, ipmrad;

static int sddx, sddy, sdnx, sdny;
static int sdcx, sdcy, isddsk, isdrad;

static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250;
static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250;
static int pmmin1 = 0, pmmax1 = 250, pmmin2 = 0, pmmax2 = 250;
static int sdmin1 = 0, sdmax1 = 250, sdmin2 = 0, sdmax2 = 250;

static int grnlo = 0, grnhi = 249;
static int ithresh = 175;
static int level = 0;

static int xgs = 63, ygs = 63;
static int xs = 0, ys = 0;

/* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

static float cpcxnew [] = {  7., 100., 225., 398., 441., 348., 225., 50., -1.};
static float cpcynew [] = {225., 348., 441., 398., 258., 100.,   7., 50., -1.};

static float fcpcx, fcpcy;
static float fx, fy;

static float pmfac = 1.0;
static float sdfac = 1.0;

static float cpdsk = 1.5,   cprad = 1.5;
static float mkdsk = 1.05,  mkrad = 1.8;
static float pmdsk = 0.95,  pmrad = 1.5;
static float sddsk = 0.00,  sdrad = 1.2;

static float cpang = 43.0;
static float mkang =  0.0;
static float pmang =  0.0;
static float sdang =  0.0;

static float cppxrs = 80.9877;
static float mkpxrs = 104.4435;
static float pmpxrs = 117.5;
static float sdpxrs = 100.0;

double	PI = 3.141592654;

mkpxrs = (511.0 / (2.0 * (1.1224 + 127.0 * 0.0104244)));   /* scale 13 Dec 90 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   /* Request input information. */

   fprintf (stderr, "Enter file name of SMM C/P image: ");
   fscanf  (stdin, "%s", namecp);
   if (strcmp (namecp, "none") == 0)
      {
      dispcp = 0;
      nres = 1;
      xcen = fcpcx = 223.5;
      ycen = fcpcy = 223.5;
      pixrs = cppxrs;
      rspix = 1.0 / pixrs;
      raddeg = PI / 180.0;
      degrad = 180.0 / PI;
      cpang = 0.0;
      scroll = 0;
      }
   else
      {
      dispcp = 1;
      nimg++;
/*
 *      fprintf (stderr, "Enter minimum radius for SMM C/P image: ");
 *      fscanf  (stdin, "%f", &cprad);
 */
      fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ");
      fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2);
      }

   fprintf (stderr, "Enter file name of MLSO Mark-III image: ");
   fscanf  (stdin, "%s", namemk);
   if (strcmp (namemk, "none") == 0)
      dispmk = 0;
   else
      {
      dispmk = 1;
      nimg++;
      fprintf (stderr, "Enter time (HR:MIN) of MLSO Mark-III image: ");
      fscanf  (stdin,  "%s", mk3time);

      fprintf (stderr, "Enter maximum radius for MLSO Mark-III image: ");
      fscanf  (stdin, "%f", &mkrad);
      cprad = mkrad;

      fprintf (stderr, "Do you wish to rescale Mark-III image intensities [y/n] ? ");
      fscanf  (stdin, "%s", answer);
      if (strncmp (answer, "y", 1) == 0)
         {
         fprintf (stderr, "Enter Mark-III oldmin, oldmax, newmin, newmax: ");
         fscanf  (stdin, "%d,%d,%d,%d", &mkmin1, &mkmax1, &mkmin2, &mkmax2);
         }
      }

   fprintf (stderr, "Enter file name of MLSO Prominence Monitor image: ");
   fscanf  (stdin, "%s", namepm);
   if (strcmp (namepm, "none") == 0)
      disppm = 0;
   else
      {
      disppm = 1;
      nimg++;

/*
 *      fprintf (stderr, "Enter time (HR:MIN) of Prominence Monitor image: ");
 *      fscanf  (stdin,  "%s", pmontime);
 *
 *      fprintf (stderr, "Enter maximum radius for Prominence Monitor image: ");
 *      fscanf  (stdin, "%f", &pmrad);
 */

      fprintf (stderr, "Do you wish to rescale Pmon image intensities [y/n] ? ");
      fscanf  (stdin, "%s", answer);
      if (strncmp (answer, "y", 1) == 0)
         {
         fprintf (stderr, "Enter Pmon oldmin, oldmax, newmin, newmax: ");
         fscanf  (stdin, "%d,%d,%d,%d", &pmmin1, &pmmax1, &pmmin2, &pmmax2);
         }

      fprintf (stderr, "Enter threshold DN level for Pmon image: ");
      fscanf  (stdin, "%d", &ithresh);
      }

   fprintf (stderr, "Enter file name of Solar Disk image: ");
   fscanf  (stdin, "%s", namesd);

   if (strcmp (namesd, "none") == 0)
      dispsd = 0;
   else
      {
      dispsd = 1;
      nimg++;
      fprintf (stderr, "Do you wish to rescale Solar Disk image intensities [y/n] ? ");
      fscanf  (stdin, "%s", answer);
      if (strncmp (answer, "y", 1) == 0)
         {
         fprintf (stderr, "Enter Solar Disk oldmin, oldmax, newmin, newmax: ");
         fscanf  (stdin, "%d,%d,%d,%d", &sdmin1, &sdmax1, &sdmin2, &sdmax2);
         }
      }

   icprad = (int) (cprad * cppxrs);
   icpdsk = (int) (cpdsk * cppxrs);

   imkrad = (int) (mkrad * mkpxrs);
   imkdsk = (int) (mkdsk * mkpxrs);

   strcpy (nampgm, argv [0]);

   /*** Initialize SGI Graphics Library. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen (nampgm);
   gl_wclear (BLACK);
   onemap ();			/* Set single-map mode for color table. */
   get_cmap ();			/* Save color map. */

   multimap ();			/* Set multi-map mode for color table. */
   gconfig ();
   setmap (2);			/* Select color map #2. */

   fminit ();			/* Initialize SGI font manager. */

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*
    * Read SMM C/P image file.
    * Write black disk.
    * Magnify image.
    */

   if (dispcp == 1)
      {
      fprintf (stdout, "<--Read C/P image.\n");
      cpims = rdcp (namecp, &cpdx, &cpdy, &cpcx, &cpcy);
      sprintf (cptime, "%02d:%02d", H.hour, H.minute);
      fcpcx = cpcxnew [H.ism];
      fcpcy = cpcynew [H.ism];
      fprintf (stdout, "   fcpcx: %6.2f   fcpcy: %6.2f\n", fcpcx, fcpcy);
      imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
      sleep (1);

      /*** Rescale brightness of C/P image. ***/

      fprintf (stdout, "~~~Rescale C/P image . . . %d-%d --> %d-%d\n",
               cpmin1, cpmax1, cpmin2, cpmax2);
      rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1);

      fprintf (stdout, "   pmmkcp> xs:%d   ys:%d   cpnx:%d   cpny:%d\n",
               xs, ys, cpnx, cpny);
      imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
      sleep (1);

      /*** Write black disk in C/P occulting disk. ***/

      fprintf (stdout, "-0-Write disk into C/P image . . . cpcx:%d   cpcy:%d   icprad:%d\n",
               cpcx, cpcy, icprad);
      wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level);

      fprintf (stdout, "   pmmkcp> xs:%d   ys:%d   cpnx:%d   cpny:%d\n",
               xs, ys, cpnx, cpny);
      imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
      sleep (1);

      /*** Magnify C/P image. ***/

      fprintf (stdout, "<->Magnify C/P image.\n");
      fx = fy = mkpxrs / cppxrs;
      cpimr = (unsigned short *) malloc (cpdx * cpdy * sizeof(short));
      expcon (cpims, cpdx, cpdy, cpimr, cpdx, cpdy, fx, fy,
              (float) cpcx, (float) cpcy, fcpcx, fcpcy);
      free (cpims);

      gl_wclear (BLACK);
      fprintf (stdout, "   pmmkcp> xs:%d   ys:%d   cpnx:%d   cpny:%d\n",
               xs, ys, cpnx, cpny);
      imgwgl (cpimr, nbypix, xs, ys, cpnx, cpny, "ru");
      sleep (1);
      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*
    * Read MLSO Mark-III image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Rescale intensity levels.
    */

   if (dispmk == 1)
      {
      fprintf (stdout, "<--Read Mark-III image.\n");
      mkims = rdmk3 (namemk, &mkdx, &mkdy, &mkcx, &mkcy);
      fprintf (stdout, "   mkdx:%d   mkdy:%d   mkcx:%d   mkcy:%d\n",
               mkdx, mkdy, mkcx, mkcy);
      gl_wclear (BLACK);
      imgwgl (mkims, nbypx, 0, 0, mkdx, mkdy, "ru");
      sleep (1);

      /*** Write black disk into Mark-III occulting disk. ***/

      fprintf (stdout, "-0-Write disk in Mark-III image . . . radius:%f\n", mkdsk);
      wdisk (mkims, mkdx, mkdy, mkcx, mkcy, imkdsk, level);
      imgwgl (mkims, nbypx, 0, 0, mkdx, mkdy, "ru");
      sleep (1);

      /*** Rotate Mark-III image. ***/

      mkang -= cpang;
      mkang -= (float) scroll;
      fprintf (stdout, "~^~Rotate Mark-III image . . . mkang: %f\n", mkang);
      mkimr = rotate_img (mkims, mkdx, mkdy, mkcx, mkcy, mkang);
      free (mkims);
      imgwgl (mkimr, nbypx, 0, 0, mkdx, mkdy, "ru");
      sleep (1);

      /*** Mask Mark-III image beyond specified radius. ***/

      fprintf (stdout, "|-|Mask Mark-III image beyond %f Rsun.\n", mkrad);
      mskimg (mkimr, 0, 0, mkdx, mkdy, mkcx, mkcy, imkrad);
      imgwgl (mkimr, nbypx, 0, 0, mkdx, mkdy, "ru");
      sleep (1);

      /*** Magnify Mark-III image. ***/

      fprintf (stdout, "<->Magnify Mark-III image.\n");
      mknx = cpnx;
      mkny = cpny;
      fx = 1.0;
      fy = 1.0;
      mkims = (unsigned short *) malloc (mknx * mkny * sizeof(short));
      for (i=0; i<mknx*mkny; i++)  *(mkims+i) = 0;
      expcon (mkimr, mkdx, mkdy, mkims, mknx, mkny, fx, fy,
              (float) mkcx, (float) mkcy, fcpcx, fcpcy);
      free (mkimr);
 
      gl_wclear (BLACK);
      imgwgl (mkims, nbypx, xs, xs, mknx, mkny, "ru");
      sleep (1);

      /*** Rescale brightness of Mark-III image. ***/

      fprintf (stdout, "~~~Rescale Mark-III image.\n");
      rescale (mkims, mknx, mkny, mkmin1, mkmax1, mkmin2, mkmax2, grnlo, grnhi-1);
      imgwgl (mkims, nbypx, xs, ys, mknx, mkny, "ru");
      sleep (1);

      mkcx = (int) fcpcx;
      mkcy = (int) fcpcy;
      mskimg (mkims, 0, 0, mknx, mkny, mkcx, mkcy, imkrad);
      imgwgl (mkims, nbypx, xs, ys, mknx, mkny, "ru");
      sleep (1);
      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*
    * Read MLSO Prominence Monitor image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Minify image.
    * Rescale intensity levels.
    */

   if (disppm == 1)
      {
      fprintf (stdout, "<--Read Pmon image.\n");
      pmims = rdpmon (namepm, &pmdx, &pmdy, &pmcx, &pmcy, pmontime,
                      &pmpxrs, &pmang);
      ipmdsk = (int) (pmdsk * pmpxrs);
      ipmrad = pmrad * pmpxrs;
      fprintf (stdout, "   pmdx: %d   pmdy: %d   pmcx: %d   pmcy: %d\n",
               pmdx, pmdy, pmcx, pmcy);

      /*** Rescale brightness of Pmon image. ***/

      fprintf (stdout, "~~~Rescale Pmon image.\n");
      rescale (pmims, pmdx, pmdy, pmmin1, pmmax1, pmmin2, pmmax2, grnlo, grnhi);

      gl_wclear (BLACK);
      imgwgl (pmims, nbypx, 0, 0, pmdx, pmdy, "ru");
      sleep (1);

      /*** Write black disk at center of Pmon occulting disk. ***/

      fprintf (stdout, "-0-Write %6.2f Rsun disk into Pmon image.\n", pmdsk);
      wdisk (pmims, pmdx, pmdy, pmcx, pmcy, ipmdsk, level);
      imgwgl (pmims, nbypx, 0, 0, pmdx, pmdy, "ru");
      sleep (1);

      /*** Rotate Pmon image. ***/

      pmang -=  cpang;
      pmang -= (float) scroll;
      fprintf (stdout, "~^~Rotate Pmon image . . . pmang: %f\n", pmang);
      pmimr = rotate_img (pmims, pmdx, pmdy, pmcx, pmcy, pmang);
      free (pmims);
      imgwgl (pmimr, nbypx, 0, 0, pmdx, pmdy, "ru");
      sleep (1);

      /*** Magnify Pmon image. ***/

      fprintf (stdout, "<->Magnify Pmon image.\n");
      pmnx = cpnx; pmny = cpny;
      fx = fy = mkpxrs / pmpxrs;
      pmims = (unsigned short *) malloc (pmnx * pmny * sizeof(short));
      expcon (pmimr, pmdx, pmdy, pmims, pmnx, pmny, fx, fy,
              (float) pmcx, (float) pmcy, fcpcx, fcpcy);
      free (pmimr);
      imgwgl (pmims, nbypx, xs, ys, pmnx, pmny, "ru");
      sleep (1);

      /*** Mask Pmon image beyond specified radius. ***/

      fprintf (stdout, "|-|Mask Pmon image beyond %4.2f Rsun.\n", pmrad);
      pmfac = fx;
      ipmrad = pmrad * pmfac * pmpxrs;
      fprintf (stdout, "   pmfac: %6.2f   pmpxrs: %6.2f   ipmrad: %d\n",
               pmfac, pmpxrs, ipmrad);
      pmcx = (int) fcpcx;
      pmcy = (int) fcpcy;
      mskimg (pmims, 0, 0, pmnx, pmny, pmcx, pmcy, ipmrad);

      gl_wclear (BLACK);
      imgwgl (pmims, nbypx, xs, ys, pmnx, pmny, "ru");
      sleep (1);

      /*
       *   fprintf (stdout, "~~~Rescale Pmon image.\n");
       *   rescale (pmims, pmnx, pmny, pmmin1, pmmax1, pmmin2, pmmax2);
       *   gl_wclear (BLACK);
       *   imgwgl (pmims, nbypx, xs, ys, pmnx, pmny, "ru");
       */
      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*
    * Read Solar Disk Monitor image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Minify image.
    * Rescale intensity levels.
    */

   if (dispsd == 1)
      {
      fprintf (stdout, "<--Read Solar Disk image.\n");
      sdims = rdpmon (namesd, &sddx, &sddy, &sdcx, &sdcy, sdtime, &sdpxrs, &sdang);
      isddsk = (int) (sddsk * sdpxrs);
      isdrad = sdrad * sdpxrs;
      fprintf (stdout, "   sddx: %d   sddy: %d   sdcx: %d   sdcy: %d\n",
               sddx, sddy, sdcx, sdcy);

      /*** Rescale brightness of Solar Disk image. ***/

      fprintf (stdout, "~~~Rescale Solar Disk image.\n");
      rescale (sdims, sddx, sddy, sdmin1, sdmax1, sdmin2, sdmax2, grnlo, grnhi);

      gl_wclear (BLACK);
      imgwgl (sdims, nbypx, 0, 0, sddx, sddy, "ru");
      sleep (1);

      /*** Write black disk at center of Solar Disk occulting disk. ***/

      fprintf (stdout, "-0-Write %6.2f Rsun disk into Solar Disk image.\n", sddsk);
      wdisk (sdims, sddx, sddy, sdcx, sdcy, isddsk, level);
      imgwgl (sdims, nbypx, 0, 0, sddx, sddy, "ru");
      sleep (1);

      /*** Rotate Solar Disk image. ***/

      sdang -=  cpang;
      sdang -= (float) scroll;
      fprintf (stdout, "~^~Rotate Solar Disk image . . . sdang: %f\n", sdang);
      sdimr = rotate_img (sdims, sddx, sddy, sdcx, sdcy, sdang);
      free (sdims);
      imgwgl (sdimr, nbypx, 0, 0, sddx, sddy, "ru");
      sleep (1);

      /*** Magnify Solar Disk image. ***/

      fprintf (stdout, "<->Magnify Solar Disk image.\n");
      sdnx = cpnx; sdny = cpny;
      fx = fy = mkpxrs / sdpxrs;
      sdims = (unsigned short *) malloc (sdnx * sdny * sizeof(short));
      expcon (sdimr, sddx, sddy, sdims, sdnx, sdny, fx, fy,
              (float) sdcx, (float) sdcy, fcpcx, fcpcy);
      free (sdimr);
      imgwgl (sdims, nbypx, xs, ys, sdnx, sdny, "ru");
      sleep (1);

      /*** Mask Solar Disk image beyond specified radius. ***/

      fprintf (stdout, "|-|Mask Solar Disk image beyond %4.2f Rsun.\n", sdrad);
      sdfac = fx;
      isdrad = sdrad * sdfac * sdpxrs;
      fprintf (stdout, "   sdfac: %6.2f   sdpxrs: %6.2f   isdrad: %d\n",
               sdfac, sdpxrs, isdrad);
      sdcx = (int) fcpcx;
      sdcy = (int) fcpcy;
      mskimg (sdims, 0, 0, sdnx, sdny, sdcx, sdcy, isdrad);

      gl_wclear (BLACK);
      imgwgl (sdims, nbypx, xs, ys, sdnx, sdny, "ru");
      sleep (1);
      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*** Add images together. ***/

   if (nimg == 4)
      imgnew = merge4s (cpimr, mkims, pmims, sdims, cpnx, cpny, ithresh);



   if (nimg == 3)
      {
      if (disppm == 0)  ithresh = grnhi;

      if (dispcp && dispmk && disppm)
         imgnew = merge3s (cpimr, mkims, pmims, cpnx, cpny, ithresh);

      if (dispcp && dispmk && dispsd)
         imgnew = merge3s (cpimr, mkims, sdims, cpnx, cpny, ithresh);

      if (dispcp && dispsd && disppm)
         imgnew = merge3s (cpimr, sdims, pmims, cpnx, cpny, ithresh);

      if (dispmk && dispsd && disppm)
         imgnew = merge3s (mkims, sdims, pmims, cpnx, cpny, ithresh);
      }



   if (nimg == 2)
      {
      if (disppm == 0)  ithresh = grnhi;

      if (dispcp && dispmk)
         imgnew = merge2s (cpimr, mkims, cpnx, cpny, ithresh);

      if (dispcp && disppm)
         imgnew = merge2s (cpimr, pmims, cpnx, cpny, ithresh);

      if (dispcp && dispsd)
         imgnew = merge2s (cpimr, sdims, cpnx, cpny, ithresh);


      if (dispmk && disppm)
         imgnew = merge2s (mkims, pmims, cpnx, cpny, ithresh);

      if (dispmk && dispsd)
         imgnew = merge2s (mkims, sdims, cpnx, cpny, ithresh);


      if (dispsd && disppm)
         imgnew = merge2s (sdims, pmims, cpnx, cpny, ithresh);
      }


   if (dispcp)
      free (cpimr);
   if (dispmk)
      free (mkims);
   if (disppm)
      free (pmims);
   if (dispsd)
      free (sdims);

   /*** Display composite image with annotation. ***/

   gl_wclear (BLACK);
   ldisk = 1;
   glcp_label (0, 1, 1, 1, cp_grey, 1);
   glcp_annot (0, 1);

   imgwgl (imgnew, nbypx, xgs, ygs, cpnx, cpny, "ru");/* Write merged image.*/
   free (imgnew);

/*   wsolid (imgch4, 0, 0, 512, 61, "ru"); */	/* Erase area below image. */
   select_font ("Charter-Black", 12);	/* Helvetica-Bold, 16x17 characters. */

   ybeg = 2;
   i = 0;

   if (dispsd)
      {
      ypos = ybeg + yinc * i++;
      glwstr ("Solar Disk                     ", 63, ypos, cp_green);
      glwstr (sdtime, 444, ypos, cp_red);
      printf ("sd ypos: %d\n", ypos);
      }

   if (disppm)
      {
      ypos = ybeg + yinc * i++;
      glwstr ("MLSO PROMINENCE MONITOR        ", 63, ypos, cp_green);
      glwstr (pmontime, 444, ypos, cp_red);
      printf ("pm ypos: %d\n", ypos);
      }

   if (dispmk)
      {
      ypos = ybeg + yinc * i++;
      glwstr ("MLSO MARK-III K-CORONAMETER    ", 63, ypos, cp_green);
      glwstr (mk3time, 444, ypos, cp_red);
      printf ("mk ypos: %d\n", ypos);
      }

   if (dispcp && nimg < 4)
      {
      ypos = ybeg + yinc * i++;
      glwstr ("SMM CORONAGRAPH/POLARIMETER    ", 63, ypos, cp_green);
      glwstr (cptime, 444, ypos, cp_red);
      printf ("cp ypos: %d\n", ypos);
      }

   fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
   xcen = fcpcx;
   ycen = 447 - fcpcy;
   fx = fy = mkpxrs / cppxrs;
   pixrs = pixrs * fx;
   if (dispcp == 0)  scroll = -43;
   glcp_north ((int) fcpcx, (int) fcpcy, 0.1, 0.3, cp_yellow);
   glcp_scalers (0, cp_yellow);
   glcp_suncir (0.4, 1.0, 0.1, 90.0, 0.4, 1.0, 0.2, 30.0, cp_yellow);

   /*** Write composite image to disc file (WHIP pic format). ***/

   npximg = 512 * 512;
   imgnew = (unsigned short *) malloc (npximg * sizeof (short));
   cimg = (unsigned char *) malloc (npximg);
   rectread (0, 0, 511, 511, imgnew);
   for (i = 0; i < npximg; i++)
      *(cimg + i) = (unsigned char) (*(imgnew + i) & 0377);
   imgpic (cimg, 1, 512, 512, "ru", namout);

   sleep (10);
   gexit ();
   exit (0);
}
