/* polimg2.c
 *----------------------------------------------------------------------
 * Performs radial and theta scans using polaroid images.
 * Produces images of B, It, Ir, pB, %p.
 *----------------------------------------------------------------------
 * POLIMG is a polarization analysis routine modified for the Sun-4/280.
 * The Stokes vector is calculated from three C/P images taken through
 * the three polaroid filters: +60, 0, and -60 degrees.
 *----------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 13 January 1987
 * 14 May 1987: No Munro calibration, stray light, vignetting, or F-corona
 *              corrections.
 * 14 November 1988: remove Grinnell commands for use on Sun-4/280.
 *----------------------------------------------------------------------
 *... hardware:     Sun-4/280 cpu
 *                  32 bit word length
 *... software:     SunOS 4.0 (UNIX 4.3 bsd) operating system
 *                  C programming language
 *----------------------------------------------------------------------
 *... dictionary:
 *  s		: input polaroid vector (p1,p2,p3).
 *  ss		: output Stoke's vector (I,Q,U).
 *  xk1, xk2	: max and min transmisions of the polaroids.
 *
 * B: total brightness; pB: polarization * brightness
 * It: tangential intensity; Ir: radial intensity
 * p: polarization fraction
 *
 * B  = It + Ir
 * pB = It - Ir
 *
 * p0: clear (non-polaroid) image.
 * p1: +60 degree polaroid (w.r.t. pylon).
 * p2:   0 degree polaroid (aligned with C/P pylon).
 * p3: -60 degree polaroid (w.r.t. pylon).
 *----------------------------------------------------------------------
 */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.h"

FILE *lun;

int ixc0, iyc0, ixc1, iyc1, ixc2, iyc2, ixc3, iyc3;
int isprint, ixfix, iyfix;

/*---------------------------------------------------------------------- */

main (argc, argv)

int argc;
char *argv [];

{
extern char *mkname ();	/* Create metacode file name using clear image name. */
extern int poldef ();	/* Polaroid image definition routine. */
extern int stokes ();	/* Compute stokes vector, Br, Bt, pol, pB. */

static char answer [2];
static char namp0 [80], namp1 [80], namp2 [80], namp3 [80];
static char namemc [80];
static char namdat [80];

static float rinv [3][3];
static float pi = 3.141592654;

/* --------------------------------------------------------------------- */

   /*** Program greeting. ***/

   fprintf (stdout, "POLIMG: generates images of Br, Bt, pol, pB.\n");

   /* POLDEF :
    * Request input file names from user.
    * Read Mueller matrix from disk file 'polmtx.dat'.
    */

   poldef (namp0, namp1, namp2, namp3, rinv);
   strcpy (namemc,mkname(namp0, "mc"));

   fprintf (lun, "clear: %s\n", namp0);
   fprintf (lun, "p1: %s\n", namp1);
   fprintf (lun, "p2: %s\n", namp2);
   fprintf (lun, "p3: %s\n", namp3);
   fprintf (lun, "transposed Mueller matrix :\n");
   fprintf (lun, "%f %f %f\n%f %f %f\n%f %f %f\n",
            rinv [0][0], rinv [0][1], rinv [0][2], 
            rinv [1][0], rinv [1][1], rinv [1][2], 
            rinv [2][0], rinv [2][1], rinv [2][2]);

   lsv = 0;		/* No removal of stray light or vignetting. */
   lfc = 0;		/* No removal of F-corona. */
/*
 *   printf ("Remove F-corona ? ");
 *   scanf  ("%s", answer);
 *   if (strncmp (answer, "y", 1) == 0)  lfc = 1;  else lfc = 0;
 */

   /*** Compute Stokes vector I, Q, U. ***/
   /*** ....... p1,p2,p3 ---> i,q,u ....... ***/

   stokes (namp0, namp1, namp2, namp3, rinv);

   /*** Close all files. ***/

   fclose (lun);
   exit (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
mkname (namvid, ext)

char *namvid;
char *ext;

{
int lenvid;
int idot;
int i;
char paknam[80];

	lenvid = strlen(namvid);
	idot = -1;
        for (i=0; i<lenvid; i++) 
		{if(namvid[i] == '.') idot = i; paknam[i] = namvid[i];}
	if (idot == -1)
		{
		printf ("mkname: invalid file name: %s\n", namvid);
		exit (-1);
		}
	paknam[idot+1] = '\0';
	strcat (paknam, ext);

	return (paknam);
}
