/* polscn.c
 *----------------------------------------------------------------------
 * Performs radial and theta scans using polaroid images.
 * Produces plots of b, pb, %p, angle.
 *
 * POLSCN is a polarization analysis routine modified for the VAX-11/750.
 * The Stokes vector is calculated from three C/P images taken through
 * the three polaroid filters: +60, 0, and -60 degrees.
 *----------------------------------------------------------------------
 * 'polscn' evolved (30 dec 82) from 'pol.ftn' by Kevin Cole.
 *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 23 November 1983 :
 *    Removed fcorona logic.
 *    Added option to save pb scan in a disk file.
 *  6 July 1984: conversion from 11/34 (f4p) to VAX 750 (F77).
 * 15 February 1985: conversion from F77 to C.
 * 17 June 1985: use F77 subroutines for matinv & mxm.
 * 29 July 1985: atan2 function returns result in radians.
 * 1 August 1985: add Munro calibration correction factor (1.9).
 * 2 August 1985: add clear intensity frame.
 * 30 September 1985: si[i] = ss[0] * 1.9;
 * 23 October 1985: Display clear image on Grinnell; Add F-corona removal.
 * 31 October 1985: fix overlay color problem caused by cpview call.
 *  4 November 1985: Correct title labels.
 *  9 December 1985: Fix title for p0123 plot.
 * 12 December 1985: Clean up I,Q,U plot labels.
 *  9 July 1986: Add 'scatter' plot.
 * 11 July 1986: Remove 'rmsv' command (now default).  Rearrange command order.
 * 26 August 1986: Generate metacode files only if terminal is non-graphics.
 * 11 November 1988: Prompt user for stray light & vignetting removal options.
 * 17 June 1991: Adapt for SGI/IRIS GL windows.
 * 18 June 1991: If lfc=1, remove F-corona from both p0 and ci;
 *  6 January 1992: correct pB computation.
 *----------------------------------------------------------------------
 *... hardware:     SGI IRIS 4D/25 G
 *                  32 bit word length
 *... software:     IRIX 3.3.1 operating system
 *                  C programming language
 *----------------------------------------------------------------------
 *... dictionary:
 *  s		: input polaroid vector (p1,p2,p3).
 *  ss		: output Stoke's vector (I,Q,U).
 *  xk1, xk2	: max and min transmisions of the polaroids.
 *----------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/stanger/lib/src/gl/img_scale.h"
#include "/usr/people/stanger/lib/src/gl/gl_color.h"
#include "/usr/people/stanger/lib/src/gl/ov_color.h"
#include "scan.h"

#define ND 3600

FILE *lun;

int pcount = 0;	/* Plot count for 'xyplot' routine. */
int ixc0, iyc0, ixc1, iyc1, ixc2, iyc2, ixc3, iyc3;
int isprint, ixfix, iyfix;
int DSP = 0;	/* Display usage flag. */

/*---------------------------------------------------------------------- */

main (argc, argv)

int argc;
char *argv [];
{
extern char *mcname ();	/* Create metacode file name using clear image name. */
extern int calscn ();	/* Radial/Theta scan driver for calibrated images.*/
extern int remstr ();	/* Stray light removal. */
extern int remvig ();	/* Vignetting removal. */
extern int rmfc ();	/* F-corona removal. */
extern int mxm_ ();	/* Matrix multiplication routine. */
extern int faverage ();	/* Floating point array averaging routine. */
extern int xyplot ();	/* Plot routine. */
extern int poldef ();	/* Polaroid image definition routine. */
extern int polcmd ();	/* Command help list. */
extern float stray ();	/* Stray light function. */
extern float vignet ();	/* Vignetting function. */
extern double sqrt (), fabs(), atan2();	/* Math routines. */
extern int cpview ();	/* Grinnell image display function. */

int xset = 1;
int ncmd = 10;
int idcmd;
int ic;
int nval;
int i;
int ixscale = 0;	/* x-axis scaling type; 0=linear. */
int iyscale = 0;	/* y-axis scaling type; 0=linear. */
int ibkg = 0;		/* Background flag; 0=tick marks only. */
int plotit = 1;
int clip = 0;
int zero = 0, two = 2, minus1 = -1;
int n1 = 1;
int n3 = 3;
int pltchr = 43;

static char answer [2];
static char gksnam [] = "polscn.cgm";
static char gksout [80];
static char namp0 [80], namp1 [80], namp2 [80], namp3 [80];
static char namemc [80];
static char nampgm [80];
static char lext [8] = "-svfc";
static char ltit [42] = "Title";
static char labx [42] = "X-axis";
static char title [42] = "Title";
static char labelx [42] = "x-label";
static char labely [42] = "y-label";

static char scntyp [7];
static char req [6];
static char comand [10][7] = {"scan", "fcor", "pol",    "pb",   "angle",
                              "brt",  "cmp",  "stokes", "help", "exit"};

static float xd [2], yd [2];
static float rinv [3][3];
static float s [3], ss [3];
static float p0 [ND], p1 [ND], p2 [ND], p3 [ND];
static float p123 [ND], p0123 [ND];
static float ci [ND], cq [ND], cu [ND];
static float pct [ND], pangle [ND], pb [ND];
static float scandx [ND];
static float pi = 3.141592654;
static float Cexp = 1.e-11;	/* Photometric scaling factor [B/Bsun]. */

float degrad;
float th;
float si, sq, su;
float zi, zq, zu;
float alpha, delang;
float pol, polbrt;
float cc = 1.0, str = 0.0, vig = 1.0;
float fsign;
float clear;
float pmin, pmax;

/* --------------------------------------------------------------------- */
   strcpy (nampgm, argv [0]);

   degrad = 180.0 / pi;

   /*** Program greeting. ***/

   fprintf (stderr, "POLSCN: performs scans on C/P polaroid images.\n");
   fprintf (stderr, "Enter terminal type: v=video, g=graphics: ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp ("g", answer, 1) == 0)
      igterm = 1;
   else
      igterm = 0;

   fprintf (stderr, "Do you wish to use the IRIS display (y/n) ? ");
   fscanf  (stdin,  "%s", answer);
   igr = 0;
   if (strncmp ("y", answer, 1) == 0)
      {
      igr = 1;
      DSP = 1;
      foreground ();
/*
 *      if (H.ires == 0)
 *      prefposition (266, 1279,   0, 1023);
 *      else
 */
      prefposition (760, 1271, 480, 991);/*512x512 window: upper right corner.*/
      fminit ();			/* Initialize SGI font manager. */

      winopen (nampgm);
      gl_wclear (BLACK);
      onemap ();
      gconfig ();
      pixmode (PM_ADD24, 512);		/* i.e., effectively use color map #2.*/
      }

   /* POLDEF :
    * Request input file names from user.
    * Read Mueller matrix from disk file 'polmtx.dat'.
    */

   poldef (namp0, namp1, namp2, namp3, rinv);
   strcpy (namemc, mcname (namp0));
   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
   putenv (gksout);

   /*
    * If terminal is non-graphics, generate metacode file only; no plots.
    */

   if (igterm == 0)
      {
/*      gks_chanam (namemc); */
      pcount += 1;
      igterm = 1;
      }
   else
   if (igterm == 1)
      {
/*      gksini_ (&plotit, gksnam); */
      opngks_ ();
/*      gsclip_ (&clip); */
      }

   fprintf (lun, "transposed Mueller matrix :\n");
   fprintf (lun, "%f %f %f\n%f %f %f\n%f %f %f\n",
           rinv [0][0], rinv [0][1], rinv [0][2], 
           rinv [1][0], rinv [1][1], rinv [1][2], 
           rinv [2][0], rinv [2][1], rinv [2][2]);

   /*** Display clear (unpolarized) image on Grinnell. ***/

   if (DSP)
      {
      fprintf (stderr, "Displaying image %s.  Please wait.\n", namp0);

      glcp_view (namp0, 1);

      /*** Overlay set-up. ***/

      overlay (2);		/* Establish two bitplanes for overlay. */
      gconfig ();

      drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
      mapcolor (ov_black,   0,   0,   0);	/* color 0 -> black. */
      mapcolor (ov_red,   255,   0,   0);	/* color 0 -> red.   */
      mapcolor (ov_green,   0, 255,   0);	/* color 0 -> green. */
      mapcolor (ov_blue,    0,   0, 255);	/* color 0 -> blue.  */
      gl_wclear (ov_black);
      }

   /*** Go to scan section. ***/

   goto c0;

   /*** Display command menu. ***/

next:

   /*** Get next command. ***/

   fprintf (stderr, "? ");
   fscanf  (stdin,  "%s", req);
   if (strlen (req) <= 0)  exit (-1);
   idcmd = 0;

   idcmd = -1;
   for (ic=0; ic<ncmd; ic++)
	{
	if (strncmp (req, &comand [ic][0], 2) == 0) idcmd = ic;
	}

   if (idcmd == -1)
     {
      fprintf (stderr, "*** invalid command\n");
      goto next;
     }

   switch (idcmd)
   {

   /*** Process 'scan' ... Radial and theta scans. ***/

case 0:
c0:

   lcc  = 0;				/* No Munro correction factor. */
   lstr = 0;				/* Stray light flag. */
   lvig = 0;				/* Vignetting flag. */
   lfc  = 0;				/* F-corona flag. */
   fflush (stdin);

   fprintf (stderr, "Enter scan type [ radial or theta ]: \n");
   fscanf  (stdin,  "%s", scntyp);

   /*** Radial scan limits. ***/

   if (scntyp [0] == 'r')
      {
      fprintf (stderr, "Enter min,max,delta radius [R/Rsun]: \n");
      fscanf  (stdin,  "%f,%f,%f", &rmin, &rmax, &rinc);
      fprintf (stderr, "Enter theta angle [degrees]: \n");
      fscanf  (stdin,  "%f", &thmin);
      thmax = thmin;
      theta = thmin;
      }

   /*** Theta scan limits. ***/

   else
      {
      fprintf (stderr, "Enter min,max,delta theta [degrees]: \n");
      fscanf  (stdin,  "%f,%f,%f", &thmin, &thmax, &thinc);
      if (thmin > thmax)  thmax = thmax + 360.0;
      fprintf (stderr, "Enter radius [R/Rsun]: \n");
      fscanf  (stdin,  "%f", &rmin);
      rmax = rmin;
      radius = rmin;
      }

   /*** Do a multiple-point average of p1,p3,p3. ***/

   fprintf (stderr, "Enter # points to average in scans : ");
   fscanf  (stdin,  "%d", &nave);
   if (nave <= 0)  nave = 1;

   /*** Extract scan data from clear (p0) + 3 polaroid (p1,p2,p3) images. ***/

   xcen = ixc0;
   ycen = iyc0;
   calscn (scntyp, namp0, p0, scandx, &nval);

   xcen = ixc1;
   ycen = iyc1;
   calscn (scntyp, namp1, p1, scandx, &nval);

   xcen = ixc2;
   ycen = iyc2;
   calscn (scntyp, namp2, p2, scandx, &nval);

   xcen = ixc3;
   ycen = iyc3;
   calscn (scntyp, namp3, p3, scandx, &nval);

   fprintf (stderr, "Apply Munro calibration correction [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp ("y", answer, 1) == 0)  lcc = 1;

   fprintf (stderr, "Remove stray light [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp ("y", answer, 1) == 0)  lstr = 1;

   fprintf (stderr, "Remove vignetting [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp ("y", answer, 1) == 0)  lvig = 1;

   fprintf (lun,
           "lcc: %d  lstr: %d  lvig: %d  lfc: %d  cc: %g  str: %g  vig: %g\n",
            lcc, lstr, lvig, lfc, cc, str, vig);
   fprintf (lun, "stray (%5.2f): %g\n\n", radius, stray (radius));

   if (nval > ND) goto next;

   if (scntyp [0] == 'r')	/*** Radial scan labels. ***/
      {
      sprintf (ltit, "%14.14s radial scan %5.1f deg", namp0, thmin);
      strcpy (lext, " -");
      if (lcc)  strcat (lext, "c");  else strcat (lext, " ");
      if (lstr) strcat (lext, "s");  else strcat (lext, " ");
      if (lvig) strcat (lext, "v");  else strcat (lext, " ");
      if (lfc)  strcat (lext, "f");  else strcat (lext, " ");
      if ((strcmp (lext, " -    ")) != 0)  strcat (ltit, lext);
      sprintf (labx, "radius [R/Rsun]");

      fprintf (stderr, "%s  radial scan . . . angle : %f\n", namp0, thmin);
      fprintf (lun   , "%s  radial scan . . . angle : %f\n", namp0, thmin);
      fprintf (lun   , "radius        p1       p2       p3         i         q         u        ci        cq        cu       str   vig        pb   pol  angle\n");
      }
   else				/*** Theta scan labels. ***/
      {
      sprintf (ltit, "%14.14s theta scan %3.1f Rsun", namp0, rmin);
      strcpy (lext, " -");
      if (lcc)  strcat (lext, "c");  else strcat (lext, " ");
      if (lstr) strcat (lext, "s");  else strcat (lext, " ");
      if (lvig) strcat (lext, "v");  else strcat (lext, " ");
      if (lfc)  strcat (lext, "f");  else strcat (lext, " ");
      if ((strcmp (lext, " -    ")) != 0)  strcat (ltit, lext);
      sprintf (labx, "position angle [degrees]");

      fprintf (stderr, "%s  theta scan  . . . radius : %f\n", namp0, rmin);
      fprintf (lun   , "%s  theta scan  . . . radius : %f\n", namp1, rmin);
      fprintf (lun   , " theta        p1       p2       p3         i         q         u    cleari        cq        cu       str   vig        pb   pol  alpha\n");
      }

   /*** Compute Stokes vector I, Q, U. ***/
   /*** ....... p1,p2,p3 ---> i,q,u ....... ***/

   pmin =  32767.0;
   pmax = -32767.0;
   radius = rmin;
   th = thmin;

   for (i=0; i<nval; i++)
      {
      if (scntyp [0] == 't')  th = scandx [i];
      if (th > 360.0)  th -= 360.0;

      if (scntyp [0] == 'r')  radius = scandx [i];

      /*** Remove stray light & vignetting. ***/

      if (lcc)   cc  = calcor (radius);  else cc  = 1.0;
      if (lstr)  str = stray  (radius);  else str = 0.0;
      if (lvig)  vig = vignet (radius);  else vig = 1.0;

      /*** Establish polaroid input intensity vector (s). ***/

      s [0] = p1 [i];
      s [1] = p2 [i];
      s [2] = p3 [i];

      /*** Compute ss : the resultant Stokes vector. ***/

      mxm_ (rinv, &n3, s, &n3, ss, &n1);
      si = ss [0];
      sq = ss [1];
      su = ss [2];

      /*** Corrected Stokes vector 
       *** derived from 3 polaroids (+60,0,-60 deg).
       ***/

      zi = ci [i] = (si * cc - str) * vig;	/* I component. */
      zq = cq [i] = sq * vig * cc;		/* Q component. */
      zu = cu [i] = su * vig * cc;		/* U component. */

      /*** Clear polaroid intensity. ***/

      clear = (p0 [i] * cc - str) * vig;
      p0 [i] = clear;
      p123 [i] = zi;
      if (zi == 0.0 || (scandx [i] > 150.0 && scandx [i] < 210.0))
         p0123 [i] = 1.0;
      else
         p0123 [i] = clear / zi;
      if (pmin > clear) pmin = clear;
      if (pmin > zi) pmin = zi;
      if (pmax < clear) pmax = clear;
      if (pmax < zi) pmax = zi;

      /*** Polarization * Brightness. ***/

      polbrt = sqrt ((double) (zq*zq + zu*zu));
      pol = -1.0;
      if (zi > 0.0)  pol = polbrt / zi;

      /*** Polarization deviation angle. ***/

      if (zq != 0.0)
         alpha = 0.5 * degrad * (float) atan2 ((double) zu, (double) zq);
      else
	 alpha = 0.0;
      alpha = alpha - 90.0;
      if (alpha <   0.0)  alpha = alpha + 360.0;
      if(alpha > 360.0)  alpha = alpha - 360.0;

      delang = alpha - th;
      fsign = 1.0;
      if (delang != 0.0) fsign = delang / fabs ((double) delang);
      while (fabs ((double) delang) > 45.0) delang -= 90.0 * fsign;

      fprintf (lun   ,"%6.2f  %8.2e %8.2e %8.2e  %8.2e %9.2e %9.2e  %8.2e %9.2e %9.2e  %8.2e %5.2f  %8.2e %5.3f %6.2f\n",
         scandx [i], s [0], s [1], s [2], si, sq, su, p0 [i], zq, zu, str, vig,
         polbrt, pol, delang);
      }
   polcmd ();
   break;

   /*** 'fcor' ... Remove F-corona. ***/

case 1:
   if (lstr == 0 || lvig == 0)
      {
      fprintf (stderr, "Stray light & vignetting must be removed first.\n");
      break;
      }
   if (lfc == 1)
      {
      fprintf (stderr, "F-corona has already been removed.\n");
      }

   /*** Radial scan : ***/

   if (scntyp [0] == 'r' && lfc == 0)
      {
      lfc = 1;
      sprintf (ltit, "%14.14s radial scan %5.1fdeg", namp0, thmin);
      strcpy (lext, " -");
      if (lcc)  strcat (lext, "c");  else strcat (lext, " ");
      if (lstr) strcat (lext, "s");  else strcat (lext, " ");
      if (lvig) strcat (lext, "v");  else strcat (lext, " ");
      if (lfc)  strcat (lext, "f");  else strcat (lext, " ");
      if ((strcmp (lext, " -    ")) != 0)  strcat (ltit, lext);
      rmfc (scandx, nval, ci, nval, theta, 1);
      rmfc (scandx, nval, p0, nval, theta, 1);
      }

   /*** Theta scan : ***/

   if (scntyp[0] == 't' && lfc == 0)
      {
      lfc = 1;
      sprintf (ltit, "%14.14s theta scan %3.1f Rsun", namp0, rmin);
      strcpy (lext, " -");
      if (lcc)  strcat (lext, "c");  else strcat (lext, " ");
      if (lstr) strcat (lext, "s");  else strcat (lext, " ");
      if (lvig) strcat (lext, "v");  else strcat (lext, " ");
      if (lfc)  strcat (lext, "f");  else strcat (lext, " ");
      if ((strcmp (lext, " -    ")) != 0)  strcat (ltit, lext);
      radius = rmin;
      rmfc (&radius, 1, ci, nval, scandx, nval);
      rmfc (&radius, 1, p0, nval, scandx, nval);
      }

   for (i=0; i<nval; i++)
      {
      if (ci [i] < 0.0)  ci [i] = 0.0;
      if (p0 [i] < 0.0)  p0 [i] = 0.0;
      }

/*
 *   if (igterm)
 *      {
 *      xyplot (scandx, p0, nval, namemc, ltit, labx, "I (p0)  [B/Bsun]",
 *              ixscale, iyscale, ibkg);
 *      xyplot (scandx, ci, nval, namemc, ltit, labx, "I (p1,p2,p3)  [B/Bsun]",
 *              ixscale, iyscale, ibkg);
 *      }
 */
   polcmd ();
   break;

   /*** 'pol' ... Polarization fraction. ***/

case 2:

   for (i=0; i<nval; i++)
      {
      if (ci[i] != 0.0)
         pct[i] = sqrt ((double) (cq[i]*cq[i] + cu[i]*cu[i] )) / ci[i];
      else
         pct[i] = 1.0;
     if (pct[i] > 1.0 )  pct[i] = 1.0;
     if (pct[i] < 0.0 )  pct[i] = 0.0;
     }

   if (igterm)
      xyplot (scandx, pct, nval, namemc, ltit, labx, "Polarization Fraction",
              ixscale, iyscale, ibkg);

   polcmd ();
   break;

   /*** 'pb' ... Polarization * Brightness. ***/

case 3:

   for (i=0; i<nval; i++)
      pb [i] = sqrt ((double) (cq [i] * cq [i] + cu [i] * cu [i]) );
   if (igterm)
      xyplot (scandx, pb, nval, namemc, ltit, labx, "pB  [B/Bsun]",
              ixscale, iyscale, ibkg);
   polcmd ();
   break;

   /*** 'angle'... Polarization angle (deviation). ***/

case 4:

   for (i=0; i<nval; i++)
      {
      th = thmin;
      if (scntyp [0] == 't')  th = scandx [i];
      if (th > 360.0)         th = th - 360.0;

      alpha = 0.5 * degrad * (float) atan2 ((double) cu[i], (double) cq[i]);
/*
 *      printf ("cu: %f  cq: %f  alpha: %f\n", cu[i], cq[i], alpha);
 */
      alpha = alpha - 90.0;
      if (alpha <   0.0)  alpha += 360.0;
      if (alpha > 360.0)  alpha -= 360.0;

      delang = alpha - th;
      fsign = 1.0;
      if (delang != 0.0)  fsign = delang / (float) fabs((double) delang);
      while ((float) fabs((double) delang) > 45.)  delang -= 90.0 * fsign;
      pangle [i] = delang;
      }

   faverage (nave, pangle, nval);

   if (igterm)
      xyplot (scandx, pangle, nval, namemc, ltit, labx,
              "Polarization deviation angle [degrees]", ixscale, iyscale, ibkg);
   polcmd ();
   break;

   /*** "brt"...Plot clear intensity, polaroid intensity & ratio: clear/zi. ***/

case 5:

   if (igterm)
      {
      xyplot (scandx, p0, nval, namemc, ltit, labx,
              "I (p0)  [B/Bsun]", ixscale, iyscale, ibkg);

      xyplot (scandx, ci, nval, namemc, ltit, labx,
              "I (p1,p2,p3)  [B/Bsun]", ixscale, iyscale, ibkg);

      for (i=0; i<nval; i++)
         {
         if (ci [i] == 0.0 || (scandx [i] > 150.0 && scandx [i] < 210.0))
            p0123 [i] = 1.0;
         else
            p0123 [i] = p0 [i] / ci [i];
         }
      xyplot (scandx, p0123, nval, namemc, ltit, labx,
              "ratio   I (p0) / I (p1,p2,p3)", ixscale, iyscale, ibkg);
      }
   polcmd ();
   break;

   /*** "cmp" . . . Plot clear vs. stokes intensity. ***/

case 6:

   if (igterm)
      {
      sprintf (labelx, "Clear I (p0)   [B/Bsun]");
      sprintf (labely, "Stokes I (p1,p2,p3)   [B/Bsun]");
      sprintf (title, "Clear (p0) vs. Polaroids (p1+p2+p3)");
      strcpy (lext, " -");
      if (lcc)  strcat (lext, "c");  else strcat (lext, " ");
      if (lstr) strcat (lext, "s");  else strcat (lext, " ");
      if (lvig) strcat (lext, "v");  else strcat (lext, " ");
      if (lfc)  strcat (lext, "f");  else strcat (lext, " ");
      if ((strcmp (lext, " -    ")) != 0)  strcat (title, lext);

      xypplot (p0, ci, nval, namemc, title, labelx, labely,
               ixscale, iyscale, ibkg, pmin, pmax);
      }
   polcmd ();
   break;

   /*** 'stokes' ... Plot Stokes vector components I,Q,U. ***/

case 7:

   if (igterm)
      {
      xyplot (scandx, ci, nval, namemc, ltit, labx, "I (p1,p2,p3) [B/Bsun]",
              ixscale, iyscale, ibkg);
      xyplot (scandx, cq, nval, namemc, ltit, labx, "Q (p1,p2,p3) [B/Bsun]",
              ixscale, iyscale, ibkg);
      xyplot (scandx, cu, nval, namemc, ltit, labx, "U (p1,p2,p3) [B/Bsun]",
              ixscale, iyscale, ibkg);
      }
   polcmd ();
   break;

   /*** 'help'... Print command list. ***/

case 8:

   polcmd ();
   break;

   /*** 'exit' ... exit program. ***/

case 9:
default:

   if (igr == 1)
      {
      drawmode (OVERDRAW);
      gl_wclear (ov_black);

      drawmode (NORMALDRAW);
      onemap ();		/* Set single-map mode for color table. */
      gconfig ();
      pixmode (PM_ADD24, 0);	/* i.e., effectively use color map #0.*/
      gexit();
      }

   if (igterm)
      {
      clsgks_ ();
/*      unlink (gksnam); */
      }

   sprintf (gksout, "NCARG_GKS_OUTPUT=gks.cgm");
   putenv (gksout);

   /*** Close all files. ***/

   fclose (lun);
   exit (0);

   }
   goto next;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
mcname (namvid)

char *namvid;

{
int lenvid;
int idot;
int i;
char paknam [80];

   lenvid = strlen(namvid);
   idot = -1;
   for (i = 0; i < lenvid; i++) 
      {
      if (namvid [i] == '.') idot = i;
      paknam [i] = namvid [i];
      }

   if (idot == -1)
      {
      printf ("mcname: invalid file name: %s\n", namvid);
      exit (-1);
      }

   paknam [idot+1] = 'c';
   paknam [idot+2] = 'g';
   paknam [idot+3] = 'm';
   paknam [idot+4] = '\0';

   return (paknam);
}
