/*==============================================================================
 * GRID.C	- This file contains various graphic routines for drawing 
 *		  grids on the display screen.
/*==============================================================================
 * Kevin Cole * Solar Maximum Mission * October 23, 1984
 *
 * FUNCTIONS INCLUDED HERE,
 *   grid (image,imgtype)         - draw tick marks along the image boundary.
 *   smmtogrin (image,x,y,ix,iy)  - SMM coordinate conversion routine.
 *   crosshair (x, y, dx, dy)     - draw a cross hair at x,y with lengths dx,dy.
 *   tgrid (image, tmin, tmax, r) - draw a tangential grid.
 *   rgrid (image, tmin, tmax, tstep) - draw a radial grid.
 *==============================================================================
 */
#include "scan.ext"
#include "trig.h"

/*------------------------------------------------------------------------------
 * G R I D ()	- draws tick marks at one solar radius interval along the
 *		  boundary of the display area. 
 *
 * external routines required by GRID(),
 *     drawbox()	- draw a box surrounding the image display area.
 *     smmtogrin()	- convert from Solar Max hi resolution frame coord. to
 *			  512 x 512 image display coordinates.
 *     crosshair()	- draw a cross hair on the image display.
 *     ggetdx()		- returns image display area extent in x-pixels.
 *     ggetdy()		- returns image display area extent in y-pixels.
 *     ginitx()	 	- returns initial display area position in x-pixels.
 *			  x = 0 at left of screen.
 *     ginity()		- returns initial display area position in y pixels.
 *			  y = 0 at top of screen.
 *     wvec()		- draw a vector on the display screen.
 *
 * extern constants required by GRID(),
 *     gMINCOL		- Minimum horizontal-pixel screen location.
 *     gMAXCOL		- Size of the display screen in horizontal-pixels.
 *     gMINROW	 	- Minimum vertical-pixel screen location.
 *     gMAXROW		- Maximum vertical-pixel screen location.
 *------------------------------------------------------------------------------
 */

grid (image, imgtype)
struct IMAGE *image;
char *imgtype;
{
    extern drawbox(), smmtogrin(), crosshair();
    extern ginitx(), ginity(), ggetdx(), ggetdy();
    int ticlength = 10;			/* length of a tick mark in pixels */
    int x, y, xg, yg,  dx, dy, ix, iy;
    int xl, xr, yt, yb;			/* xleft, xright, ytop, ybottom       */

/* Get the image display area size in pixels. */

    dx = ggetdx ();  dy = ggetdy ();

/* Get the initial x and y screen positions.   */

    ix = ginitx ();   iy = ginity ();

/* Clip the max and min pixels locations to the screen boundaries. */

    if ((xl = ix - ticlength)      < gMINCOL) xl = gMINCOL;
    if ((xr = ix + ticlength + dx) > gMAXCOL) xr = gMAXCOL;
    if ((yt = iy + ticlength)      > gMAXROW) yt = gMAXROW;
    if ((yb = iy - dy - ticlength) < gMINROW) yb = gMINROW;

/* Draw a box around the image display area. */

    drawbox ();

/* Convert the image center position to Grinnell coordinates */

    smmtogrin (image, image->xcen, image->ycen, &xg, &yg);

/* Draw a crosshair at (xg,yg) with length 10. */

    crosshair (xg, yg, 10, 10);

/* Begin the display grid by drawing the first tic mark at the center position
 * projected onto the surrounding box. 
 * 
 * first the left hand side...
 */
    wvec (ix, yg, xl, yg);

/* Next, the right hand side. */ 

    wvec (ix+dx, yg, xr , yg);

/* The top. */

    wvec (xg, yt, xg, iy);

/* The bottom. */

    wvec (xg, yb,  xg, iy - dy);

/* Now, draw the remaining ticks along each boundary. */

/* Draw the right hand side tics. */

    y = yg;
    while ( (y += image->pixsr) < iy) 
	wvec (ix+dx, y, xr, y);
    y = yg;
    while (((y -= image->pixsr) < iy) && (y > (iy - dy))) 
	wvec (ix+dx, y, xr, y);

/* Draw the left hand side tics. */

    y = yg;
    while (((y += image->pixsr) < iy) && ( y > (iy - dy))) 
	wvec (ix, y, xl, y);
    y = yg;
    while (((y -= image->pixsr) < iy) && (y > (iy - dy))) 
	wvec (ix, y, xl, y);

/* Draw ticks along the top of the image. */

    x = xg;
    while (((x += image->pixsr) > ix) && (x < (ix + dx)))
	wvec (x, iy, x, yt);
    x = xg;
    while (((x -= image->pixsr) > ix) && (x < (ix + dx)))
	wvec (x, iy, x, yt);

/* Draw ticks along the bottom of the image. */

    x = xg;
    while (((x += image->pixsr) > ix) && (x < (ix + dx)))
	wvec (x, iy - dy, x, yb);
    x = xg;
    while (((x -= image->pixsr) > ix) && (x < (ix + dx)))
	wvec (x, iy- dy, x, yb);
}


/*-----------------------------------------------------------------------------
 * S M M TO G R I N ()	- Convert from Solar Max Hi resolution frame coordinates
 *			  to Grinnell 512x512 frame buffer coordinates.
 *
 * external routines required;
 *     ginitx()	   - returns the initial x position of the Image Display window.
 *     ginity()	   - returns the initial y position of the Image Display window.
 *-----------------------------------------------------------------------------
 */
smmtogrin (image, x, y, xg, yg)
struct IMAGE *image;
float x, y;			/* SMM Hi Resolution frame coordinates */
int *xg, *yg;			/* Returns the grinnell coordinates    */
{
    extern int ginitx(), ginity();      /* initial x,y screen coordinates */
    int nres;

    nres = 1;				/* initialize for low res images  */
    if(image->ires == 0) nres = 2;  	/* if hi res image                */

    *xg = (int) x/nres + ginitx();
    *yg = ginity() - (int) y/nres;
}


/*-----------------------------------------------------------------------------
 * C R O S S H A I R ()   - Draws a cross hair with center (screen coordinates)
 *		            at x,y with lengths dx and dy.
 *
 * If the coordinates fall outside the displayable screen area. The the 
 * coordinates are clipped to the screen boundary.
 *
 * externals required;
 *    wvec()	- draw a vector on the display screen.
 *    gMINROW, gMAXROW  - integers. Minimum and maximum vertical pixels.
 *    gMINCOL, gMAXCOL  - integers. Minimum and maximum horizontal pixels.
 *-----------------------------------------------------------------------------
 */
crosshair (x, y, dx, dy)
int x, y;				/* cross hair center position */
int dx, dy;				/* cross hair extent in pixels */
{
/* Clip the coordinates to the display screen area. */

   if (x < gMINCOL) x = gMINCOL;
   if (x > gMAXCOL) x = gMAXCOL;
   if (y > gMAXROW) y = gMAXROW;
   if (y < gMINROW) y = gMINROW;

   if (x + dx < gMAXCOL) 
      wvec (x, y, x+dx, y);
   else
      wvec (x, y, gMAXCOL, y);

   if (x - dx > gMINCOL)
      wvec (x, y, x-dx, y);
   else
      wvec (x, y, gMINCOL, y);

   if (y+dy < gMAXROW)
      wvec (x, y, x, y+dx);
   else
      wvec (x, y, x, gMAXROW);

   if (y-dy > gMINROW)
      wvec (x, y, x, y-dx);
   else
      wvec (x, y, x, gMINROW);
}

/*-----------------------------------------------------------------------------
 * R G R I D ()		- draw a grid of radial lines
 *
 * functions required:
 *  coord()
 *-----------------------------------------------------------------------------
 */
rgrid (icb, tmin, tmax, tstep)

struct IMAGE  *icb;     /* Pointer to image structure.			*/
float   tmin, tmax;     /* Theta (degrees) min and max values		*/
float   tstep;		/* stepsize (degrees) for theta			*/
{
   float   Tmin, Tmax;
   float   x1, y1, x2, y2;
   float   r, rstep = 0.1, rmin, rmax, th;
   int     nres;
   int     ixg1, iyg1, ixg2, iyg2;
   int     ierr;

   if (scans->GRN)  disp (scans->GRIDOVL);

   Tmin = tmin;
   Tmax = tmax;
   if (Tmin < 0) Tmin += 360.0;
   if (Tmax < 0) Tmax += 360.0;
   if (Tmax < Tmin)  Tmax += 360.0;

/*
 *   if (Tmax < Tmin)
 *      {
 *      temp = Tmax;
 *      Tmax = Tmin;
 *      Tmin = temp;
 *   }
 */

   /*** Set rmax large enough to extend to the edge of the display. ***/

   rmin = 1.1;
   rmax = 5.25;

   for (th = Tmin; th <= Tmax; th += tstep)
      for (r = rmin; r < rmax; r += rstep)
         {

         /*** Coordinate transformation :  (theta,angle) --> (pixel,line). ***/

/*
 *       if ((ierr = coord (icb, &rmin, &th, &x1, &y1, 1)) != 0)  return (ierr);
 *       if ((ierr = coord (icb, &rmax, &th, &x2, &y2, 1)) != 0)  return (ierr);
 */
         if ((ierr = coord (icb, &r, &th, &x1, &y1, 1)) != 0)  return (ierr);

         /*** Draw "X" to mark scan point on Grinnell display. ***/
 
         nres = 1;  			   /* initialize for low res images */
         if (icb->ires == 0)  nres = 2;	   /* if hi res image               */

         ixg1 = (int) (x1 / nres + ginitx () + 0.5);
         iyg1 = (int) (ginity () - y1 / nres + 0.5);
         ixg2 = (int) (x2 / nres + ginitx () + 0.5);
         iyg2 = (int) (ginity () - y2 / nres + 0.5);
         if (ixg1 < ginitx ())              ixg1 = ginitx();
         if (ixg1 > ginitx () + ggetdx ())  ixg1 = ginitx () + ggetdx ();
         if (iyg1 > ginity ())              iyg1 = ginity ();
         if (iyg1 < ginity () - ggetdy ())  iyg1 = ginity () - ggetdy ();
         if (ixg2 < ginitx ())              ixg2 = ginitx();
         if (ixg2 > ginitx () + ggetdx ())  ixg2 = ginitx () + ggetdx ();
         if (iyg2 > ginity ())              iyg2 = ginity ();
         if (iyg2 < ginity () - ggetdy ())  iyg2 = ginity () - ggetdy ();
         wvec (ixg1, iyg1, ixg1, iyg1);
         }
   return (0);
}


/*-----------------------------------------------------------------------------
 * T G R I D ()		- draw a grid of theta lines
 *
 * functions required:
 *  coord()
 *-----------------------------------------------------------------------------
 */
tgrid (icb, rmin, rmax, rstep)

struct IMAGE  *icb;     /* Pointer to image structure.			*/
float   rmin, rmax;     /* Theta (degrees) min and max values		*/
float   rstep;		/* stepsize (degrees) for theta			*/
{
   float   Rmin, Rmax, r;
   float   th, tmin = 0.0, tmax = 360.0, tstep = 2.0;
   float   x, y;
   int     nres;
   int     ixg, iyg;
   int     ixmin = 63, ixmax = 510;
   int     iymin = 63, iymax = 510;
   int     ierr;

   if (scans->GRN)  disp (scans->GRIDOVL);

   Rmin = rmin;
   Rmax = rmax;

   for (r = rmin; r <= rmax; r += rstep)
      for (th = tmin; th <= tmax; th += tstep)
         {

         /*** Coordinate transformation :  (theta,angle) --> (pixel,line). ***/

         if ((ierr = coord (icb, &r, &th, &x, &y, 1)) != 0)  return (ierr);

         /*** Draw "X" to mark scan point on Grinnell display. ***/
 
         nres = 1;  			     /* initialize for low res images */
         if (icb->ires == 0)  nres = 2;	     /* if hi res image               */

         ixg = (int) (x / nres + ginitx () + 0.5);
         iyg = (int) (ginity () - y / nres + 0.5);
	 if (ixg > ixmin && ixg < ixmax && iyg > iymin & iyg < iymax)
            wvec (ixg, iyg, ixg, iyg);
         }
   return (0);
}
/*-----------------------------------------------------------------------------
 * P G R I D ()		- draw a polar grid
 *
 * functions required:
 *  coord()
 *-----------------------------------------------------------------------------
 */
pgrid (icb, rmin, rmax, rstep, tmin, tmax, tstep)

struct IMAGE  *icb;     /* Pointer to image structure.			*/
float   rmin, rmax;	/* Radial min and max values [rsun]		*/
float   rstep;		/* Radial stepsize [rsun]			*/
float   tmin, tmax;     /* Theta (degrees) min and max values		*/
float   tstep;		/* stepsize (degrees) for theta			*/
{
   float   Rmin, Rmax, Tmin, Tmax;
   float   r, temp, x1, y1, x2, y2, th;
   int     nres;
   int     ixg2, iyg2, ixg1, iyg1, ierr;

   Tmin = tmin;
   Tmax = tmax;
   if (Tmin < 0) Tmin += 360.0;
   if (Tmax < 0) Tmax += 360.0;
   if (Tmax < Tmin)  Tmax += 360.;

   Rmin = rmin;
   Rmax = rmax;
   if (Rmax < Rmin)
      {
      temp = Rmax;
      Rmax = Rmin;
      Rmin = temp;
      }

   sprintf (wbuff, "Tmin:%f  Tmax:%f  Rmin:%f  Rmax:%f",
	    Tmin, Tmax, Rmin, Rmax);
/*   disp_err (wbuff); */

   if (scans->GRN)  disp (scans->GRIDOVL);

   for (th = Tmin; th <= Tmax; th += tstep)
      {

      /*** Coordinate transformation :  (theta,angle) --> (pixel,line). ***/

      if ((ierr = coord (icb, &Rmin, &th, &x1, &y1, 1)) != 0)  return (ierr);
      if ((ierr = coord (icb, &Rmax, &th, &x2, &y2, 1)) != 0)  return (ierr);

      /*** Draw "X" to mark scan point on Grinnell display. ***/
 
      nres = 1;  			     /* initialize for low res images */
      if (icb->ires == 0)  nres = 2;   	     /* if hi res image               */

      ixg1 = (int) x2 / nres + ginitx ();
      iyg1 = ginity () - (int) y2 / nres;
      ixg2 = (int) x1 / nres + ginitx ();
      iyg2 = ginity () - (int) y1 / nres;
      if (ixg1 < ginitx ())  ixg1 = ginitx ();
      if (ixg1 > ginitx () + ggetdx ())  ixg1 = ginitx () + ggetdx () -1;
      if (iyg1 > ginity ())  iyg1 = ginity ();
      if (iyg1 < ginity () - ggetdy ())  iyg1 = ginity () - ggetdy ();
      if (ixg2 < ginitx())  ixg2 = ginitx ();
      if (ixg2 > ginitx () + ggetdx ())  ixg2 = ginitx () + ggetdx () - 1;
      if (iyg2 > ginity ())  iyg2 = ginity ();
      if (iyg2 < ginity () - ggetdy ())  iyg2 = ginity () - ggetdy ();
      wvec (ixg1, iyg1, ixg2, iyg2);
      }
   for (r = Rmin; r <= Rmax; r += rstep)
      (void) tarc (icb, Tmin, Tmax, r);
   return (0);
}


/*------------------------------------------------------------------------------
 * T H A R C ()		- draw an arc.
 *
 * functions required:
 *   coord()
 *   ginitx(), ginity()
 *------------------------------------------------------------------------------
 */

tarc (image, thmin, thmax, radius)
struct IMAGE *image;    /* Pointer to image array. 			*/
float   thmin, thmax;   /* Solar position angle limits for theta scan. 	*/
float   radius;         /* Radius (R/Rsun units). 			*/
{
    extern int ginitx(), ginity();
    float   x, y;
    float   r, th;
    float   tinc;
    int  nres, ierr, ixgp, iygp, ixg, iyg;

/* Theta increment: set default to angle separating two adjacent pixels. */

/*
 *   tinc = RDDG / (radius * image->pixsr);
 *   tinc *= 2.0;
 */
    tinc = 1.0;

   r = radius;
   if (thmax < thmin)  thmax += 360.0;

   for (th = thmin; th <= thmax; th += tinc)
      {

      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = coord (image, &r, &th, &x, &y, 1)) != 0)  return (ierr);

      nres = 1;  			 /* initialize for low res images */
      if (image->ires == 0)  nres = 2;    /* if hi res image               */

      /*** Draw "X" to mark theta scan point on Grinnell display. ***/

      ixg = (int) (x / nres + ginitx () + 0.5);
      iyg = (int) (ginity () - y / nres + 0.5);
      if (ixg < ginitx ())  ixg = ginitx ();
      if (ixg > ginitx () + ggetdx ())  ixg = ginitx () + ggetdx ();
      if (iyg > ginity ())  iyg = ginity ();
      if (iyg < ginity () - ggetdy ())  iyg = ginity () - ggetdy ();
      ixgp = ixg + 1;
      iygp = iyg + 1;
      if (ixgp < ginitx ())  ixgp = ginitx ();
      if (ixgp > ginitx () + ggetdx ())  ixgp = ginitx () + ggetdx ();
      if (iygp > ginity ())  iygp = ginity ();
      if (iygp < ginity () - ggetdy ())  iygp = ginity () - ggetdy ();
/*      wvec (ixg, iyg, ixgp, iygp); */
      wvec (ixg, iyg, ixg, iyg);
      }
   return (0);
}
