/* lscan.c
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Line Scan procedure.
 * Extracts values from an image along a line between two arbitrary end points.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984
 * 17 March 1986: add stray light/vignetting * F-corona removal features.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 */

#include <math.h>
#include "scan.ext"
#include "max.c"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

lscan (ifc, ip1, ir1, ip2, ir2, scntyp, scan, scandx, ns)

struct IMAGE  *ifc;	
int	ip1, ir1, ip2, ir2;	/* Pixel,line coordinate scan limits. */
int	*ns;			/* Number of values in scan. */
int	scntyp;		        /* Scan type: radial=1 or line=2. */
float	scan[];			/* Dependent axis scan array. */
float	scandx[];		/* Independent axis scan array. */
{
    float stray(), vignet(), fcorona();
    short   *ims;		/* Pointer to 2 byte image array. */
    float   x, y, slope, yoff;
    float   x1, x2, y1, y2;
    float   dist, dstinc, dstsun;
    float   r1, r2, th1, th2;
    float   r, th;
    float   pix1, rec1, pix2, rec2;
    int	    i, nres;
    int	    ierr;			/* Error. */
    int	    ival1, ival2;
    int	    ix, ixinc, iy, iyinc;
    int	    ix1, ix2, iy1, iy2;
    int	    ixg, iyg;
    int	    nx, nxdif;
    int	    ny, nyinc, nydif;

    ims = (short *) ifc->addr;

    nres = 1;
    if(ifc->ires == 0) nres = 2;

   /*** Clear scan arrays. ***/

    for (i=0; i < MAXSCN; i++)  scandx[i] = scan[i] = 0.;

   /*** Check scan size. ***/
    ierr = 0;
    *ns = max(abs(ip2-ip1),abs(ir2-ir1)) + 1;

    if(*ns > MAXSCN)
        return(-1);

   /*** Compute distance (in Rsun units) between scan values. ***/
   /*   (line scans only) */

	 dist = sqrt((double)(ip2-ip1)*(ip2-ip1) + (double)(ir2-ir1)*(ir2-ir1));
	 dstinc = dist / ((*ns-1) * ifc->pixsr);
	 dstsun = -dstinc;

   /*** Orient start/end points so that start point is closest to disk. ***/
   /*   (radial scans only) */

	pix1 = (float) ip1;
	rec1 = (float) ir1;
	pix2 = (float) ip2;
	rec2 = (float) ir2;

	if((ierr = coord(ifc,&r1,&th1,&pix1,&rec1,-1)) != 0) return(ierr);
	if((ierr = coord(ifc,&r2,&th2,&pix2,&rec2,-1)) != 0) return(ierr);

	/*** Reverse start/end points. ***/

	if(r1 > r2 && scntyp == 1)
	{
	 ix1 = ip2;
	 ix2 = ip1;
	 iy1 = ir2;
	 iy2 = ir1;
	}
	else
	/*** Leave start/end points as given. ***/
	{
	 ix1 = ip1;
	 ix2 = ip2;
	 iy1 = ir1;
	 iy2 = ir2;
	}

        /** Equation of a line:   y = slope * x + yoff. ***/

	x1 = (float) ix1;
	x2 = (float) ix2;
	y1 = (float) iy1;
	y2 = (float) iy2;
	slope = -1.0e+9;
	yoff = 0.;
	if(ix1 != ix2)  slope = (y2 - y1) / (x2 - x1);
	if(ix1 != ix2)  yoff  = (x2*y1 - x1*y2) / (x2 - x1);

   if(fabs((double) slope) < 1.0)
   {
   /*** ...... pixel-driven loop : ***/

      nxdif = ix2 - ix1;
      if(nxdif < 0)  nxdif = -nxdif;
      nx = nxdif + 1;
      ix = ix1 - 1;
      ixinc = 1;
      if(ix1 > ix2)
	{
         ix = ix1 + 1;
         ixinc = -1;
	}

	for (i = 0; i < nx; i++)
	{
            ix = ix + ixinc;
            x = (float) ix;
            y = (float) iy1;
            if(iy1 != iy2)  y = slope * x + yoff;
            iy = (int) y;

   /*** 1 byte per pixel:  clear all high order bits above low-order byte. ***/

	    if(ifc->bppx == 1)
	    {
	        ival1 = (*(ifc->addr
			+(ix - ifc->ip)+(iy - ifc->ir)*ifc->dx) & 0377);
	        ival2 = (*(ifc->addr
			+(ix - ifc->ip)+(iy - ifc->ir+1)*ifc->dx) & 0377);
	    }
	    else
	    /* 2 bytes per pixel. */
	    {
	        ival1 = *(ims + (ix - ifc->ip) + (iy - ifc->ir)*ifc->dx);
	        ival2 = *(ims + (ix - ifc->ip) + (iy - ifc->ir+1)*ifc->dx);
	    }

            /*** Linear interpolation in vertical axis. ***/

            scan[i] = (float) ((ival1 + (ival2-ival1) * (y-iy)) * ifc->scale);

            /*** Determine independent axis coordinate value. ***/

            if((ierr = coord(ifc, &r,&th,&x,&y,-1)) != 0) return(ierr);
	     if (ifc->dtype == 3)
		{
		if (scnst->lsv == 1) scan[i] = (scan[i] - stray(r)) * vignet(r);
		if (scnst->lfc == 1) scan[i] -= fcorona(r,th);
		}

            dstsun = dstsun + dstinc;
            if(scntyp == 1)  scandx[i] = (float) r;
            if(scntyp == 2)  scandx[i] = (float) dstsun;

            /*** Plot 2 nearest points on Grinnell in graphics overlay. ***/

            if(scnst->DSP)
            {
           	ixg = ix/nres + ginitx();
           	iyg = ginity() - iy/nres;
		gl_wvec(ixg,iyg,ixg,iyg+1, scnst->LSCANOVL);
            }
	    *ns = nx;
	}
   }
   else
      {
      /*** ...... line-driven loop : ***/
      nydif = iy2 - iy1;
      if(nydif < 0)  nydif = -nydif;
      ny = nydif + 1;
      iy = iy1 - 1;
      iyinc = 1;
      if (iy1 > iy2)
         {
         iy = iy1 +1;
         iyinc = -1;
         }

      for (i = 0; i < ny; i++)
         {
         iy = iy + iyinc;
         x = (float) ix1;
         y = (float) iy;
         if(ix1 != ix2)  x = (y-yoff) / slope;
         ix = (int) x;

        /*** Extract 2 adjacent pixels (in same line) from image. ***/

        /* 1 byte per pixel:  clear all high order bits above low-order byte. */

        if (ifc->bppx == 1)
           {
           ival1 = (*(ifc->addr
                   +(ix - ifc->ip) +(iy- ifc->ir)*ifc->dx) & 0377);
           ival2 = (*(ifc->addr
                   +(ix - ifc->ip+1)+(iy- ifc->ir)*ifc->dx) & 0377);
           }
        else

           /* 2 bytes per pixel. */

           {
           ival1 = *(ims + (ix- ifc->ip)   + (iy- ifc->ir)*ifc->dx);
           ival2 = *(ims + (ix- ifc->ip+1) + (iy- ifc->ir)*ifc->dx);
           }

         /*** Linear interpolation in horizonal axis. ***/

         scan[i] = (float) ((ival1 + (ival2-ival1) * (x-ix)) * ifc->scale);
	 if (scnst->lsv == 1) scan[i] = (scan[i] - stray(r)) * vignet(r);

         /*** Determine independent coordinate value for scan. ***/

         if ((ierr = coord(ifc,&r,&th,&x,&y,-1)) != 0)  return (ierr);
         if (ifc->dtype == 3)
	    {
            if (scnst->lsv == 1) scan[i] = (scan[i] - stray(r)) * vignet(r);
            if (scnst->lfc == 1) scan[i] -= fcorona(r,th);
            }
         dstsun = dstsun + dstinc;
         if (scntyp == 1)  scandx[i] = (float) r;
         if (scntyp == 2)  scandx[i] = (float) dstsun;

         /*** Plot 2 nearest points on Grinnell in graphics overlay. ***/

         if (scnst->DSP)
            {
            ixg = ix/nres + ginitx ();
            iyg = ginity() - iy/nres;
            gl_wvec (ixg, iyg, ixg+1, iyg, scnst->LSCANOVL);
	    }
	 }
      *ns = ny;
      }
   return (ierr);
}
