/*************************************************************************
*    void calscan(xval,yval,dnval,nval)				         *
**************************************************************************
*    Subroutine to calibrate scan arrays (data produced by the "SCAN"    *
*    program) of raw SMM images.					 *
*    This routine does NOT include the geometric distortion corrections. *
*									 *
*    Input:								 *
*    xval   = array, pixel positions of the data 			 *
*    yval   = array, line positions of the data			         *
*    dnval  = array, raw vidicon digital numbers			 *
*    nval   = number of points in the arrays				 *
*									 *
*    Output:								 *
*    dnval  = array, calibrated data in B/Bsun units			 *
*									 *
*    original author: Michele D. De La Pena   11 May 1987		 *
*    modified by: Andrew L. Stanger                                      *
*    12 May 1989: use 448x448 resolution parameter files.                *
*                 use 3rd order polynomial transfer curve equation.      *
*    9 April 1991: adapt for SGI IRIS.                                   *
*************************************************************************/
#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"

void
calscan (xval, yval, dnval, nval)

float xval[], yval[], dnval[] ;
int nval;
{
   extern char *malloc () ;		/* System memory allocation function. */
   /*
    *** 3rd order polynomial parameter files.
    *** These files are 448x448 resolution.
    */
   static char pardir[] = "/picard/cp/cal/param/";   /* parameter directory. */
   static char nama0 [80];	/* a0 parameter file name. */
   static char nama1 [80];	/* a1  parameter file name. */
   static char nama2 [80];	/* a2 parameter file name. */
   static char nama3 [80];	/* a3 parameter file name. */
   static char fila0 [80];	/* a0 parameter path name. */
   static char fila1 [80];	/* a1 parameter path name. */
   static char fila2 [80];	/* a2 parameter path name. */
   static char fila3 [80];	/* a3 parameter path name. */
   int fda0, fda1, fda2, fda3 ;	/* File descriptors for parameter files. */
   /*
    *** Equivalent second adjustment for non-green filter exposures.
    */
   static float excolr[8] = {1.266,15.798,0.200,1.025,0.444,1.0,94.16,94.16} ;
   float escorr,expdur ;
   /*
    *** Radiance coefficients of 10**-11 B/Bsun units.
    */
   static float radfil[8] = {376.0,463.0,393.0,334.0,324.0,324.0,522.0,522.0} ;
   float radfac ;
   /*
    *** Define output pixel,line arrays.
    *** [0,0] = upper left corner of an SMM image.
    */
   int i,j,k ;		/* Index variables.			*/
   int jk ;		/* Index variable.			*/
   int len;		/* Size (bytes) of the parameter files. */
   float *a0, *a1, *a2, *a3 ;	/* Arrays for polynomial parameters. */
   float z, z2, z3, eq;
   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   len = 448 * 448 * sizeof (float) ;
   /*
    *** Convert raw DN values to exposure units.
    *** First, determine the equivalent second correction based upon
    *** the color filter in use.
    */
   escorr = excolr[H.ifw] ;
   expdur = (float)H.exp ;
   radfac = radfil[H.ifw] ;
   /*
    *** Open the Alpha, Beta and Offset parameter files.
    */
   sprintf (nama0, "a0.%dlo", H.year);
   sprintf (fila0, "%s%s", pardir, nama0);
   if ( (fda0 = open(fila0,0)) == -1 )
      {
      fprintf(stderr,"Parameter file %s cannot be opened.\n",fila0) ;
      exit(-1) ;
      }
   a0 = (float *) malloc (len);

   sprintf (nama1, "a1.%dlo", H.year);
   sprintf (fila1, "%s%s", pardir, nama1);
   if ( (fda1 = open(fila1,0)) == -1 )
      {
      fprintf(stderr,"Parameter file %s cannot be opened.\n",fila1) ;
      exit(-1) ;
      }
   a1 = (float *) malloc(len);

   sprintf (nama2, "a2.%dlo", H.year);
   sprintf (fila2, "%s%s", pardir, nama2);
   if ( (fda2 = open(fila2,0)) == -1 )
      {
      fprintf(stderr,"Parameter file %s cannot be opened.\n",fila2) ;
      exit(-1) ;
      }
   a2 = (float *) malloc(len);

   sprintf (nama3, "a3.%dlo", H.year);
   sprintf (fila3, "%s%s", pardir, nama3);
   if ( (fda3 = open(fila3,0)) == -1 )
      {
      fprintf(stderr,"Parameter file %s cannot be opened.\n",fila3) ;
      exit(-1) ;
      }
   a3 = (float *) malloc(len);
   /*
    *** Position the pointer at the beginning of the parameter files.
    */
   fprintf (stdout, "parameter files: %s %s %s %s\n",
            nama0, nama1, nama2, nama3);
   lseek(fda0,(long)0,0) ;
   lseek(fda1,(long)0,0) ;
   lseek(fda2,(long)0,0) ;
   lseek(fda3,(long)0,0) ;
   /*
    *** Read in the parameter data.
    */
   if ( read(fda0,a0,len) <= 0 )
      {
      fprintf(stderr,"Problems reading data on input file %s.\n",fila0);
      fprintf(stderr,"Exiting.\n\n") ;
      exit(-1) ;
      }

   if ( read(fda1,a1,len) <= 0 )
      {
      fprintf(stderr,"Problems reading data on input file %s.\n",fila1);
      fprintf(stderr,"Exiting.\n\n") ;
      exit(-1) ;
      }

   if ( read(fda2,a2,len) <= 0 )
      {
      fprintf(stderr,"Problems reading data on input file %s.\n",fila2);
      fprintf(stderr,"Exiting.\n\n") ;
      exit(-1) ;
      }

   if ( read(fda3,a3,len) <= 0 )
      {
      fprintf(stderr,"Problems reading data on input file %s.\n",fila3);
      fprintf(stderr,"Exiting.\n\n") ;
      exit(-1) ;
      }
   /*
    *** Compute the B/Bsun values from the raw DN values.
    *** B/Bsun = (equivalent seconds/exposure duration)*radiance
    */
   for ( i=0 ; i<nval ; i++ )
      {
      j = (int) xval[i] ;
      k = (int) yval[i] ;
      if ( H.ires == 0 )
         {
         j /= 2 ;
         k /= 2 ;
         }
      k *= 448 ;

      z = *(dnval + i);
      z2 = z * z;
      z3 = z2 * z;
      jk = j + k;
      eq = (*(a0+jk) + *(a1+jk) * z + *(a2+jk) * z2 + *(a3+jk) * z3);
      *(dnval + i) = ((eq * escorr) / expdur) * radfac;
      }

   free (a0) ;
   free (a1) ;
   free (a2) ;
   free (a3) ;

   close (fda0) ;
   close (fda1) ;
   close (fda2) ;
   close (fda3) ;

}	/*  end function calscan.c  */
