/* ptscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Theta Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984: Re-created on Vax-11/750; C language; UNIX 4.2 bsd.
 * 18 July 1984: Add stray light & vignetting removal capabilities.
 * 26 July 1984: Add calibration correction.  Add F-corona removal option.
 * 8 February 1985: Add average routine.
 * 6 December 1985: Apply fixcal to all calibrated images.
 * 16 December 1985: Replace 'coord' with 'rcoord'.
 * 17 November 1986: Replace 'nres' by 'gres' for Grinnell display.
 * 26 March 1987: add polarization filter corrections.
 * 27 May 1987: Added call to CALSCAN. Michele D. De La Pena
 * 5 August 1988: Remove Munro calibration correction.
 * 9 April 1991: adapt for SGI IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

#include "/home/riker/stanger/lib/include/cplib.ext"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

ptscan (namimg, image, thmin, thmax, thinc, radius, scan, scandx, ns)

char	namimg[] ;	/* Image file name. */
char	*image ;	/* Pointer to image array. */

float	thmin, thmax ;	/* Solar position angle limits for theta scan. */
float	thinc ;		/* Angular increment for theta scan. */
float	radius ;	/* Radius (R/Rsun units). */
float	scan[] ;	/* Dependent axis scan array. */
float	scandx[] ;	/* Independent axis scan array. */

int	*ns ;		/* Number of values in scan. */

{
extern void calscan() ;        /* Apply calibration parameters to raw data. */
extern void faverage() ;       /* Smoothing routine. */

extern float calcor() ;	       /* Munro calibration correction. */
extern float cpstray() ;       /* Stray light function. */
extern float vignet() ;	       /* Vignetting function. */
extern float fcorona() ;       /* F-corona function. */
extern float blint() ;	       /* Bilinear interpolation function. */
extern float polcor ;	       /* Polaroid transmission correction factor. */

extern int rcoord() ;	/* Coordinate transformation: polar <--> cartesian. */

float r, th ;
float tinc ;
float rbuf[1] ;
float Bexp ;		           /* "Generic" scale factor. */
float cc, str, vig, fcc, pc ;
float cval[MAXSCN] ;               /* Intermediate scan values. */
float xval[MAXSCN],yval[MAXSCN] ;  /* Pixel/line values for CALSCAN. */
float	x, y ;

int it ;
int ibounds;
int ixg, iyg ;
int ierr ;
int nr ;
int nscan ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Set scale factor. ***/

   Bexp = pscale ;
   str = 0.0 ;
   vig = 1.0 ;
   fcc = 0.0 ;
   cc = 1.0 ;
   pc = 1.0 ;
   /*
    *   if (polcor == 1.0)  pc = 1.;
    *   else pc = 0.5;
    */

   /*
    * Theta increment.
    * Set default to angle separating two adjacent pixels.
    */

   tinc = thinc ;
   if(thinc <= 0.0)  tinc = degrad / (radius*pixrs) ;

   r = radius ;

   /*** Clear scan arrays. ***/

   for (it = 0; it < MAXSCN; it++)  *(scan + it) = *(scandx+ it) = 0.0 ;
 
   /* ******* Theta scan loop ******* */
 
   if (thmax < thmin)  thmax += 360.0 ;

   *ns = 0 ;
   it = -1 ;
   for (th = thmin; th <= thmax; th += tinc)
      {
      it++ ;
 
      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = rcoord(&r,&th,&x,&y,1,scroll)) != 0)  return(ierr) ;
      xval [it] = x ;
      yval [it] = y ;
      ibounds = 0;
      if (x < 0.0 || x > 447.0)
	 {
	 ibounds = 1;
	 fprintf (stdout, "*** Invalid x: %3.0f\n");
	 }
      if (y < 0.0 || y > 447.0)
	 {
	 ibounds = 1;
	 fprintf (stdout, "   *** Invalid y: %3.0f\n");
	 }
      if (ibounds > 0)  fprintf (stdout, "   r: %5.2f   th: %7.2f\n", r, th);

      /*** Draw "X" to mark theta scan point on Grinnell display. ***/
 
      /*
       *   if (igr > 0)
       *      {
       *      ixg = (int) x/gres + 62 ;
       *      iyg = 511 - (int) y/gres ;
       *      wvec(ixg,iyg,ixg+1,iyg+1) ;
       *      wvec(ixg+1,iyg,ixg,iyg+1) ;
       *      }
       */

      /*
       * Perform bilinear interpolation to compute scan point from image.
       * Use 4 nearest neighbors.
       */

      *(scandx + it) = (float) th ;

      cval[it] = blint(image, &x, &y) ;

      }
   *ns = it + 1 ;

   if ( rawtocal == 1 )
      {
      calscan(xval,yval,cval,*ns) ;
      Bexp = 1.e-11;
      }
   /*
    * Apply the calibration to the data,
    * compute stray light, vignetting & F-corona values.
    * Apply Munro calibration correction.
    */
   th = thmin ;
   for ( it=0 ; it<*ns ; it++ )
      {
      /*
       * if (lcc == 1) cc = calcor(r) ;
       * if (lstr == 1) str = cpstray(r) * pc ;
       * if (lfc == 1) fcc = fcorona(r,th) * pc ;
       * if (lvig == 1) vig = vignet(r) ;
       * cval[it] *= (Bexp * cc * polcor) ;
       * *(scan + it) = (cval[it] - str) * vig - fcc ;
       */

      /***   cc = calcor(r) ; ***/

      *(scan + it) = cval[it] * Bexp * cc ;
      if (*(scan + it) < 0.0)  *(scan + it) = 0.0;
      /*
       * printf ("ptscan> th: %6.2f  cval: %g  scan: %g\n",
       *      th, cval[it], scan[it]);
       */
      th += tinc ;
      }

   /* 
    * Smooth data using 'nave' point average.
    */
   if ( rawtocal == 1 )  nave = 3 ;  else nave = 1;
   faverage (nave, scan, *ns) ;

   nscan = *ns ;

/*
 *   for (it=0; it<*ns; it++)
 *       printf("r:%7.3f  th:%7.2f  scan:%9.2e\n", r, scandx[it], scan[it]);
 */
   return(0) ;
}
