/* sg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Generates SMM C/P synoptic file on VAX.
 * Performs theta scans (tscan) on north, east, south & west sector images.
 * Stores scan data into a synoptic map structure, indexed on position angle.
 * Theta scan data is sampled every 1 degree.
 * Time resolution is 55 images per Carrington rotation (27.273 days), or
 *    one (n,e,s,w) set approximately every 12 hours.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * original author: Rainer Illing  25 July 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Modifications by Andrew L. Stanger :
 * 15 April 1987: Modify for roll and east photometric adjustment (calfac).
 * 16 April 1987: Add modulo 360 for latitude computation from scandx.
 * 23 April 1987: Round off offset value.
 * 29 April 1987: Generalize program to handle 1980,84,85,86 images.
 * 29 January 1988: Use look-up tables to determine carrington rotations.
 * 20 May 1988: adjust theta limits for sectors depending upon radius.
 * 25 May 1988: change theta scan values to short integers (instead of integers)
 * 1 June 1988: remove photometric adjustment for east.
 * 2 June 1988: offset for non-existent data bins is +1000.
 * 23 June 1988: modify cquad logic (eliminate double roll compensation).
 * 3 August 1988: replace tscan with ptscan (will calibrate theta scan data).
 * 5 August 1988: remove Munro calibration correction from ptscan.
 *                change roll adjustment: rolladj = - scroll.
 * 9 April 1991: adapt for SGI IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *	v1.60	16 April 1987	Accomodates arbitrary roll angles.
 *      v3.00   8 January 1988  Look up tables for Carrington rotations.
 *                              Data file contains two samples/doy.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <sys/types.h>
#include <errno.h>
/* #include <sv.h> */

#include "/usr/people/smm/lib/include/cplib.h"
#include "synst.h"		/* synoptic data file structure definition. */
struct	S	STT;
struct	S	STZERO;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

extern float	synrot();	/* Synodic rotation DOY start. */
extern float	syndur();	/* Synodic rotation duration. */
extern char	*rdimg();	/* Read image from disk to memory. */

register	struct	S	*ST = &STT;
register	struct	S	*ST0 = &STZERO;

char	*image;			/* Pointer to image array. */
static char	namsyn[80];	/* Synoptic Map data file name. */
static char	namlst[80];	/* File containing image name list. */
static char	namimg[80];	/* Image file name. */
static char	answer[2];	/* Storage for y/n response. */
static char	filcal[2];	/* is file calibrated? answer	*/
static char	smname[9][3] = {"sw","s ","se","e ","ne","n ","nw","w ","4q"};

static float scan[MAXSCN];	/* Pointer to independent scan array. */
static float scandx[MAXSCN];	/* Pointer to dependent scan array. */
float	begdoy;		/* First DOY for a synoptic map file. */
float	cmprot;		/* Central Meridian Passage Carrington Rotation. */
float	eastcr;		/* East limb carrington rotation. */
float	enddoy;		/* Last  DOY for a synoptic map file. */
float	imgdoy;		/* Image date in units of DOY [0-365 or 0-366]. */
float	mapdoy;		/* Map date in units of DOY [relative to map year]. */
float	radius;		/* Solar radius (Rsun). */
float	recdoy;		/* Initial image date in data file (in units of DOY). */
float	rolladj;	/* Roll adjustment angle (degrees). */
float	rotdur;		/* Duration of carrington rotation (days). */
float	thmin, thmax;	/* Theta limits (deg.). */
float	thinc;		/* Theta increment (deg.). */
float	westcr;		/* West limb carrington rotation. */

double	calfac = 1.0;	/* factor for scan value scaling. */
double  zoffset,zz;

long	offset;		/* offset for fseek.*/

register	i;
int	doy,hr,min;
int	ierr;
int	ioff;
int	iradius;
int	iovw = 0;	/* overwrite parameter. */
int	iyear;		/* Year for synoptic map. */
int	jj;
int	lsv;
int	nread;
int	numrec;		/* # theta scan records in synoptic map file. */
int	ndaycy;		/* # days in current year. */
int	ndaypy;		/* # days in previous year. */
int	nval;		/* # values in scan. */
/* int	posroll; */	/* roll (0-360 degrees). */
int	cquad;		/* coronal quadrant index. */
int	rstrt;		/* Starting carrington rotation number for map data. */
int	sizST;

FILE	*LIST, *SYN, *fopen();
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Initialization. ***/

   sizST = sizeof(STT);
   igterm = 0;		/* no graphics			*/
   igr = 0;		/* no Grinnell			*/
   lfc = 0;		/* no F-corona subtraction	*/

   /*** Request name of file containing input images. ***/

   fprintf (stderr,"$%s V3.00 \n", argv [0]);
   fprintf (stderr,"Enter name of file containing input images:  ");
   scanf ("%s",namlst);

   fprintf (stderr,"Are these calibrated images? [y/n]  ");
   scanf ("%s",filcal);

   calfac = 1.e+11;

   if (*filcal == 'y')
      rawtocal = 0 ;
   else
      rawtocal = 1 ;

   /*** Open file containing image list. ***/

   if ((LIST = fopen(namlst,"r")) == NULL)
      {
      fprintf (stderr,"cannot open file %s\n",namimg);
      exit(-1);
      }

   /*** Request year & radius for synoptic map data file. ***/

   fprintf (stderr, "Enter year for synoptic map file: ");
   fscanf  (stdin, "%d", &iyear);
   if (iyear > 1900) iyear -= 1900;

   fprintf(stderr,"Enter solar radius (Rsun) for theta scan : ");
   scanf("%f", &radius);
   printf ("scan radius = %g\n",radius);
   iradius = (int) (radius * 10. + 0.5);

   sprintf (namsyn, "csyn%02d.r%d", iyear, iradius);
   fprintf (stdout, "namsyn: %s\n", namsyn);

   /*** Set start/stop day-of-years. ***/

   switch (iyear)
      {
      case 80:
         {
         begdoy = 68.;
         enddoy = 267.;
	 ndaypy = 365;
	 ndaycy = 366;
         break;
         }
      case 84:
         {
         begdoy = 138.;
         enddoy = 397.;
	 ndaypy = 365;
	 ndaycy = 366;
         break;
         }
      case 85:
         {
         begdoy = -10.;
         enddoy = 386.;
	 ndaypy = 366;
	 ndaycy = 365;
         break;
         }
      case 86:
         {
         begdoy = -29.;
         enddoy = 383.;
	 ndaypy = 365;
	 ndaycy = 365;
         break;
         }
      case 87:
         {
         begdoy = 89.;
         enddoy = 400.;
	 ndaypy = 365;
	 ndaycy = 366;
         break;
         }
      case 88:
         {
         begdoy = -22.;
         enddoy = 389.;
	 ndaypy = 365;
	 ndaycy = 366;
         break;
         }
      case 89:
         {
         begdoy = -26.;
         enddoy = 399.;
	 ndaypy = 366;
	 ndaycy = 365;
         break;
         }
      default:
         {
         begdoy = 0.;
         enddoy = 0.;
         }
      }
   numrec = 2 * (int) (enddoy - begdoy + 1.);

   /*** Open synoptic map data file. ***/

   fprintf (stderr,"Is this a new file? [y/n] ");
   scanf ("%s",answer);
   if (*answer == 'y')
      {
      fprintf (stderr, "If old file (%s) exists, it will be overwritten.\n", namsyn);
      fprintf (stderr, "Are you sure that you want to create a new file (%s) ? ", namsyn);
      fscanf (stdin, "%s", answer);
      if (answer[0] != 'y')  exit(-2);

      i = creat (namsyn, 0644);
      close (i);
      if ((SYN = fopen(namsyn,"w+")) == NULL)
         {
         fprintf (stderr,"cannot open file %s\n",namimg);
         exit(-1);
         }

      /*** Initialize synoptic map data file. ***/

      ST->nec = 0.;
      ST->ec = 0.;
      ST->sec = 0.;
      ST->swc = 0.;
      ST->wc = 0.;
      ST->nwc = 0.;
      for (i=0; i<361; i++)  ST->th[i] = 0.;

      for (i=0; i<numrec; i++)
         {
         offset = i * sizST;
         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error: new rec  %d\n",i);
            exit (-1);
            }
         /*
          * Set times for each record to the nominal DOY date + 2000.
          * This indicates that no scan data is stored for that segment.
          * Each record represents one half of a day
          * (time period 00:00-11:59 or 12:00-23:59).
          */
         mapdoy = (float) i * 0.5 + begdoy;
	 recdoy = mapdoy + 2000;
         ST->net = recdoy;
         ST->et  = recdoy;
         ST->set = recdoy;
         ST->swt = recdoy;
         ST->wt  = recdoy;
         ST->nwt = recdoy;
         fprintf (stdout, "record: %03d   DOY: %7.2f\n", i, mapdoy);
         if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"write error: new rec  %d\n",i);
            exit (-2);
            }
         }
      rewind (SYN);
      }
   else
      {
      if ((SYN = fopen(namsyn,"r+")) == NULL)
         {
         fprintf (stderr,"cannot open file %s\n",namsyn);
         exit(-1);
         }
      }

   /******* Image loop. *******/

   while (fscanf(LIST,"%s",namimg) != EOF)
      {

      /*** Open input image disk file and read housekeeping record. ***/

      image = rdimg(namimg);
      hkline (stdout);
      rolladj = - (float) scroll;

      /* Extract image date in units of DOY. */

      imgdoy = H.doy + (H.hour+H.minute/60.)/24.;
      mapdoy = imgdoy;
      if (H.year > iyear)  mapdoy = imgdoy + ndaycy;
      if (H.year < iyear)  mapdoy = imgdoy - ndaypy;

      /*** Carrington Rotation at c.m.p. (Central Meridian Passage). ***/

      cmprot = synrot (H.year, imgdoy);
      rotdur = syndur ((int) cmprot);
      eastcr = cmprot + 0.25; 
      westcr = cmprot - 0.25; 
      fprintf (stdout, "mapdoy:%6.2f   cmprot:%7.2f   east c.r.:%7.2f   west c.r.:%7.2f   duration:%6.2f\n",
              mapdoy, cmprot, eastcr, westcr, rotdur);

      switch (H.ism)
         {
         case 1: cquad = 3;		/* South sector. */
                 break;
         case 3: cquad = 2;		/* East  sector. */
                 break;
         case 5: cquad = 1;		/* North sector. */
                 break;
         case 7: cquad = 4;		/* West  sector. */
                 break;
         default: cquad = -1;
                  break;
         }

      /*** Theta scan by sectors. ***/

      switch (cquad)
      {

/******* North Quadrant -- two parts: NE & NW. *******/

      case 1:
         {
         fprintf (stdout,"north file: %s\n",namimg);
         thmin = 315. + rolladj;
         if (radius > 2.5 && radius < 3.2)
              thmax =  40. + rolladj;
         else thmax =  45. + rolladj;
         if (thmin < 0.)  thmin += 360;
         if (thmax < 0.)  thmax += 360;
         if (thmin >= 360.)  thmin -= 360;
         if (thmax >  360.)  thmax -= 360;
         thinc = 1.;

         /*** Find correct record in synoptic file. ***/
         
         ioff = (int) ((mapdoy - begdoy) * 2);
         offset = (long) (ioff * sizST);

         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error r: %s\n",namimg);
            break;
            }
         if (nread = fread ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"read error> %s   nread: %d\n", namimg, nread);
            break;
            }

         /*
          * If correct bin is filled, check adjacent bins
          * (lower & higher) in time.
          */

         if (ST->net < 1000.)
            {

            /* Check lower bin.
             * Move current bin to lower one if time is  smaller than current
             * image, and put current image scan in current bin.
             */

         if (fseek(SYN,offset-sizST,0) == -1)
            {
            fprintf(stderr,"seek error r: %s\n",namimg);
            break;
            }
         if (fread ((char *) ST0, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"read error: %s\n",namimg);
            break;
            }
         if (ST0->net > 1000.)
            {
            if (fseek(SYN,offset-sizST,0) == -1)
               {
               fprintf(stderr,"seek error r: %s\n",namimg);
               break;
               }
            if (ST->net < mapdoy)
               {
               fprintf (stdout, "***Lower bin empty; filled with current bin.\n");
               if (fwrite ((char *)ST, sizST, 1, SYN) == 0)
                  {
                  fprintf(stderr,"write error: %s\n",namimg);
                  break;
                  }
               }
            else	/*** Put current image scan into lower bin. ***/
               {
               fprintf (stdout, "***Lower bin empty; filled with current image.\n");
               offset -= sizST;
               }
            cpST (ST0, ST);
            }

            /*** Check higher bin if lower bin is already filled. ***/

            if (ST0->net < 1000.)
               {
               if (fseek(SYN,offset+sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
            if (fread ((char *) ST0, sizST, 1, SYN) == 0)
               {
               fprintf(stderr,"read error: %s\n",namimg);
               break;
               }
            if (ST0->net > 1000.)
               {
               if (fseek(SYN,offset+sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (ST->net < mapdoy)
                  {
                  fprintf (stdout, "***High bin empty; fill with new image.\n");
                  offset += sizST;
                  }
               else
                  {
                  fprintf (stdout, "***High bin empty; fill with current bin.\n");
                  if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                     {
                     fprintf(stderr,"write error: %s\n",namimg);
                     break;
                     }
                  }
               cpST (ST0, ST);
               }
            else
               {
               fprintf (stdout, "!!!!!!! Both low & high bins also filled.\n");
               }
            }
         }

         if ((ST->net < 1000.) && (iovw == 0))
            {
            fprintf(stdout,"previous data: no overwrite\n");
            break;
            }

         /*** Perform theta scan. ***/

         ST->net = mapdoy;
         ST->nwt = mapdoy;
         ST->nec = eastcr;
         ST->nwc = westcr;
         ptscan (namimg,image,thmin,thmax,thinc,radius,scan,scandx,&nval);

         i = (int) (offset / sizST);
         fprintf (stdout, "ioff:%d   net:%7.2f   nwt:%7.2f\n", i, ST->net, ST->nwt);

         /*** Insert the scan values into the data file structure. ***/

         for (i=0; i<nval; i++)
            {
            jj = (int) scandx[i] % 360;
            ST->th[jj] = (short) (scan[i] * calfac);
            /*
	     * printf ("sg4> i: %3d  jj: %3d  th: %6.2f  ST->th: %3d\n",
	     *        i, jj, scandx[i], ST->th[jj]);
	     */
            }

         /*** Write the structure record to the file. ***/
			
         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error w: %s\n",namimg);
            break;
            }
         if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"write error: %s\n",namimg);
            break;
            }

         /*** Relinquish storage for image array. ***/

         free(image);
         break;
         }   /*** End of north quadrant. ***/

/******* East Quadrant *******/

      case 2:
         {
         fprintf (stdout,"east file: %s\n",namimg);
         if (radius > 2.5 && radius < 3.2)
              thmin =  40. + rolladj;
         else thmin =  45. + rolladj;
         thmax = 140. + rolladj;
         if (thmin < 0.)  thmin += 360;
         if (thmax < 0.)  thmax += 360;
         if (thmin >= 360.)  thmin -= 360;
         if (thmax >  360.)  thmax -= 360;
         thinc = 1.;

         /*** Find correct record in synoptic file. ***/
         
         ioff = (int) ((mapdoy - begdoy) * 2);
         offset = (long) (ioff * sizST);
         fprintf (stdout, "ioff: %d   offset:%d\n", ioff, offset);

         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error r: %s\n",namimg);
            break;
            }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"read error: %s\n",namimg);
            break;
            }

         /* If correct bin is filled, check adjacent bins
          * (lower & higher) in time.
          */

         if (ST->et < 1000.)
            {

            /* Check lower bin.
             * Move current bin to lower one if time is  smaller than current
             * image, and put current image scan in current bin.
             */

            if (fseek(SYN,offset-sizST,0) == -1)
               {
               fprintf(stderr,"seek error r: %s\n",namimg);
               break;
               }
            if (fread ((char *) ST0, sizST, 1, SYN) == 0)
               {
               fprintf(stderr,"read error: %s\n",namimg);
               break;
               }
            if (ST0->et > 1000.)
               {
               if (fseek(SYN,offset-sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (ST->et < mapdoy)
                  {
                  fprintf (stdout, "***Lower bin empty; filled with current bin.\n");
                  if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                     {
                     fprintf(stderr,"write error: %s\n",namimg);
                     break;
                     }
                  }
               else	/* Put current image scan into lower bin. */
                  {
                  fprintf (stdout, "***Lower bin empty; filled with current image.\n");
                  offset -= sizST;
                  }
               cpST (ST0, ST);
               }

            /*** Check higher bin if lower bin is already filled. ***/

            if (ST0->et < 1000.)
               {
               if (fseek(SYN,offset+sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (fread ((char *) ST0, sizST, 1, SYN) == 0)
                  {
                  fprintf(stderr,"read error: %s\n",namimg);
                  break;
                  }
               if (ST0->et > 1000.)
                  {
                  if (fseek(SYN,offset+sizST,0) == -1)
                     {
                     fprintf(stderr,"seek error r: %s\n",namimg);
                     break;
                     }
                  if (ST->et < mapdoy)
                     {
                     fprintf (stdout, "***High bin empty; fill with new image.\n");
                     offset += sizST;
                     }
                  else	/* Put current image scan into higher bin. */
                     {
                     fprintf (stdout, "***High bin empty; fill with current bin.\n");
                     if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                        {
                        fprintf(stderr,"write error: %s\n",namimg);
                        break;
                        }
                     }
                  cpST (ST0, ST);
                  }
               else
                  {
                  fprintf (stdout, "!!!!!!! Both low & high bins also filled.\n");
                  }
               }
            }

         if ((ST->et < 1000.) && (iovw == 0))

            {
            fprintf(stdout,"previous data: no overwrite\n");
            break;
            }

         /*** Peform theta scan. ***/

         ST->et = mapdoy;
         ST->ec = eastcr;
         ptscan (namimg,image,thmin,thmax,thinc,radius,scan,scandx,&nval);

         i = (int) (offset / sizST);
         fprintf (stdout, "ioff:%d   et:%7.2f   ec:%8.2f\n", i, ST->et, ST->ec);

         /*** Insert the scan values into the data file structure. ***/

         for (i=0; i<nval; i++)
            {
            jj = (int) scandx[i] % 360;
            ST->th[jj] = (short) (scan[i] * calfac);
            /*
	     * printf ("sg4> i: %3d  jj: %3d  th: %6.2f  ST->th: %3d\n",
	     *        i, jj, scandx[i], ST->th[jj]);
	     */
            }

         /*** Write the structure record to the file. ***/

         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error w: %s\n",namimg);
            break;
            }
         if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"write error: %s\n",namimg);
            break;
            }

         /* Relinquish storage for image array. */

         free(image);
         break;
         }   /*** End of east quadrant. ***/

/******* South Quadrant -- two parts: SE & SW. *******/

      case 3:
         {
         fprintf (stdout,"south file: %s\n",namimg);
         thmin = 140. + rolladj;
         thmax = 225. + rolladj;
         if (thmin < 0.)  thmin += 360;
         if (thmax < 0.)  thmax += 360;
         if (thmin >= 360.)  thmin -= 360;
         if (thmax >  360.)  thmax -= 360;
         thinc = 1.;

         /*** Find correct record in synoptic file. ***/
         
         ioff = (int) ((mapdoy - begdoy) * 2);
         offset = (long) (ioff * sizST);
         fprintf (stdout, "ioff: %d   offset:%d\n", ioff, offset);

         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error r: %s\n",namimg);
            break;
            }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"read error: %s\n",namimg);
            break;
            }

         /* If correct bin is filled, check adjacent bins
          * (lower & higher) in time.
          */

         if (ST->set < 1000.)
            {

            /* Check lower bin.
             * Move current bin to lower one if time is  smaller than current
             * image, and put current image scan in current bin.
             */

            if (fseek(SYN,offset-sizST,0) == -1)
               {
               fprintf(stderr,"seek error r: %s\n",namimg);
               break;
               }
            if (fread ((char *) ST0, sizST, 1, SYN) == 0)
               {
               fprintf(stderr,"read error: %s\n",namimg);
               break;
               }
            if (ST0->set > 1000.)	/* Lower bin is empty. */
               {
               if (fseek(SYN,offset-sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (ST->set < mapdoy)
                  {
                  fprintf (stdout, "***Lower bin empty; fill with current bin.\n");
                  if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                     {
                     fprintf(stderr,"write error: %s\n",namimg);
                     break;
                     }
                  }
               else	/*** Put current image scan into lower bin. ***/
                  {
                  fprintf (stdout, "***Lower bin empty; fill with new image.\n");
                  offset -= sizST;
                  }
               cpST (ST0, ST);
               }

            /*** Check higher bin if lower bin is already filled. ***/

            if (ST0->set < 1000.)
               {
               if (fseek(SYN,offset+sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (fread ((char *) ST0, sizST, 1, SYN) == 0)
                  {
                  fprintf(stderr,"read error: %s\n",namimg);
                  break;
                  }
               if (ST0->set > 1000.)	/* High bin is empty. */
                  {
                  if (fseek(SYN,offset+sizST,0) == -1)
                     {
                     fprintf(stderr,"seek error r: %s\n",namimg);
                     break;
                     }
                  if (ST->set < mapdoy)
                     {
                     fprintf (stdout, "***High bin empty; fill with new image.\n");
                     offset += sizST;
                     }
                  else	/*** Put current image scan into higher bin. ***/
                     {
                     fprintf (stdout, "***High bin empty; fill with current bin.\n");
                     if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                        {
                        fprintf(stderr,"write error: %s\n",namimg);
                        break;
                        }
                     }
                  cpST (ST0, ST);
                  }
               else
                  {
                  fprintf (stdout, "!!!!!!! Both low & high bins are filled.\n");
                  }
               }
            }


         if ((ST->set < 1000.) && (iovw == 0))
            {
            fprintf(stdout,"previous data: no overwrite\n");
            break;
            }

         /*** Peform theta scan. ***/

         ST->set = mapdoy;
         ST->swt = mapdoy;
         ST->sec = eastcr;
         ST->swc = westcr;
         ptscan (namimg,image,thmin,thmax,thinc,radius,scan,scandx,&nval);

         i = (int) (offset / sizST);
         fprintf (stdout, "ioff:%d   set:%7.2f   swt:%7.2f\n", i, ST->set, ST->swt);

         /*** Insert the scan values into the data file structure. ***/

         for (i=0; i<nval; i++)
             {
             jj = (int) scandx[i] % 360;
             ST->th[jj] = (short) (scan[i] * calfac);
            /*
	     * printf ("sg4> i: %3d  jj: %3d  th: %6.2f  ST->th: %3d\n",
	     *        i, jj, scandx[i], ST->th[jj]);
	     */
             }

         /*** Write the structure record to the file. ***/
			
         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error w: %s\n",namimg);
            break;
            }
         if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"write error: %s\n",namimg);
            break;
            }

         /*** Relinquish storage for image array. ***/

         free(image);
         break;
         }   /*** End of south quadrant. ***/


/******* West Quadrant *******/

      case 4:
         {
         fprintf (stdout,"west file: %s\n",namimg);
         thmin = 225. + rolladj;
         thmax = 315. + rolladj;
         if (thmin < 0.)  thmin += 360;
         if (thmax < 0.)  thmax += 360;
         if (thmin >= 360.)  thmin -= 360;
         if (thmax >  360.)  thmax -= 360;
         thinc = 1.;

         /*** Find correct record in synoptic file. ***/
         
         ioff = (int) ((mapdoy - begdoy) * 2);
         offset = (long) (ioff * sizST);
         fprintf (stdout, "ioff: %d   offset:%d\n", ioff, offset);

         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error r: %s\n",namimg);
            break;
            }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"read error: %s\n",namimg);
            break;
            }

         /* If correct bin is filled, check adjacent bins
          * (lower & higher) in time.
          */

         if (ST->wt < 1000.)
            {

            /* Check lower bin.
             * Move current bin to lower one if time is  smaller than current
             * image, and put current image scan in current bin.
             */

            if (fseek(SYN,offset-sizST,0) == -1)
               {
               fprintf(stderr,"seek error r: %s\n",namimg);
               break;
               }
            if (fread ((char *) ST0, sizST, 1, SYN) == 0)
               {
               fprintf(stderr,"read error: %s\n",namimg);
               break;
               }
            if (ST0->wt > 1000.)
               {
               if (fseek(SYN,offset-sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (ST->wt < mapdoy)
                  {
                  fprintf (stdout, "***Lower bin empty; filled with current bin.\n");
                  if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                     {
                     fprintf(stderr,"write error: %s\n",namimg);
                     break;
                     }
                  }
               else   /*** Put current image scan into lower bin. ***/
                  {
                  fprintf (stdout, "***Lower bin empty; filled with current image.\n");
                  offset -= sizST;
                  }
               cpST (ST0, ST);
               }

            /*** Check higher bin if lower bin is already filled. ***/

            if (ST0->wt < 1000.)
               {
               if (fseek(SYN,offset+sizST,0) == -1)
                  {
                  fprintf(stderr,"seek error r: %s\n",namimg);
                  break;
                  }
               if (fread ((char *) ST0, sizST, 1, SYN) == 0)
                  {
                  fprintf(stderr,"read error: %s\n",namimg);
                  break;
                  }
               if (ST0->wt > 1000.)
                  {
                  if (fseek(SYN,offset+sizST,0) == -1)
                     {
                     fprintf(stderr,"seek error r: %s\n",namimg);
                     break;
                     }
                  if (ST->wt < mapdoy)
                     {
                     fprintf (stdout, "***High bin empty; fill with new image.\n");
                     offset += sizST;
                     }
                  else   /*** Put current image scan into higher bin. ***/
                     {
                     fprintf (stdout, "***High bin empty; fill with current bin.\n");
                     if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
                        {
                        fprintf(stderr,"write error: %s\n",namimg);
                        break;
                        }
                     }
                  cpST (ST0, ST);
                  }
               else
                  {
                  fprintf (stdout, "!!!!!!! Both low & high bins also filled.\n");
                  }
               }
            }


         if ((ST->wt < 1000.) && (iovw == 0))
	    {
            fprintf(stdout,"previous data: no overwrite\n");
            break;
            }

         /*** Perform theta scan. ***/

         ST->wt = mapdoy;
         ST->wc = westcr;
         ptscan (namimg,image,thmin,thmax,thinc,radius,scan,scandx,&nval);

         i = (int) (offset / sizST);
         fprintf (stdout, "ioff:%d   wt:%7.2f   wc:%8.2f\n", i, ST->wt, ST->wc);

         /*** Insert the scan values into the data file structure. ***/

         for (i=0; i<nval; i++)
            {
            jj = (int) scandx[i] % 360;
            ST->th[jj] = (short) (scan[i] * calfac);
            /*
	     * printf ("sg4> i: %3d  jj: %3d  th: %6.2f  ST->th: %3d\n",
	     *        i, jj, scandx[i], ST->th[jj]);
	     */
            }

         /*** Write the structure record to the file. ***/
			
         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error w: %s\n",namimg);
            break;
            }
         if (fwrite ((char *) ST, sizST, 1, SYN) == 0)
            {
            fprintf(stderr,"write error: %s\n",namimg);
            break;
            }

         /*** Relinquish storage for image array. ***/

         free(image);
         break;
         }   /*** End of west quadrant. ***/

      default:
         {
         fprintf (stderr, "Invalid sector: %s\n", &smname[H.ism][0]);
         free (image);
         break;
         }
      
      }   /*** End of quadrant choices. ***/


      }   /******* End of image loop. *******/

   /* Close file. */

   fflush(SYN);
   fclose(SYN);
}
