;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Document name: solargeom.pro
; Created by:    Randy Meisner, HAO/NCAR, Boulder, CO, February 20, 1998
;
; Last Modified: Tue Mar 24 15:42:40 1998 by meisner (Randy Meisner) on hoth
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
PRO solargeom, im, minint, thresh, xc, yc, rx, ry, tv=tv, help=help
;+
;
; PROJECT:
;       SunRISE
;
; NAME:
;       solargeom     (procedure)
;
; PURPOSE:
;       To determine the solar geometry of a full disk image, using the zero
;       point crossings of the Laplacian of the image and least-squares
;       fitting of the points to determine the location of the limb.
;
; CALLING SEQUENCE:
;
;       solargeom, im, minint, thresh, xc, yc, rx, ry [,tv=tv, help=help]
;
; INPUTS:
;       im - full disk solar image.
;   minint - the minimum intensity value at/near the solar limb.
;   thresh - minimum amplitude threshold to consider in Laplacian image to
;            denote the location of an edge.
;
; OPTIONAL INPUTS: 
;       None.
;
; OUTPUTS:
;       xc - image x coordinate of the center of the solar disk.
;       yc - image y coordinate of the center of the solar disk.
;       rx - solar disk radius in the x dimension.
;       ry - solar disk radius in the y dimension.
;
; OPTIONAL OUTPUTS:
;       None.
;
; KEYWORD PARAMETERS: 
;       /help.  Will call doc_library and list header, and return
;         /tv.  Will display the image with the fitted points.
;
; CALLS:
;       initgeom.pro, linchk.pro, fitellipse.pro, fitcircle.pro
;
; COMMON BLOCKS:
;       None.
;
; RESTRICTIONS: 
;       None.
;
; SIDE EFFECTS:
;       None.
;
; CATEGORY:
;       Part of the SunRISE/PSPT image processing software.
;
; PREVIOUS HISTORY:
;       Written February 20, 1998, by Randy Meisner, HAO/NCAR, Boulder, CO
;
; MODIFICATION HISTORY:
;       
;
; VERSION:
;       Version 1, February 20, 1998
;-
;
;ON_ERROR, 2

IF(N_ELEMENTS(help) GT 0) THEN BEGIN
  doc_library,'solargeom'
  RETURN
ENDIF

IF(N_PARAMS() NE 7) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  solargeom, im, minint, thresh, xc, yc, rx, ry ' + $
     '[,tv=tv, help=help]'
  PRINT, ''
  RETURN
ENDIF

; Get image size information.

sz = size(im)

nx = sz(1) & ny = sz(2)

;minint = find_minint(im)


; Get initial estimate of image geometry.

initgeom, im, minint, ixc, iyc, ir, /print


; Generate Laplacian image.

kern = [[-1., -1., -1.], [-1., 8., -1.], [-1., -1., -1.]]
 
lap = convol(im, kern)


; Set edges of Laplacian image to zero - remove edge effects.
 
dx = fix(nx/100.) & dy = fix(ny/100.)
 
lap(*, 0:dy) = 0.0           ; bottom
 
lap(0:dx, *) = 0.0           ; left
 
lap(nx-dx:nx-1, *) = 0.0     ; right
 
lap(*, ny-dy:ny-1) = 0.0     ; top


; Initialize arrays to store limb points found.
 
xpts = [-999.] & ypts = [-999.]


; Setup progress reporting.

prog = ' '

print, 'Working:  ', format = '(a, $)'


; Loop over and extract each column and row in the image and check for solar
; limb points.

FOR i = 0, nx-1 DO BEGIN
   
   IF(i mod 25 eq 0) THEN print, ' .', format = '(a, $)'

   line = lap(*, i)             ; extract a row
   
                                ; check row for limb points
   
   linchk, smooth(line, 3), i, ixc, iyc, ir, thresh, edge, /row
   
   
                                ; if limb points found, then store coordinates.
   
   IF(edge(0) NE -1) THEN BEGIN
      
      nedge = n_elements(edge)
      
                                ; if more than 2 qualifying limb points are
                                ; found, then use the first and last points.
                                ; If only 2 points are found, use them - same
                                ; if only 1 point is found.
      
      CASE nedge OF
         
         2 : BEGIN
            
            xpts = [xpts, edge(0), edge(1)]
            
            ypts = [ypts, i, i]
            
         END
         
         1 : BEGIN
            
            xpts = [xpts, edge(0)]
            
            ypts = [ypts, i]
            
         END
         
         ELSE : BEGIN
            
            xpts = [xpts, edge(0), edge(nedge-1)]
            
            ypts = [ypts, i, i]
            
         END
         
      ENDCASE
      
   ENDIF
                                ; columns
   
                                ; Process the columns in the same way as the
                                ; rows.
   
   line = lap(i, *)
   
   linchk, smooth(line, 3), i, ixc, iyc, ir, thresh, edge, /col
   
   IF(edge(0) NE -1) THEN BEGIN
      
      nedge = n_elements(edge)
      
      CASE nedge OF
         
         2 : BEGIN
            
            xpts = [xpts, i, i]
            
            ypts = [ypts, edge(0), edge(1)]
            
         END
         
         1 : BEGIN
            
            xpts = [xpts, i]
            
            ypts = [ypts, edge(0)]
            
         END
         
         ELSE : BEGIN
            
           xpts = [xpts, i, i]
            
            ypts = [ypts, edge(0), edge(nedge-1)]
            
         END
         
      ENDCASE
      
   ENDIF
   
ENDFOR

; Transpose limb points for input into fitellipse.pro.

xpts = transpose(xpts(where(xpts NE -999.0)))
ypts = transpose(ypts(where(ypts NE -999.0)))


; Least-squares fit the limb points to an ellipse.

fitellipse, xpts, ypts, coeff, xc, yc, rx, ry, phi, err = err


; For comparison, least-squares fit the limb points to a circle.

fitcircle, xpts, ypts, cx, cy, cr


; Print the results to the screen.
 
print, ''
print, 'XC', 'YC', 'RX', 'RY', 'PHI', 'Ellipse Fit:', xc, yc, rx, $
   ry, phi, format = '(14x, a, 6x, a, 7x, a, 6x, a, 6x, a, /, a, 1x,' + $
   'f7.2, 1x, f7.2, 1x, f7.2, 1x, f7.2, 1x, f7.2)'
 
print, ' Circle Fit:', cx, cy, cr, $
   format = '(/, a, 1x, f7.2, 1x, f7.2, 1x, f7.2)'
 
print, ''
print, n_elements(xpts), ' Points fit.'
print, ''


; Display the limb/edge points found, if requested.
 
IF(keyword_set(tv)) THEN BEGIN
  
   set_plot, 'Z'
   device, set_resolution = [nx, ny], set_colors = 256
   tv, bytscl(im)
   FOR i = 0, n_elements(xpts)-1 DO plots,[xpts(i),xpts(i)], $
      [ypts(i),ypts(i)], /dev, color = 255, psym = 1
   a = tvrd()
   device,/close
   set_plot, 'X'
   a = rebin(a, 512, 512)
   tvscl, a
  
ENDIF

RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'solargeom.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
