/* sptofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert a binary Spartan 201-4 image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   24 Septbember 1996
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2014lib.h"

#define MAXLIN 180		/* Maximum # lines allocated for FITS header. */
char fitbuf   [MAXLIN][80] ;	/* FITS header buffer. */
char outbuf   [84] ;

sptofits (hkbuf, imbuf, namfts, namext, hduflag, pmode)

   unsigned char hkbuf [NBYHK] ;	/* Housekeeping array. */

   unsigned char imbuf [NY][NX] ;	/* Image array. */

   char namfts [] ;			/* Name of output FITS file. */

   int pmode ;				/* Print mode. */


{
   extern char *malloc () ;		/* Function to allocate memory. */
   char *padbuf ;			/* Pointer to padding buffer. */
   unsigned char *cibuf ;		/* unsigned char image pointers. */

   unsigned char cmin, cmax ;		/* Data min/max values. */
   unsigned char cval ;			/* Data value. */


   static char kazimuth  [12] = "AZIMUTH " ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kccd_temp1[12] = "CCDTEMP1" ;
   static char kccd_temp2[12] = "CCDTEMP2" ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcode_obs [12] = "CODE-OBS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kocc_disc [12] = "OCC-DISC" ;
   static char kdisp_dir [12] = "DISP-DIR" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kspver    [12] = "SP201VER " ;
   static char kelev     [12] = "ELEV    " ;
   static char kend      [12] = "END     " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kfold_mir [12] = "FOLD_MIR" ;
   static char kgain_det [12] = "GAIN-DET" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char kimage_id [12] = "IMAGE_ID" ;
   static char khalfwave [12] = "HALFWAVE" ;
   static char kheat_mir [12] = "HEAT_MIR" ;
   static char khour_ang [12] = "HOUR-ANG" ;
   static char kinstrume [12] = "INSTRUME" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char knorthang [12] = "NORTHANG" ;
   static char kobject   [12] = "OBJECT  " ;
   static char koptdev   [12] = "OPTDEV" ;
   static char koptpos   [12] = "OPTPOS" ;
   static char korbit    [12] = "ORBIT   " ;
   static char korigin   [12] = "ORIGIN  " ;
   static char knimsum   [12] = "NIMSUM  " ;
   static char kpitchave [12] = "PITCHAVE" ;
   static char kpitchmin [12] = "PITCHMIN" ;
   static char kpitchmax [12] = "PITCHMAX" ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char kpol      [12] = "POL     " ;
   static char krsun     [12] = "RSUN    " ;
   static char ksolar_b0 [12] = "SOLAR_B0" ;
   static char ksolar_l0 [12] = "SOLAR_L0" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ksolar_ra [12] = "SOLAR_RA" ;
   static char ksolardec [12] = "SOLARDEC" ;
   static char kstimlamp [12] = "STIMLAMP" ;
   static char ktelescop [12] = "TELESCOP" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char ktype_dp  [12] = "TYPE-DP " ;
   static char kversion  [12] = "VERSION " ;
   static char kwavelen  [12] = "WAVELEN " ;
   static char kwavewid  [12] = "WAVEWID " ;
   static char kyaw_ave  [12] = "YAW_AVE " ;
   static char kyaw_min  [12] = "YAW_MIN " ;
   static char kyaw_max  [12] = "YAW_MAX " ;

   static char atype1   [24] ;			/* Coord type axis 1. */
   static char atype2   [24] ;			/* Coord type axis 2. */
   static char bunit    [32] ;			/* Brightness unit. */
   static char ctime    [12] = "HH:MM:SS" ;	/* File creation time. */
   static char ctype1   [24] ;			/* Coord type axis 1. */
   static char ctype2   [24] ;			/* Coord type axis 2. */
   static char ctypenam [24] = "GEOCENTRIC";	/* Coordinate system name. */
   static char cunit1   [24] ;			/* Coord units axis 1. */
   static char cunit2   [24] ;			/* Coord units axis 2. */
   static char datcom   [72] ;			/* Data min/max comment. */
   static char cdate    [12] = "DD/MM/YY" ;	/* File creation date. */
   static char code_obs [12] = "???     " ;	/* Observation code. */
   static char date_cal [12] = "DD/MM/YY" ;	/* Calibration date. */
   static char date_obs [12] = "DD/MM/YY" ;	/* Observation date. */
   static char eq       [4]  = "= " ;
   static char instrume [12] = "CCD" ;		/* Instrument name. */
   static char instcom  [28] = "RCA SID501 CCD" ;
   static char object   [12] = "SUN" ;		/* Object. */
   static char origin   [12] = "HAO" ;		/* Origin. */
   static char origincom [28] = "HIGH ALTITUDE OBSERVATORY" ;
   static char pixcom   [48] ;			/* Data type comment. */
   static char refcom   [24] ;			/* Coord ref comment. */
   static char disp_dir [ 4] ;			/* Scan direction:R/L,U/D,O/I.*/
   static char telescop [12] = "SPARTAN 201" ;	/* Telescope name. */
   static char time_obs [12] = "HH:MM:SS" ;	/* Observation time. */
   static char tscopcom [24] = "SPARTAN 201 CORONAGRAPH";
   static char type_obs [24] ;			/* Observation type. */
   static char type_dp  [24] = "IMAGE FORMAT";	/* Data Processing type. */
   static char version  [24] ;

   int nimsum ;

   static char namdev [5][20] =
      {"FOLDING MIRROR",  "HEAT DUMP MIRROR", "POLAROID WHEEL",
       "HALF WAVE PLATE", "STIMULATOR LAMP"} ;

   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */

   short *sibuf ;			/* Short image pointers. */
   short smin, smax ;			/* Data min/max values. */
   short sval = 0 ;			/* Data value. */

   int bitpix ;				/* # bits per pixel. */
   int doy_obs ;			/* Observation day-of-year. */
   int gcount = 0 ;			/* gcount. */
   int iyear ;				/* year - 1900. */
   int i, ix, iy ;			/* Indices. */
   int iynum ;				/* # lines in FITS header. */
   int nhdu ;				/* # HDU blocks to write. */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* Integer value. */
   int kx, ky ;				/* Image indices. */
   int ndim   = 0 ;			/* # dimensions for image (2=XY). */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyfits = 2880 ;			/* # bytes per FITS logical record. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int pcount  = 0 ;			/* pcount. */
   int nrec ;				/* # fits records for image. */
   int nelwr   = 0 ;
   int rcode ;
   int scode ;
   int wcode ;
   int year_obs ;			/* Observation year. */

   float *fibuf ;			/* Float image pointers. */

   float northdeg =  90.0 ;		/* Solar north w.r.t. +X axis. */
   float bscale   =   1.0 ;		/* Brightness intensity scale factor. */
   float bzero    =   0.0 ;		/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float crpix1, crpix2 ;		/* Reference pixel location. */
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crradius = 42.7 ;		/* Solar radius [pixels] = 960.75/22.5*/
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin, dmax ;			/* Data min/max values. */
   float swver = 1.00 ;			/* Spartan FITS s/w version number. */
   float fmin, fmax ;			/* Data min/max values. */
   float fval ;				/* Data value. */
   float xcenter, ycenter, diam ;	/* X/Y centers + disk diameter. */
   float vmin, vmax ;			/* Data min/max values. */
   float wmin =    0.0 ;		/* Display min value. */
   float wmax = 4000.0 ;		/* Display max value. */
   float foldfac = 1.0 ;		/* Fold Mirror display factor. */
   float heatfac = 1.0 ;		/* Heat Mirror display factor. */
   float polfac  = 1.0 ;		/* Polaroid    display factor. */
   float wavefac = 1.0 ;		/* Half Wave   display factor. */
   FILE *FF ;				/* FITS file ID. */

   if (pmode)
      {
      fprintf (stdout, "sptofits> namfts: %s  namext: %s   hduflag: %d\n",
	       namfts, namext, hduflag) ;

      fprintf (stdout, "\n------- sptofits> %s -------\n\n",
	       namfts) ;
      }

   nx = SPII->xdim ;			/* X dimension. */
   ny = SPII->ydim ;			/* Y dimension. */
   npximg = nx * ny ;			/* # pixels in image. */
   nbyimg = npximg * SPII->nbypx ;	/* # bytes in image. */
   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			nx, ny, SPII->nbypx, npximg, nbyimg) ;

   /*** Find min/max data values. ***/
   /*** Find disk center for short integer images. ***/

   switch (SPII->nbypx)
      {
      case 1:
	 cibuf = (unsigned char *) imbuf ;
	 cmin  = 255 ;
	 cmax  =   0 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       cval = *(cibuf + ix + iy*nx) ;
               if (cval < cmin)  cmin = cval ;
               if (cval > cmax)  cmax = cval ;
	       }
	    }
         SPII->dmin = (int) cmin ;
         SPII->dmax = (int) cmax ;
	 vmin = (float) cmin ;
	 vmax = (float) cmax ;
         break ;

      case 2:
	 sibuf = (short *) imbuf ;
	 smin  =  32767 ;
	 smax  = -32767 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       sval = *(sibuf + ix + iy*nx) ;
               if (sval < smin)  smin = sval ;
               if (sval > smax)  smax = sval ;
	       }
	    }
         SPII->dmin = (int) smin ;
         SPII->dmax = (int) smax ;
	 vmin = (float) smin ;
	 vmax = (float) smax ;
         break ;

      case 4:
	 fibuf = (float *) imbuf ;
	 fmin  =  1.0e+30 ;
	 fmax  = -1.0e+30 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    for (ix = 0 ; ix < nx ; ix++)
	       {
               fval = *(fibuf + ix + iy*nx) ;
               if (fval < fmin)  fmin = fval ;
               if (fval > fmax)  fmax = fval ;
	       }
	    }
         SPII->dmin = (int) fmin ;
         SPII->dmax = (int) fmax ;
	 vmin = fmin ;
	 vmax = fmax ;
         break ;

      default:
	 fprintf (stderr, "Invalid Spartan image . . . nbypx: %d\n", SPII->nbypx) ;
	 exit (-1) ;
      }

   iyear = SPII->year ;
   if (SPII->year > 1900)  iyear = SPII->year - 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, SPII->month, SPII->day) ;

   sprintf (date_obs, "%02d/%02d/%02d", SPII->day,  SPII->month, iyear) ;
   sprintf (time_obs, "%02d:%02d:%02d", SPII->hour, SPII->minute, SPII->second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   sprintf (cdate, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (ctime, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   iyear = SPII->cal_year ;
   if (SPII->cal_year > 1900)  iyear = SPII->cal_year - 1900 ;
   sprintf (date_cal, "%02d/%02d/%02d", SPII->cal_day, SPII->cal_month, iyear) ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (SPII->xdim > 0)
      ndim++ ;
   if (SPII->ydim > 1)
      ndim++ ;

   /*
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Set parameters.
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;
   strcpy  (origin,   "HAO") ;
   sprintf (datcom,   "%s", instrume) ;
   strcpy  (atype1,   "X [EAST->WEST ]") ;
   strcpy  (atype2,   "Y [SOUTH->NORTH]") ;
   strcpy  (ctypenam, "HELIOCENTRIC") ;
   strcpy  (cunit1,   "ARCSEC") ;
   strcpy  (cunit2,   "ARCSEC") ;
   strcpy  (refcom,   "SUN CENTER") ;
   strcpy  (disp_dir, "RU") ;		/* Display direction: right, up. */

   xcenter = (float) (243) ;			/* Sun center (X pixels). */
   ycenter = (float) (120) ;			/* Sun center (Y pixels). */
   crval1 = 0.0 ;				/* Sun center (X arcsec). */
   crval2 = 0.0 ;				/* Sun center (Y arcsec). */
   cdelt1 = 22.5 ;				/* arcsec/pixel. */
   cdelt2 = 22.5 ;				/* arcsec/pixel. */
   crradius = SPII->rsun / cdelt1 ;		/* Solar radius (pixels). */

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Spartan data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   /*** Set object.
    *** Set coordinate type(s).
    *** Set bitpix.
    *** Set display scaling.
    ***/

   /*** Default designations. ***/

   strcpy (object,   "UNKNOWN") ;   
   strcpy (type_obs, "UNKNOWN") ;
/*   strcpy (type_dp , "UNKNOWN") ; */
   strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
   strcpy (ctype1,   "X") ;
   strcpy (ctype2,   "Y") ;
   strcpy (bunit,    "CCD") ;
   bitpix =  16 ;			/* short integer image. */
   wmin = 0.0 ;
   wmax = 0.0 ;

   strcpy (object, "SUN") ;

   if (strncmp (&SPII->optpos [0], "CAL", 3) == 0)
      strcpy (type_obs, "CAL") ;
   else
      strcpy (type_obs, "WHITE-LIGHT") ;

   nimsum = SPII->nimsum ;

   /*** Determine display magnitude range. */

   if (strncmp (&SPII->optpos [0], "COR", 3) == 0)   foldfac  = 1.0 ;
   else foldfac = 0.5 ;

   if (strncmp (&SPII->optpos [1], "COR", 3) == 0)   heatfac  = 1.0 ;
   else heatfac = 0.5 ;

   if (strncmp (&SPII->optpos [2], "CLEAR", 5) == 0) polfac   = 1.0 ;
   else polfac  = 0.333 ;

/*
 *   if (strncmp (&SPII->optpos [3], "0 DEG", 5) == 0)  wavefac = 1.0 ;
 *   else wavefac = 2.0 ;
 */

   wmin =    0.0 ;
   wmax =    0.0 ;
/*   wmax = foldfac * heatfac * polfac * wavefac * 1500.0 * (float) nimsum ; */

   /*** Create FITS file. ***/

   if ((FF = fopen (namfts, "w")) == NULL)
      {
      fprintf (stderr, "Unable to create file: %s\n", namfts) ;
      exit (-2) ;
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- PRIMARY HDU ---\n") ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   sprintf (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: HAO/SPARTAN") ;
   else
      {
      sprintf (&fitbuf [iy][0],    "XTENSION= '%-8s'", namext) ;
      sprintf (&fitbuf [iy++][31],  "/ HAO/SPARTAN %s EXTENSION", type_obs) ;
      }

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF DIMENSIONS",
	                        knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
	                        knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / SIZE OF %s DIMENSION",
	                           knaxis2, eq, ny, atype2) ;
   else
      sprintf (&fitbuf [iy++][0],  "          ^                                                                    ^") ;

   /*** Extension information. ***/

   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;
      }
   else
      {
      sprintf (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      }

   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s %s", atype1, ctypenam) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [INDEX:PIXELS]",
			        kcrpix1  , eq, xcenter, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
			        kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			        kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s AXIS SOLAR NORTH ANGLE [DEG]",
			        kcrota1  , eq, SPII->nangle, atype1) ;

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s %s", atype2, ctypenam) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [INDEX:PIXELS]",
			           kcrpix2, eq, ycenter, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom, cunit2) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / %s AXIS SOLAR NORTH ANGLE [DEG]",
			        kcrota2  , eq, SPII->nangle, atype2) ;
      sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2  , eq, cdelt2, atype2, cunit2) ;
      }
   else
      {
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^                                                                    ^") ;
      }

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM NAME") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f", kcrradius, eq, crradius) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR RADIUS [PIXELS]") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR RADIUS [ARCSEC]",
			        krsun    , eq, SPII->rsun) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / OCCULTING DISC DIAMETER [SOLAR RADII]",
				   kocc_disc, eq, SPII->odisk) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / ORBIT NUMBER",
			        korbit, eq, SPII->orbit_num) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / IMAGE NUMBER",
			        kimage_id, eq, SPII->image_num) ;

   /*** Date & Time of observation. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
				doy_obs, year_obs) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH:MM:SS [UT]") ;

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ %s", origincom) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-9s'",     ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ %s", tscopcom) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kinstrume, eq, instrume) ;
   sprintf (&fitbuf [iy++][31], "/ %s", instcom) ;

   /*** Observation source & type. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ SOURCE OBJECT") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", ktype_dp,   eq, type_dp) ;
   sprintf (&fitbuf [iy++][31], "/ DATA PROCESSING TYPE") ;

   for (i = 0; i < 4; i++)
   {
   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptdev, i, eq,
				&SPII->optdev [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %s NAME", &namdev [i][0]) ;

   sprintf (&fitbuf [iy  ][0],  "%-6s%02d%2s'%-8s'", koptpos, i, eq,
				&SPII->optpos [i][0]) ;
   sprintf (&fitbuf [iy++][31], "/ %s POSITION", &namdev [i][0]) ;
   }

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f",
			        kwavelen,  eq, SPII->wavelen) ;
   sprintf (&fitbuf [iy++][31], "/ CENTRAL WAVE LENGTH OF OBSERVATION [NM]") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.2f",
			        kwavewid,  eq, SPII->wavewid) ;
   sprintf (&fitbuf [iy++][31], "/ BANDPASS OF FILTER [NM] FWHM") ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

/*
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
			        ksolar_p0, eq, SPII->pangle) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
			        ksolar_b0, eq, SPII->bangle) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE",
			        ksolar_l0, eq, SPII->langle) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR RIGHT ASCENSION [DEGREES]",
			        ksolar_ra, eq, SPII->ra) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR DECLINATION [DEGREES]",
			        ksolardec, eq, SPII->dec) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR HOUR ANGLE [DEGREES] (W.R.T.ZENITH E- W+)",
			        khour_ang, eq, SPII->hangle) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR ELEVATION [DEGREES]",
			        kelev    , eq, SPII->elev) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / SOLAR AZIMUTH [DEGREES]",
			        kazimuth , eq, SPII->azimuth) ;
   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
 */

/* -------------------------------------------------------------------------- */

   /*** Number of Images Summed. ***/

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20d / NUMBER OF IMAGES SUMMED",
			        knimsum  , eq, SPII->nimsum) ;
   /*** Exposure Duration. ***/

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / EXPOSURE DURATION [SECONDS]",
			      kexpdur  , eq, SPII->expdur) ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kbunit, eq, bunit) ;
   sprintf (&fitbuf [iy++][31], "/ PIXEL MAGNITUDE UNITS") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PHYSICAL = DATA * BSCALE + BZERO",
			        kbscale, eq, SPII->bscale) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s%20.3f", kbzero , eq, SPII->bzero ) ;
   sprintf (&fitbuf [iy++][31], "/ PHOTOMETRIC SCALING OFFSET [SEE BSCALE]") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MINIMUM DATA VALUE [%s]",
			        kdatamin , eq, vmin, datcom) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MAXIMUM DATA VALUE [%s]",
			        kdatamax , eq, vmax, datcom) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MIN VALUE FOR DISPLAY [UNSCALED]",
			        kdispmin , eq, wmin) ;
   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.2f / MAX VALUE FOR DISPLAY [UNSCALED]",
			        kdispmax , eq, wmax) ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kdisp_dir,  eq, disp_dir) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION (AXIS 1 2 3): R/L  U/D  O/I") ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / CCD TEMPERATURE @ FIRST IMAGE [DEG.CELSIUS]",
			        kccd_temp1, eq, SPII->ccd_temp_beg) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / CCD TEMPERATURE @ LAST  IMAGE [DEG.CELSIUS]",
			        kccd_temp2, eq, SPII->ccd_temp_end) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PES YAW AVERAGE VOLTAGE",
			        kyaw_ave, eq, SPII->pes_yaw_ave) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PES YAW MINIMUM VOLTAGE",
			        kyaw_min, eq, SPII->pes_yaw_min) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PES YAW MAXIMUM VOLTAGE",
			        kyaw_max, eq, SPII->pes_yaw_max) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PES PITCH AVERAGE VOLTAGE",
			        kpitchave, eq, SPII->pes_pitch_ave) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PES PITCH MINIMUM VOLTAGE",
			        kpitchmin, eq, SPII->pes_pitch_min) ;

   sprintf (&fitbuf [iy++][0],  "%-8s%2s%20.3f / PES PITCH MAXIMUM VOLTAGE",
			        kpitchmax, eq, SPII->pes_pitch_max) ;

   sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;
   sprintf (version, "Spartan v%4.2f", swver) ;
   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kversion, eq, version) ;
   sprintf (&fitbuf [iy++][31], "/ Spartan FITS S/W VERSION") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'", kdate,     eq, cdate) ;
   sprintf (&fitbuf [iy++][31], "/ FITS FILE GENERATION DATE: DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",     kdate_cal, eq, date_cal) ;
   sprintf (&fitbuf [iy++][31], "/ Spartan CALIBRATION DATE: DD/MM/YY [UT]") ;

   sprintf (&fitbuf [iy++][0],  "                                                                                ") ;

   sprintf (&fitbuf [iy++][0],  "COMMENT   THE SPARTAN-201 INSTRUMENT IS A WHITE LIGHT CORONAGRAPH,") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   DESIGNED & BUILT BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   IT OPERATED IN EARTH ORBIT AFTER RELEASE FROM NASA'S SPACE SHUTTLE") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   DURING THE FOLLOWING FLIGHTS:") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT      SP201-01 DISCOVERY STS-56  11-13 April 1993 ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT      SP201-02 DISCOVERY STS-64  13-15 September 1994") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT      SP201-03 ENDEAVOR  STS-69   8-10 September 1995") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT      SP201-04 COLUMBIA  STS-87  21    NOVEMBER 1997") ;
/*   sprintf (&fitbuf [iy++][0],  "                                                                                ") ; */
   sprintf (&fitbuf [iy++][0],  "COMMENT   THE DETECTOR IS AN RCA SID501 CCD WITH 320 X 256 30-MICRON PIXELS.") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   OPTICAL RESOLUTION IS 22.5 ARCSECONDS,  WITH A 6 SOLAR RADIUS FIELD") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   OF VIEW.  THE OCCULTING DISK, WHICH IS PLACED NEAR THE RIGHT EDGE OF") ;
   sprintf  (&fitbuf [iy++][0], "COMMENT   THE DETECTOR, HAS A RADIUS IS 1.25 SOLAR RADII.") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.") ;
/*   sprintf (&fitbuf [iy++][0],  "                                                                                ") ; */
   sprintf (&fitbuf [iy++][0],  "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;
   fprintf (stdout, "sptofits> %s ... hdu nelwr: %d\n", namfts, nelwr) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (imbuf, SPII->nbypx, npximg, FF) ;
   fprintf (stdout, "sptofits> %s ... img nelwr: %d\n", namfts, nelwr) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FF) ;
   fprintf (stdout, "sptofits> %s ... pad nelwr: %d\n", namfts, nelwr) ;
      free (padbuf) ;
      }
   if (pmode)
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg) ;

   close (FF) ;
   return (0) ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
