/* metconv.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert Spartan MET (Mission Elapsed Time) from counts
 * to hours, minutes, seconds.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   1 October 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * INPUT:
 *    Image S/C time (met).
 * OUTPUT :
 *    kyear, kdoy, khour, kmkn, ksec: returned GMT values (integer).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

int
metconv (met, kyear,  kdoy,  khour,  kmin,  ksec)

int met;					/* Image MET. */
int *kyear, *kdoy, *khour, *kmin;		/* Image GMT. */
float *ksec ;

{
int i;
int nyear, ndoy, nhour, nmin, nsec;
int iday, idaymo, imonth;
int iyear, idoy, ihour, imin, isec;

static int ndaymo[12] = {31,28,31,30,31,30,31,31,30,31,30,31};	/* Non-leap. */
static int ldaymo[12] = {31,29,31,30,31,30,31,31,30,31,30,31};	/* Leap year.*/

double sec0;
double sec, ysec, dsec, hsec, xsec;

static double secyr = 31536000.0;	/* seconds/year. */
static double secday = 86400.0;		/* seconds/day. */
static double sechr = 3600.0;		/* seconds/hour. */
static double secmin = 60.0;		/* seconds/minute. */
static double seccen = 0.008;		/* MET clock interval = 8 msec. */
static double secoff = 0.0;		/* MET offset. */

int   iyear0 = 0 ;			/* Initial year. */
int   idoy0  = 0 ;			/* Initial day-of-year. */
int   ihour0 = 0 ;			/* Initial hours. */
int   imin0  = 0 ;			/* Initial minutess. */
float fsec0  = 0.0 ;			/* Initial seconds. */
int   sct0   = 0 ;			/* Initial MET. */

/*----------------------------------------------------------------------------*/

   sec0 = (double) fsec0;

   /* Compute GMT time from MET (Mission Elapsed Time in S/C counts).
    * centon: MET clock interval.
    * seccen: seconds/centon (=0.008)
    * secoff: offset in seconds
    * y(seconds) = m * x(centons) + b
    * where m = seccen and b = secoff.
    * secyr = seconds/year.
    */

   /*** Convert MET into seconds. ***/

   sec = (met * seccen) + secoff;

   /* Determine year (relative to 1900) and day-of-year.
    *    nyear  = sec / secyr
    *    ysec = sec - nyear*secyr
    *    iyear = iyear0 + nyear
    */

   nyear = 0;
   ysec = sec;
   iyear = iyear0;

   ndoy  = (int) (ysec / secday);
   idoy  = idoy0 + ndoy;

   /* Determine hours, minutes and seconds. */

   dsec  = (ysec - ndoy*secday);
   nhour = (int) (dsec / sechr);
   ihour = ihour0 + nhour;

   hsec  = (dsec - nhour*sechr);
   nmin  = (int) (hsec / secmin);
   imin  = imin0 + nmin;

   xsec  = (hsec - nmin*secmin);
   while (xsec >= 60.0) xsec -= 60.0 ;
   isec  = (int) (sec0 + xsec + 0.5);


   /* Correct for rollover in time units.
    *
    *   seconds:
    */
   while (isec > 59)
      {
      isec -= 60;
      imin +=  1;
      }

   /*    minutes: */

   while (imin > 59)
      {
      imin -=  60;
      ihour +=  1;
      }

   /*    hours: */

   while (ihour > 23)
      {
      ihour -= 24;
      idoy +=   1;
      }

   /*    Day-Of-Year:
    *   (check for leap year)
    */

   while (idoy > 365)
      {
      if ((iyear/4)*4 == iyear)
         {
         if (idoy <= 366)  break;
         iyear++;
         idoy -= 366;
         }
      else
         {
         iyear++;
         idoy -= 365;
         }
      }	


   /* Determine month and day-of-month. */

   /* Leap year case: */

   if ((iyear/4)*4 == iyear)
      {
      idaymo = 367;
      for (i=0; i<12; i++)
         {
         imonth = 11 - i;
         idaymo = idaymo - ldaymo[imonth];
         if(idoy >= idaymo) break;
         }
      }

   /* Non-leap year case: */

   else
      {
      idaymo = 366;
      for (i=0; i<12; i++)
         {
         imonth = 11 - i;
         idaymo = idaymo - ndaymo[imonth];
         if(idoy >= idaymo) break;
         }
      }

   iday = idoy - idaymo + 1;		/* Day of month. */

   *kyear = iyear;
   *kdoy  = idoy;
   *khour = ihour;
   *kmin  = imin;
   *ksec  = xsec;
   return (0);
}
