/* sphskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill HAO/SPARTAN housekeeping structure
 * using information in SPARTAN image header information.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    24 September 1996
 * 1 Oct 96: Add gyro_pitch_set for roll angle specification versus orbit_num.
 * 13 May 1998: Initial GMT is now stored in "sp2014lib.h".
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * 11 Nov 1998: Add NCP offsets to compute geocentric pole location.
 *              Subtract P-angle to determine heliocentric pole location.
 *  1 Dec 1998: Add P-angle & NCP angle to image structure.
 *  2 Dec 1998: Change orbit number for images with MET between
 *              01-12:06:16.06 and 01-13:40:17.94. (should be orbit 23, not 24).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"

sphskp (hkbuf, pmode)

unsigned char hkbuf [] ;		/* Spartan image housekeeping array. */
int pmode ;				/* Print mode: 0=silent, 1=verbose. */

{
   extern FILE *FP_ELOG ;
   extern int rel_pitch_prev ;
   extern int rel_pitch_new  ;
   extern int pitch_orb_prev ;
   extern int pitch_img_prev ;
   static int irel = 0 ;
   int iyear ;
   int idoy  ;
   int imonth ;
   int iday ;
   int ihour ;
   int iminute ;
   float isecond ;

   int metsec1, metsec2 ;
   int metsec ;
   int secday  = 86400 ;
   int sechour =  3600 ;
   int secmin =     60 ;

   /*** Data Block Housekeeping: ***/
   /* sync pattern: bytes  0 -  5 */

   int met ;			/* bytes  6 -  9     */
   int rel_pitch_ang ;		/* bytes 10 - 11     */
   int mech_pos1 ;		/* byte  12          */
   int fold_mir_pos ;		/* byte  12 bit  0   */
   int hdump_mir_pos ;		/* byte  12 bit  1   */
   int pw_pos ;			/* byte  12 bits 2-3 */
   int half_wave_pos ;		/* byte  12 bits 4-5 */
   int mech_pos2 ;		/* byte  13          */
   int num_img_sum ;		/* byte  13 bits 0-3 */
   int stim_lamp_stat ;		/* byte  13 bit  5   */
   int ccd_expdur ;		/* bytes 14 - 15     */
   int orbit_num ;		/* byte  16          */
   int image_num ;		/* byte  17          */
   int ccd_temp_beg ;		/* bytes 18 - 19     */
   int ccd_temp_end ;		/* bytes 20 - 21     */
   int pes_yaw_mean ;		/* bytes 22 - 23     */
   int pes_yaw_min ;		/* bytes 24 - 25     */
   int pes_yaw_max ;		/* bytes 26 - 27     */
   int pes_pitch_mean ;		/* bytes 28 - 29     */
   int pes_pitch_min ;		/* bytes 30 - 31     */
   int pes_pitch_max ;		/* bytes 32 - 33     */
   int spare1 ;			/* bytes 34 - 35     */
   int spare2 ;			/* byte  36          */
   int spare3 ;			/* byte  37          */
   int spare4 ;			/* byte  38          */
   int spare5 ;			/* byte  39          */
   int spare6 ;			/* byte  40          */
   int spare7 ;			/* byte  41          */
   int spare8 ;			/* byte  42          */
   int spare9 ;			/* byte  43          */
   int cur_ccd_line ;		/* bytes 44 - 45     */

   FILE *FP ;
   int doy ;
   int i ;
   int ipos ;
   float roll_ang ;
   float pitch_id, pitch_beg, pitch_end ;
   float ncp_ang, nhp_ang ;
   float wlc_ang = -90.0 ;	/* solar north = 90 deg CCW from image +X axis*/

   static char namdev [5][12] =
      {"FOLD-MIR","HEAT-MIR","POL-WHL","HALFWAVE","STIMLAMP"} ;

   static char nampos [5][4][12] =
      {
      "CAL",        "CORONA",     "?",          "?",

      "CORONA",     "CAL",        "?",          "?",

      "CLEAR",      "0 DEG",      "30 DEG",     "60 DEG",

      "90 DEG",     "60 DEG",     "30 DEG",     "0 DEG",

      "OFF",        "ON",         "?",          "?",
      } ;

   /*** NHP: Northern Heliospheric Pole.
    *** NHP = NCP + P-angle
    *** NHP = NCP + 24.5 degrees	DOY 305 1998 00:00:00 (1 November)
    *** NHP = NCP + 24.4 degrees	DOY 306 1998 00:00:00 (2 November)
    *** NHP = NCP + 24.2 degrees	DOY 307 1998 00:00:00 (3 November)
    ***/

   /*** Pitch angle set by ground command.  Specified for each orbit. ***/

   /*** NCP: Northern Celestial Pole.  Pitch offset in degrees. ***/
   /*** Pitch offsets are given for each orbit. ***/
   /*** NCP = pitch_orb [i] + rel_pitch_ang. ***/

   static float ncp_orb [28] =
      { 68.0,  68.0,  68.0,  73.8,  73.8,  90.2, 270.2, 290.8, 271.2, 272.6,
       273.8, 274.7,  23.2,  23.2,  23.2,  23.2,  23.2,  23.2,  23.2,  23.2,
        23.2,  23.2,  23.2,  23.2, 357.3, 357.3, 196.1,   0.0
      } ;

   static float pang_orb [28] =
      { 24.42, 24.41, 24.40, 24.38, 24.37, 24.36, 24.35, 24.34, 24.33, 24.31,
	24.30, 24.29, 24.28, 24.27, 24.26, 24.24, 24.23, 24.22, 24.21, 24.20,
	24.18, 24.16, 24.15, 24.14, 24.12, 24.11, 24.10, 24.09
      } ;

   static float rsun_orb [28] =
      { 967.000, 967.017, 967.034, 967.050, 967.067, 967.084, 967.101, 967.118,
	967.134, 967.151, 967.168, 967.185, 967.201, 967.218, 967.235, 967.251,
	967.268, 967.285, 967.302, 967.318, 967.335, 967.351, 967.368, 967.385,
	967.401, 967.418, 967.435, 967.451
      } ;

   /*** Decode Image Housekeeping. ***/
   /*** Fill Raw Spartan housekeeping structure. ***/

   for (i = 0; i < 6; i++)
      SPIH->sync [i] = hkbuf [i] ;
   SPIH->met = int4 (&hkbuf [6]) ;
   SPIH->rel_pitch_ang = int2 (&hkbuf [10]) ;

   SPIH->fold_mir_pos = hkbuf [12] & 001 ;
   SPIH->heat_mir_pos = (hkbuf [12] & 002) >> 1 ;
   SPIH->pw_pos = (hkbuf [12] & 014) >> 2 ;
   SPIH->half_wave_pos = (hkbuf [12] & 060) >> 4 ;
   SPIH->num_img_sum = (hkbuf [13] & 017) ;
   SPIH->stim_lamp_stat = (hkbuf [13] & 040) >> 5 ;

   SPIH->pos [0] = SPIH->fold_mir_pos ;
   SPIH->pos [1] = SPIH->heat_mir_pos ;
   SPIH->pos [2] = SPIH->pw_pos ;
   SPIH->pos [3] = SPIH->half_wave_pos ;
   SPIH->pos [4] = SPIH->stim_lamp_stat ;

   SPIH->ccd_expdur = int2 (&hkbuf [14]) ;
   SPIH->orbit_num = hkbuf [16] ;
   SPIH->image_num = hkbuf [17] ;
   SPIH->ccd_temp_beg = int2 (&hkbuf [18]) ;
   SPIH->ccd_temp_end = int2 (&hkbuf [20]) ;
   SPIH->pes_yaw_mean = int2 (&hkbuf [22]) ;
   SPIH->pes_yaw_min  = int2 (&hkbuf [24]) ;
   SPIH->pes_yaw_max  = int2 (&hkbuf [26]) ;
   SPIH->pes_pitch_mean = int2 (&hkbuf [28]) ;
   SPIH->pes_pitch_min  = int2 (&hkbuf [30]) ;
   SPIH->pes_pitch_max  = int2 (&hkbuf [32]) ;
   SPIH->cur_ccd_line = int2 (&hkbuf [44]) ;

   /*** SP201-5 ACS command sequencer error:
    *** images 24-1 through 24-51 (MET 01-12:06:16.06 -> 01-13:40:17.94)
    *** are incorrectly labeled.  The orbit should be 23, not 24 !
    ***/

   metsec1 = 1 * secday + 12 * sechour +  6 * secmin + 16 ;	/* Start time */
   metsec2 = 1 * secday + 13 * sechour + 40 * secmin + 18 ;	/* Stop  time */
   
   metsec = SPIH->met * 0.008 ;		/* Convert MET to seconds. */

   if (metsec >= metsec1 && metsec <= metsec2)
      SPIH->orbit_num -= 1 ;

   /*** Print header. ***/

   if (pmode)
      {
      fprintf (stdout, "SPIH->met           : %10d\n", SPIH->met           ) ;
      fprintf (stdout, "SPIH->rel_pitch_ang : %10d\n", SPIH->rel_pitch_ang ) ;
      fprintf (stdout, "SPIH->fold_mir_pos  : %10d\n", SPIH->fold_mir_pos  ) ;
      fprintf (stdout, "SPIH->heat_mir_pos  : %10d\n", SPIH->heat_mir_pos ) ;
      fprintf (stdout, "SPIH->pw_pos        : %10d\n", SPIH->pw_pos        ) ;
      fprintf (stdout, "SPIH->half_wave_pos : %10d\n", SPIH->half_wave_pos ) ;
      fprintf (stdout, "SPIH->num_img_sum   : %10d\n", SPIH->num_img_sum   ) ;
      fprintf (stdout, "SPIH->stim_lamp_stat: %10d\n", SPIH->stim_lamp_stat) ;
      fprintf (stdout, "SPIH->ccd_expdur    : %10d\n", SPIH->ccd_expdur    ) ;
      fprintf (stdout, "SPIH->orbit_num     : %10d\n", SPIH->orbit_num     ) ;
      fprintf (stdout, "SPIH->image_num     : %10d\n", SPIH->image_num     ) ;
      fprintf (stdout, "SPIH->ccd_temp_beg  : %10d\n", SPIH->ccd_temp_beg  ) ;
      fprintf (stdout, "SPIH->ccd_temp_end  : %10d\n", SPIH->ccd_temp_end  ) ;
      fprintf (stdout, "SPIH->pes_yaw_mean  : %10d\n", SPIH->pes_yaw_mean  ) ;
      fprintf (stdout, "SPIH->pes_yaw_min   : %10d\n", SPIH->pes_yaw_min   ) ;
      fprintf (stdout, "SPIH->pes_yaw_max   : %10d\n", SPIH->pes_yaw_max   ) ;
      fprintf (stdout, "SPIH->pes_pitch_mean: %10d\n", SPIH->pes_pitch_mean) ;
      fprintf (stdout, "SPIH->pes_pitch_min : %10d\n", SPIH->pes_pitch_min ) ;
      fprintf (stdout, "SPIH->pes_pitch_max : %10d\n", SPIH->pes_pitch_max ) ;
      fprintf (stdout, "SPIH->cur_ccd_line  : %10d\n", SPIH->cur_ccd_line  ) ;
      }

   /*** MET = 0 corresponds to 1 November 1998 DOY 305 19:04:44 [SP201-5] ***/

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPIH->met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   month_day (iyear, idoy, &imonth, &iday) ;

   SPII->swvers		= 1.00 ;
   SPII->dtype		= 1 ;
   SPII->xdim		= 320 ;
   SPII->ydim		= 256 ;
   SPII->zdim		= 1 ;
   SPII->nbypx		= sizeof (short) ;
   SPII->dmin		= 0 ;
   SPII->dmax		= 0 ;
   SPII->smin		= 0 ;
   SPII->smax		= 0 ;
   SPII->year		= iyear ;
   if (SPII->year < 1900)   SPII->year += 1900 ;
   SPII->month		= imonth ;
   SPII->day		= iday ;
   SPII->doy		= idoy ;
   SPII->hour		= ihour ;
   SPII->minute		= iminute ;
   SPII->second		= (int) isecond ;
   SPII->cal_year	= 0 ;
   SPII->cal_month	= 0 ;
   SPII->cal_day	= 0 ;
   SPII->nimsum		= SPIH->num_img_sum ;
   SPII->orbit_num	= SPIH->orbit_num ;
   SPII->image_num	= SPIH->image_num ;
   SPII->expdur	= (float) SPIH->ccd_expdur * 0.008 ;	/* seconds. */
   SPII->ccd_temp_beg   = ((SPIH->ccd_temp_beg  * 366.80) / 4096.0 - 135.70) ;
   SPII->ccd_temp_end   = ((SPIH->ccd_temp_end  * 366.80) / 4096.0 - 135.70) ;
   SPII->pes_yaw_ave    = (SPIH->pes_yaw_mean   * 5.0 )   / 4096.0 ;
   SPII->pes_yaw_min    = (SPIH->pes_yaw_min    * 5.0 )   / 4096.0 ;
   SPII->pes_yaw_max    = (SPIH->pes_yaw_max    * 5.0 )   / 4096.0 ;
   SPII->pes_pitch_ave  = (SPIH->pes_pitch_mean * 5.0 )   / 4096.0 ;
   SPII->pes_pitch_min  = (SPIH->pes_pitch_min  * 5.0 )   / 4096.0 ;
   SPII->pes_pitch_max  = (SPIH->pes_pitch_max  * 5.0 )   / 4096.0 ;

   SPII->odisk		= 1.25 ;	/* Occulting Disk: solar radii */
   SPII->bscale		= 1.0 ;		/* Intensity scaling multiplier. */
   SPII->bzero		= 0.0 ;		/* Intensity scaling offset. */
   SPII->pixrs		= 0.0 ;		/* # pixels per solar radius. */
   SPII->pangle		= 0.0 ;		/* Solar P-angle. */
   SPII->bangle		= 0.0 ;		/* Solar B-angle. */
   SPII->langle		= 0.0 ;		/* Solar L-angle. */
   SPII->ra		= 0.0 ;		/* Right Ascension. */
   SPII->dec		= 0.0 ;		/* Declination. */
   SPII->hangle		= 0.0 ;		/* Hour Angle. */

   /*** Add North Celestial Pole offset to relative pitch angle
    *** to get the Geocentric north pole position.
    *** Adjust by P-angle to get Heliospheric north pole position.
    ***/

   SPII->ocp_ang = ncp_orb [SPIH->orbit_num] ;	/* North Celestial Pole (deg).*/
   SPII->rcp_ang = SPIH->rel_pitch_ang * 0.5 ;	/* Relative Pitch Angle (deg).*/
   SPII->pangle = pang_orb [SPII->orbit_num] ;	/* Solar P-angle (deg). */
   SPII->wlc_ang = wlc_ang ;		/* WLC angle w.r.t. Spartan S/C. */

   while (SPII->ocp_ang >  360.0)  SPII->ocp_ang -= 360.0 ;
   while (SPII->ocp_ang <    0.0)  SPII->ocp_ang += 360.0 ;

   while (SPII->rcp_ang >  360.0)  SPII->rcp_ang -= 360.0 ;
   while (SPII->rcp_ang <    0.0)  SPII->rcp_ang += 360.0 ;

   ncp_ang = SPII->ocp_ang - SPII->rcp_ang + SPII->wlc_ang ;
   nhp_ang = ncp_ang - SPII->pangle ;

   while (nhp_ang >  360.0)  nhp_ang -= 360.0 ;
   while (nhp_ang <    0.0)  nhp_ang += 360.0 ;
   
/*
   fprintf (stdout, "orb/img: %2d-%2d  rel_pitch: %4d   geo_ang: %7.2f pang: %7.2f  nhp_ang: %7.2f\n",
	    SPII->orbit_num, SPII->image_num, SPIH->rel_pitch_ang,
	    geo_ang, SPII->pangle, nhp_ang) ;
 */
/*
 *   fprintf (FP_ELOG, "orb/img: %2d-%2d  i:%2d  nangle:%7.2f\n",
 *      SPII->orbit_num, SPII->image_num, i, nhp_ang) ;
 *   fflush (FP_ELOG) ;
 */

   SPII->nangle		= nhp_ang ;			/* Solar North Angle. */
   while (SPII->nangle >  360.0)  SPII->nangle -= 360 ;
   while (SPII->nangle <    0.0)  SPII->nangle += 360 ;
   SPII->elev		= 0.0 ;		/* Elevation. */
   SPII->azimuth	= 0.0 ;		/* Azimuth. */
   SPII->rsun		= 967.00 ;	/* Solar Radius (arcsec). */
   SPII->rsun		= rsun_orb [SPII->orbit_num] ;	/* Rsun (arcsec). */
   SPII->xcen		= 0.0 ;		/* Sun center x-axis. */
   SPII->ycen		= 0.0 ;		/* Sun center y-axis. */
   SPII->rmin		= 0.0 ;		/* Field-Of-View inner radius. */
   SPII->rmax		= 0.0 ;		/* Field-Of-View outer radius. */
   SPII->wavelen	= 477.0 ;	/* Central Wavelength (nm). */
   SPII->wavewid	=  60.0 ;	/* Filter bandpass (nm), FWHM +/-6 nm.*/

   for (i = 0; i < 5; i++)
      {
      ipos = SPIH->pos [i] ;
      strcpy (&SPII->optdev [i][0], &namdev [i][0]) ;		/* Dev. Name. */
      SPII->optdev [i][11] = '\0';

      strcpy (&SPII->optpos [i][0], &nampos [i][ipos][0]) ;	/* Position. */
      SPII->optpos [i][11] = '\0';
      }

   /*** Print housekeeping. ***/

   if (pmode)
      {
      fprintf (stdout, "SPII->swvers: 		%10d\n",	SPII->swvers) ;
      fprintf (stdout, "SPII->dtype:		%10d\n",	SPII->dtype) ;
      fprintf (stdout, "SPII->xdim:		%10d\n",	SPII->xdim) ;
      fprintf (stdout, "SPII->ydim:		%10d\n",	SPII->ydim) ;
      fprintf (stdout, "SPII->zdim:		%10d\n",	SPII->zdim) ;
      fprintf (stdout, "SPII->nbypx:		%10d\n",	SPII->nbypx) ;
      fprintf (stdout, "SPII->dmin:		%10d\n",	SPII->dmin) ;
      fprintf (stdout, "SPII->dmax:		%10d\n",	SPII->dmax) ;
      fprintf (stdout, "SPII->smin:		%10d\n",	SPII->smin) ;
      fprintf (stdout, "SPII->smax:		%10d\n",	SPII->smax) ;
      fprintf (stdout, "SPII->year:		%10d\n",	SPII->year) ;
      fprintf (stdout, "SPII->month:		%10d\n",	SPII->month) ;
      fprintf (stdout, "SPII->day:		%10d\n",	SPII->day) ;
      fprintf (stdout, "SPII->doy:		%10d\n",	SPII->doy) ;
      fprintf (stdout, "SPII->hour:		%10d\n",	SPII->hour) ;
      fprintf (stdout, "SPII->minute:		%10d\n",	SPII->minute) ;
      fprintf (stdout, "SPII->second:		%10d\n",	SPII->second) ;
      fprintf (stdout, "SPII->cal_year:		%10d\n",	SPII->cal_year) ;
      fprintf (stdout, "SPII->cal_month:	%10d\n",	SPII->cal_month) ;
      fprintf (stdout, "SPII->cal_day:		%10d\n",	SPII->cal_day) ;
      fprintf (stdout, "SPII->nimsum:		%10d\n",	SPII->nimsum) ;


      fprintf (stdout, "SPII->expdur:		%10.4\n",	SPII->expdur) ;
      fprintf (stdout, "SPII->ccd_temp_beg:	%10.4\n",	SPII->ccd_temp_beg) ;
      fprintf (stdout, "SPII->ccd_temp_end:	%10.4\n",	SPII->ccd_temp_end) ;
      fprintf (stdout, "SPII->pes_yaw_ave:	%10.4\n",	SPII->pes_yaw_ave) ;
      fprintf (stdout, "SPII->pes_yaw_min:	%10.4\n",	SPII->pes_yaw_min) ;
      fprintf (stdout, "SPII->pes_yaw_max:	%10.4\n",	SPII->pes_yaw_max) ;
      fprintf (stdout, "SPII->pes_pitch_ave:	%10.4\n",	SPII->pes_yaw_ave) ;
      fprintf (stdout, "SPII->pes_pitch_min:	%10.4\n",	SPII->pes_yaw_min) ;
      fprintf (stdout, "SPII->pes_pitch_max:	%10.4\n",	SPII->pes_yaw_max) ;

      fprintf (stdout, "SPII->odisk:		%10.4\n",	SPII->odisk) ;
      fprintf (stdout, "SPII->bscale:		%10.4f\n",	SPII->bscale) ;
      fprintf (stdout, "SPII->bzero:		%10.4f\n",	SPII->bzero) ;
      fprintf (stdout, "SPII->pixrs:		%10.4f\n",	SPII->pixrs) ;
      fprintf (stdout, "SPII->pangle:		%10.4f\n",	SPII->pangle) ;
      fprintf (stdout, "SPII->bangle:		%10.4f\n",	SPII->bangle) ;
      fprintf (stdout, "SPII->langle:		%10.4f\n",	SPII->langle) ;
      fprintf (stdout, "SPII->hangle:		%10.4f\n",	SPII->hangle) ;
      fprintf (stdout, "SPII->nangle:		%10.4f\n",	SPII->nangle) ;
      fprintf (stdout, "SPII->elev:		%10.4f\n",	SPII->elev) ;
      fprintf (stdout, "SPII->azimuth:		%10.4f\n",	SPII->azimuth) ;
      fprintf (stdout, "SPII->rsun:		%10.4f\n",	SPII->rsun) ;
      fprintf (stdout, "SPII->xcen:		%10.4f\n",	SPII->xcen) ;
      fprintf (stdout, "SPII->ycen:		%10.4f\n",	SPII->ycen) ;
      fprintf (stdout, "SPII->rmin:		%10.4f\n",	SPII->rmin) ;
      fprintf (stdout, "SPII->rmax:		%10.4f\n",	SPII->rmax) ;
      fprintf (stdout, "SPII->wavelen:		%10.4f\n",	SPII->wavelen) ;
      fprintf (stdout, "SPII->wavewid:		%10.4f\n",	SPII->wavewid) ;
      for (i = 0; i < 5; i++)
	 {
         fprintf (stdout, "SPII->optdev[%d]: %-16s	SPII->optpos[%d]: %s\n",
		  i, &SPII->optdev [i][0], i, &SPII->optpos[i][0]) ;
	 }
      }

   return (0) ;
}
