/* srfaw.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a sequence Sun rasterfile images,
 * load colormap to LUT, load image to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Supports only 8-bit greyscale, 8-bit colormap, and 24-bit RGB image types.
 * Also display 32-bit RGBA images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 28 January 92
 * 20 August 92: add ability to display a sequence of images using keyboard.
 *  3 September 92: add forward/backward control.
 *                  add option to use selectable colormap.
 * 14 October 92: use mouse buttons to go forward or backward; use ESC to quit.
 * 20 August 1993: displays RGBA sun rasterfile images (32-bit color).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "/usr/people/stanger/lib/include/rasterfile.h"

main (argc, argv)

int argc;
char *argv [];
{
   extern char *malloc ();
   extern int gl_wbox1 ();		/* Write colormap image to GL window. */
   FILE *LIST;
   short val;

   int gl_wrgb ();			/* Write RGB image to GL window. */
   int gl_wrgba ();			/* Write RGBA image to GL window. */

   static char dir2 [4] = "rd";
   static char namlst [80];
   static char namsrf [80];
   static char lutbuf [768];

   static char flist [60][80];
   unsigned char *imgbuf;
   unsigned char *imgr, *imgg, *imgb;
   static unsigned short int sred [256], sgreen [256], sblue [256];

   static int srfhdr [8];

   int begindex = 512, numindex = 256;
   int colmax = 0;
   int dxg = 1280, dyg = 1024;
   int dzs = 0;
   int fd;				/* Image file descriptor. */
   int findex = 0;
   int i;
   int ians;				/* KB response. */
   int imgsize;
   int lutnum = 2;			/* LUT number. */
   int lutoff = 512;			/* LUT offset. */
   int maxdim;
   int nbypx;
   int ngr = 0, imgch = 0;
   int npximg;
   int nread;
   int numfil;				/* # images in file list. */
   int prevdxs = 0, prevdys = 0;	/* Previous image size. */
   int seq;
   int xs, ys;

   int magic;
   int dxs, dys;
   int bitpix;
   int imgsiz, imgtyp;
   int coltyp, colsiz;
   int xll, yll;
   int xur, yur;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   /*** Get sun rasterfile from command line. ***/

   if (argc == 2)
      {
      strcpy (namsrf, argv [1]);
      }
   else
   if (argc == 3)
      {
      sscanf (argv [1], "%d", &lutnum);
      strcpy (namsrf, argv [2]);
      }
   else
      {
      fprintf (stderr, "usage: %s [lutnum] sun_rasterfile_name | @namelist | ^namelist\n",
               argv [0]);
      exit (-1);
      }

   lutoff = lutnum * 256;
   begindex = lutoff;

   if (namsrf [0] == '@' || namsrf [0] == '^')
      {
      if (namsrf [0] == '@')
         seq = 1;
      else
      if (namsrf [0] == '^')
         seq = 2;

      strcpy (namlst, &namsrf [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = 0;
      while (fscanf (LIST, "%s", namsrf) != EOF)
         {
         strcpy (&flist [i++][0], namsrf);
         }
      numfil = i;
      fclose (LIST);
      }

   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "position cursor within %s window.\n", argv [0]);
      fprintf (stderr, "click `left mouse'   to go forward.\n");
      fprintf (stderr, "click `middle mouse' to go backward.\n");
      fprintf (stderr, "type  `ESC'          to quit.\n");
      }
   else
      {
      fprintf (stderr, "position cursor within %s window.\n", argv [0]);
      fprintf (stderr, "type  `ESC'          to quit.\n");
      }

   /*** Initialize GL software. ***/

   foreground ();

loop:

   if (seq)
      strcpy (namsrf, &flist [findex][0]);

   /*** Read Sun rasterfile header. ***/

   if ((fd = open (namsrf, 0)) < 0)
      {
      fprintf (stderr, "Unable to open image file: %s\n", namsrf);
      exit (-3);
      }
   nread = read (fd, srfhdr, 32);

   /*** Decode Sun rasterfile header. ***/

   magic  = srfhdr [0];			/* Sun rasterfile magic number. */
   dxs    = srfhdr [1];			/* Image width (pixels). */
   dys    = srfhdr [2];			/* Image height (pixels). */
   bitpix = srfhdr [3];			/* Image depth (bits/pixel). */
   imgsiz = srfhdr [4];			/* Image size (bytes). */
   imgtyp = srfhdr [5];			/* Image type. */
   coltyp = srfhdr [6];			/* Colormap type. */
   colsiz = srfhdr [7];			/* Colormap size (bytes). */

   xs = 0;
   ys = dys - 1;
   nbypx = bitpix / 8;
   npximg = dxs * dys;
   imgsize = npximg * nbypx;

/*
 *   fprintf (stdout, "dxs: %4d   dys: %4d   bitpix: %2d   imgsiz: %7d\n",
 *            dxs, dys, bitpix, imgsiz);
 *   fprintf (stdout, "imgtyp: %d   coltyp: %d   colsiz: %d\n",
 *            imgtyp, coltyp, colsiz);
 */

   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "%s: Not a sun rasterfile.\n", namsrf);
      exit (-4);
      }

   if (dxs > dxg || dys > dyg)
      {
      fprintf (stderr, "Invalid image size . . . dxs: %3d . . . dys: %3d\n",
	       dxs, dys);
      exit (-5);
      }

   if (imgtyp != RT_STANDARD)
      {
      fprintf (stderr, "Only RT_STANDARD type sun rasterfiles are supported.\n");
      exit (-6);
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz);
      exit (-7);
      }

   /*** Copy RGB lookup tables into unsigned short int arrays. ***/

   if (colsiz > 0)
      {
      nread = read (fd, lutbuf, colsiz);
      colmax = colsiz / 3;
      for (i = 0; i < colmax; i++)
         {
         sred [i]   = (unsigned short int) (lutbuf [i]            & 0377);
         sgreen [i] = (unsigned short int) (lutbuf [i + colmax]   & 0377);
         sblue [i]  = (unsigned short int) (lutbuf [i + colmax*2] & 0377);
         }
      }

   /*** Read image from file into memory. ***/

   imgbuf = (unsigned char *)  malloc (imgsize);
   nread = read (fd, imgbuf, imgsize);
   close (fd);

   /*** Compute lower left and upper right coordinates for GL window. ***/

   if (dxs <= 1270)  xur = 1270; else xur = 1279;
   if (dys <=  995)  yur =  995; else yur = 1023;
   xll = xur - dxs + 1;
   yll = yur - dys + 1;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n");
      exit (-1);
      }
/*
 *   fprintf (stdout, "xll: %4d  yll: %4d  xur: %4d  yur: %4d\n",
 *            xll, yll, xur, yur);
 */

   /*** Open GL window. ***/

   prefposition (xll, xur, yll, yur);

   if (dxs != prevdxs || dys != prevdys)
      {
      printf ("winopen . . . prevdxs: %4d  prevdys: %4d   dxs: %4d  dys: %4d\n",
              prevdxs, prevdys, dxs, dys);
      winopen (argv [0]);
      doublebuffer ();
      gconfig ();
      }
   prevdxs = dxs;
   prevdys = dys;

   /*** Write greyscale or colormap (pseudo-color) image to GL window. ***/

   if (imgtyp == RT_STANDARD && bitpix == 8)
      {
      gl_wbox1 (imgbuf, lutoff, nbypx, xs, ys, dxs, dys, dir2, 0);
      }

   /*** Write RGB image to GL window. ***/
   /*
    * RT_STANDARD RGB sun rasterfile:
    * Each triplet (adjacent 3 bytes) contains
    * Blue, Green & Red pixel intensities, in that order.
    */

   if (imgtyp == RT_STANDARD && bitpix == 24)
      gl_wrgb (imgbuf, dxs, dys);

   if (imgtyp == RT_STANDARD && bitpix == 32)
      gl_wrgba (imgbuf, dxs, dys);

   swapbuffers ();

   /*** Write color map (if present) to appropriate look-up tables. ***/

   if (colsiz > 0)
      loadcm (begindex, numindex, sred, sgreen, sblue);

   fprintf (stderr, "%s > ", namsrf);

   /*** Window remains on screen until terminated by mouse. ***/

   /*** Declare devices. ***/

   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               free (imgbuf);
               if (findex >= numfil)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.");
                     findex = numfil - 1;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               fprintf (stderr, "\n"); 
               goto loop;
               }
            break;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex--;
               free (imgbuf);
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = numfil - 1;
                  }
               fprintf (stderr, "\n"); 
               goto loop;
               }
            break;

         case REDRAW:
            gl_wbox1 (imgbuf, lutoff, nbypx, xs, ys, dxs, dys, dir2, 0);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               free (imgbuf);
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

quit:
   gexit ();
   exit (0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * gl_wrgb: Write RGB image (standard Sun rasterfile format) to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_wrgb (imgbuf, dxs, dys)

unsigned char *imgbuf;
int dxs, dys;

{
   unsigned char *ibuf;
   unsigned long *rgb, *imgrgb;
   int i, iy;
   int red, green, blue;
   int npximg;

   npximg = dxs * dys;
   imgrgb = (unsigned long *) malloc (dxs * dys * sizeof (long));

   for (i = 0; i < npximg; i++)
      {
      rgb = imgrgb + i;
      ibuf = imgbuf + i * 3;

      blue  = (long) *ibuf & 0377;
      green = (long) *(ibuf + 1) & 0377;
      red   = (long) *(ibuf + 2) & 0377;
      *rgb = (blue << 16 | green << 8 | red) & 077777777;
      }

   RGBmode ();
   gconfig ();

   for (i = 0; i < dys; i++)
      {
      iy = dys - i - 1;
      lrectwrite (0, iy, dxs-1, iy, imgrgb + i * dxs);
      }

   return (0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * gl_wrgba: Write RGBA image (standard Sun rasterfile format) to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_wrgba (imgbuf, dxs, dys)

unsigned char *imgbuf;
int dxs, dys;

{
   unsigned char *ibuf;
   unsigned long *abgr, *imgabgr;
   unsigned long red, green, blue, alpha;
   int i, iy;
   int npximg;

   npximg = dxs * dys;
   imgabgr = (unsigned long *) malloc (dxs * dys * sizeof (long));

   for (i = 0; i < npximg; i++)
      {
      abgr = imgabgr + i;
      ibuf = imgbuf + i * 4;

      alpha = (unsigned long) *(ibuf) & 0377;
      blue  = (unsigned long) *(ibuf + 1) & 0377;
      green = (unsigned long) *(ibuf + 2) & 0377;
      red   = (unsigned long) *(ibuf + 3) & 0377;
      *abgr = (alpha << 24 | blue << 16 | green << 8 | red) & 037777777777;
      }

   RGBmode ();
   gconfig ();

   for (i = 0; i < dys; i++)
      {
      iy = dys - i - 1;
      lrectwrite (0, iy, dxs-1, iy, imgabgr + i * dxs);
      }

   free (imgabgr);
   return (0);
}
