/* cptofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert SMM C/P image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR   27 October 1993
 *  3 February 1994: Change order of data units: image, hk+bm, colormap.
 * 28 March 1994: reverse image top-to-bottom.
 * 18 August 1994: change CPVER to VERSION.
 * 19 March 1996: change cptilt from +43 to -43 degrees. CROTA1=cptilt+scroll.
 *                crpix2 = ny - ycen, since image is reversed top to bottom.
 *                version = 'SMM C/P 1.02'
 * 25 March 1996: change wmin/wmax to SCALED values.
 * 10 July 1996: version = "SMM C/P 1.03'
 *               Add DIRAXIS1='RIGHT' and DIRAXIS2='UP'.
 *  4 March 1997: CDELT1 & CDELT2 changed from Rsun to Arcsecond units.
 *  5 March 1997: Add CRRADIUS (pixels/solar radius).  Change RSUN units
 *                from pixels/solar radius to arcsec/solar radius.
 *  2 September 1997: Corrected 5303 filter label from FE-IV to FE-XIV.
 * 13 November 1997: Subtract SCROLL from CPTILT to get orientation.
 * 13 November 1997: Change cptilt to +43 degrees.  CROTA1 = - (cptilt+scroll).
 * 23 February 1998: Object='corona' or 'calib', depending upon calmir positon.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"
#define MAXLIN 180		/* Maximum # lines allocated for FITS header. */
char fitbuf   [MAXLIN][80] ;	/* FITS header buffer. */

cptofits (namimg, namext, hduflag, pmode)

   char namimg [] ;		/* Name of primary file. */

   char namext [] ;		/* Name of extension file. */

   int hduflag ;		/* 1: primary image; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */
				/* 4: extension data: C/P housekeeping. */

   int pmode ;			/* Print mode. */


{
   extern unsigned char *rdfimg () ;
   extern char *malloc () ;
   extern float sunrad () ;
   char *padbuf ;
   static char namfit [80] ;			/* FITS file name. */
   static char outbuf [84] ;
   static char pixcom [48] ;
   static char eq [4] = "= " ;
   static char diraxis1  [12] = "RIGHT" ;	/* Direction of axis 1: right.*/
   static char diraxis2  [12] = "UP" ;		/* Direction of axis 2: up. */

   static char kbitpix   [12] = "BITPIX  " ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;

   static char kangnorth [12] = "ANGNORTH" ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kcalmir   [12] = "CALMIR  " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcolorfil [12] = "COLORFIL" ;
   static char kcomment  [12] = "COMMENT " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kcptilt   [12] = "CPTILT  " ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kdataform [12] = "DATAFORM" ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdiraxis1 [12] = "DIRAXIS1" ;
   static char kdiraxis2 [12] = "DIRAXIS2" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kend      [12] = "END     " ;
   static char kframe    [12] = "FRAME   " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kexpunits [12] = "EXPUNITS" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char kglfilter [12] = "GLFILTER" ;
   static char khistory  [12] = "HISTORY " ;
   static char kinstrume [72] = "INSTRUME" ;
   static char korigin   [72] = "ORIGIN  " ;
   static char kobject   [72] = "OBJECT  " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char kpolaroid [12] = "POLAROID" ;
   static char kquantity [12] = "QUANTITY" ;
   static char krsun     [12] = "RSUN    " ;
   static char kscroll   [12] = "SCROLL  " ;
   static char ksector   [12] = "SECTOR  " ;
   static char ktelescop [12] = "TELESCOP" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [72] = "TYPE-OBS" ;
   static char kversion  [12] = "VERSION " ;

   static char atype1   [12] = "X" ;
   static char atype2   [12] = "Y" ;
   static char bunit    [24] ;
   static char calmir   [12] ;
   static char colorfil [12] ;
   static char ctype1   [12] ;
   static char ctype2   [12] ;
   static char ctypenam [12] = "GEOCENTRIC" ;
   static char cunit1   [24] ;			/* Coord units axis 1. */
   static char cunit2   [24] ;			/* Coord units axis 2. */
   static char date     [12] ;
   static char date_obs [12] ;
   static char datcom   [24] ;			/* data scale comment. */
   static char dform    [24] ;
   static char glfilter [24] ;
   static char object   [16] = "CORONA" ;
   static char origin   [12] = "HAO" ;
   static char polaroid [24] ;
   static char refcom   [48] = "SUN CENTER" ;
   static char telescop [12] = "SMM C/P" ; 
   static char time     [12] ;
   static char time_obs [12] ;
   static char type_obs [72] = "WHITE LIGHT" ;
   static char namfil [6][12] = {"RED", "H-ALPHA", "OPEN", "BLUE", "WIDEBAND",
				 "GREEN"} ;
   static char namcm  [4][8]  = {"MOVE", "OUT", "IN", "BAD"} ;
   static char namgls [4][8]  = {"MOVE", "FE-XIV", "STOW", "FAIL"} ;
   static char namobj [4][8]  = {"UNKNOWN", "CORONA", "CALIB", "UNKNOWN"};
   static char nampol [4][8]  = {"CLEAR", "+60 DEG", "-60 DEG", "  0 DEG"} ;
/*   static char nampol [4][8]  = {"P0", "P1", "P3", "P2"} ; */
   static char version  [24] ;			/* SMM C/P version. */

   unsigned char *cpbuf ;		/* SMM C/P file  array pointer. */
   unsigned char *imbuf ;		/* SMM C/P image array pointer. */
   unsigned char *cbuf ;
   short *sbuf ;
   float *fbuf ;

   char  cmin, cmax ;			/* min/max values. */
   short smin, smax ;			/* min/max values. */
   float fmin, fmax ;			/* min/max values. */

   int bitpix ;				/* # bits per pixel. */
   int bphk = 8 ;			/* bits/pixel for hskp & bitmap. */
   int gcount = 1 ;			/* Extension gcount. */
   int iday, imonth, iyear ;		/* date. */
   int i, ix, iy ;			/* indices. */
   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* Integer value. */
   int iynum ;				/* # lines in FITS header. */
   int len ;				/* String length. */
   int nbypx ;				/* # bytes per pixel. */
   int ndim = 2 ;			/* # dimensions for image (2=XY). */
   int nhdu ;				/* # HDU blocks to write. */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyfits = 2880 ;			/* # bytes per FITS logical record. */
   int nbyhk ;				/* # bytes in hskp & bitmap records. */
   int nrechd = 2 ;			/* # records for hskp & bitmap. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int nrec ;				/* # fits records for image. */
   int nwrite ;				/* # bytes written to disk. */
   int pcount = 0 ;			/* Extension pcount. */
   int swrite ;				/* # bytes written to disk. */

   float angnorth ;			/* angle of solar north. */
   float bscale = 1.0 ;			/* Brightness intensity scale factor. */
   float bzero  = 0.0 ;			/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float cptilt =  43.0 ;		/* Axis tilt for S/C roll = 0. */
   float crpix1, crpix2 ;		/* Reference pixel location.*/
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin = 0.0 ;			/* Data minimum value. */
   float dmax = 0.0 ;			/* Data maximum value. */
   float expdur ;			/* Exposure duration (seconds). */
   float htilt  =  43.0 ;		/* Horiz.Axis tilt including S/C roll.*/
   float secrs  = 960.0 ;		/* Arcseconds / solar radius. */
   float vtilt  =  43.0 ;		/* Vert. Axis tilt including S/C roll.*/
   float wmin = 0.0 ;			/* Display min value. */
   float wmax = 0.0 ;			/* Display max value. */

   FILE *FF ;				/* FITS file ID. */

   nwrite = 0 ;
   swrite = 0 ;

   sprintf (namfit, "%s.fts", namimg) ;

   printf ("\n------- cptofits> %s -> %s -------\n\n", namimg, namfit) ;

   if (hduflag == 1 || hduflag == 2)
      fprintf (stdout, " --- PRIMARY   HDU ---\n") ;
   else
      fprintf (stdout, " --- EXTENSION HDU ---\n") ;

   /*** Read SMM C/P image from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2 || hduflag == 4)
      cpbuf = rdfimg (namimg) ;		/* Read C/P image into memory. */
   else
      cpbuf = rdfimg (namext) ;		/* Read C/P image into memory. */

   if (nbypix == 4)
      nrechd = 1 ;
   else
      nrechd = 2 ;

   nbyhk = npix ;				/* Size of hskp data. */
   imbuf   = cpbuf + npixl * nbypix * nrechd ;	/* Start of pixel data. */

   if (hduflag == 4)
      {
      nx = npix ;
      ny = 2 ;				/* 1st record is hskp, 2nd is bitmap. */
      nbypx = 1 ;			/* # bytes per element. */
      nbyimg  = nx * ny ;		/* # bytes in "image". */
      npximg  = 0 ;
      }
   else
      {
      nx = ny = npixl ;			/* # pixels for X & Y axes. */
      nbypx = nbypix ;			/* # bytes per pixel. */
      npximg = nx * ny ;		/* # pixels in image. */
      nbyimg = npximg * nbypix ;	/* # bytes in image. */
      }

   /*** Set object field. ***/

   strcpy (object, &namobj [H.icm][0]) ;

   /*** Determine month & day-of-month. ***/

   iyear = H.year ;
   if (H.year > 1900)  iyear = H.year - 1900 ;
   month_day (H.year, H.doy, &imonth, &iday) ;

   /*** Format observation date & time parameters. ***/

   sprintf (date_obs, "%02d/%02d/%02d", iday, imonth, iyear) ;
   sprintf (time_obs, "%02d:%02d:%02d", H.hour, H.minute, H.second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   sprintf (date, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (time, "%02d:%02d:%02d", ichour, icmin, icsec) ;

   /*** Set solar north angle. ***/

   htilt    = - (cptilt + scroll) ;		/* Angle of north w.r.t. +Y axis. */
   vtilt    = - (cptilt + scroll) ;		/* Angle of north w.r.t. +Y axis. */
   angnorth = - (cptilt + scroll) ;		/* Angle of north w.r.t. +Y axis. */

   /*** Coordinate system. ***/

   sprintf (ctype1, "%-8s", atype1) ;
   sprintf (ctype2, "%-8s", atype2) ;
   crpix1 = H.xcen ;				/* X sun center: pixels. */
   crpix2 = (float) 447*(2-H.ires) - H.ycen ;	/* Y sun center: pixels. */
   crval1 = crval2 = 0.0 ;			/* Heliocentric sun center. */
   secrs = sunrad (H.year, H.doy, H.hour) ;	/* Arcseconds / Rsun. */
   cdelt1 = cdelt2 = 1.0 / H.pixrs * secrs ;	/* Arcseconds / pixel. */
   strcpy (cunit1, "ARCSECONDS") ;
   strcpy (cunit2, "ARCSECONDS") ;
   sprintf (version, "SMM C/P 1.03") ;

   /*** Set brightness unit. ***/
   /*** Set data form. ***/
   /*** Set display intensity min/max. ***/
   /*** Set bscale & bzero.  ***/

   switch (H.idtype)
      {
      case 1:	strcpy (bunit, "DN") ;
		strcpy (dform, "RAW VIDEO") ;
		wmin = 0.0 ;	wmax = 255.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 2:	strcpy (bunit, "1.024 SEC") ;
		strcpy (dform, "EQUIV.SEC") ;
		wmin = 0.0 ;	wmax = 30.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 3:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "B-TOTAL") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 4:	strcpy (bunit, "NELEC/CM2") ;
		strcpy (dform, "ELEC DEN") ;
		wmin = 0.0 ;	wmax = 5.0e+15 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 5:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "BRT DIFF") ;
		wmin = -2.0e-9 ;wmax = 2.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 6:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "B [K]") ;
		wmin = 0.0 ;	wmax = 2.5e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 7:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "B2-B1/BK") ;
		wmin = -1.0 ;	wmax = 1.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 8:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "B2-B1/B1") ;
		wmin = -1.0 ;	wmax = 1.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 9:	strcpy (bunit, "FRACTION") ;
		strcpy (dform, "POL.FRAC") ;
		wmin =  0.0 ;	wmax = 1.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 10:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "POL.BRT.") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 11:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "BTAN+BRAD") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 12:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "B [TAN] ") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 13:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "B [RAD] ") ;
		wmin = 0.0 ;	wmax = 5.0e-9 ;
		bzero = 0.0 ;	bscale = H.pscale ;
		break ;
      case 14:	strcpy (bunit, "BSUN") ;
		strcpy (dform, "BF*V + S") ;
		wmin = 0.0 ;	wmax = 2.5e-9 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 15:	strcpy (bunit, "RATIO") ;
		strcpy (dform, "B1/B2") ;
		wmin = 0.0 ;	wmax = 2.0 ;
		bzero = 0.0 ;	bscale = 1.0 ;
		break ;
      case 16:
      default:	strcpy (bunit, "B") ;
		strcpy (dform, "B") ;
		wmin = 0.0 ;	wmax = 2.0e-9 ;
		bzero = 0.0 ;	bscale = 1.0 ;
      }
   sprintf (datcom, "%s", bunit) ;
/*   squeeze (datcom, (int) 047) ; */		/* Eliminate "'" from string. */
/*   squeeze (datcom, (int) 040) ; */		/* Eliminate " " from string. */

   sprintf (colorfil, "%s",	&namfil [H.ifw][0]) ;
   if (H.iglstw == 1)
      sprintf (glfilter, "%03d:%s", H.iglpos, &namgls [H.iglstw][0]) ;
   else
      sprintf (glfilter, "%s", &namgls [H.iglstw][0]) ;
   sprintf (polaroid, "%s",	&nampol [H.ipw][0]) ;
   sprintf (calmir,   "%s",	&namcm  [H.icm][0]) ;

   /*** Set number of bits per pixel. ***/

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;

   switch (nbypx)
      {
      case 1:	bitpix =   8 ;			/* byte image. */
		strcpy (pixcom, "1-BYTE UNSIGNED PIXELS") ;
		if (hduflag == 4)  break ;
		cbuf = imbuf ;
		cmin = 255 ;
		cmax =   0 ;
                for (i = 0 ; i < npximg; i++)
                   {
                   if (*(cbuf + i) < cmin)  cmin = *(cbuf + i) ;
                   if (*(cbuf + i) > cmax)  cmax = *(cbuf + i) ;
                   }
                dmin = (float) cmin * bscale + bzero ;
                dmax = (float) cmax * bscale + bzero ;
		printf ("cmin/cmax: %3d %3d\n", cmin, cmax) ;
		break ;

      case 2:	bitpix =  16 ;			/* short integer image. */
		strcpy (pixcom, "2-BYTE SIGNED PIXELS.") ;
		sbuf = (short *) imbuf ;
		smin =  32767 ;
		smax = -32767 ;
                for (i = 0 ; i < npximg; i++)
                   {
                   if (*(sbuf + i) < smin)  smin = *(sbuf + i) ;
                   if (*(sbuf + i) > smax)  smax = *(sbuf + i) ;
                   }
                dmin = (float) smin * bscale + bzero ;
                dmax = (float) smax * bscale + bzero ;
		printf ("smin/smax: %4d %4d\n", smin, smax) ;
		break ;

      case 4:	bitpix = -32 ;			/* floating point image. */
		strcpy (pixcom, "4-BYTE FLOATING POINT PIXELS.") ;
		fbuf = (float *) imbuf ;
		fmin =  1.e+30 ;
		fmax = -1.e+30 ;
                for (i = 0 ; i < npximg; i++)
                   {
                   if (*(fbuf + i) < fmin)  fmin = *(fbuf + i) ;
                   if (*(fbuf + i) > fmax)  fmax = *(fbuf + i) ;
                   }
                dmin = (float) fmin * bscale + bzero ;
                dmax = (float) fmax * bscale + bzero ;
		printf ("fmin/fmax: %8.2g %8.2g\n", fmin, fmax) ;
		break ;

      default:	fprintf (stderr, "Invalid nbypx: %d\n", nbypx) ;  exit (-1) ;
      }
   printf ("nx/ny: %3d %3d   npximg: %6d  dmin/dmax: %8.2g   %8.2g\n",
	   nx, ny, npximg, dmin, dmax) ;

   /*** Create FITS file. ***/

   if (hduflag == 3  || hduflag == 4)			/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
      strcpy  (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: SMM C/P IMAGE") ;
   else if (hduflag == 3)
      sprintf (&fitbuf [iy++][0],  "XTENSION= 'IMAGE   '           / SMM C/P %s EXTENSION", dform) ;
   else if (hduflag == 4)
      sprintf (&fitbuf [iy++][0],  "XTENSION= 'SMMCP_HK'           / SMM C/P HOUSEKEEPING EXTENSION") ;
   else
      {
      fprintf (stderr, "Invalid hduflag value: %d\n", hduflag) ;
      return (-1) ;
      }

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                         knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                         knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                            knaxis2, eq, ny, atype2) ;
   else
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;

   /*** Extension information. ***/

   if (hduflag == 1)
      {
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;
      }
   else
   if (hduflag == 2)
      {
      strcpy  (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
      }
   else
   if (hduflag == 3 || hduflag == 4)
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION PCOUNT",
	                            kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION GCOUNT",
				    kgcount, eq, gcount) ;
      }

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kobject, eq, object) ;
   strcpy  (&fitbuf [iy++][31], "/ FIELD-OF-VIEW: APPROX. 1.5 - 5.5 SOLAR RADII") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  ktype_obs, eq, type_obs) ;
   strcpy  (&fitbuf [iy++][31], "/ SEE 'COLORFIL' FOR SPECTRAL FILTER INFORMATION") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  korigin, eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ HIGH ALTITUDE OBSERVATORY (HAO)") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR MAXIMUM MISSION CORONAGRAPH/POLARIMETER") ;
   strcpy  (&fitbuf [iy  ][0],  "INSTRUME= 'VIDICON '") ;
   strcpy  (&fitbuf [iy++][31], "/ WESTINGHOUSE SEC VIDICON: 896x896 SQUARE PIXELS") ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DAY/MONTH/YEAR [DOY %3d 19%2d]",
				 H.doy, H.year) ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HOUR/MINUTE/SECOND") ;

   /*** Coordinate system. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s", atype1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [PIXELS]",
				 kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [%s]",
				 kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s COORD INCREMENT [%s/PIXEL]",
				 kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / C/P +X = %7.2f DEGREES CW FROM ARRAY +X AXIS", 
				 kcrota1, eq, htilt, htilt) ;

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s", atype2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [PIXELS]",
			            kcrpix2, eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [%s]",
			            kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s COORD INCREMENT [%s/PIXEL]",
			            kcdelt2, eq, cdelt2, atype2, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / C/P +Y = %7.2f DEGREES CW FROM ARRAY +Y AXIS", 
				    kcrota2, eq, htilt, htilt) ;
      }

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kbunit, eq, bunit) ;
   strcpy  (&fitbuf [iy++][31], "/ MAGNITUDE UNITS") ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / %s = DATA * BSCALE + BZERO",
			        kbscale, eq, bscale, datcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e", kbzero , eq, bzero ) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MINIMUM DATA VALUE [%s]",
			         kdatamin , eq, dmin, datcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MAXIMUM DATA VALUE [%s]",
			         kdatamax , eq, dmax, datcom) ;

   /*** All keywords after this point are unique to SMM C/P. ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MIN VALUE FOR DISPLAY [%s]",
			         kdispmin , eq, wmin, datcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3e / MAX VALUE FOR DISPLAY [%s]",
			         kdispmax , eq, wmax, datcom) ;

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [iy  ][0],  "%-8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM NAME") ;

   /*** Coronagraph tilt with respect to the SMM spacecraft. ***/
   /*** C/P North is tilted 43 degrees CCW from SMM spacecraft axis. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s%20.3f", kcptilt, eq, cptilt) ;
   sprintf (&fitbuf [iy++][31], "/ CORONAGRAPH AXIS W.R.T. SMM SPACECRAFT [DEG]") ;

   /*** Solar Radius size (pixels). ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / SOLAR RADIUS [pixels]", 
				 kcrradius, eq, H.pixrs) ;

   /*** Solar Radius size (arcseconds). ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / SOLAR RADIUS [ARCSEC]",
			         krsun, eq, secrs ) ;

   /*** Display Orientation. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdiraxis1, eq, diraxis1) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 1 [X]") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdiraxis2, eq, diraxis2) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION FOR AXIS 2 [Y]") ;

   /*** Type of data values. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kdataform, eq, dform) ;
   strcpy  (&fitbuf [iy++][31], "/ SMM C/P DATA FORM") ;

   /*** Instrument configuration. ***/

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kcolorfil,   eq, colorfil) ;
   strcpy  (&fitbuf [iy++][31], "/ COLORS: RED,H-ALPHA,OPEN,BLUE,WIDEBAND,GREEN") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kglfilter,   eq, glfilter) ;
   strcpy  (&fitbuf [iy++][31], "/ GREEN LINE FILTER POSITION & STOW STATUS") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kpolaroid,   eq, polaroid) ;
/*
 *   strcpy  (&fitbuf [iy++][31], "/ POLAROID FILTER: P0=CLR,P1=+60,P2=0,P3=-60 DEG") ;
 */
   strcpy  (&fitbuf [iy++][31], "/ POLAROID FILTER: CLEAR,+60 DEG,0 DEG,-60 DEG") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", ksector,     eq, H.sec) ;
   strcpy  (&fitbuf [iy++][31], "/ SECTOR MIRROR POSITION: SW,S,SE,E,NE,N,NW,W,4Q") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kcalmir,     eq, calmir) ;
   strcpy  (&fitbuf [iy++][31], "/ CALIBRATION MIRROR : IN=CAL.PATH, OUT=CORONA") ;
   expdur = (float) H.exp * 1.024 ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20.3f", kexpdur,   eq, expdur) ;
   strcpy  (&fitbuf [iy++][31], "/ EXPOSURE DURATION [SECONDS]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d", kscroll,   eq, H.roll) ;
   strcpy  (&fitbuf [iy++][31], "/ SPACECRAFT ROLL [DEGREES]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d", kframe,    eq, H.frame) ;
   strcpy  (&fitbuf [iy++][31], "/ IMAGE FRAME COUNTER [0 -> 4095]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d", kquantity, eq, H.quan) ;
   strcpy  (&fitbuf [iy++][31], "/ IMAGE QUANTITY [% captured]") ;

   /*** FITS Conversion: software version. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'", kversion,  eq, version) ;
   strcpy  (&fitbuf [iy++][31], "/ SMM C/P FITS SOFTWARE VERSION") ;

/*
 *   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
 */
   sprintf (&fitbuf [iy++][0] , "COMMENT   THE HIGH ALTITUDE OBSERVATORY (HAO) IS A DIVISION OF THE NATIONAL") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   CENTER FOR ATMOSTPHERIC RESEARCH (NCAR), LOCATED IN BOULDER, COLORADO.") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   THE HAO CORONAGRAPH/POLARIMETER (C/P) TELESCOPE OPERATED ABOARD THE") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   NASA SOLAR MAXIMUM MISSION (SMM) SATELLITE, WHICH ORBITED THE EARTH") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   AT AN ALTITUDE OF ~500 KM FROM 14 FEB 1980 UNTIL 2 DECEMBER 1989.") ; 
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   THE HAO/SMM C/P TELESCOPE IS EXTERNALLY OCCULTED AND USES A") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   WESTINGHOUSE SEC VIDICON DETECTOR WITH 896x896 8-BIT SQUARE PIXELS") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   (HIGH RESOLUTION).  A FULL-VIEW, LOW RESOLUTION MODE YIELDS A") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   448x448 IMAGE BY PERFORMING A 2x2 PIXEL AVERAGE.                       ") ;
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   THE SECTOR MIRROR REFLECTS APPROXIMATELY ONE QUADRANT OF THE CORONA") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   AT A TIME ONTO THE VIDICON.  SECTOR POSITIONS ARE LABELED N, NE, E,") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   SE, S, SW, W, NW, AND INDICATE THE CORONAL REGION CONTAINED IN A") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   QUADRANT FOR THE NOMINAL SPACECRAFT ROLL ORIENTATION (0 DEGREES).") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   A POSITIVE ROLL MOVES THE CORONA COUNTER-CLOCKWISE (CCW).") ;
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   THE SPECTRAL FILTER WHEEL POSITIONS ARE: RED, GREEN, BLUE, WIDEBAND,") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   H-ALPHA, AND OPEN (WHICH IS OCCASIONALLY USED WITH A SEPARATE,") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   TILTABLE GREEN-LINE [530.3 NM = FE-XIV] FILTER).") ;
   strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   A CALIBRATION MIRROR IS USED TO VIEW EITHER A CALIBRATION RETICLE") ;
   sprintf (&fitbuf [iy++][0] , "COMMENT   TARGET OR THE SOLAR CORONA.  IMAGE ORIGIN: LOWER LEFT CORNER.") ;

   /*** Terminate FITS HDU. ***/

   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   iynum = iy ;
   nhdu = iynum / 36 ;
   if (nhdu * 36 != iynum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

printf ("iynum: %d  nhdu: %d\n", iynum, nhdu) ;

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nwrite = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;
   swrite += nwrite ;
printf ("swrite: %d\n", swrite) ;
   nwrite = 0 ;

   /*** Write pixel data. ***/
   /*** Reverse image top-to-bottom. ***/

   if (hduflag == 4)
      {
      nwrite  = fwrite (cpbuf,         sizeof (char), nbyhk, FF) ;
      nwrite  = fwrite (cpbuf + nbyhk, sizeof (char), nbyhk, FF) ;
      swrite += nwrite ;
printf ("swrite: %d\n", swrite) ;
      }
   else
   for (iy = 0 ; iy < ny ; iy++)
      {
      nwrite  = fwrite (imbuf + (ny - iy - 1) * nx * nbypx, nbypx, nx, FF) ;
      swrite += nwrite ;
      }

printf ("hduflag: %d  nx/ny: %3d %3d  nbypx: %d  nbyimg: %6d\n",
	hduflag, nx, ny, nbypx, nbyimg) ;
printf ("swrite: %d\n", swrite) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nwrite  = fwrite (padbuf, 1, npad, FF) ;
      swrite += nwrite ;
      free (padbuf) ;
      }

printf ("swrite: %d\n", swrite) ;

   fclose (FF) ;
   return (0) ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

month_day (year, doy, pmonth, pday)	/* Set month & day from day-of-year. */

int year, doy, *pmonth, *pday ;

{
   static int day_tab [2][13] =
      {
      {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      } ;
   int i, leap ;

   leap = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0) ;

   for (i = 1 ; doy > day_tab [leap][i] ; i++)
      doy -= day_tab [leap][i] ;

   *pmonth = i ;
   *pday = doy ;

   return (0) ;
}

