/*
 *+
 * NAME:
 *	CALDAT
 *
 * PURPOSE:
 *	Return the calendar date and time given Julian date.
 *	This is the inverse of the function JULDAY.
 * CATEGORY:
 *	Misc.
 *
 * CALLING SEQUENCE:
 *	caldat (Julian, Month, Day, Year, Hour, Minute, Second)
 *	See also: julday, the inverse of this function.
 *
 * INPUTS:
 *	JULIAN contains the Julian Day Number (which begins at noon) of the 
 *	specified calendar date.  It should be a long integer.
 * OUTPUTS:
 *	month:	Number of the desired month (1 = January, ..., 12 = December).
 *	day:	Number of day of the month.
 *	year:	Number of the desired year.
 *
 *	hour:	Hour of the day
 *	minute: Minute of the day
 *	second: Second (and fractions) of the day.
 *
 * RESTRICTIONS:
 *	Accuracy using IEEE double precision numbers is approximately
 *	1/10000th of a second.
 *
 * MODIFICATION HISTORY:
 *	Translated from "Numerical Recipies in C", by William H. Press,
 *	Brian P. Flannery, Saul A. Teukolsky, and William T. Vetterling.
 *	Cambridge University Press, 1988 (second printing).
 *
 *	DMS, July 1992.
 *	DMS, April 1996, Added HOUR, MINUTE and SECOND keyword
 *	AB, 7 December 1997, Generalized to handle array input.
 *-
 *------------------------------------------------------------------------------
 * [ALS] Andrew L. Stanger   HAO/NCAR
 *  3 Dec 2002: Conversion from IDL to C-language.
 *------------------------------------------------------------------------------
 */

#include <math.h>

int
caldat (julian, month, day, year, hour, minute, second)

long julian ;
int  *month ;
int  *day ;
int  *year ;
int  *hour ;
int  *minute ;
int  *second ;
{
    extern double floor () ;

/*
 *  compile_opt hidden
 *  ON_ERROR, 2				; Return to caller if errors
 */

  double f ;
  long igreg = 2299161L ;		/* Beginning of Gregorian calendar. */
  long ja ;
  long jb ;
  long jc ;
  long jd ;
  long je ;
  long jalpha ;
  long jul ;

  jul = (long) (julian + 0.5) ;		/* Better be long. */
  f   = julian + 0.5 - jul ;

  if (f != 0.0)				/* Get hours, minutes, seconds. */
    {
    *hour   = floor (f * 24.0) ;
    f = f - *hour / 24.0 ;
    *minute = floor (f * 1440) ;
    *second = (f - *minute / 1440.0) * 86400.0 ;
    }
  else
    {
    *hour   = 0L ;
    *minute = 0L ;
    *second = 0L ;
    }
   
   
  if (jul >= igreg)
    {
    jalpha = (long) (((jul - 1867216L) - 0.25) / 36524.25) ;
    ja = jul + 1 + jalpha - (long) (0.25 * jalpha) ;
    }
  else
    ja = jul ;

  jb = ja + 1524 ;
  jc = (long) (6680.0 + ((jb - 2439870L) - 122.1) / 365.25) ;
  jd = (long) (365 * jc + (0.25 * jc)) ;
  je = (long) ((jb - jd) / 30.6001) ;

  *day   = jb - jd - (long) (30.6001 * je) ;
  *month = je - 1 ;
  if (*month > 12)  *month = *month - 12 ;
  *year  = jc - 4715 ;
  if (*month >  2)  *year = *year - 1 ;
  if (*year  <= 0)  *year = *year - 1 ;

  return (0) ;
}
