/* carr_jd.c
 * -----------------------------------------------------------------------------
 * Convert Carrington Rotation & Latitude to modified Julian Date.
 *
 * Nick Arge   NOAA   20 Nov 2002   [carr_jd_fn.pro]
 * -----------------------------------------------------------------------------
 * Andrew Stanger   HAO/NCAR
 *  2 Dec 2002: Conversion to C-language. [carr_jd.c]
 * 20 Dec 2002: Add comments, change a few variable names.
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>

double
carr_jd (crot, clon)

double crot ;
double clon ;

{

    /* Convert from Carrington Longitude and Latitude to (Julian Date -2440000)
     * Modified Nov. 20 2002
     */

    double pi = 3.14159265358979323846264338327 ;
    double jd      = 0.0 ;
    double jd_mod  = 0.0 ;
    double jd_low  = 0.0 ;
    double jd_orig = 0.0 ;
    double carrot  = 0.0 ;
    double lonmax  = 0.0 ;
    double incl    = 7.25 ;
    double Bo      = 0.0 ;		/* Heliographic Latitude  */
    double Lo      = 0.0 ;		/* Heliographic Longitude */
    double nn ;				/* # days since epoch. */
    double incr ;			/* Day increment for loop. */
    double ww ;
    double LL ;
    double dayfr ;
    double gg ;
    double elong ;			/* Ecliptic Longitude */
    double omega ;
    double Bosin ;
    double Bocos ;
    double LoWsin ;
    double LoWcos ;
    double long_diff ;
    double LoW ;
    double rad_deg ;
    double deg_rad ;

    long MM ;
    int pmode = 0 ;
    
    rad_deg = pi / 180.0 ;
    deg_rad = 180.0 / pi ;

    if (pmode)
        printf ("carr_jd> crot: %7.2f  clon: %9.4f\n", crot, clon) ;

    if (clon == 0.0)
	{
        clon = 360.0 ;
        crot = crot + 1.0 ;
	}
    
    incl    = 7.25 ;
    jd_mod  = 0.0 ;
    jd_orig = 0.0 ;
    Lo      = 0.0 ;

    jd_mod  = crot + (360.0 - clon) / 360.0 ;	/* rotation + long. fraction */
    jd_mod  = jd_mod - 1750.0 ;			/* # rotations since C.R.1750 */
    jd_mod  = jd_mod * 27.27530 ;		/* # days since C.R.1750 */
    jd_mod  = jd_mod + 5871.91 ;		/* modified Julian date. */
    
    jd_orig = jd_mod ;
    jd_low  = jd_mod + 3.0 ;

    jd_mod  = jd_mod - 3.0 ;			/* Back up 3 days. */
    
    /*** For each of the dates labeling the bottom x-axis of the 1st plot. ***/
    
    incr = (double) (0.1) ;
    
    for (MM = 0; MM <= 10000; MM++)
	{
        jd_mod = jd_mod + incr ;

	/*
         * nn = JD - 2451545.0		[Julian date w.r.t. 2000 Jan 1 noon]
         * 2451545 - 2440000 = 11545	[jd_mod offset: 2000 Jan 1 12:00:00]
	 */
    
        nn = jd_mod - (double) 11545.0 ;   /* Days since 2000 Jan 1 12:00:00 */

        LL = 280.472 + 0.9856474 * nn ;	/* Mean longitude, abber.cor.*/
        LL = 280.460 + 0.9856474 * nn ;	/* Mean longitude, abber.cor.[Almanac]*/
        gg = 357.528 + 0.9856003 * nn ;	/* Mean anomaly. */
        elong = LL + 1.915 * sin (gg * rad_deg)
    	    + 0.020 * sin (2.0 * gg * rad_deg) ;	/* Ecliptic Longitude */
    
	/*** Longitude of solar equator's ascending node. ***/

	/*
         * Explains where the number 43243.25 comes from :
         * omega = 73.0 + 40.0 / 60.0 + 50.25 * ((jd_mod + 2440000.0 - 
         * 2396756.75) / 365.25) / 3600.0 ;
	 */

        omega = 73.0 + 40.0 / 60.0 +
                50.25 * ((jd_mod + 43243.25) / 365.25) / 3600.0 ;
    
       /*
        * Explains where the number 41780.0 comes from :
        * ww = 360.0 / 25.38 * (jd_mod + 2440000.0 - 2398220.0) ;
	*/
    
        ww = 360.0 / 25.38 * (jd_mod + 41780.0) ;
        ww = (ww / 360.0 - (long) (ww / 360.0)) * 360.0 ;
    

        Bosin = sin ((elong - omega) * rad_deg) * sin (incl * rad_deg) ;
        Bocos = sqrt (1.0 - Bosin * Bosin) ;
    
        Bo = atan2 (Bosin, Bocos) * deg_rad ;	/* Heliographic Latitude */
    

        LoWsin = cos (incl * rad_deg) * sin ((omega - elong) * rad_deg)
    	       / cos (Bo * rad_deg) ;
        LoWcos = -cos ((omega - elong) * rad_deg) / cos (Bo * rad_deg) ;
        LoW = atan2 (LoWsin, LoWcos) * deg_rad ;

        if (LoW < 0.0)  LoW = LoW + 360.0 ;
        Lo = LoW - ww ;				/* Heliographic Longitude */
    
        if (Lo < 0.0)  Lo = Lo + 360.0 ;
    
	long_diff = Lo - clon ;
	    
	if (pmode)
	    {
            printf ("incr: %7.2e  jd_mod: %9.4f  long_diff: %9.4f\n",
		     incr, jd_mod, long_diff) ;
	    printf ("carr_jd> Bo: %9.4f\n", Bo) ;
	    printf ("carr_jd> cos (Bo): %f\n",
		     cos (Bo * rad_deg));
	    printf ("LoWsin: %9.4f  LoWcos: %9.4f  LoW: %9.4f\n",
		     LoWsin, LoWcos, LoW) ;
            printf ("long_diff: %9.4f  Lo: %9.4f  clon: %9.4f  jd_mod: %9.4f\n",
		     long_diff, Lo, clon, jd_mod) ;
	    fflush (stdout) ;
	    }
    
        if ( (long_diff < 0.0) && (labs ((long) long_diff) < 100))
	    {
            jd_mod = jd_mod - incr ;
            incr = incr / 10.0 ;
            if (incr < 1.0e-6) goto DONE ;
	    }
    
        if ( (long_diff > 0.0) && (labs ((long) long_diff) > 100))
            { 
            jd_mod = jd_mod - incr ;
            incr = incr / 10.0 ;
            if (incr < 1.0e-6) goto DONE ;
            } 
        }
    
    DONE:
    
/*
 *    if (pmode)
 *	{
 *	jd = jd_mod + 2440000.0 ;
 *	caldat ((jd, &mmm, &ddd, &yyy, &hour, &minute, &second) ;
 *	}
 */

    dayfr = jd_mod - (long) (jd_mod) + 0.5 ;
    if (dayfr >= 1.0)  dayfr = dayfr - 1.0 ;
    
    return (jd_mod) ;
}
