/* crll.c
 *------------------------------------------------------------------------------
 * Use date to compute Carrington Rotation, Longitude & Latitude.
 *------------------------------------------------------------------------------
 * Functions called:
 *   jdate		Julian date
 *   carr_jd		Carrington Rotation --> Julian Date
 *   helio_long		Heliographic Longitude
 *   helio_lat		Heliographic Latitude
 *------------------------------------------------------------------------------
 * Derived from IDL function "jd_carr.pro",
 * Nick Arge   NOAA   20 Nov 2002
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR
 *  2 Dec 2002: Converted to C-language. Return rotation number. [jd_carr.c]
 * 20 Dec 2002: Also return longitude & latitude. [crll.c]
 *------------------------------------------------------------------------------
 */

#include <stdio.h>

int
crll (year, month, day, hour, minute, second, crot, clong, clat)

int year ;
int month ;
int day ;
int hour ;
int minute ;
int second ;

int    *crot ;
double *clong ;
double *clat ;

{
    extern double jdate () ;
    extern double carr_jd () ;
    extern double helio_lat () ;
    extern double helio_long () ;

    double carr_cm     = 0.0 ;
    double carr_increm = 0.0 ;
    double carr_test   = 0.0 ;
    double crdur       = 27.2753 ;		/* Average C.R. duration. */

    double jd          = 0.0 ;			/* Julian Date. */
    double jd_mod      = 0.0 ;			/* Julian Date. */
    double jd_cr0      = -41859.899931 ;	/* JD Offset for C.R. 0 */
    double jd_pred     = 0.0 ;
    double jd_diff     = 0.0 ;
    double jd_test     = 0.0 ;
    double jd_offset    = 2440000.0 ;		/* Julian Date offset. */

    double Bo          = 0.0 ;
    double Lo          = 0.0 ;

    double num_rot     = 0.0 ;
    double dayfr       = 0.0 ;

    int pmode = 0 ;			/* Print mode: 0=silent, 1=verbose. */

    carr_test   = 0.0 ;
    carr_increm = 0.0 ;

    /*** Compute the Julian date. ***/

    jd = jdate (year, month, day, hour, minute, second) ;

    /*** Use modified Julian date. ***/

    jd_mod = jd - jd_offset ;

    num_rot = (jd_mod - jd_cr0) / crdur ;   /* # rotations since ref. date. */

    if (pmode)
        printf ("num_rot: %7.2f\n", num_rot) ;

    if (num_rot >= 1000.0) carr_increm = 1000.0 ;
    else
    if (num_rot >=  100.0) carr_increm =  100.0 ;
    else
    if (num_rot >=   10.0) carr_increm =   10.0 ;
    else
    if (num_rot >=    1.0) carr_increm =    1.0 ;
    else
        carr_cm = 0.0 ;

    /*** Compute Julian date for rotation "carr_test". ***/

    carr_test = carr_increm ;
    jd_test = carr_jd (carr_test, (double) 0.0) ;

    if (pmode)
	{
        printf ("carr_increm: %9.4f\n", carr_increm) ;
        printf ("carr_test: %9.4f  jd_test: %9.4f\n", carr_test, jd_test) ;
	}

    while (jd_test <= jd_mod)
        {
        num_rot = (jd_mod - jd_test) / crdur ;

        if (num_rot >= 1000.0) carr_increm = 1000.0 ;
        else
        if (num_rot >=  100.0) carr_increm =  100.0 ;
        else
        if (num_rot >=   10.0) carr_increm =   10.0 ;
        else
        if (num_rot >=    0.0) carr_increm =    1.0 ;
  
        /*** Compute Julian date for rotation "carr_test". ***/

        carr_test = carr_test + carr_increm ;
        jd_test   = carr_jd (carr_test, (double) 0.0) ;

	if (pmode)
	    {
            printf ("num_rot: %7.2f\n", num_rot) ;
            printf ("carr_increm: %9.4f\n", carr_increm) ;
            printf ("carr_test: %9.4f  jd_test: %9.4f\n", carr_test, jd_test) ;
	    }
        }

    /*** Compute longitude & latitude. ***/

    Lo = helio_long (jd) ;
    Bo = helio_lat  (jd) ;

    jd_pred = carr_jd (carr_test, Lo) ;

    jd_diff = labs ((long) (jd_mod - jd_pred)) ;
    if (jd_diff > 20.0) 
	{
        if ((jd_mod - jd_pred) < 0.0)
	    carr_test = carr_test - 1.0 ;
	else
            carr_test = carr_test + 1.0 ;
        }

    /*** Convert Julian day to calendar date. ***/

    caldat ((long) jd, &month, &day, &year, &hour, &minute, &second) ;

    dayfr = jd_mod - (long) jd_mod + (double) 0.5 ;
    if (dayfr >= 1.0) dayfr = dayfr - 1.0 ;

    if (pmode)
	{
        printf ("jd_mod: %14.7f jd: %14.5f\n",
		 jd_mod, jd) ;
        printf ("month: %2d day: %2d year: %4d dayfr: %10.3f\n",
	         month, day, year, dayfr ) ;
        printf ("crot: %f  Lo: %f  Bo: %f\n", carr_test, Lo, Bo) ;
	}

    /*** Store values in output variables. ***/

    *crot = carr_test ;
    *clong = Lo ;
    *clat  = Bo ;

    return (0) ;
}
