/* fwint2d.c
 * -----------------------------------------------------------------------------
 * 2-dimensional linear interpolation.
 * -----------------------------------------------------------------------------
 * C language
 * Usage example :
 * -------------
 * #define xdim 1024		Example value: x dimension original image.
 * #define ydim 1024		Example value: y dimension original image.
 * #define xidim 512		Example value: x dimension output image.
 * #define yidim 512		Example value: y dimension output image.
 *
 * extern float *int2d () ;
 * float img    [ydim][xdim] ;
 * float ximg   [yidim][xidim] ;
 * float yimg   [yidim][xidim] ;
 * float imgint [yidim][xidim] ;
 * float xcen, ycen ;			Sun center in original image.
 * float rsunpix ;			Solar radius (pixels) in original image.
 * float rmax ;
 * int ix, iy ;
 * int minpix ;
 *
 * for (iy = 0 ; iy < yidim; iy++)
 *     for (ix = 0 ; ix < xidim; ix++)
 *         {
 *         ximg [iy][ix] = ix * 2.0 + 0.5 ;	Example ximg equation.
 *         yimg [iy][ix] = iy * 2.0 + 0.5 ;	Example yimg equation.
 *         }
 *
 * Example ouput image is 1/2 the size of original image.
 *
 * imgint = fwint2d (img, &xdim, &ydim, &xcen, &ycen, &rsunpix, &rmax,
 *                   imgint, ximg, yimg, &xidim, &yidim, &minpix, &bkg) ;
 *
 * -----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   10 Mar 2003
 * 12 Mar 2003 : Convert from function to subroutine.
 * 13 Mar 2003 : Adapt for IDL [CALL_EXTERNAL].
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>

/*
 * void
 * wint2d (img,                xdim,  ydim, xcen, ycen, rsunpix, rmax,
 *         imgint, ximg, yimg, xidim, yidim, minpix, bkg)
 */

void
fwint2d (argc, argv)

int argc ;
char *argv [] ;

{
    float  *img ;	/* Original image. */
    int    *xd ;	/* Original image x-dimension. */
    int    *yd ;	/* Original image y-dimension. */
    float  *xc ;	/* Original image x sun center. */
    float  *yc ;	/* Original image y sun center. */
    float  *rspix ;	/* Solar radius (pixels). */
    float  *radmax  ;	/* Maximum solar radius to use [Rsun]. */

    float  *imgint ;	/* Interpolated image. */
    float  *ximg ;	/* Array of x-axis coordinates for interpolation. */
    float  *yimg ;	/* Array of y-axis coordinates for interpolation. */
    int    *xid ;	/* New image x-dimension. */
    int    *yid ;	/* New image y-dimension. */
    int    *mpix ;	/* Minimum # pixels to use for interpolation. */
    float  *bkglev ;	/* Background intensity level to use. */

    int    xdim, ydim ;
    float  xcen, ycen ;
    float  rsunpix ;
    float  rmax ;
    int    xidim, yidim ;
    int    minpix ;
    float  bkg ;
    
    extern float fwintrb () ;
    float  xi, yi ;
    int    ix, iy ;

    /*** Get arguments from command line. ***/

    img    = (float *) argv [0] ;
    xd     = (int *)   argv [1] ;
    yd     = (int *)   argv [2] ;
    xc     = (float *) argv [3] ;
    yc     = (float *) argv [4] ;
    rspix  = (float *) argv [5] ;
    radmax = (float *) argv [6] ;
    imgint = (float *) argv [7] ;
    ximg   = (float *) argv [8] ;
    yimg   = (float *) argv [9] ;
    xid    = (int *)   argv [10] ;
    yid    = (int *)   argv [11] ;
    mpix   = (int *)   argv [12] ;
    bkglev = (float *) argv [13] ;

    xdim    = *xd ;
    ydim    = *yd ;
    xcen    = *xc ;
    ycen    = *yc ;
    rsunpix = *rspix ;
    rmax    = *radmax ;
    xidim   = *xid ;
    yidim   = *yid ;
    minpix  = *mpix ;
    bkg     = *bkglev ;
   
    printf ("fwint2d> xdim/ydim: %4d %4d\n", xdim, ydim) ;
    printf ("fwint2d> xcen/ycen: %8.3f %8.3f\n", xcen, ycen) ;
    printf ("fwint2d> rsunpix: %10.5f\n", rsunpix) ;
    printf ("fwint2d> rmax: %10.5f\n", rmax) ;
    printf ("fwint2d> minpix: %d\n", minpix) ;
    printf ("fwint2d> bkg: %8.3f\n", bkg) ;

    for (iy = 0; iy < yidim; iy++)
	{
	for (ix = 0; ix < xidim; ix++)
	    {
	    xi = *(ximg + ix + iy * xidim) ;
	    yi = *(yimg + ix + iy * xidim) ;
	    *(imgint + ix + iy * xidim) =
		fwintrb (img, xdim, ydim, xi, yi, xcen, ycen,
			 rsunpix, rmax, bkg, minpix) ;
	    }
	}
}
