/* fwintrb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Weighted nearest-neighbor interpolation : Shepard's method.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * img :	array containing image.
 * xdim/ydim:	image dimensions.
 * x,y:		x & y position in image to be interpolated.
 * xcen/ycen:	sun center coordinates [pixels].
 * rsun:	pixels/Rsun
 * rmax:	maxium radius to use [Rsun units]
 * bkg:		background intensity level.
 * numpix:	minimum number of pixels to use inside disc for interpolation.
 *
 * fwintrb returns a float precision value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float fwintdb () ;
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger   HAO/NCAR    6 Mar 2003
 *  6 Mar 2003 : Modified to NOT use a pixel beyond rsun.
 * 26 Mar 2003 : Add rmax & bkg parameters.
 * 27 Mar 2003 : Floaing point instead of double.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
fwintrb2 (img, xdim, ydim, x, y, xcen, ycen, rsun, rmax, bkg, minpix)

float  *img ;		/* Pointer to image array. */
int    xdim ;		/* X-axis dimension. */
int    ydim ;		/* Y-axis dimension. */
float  x, y ;		/* Coordinates of scan point in image. */
float  xcen ;		/* X-axis sun center (pixel units). */
float  ycen ;		/* Y-axis sun center (pixel units). */
float  rsun ;		/* Solar radius (pixel units). */
float  rmax ;		/* Maxium solar radius to use (Rsun units). */
float  bkg  ;		/* Background level to use. */
int    minpix ;		/* Min # pixels inside solar disc for interpolation. */

{
   int	  ix, iy ;
   int    pmode = 0 ;
   int    numpixdisc ;	/* # neighboring pixels with radial dist. < rsunlim. */
   float  b00, b10, b01, b11 ;
   float  fx, fy ;
   float  f00, f10, f01, f11 ;
   float  fx0, fx1, fxy ;
   float  h00, h10, h01, h11 ;
   float  hsum ;
   float  invhsq00, invhsq10, invhsq01, invhsq11 ;
   float  r00, r10, r01, r11 ;
   float  rsunlim ;
   float  sx00, sx10, sx01, sx11 ;
   float  sy00, sy10, sy01, sy11 ;
   float  w00, w10, w01, w11 ;
   float  x00, x10, x01, x11 ;
   float  y00, y10, y01, y11 ;
  
   if (pmode)
      {
      fprintf (stdout , "\n") ;
      fprintf (stdout, "fwintrb> xdim/ydim: %4d %4d  x/y: %7.2f %7.2f\n",
	       xdim, ydim, x, y) ;
      fprintf (stdout, "fwintrb> xcen/ycen: %7.2f %7.2f  rsun: %7.2f\n",
	       xcen, ycen, rsun) ;
      fprintf (stdout, "fwintrb> rmax: %5.2f  bkg: %8.2e  minpix: %d\n",
	       rmax, bkg, minpix) ;
      } 

   if (x < 0.0 || x > (float)  xdim)
      {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
      return (bkg) ;
      }

   if (y < 0.0 || y > (float)  ydim)
      {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
      return (bkg) ;
      }

   /*** X/Y coordinates of lower left corner pixel. ***/

   ix = (int) x ;
   iy = (int) y ;

   /*** Fractional distance of point from lower left corner neighbor. ***/

   fx = x - (float)  ix ;
   fy = y - (float)  iy ;

   if (pmode)
      fprintf (stdout, "fx: %9.4f  fy: %9.4f\n", fx, fy) ;

   /*** Intensity at each neighboring pixel. ***/

   b00 = *(img +  ix    +  iy    * xdim) ;
   b10 = *(img + (ix+1) +  iy    * xdim) ;
   b01 = *(img +  ix    + (iy+1) * xdim) ;
   b11 = *(img + (ix+1) + (iy+1) * xdim) ;

/*
   if (b00 != 0.0 || b10 != 0.0 || b10 != 0.0 || b11 != 0.0)
       fprintf (stdout, "b00: %9.4f  b10: %9.4f  b01: %9.4f  b11: %9.4f\n",
	        b00, b10, b01, b11) ;
 */

   /*** Distance of each neighbor pixel from sun center. ***/

   sx00 = (float)   ix      - xcen ;
   sy00 = (float)   iy      - ycen ;

   sx10 = (float)  (ix + 1) - xcen ;
   sy10 = (float)   iy      - ycen ;

   sx01 = (float)   ix      - xcen ;
   sy01 = (float)  (iy + 1) - ycen ;

   sx11 = (float)  (ix + 1) - xcen ;
   sy11 = (float)  (iy + 1) - ycen ;


   r00 = sqrt ( sx00 * sx00 + sy00 * sy00 ) ;
   r10 = sqrt ( sx10 * sx10 + sy10 * sy10 ) ;
   r01 = sqrt ( sx01 * sx01 + sy01 * sy01 ) ;
   r11 = sqrt ( sx11 * sx11 + sy11 * sy11 ) ;

   if (pmode && iy == ydim / 2)
       {
       printf ("ix/iy: %4d %4d  r00/r10/r01/r11: %7.2f  %7.2f  %7.2f  %7.2f\n",
	       ix, iy, r00, r10, r01, r11) ;
       }

   /*** Distances of each neighbor from interpolated point. ***/

   x00 = x - (float)   ix ;
   y00 = y - (float)   iy ;

   x10 = x - (float)  (ix + 1) ;
   y10 = y - (float)   iy ;

   x01 = x - (float)   ix ;
   y01 = y - (float)  (iy + 1) ;

   x11 = x - (float)  (ix + 1) ;
   y11 = y - (float)  (iy + 1) ;

   h00 = sqrt ( x00 * x00 + y00 * y00 ) ;
   h10 = sqrt ( x10 * x10 + y10 * y10 ) ;
   h01 = sqrt ( x01 * x01 + y01 * y01 ) ;
   h11 = sqrt ( x11 * x11 + y11 * y11 ) ;

   if (pmode)
   {
   fprintf (stdout, "x00: %9.4f  x10: %9.4f  x01: %9.4f  x11: %9.4f\n",
	    x00, x10, x01, x11) ;
   fprintf (stdout, "y00: %9.4f  y10: %9.4f  y01: %9.4f  y11: %9.4f\n",
	    y00, y10, y01, y11) ;
	    
   fprintf (stdout, "h00: %9.4f  h10: %9.4f  h01: %9.4f  h11: %9.4f\n",
	    h00, h10, h01, h11) ;
   }

   /*** Compute inverse square distances. ***/

   invhsq00 = 1.0 / (h00 * h00) ;
   invhsq10 = 1.0 / (h10 * h10) ;
   invhsq01 = 1.0 / (h01 * h01) ;
   invhsq11 = 1.0 / (h11 * h11) ;

   if (h00 == 0.0)  invhsq00 = 1.0 ;
   if (h10 == 0.0)  invhsq10 = 1.0 ;
   if (h01 == 0.0)  invhsq01 = 1.0 ;
   if (h11 == 0.0)  invhsq11 = 1.0 ;

   if (pmode)
      fprintf (stdout, "invhsq00: %9.4f  invhsq10: %9.4f  invhsq01: %9.4f  invhsq11: %9.4f\n",
	    invhsq00, invhsq10, invhsq01, invhsq11) ;

   /*** For each point with a radial distance exceeding rsun,
    *** set the inverse square distance to zero.
    ***/

   numpixdisc = 4 ;
   rsunlim = rsun * rmax ;			/* radius limit [pixels] */

   if (r00 > rsunlim)  {invhsq00 = 0.0 ;  numpixdisc-- ;}
   if (r10 > rsunlim)  {invhsq10 = 0.0 ;  numpixdisc-- ;}
   if (r01 > rsunlim)  {invhsq01 = 0.0 ;  numpixdisc-- ;}
   if (r11 > rsunlim)  {invhsq11 = 0.0 ;  numpixdisc-- ;}

/*
 *   if ( b00 == bkg )  {invhsq00 = 0.0 ;  numpixdisc-- ;} 
 *   if ( b10 == bkg )  {invhsq10 = 0.0 ;  numpixdisc-- ;} 
 *   if ( b10 == bkg )  {invhsq01 = 0.0 ;  numpixdisc-- ;} 
 *   if ( b11 == bkg )  {invhsq11 = 0.0 ;  numpixdisc-- ;} 
 */

   /*** Compute weights for each neighboring pixel. ***/

/*
 *   hsum = 1.0 / h00*h00 + 1.0 / h10*h10 + 1.0 / h01*h01 + 1.0 / h11*h11
 *
 *   w00 = (1.0 / (h00 * h00) ) / hsum ;
 *   w10 = (1.0 / (h10 * h10) ) / hsum ;
 *   w01 = (1.0 / (h01 * h01) ) / hsum ;
 *   w11 = (1.0 / (h11 * h11) ) / hsum ;
 */

   hsum = invhsq00 + invhsq10 + invhsq01 + invhsq11 ;

   /*** Return background intensity level if the number of
    *** nearest neighbor pixels "on" the disc is less than "minpix".
    ***/

   if (hsum == 0.0 || numpixdisc < minpix)  return (bkg) ;

   /*** Compute nearest neighbor pixel weights. ***/

   w00 = invhsq00 / hsum ;
   w10 = invhsq10 / hsum ;
   w01 = invhsq01 / hsum ;
   w11 = invhsq11 / hsum ;

   if (pmode)
      {
      fprintf (stdout, "hsum: %9.4f\n", hsum) ;
      fprintf (stdout, "w00: %9.4f  w10: %9.4f  w01: %9.4f  w11: %9.4f\n",
	       w00, w10, w01, w11) ;
      }

   /*** Select 4 nearest neighbor elements in 'img' array. ***/

   f00 = w00 * b00 ;
   f10 = w10 * b10 ;
   f01 = w01 * b01 ;
   f11 = w11 * b11 ;

   /*** Shepard's inverse distance weighted interpolation.
    *** Simply add up the weighted intensities of the neighboring pixels.
    ***/

   fxy = f00 + f10 + f01 + f11 ;

   if (pmode)
      printf ("x/y: %8.3f %8.3f  fxy: %8.2e\n", x, y, fxy) ;

   if (fxy > 250.001)
      printf ("x/y: %8.3f %8.3f  fxy: %8.2e\n", x, y, fxy) ;

   return (fxy) ;
}
