/* hblint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Heliographic Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * blint returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float blint ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, or floating-point images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 29 March 2000: Add nbypix parameter (instead of include statement).
 *                Add xdim & ydim parameters.
 *                Replace "npixl" with "xdim".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
hblint (image, nbypix, xdim, ydim, xcen, ycen, pixrs, rmax, x, y)

char	*image;		/* Pointer to image array. */
int	nbypix ;	/* # bytes per pixel. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
float	xcen ;		/* X-axis sun center. */
float	ycen ;		/* Y-axis sun center. */
float	pixrs ;		/* # pixels per solar radius */
float	rmax ;		/* Maxiumum radius allowed [Rsun units]. */
float	x, y;		/* Coordinates of scan point in image. */

{
register char	*im;
register short	*ims;
register float	*imf;
int	ix, iy;
float	fx, fy;
float	f00, f10, f01, f11;
float	fx0, fx1, fxy;

    if (x < 0 || x > xdim)
       {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
       return (-1.0) ;
       }
 
    if (y < 0 || y > ydim)
       {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
       return (-2.0) ;
       }
 
    im  = (char *) image;
    ims = (short *) image;
    imf = (float *) image;
 
    ix = (int) x;
    iy = (int) y;
 
    /*** Define pixel coordinates for 4 nearest neighbors. ***/
 
    x0 = (float) ix ;
    y0 = (float) iy ;
    x1 = x0 + 1.0 ;
    x2 = y0 + 1.0 ;
 
    /*** Distance from sun center (along each axis). ***/
 
    dx0 = x0 - xcen ;
    dx1 = x1 - xcen ;
    dy0 = y0 - ycen ;
    dy1 = y1 - ycen ;
 
    /*** Radial distance (pixel units) from sun center
     *** for each neighboring pixel.
     ***/
 
    rp00 = sqrt (dx0*dx0 + dy0*dy0) ;
    rp10 = sqrt (dx1*dx1 + dy0*dy0) ;
    rp01 = sqrt (dx0*dx0 + dy1*dy1) ;
    rp11 = sqrt (dx1*dx1 + dy1*dy1) ;
 
    /*** Radial distance (Rsun units) from sun center
     *** for each neighboring pixel.
     ***/
 
    rs00 = rp00 / pixrs ;
    rs10 = rp10 / pixrs ;
    rs01 = rp01 / pixrs ;
    rs11 = rp11 / pixrs ;
 
    fx = x - (float) ix;
    fy = y - (float) iy;
 
    /*** Select 4 nearest neighbor elements in 'image' array. ***/
 
    /*** 1 byte per pixel. ***/
 
    if (nbypix == 1)
       {
       f00 = (float) (*(im +  ix    +  iy*xdim) & 0377);
       f10 = (float) (*(im + (ix+1) +  iy*xdim) & 0377);
       f01 = (float) (*(im +  ix    + (iy+1)*xdim) & 0377);
       f11 = (float) (*(im + (ix+1) + (iy+1)*xdim) & 0377);
       }
    else
 
    /*** 2 bytes per pixel (short integer). ***/
 
    if (nbypix == 2)
       {
       f00 = (float) *(ims +  ix    +  iy*xdim);
       f10 = (float) *(ims + (ix+1) +  iy*xdim);
       f01 = (float) *(ims +  ix    + (iy+1)*xdim);
       f11 = (float) *(ims + (ix+1) + (iy+1)*xdim);
       }
    else
 
    /*** 4 bytes per pixel (floating point). ***/
 
    if (nbypix == 4)
       {
       f00 = (float) *(imf +  ix    +  iy*xdim);
       f10 = (float) *(imf + (ix+1) +  iy*xdim);
       f01 = (float) *(imf +  ix    + (iy+1)*xdim);
       f11 = (float) *(imf + (ix+1) + (iy+1)*xdim);
       }
 
    /*** Bilinear interpolation. ***/
 
    fx0 = f00 + fx * (f10 - f00);
    fx1 = f01 + fx * (f11 - f01);
    fxy = fx0 + fy * (fx1 - fx0);
 
    /*** If any of the nearest neighbor pixels fall outside
     *** of the rmax distance, do only linear interpolation.
     ***/

    if (rs00 > rmax && rs10 > rmax)
	{
	if (rs01 > rmax)	fxy = f11 ;
	else
	if (rs11 > rmax)	fxy = f01 ;
	else
				fxy = fx1 ;
	}

    if (rs00 > rmax && rs01 > rmax)
	{
	if (rs10 > rmax)	fxy = f11 ;
	else
	if (rs11 > rmax)	fxy = f10 ;
	else
				fxy = fy1 ;
	}

    if (rs00 > rmax && rs11 > rmax)
	{
	if (rs10 > rmax)	fxy = f01 ;
	else
	if (rs01 > rmax)	fxy = f10 ;
	}

	if (rs10 > rmax && rs01 > rmax)
	{
	if (rs00 > rmax)	fxy = f11 ;
	else
	if (rs11 > rmax)	fxy = f00 ;
	}

	if (rs10 > rmax && rs11 > rmax)
	{
	if (rs00 > rmax)	fxy = f01 ;
	else
	if (rs01 > rmax)	fxy = f00 ;
	else
				fxy = fy0 ;
	}

    if (rs01 > rmax && rs11 > rmax)
	{
	if (rs00 > rmax)	fxy = f10 ;
	else
	if (rs10 > rmax)	fxy = f00 ;
	else
				fxy = fx0 ;
	}

    if (rs00 > rmax)
	{
	if (rs10 < rmax && rs01 < rmax && rs11 < rmax)
	    fxy = f01 + z0110 * (f10 - f01) ;

	}

    return (fxy);
}
