/* hblintd.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Heliographic Bilinear interpolation.
 * Modified interpolation scheme is used whenever 1, 2, or 3 of the
 * nearest neighbor points lie outside of the radial distance "rmax"
 * from sun center [xcen, ycen].
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to image array.
 *         Use "char" type irregardless of the actual type (char, short, float).
 * x,y:    double precision values of the x & y position in image array.
 * hblintd returns a double precision value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    double hblintd () ;
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept char, short integer, or floating-point images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 29 March 2000: Add nbypix parameter (instead of include statement).
 *                Add xdim & ydim parameters.
 *                Replace "npixl" with "xdim".
 * 12 Feb 2003 : Modified interpolation for heliographic transformation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

double
hblintd (img, nbypix, xdim, ydim, xcen, ycen, pixrs, rmax, x, y, mod)

char	*img ;		/* Pointer to image array (always cast as type char). */
int	nbypix ;	/* # bytes per pixel: 1=char, 2=short, 4=float. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
double	xcen ;		/* X-axis sun center. */
double	ycen ;		/* Y-axis sun center. */
double	pixrs ;		/* # pixels per solar radius */
double	rmax ;		/* Maxiumum radius allowed [Rsun units]. */
double	x, y ;		/* Coordinates of point in image. */
int     mod  ;		/* modified interpolation ? =0 no, =1 yes. */

{
register char	*imc ;		/* pointer to char   image array. */
register short	*ims ;		/* pointer to short  image array. */
register float	*imf ;		/* pointer to float  image array. */
register double *imd ;		/* pointer to double image array. */

int	ix, iy ;		/* Integral components of x & y position. */
int	sum ;			/* Value for switch statement. */

double	fx, fy ;		/* Fractional distance in x & y from pixel 00 */
double  fd, fy01 ;		/* fractional distance along diagonal. */
double	m00, m10, m01, m11 ;	/* Pixel magnitudes of nearest neighbors. */
double	fx0, fx1 ;		/* Linearly interpolated values along y0 & y1 */
double  fxy ;			/* Interpolated magnitude. */

double	px0, px1 ;		/* X-axis distance from sun center. */
double  py0, py1 ;		/* Y-axis distance from sun center. */
double rp00, rp10, rp01, rp11 ;	/* radial distance [pixels] from sun center. */
double rs00, rs10, rs01, rs11 ;	/* radial distance [Rsun]  from sun center. */
double  x0, y0 ;		/* ix, iy     in double precision. */
double  x1, y1 ;		/* ix+1, iy+1 in double precision. */

    if (x < 0 || x > xdim)
       {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
       return (-1.0) ;
       }
 
    if (y < 0 || y > ydim)
       {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
       return (-2.0) ;
       }
 
    /*** Get integral components of x & y position. ***/

    ix = (int) x ;
    iy = (int) y ;
 
    /* Compute fractional distance between pixels. ***/

    fx = x - (double) ix ;
    fy = y - (double) iy ;
 
    /*** Define pixel coordinates for 4 nearest neighbors. ***/
 
    x0 = (double) ix ;
    y0 = (double) iy ;
    x1 = x0 + 1.0 ;
    y1 = y0 + 1.0 ;
 
    /*** Distance from sun center (along each axis). ***/
 
    px0 = x0 - xcen ;
    px1 = x1 - xcen ;
    py0 = y0 - ycen ;
    py1 = y1 - ycen ;
 
    /*** Radial distance (pixel units) from sun center
     *** for each neighboring pixel.
     ***/
 
    rp00 = sqrt (px0*px0 + py0*py0) ;
    rp10 = sqrt (px1*px1 + py0*py0) ;
    rp01 = sqrt (px0*px0 + py1*py1) ;
    rp11 = sqrt (px1*px1 + py1*py1) ;
 
    /*** Radial distance (Rsun units) from sun center
     *** for each neighboring pixel.
     ***/
 
    rs00 = rp00 / pixrs ;
    rs10 = rp10 / pixrs ;
    rs01 = rp01 / pixrs ;
    rs11 = rp11 / pixrs ;
 
    /*** Select 4 nearest neighbor elements in 'img' array. ***/
 
    /*** 1 byte per pixel. ***/
 
    if (nbypix == 1)
       {
       imc = (char *)   img ;
       m00 = (double) (*(imc +  ix    +  iy    * xdim) & 0377) ;
       m10 = (double) (*(imc + (ix+1) +  iy    * xdim) & 0377) ;
       m01 = (double) (*(imc +  ix    + (iy+1) * xdim) & 0377) ;
       m11 = (double) (*(imc + (ix+1) + (iy+1) * xdim) & 0377) ;
       }
    else
 
    /*** 2 bytes per pixel (short integer). ***/
 
    if (nbypix == 2)
       {
       ims = (short *)  img ;
       m00 = (double) *(ims +  ix    +  iy    * xdim) ;
       m10 = (double) *(ims + (ix+1) +  iy    * xdim) ;
       m01 = (double) *(ims +  ix    + (iy+1) * xdim) ;
       m11 = (double) *(ims + (ix+1) + (iy+1) * xdim) ;
       }
    else
 
    /*** 4 bytes per pixel (floating point). ***/
 
    if (nbypix == 4)
       {
       imf = (float *)  img ;
       m00 = (double) *(imf +  ix    +  iy    * xdim) ;
       m10 = (double) *(imf + (ix+1) +  iy    * xdim) ;
       m01 = (double) *(imf +  ix    + (iy+1) * xdim) ;
       m11 = (double) *(imf + (ix+1) + (iy+1) * xdim) ;
       }
 
    /*** 8 bytes per pixel (floating point). ***/
 
    if (nbypix == 8)
       {
       imd = (double *) img ;
       m00 = (double) *(imd +  ix    +  iy    * xdim) ;
       m10 = (double) *(imd + (ix+1) +  iy    * xdim) ;
       m01 = (double) *(imd +  ix    + (iy+1) * xdim) ;
       m11 = (double) *(imd + (ix+1) + (iy+1) * xdim) ;
       }
 
    /*** Bilinear interpolation. ***/
 
    fx0 = m00 + fx * (m10 - m00) ;
    fx1 = m01 + fx * (m11 - m01) ;
    fxy = fx0 + fy * (fx1 - fx0) ;
 
    if (mod == 0)
       return (fxy) ;

    printf ("mod: %d\n", mod) ;

    /*** If any of the nearest neighbor pixels fall outside
     *** of the rmax distance, do only linear interpolation.
     ***/

    /*** For each nearest neighbor pixel, determine whether it
     *** its distance is greater than rmax.
     *** Assign a prime number to each pixel whose distance from center
     *** exceeds rmax.  Otherwise, assign the value zero.
     *** Compute the sum of these 4 values to determine the ID
     *** to be used in doing a modified "interpolation".
     *** In cases where only 1 neighbor's distance is less than rmax,
     *** simply use the intensity of this pixel.
     *** For the case of 2 neighbors' distance being less than rmax,
     *** Do linear interpolation using these two pixels.
     *** For the situation of 3 neighbors' distance being less than rmax,
     *** do linear interpolation between diagonal neighbors.
     ***/

    sum = 0 ;
    if (rs00 > rmax) sum += 1 ;
    if (rs10 > rmax) sum += 3 ;
    if (rs01 > rmax) sum += 5 ;
    if (rs11 > rmax) sum += 7 ;

    if (sum == 16)
        printf ("x/y: %7.2f %7.2f  rp00: %7.2f  rs00: %7.2f  sum: %d\n",
		 x, y, rp00, rs00, sum) ;

    switch (sum)
        {
        case 1:					/* 00		beyond rmax */
		fy01 = 1.0 - fy ;
		fd = sqrt (fx*fx + fy01*fy01) ;
		fxy = m01 + fd * (m10 - m01) ;			break ;
        case 3:					/*    10	beyond rmax */
		fd = sqrt (fx*fx + fy*fy) ;
		fxy = m00 + fd * (m11 - m00) ;			break ;
        case 5:					/*       01	beyond rmax */
		fd = sqrt (fx*fx + fy*fy) ;
		fxy = m00 + fd * (m11 - m00) ;			break ;
        case 7:					/*          11	beyond rmax */ 
		fy01 = 1.0 - fy ;
		fd = sqrt (fx*fx + fy01*fy01) ;
		fxy = m01 + fd * (m10 - m01) ;			break ;
        case 4:					/* 00 10	beyond rmax */
		fxy = m01 + fx * (m11 - m01) ;			break ;
        case 6:					/* 00    01	beyond rmax */
		fxy = m10 + fy * (m11 - m10) ;			break ;
        case 10:				/*    10    11	beyond rmax */
		fxy = m00 + fy * (m01 - m00) ;			break ;
        case 12:				/*       01 11	beyond rmax */
		fxy = m00 + fx * (m10 - m00) ;			break ;
        case 9:					/* 00 10 01	beyond rmax */
		fxy = m11 ;					break ;
        case 11:				/* 00 10    11	beyond rmax */
		fxy = m01 ;					break ;
        case 13:				/* 00    01 11	beyond rmax */
		fxy = m10 ;					break ;
        case 15:				/*    10 01 11	beyond rmax */
		fxy = m00 ;	break ;
        case 16:				/* all		beyond rmax */
		fxy = 0.0 ;	break ;
        case 0:					/* none		beyond rmax */
		break ;
       }

    return (fxy) ;
}
