/* helio_long.c
 *------------------------------------------------------------------------------
 * PURPOSE	Convert Julian Date to Heliographic Longitude.
 *
 * AUTHOR	Nick Arge   NOAA   20 Nov 2002
 *
 * HISTORY	Modified April 27, 2001
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR    3 Dec 2002
 *  3 Dec 2002: Conversion from IDL to C-language.
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>

/*** Converts from (Julian date - 2440000) to Helographic longitude ***/

double
helio_long (JD)

double JD ;			/* Julian day. */
{
  double pi     = 3.14159265358979323846264338327 ;
  double incl   = 7.25 ;
  double JD_mod = 0.0 ;
  double JD_off = 2440000.0 ;
  double nn     = 0.0 ;
  double LL     = 0.0 ;
  double gg     = 0.0 ;
  double elong  = 0.0 ;		/* Ecliptic Longitude. */
  double omega  = 0.0 ;
  double ww     = 0.0 ;
  double Bosin  = 0.0 ;
  double Bocos  = 0.0 ;
  double LoW    = 0.0 ;
  double LoWsin = 0.0 ;
  double LoWcos = 0.0 ;
  double Bo     = 0.0 ;
  double Lo     = 0.0 ;

  double rad_deg ;
  double deg_rad ;

  rad_deg = pi / 180.0 ;
  deg_rad = 180.0 / pi ;

  JD_mod = JD - JD_off ;		/* Modified Julian Date. */

/*
 * nn = JD - 2451545.0
 * 2451545 - 2440000 = 11545		[offset for Epoch 2000]
 */

  nn = JD_mod - (double) (11545.0) ;		/* #days since epoch 2000 */
  LL = 280.460 + 0.9856474 * nn ;		/* Mean Longitude [Almanac]. */
  LL = 280.472 + 0.9856474 * nn ;		/* Mean Longitude. */
  gg = 357.528 + 0.9856003 * nn ;		/* Mean Anomaly. */
  elong = LL + 1.915 * sin (gg * rad_deg)
      + 0.020 * sin (2.0 * gg * rad_deg) ;	/* Ecliptic Longitude. */

/*
 * Explains where the number 43243.25 comes from:
 * omega = 73.0 + 40.0 / 60.0 
 *       + 50.25 * (JD_mod + 2440000.0 - 2396756.75) / 365.25) / 3600.0
 */

  omega = 73.0 + 40.0 / 60.0
	+ 50.25 * ((JD_mod + 43243.25) / 365.25) / 3600.0 ;

/*
 * Explains where the number 41780.0 comes from :
 * ww = 360.0 / 25.38 * (JD_mod + 2440000.0 - 2398220.0)
 */

  ww = 360.0 / 25.38 * (JD_mod + 41780.0) ;
  ww = (ww / 360.0 - (long) (ww / 360.0)) * 360.0 ;

  Bosin = sin ((elong - omega) * rad_deg) * sin (incl * rad_deg) ;
  Bocos = sqrt (1.0 - Bosin * Bosin) ;

  Bo = atan2 (Bosin, Bocos) * deg_rad ;

  LoWsin = cos (incl * rad_deg) * sin ((omega - elong) * rad_deg)
	 / cos (Bo * rad_deg) ;
  LoWcos = -cos ((omega - elong) * rad_deg) / cos (Bo * rad_deg) ;
  LoW = atan2 (LoWsin, LoWcos) * deg_rad ;

  if (LoW < 0.0)  LoW = LoW + 360.0 ;
  Lo = LoW - ww ;

  if (Lo  < 0.0)  Lo  = Lo  + 360.0 ;

  return (Lo) ;
}
