
/* -------------------------------------------------------------------------- */

/*
 *  obs2helio.c                         ~soi/(version)/src/modules/obs2helio.c
 *
 *  Purpose:
 *     Interpolate CCD data to estimate value that would be obtained
 *     at specified equal incrememts of heliographic longitude
 *     and sine of latitude.
 *
 *  Description:
 *
 *     Employs bi-linear or cubic convolution interpolation to map a
 *     portion of the solar disk to a rectangular array "cols" by "rows" in
 *     width and height respectively.
 *
 *     Adapted by Kay Leibrand from pipeLNU remap from proto.d remap which was 
 *     modified by PHS from Lyle Bacon version.
 *     Added transformations to handle pixel scaling, image orientation, and
 *     stretching (in process) as defined by Jesper Schou.
 *     Added choice of velocity correction and interpolation methods.
 *
 *  Reference:
 *
 *     Green, Robin M. "Spherical Astronomy," Cambridge University
 *     Press, Cambridge, 1985.
 *
 *  Responsible:  Kay Leibrand                   KLeibrand@solar.Stanford.EDU
 *
 *  Restrictions:
 *     The number of output map rows must be even.
 *     The number of output map columns must be less than MAXCOLS.
 *     Assumes orientation is a 4 character string with one of the following
 *     8 legal values: SESW SENE SWSE SWNW NWNE NWSW NENW NESE 
 *
 *  Bugs:
 *     Possible division by 0 
 *
 *  Planned updates:
 *     Optimize.
 *
 *  Revision history is at end of file.
 */

/* -------------------------------------------------------------------------- */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <soi_fun.h>
#include <soi_util.h>
#include <soi_error.h>

#define MAXCOLS	(3072)

double ccintd(double *f, int nx, int ny, double x, double y);
double linintd (double *f, int nx, int ny, double x, double y);

int obs2helio (

float	*V,		/* input velocity array				*/
			/* assumed declared V[ypixels][xpixels]		*/
float	*U,		/* output projected array			*/

int	xpixels,	/* x width of input array			*/
int	ypixels,	/* y width of input array			*/
double  x0, 	   	/* x pixel address of disk center		*/
double  y0, 	   	/* y pixel address of disk center		*/
double  B0,		/* heliographic latitude of disk center		*/
double  P,		/* angle between CCD y-axis and solar vertical	*/
			/* positive to the east (CCW)			*/
double  S,		/* subtended radius of the sun	radians    	*/
double  rsun,		/* pixels */
double  Rmax,		/* maximum disk radius to use (e.g. 0.95)	*/

int	interpolation,	/* option */
int	cols,		/* width of output array			*/
int	rows,		/* height of output array			*/
double  Lmin,		/* start of longitude range 			*/
double  Ldelta,		/* increment of longitude per col	        */
double 	Ladjust,        
double  sinBdelta,	/* increment of sin latitude per row          */

double	smajor,
double	sminor,
double	sangle,
double  xscale,
double  yscale,
char    *orientation,

int	mag_correction,		/* option */
int	velocity_correction,	/* option */
double  obs_vr,
double  obs_vw,
double  obs_vn,
double  vsign,
int	NaN_beyond_rmax)

{

double xtmp, ytmp;
float u;			/* output temp				*/

/* Variables for coordinate projection of desired point(L,B) on disk	*/

double sinBbase;  		/* base value to calculate sinB         */
double sinB0, cosB0;		/* Latitude center of disk components	*/
double sinB, cosB;		/* Latitude desired point components	*/
double sinL, cosL;		/* Longitude desired point components	*/
double sinP, cosP;		/* Position angle of N wrt CCD y-axis	*/
double sinPe, cosPe;		/* Position angle of semimajor axis	*/
double sinrho, cosrho;		/* Central angle to desired point comps */
double sinp, cosp;		/* Position angle of point wrt N on sun */
double sinp_sinrho;		/* N-S projection on disk		*/
double cosp_sinrho;		/* W-E projection on disk		*/
double rp;			/* projected radial distance correction */
double x, y;			/* CCD location of desired point	*/
double Rmax2;			/* Max disk radius to consider - squared*/

/* Variables for "stretch" transformation matrix */

double sinS, cosS, sinS2, cosS2, sinScosS;
double stretch_xx, stretch_xy, stretch_yx, stretch_yy;

/* Variables for "orientation" transformation matrix */

int orient_xx, orient_xy, orient_yx, orient_yy;
int swap, ewflip, nsflip;

/* Variables for "combined" transformation matrix */

double combo_xx, combo_xy, combo_yx, combo_yy;

/*  Variables to determine (L,B) of desired point			*/

int row, col;			/* index into output array		*/
double L;			/* Longitude of desired point		*/

/*  Function for interpolation */

float (*interpolate)();

/*  Variables to determine velocity correction				*/

double mu;
double v_rotation;
double v_limbshift;

/*  Constants used in velocity correction				*/

const double v_lat	       = 0.0;
const double v_equator         = 1974.1;
const double rotation_coef_a2  = -0.121;
const double rotation_coef_a4  = -0.178;
const double limbshift_coef_a0 = -166.;
const double limbshift_coef_a1 = -139.;
const double limbshift_coef_a2 = 700.;

/*  Variables for optimization  */

double savedL[MAXCOLS];
double savedsinL[MAXCOLS];
double savedcosL[MAXCOLS];
double cB0sB, cB0cB, sB0sB, sB0cB;
double sinB2;
int rowoffset;

long i;
double *vdp;
float *vp;

double *VD;
VD  = (double *) (malloc (xpixels * ypixels * sizeof (double)));
vdp = VD;
vp  = V;
for (i = 0; i < xpixels*ypixels; i++) *vdp++ = (double) *vp++;

if (cols>MAXCOLS) return DIMENSION_MISMATCH;
if (rows%2) return CANT_DO_ODD_NUM_LATS;

if (interpolation == CUBIC_CONVOLUTION) interpolate = ccint2;
else if (interpolation == BILINEAR) interpolate = linint2;
else return UNSUPPORTED_INTERPOLATION; 

if ((mag_correction < MCOR_LEVEL0) || (mag_correction > MCOR_LEVEL1))
return UNSUPPORTED_M_CORRECTION;

if ((velocity_correction < VCOR_SIGN_ONLY) ||
    (velocity_correction > VCOR_LEVEL2))
    return UNSUPPORTED_V_CORRECTION;

/* Calculate quantities that are constant for this image		*/

sinBbase = 0.5 - rows / 2;
sinS  = sin (sangle); cosS  = cos (sangle); 
sinS2 = sinS * sinS;  cosS2 = cosS * cosS; sinScosS = sinS * cosS;
stretch_xx = smajor * cosS2 + sminor * sinS2;
stretch_xy = stretch_yx = (smajor - sminor) * sinScosS;
stretch_yy = smajor * sinS2 + sminor * cosS2;

swap = (orientation [0] != orientation [2]);
ewflip = (orientation [1] == 'W'); nsflip = (orientation [0] == 'N');
orient_xx = orient_yy = !swap; orient_xy = orient_yx = swap; 
if (ewflip) {orient_xx = -orient_xx; orient_yx = -orient_yx;}
if (nsflip) {orient_xy = -orient_xy; orient_yy = -orient_yy;}

combo_xx = rsun * (orient_xx * stretch_xx + orient_xy * stretch_yx) / xscale;
combo_xy = rsun * (orient_xx * stretch_xy + orient_xy * stretch_yy) / xscale;
combo_yx = rsun * (orient_yx * stretch_xx + orient_yy * stretch_yx) / yscale;
combo_yy = rsun * (orient_yx * stretch_xy + orient_yy * stretch_yy) / yscale;

sinB0 = sin (B0);
cosB0 = sqrt (1.0 - sinB0 * sinB0);	/* |B0| <= 7.25 degrees		*/
sinP  = sin (P);			/* -PI <= P <= PI		*/
cosP  = cos (P);
Rmax2 = Rmax * Rmax;

for (col = 0; col < cols; col++)
   {
   savedL [col] = (L = Ldelta*col + Lmin - Ladjust);
   savedsinL [col] = (sinL = sin (L));
   savedcosL [col] = sqrt (1.0 - sinL * sinL);
   }

/* do the "remap" */

for (row = 0; row < rows; row++)
   {
   rowoffset = row * cols;
   sinB  = sinBdelta * (sinBbase + row);
   sinB2 = sinB * sinB;
   cosB  = sqrt (1.0 - sinB2);
   sB0sB = sinB0 * sinB;
   sB0cB = sinB0 * cosB;
   cB0sB = cosB0 * sinB;
   cB0cB = cosB0 * cosB;

   v_rotation = v_equator * 
		(1.0 + sinB2 * (rotation_coef_a2 + sinB2 * rotation_coef_a4));

   for (col = 0; col < cols; col++)
      {
      sinL = savedsinL [col]; 
      cosL = savedcosL [col]; 
      sinp_sinrho = sinL * cosB;
      cosp_sinrho = cB0sB - sB0cB * cosL;
      cosrho      = sB0sB + cB0cB * cosL;
      rp = 1.0 / (1.0 - S * cosrho);
      x =  rp * (sinp_sinrho * cosP - cosp_sinrho * sinP);
      y =  rp * (cosp_sinrho * cosP + sinp_sinrho * sinP);

/* Old code. Sets to missing if outside Rmax.

      if ((x*x + y*y) >= Rmax2)
	 {
         *(U + rowoffset + col) = F_MISSING; continue;
         }
*/

/* New code. Set to 0 if outside Rmax. 

      if ((x*x + y*y) >= Rmax2)
	 {
         *(U + rowoffset + col) = 0.0; continue;
         }
*/

/* Let caller decide */

      if ((x*x + y*y) >= Rmax2)
	 {
	 *(U + rowoffset + col) = (NaN_beyond_rmax) ? F_MISSING : 0.0;
	 continue;
         }

      xtmp = combo_xx*x + combo_xy*y;
      ytmp = combo_yx*x + combo_yy*y;
      x = xtmp + x0;
      y = ytmp + y0;

/* While this code is great in theory, it is slow.
 *
 *    u = vsign * interpolate (V, xpixels, ypixels, x, y); 
 */

      if (interpolation == CUBIC_CONVOLUTION)
	 {
         u = ccintd (VD, xpixels, ypixels, x, y); 
         u = vsign * u; 
         }
      else if (interpolation == BILINEAR) 
	 {
         u = vsign * linintd (VD, xpixels, ypixels, x, y); 
         }
      else
	 {
         }

      
      if (mag_correction == MCOR_LEVEL1)	/* line-of-sight correction */
	  u = u * (sB0sB + cB0cB) / cosrho;
	  
      if (velocity_correction > VCOR_SIGN_ONLY)
	 {
         mu = 1.0 - cosrho;
         v_limbshift = limbshift_coef_a0
		     + mu * (limbshift_coef_a1 + mu * limbshift_coef_a2);

         if (velocity_correction == VCOR_LEVEL1) rp=1.0;
         u = (u + v_limbshift) / rp + v_rotation * cosB * sinL*cosB0
	     - (sB0cB - cosL * cB0sB) * v_lat
             + obs_vr/rp - S*(obs_vw*sinp_sinrho + obs_vn*cosp_sinrho);

         if (velocity_correction == VCOR_LEVEL2)
	    u = cosB * cosL * u / (cosrho - S);
         }

      *(U + rowoffset + col) = u;
      }
   }
free (VD);
return NO_ERROR;
}

/* -------------------------------------------------------------------------- */

/*
 *  Bilinear interpolation, based on pipeLNU remap.
 *
 *  Reference: 
 *     Abramowitz, Milton, and Stegun, Irene, "Handbook of Mathematical
 *     Functions," Dover Publications, New York, 1964.
 *
 *  Usage:
 *    double linintd (double *f, int nx, int ny, double x, double y)
 *
 *  Bugs:
 *    Only works for double data.
 */

double linintd (double *f, int nx, int ny, double x, double y)

{
   double p0, p1, q0, q1;          /* weights                              */
   int ilow, jlow;   /* selected CCD pixels                  */
   double *fptr;                    /* input array temp                     */
   double u;                        

   ilow = floor (x);
   jlow = floor (y);
   if (ilow < 0) ilow = 0;
   if (ilow + 1 >= nx) ilow -= 1;
   if (jlow < 0) jlow = 0;
   if (jlow + 1 >= ny) jlow -= 1;
   p1 = x - ilow;
   p0 = 1.0 - p1;
   q1 = y - jlow;
   q0 = 1.0 - q1;

   /* Bilinear interpolation from four data points. */

   u = 0.0;
   fptr = f + jlow*nx + ilow;
   u += p0 * q0 * *fptr++;
   u += p1 * q0 * *fptr;
   fptr += nx - 1;
   u += p0 * q1 * *fptr++;
   u += p1 * q1 * *fptr;
   return u;
}

/* -------------------------------------------------------------------------- */

/*
 *  Cubic convolution interpolation, based on GONG software, received Apr-88
 *  Interpolation by cubic convolution in 2 dimensions with 32-bit double data
 *
 *  Reference:
 *    R.G. Keys in IEEE Trans. Acoustics, Speech, and Signal Processing, 
 *    Vol. 29, 1981, pp. 1153-1160.
 *
 *  Usage:
 *    double ccintd (double *f, int nx, int ny, double x, double y)
 *
 *  Bugs:
 *    Currently interpolation within one pixel of edge is not
 *      implemented.  If x or y is in this range or off the picture, the
 *      function value returned is zero.
 *    Only works for double data.
 */


/* Calculate the interpolation kernel. */

static void
ccker (double *u, double s)

{
  double s2, s3;

  s2 = s * s;
  s3 = s2 * s;
  u [0] = s2 - 0.5 * (s3 + s);
  u [1] = 1.5*s3 - 2.5*s2 + 1.0;
  u [2] = -1.5*s3 + 2.0*s2 + 0.5*s;
  u [3] = 0.5 * (s3 - s2);
}

/* -------------------------------------------------------------------------- */

/* Cubic convolution interpolation */

double
ccintd (double *f, int nx, int ny, double x, double y)

{
  double  ux [4], uy [4];

/* Sum changed to double to speed up things */

  double sum;

  int ix, iy, ix1, iy1, i, j;
  double s,s2, s3;

  if (x < 1.0 || x >= (double)(nx-2) ||
      y < 1.0 || y >= (double)(ny-2))
    return 0.0;

  ix = (int) x;
  iy = (int) y;
  ccker (ux,  x - (double) ix);
  ccker (uy,  y - (double) iy);

  ix1 = ix - 1;
  iy1 = iy - 1;
  sum = 0.;
  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      sum = sum + f [(iy1 + i) * nx + ix1 + j] * uy [i] * ux [j];
  return sum;
}

/* -------------------------------------------------------------------------- */

/*
 * $Id: obs2helio.c,v 1.11 2000/05/15 17:21:03 kehcheng Exp $
 * $Source: /home/soi/CM/src/functions/RCS/obs2helio.c,v $
 * $Author: kehcheng $
 */

/* $Log: obs2helio.c,v $
 * Revision 1.11  2000/05/15  17:21:03  kehcheng
 * added line-of-sight correction for remapped magnetograms
 *
 * Revision 1.10  1998/02/24 18:42:13  kehcheng
 * added NaN_beyond_rmax to obs2helio() parameter list
 *   NaN_beyond_rmax == 0: points outside r=RMAX are set zero
 *   NaN_beyond_rmax != 0: points outside r=RMAX are set F_MISSING
 *
 * Revision 1.9  1997/04/30 02:56:42  schou
 * Sets to 0 rather that F_MISSING if point is outside Rmax.
 *
 * Revision 1.8  1996/11/19  00:09:06  schou
 * Moved interpolation code from libM into this file to permit easier
 * inlining.
 * Changed some floats to doubles.
 *
 * Revision 1.7  1995/07/13  08:04:01  phil
 * use new MISSING definitins
 *
 * Revision 1.6  1995/03/17  00:37:43  kay
 * added interpolation and velocity correction options, changed handling
 * of rsun, sinBdelta, and xscale, yscale.
 *
 * Revision 1.5  1995/03/10  00:16:17  kay
 * Added transformations to handle pixel scaling, image orientation, and
 * stretching as defined by Jesper Schou
 * was tested with Jesper Schou's testmaps (regular grids, several orientations)
 *
 * Revision 1.4  1995/01/12  22:44:15  kay
 * parameter cleanup, stretch and orientation transformations, longitude shift
 *
 * Revision 1.3  1994/10/26  21:39:57  kay
 * new file
 *
 * Revision 1.2  1994/08/26  17:27:44  kay
 * based on sds_remap, used by v2helio module
 *
 */

/* -------------------------------------------------------------------------- */

/*
 *  v2helio.c                            ~soi/(version)/src/modules/v2helio.c
 *
 *  This module interpolates CCD velocity data to estimate values that
 *  would be obtained at specified equal increments of heliographic 
 *  longitude and sine of latitude.  Apodization and corrections for
 *  solar rotation and limbshift are included. 
 *
 *  Responsible:  Kay Leibrand                   KLeibrand@solar.Stanford.EDU
 *
 *  Bugs:
 *    This module is under development.  Look for ??? in comments.
 *
 *  Planned updates:
 *    Decide when and where and if default values should be used.
 *
 *  Revision history is at end of file.
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <soi_error.h>
#include <module.h>
#include <soi_fun.h>
#include <soi_names.h>
#include <soi_sds.h>
#include <soi_vds.h>

#define PI		(M_PI)
#define RADSINDEG 	(PI/180)
#define ARCSECSINRAD 	(3600*180/PI)
#define DAYSINYEAR	(365.2425)
#define SECSINDAY	(86400)
#define TAU_A		(499.004782)	/* light time for unit dist, secs/au */
#define TCARR		(25.38)		/* days */
#define RTRUE		(6.96000000e8)	/* meters */
#define AU		(1.49597870e11)	/* meters/au */
#define MAXLEN		(256)
#define NO_DATASET	(-1)
#define NO_IMAGE	(-1)

float dtime_ (float* tarray);		/* user time = tarray[0] */
float etime_ (float* tarray);		/* sys time = tarray[1] */

int v2helio_strategy
  (KEY *params, KEY **results, int (*history)(char *fmt, ...), int (*errlog)());

int (*DoIt)() = v2helio_strategy;
char strategy_name [] = "v2helio";

argument arguments [] =
   {
   {ARG_DATA_IN, "in",       "", "", ""},
   {ARG_DATA_OUT,"out",      "", "", ""},
   {ARG_FLOAT,   "MAPRMAX",  "0.95", "", ""},
   {ARG_INT,     "MAPMMAX",  "1536", "", ""},	/* determines mapcols */
						/* default value is 3*512 */
   {ARG_FLOAT,   "MAPLGMAX", "72.0", "", ""},	/* degrees */    
   {ARG_FLOAT,   "MAPLGMIN", "-72.0", "", ""},
   {ARG_FLOAT,   "MAPBMAX",  "72.0", "", ""},  
   {ARG_INT,     "SINBDIVS", "512", "", ""},	/* # of = increments in sinB */
						/* from sin(0) to sin(PI/2) */
   {ARG_STRING,  "REF_T0",   "1987.01.03_17:31:12", "", ""},
   {ARG_FLOAT,   "REF_L0",   "0.0", "", ""},

   {ARG_FLOAT,   "SOLAR_P",  "999.0", "", ""},	/* can't use D_MISSING here */

   {ARG_INT,     "INTERPO",  "1", "0", "1"},	/* 2 methods - see soi_fun.h */
   {ARG_INT,     "APODIZE",  "0", "", ""},	/* see soi_fun.h or apodize.c */
   {ARG_FLOAT,   "APINNER",  "0.90", "", ""},	/* start of apodization */
   {ARG_FLOAT,   "APWIDTH",  "0.05", "", ""},	/* width of apodization */
   {ARG_INT,     "APEL",     "0", "", ""},	/* do elliptical apodization */
						/* described by apx and apy */
   {ARG_FLOAT,   "APX",      "1.00", "", ""},	/* divide the x position by this before applying apodization */
   {ARG_FLOAT,   "APY",      "1.00", "", ""},	/* divide the y position by this before applying apodization */
   {ARG_INT,     "LGSHIFT",  "0", "0", "2"}, 	/* 0=none; 1=fixed rate; 2=nearest Degree */
   {ARG_INT,     "VCORLEV",  "2", "0", "2"}, 	/* 3 levels - see soi_fun.h*/
   {ARG_INT,     "MCORLEV",  "0", "0", "1"}, 	/* 2 levels - see soi_fun.h*/
   {ARG_INT,     "MOFFSET",  "0", "0", "1"}, 	/* 1=apply BFITZERO correction*/
   {ARG_INT,     "OUTBITS",  "no scaling", "", ""},  /* bits in scaled output */
   {ARG_FLOAT,   "OUTSCALE", "1.0", "", ""},    /* scale for output */
   {ARG_FLOAT,   "OUTBIAS",  "0.0", "", ""},    /* bias for scaled output */
   {ARG_FLAG,    "v",        "0", "-1", "1"},
   {ARG_FLAG,    "o",        "0", "-1", "1"},	/* Non 0 skips checko (SESW assumed) */
   {ARG_FLAG,    "z",        "0", "-1", "1"},	/* Non 0 sets data outside RMAX MISSING */
   {ARG_INT,     "DATASIGN",  "0", "-1", "1"}, 	/* Non 0 forces datasigh to value*/
   {ARG_INT,     "MAXMISSVALS", "0", "", "0"},  /* max. allowed MISSING pixels */
   {ARG_STRING, "protocol", "RDB.FITS", "", ""},
   {ARG_END,     "", "", "", ""}
   };

static double str2time (char *timestr); 
static void checko (char *orientation);
static int status_error
   (int (*errlog)(), char *ftn, int error, int ds, int sn);
static int parameter_error
   (int (*errlog)(), char *pname, int error, int ds, int sn, KEY **results);
static int parameter_default
   (int(*errlog)(), char *pname, double defaultp, double *p, int ds, int sn);

/* -------------------------------------------------------------------------- */

int
v2helio_strategy (KEY *params, KEY **results, int (*history)(char *fmt, ...),
		  int (*errlog)())

{
static char in_root  [MAX_STRLEN];
static char out_root [MAX_STRLEN];
static char snkey    [MAX_STRLEN];

SDS *in_sds, *out_sds;
VDS *in_vds, *out_vds;

char *tobsstr, *trefstr, *orientation;

int warning = NO_ERROR;
int sn, in_fsn, in_lsn, n, nsets;
int paramsign;
int longitude_shift, velocity_correction, interpolation, apodization, bits;
int mag_correction;
int mag_offset;
int mapped_lmax, sinb_divisions, mapcols, maprows, nmax, nmin;
int status, verbose;
int length[2];
int quality;
int protocol;

float fillvalue = F_MISSING; 
double tobs, tmearth, tref;
double smajor, sminor, sangle;
double xscale, yscale, imagescale, xs;
int xpixels, ypixels, pixels;
double obs_vr, obs_vw, obs_vn, vsign;
double b0, bmax, bmin, sinBdelta;
double longmax, longmin, longmax_adjusted, longmin_adjusted, longinterval;
double p0, p, rmax;
double x0, y0;
double obsdist, longshift, obsl0, refl0, mapl0, longrate, rtrue, rsun, S;
int obsCR;
int apel;
double apinner, apwidth, apx, apy;
double scale, bias;
double colsperdeg;

float t0, tarray [2];

int skip_checko = getkey_flag (params, "o");
int NaN_beyond_rmax = getkey_flag (params, "z");
int maxmissvals = getkey_int (params, "MAXMISSVALS");

verbose = getkey_byte (params, "v");
(*history) ("Begin %s\n",strategy_name);
(*history) (" %s\n", rcsid);

/* Determine the number of input datasets. */

if (getkeytype (params, "in_nsets") != KEYTYP_INT)
   return parameter_error
      (errlog, "in_nsets", REQUIRED_PARAMETER_MISSING, NO_DATASET, NO_IMAGE, results);
nsets = getkey_int (params,"in_nsets");

/* Determine output storage model. */

if (strcmp (GETKEY_str (params, "protocol"), "RDB.FITS_MERGE") == 0)
    protocol = VDS_FITS_MERGE;
else
    protocol = VDS_FITS;

longrate = 360.0 / TCARR -360.0 / DAYSINYEAR;		/* degrees per day */
longrate /= SECSINDAY;					/* degrees per sec */
rtrue = RTRUE / AU;					/* au */

apodization = getkey_int    (params, "APODIZE");
apinner     = getkey_double (params, "APINNER");
apwidth     = getkey_double (params, "APWIDTH");
apel        = getkey_int    (params, "APEL");
apx         = getkey_double (params, "APX");
apy         = getkey_double (params, "APY");
longitude_shift     = getkey_int (params, "LGSHIFT");
mag_correction      = getkey_int (params, "MCORLEV");
mag_offset          = getkey_int (params, "MOFFSET");
velocity_correction = getkey_int (params, "VCORLEV");
interpolation       = getkey_int (params, "INTERPO");
paramsign           = getkey_int (params, "DATASIGN");
rmax  = getkey_double (params, "MAPRMAX");
refl0 = getkey_double (params, "REF_L0");
bits  = getkey_int    (params, "OUTBITS");
scale = getkey_double (params, "OUTSCALE");
bias  = getkey_double (params, "OUTBIAS");
p0    = getkey_double (params, "SOLAR_P");

tref = str2time (trefstr = getkey_str (params, "REF_T0"));	/* secs */
if (soi_errno) return parameter_error
   (errlog, "REF_T0", soi_errno, NO_DATASET, NO_IMAGE, results); 
free (trefstr);

/* Determine mapcols and adjust longmin and longmax. */

mapped_lmax = getkey_int (params, "MAPMMAX"); 
longmax  = getkey_double (params, "MAPLGMAX");	/* degrees */
longmin  = getkey_double (params, "MAPLGMIN");	/* degrees */
longinterval = (180.0) / mapped_lmax;		/* degrees */

/* 
 * This does not always handle the case where 1/longinterval is an integer 
 * correctly.
 */

nmin = longmin / longinterval;			/* round towards 0 */
nmax = longmax / longinterval;			/* round towards 0 */
colsperdeg = mapped_lmax / 180.0;
nmin = longmin * colsperdeg;			/* round towards 0 */
nmax = longmax * colsperdeg;			/* round towards 0 */
mapcols = nmax - nmin + 1;
longmin_adjusted = nmin * longinterval;
longmax_adjusted = nmax * longinterval;

/* Determine maprows, bmax, bmin, and sinBdelta. */

sinb_divisions = getkey_int (params, "SINBDIVS");
sinBdelta = 1.0 / sinb_divisions;
bmax = getkey_double (params, "MAPBMAX");	/* degrees */
bmin = -bmax; 
nmax = sin (RADSINDEG * bmax) * sinb_divisions;	/* round towards 0 */
maprows = 2 * nmax;

/* Note that other parameters come from the data attributes. */

for (n = 0; n < nsets; n++)
   {
   sprintf (in_root, "in_%d", n); sprintf (out_root, "out_%d", n);

   if (!(in_vds = vds_open (params, in_root)))
      {
      warning = status_error (errlog, "vds_open", soi_errno, n, NO_IMAGE);
      continue;					/* go to next dataset */
      }

   if (!(out_vds = vds_new (params, out_root, protocol)))
      {
      warning = status_error (errlog, "vds_new", soi_errno, n, NO_IMAGE);
      vds_close (&in_vds);
      continue;					/* go to next dataset */ 
      }
   
        /* Copy global keywords to out. */

   sds_append_attrs(out_vds->global_attributes, in_vds->global_attributes);
   vds_setkey_str(out_vds, "CONFORMS", "TS_EQ");

   status = vds_add_var_name(out_vds, "");
   if (status)
      {
      warning = status_error (errlog, "vds_add_var_name", status, n, NO_IMAGE);
      vds_close (&in_vds);
      continue;					/* go to next dataset */ 
      }

   sprintf (snkey, "in_%d_fsn", n); in_fsn = getkey_int (params, snkey);
   sprintf (snkey, "in_%d_lsn", n); in_lsn = getkey_int (params, snkey);
   if (in_lsn == -1) in_lsn = vds_last_record (in_vds);

   t0 = dtime_(tarray); (*history) ("Setup %f\n", t0);
   for (sn = in_fsn; sn <= in_lsn; sn++)
      {
      if (!(in_sds = VDS_select_frec(in_vds, 0, sn)))
	 {
         warning = status_error (errlog, "vds_select_frec", soi_errno, n, sn);
         continue;				/* go to next image */
         }

      tobsstr = sds_search_attrvalue_str (in_sds, "T_OBS");
      if ((ALL_DATA_MISSING == (quality = sds_data_quality (in_sds))) ||
          				  (!tobsstr) || (!*tobsstr))
         {
	 SDS *out_sds = sds_create ();
         sds_copy_attr (out_sds, in_sds, "I_DREC");
         sds_copy_attr (out_sds, in_sds, "T_REC");
         sds_copy_attr (out_sds, in_sds, "QUALITY");
         sds_set_attribute (out_sds, "T_OBS", tobsstr, SDS_STRING, "");
         free (tobsstr);
         vds_insert_sds_rec (out_vds, out_sds, 0, sn);
         warning = status_error (errlog, "sds_data_quality", quality, n, sn);
         continue;				/* go to next image */
         }

      if (maxmissvals > 0)
	 {
	  int missvals = sds_getkey_int (in_sds, "MISSVALS");
	  if (missvals > maxmissvals) 
	     {
	     SDS *out_sds = sds_create();
	     sds_copy_attr (out_sds, in_sds, "I_DREC");
	     sds_copy_attr (out_sds, in_sds, "T_REC");
	     sds_set_attribute (out_sds, "T_OBS", tobsstr, SDS_STRING, "");
	     vds_insert_sds_rec (out_vds, out_sds, 0, sn);
	     sds_free_data (in_sds);
	     free (tobsstr);
	     errlog ("** skipping image %d in dataset %d (%d pixels MISSING)\n",
	             sn, n, missvals);
	     continue;
	     }
         }

      /* Assemble arguments. */

/* old code
 *      tobs = str2time (tobsstr = sds_search_attrvalue_str (in_sds,"T_OBS"));
 */
      tobs = str2time (tobsstr);			/*secs*/
      if (soi_errno) return parameter_error 
         (errlog, "T_OBS", soi_errno, n, sn, results); 

      b0 = sds_search_attrvalue_double (in_sds, "OBS_B0");
      if (soi_errno) return parameter_error 
         (errlog, "OBS_B0", REQUIRED_PARAMETER_MISSING, n, sn, results);

      obsl0 = sds_search_attrvalue_double (in_sds,"OBS_L0");
      if (soi_errno) return parameter_error 
         (errlog, "OBS_L0", REQUIRED_PARAMETER_MISSING, n, sn, results);

      obsCR = sds_getkey_int (in_sds,"OBS_CR");
      if (is_I_MISSING (obsCR)) return parameter_error 
         (errlog, "OBS_CR", REQUIRED_PARAMETER_MISSING, n, sn, results);


      if (p0 == 999.0)
	 {
	 p = sds_search_attrvalue_double (in_sds, "SOLAR_P");
	 if (soi_errno) return parameter_error 
	    (errlog, "SOLAR_P", REQUIRED_PARAMETER_MISSING, n, sn, results);
         }
      else
	 {
	 p = p0;
         }

      smajor = sds_search_attrvalue_double (in_sds, "S_MAJOR");
      if (soi_errno)  warning = parameter_default
         (errlog, "S_MAJOR", 1.0, &smajor, n, sn);
      sminor = sds_search_attrvalue_double (in_sds,"S_MINOR");
      if (soi_errno)  warning = parameter_default
         (errlog, "S_MINOR", 1.0, &sminor, n, sn);
      sangle = sds_search_attrvalue_double (in_sds,"S_ANGLE");
      if (soi_errno)  warning = parameter_default
         (errlog, "S_ANGLE", 0.0, &sangle, n, sn);

      xscale = sds_search_attrvalue_double (in_sds,"X_SCALE");
      if (soi_errno) return parameter_error 
         (errlog, "X_SCALE", REQUIRED_PARAMETER_MISSING, n, sn, results);
      yscale = sds_search_attrvalue_double (in_sds,"Y_SCALE");
      if (soi_errno) return parameter_error 
         (errlog, "Y_SCALE", REQUIRED_PARAMETER_MISSING, n, sn, results);
      imagescale = sds_search_attrvalue_double (in_sds,"IM_SCALE");
      if (soi_errno) return parameter_error 
         (errlog, "IM_SCALE", REQUIRED_PARAMETER_MISSING, n, sn, results);
   
      vsign = sds_search_attrvalue_double (in_sds,"DATASIGN");
      if (paramsign != 0) vsign = paramsign;
      if (soi_errno)  warning = parameter_default
         (errlog, "DATASIGN", 1.0, &vsign, n, sn);
      if (velocity_correction)
	 {
         obs_vr = sds_search_attrvalue_double (in_sds,"OBS_VR");
         if (soi_errno)  warning = parameter_default
            (errlog, "OBS_VR", 0.0, &obs_vr, n, sn);
         obs_vw = sds_search_attrvalue_double (in_sds,"OBS_VW");
         if (soi_errno)  warning = parameter_default
            (errlog, "OBS_VW", 0.0, &obs_vw, n, sn);
         obs_vn = sds_search_attrvalue_double (in_sds,"OBS_VN");
         if (soi_errno)  warning = parameter_default
            (errlog, "OBS_VN", 0.0, &obs_vn, n, sn);
         }

      if (!skip_checko)
	 {
	  checko (orientation = sds_search_attrvalue_str (in_sds, "ORIENT"));
	  if (soi_errno) return parameter_error
	     (errlog, "ORIENT", soi_errno, n, sn, results);
         }
      else
	 {
	 char foo[] = "SESW    ";
	 orientation = malloc (9);
	 memcpy (orientation, foo, 9);
         }

      obsdist = sds_search_attrvalue_double (in_sds, "OBS_DIST");
      if (soi_errno)  warning = parameter_default
         (errlog, "OBS_DIST", 1.0, &obsdist, n, sn);
      S = rtrue / obsdist;	/* radians - approx. arcsin (rtrue / obsdist) */
   
      rsun = sds_search_attrvalue_double (in_sds,"R_SUN");
      if (soi_errno) 
	 {
         rsun = ARCSECSINRAD * S / sqrt (1.0 - S*S) / imagescale;
         warning = parameter_default (errlog, "R_SUN", rsun, &rsun, n, sn);
         }
      xs = sds_search_attrvalue_double (in_sds, "XSCALE");
      if (!soi_errno) 
	 {
/* New meaning of R_SUN, change value to old one if X_SCALE=Y_SCALE.
 * Otherwise, give up.
 */

         if (xscale == yscale)
	    {
            rsun = rsun * xscale;
            }
         else
	    {
            return parameter_error 
            (errlog, "X_SCALE != Y_SCALE", NOT_IMPLEMENTED, n, sn, results);
            }
         }

      if (longitude_shift == 1) 
	 {
         tmearth = tobs + TAU_A * (1.0 - obsdist); 
         longshift = (obsl0 - refl0) + longrate * (tmearth - tref);/* degrees */ 
         while (longshift >  180.0) longshift -= 360.0; 
         while (longshift < -180.0) longshift += 360.0;
         }
      else
      if (longitude_shift == 2) /* Shift center to nearest Carrington Degree */
	 {
	 longshift =  obsl0 - (int) (obsl0);
	 if (longshift > 0.5) longshift -= 1.0;
         }
      else longshift = 0.0;

      mapl0 = obsl0 - longshift;

      xpixels = sds_dim0 (in_sds);
      ypixels = sds_dim1 (in_sds);
      pixels  = xpixels * ypixels;

      x0 = sds_search_attrvalue_double (in_sds, "X0");
      if (soi_errno)  warning = parameter_default
         (errlog, "X0", xpixels / 2.0, &x0, n, sn);
      y0 = sds_search_attrvalue_double (in_sds, "Y0");
      if (soi_errno)  warning = parameter_default
         (errlog, "Y0", ypixels / 2.0, &y0, n, sn);

      /* Construct the output sds. */

      length [0] = mapcols; length [1] = maprows;
      if (!(out_sds = sds_construct (2, length, SDS_FLOAT, &fillvalue)))
	 {
         warning = status_error (errlog, "sds_construct", soi_errno, n, sn);
         sds_free (&in_sds); free (orientation);
         continue;					/* go to next image */
         }

   /* ??? how many of these attributes do we really need? */
   /* ??? what attributes are missing ??? */

      sds_set_attribute (out_sds, "T_OBS", tobsstr, SDS_STRING, "");
      free (tobsstr);
      sds_copy_attr (out_sds, in_sds, "I_DREC");
      sds_copy_attr (out_sds, in_sds, "T_REC");
      sds_copy_attr (out_sds, in_sds, "QUALITY");
      sds_copy_attr (out_sds, in_sds, "MISSVALS");
      sds_set_attribute (out_sds, "SN", &sn, SDS_INT, "");
      sds_set_attribute (out_sds, "PIXELS", &pixels, SDS_INT, "");
      sds_set_attribute (out_sds, "OBS_B0", &b0, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "OBS_L0",  &obsl0, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "OBS_CR",  &obsCR, SDS_INT, "");
      sds_set_attribute (out_sds, "SOLAR_P", &p, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "X0", &x0, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "Y0", &y0, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "MAPMMAX", &mapped_lmax, SDS_INT, "");
      sds_set_attribute (out_sds, "MAPCOLS", &mapcols, SDS_INT, "");
      sds_set_attribute (out_sds, "MAPROWS", &maprows, SDS_INT, "");
      sds_set_attribute (out_sds, "MAPLGMAX", &longmax_adjusted, SDS_DOUBLE,"");
      sds_set_attribute (out_sds, "MAPLGMIN", &longmin_adjusted, SDS_DOUBLE,"");
      sds_set_attribute (out_sds, "MAPBMAX", &bmax, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "MAPBMIN", &bmin, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "SINBDELT",&sinBdelta, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "MAPRMAX", &rmax, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "APODIZE", &apodization, SDS_INT, "");
      sds_set_attribute (out_sds, "APINNER", &apinner, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "APWIDTH", &apwidth, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "APEL", &apel, SDS_INT, "");
      sds_set_attribute (out_sds, "APX", &apx, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "APY", &apy, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "MCORLEV", &mag_correction, SDS_INT, "");
      sds_set_attribute (out_sds, "VCORLEV", &velocity_correction, SDS_INT, "");
      sds_set_attribute (out_sds, "INTERPO", &interpolation, SDS_INT, "");
      sds_set_attribute (out_sds, "SHIFTFLG", &longitude_shift, SDS_INT, "");
      sds_set_attribute (out_sds, "LSHIFT",  &longshift, SDS_DOUBLE, "");
      sds_set_attribute (out_sds, "MAP_L0",  &mapl0, SDS_DOUBLE, "");

      if (mag_offset)
	 {
	 float *dat = (float *) sds_data (in_sds);
	 double bfitzero = sds_search_attrvalue_double (in_sds, "BFITZERO");
	 int i;
	 char str [80];
	 if (is_D_MISSING (bfitzero))
	    {
	    sds_append_history (out_sds, 
	      "MDI magnetogram zero level correction was requested but\n");
	      sds_append_history (out_sds,
	      "BFITZERO keyword was not found.  No correction was made.\n");
	    }
	 else
	    {
	    for (i = 0; i < pixels; ++i) dat [i] -= bfitzero;
	    sprintf (str,
	    "An MDI magnetogram zero level correction of %.4g Gauss\n",
	    bfitzero);
	    sds_append_history (out_sds, str);
	    sprintf (str,
               "was subtracted from the input image before remapping.\n");
	    sds_append_history (out_sds, str);
            }
         }

      if (status = obs2helio
	 ( 
         (float *) sds_data (in_sds), (float *) sds_data (out_sds),
         xpixels, ypixels, x0, y0, b0 * RADSINDEG, p * RADSINDEG, 
         S, rsun, rmax,
         interpolation, mapcols, maprows, 
         longmin_adjusted * RADSINDEG, longinterval * RADSINDEG,
         longshift * RADSINDEG, 
         sinBdelta, 
         smajor, sminor, sangle * RADSINDEG, 
         xscale, yscale, orientation, mag_correction,
         velocity_correction, obs_vr, obs_vw, obs_vn, vsign, 
	 NaN_beyond_rmax))
         {
         warning = status_error (errlog, "obs2helio", status, n, sn);
         continue;					/* go to next image */
         }

      if (status = apodize (
	 (float *) sds_data (out_sds),
	 b0 * RADSINDEG, mapcols, maprows,
	 longmin_adjusted * RADSINDEG, longinterval * RADSINDEG,
	 sinBdelta,
	 apodization, apinner, apwidth, apel, apx, apy))
         {
         warning = status_error (errlog, "apodize", status, n, sn);
         continue; 
         }						/* go to next image */

      if (!is_I_MISSING (bits))
      if (status = sds_set_scaling (out_sds, bits, scale, bias))
	 {
         warning = status_error (errlog, "sds_set_scaling", status, n, sn);
         continue;
         }

      if (status = vds_insert_sds_rec (out_vds, out_sds, 0, sn))
	 {
         warning = status_error (errlog, "vds_insert_sds", status, n, sn);
         continue; 
         }						/* go to next image */
      if (status = vds_flush_data_rec (out_vds, 0, sn))
	 {
         warning = status_error (errlog, "vds_flush_data_rec", status, n, sn);
         continue; 
         }						/* go to next image */
 
      out_sds = NULL;
      sds_free_data (in_sds);
      free (orientation);

      t0 = dtime_ (tarray); (*history) ("sn %d done %f\n", sn, t0);
      }						/* end loop on sn (images) */

   vds_close (&in_vds);
   if (status = vds_close (&out_vds))
      {
      warning = status_error (errlog, "vds_close", status, n, NO_IMAGE);
      continue; 
      }							/* go to next dataset */

}	/* end loop on n (datasets) */

t0 = etime_(tarray);
(*history) ("Total time spent %f %f %f\n", t0,tarray[0],tarray[1]);
setkey_int (results, "abortflg", 0);
return warning;

}	/* end v2helio_strategy */

/* -------------------------------------------------------------------------- */

/* Convert keyword time format to UNIX time_t -- no fractional seconds. */

static double
str2time (char *timestr)

{
time_t return_t = 0;
static struct tm t;
t.tm_isdst = 0;			/* t.tm_gmtoff = 0; t.tm_zone = NULL;*/ 

if (!timestr) 
   soi_errno = REQUIRED_PARAMETER_MISSING;
else
if (sscanf (timestr, "%d.%d.%d_%d:%d:%d",
   &(t.tm_year), &(t.tm_mon), &(t.tm_mday), 
   &(t.tm_hour), &(t.tm_min), &(t.tm_sec))!=6)
   soi_errno = ILLEGAL_TIME_FORMAT;
else
   {
   t.tm_mon -= 1; t.tm_year -= 1900; return_t = mktime (&t);
   soi_errno = (return_t == (time_t)-1) ? TIME_CONVERSION_FAILED : NO_ERROR;
   }
return (double) return_t;
}

/* -------------------------------------------------------------------------- */

/* Check for legal orientation string. */

static void
checko (char *orientation)

{
static char o [MAXLEN];
char *c = o;

if (!orientation) soi_errno = REQUIRED_PARAMETER_MISSING;
else if (4 != sscanf (orientation, "%[NS]%[EW]%[NS]%[EW]", c++, c++, c++, c++))
   soi_errno = ILLEGAL_ORIENTATION;
else if ((o [0] == o [2]) && (o [1] == o [3])) soi_errno = ILLEGAL_ORIENTATION;
else if ((o [0] != o [2]) && (o [1] != o [3])) soi_errno = ILLEGAL_ORIENTATION;
else soi_errno = NO_ERROR;
}

/* -------------------------------------------------------------------------- */

/* Logs error. */

static int
status_error (int (*errlog)(), char *ftn, int error, int ds, int sn)

{
(*errlog) ("*** error:   %d in %s", error, ftn);
if (ds != NO_DATASET) (*errlog) (", in dataset %d", ds);
if (sn != NO_IMAGE)   (*errlog) (", in image %d", sn);
(*errlog) (" **\n");

perrstk ((int (*) (const char*,...)) errlog);
return error;
}

/* -------------------------------------------------------------------------- */

/* Logs error and sets abort flag. */

static int
parameter_error 
   (int (*errlog)(), char *pname, int error, int ds, int sn, KEY **results)
   
{
setkey_int (results, "abortflg", 1);
(*errlog)("*** error: %d in parameter %s, dataset %d, image %d **\n", 
          error, pname, ds, sn);

perrstk ((int (*) (const char*,...)) errlog);
return error;
}

/* -------------------------------------------------------------------------- */

/* Logs warning and sets parameter to default value. */

static int
parameter_default
   (int (*errlog)(), char *pname, double defaultp, double *p, int ds, int sn)

{
*p = defaultp;
(*errlog)(" ** warning: default value %g used for %s in dataset %d, image %d **\n", 
          defaultp, pname, ds, sn);

perrstk ((int (*) (const char*,...)) errlog);
return DEFAULT_PARAMETER_USED;
}

/* -------------------------------------------------------------------------- */

/*
 *  v 1.1  96.01.02     R Bogart        changed order of args in sds_copy_attrs
 *  v 1.2  96.03.08 R Bogart	modified lsn check to account for new default
 */

/*
 * $Id: v2helio.c,v 1.49 2000/08/22 15:31:49 kehcheng Exp $
 * $Source: /home/soi/CM/src/modules/RCS/v2helio.c,v $
 * $Author: kehcheng $
 */

/* $Log: v2helio.c,v $
 * Revision 1.49  2000/08/22  15:31:49  kehcheng
 * added MOFFSET (for magnetogram zero level correction)
 * removed flatfield_file
 *
 * Revision 1.48  2000/05/15 17:22:17  kehcheng
 * added MCORLEV (for line-of-sight correction to remapped magnetograms)
 * added flatfield_file (for Ic flatfield correction before remapping)
 *
 * Revision 1.47  1999/07/27 23:32:35  kay
 * declare fillvalue as float
 *
 * Revision 1.46  1999/06/18  21:10:57  phil
 * Added "protocol" flag to allow saving as FITS_MERGE
 *
 * Revision 1.45  1999/04/28  00:25:19  kehcheng
 * *** empty log message ***
 *
 * Revision 1.44  1999/04/28 00:15:43  kehcheng
 * added parameter SOLAR_P
 * modified determination of nmin,nmax from longmin,longmax
 *
 * Revision 1.43  1999/04/27 18:42:39  schou
 * Added GONG style elliptical apodization.
 *
 * Revision 1.42  1999/01/25  19:43:15  kehcheng
 * added vds_insert_rec() call in the case of MISSVALS > MAXMISSVALS
 *
 * Revision 1.41  1998/12/01 16:29:52  kehcheng
 * added command-line argument MAXMISSVALS
 *
 * Revision 1.40  1998/06/10 17:53:46  kehcheng
 * fixed dtime_() and etime_() calls
 *
 * Revision 1.39  1998/04/24 00:08:36  kay
 * changed *(int *) sds_search_attrvalue to sds_getkey_int in calculation
 * of obsCR.
 *
 * Revision 1.38  1998/02/24  18:40:40  kehcheng
 * added flag "z" to choose value for points outside r=RMAX
 *   z=0: 0.0 (the default)
 *   z=1: F_MISSING
 *
 * Revision 1.37  1998/02/23 22:24:07  kay
 * add scaling option - float output is the default
 *
 * Revision 1.36  1997/09/25  17:22:11  kehcheng
 * added flag "o" to skip checko()
 *
 * Revision 1.35  1997/05/15  20:12:42  schou
 * Put in hack to fix new definition of R_SUN based on existence
 * of XSCALE keyword. Also considers image missing of T_OBS is
 * not set or is blank.
 *
 * Revision 1.34  1997/04/23  00:55:12  phil
 * removed copy_conform_info
 *
 * Revision 1.33  1997/04/21  18:58:15  schou
 * Now does test on existence of XSCALE to derive meaning of R_SUN.
 *
 * Revision 1.32  1997/02/03  20:13:24  schou
 * Added parameter DATASIGN to fit buggy mdical Dopplergrams.
 *
 * Revision 1.31  1997/01/29  01:57:56  todd
 * Added LSHIFT to nearest Carrington Degree
 * Added Keyword MAP_LO SHIFTFLG to output
 * Fixed type for pixels,xpixels,ypixels
 *
 * Revision 1.30  1997/01/27  23:58:52  todd
 * Added OBS_CR to output
 *
 * Revision 1.29  1997/01/27  22:40:44  schou
 * Changed default to not do longitude shift.
 * Put OBS_L0 and OBS_B0 in output.
 *
 * Revision 1.28  1996/11/19  00:19:28  schou
 * Put in timing statements.
 *
 * Revision 1.27  1996/11/12  07:11:12  phil
 * fix missing records
 *
 * Revision 1.26  1996/11/05  20:01:32  phil
 * Fix output record headers in case of missing input
 * records.
 *
 * Revision 1.25  1996/10/09  06:58:13  giles
 * Added a copy of MISSVALS from input to output.
 * A good idea?  I'm not sure, but it probably doesn't hurt
 * anything, and it is helpful to me.
 *
 * Revision 1.24  1996/09/06  22:31:52  kay
 * added vds_flush_data_rec and changed calculation of default x0 and y0
 *
 * Revision 1.23  1996/09/06  00:17:03  schou
 * No changes.
 *
 * Revision 1.22  1996/05/21  00:05:11  CM
 * fix history
 *
 * Revision 1.21  1996/05/13  21:21:02  jim
 * (*history)(char *fmt, ...)
 *
 * Revision 1.20  1996/05/02  20:42:39  kay
 * changed external prototype in call to vds_new
 *
 * Revision 1.19  1996/03/09  00:00:47  rick
 * see above
 *
 * Revision 1.18  1996/01/03  06:50:07  rick
 * see above
 *
 * Revision 1.17  1995/11/18  00:41:20  giles
 * Output dataset now has attributes I_DREC and T_REC
 *
 * Revision 1.16  1995/10/27  19:03:16  kay
 * workaround to get fsn and lsn until vds_first_record is available
 *
 * Revision 1.15  1995/08/26  00:23:52  kay
 * changed calls to sds_free and vds_close
 *
 * Revision 1.14  1995/07/14  23:28:34  kay
 * added perrstk calls, fixed compiler warnings, and corrected QUALITY typo
 *
 * Revision 1.13  1995/07/13  06:49:25  phil
 * changed MISSING to D_MISSING
 * propogate QUALITY
 *
 * Revision 1.12  1995/07/06  22:37:22  kay
 * assume in_fsn is 0, get in_lsn from vds
 *
 * Revision 1.11  1995/07/06  22:10:35  schou
 * Put in call to new apodize function.
 *
 * Revision 1.10  1995/06/21  18:27:29  phil
 * added VDS_select... option
 *
 * Revision 1.9  1995/04/20  17:29:49  kay
 * changed data quality test and changed RSUN to R_SUN
 *
 * Revision 1.8  1995/03/22  01:34:32  kay
 * corrected units and assignment of default rsun
 *
 * Revision 1.7  1995/03/17  00:47:57  kay
 * added interpolation and velocity correction options,
 * changed handling of rsun,  sinBdelta, and xscale,yscale.
 *
 * Revision 1.6  1995/03/08  21:11:33  kay
 * major revamp to integrate with vds.  also changed error handling,
 * velocity correction method and made velocity correction and
 * longitude shift options.  was tested with cat019 conforming dataset
 *
 * Revision 1.5  1995/01/24  17:08:10  kay
 * changes in error logging and status returns
 * this is a version which worked before integration with vds
 *
 * Revision 1.4  1995/01/12  22:46:13  kay
 * mostly parameter cleanup
 *
 * Revision 1.3  1994/10/27  21:43:49  kay
 * changed method of determining increments of sin(lat) & added comments
 *
 * Revision 1.2  1994/08/26  16:33:30  kay
 * tested successfully against pipeLNU remap module
 * */

/* -------------------------------------------------------------------------- */

