/*
 *  obs2helio.c                         ~soi/(version)/src/modules/obs2helio.c
 *
 *  Purpose:
 *     Interpolate CCD data to estimate value that would
 *     be obtained at specified equal incrememts of heliographic
 *     longitude and sine of latitude.
 *
 *  Description:
 *
 *     Employs bi-linear or cubic convolution interpolation to map a
 *     portion of the solar disk to a rectangular array "cols" by "rows" in
 *     width and height respectively.
 *
 *     Adapted by Kay Leibrand from pipeLNU remap from proto.d remap which was 
 *     modified by PHS from Lyle Bacon version.
 *     Added transformations to handle pixel scaling, image orientation, and
 *     stretching (in process) as defined by Jesper Schou.
 *     Added choice of velocity correction and interpolation methods.
 *
 *  Reference:
 *
 *     Green, Robin M. "Spherical Astronomy," Cambridge University
 *     Press, Cambridge, 1985.
 *
 *  Responsible:  Kay Leibrand                   KLeibrand@solar.Stanford.EDU
 *
 *  Restrictions:
 *     The number of output map rows must be even.
 *     The number of output map columns must be less than MAXCOLS.
 *     Assumes orientation is a 4 character string with one of the following
 *     8 legal values: SESW SENE SWSE SWNW NWNE NWSW NENW NESE 
 *
 *  Bugs:
 *     Possible division by 0 
 *
 *  Planned updates:
 *     Optimize.
 *
 *  Revision history is at end of file.
 *
 *  22 Nov 2002: cosmetic changes only [ALS]
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define MAXCOLS	(3072)

#define DIMENSION_MISMATCH -1
#define CANT_DO_ODD_NUM_LATS -2
#define UNSUPPORTED_INTERPOLATION -3

double ccintd  (double *f, int nx, int ny, double x, double y);
double linintd (double *f, int nx, int ny, double x, double y);

/* -------------------------------------------------------------------------- */

int
obs2helio
(
float	*V,		/* input intensity array			*/
			/* assumed declared V [ypixels][xpixels]	*/
float	*U,		/* output projected array			*/

int	xpixels,	/* x width of input array			*/
int	ypixels,	/* y width of input array			*/
double  x0, 	   	/* x pixel address of disk center		*/
double  y0, 	   	/* y pixel address of disk center		*/
double  B0,		/* heliographic latitude of disk center		*/
double  P,		/* angle between CCD y-axis and solar vertical	*/
			/* positive to the east (CCW)			*/
double  S,		/* subtended radius of the sun	radians    	*/
double  rsun,		/* pixels */
double  Rmax,		/* maximum disk radius to use (e.g. 0.95)	*/

int	interpolation,	/* 1: cubic convolution
			   2: bilinear interpolation			*/
int	cols,		/* width of output array			*/
int	rows,		/* height of output array			*/
double  Lmin,		/* start of longitude range 			*/
double  Ldelta,		/* increment of longitude per col	        */
double  sinBdelta,	/* increment of sin latitude per row          */

double	smajor,		/* see http://soi.stanford.edu/cgi-bin/keyword.pl   */
double	sminor,		/* SMAJOR, SMINOR, SANGLE, X_SCALE, Y_SCALE, ORIENT */
double	sangle,	
double  xscale,
double  yscale,
char    *orientation
)

{

double xtmp, ytmp;
float u;			/* output temp				*/

/* Variables for coordinate projection of desired point (L,B) on disk	*/

double sinBbase;  		/* base value to calculate sinB         */
double sinB0, cosB0;		/* Latitude center of disk components	*/
double sinB, cosB;		/* Latitude desired point components	*/
double sinL, cosL;		/* Longitude desired point components	*/
double sinP, cosP;		/* Position angle of N wrt CCD y-axis	*/
double sinPe, cosPe;		/* Position angle of semimajor axis	*/
double sinrho, cosrho;		/* Central angle to desired point comps */
double sinp, cosp;		/* Position angle of point wrt N on sun */
double sinp_sinrho;		/* N-S projection on disk		*/
double cosp_sinrho;		/* W-E projection on disk		*/
double rp;			/* projected radial distance correction */
double x, y;			/* CCD location of desired point	*/
double Rmax2;			/* Max disk radius to consider - squared*/

/* Variables for "stretch" transformation matrix */

double sinS, cosS, sinS2, cosS2, sinScosS;
double stretch_xx, stretch_xy, stretch_yx, stretch_yy;

/* Variables for "orientation" transformation matrix */

int orient_xx, orient_xy, orient_yx, orient_yy;
int swap, ewflip, nsflip;

/* Variables for "combined" transformation matrix */

double combo_xx, combo_xy, combo_yx, combo_yy;

/*  Variables to determine (L,B) of desired point			*/

int row, col;			/* index into output array		*/
double L;			/* Longitude of desired point		*/

/*  Function for interpolation */

float (*interpolate) ();

/*  Variables to determine velocity correction				*/

double mu;
double v_rotation;
double v_limbshift;

/*  Constants used in velocity correction				*/

const double v_lat	       = 0.0;
const double v_equator         = 1974.1;
const double rotation_coef_a2  = -0.121;
const double rotation_coef_a4  = -0.178;
const double limbshift_coef_a0 = -166.;
const double limbshift_coef_a1 = -139.;
const double limbshift_coef_a2 = 700.;

/*  Variables for optimization  */

double savedL [MAXCOLS];
double savedsinL [MAXCOLS];
double savedcosL [MAXCOLS];
double cB0sB, cB0cB, sB0sB, sB0cB;
double sinB2;
int rowoffset;

long i;
double *vdp;
float *vp;

/* Copy float array V into double array VD */

double *VD;
VD = (double *) (malloc (xpixels * ypixels * sizeof (double)));
vdp = VD;
vp = V;
for (i = 0; i < xpixels * ypixels; i++) *vdp++ = (double) *vp++;

if (cols > MAXCOLS) return DIMENSION_MISMATCH;
if (rows % 2) return CANT_DO_ODD_NUM_LATS;


/* Calculate quantities that are constant for this image */

sinBbase = 0.5 - rows / 2;
sinS  = sin (sangle); cosS = cos (sangle); 
sinS2 = sinS * sinS;  cosS2 = cosS * cosS; sinScosS = sinS * cosS;
stretch_xx = smajor * cosS2 + sminor * sinS2;
stretch_xy = stretch_yx = (smajor - sminor) * sinScosS;
stretch_yy = smajor * sinS2 + sminor * cosS2;

/*
printf ("sinBbase:   %9.2e\n", sinBbase) ;
printf ("sinS:       %9.2e\n", sinS) ;
printf ("sinS2:      %9.2e\n", sinS2) ;
printf ("stretch_xx: %9.2e\n", stretch_xx) ;
printf ("stretch_xy: %9.2e\n", stretch_xy) ;
printf ("stretch_yy: %9.2e\n", stretch_yy) ;
 */

swap   = (orientation [0] != orientation [2]);
ewflip = (orientation [1] == 'W');
nsflip = (orientation [0] == 'N');
orient_xx = orient_yy = !swap;
orient_xy = orient_yx =  swap; 
if (ewflip) {orient_xx = -orient_xx; orient_yx = -orient_yx;}
if (nsflip) {orient_xy = -orient_xy; orient_yy = -orient_yy;}

/*
printf ("orient_xx: %3d\n", orient_xx) ;
printf ("orient_yx: %3d\n", orient_yx) ;
printf ("orient_xy: %3d\n", orient_xy) ;
printf ("orient_yy: %3d\n", orient_yy) ;
 */

combo_xx = rsun * (orient_xx * stretch_xx + orient_xy * stretch_yx) / xscale;
combo_xy = rsun * (orient_xx * stretch_xy + orient_xy * stretch_yy) / xscale;
combo_yx = rsun * (orient_yx * stretch_xx + orient_yy * stretch_yx) / yscale;
combo_yy = rsun * (orient_yx * stretch_xy + orient_yy * stretch_yy) / yscale;

/*
printf ("combo_xx: %9.2e\n", combo_xx) ;
printf ("combo_xy: %9.2e\n", combo_xy) ;
printf ("combo_yx: %9.2e\n", combo_yx) ;
printf ("combo_yy: %9.2e\n", combo_yy) ;
 */

sinB0 = sin (B0);
cosB0 = sqrt (1.0 - sinB0 * sinB0);	/* |B0| <= 7.25 degrees 	*/
sinP  = sin (P);			/* -PI <= P <= PI		*/
cosP  = cos (P);
Rmax2 = Rmax * Rmax;

/*
printf ("sinB0:    %9.2e\n", sinB0) ;
printf ("cosB0:    %9.2e\n", cosB0) ;
printf ("sinP :    %9.2e\n", sinP ) ;
printf ("cosP :    %9.2e\n", cosP ) ;
printf ("Rmax2:    %9.2e\n", Rmax2) ;
 */

for (col = 0; col < cols; col++) 
   {
   savedL [col] = (L = Ldelta * col + Lmin );
   savedsinL [col] = (sinL = sin (L));
   savedcosL [col] = sqrt (1.0 - sinL * sinL);
   }

/*
   printf ("cols: %3d  rows: %3d\n", cols, rows) ;
 */

/* do the "remap" */

for (row = 0; row < rows; row++)
   {
   rowoffset = row * cols;
   sinB  = sinBdelta * (sinBbase + row);
   sinB2 = sinB * sinB;
   cosB  = sqrt (1.0 - sinB2);
   sB0sB = sinB0 * sinB;
   sB0cB = sinB0 * cosB;
   cB0sB = cosB0 * sinB;
   cB0cB = cosB0 * cosB;

/*
   printf ("\n") ;
   printf ("row: %3d  rowoffset: %d\n", row, rowoffset) ;
   printf ("sinB: %9.2e  sinB2: %9.2e  cosB: %9.2e\n", sinB, sinB2, cosB) ;
   printf ("sB0sB: %9.2e  sB0cB: %9.2e\n", sB0sB, sB0cB) ;
   printf ("cB0sB: %9.2e  cB0sB: %9.2e\n", cB0sB, cB0cB) ;
 */

   v_rotation = v_equator * (1.0 +
		sinB2 * (rotation_coef_a2 + sinB2 * rotation_coef_a4));

   for (col = 0; col < cols; col++)
      {
/*
      printf ("col: %3d  row: %3d\n", col, row) ;
 */

      sinL = savedsinL [col]; 
      cosL = savedcosL [col]; 
      sinp_sinrho = sinL * cosB;
      cosp_sinrho = cB0sB - sB0cB * cosL;
      cosrho      = sB0sB + cB0cB * cosL;
      rp = 1.0 / (1.0 - S * cosrho);
      x =  rp * (sinp_sinrho * cosP - cosp_sinrho * sinP);
      y =  rp * (cosp_sinrho * cosP + sinp_sinrho * sinP);

/*
      printf ("sinL: %9.2e  cosL: %9.2e\n", sinL, cosL) ;
      printf ("sinp_sinrho: %9.2e  cosp_sinrho: %9.2e\n",
	       sinp_sinrho, cosp_sinrho) ;
      printf ("cosrho: %9.2e  rp: %9.2e\n", cosrho, rp) ;
      printf ("x:  %7.2e  y:%9.2e\n", x, y) ;
 */

/* Old code. Sets to missing if outside Rmax.
 *
 *      if ((x*x + y*y) >= Rmax2)
 *         {
 *         *(U + rowoffset + col) = F_MISSING; continue;
 *         }
 */

      /* New code. Set to 0 if outside Rmax. */

      if ((x * x + y * y) >= Rmax2)
	 {
         *(U + rowoffset + col) = 0.0;
/*
	 printf ("col: %3d  row: %3d  U =    0.00\n", col, row) ;
 */
	 continue;
         }

      xtmp = combo_xx * x + combo_xy * y;
      ytmp = combo_yx * x + combo_yy * y;
      x = xtmp + x0;
      y = ytmp + y0;

/*
      printf ("xtmp: %9.2e  ytmp: %9.2e\n", xtmp, ytmp) ;

      printf ("x: %7.2f  y:%7.2f\n", x, y) ;
 */

      if (interpolation == 1)
         {
         u = ccintd (VD, xpixels, ypixels, x, y); 
         }
      else if (interpolation == 2)
	 {
         u = linintd (VD, xpixels, ypixels, x, y); 
         }
      else return UNSUPPORTED_INTERPOLATION; 
      
      *(U + rowoffset + col) = u;

/*
      printf ("col: %3d  row: %3d  U = %9.2e\n",
               col, row, *(U + rowoffset + col)) ;
 */
      }
   }
free (VD);
return 0;
}

/* -------------------------------------------------------------------------- */

/*
 *  Bilinear interpolation, based on pipeLNU remap.
 *
 *  Reference: 
 *     Abramowitz, Milton, and Stegun, Irene, "Handbook of Mathematical
 *     Functions," Dover Publications, New York, 1964.
 *
 *  Usage:
 *    double linintd (double *f, int nx, int ny, double x, double y)
 *
 *  Bugs:
 *    Only works for double data.
 */

double
linintd (double *f, int nx, int ny, double x, double y)
{

   double p0, p1, q0, q1;	/* weights                              */
   int ilow, jlow;		/* selected CCD pixels                  */
   double *fptr;		/* input array temp                     */
   double u;                        

   ilow = floor(x);
   jlow = floor(y);
   if (ilow < 0) ilow = 0;
   if (ilow+1 >= nx) ilow -= 1;
   if (jlow < 0) jlow = 0;
   if (jlow+1 >= ny) jlow -= 1;
   p1 = x - ilow;
   p0 = 1.0 - p1;
   q1 = y - jlow;
   q0 = 1.0 - q1;

   /* Bilinear interpolation from four data points. */

   u = 0.0;
   fptr = f + jlow*nx + ilow;
   u += p0 * q0 * *fptr++;
   u += p1 * q0 * *fptr;
   fptr += nx - 1;
   u += p0 * q1 * *fptr++;
   u += p1 * q1 * *fptr;
   return u;
}

/* -------------------------------------------------------------------------- */

/*
 *  Cubic convolution interpolation, based on GONG software, received Apr-88
 *  Interpolation by cubic convolution in 2 dimensions with 32-bit double data
 *
 *  Reference:
 *    R.G. Keys in IEEE Trans. Acoustics, Speech, and Signal Processing, 
 *    Vol. 29, 1981, pp. 1153-1160.
 *
 *  Usage:
 *    double ccintd (double *f, int nx, int ny, double x, double y)
 *
 *  Bugs:
 *    Currently interpolation within one pixel of edge is not
 *      implemented.  If x or y is in this range or off the picture, the
 *      function value returned is zero.
 *    Only works for double data.
 */


/* Calculate the interpolation kernel. */

static void 
ccker (double *u, double s)
{
  double s2, s3;

  s2= s * s;
  s3= s2 * s;
  u [0] = s2 - 0.5 * (s3 + s);
  u [1] =  1.5 * s3 - 2.5 * s2 + 1.0;
  u [2] = -1.5 * s3 + 2.0 * s2 + 0.5 * s;
  u [3] =  0.5 * (s3 - s2);
}

/* -------------------------------------------------------------------------- */

/* Cubic convolution interpolation */

double 
ccintd (double *f, int nx, int ny, double x, double y)
{
  double  ux [4], uy [4];

/* Sum changed to double to speed up things */

  double sum;

  int ix, iy, ix1, iy1, i, j;
  double s,s2, s3;

  if (x < 1. || x >= (double) (nx - 2) ||
      y < 1. || y >= (double) (ny - 2))
    return 0.0;

  ix = (int) x;
  iy = (int) y;
  ccker (ux,  x - (double) ix);
  ccker (uy,  y - (double) iy);

  ix1 = ix - 1;
  iy1 = iy - 1;
  sum = 0.0;
  for (i = 0; i < 4; i++)
    for (j = 0; j < 4; j++)
      sum = sum + f [(iy1 + i) * nx + ix1 + j] * uy [i] * ux [j];
  return sum;
}

/* -------------------------------------------------------------------------- */

/*
$Id: obs2helio.c,v 1.1 2002/11/20 21:49:15 stanger Exp stanger $
$Source: /home/stanger/syn/RCS/obs2helio.c,v $
$Author: stanger $
*/
/* $Log: obs2helio.c,v $
 * Revision 1.1  2002/11/20 21:49:15  stanger
 * Initial revision
 *
 * Revision 1.11  2000/05/15  17:21:03  kehcheng
 * added line-of-sight correction for remapped magnetograms
 *
 * Revision 1.10  1998/02/24 18:42:13  kehcheng
 * added NaN_beyond_rmax to obs2helio() parameter list
 *   NaN_beyond_rmax == 0: points outside r=RMAX are set zero
 *   NaN_beyond_rmax != 0: points outside r=RMAX are set F_MISSING
 *
 * Revision 1.9  1997/04/30 02:56:42  schou
 * Sets to 0 rather that F_MISSING if point is outside Rmax.
 *
 * Revision 1.8  1996/11/19  00:09:06  schou
 * Moved interpolation code from libM into this file to permit easier
 * inlining.
 * Changed some floats to doubles.
 *
 * Revision 1.7  1995/07/13  08:04:01  phil
 * use new MISSING definitins
 *
 * Revision 1.6  1995/03/17  00:37:43  kay
 * added interpolation and velocity correction options,
 * changed handling of rsun, sinBdelta,
 * and xscale,yscale.
 *
 * Revision 1.5  1995/03/10  00:16:17  kay
 * Added transformations to handle pixel scaling, image orientation, and
 * stretching as defined by Jesper Schou
 * was tested with Jesper Schou's testmaps (regular grids, several orientations)
 *
 * Revision 1.4  1995/01/12  22:44:15  kay
 * parameter cleanup, stretch and orientation transformations, longitude shift
 *
 * Revision 1.3  1994/10/26  21:39:57  kay
 * new file
 *
 * Revision 1.2  1994/08/26  17:27:44  kay
 * based on sds_remap, used by v2helio module
 * */
