/* soleph.c
 * -----------------------------------------------------------------------------
 * Compute solar ephemeris parameters.
 * -----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR    9 Dec 2002
 * Derived from David Elmore's C-language program [sun_eph.c]
 * and Nick Arge's IDL procedures [jd_carr.pro, carr_jd_fn.pro].
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>

soleph (year, month, day, hour, minute, second,
	Ro, Po, Bo, Lo, RA, DEC, crot)

int year ;		/* Year. */
int month ;		/* Month. */
int day ;		/* Day. */
int hour ;		/* Hour. */
int minute ;		/* Minute. */
int second ;		/* Second. */

float *Ro ;		/* Solar Radius [arcsec]. */
float *Po ;		/* P-angle [deg]. */
float *Bo ;		/* Heliographic Latitude  [deg]. */
float *Lo ;		/* Heliographic Longitude [deg]. */
float *RA ;		/* Right Ascension [deg]. */
float *DEC ;		/* Declination [deg]. */
int   *crot ;		/* Carrington Rotation. */

{
    extern double jdate () ;
    extern double carr_rotation () ;

    double cr         = 0.0 ;	/* Carrington Rotation. */
    double degrad     = 0.0 ;	/* degrees / radian. */
    double elong      = 0.0 ;	/* Ecliptic longitude. [radians] */
    double elongdeg   = 0.0 ;	/* Ecliptic longitude. [degrees] */
    double eobliq     = 0.0 ;	/* Obliquity of ecliptic [radians]. */
    double eobliqdeg  = 0.0 ;	/* Obliquity of ecliptic [degrees]. */
    double EPOCH      = 2000.0 ;	/* 2000 Jan 1 00:00:00 UT */
    double g          = 0.0 ;	/* Mean anomaly [radians]. */
    double gdeg       = 0.0 ;	/* Mean anomaly [degrees]. */
    double JD_1854    = 2398219.5 ; /* Julian date for 1854 Jan 1 00:00:00 UT */
    double JD_EPOCH   = 2451544.5 ; /* Julian date for 2000 Jan 1 00:00:00 UT */
    double jd         = 0.0 ;	/* Julian date.  */
    double L          = 0.0 ;	/* Mean solar longitude [radians]. */
    double Ldeg       = 0.0 ;	/* Mean solar longitude [degrees]. */
    double nday       = 0.0 ;	/* # days since EPOCH. */
    double omega      = 0.0 ;	/* Longitude of solar equator's ascending node*/
    double pi         = 3.14159265358979323846264338327 ;
    double rau        = 0.0 ;	/* Earth-Sun distance (AU). */
    double rdeg       = 0.0 ;	/* Solar radius. */
    double raddeg     = 0.0 ;	/* radians / degree. */
    double ra         = 0.0 ;	/* Right Ascension. */
    double dec        = 0.0 ;	/* Declination. */
    double P          = 0.0 ;	/* P-angle. */
    double B0         = 0.0 ;	/* B0 angle [latitude]. */
    double L0         = 0.0 ;	/* Carrington Longitude [radians]. */
    double L0deg      = 0.0 ;	/* Carrington Longitude [degrees]. */

    double LmO        = 0.0 ;   /* elong - omega */
    double solar_incl = 7.25 ;	/* Solar inclination (degrees). */
    double cos_I      = 0.0 ;
    double sin_I      = 0.0 ;
    double cos_elong  = 0.0 ;
    double sin_elong  = 0.0 ;
    double cos_eobliq = 0.0 ;
    double sin_eobliq = 0.0 ;
    double cos_LmO    = 0.0 ;
    double sin_LmO    = 0.0 ;

    raddeg = pi / 180.0 ;
    degrad = 180.0 / pi ;
    sin_I = sin (solar_incl * raddeg) ;
    cos_I = cos (solar_incl * raddeg) ;

    /*** Julian Day. ***/

    jd = jdate (year, month, day, hour, minute, second) ;

    nday = jd - 2451545.0 ;

    printf ("nday: %9.4f\n", nday) ;

    /*** Mean solar Longitude, corrected for abberation. ***/

    Ldeg = 280.460 + 0.9856474 * nday ;		/* [2000 Nautical Almanac] */
    Ldeg = 280.472 + 0.9856474 * nday ;
    L    = Ldeg * raddeg ;

    /*** Mean anomaly. ***/

    gdeg = 357.528 + 0.9856003 * nday ;
    g    = gdeg * raddeg ;

    /*** Ecliptic Longitude. ***/

    elongdeg = Ldeg + 1.915 * sin (g) + 0.02 * sin (2.0 * g) ;
    elong    = elongdeg * raddeg ;
    while (elongdeg <    0.0)  elongdeg += 360.0 ;
    while (elongdeg >  360.0)  elongdeg -= 360.0 ;
    sin_elong = sin (elong) ;
    cos_elong = cos (elong) ;

    /*** Obliquity of ecliptic. ***/

    eobliqdeg = 23.439 - 0.0000004 * nday ;
    eobliq    = eobliqdeg * raddeg ;
    sin_eobliq = sin (eobliq) ;
    cos_eobliq = cos (eobliq) ;

    /*** Right Ascension. ***/

    ra = atan (cos (eobliq) * tan (elong)) ;

    /*** Declination. ***/

    dec = asin (sin (eobliq) * sin (elong)) ;

    /*** Earth-Sun distance [AU]. ***/

    rau = 1.00014 - 0.01671 * cos (g) - 0.00014 * cos (2.0 * g) ;

    /*** Solar radius. ***/

    rdeg = 0.2666 / rau ;		/* Degrees. */

    /* Method from "Explanatory Supplement the Astronomical Ephemeris"
     * 1961, p 307-310.
     */

    /*** Omega is the longitude of the solar equator's ascending node. ***/

    omega = 73.66666666667				/* Epoch of 1850. */
	  + (EPOCH - 1850.0) * 50.25 / 3600.0		/* Current epoch. */
	  + (50.0 * 50.25 / 3600.0) * (nday * 2.0) / 36525.0 ;
    omega *= raddeg ;					/* Convert to radians.*/

    /*** Change   in 1/2 century. ***/
    /*** Fraction of 1/2 century. ***/

    LmO = elong - omega ;
    sin_LmO = sin (LmO) ;
    cos_LmO = cos (LmO) ;

    /*** P-angle. ***/

    P = atan2 (-cos_elong * sin_eobliq, cos_eobliq)
      + atan2 (-cos_LmO   * sin_I,      cos_I) ;

    /*** Latitude of sun center. ***/

    B0 = asin (sin_LmO * sin_I) ;

    /*** Carrington Longitude. ***/

    L0 = atan2 (sin_LmO * cos_I, cos_LmO)
       + ((JD_1854 - JD_EPOCH) * (360.0 / 25.38) + 180.0) * raddeg
       - ((360.0 / 25.38) * raddeg) * (nday) ;
    L0deg = L0 * degrad ;
    while (L0deg <    0.0)  L0deg += 360.0 ;
    while (L0deg >  360.0)  L0deg -= 360.0 ;

    cr = carr_rotation (jd) ;

    printf ("year month day: %4d-%02d-%02d\n", year, month, day) ;
    printf ("hour min sec  : %02d:%02d:%02d\n", hour, minute, second) ;
    printf ("jd:     %9.4f\n", jd) ;
    printf ("nday:   %9.4f\n", nday) ;
    printf ("L:      %9.4f radians   %9.4f degrees\n", L, Ldeg) ;
    printf ("g:      %9.4f radians   %9.4f degrees\n", g, gdeg) ;
    printf ("elong:  %9.4f radians   %9.4f degrees\n", elong, elongdeg) ;
    printf ("eobliq: %9.4f radians   %9.4f degrees\n", eobliq, eobliqdeg) ;
    printf ("RA:     %9.4f radians   %9.4f degrees\n", ra,  ra  * degrad) ;
    printf ("DEC:    %9.4f radians   %9.4f degrees\n", dec, dec * degrad) ;
    printf ("rau:    %9.4f AU\n", rau) ;
    printf ("rsun:   %9.4f degrees   %9.4f arcsec\n", rdeg, rdeg*3600.0) ;
    printf ("omega:  %9.4f radians   %9.4f degrees\n", omega, omega * degrad) ; 
    printf ("P:      %9.4f radians   %9.4f degrees\n", P, P * degrad) ;
    printf ("B0:     %9.4f radians   %9.4f degrees\n", B0, B0 * degrad) ;
    printf ("L0:     %9.4f radians   %9.4f degrees\n", L0, L0deg) ;

    *Ro =   (float) (rdeg * 3600.0) ;	/* Rsun [degrees]. */
    *Po =   (float) (P * degrad) ;	/* P-angle [deg]. */
    *Bo =   (float) (B0 * degrad) ;	/* Heliographic Latitude  [deg]. */
    *Lo =   (float) L0deg ;		/* Heliographic Longitude [deg]. */
    *RA =   (float) (ra * degrad) ;	/* Right Ascension [deg]. */
    *DEC =  (float) (dec * degrad) ;	/* Declination [deg]. */
    *crot = (int) cr ;		/* Carrington Rotation. */

    return (0) ;
}

/* -------------------------------------------------------------------------- */

