/* wintd.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Weighted nearest-neighbor interpolation : Shepard's method.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * wint returns a double precision value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    double wint () ;
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, floating-point,
 * or double images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger   HAO/NCAR    6 Mar 2003
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

double
wintd (image, nbypix, xdim, ydim, x, y)

unsigned char	*image ;		/* Pointer to image array. */
int	nbypix ;	/* # bytes per pixel. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
double	x, y ;		/* Coordinates of scan point in image. */

{
   char   *im ;
   short  *ims ;
   float  *imf ;
   double *imd ;
   int	 ix, iy ;
   double fx, fy ;
   double f00, f10, f01, f11 ;
   double fx0, fx1, fxy ;
   double h00, h10, h01, h11 ;
   double w00, w10, w01, w11 ;
   double x00, x10, x01, x11 ;
   double y00, y10, y01, y11 ;
  
/*
   fprintf (stderr, "wint> nbypix: %d  xdim/ydim: %4d %4d  x/y: %7.2f %7.2f\n",
	    nbypix, xdim, ydim, x, y) ;
 */

   if (x < 0.0 || x > (double) xdim)
      {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
      return (-1.0e-30) ;
      }

   if (y < 0.0 || y > (double) ydim)
      {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
      return (-2.0e-30) ;
      }

   /*** X/Y coordinates of lower left corner pixel. ***/

   ix = (int) x ;
   iy = (int) y ;

   /*** Fractional distance of point from lower left corner neighbor. ***/

   fx = x - (double) ix ;
   fy = y - (double) iy ;

   /*** Distances of each neighbor from interpolated point. ***/

   x00 = x - (double) ix ;
   y00 = y - (double) iy ;

   x10 = x - (double) (ix + 1) ;
   y10 = y - (double) iy ;

   x01 = x - (double) ix ;
   y01 = y - (double) (iy + 1) ;

   x11 = x - (double) (ix + 1) ;
   y11 = y - (double) (iy + 1) ;

   h00 = sqrt ( x00 * x00 + y00 * y00 ) ;
   h10 = sqrt ( x10 * x10 + y10 * y10 ) ;
   h01 = sqrt ( x01 * x01 + y01 * y01 ) ;
   h11 = sqrt ( x11 * x11 + y11 * y11 ) ;

   /*** Compute weights for each neighboring pixel. ***/

   hsum = 1.0 / h00*h00 + 1.0 / h10*h10 + 1.0 / h01*h01 + 1.0 / h11*h11

   w00 = (1.0 / (h00 * h00) ) / hsum ;
   w10 = (1.0 / (h10 * h10) ) / hsum ;
   w01 = (1.0 / (h01 * h01) ) / hsum ;
   w11 = (1.0 / (h11 * h11) ) / hsum ;

   /*** Select 4 nearest neighbor elements in 'image' array. ***/

   /*** 1 byte per pixel. ***/

   if (nbypix == 1)
      {
      im  = (char *) image ;
      f00 = w00 * (double) (*(im +  ix    +  iy    * xdim) & 0377) ;
      f10 = w10 * (double) (*(im + (ix+1) +  iy    * xdim) & 0377) ;
      f01 = w01 * (double) (*(im +  ix    + (iy+1) * xdim) & 0377) ;
      f11 = w11 * (double) (*(im + (ix+1) + (iy+1) * xdim) & 0377) ;
      }
   else

   /*** 2 bytes per pixel (short integer). ***/

   if (nbypix == 2)
      {
      ims = (short *) image ;
      f00 = w00 * (double) *(ims +  ix    +  iy    * xdim) ;
      f10 = w10 * (double) *(ims + (ix+1) +  iy    * xdim) ;
      f01 = w01 * (double) *(ims +  ix    + (iy+1) * xdim) ;
      f11 = w11 * (double) *(ims + (ix+1) + (iy+1) * xdim) ;
      }
   else

   /*** 4 bytes per pixel (doubleing point). ***/

   if (nbypix == 4)
      {
      imf = (float *) image ;
      f00 = w00 * (double) *(imf +  ix    +  iy    * xdim) ;
      f10 = w10 * (double) *(imf + (ix+1) +  iy    * xdim) ;
      f01 = w01 * (double) *(imf +  ix    + (iy+1) * xdim) ;
      f11 = w11 * (double) *(imf + (ix+1) + (iy+1) * xdim) ;
      }

   if (nbypix == 8)
      {
      imd = (double *) image ;
      f00 = w00 * (double) *(imd +  ix    +  iy    * xdim) ;
      f10 = w10 * (double) *(imd + (ix+1) +  iy    * xdim) ;
      f01 = w01 * (double) *(imd +  ix    + (iy+1) * xdim) ;
      f11 = w11 * (double) *(imd + (ix+1) + (iy+1) * xdim) ;
      }

   /*** Shepard's inverse distance weighted interpolation. ***/

   fxy = f00 + f10 + f01 + f11 ;

   return (fxy) ;
}
