/* wintdr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Weighted nearest-neighbor interpolation : Shepard's method.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * wint returns a double precision value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    double wint () ;
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, floating-point,
 * or double images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger   HAO/NCAR    6 Mar 2003
 *  6 Mar 2003 : modified to NOT use a pixel beyond rsun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

double
wintdr (img, xdim, ydim, x, y, xcen, ycen, rsun)

double *img ;		/* Pointer to image array. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
double	x, y ;		/* Coordinates of scan point in image. */
double  xcen ;		/* X-axis sun center (pixel units). */
double  ycen ;		/* Y-axis sun center (pixel units). */
double  rsun ;		/* Solar radius (pixel units). */

{
   int	 ix, iy ;
   int   pmode = 0 ;
   double fx, fy ;
   double f00, f10, f01, f11 ;
   double fx0, fx1, fxy ;
   double h00, h10, h01, h11 ;
   double hsum ;
   double invhsq00, invhsq10, invhsq01, invhsq11 ;
   double r00, r10, r01, r11 ;
   double sx00, sx10, sx01, sx11 ;
   double sy00, sy10, sy01, sy11 ;
   double w00, w10, w01, w11 ;
   double x00, x10, x01, x11 ;
   double y00, y10, y01, y11 ;
  
   if (pmode)
       {
       fprintf (stdout , "\n") ;
       fprintf (stdout, "wintdr> xdim/ydim: %4d %4d  x/y: %7.2f %7.2f\n",
	        xdim, ydim, x, y) ;
       fprintf (stdout, "wintdr> xcen/ycen: %7.2f %7.2f  rsun: %7.2f\n",
	        xcen, ycen, rsun) ;
       }

   if (x < 0.0 || x > (double) xdim)
      {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
      return (-1.0e-30) ;
      }

   if (y < 0.0 || y > (double) ydim)
      {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
      return (-2.0e-30) ;
      }

   /*** X/Y coordinates of lower left corner pixel. ***/

   ix = (int) x ;
   iy = (int) y ;

   /*** Fractional distance of point from lower left corner neighbor. ***/

   fx = x - (double) ix ;
   fy = y - (double) iy ;

   if (pmode)
      fprintf (stdout, "fx: %9.4f  fy: %9.4f\n", fx, fy) ;

   /*** Distance of each neighbor pixel from sun center. ***/

   sx00 = (double)  ix      - xcen ;
   sy00 = (double)  iy      - ycen ;

   sx10 = (double) (ix + 1) - xcen ;
   sy10 = (double)  iy      - ycen ;

   sx01 = (double)  ix      - xcen ;
   sy01 = (double) (iy + 1) - ycen ;

   sx11 = (double) (ix + 1) - xcen ;
   sy11 = (double) (iy + 1) - ycen ;

   r00 = sqrt ( sx00 * sx00 + sy00 * sy00 ) ;
   r10 = sqrt ( sx10 * sx10 + sy10 * sy10 ) ;
   r01 = sqrt ( sx01 * sx01 + sy01 * sy01 ) ;
   r11 = sqrt ( sx11 * sx11 + sy11 * sy11 ) ;

   if (pmode && iy == ydim / 2)
       {
       printf ("ix/iy: %4d  %4d  r00/r10/r01/r11:  %7.2f  %7.2f  %7.2f  %7.2f\n",
	       ix, iy, r00, r10, r01, r11) ;
       }

   /*** Distances of each neighbor from interpolated point. ***/

   x00 = x - (double) ix ;
   y00 = y - (double) iy ;

   x10 = x - (double) (ix + 1) ;
   y10 = y - (double) iy ;

   x01 = x - (double) ix ;
   y01 = y - (double) (iy + 1) ;

   x11 = x - (double) (ix + 1) ;
   y11 = y - (double) (iy + 1) ;

   h00 = sqrt ( x00 * x00 + y00 * y00 ) ;
   h10 = sqrt ( x10 * x10 + y10 * y10 ) ;
   h01 = sqrt ( x01 * x01 + y01 * y01 ) ;
   h11 = sqrt ( x11 * x11 + y11 * y11 ) ;

   if (pmode)
   {
   fprintf (stdout, "x00: %9.4f  x10: %9.4f  x01: %9.4f  x11: %9.4f\n",
	    x00, x10, x01, x11) ;
   fprintf (stdout, "y00: %9.4f  y10: %9.4f  y01: %9.4f  y11: %9.4f\n",
	    y00, y10, y01, y11) ;
	    
   fprintf (stdout, "h00: %9.4f  h10: %9.4f  h01: %9.4f  h11: %9.4f\n",
	    h00, h10, h01, h11) ;
   }

   /*** Compute inverse square distances. ***/

   invhsq00 = 1.0 / (h00 * h00) ;
   invhsq10 = 1.0 / (h10 * h10) ;
   invhsq01 = 1.0 / (h01 * h01) ;
   invhsq11 = 1.0 / (h11 * h11) ;

   if (h00 == 0.0)  invhsq00 = 1.0 ;
   if (h10 == 0.0)  invhsq10 = 1.0 ;
   if (h01 == 0.0)  invhsq01 = 1.0 ;
   if (h11 == 0.0)  invhsq11 = 1.0 ;

   if (pmode)
      fprintf (stdout, "invhsq00: %9.4f  invhsq10: %9.4f  invhsq01: %9.4f  invhsq11: %9.4f\n",
	    invhsq00, invhsq10, invhsq01, invhsq11) ;

   /*** For each point with a radial distance exceeding rsun,
    *** set the inverse square distance to zero.
    ***/

   if (r00 > rsun)  invhsq00 = 0.0 ;
   if (r10 > rsun)  invhsq10 = 0.0 ;
   if (r01 > rsun)  invhsq01 = 0.0 ;
   if (r11 > rsun)  invhsq11 = 0.0 ;

   /*** Compute weights for each neighboring pixel. ***/

/*
 *   hsum = 1.0 / h00*h00 + 1.0 / h10*h10 + 1.0 / h01*h01 + 1.0 / h11*h11
 *
 *   w00 = (1.0 / (h00 * h00) ) / hsum ;
 *   w10 = (1.0 / (h10 * h10) ) / hsum ;
 *   w01 = (1.0 / (h01 * h01) ) / hsum ;
 *   w11 = (1.0 / (h11 * h11) ) / hsum ;
 */

   hsum = invhsq00 + invhsq10 + invhsq01 + invhsq11 ;
   if (hsum == 0.0)  return (0.0) ;

   w00 = invhsq00 / hsum ;
   w10 = invhsq10 / hsum ;
   w01 = invhsq01 / hsum ;
   w11 = invhsq11 / hsum ;

   if (pmode)
      {
      fprintf (stdout, "hsum: %9.4f\n", hsum) ;
      fprintf (stdout, "w00: %9.4f  w10: %9.4f  w01: %9.4f  w11: %9.4f\n",
	       w00, w10, w01, w11) ;
      }

   /*** Select 4 nearest neighbor elements in 'img' array. ***/

  f00 = w00 * (double) *(img +  ix    +  iy    * xdim) ;
  f10 = w10 * (double) *(img + (ix+1) +  iy    * xdim) ;
  f01 = w01 * (double) *(img +  ix    + (iy+1) * xdim) ;
  f11 = w11 * (double) *(img + (ix+1) + (iy+1) * xdim) ;

   /*** Shepard's inverse distance weighted interpolation. ***/

   fxy = f00 + f10 + f01 + f11 ;

   return (fxy) ;
}
