/* pb0r.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C-language version adapted from IDL code: $SSW/gen/idl/solar/pb0r.pro
 * Andrew L. Stanger   HAO/NCAR   25 March 1999
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 * Project     : SOHO - CDS
 *
 * Name        : PB0R()
 *
 * Purpose     : To calculate the solar P, B0 angles and the semi-diameter.
 *
 * Explanation : Uses semi-rigorous formulae to calculate the solar P (position
 *               angle of pole) and B0 (latitude of point at disk centre) angles
 *               and also the semi-diameter of the solar disk at the date/time
 *               requested.
 *
 * Use         : IDL> ang = pb0r(date_time,/soho)
 *
 * Inputs      : date_time  -  the date/time specified in any CDS format
 *
 * Opt. Inputs : None
 *
 * Outputs     : Function returns a 3-element array with
 *                                    ang(0)  = P  (degrees)
 *                                    ang(1)  = B0 (degrees)
 *                                    ang(2)  = R  semi-diameter (arcmin or
 *                                                                arcsec if
 *                                                                keyword set)
 *
 * Opt. Outputs: None
 *
 * Keywords    : SOHO - if present the semi-diameter returned is as viewed
 *                      from the SOHO spacecraft
 *               ARCSEC - returns semi-diameter in arcsec rather than arcmins
 *               ERROR  - Output keyword containing error message;
 *                        a null string is returned if no error occurs
 *
 * Calls       : ANYTIM2UTC
 *               SUN_POS
 *
 * Common      : pb0r_common (internal common block)
 *
 * Restrictions: None
 *
 * Side effects: None
 *
 * Category    : Util, coords
 *
 * Prev. Hist. : Based on Fortran programs by Hohenkerk and Emerson (RGO)
 *
 * Written     : CDS/IDL version, C D Pike, RAL, 16-May-94
 *
 * Modified    : Update semi-diameter calculation, CDP, 20-May-94
 *		Version 3, William Thompson, GSFC, 14 November 1994
 *			Modified .DAY to .MJD
 *               Version 4, CDP, 10-Jan-96
 *                  Add SOHO/ARCSEC keywords and make NOW the default.
 *               Version 5, Liyun Wang, GSFC/ARC, March 12, 1996
 *                  Modified such that point of view can be changed to
 *                     SOHO if the env variable SC_VIEW is set to 1
 *                     (via the call to USE_SOHO_VIEW)
 *                  Added ERROR keyword
 *               Version 6, Liyun Wang, GSFC/ARC, March 14, 1996
 *                  Replaced call to GET_ORBIT with the IDL call_function
 *               Version 7, Liyun Wang, GSFC/ARC, March 21, 1996
 *                  Modified such that if no orbit file is found, earth
 *                     view is used
 *               Version 8, Liyun Wang, GSFC/ARC, April 10, 1996
 *                  Set SC_VIEW to 0 if no orbit files are found
 *               Version 9, February 6, 1997, Liyun Wang, NASA/GSFC
 *                  Changed call to ANYTIM2JD instead of CDS2JD
 *          
 * Version     : Version 9, February 6, 1997
 */

#include <math.h>
#include <stdio.h>

int
pb0r (iyear, imonth, iday, ihour, iminute, isecond,
      pangle, bangle, rsun)
 
int iyear, imonth, iday, ihour, iminute, isecond ;
double *pangle, *bangle, *rsun ;
{

   extern double moduloc () ;
   extern double julday () ;
   static double pi = 3.14159265 ;
   double raddeg ;
   double degrad ;
   double jd ;
   double de ;
   double longmed ;
   double ra, dec ;
   double appl ;
   double oblt ;
   double lambda ;
   double node ;
   double arg ;
   double t ;
   double r ;
   double d ;
   double mv, me, mm, mj ;

   degrad = 180.0 / pi ;
   raddeg = pi / 180.0 ;

/*---------------------------------------------------------------------------
 *  number of Julian days since 2415020.0
 *---------------------------------------------------------------------------
 */
   jd = julday (iyear, imonth, iday, ihour, iminute, isecond) ;
   jd -= 2415020.0L ;

/*---------------------------------------------------------------------------
 *  ignoring difference between UT and ET then ...
 *---------------------------------------------------------------------------
 */
    de = (double) jd ;

/*---------------------------------------------------------------------------
 *  get the longitude of the sun etc.
 *---------------------------------------------------------------------------
 */
   sun_pos (de, &longmed, &ra, &dec, &appl, &oblt) ;

/*---------------------------------------------------------------------------
 *  form aberrated longitude
 *---------------------------------------------------------------------------
 */
   lambda = longmed - (20.5L / 3600.0L) ;

/*---------------------------------------------------------------------------
 *  form longitude of ascending node of sun's equator on ecliptic
 *---------------------------------------------------------------------------
 */
   node = 73.666666L + (50.25L / 3600.0L) * ( (de / 365.25L) + 50.0L ) ;
   arg = lambda - node ;

/*---------------------------------------------------------------------------
 *  calculate P, the position angle of the pole.
 *---------------------------------------------------------------------------
 */
   *pangle = (atan (-tan (oblt * raddeg) * cos (appl * raddeg)) +
              atan ( -0.12722L * cos (arg * raddeg))) * degrad ;

/*---------------------------------------------------------------------------
 *  ... and B0 the tilt of the axis.
 *---------------------------------------------------------------------------
 */
   *bangle = asin ( 0.12620L * sin (arg * raddeg) ) * degrad ;

/*---------------------------------------------------------------------------
 *  ... and the semi-diameter
 *
 *
 *  Form the mean anomalies of Venus(MV),Earth(ME),Mars(MM),Jupiter(MJ)
 *  and the mean elongation of the Moon from the Sun(D).
 *
 *---------------------------------------------------------------------------
 */
   t = de / 36525.0L ;

   mv = 212.6L   + moduloc ( (58517.80L   * t), 360.0L ) ;
   me = 358.476L + moduloc ( (35999.0498L * t), 360.0L ) ;
   mm = 319.5L   + moduloc ( (19139.86L   * t), 360.0L ) ;
   mj = 225.3L   + moduloc ( ( 3034.69L   * t), 360.0L ) ;
   d  = 350.7L   + moduloc ( (445267.11L  * t), 360.0L ) ;

/*---------------------------------------------------------------------------
 *  Form the geocentric distance (r) and solar radius (rsun).
 *---------------------------------------------------------------------------
 */
   r =  1.000141L - (0.016748L - 0.0000418L * t) * cos (me * raddeg)
      - 0.000140L * cos (2.0L * me * raddeg)
      + 0.000016L * cos ((58.3L + 2.0L * mv - 2.0L * me) * raddeg)
      + 0.000005L * cos ((209.1L + mv - me)*raddeg)
      + 0.000005L * cos ((253.8L - 2.0L * mm + 2.0L * me) * raddeg)
      + 0.000016L * cos (( 89.5L - mj + me) * raddeg)
      + 0.000009L * cos ((357.1L - 2.0L * mj + 2.0L * me) * raddeg)
      + 0.000031L * cos (d * raddeg) ;

   *rsun = (0.2665685L / r) * 3600.0L ;		/* arcseconds. */

   return (0) ;
}
