/* rotate_img.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an image.
 * Use bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew Stanger   HAO/NCAR   28 June 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* -------------------------------------------------------------------------- */

unsigned char *
rotate_img (imginp, nbypix, xdim, ydim, xcen, ycen, angdeg) 

   unsigned char *imginp ;	/* Input image array pointer. */
   int nbypix ;			/* # bytes per pixel. */
   int xdim, ydim ;		/* Dimensions of image. */
   float xcen, ycen ;		/* Center of rotation. */
   float  angdeg ;		/* Rotation angle in degrees (+ = CCW). */

{
   extern void  *malloc () ;
   extern float blint () ;

   char *parg ;

   unsigned char *imgrot ;
   unsigned char cval ;
   unsigned char *cinp, *cout ;

   unsigned short *sinp, *sout ;
   unsigned short sval ;

   int ix, iy ;
   int ox, oy ;

   float bval ;
   float fox, foy ;
   float cosrot, sinrot ;
   float fx, fy ;
   float fysin, fycos ;
   float *finp, *fout ;
   float fval ;

   double angrad ;		/* Rotation angle in radians. */
   double raddeg ;		/* radians/degree. */

   /* ----------------------------------------------------------------------- */

   /*** Rotation center = array center. ***/

   xcen = xdim / 2.0 - 0.5 ;
   ycen = ydim / 2.0 - 0.5 ;

   fprintf (stderr, "rotate_img> nbypix:%3d  xdim/ydim:%3d %3d  xcen/ycen:%7.2f %7.2f\n",
	    nbypix, xdim, ydim, xcen, ycen) ;

   /*** Compute constants. ***/
   /*** Need to reverse the sign of angdeg for proper rotation direction. ***/

   raddeg = acos (-1.0) / 180.0 ;
   angrad = (double) -angdeg * raddeg ;	/* Convert degrees to radianss. */
   sinrot = (float) sin ( angrad ) ;
   cosrot = (float) cos ( angrad ) ;

/*
 *   fprintf (stdout,
 *            "angdeg: %6.2f  angrad: %7.4f  sinrot: %8.4f  cosrot: %8.4f\n",
 *             angdeg, angrad, sinrot, cosrot) ;
 */

   /*** Allocate storage for both input & output arrays. ***/

   imgrot = (unsigned char *) malloc ( xdim * ydim * nbypix ) ;

   /*** Rotate image. ***/

   if (nbypix == 1)				/* unsigned char pixels. */
      {
      cinp = (unsigned char *) imginp ;
      cout = (unsigned char *) imgrot ;

      for ( iy = 0 ; iy < ydim ; iy++ )
         {
         fy = (float) (iy) - ycen ;
         fysin = fy * sinrot - xcen ;
         fycos = fy * cosrot + ycen ;

         for ( ix = 0 ; ix < xdim ; ix++)
            {
            fx = (float) (ix) - xcen ;
            fox = fx * cosrot - fysin ;
            foy = fycos + fx * sinrot ;
            ox = (int) fox ;
            oy = (int) foy ;

            /*** Use pixel truncation to fetch pixel intensity. ***/

            if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
               cval = 0 ;
            else
               cval = *(cinp + ox + (oy * xdim)) ; 

            /*** Use bilinear interpolation to compute pixel intensity. ***/

	    bval = blint (imginp, nbypix, xdim, ydim, fox, foy) ;
	    if (bval < 0.0)  bval = 0.0 ;

            cval = (unsigned char) bval ;

	    *cout++ = cval ;

            }
         }
      }

   else
   if (nbypix == 2)				/* unsigned short int pixels. */
      {
      sinp = (unsigned short *) imginp ;
      sout = (unsigned short *) imgrot ;

      for ( iy = 0 ; iy < ydim ; iy++ )
         {
         fy  = (float) (iy) - ycen ;
         fysin = fy * sinrot - xcen ;
         fycos = fy * cosrot + ycen ;

         for ( ix = 0 ; ix < xdim ; ix++)
            {
            fx = (float) (ix) - xcen ;
            fox = fx * cosrot - fysin ;
            foy = fycos + fx * sinrot ;
            ox = (int) fox ;
            oy = (int) foy ;

            /*** Use pixel truncation to fetch pixel intensity. ***/

            if ( (ox <= 0) || (ox >= xdim-1) || (oy <= 0) || (oy >= ydim-1) )
               sval = 0 ;
            else
               sval = *(sinp + ox + (oy * xdim)) ; 

            /*** Use bilinear interpolation to compute pixel intensity. ***/

	    bval = blint (imginp, nbypix, xdim, ydim, fox, foy) ;
	    if (bval < 0.0)  bval = 0.0 ;

            sval = (unsigned short) bval ;

	    *sout++ = sval ; 

/*
 *	    if (ix == iy)
 *	       fprintf (stderr, "ix/iy: %4d   sval: %7.2f  %d\n",
 *			ix, bval, sval) ;
 */
            }
         }
      }
   else
   if (nbypix == 4)				/* floating point pixels. */
      {
      finp = (float *) imginp ;
      fout = (float *) imgrot ;

      for ( iy = 0 ; iy < ydim ; iy++ )
         {
         fy = (float) (iy) - ycen ;
         fysin = fy * sinrot - xcen ;
         fycos = fy * cosrot + ycen ;

         for ( ix = 0 ; ix < xdim ; ix++)
            {
            fx = (float) (ix) - xcen ;
            fox = fx * cosrot - fysin ;
            foy = fycos + fx * sinrot ;
            ox = (int) fox ;
            oy = (int) foy ;

            /*** Use pixel truncation to fetch pixel intensity. ***/

            if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
               fval = 0 ;
            else
	       fval = *(finp + ox + (oy * xdim)) ;

            /*** Use bilinear interpolation to compute pixel intensity. ***/

	    bval = (float) blint (imginp, nbypix, xdim, ydim, fox, foy) ;
	    if (bval < 0.0)  bval = 0.0 ;

	    fval = bval ;

            *fout++ = fval ;

            }
         }
      }

   return (imgrot) ;
}

/* -------------------------------------------------------------------------- */

