/* xysplot.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 *   User specifies fxmin, fxmax, fymin, fymax.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   9 February 1987
 *   12 May 1989: remove cpgraf.h include file; make ig an argument of xyplot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   fxmin, fxmax : min & max values for x-axis.
 *   fymin, fymax : min & max values for y-axis.
 *   nval : # values of x = # values of y.
 *   namemc : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   ig = 0 for non-graphics terminal, =1 for graphics terminal.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   ltype	x-axis	y-axis
 *   -----      ------  ------
 *	1	linear	linear
 *	2	linear	log
 *	3	log	linear
 *	4	log	log
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
extern int pcount;

xysplot (x, y, fxmin, fxmax, fymin, fymax, nval, namemc, title, labx, laby,
        ixscale, iyscale, ibkg, ig)

float	x [], y [];
float	fxmin, fxmax;
float	fymin, fymax;	
int	nval;
char	namemc [];
char	title [], labx [], laby [];
int	ixscale, iyscale;
int	ibkg;
int	ig;

{
int ltype;
int i;
int lentit;
int lenlabx;
int lenlaby;
static int igph = 5;
static int majrx = 10;
static int majry = 10;
static int ixlab = 1, iylab= 1;
int minrx, minry;

float dxx, dxn;
float dyy, dyn;
float xmin, xmax, ymin, ymax;
float xxmin, xxmax, yymin, yymax;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   pcount ++;
   if (pcount == 1)  q8qini (namemc, ig);

   /*** Set scaling type. ***/

   if (ixscale == 0 && iyscale == 0)  ltype = 1;	/* x linear, y linear */
   if (ixscale == 0 && iyscale == 1)  ltype = 2;	/* x linear, y log    */
   if (ixscale == 1 && iyscale == 0)  ltype = 3;	/* x log,    y linear */
   if (ixscale == 1 && iyscale == 1)  ltype = 4;	/* x log,    y log    */
   if (ltype < 1 || ltype > 4)  ltype = 1;

   /*** Determine min & max limits for axes ranges. ***/

   xxmin = 1.e+30;
   xxmax = -1.e+30;
   yymin = 1.e+30;
   yymax = -1.e+30;

   for (i=0; i<nval; i++)
      {
      if (xxmin > x [i])  xxmin = x [i];
      if (xxmax < x [i])  xxmax = x [i];
      if (yymin > y [i])  yymin = y [i];
      if (yymax < y [i])  yymax = y [i];
      }

   if (fxmin == 0. && fxmax == 0.)
      {
      xmin = xxmin;
      xmax = xxmax;
      }
      else
      {
      xmin = fxmin;
      xmax = fxmax;
      }
   if (fymin == 0. && fymax == 0.)
      {
      ymin = yymin;
      ymax = yymax;
      }
      else
      {
      ymin = fymin;
      ymax = fymax;
      }
   printf ("xmin:%g  xmax:%g  ymin:%g  ymax:%g\n", xmin, xmax, ymin, ymax);

   /*** Write title & axes labels. ***/

   lentit = strlen (title);
   lenlabx = strlen (labx);
   lenlaby = strlen (laby);
   set (0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 1);
   pwrit (0.6,  0.9,  title, lentit,  2,  0, 0);
   pwrit (0.6,  0.08, labx,  lenlabx, 2,  0, 0);
   pwrit (0.1,  0.5,  laby,  lenlaby, 2, 90, 0);

   /*** Determine minor * major scaling values for axes. ***/

   nicexr_ (&xmin, &xmax, &majrx, &minrx, &dxx, &dxn);
   printf ("majrx:%d  minrx:%d  dxx:%f  dxn:%f\n", majrx, minrx, dxx, dxn);

   nicexr_ (&ymin, &ymax, &majry, &minry, &dyy, &dyn);
   printf ("majry:%d  minry:%d  dyy:%f  dyn:%f\n", majry, minry, dyy, dyn);

   set (0.25, 0.95, 0.15, 0.85, xmin, xmax, ymin, ymax, 1);

   /*** Background : perimeter, grid, half axis. ***/

   switch (ibkg)
      {
      case 0: break;			/* Do not change. */
      default:
      case 1: igph = 5;  break;		/* perimeter with tick marks. */
      case 2: igph = 0;  break;		/* full grid. */
      case 3: igph = 10; break;		/* half axes. */
      }

/*
 *   periml (majrx, minrx, majry, minry);
 */

   gridal (majrx, minrx, majry, minry, ixlab, iylab, igph, xmin, ymin);

   /*** Draw curve through data points. ***/

   curve (x, y, nval);
   frame();

   return (0);
}
